package gov.anl.gss.sdm.server.model.analysis;

import gov.anl.gss.sdm.server.model.analysis.graphtheory.AdjacencyMatrix;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.CyclePartition;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.ExogenousVariableManager;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.ExogenousVariables;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.IndependentLoopSet;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.LevelPartition;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.MatrixStatistics;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.ReachabilityMatrix;
import gov.anl.gss.sdm.server.model.analysis.leea.LeeaEquationAnalyzer;
import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.analysis.pathwaysparticipationmetrics.PPMCodeGenerator;
import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.analysis.polarity.EquationAnalyzer;
import gov.anl.gss.sdm.server.model.analysis.polarity.IfThenElseExpression;
import gov.anl.gss.sdm.server.model.analysis.polarity.IfThenElseVariableRelationship;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityCodeBuilder;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResultAnalyzer;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityRunner;
import gov.anl.gss.sdm.server.model.analysis.polarity.VariableComplexity;
import gov.anl.gss.sdm.server.model.analysis.trees.CausalAnalyzer;
import gov.anl.gss.sdm.server.model.analysis.trees.VariableSubstitution;
import gov.anl.gss.sdm.server.model.analysis.variableclass.VariableClassAnalyzer;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.StandaloneGUI;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.SDFunctionsWithXLSColt;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationComponents;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelComponents;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;
import gov.anl.gss.sdm.server.model.translator.SystemDynamicsObjectManager;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;
import gov.anl.gss.sdm.server.model.translator.View;

import java.io.File;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

public class AnalysisManager {
static Logger log = Logger.getLogger(AnalysisManager.class.getSimpleName());
	
public final static Set<String> IF_THEN_ELSE = new LinkedHashSet<String>();
	public final static Set<String> LOOKUPS = new LinkedHashSet<String>();
	
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG2 = new LinkedHashSet<String>();
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY = new LinkedHashSet<String>();
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG2_POSITIVE_POLARITY = new LinkedHashSet<String>();
	
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG3 = new LinkedHashSet<String>();
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG3_NEGATIVE_POLARITY = new LinkedHashSet<String>();
	public final static Set<String> FUNCTIONS_WITH_TIME_ARG3_POSITIVE_POLARITY = new LinkedHashSet<String>();
	
	public final static Set<String> FUNCTIONS_USE_ARG1 = new LinkedHashSet<String>();
	public final static Set<String> FUNCTIONS_USE_ARG13 = new LinkedHashSet<String>();
	public final static Set<String> RANDOM = new LinkedHashSet<String>();
	
	static {
	
		FUNCTIONS_WITH_TIME_ARG2.add("SMOOTH");		// SMOOTH(X,T)
		FUNCTIONS_WITH_TIME_ARG2.add("SMOOTH3");		// SMOOTH3(X,T)
		FUNCTIONS_WITH_TIME_ARG2.add("SMOOTHI");		// SMOOTHI(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2.add("SMOOTHN");		// SMOOTH N(X,T,I,N)
		FUNCTIONS_WITH_TIME_ARG2.add("SMOOTH3I");		// SMOOTH3I(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2.add("DELAY1");		// DELAY1(X,T)
		FUNCTIONS_WITH_TIME_ARG2.add("DELAY3");		// DELAY3(X,T)
		FUNCTIONS_WITH_TIME_ARG2.add("STEP");			// STEP(H,T)
		FUNCTIONS_WITH_TIME_ARG2.add("DELAYFIXED");	// DELAY FIXED(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2.add("DELAYN");		// DELAY N(X,T,I,N)
		FUNCTIONS_WITH_TIME_ARG2.add("DELAY1I"); 
		FUNCTIONS_WITH_TIME_ARG2.add("DELAY3I");		// DELAY3I(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2.add("PULSE");
		FUNCTIONS_WITH_TIME_ARG2.add("RAMP");
		
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("SMOOTH");		// SMOOTH(X,T)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("SMOOTH3");		// SMOOTH3(X,T)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("SMOOTHI");		// SMOOTHI(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("SMOOTHN");		// SMOOTH N(X,T,I,N)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("SMOOTH3I");		// SMOOTH3I(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAY1");		// DELAY1(X,T)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAY3");		// DELAY3(X,T)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("STEP");			// STEP(H,T)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAYFIXED");	// DELAY FIXED(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAYN");		// DELAY N(X,T,I,N)
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAY1I"); 
		FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.add("DELAY3I");		// DELAY3I(X,T,I)
		FUNCTIONS_WITH_TIME_ARG2_POSITIVE_POLARITY.add("PULSE");
		
		FUNCTIONS_WITH_TIME_ARG3.add("RAMP");
		FUNCTIONS_WITH_TIME_ARG3_POSITIVE_POLARITY.add("RAMP");
		
		
		FUNCTIONS_USE_ARG1.add("SMOOTH");		// SMOOTH(X,T)
		FUNCTIONS_USE_ARG1.add("SMOOTH3");		// SMOOTH3(X,T)
		FUNCTIONS_USE_ARG1.add("SMOOTH3I");
		FUNCTIONS_USE_ARG1.add("SMOOTHI");		// SMOOTHI(X,T,I)
		FUNCTIONS_USE_ARG1.add("DELAY1");		// DELAY1(X,T)
		FUNCTIONS_USE_ARG1.add("DELAY3");		// DELAY3(X,T)
		FUNCTIONS_USE_ARG1.add("PULSE");		// PULSE(S,D)
		FUNCTIONS_USE_ARG1.add("STEP");			// STEP(H,T)
		FUNCTIONS_USE_ARG1.add("INITIAL");		// INITIAL(A)
		FUNCTIONS_USE_ARG1.add("RAMP");			// RAMP(S,T1,T2)
		FUNCTIONS_USE_ARG1.add("SUM");			// SUM( X[r!] )
		FUNCTIONS_USE_ARG1.add("INTEGER");		// INTEGER(X)
		
		
		FUNCTIONS_USE_ARG13.add("SMOOTHN");		// SMOOTH N(X,T,I,N)
		FUNCTIONS_USE_ARG13.add("SMOOTH3I");		// SMOOTH3I(X,T,I)
		FUNCTIONS_USE_ARG13.add("SMOOTHI");		// SMOOTHI(X,T,I)
		FUNCTIONS_USE_ARG13.add("DELAYFIXED");	// DELAY FIXED(X,T,I)
		FUNCTIONS_USE_ARG13.add("DELAYN");		// DELAY N(X,T,I,N)
		FUNCTIONS_USE_ARG13.add("DELAY1I"); 
		FUNCTIONS_USE_ARG13.add("DELAY3I");		// DELAY3I(X,T,I)
		FUNCTIONS_USE_ARG13.add("TREND");		// TREND(I,H,N)
		
		
		IF_THEN_ELSE.add("IFTHENELSE");
		IF_THEN_ELSE.add("IF_THEN_ELSE");
		RANDOM.add("RANDOM01");
		LOOKUPS.add("LOOKUP");
	}; 

	private String dataFile;
	private String outputFile;
	private SDObjectManager equationManager;
	private LoopManager loopManager;
	private SystemDynamicsObjectManager sdObjectManager;
	private CausalAnalyzer causalAnalyzer;
	private Map<String, Equation> equations;
	private Map<String, View> graphics;
	
	private ExogenousVariableManager exogenousVariableManager;
	private ReachabilityMatrix reach;
	
	private MessageProcessor mf = null;
	
	private String objectName;
	
	private NativeDataTypeManager ndtm;
	
	private PolarityResults polarityResultsNoIVV = new PolarityResults();
	
	private AdjacencyMatrix adjacencyMatrix;
	private ReachabilityMatrix reachabilityMatrix;
	private ReachabilityMatrix loopReachability;
	private LevelPartition levelPartition;
	private VariableClassAnalyzer variableClassAnalyzer;
	
	private boolean sdmuiMode = false;
	private String sdmuiRootDirectory;
	
	VariableComplexity variableComplexity;
	
	
	private ModelComponents modelComponents;
	
	Documentor documentor;

	public AnalysisManager(String dataFile, String outputFile) {

		this.dataFile = dataFile;
		this.outputFile = outputFile;
		this.ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
	}

	public AnalysisManager(SystemDynamicsObjectManager sdObjectManager, Map<String, Equation> equations, Map<String, View> graphics, 
			boolean fullyDefined, MessageProcessor mf, String objectName) {
		this.sdObjectManager = sdObjectManager;
		this.equations = equations;
		this.graphics = graphics;
		this.ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		this.mf = mf;
		
		this.objectName = objectName;
		
		InformationManagers.getInstance().setAnalysisManager(this);
		
		
		if (fullyDefined) {
			equationManager = new SDObjectManager(equations, graphics, fullyDefined);
			InformationManagers.getInstance().getSdObjectManager().setObjectName(objectName);
			InformationManagers.getInstance().getUserSelectedOptions().configureForMap(false);
			NumberFormatter.map = false;
		} else {
			NumberFormatter.map = true;
			InformationManagers.getInstance().getUserSelectedOptions().configureForMap(true);
			equationManager = new SDObjectManager(equations, graphics, fullyDefined);
			InformationManagers.getInstance().getSdObjectManager().setObjectName(objectName);
		}
		
		
		
		InformationManagers.getInstance().setSdObjectManager(equationManager);
		
		polarityResultsNoIVV.loadCausualLinks(equationManager);
		
		modelComponents = new ModelComponents(equations);
	}
	

	
	public void document(String projectDirectory) {
		documentor = new Documentor(this, equations, modelComponents);
		documentor.document(projectDirectory);
		
		// restore to fully defined for options
		InformationManagers.getInstance().getUserSelectedOptions().configureForMap(false);
		NumberFormatter.map = false;
	}
	
	
	
	public Documentor getDocumentor() {
		return documentor;
	}

	public void analyzeFullyConnected() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		if (modelComponents.canAnalyzeCausalTrees()) {
//			generateCausalTrees(sdObjectManager);
		} else {
			log.debug("Sorry, can't generate causal trees");
		}
		
		
			mf.addMessage(language.getText(StringConstants.GENERATING_POLARITY_CODE) + "...\n");
			mf.setBlipMode(true);
	
		
		generatePolarityFromArrows();
		
		variableComplexity = new VariableComplexity(polarityResultsNoIVV);
		variableComplexity.process();
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES)) {
			mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_ANALYSIS));
			exogenousAnalysis();
		}
		
			mf.setBlipMode(false);
			
			sdmuiMode = true;
			
		
		if (options.isSelected(StringConstants.ANALYZE_LOOPS)){ 
			mf.addMessage(language.getText(StringConstants.GENERATING_LOOPS) + "...\n");
			String len =  options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM);

			String extraStuff = " ("+language.getTextHtml(StringConstants.MAXIMUM_LOOP_LENGTH)+": "+len+")";
			mf.addMessage(language.getText(StringConstants.GENERATING_LOOPS) + 
					extraStuff+
					"...\n");

			if (modelComponents.canAnalyzeLoops()) {
				generateLoops(equations);
//				independentLoopSet();
			} else {
				log.debug("Sorry, can't generate loops");
			}
			
			
		} else {

			mf.addMessage(language.getText(StringConstants.LOOPS_NOT_REQUESTED) + "...\n");
			loopManager = new LoopManager();
		}
	}
	
	public void exogenousAnalysis() {

		// make sure that the input/output link information is available at
		// the equation level
		if (!InformationManagers.getInstance().getModelManager().isMap())
			InformationManagers.getInstance().getSdObjectManager().broadcastLinks();

		EquationManager em = InformationManagers.getInstance().getEquationManager();
		int numExoFromEqn = 0;
		Set<String> exoVars = new LinkedHashSet<String>();
		for (Equation eqn : em.getAllEquationsList()) {
//			if (Parser.isSystemVariable(eqn.getVariableName()))
//				continue;
			if (eqn.isDefinesSubscript())
				continue;
			if (eqn.isExogenous()) {
				exoVars.add(Parser.getVariableName(eqn.getVariableName()));
//				System.out.println("Var: "+eqn.getVariableName()+" exo: "+eqn.isExogenous()+" numInLinmks "+eqn.getNumInputLinks()+" EQN: "+eqn.getVensimEquationOnly());
				numExoFromEqn++;
			}
		}
		System.out.println("numExoFromEqn = "+numExoFromEqn);
		System.out.println("numExoFrom exoVars = "+exoVars.size());

		AdjacencyMatrix adj = new AdjacencyMatrix(equationManager,polarityResultsNoIVV);
		adj.build("EV");

		reach = new ReachabilityMatrix(equationManager, polarityResultsNoIVV);
		reach.setPredecessor(adj.getPredecessor());
		reach.setSuccessor(adj.getSuccessor());

		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		reach.setMaxLevels(options.getOptionValue(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH));

		reach.buildViaTree(mf, adj);
		//		reach.build(adj);

		ExogenousVariables ev = new ExogenousVariables(reach);
		ev.process(mf);
		exogenousVariableManager = new ExogenousVariableManager(ev);

		log.debug("Number of exogenous variables: "+exogenousVariableManager.getNumberOfExogenousVariables());
		log.debug("Number of endogenous variables: "+exogenousVariableManager.getNumberOfEndogenousVariables());

	}
	
	private ReachabilityMatrix getReachabilityMatrix() {
		
		boolean disregard = false; // true;
		if (disregard)
			return null;
		if (reach != null)
			return reach;
		
		PolarityResults pr = new PolarityResults();

		InformationManagers.getInstance().getSdObjectManager().loadPolarityArrowsFromEquations(pr);
		AdjacencyMatrix adj = new AdjacencyMatrix(equationManager, pr);
		adj.build("GRM");
		
		reach = new ReachabilityMatrix(equationManager, pr);
		reach.build("GRM", adj);
		
		return reach;
	}
	
	private boolean loopComputationManageable() {
		boolean manageable = true;
		PolarityResults pr = new PolarityResults();
		InformationManagers.getInstance().getSdObjectManager().loadPolarityArrowsFromEquations(pr);
		AdjacencyMatrix adj = new AdjacencyMatrix(equationManager, pr);
		adj.build("LCM");
		MatrixStatistics stats = new MatrixStatistics(adj);
		stats.report();
		
		ReachabilityMatrix reach = new ReachabilityMatrix(equationManager, pr);
		reach.build("LCM", adj);
		stats = new MatrixStatistics(reach);
		stats.report();
		return false;
	}
	
	private void generatePolarityFromArrows() {
		InformationManagers.getInstance().getSdObjectManager().loadPolarityResultsFromGraphics(polarityResultsNoIVV);
	}
	
	private void generatePolarityArrowsFromEquations() {
		InformationManagers.getInstance().getSdObjectManager().loadPolarityArrowsFromEquations(polarityResultsNoIVV);
	}
	
	public void independentLoopSet() {
		
		Map<Integer, CyclePartition> cyclePartitions = new LinkedHashMap<Integer, CyclePartition>();
		adjacencyMatrix = new AdjacencyMatrix(equationManager, null);
		adjacencyMatrix.build("ILS");
		
	
		
		reachabilityMatrix = new ReachabilityMatrix(equationManager, null);
		reachabilityMatrix.build("ILS",adjacencyMatrix);
		
//		AdjacencyMatrix loopAdj = loopManager.getLoopDerivedAdjacency();
//		loopReachability = new ReachabilityMatrix(equationManager);
//		loopReachability.build(loopAdj);
		
//		levelPartition = new LevelPartition(loopReachability);
		levelPartition = new LevelPartition(reachabilityMatrix);
		levelPartition.build();
		
		adjacencyMatrix.blockOrder(levelPartition);
		
		boolean verified = verifyLevelPartitionWithLoops(levelPartition, loopManager);
		if (!verified) {
			System.out.println("verifyLevelPartitionWithLoops = false");
		}
		IndependentLoopSet[] ils = new IndependentLoopSet[levelPartition.getNumLevels()+1];
		for (int level = 1; level <= levelPartition.getNumLevels(); level++) {
			CyclePartition cyclePartition = new CyclePartition(level, levelPartition, /*loopReachability*/ reachabilityMatrix );
			cyclePartition.build();
			cyclePartitions.put(level, cyclePartition);
			if (cyclePartition.hasCyclePartitions()) {
				cyclePartition.printCyclePartitions();
				ils[level] = new IndependentLoopSet(loopManager, cyclePartition);
			}
		}
		
		
	}
	
	public boolean verifyLevelPartitionWithLoops(LevelPartition levelPartition, LoopManager loopManager) {
		System.out.println("### verifyLevelPartitionWithLoops");
		boolean verified = true;
		for (int level = 1; level <= levelPartition.getNumLevels(); level++) {
			Set<String> varsInLevel = levelPartition.getVariablesInLevel(level);
			System.out.println("Level "+level+" num vars in level = "+varsInLevel.size());
			for (String var : varsInLevel) {
				Set<Loop> loops = loopManager.getLoopsContaining(var);
				if (loops == null)
					continue;

				for (Loop loop : loops) {
					for (String loopVar : loop.getLoop()) {
						
						if (!varsInLevel.contains(Utilities.idToName(loopVar))) {
							verified = false;
							System.out.println("Level Loop Mismatch "+level+" "+var+" "+loopVar);
						}
					}
				}
			}
		}
		return verified;
	}

	public void analyzeFullyDefined() {
		
//		MicMac micmac = new MicMac(InformationManagers.getInstance().getEquationManager().getNumberOfVariables());
//		micmac.run();

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		if (modelComponents.canAnalyzeCausalTrees()) {
//			generateCausalTrees(sdObjectManager);
		} else {
			log.debug("Sorry, can't generate causal trees");
		}
		
		
//		if (InformationManagers.getInstance().getReuse().isPolarity()) {
//			mf.addMessage(language.getText(StringConstants.REUSE_POLARITIES)); //    "Reusing Arrow Polarity Data");
//			polarityResultsNoIVV = new PolarityResults();
//			polarityResultsNoIVV.setEquationManager(equationManager);
//			File fromF = new File(InformationManagers.getInstance().getReuse().getDocGraphDirectory()+
//					"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
//			File toF = new File(InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+
//					"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
//			try {
//				if (!fromF.getCanonicalPath().equals(toF.getCanonicalPath())) {
//					FileUtils.copyFile(fromF, toF);
//				}
//				List<String> fromFile = Utilities.getAsList(InformationManagers.getInstance().getReuse().getDocGraphDirectory()+
//						"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
//				int row = 0;
//				for (String line : fromFile) {
//					if (row++ == 0)
//						continue;
//					String[] parts = line.split("\t");
//					polarityResultsNoIVV.setPolarityForArrow(parts[0], parts[1], parts[2]);
//				}
//			} catch (IOException e) {
//				generatePolarity(language);
//			}	
//		} else {
//			generatePolarity(language);
//		}
		
	
		variableComplexity = new VariableComplexity(polarityResultsNoIVV);
		variableComplexity.process();
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES)) {
			mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_ANALYSIS)); // "Exogenous/Endogenous Analysis");
			exogenousAnalysis();
		}
		
			mf.setBlipMode(false);
		

		
		if ((options.isSelected(StringConstants.ANALYZE_LOOPS) || 
				(StandaloneGUI.threeDEnabled() && options.isSelected(StringConstants.GENERATE_3D))) /* && 
				!InformationManagers.getInstance().getReuse().isLoops()*/){ 
			
			if (StandaloneGUI.threeDEnabled() && (options.isSelected(StringConstants.GENERATE_3D)) && 
					!options.isSelected(StringConstants.ANALYZE_LOOPS)) {
				mf.addMessage(language.getText(StringConstants.TD_GENERATION_REQUIRES_LOOPS_BE_GENERATED));
			}
			String len =  options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM);

			String extraStuff = " ("+language.getTextHtml(StringConstants.MAXIMUM_LOOP_LENGTH)+": "+len+")";
			mf.addMessage(language.getText(StringConstants.GENERATING_LOOPS) + 
					extraStuff+
					"...\n");
			mf.setBlipMode(true);
			
			if (modelComponents.canAnalyzeLoops()) {
				generateLoops(equations);
//				independentLoopSet();
				
					mf.setBlipMode(false);
			} else {
				log.debug("Sorry, can't generate loops");
			}
		} else {
			
			mf.addMessage(language.getText(StringConstants.LOOPS_NOT_REQUESTED) + "...\n");

			loopManager = new LoopManager();
			loopManager.setSdmuiMode(sdmuiMode);
			loopManager.setSdmuiRootDirectory(sdmuiRootDirectory);
		}
	}
	
	public void analyzeFullyDefinedPolarity() {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		if (InformationManagers.getInstance().getReuse().isPolarity()) {
			mf.addMessage(language.getText(StringConstants.REUSE_POLARITIES)); //    "Reusing Arrow Polarity Data");
			polarityResultsNoIVV = new PolarityResults();
			polarityResultsNoIVV.setEquationManager(equationManager);
			File fromF = new File(InformationManagers.getInstance().getReuse().getDocGraphDirectory()+
					"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
			File toF = new File(InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+
					"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
			try {
				if (!fromF.getCanonicalPath().equals(toF.getCanonicalPath())) {
					FileUtils.copyFile(fromF, toF);
				}
				List<String> fromFile = Utilities.getAsList(InformationManagers.getInstance().getReuse().getDocGraphDirectory()+
						"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
				int row = 0;
				for (String line : fromFile) {
					if (row++ == 0)
						continue;
					String[] parts = line.split("\t");
					polarityResultsNoIVV.setPolarityForArrow(parts[0], parts[1], parts[2]);
				}
			} catch (IOException e) {
				generatePolarity(language);
			}	
		} else {
			generatePolarity(language);
		}
		
	

	}
	public void analyzeFullyDefinedLeea() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		generateLeea(language);
	}
		
	private void generateLeea(SDMDocLanguageSpecific language) {
//		generatePolarityArrowsFromEquations();
		

		mf.addMessage(language.getText(StringConstants.GENERATING_LEEA_CODE) + "...\n");
		mf.setBlipMode(true);

		generateLeeaCode(equations);
	}
	
	public void  generateLeeaCode(Map<String, Equation> equations) {
		
//		SDObjectManager mgr = InformationManagers.getInstance().getSdObjectManager();

		int numEqn = equations.size();  // InformationManagers.getInstance().getEquationManager().getVariables().size();
		int numProcessed = 0;
		int groupSize = Math.max(numEqn/10, 1);
		int group = 1;
		
		boolean canProcessCode = true;
		
		boolean databaseAvailable = InformationManagers.getInstance().getModelManager().isRunModel() && 
				InformationManagers.getInstance().getModelManager().isModelSuccessfullyExecuted();

		if (databaseAvailable) {

			// setup reference to baseline database for argument Builder
			String extendedMemoryFile = InformationManagers.getInstance().getFileOrganizer().getRunsDirectory()+"/baselineHSQLDB";
			InformationManagers.getInstance().setDataBridge(extendedMemoryFile);

		}
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		Set<String> s = InformationManagers.getInstance().getEquationManager().getVariables();
		for (String var : InformationManagers.getInstance().getEquationManager().getLhsVariables() /* InformationManagers.getInstance().getEquationManager().getVariables()  equations.keySet()*/ ) {
			PolarityResultAnalyzer.DEBUG = false;
			numProcessed++;
			if (numProcessed % groupSize == 0) {
				if (group < 10)
					mf.addMessage(language.getText(StringConstants.PROGRESS)+":  "+group*10+"% "+
				language.getText(StringConstants.OF)+" "+numEqn);
				group++;
			}

			Equation eqn = InformationManagers.getInstance().getEquationManager().getEquationFor(var); // equations.get(var);
				if (PolarityResultAnalyzer.DEBUG) {
					log.debug("#######EQN: "+eqn.getVensimEquationOnly());
				}

				if (!eqn.isAssignment()) {
					continue;
				}
				if (eqn.isDefinesLookup() || 
						eqn.isUsesTimeSeries() || 
						eqn.isDefinesLookupGetXls() || 
						eqn.isGetExternalData() || 
						eqn.isArrayInitialization() ||
						eqn.isTypeString() || 
						eqn.isDefinesSubscript()) {
					continue;
				}

				// This root is what we will be processing on
				// we may be modifying the contents of the tree based on need
				Node root = eqn.getCopyOfTree();

				// cases determined without code generation
				
				// this needs to execute the code!
				// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//				if (eqn.isReferencesLookup() ) {
//					
//					Node rootNoLookup = eqn.getCopyOfTree();
//					removeFunctions(rootNoLookup, LOOKUPS);
//					
//
//					EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
//							Node.generateExpression(TreeTraversal.getRhs(rootNoLookup))+"~~autogenerated9");
//					EquationAnalyzer ea = new EquationAnalyzer(lEqn, eqn, polarityResultsNoIVV, /*polarityResultsWithIVV*/ null, ndtm);
//					if (ea.isCannotProcess()) {
//						System.out.println("Had problem with: "+lEqn.getVensimEquation());
//						continue;
//					}
//					
//					String polarity = null;
//
//					boolean containsExpressionInLookupArgument = false;
//
//					// change lookup references to (slope * arg) computation
//
//					canProcessCode = true;
//
//
//					for (Node lookupFunction : TreeTraversal.getFunctionCallsInTree(root, LOOKUPS)) {
//
//						Node lookup = TreeTraversal.getFunctionArgument(lookupFunction, 1);
//						Node lookupArg = TreeTraversal.getFunctionArgument(lookupFunction, 2);
//
//						// note that there can be a number of variables in the lookup arg that will
//						// need to be processed individually
//
//						Set<String> variablesInLookupArg = new LinkedHashSet<String>();
//						TreeTraversal.getVariablesInTree(lookupArg, variablesInLookupArg);
//
//						String tail = lookupArg.getToken();
//						String head = eqn.getLhs(); 
//
//						// get the proper identifiers
//						NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
//						tail = ndtm.getOriginalName(tail);
//						head = ndtm.getOriginalName(head);
//
//						String lookupName = lookup.getToken().replace("lookup.", "");
//						Equation lookupEqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lookupName);
//						if (lookupEqn == null) {
//							setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, ArrowPolarity.LOOKUP_NO_EQN);
//
//							polarityResultsNoIVV.setPolarityForArrow(lookupName, head, ArrowPolarity.LOOKUP_NO_EQN);
//
//							canProcessCode = false;
//						} else if (lookupEqn.isDefinesLookupGetXls()) {
//
//							setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, ArrowPolarity.LOOKUP_EXTERNAL);
//
//
//							polarityResultsNoIVV.setPolarityForArrow(lookupName, head, ArrowPolarity.LOOKUP_EXTERNAL);
//							canProcessCode = false;
//						} else {
//
//							// here polarity is for the lookup (increasing to the right vs. decreasing to the right)
//							polarity = determineLookupPolarity(lookupEqn);
//							if (polarity.equals(ArrowPolarity.UNKNOWN)) {
//
//								setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, ArrowPolarity.LOOKUP_INCONSISTENT);
//
//								polarityResultsNoIVV.setPolarityForArrow(lookupName, head, ArrowPolarity.LOOKUP_INCONSISTENT);
//								
//							} else {
//
//								polarityResultsNoIVV.setPolarityForArrow(lookupName, head, polarity);
//
//								// note: this can be an expression that we'll need to process
//
//								String token = lookupArg.getToken();
//								if (Parser.isOperator(token) || Parser.isFunctionInvocation(token)) {
//									containsExpressionInLookupArgument = true;
//								} else {
//									polarityResultsNoIVV.setPolarityForArrow(ndtm.getOriginalName(lookupArg.getToken()), head, polarity);
//								}
//							}
//						}
//
//						String slope;
//						Node replacement = null;
//
//						if (polarity != null) {
//
//							if (polarity.equals(ArrowPolarity.NEGATIVE))
//								slope = "-1.0";
//							else
//								slope = "1.0";
//							if (containsExpressionInLookupArgument) {
//								replacement = new Node("*");
//								Node slop = new Node(slope);
//								replacement.setChild(slop);
//								slop.setParent(replacement);
//
//								slop.setNext(lookupArg);
//								lookupArg.setPrevious(slop);
//
//								lookupArg.setParent(replacement);
//							} else {
//								// this is the replacement node which will be just the "slope" value as pos/neg
//								replacement = new Node(slope);
//							}
//
//
//
//							Node.replaceNode(lookupFunction, replacement);
//						}	else {
//						}
//					}
//
//				}
				// end
				// this needs to execute the code!
				// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
				if (canProcessCode) {
					getRidOfRandom(root);

					if (containsIfThenElse(root)) {


						if (root.getChild() == null) {
							System.out.println("Skipping ITE: "+eqn.getVensimEquation());
							continue;
						}

						IfThenElseExpression iteExpression = new IfThenElseExpression(root);
						initializeITESwitchPolaity(iteExpression);

						iteExpression.setConditionVariablePolarity(getPolarityResultsNoIVV());

						EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
								Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated8");

						LeeaEquationAnalyzer ea = new LeeaEquationAnalyzer(lEqn, eqn, polarityResultsNoIVV, null, ndtm);
						ea.initialize();
						if (ea.isCannotProcess()) {
							System.out.println("Had problem with: "+lEqn.getVensimEquation());
							continue;
						}

					} else {  

						EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
								Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated9");

						LeeaEquationAnalyzer ea = new LeeaEquationAnalyzer(lEqn, eqn, polarityResultsNoIVV,null, ndtm);
						ea.initialize();
						if (ea.isCannotProcess()) {
							System.out.println("Had problem with: "+lEqn.getVensimEquation());
							continue;
						}



					}
				} else {
					canProcessCode = true;
				}
		}

		if (databaseAvailable) {
			InformationManagers.getInstance().getDataBridge().shutdown();
		}
		mf.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+ 
				language.getText(StringConstants.OF)+" "+numEqn+"\n");
	}
		
	
	private void generatePolarity(SDMDocLanguageSpecific language) {
		generatePolarityArrowsFromEquations();
		

		mf.addMessage(language.getText(StringConstants.GENERATING_POLARITY_CODE) + "...\n");
		mf.setBlipMode(true);

		generatePolarityCode(equations);

		if (SDM_Doc.DUMP_TABULAR_DATA) {
			polarityResultsNoIVV.writeArrowsToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure/arrowPolarityFromEquationsNoIVV.tab");
			// ############## WRITE_BRIDGE_FILES
			if (StaticSwitches.WRITE_BRIDGE_FILES) {
				polarityResultsNoIVV.writeArrowThicknessToFile(
						InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure/arrowThicknessNoIVV.tab");
			}
		}

		if (sdmuiMode) {
			polarityResultsNoIVV.writeArrowsToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure"+"/arrowPolarityFromEquations.tab");
			// ############## WRITE_BRIDGE_FILES
			if (StaticSwitches.WRITE_BRIDGE_FILES) {
				polarityResultsNoIVV.writeArrowThicknessToFile(
						InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure"+"/arrowThickness.tab");
			}
		}
	}
	
	private void substituteVariables(Map<String, Equation> equations) {
		
		VariableSubstitution vs = new VariableSubstitution(equations);
		vs.processAllStocks();
		
	}
	
	private void buildPPMCode(Map<String, Equation> equations) {
		

		PPMCodeGenerator vs = new PPMCodeGenerator(equations, causalAnalyzer, sdObjectManager, this);
		vs.processAllStocks();
		
	}


	private void generateLoops(Map<String, Equation> equations) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		boolean generateWithIVV = options.isSelected(StringConstants.ANALYZE_LOOPS_WITH_IVV);
		boolean unlimited = false; // options.isSelected(StringConstants.LOOP_SIZE_AUTO_DISCOVER);
		String max = options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM);
		int maxSize = Integer.MAX_VALUE - 10;
		maxSize = Integer.parseInt(max);

		loopManager = new LoopManager(equationManager, polarityResultsNoIVV, null/*polarityResultsWithIVV*/, 
				InformationManagers.getInstance().getNativeDataTypeManager(), getReachabilityMatrix(), generateWithIVV,
				unlimited, maxSize, mf);
		
		loopManager.setSdmuiMode(sdmuiMode);
		loopManager.setSdmuiRootDirectory(sdmuiRootDirectory);
		loopManager.process();
	}

	public void generateCausalTrees(SystemDynamicsObjectManager sdObjectManager) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		log.debug("\n\n>>>>>>>>>>>>>> Generate Causal Trees <<<<<<<<<<<<<<<<<\n");
		mf.addMessage(language.getText(StringConstants.GENERATING_CAUSAL_TREES) + "...\n");
				
		causalAnalyzer = new CausalAnalyzer(sdObjectManager);
		causalAnalyzer.generateCausalTrees();
	}
	
	private Set<String> deannotate(Set<String> set) {
		Set<String> newSet = new LinkedHashSet<String>();
		for (String element : set) {
			newSet.add(ArrayReference.deannotate(element));
		}
		return newSet;
	}

	public void  generatePolarityCode(Map<String, Equation> equations) {
		
		SDObjectManager mgr = InformationManagers.getInstance().getSdObjectManager();
		
		Set<String> mgrVariables = mgr.getVariables();
		Set<String> mgrLhs = mgr.getLhsVariables();
		
		Set<String> fromEquations = new LinkedHashSet<String>(deannotate(equations.keySet()));
		Set<String> fromMananger = new LinkedHashSet<String>(InformationManagers.getInstance().getEquationManager().getVariables());
		Set<String> inter = Utilities.intersect(fromEquations, fromMananger);
		Set<String> minus = Utilities.minus(fromMananger, fromEquations);

		int numEqn = equations.size();  // InformationManagers.getInstance().getEquationManager().getVariables().size();
		int numProcessed = 0;
		int groupSize = Math.max(numEqn/10, 1);
		int group = 1;
		
		boolean canProcessCode = true;
		boolean stripLookup = false;
		
		boolean databaseAvailable = InformationManagers.getInstance().getModelManager().isRunModel() && 
				InformationManagers.getInstance().getModelManager().isModelSuccessfullyExecuted();
//		databaseAvailable = false;
		
		if (databaseAvailable) {

			// setup reference to baseline database for argument Builder
			String extendedMemoryFile = InformationManagers.getInstance().getFileOrganizer().getRunsDirectory()+"/baselineHSQLDB";
			InformationManagers.getInstance().setDataBridge(extendedMemoryFile);

		}
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		Set<String> s = InformationManagers.getInstance().getEquationManager().getVariables();
		for (String var : InformationManagers.getInstance().getEquationManager().getVariables() /* InformationManagers.getInstance().getEquationManager().getVariables()  equations.keySet()*/ ) {
			PolarityResultAnalyzer.DEBUG = false;
			numProcessed++;
			

			if (numProcessed % groupSize == 0) {
				if (group < 10)
					mf.addMessage(language.getText(StringConstants.PROGRESS)+":  "+group*10+"% "+
				language.getText(StringConstants.OF)+" "+numEqn);
				group++;
			}

			Set<Equation> setOfEqn = InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var);
//			if (setOfEqn.size() > 1)
//				System.out.println("Multiple equations for "+var);

			for (Equation eqn : setOfEqn) {
//			Equation eqn = InformationManagers.getInstance().getEquationManager().getEquationFor(var); // equations.get(var);
			if (eqn == null) {
				System.out.println("POLARITY NULL FOR VAR: "+var);
				continue;
			}
			

			
				if (PolarityResultAnalyzer.DEBUG) {
					log.debug("#######EQN: "+eqn.getVensimEquationOnly());
				}

				if (!eqn.isAssignment()) {
					continue;
				}
				if (eqn.isDefinesLookup() || 
						eqn.isUsesTimeSeries() || 
						eqn.isDefinesLookupGetXls() || 
						eqn.isGetExternalData() || 
						eqn.isArrayInitialization() ||
						eqn.isTypeString() || 
						eqn.isDefinesSubscript()) {
					continue;
				}
				
//				if (eqn.getVensimEquationOnly().startsWith("\"Quit_ratio_from_cigarette_price_in"))
//					System.out.println("Quit_ratio_from_cigarette_price_in");

				// This root is what we will be processing on
				// we may be modifying the contents of the tree based on need
				Node root = eqn.getCopyOfTree();

				// cases determined without code generation
				if (eqn.isReferencesLookup() ) {
					
					Node rootNoLookup = eqn.getCopyOfTree();
					removeFunctions(rootNoLookup, LOOKUPS);
					

					EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
							Node.generateExpression(TreeTraversal.getRhs(rootNoLookup))+"~~autogenerated10");
					EquationAnalyzer ea = new EquationAnalyzer(lEqn, eqn, polarityResultsNoIVV, /*polarityResultsWithIVV*/ null, ndtm);
					if (ea.isCannotProcess()) {
						System.out.println("Had problem with: "+lEqn.getVensimEquation());
						continue;
					}
					
					String polarity = null;

					boolean containsExpressionInLookupArgument = false;

					// change lookup references to (slope * arg) computation

					canProcessCode = true;


					for (Node lookupFunction : TreeTraversal.getFunctionCallsInTree(root, LOOKUPS)) {

						Node lookup = TreeTraversal.getFunctionArgument(lookupFunction, 1);
						Node lookupArg = TreeTraversal.getFunctionArgument(lookupFunction, 2);

						// note that there can be a number of variables in the lookup arg that will
						// need to be processed individually

						Set<String> variablesInLookupArg = new LinkedHashSet<String>();
						TreeTraversal.getVariablesInTree(lookupArg, variablesInLookupArg);

						String tail = lookupArg.getToken();
						String head = eqn.getLhs(); 

						// get the proper identifiers
						NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
						tail = ndtm.getOriginalName(tail);
						head = ndtm.getOriginalName(head);

						String lookupName = lookup.getToken().replace("lookup.", "");
						Equation lookupEqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lookupName);
						if (lookupEqn == null) {
							setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, StringConstants.LOOKUP_NO_EQN);

							polarityResultsNoIVV.setPolarityForArrow(lookupName, head, StringConstants.LOOKUP_NO_EQN);

							canProcessCode = false;
						} else if (lookupEqn.isDefinesLookupGetXls()) {

							setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, StringConstants.LOOKUP_EXTERNAL);


							polarityResultsNoIVV.setPolarityForArrow(lookupName, head, StringConstants.LOOKUP_EXTERNAL);
							canProcessCode = false;
						} else {

							// here polarity is for the lookup (increasing to the right vs. decreasing to the right)
							polarity = determineLookupPolarity(lookupEqn);
							if (polarity.equals(ArrowPolarity.UNKNOWN)) {

								setPolarityForMultipleTails(polarityResultsNoIVV, variablesInLookupArg, head, StringConstants.LOOKUP_INCONSISTENT);

								polarityResultsNoIVV.setPolarityForArrow(lookupName, head, StringConstants.LOOKUP_INCONSISTENT);
								stripLookup = true;
							} else {

								polarityResultsNoIVV.setPolarityForArrow(lookupName, head, polarity);

								// note: this can be an expression that we'll need to process

								String token = lookupArg.getToken();
								if (Parser.isOperator(token) || Parser.isFunctionInvocation(token)) {
									containsExpressionInLookupArgument = true;
								} else {
									polarityResultsNoIVV.setPolarityForArrow(ndtm.getOriginalName(lookupArg.getToken()), head, polarity);
								}
							}
						}

						String slope;
						Node replacement = null;

						if (polarity != null) {

							if (polarity.equals(ArrowPolarity.NEGATIVE))
								slope = "-1.0";
							else
								slope = "1.0";
							if (containsExpressionInLookupArgument) {
								replacement = new Node("*");
								Node slop = new Node(slope);
								replacement.setChild(slop);
								slop.setParent(replacement);

								slop.setNext(lookupArg);
								lookupArg.setPrevious(slop);

								lookupArg.setParent(replacement);
							} else {
								// this is the replacement node which will be just the "slope" value as pos/neg
								replacement = new Node(slope);
							}



							Node.replaceNode(lookupFunction, replacement);
						}	else {
						}
					}

				} // else {
				if (canProcessCode) {
					getRidOfRandom(root);

					if (containsIfThenElse(root)) {
						
						if (root.getChild().getToken().contains("accumulate_values"))
							System.out.println("Zero arg issue?");

						if (root.getChild() == null) {
							System.out.println("Skipping ITE: "+eqn.getVensimEquation());
							continue;
						}

						IfThenElseExpression iteExpression = new IfThenElseExpression(root);
						initializeITESwitchPolaity(iteExpression);

						iteExpression.setConditionVariablePolarity(getPolarityResultsNoIVV());

						EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
								Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated8");

						EquationAnalyzer ea = new EquationAnalyzer(lEqn, eqn, polarityResultsNoIVV, null, ndtm);
						ea.initialize();
						if (ea.isCannotProcess()) {
							System.out.println("Had problem with: "+lEqn.getVensimEquation());
							continue;
						}

					} else {  
//
//						System.out.println("Vensim: "+eqn.getVensimEquationOnly());
//						System.out.println("Expres: "+Node.generateExpression(TreeTraversal.getRhs(root)));
//						System.out.println("Tokens: "+eqn.getRhsFromTokens());

						EquationComponents lEqn = null;

//						if (eqn.getVensimEquationOnly().contains("INTEG")) {

							lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
									Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated11");
//						} else {
//							lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
//									eqn.getRhsFromTokens()+"~~autogenerated11");
//						}

						EquationAnalyzer ea = new EquationAnalyzer(lEqn, eqn, polarityResultsNoIVV,null, ndtm);
						ea.initialize();
						if (ea.isCannotProcess()) {
							System.out.println("Had problem with: "+lEqn.getVensimEquation());
							continue;
						}



					}
				} else {
					canProcessCode = true;
					stripLookup = false;
				}
			}
		}

		if (databaseAvailable) {
			InformationManagers.getInstance().getDataBridge().shutdown();
		}
		mf.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+ 
				language.getText(StringConstants.OF)+" "+numEqn+"\n");
	}
	
	private void setPolarityForMultipleTails(PolarityResults pr, Set<String> vars, String head, String polarity) {
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		for (String t : vars) {
			String tail = ndtm.getOriginalName(t);
			pr.setPolarityForArrow(tail, head, polarity);
		}
	}
	

	private void initializeITESwitchPolaity(IfThenElseExpression iteExpression) {
		
		IfThenElseVariableRelationship rep = iteExpression.getRelationships();
		String lhs = rep.getLhs();
		List<Set<String>> arg1Vars = rep.getArg1Vars();
		for (Set<String> setVars : arg1Vars) {
			for (String var : setVars) {
				polarityResultsNoIVV.setPolarityForArrow(var, lhs, StringConstants.ITE_SWITCH);
			}
		}
		
	}
	
	   private void removeFunctions(Node root, Set<String> functions) {

	    	for (Node n : TreeTraversal.getFunctionCallsInTree(root, functions)) {
	    		// want to replace function invocation with just the first argument
	    		Node constant = new Node("999999");
	    		Node.replaceNode(n, constant);
	    	}
	    }
	
	   private void getRidOfRandom(Node root) {
	    	for (Node n : TreeTraversal.getFunctionCallsInTree(root, AnalysisManager.RANDOM)) {
	    		// want to replace function invocation with just the first argument
	    		Node constant = new Node("999999");
	    		Node.replaceNode(n, constant);
	    	}
	    }

	
	private void analyzePolarityForGeneratedEquation(String lhs, String rhs) {
		
		EquationComponents lEqn = new EquationComponents(lhs+" = " + rhs+"~~autogenerated10");

		Equation t = new Equation(lEqn, 1);
		
		PolarityCodeBuilder polarityCodeBuilder = new PolarityCodeBuilder(t, t, false);
		polarityCodeBuilder.buildCode();
		
		log.debug("###"+polarityCodeBuilder.getEquation().getVensimEquation());
		log.debug(">>>> "+polarityCodeBuilder.getGeneratedCode());
		// no functions for now, only stocks (which still need work)
		// we've pruned stock trees to remove integ function, so any functions that we see prevents processing
		
		if (SDFunctionsWithXLSColt.containsNoPolarityFunction(polarityCodeBuilder.getGeneratedCode())) {
			log.debug("\nGenPolCodeIVV: "+t.getVensimEquationOnly());
			log.debug(polarityCodeBuilder.getGeneratedCode());
			log.debug("Need to deal with this, SKIP");
			return;
		}

		PolarityRunner runner = new PolarityRunner(polarityCodeBuilder);
		PolarityResultAnalyzer polarityResultAnalyzer = new PolarityResultAnalyzer(runner);
		polarityResultAnalyzer.analyze(polarityResultsNoIVV);
//		polarityResultAnalyzer.analyze(polarityResultsWithIVV);
	}
	
	private String determineLookupPolarity(Equation eqn) {
		String polarity = ArrowPolarity.UNKNOWN;
		MutableBoolean positiveSlope = new MutableBoolean(false);
		MutableBoolean negativeSlope = new MutableBoolean(false);
		MutableBoolean nonZeroSlope = new MutableBoolean(false);
		if (!eqn.getSlopeDirections(positiveSlope, negativeSlope, nonZeroSlope))
			return polarity;
		
		if (nonZeroSlope.value()) {
			if (positiveSlope.value() && negativeSlope.value()) {
				polarity = ArrowPolarity.UNKNOWN;
			} else if (positiveSlope.value()) {
				polarity = ArrowPolarity.POSITIVE;
			} else {
				polarity = ArrowPolarity.NEGATIVE;
			}
			
		} else {
			// this is a horizontal line and by definition positive?
			polarity = ArrowPolarity.POSITIVE;
		}
		
		return polarity;
	}

	
    private boolean startsWithIfThenElse(Node root) {
    	Node rhs = TreeTraversal.getRhs(root);
    	if (Parser.isFunctionInvocation(rhs) && Parser.getFunctionName(rhs).equals("IFTHENELSE"))
    		return true;
    	else
    		return false;
    }
	
    private boolean containsIfThenElse(Node root) {
    	if (TreeTraversal.getFunctionCallsInTree(root, IF_THEN_ELSE).size() > 0) 
    		return true;
    	else
    		return false;
    }

	public SDObjectManager getEquationManager() {
		return equationManager;
	}

	public SystemDynamicsObjectManager getSdObjectManager() {
		return sdObjectManager;
	}

	public void setLoopManager(LoopManager loopManager) {
		this.loopManager = loopManager;
	}

	public LoopManager getLoopManager() {
		if (loopManager == null)
			return new LoopManager();
		return loopManager;
	}

	public PolarityResults getPolarityResultsNoIVV() {
		return polarityResultsNoIVV;
	}
	
//	public PolarityResults getPolarityResultsWithIVV() {
//		return polarityResultsWithIVV;
//	}

	public ModelComponents getModelComponents() {
		return modelComponents;
	}

	/**
	 * @return the objectName
	 */
	public String getObjectName() {
		return objectName;
	}

	/**
	 * @param objectName the objectName to set
	 */
	public void setObjectName(String objectName) {
		this.objectName = objectName;
	}

	/**
	 * @return the sdmuiMode
	 */
	public boolean isSdmuiMode() {
		return sdmuiMode;
	}

	/**
	 * @param sdmuiMode the sdmuiMode to set
	 */
	public void setSdmuiMode(boolean sdmuiMode) {
		this.sdmuiMode = sdmuiMode;
	}

	/**
	 * @return the sdmuiRootDirectory
	 */
	public String getSdmuiRootDirectory() {
		return sdmuiRootDirectory;
	}

	/**
	 * @param sdmuiRootDirectory the sdmuiRootDirectory to set
	 */
	public void setSdmuiRootDirectory(String sdmuiRootDirectory) {
		this.sdmuiRootDirectory = sdmuiRootDirectory;
	}

	public VariableComplexity getVariableComplexity() {
		return variableComplexity;
	}
	public MessageProcessor getMessageProcessor() {
		return mf;
	}

	public ExogenousVariableManager getExogenousVariableManager() {
		return exogenousVariableManager;
	}
	
	
}
