package gov.anl.gss.sdm.server.model.analysis;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.VariableManager;
import gov.anl.gss.sdm.server.model.translator.VariableNameEquivalence;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class EquationManager {
static Logger log = Logger.getLogger(EquationManager.class.getSimpleName());
	
//	private Map<String, Set<Equation>> variableToEquation = new LinkedHashMap<String, Set<Equation>>();
	
	private Map<String, Set<String>> byVariableType = new LinkedHashMap<String, Set<String>>();
	
	private Map<String, String> variableToID;
	private Map<String, String> idToVariable;
	
	private Set<String> variablesOriginal = new LinkedHashSet<String>();
	private Set<String> variablesCommon = new LinkedHashSet<String>();
	
	private Set<String> variablesLhsOriginalAsIs = new LinkedHashSet<String>();
	
	private Set<String> variablesLhsOriginal = new LinkedHashSet<String>();
	private Set<String> variablesLhsCommon = new LinkedHashSet<String>();
	
	private Map<String, Equation> variablesLhsOriginalToEquation = new LinkedHashMap<String, Equation>();
	private Map<String, Set<Equation>> variablesOriginalToEquation = new LinkedHashMap<String, Set<Equation>>();
	
	private Map<String, Equation> variablesLhsCommonToEquation = new LinkedHashMap<String, Equation>();
	private Map<String, Set<Equation>> variablesCommonToEquation = new LinkedHashMap<String, Set<Equation>>();
	
	private Map<String, String> commonToOriginal = new LinkedHashMap<String, String>();
	
	private List<Equation> allEquationsList = new ArrayList<Equation>();


	public void clear() {
		if (variableToID != null)
			variableToID = null; 
		if (idToVariable != null)			
			idToVariable = null;
		if (variablesOriginalToEquation != null)			
			variablesOriginalToEquation.clear();
		if (variablesCommonToEquation != null)			
			variablesCommonToEquation.clear();
		if (variablesLhsOriginalToEquation != null)			
			variablesLhsOriginalToEquation.clear();
		if (variablesLhsCommonToEquation != null)			
			variablesLhsCommonToEquation.clear();
		if (allEquationsList != null)			
			allEquationsList.clear();
	}
	
	public void reinitialize(VariableManager vm) {
//		clear();
		variableToID = new LinkedHashMap<String, String>();
		idToVariable = new LinkedHashMap<String, String>();
		
		for (String key : vm.getVariableToID().keySet()) {
			variableToID.put(key,  vm.getVariableToID().get(key));
		}
		
		for (String key : vm.getIdToVariable().keySet()) {
			idToVariable.put(key,  vm.getIdToVariable().get(key));
		}
	}
	
	static {};
	
	public Set<String> getLhsVariables() {
		return variablesLhsOriginal;
	}
	
	
	public void registerEquation(Equation equation) {

		String variable =  null;
		if (Parser.isQuotedString(equation.getLhs())) {
			variable = InformationManagers.getInstance()
					.getNativeDataTypeManager().getOriginalName(equation.getLhs());
		} else {
				variable = Parser.noSubscripts(InformationManagers.getInstance()
						.getNativeDataTypeManager().getOriginalName(equation.getLhs()));
		}
		registerVariable(variable, equation);
		registerLhsVariable(equation.getLhs(), equation);
		if (!allEquationsList.contains(equation))
			allEquationsList.add(equation);
	}
	
	public void registerLhsVariable(String variable, Equation equation) {
		
		// save variable names in their original form and as common form
		// do not include subscripts
		
		String varOriginal = Parser.removeAnySubscripts(Parser.removeAnnotation(variable));
		
		variablesLhsOriginalAsIs.add(varOriginal);
		
		String varOriginalQuoted;
		String varOriginalNoQuote;
		
		if (variable.startsWith("\"")) {
			varOriginalQuoted = varOriginal;
			varOriginalNoQuote = Parser.removeQuotes(varOriginal); // varOriginal.replace("\"", "");
		} else {
			varOriginalQuoted = Parser.quoted(varOriginal); // "\""+varOriginal+"\"";
			varOriginalNoQuote = varOriginal;
		}
		
		
		String commonVariable =  VariableNameEquivalence.getCommonName(varOriginal);
		String commonVariableNoQuote = Parser.removeQuotes(commonVariable); // commonVariable.replace("\"", "");
		
		
		
		variablesLhsOriginal.add(varOriginalQuoted);
		variablesLhsOriginal.add(varOriginalNoQuote);
		variablesLhsCommon.add(commonVariable);

		commonToOriginal.put(commonVariable, varOriginal);
		commonToOriginal.put(commonVariableNoQuote, varOriginal);
		
		variablesLhsOriginalToEquation.put(varOriginalQuoted, equation);
		variablesLhsOriginalToEquation.put(varOriginalNoQuote, equation);
		variablesLhsCommonToEquation.put(commonVariable, equation);
		variablesLhsCommonToEquation.put(commonVariableNoQuote, equation);
	}
	
	public void registerVariable(String variable, Equation equation) {
		
		// save variable names in their original form and as common form
		// do not include subscripts
		
		String varOriginalQuoted;
		String varOriginalNoQuote;
		String commonVariableQuoted;
		String commonVariableNoQuote ;
		
		if (variable.startsWith("\"")) {
			varOriginalQuoted = variable;
			varOriginalNoQuote = Parser.removeQuotes(variable); // variable.replace("\"", "");
			
		} else {
			varOriginalQuoted = Parser.quoted(variable); // "\""+variable+"\"";
			varOriginalNoQuote = variable;
		}
		
		commonVariableQuoted =  VariableNameEquivalence.getCommonName(varOriginalQuoted);
		commonVariableNoQuote = VariableNameEquivalence.getCommonName(varOriginalNoQuote);;
		
		
		
		variablesOriginal.add(varOriginalQuoted);
		variablesCommon.add(commonVariableQuoted);
		commonToOriginal.put(commonVariableQuoted, variable);
		
		if (!(ArrayReference.isArrayReference(varOriginalNoQuote) && !ArrayReference.isArrayReference(varOriginalQuoted))) {
			variablesOriginal.add(varOriginalNoQuote);
			variablesCommon.add(commonVariableNoQuote);
			commonToOriginal.put(commonVariableNoQuote, variable);
		}
		

		
		
		if (!variablesOriginalToEquation.containsKey(varOriginalQuoted)) {
			variablesOriginalToEquation.put(varOriginalQuoted, new LinkedHashSet<Equation>());
		}
		
		if (!variablesOriginalToEquation.containsKey(varOriginalNoQuote)) {
			variablesOriginalToEquation.put(varOriginalNoQuote, new LinkedHashSet<Equation>());
		}
		

		if (!variablesCommonToEquation.containsKey(commonVariableQuoted)) {
			variablesCommonToEquation.put(commonVariableQuoted, new LinkedHashSet<Equation>());
		}
		

		if (!variablesCommonToEquation.containsKey(commonVariableNoQuote)) {
			variablesCommonToEquation.put(commonVariableNoQuote, new LinkedHashSet<Equation>());
		}
		
		
		variablesOriginalToEquation.get(varOriginalQuoted).add(equation);
		variablesOriginalToEquation.get(varOriginalNoQuote).add(equation);
		
		variablesCommonToEquation.get(commonVariableQuoted).add(equation);
		variablesCommonToEquation.get(commonVariableNoQuote).add(equation);
	}
	
	public int getNumberOfVariables() {
		
		return getVariables().size(); // variableToEquation.size();
	}
	
	public Set<String> getVariablesNoSystem() {
		Set<String> aSet = new LinkedHashSet<String>();
		for (String v :  getVariables()) { //variablesLhsOriginalAsIs /*variablesOriginal*/ ) {
			if (/*!Parser.isSystemVariable(v) && */ !Parser.isNaReplacement(v))
				aSet.add(v);
		}
		return aSet;
	}
	
	public Set<String> getVariablesAsList() {
//		return variablesOriginal;
		return variablesLhsOriginalAsIs;
//		Set<String> originals = new LinkedHashSet<String>();
//		for (String s : variableToEquation.keySet()) {
//			originals.add(commonToOriginal.get(s));
//		}
//		return originals;
	}
	
	public Set<String> getVariablesLeaveQuotes() {
//		return variablesOriginal;
//		return variablesLhsOriginalAsIs;
//		Set<String> originals = new LinkedHashSet<String>(variablesOriginal);
		Set<String> originals = new LinkedHashSet<String>();
		for (String s : variablesOriginal) {
			String v = Parser.quoted(s);
			if (!originals.contains(v))
				originals.add(v);
//			if (s.startsWith("\"") ) {
//				if (!variablesOriginal.contains(Parser.removeQuotes(s))) {
//					originals.add(s);
//				} else {
//					originals.add(Parser.removeQuotes(s));
//				}
//			} else {
//				originals.add(s);
//			}
		}
		return originals;
	}
	
	public Set<String> getVariables() {
//		return variablesOriginal;
//		return variablesLhsOriginalAsIs;
		Set<String> originals = new LinkedHashSet<String>();
		
		// variablesOriginal will contain quoted and unquoted for all variables unless
		// the unquoted version would be an array reference while the quoted is not
		// so if there are quoted and unquoted vars, select the quoted only
		
		for (String s : variablesOriginal) {
			if (s.startsWith("\"") ) {
				if (!variablesOriginal.contains(Parser.removeQuotes(s))) {
					originals.add(s);
				} else {
					originals.add(s);
//					originals.add(Parser.removeQuotes(s));
				}
			} else {
//				originals.add(s);
			}
		}
		return originals;
	}
	
	public Set<String> getVariablesLhsOriginalAsIs() {
//		
		return variablesLhsOriginalAsIs;
	}
	
	public int getNumberOfEquationsFor(String variable) {
		return getAllEquationsFor(variable).size();
	}
	
	public Set<Equation> getAllEquationsFor(String tvariable) {
		String variable = Parser.removeAnnotation(Parser.removeAnySubscripts(tvariable)); 
		// we may be pasing in the common name
		
		if (variablesOriginalToEquation.containsKey(variable))
			return variablesOriginalToEquation.get(variable);
		
		if (variablesCommonToEquation.containsKey(variable))
			return variablesCommonToEquation.get(variable);
		
		return new LinkedHashSet<Equation>();  // variable MJB 6/2/2014
	}
	
	public List<String> getViews(String variableName) {
		String commonName = VariableNameEquivalence.getCommonName(variableName);
		List<String> al = new ArrayList<String>();
		if (variablesCommonToEquation.get(commonName) == null) {
			System.out.println("CommonNameProblem "+variableName);
			commonName = VariableNameEquivalence.getCommonName(Parser.removeQuotes(variableName)); // variableName.replace("\"", ""));
		}
		if (variablesCommonToEquation.get(commonName) == null) {
			System.out.println("Cannot get views for variable: "+variableName);
			return al;
		}
		for (Equation eqn : variablesCommonToEquation.get(commonName)) {
			for (String view : eqn.getViews()) {
				if (!al.contains(view))
					al.add(view);
			}
		}
		
		return al;
	}
	
//	public Set<Equation> getAnEquationfor(String variable) {
//		return variableToEquation.get(variable);
//	}
	
	public void applySelectorToAll(String variableName, VariableSelector selector, MutableInteger numEqn, MutableInteger numSelect) {
		numEqn.setValue(0);
		numSelect.setValue(0);
		String commonName = VariableNameEquivalence.getCommonName(variableName);
		if (variablesCommonToEquation.get(commonName) == null) {
			commonName = VariableNameEquivalence.getCommonName(Parser.removeQuotes(variableName)); // variableName.replace("\"", ""));
		}
		if (variablesCommonToEquation.get(commonName) == null) {
			System.out.println("applySelectorToAll commonName problem for var: "+variableName);
			return;
		}
		for (Equation eqn : variablesCommonToEquation.get(commonName)) {
			numEqn.add(1);
			if (selector.isSelectSingleEquation(eqn)) {
				numSelect.add(1);
			}
		}
	}
	
	public int getTotalEquationCount(String variableName) {
		int count = 0;
		String commonName = VariableNameEquivalence.getCommonName(variableName);
		
		if ( variablesCommonToEquation.get(commonName) == null) {
			System.out.println("getTotalEquationCount: no common name for variable <"+variableName+">");
			return -1;
		}
		for (Equation eqn : variablesCommonToEquation.get(commonName)) {
			count += eqn.getTotalEquationCount();
		}
		
		return count;
	}
	
	public String getOriginalName(String commonOrOriginalName) {
		if (commonToOriginal.containsKey(commonOrOriginalName))
			return commonToOriginal.get(commonOrOriginalName);
		
		return commonOrOriginalName;
		
	}
	
	public String getVariableID(String variableName) {
//		String commonName = VariableNameEquivalence.getCommonName(variableName);
		if (getAnEquationFor(variableName) == null) {
			System.out.println("OOPS");
			return "-1";
		} else {
			int n = getAnEquationFor(variableName).getVariableNumber();
			if (n >= getNumberOfVariables())
				System.out.println("Huh?");
			return Integer.toString(getAnEquationFor(variableName).getVariableNumber());
		}
	}
	
	public Equation asSingleton(Set<Equation> equationSet) {
		if (equationSet.size() != 1)
			return null;
		Iterator<Equation> anEqn = equationSet.iterator();
		return anEqn.next();
	}
	
	public Equation getEquationFor(String screenName) {
		
		String variable = Parser.removeAnnotation(screenName); 
		
		// if the screen name contains subscripts, remove them
		variable = Parser.removeAnySubscripts(variable);
		
		if (variablesLhsOriginalToEquation.containsKey(variable)) {
			return variablesLhsOriginalToEquation.get(variable);
		}
		
		String commonName = VariableNameEquivalence.getCommonName(variable);
		if (variablesLhsCommonToEquation.containsKey(commonName)) {
			return variablesLhsCommonToEquation.get(commonName);
		}
		
		if (variablesOriginalToEquation.containsKey(variable)) {
			return asSingleton(variablesOriginalToEquation.get(variable));
		}
		
		if (variablesCommonToEquation.containsKey(commonName)) {
			return asSingleton(variablesCommonToEquation.get(commonName));
		}
		
		return null;
	}
	
	
	

	public Equation getAnEquationFor(String screenName) {
		
		
		String variable = Parser.removeAnnotation(Parser.removeAnySubscripts(screenName)); 
		String commonName = VariableNameEquivalence.getCommonName(variable);
		
		Set<Equation> eqns = getAllEquationsFor(variable);
		Set<Equation> eqnsCommon = getAllEquationsFor(commonName);
		
		if (eqns == null || eqns.size() == 0) {
			if (eqnsCommon == null || eqnsCommon.size() == 0) {
				return null;
			} else {
				Iterator<Equation> anEqn = eqnsCommon.iterator();
				Equation eqn = anEqn.next();
				return eqn;
			}
		}
		
//		if (eqns.size() == 0)
//			return null;
		
		Iterator<Equation> anEqn = eqns.iterator();
		Equation eqn = anEqn.next();
		return eqn;
	}
	
	public List<String> orderedLHS() {
		List<String> sList = new ArrayList<String>(/*variablesLhsOriginal*/);
		
		for (String s : variablesLhsOriginal) {
			if (s.startsWith("\""))
				sList.add(s);
		}
		
//		for (String s : variablesLhsCommon) {
//			sList.add(commonToOriginal.get(s));
//		}
//		Collections.sort(sList);
		
		Collections.sort(sList, new Comparator<String>() {
			@Override
			public int compare(String o1, String o2) {
				int initialPartition = Utilities.systemVariableCompare(o1, o2);
				if (initialPartition != 0)
					return initialPartition;
				
				return o1.compareToIgnoreCase(o2);

			}
		});
		return sList;
		
//		List<String> eList = new ArrayList<String>();
//		for (String s : sList) {
//			eList.add(getAnEquationFor(s).getLhs());
//		}
//		return eList;
	}
	
	public List<String> dequoted(Set<String> varSet) {
		List<String> dequote = new ArrayList<String>();
		for (String var : varSet) {
			if (var.startsWith("\""))
				dequote.add(var);
		}
		
		return dequote;
	}
	
	public List<String> orderedVariables() {
		List<String> sList = new ArrayList<String>(dequoted(variablesOriginal));
		
		// this is used solely for report generation
		
//		for (String s : variablesCommon) {
//			sList.add(commonToOriginal.get(s));
//		}
//		Collections.sort(sList);
		
		try {
			Collections.sort(sList, new Comparator<String>() {
				@Override
				public int compare(String arg0, String arg1) {
					try {
						// first separate non-control and control
						boolean isSys0 = Parser.isSystemVariable(arg0);
						boolean isSys1 = Parser.isSystemVariable(arg1);
						if (!isSys0 && isSys1) {
							return -1;
						} else if (isSys0 && !isSys1) {
							return 1;
						} else {
							return arg0.compareToIgnoreCase(arg1);
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return sList;
	}
	
	
	public String getVariableIDForName(String name) {
		if (!mapsInitialized())
			initializeMaps();
		String commonName = VariableNameEquivalence.getCommonName(name);
		if (variableToID.containsKey(commonName))
			return variableToID.get(commonName);
		else if (variableToID.containsKey(name))
				return variableToID.get(name);
		else if (variableToID.containsKey(Parser.quoted(name))) // "\""+name+"\""))
			return variableToID.get(name);
		return "NoIDfor "+name;
	}
	
	public String getVariableNameForID(String id) {
		if (!mapsInitialized())
			initializeMaps();
		if (idToVariable.containsKey(id)) {
			return /*commonToOriginal.get(*/ idToVariable.get(id)/* ) */;
		} else {
			
			try {
				int n = Integer.parseInt(id);
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				return id;
			}
			return "NoNamefor "+id;
		}
	}
	
	public String getVariableNameForID(int id) {
		if (!mapsInitialized())
			initializeMaps();
		if (idToVariable.containsKey(asString(id))) {
			return idToVariable.get(asString(id));
//			return commonToOriginal.get(idToVariable.get(asString(id)));
		} else {
			return "NoNamefor "+id;
		}
	}
	
	private boolean mapsInitialized() {
		return variableToID != null;
	}
	
	private String asString(int n) {
		return Integer.toString(n);
	}
	
	private int asInt(String n) {
		return Integer.parseInt(n);
	}
	
	private void initializeMaps() {

		variableToID = new LinkedHashMap<String, String>();
		idToVariable = new LinkedHashMap<String, String>();

		//		List<String> variables = new ArrayList<String>(variableToEquation.keySet());  // get Variables()?
		List<String> variables = new ArrayList<String>(variablesOriginal);
		for (String v : variables) {

			Equation e = getAnEquationFor(v);
			if (e == null) {
				System.out.println("No Equation for "+v);
				System.out.println("BAD");
				continue;
			}

			int vn = e.getVariableNumber();

			// if the nonquoted variable is an array reference while
			// the quoted is not, do not add the unquoted variable.

			String noQuotes = v.replaceAll("^\"", "").replaceAll("\"$", "");
			
			boolean noAddNoQuotes = ArrayReference.isArrayReference(noQuotes) && !ArrayReference.isArrayReference(v);

			if (!v.equals(noQuotes)) {
				variableToID.put(v, asString(vn));
				idToVariable.put(asString(vn), v);
			}
//			if (!noAddNoQuotes) {
				variableToID.put(noQuotes, asString(vn));
				idToVariable.put(asString(vn), noQuotes);
//			} 

			log.debug("var: "+v+" id: "+asString(vn));
			//			System.out.println("var: "+noQuotes+" id: "+asString(vn));
		}
	}
	
	
	public List<Equation> getAllEquationsList() {
		return allEquationsList;
	}


}
