package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class AdjacencyMatrix extends GraphTheoryMatrix {
static Logger log = Logger.getLogger(AdjacencyMatrix.class.getSimpleName());

	Map<String, Integer> varToNumMap = new HashMap<String, Integer>();
	List<String> blockOrdered = new ArrayList<String>();
	List<String> levels = new ArrayList<String>();
	String[][] blockOrderedDiagram;
	int nVars = 0;
	
//	int RADIUS = 500;
	
	Map<Integer, String> columnToName = new HashMap<Integer, String>();
	Map<String, Integer> nameToColumn = new HashMap<String, Integer>();
	
	public AdjacencyMatrix(SDObjectManager objManager, PolarityResults polarityResults) {
		super(objManager, polarityResults);
	}
	
	public AdjacencyMatrix(SDObjectManager objManager) {
		super(objManager, null);
	}
	
	public AdjacencyMatrix() {
		super();
	}
	
	public void build(String tag) {
		
		// protected Map<String, Map<String, String>> matrix;
		
//		for (ArrowPolarity arrow : polarityResults.getArrows()) {
//			addSuccessor(arrow.getTail(), arrow.getHead());
//		}
		
		// complicating issue: in edges, some vars are quoted some are not
		
		List<String[]> edges;
		edges = objManager.getEdges();
		for (String[] edge : edges) {
				addSuccessor(Parser.quoted(edge[0]), Parser.quoted(edge[1]));
		}
		
		for (String v : InformationManagers.getInstance().getEquationManager().getVariablesLeaveQuotes()) {
			getRowSet(v);
			getColSet(v);
		}
		
		if (SDM_Doc.DUMP_TABULAR_DATA) {
			// ############## WRITE_DEBUG_FILES
//			if (StaticSwitches.WRITE_DEBUG_FILES) {
				this.dumpMatrixToFile(/*tag+*/"VariableAdjacencyMatrix(UsedBy).tab", successor);
//				this.dumpMatrixToFile(tag+"_AdjacencyMatrixPredecessor.tab", predecessor);
//			}
		}
		log.debug("##### Adjacency Matrix Build End #####");
	}
	
	public void blockOrder(LevelPartition levelPartition) {
		// within each level, order alphabetically;
		
		int varNum = 0;
		for (int level = 1; level <= levelPartition.getNumLevels(); level++) {
			Set<String> varsInLevel = levelPartition.getVariablesInLevel(level);
			for (String var : Utilities.sortIgnoreCase(varsInLevel)) {
				blockOrdered.add(var);
				levels.add(Integer.toString(level));
				varToNumMap.put(var, varNum);
				varNum++;
			}
		}
		blockOrderedDiagram = new String[blockOrdered.size()+1][blockOrdered.size()+2];
		blockOrderedDiagram[0][0] = "Variable";
		blockOrderedDiagram[0][1] = "";
		int irow = 0;
		for (int icol = 0; icol < blockOrdered.size(); icol++) {
			blockOrderedDiagram[irow][icol+2] = Integer.toString(icol+1);
		}
		for (int row = 0; row < blockOrdered.size(); row++) {
			blockOrderedDiagram[row+1][0] = blockOrdered.get(row);
			blockOrderedDiagram[row+1][1] = "L "+levels.get(row)+" " +Integer.toString(row+1);
		}
		for (int row = 0; row < blockOrdered.size(); row++) {
			for (int col = 0; col < blockOrdered.size(); col++) {
				blockOrderedDiagram[row+1][col+2] = "";
			}
		}
//		Arrays.fill(blockOrderedDiagram, "");
		
		for (String var : blockOrdered) {
			
			int row = varToNumMap.get(var);
			for (String succ : getRowSet(var)) {
				int col = varToNumMap.get(succ);
				blockOrderedDiagram[row+1][col+2] = "*";
			}
		}
		
		dumpBlockOrderedDiagramToFile("BlockOrderedAdjacency.tab", blockOrderedDiagram);
	}
	
	public void dumpBlockOrderedDiagramToFile(String file, String[][] diagram) {
		try {
			BufferedWriter bw = Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
			for (int row = 0; row < diagram.length; row++) {
				StringBuilder aRow = new StringBuilder();
				for (int col = 0; col < diagram[row].length; col++) {
					if (col > 0) {
						aRow.append("\t");
					}
					aRow.append(diagram[row][col]);
				}
				aRow.append("\n");
				bw.append(aRow);
			}
			
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Map<String, Integer> getVarToNumMap() {
		return varToNumMap;
	}

	public List<String> getBlockOrdered() {
		return blockOrdered;
	}
	
	public int loadFromFile(String matrixFile) {
		List<String> contents = Utilities.getAsList(matrixFile);
		
		// first line contains tab separated var names starting in column 2
		
		String[] variables = contents.get(0).split("\t");
		for (int i = 1; i < variables.length; i++) {
			columnToName.put(i, variables[i]);
			nameToColumn.put(variables[i], i);
			successor.put(variables[i], new LinkedHashSet<String>());
			predecessor.put(variables[i], new LinkedHashSet<String>());
			if (!Parser.isSystemVariable(variables[i]))
				nVars++;
		}
		int rowPtr = 1;
		while (rowPtr < contents.size()) {
			String[] adj = contents.get(rowPtr).split("\t");
			for (int i = 1; i < adj.length; i++) {
				if (adj[i].equals("1")) {
				addSuccessor(adj[0], columnToName.get(i));
				}
				
			}
			rowPtr++;
		}
		return nVars;
	}

	public Map<Integer, String> getColumnToName() {
		return columnToName;
	}

	public Map<String, Integer> getNameToColumn() {
		return nameToColumn;
	}
	
	
	

		
}
