package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class AdjacencyMatrixToMap extends AdjacencyMatrix{

private String groupFile;
private Map<String, String> variableGroup = new LinkedHashMap<String, String>();
private Map<String, String> groupShapeColor = new LinkedHashMap<String, String>();
private Map<String, String> groupTextColor = new LinkedHashMap<String, String>();
private Set<String> groups = new LinkedHashSet<String>();


	
	public AdjacencyMatrixToMap(SDObjectManager objManager, PolarityResults polarityResults) {
		super(objManager, polarityResults);
	}
	
	public AdjacencyMatrixToMap(SDObjectManager objManager) {
		super(objManager, null);
	}
	
	public AdjacencyMatrixToMap() {
		super();
	}
	
	public boolean loadGroupFile(List<String> groups, List<String> shapeColors, List<String> textColors) {
		
		return loadGroupFile(groupFile, groups, shapeColors, textColors);
	}
	
	public boolean loadGroupFile(String groupFile, List<String> groupss, List<String> shapeColors, 
			List<String> textColors) {
		List<String> contents = Utilities.getAsList(groupFile);
		groups.addAll(groupss);
		
		int rowPtr = 1;
		while (rowPtr < contents.size()) {
			String[] adj = contents.get(rowPtr).split("\t");
			String var = adj[0];
			String group = adj[1];
			
			variableGroup.put(var, group);
			
			rowPtr++;
		}
		for (int i = 0; i < groups.size(); i++) {
			groupShapeColor.put(groupss.get(i), shapeColors.get(i));
			groupTextColor.put(groupss.get(i), textColors.get(i));
		}
//		groupColor.put("Manage", "128-0-128");
//		groupColor.put("Connect", "0-128-0");
//		groupColor.put("Supply", "255-192-128");
//		groupColor.put("Distribute", "0-0-255");
		
		return true;
	}
	
	public List<String> getVariablesOrdered() {
		List<String> groupedVars = new ArrayList<String>();
		for (String g : Utilities.ordered(getGroups())) {
			groupedVars.addAll(getVariablesInGroup(g));
		}
		
		
		return groupedVars;
	}
	
	public List<String> getVariablesInGroup(String group) {
		List<String> varsInGroup = new ArrayList<String>();
		
		for (String var : variableGroup.keySet()) {
			if (variableGroup.get(var).equals(group))
				varsInGroup.add(var);
		}
		
		Collections.sort(varsInGroup);
		
		return varsInGroup;
	}
	
	public Set<String> getGroups() {
		return groups;
	}
	

	
	public String getGroupForVariable(String variable) {
		return variableGroup.get(variable);
	}
	
	public String getShapeColorForVariable(String variable) {
		return groupShapeColor.get(variableGroup.get(variable));
	}
	
	public String getShapeColorForGroup(int group) {
		return groupShapeColor.get(group);
	}
	
	public String getTextColorForVariable(String variable) {
		return groupTextColor.get(variableGroup.get(variable));
	}
	
	public String getTextColorForGroup(int group) {
		return groupTextColor.get(group);
	}


	public String getGroupFile() {
		return groupFile;
	}

	public void setGroupFile(String groupFile) {
		this.groupFile = groupFile;
	}
	
}
