package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class CyclePartition {
static Logger log = Logger.getLogger(CyclePartition.class.getSimpleName());
	
	private int level;
	private Map<String, Set<String>> partition;
	private LevelPartition levelPartition;
	private ReachabilityMatrix reachabilityMatrix;
	
	public CyclePartition(int level, LevelPartition levelPartition, ReachabilityMatrix reachabilityMatrix) {
		this.level = level;
		this.levelPartition = levelPartition;
		this.reachabilityMatrix = reachabilityMatrix;
		partition = new LinkedHashMap<String, Set<String>>();
	}
	
	public void build() {
		Set<String> vars = levelPartition.getVariablesInLevel(level);
		for (String s : vars) {

			Set<String> successor = reachabilityMatrix.getSuccessor(s);
			Set<String> predecessor = reachabilityMatrix.getPredecessor(s);

			String name = getName(successor,predecessor);
			if (!partition.containsKey(name))
				partition.put(name, new LinkedHashSet<String>());
			partition.get(name).add(s);			
		}
	}
	
	public void printCyclePartitions() {
		System.out.println("### Cycle Partitions - Level "+level);
		
		for (String s : partition.keySet()) {
			List<String> al = new ArrayList<String>(partition.get(s));
			
			System.out.println("level: "+level+" size = "+al.size()+" key: "+s);
			
			if (al.size() < 2)
				continue;
			Collections.sort(al);
			System.out.println("\n>>> "+s+" size "+al.size());
			for (String v : al) {
				System.out.println("   "+v);
			}
		}
	}
	
	private String getName(Set<String> aSet1, Set<String> aSet2) {
		
		StringBuilder sb = new StringBuilder();
		List<String> al = new ArrayList<String>(aSet1);
		Collections.sort(al);
		for (String s : al)
			sb.append("#"+s);
		sb.append("__");
		al = new ArrayList<String>(aSet2);
		Collections.sort(al);
		for (String s : al)
			sb.append("#"+s);
		
		return sb.toString();
	}
	
	public Set<Set<String>> getCyclePartitions() {
		Set<Set<String>> partitions = new LinkedHashSet<Set<String>>();
		for (Set<String> s : partition.values())
			if (s.size() > 1)
				partitions.add(s);
		return partitions;
	}
	
	public boolean hasCyclePartitions() {
		for (Set<String> s : partition.values())
			if (s.size() > 1)
				return true;
		return false;
	}

	public ReachabilityMatrix getReachabilityMatrix() {
		return reachabilityMatrix;
	}

}
