package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

public class ExogenousVariableManager {
	
	ExogenousVariables exogenousVariables;
	
	public ExogenousVariableManager(ExogenousVariables exogenousVariables) {
		this.exogenousVariables = exogenousVariables;
	}
	
	public ExogenousVariableRow getExogenousVariableRow(String screenName) {
		return exogenousVariables.getExogenousVariableRow(Parser.quoted(screenName));
	}
	
	public int getNumberOfExogenousVariables() {
		return exogenousVariables.getNumberOfExogenousVariables();
	}
	
	public int getNumberOfEndogenousVariables() {
		return exogenousVariables.getNumberOfEndogenousVariables();
	}
	
	public static boolean isIgnoreVar(String var) {
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
			if (ignore(eqn))
				return true;
		}
		return false;
	}
	public static boolean isExogenousVar(String var) {
		boolean isExo = true;
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
			if (!eqn.isExogenous())
				isExo = false;
		}
		return isExo;
	}
	public static boolean isEndogenousVar(String var) {
		boolean isEndo = false;
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
			if (!eqn.isExogenous())
				isEndo = true;;
		}
		return isEndo;
	}
	
	public static void setAllEqnExogenous(String var) {
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
			eqn.setExogenous(true);
		}
	}
	
	public static void setAllEqnEndogenous(String var) {
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
			eqn.setExogenous(false);
		}
	}
	
	public static boolean ignore(Equation equation) {

		if (equation.isDefinesSubscript())
			return true;
		if (/* equation.isControlGroup() || */ isNAorTime(equation.getLhsAsInMdl()))
			return true;
		return false;
	}
	
	public static boolean isNAorTime(String s) {
		if (s == null)
			return false;
		return s.toUpperCase().equals("TIME") || s.toUpperCase().equals("NAREPLACEMENT");
	}

}
