package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class ExogenousVariableRow {
	
	private static final String NO_VALUE = "NA";
	
	String name;
	
	int numDirectInputs;
	int numDirectExogenous;
	int numIndirectExogenous;
	int indirectMin = Integer.MAX_VALUE;
	int indirectMax = Integer.MIN_VALUE;
	
	String referenceNoPath;
	public String getReferenceNoPathNoHTML() {
		return referenceNoPathNoHTML;
	}

	public String getReferencePathNoHTML() {
		return referencePathNoHTML;
	}

	String referencePath;
	
	String referenceNoPathNoHTML;
	String referencePathNoHTML;
	
	Set<String> noPath;
	Set<String> path;

	//	float indirectMedian;
//	float indirectMean;
	List<ExoDistance> distances = new ArrayList<>();
	
	boolean exogenous = false;
	
	public ExogenousVariableRow(String name) {
		this.name = name;
//		reference = Anchors.getFileAnchorReference("../Unconnected.html", "List");
	}
	
	public void reset() {
		numDirectInputs = 0;
		numDirectExogenous = 0;
		numIndirectExogenous = 0;
		indirectMin = Integer.MAX_VALUE;
		indirectMax = Integer.MIN_VALUE;

		referenceNoPath = null;
		referencePath = null;

		if (noPath != null)
			noPath.clear();
		if (path != null)
			path.clear();

		distances = new ArrayList<>();
	}
	
	public boolean canChangeFromEndoToExo() {
		return numDirectInputs > 0 && numDirectExogenous == numDirectInputs;
	}
	
	public void incrementNumDirectInputs() {
		numDirectInputs++;
	}
	
	public void incrementDirectExogenous() {
		numDirectExogenous++;
	}
	
	public void incrementNumIndirectInputs() {
		numIndirectExogenous++;
	}
	
	public void addDistance(String var, int dist, boolean varIsExogenous) {
		// only for indirect 
		ExoDistance d = new ExoDistance(var, dist, varIsExogenous);
		distances.add(d);
		if (dist == 1)
			return;
		if (dist < indirectMin)
			indirectMin = dist;
		if (dist > indirectMax)
			indirectMax = dist;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int getNumDirectInputs() {
		return numDirectInputs;
	}

	public void setNumDirectInputs(int numDirectInputs) {
		this.numDirectInputs = numDirectInputs;
	}

	public int getNumDirectExogenous() {
		return numDirectExogenous;
	}

	public void setNumDirectExogenous(int numDirectExogenous) {
		this.numDirectExogenous = numDirectExogenous;
	}

	public int getNumIndirectExogenous() {
		return numIndirectExogenous;
	}

	public void setNumIndirectExogenous(int numIndirectExogenous) {
		this.numIndirectExogenous = numIndirectExogenous;
	}

	public int getIndirectMin() {
		
		List<ExoDistance> tmp = getExoDistanceIndirect();
		
		if (tmp.size() == 0)
			return -99;
		return indirectMin  == Integer.MAX_VALUE ? -99 : indirectMin;
	}

	public void setIndirectMin(int indirectMin) {
		this.indirectMin = indirectMin;
	}

	public int getIndirectMax() {
		List<ExoDistance> tmp = getExoDistanceIndirect();
		
		if (tmp.size() == 0)
			return -99;
		return indirectMax == Integer.MIN_VALUE ? -99 : indirectMax;
	}

	public void setIndirectMax(int indirectMax) {
		this.indirectMax = indirectMax;
	}

	public float getIndirectMedian() {
//		Collections.  (distances);
		
		List<ExoDistance> tmp = getExoDistanceIndirect();
		
		if (tmp.size() == 0)
			return -99.9f;
		if (tmp.size() % 2 == 1) {
			return tmp.get(tmp.size()/2).getDistance();
		} else {
			int i = tmp.size()/2 - 1;
			return (tmp.get(i).getDistance() + tmp.get(i+1).getDistance()) / 2;
		}
	}
	
	public List<ExoDistance> getExoDistanceIndirect() {
		List<ExoDistance> tmp = new ArrayList<>();
		for (int i = 0; i < distances.size(); i++) {
			if (distances.get(i).getDistance() > 1.0f &&
					distances.get(i).isVarIsExogenous())
				tmp.add(distances.get(i));
		}
		Collections.sort(tmp);
		return tmp;
	}
	public List<ExoDistance> getExoDistanceAll() {
		List<ExoDistance> tmp = new ArrayList<>();
		for (int i = 0; i < distances.size(); i++) {
			if (distances.get(i).isVarIsExogenous())
				tmp.add(distances.get(i));
		}
		Collections.sort(tmp);
		return tmp;
	}


	public float getIndirectMean() {
		
		List<ExoDistance> tmp = getExoDistanceIndirect();
		
		float sum = 0.0f;
		if (tmp.size() == 0)
			return -99.9f;

			int count = tmp.size();
//			for (ExoDistance exo : tmp) {
//				if (!exo.isVarIsExogenous())
//					count++;
//			}
			if (count == 0)
				return -99.9f;
			for (ExoDistance exo : tmp) {
//				if (exo.isVarIsExogenous())
//					continue;
				sum += exo.getDistance();
			}
			return sum / count;
	}
	
	public float getDirectPercentage() {
		if (numDirectInputs == 0)
			return -99.9f;
		return (float) numDirectExogenous / (float) numDirectInputs;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		sb.append(name); sb.append(",");
		sb.append(numDirectInputs); sb.append(",");
		sb.append(numDirectExogenous); sb.append(",");
		sb.append(getDirectPercentage()); sb.append(",");
		sb.append(numIndirectExogenous); sb.append(",");
		sb.append(indirectMin); sb.append(",");
		sb.append(getIndirectMean()); sb.append(",");
		sb.append(getIndirectMedian()); sb.append(",");
		sb.append(indirectMax);
		// 2147483647,NA,NA,-2147483648
		return sb.toString().replace("-99.9", "NA").replace("-99", "NA").replace("2147483647", "NA").replace("-2147483648", "NA");
	}
	
	public boolean isExogenousData() {
		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(name);
		if (eqn == null)
			return false;
		return eqn.isData(); 
	}
	
	public String getNameForHTML() {
		return name;
	}
	public String getNumDirectForHTML() {
		return NumberFormatter.formatForCountNotConditional(numDirectInputs);
	}
	public String getNumDirectExogenousForHTML() {
		if (numDirectInputs == 0)
			return NO_VALUE;
		return NumberFormatter.formatForCountNotConditional(numDirectExogenous);
	}
	public String getExogenousPercentForHTML() {
		if (getDirectPercentage() < 0.0f)
			return NO_VALUE;
		return NumberFormatter.formatForPercent(getDirectPercentage()*100.0);
	}
	public String getNumIndirectForHTML() {
		if (numDirectInputs == 0)
			return NO_VALUE;
		return NumberFormatter.formatForCountNotConditional(numIndirectExogenous);
	}
	public String getIndirectMinLengthForHTML() {
		int min = getIndirectMin();
		if (min < 0)
			return NO_VALUE;
		return NumberFormatter.formatForCountNotConditional(indirectMin);
	}
	public String getIndirectMeanLengthForHTML() {
		float mean = getIndirectMean();
		if (mean < 0)
			return NO_VALUE;
		return NumberFormatter.formatForRatio(getIndirectMean());
	}
	public String getIndirectMedianLengthForHTML() {
		float median = getIndirectMedian();
		if (median < 0)
			return NO_VALUE;
		return NumberFormatter.formatForRatio(getIndirectMedian());
	}
	public String getIndirectMaxLengthForHTML() {
		int max = getIndirectMax();
		if (max < 0)
			return NO_VALUE;
		return NumberFormatter.formatForCountNotConditional(indirectMax);
	}
	
	public String getVariableType() {
		if (isExogenous() /* || isExogenousData() */)
			return "Exogenous";
		else
			return "Endogenous";
	}
	
	public boolean isExogenous() {
//		return numDirectInputs == 0;
		return exogenous;
	}
	
	public boolean isEndogenous() {
		return !isExogenous();
	}

	public void setExogenous(boolean exogenous) {
		this.exogenous = exogenous;
	}
	
	public boolean hasNoPath() {
		return noPath != null;
	}
	
	public boolean hasPath() {
		return path != null;
	}
	
	public void setNoPath(Set<String> noPath) {
		this.noPath = noPath;
	}
	
	public void setPath(Set<String> path) {
		this.path = path;
	}
	
	public String getAnchorName() {
		return /*"EVR"+*/ name;
	}
	
	public String getAnchorReference() {
		return "#" + getAnchorName();
	}
	
	public String getNoPathHtml() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		StringBuilder sb = new StringBuilder();
		
		sb.append(HTMLConstants.TABLE_START.replace(HTMLConstants.LINK_NAME, name.replace(" ", ""))
				.replace("$$$TOP$$$", language.getTextHtml(StringConstants.TOP))
				.replace(HTMLConstants.TABLE_NAME, Parser.removeQuotes(name)));
		sb.append(HTMLConstants.ROW_START);
		sb.append(HTMLConstants.COLUMN_HEADER.replace(HTMLConstants.COLUMN_NAME, 
				language.getTextHtml(StringConstants.EXOGENOUS_UNCONNECTED)));
		sb.append(HTMLConstants.ROW_END);
			
			List<String> nop = Utilities.ordered(noPath);
			
			if (nop.size() == 0) {
				sb.append(HTMLConstants.ROW_START);
				sb.append(HTMLConstants.COLUMN_START);
				sb.append(language.getTextHtml(StringConstants.NONE));
				sb.append(HTMLConstants.COLUMN_END);
				sb.append(HTMLConstants.ROW_END+"\n");
			} else {
				for (int i = 0; i < nop.size(); i++) {  // -1
					sb.append(HTMLConstants.ROW_START);
					sb.append(HTMLConstants.COLUMN_START);
					sb.append(Parser.removeQuotes(nop.get(i)));
					sb.append(HTMLConstants.COLUMN_END);
					sb.append(HTMLConstants.ROW_END+"\n");
				}
			}
			sb.append(HTMLConstants.TABLE_END);
			return sb.toString();
		
	}
	
	public String getPathHtml() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		StringBuilder sb = new StringBuilder();
		
		sb.append(HTMLConstants.TABLE_START.replace(HTMLConstants.LINK_NAME, Anchors.createAnchorName(name))
				.replace("$$$TOP$$$", language.getTextHtml(StringConstants.TOP))
				.replace(HTMLConstants.TABLE_NAME, Parser.removeQuotes(name)));
		sb.append(HTMLConstants.ROW_START);
	
		
		sb.append(HTMLConstants.COLUMN_HEADER.replace(HTMLConstants.COLUMN_NAME, 
				language.getTextHtml(StringConstants.EXOGENOUS_CONNECTED)));
		sb.append(HTMLConstants.COLUMN_HEADER.replace(HTMLConstants.COLUMN_NAME, 
				language.getTextHtml(StringConstants.DISTANCE)));
		sb.append(HTMLConstants.ROW_END);
			
			List<ExoDistance> hasp = getExoDistanceAll();
			
			if (distances.size() == 0) {
				sb.append(HTMLConstants.ROW_START);
				sb.append(HTMLConstants.COLUMN_START);
				sb.append(language.getTextHtml(StringConstants.NONE));
				sb.append(HTMLConstants.COLUMN_END);
				sb.append(HTMLConstants.COLUMN_START);
				sb.append(language.getTextHtml(" "));
				sb.append(HTMLConstants.COLUMN_END);
				sb.append(HTMLConstants.ROW_END+"\n");
			} else {
				for (int i = 0; i < hasp.size(); i++) { // -1
					sb.append(HTMLConstants.ROW_START);
					sb.append(HTMLConstants.COLUMN_START);
					sb.append(Parser.removeQuotes(hasp.get(i).getVar()));
					sb.append(HTMLConstants.COLUMN_END);
					sb.append(HTMLConstants.COLUMN_START);
					sb.append((int) hasp.get(i).getDistance());
					sb.append(HTMLConstants.COLUMN_END);
					sb.append(HTMLConstants.ROW_END+"\n");
				}
			}
			sb.append(HTMLConstants.TABLE_END);
			return sb.toString();
		
	}
	
	public Set<String> getPathToExo() {
		List<ExoDistance> hasp = getExoDistanceAll();
		Set<String> path = new LinkedHashSet<String>();
		for (int i = 0; i < hasp.size(); i++) 
			path.add(hasp.get(i).getVar());
		
		return path;
	}

	public String getReferenceNoPath() {
		if (noPath == null)
			return "0";
		return referenceNoPath;
	}

	public void setReferenceNoPath(String file, int num) {
		if (num > 0) {
			referenceNoPath = Anchors.getFileAnchorReference(file, name, Integer.toString(num));
			referenceNoPathNoHTML = Integer.toString(num);
		} else {
			referenceNoPath = "0";
			referenceNoPathNoHTML = referenceNoPath;
		}
	}
	
	public String getReferencePath() {
		if (path == null)
			return "0";
		return referencePath;
	}

	public void setReferencePath(String file, int num) {
		if (num > 0) {
			referencePath = Anchors.getFileAnchorReference(file, name, Integer.toString(num));
			referencePathNoHTML = Integer.toString(num);
		} else {
			referencePath = "0";
			referencePathNoHTML = referencePath;
		}
	}


}
