package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.DocumentHeader;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ExogenousVariables {
	
	
	ReachabilityMatrix reach;
	Map<String, ExogenousVariableRow> evrMap = new HashMap<>();
	Set<String> varsToIgnore = new LinkedHashSet<String>();
	Set<String> allVars = new LinkedHashSet<String>();
	
	Integer numExo = null;
	Integer numEndo = null;
	

	
	public ExogenousVariables(ReachabilityMatrix reach) {
		this.reach = reach;
	}
	
	public void process(MessageProcessor mf) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<String> rowNames = Utilities.ordered(reach.keySet());
		List<String> columnNames = new ArrayList<>(rowNames);
		
		for (String col : columnNames) {
			allVars.add(col);
			if (ignore(col)) {
				varsToIgnore.add(col);
			}
		}
		
		mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_PATH));
		for (String col : columnNames) {			
			boolean exogenousVar = ExogenousVariableManager.isExogenousVar(col);

			ExogenousVariableRow evr = new ExogenousVariableRow(col);

			evrMap.put(col, evr);
			evr.setExogenous(reach.getColSet(col).size() == 0 || exogenousVar);
		}

		mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_DISTANCE));
		boolean done = false;
		int loopCount = -1;
		
		mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_REFERENCE));
		
		while (!done) {
			System.out.println("#########################################################################");
			loopCount++;
			if (loopCount > 10)
				break;
			clearEvrCounts();
			for (String col : columnNames) {				
				ExogenousVariableRow evr = evrMap.get(col);
				
				if (evr != null) {
					for (String row : rowNames) {
						if (varsToIgnore.contains(row))
							continue;
						if (row.equals(col))
							continue;
						int dist = reach.getDistance(row, col);
						if (dist >= 0) {
							if (dist == 0) {
								System.out.println("How 0");
							}
							if (dist == 1) {

								evr.addDistance(row, dist, evrMap.get(row).isExogenous());

								evr.incrementNumDirectInputs();

								if (evrMap.get(row).isExogenous()) {
									evr.incrementDirectExogenous();
								}
							} else {
								evr.addDistance(row, dist, evrMap.get(row).isExogenous());
								if (evrMap.get(row).isExogenous()) {
									evr.incrementNumIndirectInputs();
								}
							}
						}
					}
				}
			}
			
			int currentNumberOfExo = getNumberOfExogenousVariables();
			
//			System.out.println("##### Exogenous Variables Exogenous Variables #####");
//			Utilities.printToConsole(getExogenousVariables());
//			System.out.println("##### Exogenous Variables Exogenous Variables #####");
			
			for (String col : columnNames) {
				boolean ignoreVar = ExogenousVariableManager.isIgnoreVar(col);

				ExogenousVariableRow evr = evrMap.get(col);

				if (evr.isEndogenous() && !ignoreVar) {
					Set<String> noPath = new LinkedHashSet<String>(getExogenousVariables());
					noPath.removeAll(varsToIgnore);
					Set<String> hasPath = null;
					if (reach.getHasPath().get(col) != null) {
						hasPath = new LinkedHashSet<String>(reach.getHasPath().get(col));
					}

					noPath.removeAll(evr.getPathToExo());


					evr.setNoPath(noPath);
					evr.setPath(hasPath);

//					System.out.println("<"+col+"> "+" Exo/Path/NoPath "+currentNumberOfExo+" / "+(noPath != null ? noPath.size() : 0)+" / "+(hasPath != null ? hasPath.size() : 0));

				}
			}
//			mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_REFERENCE));
			for (String col : columnNames) {
				ExogenousVariableRow evr = evrMap.get(col);
				if (evr != null) {
					evr.setReferenceNoPath(InformationManagers.getInstance()
							.getFileOrganizer().getRelativeLoopsDirectory()+"/"+getHtmlFileNoPath(),
							currentNumberOfExo - (evr.getNumDirectExogenous()+
									evr.getNumIndirectExogenous()));

					evr.setReferencePath(InformationManagers.getInstance()
							.getFileOrganizer().getRelativeLoopsDirectory()+"/"+getHtmlFilePath(),
							(evr.getNumDirectExogenous()+
									evr.getNumIndirectExogenous()));
				}
			}
			done = areWeDone();
		}
		
		// at this point, make sure that all equations are assigned the appropriate exo/endo designation
		
		Set<String> exoVars = getExogenousVariables();
		for (String var : allVars) {
			if (exoVars.contains(var))
				ExogenousVariableManager.setAllEqnExogenous(var);
			else
				ExogenousVariableManager.setAllEqnEndogenous(var);
		}
		
		mf.addMessage(language.getText(StringConstants.EXOGENOUS_ENDOGENOUS_TO_FILE));
		writeNoPathToHtmlFile(getHtmlFileNoPath(), language.getText(StringConstants.ENDOGENOUS_EXOGENOUS_UNCONNECTED_VARIABLES));
		writePathToHtmlFile(getHtmlFilePath(), language.getText(StringConstants.ENDOGENOUS_EXOGENOUS_CONNECTED_VARIABLES));
		mf.endTask();
	}
	
	public boolean areWeDone() {

		boolean done = true;

		for (ExogenousVariableRow evr : evrMap.values()) {
			if (evr.isEndogenous() && evr.canChangeFromEndoToExo()) {
				evr.setExogenous(true);
				ExogenousVariableManager.setAllEqnExogenous(evr.getName());
				done = false;
//				System.out.println("NOT DONE: elevating "+evr);
			}
		}
		return done;
	}
	
	public void clearEvrCounts() {
		for (ExogenousVariableRow evr : evrMap.values()) {
			evr.reset();
		}
	}
	
	public String getHtmlFileNoPath() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		return 
				// mm.getModelName() + 
				"_Unconnected"
				+ "_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
	}
	
	public String getHtmlFilePath() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		return 
//				mm.getModelName() + 
				"_Connected"
				+ "_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
	}
	
	public ExogenousVariableRow getExogenousVariableRow(String screenName) {
		return evrMap.get(screenName);
	}
	

	
	private boolean ignore(String s) {
//		return varsToIgnore.contains(s);
		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(s);
		if (eqn == null)
			return true;
		if (Parser.isWithLookup(eqn.getLhs()))
				return true;
		if (eqn.isDefinesSubscript())
			return true;
//		if (eqn.isControlGroup())
//			return true;
		return s.toUpperCase().equals("TIME") || s.toUpperCase().equals("NAREPLACEMENT");
	}
	
	public int getNumberOfExogenousVariables() {
		
		int count = 0;
		for (String col : evrMap.keySet()) {
			if (varsToIgnore.contains(col)  || ignore(col))
				continue;
			if (evrMap.get(col).isExogenous())
				count++;
		}
		numExo = new Integer(count);
		return count;
	}
	
	public Set<String> getExogenousVariables() {
		Set<String> exogenousVariables = new LinkedHashSet<String>();
		for (String col : evrMap.keySet()) {
			if (varsToIgnore.contains(col) || ignore(col))
				continue;
			if (evrMap.get(col).isExogenous())
				exogenousVariables.add(evrMap.get(col).getName());
		}
		return exogenousVariables;
	}
	
	public int getNumberOfEndogenousVariables() {
//		if (numEndo != null)
//			return numEndo.intValue();
		
		int count = 0;
		for (String col : evrMap.keySet()) {
			if (varsToIgnore.contains(col))
				continue;
			if (evrMap.get(col).isEndogenous())
				count++;
		}
		numEndo = new Integer(count);
		return count;
	}
	
	public boolean isExogenousData(String s) {
		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(s);
		if (eqn == null)
			return false;
		return eqn.isData(); 
	}
	
	public void writeNoPathToHtmlFile(String file, String title) {
		writeNoPathToHtmlFile(
				Utilities.openFileForWritingCharset(InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory()+"/"+ file,
						SDMDocLanguageSpecific.getInstance().getFileCharSet()),
						title);
	}
	
	public void writePathToHtmlFile(String file, String title) {
		writePathToHtmlFile(
				Utilities.openFileForWritingCharset(InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory()+"/"+ file,
						SDMDocLanguageSpecific.getInstance().getFileCharSet()),
						title);
	}

	public void writeNoPathToHtmlFile(BufferedWriter bw, String title) {

		writeNoPathHtmlHeader(bw, title);
		try {
			bw.append(DocumentHeader.STYLE);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		List<String> keys = Utilities.ordered(evrMap.keySet());
		
		for (String var : keys) {
			writeNoPathHtml(bw, evrMap.get(var));
		}
		writeNoPathHtmlFooter(bw);
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writePathToHtmlFile(BufferedWriter bw, String title) {

		writePathHtmlHeader(bw, title);
		try {
			bw.append(DocumentHeader.STYLE);
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		List<String> keys = Utilities.ordered(evrMap.keySet());
		
		for (String var : keys) {
			writePathHtml(bw, evrMap.get(var));
		}
		writePathHtmlFooter(bw);
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getHtmlTail() {
		return "\n</html>\n";
	}

	public String getHtmlHead(String title) {
		StringBuilder sb = new StringBuilder();
		sb.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
				+ "<html>"
				+ "<head>"
				+ "<meta http-equiv='Content-Type' content='Type=text/html; charset="+
				SDMDocLanguageSpecific.getInstance().getHtmlCharSet()+"'>"
				+ "<title>" + title + "</title>" + "</head>\n");

		return sb.toString();
	}

	public void writeNoPathHtmlHeader(BufferedWriter bw, String title) {
		try {
			bw.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
					+ SDMDocLanguageSpecific.getInstance().getHtmlHeader()
					+ "\n<head>"
					+ "\n<meta http-equiv='Content-Type' content='Type=text/html; charset="+
					SDMDocLanguageSpecific.getInstance().getHtmlCharSet()+"'>"
					+ "\n<title>" + title + "</title>" + "</head>"+
					"\n<h1>"+title+"</h1>"+"\n<h2>"+
					SDMDocLanguageSpecific.getInstance().getTextHtml(
							StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+": "+
					InformationManagers.getInstance().getUserSelectedOptions().
					getOptionValue(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+"</h2>");

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public void writePathHtmlHeader(BufferedWriter bw, String title) {
		try {
			bw.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
					+ SDMDocLanguageSpecific.getInstance().getHtmlHeader()
					+ "\n<head>"
					+ "\n<meta http-equiv='Content-Type' content='Type=text/html; charset="+
					SDMDocLanguageSpecific.getInstance().getHtmlCharSet()+"'>"
					+ "\n<title>" + title + "</title>" + "</head>"+
					"\n<h1>"+title+"</h1>"+"\n<h2>"+
					SDMDocLanguageSpecific.getInstance().getTextHtml(
							StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+": "+
					InformationManagers.getInstance().getUserSelectedOptions().
					getOptionValue(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+"</h2>");

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void writeNoPathHtmlFooter(BufferedWriter bw) {
		try {
			bw.append("</html>\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writePathHtmlFooter(BufferedWriter bw) {
		try {
			bw.append("</html>\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void writeNoPathHtml(BufferedWriter bw, ExogenousVariableRow evr) {
		try {
			if (evr.hasNoPath())
				bw.append(evr.getNoPathHtml());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}	
	
	public void writePathHtml(BufferedWriter bw, ExogenousVariableRow evr) {
		try {
			if (evr.hasPath())
				bw.append(evr.getPathHtml());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
