package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class GeodeticCycle {
static Logger log = Logger.getLogger(GeodeticCycle.class.getSimpleName());

	private Map<String, Map<String, Integer>> D;
	private Map<String, Map<String, Integer>> L;
	
	private Set<Loop> geodeticCycles = new LinkedHashSet<Loop>();
	private Map<Pair, List<Loop>> geodeticCyclesUsingLoops = new LinkedHashMap<Pair, List<Loop>>();
	
	private Set<Loop> geo = new LinkedHashSet<Loop>();
	
	private List<String> currentKeySet;
	private LoopManager loopManager;
	
	private boolean unique = true;
	
	public GeodeticCycle(LoopManager loopManager, List<String> currentKeySet, Map<String, Map<String, Integer>> distance) {
		this.currentKeySet = currentKeySet;
		this.D = distance;
		this.loopManager = loopManager;
		geodeticCyclesUsingLoops = new LinkedHashMap<Pair, List<Loop>>();
		L = computeL();
		identifyGeodeticCycles();
		identifyGeodeticCyclesUsingLoops();
	}
	
	public List<Loop> nonCyclePartitionLoops(List<Loop> allLoops) {
		List<Loop> throwOut = new ArrayList<Loop>();
		for (Loop l : allLoops) {
			for (String id : l.getLoop()) {
				if (!currentKeySet.contains(Utilities.idToName(id))) {
					throwOut.add(l);
					break;
				}
			}
		}

		return throwOut;
		
	}
	
	public Set<Loop> identifyGeodeticCyclesUsingLoops() {
		
		// need to make sure that all elements of loop are in the cycle partition
		// are there some that aren't?

		for (String keyr : currentKeySet) {
			for (String keyc : currentKeySet) {
				if (keyr.equals(keyc))
					continue;
				Pair p = PairFactory.getPair(keyr, keyc);
				
				if (!geodeticCyclesUsingLoops.containsKey(p)) {
					geodeticCyclesUsingLoops.put(p, new ArrayList<Loop>());
					List<Loop> al = geodeticCyclesUsingLoops.get(p);
					List<Loop> allLoops = loopManager.getLoopsOrderedBySizeContaining(p);
					List<Loop> throwOut = nonCyclePartitionLoops(allLoops);
//					if (throwOut.size() > 0)
//						System.out.println("ThrowOut");
					if (allLoops.size() == 0) {
//						System.out.println("Not Loop Contain Pair: "+p);
					} else {
						int minSize = allLoops.get(0).getSize();
						for (Loop loop : allLoops) {
							if (loop.getSize() == minSize)
								if (!al.contains(loop))
									al.add(loop);
						}
					}
				}
			}
		}
		// collapse into a set. Set unique value.

		for (Pair p : geodeticCyclesUsingLoops.keySet()) {
			List<Loop> al = geodeticCyclesUsingLoops.get(p);
			if (al.size() == 0) {
				log.debug("No Loop: "+p);
				continue;
			} else if (al.size() > 1) {
				unique = false;
			}
			geo.add(al.get(0));
		}
		return geo;

	}
	
	
	private void identifyGeodeticCycles() {
		int maxLength = findMax(L);
		
		for (int currentLen = 2; currentLen <= maxLength; currentLen++) {
			Set<Pair> pairs = new LinkedHashSet<Pair>();
			
			// all vars are reachable from each other (cycle partition condition
			// check all combinations skipping
			for (String keyr : currentKeySet) {
				for (String keyc : currentKeySet) {
					if (keyr.equals(keyc))
						continue;

					if (L.get(keyr).get(keyc) == currentLen)
						pairs.add(PairFactory.getPair(keyr, keyc));
				}
			}
			
			List<Pair> pairsInLoop;
			List<Pair> pairsInPairs = new ArrayList<Pair>(pairs);
			for (Pair pair : pairsInPairs) {
				Set<Loop> loops = loopManager.getLoopsOfLengthContaining(currentLen, pair);
				pairsInLoop = loopTrack(pair.getV1(), pair.getV2());
				pairs.removeAll(pairsInLoop);
				
				if (isNonRepeated(pairsInLoop) ) {
					Loop candidate = asLoop(pairsInLoop);
					if (!isAlreadyAdded(candidate))
						geodeticCycles.add(candidate);
				} else {
					System.out.println("NOT LEGAL LOOP ");
				}
			}
		}
	}
	
	private boolean isAlreadyAdded(Loop loop) {
		loop.setLoopStartVariable();
		Iterator<Loop> lIter = geodeticCycles.iterator();
		while (lIter.hasNext()) {
			Loop temp = new Loop(lIter.next());
			temp.setLoopStartVariable();
			if (loop.isEquivalentTo(temp))
				return true;
		}
		return false;
		
	}
	
	private Loop asLoop(List<Pair> pairs) {
		List<String> loop = new ArrayList<String>();
		int p = 0;
		for (Pair pair : pairs) {
			if (p++ == 0)
				loop.add(pair.getV1());
			loop.add(pair.getV2());
		}
		// complete the loop
		loop.add(pairs.get(0).getV1());

		return new Loop(loop);
	}
	
	private boolean isNonRepeated(List<Pair> pairsInLoop) {
		Set<String> visited = new LinkedHashSet<String>();
		for (Pair pair : pairsInLoop) {
			String node = pair.getV1();
			if (visited.contains(node))
				return false;
			visited.add(node);
		}
		return true;
	}
	
	
	private Set<String> getSet(Map<String, Set<String>> pairs, String key) {
		if (!pairs.containsKey(key))
			pairs.put(key, new LinkedHashSet<String>());
		return pairs.get(key);
	}
	
	private Map<String, Map<String, Integer>> computeL() {
		Map<String, Map<String, Integer>> l = new LinkedHashMap<String, Map<String, Integer>>();
		for (String keyr : currentKeySet) {
			for (String keyc : currentKeySet) {
				if (D.get(keyr) == null ||
						D.get(keyr).get(keyc) == null || 
						D.get(keyc) == null ||
						D.get(keyc).get(keyr) == null) {
					System.out.println("NO D values for: <"+keyr+"> <"+keyc+">");
					setValue(l, keyr, keyc,Integer.MIN_VALUE);
				} else {
//					System.out.println("YES Have D values for: <"+keyr+"> <"+keyc+">");
					setValue(l, keyr, keyc, D.get(keyr).get(keyc).intValue() +  D.get(keyc).get(keyr).intValue());
				}
			}
		}
		if (SDM_Doc.DUMP_TABULAR_DATA) 
		dumpToDistanceFile("Length.tab", l, ordered(currentKeySet));
		
		return l;
	}
	
	private int findMax(Map<String, Map<String, Integer>> l) {
		int max = Integer.MIN_VALUE;
		for (String keyr : currentKeySet) {
			for (String keyc : currentKeySet) {
				int v = l.get(keyr).get(keyc);
				max = Math.max(v, max);
			}
		}
		
		return max;
	}
	
	private void setValue(Map<String, Map<String, Integer>> l, String row, String col, int value) {
		if (!l.containsKey(row))
			l.put(row, new LinkedHashMap<String, Integer>());
		l.get(row).put(col, value);
	}
	

	private List<Pair> loopTrack(String u, String v) {
		List<String> loop = new ArrayList<String>();
		
		// u -> v
		loop.add(u);
		int duv = D.get(u).get(v);
		for (int i = 1; i <= duv-1; i++) {
			Set<String> rh = new LinkedHashSet<String>();
			Set<String> lh = new LinkedHashSet<String>();
			List<Integer> predValue = dPredecessorValue(v);
			List<Integer> succValue = dSuccessorValue(loop.get(loop.size()-1));
			for (int n = 0; n < currentKeySet.size(); n++) {
				if (predValue.get(n) == duv-1)
					rh.add(currentKeySet.get(n));
				if (succValue.get(n) == 1)
					lh.add(currentKeySet.get(n));
			}
			for (String vertex : rh) {
				if (lh.contains(vertex)) {
					loop.add(vertex);
					break;
				}
			}
		}
		
		// v -> u
		
		loop.add(v);
		for (int i = 1; i <= duv-1; i++) {
			Set<String> rh = new LinkedHashSet<String>();
			Set<String> lh = new LinkedHashSet<String>();
			List<Integer> predValue = dPredecessorValue(u);
			List<Integer> succValue = dSuccessorValue(loop.get(loop.size()-1));
			for (int n = 0; n < currentKeySet.size(); n++) {
				if (predValue.get(n) == duv-1)
					rh.add(currentKeySet.get(n));
				if (succValue.get(n) == 1)
					lh.add(currentKeySet.get(n));
			}
			for (String vertex : lh) {
				if (rh.contains(vertex)) {
					loop.add(vertex);
					break;
				}
			}
		}
		
		List<Pair> pairs = new ArrayList<Pair>();
		for (int i = 0; i < loop.size()-1; i++) {
			pairs.add(PairFactory.getPair(loop.get(i), loop.get(i+1)));
		}
		return pairs;
	}
	
	private List<Integer> dSuccessorValue(String variable) {
		List<Integer> values = new ArrayList<Integer>();
		Map<String, Integer> row = D.get(variable);
		for (String v : currentKeySet) {
			values.add(row.get(v));
		}
		return values;
	}
	
	private List<Integer> dPredecessorValue(String variable) {
		List<Integer> values = new ArrayList<Integer>();
		for (String v : currentKeySet) {
			values.add(D.get(v).get(variable));
		}
		return values;
	}

	public Set<Loop> getGeodeticCycles() {
		return geodeticCycles;
	}
	
	private void dumpToDistanceFile(String file, Map<String, Map<String, Integer>> distance, List<String> keySet) {
		try {
			BufferedWriter bw = Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
			for (String var1 : keySet) {
				bw.append("\t");
				bw.append(var1);
			}
			bw.append("\n");
			
			for (String var1 : keySet) {
				bw.append(var1);
				Map<String, Integer> dist = distance.get(var1);
				for (String var2 : keySet) {
					if (dist == null || dist.get(var2) == null) {
						bw.append("\t"+Integer.MAX_VALUE);
					} else {
					bw.append("\t"+dist.get(var2).intValue());
//					log.debug(var1+" -> "+var2+ " = "+dist.get(var2).intValue());
					}
				}
				bw.append("\n");
			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private List<String> ordered(List<String> aSet) {
		List<String> aList = new ArrayList<String>(aSet);
		Collections.sort(aList);
		return aList;
	}

	public boolean isUnique() {
		return unique;
	}
}
