package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class IndependentLoopSet {
static Logger log = Logger.getLogger(IndependentLoopSet.class.getSimpleName());
	
	/**
	 * Returns a set of ILS for cyclePartition
	 * CyclePartition is for a single level
	 */
	
	private LoopManager loopManager;
	private CyclePartition cyclePartition;
	private Set<List<Loop>> allILS;
	
	private Map<String, Map<String, Integer>> shortestDistanceFromLoops;
	
	private Map<String, Map<String, Integer>> distance;

	
	public IndependentLoopSet(LoopManager loopManager, CyclePartition cyclePartition) {
		this.loopManager = loopManager;
		this.cyclePartition = cyclePartition;
		this.distance = cyclePartition.getReachabilityMatrix().getDistance();

		allILS = new LinkedHashSet<List<Loop>>();
		build();
	}
	
	private void build() {

		// get cycle partitions

		Set<Set<String>> partitions = cyclePartition.getCyclePartitions();
		for (Set<String> cp : partitions) {
			if (SDM_Doc.DUMP_TABULAR_DATA)
			dumpToDistanceFile("Distance.tab", cyclePartition.getReachabilityMatrix().getDistance(), ordered(cp));
			
			shortestDistanceFromLoops = computeShortestDistanceFromLoops(cp);
			if (SDM_Doc.DUMP_TABULAR_DATA)
				dumpToDistanceFile("LoopDistance.tab", shortestDistanceFromLoops, ordered(cp));
			
//			List<Loop> allLoopsInCyclePartition; 
			GraphTheoryMatrix B = new GraphTheoryMatrix();
			Map<Loop, GraphTheoryMatrix> E = new LinkedHashMap<Loop, GraphTheoryMatrix>();
			Map<Loop, Integer> c = new LinkedHashMap<Loop, Integer>();
			List<Loop> toRemove = new ArrayList<Loop>();
			List<Loop> sils = new ArrayList<Loop>();
			
			
//			allLoopsInCyclePartition = loopManager.getLoopsOrderedBySizeContaining(cp);
			
//			GeodeticCycle(List<String> currentKeySet, Map<String, Map<String, Integer>> distance, 
//					List<Loop> allLoopsInCyclePartition)
			
			GeodeticCycle geodeticCycles = new GeodeticCycle(loopManager, new ArrayList<String>(cp), distance);
			List<Loop> allLoopsInCyclePartition = new ArrayList<Loop>(geodeticCycles.identifyGeodeticCyclesUsingLoops());
			List<Loop> allLoopsInCyclePartitionGeodetic = new ArrayList<Loop>(geodeticCycles.getGeodeticCycles());
			
			System.out.println("### ILS - allLoops "+allLoopsInCyclePartition.size());
			for (Loop loop : allLoopsInCyclePartition) {
				// E<loop>, j j+1 = 1
				E.put(loop, new GraphTheoryMatrix());
				GraphTheoryMatrix gtm = E.get(loop);
				List<String> vars = loop.getLoop();
				for (int j = 0; j < vars.size()-1; j++) {
					gtm.addSuccessor(vars.get(j), vars.get(j+1));
				}
			}

			while (allLoopsInCyclePartition.size() > 0) {

				c.clear();
				toRemove.clear();
				
				for (Loop loop : allLoopsInCyclePartition) {
					int cnt = count(E.get(loop), B);
					c.put(loop, cnt);
					if (cnt == 0) {
						toRemove.add(loop);
					}
				}
				allLoopsInCyclePartition.removeAll(toRemove);
				Loop toAdd = selectLoop(c);
				if (toAdd != null) {
					sils.add(toAdd);
					allLoopsInCyclePartition.remove(toAdd);
					B.add(E.get(toAdd));
				}
				
				
			}
			allILS.add(sils);
			System.out.println("### SILS - allLoops "+sils.size());
			int silsCnt = 0;
			for (Loop loop : sils) {
				System.out.println("###SILS Loop ### "+(++silsCnt)+" size: "+loop.getSize());
				for (int i = 0; i < loop.getSize(); i++) {
					System.out.println(" "+Utilities.idToName(loop.getLoop().get(i)));
				}
			}
		}

	}
	

	
	private void dumpToDistanceFile(String file, Map<String, Map<String, Integer>> distance, List<String> keySet) {
		try {
			BufferedWriter bw = Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
			for (String var1 : keySet) {
				bw.append("\t");
				bw.append(var1);
			}
			bw.append("\n");

			for (String var1 : keySet) {
				bw.append(var1);
				Map<String, Integer> dist = distance.get(var1);
				if (dist == null) {
					for (String var2 : keySet) {
						bw.append("\t"+999);
					}
				} else {
					for (String var2 : keySet) {
						if (dist.get(var2) == null) {
							bw.append("\t"+999);
						} else {
							bw.append("\t"+dist.get(var2).intValue());
						}
						//					log.debug(var1+" -> "+var2+ " = "+dist.get(var2).intValue());
					}
				}
				bw.append("\n");
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	private Loop selectLoop(Map<Loop, Integer> c) {
		if (c.size() == 0)
			return null;
		
		// find minimum value
		
		int minVal = Integer.MAX_VALUE;
		int minLength = Integer.MAX_VALUE;
		Loop minLoop = null;
		
		for (Loop l : c.keySet()) {
			int cnt = c.get(l);
			if (cnt < minVal) {
				minVal = cnt;
			}
		}
		for (Loop l : c.keySet()) {
			int cnt = c.get(l);
			if (cnt == minVal) {
				if (l.getSize() < minLength) {
					minLength = l.getSize();
					minLoop = l;
				}
			}
		}
		
		return minLoop;
	}
	
	public int count(GraphTheoryMatrix E, GraphTheoryMatrix B) {
		int cnt = 0;
		
		// count edges in E not in B
		for (String var : E.keySet()) {
			Set<String> eEdge = E.getSuccessorCopy(var);
			Set<String> bEdge = B.getSuccessorCopy(var);
			eEdge.removeAll(bEdge);
			cnt += eEdge.size();
		}
		
		
		return cnt;
	}

	public Set<List<Loop>> getAllILS() {
		return allILS;
	}
	

	private List<String> ordered(Set<String> aSet) {
		List<String> aList = new ArrayList<String>(aSet);
		Collections.sort(aList);
		return aList;
	}
	
	private Map<String, Map<String, Integer>> computeShortestDistanceFromLoops(Set<String> aSet) {
		
		Map<String, Map<String, Integer>> shortDist = new LinkedHashMap<String, Map<String, Integer>>();
		
		for (Loop loop : loopManager.getLoopsLimitedTo(aSet)) {
			for (int i = 0; i < loop.getSize(); i++) {
				Loop l = loop.getLoopStartingAt(loop.getLoop().get(i));
				for (int j = 1; j < l.getSize(); j++)
					setShortDist(shortDist, l.getLoop().get(0), l.getLoop().get(j), j);
			}
		}
		
		return shortDist;
		
	}
	
	private void setShortDist(Map<String, Map<String, Integer>> shortDist, String v1, String v2, int dist) {
		Map<String, Integer> v1Map = getV1Map(shortDist, v1);
		int currentDist = getV2Dist(v1Map, v2);
		if (dist < currentDist)
			setV2Dist(v1Map, v2, dist);
	}
	
	private Map<String, Integer> getV1Map(Map<String, Map<String, Integer>> shortDist, String v1) {
		if (!shortDist.containsKey(v1))
			shortDist.put(v1, new LinkedHashMap<String, Integer>());
		return shortDist.get(v1);
	}
	
	private int getV2Dist(Map<String, Integer> v1Map, String v2) {
		if (v1Map.containsKey(v2))
			return v1Map.get(v2).intValue();
		else
			return Integer.MAX_VALUE;
	}
	
	private void setV2Dist(Map<String, Integer> v1Map, String v2, int value) {
		v1Map.put(v2, value);
	}


}
