package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class LevelPartition {
static Logger log = Logger.getLogger(LevelPartition.class.getSimpleName());
	
	private ReachabilityMatrix reachabilityMatrix;
	private Map<String, Integer> partition;
	private Map<Integer, Set<String>> variablesByLevel;
	private int numLevels;
	
	public LevelPartition(ReachabilityMatrix reachabilityMatrix) {
		this.reachabilityMatrix = new ReachabilityMatrix(reachabilityMatrix);
		partition = new LinkedHashMap<String, Integer>();
		variablesByLevel = new LinkedHashMap<Integer, Set<String>>();
	}
	
	private Set<String> intersection(Set<String> setA, Set<String> setB) {
		Set<String> intersect = new LinkedHashSet<String>(setA);
		intersect.retainAll(setB);
		return intersect;
	}
	
	public void build() {
		int level = 1;
		Set<String> thisLevel = new LinkedHashSet<String>();
		int numKeysIn = 0;
		int numKeysOut = 0;
		while(reachabilityMatrix.keySet().size() > 0) {
			numKeysIn = reachabilityMatrix.keySet().size();
			for (String v : reachabilityMatrix.keySet()) {
				Set<String> predecessor = reachabilityMatrix.getPredecessor(v);
				Set<String> successor = reachabilityMatrix.getSuccessor(v);
				

				if (successor.size() == intersection(predecessor, successor).size()) {
					thisLevel.add(v);
					numLevels = level;
				}
			}
			if (!variablesByLevel.containsKey(level)) {
				variablesByLevel.put(level, new LinkedHashSet<String>());
			}
			for (String v : thisLevel) {
				variablesByLevel.get(level).add(v);
				partition.put(v, level);
				reachabilityMatrix.remove(v);
			}
			
			numKeysOut = reachabilityMatrix.keySet().size();
			if (numKeysOut != numKeysIn) {
				System.out.println(" Made progress at level "+level);
				System.out.println(" vars in level "+thisLevel);
			} else {
				System.out.println(" Made NO progress at level "+level+" abort");
				break;
			}
			thisLevel.clear();
			level++;
		}
		print();
	}
	
	private void print() {
		System.out.println("### Level Partition ###");
		List<Integer> al = new ArrayList<Integer>(variablesByLevel.keySet());
		Collections.sort(al);
		for (Integer key : al) {
			
			List<String> al2 = new ArrayList<String>(variablesByLevel.get(key));
			System.out.println(">>> Level "+key.intValue()+" size = "+al2.size());
			Collections.sort(al2);
			for (String s : al2)
				System.out.println("   "+s);
		}
	}

	public int getNumLevels() {
		return numLevels;
	}
	
	public Set<String> getVariablesInLevel(int level) {
		return variablesByLevel.get(level);
	}
	
	

}
