package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

public class MatrixStatistics {
	
	private GraphTheoryMatrix matrix;
	
	private long numRows = 0L;
	private long size = 0L;
	private long numNonZero = 0L;
	private double density;
	private List<RowSum> rowSums = new ArrayList<RowSum>();
	
	private class RowSumComparator implements Comparator<RowSum> {

			@Override
			public int compare(RowSum o1, RowSum o2) {
				int o1Val = o1.getSum().intValue();
				int o2Val = o2.getSum().intValue();
				if (o1Val < o2Val) 
					return 1;
				if (o1Val == o2Val)
					return 0;
				return -1;
				
			}
		}
		
	
	private class RowSum {
		
		String rowName;
		Integer sum;
		
		public RowSum(String name, Integer sum) {
			this.rowName = name;
			this.sum = sum;
		}

		/**
		 * @return the sum
		 */
		public Integer getSum() {
			return sum;
		}

		/**
		 * @return the rowName
		 */
		public String getRowName() {
			return rowName;
		}
		
	}
	
	public MatrixStatistics(GraphTheoryMatrix matrix) {
		this.matrix = matrix;
		analyze();
		sumRows();
		generateDistribution();
	}
	
	private void generateDistribution() {
		int[] buckets = new int[100];
		
		for (RowSum rs : rowSums) {
			double fract = rs.getSum().doubleValue() / numRows;
			int bucket = (int) (fract * 100.0);
			buckets[bucket]++;
		}
		
//		for (int i = 0; i < buckets.length; i++) {
//			System.out.println("Bucket: "+i+" Count: "+buckets[i]);
//		}
	}
	
	private void sumRows() {
		for (String rowName : matrix.getSuccessor().keySet()) {
			RowSum rs = new RowSum(rowName, matrix.getSuccessor(rowName).size());
			rowSums.add(rs);
		}
		
		Collections.sort(rowSums, new RowSumComparator());
		
//		for (RowSum rs : rowSums) {
//			System.out.println("Row: "+rs.getRowName()+" Sum: "+rs.getSum()+" % = "+rs.getSum().doubleValue() / numRows);
//		}
	}
	
	private void analyze() {
		numRows = matrix.getSuccessor().keySet().size();
		size = numRows * numRows;
		numNonZero = countNonZero();
		density = computeDensity();
	}
	
	public void report() {
		System.out.println("#########");
		System.out.println("Num Rows:    "+numRows);
		System.out.println("Size:        "+size);
		System.out.println("Num NonZero: "+numNonZero);
		System.out.println("Density:     "+density);
	}
	
	private double computeDensity() {
		return (double) numNonZero / (double) size;
		
	}
	
	private long countNonZero() {
		long count = 0;
		for (Set<String> s : matrix.getSuccessor().values()) {
			count += s.size();
		}
		return count;
	}

	public GraphTheoryMatrix getMatrix() {
		return matrix;
	}

	public long getNumRows() {
		return numRows;
	}

	public long getSize() {
		return size;
	}

	public long getNumNonZero() {
		return numNonZero;
	}

	public double getDensity() {
		return density;
	}

}
