package gov.anl.gss.sdm.server.model.analysis.graphtheory;

import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class ReachabilityMatrix extends GraphTheoryMatrix {
	static Logger log = Logger.getLogger(ReachabilityMatrix.class.getSimpleName());

	private int maxLevels = 25;


	private Map<String, Map<String, Integer>> minDistance;
	private Map<String, Map<String, Integer>> maxDistance;
	private Map<String, Map<String, Set<Integer>>> reachableByName;
	private Map<String, Map<Integer, Set<String>>> reachableByLength;

	Map<String, Set<String>> hasPath = new LinkedHashMap<String, Set<String>>();

	Set<String> endogenousVariables = new LinkedHashSet<String>();
	Set<String> exogenousVariables = new LinkedHashSet<String>();

	MessageProcessor myMF = null;

	public ReachabilityMatrix(SDObjectManager objManager, PolarityResults polarityResults) {
		super(objManager, polarityResults);
		minDistance = new LinkedHashMap<String, Map<String, Integer>>();
		maxDistance = new LinkedHashMap<String, Map<String, Integer>>();
	}
	
	public ReachabilityMatrix(SDObjectManager objManager) {
		super(objManager, null);
		minDistance = new LinkedHashMap<String, Map<String, Integer>>();
		maxDistance = new LinkedHashMap<String, Map<String, Integer>>();
	}

	public ReachabilityMatrix(GraphTheoryMatrix gtm) {
		super(gtm.getObjManager(), gtm.getPolarityResults());

		minDistance = new LinkedHashMap<String, Map<String, Integer>>();
		maxDistance = new LinkedHashMap<String, Map<String, Integer>>();
		for (String row : gtm.keySet()) {
			Set<String> rowSet = gtm.getRowSet(row);
			minDistance.put(row, new LinkedHashMap<String, Integer>());
			maxDistance.put(row, new LinkedHashMap<String, Integer>());
			for (String col : rowSet) {
				addSuccessor(row,col);

			}
		}

	}

	
	public void buildViaTree(MessageProcessor mf, AdjacencyMatrix adjMatrix) {
		myMF = mf;
		initializeWith(adjMatrix);
		//		BufferedWriter bw = null;
		BufferedWriter bw = Utilities.openFileForWriting(InformationManagers.getInstance()
				.getFileOrganizer().getStructureDirectory()+"/distance.csv");
		try {
			bw.append("Length,Start,End,Path\n");
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		int exoCount = 0;
		int endoCount = 0;
		
		Set<String> ks = new LinkedHashSet<String>(adjMatrix.keySet());
		Set<String> vs = new LinkedHashSet<String>(InformationManagers.getInstance().getEquationManager().getVariables());
		
		Set<String> u = Utilities.union(ks, vs);
		Set<String> i = Utilities.intersect(ks, vs);
		Set<String> m = Utilities.minus(u, i);

		System.out.println("adjMatrix.keySet() size = "+adjMatrix.keySet().size());
		System.out.println("equation manager vars size = "+
		InformationManagers.getInstance().getEquationManager().getVariables().size());
		
		Set<String> adjKeys = new LinkedHashSet<String>(adjMatrix.keySet());
		Set<String> eqnKeys = new LinkedHashSet<String>(InformationManagers.getInstance().getEquationManager().getVariables());
		
		Set<String> sdoVars = InformationManagers.getInstance().getSdObjectManager().getVariables();
		
		
		Set<String> minus = Utilities.minus(eqnKeys, adjKeys);
		Set<String> inter = Utilities.intersect(adjKeys, eqnKeys);
		
		minus = Utilities.minus(eqnKeys, sdoVars);
		inter = Utilities.intersect(eqnKeys, sdoVars);
		
		minus = Utilities.minus(sdoVars, eqnKeys);
		
		minus = Utilities.minus(sdoVars, adjKeys);
		minus = Utilities.minus(adjKeys, sdoVars);

		for (String var : adjMatrix.keySet()) {
			boolean ignoreVar = ExogenousVariableManager.isIgnoreVar(var);
			boolean exogenousVar = ExogenousVariableManager.isExogenousVar(var);
			boolean endogenousVar = ExogenousVariableManager.isEndogenousVar(var);
			
			Equation eqn = InformationManagers.getInstance().getEquationManager().
					getAnEquationFor(!Parser.isQuotedString(var) ? Parser.quoted(var) : var);
			if (eqn == null) {
				System.out.println("Exo Analysis No EQN for: "+var);
				continue;
			} 
				if (ignoreVar)
					continue;
				if (exogenousVar) {
					if (!ignoreInExogenousCount(eqn)) {
						exoCount++;
					
					exogenousVariables.add(var);
				}
				} else {
					endoCount++;
					endogenousVariables.add(var);
				}
		}
		log.debug("Exogenous Variable Count: "+exoCount);
		log.debug("Endogenous Variable Count: "+endoCount);
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		System.out.println("##### Reachability Matrix Exogenous Variables #####");
//		Utilities.printToConsole(exogenousVariables);
//		System.out.println("##### Reachability Matrix Exogenous Variables #####");

		mf.addMessage(language.getText(StringConstants.EXOGENOUS_VARIABLE_COUNT)+": "+exoCount);
		mf.addMessage(language.getText(StringConstants.ENDOGENOUS_VARIABLE_COUNT)+": "+endoCount);
		mf.addMessage(language.getText(StringConstants.ENDOGENOUS_LEVEL_LIMIT)+": "+maxLevels);
		inter = Utilities.intersect(endogenousVariables, exogenousVariables);
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();

		if (exoCount > 0 && options.isSelected(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES)) {
			int interval = Math.max(exoCount / 10, 1);
			int num = 0;
			int pct = 0;
			int numEqn = 0;
			boolean printed100 = false;
			for (String var : exogenousVariables /*adjMatrix.keySet() */ ) {
				Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
				if (eqn != null) {
					numEqn++;
//					if (eqn.isExogenous()) {
						if (!ignoreInExogenousCount(eqn)) {
							num++;
							
							if (num > exoCount) {
								System.out.println("num > exoCount");
							}
							
							buildTree(bw, var, adjMatrix);

							pct = (num*100) / exoCount;
							
							if (pct > 100) {
								System.out.println("pct > 100");
							}
							
							if (num > 0 &&pct % 10.0 == 0 && pct*exoCount/100 == num
									&& pct < 100 && pct > 0) {
								mf.addMessage(language.getText(StringConstants.EXOGENOUS_PROGRESS)+": "+pct+"% "+
										language.getText(StringConstants.OF)+" "+exoCount);
							}
						} else {
							System.out.println("IGNORING IN EXO COUNT: "+var);
						}
//					}
				}
			}
			if (!printed100)
				mf.addMessage(language.getText(StringConstants.EXOGENOUS_PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+exoCount);
		}

		// hasPath contains for each endo, the exo that connect. Need set of all exo and then remove
		// those with path


		if (bw != null)
			try {
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}
	
//	private boolean isIgnoreVar(String var) {
//		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
//			if (ignore(eqn))
//				return true;
//		}
//		return false;
//	}
//	private boolean isExogenousVar(String var) {
//		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
//			if (eqn.isExogenous())
//				return true;
//		}
//		return false;
//	}
//	private boolean isEndogenousVar(String var) {
//		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
//			if (eqn.isExogenous())
//				return false;
//		}
//		return true;
//	}
	
	

	private boolean ignoreInExogenousCount(Equation eqn) {
		// Parser.isSystemVariable(
		// isNaReplacement
		return /*eqn.isControlGroup() || */
				Parser.isNaReplacement(eqn.getLhsAsInMdl()) ||
				Parser.isWithLookup(eqn.getLhs());
	}
	
//	public boolean ignore(Equation equation) {
//
//		if (equation.isDefinesSubscript())
//			return true;
//		if (equation.isControlGroup() || isNAorTime(equation.getLhsAsInMdl()))
//			return true;
//		return false;
//	}
//	
//	public boolean isNAorTime(String s) {
//		if (s == null)
//			return false;
//		return s.toUpperCase().equals("TIME") || s.toUpperCase().equals("NAREPLACEMENT");
//	}

	public Map<String, Set<Integer>> buildTree(BufferedWriter bw, String startVariable, AdjacencyMatrix adjMatrix) {

		Node root = new Node(startVariable);
		Set<String> visited = new LinkedHashSet<String>();

		// start building levels
		expandTree(root, adjMatrix, visited, 0);
//		Node.printTreeToConsole(root);
//		Node.printTreeToConsole(root);
		Map<String, Set<Integer>> distances = computeDistances(root);


		Node.getConnected(root, hasPath);

		//		Node.printPaths(bw, root);
		return distances;
	}

	public void expandTree(Node node, AdjacencyMatrix adjMatrix, Set<String> visited, int level) {

		if (level >= maxLevels) {
			return;
		}
		if (node == null) {
			return;
		}
		if (adjMatrix.getSuccessor(node.getToken()) == null) {
			return;
		}
		
		Set<String> localVisited = new LinkedHashSet<String>(visited);
		Set<String> children = new LinkedHashSet<String>(adjMatrix.getSuccessor(node.getToken()));
		children.remove(node.getToken());
		children.removeAll(localVisited);
		if (children.size() == 0) {
			return;
		}
		Node myChild = node.addChildren(children);
		//		visited.addAll(children);
		Node current = myChild;
		int nextLevel = level + 1;
		while (current != null) {
			if (!localVisited.contains(current.getToken())) {
				localVisited.add(current.getToken());
				expandTree(current, adjMatrix, localVisited, nextLevel);

			}
			current = current.getNext();
		}
	}

	public Map<String, Set<Integer>> computeDistances(Node node) {
		Map<String, Set<Integer>> distances = new LinkedHashMap<String, Set<Integer>>();

		if (!minDistance.containsKey(node.getToken())) {
			minDistance.put(node.getToken(), new LinkedHashMap<String, Integer>());
			maxDistance.put(node.getToken(), new LinkedHashMap<String, Integer>());
		}
		loadDistances(node, 0, new ArrayList<String>());

		return distances;
	}

	public void loadDistances(Node node, int level, List<String> al) {
		if (node == null)
			return;
		al.add(node.getToken());
		int length = al.size();
		if (length > 0) {
			if (!minDistance.containsKey(al.get(0))) {
				minDistance.put(al.get(0), new LinkedHashMap<String, Integer>());
				maxDistance.put(al.get(0), new LinkedHashMap<String, Integer>());
			}
			if (minDistance.get(al.get(0)).containsKey(node.getToken())) {
				if (minDistance.get(al.get(0)).get(node.getToken()) > level /*length*/)
					minDistance.get(al.get(0)).put(node.getToken(), level /*length*/);

				if (maxDistance.get(al.get(0)).get(node.getToken()) < level /*length*/)
					maxDistance.get(al.get(0)).put(node.getToken(), level /*length*/);

			} else {
				minDistance.get(al.get(0)).put(node.getToken(), level);
				maxDistance.get(al.get(0)).put(node.getToken(), level);
			}
		}

		loadDistances(node.getChild(), level+1, new ArrayList<String>(al));
		loadDistances(node.getNext(), level, new ArrayList<String>(al));
	}

	public void buildNew(AdjacencyMatrix adjMatrix) {

		boolean makingProgress = false;

		// initializes the successors and predecessors of the GTM given adjacency matrix.
		// successor is tail -> set(head) and predecessor is head -> set(tail).
		initializeWith(adjMatrix);

		// set the distance of each adjacent pair to 1	

		reachableByLength = new LinkedHashMap<String, Map<Integer, Set<String>>>();
		reachableByName = new LinkedHashMap<String, Map<String, Set<Integer>>>();
		List<String> variableNames = new ArrayList<String>(keySet());
		int pathLength = 1;
		for (String var : variableNames) {
			Set<String> singleStepReachable = getRowSet(var);
			reachableByLength.put(var, new LinkedHashMap<Integer, Set<String>>());
			Set<String> reach = new LinkedHashSet<String>();
			reach.addAll(singleStepReachable);
			reachableByLength.get(var).put(pathLength, reach);


			for (String s : singleStepReachable) {
				reachableByName.get(var).put(s, new LinkedHashSet<Integer>());
				reachableByName.get(var).get(s).add(pathLength);
			}
		}


		// compute the reachability and (shortest) distance from one variable to another
		tmpsuccessor = copyOfSuccessor();
		do {			

			makingProgress = false;

			// get the list of all variables for which we need to process
			List<String> al = new ArrayList<String>(tmpKeySet());

			pathLength++;

			for (String var : al) {

				Set<String> reach = new LinkedHashSet<String>();
				reachableByLength.get(var).put(pathLength, reach);
				Set<String> nextReach = reachableByLength.get(var).get(pathLength);
				Set<String> currentReach = reachableByLength.get(var).get(pathLength-1);
				for (String cvar : currentReach) {
					nextReach.addAll(reachableByLength.get(cvar).get(1));
				}

			}
		} while (makingProgress);

		List<String> alDist = new ArrayList<String>(keySet());
		for (String key : alDist) {
			if (!minDistance.containsKey(key))
				minDistance.put(key,  new LinkedHashMap<String, Integer>());
			Map<String, Integer> minDist = minDistance.get(key);

			if (!maxDistance.containsKey(key))
				maxDistance.put(key,  new LinkedHashMap<String, Integer>());
			Map<String, Integer> maxDist = minDistance.get(key);

			Set<String> rchSet = getRowSet(key);
			for (String s : rchSet) {
				minDist.put(s,1);
				maxDist.put(s,1);
			}
		}


		//		printMatrix();

		//		printDistance();
		if (SDM_Doc.DUMP_TABULAR_DATA) {
			dumpMatrixToFile("ReachabilitySuccessor.tab", successor);
			dumpMatrixToFile("ReachabilityPredecessor.tab", predecessor);
			dumpDistanceToFile(minDistance, "minDistance.tab");
			dumpDistanceToFile(maxDistance, "maxDistance.tab");
		}
	}

	public void build(String tag, AdjacencyMatrix adjMatrix) {

		boolean makingProgress = false;

		// initializes the successors and predecessors of the GTM given adjacency matrix.
		// successor is tail -> set(head) and predecessor is head -> set(tail).
		initializeWith(adjMatrix);

		// set the distance of each adjacent pair to 1

		List<String> alDist = new ArrayList<String>(keySet());
		for (String key : alDist) {
			if (!minDistance.containsKey(key))
				minDistance.put(key,  new LinkedHashMap<String, Integer>());
			Map<String, Integer> minDist = minDistance.get(key);

			if (!maxDistance.containsKey(key))
				maxDistance.put(key,  new LinkedHashMap<String, Integer>());
			Map<String, Integer> maxDist = maxDistance.get(key);

			Set<String> rchSet = getRowSet(key);
			for (String s : rchSet) {
				minDist.put(s,1);
				maxDist.put(s,1);
			}
		}

		int loop = 1;
		int numDiff = 0;


		// compute the reachability and (shortest) distance from one variable to another
		do {

			log.debug("ReachabilityMatrix Loop "+(loop));

			makingProgress = false;

			// get the list of all variables for which we need to process
			List<String> al = new ArrayList<String>(keySet());

			for (String key : al) {

				// get the current set of computed distances
				Map<String, Integer> minDist = minDistance.get(key);
				if (minDist == null)
					continue;
				Map<String, Integer> maxDist = maxDistance.get(key);
				//				if (minDist == null)
				//					continue;

				// rchSet is the set of variables currently determined to be reachable
				// directly from key.    key -> {rchSet}
				Set<String> rchSet = getRowSet(key);
				Set<String> expanded = new LinkedHashSet<String>();

				// this is the set of reachable variables from key
				for (String s : rchSet) {
					Set<String> row = getRowSet(s);

					// one level of inderection
					// key -> s -> {set}
					if (row != null) {
						expanded.addAll(row);
					}
				}

				// expanded now contains a list of variables that are reachable through one other variable

				// add elements not already in rshSet, the Integer value is the Loop + 1
				for (String s : expanded) {
					if (!minDist.containsKey(s)) {
						minDist.put(s, loop+1);
					} else {
						if (minDist.get(s) > loop+1) {
							minDist.put(s, loop+1);
						}
					}
					maxDist.put(s, loop+1);
				}

				int preSize = rchSet.size();
				merge(key, rchSet, expanded);



				int postSize = rchSet.size();
				numDiff = postSize - preSize;
				if (postSize != preSize) {
					makingProgress = true;
					//					log.debug("key <"+key+"> numDiff = "+numDiff);
				}
			}
			loop++;
		} while (makingProgress);


		//		printMatrix();

		//		printDistance();
		if (SDM_Doc.DUMP_TABULAR_DATA) {
			dumpMatrixToFile(tag+"_ReachabilitySuccessor.tab", successor);
			dumpMatrixToFile(tag+"_ReachabilityPredecessor.tab", predecessor);
			dumpDistanceToFile(minDistance, tag+"_minDistance.tab");
			dumpDistanceToFile(maxDistance, tag+"_maxDistance.tab");
		}
	}

	private String setToString(Set<String> set) {
		StringBuilder sb = new StringBuilder();
		for (String s : ordered(set))
			sb.append("|"+s);
		return sb.toString();
	}

	private void printReachSet(Set<String> set) {
		log.debug("AddSucc: <reach> ");
		for (String s : ordered(set))
			log.debug("|"+s);
		log.debug("\n");;
	}

	public void dumpDistanceToFile(Map<String, Map<String, Integer>> distance, String file) {

		try {
			BufferedWriter bw = Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
			for (String var1 : ordered(distance.keySet())) {
				bw.append("\t");
				bw.append(var1);
			}
			bw.append("\n");

			for (String var1 : ordered(distance.keySet())) {
				bw.append(var1);
				Map<String, Integer> dist = distance.get(var1);
				for (String var2 : ordered(distance.keySet())) {
					if (dist.containsKey(var2))
						bw.append("\t"+dist.get(var2).intValue());
					else
						bw.append("\t");
				}
				bw.append("\n");
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void printDistance() {
		log.debug("### Distance Matrix");
		for (String var1 : ordered(minDistance.keySet())) {
			Map<String, Integer> dist = minDistance.get(var1);
			for (String var2 : ordered(dist.keySet())) {
				log.debug(var1+" -> "+var2+ " = "+dist.get(var2).intValue());
			}
		}
	}

	private List<String> ordered(Set<String> aSet) {
		List<String> aList = new ArrayList<String>(aSet);
		Collections.sort(aList);
		return aList;
	}

	public Map<String, Map<String, Integer>> getDistance() {
		return minDistance;
	}

	public int getDistance(String from, String to) {
		int dist = -1;
		if (minDistance.containsKey(from)) {
			if (minDistance.get(from).containsKey(to)) {
				dist = minDistance.get(from).get(to).intValue();
			}
		} else {
			if (isReachable1(from, to)) {
				if (from.equals(to))
					dist = 0;
				else
					dist = 1;
			}
		}
		return dist;
	}

	public boolean isReachable(String var1, String var2) {
		//		Map<String, Set<String>>

		if (successor.containsKey(var1) && successor.containsKey(var2)) {
			return successor.get(var1).contains(var2) && successor.get(var2).contains(var1);
		} else {
			return false;
		}
	}

	public boolean isReachable1(String var1, String var2) {
		//		Map<String, Set<String>>

		if (successor.containsKey(var1)) {
			return successor.get(var1).contains(var2);
		} else {
			return false;
		}
	}


	public int getMaxLevels() {
		return maxLevels;
	}

	public void setMaxLevels(int maxLevels) {
		this.maxLevels = maxLevels;
	}

	public void setMaxLevels(String maxLevels) {
		try {
			this.maxLevels = Integer.parseInt(maxLevels);
		} catch (NumberFormatException e) {
			// TODO Auto-generated catch block
			System.out.println("Trying to set maxLevels to <"+maxLevels+">");
//			e.printStackTrace();
			this.maxLevels = 25;
		}
	}

	public Map<String, Set<String>> getHasPath() {
		return hasPath;
	}



	public Set<String> getExogenousVariables() {
		return exogenousVariables;
	}

	public Set<String> getEndogenousVariables() {
		return endogenousVariables;
	}


}
