package gov.anl.gss.sdm.server.model.analysis.leea;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArgumentBuilder;
import gov.anl.gss.sdm.server.model.analysis.polarity.ArgumentBuilderInterface;
import gov.anl.gss.sdm.server.model.translator.ArrayReferenceNative;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.Map;

public class LeeaArgumentBuilder extends ArgumentBuilder implements ArgumentBuilderInterface {
	
	int numIter;

	public LeeaArgumentBuilder(int numArgs, Map<String, String> idToNameMap) {
		super(numArgs, idToNameMap);
	}
	
	public boolean initialize() {

		boolean arrayReference = false;

		for (String taggedVar : idToNameMap.values()) {
			String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(taggedVar);
			if (ArrayReferenceNative.isArrayReference(origName))
				arrayReference = true;
		}

		if (!arrayReference && databaseAvailable) {

			LeeaParameters.setAnalyzeTimeSteps(new double[] {2.0, 30.0, 41.0});

			double[] analyzeTime = LeeaParameters.getAnalyzeTimeSteps();
			numIter = analyzeTime.length * (numArgs+1);
			arguments = new double[numIter][numArgs];
			simTime = new double[numIter];
			
			double[] time = InformationManagers.getInstance().getDataBridge().getTimeAsArray();
			double start = time[0];
			dt = time[1]-time[0];
			if (start!=0)
				start = 0;
			
			double[][] allValues = new double[numArgs][time.length];
			
			for (int iarg = 0; iarg < numArgs; iarg++) {
				String taggedVar = idToNameMap.get(asIdToNameMapKey(iarg));
				String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(taggedVar);
				allValues[iarg] = InformationManagers.getInstance().getDataBridge().getValuesAsArray(origName);
			}

			int index = 0;
//			simTime = new double[analyzeTime.length];
			for (int i = 0; i < analyzeTime.length; i++) {
				int timeIndex = (int) ((analyzeTime[i] - start)/dt);
				
				// establish the baseline value 
				for (int iarg = 0; iarg < numArgs; iarg++) {
					double val = allValues[iarg][timeIndex];
					arguments[index][iarg] = val;
					simTime[index] = analyzeTime[i]+time[0];
				}
				index++;
				// now alter the arguments one at a time left to right
				for (int varyArg = 0; varyArg < numArgs; varyArg++) {
				
					for (int iarg = 0; iarg < numArgs; iarg++) {
						double val = allValues[iarg][timeIndex];
						arguments[index][iarg] = iarg == varyArg ? val+Math.abs(0.1*val) : val;
						simTime[index] = analyzeTime[i]+time[0];
					}
					index++;
				}
			}
			return true;
		} else {
			return false;
		}
	}
	

//	public double getDt() {
//		return dt;
//	}

}
