package gov.anl.gss.sdm.server.model.analysis.leea;

import gov.anl.gss.sdm.server.model.analysis.polarity.CodeBuilderInterface;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityCodeBuilder;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.List;

public class LeeaCodeBuilder extends PolarityCodeBuilder implements CodeBuilderInterface {

	public LeeaCodeBuilder(Equation equation, Equation originalEquation, boolean ivv) {
		super(equation, originalEquation, ivv);
	}
	
	public void setDt(double dt) {
		this.dt = dt;
	}
	
	@Override
	public StringBuilder createClassSourceCode() {
		 StringBuilder src = new StringBuilder();
		 src.append(getHeader(dt));
		 src.append("double "+getGeneratedCode()+";\n");
		 src.append(getTail());
		 return src;
	}
	
	protected String getHeader(double dt) {
		StringBuilder src = new StringBuilder();
		src.append(" import gov.anl.gss.sdm.server.model.support.SDFunctionsWithXLSColt;\n");
		src.append(" import gov.anl.gss.sdm.server.model.support.SDModel;\n");
        src.append("public class PCalculator {\n");
        src.append("protected SDModel sdModel =  new SDModel(\"leea\", new String[] {});\n");
        src.append("protected SDFunctionsWithXLSColt sdFunctions = new SDFunctionsWithXLSColt(sdModel);\n");
        
        src.append("public double compute(double[] rhs, double time) {\n");
        src.append("sdModel.setTIMESTEP("+dt+");\n");
        src.append("sdFunctions.setLeeaMode(true);\n");
		 return src.toString();
	}
	
	@Override
    public void  buildCode() {
    	if (polarityRoot == null) {
    		polarityRoot = equation.getCopyOfTree();
    		return;
    	}
    	
    	try {
			getRidOfRandom();
//			transformEquation();

			numIDs = insertIDs();
			generatedCode = new String(build());
//			System.out.println("\n#####\n");
//			System.out.println("EQN:   "+equation.getVensimEquationOnly());
//			System.out.println("P EQN: "+generatedCode);

		} catch (Exception e) {
			// TODO Auto-generated catch block
			return;
		}


    }
	
	@Override
    public String build() {
    	StringBuilder sb = new StringBuilder();
    	sb.append(replaceMathFunctions(equation.getLEEAEquationFromTree(polarityRoot)));
    	String sbString = sb.toString().replace("memory.Time", "time");
    	if (sbString.contains("DBREPLACELOOKUP")) {
    		sbString = replaceLookups(sbString);
    	}
    	return sbString;
    }
	
	private String replaceLookups(String s) {
		List<String> tokens = Parser.tokenize(s);
		StringBuilder sb = new StringBuilder();
		
		for (String tok : tokens) {
			if (tok.startsWith("DBREPLACELOOKUP")) {
				String taggedVar = tok.replace("DBREPLACELOOKUP ", "");
				String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(taggedVar);
				String asArray = InformationManagers.getInstance().getDataBridge().getLookupValuesAsString(origName);
				sb.append(asArray);
			} else {
				sb.append(tok);
			}
		}
		return sb.toString();
	}

}
