package gov.anl.gss.sdm.server.model.analysis.leea;

import gov.anl.gss.sdm.server.model.analysis.polarity.CodeBuilderInterface;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityCalculationResult;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResultAnalyzerInterface;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.analysis.polarity.RunnerInterface;

import java.util.List;

public class LeeaResultsAnalyzer implements PolarityResultAnalyzerInterface {
	
	private RunnerInterface runner;
	
	public LeeaResultsAnalyzer(RunnerInterface runner) {
		this.runner = runner;
	}

	@Override
	public void analyze(PolarityResults results) {
		// TODO Auto-generated method stub
		
		CodeBuilderInterface cb = runner.getCodeBuilder();
		LeeaArgumentBuilder lab = new LeeaArgumentBuilder(cb.getNumIDs(), cb.getIdToNameMap());
		runner.run(lab);

		// Did we actually run anything?
		
		if (runner.getArgumentBuilder() == null || runner.getArgumentBuilder().getNumArgs() == 0) {
			return;
		}
		List<PolarityCalculationResult> result = runner.getResults();
		
		int numResults = result.size();
		int numInputs = 0;
		if (numResults > 0) {
			numInputs = result.get(0).getNumInputs();
		}
		
		double[] analyzeTime = LeeaParameters.getAnalyzeTimeSteps();
		int numTime = analyzeTime.length;
		
		int index = 0;
		for (int time = 0; time < numTime; time++) {
			PolarityCalculationResult base = result.get(index++);
			PolarityCalculationResult[] pDeriv = new PolarityCalculationResult[numInputs];
			for (int arg = 0; arg < numInputs; arg++) {
				pDeriv[arg] = result.get(index++);
			}
			analyze(analyzeTime[time], base, pDeriv);
		}
	}
	
	private void analyze(double time, PolarityCalculationResult base, PolarityCalculationResult[] pDeriv) {
		
		for (int arg = 0; arg < pDeriv.length; arg++) {
			double deltaArg = pDeriv[arg].getInputs()[arg] - base.getInputs()[arg];
			double deltaLhs = pDeriv[arg].getLhs() - base.getLhs();
			double gain = deltaLhs / deltaArg;
			System.out.println("\n#####\n");
			
			
			System.out.println("ORIG: "+runner.getCodeBuilder().getOriginalEquation());
			System.out.println("SIMPL: "+runner.getCodeBuilder().getEquation());
			System.out.println("CONV: "+runner.getCodeBuilder().getGeneratedCode());
			System.out.println("\nTime: "+time+" arg = "+arg+"\n");
			System.out.println("Base: "+base);
			System.out.println("pDeriv "+pDeriv[arg]);
			System.out.println("deltaLHS = "+deltaLhs+" deltaArg = "+deltaArg + " gain = "+gain);
		}
		
	}

}
