package gov.anl.gss.sdm.server.model.analysis.leea;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArgumentBuilderInterface;
import gov.anl.gss.sdm.server.model.analysis.polarity.CodeBuilderInterface;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityCalculationResult;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.analysis.polarity.RunnerInterface;
import gov.anl.gss.sdm.server.model.dynamic.DynaComp;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class LeeaRunner implements RunnerInterface {
	
static Logger log = Logger.getLogger(LeeaRunner.class.getSimpleName());

	private CodeBuilderInterface codeBuilder;
	private PolarityResults polarityResults;
	private ArgumentBuilderInterface argumentBuilder; 
	
	private List<PolarityCalculationResult> results = new ArrayList<PolarityCalculationResult>();

	public LeeaRunner(CodeBuilderInterface codeBuilder) {
		this.codeBuilder = codeBuilder;
	}

	public PolarityResults run(ArgumentBuilderInterface argumentBuilder) {
		
		this.argumentBuilder = argumentBuilder;
		int numArgs = codeBuilder.getNumIDs();
		
		if (numArgs == 0) {
			return polarityResults;
		}
		
//		argumentBuilder = new ArgumentBuilder(numArgs, polarityCodeBuilder.getIdToNameMap());
		argumentBuilder.initialize();
		
		double[][] rhs = argumentBuilder.getArguments();
		double[] simTime = argumentBuilder.getSimTime();
		
		try {
			log.debug("");
			DynaComp dyna = new DynaComp(codeBuilder.getClassName(), codeBuilder.createClassSourceCode());
			Class[] cArg = new Class[2];
			cArg[0] = double[].class;
			cArg[1] = double.class;
			Object o;
			
				o = dyna.getInstance();
				Method m = o.getClass().getMethod("compute",cArg);
			

			for (int rhsArg = 0; rhsArg < rhs.length; rhsArg++) {
				double lhs = (Double) m.invoke(o,rhs[rhsArg],simTime[rhsArg]);
//				log.debug("Results: "+lhs+" "+inputs(rhs[rhsArg]));
				results.add(new PolarityCalculationResult(lhs, rhsArg, rhs[rhsArg]));
			}
			
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			log.debug("NoSuchMethodException");
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			log.debug("SecurityException");
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			log.debug("IllegalAccessException");
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			log.debug("IllegalArgumentException");
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			log.debug("InvocationTargetException");
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			log.debug("o = dyna.getInstance(); Exception");
			e.printStackTrace();
			return polarityResults;
		}

		return polarityResults;
	}

	private String inputs(double[] in) {
		StringBuilder sb = new StringBuilder();
		sb.append("inputs: ");
		for (int i = 0; i < in.length; i++) {
			if (i > 0)
				sb.append(", ");
			sb.append(in[i]);
		}
		
		return sb.toString();
	}

	public List<PolarityCalculationResult> getResults() {
		return results;
	}

	public CodeBuilderInterface getCodeBuilder() {
		return codeBuilder;
	}

	public PolarityResults getPolarityResults() {
		return polarityResults;
	}

	public ArgumentBuilderInterface getArgumentBuilder() {
		return argumentBuilder;
	}
}
