package gov.anl.gss.sdm.server.model.analysis.leea.gui;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;

import java.awt.Dimension;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;

public class MessagePanel extends JPanel implements MessageProcessor {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String BLIP = "#";
	private static final long BLIP_PAUSE = 3000L;
	
	private static final int SIZE_ADJUST_WIDTH = 30;
	private static final int SIZE_ADJUST_HEIGHT = 60;
	
	private static final int SIZE_ADJUST_BORDER = 15;
	
	private JPanel masterPanel;
//	private JEditorPane editorPane;
	private JTextPane editorPane;
	
	private boolean blipMode = false;
	private boolean blipModeSave = false;
	private int blipStart = 0;
	private Thread blipThread;
	
	public MessagePanel() {
		super();
		create();
		showPanel();
	}
	

	public void setCaretPosition(int pos) {
		editorPane.setCaretPosition(pos);
	}
	
	public void viewTop() {
		setCaretPosition(0);
	}
	
	public void create() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		masterPanel = new JPanel();
		
		editorPane = new JTextPane();
		
		DefaultCaret caret = (DefaultCaret)editorPane.getCaret();
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		
		masterPanel.setPreferredSize(new Dimension(StandaloneGUI.MESSAGES_WIDTH, 
				StandaloneGUI.MESSAGES_HEIGHT-SIZE_ADJUST_BORDER));
		masterPanel.setMinimumSize(new Dimension(StandaloneGUI.MESSAGES_WIDTH, 
				StandaloneGUI.MESSAGES_HEIGHT-SIZE_ADJUST_BORDER));
		
		editorPane.setEditable(false);
		JScrollPane jsp = new JScrollPane(editorPane);
		jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		jsp.setPreferredSize(new Dimension(StandaloneGUI.MESSAGES_WIDTH-SIZE_ADJUST_WIDTH, 
				StandaloneGUI.MESSAGES_HEIGHT-SIZE_ADJUST_HEIGHT));
		jsp.setMinimumSize(new Dimension(StandaloneGUI.MESSAGES_WIDTH-SIZE_ADJUST_WIDTH, 
				StandaloneGUI.MESSAGES_HEIGHT-SIZE_ADJUST_HEIGHT));
		jsp.setMaximumSize(new Dimension(StandaloneGUI.MESSAGES_WIDTH-SIZE_ADJUST_WIDTH, 
				StandaloneGUI.MESSAGES_HEIGHT-SIZE_ADJUST_HEIGHT));
		masterPanel.add(jsp, "Center");
		add(masterPanel);
		
	}
	
	public void addMsgs(List<String> list) {
		for (String text: list) {
			addMsg(text+"\n");
		}
	}
	
	
	public void addMsg(String text) {
		try {
			
		      Document doc = editorPane.getDocument();
		      doc.insertString(doc.getLength(), text, null);
		   } catch(BadLocationException exc) {
		      exc.printStackTrace();
		   }
	

	}
	
	public void blip() {
		if (blipMode)
			addMsg(BLIP);
	}
	
	public void addTimeStamp() {
		String timeStamp = new Date().toString();
		addMsg("\n#####  "+timeStamp+"  #####");
		
	}
	
	public void addMessage(String text) {

		blipModeSave = blipMode;
		// if in blip mode, do a clear first
		if (blipMode) {
			blipClear();
		}
		
		String pattern = "HH:mm:ss";
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);

		String date = simpleDateFormat.format(new Date());



		addMsg("\n"+date+" | "+text);
		blipMode = blipModeSave;
		// need to define the new blipStart
		if (blipMode) {
			Document doc = editorPane.getDocument();
			blipStart = doc.getLength();
		}
		
		if (!masterPanel.isVisible())
			showPanel();
		pause(20L);



	}
	
	public void addMessageNP(String text) {
		
		addMsg(text);
		
//		pause(250L);
//		editorPane.revalidate();
//	      editorPane.repaint();
	}
	
	public void addMessageNPNoView(String text) {
		addMsg(text);
		this.setVisible(false);
//		pause(250L);
//		editorPane.revalidate();
//	      editorPane.repaint();
	}
	
	public void pause(long t) {
		try {
		    Thread.sleep(t);                 //1000 milliseconds is one second.
		} catch(InterruptedException ex) {
		    Thread.currentThread().interrupt();
		}
	}
	
	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
	}
	
	public void disposeMessageFrame() {
//		this.dispose();
		hidePanel();
	}
	
	public void showPanel() {
		masterPanel.setVisible(true);
	}
	
	public void hidePanel() {
		masterPanel.setVisible(false);
	}
	public boolean isBlipMode() {
		return blipMode;
	}
	public void setBlipMode(boolean blipMode) {
		
		if (!blipMode && !this.blipMode)
			return;
		
		this.blipMode = blipMode;
		
		Document doc = editorPane.getDocument();
		if (blipMode) {
			
			if (blipThread != null)
				blipThread.interrupt();
			
			blipStart = doc.getLength();
			
			blipThread = new Thread(new Runnable() {
			
			        public void run() {
			            try {
			                while (!Thread.currentThread().isInterrupted()) {
			                    Thread.sleep(BLIP_PAUSE);
			                    blip();
			                }
			            } catch (InterruptedException e) {
//			                Thread.currentThread().interrupt();
			            }
			        }
			    });
			    blipThread.start();
			
		} else {
			if (blipThread != null)
				blipThread.interrupt();
			blipClear();
			blipThread = null;
		}
	}
	
	public void blipClear() {
		Document doc = editorPane.getDocument();
		int len = doc.getLength() - blipStart;
		try {
			doc.remove(blipStart, len);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
		}
	}

	@Override
	public void endTask() {
		
		blipModeSave = blipMode;
		// if in blip mode, do a clear first
		if (blipMode) {
			blipClear();
		}
		addMsg("\n");
		blipMode = blipModeSave;
		// need to define the new blipStart
		if (blipMode) {
			Document doc = editorPane.getDocument();
			blipStart = doc.getLength();
		}
		
		if (!masterPanel.isVisible())
			showPanel();
		pause(30L);
		
	}


	@Override
	public void create(int xLoc, int yLoc) {
		// TODO Auto-generated method stub
		
	}


}
