package gov.anl.gss.sdm.server.model.analysis.leea.gui;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.xmile.XmileReader;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingWorker;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class ModelControlButtonsPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public final static String[] RECOGNIZED_FILE_TYPES = {"mdl", "xmile", "stmx"};
	private static String FC_DIR_FILE = "data/ChooserDir.txt";
	
	private JButton selectButton;
	private JButton runButton;
	private JButton loadButton;
	private JLabel then;
	private JLabel or;
	
	private StandaloneGUI gui;
	
	public ModelControlButtonsPanel(StandaloneGUI gui) {
		super();
		this.gui = gui;
		GridBagLayout grid = new GridBagLayout();
		setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		
		setBorder(BorderFactory.createTitledBorder(null, "", TitledBorder.LEFT , 
				TitledBorder.TOP,  new Font("Arial", Font.BOLD,12) , Color.black));
		
		makeButtons();
		makeLabels();
		
		c.insets = new Insets(3,10,3,10); // top padding
		c.fill = GridBagConstraints.CENTER;
		c.weightx = .50;
		c.weighty = .50;

		c.gridwidth = 1;
		c.gridy = 0;
		
		c.gridx = 0;
		add(selectButton, c);
		
		c.gridx++;
		add(then, c);
		
		c.gridx++;
		add(runButton, c);
		
		c.gridx++;
		add(or, c);
		
		c.gridx++;
		add(loadButton, c);

		
		
	}
	
	private void makeButtons() {
		
		selectButton = new JButton("Select Model");
		runButton = new JButton("Run Model");
		loadButton = new JButton("Load Vensim Data");
		
		selectButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter("Model Files", RECOGNIZED_FILE_TYPES);
				JFileChooser chooser = new JFileChooser(loadFCDir());
				
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(200, 200);


				int option = chooser.showOpenDialog(selectButton);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					saveFCDir(dir);
					gui.setModelFile(sf.getAbsolutePath());
					MessageProcessor mf = gui.getMessageProcessor();
					mf.addMessage("Selected File: "+gui.getModelFile());
				} else {
				}
			}
		});
		
		runButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific
						.getInstance();
				//				final MDLProcessor sdmDoc = null;
				boolean internalError = false;

				if (gui.getModelFile().length() == 0) {
					JOptionPane.showMessageDialog(null,
							language.getText(StringConstants.PLEASE_SPECIFY_SYSTEM_DYNAMICS_MODEL_FILENAME),
							language.getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
				} else if (!new File(gui.getModelFile()).exists()) {
					JOptionPane.showMessageDialog(null,
							language.getText(StringConstants.SPECIFIED_SYSTEM_DYNAMICS_MODEL_NOT_FOUND),
							language.getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
				} else {
					try {

						gui.setCursor(Cursor
								.getPredefinedCursor(Cursor.WAIT_CURSOR));
						MessageProcessor mf = gui.getMessageProcessor();

						String originalModelFile = gui.getModelFile();
						String modelFile = gui.getModelFile();

						boolean xmileOrigin = false;

						if (Utilities.isOfXmlOrigin(modelFile)) {

							//						if (Utilities.stringEndsWith(modelFile, "\\.[xX][mM][iI][lL][eE]$")) {
							xmileOrigin = true;
							mf.addMessage(" ### "+language.getText(StringConstants.TRANSLATING)+" "
									+ new File(modelFile).getName()
									+ " ###\n");
							XmileReader xmileReader = new XmileReader(modelFile);
							xmileReader.readAndTranslate();
							//							modelFile = modelFile.replaceAll("\\.[xX][mM][iI][lL][eE]$","_xmile.mdl");
							modelFile = Utilities.getConvertedFileName(modelFile);
						}



						mf.addMessage(" ### "
								+ new File(modelFile).getName()
								+ " ###\n");

						mf.addMessage(language
								.getText(StringConstants.CONSTRUCTOR) + "...\n");

						mf.setBlipMode(true);

						MessageProcessor ef = mf;

						String obj = Utilities.makeLegalJavaName(new File(modelFile).getName().replace(".MDL", "").replace(".mdl", "").toUpperCase());

						String root = new File(gui.getModelFile()).getParent();

						final MDLProcessor sdmDoc = new MDLProcessor(originalModelFile, modelFile, obj, root, mf, ef, getUserSelectedOptions());

						mf.setBlipMode(false);
						mf.addMessage(language
								.getText(StringConstants.EXECUTING) + "...\n");



						try {
							mf.setBlipMode(true);
							final MessageProcessor msgProc = mf;

							SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
								@Override
								protected Void doInBackground() throws Exception {


									try {
										String out = sdmDoc.getHTMLFileName();
										File f = new File(out);
										if (f.exists())
											f.delete();
										sdmDoc.processSDA(false);
									} catch (Exception e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									String out = sdmDoc.getHTMLFileName();
									if (!new File(out).exists()) {
										JOptionPane.showMessageDialog(null,
												SDMDocLanguageSpecific.getInstance().
												getText(StringConstants.HTML_REPORT_NOT_GENERATED),
												SDMDocLanguageSpecific.getInstance().
												getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
									} else {
										//										if (Desktop.isDesktopSupported()) {
										//											msgProc.addMessage(SDMDocLanguageSpecific.getInstance().
										//													getText(StringConstants.OPENING_HTML_IN_BROWSER));
										//											try {
										//												Desktop.getDesktop().browse(
										//														new File(out).toURI());
										//											} catch (IOException e) {
										//												// TODO Auto-generated catch block
										//												e.printStackTrace();
										//											}
										//										}
										gui.setModelRun(true);
										gui.updateWithModelRun(sdmDoc);
										
									} 

									return null;

								}


							};


							worker.execute();

						} catch (Exception e) {

							String errMsg = e.toString();
							String myMsg = "\n\n"+
									language.getText(StringConstants.JAVA_ERROR_MESSAGES)+"\n\n";
							//							log.log(Level.FATAL, "Internal Error Detected", e);
							mf.addMessage(errMsg + "\n");
							mf.setBlipMode(false);
							JOptionPane.showMessageDialog(null, errMsg + myMsg,
									language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
									JOptionPane.ERROR_MESSAGE);
							e.printStackTrace();
							internalError = true;
							mf.setBlipMode(false);

						}
					} finally {
						gui.setCursor(Cursor
								.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				}
			}
		});
		
		loadButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
			}
		});
		
	}
	
	private UserSelectedOptions getUserSelectedOptions() {
		return new UserSelectedOptions(false); // false = do not load from file
	}
	
	private void makeLabels() {
		
		then = new JLabel("then");
		or = new JLabel("or");
		
		then.setFont(new Font("Arial", Font.BOLD,12));
		or.setFont(new Font("Arial", Font.BOLD,12));
	}
	
	private void setDependencies() {
		
	}
	
	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

}
