package gov.anl.gss.sdm.server.model.analysis.leea.gui;

import gov.anl.gss.sdm.server.model.analysis.EquationManager;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.TypeCounter;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.tools.SDModel;
import gov.anl.gss.sdm.server.model.tools.SDModelProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

public class ModelInformationRaw {
	
	private MDLProcessor sdmDoc;
	
	private String modelName;
	private String modelData;
	
	private int levels;
	private int variables;
	private int constants;
	private int tableFunctions;
	private int data;
	private String initialTime;
	private String finalTime;
	private int nLevels;
	private int nEdges;
	private int nStrongComp;
	private int nEdgesStrongComp;
	private int nNodes;
	private long nLoops;
	private int nGeodeticLoops;

	public ModelInformationRaw() {
		
	}
	
	public ModelInformationRaw(MDLProcessor sdmDoc) {
		this.sdmDoc = sdmDoc;
		build();
	}
	
	private void build() {
		SDModelProcessor sdmProc = sdmDoc.getSdmProc();
		SDModel sdModel = sdmProc.getSdModel();
		
		EquationManager em = InformationManagers.getInstance().getEquationManager();
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		
		setModelName(mm.getMdlName());
		setModelData("???");
		
		setInitialTime(sdModel.getStartTime());
		setFinalTime(sdModel.getEndTime());
		
		Documentor d = InformationManagers.getInstance().getAnalysisManager().getDocumentor();
		TypeCounter tc = d.getTypeCounter();
		
		setLevels(tc.getLevel());
		setVariables(tc.getAllTypes()-tc.getLevel());
		setConstants(tc.getConstant());
		setTableFunctions(tc.getLookup());
		setData(-1);
		
		setnLevels(tc.getLevel());
		setnEdges(d.getAnalysisManager().getPolarityResultsNoIVV().getArrows().size());
		setnStrongComp(-1);
		setnEdgesStrongComp(-1);
		setnNodes(tc.getAllTypes());
		setnLoops(InformationManagers.getInstance().getAnalysisManager().getLoopManager().getNumberOfLoopsNoIVV());
		setnGeodeticLoops(-1);

	}

	public void setModelName(String modelName) {
		this.modelName = modelName;
	}

	public void setModelData(String modelData) {
		this.modelData = modelData;
	}

	public void setLevels(int levels) {
		this.levels = levels;
	}

	public void setVariables(int variables) {
		this.variables = variables;
	}

	public void setConstants(int constants) {
		this.constants = constants;
	}

	public void setTableFunctions(int tableFunctions) {
		this.tableFunctions = tableFunctions;
	}

	public void setData(int data) {
		this.data = data;
	}

	public void setInitialTime(String initialTime) {
		this.initialTime = initialTime;
	}

	public void setFinalTime(String finalTime) {
		this.finalTime = finalTime;
	}

	public void setnLevels(int nLevels) {
		this.nLevels = nLevels;
	}

	public void setnEdges(int nEdges) {
		this.nEdges = nEdges;
	}

	public void setnStrongComp(int nStrongComp) {
		this.nStrongComp = nStrongComp;
	}

	public void setnEdgesStrongComp(int nEdgesStrongComp) {
		this.nEdgesStrongComp = nEdgesStrongComp;
	}

	public void setnNodes(int nNodes) {
		this.nNodes = nNodes;
	}

	public void setnLoops(long nLoops) {
		this.nLoops = nLoops;
	}

	public void setnGeodeticLoops(int nGeodeticLoops) {
		this.nGeodeticLoops = nGeodeticLoops;
	}

	public String getModelName() {
		return modelName;
	}

	public String getModelData() {
		return modelData;
	}

	public int getLevels() {
		return levels;
	}

	public int getVariables() {
		return variables;
	}

	public int getConstants() {
		return constants;
	}

	public int getTableFunctions() {
		return tableFunctions;
	}

	public int getData() {
		return data;
	}

	public String getInitialTime() {
		return initialTime;
	}

	public String getFinalTime() {
		return finalTime;
	}

	public int getnLevels() {
		return nLevels;
	}

	public int getnEdges() {
		return nEdges;
	}

	public int getnStrongComp() {
		return nStrongComp;
	}

	public int getnEdgesStrongComp() {
		return nEdgesStrongComp;
	}

	public int getnNodes() {
		return nNodes;
	}

	public long getnLoops() {
		return nLoops;
	}

	public int getnGeodeticLoops() {
		return nGeodeticLoops;
	}

}
