package gov.anl.gss.sdm.server.model.analysis.leea.gui;


import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;

import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.log4j.PropertyConfigurator;

public class StandaloneGUI  extends JFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public final static String version = "0.1";

	public final static int DASHBOARD_WIDTH = 1400;
	public final static int DASHBOARD_HEIGHT = 1200;

	public final static double COLUMN_WIDTH_FRACTION_LEFT = .40;
	public final static double ROW_1_HEIGHT_FRACTION = .30; // 30
	public final static double ROW_2_HEIGHT_FRACTION = .55; // 55
	public final static double ROW_3_HEIGHT_FRACTION = .15; // 15

	public final static double ROW_3_WIDTH = DASHBOARD_WIDTH;


	public final static int MODEL_CONTROL_INFORMATION_LENGTH = 23;

	// ===========================================================

	public final static double COLUMN_WIDTH_FRACTION_RIGHT = 1.0 - COLUMN_WIDTH_FRACTION_LEFT;


	public final static int COLUMN_WIDTH_LEFT = (int) (DASHBOARD_WIDTH * COLUMN_WIDTH_FRACTION_LEFT);
	public final static int COLUMN_WIDTH_RIGHT = DASHBOARD_WIDTH - COLUMN_WIDTH_LEFT;
	public final static int ROW_1_HEIGHT = (int) (DASHBOARD_HEIGHT * ROW_1_HEIGHT_FRACTION);
	public final static int ROW_2_HEIGHT = (int) (DASHBOARD_HEIGHT * ROW_2_HEIGHT_FRACTION);

	// ===========================================================

	public final static int MODEL_CONTROL_GRID_WIDTH = 1;
	public final static int SDA_CONTROL_GRID_WIDTH = 1;
	public final static int BEHAVIOR_GRID_WIDTH = 1;
	public final static int SDA_OUTPUT_GRID_WIDTH = 1;

	// ===========================================================

	public final static int MODEL_CONTROL_BUTTONS_HEIGHT = 50;

	public final static int MODEL_CONTROL_WIDTH = COLUMN_WIDTH_LEFT;
	public final static int MODEL_CONTROL_HEIGHT = ROW_1_HEIGHT;

	public final static int MODEL_CONTROL_BUTTONS_WIDTH = MODEL_CONTROL_WIDTH;


	public final static int MODEL_CONTROL_INFORMATION_WIDTH = MODEL_CONTROL_WIDTH;
	public final static int MODEL_CONTROL_INFORMATION_HEIGHT = MODEL_CONTROL_HEIGHT - MODEL_CONTROL_BUTTONS_HEIGHT;

	// ===========================================================

	public final static int SDA_CONTROL_DDWA_BUTTONS_HEIGHT = 400;
	public final static int SDA_CONTROL_LEEA_BUTTONS_HEIGHT = ROW_2_HEIGHT - SDA_CONTROL_DDWA_BUTTONS_HEIGHT;

	public final static int SDA_CONTROL_WIDTH = COLUMN_WIDTH_LEFT;
	public final static int SDA_CONTROL_HEIGHT = ROW_2_HEIGHT;

	public final static int SDA_CONTROL_DDWA_BUTTONS_WIDTH = SDA_CONTROL_WIDTH/2;


	public final static int SDA_CONTROL_TIME_BUTTONS_WIDTH = SDA_CONTROL_WIDTH/2;
	public final static int SDA_CONTROL_TIME_BUTTONS_HEIGHT = SDA_CONTROL_DDWA_BUTTONS_HEIGHT;

	public final static int SDA_CONTROL_LEEA_BUTTONS_WIDTH = SDA_CONTROL_WIDTH/2;


	public final static int SDA_CONTROL_TIME_POINTS_WIDTH = SDA_CONTROL_WIDTH/2;
	public final static int SDA_CONTROL_TIME_POINTS_HEIGHT = SDA_CONTROL_LEEA_BUTTONS_HEIGHT;



	// ===========================================================

	public final static int BEHAVIOR_WIDTH = COLUMN_WIDTH_RIGHT;
	public final static int BEHAVIOR_HEIGHT = MODEL_CONTROL_HEIGHT;

	public final static int BEHAVIOR_BUTTONS_WIDTH = 200;
	public final static int BEHAVIOR_BUTTONS_HEIGHT = BEHAVIOR_HEIGHT-20;

	public final static int BEHAVIOR_GRAPH_WIDTH = BEHAVIOR_WIDTH - BEHAVIOR_BUTTONS_WIDTH;
	public final static int BEHAVIOR_GRAPH_HEIGHT = BEHAVIOR_HEIGHT-20;

	// ===========================================================

	public final static int SDA_OUTPUT_WIDTH = COLUMN_WIDTH_RIGHT;
	public final static int SDA_OUTPUT_HEIGHT = SDA_CONTROL_HEIGHT;

	// ===========================================================

	public final static int MESSAGES_WIDTH = DASHBOARD_WIDTH;
	public final static int MESSAGES_HEIGHT = DASHBOARD_HEIGHT - MODEL_CONTROL_HEIGHT - SDA_CONTROL_HEIGHT;

	private String modelFile;

	private ModelControlPanel mcp;
	private VariablesBehaviorPanel bvp;
	private SDAControlPanel sdacp;
	private SDAOutputPanel sdaop;
	private MessagesPanel mp;
	
	private boolean modelRun = false;


	void createAndShowGUI() {

		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// load all language specific text strings into the static elements of the class
		SDMDocLanguageSpecific.initialize();

		this.setTitle(
				"SDA" + "/Java "
						+ "Version: " + version
						+ (System.getProperty("sun.arch.data.model").equals("32") ? " (x86) " : " (x64) "));

		setPreferredSize(new Dimension(DASHBOARD_WIDTH, DASHBOARD_HEIGHT));
		setMinimumSize(new Dimension(DASHBOARD_WIDTH, DASHBOARD_HEIGHT));


		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE); // JFrame.EXIT_ON_CLOSE  JFrame.DO_NOTHING_ON_CLOSE

		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				System.exit(0);
			}
		});

		GridBagLayout grid = new GridBagLayout();
		setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();

		c.gridwidth = 1;

		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(3,5,3,5); // top padding
		//		c.fill = GridBagConstraints.CENTER;
		c.weightx = .50;
		c.weighty = .50;

		c.gridx = 0;
		c.gridy = 0;
		mcp = new ModelControlPanel(this);
		add(mcp, c);

		c.gridx = 1;
		c.gridy = 0;
		bvp = new VariablesBehaviorPanel(this);
		add(bvp ,c);

		c.gridx = 0;
		c.gridy = 1;
		sdacp = new SDAControlPanel(this);
		add(sdacp, c);

		c.gridx = 1;
		c.gridy = 1;
		sdaop = new SDAOutputPanel(this);
		add(sdaop, c);


		c.gridwidth = 2;
		c.gridx = 0;
		c.gridy = 2;
		mp = new MessagesPanel(this);
		add(mp,c);


		// #################################

		// Display the window.
		pack();
		setVisible(true);
		setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
	}
	
	public void updateWithModelRun(MDLProcessor sdmDoc) {
		mcp.updateWithModelRun(sdmDoc);
		bvp.updateWithModelRun(sdmDoc);
		sdacp.updateWithModelRun(sdmDoc);
		sdaop.updateWithModelRun(sdmDoc);
		updateWithChange();
	}
	
	public void updateWithChange() {
		invalidate();
		validate();
	}
	
	public void setSelectectModel(String s) {
		mcp.setSelectecdModel(s);
		invalidate();
		validate();
	}

	public MessageProcessor getMessageProcessor() {

		return mp.getMessageProcessor();
	}
	

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.

		String log4jConfPath = "./data/log4j.properties";
		PropertyConfigurator.configure(log4jConfPath);
		StaticSwitches.initialize();

		final StandaloneGUI gui = new StandaloneGUI();
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}



	public String getModelFile() {
		return modelFile;
	}



	public boolean isModelRun() {
		return modelRun;
	}

	public void setModelRun(boolean modelRun) {
		this.modelRun = modelRun;
	}

	public void setModelFile(String modelFile) {
		this.modelFile = modelFile;
		this.setSelectectModel(new File(modelFile).getName());
	}
}
