package gov.anl.gss.sdm.server.model.analysis.leea.gui;

import gov.anl.gss.sdm.server.model.analysis.EquationManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.tools.SDModelProcessor;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

public class VariablesBehaviorButtonsPanel   extends JPanel{

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private String title = "Behavior of Model Variables";
	private JComboBox<String> levelsList = new JComboBox<String>();
	private JLabel levelsLabel = new JLabel("Levels");
	private JComboBox<String> variablesList = new JComboBox<String>();
	private JLabel variablesLabel = new JLabel("Variables");
	private JButton export = new JButton("Export as PDF");
	
	private StandaloneGUI gui;
	
	private ActionListener levelsAL;
	private ActionListener variablesAL;
	

	public VariablesBehaviorButtonsPanel(StandaloneGUI gui, final VariablesBehaviorPanel parent, MDLProcessor sdmDoc) {
		super();
		this.gui = gui;
		setBorder(BorderFactory.createTitledBorder(null, "", TitledBorder.LEFT , 
				TitledBorder.TOP,  new Font("Arial", Font.BOLD,12) , Color.black));
		
		setPreferredSize(new Dimension(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, StandaloneGUI.BEHAVIOR_BUTTONS_HEIGHT-50));
		setMinimumSize(new Dimension(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, StandaloneGUI.BEHAVIOR_BUTTONS_HEIGHT-50));
		levelsLabel.setFont(new Font("Arial", Font.BOLD,12));
		variablesLabel.setFont(new Font("Arial", Font.BOLD,12));
		
		levelsAL = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				String variable = levelsList.getSelectedItem().toString();
				parent.setBehaviorImage(variable);
			}
		};
		
		variablesAL = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
					String variable = variablesList.getSelectedItem().toString();
					parent.setBehaviorImage(variable);
				}
		};
		
		levelsList.addActionListener(levelsAL);
		variablesList.addActionListener(variablesAL);
		
		levelsList.setSize(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, 25);
		levelsList.setPreferredSize(new Dimension(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, 25));
		
		variablesList.setSize(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, 25);
		variablesList.setPreferredSize(new Dimension(StandaloneGUI.BEHAVIOR_BUTTONS_WIDTH, 25));
		
		export.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
			}
		});
		
		if (sdmDoc == null) {
			makeUpData();
		} else {
			useRealData(sdmDoc);
		}
		GridBagLayout grid = new GridBagLayout();
		setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		
		c.insets = new Insets(3,10,3,10); // top padding
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = .50;
		c.weighty = .0;
		
		c.gridwidth = 1;
		c.gridy = 0;
		c.gridx = 0;
		
		add(levelsLabel, c);
		c.gridy++;
		add(levelsList, c);
		c.gridy++;
		
		add(variablesLabel, c);
		c.gridy++;
		add(variablesList, c);
		c.gridy++;

		c.insets = new Insets(50,10,10,10); // top padding
		add(export, c);
		
		disablePanel();
	}
	
	public void enablePanel() {
		levelsList.setEnabled(true);
		variablesList.setEnabled(true);
		export.setEnabled(true);
	}
	
	public void disablePanel() {
		levelsList.setEnabled(false);
		variablesList.setEnabled(false);
		export.setEnabled(false);
	}
	
	private void useRealData(MDLProcessor sdmDoc) {

		SDModelProcessor sdmProc = sdmDoc.getSdmProc();

		EquationManager em = InformationManagers.getInstance().getEquationManager();
		
		levelsList.removeActionListener(levelsAL);
		variablesList.removeActionListener(variablesAL);
		
		for (String v : Utilities.ordered(em.getVariablesNoSystem())) {
			Equation eqn = em.getEquationFor(v);
			if (eqn.isStock())
				levelsList.addItem(v);
			else
				variablesList.addItem(v);
		}
		
		levelsList.addActionListener(levelsAL);
		variablesList.addActionListener(variablesAL);

	}
	
	private void makeUpData() {
		levelsList.addItem(" ");		
		variablesList.addItem(" ");
		
	}

}
