package gov.anl.gss.sdm.server.model.analysis.loops;

import java.io.File;

import gov.anl.gss.sdm.server.model.documentor.gui.JCheckBoxLanguage;
import gov.anl.gss.sdm.server.model.documentor.gui.JRadioButtonLanguage;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

public class CyclesConfiguration {
	
	// these two parameters control whether cycles are recorded in memory
	// or written to a file (or both) (or neither)
	private boolean storeCyclesInMemory = false;
	private boolean writeCyclesToFile = true;
	private boolean processCyclesInMemory = false;
	private boolean processCycleWhenFound = true;
	
	private JRadioButtonLanguage summaryReportRB = new JRadioButtonLanguage(StringConstants.SUMMARY_REPORTS);
	private JRadioButtonLanguage detailedReportRB = new JRadioButtonLanguage(StringConstants.DETAILED_REPORTS);
	
	private final JCheckBoxLanguage writeLoopsToFileCB = new JCheckBoxLanguage(StringConstants.WRITE_LOOPS_TO_FILE);
	
	private CyclesFile cyclesFile;
	
	private String filenameNoIVV; // InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory()+"/cycles.dat"
	private String filenameWithIVV;
	public CyclesConfiguration() {
		filenameNoIVV = InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory()+"/cyclesNoIVV.dat";
		filenameWithIVV = InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory()+"/cyclesWithIVV.dat";
		
		setCyclesFile(new CyclesFile(getFilename(false)));
		
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();
		if (options == null) {
			options = new UserSelectedOptions(true);
			InformationManagers.getInstance().setUserSelectedOptions(options);
		}
			writeCyclesToFile = options.isSelected(StringConstants.WRITE_LOOPS_TO_FILE); // true;      // options.isSelected(StringConstants.WRITE_LOOPS_TO_FILE); // true;
			storeCyclesInMemory = options.isSelected(StringConstants.DETAILED_REPORTS); // false;     // options.isSelected(StringConstants.DETAILED_REPORTS); // false;
			
			processCyclesInMemory = options.isSelected(StringConstants.DETAILED_REPORTS); // false ;  // options.isSelected(StringConstants.DETAILED_REPORTS); // true;
			processCycleWhenFound = options.isSelected(StringConstants.SUMMARY_REPORTS);//true;   // options.isSelected(StringConstants.SUMMARY_REPORTS);
	}
	
	public void configureForStanaloneLoopLengthAnalysis() {
		writeCyclesToFile = true;      // options.isSelected(StringConstants.WRITE_LOOPS_TO_FILE); // true;
		storeCyclesInMemory = false;     // options.isSelected(StringConstants.DETAILED_REPORTS); // false;
		
		processCyclesInMemory =  false ;  // options.isSelected(StringConstants.DETAILED_REPORTS); // true;
		processCycleWhenFound = true;   // options.isSelected(StringConstants.SUMMARY_REPORTS);
	}

	public boolean isRequiresLoadAndProcess() {
		return !storeCyclesInMemory && !processCyclesInMemory && !processCycleWhenFound && writeCyclesToFile;
	}
	public boolean isStoreCyclesInMemory() {
		return storeCyclesInMemory;
	}

	public void setStoreCyclesInMemory(boolean storeCyclesInMemory) {
		this.storeCyclesInMemory = storeCyclesInMemory;
	}

	public boolean isProcessCyclesInMemory() {
		return processCyclesInMemory;
	}

	public void setProcessCyclesInMemory(boolean processCyclesInMemory) {
		this.processCyclesInMemory = processCyclesInMemory;
	}

	public boolean isProcessCycleWhenFound() {
		return processCycleWhenFound;
	}

	public void setProcessCycleWhenFound(boolean processCycleWhenFound) {
		this.processCycleWhenFound = processCycleWhenFound;
	}

	public boolean isWriteCyclesToFile() {
		return writeCyclesToFile;
	}

	public void setWriteCyclesToFile(boolean writeCyclesToFile) {
		this.writeCyclesToFile = writeCyclesToFile;
	}
	
	public String getFilename(boolean withIVV) {
		return withIVV ? getFilenameWithIVV() :	getFilenameNoIVV();
	}

	public String getFilenameNoIVV() {
		return filenameNoIVV;
	}

	public void setFilenameNoIVV(String filename) {
		this.filenameNoIVV = filename;
	}
	
	public String getFilenameWithIVV() {
		return filenameWithIVV;
	}

	public void setFilenameWithIVV(String filename) {
		this.filenameWithIVV = filename;
	}
	
	public boolean fileExists(boolean withIVV) {
		File f = new File(getFilename(withIVV));
		return f.exists();
	}

	public CyclesFile getCyclesFile() {
		return cyclesFile;
	}

	public void setCyclesFile(CyclesFile cyclesFile) {
		this.cyclesFile = cyclesFile;
	}

}
