package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

public class CyclesFile {
	
	private static final String CONDENSED_MARKER = "#";
	
	private String fileName;
	private BufferedWriter bw = null;
	private BufferedReader br = null;
	
	
	private Vector previousLoop = null;

	public CyclesFile(String fileName) {
		this.fileName = fileName;
	}
	
	public Vector readSingle() {
		return read();
	}
	
	public boolean openForWrite() {
		try {
			if (bw != null)
				return false;
			if (br != null) {
				br.close();
				br = null;
			}
			bw = Utilities.openFileForWriting(fileName);
			return true;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
	}
	
	public boolean openForRead(String f) {
		try {
			if (br != null)
				return false;
			if (bw != null) {
				bw.close();
				bw = null;
			}
			br = Utilities.openForRead(f);
			return true;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
	}
	
	public boolean openForRead() {
		return openForRead(fileName);
	}
	
	public boolean closeRead() {
		try {
			if (br != null) {
				br.close();
				br = null;
				
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	
	public boolean closeWrite() {
		try {
			if (bw != null) {
				bw.close();
				bw = null;
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	private Vector condense(Vector loop) {
		
		// "condense" will compare this loop and the previous loop
		// and replace the leading elements identical between the two
		// with the CONDENSED_MARKER and a count of the number of identical items
		
		Vector condensed = new Vector();
		
		// if this is the first loop, just set as previous and return it as is
		if (previousLoop == null) {
			previousLoop = new Vector(loop);
			return loop;
		}
		
		// search through the loop sequentially looking for identical entries in the same position
		
		int minSize = Math.min(previousLoop.size(), loop.size());
		
		int identical = 0;
		for (int i = 0; i < minSize; i++) {
			if (loop.get(i).equals(previousLoop.get(i))) {
				identical++;
			} else {
				break;
			}
			
		}
		
		if (identical > 0) {
			condensed.add(CONDENSED_MARKER+identical);
			for (int i = identical; i < loop.size(); i++) {
				condensed.add(loop.get(i));
			}
			
			previousLoop =  new Vector(loop);
			return condensed;
		} else {
			previousLoop =  new Vector(loop);
			return loop;
		}
		
		
	}
	
	private Vector expand(Vector condensed) {
		Vector expanded = new Vector();
		
		if (condensed.get(0).toString().startsWith(CONDENSED_MARKER)) {
			
			int identical = Integer.parseInt(((String) condensed.get(0)).substring(1));
			for (int i = 0; i < identical; i++) {
				if (i >= previousLoop.size()) {
					System.out.println("BAD EXPAND");
				}
				expanded.add(previousLoop.get(i));
			}
			for (int i = 1; i < condensed.size(); i++) {
				expanded.add(condensed.get(i));
			}
			
			previousLoop = new Vector(expanded);
			return expanded;
		} else {
			previousLoop = new Vector(condensed);
			return condensed;
		}
		
		
	}
	
	public boolean write(Vector vector) {
		
		try {
			bw.append(condense(vector).toString().replace(" ", "")+"\n");
			return true;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
	}
	
	public Vector read() {
		String aLine;
		try {
			aLine = br.readLine();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
		if (aLine == null)
			return null;
		String[] v = aLine.replaceAll(" ", "").replace("[", "").replace("]",  "").split(",");
		Vector cycle = new Vector();
		for (int j = 0; j < v.length; j++) {
			cycle.add(v[j]);
		}
		return expand(cycle);
		
	}
	
	public List<List<String>> loadCyclesFromFile(String f, int maxLen) {
		openForRead(f);
		List<List<String>> allCycles = new ArrayList<List<String>>();
		List<String> aCycle = read();
		while(aCycle != null) {
			if (aCycle.size() <= maxLen)
				allCycles.add(aCycle);
			aCycle = read();
		}
		return allCycles;
		
	}
}
