package gov.anl.gss.sdm.server.model.analysis.loops;

import java.util.ArrayList;
import java.util.List;

import de.normalisiert.utils.graphs.ElementaryCyclesSearch;
import edu.uci.ics.jung.algorithms.matrix.GraphMatrixOperations;
import edu.uci.ics.jung.graph.DirectedSparseGraph;
import edu.uci.ics.jung.graph.util.EdgeType;
import edu.uci.ics.jung.graph.util.Pair;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.VariableManager;


public class GraphCyclesThread extends Thread /*implements Runnable*/ {

	private static int cnt = 0;
	
	private DirectedSparseGraph<String,String> g = null;
	
	private List<List<String>> cycles;
	private List<List<String>> vars = null;
	private int maxLength;
	private MessageProcessor mf;
	private long numCycles;
	ElementaryCyclesSearch ecs;
	boolean myInterrupted = false;
	private CyclesConfiguration config;
	private boolean withIVV;
	
	public long getNumCycles() {
		return numCycles;
	}

	public GraphCyclesThread(List<String[]> th, int maxLength, MessageProcessor mf,
			CyclesConfiguration config, boolean withIVV) {
		super("graphCycle"+cnt);
		this.withIVV = withIVV;
		cnt++;
		this.maxLength = maxLength;
		g = new DirectedSparseGraph<String,String>();
		this.mf = mf;
		this.config = config;
		
		for (String[] line : th){
			String edge = line[0] + " -> " + line[1];
			g.addEdge(edge, new Pair<String>(line), EdgeType.DIRECTED);	
		}
	}
	
	public void processSingleCycles(int maxLen, boolean withIVV) {
		if (ecs != null) {
			ecs.processSingleCycles(maxLen, withIVV);
		}
	}
	
	public int longestLoopSize() {
		int longest = Integer.MIN_VALUE;
		for (List<String> cycle : cycles) {
			longest = Math.max(longest, cycle.size());
		}
		return longest;
		
	}
	
//	public List<List<String>> analyzeGraph(int maxLength){
	public void analyzeGraph() throws java.lang.OutOfMemoryError, InterruptedException {
		
		analyzeGraph1(maxLength);
		return;
	}
	
	/**
	 * Loop counting method #1 uses the de.normalisiert.util.graphs code
	 * 
	 * @return the list of cycles.
	 */
//	private List<List<String>> analyzeGraph1(int maxLength){
		@SuppressWarnings("unchecked")
		private void analyzeGraph1(int maxLength) throws java.lang.OutOfMemoryError {
			double[][] adjMat = GraphMatrixOperations.graphToSparseMatrix(g).toArray();
			boolean [][] adjMatrix = new boolean[adjMat.length][]; 

			for (int i=0; i<adjMat.length; i++){
				adjMatrix[i] = booleanArray(adjMat[i]);
			}

			ecs = new ElementaryCyclesSearch(adjMatrix, 
					g.getVertices().toArray(), maxLength, this);

			ecs.setWithIVV(withIVV);
			
			ecs.setCyclesFile(config.getCyclesFile());
			ecs.setStoreCycles(config.isStoreCyclesInMemory());
			
			// here
			UserSelectedOptions options = InformationManagers.getInstance()
					.getUserSelectedOptions();
			
			ecs.setWriteCycles( options.isSelected(StringConstants.WRITE_LOOPS_TO_FILE) && 
					(!InformationManagers.getInstance().getReuse().isLoops() || 
					(!config.fileExists(withIVV) && config.isWriteCyclesToFile()))
					);
			ecs.setProcessCycleWhenFound(config.isProcessCycleWhenFound());

			
			if (ecs.isWriteCycles()) {
				ecs.setCyclesFileName(config.getFilename(withIVV));
				ecs.openForWrite();
			}
			
			

			cycles = ecs.getElementaryCycles(mf);

			if (config.isWriteCyclesToFile()) {
				ecs.closeWrite();
				if (config.isProcessCyclesInMemory())
					cycles = ecs.loadCyclesFromFile();
			}
			
			


			if (cycles == null) {
				numCycles = -1;
			} else {
				System.out.println(ecs.getHisto());
				numCycles = ecs.getNumCycles();
			}

			return;
		}
		
		public void countCycles(int maxLength) throws java.lang.OutOfMemoryError {
		double[][] adjMat = GraphMatrixOperations.graphToSparseMatrix(g).toArray();
		boolean [][] adjMatrix = new boolean[adjMat.length][]; 

		for (int i=0; i<adjMat.length; i++){
			adjMatrix[i] = booleanArray(adjMat[i]);
		}
			
		ecs = new ElementaryCyclesSearch(adjMatrix, 
				g.getVertices().toArray(), maxLength, this);
		ecs.setStoreCycles(false);
		
		cycles = ecs.getElementaryCycles(mf);
		if (cycles == null) {
			numCycles = -1;
		} else {
			numCycles = ecs.getNumCycles();
//			System.out.println(ecs.getHisto());
		}

		return;
	}
	
	/**
	 * Converts a binary (0,1) double array to a boolean (F,T) array.
	 * 
	 * @param d the binary dounle array containing 1 or 0.
	 * @return the corresponding binary array.
	 */
	private boolean[] booleanArray(double [] d){
		boolean [] b = new boolean [d.length];
		
		for (int i=0; i<d.length; i++){
			if (d[i] == 1){
				b[i] = true;
			}
			else{
				b[i] = false;
			}
		}
		return b;
	}

	@Override
	public void run() {
//		try {
			try {
				while (!Thread.interrupted()) {
					analyzeGraph();
					if (isAlive())
						return;
				}
			} catch (java.lang.OutOfMemoryError  | InterruptedException x) {
				// TODO Auto-generated catch block
				x.printStackTrace();
				System.out.println("Graph Cycles terminating with ERROR or Interruption");
				cycles = null;
				return;
			}
	}

	/**
	 * @return the cycles
	 */
	public List<List<String>> getCycles() {
		return asVariables();
	}
	
	public List<List<String>> getCyclesAsIDs() {
		return cycles;
	}
	

	
	public List<List<String>> asVariables() {

		List<List<String>> vars = new ArrayList<List<String>>();
		for (List<String> l : cycles) {
			List<String> cycle = new ArrayList<String>();
			for (String s : l)
				cycle.add(Utilities.idToName(s));
			vars.add(cycle);
		}
		return vars;
	}

	public int getMaxLength() {
		return maxLength;
	}
	
	@Override
	public void interrupt() {
		myInterrupted = true;
//		System.out.println("Graph Cycle Thread - received interupt");
		super.interrupt();
		return;
	}

	public boolean isMyInterrupted(String src) {
		if (myInterrupted && mf != null) {
//			mf.disposeMessageFrame();
//			mf = null;
		}
		return myInterrupted;
	}

	public boolean isWithIVV() {
		return withIVV;
	}

}
