package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

public class HeapMonitorThread extends Thread {
	
	private MessageProcessor mf;
	private HeapMonitorLoopManagerInterface loopManager;
	private static int cnt;
	
	public HeapMonitorThread(HeapMonitorLoopManagerInterface lm , MessageProcessor mf) {
		super("heapMonitor"+cnt);
		cnt++;
		this.loopManager = lm;
		this.mf = mf;
	}
	
	public static String getMemoryStats() {
		Runtime runtime = Runtime.getRuntime();
		long freeMemory = runtime.freeMemory();
		long totalMemory = runtime.totalMemory();
		double pct = ((double) freeMemory / (double) totalMemory) * 100.0;
//		if (pct < 25.0)
			return(NumberFormatter.formatForPercent(pct)+"%");
	}
	
	public static void printMemoryStats() {
		Runtime runtime = Runtime.getRuntime();
		long freeMemory = runtime.freeMemory();
		long totalMemory = runtime.totalMemory();
		double pct = ((double) freeMemory / (double) totalMemory) * 100.0;
			System.out.println("Free: "+freeMemory+" / Total: "+totalMemory+" "+pct+"%");
	}

	@Override
	public void run() {
		try {
			long delayMillis = 4000; // 10000; // 2000;
			Runtime runtime = Runtime.getRuntime();
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			while (!Thread.interrupted()) {

				long freeMemory = runtime.freeMemory();
				long totalMemory = runtime.totalMemory();
				double pct = ((double) freeMemory / (double) totalMemory) * 100.0;
				printMemoryStats();
				if (pct < 10.0) {
					System.out.println("Heap Space Low: "+NumberFormatter.formatForPercent(pct)+"%");
					mf.addMessage(language.getText(StringConstants.HEAP_SPACE_LOW)+": "+NumberFormatter.formatForPercent(pct)+"%");
					loopManager.garbageCollect(true);
					Thread.sleep(delayMillis);
					freeMemory = runtime.freeMemory();
					totalMemory = runtime.totalMemory();
					pct = ((double) freeMemory / (double) totalMemory) * 100.0;
					printMemoryStats();
					if (pct < 5.0) {
						System.out.println("Heap Monitor space problem -- will interupt cycles");
						mf.addMessage(language.getText(StringConstants.INSUFFICIENT_HEAP_SPACE)+". "+
								language.getText(StringConstants.TERMINATING_SEARCH)+". "+
								language.getText(StringConstants.AVAILABLE)+" = "+
						NumberFormatter.formatForPercent(pct)+"%");
						System.out.println("Heap Monitor returning");
						return;
					}
				}
				Thread.sleep(delayMillis);
			}

		}
		catch (InterruptedException x) {
			System.out.println("HeapMonitor Interupted");
			return;
		}

	}
	
	public static double freeMemory() {
		Runtime runtime = Runtime.getRuntime();
			long freeMemory = runtime.freeMemory();
			long totalMemory = runtime.totalMemory();
			double pct = ((double) freeMemory / (double) totalMemory) * 100.0;
			return pct;
	}

}
