package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;

public class Loop implements Serializable {
/**
	 * 
	 */
	private static final long serialVersionUID = -5789736210084665033L;

static Logger log = Logger.getLogger(Loop.class.getSimpleName());

private final static String INDENT = HTMLConstants.SPACE+HTMLConstants.SPACE+HTMLConstants.SPACE+HTMLConstants.SPACE+HTMLConstants.SPACE;

// loop will always be numbers
	protected List<String> loop;
	protected List<String> arrowPolarity;
//	private Map<String, Map<String, String>> polarity;
	
	protected String nameWithNumbers = null;
	protected String orderedNameWithNumbers = null;
	protected String nameWithVariables = null;
	protected String orderedNameWithVariables = null;
	
	protected String generatingVariable;
	
	protected int loopNumber;
	protected int extendedLoopNumber = 0;
	
	protected int posPolarityCount = 0;
	protected int negPolarityCount = 0;
	protected int polarityProblemCount = 0;
	
	protected int effectiveLoopCount = -1;
	
	protected boolean onlyIVV = false;
	
	public Loop() {
		loop = new ArrayList<String>();
		arrowPolarity = new ArrayList<String>();
		
	}
	
	public Loop(Loop otherLoop) {
		this();
		System.out.println("Loop create otherLoop: "+otherLoop);
		
		// note: these loops elements will be IDs
		for (String s : otherLoop.getLoop()) {
			loop.add(s);
		}
//		setNames();
//		effectiveLoopCount = this.determineEffectiveLoopCount();
		
	}
	
	public Loop(List<String> aLoop) {
		this();
		boolean isInteger = Parser.isInteger(aLoop.get(0));
		for (String s : aLoop) {
			if (isInteger) {
				loop.add(s);
			} else {
				loop.add(nameToID(s));
			}
		}
//		setNames();
		
//			effectiveLoopCount = this.determineEffectiveLoopCount();
		
	}
	
	public int dist(String varA, String varB) {
		int d = 0;
		String numA = nameToID(varA);
		String numB = nameToID(varB);
		Loop ordered = getLoopStartingAt(varA);
		for (d = 0; d < ordered.getSize(); d++) {
			if (ordered.getLoop().get(d).equals(numB))
				return d;
		}
		
		return -1;
	}
	
	public int length() {
		return loop.size();
	}
	
	public List<String> getTruncatedLoop() {
		List<String> al = new ArrayList<String>();
		for (int i = 0; i < getLoop().size()-1; i++)
			al.add(getLoop().get(i));
		return al;
	}
	
	public void setNames() {
		generateNameWithNumbers();
		generateOrderedNameWithNumbers();
		generateOrderedNameWithVariables();
		generateNameWithVariables();
	}

	public List<String> getLoop() {
//		return asName(loop);
		return loop;
	}
	
	public List<String> getLoopWithVariables() {
//		return asName(loop);
//		String[] t = getNameWithVariables().split(">");
//		String[] t = orderedNameWithNumbers.split(">");
//		String name = idToName(t[0]);
//		Loop l = getLoopStartingAt(name);
		List<String> n =  new ArrayList<String>();
		for (String s : loop)
			n.add(s);
		
		return n;
	}
	
//	public List<String> asName(Loop lp) {
//		return asName(lp.getLoop());
//	}
	
	public List<String> withVariableName(List<String> lp) {
		List<String> al = new ArrayList<String>();
		for (String id : lp)
			al.add(idToName(id));
		return al;
	}
	
	public String getOrderedLoopName() {
		List<String> t = new ArrayList<String>();
		for (String s : loop) {
			t.add(idToName(s));
		}
		Collections.sort(t);
		
		// translation to ID takes place in getLoopStartAt
		Loop l = getLoopStartingAt(t.get(0));
		return l.getNameWithVariables();
	}
	
	public String generateOrderedNameWithNumbers() {
		List<String> t = new ArrayList<String>();
		for (String s : loop) {
			t.add(s);
		}
		
		if (t.size() == 0) {
			System.out.println("t.size = 0");
		}
		
//		Collections.sort(t);
		t = sortNumberedList(t);
		
		// translation to ID takes place in getLoopStartAt
		orderedNameWithNumbers = getNumberedNameStartingAt(t.get(0));
		return orderedNameWithNumbers;
	}
	
	public List<String> sortNumberedList(List<String> t) {
		List<String> strings = new ArrayList<String>();
		List<Integer> numbers = new ArrayList<Integer>();
		for (String s : t) {
			Integer i;
			try {
				i = new Integer(s);
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				i = 0;
				e.printStackTrace();
			}
			numbers.add(i);
		}
		Collections.sort(numbers);
		for (Integer i : numbers) {
			strings.add(i.toString());
		}
		
		
		return strings;
	}
	
	public void setLoopStartVariable() {

		String[] n = getOrderedNameWithNumbers().split(">");
		String name = idToName(n[0]);
		Loop l = getLoopStartingAt(name);
		
		loop = l.getLoop();
		arrowPolarity = l.getArrowPolarity();
	}
	
	public String getNumberedNameStartingAt(String id) {
		StringBuilder sb = new StringBuilder();
		List<String> al = new ArrayList<String>();
		if (loop.contains(id)) {
			int first = loop.indexOf(id);
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
			}
		}
		
		int l = 0;
		for (String s : al) {
			if (l++ > 0)
				sb.append(">");
			sb.append(s);
		}
		return sb.toString();
	}
	
	public Loop getNumberedLoopStartingAt(String id) {
		List<String> al = new ArrayList<String>();
		if (loop.contains(id)) {
			int first = loop.indexOf(id);
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
			}
		}
		
//		return new Loop(asName(al));
		return new Loop(al);
	}
	
	public Loop getLoopStartingAt(String variable) {
		List<String> al = new ArrayList<String>();
		List<String> pol = new ArrayList<String>();
		if (loop.contains(nameToID(variable))) {
			int first = loop.indexOf(nameToID(variable));
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
				if (arrowPolarity.size() > 0)
					pol.add(arrowPolarity.get(i));
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
				if (arrowPolarity.size() > 0)
				pol.add(arrowPolarity.get(i));
			}
		} else {
			int first = loop.indexOf(variable);
			
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
				pol.add(StringConstants.UNDEFINED);
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
				pol.add(StringConstants.UNDEFINED);
			}
		}
		
//		return new Loop(asName(al));
		Loop newLoop = new Loop(al);
		newLoop.setArrowPolarity(pol);
		return newLoop;
	}
	
	public Loop getTruncatedLoopStartingAt(String variable) {
		List<String> al = new ArrayList<String>();
		if (loop.contains(nameToID(variable))) {
			int first = loop.indexOf(nameToID(variable));
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
			}
		}
		if (al.size() == 0) {

		} else {
			al.remove(al.size()-1);
		}
//		return new Loop(asName(al));
		return new Loop(al);
	}
	
	public boolean isEquivalentTo(Loop otherLoop) {

		if (getSize() != otherLoop.getSize())
			return false;
		
		for (int i = 0; i < getSize(); i++)
			if (!Parser.isEquivalentString(loop.get(i), otherLoop.getLoop().get(i)))
			
				return false;
		
		return true;
	}
	public String getLast() {
		return idToName(loop.get(loop.size()-1));
	}
	
	public int getSize() {
		return loop.size()-1;
	}
	
	public boolean isLoop() {
		if (loop.size() > 1) {
			String p0 = loop.get(0);
			if (p0 == null) {
				System.out.println("HUH?");
			}
			String pl = loop.get(loop.size()-1);
			if (pl == null) {
				System.out.println("HUH?");
			}
			boolean b = p0.equals(pl);
		}
		return loop.size() > 1 && loop.get(0).equals(loop.get(loop.size()-1));
	}
	
	public String getNameWithVariables() {
		if (nameWithVariables == null)
			generateNameWithVariables();
		return nameWithVariables;
	}

	public String getName() {
		if (nameWithNumbers == null)
			generateNameWithNumbers();
		return nameWithNumbers;
	}
	
	public void add(String var) {
		loop.add(nameToID(var));
		nameWithNumbers = null;
	}
	
	private void generateOrderedNameWithVariables() {
		if (orderedNameWithNumbers == null)
			generateOrderedNameWithNumbers();
		String[] parts = orderedNameWithNumbers.split(">");
		
		StringBuilder sb = new StringBuilder();
		for (String s : parts) {
			if(sb.length() > 0)
				sb.append(">");
			sb.append(idToName(s));
		}
		orderedNameWithVariables = sb.toString();
//		System.out.println("orderedNameWithVariables; "+orderedNameWithVariables);
//		System.out.println("orderedNameWithNumbers; "+orderedNameWithNumbers);
	}
	
	private void generateNameWithVariables() {
		List<String> al = new ArrayList<String>(loop);
		if (isLoop()) {
			al.remove(al.size()-1);
		}
		StringBuilder sb = new StringBuilder();
		for (String s : al) {
			if(sb.length() > 0)
				sb.append(">");
			sb.append(idToName(s));
		}
		nameWithVariables = sb.toString();
	}
	
	private void generateNameWithNumbers() {
		List<String> al = new ArrayList<String>(loop);
		if (isLoop()) {
			al.remove(al.size()-1);
		}
		StringBuilder sb = new StringBuilder();
		for (String s : al) {
			if(sb.length() > 0)
				sb.append(">");
			sb.append(s);
		}
		nameWithNumbers = sb.toString();
	}
	
	public void print(int loopNum) {
		if (!isLoop())
			return;
		
//		log.debug("Name: "+getName());
		log.debug("Loop Number "+loopNumber+" of length "+(loop.size()-1));
		for (int i = 0; i < loop.size()-1; i++) {
			String s = loop.get(i);
			if (i > 0)
				log.debug("   ");
			log.debug("     "+s);
		}
	}
	
	public String getLoopIdentifier() {
		return "Loop "+NumberFormatter.formatForIntComma(loopNumber)+"."+extendedLoopNumber+" "+getLoopPolarity()+" Length "+(loop.size()-1);
	}
	
	public void setLoopIdentifier(String id) {
//		return "Loop "+loopNumber+"."+extendedLoopNumber+" "+getLoopPolarity()+" Length "+(loop.size()-1);
		String[] fields = id.split("\\.");
		loopNumber = Integer.parseInt(fields[0].replace("L", ""));
		extendedLoopNumber = Integer.parseInt(fields[1]);
	}
	
	public String getLoopIdentifierNoPolarityNoSize() {
		return "Loop "+NumberFormatter.formatForIntComma(loopNumber)+"."+extendedLoopNumber;
	}
	
	public String getLoopIdentifierHTML() {
		return "L"+NumberFormatter.formatForIntComma(loopNumber)+"."+extendedLoopNumber;
	}
	
	
	public String getHtmlLoopIdentifier() {
		
		return SDMDocLanguageSpecific.getInstance().getTextHtml(StringConstants.LOOP)+" "+
				NumberFormatter.formatForIntComma(loopNumber)+"."+extendedLoopNumber+
				" ("+getHtmlLoopPolarity()+") "+
					SDMDocLanguageSpecific.getInstance().getTextHtml(StringConstants.LENGTH)+
					" "+(loop.size()-1);
	}
	
	public void print() {
		if (!isLoop())
			return;
		
//		log.debug("Name: "+getName());
		log.debug("Loop Number "+loopNumber+" of length "+(loop.size()-1));
		for (int i = 0; i < loop.size()-1; i++) {
			String s = loop.get(i);
			if (i > 0)
				log.debug("   ");
			log.debug("     "+s);
		}
	}
	
//	public void printWithPolarity() {
//		log.debug("\nName: "+getNameWithVariables()+ " ("+getLoopPolarity()+")");
//		log.debug("Length = "+(loop.size()-1));
//		for (int i = 0; i < loop.size()-1; i++) {
//			String tail = idToName(loop.get(i));
//			String head = idToName(loop.get(i+1));
//			String polarity;
//			if (arrowPolarity.size() > 0) {
//			
//				polarity = arrowPolarity.get(i);
//			} else {
//				polarity = "#";
//			}
//
//			log.debug("   "+tail+" ---> "+head+" ("+polarity+")");
//		}
//	}
	
	public void writeOneLine(BufferedWriter bw) {
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < loop.size()-1; i++) {
			if(i > 0)
				sb.append(">");
			sb.append(idToName(loop.get(i)));
		}
		try {
				bw.append(sb.toString()+"\n");
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	public void writeTabWithPolarity(BufferedWriter bw) {
		try {
			bw.append(loopNumber+"\t");
			bw.append((loop.size()-1)+"\t");
			bw.append(getLoopPolarity());
			for (int i = 0; i < loop.size(); i++) {
				String tail = idToName(loop.get(i));
				bw.append("\t"+tail);
			}
			bw.append("\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeWithPolarity(BufferedWriter bw) {
		try {
			bw.append("\n"+getLoopIdentifier()+"\n");
			for (int i = 0; i < loop.size()-1; i++) {
				String tail = idToName(loop.get(i));
				String head = idToName(loop.get(i+1));
				String polarity = arrowPolarity.get(i);
				bw.append(" "+tail+" "+polarity+"> "+head+"\n");
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String getHtmlWithPolarity(int num, int total) {
		StringBuilder sb = new StringBuilder();
			sb.append(HTMLConstants.BREAK+   Anchors.getAnchorDefinition(getAnchorName())+HTMLConstants.BOLD_START+
					(this.isOnlyIVV() ? HTMLConstants.BLUE_START : "")+ 
					"("+NumberFormatter.formatForIntComma(num)+"/"+NumberFormatter.formatForIntComma(total)+") "+
					getHtmlLoopIdentifier()+HTMLConstants.BOLD_END+HTMLConstants.BREAK);
			for (int i = 0; i < loop.size()-1; i++) {
				String tail = StringEscapeUtils.escapeHtml4( idToName(loop.get(i)));
				String head = StringEscapeUtils.escapeHtml4( idToName(loop.get(i+1)));
				String polarity = arrowPolarity.get(i);
				
				sb.append(HTMLConstants.SPACE+HTMLConstants.SPACE+tail);
				sb.append(HTMLConstants.SPACE);
				if (polarity.equals(ArrowPolarity.POSITIVE))
					sb.append("+");
				else if (polarity.equals(ArrowPolarity.NEGATIVE))
					sb.append(HTMLConstants.DASH);
				else
					sb.append("?");
				sb.append(HTMLConstants.GT);
				sb.append(HTMLConstants.SPACE);
				sb.append(HTMLConstants.SPACE+head+(this.isOnlyIVV() ? HTMLConstants.COLOR_END : "")+HTMLConstants.BREAK+"\n");
				
			}
			return sb.toString();
		
	}
	
	public String getLoopPolarity() {
		
		if (polarityProblemCount > 0)
			return ArrowPolarity.UNKNOWN;
		else if (negPolarityCount % 2 == 0)
			return ArrowPolarity.POSITIVE;
		else
			return ArrowPolarity.NEGATIVE;
		
	}
	
	public String getHtmlLoopPolarity() {
		
		if (polarityProblemCount > 0)
			return ArrowPolarity.UNKNOWN;
		else if (negPolarityCount % 2 == 0)
			return ArrowPolarity.POSITIVE;
		else
			return HTMLConstants.DASH;
		
	}
	
	public boolean isPolarityPositiveLoop() {
		if (getLoopPolarity().equals(ArrowPolarity.POSITIVE))
				return true;
		else
			return false;
		
	}
	
	public boolean isPolarityNegativeLoop() {
		if (getLoopPolarity().equals(ArrowPolarity.NEGATIVE))
				return true;
		else
			return false;
		
	}
	
	public boolean isPolarityProblemLoop() {
		return polarityProblemCount > 0;
		
	}
	
	public List<ArrowPolarity> getArrows() {
		List<ArrowPolarity> al = new ArrayList<ArrowPolarity>();
		for (int i = 0; i < loop.size()-1; i++) {
			String tail = idToName(loop.get(i));
			String head = idToName(loop.get(i+1));
			al.add(new ArrowPolarity(tail, head, StringConstants.UNDEFINED));
		}
		return al;
	}
	
	public void determineMinimumProcessLoopPolarity(PolarityResults polarityResults) {
		for (int i = 0; i < loop.size()-1; i++) {
			String polarity = "?";
			
			if (polarityResults != null) {
				polarity = polarityResults.getPolarityForArrow(loop.get(i), loop.get(i+1));
				if (polarity == null) {
					polarity = "?";
				}
			} else {
				polarity = "?";
			}
			
			if (polarity.equals(ArrowPolarity.POSITIVE))
				posPolarityCount++;
			
			else if (polarity.equals(ArrowPolarity.NEGATIVE))
				negPolarityCount++;
			
			else if (polarity.equals(ArrowPolarity.UNKNOWN) || polarity.startsWith(ArrowPolarity.INITIALIZATION))
				polarityProblemCount++;
			
			arrowPolarity.add(polarity);

		}
	}
	
	public void determineLoopPolarity(PolarityResults polarityResults) {
		for (int i = 0; i < loop.size()-1; i++) {
			String tail = idToName(loop.get(i));
			String head = idToName(loop.get(i+1));
			String polarity;
			
			if (polarityResults != null) {
//				polarity = polarityResults.getPolarityForArrow(tail, head);
				polarity = polarityResults.getPolarityForArrowID(loop.get(i), loop.get(i+1));
				if (polarity == null) {
//					System.out.println("NO Polarity "+loop.get(i)+" "+loop.get(i+1));
					polarity = "?";
				}
			} else {
				polarity = "?";
			}
			
			if (polarity.equals(ArrowPolarity.POSITIVE))
				posPolarityCount++;
			
			else if (polarity.equals(ArrowPolarity.NEGATIVE))
				negPolarityCount++;
			
			else if (polarity.equals(ArrowPolarity.UNKNOWN) || polarity.startsWith(ArrowPolarity.INITIALIZATION))
				polarityProblemCount++;
			
			arrowPolarity.add(polarity);
//			System.out.println("DetLoopPol: "+tail+" => "+head+"  "+polarity);
		}
	}
	
	public int determineEffectiveLoopCount() {
		
		if (!isLoop())
			return 0;
		List<Integer> aList = new ArrayList<Integer>();
		
		for (int i = 0; i < loop.size()-1; i++) {
			String head = idToName(loop.get(i+1));
			int sum = 0;
			if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(head) != null) {
			for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(head))
				sum += eqn.getTotalEquationCount();
				aList.add(sum);
		}
		}
		
		int min = Integer.MAX_VALUE;
		int max = Integer.MIN_VALUE;
		
		for (int i = 0; i < aList.size(); i++) {
			Integer a = aList.get(i);
			min = Math.min(min, a.intValue());
			max = Math.max(max, a.intValue());
		
		}
		
		// set to false to generate output to console
		boolean printed = true;
		for (int i = 0; i < aList.size(); i++) {
			Integer a = aList.get(i);
			if (a.intValue() != min && a.intValue() != max) {
				if (!printed) {
					System.out.println("Eff Loop Cnt: "+" "+getNameWithVariables());
					System.out.println("Eff Loop Cnt: "+" "+getName());
					System.out.println("isLoop? "+isLoop());
					for (Integer b : aList) {
						System.out.println("  "+b.intValue());
					}
					printed = true;
				}
			}
		}

		
		return max;
	}
	
	public int compareTo(Loop loop2) {
    	if (getSize() != loop2.getSize()) {
    		return getSize() > loop2.getSize() ? -1 : 1;
    	}
    	for (int i = 0; i < getSize(); i ++) {
    		if (!getLoop().get(i).equals(loop2.getLoop().get(i))) {
    			return idToName(getLoop().get(i)).compareTo(idToName(loop2.getLoop().get(i)));
    		}
    	}
      return 0;
    }

	public String getGeneratingVariable() {
		return generatingVariable;
	}

	public void setGeneratingVariable(String generatingVariable) {
		this.generatingVariable = generatingVariable;
	}
	
	public boolean contains(String s) {
		return loop.contains(nameToID(s));
	}
	
	private String nameToID(String name) {
		//		return name;

		return InformationManagers.getInstance().getEquationManager().getVariableIDForName(name);
	}
	
	private String idToName(String id) {
//		return id;
		return /*Parser.removeQuotes(*/ InformationManagers.getInstance().getEquationManager().getVariableNameForID(id); // );
	}
	
	public String toString() {
		return getName();
	}
	
	public String getAnchorName() {
		return "L"+NumberFormatter.formatForIntComma(getLoopNumber()) +"."+getExtendedLoopNumber();
	}
	
	public String getAnchorReference() {
		return "#" + getAnchorName();
	}

	public int getLoopNumber() {
		return loopNumber;
	}

	public void setLoopNumber(int loopNumber) {
		this.loopNumber = loopNumber;
	}

	public int getExtendedLoopNumber() {
		return extendedLoopNumber;
	}

	public void setExtendedLoopNumber(int extendedLoopNumber) {
		this.extendedLoopNumber = extendedLoopNumber;
	}

	public boolean isOnlyIVV() {
		return onlyIVV;
	}

	public void setOnlyIVV(boolean onlyIVV) {
		this.onlyIVV = onlyIVV;
	}

	public int getEffectiveLoopCount() {
		if (effectiveLoopCount == -1)
			determineEffectiveLoopCount();
		return effectiveLoopCount;
	}

	/**
	 * @return the orderedNameWithNumbers
	 */
	public String getOrderedNameWithNumbers() {

		if (orderedNameWithNumbers == null) 
			generateOrderedNameWithNumbers();		
		return orderedNameWithNumbers;
	}

	/**
	 * @return the orderedNameWithVariables
	 */
	public String getOrderedNameWithVariables() {

		if (orderedNameWithVariables == null)
			generateOrderedNameWithNumbers();
		return orderedNameWithVariables;
	}

	public void setArrowPolarity(List<String> arrowPolarity) {
		this.arrowPolarity = arrowPolarity;
	}

	public List<String> getArrowPolarity() {
		return arrowPolarity;
	}

//	public Map<String, Map<String, String>> getPolarity() {
//		return polarity;
//	}
}
