package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.LoopDiscoverWorker;
import gov.anl.gss.sdm.server.model.documentor.gui.MessageFrame;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.EquationLookup;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.VariableManager;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class LoopDiscovery {
	
	private List<List<String>> discoveredCycles = null;
	
	private PolarityResults polarityResultsNoIVV = new PolarityResults();
//	private PolarityResults polarityResultsWithIVV = new PolarityResults();
	
	private List<String[]> edges = new ArrayList<String[]>();
	
	private VariableManager variableManager = new VariableManager();
	
	private String arrowPolarityTab;
	private int maxLength;
	private int startLength;
//	private MessageGobbler mf = new MessageGobbler();
	private MessageProcessor mf = new MessageGobbler();
	private String file;
	
	private MessageProcessor mp;
	
	private Date startDate;
	private Date endDate;
	
	private LoopDiscoverWorker ldw = null;
	
	private LoopLengthAnalyzer lla = new LoopLengthAnalyzer(null);
	
	SimpleDateFormat timeStamp = new SimpleDateFormat("HH:mm:ss");
	
	private GraphCyclesThread gc = null;
	
	
	
	public LoopDiscovery(String file, String startLength, String maxLength, MessageProcessor mp ) {
		this.file = file;
		String timeStamp = new SimpleDateFormat("yyyy.MM.dd.HH.mm.ss").format(new Date());
		System.out.println("Start: "+timeStamp);
//		this.startLength = Integer.parseInt(startLength);
		this.maxLength = Integer.parseInt(maxLength);
		this.startLength = this.maxLength;
		this.arrowPolarityTab = file;
		this.mp = mp;
		mf = mp;
		
		InformationManagers.getInstance().setVariableManager(variableManager);
		
	}
	
	public void initialize(LoopDiscoverWorker ldw) {
		SDMDocLanguageSpecific.initialize();
		this.ldw = ldw;
		
		mp.addMessage("Initializing");

		List<String> fromFile = Utilities.getAsList(arrowPolarityTab);
		int row = 0;
		for (String line : fromFile) {
			if (row++ == 0)
				continue;
			String[] parts = line.split("\t");

			variableManager.addVariable(parts[0]);
			variableManager.addVariable(parts[1]);
			edges.add(new String[] {variableManager.getID(parts[0]), variableManager.getID(parts[1])});
//			polarityResultsNoIVV.setPolarityForArrow(variableManager.getID(parts[0]), variableManager.getID(parts[1]), parts[2]);
//			polarityResultsNoIVV.setPolarityForArrow(parts[0], parts[1], parts[2]);
			
			

		}
		InformationManagers.getInstance().getEquationManager().reinitialize(variableManager);
		
		row = 0;
		for (String line : fromFile) {
			if (row++ == 0)
				continue;
			String[] parts = line.split("\t");

//			polarityResultsNoIVV.setPolarityForArrow(variableManager.getID(parts[0]), variableManager.getID(parts[1]), parts[2]);
			polarityResultsNoIVV.setPolarityForArrow(parts[0], parts[1], parts[2]);
		}
	}
	
	public void doit() {
		
		InformationManagers.getInstance().setLoopManager(new LoopManager());
		
		mp.addMessage("Loop Length Analysis - Size Range: "+startLength+" to "+maxLength+" by 1");
		
		for (int length = startLength; length <= maxLength; length++) {
			if (ldw.isCancelled()) {
				if (gc != null) {
					gc.interrupt();
				}
				return;
			}
			lla = new LoopLengthAnalyzer(null);
			mp.addMessage("Begin for length: "+length);
			System.out.println("== "+length+"==");
			InformationManagers.getInstance().getLoopManager().clearLla();
			InformationManagers.getInstance().getLoopManager().clearPreviousLoops();
			discover(length);
		}
	}
	
	public void discover(int length) {
		startDate = new Date();
		
		CyclesConfiguration cc = new CyclesConfiguration();
		cc.configureForStanaloneLoopLengthAnalysis();
		gc = new GraphCyclesThread(edges,length, mf, cc, false);
		try {
			gc.analyzeGraph();
		} catch (OutOfMemoryError | InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		endDate = new Date();
		
		LoopManager lm = InformationManagers.getInstance().getLoopManager();
		if (lm == null) {
			discoveredCycles = gc.getCyclesAsIDs();
			for (int i = 0; i < discoveredCycles.size(); i++) {
				List<String> aLoop = discoveredCycles.get(i);
				aLoop.add(aLoop.get(0));
				Loop l = new Loop(aLoop);
				l.determineMinimumProcessLoopPolarity(polarityResultsNoIVV);
				lla.addLoopInformation(l);
			}
		} else {
			lla = lm.getLla();
		}
		
		System.out.println(timeStamp.format(startDate)+","+timeStamp.format(endDate)+" Length: "+length+" Loop Count "+gc.getNumCycles());
		
		lla.aggregate(length);
		lla.write(Utilities.openFileForWriting(getOutputFileName(length)));
	}
	
	private String getOutputFileName(int length) {
		File f = new File(file);
		String p = f.getParentFile().getParentFile().getAbsolutePath();
		p += "./LoopDiscoveryLoopLengthAnalysis_"+length+".tab";
		return p;
	}
	
	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		
		LoopDiscovery ld = new LoopDiscovery(args[2], args[0], args[1], new MessageFrame("LoopDiscovery"));
		ld.initialize(null);
		ld.doit();
		
	}

}
