package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

public class LoopHistogram {
	
	private Map<String, MutableInteger> histo = new LinkedHashMap<String, MutableInteger>();
	
	public LoopHistogram(Set<Loop> loops) {
		buildHisto(loops);
	}
	
	public void buildHisto(Set<Loop> loops) {
		for (Loop loop : loops) {
			String length = Integer.toString(loop.getSize());
			if (!histo.containsKey(length))
				histo.put(length, new MutableInteger(1));
			else
				histo.get(length).add(1);
			
		}
	}
	
	public String getHisto() {
		StringBuilder sb = new StringBuilder();
		int maxKey = getMaxKey();
		if (maxKey < 0) {
			return "no loops with which to generate histo!";
		}
		int[] h = new int[maxKey+1];
		for (int i = 0; i < maxKey+1; i++) {
			String asString = Integer.toString(i);
			if (histo.containsKey(asString))
				h[i] = histo.get(asString).value();
			else
				h[i] = 0;
		}
		
		for (int i = 2; i <= maxKey; i++) {
			sb.append(i+": ");
			sb.append(h[i]+"\n");
		}
		
		return sb.toString();
	}
	
	private int getMaxKey() {
		int max = Integer.MIN_VALUE;
		for (String k : histo.keySet()) {
			int v = Integer.parseInt(k);
			if (v > max)
				max = v;
		}
		return max;
	}

}
