package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class LoopLengthAnalyzer {

	LoopLengthAnalyzerInterface loopManager;
	List<SingleLoopLengthAnalysis> withoutIVV;
	List<SingleLoopLengthAnalysis> withIVV;
	
	String common1 = "";
	String common2 = "";
	String common3 = "";
	
	Map<Integer, SingleLoopLengthAnalysis> lengthMap = new HashMap<Integer, SingleLoopLengthAnalysis>();
	
	public LoopLengthAnalyzer(LoopLengthAnalyzerInterface manager) {
		loopManager = manager;
		withoutIVV = new ArrayList<SingleLoopLengthAnalysis>();
		withIVV = new ArrayList<SingleLoopLengthAnalysis>();
	}
	
	public void analyze(int minLoopLength, int maxLoopLength) {
		Set<Loop> upToSizeSet = new LinkedHashSet<Loop>();
		
		for (int len = minLoopLength; len <= maxLoopLength; len++) {
			upToSizeSet.addAll(loopManager.getLoopsOfLength(len));
			withoutIVV.add(analyze(upToSizeSet, len));
		}
		
		upToSizeSet = new LinkedHashSet<Loop>();
		for (int len = minLoopLength; len <= maxLoopLength; len++) {
			upToSizeSet.addAll(loopManager.getIVVLoopsOfLength(len));
			withIVV.add(analyze(upToSizeSet, len));
		}
		
		
	}
	
//	int loopLength,
//	int numLoops,
//	int numPositive,
//	int numNegative,
//	int numUnk,
//	int shortest,
//	int longest,
//	String common1,
//	String common2,
//	String common3
//	) {
	
	public void extendTo(int maximumLoopLength) {
		for (int st = 2; st < maximumLoopLength; st++) {
			SingleLoopLengthAnalysis slla = new SingleLoopLengthAnalysis(st,0,0,0,0,0,0,"","","");
			lengthMap.put(st, slla);
		}
	}
	
	public void addLoopInformation(Loop loop) {
		int len = loop.length()-1;
		if (!lengthMap.containsKey(len)) {
			lengthMap.put(len,  new SingleLoopLengthAnalysis(len));
		}
		lengthMap.get(len).addLoop(loop);
	}
	
	public void aggregate(int length) {
		List<Integer> sizes = new ArrayList<Integer>(lengthMap.keySet());
		if (sizes.size() == 0) {
			extendTo(length);
			sizes = new ArrayList<Integer>(lengthMap.keySet());
		}
		
		if (sizes.size() == 0)
			return;
		Collections.sort(sizes);
		List<Integer> all = new ArrayList<Integer>();
		for (int i = 2; i <= sizes.get(sizes.size()-1); i++) {
			all.add(i);
		}
		int cumulNum = 0;
		int cumulPos = 0;
		int cumulNeg = 0;
		int cumulUnk = 0;
		int cumulMin = Integer.MAX_VALUE;
		int cumulMax = 0;
		
		for (int siz : all) {
			if (lengthMap.containsKey(siz)) {
				SingleLoopLengthAnalysis sla = lengthMap.get(siz);
				cumulNum += sla.getNumLoops();
				cumulPos += sla.getNumPositive();
				cumulNeg += sla.getNumNegative();
				cumulUnk += sla.getNumUnk();
				if (sla.getNumLoops() > 0)
					cumulMax = siz;
				if (sla.getNumLoops() > 0)
					cumulMin = Math.min(cumulMin, siz);
			}
			SingleLoopLengthAnalysis slla = new SingleLoopLengthAnalysis(siz,cumulNum,cumulPos,cumulNeg,cumulUnk,cumulMin,cumulMax,"","","");
			withoutIVV.add(slla);
		}
	}
	
	public void write(BufferedWriter bw) {
		try {
			bw.append(SingleLoopLengthAnalysis.getColumnHeadersNoCommon()/* +"\t"+SingleLoopLengthAnalysis.getColumnHeaders()*/+"\n");
			for (int i = 0; i < withoutIVV.size(); i++) {
				SingleLoopLengthAnalysis sllaNoIVV = withoutIVV.get(i);
//				SingleLoopLengthAnalysis sllaWithIVV = withIVV.get(i);
				
				bw.append(sllaNoIVV.asTabbedRecordNoCommon()/* +"\t"+sllaWithIVV.asTabbedRecord() */ +"\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private SingleLoopLengthAnalysis analyze(Set<Loop> loops, int len) {
		int loopLength = len;
		int numLoops = loops.size();
		int numPositive = loopManager.getNumberOfPositivePolarityLoops(loops);
		int numNegative = loopManager.getNumberOfNegativePolarityLoops(loops);
		int numUnk = loopManager.getNumberOfProblemPolarityLoops(loops);
		int shortest = loopManager.getMinimumLengthLoopSize(loops);
		int longest = loopManager.getMaximumLengthLoopSize(loops);
		createCommonCounts(loops, len);
		
		return new SingleLoopLengthAnalysis(loopLength, numLoops, numPositive, numNegative, numUnk, shortest, longest,
				common1, common2, common3);
	}
	
	private void createCommonCounts(Set<Loop> loops, int len) {
		
		Map<String, MutableInteger> counts = new HashMap<String, MutableInteger>();
		for (Loop loop : loops) {
			for (String var : loop.getLoopWithVariables()) {
				if (!counts.containsKey(var))
					counts.put(var, new MutableInteger(0));
				counts.get(var).add(1);
			}
		}
		Set<Integer> c = new LinkedHashSet<Integer>();
		for (MutableInteger mi : counts.values()) {
			Integer i = new Integer(mi.value());
			if (!c.contains(i))
				c.add(i);
		}
		
		List<Integer> is = new ArrayList<Integer>(c);
		Collections.sort(is);
		Collections.reverse(is);
		int[] num = new int[] {0,0,0};
		
		StringBuilder[] sb = new StringBuilder[3];
		
		for (int i = 0; i < 3; i++) {
			StringBuilder x = new StringBuilder();
			sb[i] = x;
			if (i >= is.size())
				break;
			Integer cnt = is.get(i);
			num[i] = cnt.intValue();
			for (String var : counts.keySet()) {
				if (counts.get(var).value() == cnt.intValue()) {
					if (x.length()>0)
						x.append(",");
					x.append(var);
				}
			}
		}
		
		common1 = num[0]+" ["+sb[0]+"]";
		common2 = num[1]+" ["+sb[1]+"]";
		common3 = num[2]+" ["+sb[2]+"]";
		
	}
}
