package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.analysis.graphtheory.AdjacencyMatrix;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.GraphTheoryMatrix;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.Pair;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.Position;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.ReachabilityMatrix;
import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.ColumnSizing;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.DocumentHeader;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.DatabaseManager;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;
import gov.anl.gss.sdm.server.model.translator.VariableManager;

import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.math3.linear.MatrixUtils;
import org.apache.commons.math3.linear.RealMatrix;
import org.apache.commons.math3.linear.RealVector;
import org.apache.log4j.Logger;

public class LoopManager implements LoopLengthAnalyzerInterface, HeapMonitorLoopManagerInterface {
	static Logger log = Logger.getLogger(LoopManager.class.getSimpleName());

	public final static String NO_IVV = "NoIVV";
	public final static String WITH_IVV = "WithIVV";
	
	
	GraphCyclesThread gc = null;
	
	int failLength = 0;
	int successLength = 0;
	int currentLength = 0;
	
	long singleCycleCount = 0;

	public final static int NO_LOOPS = -1;
	private final static int MAX_LOOP_SIZE = 1000;
	public final static int MAX_LOOPS_FOR_POSTPROCESS = 100000; // 100000;
	public final static int MAX_LONGEST_FOR_POSTPROCESS = 50000; // 100000;
	public final static int HARDCODED_LOOP_COUNT_LIMIT = 10000000;
	
	public final static int MAX_SEARCH_TIME_SECONDS = 7200; // -> two hour,
															// 1800; // -> 30
															// minutes
	
	private List<String[]> cachedIdEdges = null;

	private boolean useIncore = true;
	private boolean useDB = false;

	private long allLoopsWithIVVSize = 0;
	private long allLoopsNoIVVSize = 0;

	private GraphCyclesThread foundCycles = null;
	
	private boolean usingMinimumMemory = true;
	
	private CyclesConfiguration config = new CyclesConfiguration();

	private MutableInteger foundOtherLoop = new MutableInteger(0);

	private int shortestLoopLengthNoIVV = Integer.MAX_VALUE;
	private int longestLoopLengthNoIVV = Integer.MIN_VALUE;

	private int shortestLoopLengthWithIVV = Integer.MAX_VALUE;
	private int longestLoopLengthWithIVV = Integer.MIN_VALUE;

	private int currentLoopCount = 0;

	private VariableRelationships variableRelationships = new VariableRelationships();
	private VariableLoops variableLoops = new VariableLoops();

	private Set<String> variablesNotVisited;

	private Set<Loop> allLoopsNoIVV = new LinkedHashSet<Loop>();
	private Set<String> knownLoopNamesNoIVV = new LinkedHashSet<String>();
	private Map<String, Set<Loop>> loopMembershipNoIVV;

	private Set<Loop> allLoopsWithIVV = new LinkedHashSet<Loop>();;
	private Set<String> knownLoopNamesWithIVV = new LinkedHashSet<String>();
	private Map<String, Set<Loop>> loopMembershipWithIVV;

	private NativeDataTypeManager ndtm;
	private PolarityResults polarityResultsNoIVV;
//	private PolarityResults polarityResultsWithIVV;
	
	private LoopLengthAnalyzer lla = new LoopLengthAnalyzer(this);
	
	private int maxLengthForMessage;
	
	private MutableInteger[] loopCountsByLengthPositive;
	private MutableInteger[] loopCountsByLengthNegative;
	private MutableInteger[] loopCountsByLengthUnknown;
	
	private Map<Boolean, Map<String, VariableLoopPolarityCounts>> variableLoopPolarityCounts = 
			new LinkedHashMap<Boolean, Map<String, VariableLoopPolarityCounts>>();

	// private Map<String, String> olivaRssdNameMap = new LinkedHashMap<String,
	// String>();
	// private Map<String, String> rssdOlivaNameMap = new LinkedHashMap<String,
	// String>();

	private SDObjectManager vensimEquationManager;

	private GraphTheoryMatrix loopDerivedReachability;
	private AdjacencyMatrix loopDerivedAdjacency;

	private ReachabilityMatrix reachability;

	private MessageProcessor mf;

	private boolean sdmuiMode = false;
	private String sdmuiRootDirectory;

	private Map<String, Set<String>> neverConsider;
	private boolean unlimitedSize = true;
	private int maximumLoopLength = Integer.MAX_VALUE - 10;
	private boolean generateWithIVV = true;
	
	private List<VariableLoopCount> variableLoopCount = new ArrayList<VariableLoopCount>();

	public LoopManager() {
		allLoopsNoIVV = new LinkedHashSet<Loop>();
		loopMembershipNoIVV = new LinkedHashMap<String, Set<Loop>>();

		allLoopsWithIVV = new LinkedHashSet<Loop>();
		loopMembershipWithIVV = new LinkedHashMap<String, Set<Loop>>();

		setUseIncore(true);
	}

	public LoopManager(SDObjectManager vensimEquationManager,
			PolarityResults polarityResultsNoIVV,
			PolarityResults polarityResultsWithIVV, NativeDataTypeManager ndtm,
			ReachabilityMatrix reachability, boolean generateWithIVV,
			boolean unlimitedSize, int maxSize, MessageProcessor mf) {
		this();
		this.vensimEquationManager = vensimEquationManager;
		this.ndtm = ndtm;
		this.polarityResultsNoIVV = polarityResultsNoIVV;
//		this.polarityResultsWithIVV = polarityResultsWithIVV;
		this.generateWithIVV = generateWithIVV;
		this.reachability = reachability;
		this.unlimitedSize = unlimitedSize;
		maxLengthForMessage = maxSize;
		this.mf = mf;
		if (mf != null) {
			mf.setBlipMode(true);
		} else {
			mf = new MessageGobbler();
		}
		this.maximumLoopLength = maxSize;
		
		
		variableLoopPolarityCounts.put(true, new LinkedHashMap<String, VariableLoopPolarityCounts>());
		variableLoopPolarityCounts.put(false, new LinkedHashMap<String, VariableLoopPolarityCounts>());
		
		variableLoopPolarityCounts.get(true).put("__ALL__", new VariableLoopPolarityCounts());
		variableLoopPolarityCounts.get(false).put("__ALL__", new VariableLoopPolarityCounts());
		
		loopCountsByLengthPositive = new MutableInteger[maximumLoopLength];
		loopCountsByLengthNegative = new MutableInteger[maximumLoopLength];
		loopCountsByLengthUnknown = new MutableInteger[maximumLoopLength];
	}

	public void process() {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		loadNeverConsider();

		boolean useTimeAndMemoryRestrictedLoopDiscovery = true;
		boolean noIVVsuccess = true;

		boolean withIVVsuccess = true;
		List<List<String>> cycles = null;

		boolean allow = false;
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();

		System.out.println("############ NO Auto Discover ##################");

		try {
			Thread.sleep(10L);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		Reuse reuse = InformationManagers.getInstance().getReuse();

		//			here we can check for reuse of loops.
		//			Need i=to implement CyclesNoIVV and CyclesWithIVV files
		if (reuse.isLoops()) {
			if (options.isSelected(StringConstants.SUMMARY_REPORTS)) {
				PostProcessCycleMinimumMemory(false);
			} else {
				config.setCyclesFile(new CyclesFile(config.getFilename(false)));
				processFoundCycles(false, config.getCyclesFile().loadCyclesFromFile(config.getFilename(false), maximumLoopLength));
			}
		} else {
			if (useTimeAndMemoryRestrictedLoopDiscovery) {
				mf.addMessage(language.getText(StringConstants.USING_TIME_AND_MEMORY_RESTRICTED_LOOP_DISCOVERY));
				noIVVsuccess = loopTimingAndMemory(maximumLoopLength, false);
				if (foundCycles != null)
					this.processFoundCycles(false, foundCycles.getCycles());
				foundCycles = null;
			} else {

			}
		}
		singleCycleCount = 0;
		if (generateWithIVV) {
			if (reuse.isLoops() && config.fileExists(true)) {
				if (options.isSelected(StringConstants.SUMMARY_REPORTS)) {
					PostProcessCycleMinimumMemory(true);
				} else {
					config.setCyclesFile(new CyclesFile(config.getFilename(false)));
					processFoundCycles(true, config.getCyclesFile().loadCyclesFromFile(config.getFilename(false), maximumLoopLength));
				}
			} else {
				if (useTimeAndMemoryRestrictedLoopDiscovery) {
					withIVVsuccess = loopTimingAndMemory(maximumLoopLength, true);
					if (foundCycles != null)
					this.processFoundCycles(true, foundCycles.getCycles());
					foundCycles = null;
				} else {

				}
			}
		}

		//			mf.addMessage(maxLenMessage()+"100% -- # " + language.getText(StringConstants.LOOPS)
		//					+ " = " + allLoopsNoIVV.size() + "|" + allLoopsWithIVV.size()
		//					+ "\n");

		log.debug("Number of Loops No IVV: " + allLoopsNoIVV.size());
		log.debug("Number of Loops W/ IVV: " + allLoopsWithIVV.size());


		//		}

		boolean performExpandedLoopProcessing = !InformationManagers.getInstance().isLimitLoopProcessing() &&
				(options.isSelected(StringConstants.HYPERLINK_LOOP) || options.isSelected(StringConstants.DETAILED_REPORTS));
		// set loop start variables



		if (performExpandedLoopProcessing) {
			mf.addMessage(language.getText(StringConstants.SET_LOOP_START_VARIABLES));
			setLoopStartVariables();
			// assign Loop Numbers
			mf.addMessage(language.getText(StringConstants.ASSIGN_LOOP_NUMBERS));
			assignLoopNumbers();
			
			if (options.isSelected(StringConstants.HYPERLINK_LOOP)) {

				mf.addMessage(language.getText(StringConstants.WRITE_ALL_KNOWN_LOOPS));
				writeAllKnownLoops();
			}
		
		}

		options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.ANALYZE_LOOPS)) {
			if (options.isSelected(StringConstants.VARIABLE_RELATIONSHIPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
				if (noIVVsuccess) {
					boolean limitToLongestLoop = false;
					if (getNumberOfLoopsNoIVV() <= MAX_LOOPS_FOR_POSTPROCESS || (getNumberOfLoopsNoIVV() > MAX_LOOPS_FOR_POSTPROCESS
							&& options
							.isSelected(StringConstants.OVERRIDE_LOOP_COUNT))) {

						if (mf != null
								&& getNumberOfLoopsNoIVV() > MAX_LOOPS_FOR_POSTPROCESS
								&& options
								.isSelected(StringConstants.OVERRIDE_LOOP_COUNT)) {
							limitToLongestLoop = false;
							mf.addMessage(language.getText(StringConstants.OVERRIDING_LOOP_LIMIT_PER_USER_REQUEST));
						}

						if (mf != null) {
							mf.addMessage(language.getText(StringConstants.DETERMINING_VARIABLE_RELATIONSHIPS_FROM_DISCOVERED_LOOPS));
						}

						gatherVariableRelationships(limitToLongestLoop);

						variableRelationships.writeVariableRelationships();
					} else {

						mf.addMessage(language.getText(StringConstants.TOO_MANY_LOOPS_TO_GENERATE_VARIABLE_RELATIONSHIP_REPORT)+" "
								+ getNumberOfLoopsNoIVV());
					}
				} else {

					mf.addMessage(language.getText(StringConstants.LOOP_DISCOVERY_FAILURE_PRECLUDES_GATHERING_VARIABLE_LOOPS));
				}
			}
			if (options.isSelected(StringConstants.VARIABLE_LOOPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {

				if (noIVVsuccess) {
					if (getNumberOfLoopsNoIVV() <= MAX_LOOPS_FOR_POSTPROCESS
							|| options
							.isSelected(StringConstants.OVERRIDE_LOOP_COUNT)) {

						if (getNumberOfLoopsNoIVV() > MAX_LOOPS_FOR_POSTPROCESS
								&& options
								.isSelected(StringConstants.OVERRIDE_LOOP_COUNT)) {
							mf.addMessage(language.getText(StringConstants.OVERRIDING_LOOP_LIMIT_PER_USER_REQUEST));
						}

						mf.addMessage(language.getText(StringConstants.GENERATING_VARIABLE_LOOPS_REPORT));
						gatherVariableLoops();

						// ############## WRITE_DEBUG_FILES
						if (StaticSwitches.WRITE_DEBUG_FILES) {
							variableLoops.writeVariableLoops();
						}
					} else {
						mf.addMessage(language.getText(StringConstants.TOO_MANY_LOOPS_TO_GENERATE_VARIABLE_LOOP_REPORT)+": "
								+ getNumberOfLoopsNoIVV());
					}
				} else {
					mf.addMessage(language.getText(StringConstants.LOOP_DISCOVERY_FAILURE_PRECLUDES_GATHERING_VARIABLE_LOOPS));
				}
			}
		}

		mf.setBlipMode(false);

	}

	private void writeLoopsToDB() {
		DatabaseManager db = InformationManagers.getInstance()
				.getDatabaseManager();

		List<Loop> al = orderByLoopNumber(allLoopsNoIVV);
		db.writeLoops(al, NO_IVV);

		al = orderByLoopNumber(allLoopsWithIVV);
		db.writeLoops(al, WITH_IVV);

	}

	public void releaseLoopStorage() {

		if (allLoopsNoIVV != null) {
			allLoopsNoIVVSize = allLoopsNoIVV.size();
		} else {
			allLoopsNoIVVSize = 0;
		}
		if (allLoopsWithIVV != null) {
			allLoopsWithIVVSize = allLoopsWithIVV.size();
		} else {
			allLoopsWithIVVSize = 0;
		}
		allLoopsWithIVV = null;
		knownLoopNamesWithIVV = null;
		loopMembershipWithIVV = null;

		allLoopsNoIVV = null;
		knownLoopNamesNoIVV = null;
		loopMembershipNoIVV = null;

		setUseDB(true);
	}

	// private void panic() {
	// System.out.println("Panic!");
	// System.out.println("Panic!");
	// }
	
	private Set<String> variablesInLongestLoop() {
		Set<Loop> longest = getLoopsOfLength(getMaximumLoopLength());
		Set<String> varsInLongest = getVarsInLoops(longest);
		return varsInLongest;
	}
	
	private Set<String> getVarsInLoops(Set<Loop> longest) {
		Set<String> varsInLongest = new LinkedHashSet<String>();
		for (Loop l : longest) {
			for (String v : l.loop) {
				varsInLongest.add(v);
			}
		}
		
		return varsInLongest;
	}
	
	private void gatherVariableRelationships(boolean limitToLongestLoop) {
		
//		List<String> varsQuote = new ArrayList<String>(
//				InformationManagers.getInstance().getEquationManager().getVariables()); 
//	
//		gatherVariableRelationships(varsQuote, limitToLongestLoop);
		
		System.out.println("gatherVariableRelationships limitToLongestLoop "+limitToLongestLoop);
		
		Set<Loop> loopsToprocess = null;
		
		if (limitToLongestLoop) {
			loopsToprocess = getLoopsOfLength(getLongestLoopLengthNoIVV());
			int sizeAll = loopsToprocess.size();
			loopsToprocess = limitLongest(loopsToprocess);
			
			mf.addMessage("Using "+loopsToprocess.size()+" longest loops out of "+sizeAll);
			
//			JOptionPane.showMessageDialog(null,
//					"Using "+loopsToprocess.size()+" longest loops out of "+sizeAll,
//					"Variable Relationship Limit", 
//					JOptionPane.INFORMATION_MESSAGE);
		} else {
			loopsToprocess = new LinkedHashSet<Loop>(getLoopsOrderedBySize());
		}
		
		System.out.println("gatherVariableRelationships numLoops "+loopsToprocess.size());
		
		gatherVariableRelationships(loopsToprocess);
	}
	
	private Set<Loop> limitLongest(Set<Loop> longest) {

		int countLimiter = 0;
		Set<Loop> limited = new LinkedHashSet<Loop>();
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		int max = Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_LONGEST_LOOPS));

		for (Loop l : longest) {
			if (countLimiter++ >= max)
				break;
			limited.add(l);
		}
		return limited;

	}
	/*
	 * Set<Loop> longestLoops = getLoopsOfLength(getMaximumLoopLength());
		longestLoops = limitLongest(longestLoops);
	 */
	
	private void gatherVariableRelationships(Set<Loop> loopsToProcess) {
		int numLoops = loopsToProcess.size();
		int numLoopsProcessed = 0;
		int num10 = numLoops / 10;
		int num10Processed = 0;
		int num10Reached = 0;
		int processedPairs = 0;
		
		for (Loop l : loopsToProcess) {
			processedPairs += processLoop(l);
			numLoopsProcessed++;
			num10Processed++;
			if (num10Processed == num10) {
				num10Reached++;
				mf.addMessage(num10Reached+"0%"); // + " Processed Loops: "+NumberFormatter.formatForIntComma(numLoopsProcessed)+" Pairs: "+
//						NumberFormatter.formatForIntComma(processedPairs)+
//						" Free Memory: "+NumberFormatter.formatForPercent(HeapMonitorThread.freeMemory()));
				num10Processed = 0;
			}
			if (numLoopsProcessed == numLoops && num10Reached != 10) {
				mf.addMessage("100%");
			}
		}
	}
	
	private int processLoop(Loop loop) {
		/*
		 * The list of variables that are in the loop
		 */
		List<String> varList = loop.withVariableName(loop.getLoop());
		varList.remove(varList.size()-1);
		int numVars = varList.size();
		
		int processedPairs = 0;
		
		/*
		 * Loop over the variables in the list
		 */
		for (int varPtr = 0; varPtr < numVars-1; varPtr++) {
			
			/*
			 * want to compute distances from this variable to all other variables
			 * that appear in the loop
			 */
			String startVar = varList.get(varPtr);
			int startPos = varPtr;
			
			/*
			 * endPos points to the variable to which the distance is computed
			 */
			int endPos = varPtr+1;
			if (endPos >= varList.size()) {
				endPos = 0;
			}
			String endVar = varList.get(endPos);
			
//			if (variableRelationships.pairAlreadyProcessed(startVar, endVar))
//					continue;
			
			/*
			 * Since loop wraps around, we will need to adjust ptr sequence 
			 */
		
			while(endPos < varList.size()) {
				endVar = varList.get(endPos);
				processPair(loop, numVars, startVar, endVar, startPos, endPos);
				processedPairs++;
				endPos++;

			}
		}
		return processedPairs;
	}
	
	public void processPair(Loop loop, int numVars, String startVar, String endVar, 
			int startPos, int endPos) {
		int distAB = 0;
		int distBA = 0;
		
		if (startPos < endPos) {
			distAB = endPos - startPos;
		} else {
			distAB = numVars - startPos + endPos;
		}
		distBA = numVars - distAB;		
		
		VariableRelationship vr = variableRelationships.getVariableRelationship(startVar, endVar);
		vr.addLoop(loop, distAB, distBA);
	}

	private void gatherVariableRelationships(List<String> varsQuote, boolean limitToLongestLoop) {
		Set<String> processed = new LinkedHashSet<String>();
		int varCount = 0;
		
		List<String> vars = new ArrayList<String>();
		for (String v : varsQuote) {
			vars.add(Parser.removeQuotes(v)); // .replace("\"", ""));
		}
		Collections.sort(vars);
		
		Set<Loop> longestLoops = getLoopsOfLength(getMaximumLoopLength());
		longestLoops = limitLongest(longestLoops);
		
		System.out.println("Num Vars: "+vars.size()+" Num Combos: "+(vars.size()*vars.size()));
		System.out.println("limit to longest loops = "+limitToLongestLoop);
		System.out.println("Number of Loops of MAX Length "+getMaximumLoopLength()+" = "+longestLoops.size());
		
		for (String varA : vars) {
			Set<Loop> setA = getLoopsContaining(varA);
			if (limitToLongestLoop) {
				setA = intersect(setA, longestLoops);
			}
			
			for (String varB : vars) {
				if (varA.equals(varB))
					continue;
				if (processed.contains(varA + "#" + varB)
						|| processed.contains(varB + "#" + varA))
					continue;
				
//				 if (countLimiter++ > MAX_LOOPS_FOR_POSTPROCESS)
//				 break;
				
				varCount++;
				processed.add(varA + "#" + varB);

				Set<Loop> setB = getLoopsContaining(varB);
				if (limitToLongestLoop) {
					setB = intersect(setB, longestLoops);
				}
				
				if (varCount % 50 == 0)
					System.out.println("Combo Count: "+varCount);

				Set<Loop> intersect = intersection(setA, setB);
				if (intersect.size() == 0)
					continue;
				VariableRelationship vr = new VariableRelationship(varA, varB);
				vr.addLoops(intersect);

				variableRelationships.put(vr);
			}
		}
	}

	/**
	 * Alternate method for loop-variable relationship that uses linear algebra
	 * approach.
	 * 
	 * TODO This method counts A-B and B-A variable relationships separately,
	 * and need to be filtered to only include the first, as is done in the
	 * original method.
	 */
	private void gatherVariableRelationships2() {
		Set<String> processed = new LinkedHashSet<String>();
		// List<String> vars = new
		// ArrayList<String>(InformationManagers.getInstance().getEquationManager().getVariables());
		List<String> vars = new ArrayList<String>(InformationManagers
				.getInstance().getMemoryMapper().getStocks());
		Collections.sort(vars);

		// TODO correct loop count?
		long numLoop = allLoopsNoIVV != null ? allLoopsNoIVV.size()
				: allLoopsNoIVVSize;
		Map<Integer, Loop> loopIDMap = new LinkedHashMap<Integer, Loop>();

		// Construct a matrix with var rows and loop columns whos entries are
		// 1,0
		// to indicate variable membership in a loop
		RealMatrix varLoopMat = MatrixUtils.createRealMatrix(vars.size(),
				(int) (numLoop + 1));
		for (int i = 0; i < vars.size(); i++) {
			Set<Loop> loops = getLoopsContaining(vars.get(i));

			// Set the column values for this var row to 1 for loop indices
			for (Loop loop : loops) {
				varLoopMat.setEntry(i, loop.getLoopNumber(), 1);
				loopIDMap.put(loop.getLoopNumber(), loop);
			}
		}

		// Construct a variable - variable matrix who's entries indicate how
		// many
		// shared loops exist between each variable.
		RealMatrix tran = varLoopMat.transpose();
		RealMatrix varvarMat = varLoopMat.multiply(tran);
		tran = null;

		for (int i = 0; i < varvarMat.getRowDimension(); i++) {
			double[] val = varvarMat.getRow(i);

			System.out.println(i + " / " + varvarMat.getRowDimension());

			for (int j = 0; j < val.length; j++) {
				// Don't consider variable self relationship
				if (i == j)
					continue;

				if (val[j] > 0) {
					RealVector varLoopVec = varLoopMat.getRowVector(i)
							.ebeMultiply(varLoopMat.getRowVector(j));

					Set<Loop> intersect = new LinkedHashSet();
					for (int k = 0; k < varLoopVec.getDimension(); k++) {
						double v = varLoopVec.getEntry(k);

						if (v > 0) {
							intersect.add(loopIDMap.get(k));
						}
					}
					if (intersect.size() > 0) {
						// TODO this part is still very slow due to Loop.dist()
						VariableRelationship vr = new VariableRelationship(
								vars.get(i), vars.get(j));
						vr.addLoops(intersect);
						variableRelationships.put(vr);
					}
				}
			}
		}
	}

	private void gatherVariableRelationships3() {
		Set<String> processed = new LinkedHashSet<String>();
		// List<String> vars = new
		// ArrayList<String>(InformationManagers.getInstance().getEquationManager().getVariables());
		List<String> vars = new ArrayList<String>(InformationManagers
				.getInstance().getMemoryMapper().getStocks());
		Collections.sort(vars);

		for (String varA : vars) {
			for (String varB : vars) {
				if (varA.equals(varB))
					continue;
				if (processed.contains(varA + "#" + varB)
						|| processed.contains(varB + "#" + varA))
					continue;
				// if (countLimiter++ > 100)
				// break;
				processed.add(varA + "#" + varB);

				Set<RelationshipLoop> intersect = InformationManagers
						.getInstance()
						.getDatabaseManager()
						.getDb()
						.getRelationshipLoops(varA, varB, Utilities.nameToID(varA),
								Utilities.nameToID(varB));
				if (intersect.size() == 0)
					continue;

				// we want a set of RelationshipLoops. These are constructed in
				// SDMDocDatabase
				VariableRelationship vr = new VariableRelationship(varA, varB);
				vr.addRelationshipLoops(intersect);

				variableRelationships.put(vr);
			}
		}
	}
	
	public List<VariableLoopCount> gatherVariableLoopCountInformation() {
		
		List<VariableLoopCount> al = new ArrayList<VariableLoopCount>();

		if (isUseDB()) {
			return gatherVariableLoopCountInformationDB();
		} else {
			Set<String> processed = new LinkedHashSet<String>();
			List<String> varsQuote = new ArrayList<String>(
					InformationManagers.getInstance().getEquationManager().getVariables()); 
			List<String> vars = new ArrayList<String>();
			for (String v : varsQuote) {
				vars.add(Parser.removeQuotes(v)); // v.replace("\"", ""));
			}
			
			Collections.sort(vars);
			
			for (String varA : vars) {
				if (processed.contains(varA))
					continue;
				processed.add(varA);
				Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(varA);
//				int count = getLoopsContaining(varA).size();
				int count = getNumberOfLoopsWith(varA);
				
				al.add(new VariableLoopCount(varA, eqn, count));
			}
		}
		
		return al;
	}
	
	private List<VariableLoopCount> gatherVariableLoopCountInformationDB() {
		
		List<VariableLoopCount> al = new ArrayList<VariableLoopCount>();
		
		Set<String> processed = new LinkedHashSet<String>();
		List<String> vars = new ArrayList<String>(
				InformationManagers.getInstance().getEquationManager().getVariables());
		Collections.sort(vars);

		for (String varA : vars) {
			if (processed.contains(varA))
				continue;
			processed.add(varA);
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(varA);
			int count = InformationManagers.getInstance()
					.getDatabaseManager().getDb()
					.getNoIVVLoopsContaining(varA, Utilities.nameToID(varA)).size();
			
			al.add(new VariableLoopCount(varA, eqn, count));
		}
		
		return al;
	}

	private void gatherVariableLoops() {
		
		if (variableLoops.size() > 0)
			return;

		if (isUseDB()) {
			gatherVariableLoopsDB();
		} else {
			Set<String> processed = new LinkedHashSet<String>();
			List<String> varsQuote = new ArrayList<String>(
					InformationManagers.getInstance().getEquationManager().getVariables()); // InformationManagers.getInstance().getEquationManager().getVariables()
														// vensimEquationManager.getAllKnowVariables()
			List<String> vars = new ArrayList<String>();
			for (String v : varsQuote) {
				vars.add(Parser.removeQuotes(v)); // v.replace("\"", ""));
			}
			Collections.sort(vars);
			for (String varA : vars) {
				if (processed.contains(varA))
					continue;
				processed.add(varA);
				Set<Loop> setA = getLoopsContaining(varA);
				VariableLoop vr = new VariableLoop(varA);
				vr.addLoops(setA);
				variableLoops.put(vr);
			}
		}
	}

	private void gatherVariableLoopsDB() {
		Set<String> processed = new LinkedHashSet<String>();
		List<String> vars = new ArrayList<String>(
				InformationManagers.getInstance().getEquationManager().getVariables());
		Collections.sort(vars);

		for (String varA : vars) {
			if (processed.contains(varA))
				continue;
			processed.add(varA);
			Set<Loop> setA = InformationManagers.getInstance()
					.getDatabaseManager().getDb()
					.getNoIVVLoopsContaining(varA, Utilities.nameToID(varA));
			VariableLoop vr = new VariableLoop(varA);
			vr.addLoops(setA);
			variableLoops.put(vr);
		}
	}

	private Set<Loop> intersection(Set<Loop> setA, Set<Loop> setB) {
		Set<Loop> intersect = new LinkedHashSet<Loop>(setA);
		intersect.retainAll(setB);
		return intersect;
	}

	private void analyzeEffectiveLoopCount() {

	}

	// public void getLoopLengthCounts(Map<Integer, MutableInteger> noIVV,
	// Map<Integer, MutableInteger> withIVV) {
	// if (allLoopsNoIVV == null)
	// panic();
	// if (allLoopsWithIVV == null)
	// panic();
	// getLoopLengthCounts(noIVV, allLoopsNoIVV);
	// getLoopLengthCounts(withIVV, allLoopsWithIVV);
	// }
	//
	// public void getLoopLengthCounts(Map<Integer, MutableInteger> countMap,
	// Set<Loop> allLoops) {
	// for (Loop loop : allLoops) {
	// int len = loop.getSize();
	// if (!countMap.containsKey(len))
	// countMap.put(len, new MutableInteger(0));
	// countMap.get(len).add(1);
	// }
	// }

	public boolean isReachable(String var1, String var2) {
		return reachability.isReachable(var1, var2);
	}

	private void setLoopStartVariables() {
		for (Loop loop : getAllLoopsNoIVV()) {
			loop.setLoopStartVariable();
		}
		for (Loop loop : getAllLoopsWithIVV()) {
			loop.setLoopStartVariable();
		}
	}

	private void assignLoopNumbers() {

		Map<Integer, MutableInteger> extended = new LinkedHashMap<Integer, MutableInteger>();
		for (int i = 0; i <= getAllLoopsNoIVV().size(); i++) {
			extended.put(i, new MutableInteger(0));
		}
		int major = 1;
		int minor = 0;
		List<Loop> noIVV = orderSetBySizeAndVariables(getAllLoopsNoIVV());
		for (Loop loop : noIVV) {
			loop.setLoopNumber(major++);
			loop.setExtendedLoopNumber(minor);
		}

		major = 1;
		int noIVVptr = 0;
		int withIVVptr = 0;
		Loop with;
		Loop no;

		// order the withIVV. withIVV is always a superset of noIVV
		List<Loop> withIVV = orderSetBySizeAndVariables(getAllLoopsWithIVV());

		// until we are done
		while (withIVVptr < withIVV.size()) {
			with = withIVV.get(withIVVptr);

			// ptr points sequentially to noIVV until at the end of the list
			int ptr = Math.min(noIVVptr, noIVV.size() - 1);
			no = noIVV.get(ptr);

			// two ordered lists, two pts
			// each loop in noIVV always in withIVV
			// find thouse not in IVV

			int compare = no.compareTo(with);

			if (compare < 0) {
				// with needs to go before the no
				int prev = no.getLoopNumber() - 1;
				MutableInteger mi = extended.get(prev);
				mi.add(1);
				with.setLoopNumber(prev);
				with.setExtendedLoopNumber(mi.value());
				with.setOnlyIVV(true);
				// do not increment noIVVptr as the next withIVV loop may be <
				// no
				withIVVptr++;
			} else if (compare == 0) {
				// if loops are equal, then share same loop and extended numbers
				with.setLoopNumber(no.getLoopNumber());
				with.setExtendedLoopNumber(no.getExtendedLoopNumber());

				// increment both pointers
				noIVVptr++;
				withIVVptr++;
			} else if (compare > 0) {
				// with needs to go after no
				if (noIVVptr >= noIVV.size()) {
					// we are already past the last no
					// so lets just leave major the same and increment minors
					no = noIVV.get(noIVV.size() - 1);
					while (withIVVptr < withIVV.size()) {
						with = withIVV.get(withIVVptr++);
						with.setLoopNumber(no.getLoopNumber());
						MutableInteger mi = extended
								.get(no.getLoopNumber() - 1);
						mi.add(1);
						with.setExtendedLoopNumber(mi.value());
						with.setOnlyIVV(true);
					}

				} else {
					// after no
					with.setLoopNumber(no.getLoopNumber());
					MutableInteger mi = extended.get(no.getLoopNumber() - 1);
					mi.add(1);
					with.setExtendedLoopNumber(mi.value());
					with.setOnlyIVV(true);
					// noIVVptr++;
					withIVVptr++;
				}
			} else {

			}

		}
	}

	// private void printAllKnownLoops() {
	// int i = 1;
	// for (Loop loop : orderSetBySize(getAllLoopsNoIVV())) {
	// log.debug(" ### Loop ###");
	// loop.print(i++);
	// }
	// }

	public void writeVariableRelationships(String html) {
		StringBuilder sb = new StringBuilder();
		sb.append(getHtmlHead("Variable Relationships"));
		sb.append(DocumentHeader.STYLE);
		sb.append(html);
		sb.append(getHtmlTail());
		Utilities.writeToFileCharset(
				getGeneratedHtmlVariableRelationshipLoopsFileName(),
				sb.toString());
	}

	public void writeVariableLoops(String html) {
		StringBuilder sb = new StringBuilder();
		sb.append(getHtmlHead("Variable Loops"));
		sb.append(DocumentHeader.STYLE);
		sb.append(html);
		sb.append(getHtmlTail());
		Utilities.writeToFileCharset(getGeneratedHtmlVariableLoopsFileName(),
				sb.toString());
	}

	public String getHtmlLoopFileNameNoIVV() {
		return getHtmlLoopFileName("NoIVV");
	}

	public String getHtmlLoopFileNameWithIVV() {
		return getHtmlLoopFileName("WithIVV");
	}

	public String getHtmlLoopFileName(String type) {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		return StringEscapeUtils.escapeHtml4(InformationManagers.getInstance()
				.getFileOrganizer().getRelativeLoopsDirectory()
				+ "/"
			//	+ mm.getModelName()
				+ "_Loops"
				+ type
				+ "_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".txt"); // ".html");

	}

	public String getHtmlVariableLoopsFileName() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String res = InformationManagers.getInstance().getFileOrganizer()
				.getRelativeLoopsDirectory()
				+ "/"
				// + mm.getModelName()
				+ "_Variable_Loops_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
		return StringEscapeUtils.escapeHtml4(res);

	}

	public String getHtmlVariableRelationshipLoopsFileName() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String res = InformationManagers.getInstance().getFileOrganizer()
				.getRelativeLoopsDirectory()
				+ "/"
			//	+ mm.getModelName()
				+ "_VariableRelationship_Loops_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
		return StringEscapeUtils.escapeHtml4(res);

	}

	public String getGeneratedHtmlVariableLoopsFileName() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String res = InformationManagers.getInstance().getFileOrganizer()
				.getLoopsDirectory()
				+ "/"
			//	+ mm.getModelName()
				+ "_Variable_Loops_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
		return StringEscapeUtils.escapeHtml4(res);

	}

	public String getGeneratedHtmlVariableRelationshipLoopsFileName() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String res = InformationManagers.getInstance().getFileOrganizer()
				.getLoopsDirectory()
				+ "/"
			//	+ mm.getModelName()
				+ "_VariableRelationship_Loops_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage()
				+ ".html";
		return StringEscapeUtils.escapeHtml4(res);

	}

	private void writeNeverConsider() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();

		String file = InformationManagers.getInstance().getFileOrganizer()
				.getLoopsDirectory()
				+ "/" + /*mm.getModelName() + */ "_neverConsider.tab";
		BufferedWriter bw = Utilities.openFileForWriting(file);

		try {
			for (String var : neverConsider.keySet()) {
				bw.append(var + "\n");
				for (String v : neverConsider.get(var)) {
					bw.append("\t" + v + "\n");
				}
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeEdges(String tag, List<String[]> edges) {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String file = InformationManagers.getInstance().getFileOrganizer()
				.getLoopsDirectory()
				+ "/" + /* mm.getModelName() +*/ "_" + tag + "_edges.tab";
		BufferedWriter bw = Utilities.openFileForWriting(file);
		try {
			for (String[] edge : edges) {
				bw.append(edge[0] + "\t" + edge[1] + "\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

//	private List<String[]> loadEdges(boolean withIVV) {
//
//		List<String[]> edges = vensimEquationManager.getEdges();
//
//		if (withIVV) {
//			writeEdges("WithIVV", edges);
//			return edges;
//		} else {
//			edges = filterEdges(edges, neverConsider);
//			writeEdges("NoIVV", edges);
//			return edges;
//		}
//	}

	private List<String[]> loadEdgesNoWrite(boolean withIVV) {

		List<String[]> edges;
		if (cachedIdEdges != null)
			return cachedIdEdges;

		if (!InformationManagers.getInstance().getModelManager().isMap())
			edges = vensimEquationManager.getEdges();
		else
			edges = InformationManagers.getInstance().getGraphicsProcessor().getEdges();

		if (!withIVV) {
			edges = filterEdges(edges, neverConsider);
		}
		cachedIdEdges =  convertToID(edges);
		return cachedIdEdges;
	}
	
	private List<String[]> convertToID(List<String[]> edges) {
		List<String[]> ids = new ArrayList<String[]>();

		for (String[] anEdge : edges) {
			ids.add(new String[] {Utilities.nameToID(anEdge[0]), Utilities.nameToID(anEdge[1])});
		}
		return ids;
	}

	private List<String[]> filterEdges(List<String[]> edges,
			Map<String, Set<String>> remove) {
		List<String[]> filtered = new ArrayList<String[]>();
		for (String[] edge : edges) {
			String tail = edge[0];
			String head = edge[1];
			if (remove.containsKey(tail)
					&& (remove.get(tail).contains(head) || remove.get(tail)
							.contains(tail)))
				continue;
			filtered.add(edge);
		}

		return filtered;
	}

	private void writeAllKnownLoops() {
		
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		List<Loop> al = orderByLoopNumber(allLoopsNoIVV);
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();

		String file = /*mm.getModelName() + */ "_LoopsNoIVV_"
				+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage();
		
		if(options.isSelected(StringConstants.ANALYZE_LOOPS)) {
			// ############## WRITE_DEBUG_FILES
			if (StaticSwitches.WRITE_DEBUG_FILES) {
//				writeLoopsToTextFile(file + ".txt", al);
				writeLoopsToTabFile(InformationManagers.getInstance()
						.getFileOrganizer().getStructureDirectory()
						+ "/loopsNoIVV.tab", al);
			}
			
//			writeLoopsToHtmlFile(file + ".html", al, "No IVV");
			writeLoopsToTextFile(file + ".txt", al);
			writeLoopsToHtmlFile(file, ".html", al, "No IVV");
		}
		
		if(options.isSelected(StringConstants.ANALYZE_LOOPS_WITH_IVV)) {

			al = orderByLoopNumber(allLoopsWithIVV);

			file = /*mm.getModelName() + */ "_LoopsWithIVV_"
					+ SDMDocLanguageSpecific.getInstance().getDisplayLanguage();
			// ############## WRITE_DEBUG_FILES
			if (StaticSwitches.WRITE_DEBUG_FILES) {
				writeLoopsToTextFile(file + ".txt", al);

				writeLoopsToTabFile(InformationManagers.getInstance()
						.getFileOrganizer().getStructureDirectory()
						+ "/loopsWithIVV.tab", al);
			}
			
			writeLoopsToTextFile(file + ".txt", al);
			writeLoopsToHtmlFile(file + ".html", al, "With IVV");
		}
	}

	private void analyzeLoopDerived(GraphTheoryMatrix loopDerivedReachability) {

		Map<String, String> rowNameValueMap = new LinkedHashMap<String, String>();
		Map<String, Set<String>> rowValueNameMap = new LinkedHashMap<String, Set<String>>();

		Map<String, String> colNameValueMap = new LinkedHashMap<String, String>();
		Map<String, Set<String>> colValueNameMap = new LinkedHashMap<String, Set<String>>();

		for (String s : loopDerivedReachability.keySet()) {
			String rv = loopDerivedReachability.getRowValue(s);
			String cv = loopDerivedReachability.getColValue(s);
			rowNameValueMap.put(s, rv);
			colNameValueMap.put(s, cv);
			if (!rowValueNameMap.containsKey(rv))
				rowValueNameMap.put(rv, new LinkedHashSet<String>());
			if (!colValueNameMap.containsKey(cv))
				colValueNameMap.put(cv, new LinkedHashSet<String>());
		}

		for (String s : rowNameValueMap.keySet()) {
			String rv = rowNameValueMap.get(s);
			rowValueNameMap.get(rv).add(s);
		}

		for (String s : colNameValueMap.keySet()) {
			String cv = colNameValueMap.get(s);
			colValueNameMap.get(cv).add(s);
		}

		for (String s : rowValueNameMap.keySet()) {

			Set<String> names = rowValueNameMap.get(s);

			log.debug("\n\n### Row Group Name " + s);
			log.debug("Num Members: " + names.size());
			for (String n : names) {
				log.debug("  " + n);
			}
		}

		for (String s : colValueNameMap.keySet()) {
			Set<String> names = colValueNameMap.get(s);

			log.debug("\n\n### Column Group Name " + s);
			log.debug("Num Members: " + names.size());
			for (String n : names) {
				log.debug("  " + n);
			}

		}
		log.debug("\n\n");

	}

	private void loadNeverConsider() {
		neverConsider = vensimEquationManager.getAllInitialValueVariables();
		// ############## WRITE_DEBUG_FILES
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			writeNeverConsider();
		}
	}

	public boolean isInitialValueVariableUse(String tailVariable,
			String headVariable) {
		if (!neverConsider.containsKey(tailVariable))
			return false;
		return neverConsider.get(tailVariable).contains(headVariable)
				|| neverConsider.get(tailVariable).contains(tailVariable);
	}
	
	public void processCycleMinimumMemory(List<String> aLoop, boolean withIVV) 
			throws java.lang.OutOfMemoryError{

		
		singleCycleCount++;
		if (singleCycleCount % 100000 == 0) {
			System.out.println("Single Cycle Count: "+singleCycleCount);
		}

		Map<String, VariableLoopPolarityCounts> countsMap = variableLoopPolarityCounts.get(withIVV);
		if (countsMap == null) {
			countsMap = new LinkedHashMap<String, VariableLoopPolarityCounts>();
			countsMap.put("__ALL__", new VariableLoopPolarityCounts());
		}

		aLoop.add(aLoop.get(0));

		//				Loop l = new MinimumProcessLoop(getCycleAsVariable(aLoop));
		Loop l = new MinimumProcessLoop(aLoop);

		/// HERE @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

		shortestLoopLengthNoIVV = Math.min(shortestLoopLengthNoIVV,	l.getSize());
		longestLoopLengthNoIVV = Math.max(longestLoopLengthNoIVV, l.getSize());
		l.determineMinimumProcessLoopPolarity(polarityResultsNoIVV);
		String polarity = l.getLoopPolarity();
		if (!withIVV) {
			shortestLoopLengthNoIVV = Math.min(shortestLoopLengthNoIVV,	l.getSize());
			longestLoopLengthNoIVV = Math.max(longestLoopLengthNoIVV, l.getSize());
//			l.determineMinimumProcessLoopPolarity(polarityResultsNoIVV);
//			polarity = l.getLoopPolarity();
			lla.addLoopInformation(l);
		} else {
			shortestLoopLengthWithIVV = Math.min(shortestLoopLengthWithIVV,	l.getSize());
			longestLoopLengthWithIVV = Math.max(longestLoopLengthWithIVV, l.getSize());
//			l.determineMinimumProcessLoopPolarity(polarityResultsNoIVV);
//			polarity = l.getLoopPolarity();
			lla.addLoopInformation(l);
		}

		aLoop.remove(0);
		for (String varN : aLoop) {
			String var = Parser.quoted(Utilities.idToName(varN));
			if (!countsMap.containsKey(var))
				countsMap.put(var, new VariableLoopPolarityCounts());
			countsMap.get(var).count(polarity, aLoop.size());
		}
		countsMap.get("__ALL__").count(polarity, aLoop.size());		
	}
	
	public void processSingleCyclesFromFile(int maxLen, boolean withIVV) {
		
		CyclesFile cyclesFile = config.getCyclesFile();
		cyclesFile.closeWrite();
		cyclesFile.openForRead();
		Vector v = cyclesFile.read();
		while (v != null) {
			if (v.size() <= maxLen)
				processCycleMinimumMemory(v, withIVV);
			v = cyclesFile.read();
		}
		
		cyclesFile.closeRead();
	}
	
	private void PostProcessCycleMinimumMemory(boolean withIVV) {

		if ( InformationManagers.getInstance().getReuse().isLoops()) {
			config.setCyclesFile(new CyclesFile(config.getFilename(withIVV)));
			processSingleCyclesFromFile(maximumLoopLength, withIVV);
		} else if (config.isRequiresLoadAndProcess()) {
			if (foundCycles != null) {
				foundCycles.processSingleCycles(maximumLoopLength, withIVV);
			} else {
				System.out.println("Cannot foundCycles.processSingleCycles(maximumLoopLength, withIVV) as foundCycles is null");
			}
		}
		
		if (withIVV)
			return;

		lla.aggregate(longestLoopLengthNoIVV != Integer.MIN_VALUE ? longestLoopLengthNoIVV : 2);
		lla.write(Utilities.openFileForWriting(InformationManagers.getInstance().
				getFileOrganizer().getStructureDirectory()+"/LoopLengthAnalysis.tab"));
	}
	
	public List<String> getCycleAsVariable(List<String> cycle) {
		List<String> vars = new ArrayList<String>();
		for (String l : cycle) {
				vars.add(Utilities.idToName(l));
		}
		return vars;
	}

	
	private void processCyclesMinimumMemory(List<List<String>> cycles, boolean withIVV) 
			throws java.lang.OutOfMemoryError{
		usingMinimumMemory = true;
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		Map<String, VariableLoopPolarityCounts> countsMap = variableLoopPolarityCounts.get(withIVV);
		
		System.out.println("processCyclesMinimumMemory: Total Cycles = "+cycles.size());
		mf.addMessage(maxLenMessage()+language.getText(StringConstants.TOTAL_CYCLES)+" = "+NumberFormatter.formatForIntComma(cycles.size()));
		
		int numLoops = cycles.size();
		int interval = Math.max(numLoops / 10, 1);
		
		if (numLoops == 0)
			return;
		
		int num = 1;
		int pct = 10;
		
//		lla.extendTo(maximumLoopLength);


		
			int cyclesProcessed = 0;
			for (List<String> aLoop : cycles) {
//				if (cyclesProcessed > 10000)
//					break;
				aLoop.add(aLoop.get(0));
				
//				Loop l = new Loop(aLoop);
				Loop l = new MinimumProcessLoop(aLoop);
				
				/// HERE @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
//				polarityResultsNoIVV.createArrowsFromLoop(l);
				
				shortestLoopLengthNoIVV = Math.min(shortestLoopLengthNoIVV,	l.getSize());
				longestLoopLengthNoIVV = Math.max(longestLoopLengthNoIVV, l.getSize());
				l.determineMinimumProcessLoopPolarity(polarityResultsNoIVV);
				String polarity = l.getLoopPolarity();
				if (!withIVV) {
					lla.addLoopInformation(l);
				}

				aLoop.remove(0);
				for (String var : aLoop) {
					if (!countsMap.containsKey(var))
						countsMap.put(var, new VariableLoopPolarityCounts());
					countsMap.get(var).count(polarity, aLoop.size());
				}
				countsMap.get("__ALL__").count(polarity, aLoop.size());
				cyclesProcessed++;
				if (num % interval == 0 && pct < 100) {
					mf.addMessage(maxLenMessage()+language.getText(StringConstants.PROGRESS)+": "+pct+"% "+language.getText(StringConstants.OF)+" "+
				NumberFormatter.formatForIntComma(cycles.size()));
					pct += 10;
				}
				num++;
				
			}
			
//			lla.analyze(2, maximumLoopLength);
		
			if (!withIVV) {
				lla.aggregate(longestLoopLengthNoIVV != Integer.MIN_VALUE ? longestLoopLengthNoIVV : 2);
				lla.write(Utilities.openFileForWriting(InformationManagers.getInstance().
						getFileOrganizer().getStructureDirectory()+"/LoopLengthAnalysis.tab"));
			}

			mf.addMessage(maxLenMessage()+language.getText(StringConstants.PROGRESS)+": 100% "+
			language.getText(StringConstants.OF)+" "+NumberFormatter.formatForIntComma(numLoops)+"\n");
		}
	

	private void processCycles(List<List<String>> cycles, boolean withIVV)
			throws java.lang.OutOfMemoryError {
		usingMinimumMemory = false;
		
		System.out.println("processCycles: Total Cycles = "+cycles.size());
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<Loop> someLoops = new ArrayList<Loop>();

		int hardcodedLoopCountLimit = HARDCODED_LOOP_COUNT_LIMIT;
		int loopCount = 1;
		
		for (int i = cycles.size()-1; i >= 0; i--) {

			List<String> aLoop = cycles.get(i);
			
//			for (List<String> aLoop : cycles) {
				//			System.out.println("cycle: "+aLoop);
				if (loopCount++ > hardcodedLoopCountLimit)
					break;
				aLoop.add(aLoop.get(0));
				someLoops.add(new Loop(aLoop));
//			}
				cycles.remove(i);
		}

		for (Loop loop : someLoops) {
			if (!withIVV) {
//				polarityResultsNoIVV.createArrowsFromLoop(loop);
				addLoopNoIVV(loop);
				lla.addLoopInformation(loop);
			} else {
//				polarityResultsWithIVV.createArrowsFromLoop(loop);
				addLoopWithIVV(loop);
			}
		}
		if (!withIVV) {
			lla.aggregate(longestLoopLengthNoIVV != Integer.MIN_VALUE ? longestLoopLengthNoIVV : 2);
			lla.write(Utilities.openFileForWriting(InformationManagers.getInstance().
					getFileOrganizer().getStructureDirectory()+"/LoopLengthAnalysis.tab"));
		}

	}



	private boolean addLoopNoIVV(Loop loop) {

		// if (loop.length() > maximumLoopLength) {
		// return false;
		// }

		String loopName = loop.getOrderedNameWithNumbers();

		shortestLoopLengthNoIVV = Math.min(shortestLoopLengthNoIVV,
				loop.getSize());
		longestLoopLengthNoIVV = Math.max(longestLoopLengthNoIVV,
				loop.getSize());

		if (knownLoopNamesNoIVV.contains(loopName)) {
			return false;
		} else {
			allLoopsNoIVV.add(loop);
			loop.determineLoopPolarity(polarityResultsNoIVV);
			knownLoopNamesNoIVV.add(loopName);
			for (String s : loop.getLoopWithVariables()) {
				if (!loopMembershipNoIVV.containsKey(s))
					loopMembershipNoIVV.put(s, new LinkedHashSet<Loop>());
				loopMembershipNoIVV.get(s).add(loop);
			}
			return true;
		}
	}

	private boolean addLoopWithIVV(Loop loop) {
		// make sure that we at least initialize arrows with no polarity
		// polarityResults.createArrowsFromLoop(loop);

		// private int shortestLoopLengthWithIVV = Integer.MAX_VALUE;
		// private int longestLoopLengthWithIVV = Integer.MIN_VALUE;

		shortestLoopLengthWithIVV = Math.min(shortestLoopLengthWithIVV,
				loop.getSize());
		longestLoopLengthWithIVV = Math.max(longestLoopLengthWithIVV,
				loop.getSize());

		String loopName = loop.getOrderedNameWithNumbers();
		// loop.print();
		if (knownLoopNamesWithIVV.contains(loopName)) {
			return false;
		} else {
			allLoopsWithIVV.add(loop);
			loop.determineLoopPolarity(/*polarityResultsWithIVV*/
					polarityResultsNoIVV);
			knownLoopNamesWithIVV.add(loopName);
			for (String s : loop.getLoopWithVariables()) {
				if (!loopMembershipWithIVV.containsKey(s))
					loopMembershipWithIVV.put(s, new LinkedHashSet<Loop>());
				loopMembershipWithIVV.get(s).add(loop);
			}
			return true;
		}
	}
	
	public VariableLoopPolarityCounts getVariableLoopPolarityCounts(Map<String, VariableLoopPolarityCounts> map, String key) {
		if (map.containsKey(key)) {
			VariableLoopPolarityCounts vlpc = map.get(key);
			return vlpc;
		} else if (map.containsKey(Parser.quoted(key))) {
			VariableLoopPolarityCounts vlpc = map.get(Parser.quoted(key));
			return vlpc;
		} else {
			return null;
		}
	}

	public long getMaximumLengthLoopSize(String s, String polarity) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return 0;
			else
				return vlpc.getMaxForPolarity(polarity);
		} else {

			Set<Loop> loopSet = getLoopsContaining(s);
			Set<Loop> loopSetPolarity;
			if (polarity.equals("+")) {
				loopSetPolarity = getPositiveLoops(loopSet);
			} else if (polarity.equals("-")) {
				loopSetPolarity = getNegativeLoops(loopSet);
			} else {
				loopSetPolarity = getProblemLoops(loopSet);
			}
			return getMaximumLengthLoopSize(loopSetPolarity);
		}
	}

	public long getMaximumLengthLoopSizeIVV(String s, String polarity) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null)
				return 0;
			else
				return vlpc.getMaxForPolarity(polarity);
		} else {

			Set<Loop> loopSet = getLoopsContainingIVV(s);
			Set<Loop> loopSetPolarity;
			if (polarity.equals("+")) {
				loopSetPolarity = getPositiveLoops(loopSet);
			} else if (polarity.equals("-")) {
				loopSetPolarity = getNegativeLoops(loopSet);
			} else {
				loopSetPolarity = getProblemLoops(loopSet);
			}
			return getMaximumLengthLoopSize(loopSetPolarity);
		}
	}

	public long getMinimumLengthLoopSize(String s, String polarity) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return 0;
			else
				return vlpc.getMinForPolarity(polarity);
		} else {

			Set<Loop> loopSet = getLoopsContaining(s);
			Set<Loop> loopSetPolarity;
			if (polarity.equals("+")) {
				loopSetPolarity = getPositiveLoops(loopSet);
			} else if (polarity.equals("-")) {
				loopSetPolarity = getNegativeLoops(loopSet);
			} else {
				loopSetPolarity = getProblemLoops(loopSet);
			}
			return getMinimumLengthLoopSize(loopSetPolarity);
		}
	}

	public long getMinimumLengthLoopSizeIVV(String s, String polarity) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null) {
				return 0;
			} else {
				return vlpc.getMinForPolarity(polarity);
			} 
		} else {


			Set<Loop> loopSet = getLoopsContainingIVV(s);
			Set<Loop> loopSetPolarity;
			if (polarity.equals("+")) {
				loopSetPolarity = getPositiveLoops(loopSet);
			} else if (polarity.equals("-")) {
				loopSetPolarity = getNegativeLoops(loopSet);
			} else {
				loopSetPolarity = getProblemLoops(loopSet);
			}
			return getMinimumLengthLoopSize(loopSetPolarity);
		}
	}

	public long getNumberOfLoopsNoIVV() {
		if (usingMinimumMemory) {
			
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(false);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return (long) vlpcAll.getTotalCount();
			} else {
				return 0;
			}
		} else {
			if (allLoopsNoIVV == null) {
				return allLoopsNoIVVSize;
			}
			return (long) allLoopsNoIVV.size();
		}
	}

	public long getNumberOfLoopsWithIVV() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(true);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getTotalCount();
			} else {
				return 0;
			}
		} else {
			if (allLoopsWithIVV == null)
				return allLoopsWithIVVSize;
			return allLoopsWithIVV.size();
		}
	}

	public int getMinimumLengthLoopSize(Set<Loop> loopSet) {
		int minLength = Integer.MAX_VALUE;
		for (Loop loop : loopSet) {
			if (loop.getSize() < minLength)
				minLength = loop.getSize();
		}

		return loopSet.size() == 0 ? 0 : minLength;
	}

	public int getMaximumLengthLoopSize(Set<Loop> loopSet) {
		int maxLength = Integer.MIN_VALUE;
		for (Loop loop : loopSet) {
			if (loop.getSize() > maxLength)
				maxLength = loop.getSize();
		}

		return loopSet.size() == 0 ? 0 : maxLength;
	}

	public Set<Loop> getPositiveLoops(Set<Loop> loopSet) {
		Set<Loop> returnLoops = new LinkedHashSet<Loop>();
		for (Loop loop : loopSet) {
			if (loop.isPolarityPositiveLoop()) {
				returnLoops.add(loop);
			}
		}

		return returnLoops;
	}

	public Set<Loop> getNegativeLoops(Set<Loop> loopSet) {
		Set<Loop> returnLoops = new LinkedHashSet<Loop>();
		for (Loop loop : loopSet) {
			if (loop.isPolarityNegativeLoop()) {
				returnLoops.add(loop);
			}
		}

		return returnLoops;
	}

	public Set<Loop> getProblemLoops(Set<Loop> loopSet) {
		Set<Loop> returnLoops = new LinkedHashSet<Loop>();
		for (Loop loop : loopSet) {
			if (loop.isPolarityProblemLoop()) {
				returnLoops.add(loop);
			}
		}

		return returnLoops;
	}

	public int getNumberOfPositivePolarityLoops() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(false);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getPositiveCount();
			} else {
				return 0;
			}
		} else {
			int count = 0;
			if (isUseDB()) {
				count = InformationManagers.getInstance().getDatabaseManager()
						.getDb().getNumLoopsWithPolarity(ArrowPolarity.POSITIVE);
			} else {
				for (Loop loop : allLoopsNoIVV) {
					if (loop.isPolarityPositiveLoop())
						count++;
				}
			}
			return count;
		}
	}
	
	public int getNumberOfPositivePolarityLoopsWithIVV() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(true);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getPositiveCount();
			} else {
				return 0;
			}
		} else {
			int count = 0;
			if (isUseDB()) {
				count = InformationManagers.getInstance().getDatabaseManager()
						.getDb().getNumLoopsWithPolarity(ArrowPolarity.POSITIVE);
			} else {
				for (Loop loop : allLoopsWithIVV) {
					if (loop.isPolarityPositiveLoop())
						count++;
				}
			}
			return count;
		}
	}

	public int getNumberOfNegativePolarityLoops() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(false);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getNegativeCount();
			} else {
				return 0;
			}
		} else {
			int count = 0;
			if (isUseDB()) {
				InformationManagers.getInstance().getDatabaseManager().getDb()
				.getNumLoopsWithPolarity(ArrowPolarity.NEGATIVE);
			} else {
				for (Loop loop : allLoopsNoIVV) {
					if (loop.isPolarityNegativeLoop())
						count++;
				}
			}

			return count;
		}
	}
	
	public int getNumberOfNegativePolarityLoopsWithIVV() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(true);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getNegativeCount();
			} else {
				return 0;
			}
		} else {
			int count = 0;
			if (isUseDB()) {
				InformationManagers.getInstance().getDatabaseManager().getDb()
				.getNumLoopsWithPolarity(ArrowPolarity.NEGATIVE);
			} else {
				for (Loop loop : allLoopsWithIVV) {
					if (loop.isPolarityNegativeLoop())
						count++;
				}
			}

			return count;
		}
	}

	public int getNumberOfProblemPolarityLoops() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(false);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getUnknownCount();
			} else {
				return 0;
			}
		} else {

			int count = 0;
			if (isUseDB()) {
				InformationManagers.getInstance().getDatabaseManager().getDb()
				.getNumLoopsWithPolarity(ArrowPolarity.UNKNOWN);
			} else {
				for (Loop loop : allLoopsNoIVV) {
					if (loop.isPolarityProblemLoop())
						count++;
				}
			}

			return count;
		}
	}
	
	public int getNumberOfProblemPolarityLoopsWithIVV() {
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(true);
			if (map == null)
				return 0;
			if (map.containsKey("__ALL__")) {
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				return vlpcAll.getUnknownCount();
			} else {
				return 0;
			}
		} else {

			int count = 0;
			if (isUseDB()) {
				InformationManagers.getInstance().getDatabaseManager().getDb()
				.getNumLoopsWithPolarity(ArrowPolarity.UNKNOWN);
			} else {
				for (Loop loop : allLoopsWithIVV) {
					if (loop.isPolarityProblemLoop())
						count++;
				}
			}

			return count;
		}
	}

	public int getNumberOfPositivePolarityLoops(Set<Loop> loopSet) {
		int count = 0;
		for (Loop loop : loopSet) {
			if (loop.isPolarityPositiveLoop())
				count++;
		}
		return count;
	}

	public int getNumberOfNegativePolarityLoops(Set<Loop> loopSet) {

		int count = 0;
		for (Loop loop : loopSet) {
			if (loop.isPolarityNegativeLoop())
				count++;
		}

		return count;
	}

	public int getNumberOfProblemPolarityLoops(Set<Loop> loopSet) {

		int count = 0;
		for (Loop loop : loopSet) {
			if (loop.isPolarityProblemLoop())
				count++;
		}

		return count;
	}

	public int getEffectiveNumberOfPositivePolarityLoops(Set<Loop> loopSet) {
		int count = 0;
		for (Loop loop : loopSet) {
			if (loop.isPolarityPositiveLoop())
				count += loop.getEffectiveLoopCount();
		}

		return count;
	}

	public int getEffectiveNumberOfNegativePolarityLoops(Set<Loop> loopSet) {
		int count = 0;
		for (Loop loop : loopSet) {
			if (loop.isPolarityNegativeLoop())
				count += loop.getEffectiveLoopCount();
		}

		return count;
	}

	public int getEffectiveNumberOfProblemPolarityLoops(Set<Loop> loopSet) {
		// log.debug("Problem");
		int count = 0;
		for (Loop loop : loopSet) {
			// loop.printWithPolarity();
			if (loop.isPolarityProblemLoop())
				count += loop.getEffectiveLoopCount();
		}

		return count;
	}

	public List<Loop> getLoopsOrderedBySizeContaining(Pair p) {
		Set<Loop> setOfLoops = new LinkedHashSet<Loop>();
		setOfLoops.addAll(intersect(getLoopsContaining(p.getV1()),
				getLoopsContaining(p.getV2())));
		return orderSetBySize(setOfLoops);
	}

	public List<Loop> getLoopsOrderedBySizeContaining(Set<String> setOfVariables) {
		Set<Loop> setOfLoops = new LinkedHashSet<Loop>();
		for (String var : setOfVariables) {
			setOfLoops.addAll(getLoopsContaining(var));
		}

		return orderSetBySize(setOfLoops);
	}
	
	public Set<Loop> getLoopsContainingWithName(String st) {

		String s = Parser.removeQuotes(st); // st.replace("\"", "");
		if (loopMembershipNoIVV == null) {
			return InformationManagers.getInstance().getDatabaseManager()
					.getDb().getNoIVVLoopsContaining(s, Utilities.nameToID(s));
		} else {

			if (loopMembershipNoIVV.containsKey(loopMembershipKey(s))) {
				return loopMembershipNoIVV.get(loopMembershipKey(s));
			} else {
				return new LinkedHashSet<Loop>();
			}
		}
	}

	public Set<Loop> getLoopsContaining(String st) {

		String s = Parser.removeQuotes(st); // st.replace("\"", "");
		if (loopMembershipNoIVV == null) {
			return InformationManagers.getInstance().getDatabaseManager()
					.getDb().getNoIVVLoopsContaining(s, Utilities.nameToID(s));
		} else {
			String lmk = loopMembershipKey(s);
			if (loopMembershipNoIVV.containsKey(lmk)) {
				return loopMembershipNoIVV.get(lmk);
			} else {
				return new LinkedHashSet<Loop>();
			}
		}
	}

	public Set<Loop> getLoopsContainingIVV(String s) {
		if (loopMembershipWithIVV == null) {
			return InformationManagers.getInstance().getDatabaseManager()
					.getDb().getWithIVVLoopsContaining(s, Utilities.nameToID(s));
		} else {
			if (loopMembershipWithIVV.containsKey(loopMembershipKey(s)))
				return loopMembershipWithIVV.get(loopMembershipKey(s));
			else
				return new LinkedHashSet<Loop>();
		}
	}
	
//	public String nameForID(String s) {
//		if (Parser.isNumber(s)) {
////			return s;
//			return Utilities.idToName(s);
//		} else {
//			return s;
////			return Utilities.nameToID(s);
//		}
//	}

	public String loopMembershipKey(String s) {
	
		if (Parser.isNumber(s)) {
			return s;
//			return idToName(s);
		} else {
//			return s;
			return Utilities.nameToID(s);
		}
	}

	public int getNumberOfLoopsWith(String s) {

		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return 0;
			else
				return vlpc.getTotalCount();
		} else {
			if (loopMembershipNoIVV == null) {
				return InformationManagers.getInstance().getDatabaseManager()
						.getDb().getNoIVVLoopsContaining(s, Utilities.nameToID(s)).size();
			} else {
				// this input can be either an id or a varaible name
				// determine which and act accordingly
				if (loopMembershipNoIVV.containsKey(loopMembershipKey(s)))
					return loopMembershipNoIVV.get(loopMembershipKey(s)).size();
				else
					return 0;
			}
		}
	}

	public int getNumberOfLoopsWithIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null)
				return 0;
			else
				return vlpc.getTotalCount();
		} else {
	}
		if (loopMembershipWithIVV == null) {
			return InformationManagers.getInstance().getDatabaseManager()
					.getDb().getWithIVVLoopsContaining(s, Utilities.nameToID(s)).size();
		} else {
			// this input can be either an id or a varaible name
			// determine which and act accordingly

			if (loopMembershipWithIVV
					.containsKey(loopMembershipKey(loopMembershipKey(s))))
				return loopMembershipWithIVV.get(
						loopMembershipKey(loopMembershipKey(s))).size();
			else
				return 0;
		}

	}

	public String getPercentagePositiveCounts(String s) {
		String numPos;
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null) {
				return NumberFormatter
						.formatForCountNotConditional(0);
			}
			else {
				return NumberFormatter
						.formatForCountNotConditional(vlpc.getPositiveCount());
			} 
		} else {
			numPos = NumberFormatter
					.formatForCountNotConditional(getNumberOfPositivePolarityLoops(getLoopsContaining(s)));
			return numPos;

		}
	}

	public String getPercentagePositiveCountsIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null)
				return NumberFormatter
						.formatForCountNotConditional(0);
			else {
				return NumberFormatter
						.formatForCountNotConditional(vlpc.getPositiveCount());
			} 
		} else {
			String numPos = NumberFormatter
					.formatForCountNotConditional(getNumberOfPositivePolarityLoops(getLoopsContainingIVV(s)));
			return numPos;
		}

	}

	public String getPercentageCounts(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return NumberFormatter
						.formatForCountNotConditional(0);
			else {
				
				VariableLoopPolarityCounts vlpcAll = variableLoopPolarityCounts.get(false).get("__ALL__");
				return NumberFormatter
						.formatForPercent((double) vlpc.getTotalCount() / (double) vlpcAll.getTotalCount() * 100.0) ;
			} 
		} else {
			String pct = NumberFormatter
					.formatForPercent(getPercentageOfLoopsWith(s));
			return pct;
		}

	}

	public String getPercentageCountsIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null)
				return NumberFormatter
						.formatForCountNotConditional(0);
			else {
				VariableLoopPolarityCounts vlpcAll = variableLoopPolarityCounts.get(true).get("__ALL__");
				return NumberFormatter
						.formatForPercent((double) vlpc.getTotalCount() / (double) vlpcAll.getTotalCount() * 100.0) ;
			} 
		} else {
			String pct = NumberFormatter
					.formatForPercent(getPercentageOfLoopsWithIVV(s));
			return pct;
		}

	}

	public String getPercentageProblemCounts(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null) {
				return NumberFormatter
						.formatForCountNotConditional(0);
			} else {
				return NumberFormatter
						.formatForCountNotConditional( vlpc.getUnknownCount()) ;
			} 
		} else {
		String numNeg = NumberFormatter
				.formatForCountNotConditional(getNumberOfProblemPolarityLoops(getLoopsContaining(s)));
		return numNeg;
		}
	}

	public String getPercentageProblemCountsIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null) {
				return NumberFormatter
						.formatForCountNotConditional(0);
			} else {
				
				VariableLoopPolarityCounts vlpcAll = variableLoopPolarityCounts.get(true).get("__ALL__");
				if (vlpcAll.getUnknownCount() != 0) {
				return NumberFormatter
						.formatForPercent((double) vlpc.getUnknownCount() / (double) vlpcAll.getUnknownCount() * 100.0) ;
				} else {
					return NumberFormatter
							.formatForCountNotConditional( vlpc.getUnknownCount()) ;
				}
			} 
		} else {
			String numNeg = NumberFormatter
					.formatForCountNotConditional(getNumberOfProblemPolarityLoops(getLoopsContainingIVV(s)));
			return numNeg;
		}
	}

	public String getPercentageNegativeCounts(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return NumberFormatter
						.formatForCountNotConditional(0);
			else {
				return NumberFormatter
						.formatForCountNotConditional(vlpc.getNegativeCount()) ;
			} 
		} else {
		String numNeg = NumberFormatter
				.formatForCountNotConditional(getNumberOfNegativePolarityLoops(getLoopsContaining(s)));
		return numNeg;
		}
	}

	public String getPercentageNegativeCountsIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null)
				return NumberFormatter
						.formatForCountNotConditional(0);
			else {
				return NumberFormatter
						.formatForCountNotConditional(vlpc.getNegativeCount()) ;
			}
		} else {
			String numNeg = NumberFormatter
					.formatForCountNotConditional(getNumberOfNegativePolarityLoops(getLoopsContainingIVV(s)));
			return numNeg;
		}
	}

	public String getPositiveNegativeRatio(String s) {
		
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return StringEscapeUtils.escapeHtml4("NA");
			else {
				int neg = vlpc.getNegativeCount();
				int pos = vlpc.getPositiveCount();
				if (neg == 0) {
					if (pos == 0)
						return StringEscapeUtils.escapeHtml4("NA");
					else
						return SDMDocLanguageSpecific.getInstance().getTextHtml(
								StringConstants.INFINITE);

				}
				double ratio = (double)pos
						/ (double) neg;
				String rat = NumberFormatter.formatForRatio(ratio);
				return rat;
			} 
		} else {
			if (getNumberOfNegativePolarityLoops(getLoopsContaining(s)) == 0) {
				if (getNumberOfPositivePolarityLoops(getLoopsContaining(s)) == 0)
					return StringEscapeUtils.escapeHtml4("NA");
				else
					return SDMDocLanguageSpecific.getInstance().getTextHtml(
							StringConstants.INFINITE);

			}
			double ratio = (double) getNumberOfPositivePolarityLoops(getLoopsContaining(s))
					/ (double) getNumberOfNegativePolarityLoops(getLoopsContaining(s));
			String rat = NumberFormatter.formatForRatio(ratio);
			return rat;
		}
	}
	
	public String getPositiveNegativeRatioWithIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(true), s);
			if (vlpc == null) {
				return StringEscapeUtils.escapeHtml4("NA");
			} else {
				
			
				int neg = vlpc.getNegativeCount();
				int pos = vlpc.getPositiveCount();
				if (neg == 0) {
					if (pos == 0)
						return StringEscapeUtils.escapeHtml4("NA");
					else
						return SDMDocLanguageSpecific.getInstance().getTextHtml(
								StringConstants.INFINITE);

				}
				double ratio = (double)pos
						/ (double) neg;
				String rat = NumberFormatter.formatForRatio(ratio);
				return rat;
			} 
		} else {
			if (getNumberOfNegativePolarityLoops(getLoopsContainingIVV(s)) == 0) {
				if (getNumberOfPositivePolarityLoops(getLoopsContainingIVV(s)) == 0)
					return StringEscapeUtils.escapeHtml4("NA");
				else
					return SDMDocLanguageSpecific.getInstance().getTextHtml(
							StringConstants.INFINITE);

			}
			double ratio = (double) getNumberOfPositivePolarityLoops(getLoopsContainingIVV(s))
					/ (double) getNumberOfNegativePolarityLoops(getLoopsContainingIVV(s));
			String rat = NumberFormatter.formatForRatio(ratio);
			return rat;
		}
	}
	public String getPercentagePositiveNegativeCounts(String s) {
		return getPercentagePositiveNegativeCounts(s, true);
	}

	public String getPercentagePositiveNegativeCounts(String s, boolean addHTML) {
		String pct;
		String numPos;
		String numNeg;
		String numProb;
		long minPos;
		long minNeg;
		long minPrb;
		long maxPos;
		long maxNeg;
		long maxPrb;

		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null) {
				pct = "0.0";
				numPos = "0";
				numNeg = "0";
				numProb = "0";

				minPos = 0;
				minNeg = 0;
				minPrb = 0;
				maxPos = 0;
				maxNeg = 0;
				maxPrb =0;

			} else {


				pct = NumberFormatter
						.formatForPercent(getPercentageOfLoopsWith(s));

				numPos = NumberFormatter
						.formatForCountNotConditional(vlpc.getPositiveCount());
				numNeg = NumberFormatter
						.formatForCountNotConditional(vlpc.getNegativeCount());
				numProb = NumberFormatter
						.formatForCountNotConditional(vlpc.getUnknownCount());


				minPos = vlpc.getMinForPolarity("+");
				minNeg = vlpc.getMinForPolarity("-");
				minPrb = vlpc.getMinForPolarity("?");
				maxPos = vlpc.getMaxForPolarity("+");
				maxNeg = vlpc.getMaxForPolarity("-");
				maxPrb = vlpc.getMaxForPolarity("?");

			} 

			if (addHTML) {

				String posRange = "&nbsp;[" + minPos + "," + maxPos + "]";
				String negRange = "&nbsp;[" + minNeg + "," + maxNeg + "]";
				String prbRange = "&nbsp;[" + minPrb + "," + maxPrb + "]";

				return "&nbsp;("
				+ pct
				+ "%)"
				+ HTMLConstants.BLUE_START
				+ "&nbsp;(+)&nbsp;"
				+ numPos
				+ "&nbsp;"
				+ posRange
				+ HTMLConstants.COLOR_END
				+ HTMLConstants.RED_START
				+ "&nbsp;(-)&nbsp;"
				+ numNeg
				+ "&nbsp;"
				+ negRange
				+ HTMLConstants.COLOR_END
				+ (numProb.equals("0") ? "" : HTMLConstants.BOLD_START
						+ "&nbsp;(?)&nbsp;" + numProb + "&nbsp;" + prbRange
						+ HTMLConstants.BOLD_END);
			} else {
				String posRange = " [" + minPos + "," + maxPos + "]";
				String negRange = " [" + minNeg + "," + maxNeg + "]";
				String prbRange = " [" + minPrb + "," + maxPrb + "]";

				return  " "
				+ pct
				+ " (+) "
				+ numPos
				+ " "
				+ posRange
				+ " (-) "
				+ numNeg
				+ " "
				+ negRange
				+ (numProb.equals("0") ? "" : HTMLConstants.BOLD_START
						+ " (?) " + numProb + " " + prbRange);
			}
		} else {
			pct = NumberFormatter
					.formatForPercent(getPercentageOfLoopsWith(s));

			numPos = NumberFormatter
					.formatForCountNotConditional(getNumberOfPositivePolarityLoops(getLoopsContaining(s)));
			numNeg = NumberFormatter
					.formatForCountNotConditional(getNumberOfNegativePolarityLoops(getLoopsContaining(s)));
			numProb = NumberFormatter
					.formatForCountNotConditional(getNumberOfProblemPolarityLoops(getLoopsContaining(s)));

			minPos = getMinimumLengthLoopSize(s, "+");
			minNeg = getMinimumLengthLoopSize(s, "-");
			minPrb = getMinimumLengthLoopSize(s, "?");
			maxPos = getMaximumLengthLoopSize(s, "+");
			maxNeg = getMaximumLengthLoopSize(s, "-");
			maxPrb = getMaximumLengthLoopSize(s, "?");

			if (addHTML) {
				String posRange = "&nbsp;[" + minPos + "," + maxPos + "]";
				String negRange = "&nbsp;[" + minNeg + "," + maxNeg + "]";
				String prbRange = "&nbsp;[" + minPrb + "," + maxPrb + "]";

				return "&nbsp;("
				+ pct
				+ "%)"
				+ HTMLConstants.BLUE_START
				+ "&nbsp;(+)&nbsp;"
				+ numPos
				+ "&nbsp;"
				+ posRange
				+ HTMLConstants.COLOR_END
				+ HTMLConstants.RED_START
				+ "&nbsp;(-)&nbsp;"
				+ numNeg
				+ "&nbsp;"
				+ negRange
				+ HTMLConstants.COLOR_END
				+ (numProb.equals("0") ? "" : HTMLConstants.BOLD_START
						+ "&nbsp;(?)&nbsp;" + numProb + "&nbsp;" + prbRange
						+ HTMLConstants.BOLD_END);
			}	else {
				String posRange = " [" + minPos + "," + maxPos + "]";
				String negRange = " [" + minNeg + "," + maxNeg + "]";
				String prbRange = " [" + minPrb + "," + maxPrb + "]";

				return " "
				+ pct
				+ " (+) "
				+ numPos
				+ " "
				+ posRange
				+ " (-) "
				+ numNeg
				+ " "
				+ negRange
				+ (numProb.equals("0") ? "" : 
						" (?) " + numProb + " " + prbRange);
			}
		}
	}

	public double getPercentageOfLoopsWith(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return 0;
			else {
	
				VariableLoopPolarityCounts vlpcAll = variableLoopPolarityCounts.get(false).get("__ALL__");
				return (double) vlpc.getTotalCount() / (double) vlpcAll.getTotalCount() * 100.0;
			} 
		} else {
		if (isUseDB()) {
			if (allLoopsNoIVVSize == 0)
				return 0.0;
			else
				return 100.0
						* ((double) InformationManagers.getInstance()
								.getDatabaseManager().getDb()
								.getNoIVVLoopsContaining(s, Utilities.nameToID(s)).size())
						/ ((double) allLoopsNoIVVSize);
		} else {
			if (allLoopsNoIVV.size() == 0)
				return 0.0;
			return 100.0 * ((double) getNumberOfLoopsWith(s))
					/ ((double) allLoopsNoIVV.size());
		}
		}
	}

	public double getPercentageOfLoopsWithIVV(String s) {
		if (usingMinimumMemory) {
			VariableLoopPolarityCounts vlpc =  getVariableLoopPolarityCounts(variableLoopPolarityCounts.get(false), s);
			if (vlpc == null)
				return 0;
			else {
				VariableLoopPolarityCounts vlpcAll = variableLoopPolarityCounts.get(false).get("__ALL__");
				return (double) vlpc.getTotalCount() / (double) vlpcAll.getTotalCount() * 100.0;
			} 
		} else {
			if (isUseDB()) {
				if (allLoopsWithIVVSize == 0)
					return 0.0;
				else
					return 100.0
							* ((double) InformationManagers.getInstance()
									.getDatabaseManager().getDb()
									.getWithIVVLoopsContaining(s, Utilities.nameToID(s))
									.size()) / ((double) allLoopsWithIVVSize);
			} else {
				if (allLoopsWithIVV.size() == 0)
					return 0.0;
				return 100.0 * ((double) getNumberOfLoopsWithIVV(s))
						/ ((double) allLoopsWithIVV.size());
			}
		}
	}

	// public void printLoops() {
	// if (allLoopsNoIVV == null)
	// panic();
	// log.debug("### Recognized Loops: "+allLoopsNoIVV.size());
	// log.debug("   Number Positive Polarity: "+getNumberOfPositivePolarityLoops());
	// log.debug("   Number Negative Polarity: "+getNumberOfNegativePolarityLoops());
	// log.debug("   Number Problem Polarity:  "+getNumberOfProblemPolarityLoops());
	// for (Loop loop : getLoopsOrderedBySize()) {
	// // loop.printWithPolarity();
	// log.debug(loop.getName());
	// }
	// }

	public List<Loop> getLoopsOrderedBySize() {
		return orderSetBySize(new LinkedHashSet<Loop>(allLoopsNoIVV));
	}

	public List<Loop> getLoopsOrderedBySizeIVV() {
		return orderSetBySize(new LinkedHashSet<Loop>(allLoopsWithIVV));
	}

	public List<Loop> getLoopsLimitedTo(Set<String> set) {
		Set<Loop> loops = new LinkedHashSet<Loop>();
		// if a loop contains a single variable outside the set, discard it
		if (isUseDB()) {
			for (String s : set) {
				loops.addAll(InformationManagers.getInstance()
						.getDatabaseManager().getDb()
						.getWithIVVLoopsContaining(s, Utilities.nameToID(s)));
			}
		} else {
			for (Loop loop : allLoopsNoIVV) {
				boolean keep = true;
				for (String var : loop.getLoop()) {
					String name = Utilities.idToName(var);
					if (!set.contains(Utilities.idToName(var))) {
						keep = false;
						break;
					}
				}
				if (keep)
					loops.add(loop);
			}

		}
		return orderSetBySize(loops);
	}

	public List<Loop> getLoopsOrderedBySize(String variable) {
		if (isUseDB()) {
			return orderSetBySize(InformationManagers.getInstance()
					.getDatabaseManager().getDb()
					.getWithIVVLoopsContaining(variable, Utilities.nameToID(variable)));
		}
		return orderSetBySize(loopMembershipNoIVV
				.get(loopMembershipKey(variable)));
	}

	public List<Loop> getLoopsWithIVVOrderedBySize(String variable) {
		if (loopMembershipWithIVV == null)
			return new ArrayList<Loop>();
		return orderSetBySize(loopMembershipWithIVV
				.get(loopMembershipKey(variable)));
	}

	public static List<Loop> orderByLoopNumber(Set<Loop> set) {
		return orderByLoopNumber(new ArrayList<Loop>(set));
	}
	
	public static List<Loop> orderByLoopNumberReverse(List<Loop> list) {
		List<Loop> al = new ArrayList<Loop>(list);
		// for (Loop l : al) {
		// System.out.println(l);
		// }
		try {
			Collections.sort(al, new Comparator<Loop>() {
				@Override
				public int compare(Loop arg0, Loop arg1) {
					try {
						if (arg0.getLoopNumber() == arg1.getLoopNumber()) {
							return arg0.getExtendedLoopNumber() < arg1
									.getExtendedLoopNumber() ? 1 : -1;
						} else {

						}
						return arg0.getLoopNumber() < arg1.getLoopNumber() ? 1
								: -1;
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return al;
	}

	public static List<Loop> orderByLoopNumber(List<Loop> list) {
		List<Loop> al = new ArrayList<Loop>(list);
		// for (Loop l : al) {
		// System.out.println(l);
		// }
		try {
			Collections.sort(al, new Comparator<Loop>() {
				@Override
				public int compare(Loop arg0, Loop arg1) {
					try {
						if (arg0.getLoopNumber() == arg1.getLoopNumber()) {
							return arg0.getExtendedLoopNumber() < arg1
									.getExtendedLoopNumber() ? -1 : 1;
						} else {

						}
						return arg0.getLoopNumber() < arg1.getLoopNumber() ? -1
								: 1;
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return al;
	}

	public List<Loop> orderSetBySizeAndVariables(Set<Loop> set) {
		return orderSetBySizeAndVariables(new ArrayList<Loop>(set));
	}

	public List<Loop> orderSetBySizeAndVariables(List<Loop> list) {
		List<Loop> al = new ArrayList<Loop>(list);
		Collections.sort(al, new Comparator<Loop>() {
			@Override
			public int compare(Loop loop1, Loop loop2) {
				if (loop1.getSize() != loop2.getSize()) {
					return loop1.getSize() < loop2.getSize() ? -1 : 1;
				}
				for (int i = 0; i < loop1.getSize(); i++) {
					if (!Utilities.idToName(loop1.getLoop().get(i)).equals(
							Utilities.idToName(loop2.getLoop().get(i)))) {
						return Utilities.idToName(loop1.getLoop().get(i)).compareTo(
								Utilities.idToName(loop2.getLoop().get(i)));
					}
				}
				return 0;
			}
		});
		return al;
	}

	public List<Loop> orderSetBySize(Set<Loop> set) {
		List<Loop> loopListBySize = new ArrayList<Loop>();
		if (set == null)
			return loopListBySize;
		Map<Integer, Set<Loop>> bySize = new LinkedHashMap<Integer, Set<Loop>>();
		for (Loop loop : set) {
			Integer size = loop.getLoop().size();
			if (!bySize.containsKey(size))
				bySize.put(size, new LinkedHashSet<Loop>());
			Set<Loop> aSet = bySize.get(size);
			aSet.add(loop);
		}
		List<Integer> orderedListOfSizes = new ArrayList<Integer>(
				bySize.keySet());
		Collections.sort(orderedListOfSizes);

		for (Integer size : orderedListOfSizes) {
			Set<Loop> aSet = bySize.get(size);
			loopListBySize.addAll(aSet);
		}
		return loopListBySize;
	}

	public void printLoopMembership() {
		log.debug("Loop Membership:");
		List<String> al = new ArrayList<String>(
				vensimEquationManager.getAllKnowVariables());
		Collections.sort(al);
		for (String s : al) {
			log.debug(">>> " + s + ": " + getNumberOfLoopsWith(s) + "\n");
			// log.debug("   Number Positive Polarity: "+getNumberOfPositivePolarityLoops(getLoopsContaining(s)));
			// log.debug("   Number Negative Polarity: "+getNumberOfNegativePolarityLoops(getLoopsContaining(s)));
			// log.debug("   Number Problem Polarity:  "+getNumberOfProblemPolarityLoops(getLoopsContaining(s)));
			// int loopNum = 1;
			// for (Loop loop : getLoopsOrderedBySize(s)) {
			// loop.getLoopStartingAt(s).print(loopNum++);
			//
			// }
		}
	}

	// public void printLoopMembership(String variable) {
	// if (loopMembershipNoIVV == null)
	// panic();
	// log.debug("Loop Membership for "+variable);
	// if (loopMembershipNoIVV.containsKey(loopMembershipKey(variable))) {
	// int loopNum = 1;
	// for (Loop loop :
	// orderSetBySize(loopMembershipNoIVV.get(loopMembershipKey(variable)))) {
	// loop.getLoopStartingAt(variable).print(loopNum++);
	// }
	// }
	// }

	public void printRotatingLoops(Loop loop) {
		int loopNum = 1;
		for (String s : loop.getLoop()) {
			log.debug("\n\nRotate on " + s);
			loop.getLoopStartingAt(s).print(loopNum++);
		}
	}

//	private String Utilities.nameToID(String name) {
//		// return name;
//		return InformationManagers.getInstance().getEquationManager().getVariableIDForName(name);
//	}

//	private String idToName(String id) {
//		// return id;
//		return InformationManagers.getInstance().getEquationManager().getVariableNameForID(id);
//	}

	public ColumnSizing determineSizing() {
		long loopCountMax = Integer.MIN_VALUE;
		long posCountMax = Integer.MIN_VALUE;
		long negCountMax = Integer.MIN_VALUE;
		long prbCountMax = Integer.MIN_VALUE;
		long posRangeMax;
		
		if (usingMinimumMemory) {
			Map<String, VariableLoopPolarityCounts> map = variableLoopPolarityCounts.get(false);
			Map<String, VariableLoopPolarityCounts> mapWithIVV = variableLoopPolarityCounts.get(true);
			
				VariableLoopPolarityCounts vlpcAll = map.get("__ALL__");
				VariableLoopPolarityCounts vlpcAllWithIVV = mapWithIVV.get("__ALL__");
				posRangeMax = Math.max(vlpcAll.getMaxPositive(), vlpcAllWithIVV.getMaxPositive());
			
		} else 	if (isUseIncore()) {
			List<Loop> loops = getLoopsOrderedBySize();
			List<Loop> loopsIVV = getLoopsOrderedBySizeIVV();

			if (loops.size() == 0 || loopsIVV.size() == 0) {
				if (loopsIVV.size() == 0) {
					posRangeMax = 2;
				} else {
					posRangeMax = loopsIVV.get(loops.size() - 1).getSize();
				}
			} else {
				posRangeMax = Math.max(loops.get(loops.size() - 1).getSize(),
						loopsIVV.get(loopsIVV.size() - 1).getSize());
			}
		} else {
			posRangeMax = Math.max(InformationManagers.getInstance()
					.getDatabaseManager().getDb().getMaxSizeNoIVVLoop(),
					InformationManagers.getInstance().getDatabaseManager()
							.getDb().getMaxSizeWithIVVLoop());
			posRangeMax = Math.max(2, posRangeMax);
		}

		long posRangeMin = posRangeMax;
		long negRangeMin = posRangeMin;
		long negRangeMax = posRangeMax;

		long prbRangeMin = posRangeMin;
		long prbRangeMax = posRangeMax;

		if (isUseDB()) {
			loopCountMax = posRangeMax;
			posCountMax = posRangeMax;
			negCountMax = posRangeMax;
			prbCountMax = posRangeMax;
		} else {
			for (String key : loopMembershipNoIVV.keySet()) {
				Set<Loop> set = getLoopsContaining(key);
				loopCountMax = Math.max(loopCountMax, set.size());
				posCountMax = Math.max(posCountMax,
						getNumberOfPositivePolarityLoops(set));
				negCountMax = Math.max(negCountMax,
						getNumberOfNegativePolarityLoops(set));
				prbCountMax = Math.max(prbCountMax,
						getNumberOfProblemPolarityLoops(set));
			}

			for (String key : loopMembershipWithIVV.keySet()) {
				Set<Loop> set = getLoopsContaining(key);
				loopCountMax = Math.max(loopCountMax, set.size());
				posCountMax = Math.max(posCountMax,
						getNumberOfPositivePolarityLoops(set));
				negCountMax = Math.max(negCountMax,
						getNumberOfNegativePolarityLoops(set));
				prbCountMax = Math.max(prbCountMax,
						getNumberOfProblemPolarityLoops(set));
			}
		}

		double percentageMax = 100.0
				* (double) loopCountMax
				/ Math.max((double) (allLoopsNoIVV == null ? allLoopsNoIVVSize
						: allLoopsNoIVV.size()),
						(double) (allLoopsNoIVV == null ? allLoopsNoIVVSize
								: allLoopsNoIVV.size()));

		return new ColumnSizing(loopCountMax, percentageMax, posCountMax,
				posRangeMin, posRangeMax, negCountMax, negRangeMin,
				negRangeMax, prbCountMax, prbRangeMin, prbRangeMax);
		}

	public Set<Loop> getLoopsOfLengthContaining(int length, Pair pair) {
		Set<Loop> resultSet = getLoopsOfLength(length);
		resultSet = intersect(
				intersect(resultSet, getLoopsContaining(pair.getV1())),
				getLoopsContaining(pair.getV2()));
		return resultSet;
	}

	public Set<Loop> getLoopsOfLength(int length) {

		Set<Loop> loopsOfLength = new LinkedHashSet<Loop>();
		if (isUseDB()) {
			loopsOfLength.addAll(InformationManagers.getInstance()
					.getDatabaseManager().getDb().getLoopsOfLength(length));
		} else {
			for (Loop loop : allLoopsNoIVV)
				if (loop.getSize() == length)
					loopsOfLength.add(loop);
		}
		return loopsOfLength;
	}
	
	public Set<Loop> getIVVLoopsOfLength(int length) {

		Set<Loop> loopsOfLength = new LinkedHashSet<Loop>();
		if (isUseDB()) {
			loopsOfLength.addAll(InformationManagers.getInstance()
					.getDatabaseManager().getDb().getLoopsOfLength(length));
		} else {
			for (Loop loop : allLoopsWithIVV)
				if (loop.getSize() == length)
					loopsOfLength.add(loop);
		}
		return loopsOfLength;
	}

	public Set<Loop> intersect(Set<Loop> set1, Set<Loop> set2) {
		Set<Loop> newSet = new LinkedHashSet<Loop>();
		for (Loop loop : set1)
			if (set2.contains(loop))
				newSet.add(loop);
		return newSet;
	}
	
	public AdjacencyMatrix createLoopDerivedAdjacency(Set<Loop> allLoops) {
		AdjacencyMatrix adjacency = new AdjacencyMatrix(vensimEquationManager);
		for (String variable : vensimEquationManager.getAllKnowVariables()) {
			for (Loop loop : getLoopsContaining(variable)) {
				List<String> lsa = loop.getLoopStartingAt(variable).getLoop();
				String start = Utilities.idToName(lsa.get(0));
				for (int i = 0; i < loop.getSize()-1; i++) {
					adjacency.addSuccessor(Utilities.idToName(start), Utilities.idToName(lsa.get(i+1)));
				}
			}
		}
		if (SDM_Doc.DUMP_TABULAR_DATA)
			adjacency.dumpMatrixToFile(
					"LoopDerivedAdjacencySuccessor.tab",
					adjacency.getSuccessor());
		return adjacency;

	}

	public GraphTheoryMatrix createLoopDerivedReachability(Set<Loop> allLoops) {
		GraphTheoryMatrix reachability = new GraphTheoryMatrix();
		for (String variable : vensimEquationManager.getAllKnowVariables()) {
			for (Loop loop : getLoopsContaining(variable)) {
				List<String> lsa = loop.getLoopStartingAt(variable).getLoop();
				String start = Utilities.idToName(lsa.get(0));
				for (int i = 0; i < loop.getSize(); i++) {
					reachability.addSuccessor(start, Utilities.idToName(lsa.get(i)));
				}
			}
		}
		if (SDM_Doc.DUMP_TABULAR_DATA)
			reachability.dumpMatrixToFile(
					"LoopDerivedReachabilitySuccessor.tab",
					reachability.getSuccessor());
		return reachability;

	}

	public Set<Loop> getAllLoopsNoIVV() {
		if (isUseDB())
			return new LinkedHashSet<Loop>();
		return allLoopsNoIVV;
	}

	public Set<Loop> getAllLoopsWithIVV() {
		if (isUseDB())
			return new LinkedHashSet<Loop>();
		return allLoopsWithIVV;
	}
	
	public AdjacencyMatrix getLoopDerivedAdjacency() {
		if (loopDerivedAdjacency == null)
			loopDerivedAdjacency = createLoopDerivedAdjacency(getAllLoopsNoIVV());
		return loopDerivedAdjacency;
	}

	public GraphTheoryMatrix getLoopDerivedReachability() {
		if (loopDerivedReachability == null)
			loopDerivedReachability = createLoopDerivedReachability(getAllLoopsNoIVV());
		return loopDerivedReachability;
	}

	// public Set<String> mapOlivaToRssd(List<String> set) {
	// Set<String> newSet = new LinkedHashSet<String>();
	// for (String s : set) {
	// newSet.add(mapOlivaToRssd(s));
	// }
	// return newSet;
	// }
	//
	// public Set<String> mapOlivaToRssd(Set<String> set) {
	// Set<String> newSet = new LinkedHashSet<String>();
	// for (String s : set) {
	// newSet.add(mapOlivaToRssd(s));
	// }
	// return newSet;
	// }

	// public String mapOlivaToRssd(String s) {
	// return olivaRssdNameMap.get(s);
	// }

	// public String compareToOliva() {
	// StringBuilder sb = new StringBuilder();
	//
	// // first define our variable name equivalence maps
	// for (Loop l : allLoopsNoIVV) {
	// for (String rssdName : l.getLoop()) {
	// String oEquiv = rssdName.replace(" ", "");
	// olivaRssdNameMap.put(oEquiv, rssdName);
	// rssdOlivaNameMap.put(rssdName, oEquiv);
	// }
	// }
	//
	// Map<Loop, String> olivaMatch = new LinkedHashMap<Loop, String>();
	// Map<Loop, String> rssdMatch = new LinkedHashMap<Loop, String>();
	//
	// for (Loop l : allLoopsNoIVV) {
	// rssdMatch.put(l, "NoMatch");
	// }
	//
	// OlivaLoops oLoops = new OlivaLoops();
	//
	// for (Loop l : oLoops.getAllLoops()) {
	// olivaMatch.put(l, "NoMatch");
	// }
	//
	//
	// for (Loop oLoop : oLoops.getAllLoops()) {
	// // get Loop contents and Starting var
	// List<String> vars = oLoop.getLoop();
	// String start = vars.get(0);
	//
	// List<Loop> rssdLoops = this.getLoopsLimitedTo(mapOlivaToRssd(vars));
	// for (Loop rssdLoop : rssdLoops) {
	// Loop l = rssdLoop.getTruncatedLoopStartingAt(mapOlivaToRssd(start));
	// if (l.isEquivalentTo(oLoop)) {
	// olivaMatch.put(oLoop, "Match");
	// rssdMatch.put(rssdLoop, "Match");
	// }
	// }
	//
	// }
	//
	// // now report on comparison
	//
	// int noMatch = 0;
	//
	// for (Loop l : olivaMatch.keySet()) {
	// String comp = olivaMatch.get(l);
	// if (comp.equals("NoMatch")) {
	// noMatch++;
	// sb.append("### Oliva No Match ###\n");
	// for (String v : l.getLoop()) {
	// sb.append(" "+v+"\n");
	// }
	// }
	// }
	//
	// sb.append("##### Oliva No Match ##### "+noMatch);
	// noMatch = 0;
	// for (Loop l : orderByLoopNumber(rssdMatch.keySet())) {
	// String comp = rssdMatch.get(l);
	// if (comp.equals("NoMatch")) {
	// noMatch++;
	// sb.append("### RSSD No Match ###\n");
	// sb.append(l.getLoopIdentifier()+"\n");
	// for (String v : l.getTruncatedLoop()) {
	// sb.append(" "+v+"\n");
	// }
	// }
	// }
	// sb.append("##### RSSD No Match ##### "+noMatch);
	//
	//
	// return sb.toString();
	// }

	// public void writeLoopsToFile(String directory, String file) {
	// writeLoopsToFile(Utilities.openFileForWriting(directory, file));
	// }

	public void writeOneLineLoopsToTextFile(String file, List<Loop> loops) {
		writeOneLineLoopsToTextFile(
				Utilities.openFileForWriting(InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory(), file),
				loops);
	}

	public void writeOneLineLoopsToTextFile(BufferedWriter bw, List<Loop> loops) {

		for (Loop loop : loops) {
			loop.writeOneLine(bw);
		}

		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void writeLoopsToTabFile(String file, List<Loop> loops) {
		if (sdmuiMode)
			// writeLoopsToTabFile(Utilities.openFileForWriting(
			// sdmuiRootDirectory, file), loops);
			writeLoopsToTabFile(Utilities.openFileForWriting(file), loops);
	}

	public void writeLoopsToTabFile(BufferedWriter bw, List<Loop> loops) {

		for (Loop loop : loops) {
			loop.writeTabWithPolarity(bw);
		}

		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void writeLoopsToTextFile(String file, List<Loop> loops) {
		writeLoopsToTextFile(
				Utilities.openFileForWriting(InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory(), file),
				loops);
	}

	public void writeLoopsToTextFile(BufferedWriter bw, List<Loop> loops) {

		for (Loop loop : loops) {
			loop.writeWithPolarity(bw);
		}

		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeLoopsToHtmlFile(String file, String extension, List<Loop> loops, String title) {

		// need to support multiple files for quicker access
		
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();
		int fileNumber = 0;
		int loopNumber = 1;
		BufferedWriter bw = null;
		int num = 1;
		
		for (Loop loop : loops) {
			
			if (loopNumber++ % Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_LOOPS_PER_FILE))-1 == 0) {
				if (bw != null) {
					writeLoopHtmlFooter(bw);
					try {
						bw.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				bw = openForWriting(file, extension, fileNumber++); 
				writeLoopHtmlHeader(bw, title);
			}

			writeLoopHtmlLoop(bw, loop, num++, loops.size());

		}
		if (bw != null) {
			writeLoopHtmlFooter(bw);
			try {
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		bw = Utilities.openFileForWritingCharset(InformationManagers.getInstance()
				.getFileOrganizer().getLoopsDirectory()+"/"+ file+extension,
				SDMDocLanguageSpecific.getInstance().getFileCharSet());
		
		writeLoopHtmlHeader(bw, title);
		
		for (int i = 0; i < fileNumber-1; i++) {
			try {
				String f = InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory()+"/"+ file+"_"+i+extension; 
				bw.append("<object name=\"foo\" type=\"text/html\" data=\""+f+"\"></object>");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		if (bw != null) {
			writeLoopHtmlFooter(bw);
			try {
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public BufferedWriter openForWriting(String file, String extension, int fileNumber) {
		BufferedWriter bw = Utilities.openFileForWritingCharset(InformationManagers.getInstance()
				.getFileOrganizer().getLoopsDirectory()+"/"+ file+"_"+fileNumber+extension,
				SDMDocLanguageSpecific.getInstance().getFileCharSet());
		return bw;
	}

	public void writeLoopsToHtmlFile(String file, List<Loop> loops, String title) {
		writeLoopsToHtmlFile(
				Utilities.openFileForWritingCharset(InformationManagers.getInstance()
						.getFileOrganizer().getLoopsDirectory()+"/"+ file,
						SDMDocLanguageSpecific.getInstance().getFileCharSet()),
						loops, title);
	}

	public void writeLoopsToHtmlFile(BufferedWriter bw, List<Loop> loops,
			String title) {

		writeLoopHtmlHeader(bw, title);
		int num = 1;
		for (Loop loop : loops) {
			writeLoopHtmlLoop(bw, loop, num++, loops.size());
		}
		writeLoopHtmlFooter(bw);
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getHtmlTail() {
		return "\n</html>\n";
	}

	public String getHtmlHead(String title) {
		StringBuilder sb = new StringBuilder();
		sb.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
				+ "<html>"
				+ "<head>"
				+ "<meta http-equiv='Content-Type' content='Type=text/html; charset="+SDMDocLanguageSpecific.getInstance().getHtmlCharSet()+"'>"
				+ "<title>" + title + "</title>" + "</head>\n");

		return sb.toString();
	}

	public void writeLoopHtmlHeader(BufferedWriter bw, String title) {
		try {
			bw.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">"
					+ SDMDocLanguageSpecific.getInstance().getHtmlHeader()
					+ "<head>"
					+ "<meta http-equiv='Content-Type' content='Type=text/html; charset="+SDMDocLanguageSpecific.getInstance().getHtmlCharSet()+"'>"
					+ "<title>" + title + "</title>" + "</head>");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void writeLoopHtmlFooter(BufferedWriter bw) {
		try {
			bw.append("</html>\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void writeLoopHtmlLoop(BufferedWriter bw, Loop loop, int num,
			int total) {
		try {
			bw.append(loop.getHtmlWithPolarity(num, total));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public String getMaximumLoopLengthAsString() {
		if (isMaximumLoopLengthUnlimited())
			return SDMDocLanguageSpecific.getInstance().getTextHtml(
					StringConstants.UNLIMITED);
		else
			return Integer.toString(maximumLoopLength);
	}

	public int getMaximumLoopLength() {
		return maximumLoopLength;
	}

	public boolean isMaximumLoopLengthUnlimited() {
		return unlimitedSize;
	}

	/**
	 * @return the variablesNotVisited
	 */
	public Set<String> getVariablesNotVisited() {
		return variablesNotVisited;
	}

	/**
	 * @return the foundOtherLoop
	 */
	public MutableInteger getFoundOtherLoop() {
		return foundOtherLoop;
	}

//	private int findMaxLengthDoable(int startLength, boolean withIVV) {
//
//		failLength = 0;
//		successLength = 0;
//		currentLength = startLength;
//		boolean done = false;
//		
//		List<String> lengthList = new ArrayList<String>();
//		
//		System.out.println("### findMaxLengthDoable ### ");
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//
//		while (!done) {
//			garbageCollect(false);
//			pause();
//			String stringCurrentLength = Integer.toString(currentLength);
//			if (lengthList.contains(stringCurrentLength)) {
//				System.out.println("Already tried "+stringCurrentLength);
//			} else {
//				lengthList.add(stringCurrentLength);
//			}
//			mf.addMessage(language.getText(StringConstants.AVAILABLE_HEAP_SPACE_AT_PROCESS_START)+
//					": "+HeapMonitorThread.getMemoryStats());
//			if (loopTimingAndMemory(currentLength, withIVV, true)) {
//				successLength = currentLength;
//				mf.addMessage(maxLenMessage()+
//						language.getText(StringConstants.LOOP_COUNT)
//						+ ": "+currentLoopCount);
//				// we succeeded at this length, decide what to do
//				// if longest is shorter than what we said was max, we've found the max length
//				if (foundCycles.longestLoopSize() < currentLength)
//					return foundCycles.longestLoopSize();
//				
//				// if success at first attempt, then return. User will need to increase
//				if (failLength == 0 && foundCycles.longestLoopSize() == startLength) {
////					return foundCycles.longestLoopSize();
//				}
//				
//				if (failLength > 0 && foundCycles.longestLoopSize() == startLength) {
//					return foundCycles.longestLoopSize();
//				}
//				
//				// if not previous fail, just try longer loop length
//				if (failLength == 0) { // which really can't be true
//					mf.addMessage(maxLenMessage()+language.getText(StringConstants.INCREASE_MAXIMUM_LENGTH_FROM)+ currentLength
//							+ " "+language.getText(StringConstants.TO)+" " + (currentLength+5));
//					currentLength += 5; // *= 2;
//					this.maxLengthForMessage = currentLength;
//				} else { // this is where we should always endup
//					int newMaxLength = successLength + (failLength - successLength) / 2;
//					if (newMaxLength == successLength) {
//						return successLength;
//					}
//					mf.addMessage(maxLenMessage()+language.getText(StringConstants.LOOKING_FOR_LONGER_LOOPS)+". "+
//							language.getText(StringConstants.CURRENT_LOOP_COUNT)+": "
//							+ currentLoopCount);
//					mf.addMessage(maxLenMessage()+language.getText(StringConstants.ADJUST_MAXIMUM_LENGTH_FROM)+" " + currentLength
//							+ " "+language.getText(StringConstants.TO)+" " + (newMaxLength));
//					currentLength = newMaxLength;
//					this.maxLengthForMessage = newMaxLength;
//				}
//			} else {
//				// we've failed, so this becomes the new ceiling
//				failLength = currentLength;
//				// if this is the first time, just cut in half
//				if (failLength == startLength) {
////					maxLen = maxLen - 5; // / 2;
//					int newMaxLength = failLength - (failLength - successLength)/2;
//					if (newMaxLength == failLength) {
//						return NO_LOOPS;
//					}
//					mf.addMessage(maxLenMessage()+language.getText(StringConstants.DECREASE_MAXIMUM_LENGTH_FROM)+" " + failLength
//							+ " "+language.getText(StringConstants.TO)+" " + newMaxLength);
//					currentLength = newMaxLength;
//					this.maxLengthForMessage = newMaxLength;
//
//				} else {
////					maxLen = maxLen - 5; // / 2;
//					int newMaxLength = failLength - (failLength - successLength)/2;
//					if (newMaxLength == failLength) {
//						return NO_LOOPS;
//					}
//					mf.addMessage(maxLenMessage()+language.getText(StringConstants.DECREASE_MAXIMUM_LENGTH_FROM)+" " + failLength
//							+ " "+language.getText(StringConstants.TO)+" " + newMaxLength);
//					currentLength = newMaxLength;
//					this.maxLengthForMessage = newMaxLength;
//				}
//			}
//		}
//
//		return currentLength;
//	}
	
	private String maxLenMessage() {
		if (unlimitedSize) {
			return "(Unlimited) ";
		} else {
			return "(" + this.maxLengthForMessage + ") ";
		}
	}

	private boolean loopTimingAndMemory(int maxLength, boolean withIVV) {
		return loopTimingAndMemory(maxLength, withIVV, false);
	}

	private boolean loopTimingAndMemory(int maxLength, boolean withIVV,
			boolean discoveryMode) {

		boolean unlimited = maxLength == Integer.MAX_VALUE - 10;
		
		
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();
		double maxTimeSeconds = Double.parseDouble(options
				.getOptionValue(StringConstants.MAX_SEARCH_TIME_MINUTES)) * 60.0;

		if (discoveryMode) {
			if (withIVV)
				mf.addMessage(maxLenMessage()+language.getText(StringConstants.LOOPS_WITH_IVV)+": "
						+ language.getText(StringConstants.TRYING_MAXIMUM_LENGTH)+" = " + maxLength);
			else
				mf.addMessage(maxLenMessage()+language.getText(StringConstants.LOOPS_WITHOUT_IVV)+": "
						+ language.getText(StringConstants.TRYING_MAXIMUM_LENGTH)+" = " + maxLength);
		}

		GraphCyclesThread gc = null;
		Thread graphCycleThread = null;
		Thread heapMonitorThread = null;
		long startTime = System.nanoTime();
		long accumTime;
		boolean haveCycles = false;
		int maxLen = maxLength;
		while (!haveCycles && maxLen > 0) {

			maximumLoopLength = maxLen;
			maxLengthForMessage = maxLen;
			System.out.println("\n\nloopTimingAndMemory max length: " + maxLen);
			startTime = System.nanoTime();
			accumTime = 0L;
			gc = null;

			garbageCollect(true);

			try {
				config.setCyclesFile(new CyclesFile(config.getFilename(withIVV)));
				gc = new GraphCyclesThread(loadEdgesNoWrite(withIVV), maxLen, mf, config, withIVV);
			} catch (Exception e1) {
				// TODO Auto-generated catch block
				gc = null;
				e1.printStackTrace();
				mf.addMessage(e1.getMessage());
				mf.addMessage(language.getText(StringConstants.TRYING_TO_CORRECT));
				garbageCollect(true);
				try {
					Thread.sleep(1000);
				} catch (InterruptedException e2) {
					// TODO Auto-generated catch block
					e2.printStackTrace();
				}
			}

//			if (graphCycleThread != null && graphCycleThread.isAlive()) {
//				System.out.println("DEBUG Thread graphCycleThread");
//			}

			graphCycleThread = gc /*new Thread(gc) */;

			// get our thread ready for monitoring heap size
			HeapMonitorThread hm = new HeapMonitorThread(this, mf);

//			if (heapMonitorThread != null && heapMonitorThread.isAlive()) {
//				System.out.println("DEBUG Thread heapMonitorThread");
//			}
			heapMonitorThread = hm; /* new Thread(hm); */

			// we want to start both of these threads and then check to see if
			// cycles makes it all the way
			// through. HeapMonitor will stop it if we run short on heap space.
			// We will stop if time too long;
			graphCycleThread.start();
			graphCycleThread.setPriority(Thread.MIN_PRIORITY);
			heapMonitorThread.start();
			haveCycles = false;
			boolean done = false;
			while (!done) {
				System.out.println("Not Done Loop");
				// is cycles done?
				foundCycles = null;

				garbageCollect(false);
				if (!graphCycleThread.isAlive()) {
					System.out.println("Cycles is done computing");
					// is it because HeapMonitor stopped it?
					if (heapMonitorThread.isAlive()) {
						System.out
								.println("Heap Monitor still running - completed cycles");
						// cycles completed, so stop
						heapMonitorThread.interrupt();
						System.out
								.println("Interrupted Heap Monitor -- try join");
						try {
//							heapMonitorThread.join(1000);
							heapMonitorThread.join();
							System.out.println("HeapMonitor Done with Join");

						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						if (gc.getCycles() != null) {
							haveCycles = true;
							currentLoopCount = gc.getCycles().size();
							if (!discoveryMode)
								mf.addMessage(maxLenMessage()+language.getText(StringConstants.LOOP_DISCOVERY_COMPLETED_AT_MAXIMUM_LENGTH_OF)+
										" "
										+ (unlimited ? language.getText(StringConstants.UNLIMITED) : maxLen)
										+ "\n");
						} else {
							gc = null;
							haveCycles = false;
							mf.addMessage(maxLenMessage()+language.getText(StringConstants.LOOP_DISCOVERY_ABORTED_DUE_TO_MEMORY_LIMITATIONS));
						}
					} else {
						gc = null;
						System.out
								.println("Heap Monitor not running - Must have heap issues");
						mf.addMessage(maxLenMessage()+language.getText(StringConstants.HEAP_SPACE_LIMITS_REACHED));
						haveCycles = false;
					}

					done = true;

				} else {
					System.out.println("Cycles still computing");
					// c.interrupt();

					if (!heapMonitorThread.isAlive()) {
						System.out
								.println("Heap Monitor Not Running -- I will interupt cycles");
						graphCycleThread.interrupt();
						System.out.println("Sent interrupt -- join");
						try {
//							graphCycleThread.join(1000);
							graphCycleThread.join();
							System.out.println("GraphCycle Join Done");
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
						done = true;
					} else {
						// still running so sleep awhile
						accumTime = System.nanoTime() - startTime;
						long maxSleep = 10000L * 12L;
						long bumpTime = 60L *5L; // every 5 minutes increase sleep time
						long duration = accumTime / 1000000000L;
						long sleepTime = Math.min(maxSleep, 10000L + (duration / bumpTime) * 10000L);
						System.out.println("Elapsed Time: "+formatTime(accumTime) +" Sleep Time: "+sleepTime);
						try {
							Thread.sleep(sleepTime); // 5000
						} catch (InterruptedException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

						// here is where we check accuml time
//						accumTime = System.nanoTime() - startTime;
						if (accumTime / 1000000000.0 > maxTimeSeconds) {
							System.out
									.println("Reached Time limit interrupting monitors");
							mf.addMessage(maxLenMessage()+language.getText(StringConstants.TIME_LIMIT_EXCEEDED)+" " +
									formatTime(accumTime));
//									+ accumTime
//									/ 1000000000.0 + " "+language.getText(StringConstants.SECONDS));
							graphCycleThread.interrupt();
							heapMonitorThread.interrupt();
							System.out.println("Interrupted both");
							haveCycles = false;
							gc = null;
							done = true;
							try {
//								graphCycleThread.join(1000);
//								heapMonitorThread.join(1000);
								graphCycleThread.join();
								heapMonitorThread.join();
								System.out.println("HeapMonitor & Graph Join Done");
							} catch (InterruptedException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						} else {
							System.out.println("Time: " + formatTime(accumTime));
//									/ 1000000000.0 + " sec");
							mf.addMessage(maxLenMessage()+language.getText(StringConstants.SEARCH_TIME)+": "
									+formatTime(accumTime));
//									+ NumberFormatter
//											.formatForPercent(accumTime / 1000000000.0)
//									+ " "+language.getText(StringConstants.SECONDS));
						}
					}
				}
			}
			
			System.out.println("I am done");

			if (discoveryMode || haveCycles) {
				foundCycles = gc;
				return haveCycles;
			} 
			if (!haveCycles) {
				mf.addMessage(maxLenMessage()+language.getText(StringConstants.COULD_NOT_COMPLETE_LOOP_PROCESSING_AT_LOOP_MAXIMUM_LENGTH_OF)+
						" "
						+ (unlimited ? language.getText(StringConstants.UNLIMITED) : maxLen));
				foundCycles = null;
				
				// need to make sure that individual cycle information that may be recorded is cleared();
				
				clearPreviousLoops();
				
				gc = null;
			}
			// compute new length
			if (unlimited) {
				maxLen = 100;
				unlimited = false;
			} else {
				maxLen -= 5; // /= 2;
				mf.addMessage(maxLenMessage()+language.getText(StringConstants.REDUCING_MAXIMUM_LENGTH_TO)+" "+maxLen);
			}
			if (maxLen <= 2)
				break;
			if (!haveCycles) {
				mf.addMessage(maxLenMessage()+language.getText(StringConstants.SETTING_MAXIMUM_LENGTH_TO)+" " + maxLen
						+ language.getText(StringConstants.AND_RETRYING));
			}
		}
		return haveCycles;
	}
	
	/**
	 * provides a String representation of the given time
	 * @return {@code millis} in hh:mm:ss format
	 */
	public static final String formatTime(long millis) {
	    long secs = millis / 1000000000;
	    long hours = secs / 3600;
	    long minutes = (secs & 3600) / 60;
	    long seconds = secs % 60;
	    return String.format("%02d:%02d:%02d", secs / 3600, (secs % 3600) / 60, secs % 60);
	}
	
	public void clearPreviousLoops() {
		variableLoopPolarityCounts = new LinkedHashMap<Boolean, Map<String, VariableLoopPolarityCounts>>();
		variableLoopPolarityCounts.put(false, new LinkedHashMap<String, VariableLoopPolarityCounts>());
		variableLoopPolarityCounts.put(true, new LinkedHashMap<String, VariableLoopPolarityCounts>());
		variableLoopPolarityCounts.get(true).put("__ALL__", new VariableLoopPolarityCounts());
		variableLoopPolarityCounts.get(false).put("__ALL__", new VariableLoopPolarityCounts());
		shortestLoopLengthNoIVV = Integer.MAX_VALUE;
		longestLoopLengthNoIVV = Integer.MIN_VALUE;
		lla = new LoopLengthAnalyzer(this);
		singleCycleCount = 0;
	}
	
//	public long countCycles(int maxLen, MutableInteger mi) {
//		GraphCyclesThread gc = null;
//		try {
//			gc = new GraphCyclesThread(loadEdgesNoWrite(false), maxLen, mf, config);
//			gc.countCycles(maxLen);
//		} catch (Exception e1) {
//			// TODO Auto-generated catch block
//			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//			e1.printStackTrace();
//			mf.addMessage(e1.getMessage());
//			mf.addMessage(language.getText(StringConstants.TRYING_TO_CORRECT));
//			garbageCollect(true);
//			try {
//				Thread.sleep(1000);
//			} catch (InterruptedException e2) {
//				// TODO Auto-generated catch block
//				e2.printStackTrace();
//			}
//			return -1;
//		}
//		if (gc != null) {
//			mi.setValue(gc.getMaxLength());
//			return gc.getNumCycles();}
//		else
//			return -1;
//	}
	
	private void processFoundCycles(boolean withIVV, List<List<String>> cycles) {

		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();


		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		long numCycles = cycles.size();
		if (config.isProcessCycleWhenFound())
			numCycles = singleCycleCount;

		mf.addMessage(language.getText(StringConstants.PROCESSING_DISCOVERED_LOOPS)+". "+
				language.getText(StringConstants.LOOP_COUNT)+": " + numCycles);

		boolean overrideProcessing = false;  // true 4/29/2017
		overrideProcessing = options.isSelected(StringConstants.OVERRIDE_LOOP_COUNT) && 
				options.isSelected(StringConstants.DETAILED_REPORTS);
		if (cycles.size() > HARDCODED_LOOP_COUNT_LIMIT)
			overrideProcessing = false;

		try {
			boolean mustUseFullProcessing = false;
			if (
					(options.isSelected(StringConstants.DETAILED_REPORTS) &&
							(options.isSelected(StringConstants.HYPERLINK_LOOP) ||
							options.isSelected(StringConstants.VARIABLE_RELATIONSHIPS) || 
							options.isSelected(StringConstants.VARIABLE_LOOPS)) && 
							cycles.size() < MAX_LOOPS_FOR_POSTPROCESS) ||
							overrideProcessing) { // cycles < 5000
				if (overrideProcessing)
					mf.addMessage(language.getText(StringConstants.USER_OVERRIDE_OF_LOOP_LIMIT_WILL_DO_FULL_PROCESSING));
				mustUseFullProcessing = true;
			}

			if (mustUseFullProcessing) {
				mf.addMessage(language.getText(StringConstants.PERFORMING_FULL_LOOP_PROCESSING));
				processCycles(cycles, withIVV);
			} else {
				mf.addMessage(language.getText(StringConstants.PERFORMING_MINIMUM_MEMORY_LOOP_PROCESSING));
				if (config.isProcessCyclesInMemory()) {
					processCyclesMinimumMemory(cycles, withIVV);
				} else {
					PostProcessCycleMinimumMemory(withIVV);
				}
			}					
		} catch (OutOfMemoryError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			mf.addMessage(maxLenMessage()+language.getText(StringConstants.INSUFFICIENT_SPACE_TO_PROCESS_ALL_LOOPS)+" -- "+
					language.getText(StringConstants.TERMINATING));
			//						+ maxLen);
			//				return loopTimingAndMemory(maxLen, withIVV);

		}

		//			gc = null;
		//			System.out.println("Number of cycles: " + cycles.size());
		//			cycles = null;
		//			double estimatedTime = ((double) (System.nanoTime() - startTime)) / 1000000000.0;
		//			System.out.println(" est time " + estimatedTime);
		garbageCollect(true);
		return ;

	}
	
	public List<VariableLoopCount> getVariableLoopCounts() {
		if (variableLoopCount.size() > 0) {
			return variableLoopCount;
		}
		
		
		
		return variableLoopCount;
	}
	
	private void pause() {
		try {
			
			Thread.sleep(100L);
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
	}

	public void garbageCollect(boolean report) {

		String before = HeapMonitorThread.getMemoryStats();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		System.gc();
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
			return;
		}
		String after = HeapMonitorThread.getMemoryStats();
		if (report && HeapMonitorThread.freeMemory() < 10.0)
			mf.addMessage(maxLenMessage()+language.getText(StringConstants.HEAP_SPACE_AVAILABLE)+":" + after);
	}

	/**
	 * @return the sdmuiMode
	 */
	public boolean isSdmuiMode() {
		return sdmuiMode;
	}

	/**
	 * @param sdmuiMode
	 *            the sdmuiMode to set
	 */
	public void setSdmuiMode(boolean sdmuiMode) {
		this.sdmuiMode = sdmuiMode;
	}

	/**
	 * @return the sdmuiRootDirectory
	 */
	public String getSdmuiRootDirectory() {
		return sdmuiRootDirectory;
	}

	/**
	 * @param sdmuiRootDirectory
	 *            the sdmuiRootDirectory to set
	 */
	public void setSdmuiRootDirectory(String sdmuiRootDirectory) {
		this.sdmuiRootDirectory = sdmuiRootDirectory;
	}

	public VariableRelationships getVariableRelationships() {
		return variableRelationships;
	}

	public VariableLoops getVariableLoops() {
		return variableLoops;
	}

	public int getShortestLoopLength() {
		return shortestLoopLengthNoIVV == Integer.MAX_VALUE ? 0
				: shortestLoopLengthNoIVV;
	}

	public int getLongestLoopLength() {
		return longestLoopLengthNoIVV == Integer.MIN_VALUE ? 0
				: longestLoopLengthNoIVV;
	}

	public int getShortestLoopLengthNoIVV() {
		return shortestLoopLengthNoIVV == Integer.MAX_VALUE ? 0
				: shortestLoopLengthNoIVV;
	}

	public int getLongestLoopLengthNoIVV() {
		return longestLoopLengthNoIVV == Integer.MIN_VALUE ? 0
				: longestLoopLengthNoIVV;
	}

	public int getShortestLoopLengthWithIVV() {
		return shortestLoopLengthWithIVV == Integer.MAX_VALUE ? 0
				: shortestLoopLengthWithIVV;
	}

	public int getLongestLoopLengthWithIVV() {
		return longestLoopLengthWithIVV == Integer.MIN_VALUE ? 0
				: longestLoopLengthWithIVV;
	}

	public boolean isUseIncore() {
		return useIncore;
	}

	public void setUseIncore(boolean useIncore) {
		this.useIncore = useIncore;
		this.useDB = !useIncore;
	}

	public boolean isUseDB() {
		return useDB;
	}

	public void setUseDB(boolean useDB) {
		this.useDB = useDB;
		this.useIncore = !useDB;
	}

	public boolean isUsingMinimumMemory() {
		return usingMinimumMemory;
	}

	public ReachabilityMatrix getReachability() {
		return reachability;
	}

	public long getSingleCycleCount() {
		return singleCycleCount;
	}

	public LoopLengthAnalyzer getLla() {
		return lla;
	}
	
	public void clearLla() {
		lla = new LoopLengthAnalyzer(this);
	}
}
