package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class LoopsBySize {

	private Map<Integer, Set<Loop>> loopsBySize;

	public LoopsBySize() {
		loopsBySize = new LinkedHashMap<Integer, Set<Loop>>();
	}

	public void addLoop(Loop loop) {
		if (!loopsBySize.containsKey(loop.getSize())) {
			loopsBySize.put(new Integer(loop.getSize()), new LinkedHashSet<Loop>());
		}
		loopsBySize.get(loop.getSize()).add(loop);
	}

	public List<Integer> getSizeCounts(int maxLength) {
		List<Integer> al = new ArrayList<Integer>();
		for (int i = 2; i <= maxLength; i++) {
			if (loopsBySize.containsKey(i)) {
				al.add(loopsBySize.get(i).size());
			} else {
				al.add(0);
			}
		}
		return al;
	}
	
	public List<Loop> getLoopsOfLength(int length) {
		List<Loop> al = new ArrayList<Loop>();
		if (!loopsBySize.containsKey(length)) {
			return al;
		} else {
			al.addAll(loopsBySize.get(length));
			return al;
		}
	}
	
	public List<List<List<String>>> getTableData(int maxLength) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		
//		String titlel = "Something";
//		TableUtilities.addTitle(table, titlel);

		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);
		
		// TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));

//		TableUtilities.addColumn(row, TableUtilities.createColumn("Length"));
//		TableUtilities.addColumn(row, TableUtilities.createColumn("Num Loops"));
//		TableUtilities.addColumn(row, TableUtilities.createColumn("Cumul"));
//		TableUtilities.addColumn(row, TableUtilities.createColumn("Loops"));
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.LENGTH)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.LOOP_COUNT)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.CUMULATIVE)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.LOOP_DETAIL)));
		
		int cumul = 0;
		
		List<Integer> al = getSizeCounts(maxLength);
		for (int i = 0; i < al.size(); i++) {
			row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			
			cumul += al.get(i);
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForIntComma(i+2)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForIntComma(al.get(i))));
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForIntComma(cumul)));
			
		}
		

		return table;
	}

	
}
