package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;

public class SingleLoopLengthAnalysis {
	
	private static final String COLUMN_HEADERS = "Up To Length\tNumber"
			+ "\tPositive\tNegative\tUnknown\tMin Length\tMax Length\tP/N"
			+ "\tCommon 1\tCommon 2\tCommon 3";
	
	private static final String COLUMN_HEADERS_NO_COMMON = "Up To Length\tNumber"
			+ "\tPositive\tNegative\tUnknown\tMin Length\tMax Length\tP/N"
			;
	
	private int loopLength;
	private int numLoops;
	private int numPositive;
	private int numNegative;
	private int numUnk;
	private int shortest;
	private int longest;
	private double pn;
	private String common1; 
	private String common2; 
	private String common3; 
	
	public SingleLoopLengthAnalysis(int loopLength) {
		this.loopLength = loopLength;
	}
	
	public SingleLoopLengthAnalysis(
			int loopLength,
			int numLoops,
			int numPositive,
			int numNegative,
			int numUnk,
			int shortest,
			int longest,
			String common1,
			String common2,
			String common3
			) {
		
		this.loopLength = loopLength;
		this.numLoops = numLoops;
		this.numPositive = numPositive;
		this.numNegative = numNegative;
		this.numUnk = numUnk;
		this.shortest = shortest;
		this.longest = longest;
		this.common1 = common1;
		this.common2 = common2;
		this.common3 = common3;
		
		pn = (double) numPositive / (double) numNegative;
		
	}
	
	public void addLoop(Loop loop) {
		numLoops++;
		switch (loop.getLoopPolarity()) {
		case  ArrowPolarity.POSITIVE:
			numPositive++;
			break;
		case  ArrowPolarity.NEGATIVE:
			numNegative++;
			break;
		case  ArrowPolarity.UNKNOWN:
			numUnk++;
			break;
		}
		
	}
	
	public String asTabbedRecordNoCommon() {
		String pn;
		String min;
		String max;
		if (numLoops == 0) {
			pn = "NA";
			min = "NA";
			max = "NA";
		} else {
			pn = numNegative == 0 ? "Infinite" : NumberFormatter.formatForRatio((double) numPositive / (double) numNegative);
			min = Integer.toString(shortest);
			max = Integer.toString(longest);
		}
		StringBuilder sb = new StringBuilder();
		
		sb.append(loopLength+"\t");
		sb.append(numLoops+"\t");
		sb.append(numPositive+"\t");
		sb.append(numNegative+"\t");
		sb.append(numUnk+"\t");
		sb.append(min+"\t");
		sb.append(max+"\t");
		sb.append(pn);
				
		return sb.toString();
	}
	
	public String asTabbedRecord() {
		StringBuilder sb = new StringBuilder();
		
		sb.append(loopLength+"\t");
		sb.append(numLoops+"\t");
		sb.append(numPositive+"\t");
		sb.append(numNegative+"\t");
		sb.append(numUnk+"\t");
		sb.append(shortest+"\t");
		sb.append(longest+"\t");
		sb.append(pn+"\t");
		sb.append(common1+"\t");
		sb.append(common2+"\t");
		sb.append(common3);
		
		return sb.toString();
	}
	
	public static String getColumnHeaders() {
		return COLUMN_HEADERS;
	}
	
	public static String getColumnHeadersNoCommon() {
		return COLUMN_HEADERS_NO_COMMON;
	}

	public int getLoopLength() {
		return loopLength;
	}

	public int getNumLoops() {
		return numLoops;
	}

	public int getNumPositive() {
		return numPositive;
	}

	public int getNumNegative() {
		return numNegative;
	}

	public int getNumUnk() {
		return numUnk;
	}

	public int getShortest() {
		return shortest;
	}

	public int getLongest() {
		return longest;
	}

}
