package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringEscapeUtils;

public class VariableLoop {
	
	private static final String BREAK = "</br>\n";
	private static final String COLUMN_START = "<td style='text-align:center'>";
	private static final String COLUMN_START_LEFT = "<td style='text-align:left;border-width:0'>";
	private static final String COLUMN_START_LEFT_WIDE = "<td style='text-align:left;border-width:0;min-width:400px'>";
//	private static final String COLUMN_START_LEFT_WIDE = "<td style='text-align:left;border-width:0'>"; //min-width:400px'>";
	private static final String COLUMN_END = "</td>\n";
	
	private static final String ROW_START = "<tr>\n";
	private static final String ROW_END = "</tr>\n";
	
	private static final String TABLE_START = "\n<table class='nolines'>\n";
	private static final String TABLE_START_LINES = "\n<table class='sample'>\n";
	private static final String TABLE_END = "</table>\n";

	private String variable;
	private Set<Loop> loops;
	private LoopsBySize loopsBySize;
	
	private static int NUM_LOOPS_PER_ROW = 6;
	private final static int MAX_LOOPS_PER_VAR = 20;
	
	
	public VariableLoop(String variable) {
		this.variable = variable;
		loops = new LinkedHashSet<Loop>();
		loopsBySize = new LoopsBySize();
		LoopManager lm = InformationManagers.getInstance().getAnalysisManager().getLoopManager();
		if (lm.getNumberOfLoopsNoIVV() > 10000)
			NUM_LOOPS_PER_ROW = 5;
		if (lm.getNumberOfLoopsNoIVV() > 100000)
			NUM_LOOPS_PER_ROW = 4;
		if (lm.getNumberOfLoopsNoIVV() > 1000000)
			NUM_LOOPS_PER_ROW = 3;
	}
	
	public String getRow(int maxLength) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String file = /*mm.getModelName()+*/"_variableLoops.tab";
		
		String taggedModelname = InformationManagers.getInstance().getFileOrganizer().getTaggedModelname();
//		BufferedWriter bw = Utilities.openFileForWriting(
//				InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
//		LoopManager loopManager = InformationManagers.getInstance().getAnalysisManager().getLoopManager();
		
		StringBuilder sb = new StringBuilder();
		sb.append(ROW_START);
		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorDefinition("VariableLoops"+variable));
		sb.append(Anchors.getFileAnchorReference("../../"+taggedModelname+"_"+language.getDisplayLanguage()+".html", variable));
//		sb.append(variable);
		sb.append(COLUMN_END);

		sb.append(COLUMN_START);
		sb.append(NumberFormatter.formatForIntComma(getNumLoops()));
		sb.append(COLUMN_END);
		
//		List<Loop> toBeOrdered = new ArrayList<Loop>();
//		for (Loop rl : loops) {
//			toBeOrdered.add(rl);
//		}
		
//		sb.append(COLUMN_START);
//		sb.append(getLinksAsCSV(LoopManager.orderByLoopNumber(toBeOrdered)));
//		sb.append(COLUMN_END);
		
		sb.append(COLUMN_START);
//		sb.append(getLinksAsCSV(LoopManager.orderByLoopNumberReverse(toBeOrdered)));
		sb.append(getLoopCountTable(maxLength));
		sb.append(COLUMN_END);
		
		sb.append(ROW_END);
		
//		List<Integer> al = loopsBySize.getSizeCounts(maxLength);
//		reportSize(maxLength);
		
		return sb.toString();
	}
	
	
//	private void reportSize(int maxLength) {
//		System.out.println("Var: "+variable+" Table: ");
//		TableUtilities.printTable(loopsBySize.getTableData(maxLength));
//	}
//	
//	private String getLinksAsCSV(List<Loop> orderedLoops) {
//
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//
//		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
//		boolean limitMaxLoops = options.isSelected(StringConstants.LIMIT_V_LOOPS);
//		int maxLoopsPerVariable = Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_V_LOOPS));
//		ModelManager mm = InformationManagers.getInstance().getModelManager();		
//		StringBuilder sb = new StringBuilder();
//		
//		int numLoops = orderedLoops.size();
//		int commas = 0;
//		int numCommas = Math.min(maxLoopsPerVariable, numLoops);
//
//		if (options.isSelected(StringConstants.HYPERLINK_LOOP)) {
//			sb.append(TABLE_START);
//
//			
//			int loopsProcessed = 0;
//			int col = 1;
//			sb.append(ROW_START);
//			sb.append(COLUMN_START_LEFT);
//			for (Loop loop : orderedLoops) {
//				loopsProcessed++;
//				if (limitMaxLoops && loopsProcessed > maxLoopsPerVariable)
//					break;
//				if (col++ > NUM_LOOPS_PER_ROW) {
//					sb.append(ROW_END);
//					sb.append(ROW_START);
//					sb.append(COLUMN_START_LEFT);
//					col = 1;
//				}
//				//			sb.append(COLUMN_START);
//				// <a href='world2_DOC_GRAPH_/world2_LoopsNoIVV_English.html'>Loop List No IVV</a></td>
//			
//				int loopNum = loop.getLoopNumber();
//				int fileNum = (loopNum-1)/Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_LOOPS_PER_FILE));
//				sb.append("<a href='");
//				sb.append(StringEscapeUtils.escapeHtml4(mm.getModelName())+"_LoopsNoIVV_"+language.getDisplayLanguage()+"_"+fileNum+".html");
//				sb.append("#");
//				sb.append(loop.getLoopIdentifierHTML());
//				sb.append("' target=\"_blank\">");
//				sb.append(loop.getLoopIdentifierNoPolarityNoSize());
//				
//				if (commas++ < numCommas-1)
//					sb.append(",");
//				sb.append("</a>");
//				sb.append("&nbsp;");
//				
//			}
//			
//			sb.append(COLUMN_END);
//			sb.append(ROW_END);
//			sb.append(TABLE_END);
//			
//		} else {
//			sb.append(TABLE_START);
//			sb.append(ROW_START);
//			sb.append(COLUMN_START_LEFT);
//			
////			List<Loop> toBeOrdered = new ArrayList<Loop>();
////			for (Loop rl : loops) {
////				toBeOrdered.add(rl);
////			}
//			int loopsProcessed = 0;
//			int col = 1;
//		
//			for (Loop loop : orderedLoops) {
//				loopsProcessed++;
//				if (limitMaxLoops && loopsProcessed > maxLoopsPerVariable)
//					break;
//				if (col++ > NUM_LOOPS_PER_ROW) {
//					sb.append(ROW_END);
//					sb.append(ROW_START);
//					sb.append(COLUMN_START_LEFT);
//					col = 1;
//				}
//
//				sb.append(loop.getLoopIdentifierNoPolarityNoSize());
//				if (commas++ < numCommas-1)
//					sb.append(",");
//				sb.append("&nbsp;");
//				
//			}
//			sb.append(COLUMN_END);
//			sb.append(ROW_END);
//			sb.append(TABLE_END);
//		}
//		return sb.toString();
//	}
	
	private String getLoopCountTable(int maxLength) {

		StringBuilder sb = new StringBuilder();

			sb.append(TABLE_START_LINES);
			
			List<List<List<String>>> table = loopsBySize.getTableData(maxLength);
		
			for (int r = 0; r < TableUtilities.getNumRows(table); r++) {
				List<List<String>> row = TableUtilities.getRow(table, r);
				sb.append(ROW_START);
				for (int c = 0; c < TableUtilities.getNumColumns(row); c++) {
					sb.append(COLUMN_START);
					sb.append(TableUtilities.getColumn(row, c).get(0));
					sb.append(COLUMN_END);
				}
				
				// title inserted in getTableData
				
				if (r > 0) {
					sb.append(COLUMN_START);
					sb.append(getLinksAsCSV(r+1));
					sb.append(COLUMN_END);
					sb.append(ROW_END);
				}
			}
			sb.append(TABLE_END);
		return sb.toString();
	}
	
	private String getLinksAsCSV(int length) {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		boolean limitMaxLoops = options.isSelected(StringConstants.LIMIT_V_LOOPS);
		int maxLoopsPerVariable = Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_V_LOOPS));
		ModelManager mm = InformationManagers.getInstance().getModelManager();	
		
		
		StringBuilder sb = new StringBuilder();
		List<Loop> orderedLoops = LoopManager.orderByLoopNumber(loopsBySize.getLoopsOfLength(length));
		
		
		int numLoops = orderedLoops.size();
		int commas = 0;
		int numCommas = Math.min(maxLoopsPerVariable, numLoops);

		if (options.isSelected(StringConstants.HYPERLINK_LOOP)) {
			sb.append(TABLE_START);

			
			int loopsProcessed = 0;
			int col = 1;
			sb.append(ROW_START);
			sb.append(COLUMN_START_LEFT_WIDE);
			for (Loop loop : orderedLoops) {
				loopsProcessed++;
				
				if (limitMaxLoops && loopsProcessed > maxLoopsPerVariable)
					break;
				if (col++ > NUM_LOOPS_PER_ROW) {
					sb.append(ROW_END);
					sb.append(ROW_START);
					sb.append(COLUMN_START_LEFT_WIDE);
					col = 2;
				}
				//			sb.append(COLUMN_START);
				// <a href='world2_DOC_GRAPH_/world2_LoopsNoIVV_English.html'>Loop List No IVV</a></td>
			
				int loopNum = loop.getLoopNumber();
				int fileNum = (loopNum-1)/Integer.parseInt(options.getOptionValue(StringConstants.MAX_LIMIT_LOOPS_PER_FILE));
				sb.append("<a href='");
				sb.append(/*StringEscapeUtils.escapeHtml4(mm.getModelName())+*/ "_LoopsNoIVV_"+language.getDisplayLanguage()+"_"+fileNum+".html");
				sb.append("#");
				sb.append(loop.getLoopIdentifierHTML());
				sb.append("' target=\"_blank\">");
				sb.append(loop.getLoopIdentifierNoPolarityNoSize());
				
				if (commas++ < numCommas-1)
					sb.append(",");
				sb.append("</a>");
				sb.append("&nbsp;");
				
			}
			
			sb.append(COLUMN_END);
			sb.append(ROW_END);
			sb.append(TABLE_END);
			
		} else {
			sb.append(TABLE_START);
			sb.append(ROW_START);
			sb.append(COLUMN_START_LEFT_WIDE);
			
//			List<Loop> toBeOrdered = new ArrayList<Loop>();
//			for (Loop rl : loops) {
//				toBeOrdered.add(rl);
//			}
			int loopsProcessed = 0;
			int col = 1;
		
			for (Loop loop : orderedLoops) {
				loopsProcessed++;
				if (limitMaxLoops && loopsProcessed > maxLoopsPerVariable)
					break;
				if (col++ > NUM_LOOPS_PER_ROW) {
					sb.append(ROW_END);
					sb.append(ROW_START);
					sb.append(COLUMN_START_LEFT_WIDE);
					col = 2;
				}

				sb.append(loop.getLoopIdentifierNoPolarityNoSize());
				if (commas++ < numCommas-1)
					sb.append(",");
				sb.append("&nbsp;");
				
			}
			sb.append(COLUMN_END);
			sb.append(ROW_END);
			sb.append(TABLE_END);
		}
		return sb.toString();
	}
	
	
	
	public void addLoop(Loop loop) {
		loops.add(loop);
		loopsBySize.addLoop(loop);
	}
	
	public void addLoops(Set<Loop> loops) {
		
		for (Loop loop : loops)
			addLoop(loop);
	}
	
	public int getNumLoops() {
		return loops.size();
	}

	public String getVariable() {
		return variable;
	}
	
	public String getVariableHTML() {
		return StringEscapeUtils.escapeHtml4(variable);
	}

	public Set<Loop> getLoops() {
		return loops;
	}
	
}
