package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.support.MutableInteger;

public class VariableLoopPolarityCounts {
	
	private MutableInteger positive;
	private long minPositive = Integer.MAX_VALUE;
	private long maxPositive = Integer.MIN_VALUE;
	private MutableInteger negative;
	private long minNegative = Integer.MAX_VALUE;
	private long maxNegative = Integer.MIN_VALUE;
	private MutableInteger unknown;
	private long minUnknown = Integer.MAX_VALUE;
	private long maxUnknown = Integer.MIN_VALUE;
	
	public VariableLoopPolarityCounts() {
		positive = new MutableInteger(0);
		negative = new MutableInteger(0);
		unknown = new MutableInteger(0);
	}
	
	public void count(String polarity, long size) {
		if (polarity.equals(ArrowPolarity.POSITIVE)) {
			positive.add(1);
			minPositive = Math.min(minPositive, size);
			maxPositive = Math.max(maxPositive, size);
		} else if (polarity.equals(ArrowPolarity.NEGATIVE)) {
			negative.add(1);
			minNegative = Math.min(minNegative, size);
			maxNegative = Math.max(maxNegative, size);
		} else {
			unknown.add(1);
			minUnknown = Math.min(minUnknown, size);
			maxUnknown = Math.max(maxUnknown, size);
		}
	}
	
	public long getMaxForPolarity(String polarity) {
		if (polarity.equals(ArrowPolarity.POSITIVE)) {
			if (maxPositive == Integer.MIN_VALUE)
				return 0;
			else
				return maxPositive;
		} else if (polarity.equals(ArrowPolarity.NEGATIVE)) {
			if (maxNegative == Integer.MIN_VALUE)
				return 0;
			else
				return maxNegative;
		} else {
			if (maxUnknown == Integer.MIN_VALUE)
				return 0;
			else
				return maxUnknown;
		}
	}
	
	public long getMinForPolarity(String polarity) {
		if (polarity.equals(ArrowPolarity.POSITIVE)) {
			if (minPositive == Integer.MAX_VALUE)
				return 0;
			else
				return minPositive;
		} else if (polarity.equals(ArrowPolarity.NEGATIVE)) {
			if (minNegative == Integer.MAX_VALUE)
				return 0;
			else
				return minNegative;
		} else {
			if (minUnknown == Integer.MAX_VALUE)
				return 0;
			else
				return minUnknown;
		}
	}
	
	public int getTotalCount() {
		return getPositiveCount() + getNegativeCount() + getUnknownCount();
	}
	
	public int getPositiveCount() {
		return positive.value();
	}
	
	public int getNegativeCount() {
		return negative.value();
	}
	
	public int getUnknownCount() {
		return unknown.value();
	}

	public long getMinPositive() {
		if (minPositive == Integer.MAX_VALUE)
			return 0;
		else
			return minPositive;
	}

	public long getMaxPositive() {
		if (maxPositive == Integer.MIN_VALUE)
			return 0;
		else
			return maxPositive;
	}

	public long getMinNegative() {
		if (minNegative == Integer.MAX_VALUE)
			return 0;
		else
			return minNegative;
	}

	public long getMaxNegative() {
		if (maxNegative == Integer.MIN_VALUE)
			return 0;
		else
			return maxNegative;
	}

	public long getMinUnknown() {
		if (minUnknown == Integer.MAX_VALUE)
			return 0;
		else
			return minUnknown;
	}

	public long getMaxUnknown() {
		if (maxUnknown == Integer.MIN_VALUE)
			return 0;
		else
			return maxUnknown;
	}

}
