package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Node;

import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

public class VariableLoopTree {
static Logger log = Logger.getLogger(VariableLoopTree.class.getSimpleName());
	
	private Node root;
	private SDObjectManager vensimEquationManager;
	private LoopManager loopManager;
	
	private List<Loop> loops = new ArrayList<Loop>(); // null;
	private Set<String> namedLoops = new LinkedHashSet<String>();
	
	private Set<String> internalLoopsFound = new LinkedHashSet<String>();
	private Set<Loop> internalLoops = new LinkedHashSet<Loop>();	
	
	private boolean includeIVV = false;
	
	private long buildTreeCall = 0;
	
	private int loopCount=0;
	
	private int maxLength = 0;
	private int maxBranchLength = 1000;
	
	private MessageProcessor mf;
	

	
	public VariableLoopTree(LoopManager loopManager, SDObjectManager vem, String variable, boolean includeIVV, int maxBranchLength, MessageProcessor mf) {
		
		this.loopManager = loopManager;
		this.vensimEquationManager = vem;
		this.includeIVV = includeIVV;
		this.maxBranchLength = maxBranchLength+1; // allows for last branch back to root
		this.mf = mf;
		
		generateTree(variable);
		
//		getLoops();
		
//		for (Loop l : internalLoops) {
//		String ordered = l.getOrderedLoopName();
//		if (!namedLoops.contains(ordered)) {
//			loops.add(l);
//		}
//		}
		
		root = null;
	}
	
	private void generateTree(String variable) {
		root = new Node(variable);

		// can there physically be a loop?
		// to be part of a loop, it must be used by someone i.e. link from variable to another
		// and it must use someone else i.e. link from another to variable
		if (vensimEquationManager.getLoopUsedBy(variable).size() > 0 && 
				vensimEquationManager.getLoopUses(variable).size() > 0) {
			List<String> path = new ArrayList<String>();
			path.add(nameToID(variable));
			buildBranch(root, path);
		} else {
			System.out.println(variable+" cannot be part of a loop");
		}
	}
	
	private void logPath(List<String> al) {
		StringBuilder sb = new StringBuilder();
		for (String t : al) {
			sb.append("\t"+t);
		}
		log.debug("LOGPATH"+sb);
	}
	
	private boolean buildBranch /* SecondTry */ (Node node, List<String> pathToNode) {

		// use this condition to prevent full expansion of the tree branch
		// NOT currently, we want to search the entire tree branch and terminate only on loops and leaves
		if (pathToNode.size() > maxBranchLength)
			return false;
		
		// simply print a message so that we know things are progressing
		if (buildTreeCall > 0 && buildTreeCall % 500000 == 0) {
			System.out.println(new Date().toString()+" "+buildTreeCall);
		}
		
		// the variable at the "bud" node. This gets expanded to form a level of the tree branch
		String tailVariable = node.getToken();
		
		// if we are looking at the root variable and it is root but not in root position,
		// we have found a loop
		if (pathToNode.size() > 1 && tailVariable.equals(root.getToken())) {
			loopCount++;
			
			// build the loop based on path to node
			Loop anInternalLoop = generateLoop(pathToNode);
			anInternalLoop.setGeneratingVariable(pathToNode.get(0));
			
			// this is were we limit the size of loops since we are fully searching the structure
			if (anInternalLoop.getLoop().size() <= maxBranchLength) {
				
				// get the "ordered" form of the loop name (name starts with the first variable in alphabetically
				// sorted list of variables in loop.
				String ordered = anInternalLoop.getOrderedNameWithNumbers();

				// this really can't happen, but what the heck...
				// add only if a new loop
				if (!namedLoops.contains(ordered)) {
					namedLoops.add(ordered);
					loops.add(anInternalLoop);
//					System.out.println("VLT Added: "+anInternalLoop.getOrderedNameWithNumbers().hashCode()+" "+anInternalLoop.getOrderedNameWithNumbers());
				} else {
//					System.out.println("DUP! "+anInternalLoop.getOrderedNameWithNumbers());
				}
			}
			// end building this branch and move to next potential branch
			return true;
		}
		
		// MJB Feb 13 2015
		if (!loopManager.isReachable(root.getToken(), tailVariable))
			return false;
		
		// are we at a leaf? Then we can go no farther
		// a leaf is a variable that is not referenced on the RHS of any equation or
		// which has already been processed
		if (vensimEquationManager.getUsedBy(tailVariable) == null ||  /* Loop  */
				vensimEquationManager.getUsedBy(tailVariable).size() == 0) {  /* Loop  */
				return false;
		}
	
		// first time through, we need to create a child
		// from then on, we add to next, so we use loop variable to determine
		// child or next
		
		// Create all nodes at this level
		int loop = 0;
		
		Set<String> ub = new LinkedHashSet<String>(vensimEquationManager.getUsedBy(tailVariable));		 /* Loop  */
		
		for (String headVariableName : ub) {
			String headVariableID = nameToID(headVariableName);	
			// this is a hack to avoid using equation calling initial as part of a loop
			if (vensimEquationManager.getEquations().get(headVariableID).isInitial())
				continue;
//			vensimEquationManager.getLoopUsedBy(currentVariable).remove(toVariableID);
			
			if (!loopManager.isReachable(root.getToken(), headVariableName))
				continue;

			if (!includeIVV && loopManager.isInitialValueVariableUse(tailVariable, headVariableName)) {
				continue;
			}
			
			// have we encountered a loop other than the root?
			// if so, do not proceed
			if (!headVariableName.equals(root.getToken()) && pathToNode.contains(headVariableID)) {
				
				// node is current variable
				// toVariableID is the variable that completes loop
				
//				Loop anInternalLoop = generateInternalLoop(node, toVariableID);
//				
//	
//				if (anInternalLoop.getLoop().size() <= maxBranchLength) {
//					String ordered = anInternalLoop.getOrderedLoopName();
//
//					if (!internalLoopsFound.contains(ordered)) {
//						loopManager.getFoundOtherLoop().add(1);
//						internalLoopsFound.add(ordered);
//						internalLoops.add(anInternalLoop);
//					}
//				}
				continue;
			}

			Node newNode = new Node(headVariableName);
			newNode.setParent(node);
			if (loop++ == 0) {
				node.setChild(newNode);
			} else {
				Node child = node.getChild();
				while (child.getNext() != null)
					child = child.getNext();
				child.setNext(newNode);
				newNode.setPrevious(child);
			}
		}
		
		Node aNode = node.getChild();
		while (aNode != null) {
			List<String> nextPath = new ArrayList<String>(pathToNode);
				nextPath.add(aNode.getToken());
				buildTreeCall++;
				
//				loopManager.getVariablesNotVisited().remove(aNode.getToken());

				if (!buildBranch(aNode, nextPath)) {
					// we did not generate a loop
					// cleanup -- need to remove aNode from the structure
					// it can be either the child of the parent or in the parent's child's next stream
					Node nextNode = aNode.getNext();
					Node.pruneNode(aNode);
					aNode = nextNode;
				} else {
					
					aNode.setDoNotPrune(true);
					aNode = aNode.getNext();
				}
		}
		
		// processed this level, let's prune non loops
//		System.out.println("isDNP: "+node.isDoNotPrune()+": "+buildTreeCall+" node:"+node.getToken()+" p.size: "+pathToNode.size());

		return node.isDoNotPrune();
	}
	
//	private boolean buildTreeExperiment(Node node, List<String> pathToNode) {
//		
//		if (pathToNode.size() > maxBranchLength)
//			return false;
//		
////		loopManager.getVariablesNotVisited().remove(node.getToken());
//		
//		
//		if (buildTreeCall > 0 && buildTreeCall % 1000000 == 0) {
//			System.out.println(new Date().toString()+" "+buildTreeCall);
//		}
//		
//		String currentVariable = node.getToken();
//		
//		// if we are looking at variable and it is root but not in root position we have found a loop
//		if (pathToNode.size() > 1 && currentVariable.equals(root.getToken())) {
//			loopCount++;
//			return true;
//		}
//		
//		// first check if is referenced by any other equation
//		// used by represents the variable pointed to from current
//		// if we don't point to anyone, we are done
//		if (vensimEquationManager.getUsedBy(currentVariable) == null || 
//				vensimEquationManager.getUsedBy(currentVariable).size() == 0) {
//			// this can't be a loop since the above code would have detected it
//				return false;
//		}
//	
//		// first time through, we need to create a child
//		// from then on, we add to next
//		
//		// Create all nodes at this level
//		int loop = 0;
//		
//		for (String toVariableName : vensimEquationManager.getUsedBy(currentVariable)) {
//			String toVariableID = nameToID(toVariableName);
//
//			if (!includeIVV && loopManager.isInitialValueVariableUse(currentVariable, toVariableName))
//				continue;
//
//			// have we encountered a loop other than the root?
//			// if so, do not proceed
//			if (!toVariableName.equals(root.getToken()) && pathToNode.contains(toVariableID)) {
//				
//				// node is current variable
//				// toVariableID is the variable that completes loop
//				
//				Loop anInternalLoop = generateInternalLoop(node, toVariableID);
//				if (anInternalLoop.getLoop().size() <= maxBranchLength) {
//					String ordered = anInternalLoop.getNumberedOrderedLoopName();
//					if (!internalLoopsFound.contains(ordered)) {
//						loopManager.getFoundOtherLoop().add(1);
//						internalLoopsFound.add(ordered);
//						internalLoops.add(anInternalLoop);
//					}
//				}
//				continue;
//			}
//
//			Node newNode = new Node(toVariableName);
//			newNode.setParent(node);
//			if (loop++ == 0) {
//				node.setChild(newNode);
//			} else {
//				Node child = node.getChild();
//				while (child.getNext() != null)
//					child = child.getNext();
//				child.setNext(newNode);
//				newNode.setPrevious(child);
//			}
//		}
//		
//		Node aNode = node.getChild();
//		while (aNode != null) {
//			List<String> nextPath = new ArrayList<String>(pathToNode);
//				nextPath.add(aNode.getToken());
//				buildTreeCall++;
//				
////				loopManager.getVariablesNotVisited().remove(aNode.getToken());
//
//				if (!buildBranch(aNode, nextPath)) {
//					// we did not generate a loop
//					// cleanup -- need to remove aNode from the structure
//					// it can be either the child of the parent or in the parent's child's next stream
//					Node nextNode = aNode.getNext();
//					Node.pruneNode(aNode);
//					aNode = nextNode;
//				} else {
//					
//					aNode.setDoNotPrune(true);
//					aNode = aNode.getNext();
//				}
//		}
//		
//		// processed this level, let's prune non loops
////		System.out.println("isDNP: "+node.isDoNotPrune()+": "+buildTreeCall+" node:"+node.getToken()+" p.size: "+pathToNode.size());
//		return node.isDoNotPrune();
//	}
	
	private Loop generateInternalLoop(Node node, String toVariableID) {
		
		List<String> varsInLoop = new ArrayList<String>();
		// first locate beginning of loop
		Node aNode = node;
		
		varsInLoop.add(toVariableID);
		
		while (!aNode.getToken().equals(toVariableID)) {
			varsInLoop.add(aNode.getToken());
			aNode = aNode.getParent();
		}
		
		varsInLoop.add(toVariableID);
		
		Loop loop = new Loop();
		for (int i = varsInLoop.size()-1; i >= 0; i--) {
			loop.add(varsInLoop.get(i));
		}
		
		loop.setNames();
		
		return loop;
		
		
	}
	
	private Loop generateLoop(List<String> path) {
		
		Loop loop = new Loop();
		for (String p : path) {
			loop.add(p);
		}
		
		loop.setNames();
		
		return loop;
		
		
	}
	
//	private boolean buildBranchORIG(Node node, List<String> path) {
//		
//		String currentVariable = node.getToken();
//		// first check if is referenced by any other equation
//		// used by represents the variable pointed to from current
//		// if we don't point to anyone, we are done
//		if (vensimEquationManager.getUsedBy(currentVariable) == null || 
//				vensimEquationManager.getUsedBy(currentVariable).size() == 0) {
//			return false;
//		}
//		// first time through, we need to create a child
//		// from then on, we add to next
//		int loop = 0;
//		for (String toVariableName : vensimEquationManager.getUsedBy(currentVariable)) {
//			String toVariableID = nameToID(toVariableName);
//			
//			
//			// never include ivvs
//			if (loopManager.isInitialValueVariableUse(currentVariable, toVariableName))
//				continue;
//			List<String> nextPath = new ArrayList<String>(path);
//			// have we encountered a loop other than the root?
//			// if so, do not proceed
////			if (!toVariableName.equals(root.getToken()) && nextPath.contains(toVariableID)) {
////				continue;
////			}
//			nextPath.add(toVariableID);
//			buildTreeCall++;
////			log.debug("BT: "+toVariable+" call: "+buildTreeCall+" Path Length: "+nextPath.size());
//			
//			Node newNode = new Node(toVariableName);
//			newNode.setParent(node);
//			if (loop++ == 0) {
//				node.setChild(newNode);
//			} else {
//				Node child = node.getChild();
//				while (child.getNext() != null)
//					child = child.getNext();
//				child.setNext(newNode);
//				newNode.setPrevious(child);
//			}
//			
////			log.debug("BT: "+toVariableName+" TreeSize: "+TreeTraversal.treeSize(root));
//			
//			if (!toVariableName.equals(root.getToken()))
//				if (!buildBranch(newNode, nextPath)) {
//					if (newNode.getParent().getChild().equals(this)) {
//						// this is parent child, rather than parent child next
//					}
//				}
//		}
//		return true;
//	}

	public Node getRoot() {
		return root;
	}
	
	public List<Loop> getChains() {
		List<Loop> chains = new ArrayList<Loop>();

		// note that we will save "non-loop" loops in case we would ever need "paths"
		// start with the first loop

		Loop loop = new Loop();
		loop.add(root.getToken());

		expand(chains, loop, root, false);
		namedLoops.clear();
		return chains;
	}
	
	public List<Loop> getLoops() {
//		if (loops == null) {
//			loops = new ArrayList<Loop>();
//
//			Loop loop = new Loop();
//			loop.add(root.getToken());
//
//			expand(loops, loop, root, true);
//
//			for (Loop l : loops) {
//				if (l.getSize() > maxLength)
//					maxLength = l.getSize();
//			}
//
//			namedLoops.clear();
//		}
		
		System.out.println("Found Loops Count: "+loops.size());
		
//		for (Loop l : loops) {
//			System.out.println(l.getOrderedNameWithNumbers().hashCode()+" "+l.getOrderedNameWithNumbers());
//		}
		
		return loops;
	}
	
	private void expand(List<Loop> loops, Loop loop, Node node, boolean loopMode) {
		// node will always be non-null
		// node will always be the last element in loop

		// if there isn't a child to this node, then we've reached the end
		// save the loop and return
		if (node.getChild() == null) {
			if (loopMode) {
				if (loop.isLoop()) {
					if (!namedLoops.contains(loop.getName())) {
						namedLoops.add(loop.getName());
						loops.add(loop);
					} else {
						log.debug("Repeat: length "+loop.getName().length());
					}
				}
			} else {
				if (!namedLoops.contains(loop.getName())) {
					namedLoops.add(loop.getName());
					loops.add(loop);
				} else {
					log.debug("Repeat: length "+loop.getName().length());
				}
			}
			return;
		}

		// there is a child
		Node child = node.getChild();
		// verify usedBy
		Set<String> used = vensimEquationManager.getUsedBy(loop.getLast());
		if (!used.contains(child.getToken())) {
			log.debug("Child UsedBy Err: "+loop.getLast()+" -> "+child.getToken());
		}

		Loop childLoop = new Loop(loop);

		childLoop.add(child.getToken());
		expand(loops, childLoop, child, loopMode);


		// if there is not a next node, then we are finished and return
		if(child.getNext() == null) {
			return;
		}

		// we have a next, so we must clone the loop to this point, add next elements
		// and then expand

		Node next = child.getNext();
		while (next != null) {


			Loop newLoop = new Loop(loop);
			newLoop.setGeneratingVariable(root.getToken());
			// note: this newLoop contains the child as the last element, so it must be removed
			String last1 = newLoop.getLast();
			//			newLoop.getLoop().remove(newLoop.getLoop().size()-1);

			//			if (newLoop.getLoop().contains(next.getToken())) {
			//				log.debug("Expand: already in path: "+next.getToken());
			//			}
			// verify usedBy
			used = vensimEquationManager.getUsedBy(newLoop.getLast());
			if (!used.contains(next.getToken())) {
				//				log.debug("Next UsedBy Err: "+newLoop.getLast()+" -> "+next.getToken()+" last1 = "+last1);
			}
			newLoop.add(next.getToken());
			expand(loops, newLoop, next, loopMode);
			next = next.getNext();
		}

	}

	public int getMaxLength() {
		return maxLength;
	}
	
	private String nameToID(String name) {
		return name;
//		return InformationManagers.getInstance().getEquationManager().getVariableIDForName(name);
	}
	
	private String idToName(String id) {
//		return id;
		return InformationManagers.getInstance().getEquationManager().getVariableNameForID(id);
	}

}
