package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class VariableLoops {
	
	private static final String BREAK = "</br>\n";
	private static final String COLUMN_START = "<td>\n";
	private static final String COLUMN_END = "</td>\n";
	
	private static final String ROW_START = "<tr>\n";
	private static final String ROW_END = "</tr>\n";
	
	private static final String TABLE_START = "<br/><a name='VariableLoops' /><a href='#top'>Top</a><h3>Variable Loops</h3>"
			+ "<table class='results'>\n";
	private static final String TABLE_END = "</table>\n";
	
	protected static final String COLUMN_HEADERS = "<td style='vertical-align:bottom; color:Blue'><b><center>$Variable$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Num Loops$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$LoopDetail$</center></b></td>\n"; // +
//			"<td style='vertical-align:bottom; color:Blue'><b><center>$Long Loops$</center></b></td>\n";
	
	private Map<String, VariableLoop> variableLoops = new LinkedHashMap<String, VariableLoop>();
	
	public VariableLoops() {
		
	}
	
	public void put(VariableLoop vl) {
		variableLoops.put(vl.getVariable(), vl);
	}
	
	public int size() {
		return variableLoops.size();
	}
	
	public void writeVariableLoops() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String file = /*mm.getModelName()+*/ "_variableLoops.tab";
		
		BufferedWriter bw = Utilities.openFileForWriting(
				InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
					
		
		List<String> keys = orderVariableLoops(new ArrayList<String>(variableLoops.keySet()));
		try {
			bw.append("Var A\tNum Loops\n");
			for (String key : keys) {
				VariableLoop vr = variableLoops.get(key);
				bw.append(vr.getVariable()+"\t");
				bw.append(vr.getNumLoops()+"\t");
			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public List<String> orderVariableLoops(List<String> vars) {
//		List<String> al = new ArrayList<String>();
		Collections.sort(vars, new Comparator<String>() {
			@Override
			public int compare(String var1, String var2) {

				VariableLoop vl1 = variableLoops.get(var1);
				VariableLoop vl2 = variableLoops.get(var2);

				if (vl1.getNumLoops() != vl2.getNumLoops()) {

					return vl1.getNumLoops() < vl2.getNumLoops() ? 1 : -1;
				}


				if (!vl1.getVariable().equals(vl2.getVariable())) {
					return vl1.getVariable().compareToIgnoreCase(vl2.getVariable());
				}

				return 0;
			}
		});
		return vars;
	}
	
	public String getXML() {
		StringBuilder sb = new StringBuilder();
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<String> keys = orderVariableLoops(new ArrayList<String>(variableLoops.keySet()));
		
		LoopManager loopManager = InformationManagers.getInstance().getAnalysisManager().getLoopManager();
		
		sb.append(TABLE_START);
		sb.append(ROW_START);
		sb.append(COLUMN_HEADERS.replace("$Variable$", language.getTextHtml(StringConstants.VARIABLE))
//				.replace("$Short Loops$", language.getTextHtml(StringConstants.SHORT_LOOPS))
				.replace("$LoopDetail$", language.getTextHtml(StringConstants.LOOP_DETAIL))
				.replace("$Num Loops$", language.getTextHtml(StringConstants.NUMBER_OF_LOOPS)));
		sb.append(ROW_END);
			for (String key : keys) {
				sb.append(variableLoops.get(key).getRow(loopManager.getLongestLoopLength()));
			}
			sb.append(TABLE_END);
		return sb.toString();
	}

	public Map<String, VariableLoop> getVariableLoops() {
		return variableLoops;
	}

}
