package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class VariableRelationships {
	
	private static final String BREAK = "</br>\n";
	private static final String COLUMN_START = "<td>\n";
	private static final String COLUMN_END = "</td>\n";
	
	private static final String ROW_START = "<tr>\n";
	private static final String ROW_END = "</tr>\n";
	
	private static final String TABLE_START = "<br/><a name='VariableRelationships' /><a href='#top'>Top</a><h3>Variable Relationships</h3>"
			+ "<table class='results'>\n";
	private static final String TABLE_END = "</table>\n";
	
	protected static final String COLUMN_HEADERS = 
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Variable A$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Variable B$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Loops$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Min AB$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Max AB$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Min BA$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$Max BA$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$LoopDetail$</center></b></td>\n"; // +
//			"<td style='vertical-align:bottom; color:Blue'><b><center>$Long Loops$</center></b></td>\n";
	
	private Map<String, VariableRelationship> variableRelationships = new LinkedHashMap<String, VariableRelationship>();
	
	public VariableRelationships() {
		
	}
	
	public void put(VariableRelationship vr) {
		variableRelationships.put(vr.getVariableA()+"#"+vr.getVariableB(), vr);
	}
	
	public boolean pairAlreadyProcessed(String a, String b) {
		return variableRelationships.containsKey(a+"#"+b) || 
				variableRelationships.containsKey(b+"#"+a);
	}
	
	public VariableRelationship getVariableRelationship(String a, String b) {
		if (variableRelationships.containsKey(a+"#"+b)) {
			return variableRelationships.get(asKey(a,b));
		} else if (variableRelationships.containsKey(b+"#"+a)) {
			return variableRelationships.get(asKey(b,a));
		} else {
			VariableRelationship vr = new VariableRelationship(a,b);
			variableRelationships.put(asKey(a,b), vr);
			return vr;
		}
		
	}
	
	public String asKey(String a, String b) {
			return a+"#"+b;
	}
	
	public VariableRelationship getVariableRelationship(String key) {
		return variableRelationships.get(key);
	}
	
	public void writeVariableRelationships() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		String file = /*mm.getModelName()+*/ "_variableRelationships.tab";
		BufferedWriter bw = Utilities.openFileForWriting(
				InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
		
		List<String> keys = orderVariableRelationships(
				new ArrayList<String>(variableRelationships.keySet()));
		try {
			bw.append("Var A\tVar B\tNum Loops\tMin AB\tMax AB\tMin BA\tMax BA\n");
			for (String key : keys) {
				VariableRelationship vr = variableRelationships.get(key);
				bw.append(vr.getVariableA()+"\t");
				bw.append(vr.getVariableB()+"\t");
				bw.append(vr.getNumLoops()+"\t");
				bw.append(vr.getMinAB()+"\t");
				bw.append(vr.getMaxAB()+"\t");
				bw.append(vr.getMinBA()+"\t");
				bw.append(vr.getMaxBA()+"\n");
			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public List<String> orderVariableRelationships(List<String> vars) {
//		List<String> al = new ArrayList<String>();
		Collections.sort(vars, new Comparator<String>() {
			@Override
			public int compare(String var1, String var2) {

				VariableRelationship vr1 = variableRelationships.get(var1);
				VariableRelationship vr2 = variableRelationships.get(var2);

				if (vr1.getNumLoops() != vr2.getNumLoops()) {

					return vr1.getNumLoops() < vr2.getNumLoops() ? 1 : -1;
				}


				if (!vr1.getVariableA().equals(vr2.getVariableA())) {
					return vr1.getVariableA().compareToIgnoreCase(vr2.getVariableA());
				}

				if (!vr1.getVariableB().equals(vr2.getVariableB())) {
					return vr1.getVariableB().compareToIgnoreCase(vr2.getVariableB());
				}

				return 0;
			}
		});
		return vars;
	}
	
	
	public String getXML() {
		StringBuilder sb = new StringBuilder();

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		LoopManager loopManager = InformationManagers.getInstance().getAnalysisManager().getLoopManager();

//		List<String> keys = Utilities.sortIgnoreCase(variableRelationships.keySet());
		List<String> keys = orderVariableRelationships(
				new ArrayList<String>(variableRelationships.keySet()));

		//		sb.append(HTMLConstants.COLUMN_HEADER.replace(HTMLConstants.COLUMN_NAME, 
		//				language.getTextHtml(StringConstants.EXOGENOUS_UNCONNECTED)));
		//		
		sb.append(TABLE_START);
		sb.append(ROW_START);
		sb.append(COLUMN_HEADERS
				.replace("$Variable A$", language.getTextHtml(StringConstants.VARIABLE_A))
				.replace("$Variable B$", language.getTextHtml(StringConstants.VARIABLE_B))
				.replace("$Min AB$", language.getTextHtml(StringConstants.MIN_AB))
				.replace("$Max AB$", language.getTextHtml(StringConstants.MAX_AB))
				.replace("$Min BA$", language.getTextHtml(StringConstants.MIN_BA))
				.replace("$Max BA$", language.getTextHtml(StringConstants.MAX_BA))
//				.replace("$Short Loops$", language.getTextHtml(StringConstants.SHORT_LOOPS))
				.replace("$LoopDetail$", language.getTextHtml(StringConstants.LOOP_DETAIL))
				.replace("$Loops$", language.getTextHtml(StringConstants.NUMBER_OF_LOOPS))
				);


		sb.append(ROW_END);
		for (String key : keys) {
			sb.append(variableRelationships.get(key).getRow(loopManager.getLongestLoopLength()));
		}
		sb.append(TABLE_END);
		return sb.toString();
	}

}
