package gov.anl.gss.sdm.server.model.analysis.loops;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class VensimLoopAnalyzer {
	
	private String fileIn;
	private String fileOutOrdered;
	private String fileOutSorted;
	private List<String> loopElements;
	private List<String> sortedLoopNames = new ArrayList<String>();
	private List<String> orderedLoopNames = new ArrayList<String>();
	
	public VensimLoopAnalyzer(String fileIn, String fileOutOrdered, String fileOutSorted) {
		this.fileIn = fileIn;
		this.fileOutOrdered = fileOutOrdered;
		this.fileOutSorted = fileOutSorted;
	}
	
	public static void main(String[] args) {

		VensimLoopAnalyzer analyzer = new VensimLoopAnalyzer(args[0], args[1], args[2]);
		analyzer.execute();
	}
	
	private void execute() {
		
		loopElements = Utilities.getAsList(fileIn);
		
		int loopNum = 0;
		List<String> loop = null;
		for (String element : loopElements) {
			if (element.startsWith("Loop Number")) {
				if (loopNum++ > 0) {
					generateName(loop, orderedLoopNames, sortedLoopNames);
				}
				loop = new ArrayList<String>();
			} else {
				loop.add(element.trim().replace(" ", ""));
				
			}
		}
		generateName(loop, orderedLoopNames, sortedLoopNames);
		
		writeFile(fileOutOrdered, orderedLoopNames);
		writeFile(fileOutSorted, sortedLoopNames);
	}
	
	private void generateName(List<String> loop, List<String> orderedLoopNames, List<String> sortededLoopNames) {
		StringBuilder sbOrdered = new StringBuilder();
		StringBuilder sbSorted = new StringBuilder();
		
		for (int i = 0; i < loop.size(); i++) {
			if (i > 0)
				sbOrdered.append("#");
			sbOrdered.append(loop.get(i));
		}
		orderedLoopNames.add(sbOrdered.toString());
		
		Collections.sort(loop);
		for (int i = 0; i < loop.size(); i++) {
			if (i > 0)
				sbSorted.append("#");
			sbSorted.append(loop.get(i));
		}
		sortedLoopNames.add(sbSorted.toString());
		
		return;
	}
	
	private void writeFile(String file, List<String> al) {
		BufferedWriter bw = Utilities.openFileForWriting(
				InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), file);
		try {
			for (String s : al) {
				bw.append(s+"\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
