package gov.anl.gss.sdm.server.model.analysis.micmac;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;

import java.util.List;

public class DataPointProximitySummary implements Comparable<DataPointProximitySummary>{
	
	int ptr;

	double[] closest;
	List<Integer>[] inSlot;
	int[] count;
	List<DataPointProximity> ordered;
	
	int emptySlots = -1;
	double averageClosestDistance = -1.0;
	
	public DataPointProximitySummary(int ptr) {
		this.ptr = ptr;
	}
	
	public String toString() {
		return "ptr: "+ptr+", emptySlots: "+emptySlots+", average closest: "+NumberFormatter.formatForSmall(averageClosestDistance);
	}
	
	public void initialize() {
		getEmptySlots();
		getAverageClosestDistance();
		
	}
	
	public double getAverageClosestDistance() {
		if (averageClosestDistance == -1.0) {
			averageClosestDistance = 0.0;
			for (int i = 0; i < count.length; i++) {
				if (count[i] > 0 && closest[i] > 0.0) {
					averageClosestDistance += closest[i];
				}
			}
			averageClosestDistance = averageClosestDistance / (16 - getEmptySlots());	
		}
		return averageClosestDistance;
	}

	public int getEmptySlots() {
		if (emptySlots == -1) {
			emptySlots = 0;
			for (int i = 0; i < count.length; i++) {
				if (count[i] == 0)
					emptySlots++;
			}
				
		}
		return emptySlots;
	}

	public double[] getClosest() {
		return closest;
	}

	public void setClosest(double[] closest) {
		this.closest = closest;
	}

	public List<Integer>[] getInSlot() {
		return inSlot;
	}

	public void setInSlot(List<Integer>[] inSlot) {
		this.inSlot = inSlot;
	}

	public int[] getCount() {
		return count;
	}

	public void setCount(int[] count) {
		this.count = count;
	}

	public List<DataPointProximity> getOrdered() {
		return ordered;
	}

	public void setOrdered(List<DataPointProximity> ordered) {
		this.ordered = ordered;
	}

	public int getPtr() {
		return ptr;
	}

	@Override
	public int compareTo(DataPointProximitySummary arg0) {
		// ascending sort on emptySlots
		// then ascending sort on averageClosestDistance
		
		if (emptySlots < arg0.getEmptySlots()) {
			return -1;
		} else if (emptySlots > arg0.getEmptySlots()) {
			return 1;
		} else if (averageClosestDistance < arg0.getAverageClosestDistance()) {
			return -1;
		} else if (averageClosestDistance > arg0.getAverageClosestDistance()) {
			return 1;
		}
		return 0;
	}
}
