package gov.anl.gss.sdm.server.model.analysis.micmac;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import org.apache.commons.math3.linear.BlockRealMatrix;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.DataFormat;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

public class MicMacExcelWriter {
	
	MicMacMinimumMemory micmac;
	String file;
	Workbook workbook;
	Font headerFont;
	CellStyle headerCellStyle;
	CellStyle normalizedCellStyle;
	CellStyle numericCellStyle;
	
	MessageProcessor mf;
	
	public MicMacExcelWriter(MicMacMinimumMemory micmac, MessageProcessor mf) {

		
		this.micmac = micmac;
		this.mf = mf;
		try {
				workbook = new XSSFWorkbook();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public boolean writeFile(String file) {
		
		File aFile = new File(file);
        if (aFile.getParentFile() != null)
        	aFile.getParentFile().mkdirs();

        // Create a Font for styling header cells
        headerFont = workbook.createFont();
        headerFont.setBold(true);
//        headerFont.setFontHeightInPoints((short) 14);
        headerFont.setColor(IndexedColors.BLACK.getIndex());

        // Create a CellStyle with the font
        headerCellStyle = workbook.createCellStyle();
        headerCellStyle.setFont(headerFont);
        headerCellStyle.setAlignment(HorizontalAlignment.CENTER);
        
        DataFormat format = workbook.createDataFormat();
        normalizedCellStyle = workbook.createCellStyle();
        normalizedCellStyle.setDataFormat(format.getFormat("0.00"));
        
        numericCellStyle = workbook.createCellStyle();
        numericCellStyle.setDataFormat(format.getFormat("#,##0"));
		
        
        for (int order = 0; order < MicMac.NTH_ORDER_RESULTS; order++) {
        	
        	if (micmac.getSavedMatricies()[order] == null)
        		break;
        	
        	BlockRealMatrix m = micmac.getSavedMatricies()[order];
        	writeMatrixToSheet(MicMac.tags[order], m);
        }
        
        writeInfluenceExposureToSheet("Tabulated Results", micmac.getInfluence(), micmac.getExposure(), numericCellStyle);
        writeInfluenceExposureToSheet("Normalized Results", micmac.getInfluenceNormalized(), micmac.getExposureNormalized(), normalizedCellStyle);


        // Write the output to a file
        try {
			FileOutputStream fileOut = new FileOutputStream(file);
			workbook.write(fileOut);
			fileOut.close();

			// Closing the workbook
			workbook.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		
		return true;
	}
	
	public boolean writeMatrixToSheet(String sheetName, BlockRealMatrix matrix) {

		Sheet sheet = workbook.createSheet(sheetName);
		// Create a Row
		Row headerRow = sheet.createRow(0);

		Cell cell = headerRow.createCell(0);
		cell.setCellValue("Variables");
		cell.setCellStyle(headerCellStyle);
		
		int createdCell = 1;

		double[][] data = matrix.getData();
		for (int r = 0; r < data.length; r++) {
			String var = micmac.idVar(r); // InformationManagers.getInstance().getEquationManager().getVariableNameForID(r);
			if (var.startsWith("C_L_O_U_D"))
				continue;
				if (var.startsWith("NoNamefor"))
				continue;
			if (Parser.isSystemVariable(var) || Parser.isNaReplacement(var))
				continue;
			cell = headerRow.createCell(createdCell++);
			cell.setCellValue(var);
			cell.setCellStyle(headerCellStyle);
		}

		int createdRow = 1;
		for (int r = 0; r < data.length; r++) {
			
			String var = micmac.idVar(r); // InformationManagers.getInstance().getEquationManager().getVariableNameForID(r);
			if (Parser.isSystemVariable(var) || Parser.isNaReplacement(var))
				continue;
			if (var.startsWith("C_L_O_U_D"))
				continue;
				if (var.startsWith("NoNamefor"))
				continue;
			Row aRow = sheet.createRow(createdRow++);
			cell = aRow.createCell(0);
			cell.setCellValue(var);
			cell.setCellStyle(headerCellStyle);

			for (int c = 0; c < data.length; c++) {
				cell = aRow.createCell(c+1);
				cell.setCellValue(data[r][c]);
				cell.setCellStyle(numericCellStyle);
			}
		}

		// Resize all columns to fit the content size
		for(int i = 0; i < createdCell; i++) {
			sheet.autoSizeColumn(i);
		}
		return true;
	}
	
	public boolean writeInfluenceExposureToSheet(String sheetName, double[][] influence, double[][] exposure,
			CellStyle cellStyle) {
		
		String[] forMerge = {"B1:C1", "D1:E1","F1:G1","H1:I1","J1:K1","L1:M1",
				"N1:O1", "P1:Q1","R1:S1","T1:U1","V1:W1","X1:Y1"};
		

		
//		sheet.addMergedRegion(new CellRangeAddress(startRowIndx, endRowIndx, startColIndx,endColIndx));
//		my_sheet.addMergedRegion(CellRangeAddress.valueOf("B2:D5"));
		
//		setAlignment(CellStyle.ALIGN_RIGHT);
		
		Sheet sheet = workbook.createSheet(sheetName);
		
		// Create a Row
		Row headerRow = sheet.createRow(0);

		int createdCell = 1;
		Cell cell = null;
		
		for (int r = 0; r < micmac.getStableIterations(); /*MicMac.NTH_ORDER_RESULTS;*/ r++) {
			cell = headerRow.createCell(createdCell);
			cell.setCellValue(MicMac.tags[r]);
			cell.setCellStyle(headerCellStyle);
			sheet.addMergedRegion(CellRangeAddress.valueOf(forMerge[r]));
			createdCell += 2;
		}
		
		Row headerRow2 = sheet.createRow(1);

		cell = headerRow2.createCell(0);
		cell.setCellValue("Variables");
		cell.setCellStyle(headerCellStyle);

		int created = 1;
		for (int r = 0; r < micmac.getStableIterations(); /*MicMac.NTH_ORDER_RESULTS;*/ r++) {
			
			cell = headerRow2.createCell(created++);
			cell.setCellValue("Influence");
			cell.setCellStyle(headerCellStyle);
			
			cell = headerRow2.createCell(created++);
			cell.setCellValue("Exposure");
			cell.setCellStyle(headerCellStyle);
		}

		int createdRow = 2;
		createdCell = 1;
		for (int r = 0; r < influence.length; r++) {
			
			String var = micmac.idVar(r); // InformationManagers.getInstance().getEquationManager().getVariableNameForID(r);
			if (Parser.isSystemVariable(var) || Parser.isNaReplacement(var))
				continue;
			if (var.startsWith("C_L_O_U_D"))
				continue;
				if (var.startsWith("NoNamefor"))
				continue;
			Row aRow = sheet.createRow(createdRow++);
			cell = aRow.createCell(0);
			cell.setCellValue(var);
			cell.setCellStyle(headerCellStyle);

			createdCell = 1;
			for (int c = 0; c < micmac.getStableIterations(); /* MicMac.NTH_ORDER_RESULTS;*/ c++) {
				
				cell = aRow.createCell(createdCell++);
				cell.setCellValue(influence[r][c]);
				cell.setCellStyle(cellStyle);
				
				cell = aRow.createCell(createdCell++);
				cell.setCellValue(exposure[r][c]);
				cell.setCellStyle(cellStyle);
			}
		}

		// Resize all columns to fit the content size
		for(int i = 0; i < createdCell; i++) {
			sheet.autoSizeColumn(i);
		}
		
		return true;
	}

}
