package gov.anl.gss.sdm.server.model.analysis.micmac;

import gov.anl.gss.sdm.server.model.analysis.EquationManager;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.visualization.SFEXYPointerAnnotationMetaData;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.math3.exception.OutOfRangeException;
import org.apache.commons.math3.linear.BlockRealMatrix;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

public class MicMacMinimumMemory {
	
	public static final int NTH_ORDER_ITERATIONS = 20;
	public static final int NTH_ORDER_RESULTS = NTH_ORDER_ITERATIONS + 1;
	
	public static final String[] tags = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "10",
		"11", "12", "13", "14", "15", "16", "17", "18", "19", "20"};
	
	private SFEXYPointerAnnotationMetaData meta;
	
	private int stableIterations = 0;
	
	private static double CLOSE_ENOUGH = .01;
	
	BlockRealMatrix matrix;
	int numEquations;
//	BlockRealMatrix[] matrixArray = new BlockRealMatrix[NTH_ORDER_RESULTS];
	double[][] influence;
	double[][] exposure;
	double[][] influenceNormalized;
	double[][] exposureNormalized;
	
	BlockRealMatrix[] savedMatricies = new BlockRealMatrix[20];
	
	List<double[]> inf = new ArrayList<double[]>();
	List<double[]> exp = new ArrayList<double[]>();
	List<double[]> infN = new ArrayList<double[]>();
	List<double[]> expN = new ArrayList<double[]>();
	
	Map<String, Integer> varToID = new LinkedHashMap<String, Integer>();
	Map<Integer, String> idToVar = new LinkedHashMap<Integer, String>();
	
	public MicMacMinimumMemory(EquationManager eqnMgr) {
		
		numEquations = initializeVars(eqnMgr);
		matrix = new BlockRealMatrix(numEquations, numEquations);
		influence = new double[numEquations][NTH_ORDER_RESULTS];
		exposure = new double[numEquations][NTH_ORDER_RESULTS];
		influenceNormalized = new double[numEquations][NTH_ORDER_RESULTS];
		exposureNormalized = new double[numEquations][NTH_ORDER_RESULTS];
	}
	
	public MicMacMinimumMemory(int numEquations) {
		matrix = new BlockRealMatrix(numEquations, numEquations);
		influence = new double[numEquations][NTH_ORDER_RESULTS];
		exposure = new double[numEquations][NTH_ORDER_RESULTS];
		influenceNormalized = new double[numEquations][NTH_ORDER_RESULTS];
		exposureNormalized = new double[numEquations][NTH_ORDER_RESULTS];
		this.numEquations = numEquations;
	}
	
	private int initializeVars(EquationManager eqnMgr) {
		
	Set<String> vars = eqnMgr.getVariables();
	
//	Utilities.printToConsole(vars);

		int row = 0;
		
		for (String variable : Utilities.ordered(vars)) {
			if (variable.startsWith("C_L_O_U_D")) {
//				System.out.println("MICMAC discarding "+variable);
				continue;
			}
				if (variable.startsWith("NoNamefor")) {
//					System.out.println("MICMAC discarding "+variable);
				continue;
				}
				if (Parser.isSystemVariable(variable)) {
//					System.out.println("MICMAC discarding "+variable);
					continue;
				}
				if (Parser.isNaReplacement(variable)) {
//					System.out.println("MICMAC discarding "+variable);
					continue;
				}
				
				varToID.put(variable, row);
				idToVar.put(row, variable);
				row++;
		}
		System.out.println(varToID.keySet());
//		Utilities.printToConsole(varToID.keySet());
		
		return row;
	}
	
	public void runUntilStable(int minIterations, int maxIterations) {
		initialize();
		createFirstOrder();
		
		int save = 0;
//		savedMatricies[save++] = matrix.copy();

		stableIterations = 0;
		System.out.println("runUntilStable");
		while (true) {

			multiply(stableIterations);
			savedMatricies[save++] = matrix.copy();
			tabulateExposure(stableIterations);
			tabulateInfluence(stableIterations);
			normalizeInfluence(stableIterations);
			nomalizeExposure(stableIterations);
			if (stableIterations >= minIterations && stable(stableIterations))
				break;
			if (stableIterations+1 >=  maxIterations)
				break;
			stableIterations++;
			System.out.println("Interation: "+stableIterations);
		}
	}
	
	public BlockRealMatrix[] getSavedMatricies() {
		return savedMatricies;
	}

	private boolean stable(int iteration) {
		
//		if (inf.size() < 2)
//			return false;
//		
//		double[] last = infN.get(inf.size()-1);
//		double[] prev = infN.get(inf.size()-2);
		
//		for (int i = 0; i < last.length; i++) {
//			if (Math.abs(last[i] - prev[i]) > CLOSE_ENOUGH) {
//				return false;
//			}
//		}
		
//		System.out.println("stable? "+iteration);
		if (iteration < 2)
			return false;
		for (int i = 0; i < influenceNormalized.length; i++) {
			
			
//			System.out.println(i+" influence diff "+Math.abs(influenceNormalized[i][iteration] - influenceNormalized[i][iteration-1])+" comp to "+CLOSE_ENOUGH);
//			System.out.println(i+" exposure diff "+Math.abs(exposureNormalized[i][iteration] - exposureNormalized[i][iteration-1])+" comp to "+CLOSE_ENOUGH);
			
			if (Math.abs(influenceNormalized[i][iteration] - influenceNormalized[i][iteration-1]) > CLOSE_ENOUGH ||
					Math.abs(exposureNormalized[i][iteration] - exposureNormalized[i][iteration-1]) > CLOSE_ENOUGH) {
				return false;
			}
		}
		
		
		return true;
	}
	
	public void run() {
		initialize();
		createFirstOrder();

		for (int order = 0; order < NTH_ORDER_RESULTS; order++) {
			multiply(order);
			tabulateExposure(order);
			tabulateInfluence(order);
			normalizeInfluence(order);
			nomalizeExposure(order);
		}

		//		printExposureInfluence(influenceNormalized, exposureNormalized, "Normalized");
	}
	
	public boolean hasNonzeroData(double minX, double maxX, double minY, double maxY) {
		
		for (int i = 0; i < numEquations; i++) {
			if (Parser.isSystemVariable(idVar(i)) || Parser.isNaReplacement(idVar(i)))
				continue;
			double x = influenceNormalized[i][stableIterations];
			double y = exposureNormalized[i][stableIterations];
			if (inRange(x,y,minX,maxX,minY,maxY)) {
				if (x > 0.0 || y > 0.0)
					return true;
			}
		}
		return false;
	}
	
	public XYSeriesCollection getScatterData(double minX, double maxX, double minY, double maxY) {
		XYSeriesCollection d = new XYSeriesCollection();
		for (int i = 0; i < numEquations; i++) {
			if (Parser.isSystemVariable(idVar(i)) || Parser.isNaReplacement(idVar(i)))
				continue;
			XYSeries series = new XYSeries(idVar(i));
			double x = influenceNormalized[i][stableIterations];
			double y = exposureNormalized[i][stableIterations];
			if (inRange(x,y,minX,maxX,minY,maxY)) {
				series.add(x,y);
				d.addSeries(series);
			}
		}
	
		return d;

	}
	
	private boolean inRange(double x, double y, double minX, double maxX, double minY, double maxY) {
		return x>=minX && x<=maxX && y>=minY && y<=maxY;
	}
	private void printExposureInfluence(double[][] influence, double[][] exposure, String tag) {
		System.out.println("\n##### "+tag+" #####\n");
		for (int row = 0; row < numEquations; row++) {
			
			if (Parser.isSystemVariable(idVar(row)) || Parser.isNaReplacement(idVar(row)))
					continue;
			
			StringBuilder sb = new StringBuilder();
			sb.append(row);
//			sb.append(","+varName(row));
			sb.append(idVar(row));
			for (int order = 0; order < NTH_ORDER_ITERATIONS+1; order++) {
				sb.append(","+NumberFormatter.formatForRatio(influence[row][order]));
				sb.append(","+NumberFormatter.formatForRatio(exposure[row][order]));
			}
			System.out.println(sb.toString());
		}
	}
	
	private void tabulateExposure(int order) {

		double[][] data = matrix.getData();

		for (int col = 0; col < numEquations; col++) {
			for (int row = 0; row < numEquations; row++) {
				exposure[col][order] += data[row][col];
			}
		}

	}
	
	private void tabulateInfluence(int order) {

		double[][] data = matrix.getData();

		for (int row = 0; row < numEquations; row++) {
			for (int col = 0; col < numEquations; col++) {
				influence[row][order] += data[row][col];
			}
		}
	}
	
	private void normalizeInfluence(int order) {

		double max = Double.NEGATIVE_INFINITY;
		for (int row = 0; row < numEquations; row++) {
			max = Math.max(max, influence[row][order]);
		}
		for (int row = 0; row < numEquations; row++) {
			influenceNormalized[row][order] = max == 0.0 ? 0.0 : influence[row][order] / max;
		}
	}
	
	private void nomalizeExposure(int order) {
		double max = Double.NEGATIVE_INFINITY;
		for (int row = 0; row < numEquations; row++) {
			max = Math.max(max, exposure[row][order]);
		}
		for (int row = 0; row < numEquations; row++) {
			exposureNormalized[row][order] = max == 0.0 ? 0.0 : exposure[row][order] / max;
		}
	}
	
	private void printMatrix(BlockRealMatrix matrix, String tag) {
		System.out.println("\n######### "+ tag +" ##########\n");
		double[][] data = matrix.getData();
		for (int i = 0; i < data.length; i++) {
			StringBuilder sb = new StringBuilder();
			sb.append(i);
			sb.append(","+idVar(i));
			for (int j = 0; j < data[i].length; j++) {
				sb.append(", "+NumberFormatter.formatForRatio(data[i][j]));
			}
			System.out.println(sb.toString());
			
		}
	}
	
//	private void multiply() {
//		
//		matrixArray[0] = matrix;
////		printMatrix(matrixArray[0],tags[0]);
//		for (int i = 0; i < NTH_ORDER_ITERATIONS; i++) {
//			matrixArray[i+1] = matrixArray[i].multiply(matrixArray[i]);
////			printMatrix(matrixArray[i+1], tags[i+1]);
//		}
//	}
	
	private void multiply(int order) {
		if (order == 0)
			return;
		BlockRealMatrix tempMatrix;
		tempMatrix = matrix.multiply(matrix);
		matrix = tempMatrix;
	}
	
	private void initialize() {
		double[] zeros = new double[numEquations];
		Arrays.fill(zeros, 0.0);
		for (int row = 0; row < numEquations; row++) {
			matrix.setRow(row, zeros);
		}
		for (int i = 0; i < numEquations; i++) {
			Arrays.fill(influence[i], 0.0);
			Arrays.fill(exposure[i], 0.0);
		}
	}

	private void createFirstOrder() {
		
		String micmacTxt = InformationManagers.getInstance().getFileOrganizer().getMicmacTextFile();
		BufferedWriter bw = Utilities.openFileForWriting(micmacTxt);
		StringBuilder sb = new StringBuilder();
		
		Set<String> vars = varToID.keySet();

		try {
			for (String variable : Utilities.ordered(vars)) {

				sb.setLength(0);
				if (variable.startsWith("C_L_O_U_D")) {
//					System.out.println("MICMAC 2 discarding "+variable);
					continue;
				}
				if (variable.startsWith("NoNamefor")) {
//					System.out.println("MICMAC 2 discarding "+variable);
					continue;
				}
				if (Parser.isSystemVariable(variable)) {
//					System.out.println("MICMAC 2 discarding "+variable);
					continue;
				}
				if (Parser.isNaReplacement(variable)) {
//					System.out.println("MICMAC 2 discarding "+variable);
					continue;
				}
				int row = varID(variable);
				if (row < 0)
					continue;

				if (InformationManagers.getInstance().getEquationManager().getAnEquationFor(variable) != null) {
					Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(variable);
					List<String> usedBy = eqn.getUsedBy();
					sb.append(Parser.removeQuotes(variable)); // variable.replace("\"", "")+";");
					if (usedBy != null && usedBy.size() > 0) {
						for (String ub : usedBy) {
							if (ub.startsWith("C_L_O_U_D") || ub.startsWith("NoNamefor")) {
//								System.out.println("MICMAC 3 discarding "+variable+" ub "+ub);
								continue;
							}
							int col = varID(ub);
							if (col < 0)
								continue;
							if (row >= numEquations || col >= numEquations) {
								System.out.println("Range Error - MICMAC");
							}
							matrix.setEntry(row, col, 1.0);
							sb.append(Parser.removeQuotes(ub)); // ub.replace("\"", "")+";");
							//							System.out.println("$$$ " +variable+" ub "+ub+" ["+row+"]["+col+"]");

						}
					} else {
						//						System.out.println("$$$ No UsedBy for "+variable);
					}

				} else {
					System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>No Equation for "+variable);
				}



				try {

					bw.append(sb.toString()+"\n");

				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		} catch (OutOfRangeException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
//		printMatrix(matrix, "1st");
	}
	
	public int getStableIterations() {
		return stableIterations;
	}

	public void setStableIterations(int stableIterations) {
		this.stableIterations = stableIterations;
	}

	public SFEXYPointerAnnotationMetaData getMeta() {
		return meta;
	}

	public void setMeta(SFEXYPointerAnnotationMetaData meta) {
		this.meta = meta;
	}

	private int asInt(String s) {
		return Integer.parseInt(s);
	}
	
	public int varID(String s) {
		if (varToID.containsKey(s)) {
		return varToID.get(s);
		} else if (varToID.containsKey(Parser.quoted(s))) {
			return varToID.get(Parser.quoted(s));
		} else {
			return -1;
		}
//		return asInt(Utilities.nameToID(s));
//		return asInt(InformationManagers.getInstance().getEquationManager().getVariableID(s));
	}
	
	public String idVar(int id) {
		return idToVar.get(id);
//		return Utilities.idToName(id);
//		return InformationManagers.getInstance().getEquationManager().getVariableNameForID(id);
	}

	public BlockRealMatrix getMatrix() {
		return matrix;
	}

	public int getNumEquations() {
		return numEquations;
	}

	public double[][] getInfluence() {
		return influence;
	}

	public double[][] getExposure() {
		return exposure;
	}

	public double[][] getInfluenceNormalized() {
		return influenceNormalized;
	}

	public double[][] getExposureNormalized() {
		return exposureNormalized;
	}
}