package gov.anl.gss.sdm.server.model.analysis.micmac;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import javax.imageio.ImageIO;

import org.jfree.chart.JFreeChart;

public class MicMacPlot {
	
	public static final double FORMAT_CHANGE = 0.01;
	public static final int PLOT_SIZE = 1800;
	

	String title;
	MicMacMinimumMemory micmac;
	String fileName;
	double minX;
	double maxX;
	double minY;
	double maxY;

	public MicMacPlot(String title,
			MicMacMinimumMemory micmac,
			String fileName,
			double minX, double maxX, double minY, double maxY) {

		this.title = title;
		this.micmac = micmac;
		this.fileName = fileName;
		this.minX = minX;
		this.maxX = maxX;
		this.minY = minY;
		this.maxY = maxY;
		
	}


	public void generate() {
		MinMaxPair zoom = null;
		List<MinMaxPair> zooms = new ArrayList<MinMaxPair>();
		zooms.add(new MinMaxPair(0.0, 1.0, 0.0, 1.0));
		int num = 0;
		while (zooms.size() > 0) {
			zoom = zooms.get(0);
			zooms.remove(0);
			if (num++ == 0) {
				generateNoDataCheck(zoom);
			} else {
				generate(zoom);
			}
			zooms.addAll(micmac.getMeta().getAllZoom());
		}
	}
	
	public void generateNoDataCheck(MinMaxPair mmp) {
		
			generate(mmp.getMinX(), mmp.getMaxX(), mmp.getMinY(), mmp.getMaxY());
	}

	
	public void generate(MinMaxPair mmp) {
		if (micmac.hasNonzeroData(mmp.getMinX(), mmp.getMaxX(), mmp.getMinY(), mmp.getMaxY()))
			generate(mmp.getMinX(), mmp.getMaxX(), mmp.getMinY(), mmp.getMaxY());
	}

	public void generate(double minX, double maxX, double minY, double maxY) {

		this.minX = minX;
		this.maxX = maxX;
		this.minY = minY;
		this.maxY = maxY;

		MicMacScatterPlot plot = new MicMacScatterPlot(title.replace("##RANGE##", getRange())
				.replace("##ITERATION##", getIteration()), micmac, minX, maxX, minY, maxY);

		if (plot.hasAnnotatedDataPoints()) {
			OutputStream out = null;
			try {
				out = new FileOutputStream(new File(addRangeToName(fileName)));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			writeAsPNG( plot.getChart(), out, PLOT_SIZE,  PLOT_SIZE ); // 500,  270 // 200, 100
		}
	}
	
	private String getIteration() {
		return Integer.toString(micmac.getStableIterations());
	}
	
	private String getRange() {
		if (maxX - minX < FORMAT_CHANGE) {
			DecimalFormat form = new DecimalFormat("0.0E0");
			return " ["+form.format(minX)+
					","+form.format(maxX)+
					"]"+"-"+
					" ["+form.format(minY)+","+form.format(maxY)+"]";
	} else {
		return " ["+minX+","+maxX+"]"+"-"+" ["+minY+","+maxY+"]";
	}
	}
	
	private String addRangeToName(String f) {
		// these are .png files
		return f.replace(".png", getRange()+".png");
	}

	public void writeAsPNG( JFreeChart chart, OutputStream out, int width, int height  )
	{
		try
		{
			BufferedImage chartImage = chart.createBufferedImage( width, height, null);
			ImageIO.write( chartImage, "png", out );

			out.close();
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
