package gov.anl.gss.sdm.server.model.analysis.micmac;

import java.util.HashMap;
import java.util.Map;

public class SlotIterator {
	
	public static final int[] CLOCKWISE = new int[] {0,8,9,10,11,12,13,14,15, 7,6,5,4,3,2,1};
	public static final int[] COUNTER_CLOCKWISE = new int[] {0,1,2,3,4,5,6,7,15,14,13,12,11,10, 9, 8};
	
	int startSlot;
	Integer currentClockwise = null;
	Integer currentCounterClockwise = null;
	boolean hasMoreClockwise = true;
	boolean hasMoreCounterClockwise = true;
	
	boolean clockwiseIterating = false;
	boolean counterClockwiseIterating = false;
	
	int clockwiseGet = 0;
	int counterClockwiseGet = 0;
	
	
	Map<Integer, Integer> clockwise = null;
	Map<Integer, Integer> counterClockwise = null;
	
	
	public SlotIterator(int startSlot) {
		this.startSlot = startSlot;
		if (clockwise == null) {
			initializeMaps();
		}
	}
	
	private void initializeMaps() {
		
		clockwise = new HashMap<Integer, Integer>();
		counterClockwise = new HashMap<Integer, Integer>();
		
		for (int i = 0; i < CLOCKWISE.length-1; i++) {
			clockwise.put(CLOCKWISE[i], CLOCKWISE[i+1]);
			counterClockwise.put(COUNTER_CLOCKWISE[i], COUNTER_CLOCKWISE[i+1]);
		}
		
		clockwise.put(CLOCKWISE[CLOCKWISE.length-1], CLOCKWISE[0]);
		counterClockwise.put(COUNTER_CLOCKWISE[COUNTER_CLOCKWISE.length-1], COUNTER_CLOCKWISE[0]);
		
		if (startSlot == 0) {
		currentCounterClockwise = COUNTER_CLOCKWISE[COUNTER_CLOCKWISE.length-1];
		currentClockwise = CLOCKWISE[CLOCKWISE.length-1];
		} else {
		
		for (int i = 1; i < CLOCKWISE.length; i++) {
			if (CLOCKWISE[i] == startSlot) {
				currentClockwise = CLOCKWISE[i-1];
			}
			if (COUNTER_CLOCKWISE[i] == startSlot) {
				currentCounterClockwise = COUNTER_CLOCKWISE[i-1];
			}
		}
		}
	}
	
	public boolean isHasMoreClockwise() {
		return clockwiseGet < CLOCKWISE.length; 
	}
	
	public boolean isHasMoreCounterClockwise() {
		return counterClockwiseGet < CLOCKWISE.length; 
	}
	
	public Integer nextClockwise() {
		if (!isHasMoreClockwise())
			return null;
		clockwiseIterating = true;
		clockwiseGet++;
		if (currentClockwise == null) {
			currentClockwise = clockwise.get(startSlot);
		} else {
			currentClockwise = clockwise.get(currentClockwise);
		}
		return currentClockwise;
		
	}
	
	public Integer nextCounterClockwise() {
		if (!isHasMoreCounterClockwise())
			return null;
		counterClockwiseIterating = true;
		counterClockwiseGet++;
		if (currentCounterClockwise == null) {
			currentCounterClockwise = counterClockwise.get(startSlot);
		} else {
			currentCounterClockwise = counterClockwise.get(currentCounterClockwise);
		}
		return currentCounterClockwise;
		
	}

}
