package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.translator.ArrayReferenceNative;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

public class ArgumentBuilder implements ArgumentBuilderInterface {
static Logger log = Logger.getLogger(ArgumentBuilder.class.getSimpleName());

	public final static int NUM_ITERATIONS = 10;

	protected double[][] arguments;
	protected int numArgs;
	protected boolean databaseAvailable;
	protected double dt;
	protected double simTime[];
	
	public double[] getSimTime() {
		return simTime;
	}

	protected Map<String, String> idToNameMap;
	private Map<String, MinMeanMax> minMeanMax = new HashMap<String, MinMeanMax>();

	public ArgumentBuilder() {
	}

	public ArgumentBuilder(int numArgs, Map<String, String> idToNameMap) {
		databaseAvailable = InformationManagers.getInstance().getModelManager().isRunModel() &&
				InformationManagers.getInstance().getModelManager().isModelSuccessfullyExecuted();
		this.numArgs = numArgs;
		arguments = new double[NUM_ITERATIONS*numArgs][numArgs];
		simTime = new double[NUM_ITERATIONS*numArgs];
		this.idToNameMap = idToNameMap;
	}
	
	public boolean initialize() {
		
		boolean arrayReference = false;
		
		for (String taggedVar : idToNameMap.values()) {
			String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(taggedVar);
			if (ArrayReferenceNative.isArrayReference(origName))
				arrayReference = true;
		}
		
		// temp change -- if array reference, do not go to database
		
		if (!arrayReference && databaseAvailable) {
			for (String taggedVar : idToNameMap.values()) {
				String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(taggedVar);
				minMeanMax.put(taggedVar, InformationManagers.getInstance().getDataBridge().getMinMeanMax(origName));
			}
			dt = InformationManagers.getInstance().getDataBridge().getDt();
			initializeForAllArrows();
		} else {
			initializeForAllArrowsNoDatabase();
		}
		return true;
	}
	
	private double random() {
		double rand = Math.random();
		
		while (rand == 0.0) {
			rand = Math.random();
		}
		
		rand *= .0122;
		
		while (rand < .001) {
			rand *= 10.0;
		}
		
		return rand;
	}
	
	protected String asIdToNameMapKey(int i) {
		return "rhs["+i+"]";
	}
	
	public double getDt() {
		return dt;
	}

	private void initializeForAllArrows() {
		// this outside loop controls which index receives the iteration values
		
		// this is where we need to access actual data from database
		
		for (int varyingIndex = 0; varyingIndex < numArgs; varyingIndex++) {
			for (int i = 0; i < numArgs; i++) {
				MinMeanMax mmm = minMeanMax.get(idToNameMap.get(asIdToNameMapKey(i)));
				
				double mean = mmm.getMean();
				double min = mmm.getMin();
				double max = mmm.getMax();
				
				if (mean == 0.0) {
					mean = random();
				}
				
				while (mean != 0.0 && Math.abs(mean) < .001) {
					mean *= 10.0;
				}
				
				while (min != 0.0 && Math.abs(min) < .001) {
					min *= 10.0;
				}
				
				while (max != 0.0 && Math.abs(max) < .001) {
					max *= 10.0;
				}
				double rand = random();
				double delta = (max - min)/NUM_ITERATIONS;
				
				while (delta != 0.0 && Math.abs(delta) < .001) {
					delta *= 10.0;
				}
				
				
				for (int j = 0; j < NUM_ITERATIONS; j++) {
					if (i == varyingIndex) {
						// this is what we want to vary, so it must vary but within bounds
						if (max - min == 0.0) {
							arguments[j+varyingIndex*NUM_ITERATIONS][i] = ((j+1)*rand) + min;
						} else {
							arguments[j+varyingIndex*NUM_ITERATIONS][i] = (j+1) * delta;
						}
					} else {
							// never let this value be set to 0.0
						if (mmm.getMean() != 0.0) {
							arguments[j+varyingIndex*NUM_ITERATIONS][i] = mean + (i+1)*rand;
						} else {
							arguments[j+varyingIndex*NUM_ITERATIONS][i] = (i+1)*rand;
						}
						
					}
				}
			}
		}
	}
	
	private void initializeForAllArrowsNoDatabase() {
		// this outside loop controls which index receives the iteration values
		
		// this is where we need to access actual data from database
		
		for (int varyingIndex = 0; varyingIndex < numArgs; varyingIndex++) {
			for (int i = 0; i < numArgs; i++) {
				double rand = random();
				for (int j = 0; j < NUM_ITERATIONS; j++) {
					if (i == varyingIndex) {
						arguments[j+varyingIndex*NUM_ITERATIONS][i] = (j+1) * rand;
					} else {
						arguments[j+varyingIndex*NUM_ITERATIONS][i] = (i+1) * rand;
					}
				}
			}
		}
	}

	public double[][] getArguments() {
		return arguments;
	}

	public int getNumArgs() {
		return numArgs;
	}
}
