package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;

public class ArrowPolarity {
	
	private String tail;
	private String head;
	private String polarity;
	
	public static final String UNKNOWN = "?";
//	public static final String NONE = "";
	public static final String NEGATIVE = "-";
	public static final String POSITIVE = "+";
	public static final String ANY = "*";
	
	public static final String INITIALIZATION = "?-+?-+?-+?-+?-+?-+";
	
//	public static final String FUNCTION_SKIP = "Function";
//	public static final String UNDEFINED_LOAD = "Undefined(Load)";
//	public static final String UNDEFINED = "Undefined";
//	public static final String UNDEFINED_HEAD = "Undefined Head";
//	public static final String UNDEFINED_TAIL = "Undefined Tail";
//	public static final String INCONSISTENT = "Inconsistent";
//	
//	public static final String ITE_SWITCH = "If Then Else Switch";
//	public static final String LOOKUP_INCONSISTENT = "Lookup Inconsistent";
//	public static final String LOOKUP_EXTERNAL = "Lookup External";
//	public static final String LOOKUP_NO_EQN = "Lookup No Eqn";
	public static final String TIME_DELAY_SMOOTH = NEGATIVE;
//	public static final String NO_EFFECT = "NO EFFECT";
	
	public ArrowPolarity(String tail, String head, String polarity) {
	
		this.tail = tail;
		this.head = head;
		this.polarity = polarity;
	}
	
	public static String reverse(String polarity) {
		if (polarity.equals(POSITIVE))
			return NEGATIVE;
		else if (polarity.equals(NEGATIVE))
			return POSITIVE;
		else
			return UNKNOWN;
	}

	public String getTail() {
		return tail;
	}

	public String getHead() {
		return head;
	}

	public String getPolarity() {
		return polarity;
	}
	
	public boolean isPositivePolarity() {
		return polarity.equals(POSITIVE);
	}
	
	public boolean isNegativePolarity() {
		return polarity.equals(NEGATIVE);
	}
	
	public boolean isUnknownPolarity() {
		return !isNegativePolarity() && !isPositivePolarity();
	}
	
	public boolean isGeneratedCloud() {
		return head.startsWith(GraphicObject.CLOUD_NAME) || head.startsWith(GraphicObject.CLOUD_NAME);
	}
	
	public boolean isGeneratedRate() {
		return head.startsWith(GraphicObject.RATE_NAME) || head.startsWith(GraphicObject.RATE_NAME);
	}
	
	public boolean isGeneratedCloudOrRate() {
		return isGeneratedCloud() || isGeneratedRate();
	}
	
	public boolean equals(ArrowPolarity another) {
		if (getHead().equals(another.getHead()) && getTail().equals(another.getTail()))
			return true;
		else
			return false;
	}
	
	public static boolean isUnknownPolarity(String polarity) {
		if (polarity.equals(POSITIVE) ||
				polarity.equals(NEGATIVE) ||
				polarity.equals(ANY) ||
				polarity.startsWith(StringConstants.FUNCTION_SKIP) ||  
				polarity.equals(StringConstants.INCONSISTENT) ||
				polarity.equals(StringConstants.TIME_DELAY_SMOOTH) ||
				polarity.equals(StringConstants.NO_EFFECT) ||
				polarity.equals(StringConstants.LOOKUP_INCONSISTENT) ||
				polarity.equals(StringConstants.LOOKUP_EXTERNAL) ||
				polarity.equals(StringConstants.LOOKUP_NO_EQN) ||
				polarity.equals(StringConstants.ITE_SWITCH)
				)
			return false;
		else
			return true;
	}
}
