package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.bridge.BridgeModelResultsDatabaseImplementation;

public class DataBridge {

	private String databaseName;
	private BridgeModelResultsDatabaseImplementation db;
	
	public DataBridge(String databaseName) {
		this.databaseName = databaseName;
		db = new BridgeModelResultsDatabaseImplementation(databaseName);
	}
	
	public double[] getValuesAsArray(String variable) {
		if (variable.equals("time"))
			return getTimeAsArray();
		return db.getDb().getVarTimeVector(variable);
		
	}
	
	public double[] getTimeAsArray() {
		return db.getDb().getTimeVector();
		
	}
	
	public double getDt() {
		return db.getDt();
	}
	
	public MinMeanMax getMinMeanMax(String variable) {
		double min = Double.MAX_VALUE;
		double mean = 0; 
		double max = -Double.MAX_VALUE;
		double[] values = getValuesAsArray(variable);
		
		for (double val : values) {
			min = Math.min(min, val);
			max = Math.max(max, val);
			mean += val;
		}
		
		if ((min != 0.0 && Math.abs(min) < 0.0000001) || (mean != 0.0 && Math.abs(mean) < 0.0000001) || max == Double.MAX_VALUE) {
			System.out.println("MIN MEAN");
		}
		
		return new MinMeanMax(variable, min, mean/values.length, max);
	}
	
	public String getLookupValuesAsString(String variable) {
		double[][] v = db.getDb().getLookupValues(variable);
		
		StringBuilder sb = new StringBuilder();
		sb.append("new double[][] {");
		
		for (int i = 0; i < v.length; i++) {
			if (i > 0)
				sb.append(",");
			sb.append("{");
			for (int j = 0; j < v[i].length; j++) {
				if (j > 0)
					sb.append(",");
				sb.append(v[i][j]);
			}
			sb.append("}");
		}
		
		sb.append("}");
		
		return sb.toString();
		
	}
	
	public void shutdown() {
		db.getDb().shutdown();
	}
}
