package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.support.SDFunctionsWithXLSColt;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationComponents;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;

import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

public class EquationAnalyzer {
	static Logger log = Logger.getLogger(EquationAnalyzer.class.getSimpleName());
	
	protected final static Set<String> LIMITING = new LinkedHashSet<String>();
	protected final static Set<String> MATH_FUNCTIONS = new LinkedHashSet<String>();
	
	static {
		LIMITING.add("MIN");
		LIMITING.add("MAX");
		LIMITING.add("IFTHENELSE");
		LIMITING.add("ZIDZ");
		LIMITING.add("XIDZ");
		
		MATH_FUNCTIONS.add("MIN");
		MATH_FUNCTIONS.add("ARCCOS");
		MATH_FUNCTIONS.add("ARCSIN");
		MATH_FUNCTIONS.add("ARCTAN");
		MATH_FUNCTIONS.add("EXP");
		MATH_FUNCTIONS.add("LN");
		MATH_FUNCTIONS.add("LOG");
		MATH_FUNCTIONS.add("MAX");
		MATH_FUNCTIONS.add("SIN");
		MATH_FUNCTIONS.add("COS");
		MATH_FUNCTIONS.add("COSH");
		MATH_FUNCTIONS.add("SQRT");
		MATH_FUNCTIONS.add("RANDOM01");
		MATH_FUNCTIONS.add("TAN");
		MATH_FUNCTIONS.add("ABS");
	}
	
	protected Equation equation;
	protected Set<Equation> reduced;
	protected Equation originalEquation;
	
	protected boolean cannotProcess = false;
	protected EquationComponents lEqn;
	
	protected PolarityResults polarityResultsNoIVV; //  = new PolarityResults();
//	protected PolarityResults polarityResultsWithIVV; //  = new PolarityResults();
	protected NativeDataTypeManager ndtm;
	
	public EquationAnalyzer(EquationComponents lEqn, Equation originalEquation, PolarityResults polarityResultsNoIVV,
			PolarityResults polarityResultsWithIVV, NativeDataTypeManager ndtm) {
		
		try {
			int doNotRegister = 1;
			this.ndtm = ndtm;
			this.lEqn = lEqn;
			Equation t = new Equation(lEqn, doNotRegister);
			if (!t.isSyntacticallyCorrect())
				cannotProcess = true;
			equation = t;
			this.originalEquation = originalEquation;
			this.polarityResultsNoIVV = polarityResultsNoIVV;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			cannotProcess = true;
			System.out.println("FAILURE EqnAnalyze: "+originalEquation);
			System.out.println(e.getMessage());
			
		}
	}
	
	public void initialize() {
		try {
//			System.out.println("PolInit: "+equation.getVensimEquationOnly());
//			if (equation.getVensimEquationOnly().contains("FTE diffence non-specific needs minus tenured teachers")) {
//				System.out.println("polarity debug");
//			}
			
			reduced = reduceEquations();
//			analyze();
			
			Set<String> functionsCalled = originalEquation.getFunctionsCalled();
			functionsCalled.removeAll(LIMITING);
			functionsCalled.removeAll(AnalysisManager.FUNCTIONS_USE_ARG1);
			functionsCalled.remove("INTEG");
			functionsCalled.removeAll(MATH_FUNCTIONS);
			
//			System.out.println("\n\nORIG: "+originalEquation.getVensimEquationOnly());
//			for (Equation e : reduced) {
//				System.out.println("\t\tREDU: "+e.getVensimEquationOnly()+"\n");
//			}
			
			if (functionsCalled.size() == 0) {
				analyze();
			} else {
				polarityResultsNoIVV.skippingFunction(originalEquation);
//			polarityResultsWithIVV.skippingFunction(originalEquation);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			cannotProcess = true;
			System.out.println("FAILURE EqnAnalyze: "+originalEquation);
			System.out.println(e.getMessage());
			
		}
	}
	
	protected void analyze() {
		for (Equation eqn : reduced) {
			analyze(eqn);
		}
	}
	
	
	protected Set<Equation> reduceEquations() {
		Set<Equation> redEqn = new LinkedHashSet<Equation>();
		Equation tCopy = createCopy(equation);
		Node rootCopy = tCopy.getTreeRoot();
		
		// first split the equation into as many as necessary

		if (requiresSplit(rootCopy)) {
			split(tCopy, redEqn);
		} else {
			redEqn.add(tCopy);
		}
		
		Set<Equation> redEqn2 = new LinkedHashSet<Equation>(redEqn);
		redEqn.clear();
		
		for (Equation e : redEqn2) {

			tCopy = createCopy(e);
			rootCopy = tCopy.getTreeRoot();
			
			processTimeArguments(rootCopy, redEqn);
			processInitialValueArguments(rootCopy, redEqn);

			while(canReduceInPlace(rootCopy)) {
				reduceInPlace(rootCopy);
			}
			
			String x = rootCopy.generateExpression()+"~~autogenerated1";
			
			Equation t1 = new Equation(new EquationComponents(x), 1);
			redEqn.add(t1);
		}
		//		
		//		if (requiresSplit(rootCopy)) {
		//			split(tCopy, redEqn);
		//		} else {
		//			redEqn.add(tCopy);
		//		}

		//		while(canReduceInPlace(rootCopy)) {
		//			reduceInPlace(rootCopy);
		//		}
		//
		//		
		//		// need to transform for functions too
		//		if (!requiresSplit(rootCopy)) {
		//			redEqn.add(tCopy);
		//		} else {
		//			split(tCopy, redEqn);
		//		}
		return redEqn;

	}
	
	protected boolean canReduceInPlace(Node node) {
		if (node == null)
			return false;
		if (containsSpecialProcessFunction(node, AnalysisManager.FUNCTIONS_USE_ARG1)) {
			return true;
		} else {
			return false;
		}
	}
	
	protected void reduceInPlace(Node node) {

		List<Node> funcs = TreeTraversal.getFunctionCallsInTree(node, AnalysisManager.FUNCTIONS_USE_ARG1);
		if (funcs.size() == 0)
			return;
		Node func = funcs.get(0);
		Node arg1 = TreeTraversal.getFunctionArgument(func, 1);
		Node.replaceNode(func, arg1);

	}
	
	protected boolean requiresSplit(Node node) {
		if (node == null)
			return false;
		if (/* containsSpecialProcessFunction(node, AnalysisManager.FUNCTIONS_USE_ARG13) || */
				containsLimitingFunction(node)) {
			return true;
		} else {
			return false;
		}
	}
	
	protected void processTimeArguments(Node root, Set<Equation> redEqn) {
		List<Node> funcs = TreeTraversal.getFunctionCallsInTree(root, AnalysisManager.FUNCTIONS_WITH_TIME_ARG2);
		if (funcs.size() == 0)
			return;
		
		// just create simple equations that will cause negative polarity
		
		for (Node func : funcs) {
			Node arg = TreeTraversal.getFunctionArgument(func, 2);
			String s1a = root.getChild().generateExpression();
			String s1b = ndtm.getOriginalNameQuotedIfNecessary(s1a);
			String s2a = arg.generateExpression();
			String s2b = ndtm.getOriginalNameQuotedIfNecessary(s2a);
			String eqn1;
			
			if (AnalysisManager.FUNCTIONS_WITH_TIME_ARG2_NEGATIVE_POLARITY.contains(func.getToken()))
				eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = -1.0 * " + 
					ndtm.getOriginalNameQuotedIfNecessary(arg.generateExpression())+"~~autogenerated1";
			else {
				eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = 1.0 * " + 
						ndtm.getOriginalNameQuotedIfNecessary(arg.generateExpression())+"~~autogenerated1";
			}
			
			
			Equation t1 = new Equation(new EquationComponents(eqn1), 1);
			redEqn.add(t1);
		}
		
		funcs = TreeTraversal.getFunctionCallsInTree(root, AnalysisManager.FUNCTIONS_WITH_TIME_ARG3);
		if (funcs.size() == 0)
			return;
		
		for (Node func : funcs) {
			Node arg = TreeTraversal.getFunctionArgument(func, 3);
			String s1a = root.getChild().generateExpression();
			String s1b = ndtm.getOriginalNameQuotedIfNecessary(s1a);
			String s2a = arg.generateExpression();
			String s2b = ndtm.getOriginalNameQuotedIfNecessary(s2a);
			String eqn1;
			
			if (AnalysisManager.FUNCTIONS_WITH_TIME_ARG3_NEGATIVE_POLARITY.contains(func.getToken()))
				eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = -1.0 * " + 
					ndtm.getOriginalNameQuotedIfNecessary(arg.generateExpression())+"~~autogenerated1";
			else {
				eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = 1.0 * " + 
						ndtm.getOriginalNameQuotedIfNecessary(arg.generateExpression())+"~~autogenerated1";
			}
			
			
			Equation t1 = new Equation(new EquationComponents(eqn1), 1);
			redEqn.add(t1);
		}
		
		
	}
	
	protected void processInitialValueArguments(Node root, Set<Equation> redEqn) {
		List<Node> funcs = TreeTraversal.getFunctionCallsInTree(root, AnalysisManager.FUNCTIONS_USE_ARG13);
		if (funcs.size() == 0)
			return;
		
		// just create simple equations that will cause positive polarity
		
		for (Node func : funcs) {
			Node arg = TreeTraversal.getFunctionArgument(func, 3);
			String s1a = root.getChild().generateExpression();
			String s1b = ndtm.getOriginalNameQuotedIfNecessary(s1a);
			String s2a = arg.generateExpression();
			String s2b = ndtm.getOriginalNameQuotedIfNecessary(s2a);
			String eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = " + 
					ndtm.getOriginalNameQuotedIfNecessary(arg.generateExpression())+"~~autogenerated1";
			Equation t1 = new Equation(new EquationComponents(eqn1), 1);
			redEqn.add(t1);
		}
		
		
		
	}
	
	protected void split(Equation eqn, Set<Equation> redEqn) {

		Equation tCopy = createCopy(eqn);

//		if (containsSpecialProcessFunction(tCopy.getTreeRoot(), AnalysisManager.FUNCTIONS_USE_ARG13)) {
//			Equation t = transformARG3(tCopy.getTreeRoot());
//			if (requiresSplit(t.getTreeRoot())) {
//				split(t, redEqn);
//			} else {
//				redEqn.add(t);
//			}
//		}

		// check if there are "limiting" function calls in equation. If not, no processing required
		List<Node> funcs = TreeTraversal.getFunctionCallsInTree(tCopy.getTreeRoot(), LIMITING);
		if (funcs.size() == 0)
			return;

		Node limitingFunc = funcs.get(0);

		// create two new equations with equation equal to original
		int startArg = 1;
		int endArg = 2;

		if (limitingFunc.getToken().contains("IFTHENELSE")) {
			startArg = 2;
			endArg = 3;
		}

		if (limitingFunc.getToken().contains("XIDZ")) {
			startArg = 1;
			endArg = 3;
		}

		for (int arg = startArg; arg <= endArg; arg++) {
			Node root = tCopy.getTreeRoot();
			Equation t1 = new Equation(new EquationComponents(Node.generateExpression(root)), 1);
			root = t1.getTreeRoot();
			limitingFunc = TreeTraversal.getFunctionCallsInTree(root, LIMITING).get(0);
			Node arg1 = TreeTraversal.getFunctionArgument(limitingFunc, arg);
			Node.replaceNode(limitingFunc, arg1);
//			String eqn1 = ndtm.getOriginalNameQuotedIfNecessary(root.getChild().generateExpression())+" = " + 
//					Node.generateExpression(root.getChild().getNext())+"~~autogenerated1";
			String eqn1 = root.generateExpression()+"~~autogenerated1";
			t1 = new Equation(new EquationComponents(eqn1), 1);
			if (containsLimitingFunction(t1.getTreeRoot())) {
				split(t1, redEqn);
			} else {
				redEqn.add(t1);
			}
		}


		return;
	}
	
	public void analyze(Equation t) {
		
//		if (t.getOriginalLhs().equals("Input"))
//			System.out.println("DEBUG Input");

		PolarityCodeBuilder polarityCodeBuilder = new PolarityCodeBuilder(t, originalEquation, false);
		polarityCodeBuilder.buildCode();
		if (!polarityCodeBuilder.hasTreeRoot())
			return;
		log.debug("###"+polarityCodeBuilder.getEquation().getVensimEquation());
		log.debug(">>>> "+polarityCodeBuilder.getGeneratedCode());

		if (polarityCodeBuilder.getGeneratedCode() == null)
			return;

		// no functions for now, only stocks (which still need work)
		// we've pruned stock trees to remove integ function, so any functions that we see prevents processing
//		if (SDFunctionsWithXLSColt.containsNoPolarityFunction(polarityCodeBuilder.getGeneratedCode())) {
//			//		if (polarityCodeBuilder.getGeneratedCode().contains("sdFunction") /* && !eqn.isStock() */) {
//			log.debug("\nGenPolCode: "+originalEquation.getVensimEquationOnly());
//			log.debug(polarityCodeBuilder.getGeneratedCode());
//			log.debug("Need to deal with this, SKIP");
//
//			// functions
//			// XIDZ(A,B,X) Returns X if dividing by zero (B =0) otherwise returns A divided by B.
//			// ZIDZ(A,B) Zero (0.0) if dividing by zero (B=0) otherwise returns A divided by B.
//
//			polarityResultsNoIVV.skippingFunction(t);
//			//			polarityResultsWithIVV.skippingFunction(t);
//			return;
//		}


		PolarityRunner runner = new PolarityRunner(polarityCodeBuilder);


		PolarityResultAnalyzer polarityResultAnalyzer = new PolarityResultAnalyzer(runner);
		polarityResultAnalyzer.analyze(polarityResultsNoIVV);
		//			polarityResultAnalyzer.analyze(polarityResultsWithIVV);

		if (originalEquation.isStock()) {
			analyzePolarityForIVVArrows(originalEquation);
		}
	}
	
	protected String isolateIVV(Equation eqn) {
		Node copyRoot = eqn.getCopyOfTree();
		Node integFunction = TreeTraversal.getRhs(copyRoot);
		Node ivv = TreeTraversal.getFunctionArgument(integFunction, 2);
		return Node.generateExpression(ivv);
	}
	
	protected Equation copyOf(Equation eqn) {
		EquationComponents lEqn = new EquationComponents(eqn.getVensimEquationOnly());
		Equation t = new Equation(lEqn, 1);
		return t;
	}
	
	protected void analyzePolarityForIVVArrows(Equation eqn) {
		
		Equation t = copyOf(eqn);
		
		
		// need to get second argument
		
//		Node root = t.getTreeRoot();
//		Node integ = TreeTraversal.getRhs(root);
//		Node arg1 = TreeTraversal.getFunctionArgument(integ, 1);
//		Node arg2 = TreeTraversal.getFunctionArgument(integ, 2);
//		Node.replaceNode(integ, arg2);
		
		PolarityCodeBuilder polarityCodeBuilder = new PolarityCodeBuilder(t, originalEquation, true);
		polarityCodeBuilder.buildCode();
		
		log.debug("###"+polarityCodeBuilder.getEquation().getVensimEquation());
		log.debug(">>>> "+polarityCodeBuilder.getGeneratedCode());
		// no functions for now, only stocks (which still need work)
		// we've pruned stock trees to remove integ function, so any functions that we see prevents processing
		if (SDFunctionsWithXLSColt.containsNoPolarityFunction(polarityCodeBuilder.getGeneratedCode())) {
//		if (polarityCodeBuilder.getGeneratedCode().contains("sdFunction") /* && !eqn.isStock() */) {
			log.debug("\nGenPolCodeIVV: "+t.getVensimEquationOnly());
			log.debug(polarityCodeBuilder.getGeneratedCode());
			log.debug("Need to deal with this, SKIP");
			return;
		}

		PolarityRunner runner = new PolarityRunner(polarityCodeBuilder);
		PolarityResultAnalyzer polarityResultAnalyzer = new PolarityResultAnalyzer(runner);
		polarityResultAnalyzer.analyze(polarityResultsNoIVV);
//		polarityResultAnalyzer.analyze(polarityResultsWithIVV);
	}
	
    protected boolean containsSpecialProcessFunction(Node root, Set<String> set) {
//    	System.out.println(TreeTraversal.getFunctionCallsInTree(root, set));
//    	if (TreeTraversal.getFunctionCallsWithVariablesInTree(root, set).size() > 0) 
    		if (TreeTraversal.getFunctionCallsInTree(root, set).size() > 0) 
    		return true;
    	else
    		return false;
    }
	
    protected boolean containsLimitingFunction(Node root) {
    	if (TreeTraversal.getFunctionCallsInTree(root, LIMITING).size() > 0) 
    		return true;
    	else
    		return false;
    }
    
    protected Equation createCopy(Equation eqn) {
    	
//    	String e = ndtm.getOriginalNameQuotedIfNecessary(eqn.getLhs())+" = " + 
//				eqn.getRhsFromTokens()+"~~autogenerated2";

    	String e = Node.generateExpression(eqn.getTreeRoot())+"~~autogenerated2";
    	EquationComponents lEqn = new EquationComponents(e);
//    	EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(equation.getLhs())+" = " + 
//    			Node.generateExpression(TreeTraversal.getRhs(eqn.getTreeRoot()))+"~~autogenerated2");
		return new Equation(lEqn, 1);
    }
    
    protected Set<Equation> transformARG1(Node root) {
    	Set<Equation> set = new LinkedHashSet<Equation>();
    	Equation t1 = null;
    	Equation t2 = null;
    	
    	EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(equation.getLhs())+" = " + 
    			Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated3");
		t1 = new Equation(lEqn, 1);
		lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(equation.getLhs())+" = " +  // +" = -1.0 * " + 
    			Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated4");
		t2 = new Equation(lEqn, 1);
    	
    	// first make a copy of this equation since we will be modifying it
    	
    	List<Node> funcs = TreeTraversal.getFunctionCallsInTree(t1.getTreeRoot(), AnalysisManager.FUNCTIONS_USE_ARG1);
    	if (funcs.size() > 0) {
    		Node n = funcs.get(funcs.size()-1);
    		// want to replace function invocation with just the first argument
    		Node arg1 = TreeTraversal.getFunctionArgument(n, 1);
    		Node.replaceNode(n, arg1);
    		set.add(t1);
    	}
    	
    	// first make a copy of this equation since we will be modifying it
    	
    	funcs = TreeTraversal.getFunctionCallsInTree(t2.getTreeRoot(), AnalysisManager.FUNCTIONS_USE_ARG1);
    	if (funcs.size() > 0) {
    		Node n = funcs.get(funcs.size()-1);
    		// want to replace function invocation with just the first argument
    		Node arg2 = TreeTraversal.getFunctionArgument(n, 2);
    		if (arg2 != null) {
    			Node.replaceNode(n, arg2);
    			set.add(t2);
    		}
    	}
    	return set;
    }
    
    
    protected Equation transformARG3(Node root) {
    	Equation t = null;
    	
    	// first make a copy of this equation since we will be modifying it
    	EquationComponents lEqn = new EquationComponents(ndtm.getOriginalNameQuotedIfNecessary(equation.getLhs())+" = " + 
    			Node.generateExpression(TreeTraversal.getRhs(root))+"~~autogenerated5");
		t = new Equation(lEqn, 1);
		
    	List<Node> funcs = TreeTraversal.getFunctionCallsInTree(t.getTreeRoot(), AnalysisManager.FUNCTIONS_USE_ARG13);
    	if (funcs.size() > 0) {
    		
    		// try last To First
    		Node n = funcs.get(funcs.size()-1);
    		// want to replace function invocation with just the third argument
    		Node arg3 = TreeTraversal.getFunctionArgument(n, 3);
    		Node.replaceNode(n, arg3);
    	}
    	return t;
    }

	public boolean isCannotProcess() {
		return cannotProcess;
	}
    

}
