package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

public class IfThenElseVariableRelationship {
static Logger log = Logger.getLogger(IfThenElseVariableRelationship.class.getSimpleName());

	private String lhs;
	private List<Set<String>> arg1Vars = new ArrayList<Set<String>>();
	private List<Set<String>> arg2Vars = new ArrayList<Set<String>>();
	private List<Set<String>> arg3Vars = new ArrayList<Set<String>>();

	private Node eqnRoot;

	public IfThenElseVariableRelationship(Node eqnRoot) {
		
		this.eqnRoot = eqnRoot;
		if (eqnRoot == null)
			return;
		try {
			this.lhs = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(TreeTraversal.getLhs(eqnRoot).getToken());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		findRelationships();
	}

	private void findRelationships() {
		for (Node node : TreeTraversal.getTreeAsList(TreeTraversal.getRhs(eqnRoot))) {
			if (Parser.isFunctionInvocation(node) && Parser.getFunctionName(node).equalsIgnoreCase("IFTHENELSE")) {
				// found an ITE
				Set<String> a1 = new LinkedHashSet<String>();
				arg1Vars.add(a1);
				Set<String> a2 = new LinkedHashSet<String>();
				arg2Vars.add(a2);
				Set<String> a3 = new LinkedHashSet<String>();
				arg3Vars.add(a3);

				List<String> varsToAdd1 = toOriginalName(TreeTraversal.extractVariablesFromChildExcludingFunctions(TreeTraversal.getFunctionArgument(node, 1)));
				a1.addAll(varsToAdd1);

				List<String> varsToAdd2 = toOriginalName(TreeTraversal.extractVariablesFromChildExcludingFunctions(TreeTraversal.getFunctionArgument(node, 2)));
				a2.addAll(varsToAdd2);

				List<String> varsToAdd3 = toOriginalName(TreeTraversal.extractVariablesFromChildExcludingFunctions(TreeTraversal.getFunctionArgument(node, 3)));
				a3.addAll(varsToAdd3);
			}

		}
	}
	
	private List<String> toOriginalName(List<String> list) {
		List<String> original = new ArrayList<String>();
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		for (String v : list)
			original.add(ndtm.getOriginalName(v));
		return original;
	}

	public void setConditionVariablePolarity(PolarityResults polarityResults) {

		boolean arg2Pos = false;
		boolean arg2Neg = false;
		boolean arg2Unk = false;
		boolean arg2Any = false;
		
		boolean arg3Pos = false;
		boolean arg3Neg = false;
		boolean arg3Unk = false;
		boolean arg3Any = false;
		
		String arg2Polarity = ArrowPolarity.UNKNOWN;
		String arg3Polarity = ArrowPolarity.UNKNOWN;
		
		
		log.debug("##### ITE Condition Polarity #####");

		for (int i = 0; i < arg1Vars.size(); i++) {
			Set<String> a1 = arg1Vars.get(i);
			Set<String> a2 = arg2Vars.get(i);
			Set<String> a3 = arg3Vars.get(i);

			// gather arg2 Polarities

			arg2Pos = false;
			arg2Neg = false;
			arg2Unk = false;
			arg2Any = false;
			
			for (String v2 : a2) {
				if (Parser.isNumber(v2)) {
					log.debug(v2+" -> "+lhs+" Constant");
				} else {
					log.debug(v2+" -> "+lhs+" "+polarityResults.getPolarityForArrow(v2, lhs));
				}
				if (Parser.isNumber(v2)) {
					arg2Any = true;
				} else if (polarityResults.getPolarityForArrow(v2, lhs).equals(ArrowPolarity.POSITIVE)) {
					arg2Pos = true;
				} else if (polarityResults.getPolarityForArrow(v2, lhs).equals(ArrowPolarity.NEGATIVE)) {
					arg2Neg = true;
				} else {
					arg2Unk = true;
				}
			}
			arg2Polarity = determinePolarity(arg2Pos, arg2Neg, arg2Unk, arg2Any);
			log.debug("ARG2 Polarity = "+arg2Polarity);
			
			arg3Pos = false;
			arg3Neg = false;
			arg3Unk = false;
			arg3Any = false;
			
			for (String v3 : a3) {
				if (Parser.isNumber(v3)) {
					log.debug(v3+" -> "+lhs+" Constant");
				} else {
					log.debug(v3+" -> "+lhs+" "+polarityResults.getPolarityForArrow(v3, lhs));
				}
				
				if (Parser.isNumber(v3))
					arg3Any = true;
				if (polarityResults.getPolarityForArrow(v3, lhs).equals(ArrowPolarity.POSITIVE))
					arg3Pos = true;
				else if (polarityResults.getPolarityForArrow(v3, lhs).equals(ArrowPolarity.NEGATIVE))
					arg3Neg = true;
				else
					arg3Unk = true;
			}
			arg3Polarity = determinePolarity(arg3Pos, arg3Neg, arg3Unk, arg3Any);
			log.debug("ARG3 Polarity = "+arg3Polarity);
			
			for (String v1 : a1) {
				if (arg2Polarity.equals(arg3Polarity) || arg3Polarity.equals(ArrowPolarity.ANY)) {
					polarityResults.setPolarityForArrow(v1, lhs, arg2Polarity);
				} else if (arg2Polarity.equals(ArrowPolarity.ANY)){
					polarityResults.setPolarityForArrow(v1, lhs, arg3Polarity);
				} else {
					polarityResults.setPolarityForArrow(v1, lhs, StringConstants.ITE_SWITCH);
				}
			}
		}
	}
	
	private String determinePolarity(boolean pos, boolean neg, boolean unk, boolean any) {
		String polarity = ArrowPolarity.UNKNOWN;
		
		// a single unknown renders it to unknown
		if (unk)
			return polarity;
		
		if (pos && !neg) {
			polarity = ArrowPolarity.POSITIVE;
		} else if (!pos && neg) {
			polarity = ArrowPolarity.NEGATIVE;
		} else if (!pos && !neg && any) {
			polarity = ArrowPolarity.ANY;
		}
		return polarity;
		
	}

	public String getLhs() {
		return lhs;
	}

	public List<Set<String>> getArg1Vars() {
		return arg1Vars;
	}

	public List<Set<String>> getArg2Vars() {
		return arg2Vars;
	}

	public List<Set<String>> getArg3Vars() {
		return arg3Vars;
	}

	public Node getEqnRoot() {
		return eqnRoot;
	}
}
