package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationComponents;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class PolarityCodeBuilder implements CodeBuilderInterface {
	
static Logger log = Logger.getLogger(PolarityCodeBuilder.class.getSimpleName());
	

    
protected Equation equation;
protected Set<Equation> equations;
protected Map<String, String> idToNameMap;
protected Node polarityRoot;
protected String generatedCode;
protected int numIDs;
protected Equation originalEquation;
protected boolean ivv;

protected double dt;

    // getCopyOfTree
    
    public PolarityCodeBuilder(Equation equation, Equation originalEquation, boolean ivv) {
    	// process any special cases without actually generating code

    	this.equation = equation;
    	this.originalEquation = originalEquation;
    	this.ivv = ivv;
    	
    	idToNameMap = new LinkedHashMap<String, String>();
    	polarityRoot = equation.getTreeRoot(); // equation.getCopyOfTree();
    }
    
    public void  buildCode() {
    	if (polarityRoot == null) {
    		polarityRoot = equation.getCopyOfTree();
    		return;
    	}
    	
    	try {
			getRidOfRandom();
			transformEquation();

			numIDs = insertIDs();
			generatedCode = new String(build());

		} catch (Exception e) {
			// TODO Auto-generated catch block
			return;
		}


    }
    
    public List<String> getSimplifiedByArgument() {
		List<String> simplified = new ArrayList<>();
		
		for (int i = 0; i < numIDs; i++) {
			EquationComponents lEqn2 = new EquationComponents(Parser.removeAnnotation(generatedCode)+"~~autogenerated6");
			int doNotRegister = 1;
			Equation t2 = new Equation(lEqn2, doNotRegister);
			for (Node n : t2.getTreeAsList()) {
				if (n.getToken().contains("rhs[") && !n.getToken().contains("rhs["+i+"]")) {
					n.setToken(Double.toString(Math.random()));
				} else {
					n.setToken(Parser.removeAnnotation(n.getToken()));
				}
			}
			simplified.add(t2.generateExpression(t2.getTreeRoot()));
		}
		return simplified;
    }
    
    public boolean hasTreeRoot() {
    	return polarityRoot != null;
    }
    
    protected void getRidOfRandom() {
    	for (Node n : TreeTraversal.getFunctionCallsInTree(polarityRoot, AnalysisManager.RANDOM)) {
    		// want to replace function invocation with just the first argument
    		Node constant = new Node("999999");
    		Node.replaceNode(n, constant);
    	}
    }
    

    
    private void transformEquation() {
    	// first handle integration
    	pruneStockTree();
    	// then smooth and delay
    	transformSmoothDelay();
    }
    
    private void pruneStockTree() {
    	if (equation.isStock()) {
    		Node lhs = TreeTraversal.getLhs(polarityRoot);
    		Node rhs = TreeTraversal.getRhs(polarityRoot);
    		Node integ = TreeTraversal.getFirstFunctionCallInTree(polarityRoot, "INTEG");
    		Node arg1 = TreeTraversal.getFunctionArgument(integ, ivv ? 2 : 1);
    		lhs.setNext(arg1);
    		arg1.setParent(polarityRoot);
    		arg1.setPrevious(lhs);
    		arg1.setNext(null);
    	}
    }
    
    private void transformSmoothDelay() {
    	
    	boolean replaced = false;
    	for (Node n : TreeTraversal.getFunctionCallsInTree(polarityRoot, AnalysisManager.FUNCTIONS_USE_ARG1)) {
//    		Node.printAsTree(polarityRoot);
    		// want to replace function invocation with just the first argument
    		Node arg1 = TreeTraversal.getFunctionArgument(n, 1);
    		arg1 = TreeTraversal.generateAdditionSequence(arg1);
    		// remove other arguments - just remove next node
    		arg1.setNext(null);
    		Node.replaceNode(n, arg1);
//    		Node.printAsTree(polarityRoot);
    		replaced = true;
    	}
    	if (replaced)
    		transformSmoothDelay();
    }
    
    private void replaceVariablesWithConstants() {
    	// replace any variables that are constants with their numeric values

    	Node lhs = TreeTraversal.getLhs(polarityRoot);
    	Node rhs = TreeTraversal.getRhs(polarityRoot);

    	Set<String> vars = new LinkedHashSet<String>();
    	TreeTraversal.getVariablesInTree(rhs, vars);

    	for (Node node : TreeTraversal.getTreeAsList(rhs)) {
    		String var = node.getToken();
    		if (vars.contains(var)) {
    			Equation varEqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
    			if (varEqn != null) {
    				if (varEqn.isConstant()) {
    					Node varRoot = varEqn.getCopyOfTree();
    					// replace var with constant
    					Node varRhs = TreeTraversal.getRhs(varRoot);
    					Node.replaceNode(node, varRhs);
    				}
    			}
    		}
    	}
    }
    
    protected int insertIDs() {

    	// first change lhs
    	Node lhs = TreeTraversal.getLhs(polarityRoot);
    	Node rhs = TreeTraversal.getRhs(polarityRoot);
   
    	String lhsVar = "lhs";
    	
    	idToNameMap.put(lhsVar, lhs.getToken());
    	lhs.setToken(lhsVar);



    	int id = 0;
    	//	Iterator<String> iter = equation.getRHSVariables().iterator();
    	Set<String> vars = new LinkedHashSet<String>();
    	TreeTraversal.getVariablesInTree(rhs, vars);  // getFunctionArgumentIncludingAutoGen
    	Iterator<String> iter = vars.iterator();
    	while(iter.hasNext()) {
    		String variable = iter.next();
    		String origName = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(variable);
    		Equation e = InformationManagers.getInstance().getEquationManager().getAnEquationFor(origName);
    		if (!variable.endsWith(StringConstants.RSSD_LOOKUP) && !e.isDefinesLookup()) {
//    			if (Parser.isTimeVariable(origName)) {
//    				System.out.println("Processing time var in polarity code");
//    					replaceVariableWithVariable(rhs, variable, "time");
//    			} else {
//    				replaceVariableWithID(rhs, InformationManagers.getInstance().getNativeDataTypeManager().getLegalNameWithSubscripts(equation, variable), id);
//    				replaceVariableWithID(rhs, variable, id);
//    				id++;
//    			}
    				replaceVariableWithID(rhs, InformationManagers.getInstance().getNativeDataTypeManager().getLegalNameWithSubscripts(equation, variable), id);
    				replaceVariableWithID(rhs, variable, id);
    				id++;
    		}
    	}

    	return id;

    }
    
    private void replaceVariableWithVariable(Node node, String variable, String newVariable) {
	if (node == null)
	    return;
	if (node.getToken().equals(variable)) {
	    String rhsVar = newVariable;
	    node.setToken(rhsVar);
	    idToNameMap.put(rhsVar, variable);
	}
	replaceVariableWithVariable(node.getChild(), variable, newVariable);
	replaceVariableWithVariable(node.getNext(), variable, newVariable);
	
    }
    
    private void replaceVariableWithID(Node node, String variable, int id) {
	if (node == null)
	    return;
	if (node.getToken().equals(variable)) {
	    String rhsVar = "rhs["+id+"]";
	    node.setToken(rhsVar);
	    idToNameMap.put(rhsVar, variable);
	}
	replaceVariableWithID(node.getChild(), variable, id);
	replaceVariableWithID(node.getNext(), variable, id);
	
    }
    

    
    public String build() {
    	StringBuilder sb = new StringBuilder();
    	sb.append(replaceMathFunctions(equation.getEquationFromTree(polarityRoot)));
    	return sb.toString();
    }
    
    protected String replaceMathFunctions(String equation) {
    	String oneToOne = equation.
    			replace("sdFunctions.MIN", "Math.min").
    			replace("sdFunctions.ARCCOS", "Math.acos").
    			replace("sdFunctions.ARCSIN", "Math.asin").
    			replace("sdFunctions.ARCTAN", "Math.atan").
    			replace("sdFunctions.EXP", "Math.exp").
    			replace("sdFunctions.LN", "Math.log").
//    			replace("sdFunctions.LOG", "Math.log10").
    			replace("sdFunctions.MAX", "Math.max").
    			replace("sdFunctions.SIN", "Math.sin").
    			replace("sdFunctions.COS", "Math.cos").
    			replace("sdFunctions.COSH", "Math.cosh").
    			replace("sdFunctions.SQRT", "Math.sqrt").
    			replace("sdFunctions.RANDOM01", "Math.random").
    			replace("sdFunctions.TAN", "Math.tan")
    			;
   
//    if (!equation.contains("sdFunctions.LOG")) 
    	return oneToOne;
    
//    String trans = convertLog(oneToOne);
//    return trans;
    
    }
    
    private String convertLog(String eqn) {
    	EquationComponents lEqn = new EquationComponents(eqn.replace("sdFunctions.", "")+"~~autogenerated7");
		Equation t = new Equation(lEqn, 1);
		Node n = t.generateEquationTree();
		Set<String> logFunc = new LinkedHashSet<String>();
		logFunc.add("LOG");
		List<Node> logFuncs = TreeTraversal.getFunctionCallsInTree(n, logFunc);
		for (Node log : logFuncs) {
			translateLog(log);
		}
		
		return t.getEquationFromTree(n);
    }
    
    private void translateLog(Node log) {
    	// first get arguments
    	Node x = TreeTraversal.getFunctionArgument(log, 1);
    	Node y = TreeTraversal.getFunctionArgument(log, 2);
    	
    	Node divide = new Node("/");
    	Node mlx = new Node("Math.log");
    	Node mly = new Node("Math.log");
    	
    	// now build tree structure for this translation
    	
    	divide.setChild(mlx);
    	mlx.setParent(divide);
    	mlx.setNext(mly);
    	mly.setParent(divide);
    	mly.setPrevious(mlx);
    	
    	mlx.setChild(x);
    	x.setNext(null);
    	x.setPrevious(null);
    	x.setParent(mlx);
    	
    	mly.setChild(y);
    	y.setNext(null);
    	y.setPrevious(null);
    	y.setParent(mlx);
    	
    	Node.replaceNode(log, divide);
    }
    
    // .replace("sdFunctions.MIN", "Math.min")

    public String getGeneratedCode() {
        return generatedCode;
    }

	public int getNumIDs() {
		return numIDs;
	}
	
	public StringBuilder createClassSourceCode() {
		 StringBuilder src = new StringBuilder();
		 src.append(getHeader());
		 src.append("double "+getGeneratedCode()+";\n");
		 src.append(getTail());
		 return src;
	}
	
	protected String getHeader() {
		StringBuilder src = new StringBuilder();
		src.append(" import gov.anl.gss.sdm.server.model.support.SDFunctionsWithXLSColt;\n");
        src.append("public class PCalculator {\n");
        src.append("protected SDFunctionsWithXLSColt sdFunctions = new SDFunctionsWithXLSColt(null);\n");
        src.append("public double compute(double[] rhs, double time) {\n");
		 return src.toString();
	}
	
	protected String getTail() {
		StringBuilder src = new StringBuilder();
		
        src.append("return lhs;\n");
        src.append("}\n");
        src.append("}\n");

		 return src.toString();
	}
	
	public String getClassName() {
		return "PCalculator";
	}

	public Equation getEquation() {
		return equation;
	}
	
	public String getVariableFor(String s) {
		return this.idToNameMap.get(s);
	}

	public Map<String, String> getIdToNameMap() {
		return idToNameMap;
	}

	public Equation getOriginalEquation() {
		return originalEquation;
	}

	public double getDt() {
		return dt;
	}

}
