package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;

import java.util.List;

import org.apache.log4j.Logger;

public class PolarityResultAnalyzer implements PolarityResultAnalyzerInterface {
static Logger log = Logger.getLogger(PolarityResultAnalyzer.class.getSimpleName());
	
	public static final int ZERO = 0;
	public static final int POS = 1;
	public static final int NEG = 2;
			
	public static final int POSITIVE = 1;
	public static final int NEGATIVE = -1;
	public static final int INCONSISTENT = -2;
	public static final int NO_EFFECT = -3;
	
	public static boolean DEBUG = false;
	
	private PolarityRunner runner;
	private int[] polarity;
	double sumL = 0.0;
	double sumA = 0.0;
	
	public PolarityResultAnalyzer(PolarityRunner runner) {
		this.runner = runner;
	}
	
	public void analyze(PolarityResults results) {
		analyze(results, "ignore");
	}
	
	private void analyze(PolarityResults results, String lPolarity) {
		CodeBuilderInterface cb = runner.getCodeBuilder();
		ArgumentBuilder lab = new ArgumentBuilder(cb.getNumIDs(), cb.getIdToNameMap());
		runner.run(lab);

		// Did we actually run anything?
		
		if (runner.getArgumentBuilder() == null || runner.getArgumentBuilder().getNumArgs() == 0) {
			log.debug("No Polarity to determine");
			return;
		}
		List<PolarityCalculationResult> result = runner.getResults();
		
//		System.out.println("\n###\n");
//		System.out.println(runner.getPolarityCodeBuilder().getOriginalEquation().getVensimEquationOnly());
//		System.out.println(runner.getPolarityCodeBuilder().getEquation().getVensimEquationOnly());
//		System.out.println(runner.getPolarityCodeBuilder().getGeneratedCode());
//		for (PolarityCalculationResult p : result) {
//			System.out.println(p);
//		}
		
		polarity = new int[runner.getArgumentBuilder().getNumArgs()];
		int pos = 0;
		if (DEBUG)
			System.out.println("Equation: "+runner.getCodeBuilder().getGeneratedCode());
		for (int arg = 0; arg < runner.getArgumentBuilder().getNumArgs(); arg++) {
			double[] lhs = new double[ArgumentBuilder.NUM_ITERATIONS];
			double[] argument = new double[ArgumentBuilder.NUM_ITERATIONS];
			double[] lhsSlope = new double[ArgumentBuilder.NUM_ITERATIONS-1];
			double[] argumentSlope = new double[ArgumentBuilder.NUM_ITERATIONS-1];
			for (int seq = 0; seq < ArgumentBuilder.NUM_ITERATIONS; seq++) {
				if (result.size() == 0) {
					System.out.println("WHY FAIL?");
					return;
				}
				PolarityCalculationResult res = result.get(pos++);
//				log.debug(res.toString());
				lhs[seq] = res.getLhs();
				argument[seq] = res.getInputs()[arg];
			}
			if (DEBUG) {
				System.out.println("LHS: "+Utilities.asList(lhs));
				System.out.println("ARG: "+Utilities.asList(argument));
			}
			
//			for (int s = 0; s < lhsSlope.length; s++) {
//				lhsSlope[s] = lhs[s+1] == lhs[s] ? 0.0 : lhs[s+1] < lhs[s] ? -1.0 : 1.0;
//				argumentSlope[s] = argument[s+1] == argument[s] ? 0.0 : argument[s+1] < argument[s] ? -1.0 : 1.0;
//				log.debug("s:"+s+" lhsSlope:"+lhsSlope[s]+" argSlope:"+argumentSlope[s]);
//			}
			
			for (int s = 0; s < lhsSlope.length; s++) {
				lhsSlope[s] = lhs[s+1] == lhs[s] ? 0.0 : lhs[s+1] < lhs[s] ? -1.0 : 1.0;
				if (Double.isInfinite(lhsSlope[s]) || Double.isNaN(lhsSlope[s]))
					lhsSlope[s] = 0.0;
				argumentSlope[s] = argument[s+1] == argument[s] ? 0.0 : argument[s+1] < argument[s] ? -1.0 : 1.0;
//				log.debug("s:"+s+" lhsSlope:"+lhsSlope[s]+" argSlope:"+argumentSlope[s]);
			}
			CodeBuilderInterface pcb = runner.getCodeBuilder();
			String tail = pcb.getVariableFor("rhs["+arg+"]");
			String head = pcb.getVariableFor("lhs"); 
			
			polarity[arg] = determinePolarity(lhsSlope, argumentSlope);
			log.debug("ARGUMENT: "+arg+" Polarity: "+polarity[arg]+"  "+tail+" --> "+head);
			if (DEBUG) {
				System.out.println("ARGUMENT: "+arg+" Polarity: "+polarity[arg]+"  "+tail+" --> "+head);
			}
//			System.out.println(pcb.getEquation().getVensimEquationOnly());
//			System.out.println(pcb.getOriginalEquation().getVensimEquationOnly());
//			System.out.println("ARGUMENT: "+arg+" Polarity: "+polarity[arg]+"  "+tail+" --> "+head);
			String arrowPolarity = StringConstants.INCONSISTENT; 
			if (polarity[arg] == POSITIVE) {
				arrowPolarity = ArrowPolarity.POSITIVE;
			} else if (polarity[arg] == NEGATIVE) {
				arrowPolarity = ArrowPolarity.NEGATIVE;
			} else if (polarity[arg] == NO_EFFECT) {
				arrowPolarity = StringConstants.NO_EFFECT;
			} 
			// note tail and head are annotated -- need to get original
			NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
			tail = ndtm.getOriginalName(tail);
			head = ndtm.getOriginalName(head);
			
			if (lPolarity.equals("ignore")) {
				if (DEBUG) {
					System.out.println("ARGUMENT: "+arg+" Polarity: "+arrowPolarity+"  "+tail+" --> "+head);
				}
				results.setPolarityForArrow(tail, head, arrowPolarity);
			} else {
				// determine polarity based on arrow polarity & lookup Polarity
				if (lPolarity.equals(ArrowPolarity.POSITIVE)) {
					results.setPolarityForArrow(tail, head, arrowPolarity);
				} else if (lPolarity.equals(ArrowPolarity.NEGATIVE)) {
					results.setPolarityForArrow(tail, head, ArrowPolarity.reverse(arrowPolarity));
				} else if (lPolarity.equals(ArrowPolarity.NEGATIVE)) {
					results.setPolarityForArrow(tail, head, ArrowPolarity.reverse(arrowPolarity));
				} else {
					results.setPolarityForArrow(tail, head, ArrowPolarity.UNKNOWN);
				}
			}
			
		}
	}
	
	private int determinePolarity(double[] lhsSlope, double[] argumentSlope) {
		
		boolean[] lhs = {false, false, false};
		boolean[] rhs = {false, false, false};
		
		for (int i = 0; i < lhsSlope.length; i++) {
			if (lhsSlope[i] < 0.0) {
				lhs[NEG] = true;
			} else if (lhsSlope[i] > 0.0) {
				lhs[POS] = true;
			} else {
				lhs[ZERO] = true;
			}
			if (argumentSlope[i] < 0.0) {
				rhs[NEG] = true;
			} else if (argumentSlope[i] > 0.0) {
				rhs[POS] = true;
			} else {
				rhs[ZERO] = true;
			}
		}
		
		// determine slope
		// by implemetation, we know argument slope always increases
		// need to check only lhs
		if (lhs[NEG] && lhs[POS])
			return INCONSISTENT;
		if (lhs[NEG])
			return NEGATIVE;
		else if (lhs[POSITIVE])
			return POSITIVE;
		else
			return NO_EFFECT;
		
//		sumL = 0.0;
//		sumA = 0.0;
//		
//		for (int i = 0; i < lhsSlope.length; i++) {
//			sumL += lhsSlope[i];
//			sumA += argumentSlope[i];
//		}
//		
//		if (Math.abs(sumL) != lhsSlope.length || Math.abs(sumA) != lhsSlope.length) {
//			log.debug(runner.getPolarityCodeBuilder().getEquation().getVensimEquationOnly());
//			log.debug("Inconsistent: "+sumL+" "+lhsSlope.length+" "+sumA+" "+lhsSlope.length);
//			return INCONSISTENT;
//		}
//		
//		if ((sumL > 0.0 && sumA > 0.0) || (sumL < 0.0 && sumA < 0.0))
//			return POSITIVE;
//		return NEGATIVE;
	}
	
	

}
