package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class PolarityResults {
static Logger log = Logger.getLogger(PolarityResults.class.getSimpleName());

	Map<String, Map<String, String>> results;
	Map<String, Map<String, String>> resultsByID;
	Set<String> processed = new LinkedHashSet<String>();
	private SDObjectManager equationManager;

	public PolarityResults() {
		results = new LinkedHashMap<String, Map<String, String>>();
		resultsByID = new LinkedHashMap<String, Map<String, String>>();
	}
	
	public void loadCausualLinks(SDObjectManager equationManager /*Map<String, Set<String>> usedBy, Map<String, Set<String>> uses*/) {
		this.equationManager = equationManager;
		Map<String, Set<String>> usedBy = InformationManagers.getInstance().getSdObjectManager().getUsedBy();
		for (String tail : usedBy.keySet()) {
			Set<String> heads = usedBy.get(tail);
			for (String head : heads) {
				
				if (head.contains("sdFunc") || tail.contains("sdFunc")) {
					System.out.println("FUNC");
				}
				
				if (tail.replace(StringConstants.RSSD_LOOKUP, "").equals(head.replace(StringConstants.RSSD_LOOKUP, "")))
					continue;
				if (head.startsWith(GraphicObject.CLOUD_NAME) || tail.startsWith(GraphicObject.CLOUD_NAME))
					continue;
//				createArrow(tail, head, ArrowPolarity.UNDEFINED_LOAD);
			}
		}
	}
	
	public void skippingFunction(Equation eqn) {
		String head = InformationManagers.getInstance().getEquationLookup().getScreenName(eqn.getLhs());
		if (head == null || InformationManagers.getInstance().getSdObjectManager().getUses() == null || 
				!InformationManagers.getInstance().getSdObjectManager().getUses().containsKey(head)) {
//			System.out.println("Skip Problem: "+eqn.getVensimEquationOnly());
			return;
		}
		
		Set<String> funcs = eqn.getInvokedFunctions();
		StringBuilder sb = new StringBuilder();
		for (String f : funcs) {
			if (sb.length() > 0)
				sb.append(",");
			sb.append(f);
		}
		for (String tail : InformationManagers.getInstance().getSdObjectManager().getUses().get(head)) {
			setPolarityForArrow(tail, head, StringConstants.FUNCTION_SKIP+"["+sb.toString()+"]");
//			System.out.println("Skip: "+eqn.getVensimEquationOnly()+" "+head+" -> "+tail);
		}
	}
	
	public String getPolarityForArrowID(String tail, String head) {
		if (resultsByID.containsKey(tail) && resultsByID.get(tail).containsKey(head))
				return resultsByID.get(tail).get(head);
		else
			return ArrowPolarity.UNKNOWN;
	}

	public String getPolarityForArrow(String tail, String head) {
		
		String t;
		String h;
		
		// we are expecting tail and head to variables
		
		if (Parser.isNumber(tail)) {
			t = nonArray(Utilities.idToName(tail));
			h = nonArray(Utilities.idToName(head));
		} else {
			t = nonArray(tail);
			h = nonArray(head);
		}
		
//		t = Parser.quoted(t);
//		h = Parser.quoted(h);
		
		if (results.containsKey(t)) {
			
		} else {
			t = Parser.quoted(t);
			h = Parser.quoted(h);

			if (results.containsKey(t)) {

			} else {
				System.out.println("Neither quoted or not quoted for "+t);
			}
		}
		
		boolean located = false;
		
		String res = ArrowPolarity.UNKNOWN;
		
		if (results.containsKey(t)) {
			Map<String, String> map = results.get(t);
			if (map == null) {
				res = StringConstants.UNDEFINED_HEAD;
			} else {
				String polarity = results.get(t).get(h);
				located = true;
				if (polarity == null) {
					res = ArrowPolarity.UNKNOWN;
				} else
					res = polarity;
			}
		} else {
			res = StringConstants.UNDEFINED_TAIL;
		}
		
//		if (res.equals("?"))
//				System.out.println("Bas Polarity on Link");
		return res;
	}
	
	public void createArrowsFromLoop(Loop loop) {
		// make sure that arrows exist and initialize to unknown
		// if we can calculate polarity, it will be set to the appropriate value
		for (ArrowPolarity ap : loop.getArrows()) {
			String pair = ap.getTail()+"###"+ap.getHead();
			if (!processed.contains(pair)) {
				createArrow(ap.getTail(),  ap.getHead(),  ap.getPolarity());
				processed.add(pair);
			}
		}
		
	}
	
	public void createArrow(String tail, String head) {
		if (head.startsWith(GraphicObject.CLOUD_NAME) || tail.startsWith(GraphicObject.CLOUD_NAME))
			return;
		createArrow(tail, head, ArrowPolarity.INITIALIZATION); // UNKNOWN
	}
	
	public void createArrow(String tail, String head, String polarity) {
		
		if (tail.contains("sdFunc") || head.contains("sdFunc"))
				System.out.println("FUNC");
		
		String t = Parser.removeQuotes(nonArray(tail));
		String h = Parser.removeQuotes(nonArray(head));
		String tID = InformationManagers.getInstance().getEquationManager().getVariableIDForName(t);
		String hID = InformationManagers.getInstance().getEquationManager().getVariableIDForName(h);
		
		if (t.contains("sdFunc") || h.contains("sdFunc"))
			System.out.println("FUNC");
		
		if (tail.startsWith(GraphicObject.CLOUD_NAME))
			return;
		if (head.startsWith(GraphicObject.CLOUD_NAME))
			return;
		
		if (!resultsByID.containsKey(tID)) {
			resultsByID.put(tID, new LinkedHashMap<String, String>());
		}
		
		if (!results.containsKey(t)) {
			results.put(t, new LinkedHashMap<String, String>());
		}
		if (!results.get(t).containsKey(h)) {
			results.get(t).put(h, polarity);
			resultsByID.get(tID).put(hID, polarity);
		}
		
//		System.out.println(""+t+" -> "+h+"  "+tID+" => "+hID+"  "+polarity);
	}

	public void setPolarityForArrow(String tail, String head, String polarity) {
		
		if (tail.startsWith(GraphicObject.CLOUD_NAME))
			return;
		if (head.startsWith(GraphicObject.CLOUD_NAME))
			return;
		
		if (tail.contains("sdFunc") || head.contains("sdFunc")) {
			System.out.println("FUNC");
		}

		String t = nonArray(tail);
		String h = nonArray(head);
		String tID = InformationManagers.getInstance().getEquationManager().getVariableIDForName(t);
		String hID = InformationManagers.getInstance().getEquationManager().getVariableIDForName(h);

		if (!results.containsKey(t)) {
			results.put(t, new LinkedHashMap<String, String>());
		}
		
		if (!resultsByID.containsKey(tID)) {
			resultsByID.put(tID, new LinkedHashMap<String, String>());
		}

		if (results.get(t).containsKey(h)) {
			if (!polarity.equals(results.get(t).get(h))) {
				String currentPolarity = results.get(t).get(h);
				if (!results.get(t).get(h).equals(StringConstants.UNDEFINED_LOAD) && 
						!results.get(t).get(h).equals(ArrowPolarity.UNKNOWN) && 
						!polarity.equals(ArrowPolarity.UNKNOWN)) {
					
//					System.out.println("Attempting to Redefine Polarity <"+t+"> -> <"+h+
//							"> <"+results.get(t).get(h)+"> <"+polarity+">");
					log.debug("Attempting to Redefine Polarity <"+t+"> -> <"+h+" <"+results.get(t).get(h)+"> <"+polarity+">");
				}
				
				if (!ArrowPolarity.isUnknownPolarity(polarity)) {
					if ((currentPolarity.equals(ArrowPolarity.POSITIVE) && polarity.equals(ArrowPolarity.NEGATIVE)) ||
							(currentPolarity.equals(ArrowPolarity.NEGATIVE) && polarity.equals(ArrowPolarity.POSITIVE))) {
//						System.out.println("$$$$$$$$$$$$$$$$$ "+t + " -> "+h+" <"+StringConstants.INCONSISTENT+">");
						results.get(t).put(h, StringConstants.INCONSISTENT);
						resultsByID.get(tID).put(InformationManagers.getInstance().getEquationManager().getVariableIDForName(hID), 
								StringConstants.INCONSISTENT);
					} else {
						// if it already has a polarity, don't change if no effect (this would be based on data values)
						if (!polarity.equals(StringConstants.NO_EFFECT) || 
								(polarity.equals(StringConstants.NO_EFFECT) && (
										currentPolarity.equals(ArrowPolarity.INITIALIZATION) ||
										currentPolarity.equals(StringConstants.UNDEFINED_LOAD)))) {
							results.get(t).put(h, polarity);
							resultsByID.get(tID).put(hID, polarity);
//							System.out.println(t + " -> "+h+" <"+polarity+">");
						}
					}
				}
			}
		} else {
				results.get(t).put(h, polarity);
				resultsByID.get(tID).put(hID, polarity);
		}
		
	}
	
	private String nonArray(String tail) {
		String t = tail;
		
		if (ArrayReference.isArrayReference(t)) {
			return new ArrayReference(t).getArrayName();
		}
		
		return tail;
		
	}
	
	public List<String[]> loadEdges(Map<String, Set<String>> neverConsider, boolean withIVV) {

		List<String[]> edges = new ArrayList<String[]>();
		List<String> tails = new ArrayList<String>(results.keySet());
		Collections.sort(tails);

		for (String tail : tails) {
			List<String> heads = new ArrayList<String>(results.get(tail).keySet());
			Collections.sort(heads);
			for (String head : heads) {
				if (neverConsider.containsKey(tail) &&
						(neverConsider.get(tail).contains(head) || 
								neverConsider.get(tail).contains(tail)))
					continue;		

				String[] th = new String[2];
				th[0] = tail;
				th[1] = head;
				edges.add(th);
			}
		}

		return edges;

	}
	
	public void writeEdgesToFile(String file, Map<String, Set<String>> neverConsider) {
		try {
			// Map<String, Map<String, String>> results;
			BufferedWriter bw = Utilities.openFileForWriting(
					/* InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), */ file);
			List<String> tails = new ArrayList<String>(results.keySet());
			Collections.sort(tails);
			bw.append("tail\thead\n");
			for (String tail : tails) {
				List<String> heads = new ArrayList<String>(results.get(tail).keySet());
				Collections.sort(heads);
				for (String head : heads) {
					if (neverConsider.containsKey(tail) &&
							(neverConsider.get(tail).contains(head) || 
									neverConsider.get(tail).contains(tail)))
						continue;
					
					bw.append(tail);
					bw.append("\t");
					bw.append(head);
					bw.append("\n");
				}
				
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeArrowThicknessToFile(String file) {
		try {
			// Map<String, Map<String, String>> results;
			BufferedWriter bw = Utilities.openFileForWriting(
					/* InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), */ file);
			List<String> tails = new ArrayList<String>(results.keySet());
			Collections.sort(tails);
			
			for (String tail : tails) {
				List<String> heads = new ArrayList<String>(results.get(tail).keySet());
				Collections.sort(heads);
				for (String head : heads) {
					bw.append(tail);
					bw.append("\t");
					bw.append(head);
					bw.append("\t");
					bw.append("1");
					bw.append("\n");
				}
				
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeArrowsToFile(String file) {
		try {
			// Map<String, Map<String, String>> results;
			BufferedWriter bw = Utilities.openFileForWriting(
					/* InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), */ 
					file);
			List<String> tails = new ArrayList<String>(results.keySet());
			Collections.sort(tails);
			bw.append("tail\thead\tpolarity\n");
			for (String tail : tails) {
				List<String> heads = new ArrayList<String>(results.get(tail).keySet());
				Collections.sort(heads);
				for (String head : heads) {
					bw.append(tail);
					bw.append("\t");
					bw.append(head);
					bw.append("\t");
					bw.append(getPolarityForArrow(tail, head));
					bw.append("\n");
				}
				
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public List<ArrowPolarity> getPositivePolarityArrows() {
		return getArrowsWithPolarity(ArrowPolarity.POSITIVE);
	}
	
	public List<ArrowPolarity> getNegativePolarityArrows() {
		return getArrowsWithPolarity(ArrowPolarity.NEGATIVE);
	}
	
	public List<ArrowPolarity> getUnknownPolarityArrows() {
		List<ArrowPolarity> list = getArrows();
		removeFromList(list, getNegativePolarityArrows());
		removeFromList(list, getPositivePolarityArrows());
		return list;
	}
	
	private void removeFromList(List<ArrowPolarity> inList, List<ArrowPolarity> toRemove) {
		for (ArrowPolarity ap : toRemove) {
			remove(inList, ap);
		}
		
	}
	
	private void remove(List<ArrowPolarity> list, ArrowPolarity ap) {
		ArrowPolarity toRemove = null;
		for (ArrowPolarity apl : list) {
			if (apl.equals(ap))
				toRemove = apl;
		}
		list.remove(toRemove);
	}
	
	public List<ArrowPolarity> getArrows() {
		
		List<ArrowPolarity> ap = new ArrayList<ArrowPolarity>();
		for (String tail : results.keySet()) {
			Map<String, String> heads = results.get(tail);
			for (String head : heads.keySet()) {
				String pol = heads.get(head);
				if (!pol.equals(StringConstants.UNDEFINED))
					ap.add(new ArrowPolarity(tail, head, pol));
			}
		}
		
		return ordered(ap);
	}
	
	public List<ArrowPolarity> getArrowsWithPolarity(String polarity) {
		
		List<ArrowPolarity> ap = new ArrayList<ArrowPolarity>();
		for (String tail : results.keySet()) {
			Map<String, String> heads = results.get(tail);
			for (String head : heads.keySet()) {
				String pol = heads.get(head);
				if (pol.equals(polarity))
					ap.add(new ArrowPolarity(tail, head, pol));
			}
		}
		
		return ordered(ap);
	}
	
	public List<ArrowPolarity> ordered(List<ArrowPolarity> al) {
		Collections.sort(al, new Comparator<ArrowPolarity>() {
			@Override
			public int compare(ArrowPolarity arrow1, ArrowPolarity arrow2) {
				
				if (arrow1.getTail().equals(arrow2.getTail())) {
					return arrow1.getHead().compareToIgnoreCase(arrow2.getHead());
				} else {
					return arrow1.getTail().compareToIgnoreCase(arrow2.getTail());
				}

			}
		});
		return al;
	}

	public void setEquationManager(SDObjectManager equationManager) {
		this.equationManager = equationManager;
	}
}
