package gov.anl.gss.sdm.server.model.analysis.polarity;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class VariableComplexity {
	
	Map<String, VariableComplexityData> complexity;
	PolarityResults polarityResults;
	
	int minIn = Integer.MAX_VALUE;
	int maxIn = Integer.MIN_VALUE;
	
	int minOut = Integer.MAX_VALUE;
	int maxOut = Integer.MIN_VALUE;
	
	int minInPos = Integer.MAX_VALUE;
	int maxInPos = Integer.MIN_VALUE;
	
	int minInNeg = Integer.MAX_VALUE;
	int maxInNeg = Integer.MIN_VALUE;
	
	int minInUnk = Integer.MAX_VALUE;
	int maxInUnk = Integer.MIN_VALUE;
	
	int minOutPos = Integer.MAX_VALUE;
	int maxOutPos = Integer.MIN_VALUE;
	
	int minOutNeg = Integer.MAX_VALUE;
	int maxOutNeg = Integer.MIN_VALUE;
	
	int minOutUnk = Integer.MAX_VALUE;
	int maxOutUnk = Integer.MIN_VALUE;
	
	public VariableComplexity(PolarityResults polarityResults) {
		this.polarityResults = polarityResults;
		complexity = new HashMap<String, VariableComplexityData>();
	}
	
	public void process() {
		
		for (ArrowPolarity ap : polarityResults.getArrows()) {
			if (!complexity.containsKey(ap.getTail()))
				complexity.put(ap.getTail(), new VariableComplexityData(ap.getTail()));
			if (!complexity.containsKey(ap.getHead()))
				complexity.put(ap.getHead(), new VariableComplexityData(ap.getHead()));
			
			complexity.get(ap.getTail()).processArrowPolarity(ap);
			complexity.get(ap.getHead()).processArrowPolarity(ap);
		}
	}
	
	public List<VariableComplexityData> getSortedData() {
		List<String> keys = new ArrayList<String>(complexity.keySet());
		Collections.sort(keys);
		List<VariableComplexityData> data = new ArrayList<VariableComplexityData>();
		for (String key : keys) {
			data.add(complexity.get(key));
		}
		return data;
	}
	
	public void determineSizing() {
		for (String key : complexity.keySet()) {
			VariableComplexityData vcd = complexity.get(key);
			minIn = Math.min(minIn, vcd.getIn());
			minInPos = Math.min(minInPos, vcd.getInPos());
			minInNeg = Math.min(minInNeg, vcd.getInNeg());
			minInUnk = Math.min(minInUnk, vcd.getInUnk());
			
			maxIn = Math.max(maxIn, vcd.getIn());
			maxInPos = Math.max(maxInPos, vcd.getInPos());
			maxInNeg = Math.max(maxInNeg, vcd.getInNeg());
			maxInUnk = Math.max(maxInUnk, vcd.getInUnk());
			
			minOut = Math.min(minOut, vcd.getOut());
			minOutPos = Math.min(minOutPos, vcd.getOutPos());
			minOutNeg = Math.min(minOutNeg, vcd.getOutNeg());
			minOutUnk = Math.min(minOutUnk, vcd.getOutUnk());
			
			maxOut = Math.max(maxOut, vcd.getOut());
			maxOutPos = Math.max(maxOutPos, vcd.getOutPos());
			maxOutNeg = Math.max(maxOutNeg, vcd.getOutNeg());
			maxOutUnk = Math.max(maxOutUnk, vcd.getOutUnk());
		}
	}
	
	public VariableComplexityData getDataFor(String var) {
//		if (var == null || complexity.get(var) == null) {
//			System.out.println("var null");
//		}
		return complexity.get(var); // Parser.removeQuotes(var));
	}
	
	public void printVars() {
		for (String s : complexity.keySet()) {
			System.out.println("vc: "+s);
		}
	}

	public Map<String, VariableComplexityData> getComplexity() {
		return complexity;
	}

	public PolarityResults getPolarityResults() {
		return polarityResults;
	}

	public int getMinIn() {
		return minIn;
	}

	public int getMaxIn() {
		return maxIn;
	}

	public int getMinOut() {
		return minOut;
	}

	public int getMaxOut() {
		return maxOut;
	}

	public int getMinInPos() {
		return minInPos;
	}

	public int getMaxInPos() {
		return maxInPos;
	}

	public int getMinInNeg() {
		return minInNeg;
	}

	public int getMaxInNeg() {
		return maxInNeg;
	}

	public int getMinInUnk() {
		return minInUnk;
	}

	public int getMaxInUnk() {
		return maxInUnk;
	}

	public int getMinOutPos() {
		return minOutPos;
	}

	public int getMaxOutPos() {
		return maxOutPos;
	}

	public int getMinOutNeg() {
		return minOutNeg;
	}

	public int getMaxOutNeg() {
		return maxOutNeg;
	}

	public int getMinOutUnk() {
		return minOutUnk;
	}

	public int getMaxOutUnk() {
		return maxOutUnk;
	}

}
