package gov.anl.gss.sdm.server.model.analysis.trees;

import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;

import java.util.Map;

import org.apache.log4j.Logger;

public class VariableSubstitution {
static Logger log = Logger.getLogger(VariableSubstitution.class.getSimpleName());
	
	protected Map<String, Equation> equations;
	protected NativeDataTypeManager ndtm;
	protected String currentLHS;
	
	public VariableSubstitution(Map<String, Equation> equations) {
		this.equations = equations;
		ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
	}
	
	public void processAllStocks() {
		
		log.debug("@@@ All Variable Substitution @@@");

//		for (Equation eqn : equations.values()) {
//			eqn.printTreeTerse();
//		}
		for (Equation eqn : equations.values()) {
			if (eqn.isStock()) {
				substitute(eqn);
//				eqn.printTree();
//				log.debug("\nExp1: "+generateExpression(eqn.getTreeRoot())+"\n");
//				Node rootCopy = eqn.getCopyOfTree();
//				currentLHS = TreeTraversal.getLhs(rootCopy).getToken();
//				Node rhs = TreeTraversal.getRhs(rootCopy);
//				performVariableSubstitution(rhs);
////				Node.printTree(rootCopy);
//				
//				// We know this is a stock, so let's grab it's first argument
//				Node aNode = TreeTraversal.getRhs(rootCopy);
//				aNode = TreeTraversal.getFunctionArgument(aNode, 1);
//				log.debug("\nExp2: "+generateExpression(rootCopy)+"\n");
//				replaceNode(rhs, aNode);
//				
//				log.debug("Exp3: "+generateExpression(rootCopy)+"\n\n");
//				// we are interested only in the first argument
//				aNode.setNext(null);
			}
		}
	}
	
	public String substitute(Equation eqn) {
		log.debug("@@@ Variable Substitution Single Equation @@@");
		eqn.printTree();
		log.debug("\nExp1: "+generateExpression(eqn.getTreeRoot())+"\n");
		Node rootCopy = eqn.getCopyOfTree();
		currentLHS = TreeTraversal.getLhs(rootCopy).getToken();
		Node rhs = TreeTraversal.getRhs(rootCopy);
		performVariableSubstitution(rhs);
//		Node.printTree(rootCopy);
		
		// We know this is a stock, so let's grab it's first argument
		Node aNode = TreeTraversal.getRhs(rootCopy);
		aNode = TreeTraversal.getFunctionArgument(aNode, 1);
		log.debug("\nExp2: "+generateExpression(rootCopy)+"\n");
		replaceNode(rhs, aNode);
		aNode.setNext(null);
		log.debug("Exp3: "+generateExpression(rootCopy)+"\n\n");
		// we are interested only in the first argument
		return generateExpression(rootCopy);
		
	}
	
	public void replaceNode(Node substitutionNode, Node replacementNode) {
//		log.debug("Replace "+substitutionNode.getToken()+" with "+replacementNode.getToken());
		replacementNode.setParent(substitutionNode.getParent());
		replacementNode.setNext(substitutionNode.getNext());
		if (substitutionNode.getNext() != null)
			substitutionNode.getNext().setPrevious(replacementNode);
		if (substitutionNode.getPrevious() != null)
			substitutionNode.getPrevious().setNext(replacementNode);
		replacementNode.setPrevious(substitutionNode.getPrevious());
		// child is already set
		
		// is this right?
		if (substitutionNode.getParent().getChild().equals(substitutionNode))
			substitutionNode.getParent().setChild(replacementNode);
	}
	
	public void performVariableSubstitution(Node inputNode) {
		
		
		// first termination criteria: the node is null
		if (inputNode == null) {
//			log.debug("NULL");
			return;
		}
		
//		log.debug("performVariableSubstitution: "+inputNode.getToken());
		// substitutionNode is the node that would be replaced 
		Node substitutionNode = inputNode;
		
		if (Parser.isFunctionInvocation(inputNode.getToken())) {
			substitutionNode = TreeTraversal.getFunctionArgument(inputNode, 1);
//			log.debug("Skip ahead to "+substitutionNode.getToken());
		}
		
		// we will only perform a substituion on terminal nodes
		
		if (substitutionNode.isTerminal()) {

			// we need to know something about this node
			String possibleLHS = ndtm.getOriginalNameForSubstitution(substitutionNode.getToken());
			if (equations.containsKey(possibleLHS)) {
				// there is a equation tree for this node
				Equation eqn = equations.get(possibleLHS);

				// if this node is a stock, we are done
				if (eqn.isStock() && !possibleLHS.equals(currentLHS))
					return;
				
				// do not process getExternalData
				if (eqn.isGetExternalData())
					return;
				
				// we can substitute, get the rhs of the tree that will be inserted
				Node subRoot = equations.get(possibleLHS).getCopyOfTree();
				Node replacementNode = TreeTraversal.getRhs(subRoot);
				replaceNode(substitutionNode, replacementNode);
				
				// we now want to process this new tree
//				log.debug("Replacement Invocation");
				performVariableSubstitution(replacementNode);
				substitutionNode = replacementNode;
			}

		} else {
			// let's try it's child
//			log.debug("Child Invocation");
			substitutionNode = substitutionNode.getChild();
			performVariableSubstitution(substitutionNode);
			
//			performVariableSubstitution(substitutionNode.getChild());
		}
		
		// we always will try for a next sibling
		if (substitutionNode != null) {
			substitutionNode = substitutionNode.getNext();
//			log.debug("Next Invocation ");
			performVariableSubstitution(substitutionNode);
		}
		
	}
	
	public void performVariableSubstitutionOrig(Node inputNode) {
		
		if (inputNode == null)
			return;
		
		
//		inputNode.print("inputNode");
		
		// substitutionNode is the node that would be replaced 
		Node substitutionNode = inputNode;
		
		// if we are invoking a function, we need to jump to the first argument
		
		if (Parser.isFunctionInvocation(inputNode.getToken())) {
			substitutionNode = TreeTraversal.getFunctionArgument(inputNode, 1);
//			substitutionNode.print("function replacement");
		}
		
//		log.debug("pvs: "+substitutionNode.getToken());
		
		// substitute for this node
		String possibleLHS = ndtm.getOriginalNameForSubstitution(substitutionNode.getToken());
		if (equations.containsKey(possibleLHS) && !possibleLHS.equals(currentLHS) &&
				!equations.get(possibleLHS).isGetExternalData()) {
//			log.debug("Can sub for "+possibleLHS);
			
			// replacementNode is the node that will replace substitutionNode
			Node subRoot = equations.get(possibleLHS).getCopyOfTree();
			Node replacementNode = TreeTraversal.getRhs(subRoot);
			
//			log.debug("Replacing "+substitutionNode.getToken()+" with "+replacementNode.getToken());
			replaceNode(substitutionNode, replacementNode);
			
//			substitutionNode.print("substitutionNode");
//			
//			replacementNode.print("replacementNode 1");
			
			
			
//			replacementNode.print("replacementNode 2");
			
//			return;
		} else {
			// substitute for child
//			log.debug("Cannot sub, let's try child");
			performVariableSubstitutionOrig(substitutionNode.getChild());
		}
		
		// substitute for next
		if (substitutionNode != null) {
			substitutionNode = substitutionNode.getNext();
//			log.debug("Checking next: "+(substitutionNode != null ? substitutionNode.getToken() : "NULL"));
			performVariableSubstitutionOrig(substitutionNode);
		}
		
	}
	
	public String generateExpression(Node node) {
		StringBuilder sb = new StringBuilder();

		if (node != null) {
			

			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child  
			if (Parser.isArithmeticOperator(node.getToken()) || Parser.isEqualSign(node.getToken())
					|| Parser.isRelationalOperator(node.getToken())) {
				if (!Parser.isEqualSign(node.getToken()))
						sb.append("(");
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(generateExpression(TreeTraversal.getLhs(node))); // node.getChild()
					sb.append(ndtm.getOriginalNameForSubstitution(node.getToken()));
					sb.append(generateExpression(TreeTraversal.getRhs(node))); // node.getChild().getNext()
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(generateExpression(node.getChild())); // this is really a LHS location, but RHS of unary
				}
				if (!Parser.isEqualSign(node.getToken()))
						sb.append(")");
				return sb.toString();

			} else if (Parser.isFunctionInvocation(node.getToken())) {
//				sb.append(node.getToken());
				sb.append(generateFunctionCall(node));
			} else {
				sb.append(ndtm.getOriginalNameForSubstitution(node.getToken()));
			}

		}
		
		return sb.toString();
	}
	
	public String generateFunctionCall(Node node) {
		StringBuilder sb = new StringBuilder();
		
		if (node != null) {
			// create the entire invocation
			// node is the function name
			sb.append(ndtm.getOriginalNameForSubstitution(node.getToken()));
			sb.append("(");
			int i = 0;
			
			Node arg = TreeTraversal.getFunctionArgument(node, 1);
			while (arg != null) {
				if (i++ > 0)
					sb.append(",");
				sb.append(generateExpression(arg));
				arg = arg.getNext();
			}
			
			sb.append(")");
			
		}
		
		return sb.toString();
	}

}
