package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.shared.CustomGraphObject;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class BridgeCustomGraphManager {
	
	Map<String, CustomGraphObject> graphs;
	private ModelStructure modelStructure;
	private Set<String> requiredForCustomGraphs;

	public BridgeCustomGraphManager(ModelStructure modelStructure) {
		graphs = new LinkedHashMap<String, CustomGraphObject>();
		this.modelStructure = modelStructure;
		requiredForCustomGraphs = new LinkedHashSet<String>();
	}
	
	public void loadCustomGraphs(String file) {
		if (Utilities.fileExists(file)) {
			List<String> graphInfo = Utilities.getAsList(file);
			for (String line : graphInfo) {
				String[] l = line.split("\t");
				CustomGraphObject graph = new CustomGraphObject(l[0]);
				List<String> al = new ArrayList<String>();
				for (int i = 1; i < l.length; i++) {
					// need to verify that this variable actually exists
					// it may be a bad reference to a non-existant variable
					if (modelStructure.isKnownVariable(l[i])) {
						al.add(l[i]);
						requiredForCustomGraphs.add(l[i]);
					}
				}
				graph.setVariables(al);
				if (al.size() > 0)
					graphs.put(graph.getName(), graph);
			}
		}
	}
	
	public void addCustomGraph(CustomGraphObject cgo) {
		graphs.put(cgo.getName(), cgo);
	}
	
	public void save(String file) {
		BufferedWriter bw = Utilities.openFileForWriting(file);
		try {
			for (CustomGraphObject cgo : graphs.values())
				bw.append(cgo.asRawData());
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public List<CustomGraphObject> getCustomGraphs() {
		return new ArrayList<CustomGraphObject>(graphs.values());
	}

	public Set<String> getRequiredForCustomGraphs() {
		return requiredForCustomGraphs;
	}
}
