package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.shared.LoopObject;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class BridgeLoop {

	private List<String> loop;
	private List<String> linkPolarities;
	private List<String> linkThickness;
	
	private String id;
	private int length;
	private String polarity;
	private String extendedName = null;
	
//	Map<String, Map<String, String>> polarityMap;
	
	public BridgeLoop(String id, String length, String polarity) {
		this.id = id;
		this.length = Integer.parseInt(length);  // the initial var is repeated at end to close loop, but length considers 2 vars
		this.polarity = polarity;
		loop = new ArrayList<String>();
		linkPolarities = new ArrayList<String>();
		this.linkThickness = new ArrayList<String>();
	}
	
	public BridgeLoop(String id, String length, String polarity, List<String> vars, List<String> linkPolars, List<String> linkThickness) {
		this(id, length, polarity);
		loop = new ArrayList<String>(vars);
		linkPolarities = new ArrayList<String>(linkPolars);
		this.linkThickness = new ArrayList<String>(linkThickness);
		
	}
	
	public void addLinkPolarity( Map<String, Map<String, String>> polarity) {
		if (polarity.size() == 0) {
			for (int i = 0; i < loop.size()-1; i++) {
				linkPolarities.add("?");
			}
		} else {
		for (int i = 0; i < loop.size()-1; i++) {
			linkPolarities.add(polarity.get(loop.get(i)).get(loop.get(i+1)));
		}
		}
//		linkPolarities.add(polarity.get(loop.get(loop.size()-1)).get(loop.get(0)));
	}
	
	public void addLinkThickness( Map<String, Map<String, String>> linkThickness) {
		if (linkThickness.size() == 0) {
			for (int i = 0; i < loop.size()-1; i++) {
				this.linkThickness.add("1");
			}
		} else {
			for (int i = 0; i < loop.size()-1; i++) {
				if (linkThickness.containsKey(loop.get(i))) {
					if (linkThickness.get(loop.get(i)).containsKey(loop.get(i+1))) {
						Map<String, String> map = linkThickness.get(loop.get(i));
						String t = map.get(loop.get(i+1));
						this.linkThickness.add(t);
					} else {
						this.linkThickness.add("1");
					}
				} else {
					this.linkThickness.add("1");
				}
			}
		}
		//		linkPolarities.add(polarity.get(loop.get(loop.size()-1)).get(loop.get(0)));
	}
	
	public void addVar(String var) {
		loop.add(var);
	}

	/**
	 * @return the loop
	 */
	public List<String> getLoop() {
		return loop;
	}

	/**
	 * @return the id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return the length
	 */
	public int getLength() {
		return length;
	}

	/**
	 * @return the polarity
	 */
	public String getPolarity() {
		return polarity;
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		
		sb.append("ID: "+getId()+" length: "+getLength()+" polarity: "+polarity+"\n");
		for (String var : loop) {
			sb.append("  "+var+"\n");
		}
		
		return sb.toString();
	}
	
	public BridgeLoop getLoopStartingAt(String variable) {
		List<String> al = new ArrayList<String>();
		List<String> alp = new ArrayList<String>();
		List<String> alt = new ArrayList<String>();
		
		if (loop.contains(variable)) {
			int first = loop.indexOf(variable);
			for (int i = first; i < loop.size()-1; i++) {
				al.add(loop.get(i));
				alp.add(linkPolarities.get(i));
				alt.add(linkThickness.get(i));
			}
			for (int i = 0; i <= first; i++) {
				al.add(loop.get(i));
				alp.add(linkPolarities.get(i));
				alt.add(linkThickness.get(i));
			}
			BridgeLoop bl = new BridgeLoop(id, Integer.toString(length), polarity, al, alp, alt);
			bl.setExtendedName(extendedName);
			return bl;
		} else {
			return null;
		}
	}
	
	public LoopObject asLoopObject() {
		String name = id;
		LoopObject lo = new LoopObject(name);
		lo.setExtendedName(extendedName);
		lo.setVariable(loop.get(0));
		lo.setLoopPolarity(polarity);
		List<String> intermediates = new ArrayList<String>();
		for (int i = 1; i < loop.size()-1; i++)
			intermediates.add(loop.get(i));
		lo.setIntermediates(intermediates);
		
		List<String> polars = new ArrayList<String>(linkPolarities);
		lo.setLinkPolarities(polars);
		
		List<String> thicks = new ArrayList<String>(linkThickness);
		lo.setLinkThickness(thicks);
		return lo;
	}

	public String getExtendedName() {
		return extendedName;
	}

	public void setExtendedName(String extendedName) {
		this.extendedName = extendedName;
	}
}
