package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.support.Utilities;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class BridgeLoopManager {
	
	private Map<String, List<BridgeLoop>> loopMembershipNoIVV;
	
	private List<BridgeLoop> allLoops = new ArrayList<BridgeLoop>();
	
	private Map<String, Map<String, String>> arrowThickness;
	private Map<String,String> extendedLoopNames;
	
	public BridgeLoopManager() {
		loopMembershipNoIVV = new LinkedHashMap<String, List<BridgeLoop>>();
	}
	
	public List<BridgeLoop> orderByLoopLength(List<BridgeLoop> list) {
		List<BridgeLoop> al = new ArrayList<BridgeLoop>(list);
//		for (Loop l : al) {
//			System.out.println(l);
//		}
		try {
			Collections.sort(al, new Comparator<BridgeLoop>() {
			    @Override
			    public int compare(BridgeLoop arg0, BridgeLoop arg1) {
			    	try {
						if (arg0.getLength() == arg1.getLength()) {
							return 0;
						} else {
							return arg0.getLength() < arg1.getLength() ? -1 : 1;
						}
						
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
			    }
			  });
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return al;
	}
	
	public void loadLoops(String file,  Map<String, Map<String, String>> polarity) {
		List<String> loopInfo = Utilities.getAsList(file);
		for (String line : loopInfo) {
			String[] l = line.split("\t");
			BridgeLoop loop = new BridgeLoop(l[0], l[1], l[2]);
			allLoops.add(loop);
			for (int i = 3; i < l.length; i++) {
				loop.addVar(l[i]);
				if (i < l.length-1) {
				if (!loopMembershipNoIVV.containsKey(l[i])) {
					loopMembershipNoIVV.put(l[i], new ArrayList<BridgeLoop>());
				}
					loopMembershipNoIVV.get(l[i]).add(loop);
				}
			}
			loop.addLinkPolarity(polarity);
			loop.addLinkThickness(arrowThickness);
			if (extendedLoopNames.containsKey(l[0]))
				loop.setExtendedName(extendedLoopNames.get(l[0]));
		}
	}
	
	public List<BridgeLoop> getLoops() {
		return orderByLoopLength(allLoops);
	}
	
	public List<BridgeLoop> getLoopsWithVariable(String variable) {
		if (!loopMembershipNoIVV.containsKey(variable))
            return new ArrayList<BridgeLoop>();

		return orderByLoopLength(loopMembershipNoIVV.get(variable));
		
	}
	
	public List<BridgeLoop> getLoopsOfLength(int length) {
		List<BridgeLoop> al = new ArrayList<BridgeLoop>();
		
		for (BridgeLoop loop : allLoops) {
			if (loop.getLength() == length)
				al.add(loop);
		}
		
		return al;
	}

	public Map<String, Map<String, String>> getArrowThickness() {
		return arrowThickness;
	}

	public void setArrowThickness(Map<String, Map<String, String>> arrowThickness) {
		this.arrowThickness = arrowThickness;
	}

	public Map<String, String> getExtendedLoopNames() {
		return extendedLoopNames;
	}

	public void setExtendedLoopNames(Map<String, String> extendedLoopNames) {
		this.extendedLoopNames = extendedLoopNames;
	}

}
