package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

public class BridgeModelMeta {
	
	private String mdlFile;
	private String className;
	private boolean map;
	private boolean runnable;
	
	private String rootDirectory;
	private String model;
	
	private String initialTime;
	private String finalTime;
	private String dt;
	private String saveper;
	
	private String timeUnits;
	
	private String metaFile;
	
	
	public BridgeModelMeta(String structureDirectory, String model) {
		this.rootDirectory = structureDirectory;
		this.metaFile = rootDirectory+"/modelMeta.tab";
		this.model = model;
		File f = new File(metaFile);
		if (!f.exists()) {
			System.out.println("new meta file "+metaFile);
		}
		
	}

	public String getModellRunFileName(String runName) {
		return  rootDirectory +"/runs/"+runName+".tab";
	}
	
	public String getModelVariablesFileName() {
		return  rootDirectory + "/structure/variables.tab";
	}
	
	public boolean load() {
		File f = new File(metaFile);
		if (!f.exists()) {
			System.out.println("Meta file not found "+metaFile);
			return false;
		}
		for (String line : Utilities.getAsList(metaFile)) {
			String[] parts = line.split("\t");
			if (parts[0].equalsIgnoreCase("MDL_FILE"))
				mdlFile = parts[1];
			else if (parts[0].equalsIgnoreCase("MAP"))
				map = Boolean.parseBoolean(parts[1]);
			else if (parts[0].equalsIgnoreCase("RUNNABLE"))
				runnable = Boolean.parseBoolean(parts[1]);
			else if (parts[0].equalsIgnoreCase("INITIAL TIME"))
				initialTime = parts[1];
			else if (parts[0].equalsIgnoreCase("FINAL TIME"))
				finalTime = parts[1];
			else if (parts[0].equalsIgnoreCase("DT"))
				dt = parts[1];
			else if (parts[0].equalsIgnoreCase("SAVEPER"))
				saveper = parts[1];
			else if (parts[0].equalsIgnoreCase("TIME UNITS")) {
				if (parts.length > 1)
					timeUnits = parts[1];
				else
					timeUnits = "Not Specified";
			} 
		}
		
		this.className = Utilities.convertModelNameToClassName(mdlFile);
		
		return true;
	}
	
	public boolean save() {
		
		BufferedWriter bw = Utilities.openFileForWriting(metaFile);
		try {
			bw.append("MDL_FILE"+"\t"+mdlFile+"\n");
			bw.append("MAP"+"\t"+Boolean.toString(map)+"\n");
			bw.append("RUNNABLE"+"\t"+Boolean.toString(runnable)+"\n");
			bw.append("INITIAL TIME"+"\t"+initialTime+"\n");
			bw.append("FINAL TIME"+"\t"+finalTime+"\n");
			bw.append("DT"+"\t"+dt+"\n");
			bw.append("SAVEPER"+"\t"+saveper+"\n");
			bw.append("TIME UNITS"+"\t"+timeUnits+"\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
		
	}
	
	public String getModelName() {
		// return the model name -- which is mdl file without extension
		String modelName = mdlFile.replaceAll("\\.[Mm][Dd][Ll]", "");
		return modelName;
	}

	/**
	 * @return the mdlFile
	 */
	public String getMdlFile() {
		return mdlFile;
	}

	/**
	 * @param mdlFile the mdlFile to set
	 */
	public void setMdlFile(String mdlFile) {
		this.mdlFile = mdlFile;
		this.className = Utilities.convertModelNameToClassName(mdlFile);
	}

	/**
	 * @return the map
	 */
	public boolean isMap() {
		return map;
	}

	/**
	 * @param map the map to set
	 */
	public void setMap(boolean map) {
		this.map = map;
	}

	/**
	 * @return the runnable
	 */
	public boolean isRunnable() {
		return runnable;
	}

	/**
	 * @param runnable the runnable to set
	 */
	public void setRunnable(boolean runnable) {
		this.runnable = runnable;
	}

	/**
	 * @return the metaFile
	 */
	public String getMetaFile() {
		return metaFile;
	}

	/**
	 * @param metaFile the metaFile to set
	 */
	public void setMetaFile(String metaFile) {
		this.metaFile = metaFile;
	}

	/**
	 * @return the initialTime
	 */
	public String getInitialTime() {
		return initialTime;
	}

	/**
	 * @param initialTime the initialTime to set
	 */
	public void setInitialTime(String initialTime) {
		this.initialTime = initialTime;
	}

	/**
	 * @return the finalTime
	 */
	public String getFinalTime() {
		return finalTime;
	}

	/**
	 * @param finalTime the finalTime to set
	 */
	public void setFinalTime(String finalTime) {
		this.finalTime = finalTime;
	}

	/**
	 * @return the dt
	 */
	public String getDt() {
		return dt;
	}

	/**
	 * @param dt the dt to set
	 */
	public void setDt(String dt) {
		this.dt = dt;
	}

	/**
	 * @return the saveper
	 */
	public String getSaveper() {
		return saveper;
	}

	/**
	 * @param saveper the saveper to set
	 */
	public void setSaveper(String saveper) {
		this.saveper = saveper;
	}

	/**
	 * @return the timeUnits
	 */
	public String getTimeUnits() {
		return timeUnits;
	}

	/**
	 * @param timeUnits the timeUnits to set
	 */
	public void setTimeUnits(String timeUnits) {
		this.timeUnits = timeUnits;
	}

	/**
	 * @return the rootDirectory
	 */
	public String getRootDirectory() {
		return rootDirectory;
	}

	/**
	 * @param rootDirectory the rootDirectory to set
	 */
	public void setRootDirectory(String rootDirectory) {
		this.rootDirectory = rootDirectory;
	}

	/**
	 * @return the model
	 */
	public String getModel() {
		return model;
	}

	/**
	 * @param model the model to set
	 */
	public void setModel(String model) {
		this.model = model;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

}
