package gov.anl.gss.sdm.server.model.bridge;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class BridgeModelResults {
	
	private final static String UNKNOWN = "Unknown";
	
	private String filename;
	private String scenario;
	private double startTime;
	private double endTime;
	private double timeStep;
	private double dt;
	
	private String fileFormat = UNKNOWN;
	
	
//	private Date runDate;
	
	private boolean ok = false;
	
	private List<Double> timeVector;
	
	private Map<String, List<Double>> values;
	
	public BridgeModelResults() {
		values = new LinkedHashMap<String, List<Double>>();
	}
	
	public BridgeModelResults(BridgeRunValidator brv) {
		values = brv.getValues();
		timeVector = brv.getTimeVector();
		filename = brv.getRunFile();
		BridgeModelMeta bmm = brv.getMeta();
		startTime = Double.parseDouble(bmm.getInitialTime());
		endTime = Double.parseDouble(bmm.getFinalTime());
		timeStep = Double.parseDouble(bmm.getDt());
		dt = Double.parseDouble(bmm.getDt());
	}
	
	/**
	 * For existing model runs on disk
	 * @param filename
	 */
	public BridgeModelResults(String filename) {
		this();
		this.filename = filename;
		BridgeModelResultsLoader bmrl = new BridgeModelResultsLoader();
		ok = bmrl.load(filename);
		if (ok) {
			timeVector = bmrl.getTimeVector();
			values = bmrl.getValues();
		}
	}
	
	/**
	 * For a new model run
	 * @param filename
	 * @param scenario
	 * @param startTime
	 * @param endTime
	 * @param timeStep
	 * @param dt
	 */
	public BridgeModelResults(
			String filename,
			String scenario,
			double startTime,
			double endTime,
			double timeStep,
			double dt
			) {
		this();
		this.filename = filename;
		this.scenario = scenario;
		this.startTime = startTime;
		this.endTime = endTime;
		this.timeStep = timeStep;
		this.dt = dt;
		
		
	}
	
	public void addValue(String variable, double value) {
		values.get(variable).add(value);
	}
	

	
	private int mapTimeToIndex(double time) {
		for (int i = 0; i < timeVector.size(); i++)
			if (timeVector.get(i).doubleValue() == time)
				return i;
		return 0;
	}
	
	public List<Double> getValues(String variable) {
		return values.get(variable);
	}
	
	public double getValue(String variable, double time) {
		if (values == null) {
			System.out.println("BMR values null "+variable+" @ "+time);
			return 0.0;
		}
		if (values.get(variable) == null) {
			System.out.println("BMR missing variable"+variable+" @ "+time);
			return 0.0;
		}
		return values.get(variable).get(mapTimeToIndex(time));
	}
	
	public Map<String, Double> getAllVariableValues(double time, Set<String> selected) {
		
		Map<String, Double> map = new LinkedHashMap<String, Double>();
		int index = mapTimeToIndex(time);
		
		for (String var : values.keySet()) {
			if (!selected.contains(var))
				continue;
			if (values.get(var).size() < index) {
				map.put(var, values.get(var).get(index));
			} else {
				break;
			}
		}
		
		return map;
		
	}
	
	public double[] getValuesAsArray(String variable) {
		List<Double> list = values.get(variable);
		double[] cv = new double[list.size()];
			for (int i = 0; i < list.size(); i++) {
				cv[i] = list.get(i);
			}
		return cv;
		
	}
	
	Map<String, List<Double>> getAllVariableValues() {
		return values;
	}

	/**
	 * @return the filename
	 */
	public String getFilename() {
		return filename;
	}

	/**
	 * @param filename the filename to set
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/**
	 * @return the scenario
	 */
	public String getScenario() {
		return scenario;
	}

	/**
	 * @param scenario the scenario to set
	 */
	public void setScenario(String scenario) {
		this.scenario = scenario;
	}

	/**
	 * @return the startTime
	 */
	public double getStartTime() {
		return startTime;
	}

	/**
	 * @param startTime the startTime to set
	 */
	public void setStartTime(double startTime) {
		this.startTime = startTime;
	}

	/**
	 * @return the endTime
	 */
	public double getEndTime() {
		return endTime;
	}

	/**
	 * @param endTime the endTime to set
	 */
	public void setEndTime(double endTime) {
		this.endTime = endTime;
	}

	/**
	 * @return the timeStep
	 */
	public double getTimeStep() {
		return timeStep;
	}

	/**
	 * @param timeStep the timeStep to set
	 */
	public void setTimeStep(double timeStep) {
		this.timeStep = timeStep;
	}

	/**
	 * @return the dt
	 */
	public double getDt() {
		return dt;
	}

	/**
	 * @param dt the dt to set
	 */
	public void setDt(double dt) {
		this.dt = dt;
	}

	/**
	 * @return the cleanLoad
	 */
	public boolean isOk() {
		return ok;
	}

	/**
	 * @return the timeVector
	 */
	public List<Double> getTimeVector() {
		return timeVector;
	}

	/**
	 * @param timeVector the timeVector to set
	 */
	public void setTimeVector(List<Double> timeVector) {
		this.timeVector = timeVector;
	}

	public List<String> getVariables() {
		return new ArrayList<String>(values.keySet());
	}

	/**
	 * @return the fileFormat
	 */
	public String getFileFormat() {
		return fileFormat;
	}
}
