package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class BridgeModelResultsLoader {

	private final static String SINGLE_VALUE_ROW = "Single Value Row";
	private final static String MULTIPLE_VARIABLE_ROW = "Multiple Variable Row";
	private final static String SINGLE_VARIABLE_ROW = "Single Variable Row";
	private final static String UNKNOWN = "Unknown";
	
	private List<Double> timeVector = new ArrayList<Double>();
	
	private Map<String, List<Double>> values = new LinkedHashMap<String, List<Double>>();
	
	private String fileFormat = UNKNOWN;
	
	private boolean ok = false;
	
	private String sep = "\t";
	
	public BridgeModelResultsLoader() {
		
	}
	
	public List<Double> getTimeVector() {
		return timeVector;
	}

	public Map<String, List<Double>> getValues() {
		return values;
	}

	public boolean load(String filename) {
		
		List<String> linesOfData = Utilities.getAsList(filename);
		if (linesOfData.size() == 0)
			return false;
		
		// determine the format of the input data file
		
		if (!linesOfData.get(0).contains(sep) && linesOfData.get(0).contains(","))
			sep = ",";
		String[] firstLine = linesOfData.get(0).split(sep);
		if (firstLine[0].equals("Variable") &&
				firstLine[1].equals("Time") &&
				firstLine[2].equals("Value")) {
			return loadSingleValuePerRow(linesOfData);
		} else if (firstLine[0].equals("Time") &&
				Parser.isNumber(firstLine[1])) {
			return loadSingleVariableRow(linesOfData);
		} else if (firstLine[0].equals("Time") &&
				!Parser.isNumber(firstLine[1])) {
				return loadMultipleVariableRow(linesOfData);
		} else {
			return false;
		}
		
	}
	
	private boolean loadSingleValuePerRow(List<String> linesOfData) {
		
		fileFormat = SINGLE_VALUE_ROW;
		String previousTime = "";
		boolean skip = true;
		for (String line : linesOfData) {
			if (skip) {
				skip = false;
				continue;
			}
			String[] v = line.split(sep);
			if (!values.containsKey(v[0])) {
				values.put(v[0], new ArrayList<Double>());
			}
			if (!previousTime.equals(v[1])) {
				timeVector.add(Double.parseDouble(v[1]));
				previousTime = v[1];
			}
			values.get(v[0]).add(Double.parseDouble(v[2]));
		}
		
		ok = true;
		return true;
	}
	
	private boolean loadSingleVariableRow(List<String> linesOfData) {
		
		// vensim data files for constants have only one value
		// We know Time is first, so numTimePeriods will be set correctly
		
		fileFormat = SINGLE_VARIABLE_ROW;
		int numTimePeriods = -1;
		
		for (String line : linesOfData) {
			String[] parts = line.split(sep);
			values.put(parts[0], new ArrayList<Double>());
			
			numTimePeriods = Math.max(numTimePeriods, parts.length-1); 
			
			for (int i = 1; i <= numTimePeriods; i++) {
				List<Double> list = values.get(parts[0]);
				if (i >= parts.length) {
					list.add(list.get(list.size()-1));					
				} else {
					list.add(Double.parseDouble(parts[i]));
				}
			}
		}
		timeVector = new ArrayList<Double>(values.get("Time"));
		return true;
	}
	
	private boolean loadMultipleVariableRow(List<String> linesOfData) {
		
		// vensim data files for constants have only one value
		
		fileFormat = MULTIPLE_VARIABLE_ROW;
		
		String[] firstLine = linesOfData.get(0).split(sep);
		for (String var : firstLine) {
			values.put(var, new ArrayList<Double>());
		}
		int numVars = firstLine.length;
		
		for (int l = 1; l < linesOfData.size(); l++) {
			String[] parts = linesOfData.get(l).split(sep);
			
			for (int i = 0; i < numVars; i++) {
				List<Double> list = values.get(firstLine[i]);
				if (i > parts.length-1 || parts[i].length() == 0) {
					list.add(list.get(list.size()-1));
				} else {
				list.add(Double.parseDouble(parts[i]));
				}
			}
		}
		timeVector = new ArrayList<Double>(values.get("Time"));
		
		return true;
	}
}
