package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.File;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

public class BridgeModelState {

	private SDMBridge bridge;
	
	private String className;

	private String selectedModel;
	private String selectedVignette;
	private Set<String> selectedRuns;

	private Map<String, Boolean> runLoaded;
	private boolean modelSelected = true;
	private boolean vignetteSelected = true;
	private boolean runsSelected = true;

	private boolean modelRunnable;
	private String modelRunName;
	private boolean modelInitialized = true;
	private boolean modelRunning;
	private boolean modelRunComplete;
	private boolean metaLoaded = true;
	private boolean structureLoaded = true;
	
	private boolean modelInstantiated;

	private double currentTime;
	private double initialTime;
	private double finalTime;
	private double dt;
	
	private Set<String> availableRuns = new LinkedHashSet<String>();
	private Set<String> availableVignettes = new LinkedHashSet<String>();

	public BridgeModelState(SDMBridge bridge) {
		this.bridge = bridge;
	}
	
	public void extractFromMeta(BridgeModelMeta meta) {
		modelRunnable = meta.isRunnable();
		initialTime = Double.parseDouble(meta.getInitialTime());
		finalTime = Double.parseDouble(meta.getFinalTime());
		if (Parser.isNumber(meta.getDt())) {
			dt = Double.parseDouble(meta.getDt());
		} else {
			
		}
	}
	
	public void newRun() {
		modelInitialized = false;
		modelRunning = false;
		modelRunComplete = false;
	}
	
	public void addAvailableRun(String runName) {
		availableRuns.add(runName);
		runLoaded.put(runName, new Boolean(false));
	}
	
	public void addAvailableVignette(String vignette) {
		availableRuns.add(vignette);
	}

	public boolean canSelectModel() { 
		return true; 
	}

	public boolean canSelectVignette(String vignette) { 
		return modelSelected; 
	}
	public boolean canSelectRun() { 
		return modelSelected; 
	}
	public boolean canCreateRun() { 
		return modelSelected; 
	}
	public boolean canDone() { 
		return modelSelected; 
	}
	public boolean canStepModel() {
		return modelRunning;
	}
	public boolean canBackstepModel() {
		return modelRunning && currentTime > 0;
	}
	public boolean canGetModelLeverVariables() {
		return modelSelected && structureLoaded;
	}
	public boolean canGetModelOutputVariables() {
		return modelSelected && structureLoaded;
	}
	public boolean canGetTimeStep() {
		return metaLoaded;
	}
	public boolean canGetDT() {
		return metaLoaded;
	}
	public boolean canGetCurrentTime() {
		return metaLoaded;
	}
	public boolean canGetTimeUnits() {
		return metaLoaded;
	}
	public boolean canGetVariableValues(String scenario, String variable) {
		return modelSelected && runsSelected;
	}
	public boolean canGetVariableValue(String scenario, String variable, double time) {
		return modelSelected && runsSelected;
	}
	public boolean canGetAllVariableValues(String scenario, double time, Set<String> selectedVariables) {
		return modelSelected && runsSelected;
	}
	public boolean canGetStartTime() {
		return metaLoaded;
	}
	public boolean canGetEndTime() {
		return metaLoaded;
	}
	public boolean canGetAvailableModels(String file) {
		File f = new File(file);
		return f.exists() && f.isDirectory();
	}
	public boolean canGetVignettes(String file) {
		File f = new File(file);
		return modelSelected && f.exists() && f.isDirectory();
	}
	public boolean canGetRuns(String file) {
		File f = new File(file);
		return modelSelected && f.exists() && f.isDirectory();
	}
	public boolean canRun() {
		return modelRunnable;
	}
	
	public boolean canRunTo(double time) {
		return modelRunnable;
	}
	public boolean canGetAvailableViewNames() {
		return true;
//		return modelSelected && structureLoaded;
	}
	public boolean canGetView(String name) {
		return modelSelected && structureLoaded;
	}
	public boolean canSetValue(String variableName, double value) {
		return true;
//		return modelRunning || modelInitialized || modelInstantiated;
	}
	public boolean canGetCurrentValues(String[] variables) {
		return true;
//		return modelRunning || modelInitialized || modelInstantiated;
	}
	public boolean canGetValues(String[] variables) {
		return true;
//		return modelRunning || modelInitialized || modelInstantiated;
	}
	public boolean canGetValues(String[] variables, double time) {
		return true;
//		return ((modelRunning || modelRunComplete || modelInstantiated) && time <= currentTime) ||
//				(modelInitialized && time == initialTime);
	}
	public boolean canGetValues(String runName, String[] variable, double time) {
		return true;
//		return ((modelRunning || modelRunComplete) && time <= currentTime) ||
//				(modelInitialized && time == initialTime);
	}
	public boolean canGetVignetteVariables() {
		return modelSelected && vignetteSelected;
	}
	public boolean canCreateNewVignette(String vignette) {
		return modelSelected;
	}
	public boolean canGetAllVariableValues(String run) {
		return modelSelected && selectedRuns.contains(run);
	}
	public boolean canSaveSelectedVignette() {
		return true;
	}
	public boolean canModelIsRunnable() {
		return modelSelected && metaLoaded;
	}
	public boolean canModelIsMap() {
		return modelSelected && metaLoaded;
	}
	public boolean canGetTimeVector() {
		return modelSelected;
	}
	public boolean canGetTimeVector(String runName) {
		return modelSelected && selectedRuns.contains(runName);
	}
	public boolean canGetSavePer() {
		return modelSelected && metaLoaded;
	}
	public boolean canTunTo(double time) {
		return modelSelected && modelRunning;
	}
	public boolean canStartExecution() {
		return modelSelected && !modelInitialized;
	}
	public boolean canGetArrowsWithTail(String variable) {
		return modelSelected && structureLoaded;
	}
	public boolean canGetArrowsWithHead(String variable) {
		return modelSelected && structureLoaded;
	}
	public boolean canGetLoops() {
		return modelSelected && structureLoaded;
	}
	public boolean canGetLoopsWithVariable(String variable) {
		return modelSelected && structureLoaded;
	}
	public boolean canGetLoopsOfLength(int length) {
		return modelSelected && structureLoaded;
	}
	public boolean canCloseSelectedModel() {
		return modelRunComplete || !modelInitialized;
	}

	public boolean canGetSdmdocHtmlFile(String file) {
		File f = new File(file);
		return f.exists() && f.canRead();
	}
	public void stepTime() {
		currentTime += 1;
	}

	public void backstepTime() {
		currentTime -= 1;
	}
	/**
	 * @return the selectedModel
	 */
	 public String getSelectedModel() {
		return selectedModel;
	 }
	 /**
	  * @param selectedModel the selectedModel to set
	  */
	 public void setSelectedModel(String selectedModel) {
		 this.selectedModel = selectedModel;
	 }
	 /**
	  * @return the selectedVignette
	  */
	 public String getSelectedVignette() {
		 return selectedVignette;
	 }
	 /**
	  * @param selectedVignette the selectedVignette to set
	  */
	 public void setSelectedVignette(String selectedVignette) {
		 this.vignetteSelected = true;
		 this.selectedVignette = selectedVignette;
	 }
	 /**
	  * @return the selectedRuns
	  */
	 public Set<String> getSelectedRuns() {
		 return selectedRuns;
	 }
	 /**
	  * @param selectedRuns the selectedRuns to set
	  */
	 public void setSelectedRuns(Set<String> selectedRuns) {
		 this.selectedRuns = selectedRuns;
	 }
	 /**
	  * @return the runLoaded
	  */
	 public Map<String, Boolean> getRunLoaded() {
		 return runLoaded;
	 }
	 /**
	  * @param runLoaded the runLoaded to set
	  */
	 public void setRunLoaded(Map<String, Boolean> runLoaded) {
		 this.runLoaded = runLoaded;
	 }
	 /**
	  * @return the modelSelected
	  */
	 public boolean isModelSelected() {
		 return modelSelected;
	 }
	 /**
	  * @param modelSelected the modelSelected to set
	  */
	 public void setModelSelected(boolean modelSelected) {
		 this.modelSelected = modelSelected;
	 }
	 /**
	  * @return the vignetteSelected
	  */
	 public boolean isVignetteSelected() {
		 return vignetteSelected;
	 }
	 /**
	  * @param vignetteSelected the vignetteSelected to set
	  */
	 public void setVignetteSelected(boolean vignetteSelected) {
		 this.vignetteSelected = vignetteSelected;
	 }
	 /**
	  * @return the runsSelected
	  */
	 public boolean isRunsSelected() {
		 return runsSelected;
	 }
	 /**
	  * @param runsSelected the runsSelected to set
	  */
	 public void setRunsSelected(boolean runsSelected) {
		 this.runsSelected = runsSelected;
	 }
	 /**
	  * @return the modelRunnable
	  */
	 public boolean isModelRunnable() {
		 return modelRunnable;
	 }
	 /**
	  * @param modelRunnable the modelRunnable to set
	  */
	 public void setModelRunnable(boolean modelRunnable) {
		 this.modelRunnable = modelRunnable;
	 }
	 /**
	  * @return the modelRunName
	  */
	 public String getModelRunName() {
		 return modelRunName;
	 }
	 /**
	  * @param modelRunName the modelRunName to set
	  */
	 public void setModelRunName(String modelRunName) {
		 this.modelRunName = modelRunName;
	 }
	 /**
	  * @return the modelInitialized
	  */
	 public boolean isModelInitialized() {
		 return modelInitialized;
	 }
	 /**
	  * @param modelInitialized the modelInitialized to set
	  */
	 public void setModelInitialized(boolean modelInitialized) {
		 this.modelInitialized = modelInitialized;
	 }
	 /**
	  * @return the modelRunning
	  */
	 public boolean isModelRunning() {
		 return modelRunning;
	 }
	 /**
	  * @param modelRunning the modelRunning to set
	  */
	 public void setModelRunning(boolean modelRunning) {
		 this.modelRunning = modelRunning;
	 }
	 /**
	  * @return the modelRunComplete
	  */
	 public boolean isModelRunComplete() {
		 return modelRunComplete;
	 }
	 /**
	  * @param modelRunComplete the modelRunComplete to set
	  */
	 public void setModelRunComplete(boolean modelRunComplete) {
		 this.modelRunComplete = modelRunComplete;
	 }

	 /**
	  * @return the currentTime
	  */
	 public double getCurrentTime() {
		 return currentTime;
	 }

	 /**
	  * @param currentTime the currentTime to set
	  */
	 public void setCurrentTime(double currentTime) {
		 this.currentTime = currentTime;
	 }

	 /**
	  * @return the initialTime
	  */
	 public double getInitialTime() {
		 return initialTime;
	 }

	 /**
	  * @param initialTime the initialTime to set
	  */
	 public void setInitialTime(double initialTime) {
		 this.initialTime = initialTime;
	 }

	 /**
	  * @return the finalTime
	  */
	 public double getFinalTime() {
		 return finalTime;
	 }

	 /**
	  * @param finalTime the finalTime to set
	  */
	 public void setFinalTime(double finalTime) {
		 this.finalTime = finalTime;
	 }

	 /**
	  * @return the dt
	  */
	 public double getDt() {
		 return dt;
	 }

	 /**
	  * @param dt the dt to set
	  */
	 public void setDt(double dt) {
		 this.dt = dt;
	 }

	 /**
	  * @return the metaLoaded
	  */
	 public boolean isMetaLoaded() {
		 return metaLoaded;
	 }

	 /**
	  * @param metaLoaded the metaLoaded to set
	  */
	 public void setMetaLoaded(boolean metaLoaded) {
		 this.metaLoaded = metaLoaded;
	 }

	 /**
	  * @return the structureLoaded
	  */
	 public boolean isStructureLoaded() {
		 return structureLoaded;
	 }

	 /**
	  * @param structureLoaded the structureLoaded to set
	  */
	 public void setStructureLoaded(boolean structureLoaded) {
		 this.structureLoaded = structureLoaded;
	 }

	/**
	 * @return the bridge
	 */
	public SDMBridge getBridge() {
		return bridge;
	}

	/**
	 * @return the availableRuns
	 */
	public Set<String> getAvailableRuns() {
		return availableRuns;
	}

	/**
	 * @return the availableVignettes
	 */
	public Set<String> getAvailableVignettes() {
		return availableVignettes;
	}

	/**
	 * @return the modelInstantiated
	 */
	public boolean isModelInstantiated() {
		return modelInstantiated;
	}

	/**
	 * @param modelInstantiated the modelInstantiated to set
	 */
	public void setModelInstantiated(boolean modelInstantiated) {
		this.modelInstantiated = modelInstantiated;
	}

	public String getClassName() {
		return className;
	}

	public void setClassName(String className) {
		this.className = className;
	}

}
