package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.shared.Lever;
import gov.anl.gss.sdm.shared.Output;

import java.io.BufferedWriter;
import java.io.IOException;

public class BridgeModelVariable {

	protected String name;
	protected String description;
	protected String units;
	protected String minVal;
	protected String maxVal;
	protected String lever;
	protected String type;

	public BridgeModelVariable(Equation eqn) {
		this.name = eqn.getOriginalLhs();
		this.description = eqn.getComment();
		this.units = eqn.getUnits();
		this.minVal = eqn.getMinValFromUnits();
		this.maxVal = eqn.getMaxValFromUnits();
		this.lever = Boolean.toString(eqn.isGameOrConstant());
		this.type = eqn.getStockFlowAuxiliaryType();
		limitLevers();
	}

	public BridgeModelVariable(String name, String description, String units, String minVal, String maxVal, String lever,
			String type) {
		
		this.name = name;
		this.description = description;
		this.units = units;
		this.minVal = minVal;
		this.maxVal = maxVal;
		this.lever = lever;
		this.type = type;
		limitLevers();

	}
	
	public BridgeModelVariable(BridgeModelVariable another) {
		this.name = another.getName();
		this.description = another.getDescription();
		this.units = another.getUnits();
		this.minVal = another.getMinVal();
		this.maxVal = another.getMaxVal();
		this.lever = another.getLever();
		this.type = another.getType();
		limitLevers();
	}
	
	private void limitLevers() {
		String[] notLevers = {"time", "time step", "timestep", "time_step", "initial time", "initial_time",
				"final time", "final_time", "saveper"};
		for (String no : notLevers) {
			if (name.equalsIgnoreCase(no))
				lever = Boolean.toString(false);
		}
	}
	
// 	public Lever(int position,String name, String description,String units, 
	//       double minVal, double maxVal, int increment,String displayName, boolean editable,boolean included,
	//  String type) {
	public Lever asLever(int position) {
		return new Lever(position, name, description, units, getMinValAsDouble(), getMaxValAsDouble(), 0, name, units, false, true, type);
	}
	
	public Output asOutput(int position) {
		return new Output(position, name,  description, units, name, units, true, type);
		
	}
	
	public void saveAsVignette(BufferedWriter bw, int pos) {
		try {
			bw.append(name+"\t");
			if (description != null)
				bw.append(description.replace("\t", " ")+"\t");
			else
				bw.append("\t");
			bw.append(units+"\t");
			bw.append(/*minVal+*/"\t");
			bw.append(/*maxVal+*/"\t");
			bw.append(lever+"\t");
			bw.append(pos+"\t");
			bw.append("1\t");
			bw.append(name+"\t");
			bw.append(units+"\t");
			bw.append("TRUE\t");
			bw.append("TRUE\t");
			bw.append(type);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void saveWithType(BufferedWriter bw) {
		try {
//			bw.append(Utilities.nameToID(name)+"\t");
			bw.append(name+"\t");
			if (description != null)
				bw.append(description.replace("\t", " ")+"\t");
			else
				bw.append("\t");
			bw.append(units+"\t");
			bw.append(minVal+"\t");
			bw.append(maxVal+"\t");
			bw.append(lever+"\t");
			bw.append(type);
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void save(BufferedWriter bw) {
		try {
			bw.append(name+"\t");
			if (description != null)
				bw.append(description.replace("\t", " ")+"\t");
			else
				bw.append("\t");
			bw.append(units+"\t");
			bw.append(minVal+"\t");
			bw.append(maxVal+"\t");
			bw.append(lever+"\t");
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * @return the lever
	 */
	public boolean isLever() {
		return Boolean.parseBoolean(lever);
	}


	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}


	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}


	/**
	 * @return the description
	 */
	public String getDescription() {
		return description;
	}


	/**
	 * @param description the description to set
	 */
	public void setDescription(String description) {
		this.description = description;
	}


	/**
	 * @return the units
	 */
	public String getUnits() {
		return units;
	}


	/**
	 * @param units the units to set
	 */
	public void setUnits(String units) {
		this.units = units;
	}


	/**
	 * @return the minVal
	 */
	public double getMinValAsDouble() {
		if (minVal.trim().length() == 0) {
			return Double.MIN_VALUE;
		} else {
			return Double.parseDouble(minVal);
		}
	}


	/**
	 * @param minVal the minVal to set
	 */
	public void setMinVal(String minVal) {
		this.minVal = minVal;
	}


	/**
	 * @return the maxVal
	 */
	public double getMaxValAsDouble() {
		if (maxVal.trim().length() == 0) {
			return Double.MAX_VALUE;
		} else {
			return Double.parseDouble(maxVal);
		}
	}


	/**
	 * @param maxVal the maxVal to set
	 */
	public void setMaxVal(String maxVal) {
		
		this.maxVal = maxVal;
	}

	/**
	 * @return the minVal
	 */
	public String getMinVal() {
		return minVal;
	}

	/**
	 * @return the maxVal
	 */
	public String getMaxVal() {
		return maxVal;
	}

	/**
	 * @return the lever
	 */
	public String getLever() {
		return lever;
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	/**
	 * @param type the type to set
	 */
	public void setType(String type) {
		this.type = type;
	}

}
