package gov.anl.gss.sdm.server.model.bridge;

import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.shared.Lever;
import gov.anl.gss.sdm.shared.Output;

import java.io.BufferedWriter;
import java.io.IOException;

/**
* @author Mary Ann Bitz
* Argonne National Laboratory
* Global Security Sciences Division 
*/
/**
 * Class to hold one of the variables used in a simulation model
 */
public class BridgeVariable extends BridgeModelVariable {
	protected String position;
	protected String increment;
	protected String displayName;
	protected String displayUnits;
	protected String editable;
	protected String included;
	
	public BridgeVariable(Lever lever) {
		super(lever.getName(), lever.getDescription(), lever.getUnits(), Double.toString(lever.getMinVal()), 
				Double.toString(lever.getMaxVal()), Boolean.toString(true), lever.getType());
		this.position = Integer.toString(lever.getPosition());
		this.increment = Integer.toString(lever.getIncrement());
		this.displayName = lever.getDisplayName();
		this.displayUnits = lever.getDisplayUnits();
		this.editable = Boolean.toString(lever.isEditable());
		this.included = Boolean.toString(lever.isIncluded());
	}
	
	public BridgeVariable(Output output) {
		super(output.getName(), output.getDescription(), output.getUnits(), "", 
				"", Boolean.toString(false), output.getType());
		this.position = Integer.toString(output.getPosition());
		this.increment = "1";
		this.displayName = output.getDisplayName();
		this.displayUnits = output.getDisplayUnits();
		this.editable = Boolean.toString(false);
		this.included = Boolean.toString(output.isIncluded());
	}
	
	public BridgeVariable(BridgeModelVariable another) {
		super(another);
		position = "0";
		increment = "1";
		displayName = another.getName();
		displayUnits = another.getUnits();
		editable = Boolean.toString(false);
		included = Boolean.toString(false);
	}

	public BridgeVariable(String name, String description,String units, 
			String minVal, String maxVal, String lever, String position, String increment,
			String displayName, String displayUnits, String editable,String included, String type) {
		
		super(name, description, units, minVal, maxVal, lever, type);
		this.position=position;
		this.increment = increment;
		this.displayName=displayName;
		this.displayUnits=displayUnits;
		this.editable=editable;
		this.included=included;
	}

	public int getIncrement() {
		return Integer.parseInt(increment);
	}
	public void setIncrement(int increment) {
		this.increment = Integer.toString(increment);
	}
	public String getDisplayName() {
		return displayName;
	}
	public String getDisplayUnits() {
		return displayUnits;
	}
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}
	public void setDisplayUnits(String displayUnits) {
		this.displayUnits = displayUnits;
	}
	public boolean isIncluded() {
		return Boolean.parseBoolean(included);
	}
	public void setIncluded(boolean included) {
		this.included = Boolean.toString(included);
	}
	public boolean isEditable() {
		return Boolean.parseBoolean(editable);
	}
	public void setEditable(boolean editable) {
		this.editable = Boolean.toString(editable);
	}

	public String toString(){
		return name;
	}
	public int getPosition() {
		return Integer.parseInt(position);
	}
	public void setPosition(int position) {
		this.position = Integer.toString(position);
	}
	
	public void save(BufferedWriter bw) {
		try {
			super.save(bw);
			bw.append(position+"\t");
			bw.append(increment+"\t");
			bw.append(displayName+"\t");
			bw.append(displayUnits+"\t");
			bw.append(editable+"\t");
			bw.append(included+"\t");
			bw.append(type+"\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		Variable(String position,String name, String description,String units, 
//				String minVal, String maxVal, String increment,String displayName, String editable,String included, String lever)
	}

	/**
	 * @return the editable
	 */
	public String getEditable() {
		return editable;
	}

	/**
	 * @param editable the editable to set
	 */
	public void setEditable(String editable) {
		this.editable = editable;
	}
	
	public Lever asLever() {
		
		double max;
		double min;
		if (Parser.isNumber(maxVal)) {
			max = Double.parseDouble(maxVal);
		} else {
			max = Double.MAX_VALUE;
		}
		
		if (Parser.isNumber(minVal)) {
			min = Double.parseDouble(minVal);
		} else {
			min = Double.MIN_VALUE;
		}
		
		return new Lever(Integer.parseInt(position), name,  description, units,  min,  
				max,  Integer.parseInt(increment), displayName,  displayUnits, Boolean.parseBoolean(editable), 
				Boolean.parseBoolean(included), type);
		
	}
	
	public Output asOutput() {
		return new Output(Integer.parseInt(position), name,  description, units, 
				displayName, displayUnits, Boolean.parseBoolean(included), type);
		
	}
}
