package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.analysis.micmac.MicMacExcelWriter;
import gov.anl.gss.sdm.server.model.analysis.micmac.MicMacMinimumMemory;
import gov.anl.gss.sdm.server.model.analysis.micmac.MicMacPlot;
import gov.anl.gss.sdm.server.model.documentor.gui.StandaloneGUI;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.ColumnSizing;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Document;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Writer;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorAuxiliary;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorConstant;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorData;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorDelay;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorFlow;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorGame;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorInitial;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorLevel;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorLevelInitialization;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorLookup;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSmooth;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSpecificGroup;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSpecificModule;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSpecificView;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSubscript;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelComponents;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.visualization3d.Visualization3DData;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class Documentor {
static Logger log = Logger.getLogger(Documentor.class.getSimpleName());

	private Map<String, Equation> equations;
	private ModelComponents modelComponents;
	private AnalysisManager analysisManager;

	private ModelInformation modelInformation;
	private Warnings warnings;
	private PotentialOmissions potentialOmissions;

	private TypeCounter typeCounter;
	private ViewsCounter viewsCounter;
	private GroupsCounter groupsCounter;
	private ModulesCounter modulesCounter;

	private ViewVariables viewVariables;

	private Document document;
	
	private ColumnSizing columnSizing;
	
	private boolean subs = false;
	private boolean displayThumbNail;
	private MessageProcessor mf;


	//	private Map<String, VariableInformation> variableInformation;
	private List<ModelVariablesInformation> modelVariablesInformation; //  = new ArrayList<ModelVariablesInformation>();

	public Documentor(AnalysisManager analysisManager, Map<String, Equation> equations, ModelComponents modelComponents) {
		this.analysisManager = analysisManager;
		this.equations = equations;
		this.modelComponents = modelComponents;
		mf = analysisManager.getMessageProcessor();
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();


		this.displayThumbNail = !options.isConfiguredForMap() && 
				options.isSelected(StringConstants.INCLUDE_THUMBNAILS) && 
				InformationManagers.getInstance().getModelManager().isRunModel() ; //&&				equations.size() < 100;
		prepareVariables();

	}

	public boolean displayModule() {
		return modulesCounter.hasNonDefaultModules();
	}
	
	public MessageProcessor getMessageProcessor() {
		return mf;
	}

	public void document(String projectLocation) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		System.out.println("\n\n################### Model Documentation ###################");
		mf.addMessage(language.getText(StringConstants.PREPARING_EQUATION_VARIABLES));

		log.debug("\n\n################### Model Documentation ###################");
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		viewsCounter = new ViewsCounter(this);
		groupsCounter = new GroupsCounter(this);
		modulesCounter = new ModulesCounter(this);
		modelInformation = new ModelInformation(this);
		warnings = new Warnings(this);
		
		potentialOmissions = new PotentialOmissions(this);
		
		viewVariables = new ViewVariables(viewsCounter.getViewVariables());


		modelVariablesInformation = new ArrayList<ModelVariablesInformation>();
		modelVariablesInformation.add(new ModelVariablesInformation(this, "ALL", language.getTextHtml(StringConstants.ALL), language.getTextHtml("Variables"), new VariableSelector("ALL", "ALL")));

		for (String view : viewsCounter.getGroupingNames())
			modelVariablesInformation.add(new ModelVariablesInformation(this, "VIEW", language.getTextHtml(StringConstants.VIEW), view, new VariableSelectorSpecificView("SpecificViewVariables", "VIEW", view)));

		for (String group : groupsCounter.getGroupingNames())
			modelVariablesInformation.add(new ModelVariablesInformation(this, "GROUP", language.getTextHtml(StringConstants.GROUP), group, new VariableSelectorSpecificGroup("SpecificGroupVariables", "GROUP", group)));

		if (displayModule()) {
			for (String module : modulesCounter.getGroupingNames())
				modelVariablesInformation.add(new ModelVariablesInformation(this, "MODULE", language.getTextHtml(StringConstants.MODULE), module, new VariableSelectorSpecificModule("SpecificModuleVariables", "MODULE", module)));
		}
		
//		mf.addMessage("TypeCounter");

		typeCounter = new TypeCounter(this);

		for (String type : typeCounter.getGroupingNames()) {
//			modelVariablesInformation.add(new ModelVariablesInformation(this, "TYPE", language.getTextHtml("Type") , type, variableSelectorByType(type)));
			modelVariablesInformation.add(new ModelVariablesInformation(this, "TYPE", language.getTextHtml(StringConstants.UNITS_TYPE) , language.getText(type), variableSelectorByType(type)));
		}
		
		/*
		 * Special processing if loop analysis has not been selects
		 */
		
		
//		mf.addMessage("Continuing");
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String lm = options.getOptionValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV);
		String lmivv = options.getOptionValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV);
		String ll = options.getOptionValue(StringConstants.FEEDBACK_LOOPS);
		String llivv = options.getOptionValue(StringConstants.FEEDBACK_LOOPS_WITH_IVV);
		if (options.isSelected(StringConstants.ANALYZE_LOOPS)) {
			determineSizing();
		} else {
			options.setValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV, "boolean", UserSelectedOptions.FALSE);
			options.setValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV, "boolean", UserSelectedOptions.FALSE);
			
			options.setValue(StringConstants.FEEDBACK_LOOPS, "boolean", UserSelectedOptions.FALSE);
			options.setValue(StringConstants.FEEDBACK_LOOPS_WITH_IVV, "boolean", UserSelectedOptions.FALSE);
		}
		
		document = new Document(this, mm.getModelName(), mm.getFileName(), equations);
		
		// here we could generate the data gathering information and write to file
		// want flag if we actually want the html document generated
		
		String outputFileName = mm.getHtmlFileName();
		
		boolean writeHtmlFile = true;
		if (!writeHtmlFile) {
			return;
		}
		
		mf.addMessage(language.getText(StringConstants.WRITING_FILE)+": "+outputFileName);
		Writer writer = new Writer(outputFileName);
		writer.write(document, SDMDocLanguageSpecific.getInstance().getFileCharSet());
		
		document = null;
		System.gc();
		
		if (options.isSelected(StringConstants.MICMAC_ANALYSIS)) {
			mf.addMessage(language.getText(StringConstants.MICMAC_ANALYSIS));

//			MicMacMinimumMemory micmac = new MicMacMinimumMemory(InformationManagers.getInstance().getEquationManager().getNumberOfVariables());
			//		micmac.run();
			MicMacMinimumMemory micmac = new MicMacMinimumMemory(InformationManagers.getInstance().getEquationManager());
			
			micmac.runUntilStable(6,10);
			System.out.println("MICMAC Excel Writer");
			MicMacExcelWriter excelWriter = new MicMacExcelWriter(micmac, mf);
			System.out.println("MICMAC Excel Writer write file");
			excelWriter.writeFile(InformationManagers.getInstance().getFileOrganizer().getMicmacExcelFile());

			System.out.println("MicMacPlot");
			MicMacPlot plot = new MicMacPlot("MICMAC ##ITERATION## Iterations ##RANGE##\n"+mm.getModelName(), micmac, 
					InformationManagers.getInstance().getFileOrganizer().getMicmacPlotFile(), 0.0, 1.0, 0.0, 1.0);
			System.out.println("MicMacPlot generate");
			plot.generate();
		}
		
		options.setValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV, "boolean", lm);
		options.setValue(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV, "boolean", lmivv);
		options.setValue(StringConstants.FEEDBACK_LOOPS, "boolean", ll);
		options.setValue(StringConstants.FEEDBACK_LOOPS_WITH_IVV, "boolean", llivv);
		
		int numViews = viewsCounter.getNumberOfViews();
		if (StandaloneGUI.threeDEnabled() && numViews != 1) {
			System.out.println("3D Visualization is currently limited to models with only 1 view");
		} else {

			if (StandaloneGUI.threeDEnabled() && (options.isSelected(StringConstants.GENERATE_3D)) && 
					InformationManagers.getInstance().getModelManager().isRunModel()  ) {
				
				Visualization3DData viz = new Visualization3DData(mm.getGeneratedOutputDirectoryName(), 
						mm.getGeneratedOutputDirectoryName()+"/3D");

				if (viz.isOkToProceed()) {
					mf.addMessage(language.getText(StringConstants.WRITING_3D_DATA_TO_FILE));
					viz.forceDirected();
					viz.writeFormatted();
				} else {
					mf.addMessage(language.getText(StringConstants.COULD_NOT_GENERATE_3D_FILE));
					log.debug("Could not generate 3d");
				}
			}
		}
		
		if (options.isSelected(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS)) { 
			// zip results
			List<String> files = new ArrayList<String>();
			files.add(outputFileName);
//			files.add(mm.getMapperFile());
			
			if (options.isSelected(StringConstants.INCLUDE_MDL)) {
				files.add(mm.getFileName());
				if (mm.isXmileOrigin())
					files.add(mm.getXmileFileName());
			}
			List<String> directories = new ArrayList<String>();
			directories.add(mm.getGeneratedOutputDirectoryName());
			String zipFile = mm.getZipFileName();
			
			mf.addMessage(language.getText(StringConstants.CREATING_ZIP_FILE)+": "+zipFile);
			File tempFile = null;
			try {
				tempFile = File.createTempFile("prefix-", ".zip");
				tempFile.deleteOnExit();
				Zipper zipper = new Zipper(tempFile.getAbsolutePath(), files, directories);
				zipper.zip();
				try {
					FileUtils.copyFile(tempFile, new File(zipFile));
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		}
		LogManager.shutdown();
	}
	
	private void determineSizing() {
		
		columnSizing = analysisManager.getLoopManager().determineSizing();
	}

	public void prepareVariables() {
		for (String variableName : InformationManagers.getInstance().getEquationManager().getVariables()) {
//			VariableInformation vi = new VariableInformation(this, getAnalysisManager(), variableName);
			VariableInformation vi = InformationManagers.getInstance().getVariableInformationFactory()
					.getVariableInformation(this, getAnalysisManager(), variableName);
		}	
	}

	private VariableSelector variableSelectorByType(String type) {
		if (type.equals("Level"))
			return new VariableSelectorLevel(type, type);
		if (type.equals("Smooth"))
			return new VariableSelectorSmooth(type, type);
		if (type.equals("Delay"))
			return new VariableSelectorDelay(type, type);
		if (type.equals("LevelInitial"))
			return new VariableSelectorLevelInitialization(type, type);
		if (type.equals("Initial"))
			return new VariableSelectorInitial(type, type);
		if (type.equals("Constant"))
			return new VariableSelectorConstant(type, type);
		if (type.equals("Flow"))
			return new VariableSelectorFlow(type, type);
		if (type.equals("Auxiliary"))
			return new VariableSelectorAuxiliary(type, type);
		if (type.equals("Subscripts"))
			return new VariableSelectorSubscript(type, type);
		if (type.equals("Data"))
			return new VariableSelectorData(type, type);
		if (type.equals("Game"))
			return new VariableSelectorGame(type, type);
		if (type.equals("Lookup"))
			return new VariableSelectorLookup(type, type);
		else
			return null;
	}


	public Map<String, Equation> getEquations() {
		return equations;
//		Map<String, Equation> map = new LinkedHashMap<String, Equation>();
//		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
//			Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(var);
//			map.put(var, eqn);
//		}
//		return map;
	}

	public ModelComponents getModelComponents() {
		return modelComponents;
	}

	public AnalysisManager getAnalysisManager() {
		return analysisManager;
	}

	public ModelInformation getModelInformation() {
		return modelInformation;
	}

	public Warnings getWarnings() {
		return warnings;
	}

	public PotentialOmissions getPotentialOmissions() {
		return potentialOmissions;
	}

	public List<ModelVariablesInformation> getModelVariablesInformation() {
		return modelVariablesInformation;
	}

	public TypeCounter getTypeCounter() {
		return typeCounter;
	}

	public ViewsCounter getViewsCounter() {
		return viewsCounter;
	}

	public GroupsCounter getGroupsCounter() {
		return groupsCounter;
	}

	public ModulesCounter getModulesCounter() {
		return modulesCounter;
	}

	public ViewVariables getViewVariables() {
		return viewVariables;
	}

	public ColumnSizing getColumSizing() {
		return columnSizing;
	}

	public boolean isSubs() {
		return subs;
	}

	public void setSubs(boolean subs) {
		this.subs = subs;
	}

	/**
	 * @return the displayThumbNail
	 */
	public boolean displayThumbNail() {
		return displayThumbNail;
	}
}
