package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class GroupsCounter extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(GroupsCounter.class.getSimpleName());
	
//	public static final String ENTRY2 = "<td width='20%'><a href='' /><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String ENTRY2 = "<td width='20%'><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String EMPTY = "<td width='20%%'>&nbsp;</td>";
	
	private Map<String, Set<String>> groupVariables;

	public GroupsCounter(Documentor documentor) {
		super(documentor);
		load();
	}

	@Override
	public void load() {
		groupVariables = new LinkedHashMap<String, Set<String>>();
		VariableSelector selector = new VariableSelector("Ignore", "Ignore");
		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
			Equation equation = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
			if (selector.ignore(equation))
				continue;
//		for (Equation equation : documentor.getEquations().values()) {

			String group = equation.getGroup();
			if (!groupVariables.containsKey(group))
				groupVariables.put(group, new LinkedHashSet<String>());
			groupVariables.get(group).add(var);

		}
	}
	
	public boolean isUserDefinedGroups() {
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		boolean userDefined = false;
		for (String grp : getGroupingNames()) {
			if (!grp.equalsIgnoreCase(".control") && !grp.equalsIgnoreCase("default") && !grp.equalsIgnoreCase(mm.getModelName()))
				userDefined = true;
		}
		return userDefined;
	}

	@Override
	public List<List<List<String>>> getRows() {
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		List<List<String>> columns = initializeList();

		List<String> groups = new ArrayList<String>(groupVariables.keySet());
		Collections.sort(groups);
		int col = 0;
		for (String group : groups) {

			// Level
			columns.add(createColumn(ENTRY2, Anchors.getAnchorReference("-GROUP-"+group), NumberFormatter.formatForCountNotConditional(groupVariables.get(group).size())));
			col++;
			if (col == 5) {
				rows.add(columns);
				col = 0;
				columns = initializeList();
			}


		}

		if (col > 0) {
			while (col < 5) {
				columns.add(createColumn(EMPTY, "",""));
				col++;
			}
			rows.add(columns);
		}
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Groups"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}

		return rows;
	}
	

	
	@Override
	public List<List<List<String>>> getTableData() {
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		
		List<String> groups = new ArrayList<String>(groupVariables.keySet());
		Collections.sort(groups);
		
		for (String group : groups) {
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			TableUtilities.addColumn(row, TableUtilities.createColumn(group));
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForCountNotConditional(groupVariables.get(group).size())));
		}

		return table;
	}
	
	public void document() {
		log.debug("\n\n### Group variable Counts ###\n");
		List<String> groups = new ArrayList<String>(groupVariables.keySet());
		Collections.sort(groups);
		for (String group : groups) {
			log.debug("View: "+group+" "+groupVariables.get(group).size());
			
			for (String vv : groupVariables.get(group)) {
				log.debug("     "+vv);
			}
		}
		
	}
	
	private List<List<String>> initializeList() {
		List<List<String>> columns = new ArrayList<List<String>>();

		return columns;
	}
	
	protected List<String> createColumn(String format, String s1, String s2) {
		List<String> column = new ArrayList<String>();
		String html = format.replace("$$$ARG1$$$", s1).replace("$$$ARG2$$$", s2);
		column.add(html);
		return column;
	}
	
	public List<String> getGroupingNames() {
		List<String> al = new ArrayList<String>(groupVariables.keySet());
		Collections.sort(al);
		return al;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}


}
