package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.docx.reader.DocxReader;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.WordUtils;

public class LanguageRegistration {
	
	private final static String KEYS_FILE = "data/language/LanguageKey.docx";
	
	private Map<String, String> englishToKey;
	private Map<String, String> englishToLanguage;
	
	private String lang; // e.g. en,US
	
	public LanguageRegistration(String displayName) {
		englishToKey = load(KEYS_FILE);
		this.lang = SDMDocLanguageSpecific.getLanguageForDisplayName(displayName);
	}
	
	public void register(String file) {
		englishToLanguage = load(file);
//		if (validate())
		createBundle();
			register();
	}
	
	private boolean validate() {
		return englishToKey.keySet().size() == englishToLanguage.keySet().size();
	}
	
	public void register() {
		SDMDocLanguageSpecific.registerNewLanguage(lang);
	}
	
	private void createBundle() {
		
		List<String> eng = new ArrayList<String>(englishToKey.keySet());
		Collections.sort(eng);
		List<String> newBundle = new ArrayList<String>();
		for (String word : eng) {
			String line = englishToKey.get(word) + " = " + WordUtils.capitalize(englishToLanguage.get(word));
			newBundle.add(line);
		}
		
		String[] parts = lang.split(",");
		StringBuilder sb = new StringBuilder(parts[0]);
		if (parts.length > 1)
			sb.append("_"+parts[1]);
		if (parts.length > 2)
			sb.append("_"+parts[2]);
		Utilities.writeToFileUTF8(SDMDocLanguageSpecific.BUNDLE_FILE+"_"+sb.toString()+SDMDocLanguageSpecific.PROPERTIES, newBundle);
	}
	
	private Map<String, String> load(String file) {		
		DocxReader reader = new DocxReader(file);
		return processText(reader.getDocText());
	}
	
	private Map<String, String> processText(List<String> al) {
		Map<String, String> m = new LinkedHashMap<String, String>();

		int rowCount = 0;
		int row = 0;
		int column = 0;
		String[] cols = null;
		boolean startedTable = false;
//		for (String s : al) {
//			System.out.println(s);
//		}
		for (String s : al) {
//			System.out.println("Process: {"+s+"}");
			if (s.startsWith(DocxReader.TABLE_START)) {
				startedTable = true;
				rowCount = Integer.parseInt(s.split(" = ")[1]);
			} else if (s.startsWith(DocxReader.ROW_START)) {
				row++;
				cols = new String[2];
				column = 0;
			} else if (startedTable && s.startsWith(DocxReader.DATA_START)) {
				String text = s.replace(DocxReader.DATA_START, "");
				if (cols == null)
					System.out.println("OOPS");
				if (column > 1)
					break;
				cols[column] = text;
				column++;
				if (column == 2) {
					if (cols[0].length() > 0 && cols[1].length() > 0) {
						m.put(cols[0], cols[1]);
					} else {
//						System.out.println("Skipping blank row");
					}
				}
			} else {
				// somethings wrong!
			}
		}


		return m;
	}
}
