package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class ModulesCounter  extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(ModulesCounter.class.getSimpleName());
	
//	public static final String ENTRY2 = "<td width='20%'><a href='' /><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String ENTRY2 = "<td width='20%'><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String EMPTY = "<td width='20%%'>&nbsp;</td>";
	
	private Map<String, Set<String>> moduleVariables;

	public ModulesCounter(Documentor documentor) {
		super(documentor);
		load();
	}
	
	public boolean hasNonDefaultModules() {
		return moduleVariables.size() > 1;
	}

	@Override
	public void load() {
		moduleVariables = new LinkedHashMap<String, Set<String>>();
		for (Equation equation : documentor.getEquations().values()) {
			if (ignore(equation))
				continue;

			String module = equation.getModule();
			if (!moduleVariables.containsKey(module))
				moduleVariables.put(module, new LinkedHashSet<String>());
			moduleVariables.get(module).add(equation.getOriginalLhs());

		}
	}

	@Override
	public List<List<List<String>>> getRows() {
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		List<List<String>> columns = initializeList();

		List<String> modules = new ArrayList<String>(moduleVariables.keySet());
		Collections.sort(modules);
		int col = 0;
		for (String module : modules) {

			// Level
			columns.add(createColumn(ENTRY2, Anchors.getAnchorReference("-MODULE-"+module), NumberFormatter.formatForCountNotConditional(moduleVariables.get(module).size())));
			col++;
			if (col == 5) {
				rows.add(columns);
				col = 0;
				columns = initializeList();
			}


		}

		if (col > 0) {
			while (col < 5) {
				columns.add(createColumn(EMPTY, "",""));
				col++;
			}
			rows.add(columns);
		}
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Modules"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}

		return rows;
	}
	
	public void document() {
		log.debug("\n\n### Module variable Counts ###\n");
		List<String> modules = new ArrayList<String>(moduleVariables.keySet());
		Collections.sort(modules);
		for (String module : modules) {
			log.debug("View: "+module+" "+moduleVariables.get(module).size());
			
			for (String vv : moduleVariables.get(module)) {
				log.debug("     "+vv);
			}
		}
		
	}
	
	private List<List<String>> initializeList() {
		List<List<String>> columns = new ArrayList<List<String>>();

		return columns;
	}
	
	protected List<String> createColumn(String format, String s1, String s2) {
		List<String> column = new ArrayList<String>();
		String html = format.replace("$$$ARG1$$$", s1).replace("$$$ARG2$$$", s2);
		column.add(html);
		return column;
	}
	
	public List<String> getGroupingNames() {
		List<String> al = new ArrayList<String>(moduleVariables.keySet());
		Collections.sort(al);
		return al;
	}
	
	public boolean ignore(Equation equation) {
		
		if (equation.isDefinesSubscript())
			return true;
		if (equation.getLhs().equalsIgnoreCase("FINAL TIME") ||
				equation.getLhs().equalsIgnoreCase("INITIAL TIME") ||
				equation.getLhs().equalsIgnoreCase("TIME STEP") ||
				equation.getLhs().equalsIgnoreCase("SAVEPER") ||
				isAutogenerated(equation))
			return true;
		return false;
	}
	
	public boolean isAutogenerated(Equation equation) {
		return equation.isAutoGenerated();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		
		List<String> modules = new ArrayList<String>(moduleVariables.keySet());
		Collections.sort(modules);
		
		for (String module : modules) {
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			TableUtilities.addColumn(row, TableUtilities.createColumn(module));
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForCountNotConditional(moduleVariables.get(module).size())));
		}

		return table;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}

}
