package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorComplexity;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorComplexityStock;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSupplementary;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSupplementaryUsed;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUnused;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class PotentialOmissions extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(PotentialOmissions.class.getSimpleName());

	
	private List<Equation> unused;
	private int effectiveunusedCount = 0;
	private List<Equation> supplementary;
	private int effectivesupplementaryCount = 0;
	private List<Equation> supplementaryUsed;
	private int effectivesupplementaryUsedCount = 0;
	private List<Equation> complexVariable;
	private int effectivecomplexVariableCount = 0;
	private List<Equation> complexStock;
	private int effectivecomplexStockCount = 0;
	
	private boolean subs;
	
	public PotentialOmissions(Documentor documentor) {
		super(documentor);
		load();
		subs = documentor.isSubs();
	}
	
	public void document() {
		log.debug("### Potential Omissions ###\n");
		log.debug("Unused: "+getUnusedCount());
		log.debug("Supplementary: "+getSupplementaryCount());
		log.debug("Supplementary Used: "+getSupplementaryUsedCount());
		log.debug("Complex Variable: "+getComplexVariableCount());
		log.debug("Complex Stock: "+getComplexStockCount());
	}
	
	public void load() {
		loadUnused();
		loadSupplementary();
		loadSupplementaryUsed();
		loadComplexVariable();
		loadComplexStock();
	}
	
	private void loadSelected(VariableSelector selector, List<Equation> selected) {
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectSingleEquation(eqn))
				selected.add(eqn);
		}
	}
	
	private void loadUnused() {
		unused = new ArrayList<Equation>();
		VariableSelector unusedSelector = new VariableSelectorUnused(StringConstants.UNUSED_VARIABLES, StringConstants.UNUSED_VARIABLES);
		loadSelected(unusedSelector, unused);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (unusedSelector.isSelectMultipleEquation(eqn))
				effectiveunusedCount += eqn.getTotalEquationCount();
		}

	}
	
	private void loadSupplementary() {
		supplementary = new ArrayList<Equation>();
		VariableSelector supplementarySelector = new VariableSelectorSupplementary(StringConstants.SUPPLEMENTARY_VARIABLES,StringConstants.SUPPLEMENTARY_VARIABLES);
		loadSelected(supplementarySelector, supplementary);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (supplementarySelector.isSelectMultipleEquation(eqn))
				effectivesupplementaryCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadSupplementaryUsed() {
		supplementaryUsed = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorSupplementaryUsed(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED);
		loadSelected(selector, supplementaryUsed);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				effectivesupplementaryUsedCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadComplexVariable() {
		complexVariable = new ArrayList<Equation>();
		VariableSelector complexitySelector = new VariableSelectorComplexity(StringConstants.COMPLEX_VARIABLE,StringConstants.COMPLEX_VARIABLE);
		loadSelected(complexitySelector, complexVariable);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (complexitySelector.isSelectMultipleEquation(eqn))
				effectivecomplexVariableCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadComplexStock() {
		complexStock = new ArrayList<Equation>();
		VariableSelector stockSelector = new VariableSelectorComplexityStock(StringConstants.COMPLEX_STOCK,StringConstants.COMPLEX_STOCK);
		loadSelected(stockSelector, complexStock);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (stockSelector.isSelectMultipleEquation(eqn))
				effectivecomplexStockCount += eqn.getTotalEquationCount();
		}
	}
	
	public int getUnusedCount() {
		return unused.size();
	}
	public int getSupplementaryCount() {
		return supplementary.size();
	}
	public int getSupplementaryUsedCount() {
		return supplementaryUsed.size();
	}
	public int getComplexVariableCount() {
		return complexVariable.size();
	}
	public int getComplexStockCount() {
		return complexStock.size();
	}
	
	public String getUnusedCountWithPctNoHTML() {
		return getFormattedWithPctNoHTML(unused.size(), effectiveunusedCount);
	}
	public String getSupplementaryCountWithPctNoHTML() {
		return getFormattedWithPctNoHTML(supplementary.size(), effectivesupplementaryCount);
	}
	public String getSupplementaryUsedCountWithPctNoHTML() {
		return getFormattedWithPctNoHTML(supplementaryUsed.size(), effectivesupplementaryUsedCount);
	}
	public String getComplexVariableCountWithPctNoHTML() {
		return getFormattedWithPctNoHTML(complexVariable.size(), effectivecomplexVariableCount);
	}
	public String getComplexStockCountWithPctNoHTML() {
		return getFormattedWithPctNoHTML(complexStock.size(), effectivecomplexStockCount);
	}
	
	
	public String getUnusedCountWithPct() {
		return getFormattedWithPct(unused.size(), effectiveunusedCount);
	}
	public String getSupplementaryCountWithPct() {
		return getFormattedWithPct(supplementary.size(), effectivesupplementaryCount);
	}
	public String getSupplementaryUsedCountWithPct() {
		return getFormattedWithPct(supplementaryUsed.size(), effectivesupplementaryUsedCount);
	}
	public String getComplexVariableCountWithPct() {
		return getFormattedWithPct(complexVariable.size(), effectivecomplexVariableCount);
	}
	public String getComplexStockCountWithPct() {
		return getFormattedWithPct(complexStock.size(), effectivecomplexStockCount);
	}
	
	public String getFormattedWithPctNoHTML(int v1, int v2) {
		if (NumberFormatter.map)
			return "NA";
		return NumberFormatter.getAsCountAndPercentTotalNoHTML(v1, documentor.getModelInformation().getVariablesCount()) + (subs ? "|" +
				NumberFormatter.getAsCountAndPercentTotalNoHTML(v2, documentor.getModelInformation().getEffectiveVariableCount()) : "");
	}
	
	public String getFormattedWithPct(int v1, int v2) {
		if (NumberFormatter.map)
			return "NA";
		return NumberFormatter.getAsCountAndPercentTotal(v1, documentor.getModelInformation().getVariablesCount()) + (subs ? "|" +
				NumberFormatter.getAsCountAndPercentTotal(v2, documentor.getModelInformation().getEffectiveVariableCount()) : "");
	}
	
	private List<List<List<String>>> createFlatFileTable() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<List<String>>> table = createTable();
		
		addTitle(table,language.getTextHtml(StringConstants.POTENTIAL_OMISSIONS));
		List<String> headers = new ArrayList<String>();
		
		headers.add(language.getTextHtml(StringConstants.VARIABLE));
		headers.add(language.getTextHtml(StringConstants.NUMBER_OF_VARIABLES));
		headers.add(language.getTextHtml(StringConstants.PCT_OF_TOTAL));
		if (subs) {
			headers.add(language.getTextHtml(StringConstants.EFFECTIVE_NUMBER_OF_VARIABLES));
			headers.add(language.getTextHtml(StringConstants.EFFECTIVE_PCT_OF_TOTAL));
		}
		addColumnHeaders(table, headers);
		
		return table;
	}
	
//	@Override
	public List<List<List<String>>> getTableData() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<List<String>>> table = createFlatFileTable();
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table,StringConstants.UNUSED_VARIABLES, /*StringConstants.UNUSED_VARIABLES, */
				language.getTextHtml(StringConstants.UNUSED_VARIABLES),getUnusedCountWithPctNoHTML());
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table,StringConstants.SUPPLEMENTARY_VARIABLES,/*StringConstants.SUPPLEMENTARY_VARIABLES,*/ 
				language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES),getSupplementaryCountWithPctNoHTML());
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table,StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, /*StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, */
				language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED),getSupplementaryUsedCountWithPctNoHTML());
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table,StringConstants.COMPLEX_VARIABLE, /*StringConstants.COMPLEX_VARIABLE, */
				addRichardson(language.getTextHtml(StringConstants.COMPLEX_VARIABLE)),getComplexVariableCountWithPctNoHTML());
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table,StringConstants.COMPLEX_STOCK, /*StringConstants.COMPLEX_STOCK, */
				language.getTextHtml(StringConstants.COMPLEX_STOCK),getComplexStockCountWithPctNoHTML());
		
		return table;
	}

	@Override
	public List<List<List<String>>> getRows() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		
//		public static final String[] POTENTIAL_OMISSIONS_CHOICES = {
//			"Unused Variables", "Supplementary Variables", "Supplementary Variables Being Used", 
//			"Complex Variable Formulations (Richardson's Rule)", "Complex Stock Formulations"
//		};
		
		rowAdd(HTMLConstants.WITH_HTML, rows,StringConstants.UNUSED_VARIABLES, StringConstants.UNUSED_VARIABLES, 
				language.getTextHtml(StringConstants.UNUSED_VARIABLES),getUnusedCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows,StringConstants.SUPPLEMENTARY_VARIABLES,StringConstants.SUPPLEMENTARY_VARIABLES, 
				language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES),getSupplementaryCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows,StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, 
				language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED),getSupplementaryUsedCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows,StringConstants.COMPLEX_VARIABLE, StringConstants.COMPLEX_VARIABLE, 
				addRichardson(language.getTextHtml(StringConstants.COMPLEX_VARIABLE)),getComplexVariableCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows,StringConstants.COMPLEX_STOCK, StringConstants.COMPLEX_STOCK, 
				language.getTextHtml(StringConstants.COMPLEX_STOCK),getComplexStockCountWithPct());
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"PotentialOmissions"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		
		return rows;
	}
	
	private String addRichardson(String r) {
		String newR = new String(r);
		String rich = InformationManagers.getInstance().getUserSelectedOptions().getOptionValue(StringConstants.RULE_THRESHOLD);
		
		newR = newR.replace(")", " = "+rich+")");
		
		return newR;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}

}
