package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.documentor.gui.SDMDocFont;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.LanguageTranslationResourceBundle;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.awt.Component;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
//import java.util.ResourceBundle;
import java.util.Set;

import javax.swing.JOptionPane;

import org.apache.commons.lang3.StringEscapeUtils;

public class SDMDocLanguageSpecific {
	
	public static final String BUNDLE_START = "XmlFileStrings_";
	public static final String TOOL_TIP_START = "ToolTip_";
	public static final String PROPERTIES = ".properties";
	public static final String BUNDLE_NAME = "XmlFileStrings_"; //$NON-NLS-1$
	public static final String TOOL_TIP_NAME = "ToolTip_"; //$NON-NLS-1$
	
	public static final String BUNDLE_DIR = "data/language";
	public static final String BUNDLE_FILE = BUNDLE_DIR+"/XmlFileStrings";
	public static final String TOOL_TIP_START_FILE = BUNDLE_DIR+"/ToolTip";
	
	public static String DEFAULT_FILE = "data/language/defaultLanguage.csv";
	public static String DEFAULT_LANGUAGE = "en,US";
	public static String ENGLISH = "en,US";
	public static String ENGLISH_FILE = "en_US";
	
	public static boolean tooltipsInNativeLanguage = false;
	public static boolean tooltipsInEnglish = false;
	public boolean initialized = false;
	
	private static String startupLang;
	
	public static Map<String, List<String>> missingTranslationReport = new HashMap<String, List<String>>();
	
	private static SDMDocLanguageSpecific instance;
	private String language;
	private String country;
	private String variant;
	private Locale locale;
	private LanguageTranslationResourceBundle xmlFileStrings;
	private LanguageTranslationResourceBundle tooltipFileStrings;
	
	private static List<String> knownLanguages;
	private static Map<String, LanguageTranslationResourceBundle> allKnownLanguageBundles;
	private static Map<String, LanguageTranslationResourceBundle> allKnowLanguageToolTips;
	private static Map<String, Locale> allKnownLanguageLocales;
	private static Map<String, Locale> allSupportedLanguageLocales;
	
	private static SDMDocLanguageSpecific english;
	
	// currently only called to create "english" instance
	
	public SDMDocLanguageSpecific(String language, String x) {
		locale = allKnownLanguageLocales.get(language);
		xmlFileStrings = allKnownLanguageBundles.get(language);
		tooltipFileStrings = allKnowLanguageToolTips.get(language);
	}
	
	public SDMDocLanguageSpecific(String language, boolean setAsDefault, Component parent) {
		locale = allKnownLanguageLocales.get(language);
		xmlFileStrings = allKnownLanguageBundles.get(language);
		tooltipFileStrings = allKnowLanguageToolTips.get(language);
		if (setAsDefault && instance != null) {
			String displayName = language;
			Locale l = instance.getLocale();
			Locale s = allSupportedLanguageLocales.get(language);
			
			String dn3 = s.getDisplayName(l);
//			displayName = dn3+ " -- "+ new SDMDocLanguageSpecific(SDMDocLanguageSpecific.getLanguageForDisplayName(dn3), "").getDisplayName();
			displayName = dn3+ " -- "+ new SDMDocLanguageSpecific(language, "").getDisplayName();
//			if (allSupportedLanguageLocales.get(language) != null) {
//				displayName = allSupportedLanguageLocales.get(language).getDisplayName(instance.getLocale());
//			}
			
//			JOptionPane pane = new JOptionPane(arguments);
//		     pane.set.Xxxx(...); // Configure
//		     JDialog dialog = pane.createDialog(parentComponent, title);
//		     dialog.show();

			JOptionPane
			.showMessageDialog(
					parent,
					SDMDocLanguageSpecific.getInstance().getText(StringConstants.SETTING_DEFAULT_LANGUAGE_TO)+" "+displayName,
					SDMDocLanguageSpecific.getInstance().getText(StringConstants.SETTING_DEFAULT_LANGUAGE_TO)+" "+displayName,
					JOptionPane.INFORMATION_MESSAGE);
			saveDefaultToFile(language);
		}
		
//		System.out.println("Setting language: "+language);
	}
	
	public SDMDocLanguageSpecific() {
		this(DEFAULT_LANGUAGE, false, null);
	}
	

	public static void initialize() {

		knownLanguages = loadKnownLanguagesFromDirectorySearch();
		allKnownLanguageBundles = new LinkedHashMap<String, LanguageTranslationResourceBundle>();
		allKnowLanguageToolTips = new LinkedHashMap<String, LanguageTranslationResourceBundle>();
		allKnownLanguageLocales = new LinkedHashMap<String, Locale>();
		
		generateKnownLanguageLocalesAndBundles();
		
		loadDefaultFromFile();
		
		allSupportedLanguageLocales = new LinkedHashMap<String, Locale>();
		initializeListOfSupportedLocale();
		
		english = new SDMDocLanguageSpecific(DEFAULT_LANGUAGE, "");
		
		generateTranslationReport();
		
	}
	
	public static void registerNewLanguage(String docFile, String displayName) {
		
		String newLang = getLanguageForDisplayName(displayName);
		LanguageRegistration langReg = new LanguageRegistration(newLang);
		langReg.register(docFile);
		generateKnownLanguageLocaleAndBundle(newLang);
		
		// load LanguageKey file
		// load Language docfile
		
	}
	
	public static boolean languageTranslationBundleExists(String lang) {
		String filename = LanguageTranslationResourceBundle.getFilename(BUNDLE_DIR, BUNDLE_FILE, getLanguageForDisplayName(lang));
		return new File(filename).exists();
	}
	public static void registerNewLanguage(String newLang) {
		
		generateKnownLanguageLocaleAndBundle(newLang);
		
	}
	
	public static Set<String> minimizeLocales() {
		Set<String> keepers = new LinkedHashSet<String>();
		Map<String, Set<String>> counts = new LinkedHashMap<String, Set<String>>();
		
		Locale[] s = Locale.getAvailableLocales();
        for (int i=1; i<s.length; i++) {
        	if (!counts.containsKey(s[i].getDisplayLanguage(Locale.ENGLISH))) {
        		counts.put(s[i].getDisplayLanguage(Locale.ENGLISH), new LinkedHashSet<String>());
        	}
        	if (s[i].getDisplayCountry(Locale.ENGLISH).length() > 0) {
        		counts.get(s[i].getDisplayLanguage(Locale.ENGLISH)).add(s[i].getDisplayCountry(Locale.ENGLISH));
        	}
        }
        for (int i=1; i<s.length; i++) {
        	if (isLevel1Key(asKey(s[i])) || counts.get(s[i].getDisplayLanguage(Locale.ENGLISH)).size() > 1) {
        		keepers.add(asKey(s[i]));
        	}
        }
		
		return keepers;
	}
	
	private static boolean isLevel1Key(String s) {
		return s.split(",").length == 1;
	}
	
	public static void initializeListOfSupportedLocale() {
		Locale[] s = Locale.getAvailableLocales();
		Set<String> keepers = minimizeLocales();
        for (int i=0; i<s.length; i++) {
        	if (!keepers.contains(asKey(s[i])))
        		continue;
//           printLocaleDetails(s[i], Locale.ENGLISH);
           StringBuilder lang = new StringBuilder();
           lang.append(s[i].getLanguage());
           if (s[i].getCountry().length() > 0) {
        	   lang.append(","+s[i].getCountry());
        	   if (s[i].getVariant().length() > 0) {
            	   lang.append(","+s[i].getVariant());
               }
           }
//           supportedLangToDisplayName.put(lang.toString(), s[i].getDisplayName());
//           allSupportedLanguageLocales.put(s[i].toString() /* getLanguage() */ , s[i]);
           allSupportedLanguageLocales.put(asKey(s[i]) /* getLanguage() */ , s[i]);
        }
	}
	
	public static String asKey(Locale locale) {
//		StringBuilder sb = new StringBuilder();
//		sb.append(locale.getDisplayLanguage(Locale.ENGLISH));
//		if (locale.getDisplayCountry().length() > 0) {
//			sb.append(","+locale.getDisplayCountry(Locale.ENGLISH));
//		}
//		if (locale.getDisplayVariant().length() > 0) {
//			sb.append(","+locale.getDisplayVariant(Locale.ENGLISH));
//		}
//		String key = sb.toString();
		
		StringBuilder sb = new StringBuilder();
		sb.append(locale.getLanguage());
		if (locale.getCountry().length() > 0) {
			sb.append(","+locale.getCountry());
		}
		if (locale.getVariant().length() > 0) {
			sb.append(","+locale.getVariant());
		}
		String key = sb.toString();
		return key;
	}
	
	private static void generateKnownLanguageLocalesAndBundles() {
		for (String lang : knownLanguages) {
			generateKnownLanguageLocaleAndBundle(lang);
		}
		
	}
	
	private static void generateKnownLanguageLocaleAndBundle(String lang) {
		Locale loc = makeLocale(lang);
//		langToDisplayName.put(lang, loc.getDisplayName());
//		displayNameToLang.put(loc.getDisplayName(), lang);
		allKnownLanguageLocales.put(lang, loc);
		allKnownLanguageBundles.put(lang, LanguageTranslationResourceBundle.getBundle(BUNDLE_DIR, BUNDLE_NAME, lang));

//		public boolean tooltipsInNativeLanguage = false;
//		public boolean tooltipsInEnglish = false;
		
		// now check if Tooltip file exists
		String ttfile = LanguageTranslationResourceBundle.getFilename(BUNDLE_DIR, TOOL_TIP_NAME, lang);
		if (new File(ttfile).exists()) {
			tooltipsInNativeLanguage = true;
			allKnowLanguageToolTips.put(lang, LanguageTranslationResourceBundle.getBundle(BUNDLE_DIR, TOOL_TIP_NAME, lang));
		} else {
			ttfile = LanguageTranslationResourceBundle.getFilename(BUNDLE_DIR, TOOL_TIP_NAME, ENGLISH);
			allKnowLanguageToolTips.put(lang, LanguageTranslationResourceBundle.getBundle(BUNDLE_DIR, TOOL_TIP_NAME, ENGLISH));
			if (new File(ttfile).exists()) {
				tooltipsInEnglish = true;
			} else {
				// no tool tips
			}
		}
	}
	
	public static List<String> getKnownDisplayNames() {
		List<String> al = new ArrayList<String>(/* displayNameToLang.keySet() */);
		for (String knownLanguage : allKnownLanguageLocales.keySet()) {
			Locale localeKnownLanguage = allKnownLanguageLocales.get(knownLanguage);
			String s = localeKnownLanguage.getDisplayName(instance.getLocale())
					+" -- "+ localeKnownLanguage.getDisplayName(localeKnownLanguage);
			al.add(s);
		}
		Collections.sort(al);
		return al;
	}
	
	public static List<String> getKnownLanguages() {
		List<String> al = new ArrayList<String>(allKnownLanguageLocales.keySet() /* langToDisplayName.keySet()*/ );
		Collections.sort(al);
		return al;
	}
	
	public static List<String> getSupportedDisplayNames() {
		List<String> al = new ArrayList<String>(/* supportedDisplayNameToLang.keySet() */);
		for (String s : allSupportedLanguageLocales.keySet()) {

			
				al.add(allSupportedLanguageLocales.get(s).getDisplayName(instance.getLocale())+ 
						" -- "+ allSupportedLanguageLocales.get(s).getDisplayName(allSupportedLanguageLocales.get(s))
						);

		}
		Collections.sort(al);
		return al;
	}
	
	public static String getLanguageForDisplayName(String displayName) {
		String lang = "";
		for (String s : allSupportedLanguageLocales.keySet()) {
			String aDisplayName = allSupportedLanguageLocales.get(s).getDisplayName(instance.getLocale())
					+ " -- "+ allSupportedLanguageLocales.get(s).getDisplayName(allSupportedLanguageLocales.get(s));
			if (displayName.equals(aDisplayName))
				return asKey(allSupportedLanguageLocales.get(s));
		}
		return lang;
	}
	
	private static Locale makeLocale(String language) {
		String[] parts = language.split(",");
		if (parts.length == 1) {
			return new Locale(parts[0]);
		} else if (parts.length == 2) {
			return new Locale(parts[0], parts[1]);
		} else {
			return new Locale(parts[0], parts[1], parts[2]);
		}
	}
	
	private static List<String> loadKnownLanguagesFromDirectorySearch() {
		List<String> al = new ArrayList<String>();
		File folder = new File(BUNDLE_DIR);
		File[] listOfFiles = folder.listFiles();

		for (File file : listOfFiles) {

			if (file.getName().startsWith(BUNDLE_START) && 
					file.getName().endsWith(PROPERTIES)) {
				String name = file.getName();
				String lcv = name.replace(BUNDLE_START, "").replace(PROPERTIES, "").replace("_", ",");
				al.add(lcv);
			}
			
		}

		return al;
	}
	
    private static void loadDefaultFromFile() {
    	
    	File file = new File(DEFAULT_FILE);
    	if (!file.exists()) {
    		saveDefaultToFile(DEFAULT_LANGUAGE);
    	}
    	
    	String parts = null;
    	BufferedReader br = Utilities.openForRead(DEFAULT_FILE);
    	for (String line : Utilities.getAsList(br)) {
    		parts = line;	
    	}
    	try {
			br.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	
    	// do we have this language?
    	if (!allKnownLanguageLocales.containsKey(parts)) {
    		// don't have, reset to default
    		saveDefaultToFile(DEFAULT_LANGUAGE);
    		loadDefaultFromFile();
    	} else {    	
    		boolean setAsDefault = false;
    		getInstance(parts, setAsDefault);
    		startupLang = parts;
    	}
    	
    	
    }
    
    private static void saveDefaultToFile(String language) {
    	BufferedWriter bw = Utilities.openFileForWriting(DEFAULT_FILE);
    		try {
				bw.append(language+"\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    	
    }

	
	public String getDisplayName() {
		return locale.getDisplayName(locale);
	}
	
	public String getDisplayName(String lang) {
		return allKnownLanguageLocales.get(lang).getDisplayName();
	}
	
	public static SDMDocLanguageSpecific getInstance(String language,boolean setAsDefault, Component parent) {
//		System.out.println("GetInstance Specific language language: "+language);
		instance = new SDMDocLanguageSpecific(language, setAsDefault, parent);
		return instance;
	}
	
	public static SDMDocLanguageSpecific getInstance(String language,boolean setAsDefault) {
//		System.out.println("GetInstance Specific language language: "+language);
		instance = new SDMDocLanguageSpecific(language, setAsDefault, null);
		return instance;
	}
	
	public static SDMDocLanguageSpecific getInstance() {
		if (instance == null) {
			instance = new SDMDocLanguageSpecific();
		} else if (instance.xmlFileStrings == null) {
			SDMDocLanguageSpecific.initialize();
		} 
		return instance;
	}
	
	public String getTextHtml(String t) {
		if (xmlFileStrings.containsKey(t) && !xmlFileStrings.getString(t).equals("null"))
				return StringEscapeUtils.escapeHtml4(xmlFileStrings.getString(t));
		else
			return StringEscapeUtils.escapeHtml4(missing(t));
	}
	
	public String getTextHtmlNoMissingArrowPolarity(String t) {
		
		// need to examine t to see if it contains  reference to function.
		// function names will not be translated.
		// e.g. function[SMOOTHI] would be input -> translate(function) + [SMOOTHI]
		
		if (t.startsWith(StringConstants.FUNCTION_SKIP)) {
			String func = t.replace(StringConstants.FUNCTION_SKIP, "");
			
			if (xmlFileStrings.containsKey(StringConstants.FUNCTION_SKIP) && 
					!xmlFileStrings.getString(StringConstants.FUNCTION_SKIP).equals("null")) {
				return StringEscapeUtils.escapeHtml4(xmlFileStrings.getString(StringConstants.FUNCTION_SKIP)+
						StringEscapeUtils.escapeHtml4(func));
			} else {
				return StringEscapeUtils.escapeHtml4(missing(StringConstants.FUNCTION_SKIP)+
						StringEscapeUtils.escapeHtml4(func));
			}
		}
		
		
		if (xmlFileStrings.containsKey(t) && !xmlFileStrings.getString(t).equals("null")) {
			return StringEscapeUtils.escapeHtml4(xmlFileStrings.getString(t));
		} else if (
				t.equals(ArrowPolarity.UNKNOWN) ||
				t.equals(ArrowPolarity.NEGATIVE) ||
				t.equals(ArrowPolarity.POSITIVE) ||
				t.equals(ArrowPolarity.ANY)){
			return StringEscapeUtils.escapeHtml4(t);
		} else {
			return StringEscapeUtils.escapeHtml4(missing(t));
		}
	}
	
	public String getTextHtmlNoMissing(String t) {
		if (xmlFileStrings.containsKey(t) && !xmlFileStrings.getString(t).equals("null")) {
			return StringEscapeUtils.escapeHtml4(xmlFileStrings.getString(t));
		} else {
			return StringEscapeUtils.escapeHtml4(missing(t));
		}
	}
	
	public String getToolTipText(String key) {
		if (tooltipFileStrings.containsKey(key) && !tooltipFileStrings.getString(key).equals("null"))
			return tooltipFileStrings.getString(key);
	else
		return "No Tooltip";
	}

	public String getText(String t) {
		if (xmlFileStrings.containsKey(t) && !xmlFileStrings.getString(t).equals("null"))
				return xmlFileStrings.getString(t);
		else
			return missing(t);
	}
	
	public boolean hasTranslation(String t) {
		if (xmlFileStrings == null)
			return false;
		if (xmlFileStrings.containsKey(t)) {
			if (xmlFileStrings.getString(t).equals("null"))
				return false;
		}
		return xmlFileStrings.containsKey(t); 
	}
	
	public String getFileCharSet() {
		if (locale.getLanguage().equals(new Locale("ja").getLanguage())) {
//		if (locale.getLanguage().equals("ja")) {
			return "UTF-8"; // return "EUC-JP";
		} else {
			return "UTF-8";
		}
	}
	
	public String getHtmlHeader() {
//		if (locale.getLanguage().equals("ja")) {
		if (locale.getLanguage().equals(new Locale("ja").getLanguage())) {
			return "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"ja\" lang=\"ja\">";
		} else {
			return "<html>";
		}
	}
	
	public String getHtmlCharSet() {
//		if (locale.getLanguage().equals("ja")) {
		if (locale.getLanguage().equals(new Locale("ja").getLanguage())) {
			return "utf-8";
		} else {
			return "utf-8";
		}
	}
	
	public Font getFont(int style, int size) {
//		if (locale.getLanguage().equals("ja")) {
//		if (locale.getLanguage().equals(new Locale("ja").getLanguage())) {
			
//			System.out.println("Locale language = "+locale.getLanguage()+" font = Meiryo");
//			return new Font("Meiryo", style, size);
//		} else {
//			System.out.println("Locale language = "+locale.getLanguage()+" font = Arial");
//			return new Font("Arial", style, size);
//			return new Font("Meiryo", style, size);
//			return new Font("MS UI Gothic", style, size);
//		}
		return new Font(SDMDocFont.FONT, style, size);
	}
	
	public static void generateTranslationReport() {
		
		boolean setAsDefault = false;
		
		for (String lang : getKnownLanguages()) {
//			missingTranslationReport.put(english.getDisplayName(lang), generateTranslationReport(lang));
			missingTranslationReport.put(lang, generateTranslationReport(lang));
		}
		
		SDMDocLanguageSpecific.getInstance(startupLang, setAsDefault);
		
	}
	
	private static List<String> generateTranslationReport(String lang) {
		
		List<String> report = new ArrayList<String>();
		boolean setAsDefault = false;
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance(lang, setAsDefault);
		
//		System.out.println("\n### "+language.getDisplayName()+" START ###\n");
		for (String s : StringConstants.ALL_STRING_CONSTANTS) {
			if (!language.hasTranslation(s))
				report.add(s+" -> "+language.getText(s));
		}
		return report;
	}
	
	public String missing(String t) {
		if (english.hasTranslation(t)) {
//			return english.getText(t)+" ("+getDisplayLanguage()+" MISSING) ";
			return "<"+english.getText(t)+">"; // " ("+getDisplayLanguage()+" MISSING) ";
		} else {
//			return "No English: "+ t+" ("+getDisplayLanguage()+" MISSING) ";
			return "<"+ t+">"; // " ("+getDisplayLanguage()+" MISSING) ";
		}
	}
	
	public String getDisplayLanguage() {
		return locale.getDisplayLanguage();
	}

	public String getLanguage() {
		return language;
	}
	
	public String getDisplayCountry() {
		return locale.getDisplayCountry();
	}

	public String getCountry() {
		return country;
	}

	public Locale getLocale() {
		return locale;
	}
	
	public static Map<String, List<String>> getMissingTranslationReport() {
		return missingTranslationReport;
	}

	public static void setInstance(SDMDocLanguageSpecific instance) {
		SDMDocLanguageSpecific.instance = instance;
	}

	public static void printLocaleDetails(Locale l, Locale inLocale) {

		System.out.println(l.getCountry());
		System.out.println(l.getDisplayCountry());
		System.out.println(l.getDisplayCountry(inLocale));
		System.out.println(l.getDisplayLanguage());
		System.out.println(l.getDisplayLanguage(inLocale));
		System.out.println(l.getDisplayName());
		System.out.println(l.getDisplayName(inLocale));
		System.out.println(l.getDisplayScript());
		System.out.println(l.getDisplayScript(inLocale));
		System.out.println(l.getDisplayVariant());
		System.out.println(l.getDisplayVariant(inLocale));
		//System.out.println(l.getExtension(char key)
		//	Set<Character> 	getExtensionKeys()
		System.out.println(l.getISO3Country());
		System.out.println(l.getISO3Language());
		System.out.println(l.getLanguage());
		System.out.println(l.getScript());
		//	Set<String> 	getUnicodeLocaleAttributes()
		//	Set<String> 	getUnicodeLocaleKeys()
		//System.out.println(l.getUnicodeLocaleType(String key)
		System.out.println(l.getVariant());
		System.out.println(l.toLanguageTag());
		System.out.println(l.toString());
	}
}
