package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.List;

public class TableData {
	
	
	protected final static String[] naForMap = {StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES, StringConstants.TOTAL_NUMBER_OF_STOCKS, StringConstants.TOTAL_NUMBER_OF_MACROS, StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS,
		StringConstants.VARIABLES_WITH_SOURCE_INFORMATION, StringConstants.DIMENSIONLESS_UNIT_VARIABLES, StringConstants.FUNCTION_SENSITIVITY_PARAMETERS, StringConstants.DATA_LOOKUP_TABLES,
		StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES, StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, StringConstants.VARIABLES_NOT_IN_ANY_VIEW, 
		"Incompletely Defined Subscripted Variables",StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
		StringConstants.CASCADING_LOOKUP_FUNCTIONS, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, 
		StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,
		StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1,StringConstants.UNUSED_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, 
		"Complex Variable Formulations (Richardson", StringConstants.COMPLEX_STOCK
		};
	
	protected Documentor documentor;
	
	public TableData(Documentor documentor) {
		this.documentor = documentor;
	}
	
	protected void addTitle(List<List<List<String>>> rows, String title) {
		 // this should be an empty table, so row 0, col 0 will contain title
		List<List<String>> row = createRow();
		row.add(createColumn(true, title));
		rows.add(row);
	}
	
	protected void addColumnHeaders(List<List<List<String>>> rows, List<String> headers) {
		// this shoud be row 1
		List<List<String>> row = createRow();
		row.add(createColumn(headers));
		rows.add(row);
	}
	
	protected List<List<List<String>>> createTable() {
		return new ArrayList<List<List<String>>>();
	}
	
	protected void rowAddSeparator(List<List<List<String>>> rows) {
		rows.add(createRow(false, "", "" ,""));
		rows.add(createRow(false, "", "" ,""));
		rows.add(createRow(false, "", "" ,""));
	}
	
	protected void rowAddNoSplit(boolean withHTML, List<List<List<String>>> rows, String expandedName, /* String name,*/ String shortName, String value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t)) {
			List<List<String>> row = new ArrayList<List<String>>();
//			rows.add(createRow(withHTML, shortName ,value));
			row.add(createColumnNoSplit(shortName));
			row.add(createColumnNoSplit(value));
			rows.add(row);
		}
	}
	
	protected void rowAddNoSplit(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, long value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t)) {
			rows.add(createRow(withHTML,name, shortName ,value));
			List<List<String>> row = new ArrayList<List<String>>();
			row.add(createColumnNoSplit(name));
			row.add(createColumnNoSplit(shortName));
			row.add(createColumnNoSplit(value));
			rows.add(row);
		}
	}
	
	protected void rowAddNoSplit(boolean withHTML, List<List<List<String>>> rows, String expandedName, String shortName, long value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t)) {
			
			List<List<String>> row = new ArrayList<List<String>>();
			row.add(createColumnNoSplit(shortName));
			row.add(createColumnNoSplit(value));
			rows.add(row);
		}
	}
	
	public List<String> createColumnNoSplit(String v) {
		List<String> column = new ArrayList<String>();
		column.add(v);
		return column;
	}
	
	public List<String> createColumnNoSplit(long v) {
		List<String> column = new ArrayList<String>();
		column.add(Long.toString(v));
		return column;
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, /* String name,*/ String shortName, String value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML, shortName ,value));
	}
	
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, String value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, String extraStuff, String value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName+extraStuff ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, String extraStuff, long value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName+extraStuff ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, long value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String shortName, long value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,shortName ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, String extraStuff, int value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName+extraStuff ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, int value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String shortName, boolean value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,shortName ,value));
	}
	
	protected void rowAdd(boolean withHTML, List<List<List<String>>> rows, String expandedName, String name, String shortName, boolean value) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		String t = UserSelectedOptions.TRUE;
		if (options.getOptionValue(expandedName).equals(t))
			rows.add(createRow(withHTML,name, shortName ,value));
	}
	
	protected List<List<String>> createRow(boolean withHTML,String name, String s1) {
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,name));
		row.add(createColumn(withHTML,s1));
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String name, String s1, String s2) {
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,name));
		row.add(createColumn(withHTML,s1));
		row.add(createColumn(withHTML,s2));
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String name, String s1, int s2) {
		List<List<String>> row = createRow();
		row.add(createColumn(withHTML,name));
		row.add(createColumn(withHTML,s1));
		if (needsCondition(s1))
			row.add(createColumn(withHTML,NumberFormatter.formatForCountConditional(s2)));
		else
			row.add(createColumn(withHTML,NumberFormatter.formatForCountNotConditional(s2)));
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String name, String s1, long s2) {
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,name));
		row.add(createColumn(withHTML,s1));
		if (needsCondition(s1))
			row.add(createColumn(withHTML,NumberFormatter.formatForCountConditional(s2)));
		else
			row.add(createColumn(withHTML,NumberFormatter.formatForCountNotConditional(s2)));
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String s1, long s2) {
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,s1));
		if (needsCondition(s1))
			row.add(createColumn(withHTML,NumberFormatter.formatForCountConditional(s2)));
		else
			row.add(createColumn(withHTML,NumberFormatter.formatForCountNotConditional(s2)));
		return row;
	}
	
	protected boolean needsCondition(String name) {
		for (String s : naForMap) {
			if (s.startsWith(name))
				return true;
		}
		return false;
	}
	
	protected List<List<String>> createRow() {
		List<List<String>> row = TableUtilities.createRow();
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML) {
		List<List<String>> row = TableUtilities.createRow();
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String name, String s1, boolean s2) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,name));
		row.add(createColumn(withHTML,s1));
		row.add(createColumn(withHTML,(s2) ? language.getTextHtml(StringConstants.YES) : language.getTextHtml(StringConstants.NO)));
		return row;
	}
	
	protected List<List<String>> createRow(boolean withHTML,String s1, boolean s2) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<String>> row = createRow(withHTML);
		row.add(createColumn(withHTML,s1));
		row.add(createColumn(withHTML, s2 ? language.getTextHtml(StringConstants.YES) : language.getTextHtml(StringConstants.NO)));
		return row;
	}
	
	protected List<String> createColumn(boolean withHTML, String s1) {
		List<String> column;

		if (!withHTML) {
			column = TableUtilities.createColumn(s1);
		} else {
			column = new ArrayList<String>();
			column.add(s1);
		}

		return column;
	}
	
	protected List<String> createColumn(List<String> s1) {
		List<String> column = new ArrayList<String>();
		column.addAll(s1);
		return column;
	}

}
