package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;

public class TableUtilities {

	//	private static char[] terminators = { '+', '-', '*', '/', '(', ')', '=', ',', '^', '%', 
	//		'`', '>', '<', ':', ';', '{', '}', '|', ' ', '[', ']'};

	private static char[] terminators = { '+', '-',  '(', ')',  '%', // mjb 10/3/2020 ',',
		'{', '}', '|', ' ', '[', ']'};

	private static HashSet<Character> terminatorSet = new LinkedHashSet<Character>();


	public static List<List<List<String>>> createFlatFileTable() {
		return new ArrayList<List<List<String>>>();
	}

	public static void addTitle(List<List<List<String>>> table, String title) {
		List<List<String>> row = createRow();
		addRow(table, row);
		row.add(createColumn(title));
	}

	public static void addColumnHeaders(List<List<List<String>>> table, List<String> headers) {
		// this shoud be row 1
		List<List<String>> row = createRow();
		for (String col : headers) {
			row.add(createColumn(col));
		}
		addRow(table, row);
	}

	public static void addRow(List<List<List<String>>> table, List<List<String>> row) {

		//		List<List<String>> aRow = createRow();
		//		for (List<String> column : row) {
		//			List<String> c = createColumn(column.get(0));
		//			addColumn(aRow, c);
		//		}
		//		table.add(aRow);

		table.add(row);
	}

	public static List<List<String>> createRow() {
		List<List<String>> row = new ArrayList<List<String>>();
		return row;
	}

	public static List<List<String>> createRow(boolean withHTML) {
		List<List<String>> row = new ArrayList<List<String>>();
		return row;
	}

	public static void addColumn(List<List<String>> row, List<String> column) {
		row.add(column);
	}

	//	public static List<String> createColumn(String s1) {
	//		List<String> column = new ArrayList<String>();
	//		
	//try {
	//			//		List<String> tokens = Parser.tokenize(s1);
	//			List<String> decom = Parser.decompose(s1, false, true);
	//		} catch (Exception e) {
	//			// TODO Auto-generated catch block
	//			e.printStackTrace();
	//		}
	//		
	//		column.add(s1);
	//		return column;
	//	}

	public static String characterAt(MutableInteger position, String str) {
		String s =  str.substring(position.value(), position.value()+1);
		return str.substring(position.value(), position.value()+1);
	}

	private static void initializeTerminator() {
		for (char c : terminators) {
			terminatorSet.add(c);
		}
	}

	public static List<String> tokenizeColumnValue(String input) {
		
		if (terminatorSet.size() == 0)
			initializeTerminator();

		List<String> tokens = new ArrayList<String>();

		MutableInteger ptr = new MutableInteger();

		StringBuilder sb = new StringBuilder();

		while (ptr.value() < input.length()) {
			//			Parser.skipWhiteSpace(ptr, input);
			String character = characterAt(ptr, input);
			if (character.equals("-") && ptr.value() == 0) {
				sb.append(character);
			} else 	if (isTerminator(character.charAt(0))) {
				if (sb.toString().trim().length() > 0)
					tokens.add(sb.toString().trim());
				if (!character.equals(" "))
					tokens.add(character);
				sb.setLength(0);
			} else {
				sb.append(character);
			}

			ptr.add(1);
		}

		if (sb.length() > 0) {
			tokens.add(sb.toString().trim());
		}

		boolean candidate = true;

		if (tokens.size() > 0) {
			for (String t : tokens) {
				if (Parser.isNumber(t))
					continue;
				if (!isTerminator(t.charAt(0))) {
					candidate = false;
					break;
				}
			}
		} else {
			candidate = false;
		}

		if (tokens.size() > 1 && candidate) {
			//			System.out.println("Candidate: "+candidate+" "+tokens+"   input: "+input);
			MultiValuePattern mvp = new MultiValuePattern(tokens);

			if (matches2NumPattern(mvp)) {
				//				System.out.println("matches2NumPattern "+tokens);
			} else if (matchesNumberAndRangePattern(mvp)) {
				//				System.out.println("matchesNumberAndRangePattern "+tokens);
			} else if (matches3NumPattern(mvp)) {
				//				System.out.println("matches3NumPattern "+tokens);
			} else if (matches2NumPctPattern(mvp)) {
				//				System.out.println("matches2NumPctPattern "+tokens);
			} else if (matches4NumPattern(mvp)) {
				//				System.out.println("matches4NumPattern "+tokens);
			} else if (matches4NumPctPattern(mvp)) {
				//				System.out.println("matches4NumPctPattern "+tokens);
			} else if (matchesFeedbackPattern(mvp)) {
				//				System.out.println("matchesFeedbackPattern "+tokens);
			} else {
				System.out.println("No Match "+tokens);
				return null;
			}

			return justNumbers(tokens);
		} else {
			List<String> orig = new ArrayList<String>();
			orig.add(input);
			return orig;
		}
	}

	public static boolean matches2NumPattern(MultiValuePattern mvp) {
		//		 #|#
		return mvp.getNumMatched() == 3 && mvp.getBarCount() == 1 && mvp.getNumberCount() == 2;
	}

	public static boolean matches3NumPattern(MultiValuePattern mvp) {
		//		#|#|#
		return mvp.getNumMatched() == 5 && mvp.getBarCount() == 2 && mvp.getNumberCount() == 3;
	}

	public static boolean matches2NumPctPattern(MultiValuePattern mvp) {
		//		#(#%)
		return mvp.getNumMatched() == 5 && mvp.getPctCount() == 1 && mvp.getNumberCount() == 2
				&& mvp.getlParenCount() == 1 && mvp.getrParenCount() == 1;
	}

	public static boolean matches4NumPattern(MultiValuePattern mvp) {
		//		#{#%)|#{#%)
		return mvp.getNumMatched() == 11 && mvp.getPctCount() == 2 && mvp.getNumberCount() == 4
				&& mvp.getlParenCount() == 2 && mvp.getrParenCount() == 2 && mvp.getBarCount() == 1;
	}

	public static boolean matches4NumPctPattern(MultiValuePattern mvp) {
		//		#(#|#|#)
		return mvp.getNumMatched() == 8 && mvp.getBarCount() == 2 && mvp.getNumberCount() == 4
				&& mvp.getlParenCount() == 1 && mvp.getrParenCount() == 1;
	}

	public static boolean matchesFeedbackPattern(MultiValuePattern mvp) {
		//		#(#%)(+)# [#,#] (-) # [#,#]
		return mvp.getNumMatched() == 16 && mvp.getNumberCount() == 8
				&& mvp.getlParenCount() == 2 && mvp.getrParenCount() == 2
				&& mvp.getlBrackCount() == 2 && mvp.getrBrackCount() == 2;
	}

	public static boolean matchesNumberAndRangePattern(MultiValuePattern mvp) {
		//		#[#,#]
		return mvp.getNumMatched() == 5 && mvp.getNumberCount() == 3
				&& mvp.getlBrackCount() == 1 && mvp.getrBrackCount() == 1;
	}

	public static List<String> justNumbers(List<String> tokens) {
		List<String> numbers = new ArrayList<String>();
		for (String t : tokens) {
			if (Parser.isNumber(t))
				numbers.add(t);
		}
		return numbers;
	}

	public static  boolean isTerminator(char c) {
		if (terminatorSet.contains(c))
			return true;
		else
			return false;
	}

	public static List<String> createColumn(String s1) {
		List<String> column = new ArrayList<String>();

		List<String> tokens = tokenizeColumnValue(s1);
		if (tokens == null)
			column.add(s1);
		else
			column.addAll(tokens);
		return column;
	}

	public static List<String> createColumn(int s1) {
		List<String> column = new ArrayList<String>();
		String value = Integer.toString(s1);
		column.add(value);
		return column;
	}

	public static void printTable(List<List<List<String>>> table) {
		for (int r = 0; r < getNumRows(table); r++) {
			List<List<String>> row = getRow(table, r);
			for (int c = 0; c < getNumColumns(row); c++) {
				if (c > 0)
					System.out.print(", ");
				System.out.print(getColumn(row, c));
			}
			System.out.print("\n");
		}
	}

	public static List<List<String>> getRow(List<List<List<String>>> table, int row) {
		return table.get(row);
	}

	public static List<String> getColumn(List<List<String>> row, int column) {
		return row.get(column);
	}
	
	public static int getNumRows(List<List<List<String>>> table) {
		return table.size();
	}

	public static int getNumColumns(List<List<String>> row) {
		return row.size();
	}


}
