package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.MultiVariableSelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorAutoGenerated;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNotLookup;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationCharacteristics;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class TypeCounter extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(TypeCounter.class.getSimpleName());
	
	public static final String ENTRY5 = "   <td width='20%%'> <b>$$$ARG1$$$</b>: $$$ARG2$$$ ($$$ARG3$$$ / <b>$$$ARG4$$$</b>)$$$ARG5$$$</td>\n";
	public static final String ENTRY3 = "   <td width='20%%'> <b>$$$ARG1$$$</b>: $$$ARG2$$$ ($$$ARG3$$$)</td>\n";
	public static final String EMPTY = "   <td width='20%%'>&nbsp;</td>\n";

	private Map<String, Equation> equations;
	private Documentor documentor;
	
	int level = 0;
	int levelTotal = 0;
	int smooth = 0;
	int smoothTotal = 0;
	int delay = 0;
	int delayTotal = 0;
	int levelInitialization = 0;
	int levelInitializationTotal = 0;
	int initial = 0;
	int initialTotal = 0;
	int constant = 0;
	int constantTotal = 0;
	int flow = 0;
	int flowTotal = 0;
	int auxiliary = 0;
	int auxiliaryTotal = 0;
	int subscripts = 0;
	int subscriptsTotal = 0;
	int data = 0;
	int dataTotal = 0;
	int game = 0;
	int gameTotal = 0;
	int lookup = 0;
	int lookupTotal = 0;
	int allTypes = 0;
	int allTypesTotal = 0;
	
	public TypeCounter(Documentor documentor) {
		super(documentor);
		this.equations = documentor.getEquations();
		load();
	}
	
	@Override
	public void load() {

		MultiVariableSelector multiSelector = new MultiVariableSelector("&&");
		multiSelector.addSelector(new VariableSelectorAutoGenerated("AutoGen", "AutoGen"));
		multiSelector.addSelector(new VariableSelectorNotLookup("NotLookup", "NotLookup"));
		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
			if (multiSelector.isSelect(eqn)) {
				continue;
			}

			//		for (Equation eqn : equations.values()) {
			EquationCharacteristics ec = eqn.getEquationCharacteristics();
			if (ec == null) {
				log.debug("No EC fpr "+eqn.getVensimEquationOnly());
				continue;
			}
			if (ec.isLevel()) { 
				level++;
				levelTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isSmooth()) {
				smooth += eqn.getNumSmooth();
				smoothTotal += (eqn.getNumSmooth() + eqn.getNumSmooth3()*2) * InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isDelay()) {  
				delay += eqn.getNumDelay();
				delayTotal += (eqn.getNumDelay() + eqn.getNumDelay3()*2) * InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isLevelInitialization()) {  
				levelInitialization++;
				levelInitializationTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isInitial()) {
				initial++;
				initialTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isConstant()) {  
				constant++;
				constantTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isFlow()) {  
				flow++;
				flowTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isAuxiliary() && !ec.getEquation().isDefinesLookup() && !ec.getEquation().isAutoGenerated()) {  
				auxiliary++;
				auxiliaryTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
				
				if (InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var) > 1) {
					InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
				}
				
//				System.out.println("AUX: "+ec.getEquation().getLhsAsInMdl()+" "+auxiliary+
//						" / "+auxiliaryTotal+" "+ec.getEquation().getVensimEquationOnly());
			}
			if (ec.isSubscripts()) {  
				subscripts++;
			}
			if (/*ec.isData()*/ eqn.isDataByMeta()) {  
				data++;
				dataTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isGame()) {  
				game++;
				gameTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			if (ec.isLookup()) {  
				lookup++;
				lookupTotal += InformationManagers.getInstance().getEquationManager().getTotalEquationCount(var);
			}
			allTypes++;
		}

	}
	
	private List<List<List<String>>> createFlatFileTable() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<List<String>>> table = createTable();
		
		addTitle(table,language.getTextHtml(StringConstants.VARIABLE_TYPES));
		List<String> headers = new ArrayList<String>();
		headers.add(language.getTextHtml(StringConstants.VARIABLE_TYPES));
		headers.add(language.getTextHtml(StringConstants.COUNTS));
		addColumnHeaders(table, headers);
		
		return table;
	}
	
//	@Override
	public List<List<List<String>>> getTableData() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<List<List<String>>> table = createFlatFileTable();
		
		// Level
		addRow(HTMLConstants.WITHOUT_HTML, table, "L: "+language.getText(StringConstants.LEVEL), 
				NumberFormatter.formatForCountNotConditional(level)+"/"+ NumberFormatter.formatForCountNotConditional(levelTotal)+ "*");
		// Smooth
		addRow(HTMLConstants.WITHOUT_HTML, table, "SM: "+ language.getText(StringConstants.SMOOTH), 
				NumberFormatter.formatForCountNotConditional(smooth)+"/"+  NumberFormatter.formatForCountNotConditional(smoothTotal)+ "*");
		// Delay
		addRow(HTMLConstants.WITHOUT_HTML, table, "DE: "+ language.getText(StringConstants.DELAY), 
				NumberFormatter.formatForCountNotConditional(delay)+"/"+  NumberFormatter.formatForCountNotConditional(delayTotal)+ "*,**");
		// Level Ini
		addRow(HTMLConstants.WITHOUT_HTML, table, "LI: "+ language.getText(StringConstants.LEVEL_INITIAL), 
				NumberFormatter.formatForCountNotConditional(levelInitialization)+"/"+  NumberFormatter.formatForCountNotConditional(levelInitializationTotal));
		// Initial
		addRow(HTMLConstants.WITHOUT_HTML, table, "I: "+ language.getText(StringConstants.INITIAL), 
				NumberFormatter.formatForCountNotConditional(initial)+"/"+  NumberFormatter.formatForCountNotConditional(initialTotal));
		
		
		// Const
		addRow(HTMLConstants.WITHOUT_HTML, table, "C: "+ language.getText(StringConstants.CONSTANT), 
				NumberFormatter.formatForCountNotConditional(constant)+"/"+  NumberFormatter.formatForCountNotConditional(constantTotal));
		// Flow
		addRow(HTMLConstants.WITHOUT_HTML, table, "F: "+ language.getText(StringConstants.FLOW), 
				NumberFormatter.formatForCountNotConditional(flow)+"/"+  NumberFormatter.formatForCountNotConditional(flowTotal));
		// Aux
		addRow(HTMLConstants.WITHOUT_HTML, table, "A: "+ language.getText(StringConstants.AUXILIARY), 
				NumberFormatter.formatForCountNotConditional(auxiliary)+"/"+  NumberFormatter.formatForCountNotConditional(auxiliaryTotal));
		// Sub
		addRow(HTMLConstants.WITHOUT_HTML, table, "Sub: "+ language.getText(StringConstants.SUBSCRIPTS), NumberFormatter.formatForCountNotConditional(subscripts));
		// Data
		addRow(HTMLConstants.WITHOUT_HTML, table, "D: "+ language.getText(StringConstants.DATA),
				NumberFormatter.formatForCountNotConditional(data)+"/"+  NumberFormatter.formatForCountNotConditional(dataTotal));
		
		// Game
		addRow(HTMLConstants.WITHOUT_HTML, table, "G: "+ language.getText(StringConstants.GAME), 
				NumberFormatter.formatForCountNotConditional(game)+"/"+  NumberFormatter.formatForCountNotConditional(gameTotal));
		// Lookup
		addRow(HTMLConstants.WITHOUT_HTML, table, "T: "+ language.getText(StringConstants.LOOKUP), 
				NumberFormatter.formatForCountNotConditional(lookup)+"/"+  NumberFormatter.formatForCountNotConditional(lookupTotal)+ "*,**,***");

		return table;
	}
	
	protected void addRow(boolean withHTML, List<List<List<String>>> table, String s1, String s2) {
		List<List<String>> row = createRow();
		row.add(createColumn(withHTML, s1));
		row.add(createColumn(withHTML, s2));
		table.add(row);
		
	}

	@Override
	public List<List<List<String>>> getRows() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		List<String> col1,col2,col3,col4,col5;

		// Level
		col1 = createColumn(ENTRY5, "L", language.getText/*Html*/(StringConstants.LEVEL), 
				NumberFormatter.formatForCountNotConditional(level), NumberFormatter.formatForCountNotConditional(levelTotal), "*");
		// Smooth
		col2 = createColumn(ENTRY5, "SM", language.getText/*Html*/(StringConstants.SMOOTH), 
				NumberFormatter.formatForCountNotConditional(smooth), NumberFormatter.formatForCountNotConditional(smoothTotal), "*");
		// Delay
		col3 = createColumn(ENTRY5, "DE", language.getText/*Html*/(StringConstants.DELAY), 
				NumberFormatter.formatForCountNotConditional(delay), NumberFormatter.formatForCountNotConditional(delayTotal), "*&dagger;");
		// Level Ini
		col4 = createColumn(ENTRY3, "LI", language.getText/*Html*/(StringConstants.LEVEL_INITIAL), 
				NumberFormatter.formatForCountNotConditional(levelInitialization), NumberFormatter.formatForCountNotConditional(levelInitializationTotal), "");
		// Initial
		col5 = createColumn(ENTRY5, "I", language.getText/*Html*/(StringConstants.INITIAL), 
				NumberFormatter.formatForCountNotConditional(initial), NumberFormatter.formatForCountNotConditional(initialTotal), "");
		rows.add(createRow(col1, col2, col3, col4, col5));
		
		// Const
		col1 = createColumn(ENTRY5, "C", language.getText/*Html*/(StringConstants.CONSTANT), 
				NumberFormatter.formatForCountNotConditional(constant), NumberFormatter.formatForCountNotConditional(constantTotal), "");
		// Flow
		col2 = createColumn(ENTRY5, "F", language.getText/*Html*/(StringConstants.FLOW), 
				NumberFormatter.formatForCountNotConditional(flow), NumberFormatter.formatForCountNotConditional(flowTotal), "");
		// Aux
		col3 = createColumn(ENTRY5, "A", language.getText/*Html*/(StringConstants.AUXILIARY), 
				NumberFormatter.formatForCountNotConditional(auxiliary), NumberFormatter.formatForCountNotConditional(auxiliaryTotal), "");
		// Sub
		col4 = createColumn(ENTRY3, "Sub", language.getText/*Html*/(StringConstants.SUBSCRIPTS), 
				NumberFormatter.formatForCountNotConditional(subscripts));
		// Data
		col5 = createColumn(ENTRY5, "D", language.getText/*Html*/(StringConstants.DATA),
				NumberFormatter.formatForCountNotConditional(data), NumberFormatter.formatForCountNotConditional(dataTotal), "");
		rows.add(createRow(col1, col2, col3, col4, col5));
		
		// Game
		col1 = createColumn(ENTRY5, "G", language.getText/*Html*/(StringConstants.GAME), 
				NumberFormatter.formatForCountNotConditional(game), NumberFormatter.formatForCountNotConditional(gameTotal), "");
		// Lookup
		col2 = createColumn(ENTRY5, "T", language.getText/*Html*/(StringConstants.LOOKUP), 
				NumberFormatter.formatForCountNotConditional(lookup), NumberFormatter.formatForCountNotConditional(lookupTotal), "*&dagger;&dagger;");
		// Space
		col3 = createColumn(EMPTY, "", "", "");
		// Space
		col4 = createColumn(EMPTY, "", "", "");
		// Space
		col5 = createColumn(EMPTY, "", "", "");
		rows.add(createRow(col1, col2, col3, col4, col5));
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Types"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}

		return rows;
	}
	
	protected List<List<String>> createRow(List<String> col1, List<String> col2, List<String> col3, 
			List<String> col4, List<String> col5) {
		List<List<String>> row = new ArrayList<List<String>>();
		row.add(col1);
		row.add(col2);
		row.add(col3);
		row.add(col4);
		row.add(col5);
		return row;
	}
	
	protected List<String> createTableDataColumn(String s1) {
		List<String> column = new ArrayList<String>();
		column.add(s1);
		return column;
	}
	
	protected List<String> createColumn(String format, String s1, String s2, String s3) {
		List<String> column = new ArrayList<String>();
		String withAnchorRef = Anchors.getAnchorReference("-TYPE-"+s2);
		String html = format.replace("$$$ARG1$$$", s1).replace("$$$ARG2$$$", withAnchorRef).replace("$$$ARG3$$$", s3);
		column.add(html);
		return column;
	}
	
	protected List<String> createColumn(String format, String s1, String s2, String s3, String s4, String s5) {
		List<String> column = new ArrayList<String>();
		String withAnchorRef = Anchors.getAnchorReference("-TYPE-"+s2);
		String html = format.replace("$$$ARG1$$$", s1).replace("$$$ARG2$$$", withAnchorRef).replace("$$$ARG3$$$", s3)
				.replace("$$$ARG4$$$", s4).replace("$$$ARG5$$$", s5);
		column.add(html);
		return column;
	}
	
	public List<String> getGroupingNames() {
		List<String> groupings = new ArrayList<String>();
		
		groupings.add("Level");
		groupings.add("Smooth");
		groupings.add("Delay");
		groupings.add("LevelInitial");
		groupings.add("Initial");
		groupings.add("Constant");
		groupings.add("Flow");
		groupings.add("Auxiliary");
		groupings.add("Subscripts");
		groupings.add("Data");
		groupings.add("Game");
		groupings.add("Lookup");
		
		return groupings;
	}
	
	public void document() {
		log.debug("\n\n### Type Counts ###\n");
		log.debug("Level: "+ level);
		log.debug("Smooth: "+ smooth);
		log.debug("Delay: "+ delay);
		log.debug("Level Initialization: "+ levelInitialization);
		log.debug("Initial: "+ initial);
		log.debug("Constant: "+ constant);
		log.debug("Flow: "+ flow);
		log.debug("Auxiliary: "+ auxiliary);
		log.debug("Subscripts: "+ subscripts);
		log.debug("Data: "+ data);
		log.debug("Game: "+ game);
		log.debug("Lookup: "+ lookup);
		log.debug("All: "+ allTypes);
	}

	public int getLevel() {
		return level;
	}

	public int getSmooth() {
		return smooth;
	}

	public int getDelay() {
		return delay;
	}

	public int getLevelInitialization() {
		return levelInitialization;
	}

	public int getInitial() {
		return initial;
	}

	public int getConstant() {
		return constant;
	}

	public int getFlow() {
		return flow;
	}

	public int getAuxiliary() {
		return auxiliary;
	}

	public int getSubscripts() {
		return subscripts;
	}

	public int getData() {
		return data;
	}

	public int getGame() {
		return game;
	}

	public int getLookup() {
		return lookup;
	}

	public int getAllTypes() {
		return allTypes;
	}

	public int getSmoothTotal() {
		return smoothTotal;
	}

	public int getDelayTotal() {
		return delayTotal;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}

}
