package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.documentor.plots.LookupPlot;
import gov.anl.gss.sdm.server.model.documentor.plots.PlotManager;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.ArrayManager;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationCharacteristics;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;

public class VariableInformation extends TableData implements TableDataLoad {
static Logger log = Logger.getLogger(VariableInformation.class.getSimpleName());
	

public static final String NL = "\n";
	public static final String CELL_SECTION_MARKER = "__##__";
	
	public static final String NUMBER_MARKER = "$$$NUMBER$$$";
	
	private static final String GREEN_START = "<span style='color:Green'>";
	private static final String RED_START = "<span style='color:Red'>";
	private static final String COLOR_END = "</span>";
	
	private static final String P_START = "<p>";
	private static final String P_END = "</p>";
	
	private static final String MODULE_HEADER = "<td style='text-align:center; vertical-align:top'>";
	private static final String MODULE_FOOTER = "</td>\n";
//	private static final String BREAK = "</br>\n";
	private static final String BREAK = "<br>\n";
	private static final String START_COLUMN = "<td style='text-align:center; vertical-align:top'>\n";
	private static final String END_COLUMN = "</td>\n";
	
	
	private static final String LHS_HEADER = NL+"<td><b>";
	private static final String RHS_HEADER = "<br/>";
	
	private static final String DESCRIPTION_HEADER = NL+"<b>##Description##: </b>";
	private static final String DESCRIPTION_FOOTER = "";
	
	private static final String SOURCE_HEADER = "<b>##Source##: </b>";
	private static final String SOURCE_FOOTER = "";
	
	private static final String VIEWS_HEADER = NL+"<b>##PresentIn## $$$NUMBER$$$ ##Views##:</b><ul>";
	private static final String VIEWS_ITEM_HEADER = "<li>";
	private static final String USED_BY_HEADER = NL+"<b>##UsedBy##</b><ul>";
	private static final String USED_BY_ITEM_HEADER = "<li>";
	private static final String FEEDBACK_LOOPS_HEADER = NL+"<b>##FeedbackLoops##:</b>";
	
	private static final String LHS_FOOTER = "</b>";
	private static final String RHS_FOOTER = "";
	private static final String VIEWS_FOOTER = "</ul>";
	private static final String VIEWS_ITEM_FOOTER = "</li>";
	private static final String USED_BY_FOOTER = "</ul>";
	private static final String USED_BY_ITEM_FOOTER = "</li>";
	private static final String FEEDBACK_LOOPS_FOOTER = "";
	private static final String END_ROW = "</tr>\n";
	private static final String START_ROW = "</tr>\n";

	private String module;
	private String group;
	private String type;
	private String sortOrder;
	private String lhs;
	private String units;
	private String rhs;
	private String id;
	
	private List<List<String>> createdRow = null;
	
	private boolean skipMe = false;
	
	private String variableName;
	
	private List<String> views;
	private List<String> usedBy;
	private List<Loop> feedbackLoops;
	
	private List<String> errorMessages;
	
	private EquationCharacteristics equationCharacteristics;
	
	private Set<Equation> equations;
	private Equation anEquation;
	private AnalysisManager analysisManager;
	
	private Map<Equation, String> imageFile = new LinkedHashMap<Equation, String>();
	
	public VariableInformation(Documentor documentor, AnalysisManager analysisManager, String variableName) {
		super(documentor);
		
//		System.out.println("VI: "+variableName);
		this.equations = InformationManagers.getInstance().getEquationManager().getAllEquationsFor(variableName);
		this.analysisManager = analysisManager;
		anEquation = InformationManagers.getInstance().getEquationManager().getAnEquationFor(variableName);
		
		errorMessages = anEquation.getSemanticMessages();
		
		anEquation.setVariableInformation(this);
		for (Equation e : equations) {
			e.setVariableInformation(this);
		}
		id = "#" + anEquation.getVariableNumber();
		this.variableName = variableName;
		load();
		this.variableName = Parser.removeQuotes(variableName); // leave escaped embedded quotes // variableName.replace("\"", "");
	}
	
	public void load() {
		
		if (anEquation.isAbort() || (!anEquation.isSyntacticallyCorrect()) && !anEquation.isDefinesSubscript()) {

			skipMe = true;
			return;
		}

		Reuse reuse = InformationManagers.getInstance().getReuse();
		// if we are working on an equation that requires a image file, create it
		if (anEquation.isDefinesLookup() || anEquation.isDefinesLookupGetXls()) {
			for (Equation oneEquation : equations) {
			try {
				///////////////////////////////////////////////////////////////////////////
				
//				if (reuse.isThumbnails()) {
//					File thumbFile = new File(genFileName(var));
//					if (reuse.getDocGraphDirectory().equals(directory)) {
//						
//						if (thumbFile.exists())
//							continue;
//					} else {
//						File existing = new File(reuse.getDocGraphDirectory()+ "/"+ Utilities.makeLegal(var) + ".png");
//						try {
//							FileUtils.copyFile(existing, thumbFile);
//						} catch (IOException e) {
//							XYSeriesCollection dataset = new XYSeriesCollection();
//							dataset.addSeries(createSeries(var+seriesIndex, extendedMemory.getVarTimeVector(var) /*extendedMemory.getVarVector(var)*/,timeVector));
//								// need to create it
//								ThumbnailPlot tc = new ThumbnailPlot(dataset, genFileName(var));
//								tc.generate();
//						}
//						continue;
//					}
//					
//				} 	
				
				///////////////////////////////////////////////////////////////////////////
				if (!PlotManager.haveLookup(oneEquation)) {
					ModelManager mm = InformationManagers.getInstance().getModelManager();
					String modelFile = mm.getFileName();
					LookupPlot lp = new LookupPlot("Lookup Plot", oneEquation, modelFile);
					if (oneEquation.isDefinesLookupGetXls() && !oneEquation.isLookupXlsReadable()) {
						// ??
						continue;
					}
					imageFile.put(oneEquation, lp.getFileNameRelative()); // new File(lp.getFileName()).toURI().toString();
					PlotManager.registerLookup(oneEquation, imageFile.get(oneEquation));
				} else {
					imageFile.put(oneEquation, PlotManager.getLookupFile(oneEquation));
				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				System.out.println("Bad Lookup: "+oneEquation.getVensimEquationOnly());
				e.printStackTrace();
			}
		}
		}


		views = new ArrayList<String>();
		usedBy = new ArrayList<String>();
		feedbackLoops = new ArrayList<Loop>();

		module = anEquation.getModule();
		group = anEquation.getGroup();
		lhs = anEquation.getLhs();
		String lhst = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));
		units = anEquation.getUnits();
		rhs = anEquation.getRhs();
		
		if (anEquation.isDefinesSubscript()) {

			NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();
			ArrayManager am = InformationManagers.getInstance().getArrayManager();
			//			if (nsm.isNamedSubscript(lhs)) {
			List<String> u = am.getArraysUsingSubscript(lhs);
			for (String array : u) {
				usedBy.add(Parser.removeQuotes(array));
				for (Equation e : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(array)) {
					if (e != null) {
						for (String v : e.getViews()) {
							anEquation.addView(v);
						}
					}
				}
			}
			//			}

		} else {
			
			Set<String> aStringSet = analysisManager.getEquationManager().getUsedBy(lhst);

			if (aStringSet.size() > 0) {
				usedBy.addAll(analysisManager.getEquationManager().getUsedBy(lhst));
			}
		}

		
		if (feedbackLoops == null) {
			log.debug("feedbackLoops is NULL!");
		} else if (analysisManager.getLoopManager() == null) {
			log.debug("feedbackLoops.getLoopManager() is NULL!");
		} else if (analysisManager.getLoopManager().getLoopsContaining(lhs) == null) {
			log.debug("analysisManager.getLoopManager().getLoopsContaining(lhs) is NULL!");
		} else {
			Set<Loop> aLoopSet = analysisManager.getLoopManager().getLoopsContaining(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs));
			if (aLoopSet.size() > 0)
				feedbackLoops.addAll(analysisManager.getLoopManager().getLoopsContaining(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs)));
		}
		
		// ### this needs to be done for all equations in set
		
		for (Equation eqn : equations) {
			
		

		equationCharacteristics = new EquationCharacteristics(eqn);
		equationCharacteristics.setUnused(usedBy.size() == 0);
		eqn.setUnused(usedBy.size() == 0);
		eqn.setEquationCharacteristics(equationCharacteristics);

		// set as many other things as possible

		// determine if flow
		for (String screenName: usedBy) {
			Equation e = InformationManagers.getInstance().getEquationLookup().getEquation(screenName);
			if (e != null) {
				// if we are used by a stock, we will either be a flow, or we will be an initializer
				// or both!
				if (e.isStock()) {
					boolean found = false;
					for (String i : e.getFunctionInitialVariables()) {
						String o = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(i));
						if (o.equals(lhst)) {
							// we are part of initialization
							found = true;
							eqn.setLevelInitialization(true);
							equationCharacteristics.setLevelInitialization(true);
						}
					}
					found = false;
					for (String i : e.getNonFunctionInitialVariables()) {
						String o = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(i));
						if (o.equals(lhst)) {
							// we are part of initialization
							found = true;
							eqn.setLevelInitialization(true);
							equationCharacteristics.setLevelInitialization(true);
							
//							eqn.setFlow(true);
//							equationCharacteristics.setFlow(true);
						}
					}
					if (!found) {
//						eqn.setFlow(true);
//						equationCharacteristics.setFlow(true);
					} 

				} else {

				}
			} else {

			}
		}
		type = eqn.getType();
		if (type.equals("")) {
			type = "A";
			eqn.setAuxiliary(true);
			eqn.getEquationCharacteristics().setAuxiliary(true);
		}

		
			views = InformationManagers.getInstance().getEquationManager().getViews(variableName);
		

		if (eqn.isAssignment()) {
			if (!eqn.isStock() && !eqn.isConstant() && !eqn.isSupplementary()) {
				eqn.setAuxiliary(true);
				eqn.getEquationCharacteristics().setAuxiliary(true);
			}
		}
	}
		// ### end of individual equation processing


	}
	
	public void document() {
		log.debug("### "+lhs+" ###\n");
		log.debug("Module: "+module);
		log.debug("Group: "+group);
		log.debug("LHS: "+anEquation.getOriginalLhs());
		log.debug("Units: "+units);
		log.debug("RHS: "+rhs);
		
		log.debug("\nUsed By:");
		for (String s : usedBy) {
			log.debug("   "+s);
		}
		
		log.debug("\nFeedback Loops: "+feedbackLoops.size()+" ("+analysisManager.getLoopManager().getPercentageOfLoopsWith(lhs)+"%)");
	}
	
	public int getViewsCount() {
		return views.size();
	}
	
	public int getUsedByCount() {
		return usedBy.size();
	}
	
	public int getFeedbackLoopsCount() {
		return feedbackLoops.size();
	}

	public String getModule() {
		return module;
	}

	public String getGroup() {
		return group;
	}

	public String getType() {
		return type;
	}

	public String getSortOrder() {
		return sortOrder;
	}

	public String getLhs() {
		return lhs;
	}

	public String getUnits() {
		return units;
	}

	public String getRhs() {
		return rhs;
	}

	public List<String> getViews() {
		return views;
	}

	public List<String> getUsedBy() {
		return usedBy;
	}

	public List<Loop> getFeedbackLoops() {
		return feedbackLoops;
	}

	public Equation getAnEquation() {
		return anEquation;
	}
	
	public boolean hasImage(Equation eqn) {
		return imageFile.containsKey(eqn);
	}
	
	public String getImage(Equation eqn) {
		if (!imageFile.containsKey(eqn))
			return "";
		else
		
			return "<img src='"+Utilities.escapePound(imageFile.get(eqn))+"'/>";
	}

	public List<List<String>> createRow() {
		
		if (createdRow != null) {
//			System.out.println("VI: reuse "+variableName);
			return createdRow;
		}

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();

		List<String> columnValues = new ArrayList<String>();
		List<List<String>> row = new ArrayList<List<String>>();
		StringBuilder sb;


		if (documentor.displayModule()) {
			// module column
			columnValues.add(MODULE_HEADER+module+MODULE_FOOTER);
			row.add(columnValues);
		}

		// group Column
		columnValues= new ArrayList<String>();
		columnValues.add(MODULE_HEADER+group+MODULE_FOOTER);
		row.add(columnValues);
		// type Column
		columnValues= new ArrayList<String>();
		columnValues.add(MODULE_HEADER+id+BREAK+type+MODULE_FOOTER);
		row.add(columnValues);


		sb = new StringBuilder();
		// build details column
		columnValues= new ArrayList<String>();

		// ################### BEGIN loop for multiple equations

		String us = "&nbsp;"+RED_START+"("+StringEscapeUtils.escapeHtml4(units)+")"+COLOR_END;

		// lhs
		// if lhs is an array reference, we need just screen name first
		String thisVariableName = lhs;
		
		if (anEquation.isHasLHSArrayReference()) {
			String stringName = getScreenName(lhs);
			thisVariableName = stringName;
			sb.append(LHS_HEADER+"$$$DETAILNAME$$$"+Anchors.getAnchorDefinition(stringName)+StringEscapeUtils.escapeHtml4(stringName)+
					us+LHS_FOOTER);
		} else {
			String lhsNoQuotes = Parser.removeQuotes(lhs).replace("\\\"", "\""); // lhs.replace("\"", "");
			sb.append(LHS_HEADER+"$$$DETAILNAME$$$"+Anchors.getAnchorDefinition(lhsNoQuotes)+StringEscapeUtils.escapeHtml4(lhsNoQuotes)+us+LHS_FOOTER);
		}

		int eqnNum = 1;
		sb.append(RHS_HEADER);
		for (Equation eqn : equations) {
			// rhs
			// MJB Need to get and work with equation tree if we have an INTEG Function


			if (eqn.isStock()) {
				// convert INTEG call to calculus notation
				//			<span style='font-size:x-large'>&#8747;</span>
				//			<a href='#a64'>Associated</a>[]-<a href='#a30'>Natural</a>[] 
				//			<i>dt</i> + [<a href='#a56'>Ammount</a>[]]
				
				// need to build this in a different order. Get the tagged expressions,
				// then add extra stuff

				Node root = eqn.getTreeRoot();
				Node integ = TreeTraversal.getRhs(root);
				Node arg1 = TreeTraversal.getFunctionArgument(integ, 1);
				Node arg2 = TreeTraversal.getFunctionArgument(integ, 2);
				StringBuilder sbuf = new StringBuilder();
				String lhs;
				String a1;
				String a2;
				lhs = StringEscapeUtils.escapeHtml4(getOriginalName(eqn.getLhs()));
				if (eqn.isHasLHSArrayReference()) {
//					a1 = taggedRhs(removeSubscripts(Node.generateExpression(arg1)), false);
					a1 = taggedRhs(Node.generateExpression(arg1), false, eqn);
//					a2 = taggedRhs(removeSubscripts(Node.generateExpression(arg2)), false);
					a2 = taggedRhs(Node.generateExpression(arg2), false, eqn);
				} else {
					a1 = taggedRhs(Node.generateExpression(arg1), false, eqn);
					a2 = taggedRhs(Node.generateExpression(arg2), false, eqn); 
				}
				
				sbuf.append("<span style='font-size:x-large'>&#8747;</span>");
//				sbuf.append(Node.generateExpression(arg1).replace("\"", "")); 
				sbuf.append(a1);
				sbuf.append("&nbsp;<i>dt</i> + ");
				sbuf.append(a2);
//				sbuf.append(Node.generateExpression(arg2).replace("\"", ""));
				if (eqn.isHasLHSArrayReference()) {
					sb.append(StringEscapeUtils.escapeHtml4(getOriginalName(eqn.getLhs()))+" = " + removeSubscripts(sbuf.toString()));
				} else {
					sb.append("= " + sbuf.toString());
				}
			} else {

				if (eqn.isHasLHSArrayReference()) {
					if (!eqn.isAssignment() /* || eqn.getRHSVariables().size() == 0*/ ) {
						sb.append(StringEscapeUtils.escapeHtml4(eqn.getVensimEquationOnly()));
					} else {
						sb.append(StringEscapeUtils.escapeHtml4(getOriginalName(eqn.getLhs()))+" = " + removeSubscripts(taggedRhs(eqn.getRhs(), false, eqn)));
					}
				} else {
					if (!eqn.isAssignment() /* || eqn.getRHSVariables().size() == 0 */) {
						sb.append(StringEscapeUtils.escapeHtml4(eqn.getVensimEquationOnlyExtended()));
					} else {
						sb.append("= " + taggedRhs(eqn.getRhs(), false, eqn));
					}
				}
			}
			sb.append(RHS_FOOTER);
			sb.append(BREAK);
			
			List<String> errors = eqn.getSemanticMessages();
			if (errors.size() > 0) {
				for (String msg : errors) {
					// RED_START+"("+StringEscapeUtils.escapeHtml4(units)+")"+COLOR_END
					String[] ehtml = msg.split("\n");
					sb.append(RED_START+"<br><b>ERROR:");
					for (String e : ehtml) {
					sb.append(BREAK+StringEscapeUtils.escapeHtml4(e));
					}
					sb.append(COLOR_END+"</b><br>");;
				}
				sb.append(BREAK);
				sb.append(BREAK);
			}
			//		columnValues.add(sb.toString());
			//		row.add(columnValues);

			if (hasImage(eqn)) {
				//				sb.append(BREAK);
				sb.append(getImage(eqn));
				sb.append(BREAK);
			}

			if ((isMultiEquation() && eqnNum == getNumEquations()) || !isMultiEquation()) {
				if (eqn.getComment() != null && eqn.getComment().length() > 0) {
					if (options.isSelected("Description")) {
						// Description
						sb.append(DESCRIPTION_HEADER.replace("##Description##", language.getTextHtml(StringConstants.DESCRIPTION)));
						sb.append(StringEscapeUtils.escapeHtml4(eqn.getStrippedComment()));
						sb.append(DESCRIPTION_FOOTER);
						sb.append(BREAK);
					}
						if (options.isSelected("Meta") && eqn.isContainsSource()) {
							sb.append(SOURCE_HEADER.replace("##Source##", language.getTextHtml(StringConstants.SOURCE)));
							sb.append(StringEscapeUtils.escapeHtml4(eqn.getSource()));
							sb.append(SOURCE_FOOTER);
							sb.append(BREAK);
						
					} 
						
				}
				if (!options.isSelected("Description") && !(options.isSelected("Meta") && eqn.isContainsSource())) {
					sb.append(BREAK);
				}
			} 

			eqnNum++;
		}
		//			sb.append(BREAK);
		// ### End of multiequation loop

		//		columnValues= new ArrayList<String>();

		if (options.isSelected(StringConstants.PRESENT_IN_VIEWS)) {
			if (views.get(0).contains("Not in View")) {
				sb.append(VIEWS_HEADER.replace(NUMBER_MARKER, "0").replace("##PresentIn##", language.getTextHtml(StringConstants.PRESENT_IN)).
						replace("##Views##", language.getTextHtml(StringConstants.VIEWS)));
			} else {
				String numView = NumberFormatter.formatForCountNotConditional(views.size());
				sb.append(VIEWS_HEADER.replace(NUMBER_MARKER, numView).replace("##PresentIn##", language.getTextHtml(StringConstants.PRESENT_IN)).
						replace("##Views##", numView.equals("1") ? language.getTextHtml(StringConstants.VIEW) : language.getTextHtml(StringConstants.VIEWS)));
				// views
				Collections.sort(views);
				for (String v : views) {
					sb.append(VIEWS_ITEM_HEADER+Anchors.getAnchorReference("-VIEW-"+v)+VIEWS_ITEM_FOOTER);
				}
			}
			sb.append(VIEWS_FOOTER);

		} else {
			sb.append(BREAK);
		}
		//		sb.append(CELL_SECTION_MARKER);
		// used by

		if (options.isSelected(StringConstants.USED_BY)) {
			sb.append(USED_BY_HEADER.replace("##UsedBy##", language.getTextHtml(StringConstants.USED_BY)));
			List<String> tempUsedBy = dequote(usedBy);
			Collections.sort(tempUsedBy);
			for (String v : tempUsedBy) {
				Equation u = InformationManagers.getInstance().getEquationManager().getAnEquationFor(v);
				if (u == null) {

					if (!v.startsWith(GraphicObject.CLOUD_NAME) && !v.startsWith("RATE_"))
						log.debug("NOT CLOUD? "+v);
					continue;
				}
				//			sb.append(USED_BY_ITEM_HEADER+Anchors.getAnchorReference("-ALL-"+ v));
				sb.append(USED_BY_ITEM_HEADER+Anchors.getAnchorReference(v));

				if (u.getComment() != null && u.getComment().length() > 0) {
					sb.append(GREEN_START);
					sb.append("  "+StringEscapeUtils.escapeHtml4(u.getComment()));
					sb.append(COLOR_END);
					sb.append(USED_BY_ITEM_FOOTER);
				}

				// <span style='color:Green'>
			}
			sb.append(USED_BY_FOOTER);
		} else {
			sb.append(BREAK);
		}

		//		sb.append(CELL_SECTION_MARKER);
		// feedback loops
		if (options.isSelected(StringConstants.FEEDBACK_LOOPS)) {
			ModelManager mm = InformationManagers.getInstance().getModelManager();
			String html = FEEDBACK_LOOPS_HEADER.replace("##FeedbackLoops##", language.getTextHtml(StringConstants.FEEDBACK_LOOPS));
			// <a href='#VariableLoopsBIRTHRATENORMAL'>
			if (!InformationManagers.getInstance().isLimitLoopProcessing()) {
//				sb.append("<a href='"+StringEscapeUtils.escapeHtml4(mm.getOutputDirectoryName()+"/"+mm.getModelName())+"_Variable_Loops_"+
//				sb.append("<a href='"+StringEscapeUtils.escapeHtml4(mm.getOutputDirectoryName()+"/"+mm.getModelName())+"_LoopsNoIVV"+
//						SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+ ".html"+"#VariableLoops"+Anchors.createAnchorName(thisVariableName)+"'>");
				sb.append("\n<a href='"+"#ListFeedbackLoops"+StringEscapeUtils.escapeHtml4(Anchors.createAnchorName(thisVariableName))+"'>");
				sb.append(html);
				sb.append("</a>");
			} else {
				sb.append(html);
			}
			
			LoopManager lm = analysisManager.getLoopManager();
//			sb.append(Anchors.getAnchorReference("VariableLoops"+thisVariableName, html));
			if (lm.isUsingMinimumMemory()) {
//			sb.append("  "+NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWith(lhs))+
					sb.append("  "+NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWith(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs)))+
					analysisManager.getLoopManager().getPercentagePositiveNegativeCounts(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs)));
			} else {
			sb.append("  "+NumberFormatter.formatForCountNotConditional(feedbackLoops.size())+
					analysisManager.getLoopManager().getPercentagePositiveNegativeCounts(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs)));
			}
			sb.append(FEEDBACK_LOOPS_FOOTER);
		} else {
			sb.append(BREAK);
		}
		sb.append(END_COLUMN);
		if (documentor.displayThumbNail()) {
			String stringName = lhs;
			if (anEquation.isHasLHSArrayReference()) {
				stringName = getScreenName(lhs);
			}
			
			
			if (!anEquation.isHasLHSArrayReference()) {
				// normal scalar processing
				if (!stringName.contains(StringConstants.RSSD_LOOKUP) && !hasImage(anEquation)) {
					String outputDirRelative = Utilities.escapePound(InformationManagers.getInstance().getFileOrganizer().getThumbnailsDirectory());
					String absolute = InformationManagers.getInstance().getFileOrganizer().getThumbnailsDirectory();
					sb.append(START_COLUMN);
//					String absThumbnail = absolute+"/"+Utilities.makeLegalJavaName(stringName)+".png";
					String absThumbnail = StringEscapeUtils.escapeHtml4(absolute+"/"+Utilities.makeLegalFilename(stringName)+".png");
					if (new File(absThumbnail).exists()) {
//						sb.append("<img src='"+outputDirRelative+"/"+Utilities.makeLegalJavaName(stringName)+".png'/>");
						sb.append("<img src='"+outputDirRelative+"/"+Utilities.makeLegalFilename(stringName)+".png'/>");
					}
					sb.append(END_COLUMN);
				} else {
					sb.append(START_COLUMN);
					sb.append(END_COLUMN);
				}
			} else {
				// multiple thumbnails for array



				if (!stringName.contains(StringConstants.RSSD_LOOKUP) && !hasImage(anEquation)) {
					String outputDirRelative0 = InformationManagers.getInstance().getModelManager().getOutputDirectoryName();
					String outputDirRelative = InformationManagers.getInstance().getFileOrganizer().getThumbnailsDirectory();
					ArrayReference ar = new ArrayReference(anEquation.getOriginalLhs());
					sb.append(START_COLUMN);
//					for (SubscriptCombination outerSub : InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(ar.getSubscriptsAsArray())) {
						// Build an array reference based on the subscript value
//						String subr = "[" + outerSub.getSubscriptValue() + "]";
//						String file = stringName+subr;
						
						String absolute = InformationManagers.getInstance().getFileOrganizer().getThumbnailsDirectory();
						if (new File(absolute+"/"+Utilities.makeLegalFilename(stringName)+".png").exists()) {
//							sb.append(subr);
							sb.append(BREAK);
							sb.append("<img src='"+outputDirRelative+"/"+Utilities.makeLegalFilename(stringName)+".png'/>");
							sb.append(BREAK);
						}
//					}
					sb.append(END_COLUMN);
				} else {
					sb.append(START_COLUMN);
					sb.append(END_COLUMN);
				}
			}
		}
		sb.append(END_ROW);
		columnValues.add(sb.toString());
		//		columnValues.add(sb.toString().replace(StringConstants.RSSD_LOOKUP, ""));
		row.add(columnValues);


		createdRow = row;
		return row;
	}
	
	public List<String> dequote(List<String> orig) {
		List<String> dequoted = new ArrayList<String>();
		for (String s : orig) {
			if (s.startsWith("\""))  {
				dequoted.add(Parser.removeQuotes(s));
			} else {
				dequoted.add(s);
			}
		}
		
		return dequoted;
	}
	
	public List<List<String>> createDataTableRow() {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();

		List<List<String>> row = TableUtilities.createRow();
		
		if (documentor.displayModule()) {
			// module column
			TableUtilities.addColumn(row, TableUtilities.createColumn(module));
		}
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(group));
		TableUtilities.addColumn(row, TableUtilities.createColumn(type));
		TableUtilities.addColumn(row, TableUtilities.createColumn(id));

		// ################### BEGIN loop for multiple equations

		if (anEquation.isHasLHSArrayReference()) {
			String stringName = getScreenName(lhs);
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(stringName));
			
		} else {
			String lhsNoQuotes = Parser.removeQuotes(lhs); // lhs.replace("\"", "");
			TableUtilities.addColumn(row, TableUtilities.createColumn(lhsNoQuotes));
		}
		TableUtilities.addColumn(row, TableUtilities.createColumn(units));

		int eqnNum = 1;
		

		// fill one column with all equations, one equation per column
		
		boolean multiProcessed = false;
		
		for (Equation eqn : equations) {
			// rhs
			// MJB Need to get and work with equation tree if we have an INTEG Function


			if (eqn.isStock()) {
				// convert INTEG call to calculus notation
				//			<span style='font-size:x-large'>&#8747;</span>
				//			<a href='#a64'>Associated</a>[]-<a href='#a30'>Natural</a>[] 
				//			<i>dt</i> + [<a href='#a56'>Ammount</a>[]]
				
				// need to build this in a different order. Get the tagged expressions,
				// then add extra stuff

				Node root = eqn.getTreeRoot();
				Node integ = TreeTraversal.getRhs(root);
				Node arg1 = TreeTraversal.getFunctionArgument(integ, 1);
				Node arg2 = TreeTraversal.getFunctionArgument(integ, 2);
				StringBuilder sbuf = new StringBuilder();
				String lhs;
				String a1;
				String a2;
				lhs = StringEscapeUtils.escapeHtml4(getOriginalName(eqn.getLhs()));
				if (eqn.isHasLHSArrayReference()) {
					a1 = taggedRhs(Node.generateExpression(arg1), true, false, eqn);
					a2 = taggedRhs(Node.generateExpression(arg2), true, false, eqn);
				} else {
					a1 = taggedRhs(Node.generateExpression(arg1), true, false, eqn);
					a2 = taggedRhs(Node.generateExpression(arg2), true, false, eqn); 
				}

				sbuf.append(a1);

				sbuf.append(a2);

				if (eqn.isHasLHSArrayReference()) {
					TableUtilities.addColumn(row, TableUtilities.createColumn(getOriginalName(eqn.getLhs()) +" = " + removeSubscripts(sbuf.toString())));
				} else {
					TableUtilities.addColumn(row, TableUtilities.createColumn(sbuf.toString()));
				}
			} else { // end of stock processing -- start of other

				if (eqn.isHasLHSArrayReference() /* || eqn.getRHSVariables().size() == 0 */ ) {
					if (!eqn.isAssignment()) {
						TableUtilities.addColumn(row, TableUtilities.createColumn(eqn.getVensimEquationOnly()));
					} else {
						if (!multiProcessed) {
							multiProcessed = true;
							StringBuilder sb = new StringBuilder();
							int num = 0;
							for (int e = 0; e < equations.size(); e++) {
								if (num++ > 0)
									sb.append("  ");
								for (Equation eq2 : equations ) {
									sb.append(getOriginalName(eq2.getLhs())+" = " + 
											removeSubscripts(taggedRhs(eq2.getRhs(), true, false, eqn)));
								}
							}

							TableUtilities.addColumn(row,TableUtilities.createColumn(sb.toString()));
						}
					}
				} else {
					if (!eqn.isAssignment() /* || eqn.getRHSVariables().size() == 0 */) {
						TableUtilities.addColumn(row, TableUtilities.createColumn(eqn.getVensimEquationOnlyExtended()));
					} else {
						TableUtilities.addColumn(row, TableUtilities.createColumn(taggedRhs(eqn.getRhs(), true, false, eqn)));
					}
				}
			}

			if ((isMultiEquation() && eqnNum == getNumEquations()) || !isMultiEquation()) {
				if (eqn.getComment() != null && eqn.getComment().length() > 0) {
					if (options.isSelected("Description")) {
						// Description
						TableUtilities.addColumn(row, TableUtilities.createColumn(eqn.getStrippedComment()));
					} else {
						TableUtilities.addColumn(row, TableUtilities.createColumn(""));
					}
					if (options.isSelected("Meta") && eqn.isContainsSource()) {
						TableUtilities.addColumn(row, TableUtilities.createColumn(eqn.getSource()));

					} else {
						TableUtilities.addColumn(row, TableUtilities.createColumn(""));
					}

				} else {
					TableUtilities.addColumn(row, TableUtilities.createColumn(""));
					TableUtilities.addColumn(row, TableUtilities.createColumn(""));
				}
			} 

			eqnNum++;
		}
		
		StringBuilder sb = new StringBuilder();

		if (options.isSelected(StringConstants.PRESENT_IN_VIEWS)) {
			if (views.get(0).contains("Not in View")) {
				
//				TableUtilities.addColumn(row, TableUtilities.createColumn(""));
			} else {
				String numView = NumberFormatter.formatForCountNotConditional(views.size());
				
				// views
				
				Collections.sort(views);
				for (String v : views) {
					if (sb.length() > 0)
						sb.append(", ");
					sb.append(v);
				}
			}
			TableUtilities.addColumn(row, TableUtilities.createColumn(sb.toString()));

		} 
		//		sb.append(CELL_SECTION_MARKER);
		// used by

		StringBuilder sb2 = new StringBuilder();
		if (options.isSelected(StringConstants.USED_BY)) {
			
			Collections.sort(usedBy);
			for (String v : usedBy) {
				Equation u = InformationManagers.getInstance().getEquationManager().getAnEquationFor(v);
				if (u == null) {
					continue;
				}
				
				if (sb2.length() > 0)
					sb2.append(", ");
				sb2.append(v);

				if (u.getComment() != null && u.getComment().length() > 0) {
					sb2.append(" // "+u.getComment() + " //");
					
				}
			}
			TableUtilities.addColumn(row, TableUtilities.createColumn(sb2.toString()));
		} 
		


		// feedback loops
		StringBuilder sb3 = new StringBuilder();
		if (options.isSelected(StringConstants.FEEDBACK_LOOPS)) {
			ModelManager mm = InformationManagers.getInstance().getModelManager();
			
			LoopManager lm = analysisManager.getLoopManager();

			if (lm.isUsingMinimumMemory()) {

					sb3.append("  "+NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWith(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs)))+
					analysisManager.getLoopManager().getPercentagePositiveNegativeCounts(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs), false));
			} else {
			sb3.append("  "+NumberFormatter.formatForCountNotConditional(feedbackLoops.size())+
					analysisManager.getLoopManager().getPercentagePositiveNegativeCounts(InformationManagers.getInstance().getEquationLookup().getScreenName(lhs), false));
			}
		} 
		
		String[] fl = sb3.toString().split(" ");
		
		// 2 3 5 7 9 11 -> need to split 7 and 11
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(fl[2]));
		TableUtilities.addColumn(row, TableUtilities.createColumn(fl[3]));
		TableUtilities.addColumn(row, TableUtilities.createColumn(fl[5]));
		String[] range = getRangeValues(fl[7]);
		TableUtilities.addColumn(row, TableUtilities.createColumn(range[0]));
		TableUtilities.addColumn(row, TableUtilities.createColumn(range[1]));
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(fl[9]));
		
		range = getRangeValues(fl[11]);
		TableUtilities.addColumn(row, TableUtilities.createColumn(range[0]));
		TableUtilities.addColumn(row, TableUtilities.createColumn(range[1]));
		
//		TableUtilities.addColumn(row, TableUtilities.createColumn(sb3.toString()));


		return row;
	}
	
	private String[] getRangeValues(String range) {
		String[] rv = new String[2];
		String[] t = range.split(",");
		rv[0] = t[0].replace("[", "");
		rv[1] = t[1].replace("]", "");
		
		return rv;
	}
	
	private String taggedRhs(String rhs, boolean noEscape, Equation anEquation) {
		return taggedRhs(rhs, false, noEscape, anEquation);
	}
	
	private String taggedRhs(String rhs, boolean noHTML, boolean noEscape, Equation anEquation) {

		NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();

		String r = new String( Parser.removeQuotes(rhs)); // rhs.replace("\"", ""));
		List<String> r2 = Parser.tokenize(rhs);
		Map<String, String> encoded = new LinkedHashMap<String, String>();
		int id = 0;

		String varName = anEquation.getVariableName();
		Set<String> allVarAllEqn = new LinkedHashSet<String>();
		for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(anEquation.getVariableName())) {
			allVarAllEqn.addAll(Node.extractVariables(TreeTraversal.getRhs(eqn.getTreeRoot())));
		}

		List<String> extractedVariables = Node.extractVariables(TreeTraversal.getRhs(anEquation.getTreeRoot()));
		for (String rhsVar : extractedVariables) { 
			//		for (String rhsVar : allVarAllEqn) { 

			// need to get subscripts too. Have a list of var + subscripts (individually)

			List<String> varAndSubs = new ArrayList<String>();
			String t = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalNameRemovingAnnotation(rhsVar));
			varAndSubs.add(t);
			if (ArrayReference.isArrayReference(rhsVar)) {
				List<String> subrs = new ArrayReference(rhsVar).extractSubscripts(rhsVar);
				varAndSubs.addAll(subrs);
			}

			int item = 0;
			for (String vOrSt : varAndSubs) {
				boolean haveLinkAvailable = true;
				String vOrS = Parser.removeQuotes(vOrSt); // vOrSt.replace("\"", "");
				String marker = "###" + (id++) + "###";
				String reference;
				if (item++ == 0) {
					if (noHTML) {
							reference = vOrS;
					} else {
							reference = noEscape == true ? Anchors.getAnchorReferenceNoEscape(vOrS) : Anchors.getAnchorReference(vOrS);
					}
				} else {
					if (nsm.isNamedSubscript(vOrS)) {
						if (noHTML) {
							if (vOrS.endsWith("!")) {
								reference = vOrS.replace("!", "");
							} else {
								reference = vOrS;
							}
						} else {
							if (vOrS.endsWith("!")) {
								reference = noEscape == true ? Anchors.getAnchorReferenceNoEscape(vOrS.replace("!", ""), vOrS) : Anchors.getAnchorReference(vOrS.replace("!", ""), vOrS);
							} else {
								reference = noEscape == true ? Anchors.getAnchorReferenceNoEscape(vOrS) : Anchors.getAnchorReference(vOrS);
							}
						}
					} else {
						haveLinkAvailable = false;
						reference = null;

					}
				}

				if (haveLinkAvailable) {
					encoded.put(marker, reference);

					r = r.replace(vOrS, marker);
					for (int i = 0; i < r2.size(); i++) {
						if (r2.get(i).equals(vOrS)) {
							r2.set(i, marker);
						}
					}
				}
			}
		}

		for (String marker : encoded.keySet()) {
			r = r.replace(marker, encoded.get(marker));

			for (int i = 0; i < r2.size(); i++) {
				if (r2.get(i).equals(marker)) {
					r2.set(i, encoded.get(marker));
				}
			}
		}
		return Parser.concatenateList(r2);

	}
	
	private String getScreenName(String var) {
		return Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(var));
	}
	
	private String removeSubscripts(String var) {
		return var;
//		return Parser.noSubscriptValues(var);
	}
	
	private String getOriginalName(String var) {
		return InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(var);
	}

	public String getImageFile(Equation eqn) {
		return imageFile.get(eqn);
	}
	
	public boolean isMultiEquation() {
		return equations.size() > 0;
	}
	
	public int getNumEquations() {
		return equations.size();
	}

	public List<String> getErrorMessages() {
		return errorMessages;
	}

	/**
	 * @return the skipMe
	 */
	public boolean isSkipMe() {
		return skipMe;
	}
}
