package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.Anchors;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Logger;

public class ViewsCounter extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(ViewsCounter.class.getSimpleName());
	
//	public static final String ENTRY2 = "<td width='20%'><a href='' /><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String ENTRY2 = "<td width='20%'><b>$$$ARG1$$$</b> ($$$ARG2$$$)</td>";
	public static final String EMPTY = "<td width='20%%'>&nbsp;</td>";
	
	private static final String VIEW_HEADER = "<td><table class='sample'>\n" +
			   "<tr>\n" +	
			   "<td colspan=1 style='background:blue;color:White; font-size:large'><b>$$$VIEW$$$: $$$ANCHOR$$$ ($$$ARG2$$$ $$$VARIABLES$$$)</b></td>\n" +
			   "</tr>\n" +
			   "<tr>\n" +
			   "<td style='text-align:center; vertical-align:top'><img src='$$$OUTPUT_DIR$$$/$$$NAME$$$.png'/></td>\n" + 
			   "</tr>\n" +
			   "</table></td>\n";
	
	private Map<String, Set<String>> viewVariables;

	public ViewsCounter(Documentor documentor) {
		super(documentor);
		load();
	}
	
	public int getNumberOfViews() {
		if (viewVariables == null)
			return 0;
		return viewVariables.size();
	}

	@Override
	public void load() {
		viewVariables = new LinkedHashMap<String, Set<String>>();
		VariableSelector selector = new VariableSelector("Ignore", "Ignore");
		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
			Equation equation = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
			if (selector.ignore(equation))
				continue;
//		for (Equation equation : documentor.getEquations().values()) {
			
			for (String view : equation.getViews()) {
				if (!viewVariables.containsKey(view))
					viewVariables.put(view, new LinkedHashSet<String>());
				viewVariables.get(view).add(var);
			}
			
		}
	}
	
	public String getImages() {
		StringBuilder sb = new StringBuilder();
//		String outputDirRelative = Utilities.escapePound(InformationManagers.getInstance().getModelManager().getOutputDirectoryName());
		String outputDirRelative = Utilities.escapePound(InformationManagers.getInstance().getFileOrganizer().getRelativeViewsDirectory());
		// replace(ANCHOR_DEF_MARKER, Anchors.getAnchorDefinition("-"+"VIEW"+"-"+groupingName))
		List<String> views = new ArrayList<String>(viewVariables.keySet());
		Collections.sort(views);
		for (String view : views) {
			sb.append(VIEW_HEADER.replace("$$$NAME$$$", "__VIEW__"+Utilities.makeLegalJavaName(view)).replace("$$$OUTPUT_DIR$$$", outputDirRelative)
					.replace("$$$ARG2$$$", NumberFormatter.formatForCountNotConditional(viewVariables.get(view).size())));
		}
		
		return StringEscapeUtils.escapeHtml4(sb.toString());
	}
	
	@Override
	public List<List<List<String>>> getRows() {
		
		String outputDirRelative = InformationManagers.getInstance().getFileOrganizer().getRelativeViewsDirectory();
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		List<List<String>> columns = initializeList();

		List<String> views = new ArrayList<String>(viewVariables.keySet());
		Collections.sort(views);
		int col = 0;
		
		boolean requiresSave = false;
		
		for (String view : views) {

			if (view.contains("Not in View"))
				continue;
			// Level
			columns.add(createColumn(VIEW_HEADER, view, outputDirRelative,  viewVariables.get(view).size(), Anchors.getAnchorReferenceWithColor("-VIEW-"+view) ));
			requiresSave = true;
			col++;
			if (col > 0) {
				rows.add(columns);
				col = 0;
				columns = initializeList();
				requiresSave = false;
			}


		}
		if (requiresSave)
			rows.add(columns);
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Views"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}

		return rows;
	}
	
	public void document() {
		log.debug("\n\n### View variable Counts ###\n");
		List<String> views = new ArrayList<String>(viewVariables.keySet());
		Collections.sort(views);
		for (String view : views) {
			log.debug("View: "+view+" "+viewVariables.get(view).size());
			
			for (String vv : viewVariables.get(view)) {
				log.debug("     "+vv);
			}
		}
		
	}
	
	private List<List<String>> initializeList() {
		List<List<String>> columns = new ArrayList<List<String>>();
//		for (int i = 0; i < 5; i++) {
//			columns.add(new ArrayList<String>());
//		}
		return columns;
	}
	
//	protected List<String> createColumn(String format, String s1, String s2) {
//		List<String> column = new ArrayList<String>();
//		String html = format.replace("$$$ARG1$$$", s1).replace("$$$ARG2$$$", s2);
//		column.add(html);
//		return column;
//	}
	
	protected List<String> createColumn(String format, String s1, String s2, int s3, String anchor) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<String> column = new ArrayList<String>();
		String html = format.replace("$$$NAME$$$", "__VIEW__"+Utilities.makeLegalJavaName(s1))
				.replace("$$$OUTPUT_DIR$$$", StringEscapeUtils.escapeHtml4(s2))
				.replace("$$$VIEW$$$", language.getTextHtml(StringConstants.VIEW))
				.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES))
				.replace("$$$ARG2$$$", NumberFormatter.formatForCountNotConditional(s3))
				.replace("$$$ANCHOR$$$", anchor);
		
		// language.getTextHtml(StringConstants.MODEL_INFORMATION)
		
		column.add(html);
		return column;
	}
	
	public List<String> getGroupingNames() {
		List<String> al = new ArrayList<String>(viewVariables.keySet());
		Collections.sort(al);
		return al;
	}

	public Map<String, Set<String>> getViewVariables() {
		return viewVariables;
	}

	@Override
	public List<List<List<String>>> getTableData() {
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		String outputDirRelative = InformationManagers.getInstance().getModelManager().getOutputDirectoryName();
		
		List<String> views = new ArrayList<String>(viewVariables.keySet());
		Collections.sort(views);
		
		for (String group : views) {
			
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(group));
			TableUtilities.addColumn(row, TableUtilities.createColumn(NumberFormatter.formatForCountNotConditional(viewVariables.get(group).size())));
			
//			columns.add(createColumn(VIEW_HEADER, view, outputDirRelative,  viewVariables.get(view).size(), Anchors.getAnchorReferenceWithColor("-VIEW-"+view) ));
//			
//			
//			String html = format.replace("$$$NAME$$$", "__VIEW__"+Utilities.makeLegalJavaName(s1))
//					.replace("$$$OUTPUT_DIR$$$", s2)
//					.replace("$$$VIEW$$$", language.getTextHtml(StringConstants.VIEW))
//					.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES))
//					.replace("$$$ARG2$$$", NumberFormatter.formatForCountNotConditional(s3))
//					.replace("$$$ANCHOR$$$", anchor);
		}

		return table;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}
	

}
