package gov.anl.gss.sdm.server.model.documentor;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.FlatFileWriter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorCascadingLookup;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorEmbeddedData;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorIfThenElse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorMinMax;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNonZeroEnd;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNonmonotonic;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNotInView;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorStepPulse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUndocumented;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUnitErrorsWarnings;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class Warnings extends TableData implements TableRow, TableDataLoad {
static Logger log = Logger.getLogger(Warnings.class.getSimpleName());

	private List<Equation> undocumented;
	private int effectiveUndocumentedCount = 0;
	private List<Equation> embeddedData;
	private int effectiveEmbeddedDataCount = 0;
	private List<Equation> notInView;
	private int effectiveNotInViewCount = 0;
	private List<Equation> incompleteSubscripts;
	private int effectiveIncompleteSubscriptsCount = 0;
	private List<Equation> nonmonotonic;
	private int effectiveNonmonotonicCount = 0;
	private List<Equation> cascading;
	private int effectiveCascadingCount = 0;
	private List<Equation> nonZeroEnd;
	private int effectiveNonZeroEndCount = 0;
	private List<Equation> ifThenElse;
	private int effectiveIfThenElseCount = 0;
	private List<Equation> minMax;
	private int effectiveMinMaxCount = 0;
	private List<Equation> stepPulse;
	private int effectiveStepPulseCount = 0;
	private List<Equation> unitErrorsWarnings;
	private int effectiveUnitErrorsWarningsCount = 0;
	
	private boolean subs;
	
	public Warnings(Documentor documentor) {
		super(documentor);
		load();
		subs = documentor.isSubs();
	}
	
	public void document() {
		log.debug("### Warnings ###\n");
		log.debug("Undocumentd: "+getUndocumentedCount());
		log.debug("Embedded Data: "+getEmbeddedDataCount());
		log.debug("Not In View: "+getNotInViewCount());
		log.debug("Incomplete Subscripts: "+getIncompleteSubscriptsCount());
		log.debug("Non-Monotonic: "+getNonmonotonicCount());
		log.debug("Cascading: "+getCascadingCount());
		log.debug("Non-Zero End: "+getNonZeroEndCount());
		log.debug("IF THEN ELSE: "+getIfThenElseCount());
		log.debug("MIN MAX: "+getMinMaxCount());
		log.debug("STEP PULSE: "+getStepPulseCount());
		log.debug("Unit Errors Warnings: "+getUnitErrorsWarningsCount());
	}
	
	public void load() {
		loadUndocumented();
		loadEmbeddedData();
		loadNotInView();
		loadIncompleteSubscripts();
		loadNonmonotonic();
		loadCascading();
		loadNonZeroEnd();
		loadIfThenElse();
		loadMinMax();
		loadStepPulse();
		loadUnitErrorsWarnings();
	}
	
	private void loadSelected(VariableSelector selector, List<Equation> selected) {
		
		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
			for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
				if (selector.isSelectSingleEquation(eqn))
					selected.add(eqn);
			}
		}
	}
	
	private List<List<List<String>>> createFlatFileTable() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<List<String>>> table = createTable();
		
		addTitle(table,language.getTextHtml(StringConstants.WARNINGS));
		List<String> headers = new ArrayList<String>();
		headers.add(language.getTextHtml(StringConstants.VARIABLE));
		headers.add(language.getTextHtml(StringConstants.NUMBER_OF_VARIABLES));
		headers.add(language.getTextHtml(StringConstants.PCT_OF_TOTAL));
		if (subs) {
			headers.add(language.getTextHtml(StringConstants.EFFECTIVE_NUMBER_OF_VARIABLES));
			headers.add(language.getTextHtml(StringConstants.EFFECTIVE_PCT_OF_TOTAL));
		}
		addColumnHeaders(table, headers);
		
		return table;
	}
	
	private void loadUnitErrorsWarnings() {
		unitErrorsWarnings = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorUnitErrorsWarnings(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS);
		loadSelected(selector, unitErrorsWarnings);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				effectiveUnitErrorsWarningsCount += eqn.getTotalEquationCount();
		}

	}
	
	private void loadUndocumented() {
		undocumented = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorUndocumented(StringConstants.UNDOCUMENTED_VARIABLES, StringConstants.UNDOCUMENTED_VARIABLES);
		
		loadSelected(selector, undocumented);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				if (eqn.isSyntacticallyCorrect())
					effectiveUndocumentedCount += eqn.getTotalEquationCount();
		}
		
//		for (String var : InformationManagers.getInstance().getEquationManager().getVariables()) {
//			MutableInteger numEqn = new MutableInteger();
//			MutableInteger numSelect = new MutableInteger();
//			InformationManagers.getInstance().getEquationManager().applySelectorToAll(var, selector, numEqn, numSelect);
//			if (numEqn.value() == numSelect.value()) {
//				undocumented.addAll(InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var));
//				for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var))
//					if (eqn.isSyntacticallyCorrect())
//					effectiveUndocumentedCount += eqn.getTotalEquationCount();
//			}
//		}
		
	}
	
	private void loadEmbeddedData() {
		embeddedData = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorEmbeddedData(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, StringConstants.EQUATIONS_WITH_EMBEDDED_DATA);
		loadSelected(selector, embeddedData);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				if (eqn.isSyntacticallyCorrect())
				effectiveEmbeddedDataCount += eqn.getTotalEquationCount();
		}

	}
	
	private void loadNotInView() {
		notInView = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorNotInView(StringConstants.VARIABLES_NOT_IN_ANY_VIEW, StringConstants.VARIABLES_NOT_IN_ANY_VIEW);
		loadSelected(selector, notInView);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn) && eqn.isSyntacticallyCorrect())
				effectiveNotInViewCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadIncompleteSubscripts() {
		incompleteSubscripts = new ArrayList<Equation>();
	}
	
	private void loadNonmonotonic() {
		nonmonotonic = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorNonmonotonic(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS, StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS);
		loadSelected(selector, nonmonotonic);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn) && eqn.isSyntacticallyCorrect())
				effectiveNonmonotonicCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadCascading() {
		cascading = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorCascadingLookup(StringConstants.CASCADING_LOOKUP_FUNCTIONS, StringConstants.CASCADING_LOOKUP_FUNCTIONS);
		loadSelected(selector, cascading);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn) && eqn.isSyntacticallyCorrect())
				effectiveCascadingCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadNonZeroEnd() {
		nonZeroEnd = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorNonZeroEnd(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS);
		loadSelected(selector, nonZeroEnd);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn) && eqn.isSyntacticallyCorrect())
				effectiveNonZeroEndCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadIfThenElse() {
		ifThenElse = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorIfThenElse(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS);
		loadSelected(selector, ifThenElse);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn) && eqn.isSyntacticallyCorrect())
				effectiveIfThenElseCount += eqn.getTotalEquationCount();
		}
	}
	
	private void loadMinMax() {
		minMax = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorMinMax(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS);
		loadSelected(selector, minMax);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				effectiveMinMaxCount += eqn.getTotalEquationCount();
		}
		
		
	}
	
	private void loadStepPulse() {
		stepPulse = new ArrayList<Equation>();
		VariableSelector selector = new VariableSelectorStepPulse(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS);
		loadSelected(selector, stepPulse);
		
		Map<String, Equation> equations = documentor.getEquations();
		for (Equation eqn : equations.values()) {
			if (selector.isSelectMultipleEquation(eqn))
				effectiveStepPulseCount += eqn.getTotalEquationCount();
		}
	}
	
	public int getUndocumentedCount() {
		return undocumented.size();
	}
	public int getEmbeddedDataCount() {
		return embeddedData.size();
	}
	public int getNotInViewCount() {
		return notInView.size();
	}
	public int getIncompleteSubscriptsCount() {
		return incompleteSubscripts.size();
	}
	public int getNonmonotonicCount() {
		return nonmonotonic.size();
	}
	public int getCascadingCount() {
		return cascading.size();
	}
	public int getNonZeroEndCount() {
		return nonZeroEnd.size();
	}
	public int getIfThenElseCount() {
		return ifThenElse.size();
	}
	public int getMinMaxCount() {
		return minMax.size();
	}
	public int getStepPulseCount() {
		return stepPulse.size();
	}
	public int getUnitErrorsWarningsCount() {
		return unitErrorsWarnings.size();
	}


	public String getUndocumentedCountWithPct() {
		return getFormattedWithPct(undocumented.size(), effectiveUndocumentedCount);
	}
	public String getEmbeddedDataCountWithPct() {
		return getFormattedWithPct(embeddedData.size(), effectiveEmbeddedDataCount);
	}
	public String getNotInViewCountWithPct() {
		return getFormattedWithPct(notInView.size(), effectiveNotInViewCount);
	}
	public String getIncompleteSubscriptsCountWithPct() {
		return getFormattedWithPct(incompleteSubscripts.size(), effectiveIncompleteSubscriptsCount);
	}
	public String getNonmonotonicCountWithPct() {
		return getFormattedWithPct(nonmonotonic.size(), effectiveNonmonotonicCount);
	}
	public String getCascadingCountWithPct() {
		return getFormattedWithPct(cascading.size(), effectiveCascadingCount);
	}
	public String getNonZeroEndCountWithPct() {
		return getFormattedWithPct(nonZeroEnd.size(), effectiveNonZeroEndCount);
	}
	public String getIfThenElseCountWithPct() {
		return getFormattedWithPct(ifThenElse.size(), effectiveIfThenElseCount);
	}
	public String getMinMaxCountWithPct() {
		return getFormattedWithPct(minMax.size(), effectiveMinMaxCount);
	}
	public String getStepPulseCountWithPct() {
		return getFormattedWithPct(stepPulse.size(), effectiveStepPulseCount);
	}
	public String getUnitErrorsWarningsCountWithPct() {
		return getFormattedWithPct(unitErrorsWarnings.size(), effectiveUnitErrorsWarningsCount);
	}
	
	public String getFormattedWithPct(int v1, int v2) {
		return NumberFormatter.getAsCountAndPercentTotalNoHTML(v1, documentor.getModelInformation().getVariablesCount()) + (subs ? "|" +
		NumberFormatter.getAsCountAndPercentTotalNoHTML(v2, documentor.getModelInformation().getEffectiveVariableCount()) : "");
	}
	
//	@Override
	public List<List<List<String>>> getTableData() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		List<List<List<String>>> table = createFlatFileTable();
		
		
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES,  
				language.getTextHtml(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES), getUndocumentedCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.EQUATIONS_WITH_EMBEDDED_DATA,
				language.getTextHtml(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA),getEmbeddedDataCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.VARIABLES_NOT_IN_ANY_VIEW,
				language.getTextHtml(StringConstants.VARIABLES_NOT_IN_ANY_VIEW),getNotInViewCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
				language.getTextHtml(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS),getNonmonotonicCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.CASCADING_LOOKUP_FUNCTIONS,
				language.getTextHtml(StringConstants.CASCADING_LOOKUP_FUNCTIONS),getCascadingCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, 
				language.getTextHtml(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS),getNonZeroEndCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS),getIfThenElseCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS,
				language.getTextHtml(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS),getMinMaxCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,
				language.getTextHtml(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS),getStepPulseCountWithPct());
		rowAdd(HTMLConstants.WITHOUT_HTML, table, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS),getUnitErrorsWarningsCountWithPct());

		return table;
	}

	@Override
	public List<List<List<String>>> getRows() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<List<List<String>>> rows = new ArrayList<List<List<String>>>();
		
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES, StringConstants.UNDOCUMENTED_VARIABLES, 
				language.getTextHtml(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES), getUndocumentedCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.EQUATIONS_WITH_EMBEDDED_DATA,StringConstants.EQUATIONS_WITH_EMBEDDED_DATA,
				language.getTextHtml(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA),getEmbeddedDataCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.VARIABLES_NOT_IN_ANY_VIEW,StringConstants.VARIABLES_NOT_IN_ANY_VIEW, 
				language.getTextHtml(StringConstants.VARIABLES_NOT_IN_ANY_VIEW),getNotInViewCountWithPct());
//		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES,"",
//		language.getTextHtml("IncompletelyDefinedSubscriptedVariables"),getIncompleteSubscriptsCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
				language.getTextHtml(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS),getNonmonotonicCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.CASCADING_LOOKUP_FUNCTIONS,StringConstants.CASCADING_LOOKUP_FUNCTIONS, 
				language.getTextHtml(StringConstants.CASCADING_LOOKUP_FUNCTIONS),getCascadingCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, 
				language.getTextHtml(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS),getNonZeroEndCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS,StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS),getIfThenElseCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS,StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS),getMinMaxCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS),getStepPulseCountWithPct());
		rowAdd(HTMLConstants.WITH_HTML, rows, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, 
				language.getTextHtml(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS),getUnitErrorsWarningsCountWithPct());
		
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Warnings"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		
		return rows;
	}

	@Override
	public List<List<List<String>>> getDataTableRows() {
		// TODO Auto-generated method stub
		return null;
	}

	
}
