package gov.anl.gss.sdm.server.model.documentor;

/**
 * From http://www.journaldev.com/957/java-zip-example-to-zip-single-file-and-a-directory-recursively
 * Modified by Mark Bragen (Argonne National Laboratory)
 */

	import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;
	 
	public class Zipper {
		
		static Logger log = Logger.getLogger(Zipper.class.getSimpleName());
	     
	    private List<String> filesListInDir = new ArrayList<String>();
	    private List<String> directories;
	    private List<String> files;
	    private String zipFile;
	    
	    FileOutputStream fos;
	    ZipOutputStream zos;
	    
	    public Zipper(String zipFile, List<String> files, List<String> directories) {
	    	
	    	this.zipFile = zipFile;
	    	this.files = files;
	    	this.directories = directories;
	    }
	    
	    public void zip() {
	    	openZipFile();
	    	for (String file : files)
	    		addSingleFile(new File(file));
	    	for (String dir : directories)
	    		addDirectory(new File(dir));
	    	closeZipFile();
	    	
	    }
	 
	    private void openZipFile() {
	    	try {
				fos = new FileOutputStream(zipFile);
				zos = new ZipOutputStream(fos);
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }
	    
	    private void closeZipFile() {
	    	try {
	    		zos.close();
				fos.close();
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	    }
	 
	    /**
	     * This method zips the directory
	     * @param dir
	     * @param zipDirName
	     */
	    private void zipDirectory(File dir, String zipDirName) {
	        try {
	        	filesListInDir.clear();
	            populateFilesList(dir);
	            //now zip files one by one
	            //create ZipOutputStream to write to the zip file
	            FileOutputStream fos = new FileOutputStream(zipDirName);
	            ZipOutputStream zos = new ZipOutputStream(fos);
	            for(String filePath : filesListInDir){
	                log.debug("Zipping "+filePath);
//	               System.out.println("Zipping "+filePath+" -> "+filePath.substring(dir.getAbsolutePath().length()+1, filePath.length()));
	                //for ZipEntry we need to keep only relative file path, so we used substring on absolute path
	                ZipEntry ze = new ZipEntry(filePath.substring(dir.getAbsolutePath().length()+1, filePath.length()));
	                zos.putNextEntry(ze);
	                //read the file and write to ZipOutputStream
	                FileInputStream fis = new FileInputStream(filePath);
	                byte[] buffer = new byte[1024];
	                int len;
	                while ((len = fis.read(buffer)) > 0) {
	                    zos.write(buffer, 0, len);
	                }
	                zos.closeEntry();
	                fis.close();
	            }
	            zos.close();
	            fos.close();
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	    }
	    
	    private void addDirectory(File dir) {
	        try {
	        	filesListInDir.clear();
	            populateFilesList(dir);
	           
	            for(String filePath : filesListInDir){
	            	log.debug("Zipping "+filePath);
//	            	  System.out.println("Zipping "+filePath+" -> "+filePath.substring(dir.getAbsolutePath().length()+1, filePath.length()));
	                //for ZipEntry we need to keep only relative file path, so we used substring on absolute path
	                String zipEntryName = dir.getName() + "/" + filePath.substring(dir.getAbsolutePath().length()+1, filePath.length());
	                ZipEntry ze = new ZipEntry(zipEntryName);
	                zos.putNextEntry(ze);
	                //read the file and write to ZipOutputStream
	                FileInputStream fis = new FileInputStream(filePath);
	                byte[] buffer = new byte[1024];
	                int len;
	                while ((len = fis.read(buffer)) > 0) {
	                    zos.write(buffer, 0, len);
	                }
	                zos.closeEntry();
	                fis.close();
	            }
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	    }
	     
	    /**
	     * This method populates all the files in a directory to a List
	     * @param dir
	     * @throws IOException
	     */
	    private void populateFilesList(File dir) throws IOException {
	    	
	        File[] files = dir.listFiles();
	        for(File file : files){
	            if(file.isFile()) 
	            	filesListInDir.add(file.getAbsolutePath());
	            else 
	            	populateFilesList(file);
	        }
	    }
	 
	    /**
	     * This method compresses the single file to zip format
	     * @param file
	     * @param zipFileName
	     */
	    private static void zipSingleFile(File file, String zipFileName) {
	        try {
	            //create ZipOutputStream to write to the zip file
	            FileOutputStream fos = new FileOutputStream(zipFileName);
	            ZipOutputStream zos = new ZipOutputStream(fos);
	            //add a new Zip Entry to the ZipOutputStream
	            ZipEntry ze = new ZipEntry(file.getName());
	            zos.putNextEntry(ze);
	            //read the file and write to ZipOutputStream
	            FileInputStream fis = new FileInputStream(file);
	            byte[] buffer = new byte[1024];
	            int len;
	            while ((len = fis.read(buffer)) > 0) {
	                zos.write(buffer, 0, len);
	            }
	             
	            //Close the zip entry to write to zip file
	            zos.closeEntry();
	            //Close resources
	            zos.close();
	            fis.close();
	            fos.close();
	            System.out.println(file.getCanonicalPath()+" is zipped to "+zipFileName);
	             
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	 
	    }
	    
	    private void addSingleFile(File file) {
	        try {
	            //add a new Zip Entry to the ZipOutputStream
	            ZipEntry ze = new ZipEntry(file.getName());
	            zos.putNextEntry(ze);
	            //read the file and write to ZipOutputStream
	            FileInputStream fis = new FileInputStream(file);
	            byte[] buffer = new byte[1024];
	            int len;
	            while ((len = fis.read(buffer)) > 0) {
	                zos.write(buffer, 0, len);
	            }
	             
	            //Close the zip entry to write to zip file
	            zos.closeEntry();
	            //Close resources
	            fis.close();
	            log.debug(file.getCanonicalPath()+" is zipped to "+zipFile);
	             
	        } catch (IOException e) {
	            e.printStackTrace();
	        }
	 
	    }
	 
	}

