package gov.anl.gss.sdm.server.model.documentor.docx.writer;


import gov.anl.gss.sdm.server.model.documentor.docx.Graph;
import gov.anl.gss.sdm.server.model.documentor.docx.Table;
import gov.anl.gss.sdm.server.model.documentor.docx.reader.DocxReader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.docx4j.dml.wordprocessingDrawing.Inline;
import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.openpackaging.parts.WordprocessingML.BinaryPartAbstractImage;
import org.docx4j.openpackaging.parts.WordprocessingML.MainDocumentPart;
import org.docx4j.wml.BooleanDefaultTrue;
import org.docx4j.wml.Br;
import org.docx4j.wml.CTBorder;
import org.docx4j.wml.Drawing;
import org.docx4j.wml.HpsMeasure;
import org.docx4j.wml.Jc;
import org.docx4j.wml.JcEnumeration;
import org.docx4j.wml.ObjectFactory;
import org.docx4j.wml.P;
import org.docx4j.wml.R;
import org.docx4j.wml.RPr;
import org.docx4j.wml.STBorder;
import org.docx4j.wml.STBrType;
import org.docx4j.wml.Tbl;
import org.docx4j.wml.TblBorders;
import org.docx4j.wml.TblGrid;
import org.docx4j.wml.TblGridCol;
import org.docx4j.wml.TblPr;
import org.docx4j.wml.TblWidth;
import org.docx4j.wml.Tc;
import org.docx4j.wml.TcPr;
import org.docx4j.wml.Text;
import org.docx4j.wml.Tr;

public class ReportWriter {
	
	/**
	 * Thanks to: http://blog.iprofs.nl/2012/09/06/creating-word-documents-with-docx4j/
	 */
	
	/*
	 * Tool writes/reads a docx file with variable names and comments
	 */
	
	private String inStringFile;
	private String inDocxFile;
	private String outDocxFile;
	private List<String> excelFiles;
	
	private static final String FONT_SIZE = "24";
	
	private int figureNum = 1;
	private int tableNum = 1;
	
	ObjectFactory factory;
	WordprocessingMLPackage wordMLPackage = null;
	
	
	private List<List<List<String>>> tables;
	
	private List<Reportable> reportables;
	
	public ReportWriter() {
		factory = Context.getWmlObjectFactory();
	}
	
	public ReportWriter(List<Reportable> reportables, String outDocxFile, boolean r) {
		this();
		this.reportables = reportables;
		this.outDocxFile = outDocxFile;
	}
	
	public ReportWriter(List<List<List<String>>> tables, String outDocxFile) {
		this();
		this.tables = tables;
		this.outDocxFile = outDocxFile;
	}
    
    public List<String> docxToString() {
    	List<String> text = loadDocx();
    	text = squeezeBlanks(text);
    	return text;
   	
    }
    
    public List<String> squeezeBlanks(List<String> text) {
    	List<String> textSqueezed = new ArrayList<String>();
    	for (String s : text) {
    		String y = new String(s);
    		while (y.contains("\t")) {
    			y = y.replace("\t", " ");
    		}
    		while (y.contains("  ")) {
    			y = y.replace("  ", " ");
    		}
//    		y = y.replace("\t", " ");
    		textSqueezed.add(y);
    	}
    	return textSqueezed;
    }
    
    public void generateDocx() {
    	
    	File aFile = new File(outDocxFile);
        if (aFile.getParentFile() != null)
        	aFile.getParentFile().mkdirs();
        
        
    	createPackage();
//    	addHeader("");
    	addReportables(reportables);
    	writePackage();
    }
    
    public void stringToDocx() {
    	
    	File aFile = new File(outDocxFile);
        if (aFile.getParentFile() != null)
        	aFile.getParentFile().mkdirs();
        
        
    	createPackage();
    	addHeader("");
    	addTables(tables);
    	writePackage();
    }
    
    private void addHeader(String hdr) {
    	
    	List<String> header = new ArrayList<String>();
    	header.add(hdr);
    	addHeader(header);
    }
    
//    private void addHeader() {
//    	List<String> header = new ArrayList<String>();
//    	header.add("Systems Field Evaluation Discussion Notes");
//    	addHeader(header);
//    }
    
    
    private void addHeader(List<String> text) {
    	for (String t : text) {
    		wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Heading3",t);
    	}
    }
   
    
    
    private void createPackage() {
 
    		try {
//				wordMLPackage = WordprocessingMLPackage.load(new FileInputStream(new File("OutputDocument/ClusterTemplate.docx")));
				
				wordMLPackage = WordprocessingMLPackage.load(new FileInputStream(new File("data/language/translationTemplate.docx")));
			} catch (FileNotFoundException | Docx4JException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    }
    
    private void writePackage() {
    	try {
    		wordMLPackage.save(new java.io.File(outDocxFile));
    	} catch (Docx4JException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
    }
    
    public void addReportables(List<Reportable> reportables) {
    	for (Reportable r : reportables) {
    		if (r.isTable()) {
    			addTable(r.getTable());
    		}
    		if (r.isGraph()) {
    			addGraph(r.getGraph());
    		}
    	}
    }
    
    public void addGraph(Graph g) {
    	
    	try {
			File file = new File(g.getFilename());
			byte[] bytes = convertImageToByteArray(file);
//    	addImageToPackage(wordMLPackage, bytes);

			String caption = "Figure "+figureNum+". "+g.getTitle();
			figureNum++;
			
			wordMLPackage.getMainDocumentPart().addParagraphOfText("");
			addImageToPackage(wordMLPackage, bytes);
			wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Heading3",
					caption);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    public void addTables(List<List<List<String>>> tables) {
    	String tableHeading = "";

    	int tablePtr = 0;
    	for (List<List<String>> table : tables) {
        	Tbl t = factory.createTbl();
        	
        	addBorders(t);
        	TblGrid tblGrid = factory.createTblGrid();
        	t.setTblGrid( tblGrid );
        	
        	int r = 0;
        	for (List<String> row : table) {
        		if (r++ == 0) {
        			tableHeading = row.get(1) + " - "+row.get(0);
        		} else {
        		Tr tableRow = factory.createTr();
        		 TblGridCol colg = factory.createTblGridCol();
                 colg.setW( BigInteger.valueOf( 10 ) ); // 10
                 tblGrid.getGridCol().add( colg );
        		for (String col : row) {
        			addStyledTableCell(tableRow,  col, false, FONT_SIZE, 400);
        		}
        		t.getContent().add(tableRow);
        	}
        	}
        	if (tablePtr++ > 0)
        		addPageBreak();
        	wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Title", tableHeading);
        	wordMLPackage.getMainDocumentPart().addObject(t);
    	}

    }
    
    
    public void addTable(Table table) {
    	String tableHeading = "";

    	int tablePtr = 0;
    	Tbl t = factory.createTbl();
    	addBorders(t);
    	
    	TblGrid tblGrid = factory.createTblGrid();
    	t.setTblGrid( tblGrid );
    	
    	Jc justification = factory.createJc();
    	justification.setVal(JcEnumeration.CENTER);
    	
    	TblPr pr = t.getTblPr();
    	
    	pr.setJc(justification);
    	
//    	CTShortHexNumber c = new CTShortHexNumber();
//    	c.setVal("04A0");
//    	pr.setTblLook(c);
    	int r = 0;
    	tableHeading = table.getTitle();
    	Tr tableRow = factory.createTr();
    	TblGridCol colg = factory.createTblGridCol();
    	colg.setW( BigInteger.valueOf(100 ) );
    	tblGrid.getGridCol().add( colg );
    	for (String col : table.getColumnHeaders()) {
    		addStyledTableCell(tableRow,  col, true, FONT_SIZE, 300);
    	}
    	t.getContent().add(tableRow);
    	for (List<String> row : table.getTableData()) {

    		tableRow = factory.createTr();
    		colg = factory.createTblGridCol();
    		colg.setW( BigInteger.valueOf( 10 ) );
    		tblGrid.getGridCol().add( colg );
    		for (String col : row) {
    			addStyledTableCell(tableRow,  col, false, FONT_SIZE, 300);
    		}
    		t.getContent().add(tableRow);
    	}
    	if (tablePtr++ > 0)
    		addPageBreak();
    	
//    	PPr paragraphProperties = factory.createPPr();
//    	Jc justification = factory.createJc();
//    	justification.setVal(JcEnumeration.CENTER);
//    	paragraphProperties.setJc(justification);
    	
//    	wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Heading1", tableHeading);
    	wordMLPackage.getMainDocumentPart().addObject(t);
    	String caption = "Table "+tableNum+". "+table.getTitle();
    	tableNum++;
		
//		wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Heading3",
//				caption);

    }
    public void addTable(List<List<String>> table) {
    	String tableHeading = "";

    	int tablePtr = 0;
        	Tbl t = factory.createTbl();
        	addBorders(t);
        	TblGrid tblGrid = factory.createTblGrid();
        	t.setTblGrid( tblGrid );
        	int r = 0;
        	for (List<String> row : table) {
        		if (r++ == 0) {
        			tableHeading = row.get(1) + " - "+row.get(0);
        		} else {
        		Tr tableRow = factory.createTr();
        		 TblGridCol colg = factory.createTblGridCol();
                 colg.setW( BigInteger.valueOf( 10 ) );
                 tblGrid.getGridCol().add( colg );
        		for (String col : row) {
        			addStyledTableCell(tableRow,  col, false, FONT_SIZE, 300);
        		}
        		t.getContent().add(tableRow);
        	}
        	}
        	if (tablePtr++ > 0)
        		addPageBreak();
        	wordMLPackage.getMainDocumentPart().addStyledParagraphOfText("Title", tableHeading);
        	wordMLPackage.getMainDocumentPart().addObject(t);

    }
    
    
    
    /**
     *  In this method we create a table cell,add the styling and add the cell to
     *  the table row.
     */
    private  void addStyledTableCell(Tr tableRow, String content,
                        boolean bold, String fontSize, int width) {
        Tc tableCell = factory.createTc();
        addStyling(tableCell, content, bold, fontSize);
        tableRow.getContent().add(tableCell);
        
       // Here to
//        tableCell.getContent().add(
//            wordMLPackage.getMainDocumentPart().createParagraphOfText(
//                content));
        // Here
        if (width > 0) {
            setCellWidth(tableCell, width);
        }
        
        
        
    }
 
    /**
     *  This is where we add the actual styling information. In order to do this
     *  we first create a paragraph. Then we create a text with the content of
     *  the cell as the value. Thirdly, we create a so-called run, which is a
     *  container for one or more pieces of text having the same set of
     *  properties, and add the text to it. We then add the run to the content
     *  of the paragraph.
     *  So far what we've done still doesn't add any styling. To accomplish that,
     *  we'll create run properties and add the styling to it. These run
     *  properties are then added to the run. Finally the paragraph is added
     *  to the content of the table cell.
     */
    private  void addStyling(Tc tableCell, String content,
                    boolean bold, String fontSize) {
    
        
        P paragraph = factory.createP();
 
        Text text = factory.createText();
        text.setValue(content);
 
        R run = factory.createR();
        run.getContent().add(text);
 
        paragraph.getContent().add(run);
 
        RPr runProperties = factory.createRPr();
        if (bold) {
            addBoldStyle(runProperties);
        }
 
        if (fontSize != null && !fontSize.isEmpty()) {
            setFontSize(runProperties, fontSize);
        }
 
        run.setRPr(runProperties);
 
        tableCell.getContent().add(paragraph);
    }
 
    /**
     *  In this method we're going to add the font size information to the run
     *  properties. First we'll create a half-point measurement. Then we'll
     *  set the fontSize as the value of this measurement. Finally we'll set
     *  the non-complex and complex script font sizes, sz and szCs respectively.
     */
    private  void setFontSize(RPr runProperties, String fontSize) {
        HpsMeasure size = new HpsMeasure();
        size.setVal(new BigInteger(fontSize));
        runProperties.setSz(size);
        runProperties.setSzCs(size);
    }
 
    /**
     *  In this method we'll add the bold property to the run properties.
     *  BooleanDefaultTrue is the Docx4j object for the b property.
     *  Technically we wouldn't have to set the value to true, as this is
     *  the default.
     */
    private  void addBoldStyle(RPr runProperties) {
        BooleanDefaultTrue b = new BooleanDefaultTrue();
        b.setVal(true);
        runProperties.setB(b);
    }
    
	private  void addBorders(Tbl table) {
        table.setTblPr(new TblPr());
        CTBorder border = new CTBorder();
        border.setColor("auto");
        border.setSz(new BigInteger("4"));
        border.setSpace(new BigInteger("0"));
        border.setVal(STBorder.SINGLE);
 
        TblBorders borders = new TblBorders();
        borders.setBottom(border);
        borders.setLeft(border);
        borders.setRight(border);
        borders.setTop(border);
        borders.setInsideH(border);
        borders.setInsideV(border);
        table.getTblPr().setTblBorders(borders);
        
        
    }
	
	 /**
     *  In this method we create a cell and add the given content to it.
     *  If the given width is greater than 0, we set the width on the cell.
     *  Finally, we add the cell to the row.
     */
    private  void addTableCellWithWidth(Tr row, String content, int width){
        Tc tableCell = factory.createTc();
        tableCell.getContent().add(
            wordMLPackage.getMainDocumentPart().createParagraphOfText(
                content));
 
        if (width > 0) {
            setCellWidth(tableCell, width);
        }
        row.getContent().add(tableCell);
    }
 
    /**
     *  In this method we create a table cell properties object and a table width
     *  object. We set the given width on the width object and then add it to
     *  the properties object. Finally we set the properties on the table cell.
     */
    private  void setCellWidth(Tc tableCell, int width) {
        TcPr tableCellProperties = new TcPr();
        TblWidth tableWidth = new TblWidth();
        tableWidth.setW(BigInteger.valueOf(width));
        tableCellProperties.setTcW(tableWidth);
        tableCell.setTcPr(tableCellProperties);
    }
    
    private  void addTableCell(Tr tableRow, String content) {
        Tc tableCell = factory.createTc();
        tableCell.getContent().add(
            wordMLPackage.getMainDocumentPart().createParagraphOfText(content));
        tableRow.getContent().add(tableCell);
        }
   
    
    private void print(List<String> text) {
    	for (String t : text)
    		System.out.println(t);
    	
    }
    
    private void addPageBreak() {
        MainDocumentPart documentPart = wordMLPackage.getMainDocumentPart();

        Br breakObj = new Br();
        breakObj.setType(STBrType.PAGE);

        P paragraph = factory.createP();
        paragraph.getContent().add(breakObj);
        documentPart.getJaxbElement().getBody().getContent().add(paragraph);
    }
    
    private List<String> loadDocx() {		
		DocxReader reader = new DocxReader(inDocxFile);
		return reader.getDocText();
	}


	public String getInStringFile() {
		return inStringFile;
	}


	public void setInStringFile(String inStringFile) {
		this.inStringFile = inStringFile;
	}


	public String getInDocxFile() {
		return inDocxFile;
	}


	public void setInDocxFile(String inDocxFile) {
		this.inDocxFile = inDocxFile;
	}


	public String getOutDocxFile() {
		return outDocxFile;
	}


	public void setOutDocxFile(String outDocxFile) {
		this.outDocxFile = outDocxFile;
	}
	
	   /**
     *  Docx4j contains a utility method to create an image part from an array of
     *  bytes and then adds it to the given package. In order to be able to add this
     *  image to a paragraph, we have to convert it into an inline object. For this
     *  there is also a method, which takes a filename hint, an alt-text, two ids
     *  and an indication on whether it should be embedded or linked to.
     *  One id is for the drawing object non-visual properties of the document, and
     *  the second id is for the non visual drawing properties of the picture itself.
     *  Finally we add this inline object to the paragraph and the paragraph to the
     *  main document of the package.
     *
     *  @param wordMLPackage The package we want to add the image to
     *  @param bytes         The bytes of the image
     *  @throws Exception    Sadly the createImageInline method throws an Exception
     *                       (and not a more specific exception type)
     */
    private static void addImageToPackage(WordprocessingMLPackage wordMLPackage,
                            byte[] bytes) throws Exception {
        BinaryPartAbstractImage imagePart =
            BinaryPartAbstractImage.createImagePart(wordMLPackage, bytes);
 
        int docPrId = 1;
        int cNvPrId = 2;
            Inline inline = imagePart.createImageInline("Filename hint",
                "Alternative text", docPrId, cNvPrId, false);
 
        P paragraph = addInlineImageToParagraph(inline);
 
        wordMLPackage.getMainDocumentPart().addObject(paragraph);
    }
    
    /**
     *  We create an object factory and use it to create a paragraph and a run.
     *  Then we add the run to the paragraph. Next we create a drawing and
     *  add it to the run. Finally we add the inline object to the drawing and
     *  return the paragraph.
     *
     * @param   inline The inline object containing the image.
     * @return  the paragraph containing the image
     */
    private static P addInlineImageToParagraph(Inline inline) {
        // Now add the in-line image to a paragraph
        ObjectFactory factory = new ObjectFactory();
        P paragraph = factory.createP();
        R run = factory.createR();
        paragraph.getContent().add(run);
        Drawing drawing = factory.createDrawing();
        run.getContent().add(drawing);
        drawing.getAnchorOrInline().add(inline);
        return paragraph;
    }
    
    /**
     * Convert the image from the file into an array of bytes.
     *
     * @param file  the image file to be converted
     * @return      the byte array containing the bytes from the image
     * @throws FileNotFoundException
     * @throws IOException
     */
    private static byte[] convertImageToByteArray(File file)
            throws FileNotFoundException, IOException {
        InputStream is = new FileInputStream(file );
        long length = file.length();
        // You cannot create an array using a long, it needs to be an int.
        if (length > Integer.MAX_VALUE) {
            System.out.println("File too large!!");
        }
        byte[] bytes = new byte[(int)length];
        int offset = 0;
        int numRead = 0;
        while (offset < bytes.length               && (numRead=is.read(bytes, offset, bytes.length-offset)) >= 0) {
            offset += numRead;
        }
        // Ensure all the bytes have been read
        if (offset < bytes.length) {
            System.out.println("Could not completely read file "
                        +file.getName());
        }
        is.close();
        return bytes;
    }
}


