package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;

public class AcknowledgementsFrame extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String ACKS = "data/acknowledgements.txt";
	
	private static final String DEVELOPERS = "Developers";
	private static final String COLLABORATORS = "Collaborators";
	private static final String INTERNATIONALIZATION = "Internationalization";
	
	private JFrame masterFrame;
	private JEditorPane editorPane;
	
	public AcknowledgementsFrame() {
		create();
//		showPanel();
	}
	private void create() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		//Create and set up the window.
		final JFrame frame = new JFrame(language.getText(StringConstants.SDM_DOC_TOOL)+"/Java "+
				language.getText(StringConstants.VERSION)+StandaloneGUI.version+" "+language.getText(StringConstants.ACKNOWLEDGEMENTS));
		masterFrame = frame;
		
		//		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setPreferredSize(new SdmDimension(600, 450));
		frame.setLocation (0, 200) ;

		
		editorPane = new JEditorPane();
		editorPane.setEditable(false);
		editorPane.setFont(language.getFont(Font.PLAIN, 14));
		editorPane.setBackground(new Color(240,240,240));
		
		frame.getContentPane().add(new JScrollPane(editorPane), "Center");
		
		loadFromFile();
		
		frame.pack();
	}
	
	private void loadFromFile() {
		
		Document doc = editorPane.getDocument();
		
		try {
			doc.remove(doc.getStartPosition().getOffset(), doc.getLength());
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String line : Utilities.getAsList(ACKS)) {
			if (line.startsWith(DEVELOPERS)) {
				addMsg(language.getText(DEVELOPERS)+":\n");
			} else if (line.startsWith(COLLABORATORS)) {
				addMsg(language.getText(COLLABORATORS)+":\n");
			} else if (line.startsWith(INTERNATIONALIZATION)) {
				addMsg(language.getText(INTERNATIONALIZATION)+":\n");
			} else {
				addMsg(line+"\n");
			}
		}
	}
	
	public void addMsg(String text) {
		try {
		      Document doc = editorPane.getDocument();
		      doc.insertString(doc.getLength(), text, null);
		      editorPane.revalidate();
		      editorPane.repaint();
		      editorPane.update(editorPane.getGraphics());
		   } catch(BadLocationException exc) {
		      exc.printStackTrace();
		   }
		

	}
	

	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		masterFrame.setTitle(language.getText(StringConstants.SDM_DOC_TOOL)+"/Java "+
				language.getText(StringConstants.VERSION)+StandaloneGUI.version+" "+language.getText(StringConstants.ACKNOWLEDGEMENTS));
		loadFromFile();
	}
	
	public void showPanel() {
		masterFrame.setVisible(true);
	}
	
	public void hidePanel() {
		masterFrame.setVisible(false);
	}


}
