package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.AdjacencyToMap;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class AdjacencyToMapPanel extends JPanel {
	
	static Logger log = Logger.getLogger(AdjacencyToMapPanel.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final int TEXT_FIELD_SIZE = 60;
	private static final int NUMERIC_TEXT_FIELD_SIZE = 15;
	private static String FC_DIR_FILE = "data/ChooserDir.txt";

	private static final String GENERATE_DOCX = "GenerateDocx";
	private static final String GENERATE_VENSIM = "GenerateVensim";

	private static final String CHECK_DEFAULT = "Check";
	private static final String DO_NOT_CHECK_DEFAULT = "DoNotCheck";

	private JLabel adjacencyMatrixLabel = new JLabelLanguage(
			StringConstants.ADJACENCY_MATRIX_FILE);
	private JLabel mapFileLabel = new JLabelLanguage(
			StringConstants.VENSIM_MAP_FILE);
	
	private JLabel visualizeLabel = new JLabelLanguage(StringConstants.VISUALIZE_MAP_WITH);

	private JTextField adjacencyMatrixText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField mapFileText = new JTextField(TEXT_FIELD_SIZE);

	private JButtonLanguage adjacencyMatrixBrowseButton = new JButtonLanguage(
			StringConstants.BROWSE);
	private JButtonLanguage mapBrowseButton = new JButtonLanguage(
			StringConstants.BROWSE);
	private JButtonLanguage groupBrowseButton = new JButtonLanguage(
			StringConstants.BROWSE);

	private JButton mapButton = new JButtonLanguage(StringConstants.GENERATE_MAP);

	private JDialog parent;
	private String title;
	
	private JRadioButtonLanguage circleRB = new JRadioButtonLanguage(StringConstants.CIRCLE);
	private JRadioButtonLanguage arcsRB = new JRadioButtonLanguage(StringConstants.ARCS);
	private JRadioButtonLanguage rectangleRB = new JRadioButtonLanguage(StringConstants.RECTANGLE);
	
	private JPanel visualizationPanel = new JPanel();
	
	private JPanel visualizationParametersPanel = new JPanel();
	private JLabel vpCircleLabel = new JLabelLanguage(StringConstants.CIRCLE_PARAMETERS);
	private JLabel vpArcsLabel = new JLabelLanguage(StringConstants.ARCS_PARAMETERS);
	private JLabel vpRectangleLabel = new JLabelLanguage(StringConstants.RECTANGLE_PARAMETERS);
	
	private JTextField vpCircleRadiusTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JTextField vpCircleThreshholdTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel vpCircleThreshholdLabel = new JLabelLanguage(StringConstants.THRESHHOLD);
	private JLabel vpCircleRadiusLabel = new JLabelLanguage(StringConstants.RADIUS);
	
	private JLabel vpCircleGroupGapLabel = new JLabelLanguage(StringConstants.INTER_GROUP_GAP);
	private JTextField vpCircleGroupGapTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	
	private JLabel vpCircleShapeSizeLabel = new JLabelLanguage(StringConstants.SHAPE_SIZE);
	private JTextField vpCircleShapeSizeTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel vpArcsShapeSizeLabel = new JLabelLanguage(StringConstants.SHAPE_SIZE);
	private JTextField vpArcsShapeSizeTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	
	private JLabel vpCircleArrowWidthLabel = new JLabelLanguage(StringConstants.ARROW_WIDTH);
	private JTextField vpCircleArrowWidthTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel vpArcsArrowWidthLabel = new JLabelLanguage(StringConstants.ARROW_WIDTH);
	private JTextField vpArcsArrowWidthTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	
	private JTextField vpArcsRadiusTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JTextField vpArcsArcDegreesTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel vpArcsOffsetLabel = new JLabelLanguage(StringConstants.ARC_DEGREES);
	private JLabel vpArcsRadiusLabel = new JLabelLanguage(StringConstants.RADIUS);
	
	private JTextField vpRectangleWidthTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel vpRectangleWidthLabel = new JLabelLanguage(StringConstants.WIDTH);
	
	
	// new for groups, colors, loopLength based map
	
	private JPanel optionsPanel = new JPanel();
	private JPanel groupColorPanel = new JPanel();
	
	private JLabel optionsLabel = new JLabelLanguage(StringConstants.OPTIONAL_GENERATED_MAP_OPTIONS);
	
	private JTextField groupFileTF = new JTextField(TEXT_FIELD_SIZE);
	private JLabel groupFileLabel = new JLabelLanguage(StringConstants.GROUP_FILE);
	
	private JCheckBoxLanguage groupCB = new JCheckBoxLanguage(StringConstants.BY_GROUP);
	private JCheckBoxLanguage groupInCB = new JCheckBoxLanguage(StringConstants.BY_INPUT);
	private JCheckBoxLanguage groupOutCB = new JCheckBoxLanguage(StringConstants.BY_OUTPUT);
	
	
	private JTextField maxLoopLengthTF = new JTextField(NUMERIC_TEXT_FIELD_SIZE);
	private JLabel maxLoopLengthLabel = new JLabelLanguage(StringConstants.MAXIMUM_LOOP_LENGTH);
	private JCheckBoxLanguage loopLengthCB = new JCheckBoxLanguage(StringConstants.BY_LOOP_LENGTH);
	private JCheckBoxLanguage specificLengthCB = new JCheckBoxLanguage(StringConstants.SPECIFIC_LOOP_LENGTH);
	private JCheckBoxLanguage upToLengthCB = new JCheckBoxLanguage(StringConstants.UP_TO_LOOP_LENGTH);
	
	private List<String> groups = new ArrayList<String>();
	List<String> colors;
	
	private JButton[] groupShapeColorsB;
	private JButton[] groupTextColorsB;
	
//	private JButtonLanguage groupShapeColorButtonSpaceHolder = new 
//			JButtonLanguage(StringConstants.GROUP_SHAPE_COLOR_CHOOSER);
//	private JButtonLanguage groupTextColorButtonSpaceHolder = 
//			new JButtonLanguage( StringConstants.GROUP_TEXT_COLOR_CHOOSER);
	
	private JLabelLanguage groupShapeColorButtonSpaceHolder = new 
			JLabelLanguage(StringConstants.GROUP_SHAPE_COLOR_CHOOSER);
	private JLabelLanguage groupTextColorButtonSpaceHolder = 
			new JLabelLanguage( StringConstants.GROUP_TEXT_COLOR_CHOOSER);
	
	JPanel shapeColorPickers = new JPanel();
	JPanel textColorPickers = new JPanel();
	
	
	
//	circle - radius, angle threshhold
//	arcs - radius, angle offset
//	rectangle - width
	

	public AdjacencyToMapPanel(JDialog parent, String title) {

		this.parent = parent;
		this.title = title;

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		adjacencyMatrixLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mapFileLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		visualizeLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		optionsLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		adjacencyMatrixText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mapFileText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));


		adjacencyMatrixBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mapBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		mapButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));

		createLayout();
		createListeners();
		setDependencies();

	}

	private void createLayout() {
		setLayout(new GridBagLayout());
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		//		originalLabel.setFont(language.getFont(Font.BOLD, 12));
		//		mergedLabel.setFont(language.getFont(Font.BOLD, 12));
		//		extractedLabel.setFont(language.getFont(Font.BOLD, 12));

		circleRB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		arcsRB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		rectangleRB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		
		
		vpCircleLabel.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
		vpCircleRadiusLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpCircleRadiusTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpCircleThreshholdLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpCircleGroupGapLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpCircleThreshholdTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpCircleGroupGapTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpCircleShapeSizeTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsShapeSizeTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpCircleArrowWidthTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsArrowWidthTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpCircleShapeSizeLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsShapeSizeLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpCircleArrowWidthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsArrowWidthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpArcsLabel.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
		vpArcsRadiusLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsRadiusTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsOffsetLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpArcsArcDegreesTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		vpRectangleLabel.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
		vpRectangleWidthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		vpRectangleWidthTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		groupFileLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		groupFileTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		groupCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		groupInCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		groupOutCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		maxLoopLengthTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		maxLoopLengthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		loopLengthCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		upToLengthCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		specificLengthCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		int wid = 300;
		int hid = 25;
		groupShapeColorButtonSpaceHolder.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		groupTextColorButtonSpaceHolder.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		groupTextColorButtonSpaceHolder.setMaximumSize(new Dimension(wid,hid));
		groupShapeColorButtonSpaceHolder.setMaximumSize(new Dimension(wid,hid));
		
		groupTextColorButtonSpaceHolder.setMinimumSize(new Dimension(wid,hid));
		groupShapeColorButtonSpaceHolder.setMinimumSize(new Dimension(wid,hid));
		
		groupTextColorButtonSpaceHolder.setSize(new Dimension(wid,hid));
		groupShapeColorButtonSpaceHolder.setSize(new Dimension(wid,hid));
		
		groupShapeColorButtonSpaceHolder.setPreferredSize(new Dimension(wid,hid));
		groupTextColorButtonSpaceHolder.setPreferredSize(new Dimension(wid,hid));
		
		
		groupFileTF.setEnabled(true);
		groupCB.setEnabled(false);
		groupInCB.setEnabled(false);
		groupOutCB.setEnabled(false);
		maxLoopLengthTF.setEnabled(false);
		loopLengthCB.setEnabled(false);
		upToLengthCB.setEnabled(false);
		specificLengthCB.setEnabled(false);
		groupBrowseButton.setEnabled(true);
		
		groupBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		
		vpCircleRadiusTF.setEnabled(false);
		vpCircleThreshholdTF.setEnabled(false);
		vpCircleGroupGapTF.setEnabled(false);
		
		vpCircleShapeSizeTF.setEnabled(false);
		vpArcsShapeSizeTF.setEnabled(false);
		
		vpCircleArrowWidthTF.setEnabled(false);
		vpArcsArrowWidthTF.setEnabled(false);
	
		vpArcsRadiusTF.setEnabled(false);
		vpArcsArcDegreesTF.setEnabled(false);
		
		vpRectangleWidthTF.setEnabled(false);

		circleRB.setSelected(true);
		
		setValues(true);

		int row = -1;

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;

		c.gridy = ++row;
		c.gridx = 0;
		c.anchor = GridBagConstraints.CENTER;
		add(adjacencyMatrixLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(adjacencyMatrixText, c);
		c.gridx = 1;
		c.weightx = 0.0;
		add(adjacencyMatrixBrowseButton, c);
		
		c.gridy = ++row;
		c.gridx = 0;
		add(groupFileLabel, c);
		
		c.gridy = ++row;
		c.gridx = 0;
		add(groupFileTF, c);
		c.gridx = 1;
		add(groupBrowseButton, c);
		
//		groupShapeColorButtonSpaceHolder.setBackground(Color.gray);
//		groupShapeColorButtonSpaceHolder.setEnabled(false);
		shapeColorPickers.add(groupShapeColorButtonSpaceHolder);
		
		c.gridy = ++row;
		c.gridx = 0;
		add(shapeColorPickers, c);
		
//		groupTextColorButtonSpaceHolder.setEnabled(false);
		textColorPickers.add(groupTextColorButtonSpaceHolder);
		
		c.gridy = ++row;
		c.gridx = 0;
		add(textColorPickers, c);
		
		
		
//		add(groupColorButtonSpaceHolder)

		c.weightx = 1.0;
		c.gridy = ++row;
		c.gridx = 0;
		c.anchor = GridBagConstraints.CENTER;
		add(mapFileLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(mapFileText, c);
		c.gridx = 1;
		c.weightx = 1.0;
		add(mapBrowseButton, c);

		ButtonGroup group = new ButtonGroup();
		group.add(circleRB);
		group.add(arcsRB);
		group.add(rectangleRB);

		visualizationPanel.add(circleRB);
		visualizationPanel.add(arcsRB);
		visualizationPanel.add(rectangleRB);

		c.gridy = ++row;
		c.gridx = 0;
		c.anchor = GridBagConstraints.CENTER;
		add(visualizeLabel, c);

		c.gridy = ++row;
		c.gridx = 0;
		add(visualizationPanel, c);

		visualizationParametersPanel.setLayout(new GridBagLayout());
		GridBagConstraints vpc = new GridBagConstraints();
		

		vpc.fill = GridBagConstraints.NONE;
		vpc.anchor = GridBagConstraints.CENTER;
		vpc.weightx = 1.0;
		vpc.weighty = 1.0;
		vpc.gridwidth = 1;
		vpc.gridy = 0;
		vpc.gridx = 0;

//		vpc.gridwidth = 2;
		visualizationParametersPanel.add(vpCircleLabel, vpc);

		vpc.gridwidth = 1;
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleRadiusLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleRadiusTF, vpc);

		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleThreshholdLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleThreshholdTF, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleGroupGapLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleGroupGapTF, vpc);
		
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleShapeSizeLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleShapeSizeTF, vpc);
		
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleArrowWidthLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpCircleArrowWidthTF, vpc);


		//            END CIRCLE

		vpc.gridy = 0;
		vpc.gridx = 2;
		visualizationParametersPanel.add(vpArcsLabel, vpc);

		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsRadiusLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsRadiusTF, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsOffsetLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsArcDegreesTF, vpc);
		
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsShapeSizeLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsShapeSizeTF, vpc);
		
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsArrowWidthLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpArcsArrowWidthTF, vpc);

		//            END ARCS

		vpc.gridy = 0;
		vpc.gridx = 3;
		visualizationParametersPanel.add(vpRectangleLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpRectangleWidthLabel, vpc);
		vpc.gridy++;
		visualizationParametersPanel.add(vpRectangleWidthTF, vpc);

		c.gridy = ++row;
		c.gridx = 0;
		add(visualizationParametersPanel, c);

		//        END RECTANGLE
		
		// group, loop options

//		private JButtonLanguage[] groupColorsB;
		
		optionsPanel.setLayout(new GridBagLayout());
		GridBagConstraints opc = new GridBagConstraints();
		
		int localRow = 0;
		opc.fill = GridBagConstraints.NONE;
		opc.anchor = GridBagConstraints.WEST;
		opc.weightx = 1.0;
		opc.weighty = 1.0;
		opc.gridwidth = 1;
		opc.gridy = localRow;
		opc.gridx = 0;
		
		
		optionsPanel.add(optionsLabel, opc);
		opc.gridy = ++localRow;
		optionsPanel.add(groupCB, opc);
		opc.gridx = 1;
//		opc.gridy = ++localRow;
//		optionsPanel.add(groupFileLabel, opc);
		
//		opc.gridy = ++localRow;
//		optionsPanel.add(groupFileTF, opc);
//		
//		groupColorPanel.add(groupColorButtonSpaceHolder);
//		
//		opc.gridx = 2;
//		optionsPanel.add(groupBrowseButton, opc);
		opc.gridx = 1;
		opc.gridy = ++localRow;
		optionsPanel.add(groupColorPanel, opc);
		
		opc.gridy = ++localRow;
		optionsPanel.add(groupOutCB, opc);
		
		opc.gridy = ++localRow;
		optionsPanel.add(groupInCB, opc);
		
//		opc.gridy = ++localRow;
//		optionsPanel.add(groupColorsB, opc);
		
		opc.gridx = 0;
		
		opc.gridy = ++localRow;
		optionsPanel.add(loopLengthCB,opc);
		opc.gridx = 1;
		opc.gridy = ++localRow;
		
		optionsPanel.add(maxLoopLengthLabel,opc);
		
		opc.gridy = ++localRow;
//		opc.anchor = GridBagConstraints.CENTER;
		optionsPanel.add(maxLoopLengthTF,opc);
		maxLoopLengthTF.setHorizontalAlignment(JTextField.CENTER);
		opc.anchor = GridBagConstraints.WEST;
		
		opc.gridy = ++localRow;
		optionsPanel.add(specificLengthCB,opc);
		
		opc.gridy = ++localRow;
		optionsPanel.add(upToLengthCB,opc);

		
		c.gridy = ++row;
		c.gridx = 0;
		add(optionsPanel, c);
		
		c.fill = GridBagConstraints.NONE;

		c.gridwidth = 1;

		c.fill = GridBagConstraints.CENTER;

		c.gridy = ++row;
		c.gridx = 0;
		add(mapButton, c);

	}

	private void createListeners() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		final String c1 = CHECK_DEFAULT;
		createBrowserListener(adjacencyMatrixBrowseButton, adjacencyMatrixText, "mdl", c1, 
					language.getText(StringConstants.SELECT_ADJACENCY_MATRIX_FILE));
		final String c2 = DO_NOT_CHECK_DEFAULT;
		createBrowserListener(mapBrowseButton, mapFileText, "docx", c2, 
				language.getText(StringConstants.SELECT_ADJACENCY_MATRIX_MAP_FILE));
		final String c3 = DO_NOT_CHECK_DEFAULT;
		createBrowserListener(groupBrowseButton, groupFileTF, "tab", c3, 
				language.getText(StringConstants.SELECT_ADJACENCY_MATRIX_GROUP_FILE));


		mapButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				List<String> shapeColors = new ArrayList<String>();
				List<String> textColors = new ArrayList<String>();
				
				for (int i = 0; i < groupShapeColorsB.length; i++) {
					Color c = groupShapeColorsB[i].getBackground();
					shapeColors.add(c.getRed()+"-"+c.getGreen()+"-"+c.getBlue());
					Color ct = groupTextColorsB[i].getBackground();
					textColors.add(ct.getRed()+"-"+ct.getGreen()+"-"+ct.getBlue());
					
				}

//				colors.add("0-128-0");
//				colors.add("0-0-255");
//				colors.add("128-0-128");t
//				colors.add("255-192-128");
				
				if (groups == null || groups.size() == 0) {
					loadGroupFile(groupFileTF.getText());
				}
				
				AdjacencyToMap cc = new AdjacencyToMap(
						adjacencyMatrixText.getText(), mapFileText.getText(), 
						circleRB.isSelected() ? AdjacencyToMap.FORMAT_CIRCLE : arcsRB.isSelected() ? 
								AdjacencyToMap.FORMAT_ARC : AdjacencyToMap.FORMAT_RECTANGLE,
								Integer.parseInt(vpCircleRadiusTF.getText()), 
								Integer.parseInt(vpArcsRadiusTF.getText()),
								Integer.parseInt(vpRectangleWidthTF.getText()),
								Double.parseDouble(vpCircleThreshholdTF.getText()),
								Integer.parseInt(vpCircleGroupGapTF.getText()),
								Double.parseDouble(vpArcsArcDegreesTF.getText()),
								groupCB.isSelected(), groups, shapeColors, textColors, groupInCB.isSelected(),
								groupOutCB.isSelected(), groupFileTF.getText(), loopLengthCB.isSelected(),
								maxLoopLengthTF.getText().length() > 0 ?
										Integer.parseInt(maxLoopLengthTF.getText()) : 0,
								specificLengthCB.isSelected(), upToLengthCB.isSelected(),
								circleRB.isSelected() ? Integer.parseInt(vpCircleShapeSizeTF.getText()) : 
									arcsRB.isSelected() ? Integer.parseInt(vpArcsShapeSizeTF.getText()) : 0,
											circleRB.isSelected() ? Integer.parseInt(vpCircleArrowWidthTF.getText()) : 
												arcsRB.isSelected() ? Integer.parseInt(vpArcsArrowWidthTF.getText()) : 2
								);
				try {
					cc.execute();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					JOptionPane
							.showMessageDialog(
									null,
									language.getText(StringConstants.ADJACENCY_TO_MAP_COMPLETED),
									language.getText(StringConstants.ADJACENCY_TO_MAP),
									JOptionPane.INFORMATION_MESSAGE);
				} catch (Exception e) {
					String errMsg = e.toString();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					log.log(Level.FATAL, "Internal Error Detected", e);
					JOptionPane.showMessageDialog(null, errMsg,
							language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
							JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});
		
		circleRB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				circleAction();
				setDependencies();
			}
		});

		arcsRB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				arcsAction();
				setDependencies();
			}
		});
		
		rectangleRB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				rectangleAction();
				setDependencies();
			}
		});
		
		groupFileTF.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				groupFileAction();
				setDependencies();
			}
		});
		groupCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				groupCBAction();
				setDependencies();
			}
		});
		groupInCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				groupInCBAction();
				setDependencies();
			}
		});
		groupOutCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				groupOutCBAction();
				setDependencies();
			}
		});
		maxLoopLengthTF.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				maxLoopLengthAction();
				setDependencies();
			}
		});
		loopLengthCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				loopsLengthCBAction();
				setDependencies();
			}
		});;
		upToLengthCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				upToLengthAction();
				setDependencies();
			}
		});
		specificLengthCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				specificLengthCBAction();
				setDependencies();
			}
		});
		
	} 
	
	private boolean looksLikeAnAdjacencyMatrixFile(String f) {
		boolean couldBe = true;
		
		List<String> contents = Utilities.getAsList(f);
		// expect first line to be a list of variable names
		// varname followed by tab delimited empty cells and 1's
		
		try {
			String[] columns = contents.get(0).split("\t");
			
			if (columns.length < 3) {
				return false;
			}
			
			for (String col : columns) {
				if (col.length() > 0 && Parser.isNumber(col)) {
					couldBe = false;
				}
			}
			
			for (int i = 1; i < contents.size(); i++) {
				columns = contents.get(i).split("\t");
				if (columns[0].length() == 0 || Parser.isNumber(columns[0]) )
					couldBe = false;
				for (int c = 1; c < columns.length; c++) {
					if (columns[c].length() > 0 && !Parser.isNumber(columns[c]))
						couldBe = false;
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			couldBe = false;
		}
		
		
		return couldBe;
		
	}
	
	private boolean looksLikeAnAdjacencyMatrixGroupFile(String f) {
		boolean couldBe = true;
		List<String> contents = Utilities.getAsList(f);
		// expect first line to column headers: Variable	Group
		// varname followed by tab and then a group name

		try {
			String[] columns = contents.get(0).split("\t");
			if (columns.length < 2) {
				couldBe = false;
				return couldBe;
			}
			if (!columns[0].equals("Variable")) {
				couldBe = false;
				return couldBe;
			}
			if (!columns[1].equals("Group")) {
				couldBe = false;
				return couldBe;
			}


			for (int i = 1; i < contents.size(); i++) {
				columns = contents.get(i).split("\t");
				if (columns.length == 2) {
					if (columns[0].length() == 0 || Parser.isNumber(columns[0]))
						couldBe = false;
					if (columns[1].length() == 0 || Parser.isNumber(columns[1]))
						couldBe = false;
				} else {
					couldBe = false;
					return couldBe;
				}
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			couldBe = false;
		}

		return couldBe;
	}
	
	protected void specificLengthCBAction() {
		// TODO Auto-generated method stub
		
	}

	protected void upToLengthAction() {
		// TODO Auto-generated method stub
		
	}

	protected void loopsLengthCBAction() {
		// TODO Auto-generated method stub
		
	}

	protected void maxLoopLengthAction() {
		// TODO Auto-generated method stub
		
	}

	protected void groupOutCBAction() {
		// TODO Auto-generated method stub
		
	}

	protected void groupInCBAction() {
		// TODO Auto-generated method stub
		
	}

	protected void groupCBAction() {
		// TODO Auto-generated method stub
		
	}
	
	public List<String> loadGroupFile(String groupFile) {
		List<String> contents = Utilities.getAsList(groupFile);
		groups = new ArrayList<String>();
		
		int rowPtr = 1;
		while (rowPtr < contents.size()) {
			String[] adj = contents.get(rowPtr).split("\t");
			String var = adj[0];
			String group = adj[1];
			if (!groups.contains(group))
				groups.add(group);
			
			rowPtr++;
		}
//		groupColor.put("Manage", "128-0-128");
//		groupColor.put("Connect", "0-128-0");
//		groupColor.put("Supply", "255-192-128");
//		groupColor.put("Distribute", "0-0-255");
		
		return groups;
	}

	protected void groupFileAction() {
		// TODO Auto-generated method stub
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific
				.getInstance();
		
		colors = new ArrayList<String>();

		colors.add("0-128-0");
		colors.add("0-0-255");
		colors.add("128-0-128");
		colors.add("255-192-128");
		
		// need to see if file exists, open it, read -- does it really look like a groups file
		File grp = new File(groupFileTF.getText());
		if (grp.exists()) {
			List<String> contents = Utilities.getAsList(groupFileTF.getText());
			
			int rowPtr = 1;
			while (rowPtr < contents.size()) {
				String[] adj = contents.get(rowPtr).split("\t");
				String var = adj[0];
				String group = adj[1];
				
				if (!groups.contains(group))
						groups.add(group);
				rowPtr++;
			}
			
			addGroupButtons();
		} else {
			JOptionPane.showMessageDialog(null,
					language.getText("FIle does not exist: groupFileTF.getText()"),
					language.getText(StringConstants.GROUPS), JOptionPane.ERROR_MESSAGE);
		}
		
	}
	
	private Color convertToColor(String s) {
		String[] parts = s.split("-");
		int red = Integer.parseInt(parts[0]);
		int green = Integer.parseInt(parts[1]);
		int blue = Integer.parseInt(parts[2]);
		return new Color(red,green,blue);
	}
	
	private void addGroupButtons() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		shapeColorPickers.removeAll();
		
		groupShapeColorsB = new JButton[groups.size()];
		groupTextColorsB = new JButton[groups.size()];

		
		for (int i = 0; i < groups.size(); i++) {
			final JButton b = new JButton(groups.get(i));
			b.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
			groupShapeColorsB[i] = b;
			
			final JButton bt = new JButton(groups.get(i));
			bt.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
			groupTextColorsB[i] = bt;
			
			if (i < colors.size()) {
				Color bc = convertToColor(colors.get(i));
				b.setBackground(bc);
				b.setForeground(bc);
				bt.setBackground(Color.black);
				bt.setForeground(Color.black);
			}
			ActionListener actionListener = new ActionListener() {
			      public void actionPerformed(ActionEvent actionEvent) {
			        Color initialBackground = b.getBackground();
			        Color background = JColorChooser.showDialog(null, "Group Shape Color Chooser - "+b.getText(), initialBackground);
			        if (background != null) {
			          b.setBackground(background);
			          b.setForeground(background);
			        }
			      }
			    };
			
			    b.addActionListener(actionListener);
			    
			    actionListener = new ActionListener() {
				      public void actionPerformed(ActionEvent actionEvent) {
				        Color initialBackground = bt.getBackground();
				        Color background = JColorChooser.showDialog(null, "Group Text Color Chooser - "+bt.getText(), initialBackground);
				        if (background != null) {
				          bt.setBackground(background);
				          bt.setForeground(background);
				        }
				      }
				    };
				
				    bt.addActionListener(actionListener);
			shapeColorPickers.add(b);
			textColorPickers.add(bt);
		}
		

		shapeColorPickers.validate();
		shapeColorPickers.repaint();
		
		textColorPickers.validate();
		textColorPickers.repaint();

		
	}

	private boolean fileExists(String file) {
		return (new File(file)).exists();
	}
	
	private void circleAction() {
		circleRB.setSelected(true);
		String n = generateDefaultMapFilename();
		if (fileExists(n))
			overwriteMap(n);
		else
			mapFileText.setText(n);
		
		vpCircleRadiusTF.setEnabled(true);
		vpCircleThreshholdTF.setEnabled(true);	
		vpCircleGroupGapTF.setEnabled(true);
		vpCircleShapeSizeTF.setEnabled(true);
		vpArcsShapeSizeTF.setEnabled(false);
		vpArcsRadiusTF.setEnabled(false);
		vpArcsArcDegreesTF.setEnabled(false);
		vpRectangleWidthTF.setEnabled(false);
		
		vpCircleArrowWidthTF.setEnabled(true);
		vpArcsArrowWidthTF.setEnabled(false);
	}

	private void arcsAction() {
		arcsRB.setSelected(true);
		String n = generateDefaultMapFilename();
		if (fileExists(n))
			overwriteMap(n);
		else
			mapFileText.setText(n);
		
		vpCircleRadiusTF.setEnabled(false);
		vpCircleThreshholdTF.setEnabled(false);	
		vpCircleGroupGapTF.setEnabled(false);	
		vpCircleShapeSizeTF.setEnabled(false);
		vpArcsShapeSizeTF.setEnabled(true);
		
		vpCircleArrowWidthTF.setEnabled(false);
		vpArcsArrowWidthTF.setEnabled(true);
		
		vpArcsRadiusTF.setEnabled(true);
		vpArcsArcDegreesTF.setEnabled(true);
		vpRectangleWidthTF.setEnabled(false);
	}

	private void rectangleAction() {
		rectangleRB.setSelected(true);
		String n = generateDefaultMapFilename();
		if (fileExists(n))
			overwriteMap(n);
		else
			mapFileText.setText(n);
		
		vpCircleRadiusTF.setEnabled(false);
		vpCircleThreshholdTF.setEnabled(false);	
		vpCircleGroupGapTF.setEnabled(false);	
		vpCircleShapeSizeTF.setEnabled(false);
		vpArcsShapeSizeTF.setEnabled(false);
		vpArcsRadiusTF.setEnabled(false);
		vpArcsArcDegreesTF.setEnabled(false);
		vpRectangleWidthTF.setEnabled(true);
		
		vpCircleArrowWidthTF.setEnabled(false);
		vpArcsArrowWidthTF.setEnabled(false);
	}

	private void createBrowserListener(/*final*/ JButtonLanguage button,
			/*final*/ JTextField textField, /*final*/ String fileType,
			/*final*/  String checkDefault,
			String titleBarText) {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		final JButtonLanguage fbutton = button;
		final String ffileType = fileType;
		final JTextField ftextField = textField;
		final String fcheckDefault = new String(checkDefault);
		final String ftitleBarText = new String(titleBarText);
		fbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				
				FileFilter filter = new FileNameExtensionFilter(ffileType
						.toUpperCase() + " Files", ffileType.toLowerCase());
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				//				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setDialogTitle(ftitleBarText);
				chooser.setLocation(0, 200);

				String savedValue = ftextField.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					saveFCDir(dir);
					String path = sf.getAbsolutePath();
					ftextField.setText(path);
				} else {
					ftextField.setText(savedValue);
					if (savedValue.length() == 0) {
					}
				}

				boolean fileLooksOK = true;

				if (fbutton.equals(adjacencyMatrixBrowseButton)) {
					if (!looksLikeAnAdjacencyMatrixFile(ftextField.getText())) {
						fileLooksOK = false;
						ftextField.setText(savedValue);
						if (savedValue.length() == 0) {
						}
						JOptionPane.showMessageDialog(null,
								language.getText(StringConstants.NOT_AN_ADJACENCY_MATRIX_FILE),
								language.getText(StringConstants.NOT_AN_ADJACENCY_MATRIX_FILE),
								JOptionPane.ERROR_MESSAGE);
						
					}

				} else if (fbutton.equals(mapBrowseButton)) {

				} else if (fbutton.equals(groupBrowseButton)) {
					if (!looksLikeAnAdjacencyMatrixGroupFile(ftextField.getText())) {
						fileLooksOK = false;
						ftextField.setText(savedValue);
						if (savedValue.length() == 0) {
						}
						JOptionPane.showMessageDialog(null,
								language.getText(StringConstants.NOT_AN_ADJACENCY_MATRIX_GROUP_FILE),
								language.getText(StringConstants.NOT_AN_ADJACENCY_MATRIX_GROUP_FILE),
								JOptionPane.ERROR_MESSAGE);
					}
				}
				if (fileLooksOK) {
					setDefaults(fbutton, ftextField, fcheckDefault);
					setDependencies();
					if (fbutton.equals(groupBrowseButton))
						groupFileAction(); 
				}
			}
		});
	}
	
	private String topology() {
		if (circleRB.isSelected()) {
			return "circle";
		} else if (arcsRB.isSelected()) {
			return "arcs";
		} else {
			return "rectangle";
		}
	}
	
	private String generateDefaultMapFilename() {
		String orig = adjacencyMatrixText.getText();
		if (orig.length() == 0)
			return "";

		String defaultExtracted = orig.replace(".TAB", ".mdl").replace(".tab",
				".mdl");
		
		defaultExtracted = defaultExtracted.replace(".mdl", "_"+topology()+"_ALL.mdl");
		return defaultExtracted;
	}

	private void setDefaults(JButtonLanguage fbutton, JTextField textField, String checkDefaults) {

		if (!checkDefaults.equals(CHECK_DEFAULT))
			return;
		String orig = adjacencyMatrixText.getText();
		if (orig.length() == 0)
			return;

		File aFile;
		
		String defaultExtracted = generateDefaultMapFilename();

		aFile = new File(defaultExtracted);
		if (fbutton.equals(adjacencyMatrixBrowseButton) && aFile.exists()) {
			overwriteMap(defaultExtracted);
		} else {
			mapFileText.setText(defaultExtracted);
		}

	}
	
	private boolean overwriteMap(String defaultExtracted) {
		UIManager.put("OptionPane.yesButtonText",SDMDocLanguageSpecific.getInstance()
				.getText(StringConstants.YES));
		   UIManager.put("OptionPane.noButtonText", SDMDocLanguageSpecific.getInstance()
					.getText(StringConstants.NO));

		int dialogButton = JOptionPane.YES_NO_OPTION;
		int reply = JOptionPane
				.showConfirmDialog(
						null,
						SDMDocLanguageSpecific
								.getInstance()
								.getText(
										StringConstants.VENSIM_MAP_FILE_EXISTS_AND_WILL_BE_OVERWRITTEN)
								+ "\n"
								+ SDMDocLanguageSpecific.getInstance()
										.getText(StringConstants.OVERWRITE),
						SDMDocLanguageSpecific.getInstance().getText(
								StringConstants.WARNING), dialogButton);

		if (reply == JOptionPane.YES_OPTION) {
			mapFileText.setText(defaultExtracted);
			return true;
		} else {
			mapFileText.setText("");
			return false;
		}
	}

	private void setDependencies() {
		
		circleRB.setEnabled(false);
		arcsRB.setEnabled(false);
		rectangleRB.setEnabled(false);
		
		mapButton.setEnabled(false);
		
		if (adjacencyMatrixText.getText().length() > 0 && groupFileTF.getText().length() > 0) {
			circleRB.setEnabled(true);
			arcsRB.setEnabled(true);
			rectangleRB.setEnabled(true);
			
			groupCB.setEnabled(true);
			loopLengthCB.setEnabled(true);
			
			if (groupCB.isSelected()) {
				groupFileTF.setEnabled(true);
				groupInCB.setEnabled(true);
				groupOutCB.setEnabled(true);
				groupBrowseButton.setEnabled(true);
			} else {
				groupFileTF.setEnabled(true);
				groupInCB.setEnabled(false);
				groupOutCB.setEnabled(false);
				groupBrowseButton.setEnabled(true);
			}
			
			if (loopLengthCB.isSelected()) {
				maxLoopLengthTF.setEnabled(true);
				upToLengthCB.setEnabled(true);
				specificLengthCB.setEnabled(true);
			} else {
				maxLoopLengthTF.setEnabled(false);
				upToLengthCB.setEnabled(false);
				specificLengthCB.setEnabled(false);
			}
			
			if (circleRB.isSelected()) {
//				circleAction();
				vpCircleRadiusTF.setEnabled(true);
				vpCircleThreshholdTF.setEnabled(true);	
				vpCircleGroupGapTF.setEnabled(true);
				
				vpCircleShapeSizeTF.setEnabled(true);
				vpArcsShapeSizeTF.setEnabled(false);
				
				vpCircleArrowWidthTF.setEnabled(true);
				vpArcsArrowWidthTF.setEnabled(false);
				
				vpArcsRadiusTF.setEnabled(false);
				vpArcsArcDegreesTF.setEnabled(false);
				vpRectangleWidthTF.setEnabled(false);

			} else if (arcsRB.isSelected()) {
//				arcsAction();
				vpCircleRadiusTF.setEnabled(false);
				vpCircleThreshholdTF.setEnabled(false);	
				vpCircleGroupGapTF.setEnabled(false);
				vpArcsRadiusTF.setEnabled(true);
				vpArcsArcDegreesTF.setEnabled(true);
				vpRectangleWidthTF.setEnabled(false);
				
				vpCircleShapeSizeTF.setEnabled(false);
				vpArcsShapeSizeTF.setEnabled(true);
				
				vpCircleArrowWidthTF.setEnabled(false);
				vpArcsArrowWidthTF.setEnabled(true);

			} else if (rectangleRB.isSelected()) {
//				rectangleAction();
				vpCircleRadiusTF.setEnabled(false);
				vpCircleThreshholdTF.setEnabled(false);	
				vpCircleGroupGapTF.setEnabled(false);	
				vpArcsRadiusTF.setEnabled(false);
				vpArcsArcDegreesTF.setEnabled(false);
				vpRectangleWidthTF.setEnabled(true);
				
				vpCircleShapeSizeTF.setEnabled(false);
				vpArcsShapeSizeTF.setEnabled(false);
				
				vpCircleArrowWidthTF.setEnabled(false);
				vpArcsArrowWidthTF.setEnabled(false);
			}
		}


			if (adjacencyMatrixText.getText().length() > 0
					&& mapFileText.getText().length() > 0
					&& groupFileTF.getText().length() > 0) {
				mapButton.setEnabled(true);
			}
		
	}

	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, 12), Color.black));

		adjacencyMatrixLabel.setText(language.getText(StringConstants.ADJACENCY_MATRIX_FILE));
		mapFileLabel.setText(language.getText(StringConstants.VENSIM_MAP_FILE));
		visualizeLabel.setText(language.getText(StringConstants.VISUALIZE_MAP_WITH));
		
		optionsLabel.setText(language.getText(StringConstants.OPTIONAL_GENERATED_MAP_OPTIONS));

		adjacencyMatrixBrowseButton.setText(language.getText(StringConstants.BROWSE));
		mapBrowseButton.setText(language.getText(StringConstants.BROWSE));
		groupBrowseButton.setText(language.getText(StringConstants.BROWSE));

		mapButton.setText(language.getText(StringConstants.GENERATE_MAP));
		
		circleRB.setText(language.getText(StringConstants.CIRCLE));
		arcsRB.setText(language.getText(StringConstants.ARCS));
		rectangleRB.setText(language.getText(StringConstants.RECTANGLE));
		
		
		vpCircleLabel.setText(language.getText(StringConstants.CIRCLE_PARAMETERS));
		vpArcsLabel.setText(language.getText(StringConstants.ARCS_PARAMETERS));
		vpRectangleLabel.setText(language.getText(StringConstants.RECTANGLE_PARAMETERS));
		
		vpCircleThreshholdLabel.setText(language.getText(StringConstants.THRESHHOLD));
		vpCircleGroupGapLabel.setText(language.getText(StringConstants.INTER_GROUP_GAP));
		vpCircleRadiusLabel.setText(language.getText(StringConstants.RADIUS));
		
		vpCircleArrowWidthLabel.setText(language.getText(StringConstants.SHAPE_SIZE));
		vpArcsArrowWidthLabel.setText(language.getText(StringConstants.SHAPE_SIZE));
		
		vpCircleShapeSizeLabel.setText(language.getText(StringConstants.ARROW_WIDTH));
		vpArcsShapeSizeLabel.setText(language.getText(StringConstants.ARROW_WIDTH));
		
		vpArcsOffsetLabel.setText(language.getText(StringConstants.ARC_DEGREES));
		vpArcsRadiusLabel.setText(language.getText(StringConstants.RADIUS));
		
		vpRectangleWidthLabel.setText(language.getText(StringConstants.WIDTH));
		
		
		groupFileLabel.setText(language.getText(StringConstants.GROUP_FILE));
		
		groupCB.setText(language.getText(StringConstants.BY_GROUP));
		groupInCB.setText(language.getText(StringConstants.BY_INPUT));
		groupOutCB.setText(language.getText(StringConstants.BY_OUTPUT));
		
		
		
		maxLoopLengthLabel.setText(language.getText(StringConstants.MAXIMUM_LOOP_LENGTH));
		loopLengthCB.setText(language.getText(StringConstants.BY_LOOP_LENGTH));
		specificLengthCB.setText(language.getText(StringConstants.SPECIFIC_LOOP_LENGTH));
		upToLengthCB.setText(language.getText(StringConstants.UP_TO_LOOP_LENGTH));
		
	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void setValues(boolean value) {
		
		vpCircleRadiusTF.setHorizontalAlignment(JTextField.CENTER);
		vpCircleThreshholdTF.setHorizontalAlignment(JTextField.CENTER);
		vpCircleGroupGapTF.setHorizontalAlignment(JTextField.CENTER);
		vpArcsRadiusTF.setHorizontalAlignment(JTextField.CENTER);
		vpArcsArcDegreesTF.setHorizontalAlignment(JTextField.CENTER);
		vpRectangleWidthTF.setHorizontalAlignment(JTextField.CENTER);
		
		vpCircleShapeSizeTF.setHorizontalAlignment(JTextField.CENTER);
		vpArcsShapeSizeTF.setHorizontalAlignment(JTextField.CENTER);
		
		vpCircleArrowWidthTF.setHorizontalAlignment(JTextField.CENTER);
		vpArcsArrowWidthTF.setHorizontalAlignment(JTextField.CENTER);
		
		
		vpCircleThreshholdTF.setText("10");	
		vpCircleThreshholdTF.setInputVerifier(new IntegerVerifier("Label Translation Degree",10,80));
		
		vpCircleGroupGapTF.setText("0");
		vpCircleGroupGapTF.setInputVerifier(new IntegerVerifier("Group Gap",0,10));
		
		vpArcsArcDegreesTF.setText("120");
		vpArcsArcDegreesTF.setInputVerifier(new IntegerVerifier("Arc Degrees",100,170));
		
		
		vpCircleRadiusTF.setText("500");
		vpArcsRadiusTF.setText("500");
		vpRectangleWidthTF.setText("500");
		
		vpCircleRadiusTF.setInputVerifier(new IntegerVerifier("Radius",100,1000));
		vpArcsRadiusTF.setInputVerifier(new IntegerVerifier("Radius",100,1000));
		vpRectangleWidthTF.setInputVerifier(new IntegerVerifier("Rectangle Width",100,1000));
		
		vpCircleShapeSizeTF.setText("15");
		vpArcsShapeSizeTF.setText("15");
		
		vpCircleShapeSizeTF.setInputVerifier(new IntegerVerifier("Shape Size",1,15));
		vpArcsShapeSizeTF.setInputVerifier(new IntegerVerifier("Shape Size",1,15));
		
		vpCircleArrowWidthTF.setText("1");
		vpArcsArrowWidthTF.setText("1");
		vpCircleArrowWidthTF.setInputVerifier(new IntegerVerifier("Arrow Width",1,6));
		vpArcsArrowWidthTF.setInputVerifier(new IntegerVerifier("Arrow Width",1,6));
		
	}

}
