package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Font;
import java.util.LinkedHashMap;
import java.util.Map;

public class ArrowInformationPanel extends SDMOptionsPanel {
	
	JCheckBoxLanguage positive = new JCheckBoxLanguage(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS);
	JCheckBoxLanguage negative = new JCheckBoxLanguage(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS);
	JCheckBoxLanguage unknown = new JCheckBoxLanguage(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS);
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();

	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public ArrowInformationPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);

		loadMap();
		
		add(positive);
		add(negative);
		add(unknown);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		positive.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		negative.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		unknown.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		setValues();
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		positive.setText(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS);
		negative.setText(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS);
		unknown.setText(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS);
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.ARROW_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.ARROW_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
		}
		return true;
	}
	
	private void loadMap() {
		

		mapNameToCheckBox.put(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS, positive);
		mapNameToCheckBox.put(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS, negative);
		mapNameToCheckBox.put(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS, unknown);
		
		
	}
	
	@Override
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.ARROW_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}
