package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.CodeGenerationOptions;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class CodeGenerationOptionsPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;
	
	private static final int TEXT_FIELD_SIZE = 6;
	private static String FC_DIR_FILE = "data/ChooserDirModelRuns.txt";
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	private Map<String, JTextField> mapNameToTextField = new LinkedHashMap<String, JTextField>();
	private Map<String, JRadioButton> mapNameToRadioButton = new LinkedHashMap<String, JRadioButton>();
	
	private List<String> runLogs = new ArrayList<String>();
	
	private JLabel equationLimitL = new JLabelLanguage(StringConstants.EQUATION_LIMIT);
	private JLabel flattenL = new JLabelLanguage(StringConstants.FLATTEN);
	private JLabel commentsL = new JLabelLanguage(StringConstants.COMMENTS);
	private JLabel runnerL = new JLabelLanguage(StringConstants.RUNNER);
	private JLabel hashMapL = new JLabelLanguage(StringConstants.HASHMAP);
	private JLabel nativeL = new JLabelLanguage(StringConstants.NATIVE);
	
	
	private JTextField equationLimit = new JTextField(TEXT_FIELD_SIZE);
	
	
	private JCheckBoxLanguage commentsCB = new JCheckBoxLanguage(StringConstants.COMMENTS);
	private JCheckBoxLanguage flattenCB = new JCheckBoxLanguage(StringConstants.FLATTEN);
	private JCheckBoxLanguage runnerCB = new JCheckBoxLanguage(StringConstants.RUNNER);
	
	private JRadioButton hashMapRB = new JRadioButtonLanguage(StringConstants.HASHMAP);
	private JRadioButton nativeRB = new JRadioButtonLanguage(StringConstants.NATIVE);
	
	private CodeGenerationOptions cgo = new CodeGenerationOptions();
	
	private CodeGenerationOptionsFrame optionsFrame;

	public CodeGenerationOptionsPanel(CodeGenerationOptionsFrame optionsFrame, String title) {
		
		this.optionsFrame = optionsFrame;
		
		setPreferredSize(new SdmDimension(200, 200));
		
		equationLimitL.setLabelFor(equationLimit);
		flattenL.setLabelFor(flattenCB);
		commentsL.setLabelFor(commentsCB);
		runnerL.setLabelFor(runnerCB);
		hashMapL.setLabelFor(hashMapRB);
		nativeL.setLabelFor(nativeRB);
		
//		cgo.setDefaults();
		
		loadMap();
		
		
		// override layout
		setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.NONE;
//		c.anchor = GridBagConstraints.NORTH;
		c.weightx = .10;
		c.weighty = 1.0;

		c.gridwidth = 1;
		c.gridy = 0;
		
		c.gridx = 0;
		add(equationLimitL, c);
		
		c.weightx = 1.0;
		c.fill = GridBagConstraints.NONE;
		
		c.gridx = 1;
		add(equationLimit, c);
		
		c.gridx = 2;
		add(flattenL, c);
		c.gridx = 3;
		add(flattenCB, c);;
		
		c.gridy++;
		
		c.gridx = 0;
		add(commentsL, c);
		c.gridx = 1;
		add(commentsCB, c);
		
		c.gridx = 2;
		add(runnerL, c);
		c.gridx = 3;
		add(runnerCB, c);
		
		ButtonGroup group = new ButtonGroup();
        group.add(hashMapRB);
        group.add(nativeRB);
     
		c.gridy++;
		
		c.gridx = 2;
		add(hashMapL, c);
		c.gridx = 3;
		add(hashMapRB, c);
		
		c.gridwidth = 3;
		c.gridy++;
		
		c.anchor = GridBagConstraints.WEST;
		
		c.gridx = 0;
		add(nativeL, c);
		
		c.anchor = GridBagConstraints.CENTER;
		
		c.gridwidth = 1;
		c.gridx = 3;
		add(nativeRB, c);
	
		setValues();
	}
	
	
	public void changeLanguage() {
		
		equationLimitL.setText(StringConstants.EQUATION_LIMIT);
		flattenL.setText(StringConstants.FLATTEN);
		commentsL.setText(StringConstants.COMMENTS);
		runnerL.setText(StringConstants.RUNNER);
		hashMapL.setText(StringConstants.HASHMAP);
		nativeL.setText(StringConstants.NATIVE);
		
	}
	
	

	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.CODE_GENERATION_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)
				cb.setSelected(value);
		}
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.CODE_GENERATION_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
				cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
			} else {
				JTextField tf = mapNameToTextField.get(var);
				if (tf != null) {
				tf.setText(options.getOptionValue(var));
				} else {
					JRadioButton rb = mapNameToRadioButton.get(var);
					rb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
				}
			}
		}
	}
	
	public void saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.CODE_GENERATION_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
			} else {
				JTextField tf = mapNameToTextField.get(var);
				if (tf != null) {
					options.setValue(var, options.getOptionType(var), tf.getText());
				} else {
					JRadioButton rb = mapNameToRadioButton.get(var);
					if (rb.isSelected())
						options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
					else
						options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
				}
			}
		}
	}
	
	private void loadMap() {
		
		mapNameToTextField.put(StringConstants.EQUATION_LIMIT, equationLimit);
	
		mapNameToCheckBox.put(StringConstants.COMMENTS, commentsCB);
		mapNameToCheckBox.put(StringConstants.FLATTEN, flattenCB);
		mapNameToCheckBox.put(StringConstants.RUNNER, runnerCB);
		
		mapNameToRadioButton.put(StringConstants.HASHMAP, hashMapRB);
		mapNameToRadioButton.put(StringConstants.NATIVE, nativeRB);
		
	}
	
	public CodeGenerationOptions getAsCodeGenerationOptions() {
		
		cgo.setEquationLimit(Integer.parseInt(equationLimit.getText()));
		cgo.setFlatten(flattenCB.isSelected());
		cgo.setIncludeComments(commentsCB.isSelected());
		cgo.setGenerateRunner(runnerCB.isSelected());
		cgo.setUseHashMapMemory(hashMapRB.isSelected());
		cgo.setUseNativeMemory(nativeRB.isSelected());
		
		return cgo;
	}
	
	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	

}

