package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class GraphOutputOptionsPanel extends SDMOptionsPanel implements FocusListener, KeyListener {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;
	
	private static final int TEXT_FIELD_SIZE = 6;
	private static String FC_DIR_FILE = "data/ChooserDirModelRuns.txt";
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	private Map<String, JTextField> mapNameToTextField = new LinkedHashMap<String, JTextField>();
	
	private List<String> runLogs = new ArrayList<String>();
	
	private JLabel widthL = new JLabelLanguage(StringConstants.WIDTH);
	private JLabel heightL = new JLabelLanguage(StringConstants.HEIGHT);
	private JLabel leftL = new JLabelLanguage(StringConstants.LEFT_MARGIN);
	private JLabel bottomL = new JLabelLanguage(StringConstants.BOTTOM_MARGIN);
	private JLabel fontSizeL = new JLabelLanguage(StringConstants.FONT_SIZE);
	private JLabel viewWidthL = new JLabelLanguage(StringConstants.VIEW_WIDTH_MAX);
	private JLabel highlightXL = new JLabelLanguage(StringConstants.HIGHLIGHT_POINT);
	private JLabel highlightYL = new JLabelLanguage("");
	private JLabel richardsonL = new JLabelLanguage(StringConstants.RULE_THRESHOLD);
	
	private final JLabel thumbnailMaximumL = new JLabelLanguage(StringConstants.THUMBNAIL_MAXIMUM);
	private final JTextField thumbnailMaximum = new JTextField(TEXT_FIELD_SIZE);
	
	private JTextField width = new JTextField(TEXT_FIELD_SIZE);
	private JTextField height = new JTextField(TEXT_FIELD_SIZE);
	private JTextField left = new JTextField(TEXT_FIELD_SIZE);
	private JTextField bottom = new JTextField(TEXT_FIELD_SIZE);
	private JTextField fontSize = new JTextField(TEXT_FIELD_SIZE);
	private JTextField viewWidth = new JTextField(TEXT_FIELD_SIZE);
	private JTextField highlightX = new JTextField(TEXT_FIELD_SIZE);
	private JTextField highlightY = new JTextField(TEXT_FIELD_SIZE);
	private JTextField richardson = new JTextField(TEXT_FIELD_SIZE);
	
	private JCheckBoxLanguage highlight = new JCheckBoxLanguage(StringConstants.HIGHLIGHT_POINT);
	private JCheckBoxLanguage zip = new JCheckBoxLanguage(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS);
	private JCheckBoxLanguage zipMDL = new JCheckBoxLanguage(StringConstants.INCLUDE_MDL);
	
	private JCheckBoxLanguage micmac = new JCheckBoxLanguage(StringConstants.MICMAC_ANALYSIS);
	private JCheckBoxLanguage thumbnails = new JCheckBoxLanguage(StringConstants.INCLUDE_THUMBNAILS);
	private JCheckBoxLanguage override = new JCheckBoxLanguage(StringConstants.LIMIT_THUMBNAIL_COUNT);
	private JCheckBoxLanguage flatFilesCB = new JCheckBoxLanguage(StringConstants.CREATE_FLAT_FILES);
	
	private JCheckBoxLanguage generate3d = new JCheckBoxLanguage(StringConstants.GENERATE_3D);
	
//	private JButton useDefault = new JButtonLanguage("Use Defaults");
	private JButton thumbnailBrowse = new JButtonLanguage(StringConstants.BROWSE);
	
	private String savedThumbnailLimit = "1000";
	private boolean displayingError = false;

	public GraphOutputOptionsPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);
		
//		setPreferredSize(new Dimension(260, 260));
		
		widthL.setLabelFor(width);
		heightL.setLabelFor(height);
		leftL.setLabelFor(left);
		bottomL.setLabelFor(bottom);
		fontSizeL.setLabelFor(fontSize);
		viewWidthL.setLabelFor(viewWidth);
		highlightXL.setLabelFor(highlightX);
		highlightYL.setLabelFor(highlightY);
		richardsonL.setLabelFor(richardson);
		
		width.setInputVerifier(new IntegerVerifier("Width",0,1000));
		height.setInputVerifier(new IntegerVerifier("Height",0,1000));
		left.setInputVerifier(new IntegerVerifier("Left Margin",0,10));
		bottom.setInputVerifier(new IntegerVerifier("Bottom Margin",0,10));
		fontSize.setInputVerifier(new IntegerVerifier("Font Size",5,15));
		viewWidth.setInputVerifier(new IntegerVerifier("View Width",0,2000));
		highlightX.setInputVerifier(new IntegerVerifier("Highlight X",1,10));
		highlightY.setInputVerifier(new IntegerVerifier("Highlight Y",1,10));
		richardson.setInputVerifier(new IntegerVerifier("Richardson Threshold",1,5));
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		widthL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		widthL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		heightL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		leftL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		bottomL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		fontSizeL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		viewWidthL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		highlightXL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		highlightYL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		richardsonL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		width.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		width.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		height.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		left.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		bottom.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		fontSize.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		viewWidth.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		highlightX.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		highlightY.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		richardson.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		highlight.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		zip.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		zipMDL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		micmac.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		thumbnails.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		override.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		flatFilesCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		thumbnailMaximumL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		thumbnailMaximum.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		loadMap();
		
		generate3d.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				set3DDependencies();
			}
		});
		
		zip.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		micmac.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		micmac.setEnabled(true);
		
		flatFilesCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		flatFilesCB.setEnabled(true);
		
		thumbnails.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		override.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		thumbnailBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {

				FileFilter filter = new FileNameExtensionFilter("Model Run Result Files",
						"tab");
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(true);
				chooser.setLocation(0, 200);

//				String savedValue = statusbar.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf[] = chooser.getSelectedFiles();
					for (File mr : sf) {
						String dir = mr.getParent();
						saveFCDir(dir);
						String path = mr.getAbsolutePath();
						runLogs.add(path);
					}

				} else {
					
				}
			}
		});
		
		// override layout
		setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.NONE;
//		c.anchor = GridBagConstraints.NORTH;
		c.weightx = .10;
		c.weighty = 1.0;

		c.gridwidth = 1;
		c.gridy = 0;
		
		c.gridx = 0;
		/*
		add(widthL, c);
		
		c.weightx = 1.0;
		c.fill = GridBagConstraints.NONE;
		
		c.gridx = 1;
		add(width, c);
		
		c.gridx = 2;
		add(leftL, c);
		c.gridx = 3;
		add(left, c);;
		
		c.gridy++;
		
		c.gridx = 0;
		add(heightL, c);
		c.gridx = 1;
		add(height, c);
		
		c.gridx = 2;
		add(bottomL, c);
		c.gridx = 3;
		add(bottom, c);
		
		c.gridy++;
		
		c.gridx = 2;
		add(fontSizeL, c);
		c.gridx = 3;
		add(fontSize, c);
		
		c.gridwidth = 3;
		c.gridy++;
		
		*/
		c.anchor = GridBagConstraints.WEST;
		
		c.gridx = 0;
		add(viewWidthL, c);
		
		c.anchor = GridBagConstraints.CENTER;
		
		c.gridwidth = 1;
		c.gridx = 3;
		add(viewWidth, c);
		
		c.anchor = GridBagConstraints.WEST;
		
		c.gridwidth = 2;
		c.gridy++;
		c.gridx = 0;
		add(highlight, c);
		
		c.gridwidth = 1;
		c.anchor = GridBagConstraints.CENTER;
		c.gridx = 2;
		add(highlightX, c);
		
		c.gridx = 3;
		add(highlightY, c);
		
		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy++;
		add(richardsonL, c);
		
		c.gridwidth = 1;
		c.gridx = 3;
		add(richardson, c);
		
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		c.gridy++;
		c.gridx = 0;
		add(micmac, c);
		
		
		
		c.gridwidth = 2;
		c.gridx = 2;
		add(thumbnails, c);
		
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		c.gridy++;
		c.gridx = 0;
		add(zip, c);
		
		c.insets = new Insets(0,20,0,0);
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		c.gridy++;
		c.gridx = 0;
		add(zipMDL, c);
		
		c.insets = new Insets(0,0,0,0);
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 2;
		c.gridy++;
		c.gridx = 0;
		add(flatFilesCB, c);
		
		c.gridy -= 1;
		c.gridwidth = 2;
		c.gridx = 2;
		add(override, c);
		
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 1;
		c.gridy++;
		c.gridx = 2;
		add(thumbnailMaximumL, c);

//		c.weightx = 1.0;
//		c.fill = GridBagConstraints.HORIZONTAL;
		thumbnailMaximum.addFocusListener(this);
		thumbnailMaximum.addKeyListener(this);
		c.gridy++;
		c.gridx = 2;
		add(thumbnailMaximum, c);
		
		c.insets = new Insets(0,0,0,0);
		c.gridy++;
		c.gridx = 0;
		if (StandaloneGUI.threeDEnabled())
			add(generate3d, c);
		
		setValues();
		setDependencies();
	}
	
	private void set3DDependencies() {
		if (generate3d.isSelected()) {
			
		} 
	}
	
	private void setDependencies() {
		if (zip.isSelected()) {
			zipMDL.setEnabled(true);
		} else {
			zipMDL.setEnabled(false);
		}
		if (thumbnails.isSelected()) {
			thumbnailBrowse.setEnabled(true);
			override.setEnabled(true);
			thumbnailMaximum.setEnabled(true);
			if (Integer.valueOf(thumbnailMaximum.getText()) > 1000)
				thumbnailMaximum.setText("1000");
		} else {
			if (thumbnailMaximum.getText().length() > 0) {
				savedThumbnailLimit = thumbnailMaximum.getText();
				if (Integer.valueOf(savedThumbnailLimit) > 1000)
					savedThumbnailLimit = "1000";
				thumbnailMaximum.setEnabled(false);
			}
			thumbnailMaximum.setText("1000");
			thumbnailBrowse.setEnabled(false);
			override.setEnabled(false);
		}
		if (override.isEnabled() && override.isSelected()) {
			thumbnailMaximum.setEnabled(true);
			thumbnailMaximum.setText(savedThumbnailLimit);
			if (Integer.valueOf(thumbnailMaximum.getText()) > 1000)
				thumbnailMaximum.setText("1000");
		} else {
			thumbnailMaximum.setText("1000");
			thumbnailMaximum.setEnabled(false);
			
		}
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		widthL.setText(StringConstants.WIDTH);
		heightL.setText(StringConstants.HEIGHT);
		leftL.setText(StringConstants.LEFT_MARGIN);
		bottomL.setText(StringConstants.BOTTOM_MARGIN);
		fontSizeL.setText(StringConstants.FONT_SIZE);
		viewWidthL.setText(StringConstants.VIEW_WIDTH_MAX);
		highlightXL.setText(StringConstants.HIGHLIGHT_POINT);
		highlightYL.setText("");
		richardsonL.setText(StringConstants.RULE_THRESHOLD);
		
		thumbnailMaximumL.setText(StringConstants.THUMBNAIL_MAXIMUM);
		
		highlight.setText(StringConstants.HIGHLIGHT_POINT);
		zip.setText(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS);
		generate3d.setText(StringConstants.GENERATE_3D);
		zipMDL.setText(StringConstants.INCLUDE_MDL);
		thumbnails.setText(StringConstants.INCLUDE_THUMBNAILS);
		micmac.setText(StringConstants.MICMAC_ANALYSIS);
		override.setText(StringConstants.LIMIT_THUMBNAIL_COUNT);
		flatFilesCB.setText(StringConstants.CREATE_FLAT_FILES);
	}
	
	private JPanel labelFieldPanel(JLabel label, JTextField text) {
		JPanel panel = new JPanel();
		
		panel.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
//		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;

		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
	
		panel.add(label, c);
		
		text.setPreferredSize(new SdmDimension(60, 20));
		text.setMaximumSize(new SdmDimension(60, 20));
		text.setMinimumSize(new SdmDimension(60, 20));
		
		c.weightx = 0.0;
		c.weighty = 0.0;

		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = 0;
		panel.add(text, c);
		
		return panel;
	}
	
	@Override
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.GRAPH_OUTPUT_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)
				cb.setSelected(value);
		}
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.GRAPH_OUTPUT_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
				if (options.getOptionValue(var) == null) {
					options.setValue(var, "boolean", UserSelectedOptions.FALSE);
				}
				cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
			} else {
				JTextField tf = mapNameToTextField.get(var);
				tf.setText(options.getOptionValue(var));
			}
		}
		
		if (Integer.valueOf(thumbnailMaximum.getText()) > 1000)
			thumbnailMaximum.setText("1000");
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.GRAPH_OUTPUT_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
			} else {
				JTextField tf = mapNameToTextField.get(var);
				options.setValue(var, options.getOptionType(var), tf.getText());
			}
		}
		return true;
	}
	
	private void loadMap() {
		
//		public static final String[] GRAPH_OUTPUT_OPTIONS = {
//			"Width", "Height", "Left Margin", "Bottom Margin", "Font Size",
//			"\"View\" Width Max (% of Screen)", "Highlight Point X", "Highlight Point Y", "Richardson's Rule Threshold"
//		};
		
		mapNameToTextField.put(StringConstants.WIDTH, width);
		mapNameToTextField.put(StringConstants.HEIGHT, height);
		mapNameToTextField.put(StringConstants.LEFT_MARGIN, left);
		mapNameToTextField.put(StringConstants.BOTTOM_MARGIN,bottom);
		mapNameToTextField.put(StringConstants.FONT_SIZE, fontSize);
		mapNameToTextField.put(StringConstants.VIEW_WIDTH_MAX, viewWidth);
		mapNameToTextField.put(StringConstants.HIGHLIGHT_POINT_X,highlightX);
		mapNameToTextField.put(StringConstants.HIGHLIGHT_POINT_Y, highlightY);
		mapNameToTextField.put(StringConstants.RULE_THRESHOLD, richardson);
		mapNameToTextField.put(StringConstants.THUMBNAIL_MAXIMUM, thumbnailMaximum);
		
		mapNameToCheckBox.put(StringConstants.HIGHLIGHT_POINT, highlight);
		mapNameToCheckBox.put(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS, zip);
		mapNameToCheckBox.put(StringConstants.GENERATE_3D, generate3d);
		mapNameToCheckBox.put(StringConstants.INCLUDE_MDL, zipMDL);
		mapNameToCheckBox.put(StringConstants.INCLUDE_THUMBNAILS, thumbnails);
		mapNameToCheckBox.put(StringConstants.MICMAC_ANALYSIS, micmac);
		mapNameToCheckBox.put(StringConstants.CREATE_FLAT_FILES, flatFilesCB);
		mapNameToCheckBox.put(StringConstants.LIMIT_THUMBNAIL_COUNT, override);
	}
	
	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	@Override
	public void focusGained(FocusEvent arg0) {
		// TODO Auto-generated method stub
		if (Integer.valueOf(thumbnailMaximum.getText()) > 1000)
			thumbnailMaximum.setText("1000");
	}

	@Override
	public void focusLost(FocusEvent arg0) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		// TODO Auto-generated method stub
		if (Integer.valueOf(thumbnailMaximum.getText()) > 1000) {
			if (!displayingError) {
				displayingError = true;
				JOptionPane.showMessageDialog(null,
						language.getText(StringConstants.MAXIMUM_NUMBER_OF_THUMBNAILS_LIMITED_TO)+" 1000",
						language.getText(StringConstants.MAXIMUM_NUMBER_OF_THUMBNAILS),
						JOptionPane.ERROR_MESSAGE);
				displayingError = false;
				thumbnailMaximum.setText("1000");
			}
		}

	}

	@Override
	public void keyPressed(KeyEvent arg0) {
		// TODO Auto-generated method stub
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyTyped(KeyEvent e) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		// TODO Auto-generated method stub
		displayingError = true;
		int id = e.getID();

		if (id == KeyEvent.KEY_TYPED) {
			String c = e.getKeyChar()+"";
			if (c.equals("\n")) {
				if (Integer.valueOf(thumbnailMaximum.getText()) > 1000) {
					JOptionPane.showMessageDialog(null,
							language.getText(StringConstants.MAXIMUM_NUMBER_OF_THUMBNAILS_LIMITED_TO)+" 1000",
							language.getText(StringConstants.MAXIMUM_NUMBER_OF_THUMBNAILS),
	        			    JOptionPane.ERROR_MESSAGE);
					thumbnailMaximum.setText("1000");
				}
			}
		}
		displayingError = false;


	}
	

}

