package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.CommentProcessor;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.filechooser.FileFilter;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

public class ImportSDMDoc extends JFrame {
	
	static Logger log = Logger.getLogger(ImportSDMDoc.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1893237669846913860L;

	private static String FC_DIR_FILE = "data/ChooserDir.txt";
	private static String FEEDBACK_URL = "http://www.gss.anl.gov/system-dynamics-model-documentation-tool-sdm-doc/";
	
	private MDLProcessor sdmDocHtml = null;
	
	private String destDir;
	
	private String modelName = null;
	
	public ImportSDMDoc(String modelName, String mdlFile) {
		this.modelName = modelName;
		this.destDir = new File(mdlFile).getParent();
	}
	
	public ImportSDMDoc() {
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event-dispatching thread.
	 */
	private/* static */void createAndShowGUI() {

		SDMDocLanguageSpecific.initialize();

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		final JTextArea statusbar = new JTextArea("");

		//

		// Create and set up the window.
		final JFrame frame = new JFrame(
				language.getText(StringConstants.SDM_DOC_TOOL) + "/Java "
						+ language.getText(StringConstants.VERSION) + StandaloneGUI.version
						+ " -- " + language.getDisplayName());
		frame.setPreferredSize(new SdmDimension(600, 200));
		
		if (modelName == null) {
		frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE); // JFrame.EXIT_ON_CLOSE
		
		} else {
			frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);
		}

		JPanel panel = new JPanel();

		final JButtonLanguage genDocButton;
		genDocButton = new JButtonLanguage("Import");

		final JButtonLanguage fileChooser;

		fileChooser = new JButtonLanguage(StringConstants.BROWSE);
		// The options panel

		UserSelectedOptions uso = new UserSelectedOptions("data/SDM-Doc_UserOptions.tab" /* load/save to file */);
		
		final AboutFrame aboutFrame = new AboutFrame();
		
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		final JMenuBar menubar = new JMenuBar();

		final JMenu file = new JMenu(language.getText(StringConstants.FILE));

		final JMenuItemLanguage exitMenuItem = new JMenuItemLanguage(StringConstants.EXIT);

		exitMenuItem.setToolTipText(language
				.getText(StringConstants.EXIT_APPLICATION));

		exitMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				if (InformationManagers.getInstance() != null && InformationManagers.getInstance().getUserSelectedOptions() != null)
					InformationManagers.getInstance().getUserSelectedOptions().close();
				System.exit(0);
			}
		});

		file.add(exitMenuItem);

		genDocButton.setBounds(50, 60, 80, 30);
		genDocButton.setAlignmentX(Component.CENTER_ALIGNMENT);

		genDocButton.setEnabled(false);

		final JLabel label = new JLabel("Existing SDM-Doc HTML File");
		label.setFont(language.getFont(Font.BOLD, 18));

		final JMenu help = new JMenu(language.getText(StringConstants.HELP));
		final JMenuItemLanguage aboutMenuItem = new JMenuItemLanguage(StringConstants.ABOUT);
		final JMenuItemLanguage feedbackMenuItem = new JMenuItemLanguage(
				StringConstants.FEEDBACK);

		aboutMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				aboutFrame.changeLanguage();
				aboutFrame.update(null);
				aboutFrame.showPanel();
			}
		});

		feedbackMenuItem.setToolTipText(language.getText(StringConstants.FEEDBACK));
		feedbackMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				BrowserControl.displayURL(FEEDBACK_URL);
				// http://www.dis.anl.gov/feedback/sys_dyn_model.php?tname=your%20name&temail=your%20email&tphone=&tlocation=&tversion=5.0
			}
		});


		help.add(aboutMenuItem);
		help.add(feedbackMenuItem);
		menubar.add(file);
	
		menubar.add(help);

		frame.setJMenuBar(menubar);

		genDocButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				importSDMDoc(statusbar.getText());
				new OperationCompletedSuccessfully();
			}
		});

		fileChooser.setBounds(50, 60, 80, 30);
		fileChooser.setAlignmentX(Component.CENTER_ALIGNMENT);

		// Create a file chooser that opens up as an Open dialog
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) { // SDMDocModelNameFileFilter
				FileFilter filter = new SDMDocModelNameFileFilter(modelName, "Existing SDM-Doc Files for Model "+modelName);
//				FileFilter filter = new FileNameExtensionFilter("Model Files",
//						"mdl", "xml");
//				FileFilter filter = new FileNameExtensionFilter("Model Files",
//						"mdl");
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(0, 200);

				String savedValue = statusbar.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					saveFCDir(dir);
					String path = sf.getAbsolutePath();
					genDocButton.setEnabled(true);
					statusbar.setText(path);
				} else {
					statusbar.setText(savedValue);
					if (savedValue.length() == 0)
						genDocButton.setEnabled(false);
				}
			}
		});
		
//		if (preselectedMDL != null) {
//			statusbar.setText(preselectedMDL);
			genDocButton.setEnabled(false);
			fileChooser.setEnabled(true);
			
//		} 

		c.insets = new Insets(0, 0, 0, 0); // top padding
		c.fill = GridBagConstraints.CENTER;
		c.weightx = 1.0;
		c.weighty = 1.0;
		// c.fill = GridBagConstraints.HORIZONTAL;

		c.gridwidth = 3;
		c.gridx = 1;
		c.gridy = 0;
		panel.add(label, c);

		c.gridwidth = 3;
		c.gridx = 1;
		c.gridy = 1;
		panel.add(statusbar, c);
		statusbar.setPreferredSize(new SdmDimension(500, 25));
		statusbar.setFont(new Font("Arial", Font.PLAIN, 14));

		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 2;
//		if (preselectedMDL == null) {
			panel.add(fileChooser, c);
			fileChooser.setPreferredSize(new SdmDimension(200, 25));
			fileChooser.setFont(language.getFont(Font.PLAIN, 14));
//		}

		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 3;
		panel.add(genDocButton, c);
		genDocButton.setPreferredSize(new SdmDimension(200, 25));
		genDocButton.setFont(language.getFont(Font.PLAIN, 14));

		// panel.add(exitButton);

		frame.getContentPane().add(panel);

		// Display the window.
		frame.pack();
		frame.setVisible(true);
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));

		// Cursor.DEFAULT_CURSOR Cursor.WAIT_CURSOR
		
//		runAnyTestCode();

	}
	
	private void importSDMDoc(String htmlFile) {
		String fileHead = htmlFile.replaceAll(".html$", "");
		File sourceDirFile = new File(htmlFile).getParentFile();
		File destDirFile = new File(destDir);
		try {
			FileUtils.copyFileToDirectory(new File(htmlFile), destDirFile);
			
			File zip = new File(fileHead+".zip");
			if (zip.exists()) {
				FileUtils.copyFileToDirectory(zip, destDirFile);
			}
			
			File mdl = new File(sourceDirFile.getAbsolutePath()+"/"+modelName+".mdl");
			if (mdl.exists()) {
				FileUtils.copyFileToDirectory(mdl, destDirFile);
			}
			
			File map = new File(sourceDirFile.getAbsolutePath()+"/"+modelName+".map");
			if (map.exists()) {
				FileUtils.copyFileToDirectory(map, destDirFile);
			}
			
			File outDir = new File(sourceDirFile.getAbsolutePath()+"/"+modelName+"_DOC_GRAPH_");
			if (outDir.exists()) {
				FileUtils.copyDirectoryToDirectory(outDir, destDirFile);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void runAnyTestCode() {
		List<String> col1 = new ArrayList<String>();
		List<String> col2 = new ArrayList<String>();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String s : StringConstants.ALL_STRING_CONSTANTS) {
			col1.add(s);
			col2.add(language.getText(s));
		}
		CommentProcessor cp = new CommentProcessor();
		cp.createTable(col1, col2);
	}
	


	private void initializeLanguageMenu(JFrame frame, JMenu file,
			JMenuItemLanguage exitMenuItem, JMenuItemLanguage optionsMenuItem,
			JMenuItemLanguage acknowledgementsMenuItem, JMenu lang, JMenu help,
			JMenuItemLanguage aboutMenuItem, JMenuItemLanguage feedbackMenuItem,
			JButtonLanguage genDocButton, JLabel screenTitle, JButtonLanguage fileChooser,
			OptionsFrame optionsFrame, CodeGenerationOptionsFrame cgOptionsFrame, AcknowledgementsFrame af, JMenu tools, 
			JMenuItemLanguage extractCommentsMenuItem, JMenuItemLanguage mergeCommentsMenuItem) {
		final JFrame fframe = frame;
		final JMenu ffile = file;
		
		final JMenu ftools = tools;
		final JMenuItemLanguage fextractCommentsMenuItem = extractCommentsMenuItem;
		final JMenuItemLanguage fmergeCommentsMenuItem = mergeCommentsMenuItem;
		
		final JMenuItemLanguage fexitMenuItem = exitMenuItem;
		final JMenuItemLanguage foptionsMenuItem = optionsMenuItem;
		final JMenu flang = lang;
		final JMenu fhelp = help;
		final JMenuItemLanguage faboutMenuItem = aboutMenuItem;
		final JMenuItemLanguage ffeedbackMenuItem = feedbackMenuItem;
		final JMenuItemLanguage facknowledgementsMenuItem = acknowledgementsMenuItem;
		final JButtonLanguage fgenDocButton = genDocButton;
		final JLabel fscreenTitle = screenTitle;
		final JButtonLanguage ffileChooser = fileChooser;
		final OptionsFrame foptionsFrame = optionsFrame;
		final CodeGenerationOptionsFrame fcgOptionsFrame = cgOptionsFrame;
		final AcknowledgementsFrame faf = af;

		lang.removeAll();

		JMenuItemLanguage langRegMenuItem = new JMenuItemLanguage(
						StringConstants.REGISTER_LANGUAGE);
		final JFrame thisFrame = this;
		langRegMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageRegistrationDialog lrf = new LanguageRegistrationDialog(
						thisFrame, "underConstructionpng");
				// lrf.setVisible(true);
				initializeLanguageMenu(fframe, ffile, fexitMenuItem,
						foptionsMenuItem, facknowledgementsMenuItem, flang,
						fhelp, faboutMenuItem, ffeedbackMenuItem,
						fgenDocButton, fscreenTitle, ffileChooser,
						foptionsFrame, fcgOptionsFrame, faf, ftools, fextractCommentsMenuItem, fmergeCommentsMenuItem);
			}
		});
		lang.add(langRegMenuItem);

		for (String l : SDMDocLanguageSpecific.getKnownDisplayNames()) {
			JMenuItemSpecificLanguage langMenuItem = new JMenuItemSpecificLanguage(l);
			final String thisLang = langMenuItem.getLanguage();
			langMenuItem.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent event) {

					// here we assign the new language to be used
					boolean setAsDefault = false;
					SDMDocLanguageSpecific.getInstance(thisLang, setAsDefault);

					SDMDocLanguageSpecific language = SDMDocLanguageSpecific
							.getInstance();
					System.out.println("language select menu item " + thisLang
							+ " instance = " + language.getDisplayName());

					changeLanguage(fframe, ffile, fexitMenuItem,
							foptionsMenuItem, facknowledgementsMenuItem, flang,
							fhelp, faboutMenuItem, ffeedbackMenuItem,
							fgenDocButton, fscreenTitle, ffileChooser,
							foptionsFrame, fcgOptionsFrame, faf, ftools, fextractCommentsMenuItem, fmergeCommentsMenuItem);

				}
			});
			lang.add(langMenuItem);
		}
	}

	public void actionPerformed(ActionEvent e) {
		if ("disable".equals(e.getActionCommand())) {

		} else {

		}
	}
	
	public static void viaSDMUI(String modelName, String mdlFile) {
		final ImportSDMDoc gui = new ImportSDMDoc(modelName, mdlFile);
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		
		final ImportSDMDoc gui = new ImportSDMDoc();
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void changeLanguage(JFrame frame, JMenu file,
			JMenuItemLanguage exitMenuItem, JMenuItemLanguage optionsMenuItem,
			JMenuItemLanguage acknowledgementsMenuItem, JMenu lang, JMenu help,
			JMenuItemLanguage aboutMenuItem, JMenuItemLanguage feedbackMenuItem,
			JButtonLanguage genDocButton, JLabel screenTitle, JButtonLanguage fileChooser,
			OptionsFrame optionsFrame, CodeGenerationOptionsFrame cgOptionsFrame, AcknowledgementsFrame af, JMenu tools, 
			JMenuItemLanguage extractCommentsMenuItem, JMenuItemLanguage mergeCommentsMenuItem) {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		System.out.println("changeLanguage displayName "
				+ language.getDisplayName());

		screenTitle.setFont(language.getFont(Font.BOLD, 18));
		fileChooser.setFont(language.getFont(Font.PLAIN, 14));
		genDocButton.setFont(language.getFont(Font.PLAIN, 14));

		initializeLanguageMenu(frame, file, exitMenuItem, optionsMenuItem,
				acknowledgementsMenuItem, lang, help, aboutMenuItem,
				feedbackMenuItem, genDocButton, screenTitle, fileChooser,
				optionsFrame, cgOptionsFrame, af, tools, extractCommentsMenuItem, mergeCommentsMenuItem);

		feedbackMenuItem.setText(StringConstants.FEEDBACK);
		lang.setText(StringConstants.LANGUAGE);

		frame.setTitle(language.getText(StringConstants.SDM_DOC_TOOL)
				+ "/Java " + language.getText(StringConstants.VERSION) + StandaloneGUI.version
				+ " -- " + language.getDisplayName());

		file.setText(StringConstants.FILE);

		exitMenuItem.setText(StringConstants.EXIT);
		exitMenuItem.setToolTipText(StringConstants.EXIT_APPLICATION);

		optionsMenuItem.setText(StringConstants.OPTIONS);
		optionsMenuItem.setToolTipText(StringConstants.SELECT_USER_OPTIONS);
		
		mergeCommentsMenuItem.setText(StringConstants.MERGE_COMMENTS);
		extractCommentsMenuItem.setText(StringConstants.EXTRACT_COMMENTS);
		tools.setText(StringConstants.TOOLS);

		help.setText(StringConstants.HELP);

		aboutMenuItem.setText(StringConstants.ABOUT);
		aboutMenuItem.setToolTipText(StringConstants.ABOUT);

		acknowledgementsMenuItem.setText(StringConstants.ACKNOWLEDGEMENTS);
		acknowledgementsMenuItem.setToolTipText(StringConstants.ACKNOWLEDGEMENTS);

//		genDocButton.setText(language.getText(StringConstants.GENERATE_DOCUMENT));
		genDocButton.setText(StringConstants.GENERATE_DOCUMENT);
		screenTitle.setText(StringConstants.DEFINITION_INPUT_FILE);

		fileChooser.setText(StringConstants.BROWSE);

		af.changeLanguage();

		frame.update(frame.getGraphics());
		optionsFrame.changeLanguage();
		optionsFrame.update(optionsFrame.getGraphics());

	}


}
