package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.LanguageAdministration;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class LanguageAdministrationDialog extends SDMDocJDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private JFrame parent;
	LanguageAdministrationDialog thisDialog;
	
	/**
	 * Conastructor
	 * @param parent
	 */
	
	public LanguageAdministrationDialog(JFrame parent, String helpFile) {
		super(parent, ModalityType.DOCUMENT_MODAL, helpFile);
		initialize();
	}

	public LanguageAdministrationDialog(JFrame parent) {
		super(parent, ModalityType.MODELESS);
		initialize();
	}
	private void initialize() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setTitle(language.getText(StringConstants.LANGUAGE_ADMINISTRATION));
		setPreferredSize(new SdmDimension(650, 650)); // 600, 320
		setLocation (20, 150) ;
		create();
		pack();
		setVisible(true);
		thisDialog = this;
//		this.parent = parent;
	}

	private void create() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		addInformationMenu(this);
		setJMenuBar(myMenubar);
		
		JPanel panel = new JPanel();
		panel.setPreferredSize(new SdmDimension(600, 600)); // 600, 320
		this.getContentPane().add(panel);
		final JTextArea fileName = new JTextArea();
		fileName.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		fileName.setPreferredSize(new SdmDimension(300, 25));
		final JButton generateFile = new JButtonLanguage(StringConstants.GENERATE_TRANSLATION_FILES);
		generateFile.setEnabled(false);
		final JList<String> listbox = new JList<String>();

		//		fileName.setFont(new Font("Arial", Font.PLAIN, 14));
		//		panel.add(fileName);

		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;


		final JButton fileChooser;

		fileChooser = new JButtonLanguage(StringConstants.BROWSE);

		fileChooser.setPreferredSize(new SdmDimension(250, 25));
		fileChooser.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		fileChooser.setBounds(50, 60, 80, 30);
		fileChooser.setAlignmentX(Component.CENTER_ALIGNMENT);

		//        panel.add(fileChooser);



		// Create a file chooser that opens up as an Open dialog
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter("Docx Files","docx");
				JFileChooser chooser = new JFileChooser();
				chooser.setLocale(SDMDocLanguageSpecific.getInstance().getLocale());
//				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation (0, 200) ;
				chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

				String savedValue = fileName.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();

					String path = sf.getAbsolutePath();
					fileName.setText(path);
					
					if (listbox.getSelectedIndex() != -1)
						generateFile.setEnabled(true);
				}
				else {
					fileName.setText(savedValue);
					if (savedValue.length() == 0) {}
					//            	  genDocButton.setEnabled(false);
				}
			}
		});
		
		
		List<String> al = SDMDocLanguageSpecific.getSupportedDisplayNames();
		String[] listData = new String[al.size()];
		listData = al.toArray(listData);
		listbox.setListData(listData);


		// Create a new listbox control
		
		listbox.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		listbox.setName(language.getText(StringConstants.SELECT_ONE_OR_MORE));
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(listbox);
		scrollPane.setMinimumSize(new SdmDimension(300, 100));
		
		listbox.addListSelectionListener( new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e){
			int sel = listbox.getSelectedIndex();
	        if( sel != -1 ){
			    if (fileName.getText().length() > 0)
			    	generateFile.setEnabled(true);
			}
		    }
		} );


		

		generateFile.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				SDMDocLanguageSpecific.initialize();
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				boolean setAsDefault = false;
				SDMDocLanguageSpecific currentLanguage = new SDMDocLanguageSpecific(language.getLanguage(), setAsDefault,  parent);
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				String dir = fileName.getText();
				
				LanguageAdministration la = new LanguageAdministration();
				la.generateKeyFile(dir+"/LanguageKey.docx");
				
				for (String s : listbox.getSelectedValuesList()) {
					la.generateTranslationFile(s, SDMDocLanguageSpecific.getLanguageForDisplayName(s), 
							dir+"/LanguageTranslation"+s+".docx");
				}
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				
				SDMDocLanguageSpecific.setInstance(currentLanguage);
				listbox.clearSelection();
				JOptionPane
				.showMessageDialog(
						null,
						language.getText(StringConstants.PROCESSING_COMPLETE),
						language.getText(StringConstants.PROCESSING_COMPLETE),
						JOptionPane.INFORMATION_MESSAGE);
			}
		});

		int row = 0;
		
		JLabel label = new JLabelLanguage(StringConstants.LANGUAGE_ADMINISTRATION);
		label.setFont(language.getFont(Font.BOLD, SDMDocFont.LARGE));

		JLabel label1 = new JLabelLanguage(StringConstants.TRANSLATION_FILE_DIRECTORY);
		label1.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		JLabel label2 = new JLabelLanguage(StringConstants.SELECT_ONE_OR_MORE);
		label2.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));



		c.insets = new Insets(5,0,0,0);  //top padding
		c.fill = GridBagConstraints.VERTICAL;
		c.weightx = 0.0;
		c.weighty = 0.0;


		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label,c);
		
		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label1,c);


		c.gridwidth = 3;
		c.gridx = 0;
		c.gridy =row++;
		panel.add(fileName,c);
//		fileName.setMinimumSize(new SdmDimension(500, 25));  // 500, 25


		// fileChooser

		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(fileChooser,c);
		
		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label2,c);

		
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(scrollPane,c);

		
		c.weightx = 0.0;
		c.weighty = 0.0;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(generateFile,c);
		generateFile.setMinimumSize(new SdmDimension(220, 25));
		generateFile.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
	}
	
	/**
	 * Hide Panel
	 */
	
	public void hidePanel() {
		setVisible(false);
	}
	
	/**
	 * Show panel
	 */
	
	public void showPanel() {
		setVisible(false);
	}

}
