package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.LanguageRegistration;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class LanguageRegistrationDialog extends SDMDocJDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private JFrame parent;
	
	/**
	 * Conastructor
	 * @param parent
	 */
	
	public LanguageRegistrationDialog(JFrame parent, String helpFile) {
		super(parent, ModalityType.DOCUMENT_MODAL, helpFile);
		initialize();
	}

	public LanguageRegistrationDialog(JFrame parent) {
		super(parent, ModalityType.DOCUMENT_MODAL);
		initialize();
		
	}
	private void initialize() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setTitle(language.getText(StringConstants.REGISTER_LANGUAGE));
		setPreferredSize(new SdmDimension(600, 400));
		setLocation (20, 150) ;
		create();
		pack();
		setVisible(true);
	}

	private void create() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		JPanel panel = new JPanel();
		panel.setMinimumSize(new Dimension(600, 320));
		this.getContentPane().add(panel);
		final JTextArea fileName = new JTextArea();
		fileName.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		final JButton register = new JButtonLanguage(StringConstants.REGISTER_LANGUAGE);
		register.setEnabled(false);
		final JList<String> listbox = new JList<String>();
		
		addInformationMenu(this);
		setJMenuBar(myMenubar);

		//		fileName.setFont(new Font("Arial", Font.PLAIN, 14));
		//		panel.add(fileName);

		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;


		final JButton fileChooser;

		fileChooser = new JButtonLanguage(StringConstants.BROWSE);

		fileChooser.setPreferredSize(new SdmDimension(200, 25));
		fileChooser.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		fileChooser.setBounds(50, 60, 80, 30);
		fileChooser.setAlignmentX(Component.CENTER_ALIGNMENT);

		//        panel.add(fileChooser);



		// Create a file chooser that opens up as an Open dialog
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter("Docx Files","docx");
				JFileChooser chooser = new JFileChooser();
				chooser.setLocale(SDMDocLanguageSpecific.getInstance().getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation (0, 200) ;

				String savedValue = fileName.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();

					String path = sf.getAbsolutePath();
					fileName.setText(path);
					
					if (listbox.getSelectedIndex() != -1)
						register.setEnabled(true);
				}
				else {
					fileName.setText(savedValue);
					if (savedValue.length() == 0) {}
					//            	  genDocButton.setEnabled(false);
				}
			}
		});
		
		

		List<String> al = SDMDocLanguageSpecific.getSupportedDisplayNames();
		String[] listData = new String[al.size()];
		listData = al.toArray(listData);
		listbox.setListData(listData);


		// Create a new listbox control
		
		listbox.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		listbox.setName(language.getText(StringConstants.SELECT_LANGUAGE));
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(listbox);
		scrollPane.setMinimumSize(new SdmDimension(300, 200)); // 100
		
		listbox.addListSelectionListener( new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e){
			int sel = listbox.getSelectedIndex();
	        if( sel != -1 ){
			    if (fileName.getText().length() > 0)
			    	register.setEnabled(true);
			}
		    }
		} );


		

		register.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				// check first we will be overwriting an existing language translation file
				
				if (SDMDocLanguageSpecific.languageTranslationBundleExists(listbox.getSelectedValue())) {
					int dialogButton = JOptionPane.YES_NO_OPTION;
	                JOptionPane.showConfirmDialog (null, 
	                		language.getText(StringConstants.THIS_LANGUAGE_TRANSLATION_HAS_ALREADY_BEEN_REGISTERED)+
	                		"\n"+language.getText(StringConstants.OVERWRITE),
	                		language.getText(StringConstants.WARNING),dialogButton);

	                if(dialogButton != JOptionPane.YES_OPTION) {
	                	return;
	                }
				}
				
				
				LanguageRegistration lr = new LanguageRegistration(listbox.getSelectedValue());
				lr.register(fileName.getText());
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				JOptionPane.showMessageDialog(null,
						language.getText(StringConstants.REGISTRATION_COMPLETE),
						language.getText(StringConstants.LANGUAGE_REGISTRATION),
    					JOptionPane.INFORMATION_MESSAGE);
				hidePanel();
			}
		});

		int row = 0;

		JLabel label = new JLabelLanguage(StringConstants.TRANSLATION_FILE);
		label.setFont(language.getFont(Font.BOLD, SDMDocFont.LARGE));



		c.insets = new Insets(5,0,0,0);  //int top, int left, int bottom, int right
		c.fill = GridBagConstraints.VERTICAL;   // CENTER
		c.weightx = 0.0;
		c.weighty = 0.0;


		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label,c);


		c.gridwidth = 3;
		c.gridx = 0;
		c.gridy =row++;
		panel.add(fileName,c);
		fileName.setPreferredSize(new SdmDimension(500, 25));


		// fileChooser

		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(fileChooser,c);

		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		c.weightx = 1.0;
		c.weighty = 1.0;
		panel.add(scrollPane,c);

		c.gridx = 1;
		c.gridy = row++;
		c.weightx = 0.0;
		c.weighty = 0.0;
		panel.add(register,c);
		register.setMinimumSize(new SdmDimension(200, 25));
		register.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
	}
	
	/**
	 * Hide Panel
	 */
	
	public void hidePanel() {
		setVisible(false);
	}
	
	/**
	 * Show panel
	 */
	
	public void showPanel() {
		setVisible(false);
	}

}
