package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

public class LanguageTranslationReportDialog extends SDMDocJDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private JFrame parent;
	
	/**
	 * Conastructor
	 * @param parent
	 */
	
	public LanguageTranslationReportDialog(JFrame parent, String helpFile) {
		super(parent, ModalityType.MODELESS, helpFile /*ModalityType.DOCUMENT_MODAL*/);
		initialize();
	}

	public LanguageTranslationReportDialog(JFrame parent) {
		super(parent, ModalityType.MODELESS /*ModalityType.DOCUMENT_MODAL*/);
		initialize();
	}
	
	private void initialize() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setTitle(language.getText(StringConstants.LANGUAGE_TRANSLATION_REPORT));
		setPreferredSize(new SdmDimension(600, 400));
		setLocation (20, 150) ;
		create();
		pack();
		setVisible(true);
	}

	private void create() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		JPanel panel = new JPanel();
		panel.setMinimumSize(new Dimension(600, 320));
		this.getContentPane().add(panel);
		final JTextArea fileName = new JTextArea();
		fileName.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		final JButton viewReports = new JButtonLanguage(StringConstants.VIEW_TRANSLATION_REPORTS);
		viewReports.setEnabled(false);
		final JList<String> listbox = new JList<String>();
		
		addInformationMenu(this);
		setJMenuBar(myMenubar);

		//		fileName.setFont(new Font("Arial", Font.PLAIN, 14));
		//		panel.add(fileName);

		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;


		final JButton fileChooser;

		fileChooser = new JButtonLanguage(StringConstants.BROWSE);

		fileChooser.setMinimumSize(new SdmDimension(200, 25));
		fileChooser.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		fileChooser.setBounds(50, 60, 80, 30);
		fileChooser.setAlignmentX(Component.CENTER_ALIGNMENT);

		//        panel.add(fileChooser);



		// Create a file chooser that opens up as an Open dialog
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter("Docx Files","docx");
				JFileChooser chooser = new JFileChooser();
				chooser.setLocale(SDMDocLanguageSpecific.getInstance().getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation (0, 200) ;

				String savedValue = fileName.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();

					String path = sf.getAbsolutePath();
					fileName.setText(path);
					
					if (listbox.getSelectedIndex() != -1)
						viewReports.setEnabled(true);
				}
				else {
					fileName.setText(savedValue);
					if (savedValue.length() == 0) {}
					//            	  genDocButton.setEnabled(false);
				}
			}
		});
		
		

		List<String> al = SDMDocLanguageSpecific.getKnownDisplayNames();
		String[] listData = new String[al.size()];
		listData = al.toArray(listData);
		listbox.setListData(listData);


		// Create a new listbox control
		
		listbox.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		listbox.setName(language.getText(StringConstants.SELECT_LANGUAGE));
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setViewportView(listbox);
		scrollPane.setMinimumSize(new SdmDimension(350, 250));
		
		listbox.addListSelectionListener( new ListSelectionListener() {
		    public void valueChanged(ListSelectionEvent e){
			int sel = listbox.getSelectedIndex();
	        if( sel != -1 ){
//			    if (fileName.getText().length() > 0)
			    	viewReports.setEnabled(true);
			}
		    }
		} );


		

		viewReports.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
				int loc = 200;
				
				for (String s : listbox.getSelectedValuesList()) {
					
					String language = SDMDocLanguageSpecific.getLanguageForDisplayName(s);
					
					List<String> missing = SDMDocLanguageSpecific.missingTranslationReport.get(language);
					int numPages = Math.max(1, missing.size() / 10);
					numPages = Math.min(numPages, 5);
					int height = 200 * numPages;
					
					MessageFrame mf = new MessageFrame(StringConstants.LANGUAGE_TRANSLATION_REPORT, loc,loc,700,height);
					mf.addMsg("\n"+StringConstants.LANGUAGE_TRANSLATION_REPORT+": "+s+"\n");
					mf.showPanel();
					
					if (missing.size() > 0) {
						mf.addMsg("\nThere are "+missing.size()+" phrases that need to be translated\n\n");
						mf.addMsgs(missing);
					} else {
						mf.addMsg("All phrases translated");
					}
					mf.viewTop();
					
					loc += 20;
				}
			}
		});

		int row = 0;

		JLabel label = new JLabelLanguage(StringConstants.LANGUAGE_TRANSLATION_REPORT);
		label.setFont(language.getFont(Font.BOLD, SDMDocFont.LARGE));
		
		JLabel label2 = new JLabelLanguage(StringConstants.SELECT_ONE_OR_MORE);
		label2.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));



		c.insets = new Insets(0,0,0,0);  //top padding
		c.fill = GridBagConstraints.VERTICAL;
		c.weightx = 0.0;
		c.weighty = 0.0;


		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label,c);
		
		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(label2,c);
		
		c.gridwidth = 3;
		c.gridx = 0;
		c.gridy =row++;
//		panel.add(fileName,c);
		fileName.setMinimumSize(new SdmDimension(500, 25));


		// fileChooser

		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
//		panel.add(fileChooser,c);

		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(scrollPane,c);

		c.weightx = 0.0;
		c.weighty = 0.0;
		c.gridx = 1;
		c.gridy = row++;
		panel.add(viewReports,c);
		viewReports.setMinimumSize(new SdmDimension(200, 25));
		viewReports.setFont(language.getFont(Font.PLAIN, 14));
	}
	
	/**
	 * Hide Panel
	 */
	
	public void hidePanel() {
		setVisible(false);
	}
	
	/**
	 * Show panel
	 */
	
	public void showPanel() {
		setVisible(false);
	}

}
