package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.InputVerifier;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;

public class LoopAnalysisPanel extends SDMOptionsPanel  {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	private static final int TEXT_FIELD_SIZE = 8;
	private static final String DEFAULT_LOOP_SIZE = "30";

	// in support of detailed vs summary loop reporting

	JPanel detailedReportsPanel = new JPanel();

	private JRadioButtonLanguage summaryReportRB = new JRadioButtonLanguage(StringConstants.SUMMARY_REPORTS);
	private JRadioButtonLanguage detailedReportRB = new JRadioButtonLanguage(StringConstants.DETAILED_REPORTS);
	
	private final JCheckBoxLanguage writeLoopsToFileCB = new JCheckBoxLanguage(StringConstants.WRITE_LOOPS_TO_FILE);
	private final JCheckBoxLanguage readLoopsToFileCB = new JCheckBoxLanguage(StringConstants.READ_LOOPS_FROM_FILE);

	private final JTextField loopFileTF = new JTextField(TEXT_FIELD_SIZE);
	
	private Map<String, JRadioButtonLanguage> mapNameToRadioButton = new LinkedHashMap<String, JRadioButtonLanguage>();

	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	private Map<String, JTextField> mapNameToTextField = new LinkedHashMap<String, JTextField>();

	private final JLabel maxLoopSizeL = new JLabelLanguage(StringConstants.LOOP_SIZE_MAXIMUM);
	private final JTextField maxLoopSize = new JTextField(TEXT_FIELD_SIZE);

	private final JCheckBoxLanguage analyze = new JCheckBoxLanguage(StringConstants.ANALYZE_LOOPS);
	private final JCheckBoxLanguage analyzeIVV = new JCheckBoxLanguage(StringConstants.ANALYZE_LOOPS_WITH_IVV);

	private final JCheckBoxLanguage hyperlinkLoopsCB = new JCheckBoxLanguage(StringConstants.HYPERLINK_LOOP);

	private final JCheckBoxLanguage variableRelationships = new JCheckBoxLanguage(StringConstants.VARIABLE_RELATIONSHIPS);
	private final JCheckBoxLanguage variableLoops = new JCheckBoxLanguage(StringConstants.VARIABLE_LOOPS);

	private final JCheckBoxLanguage limitVRLoopsCB = new JCheckBoxLanguage(StringConstants.LIMIT_VR_LOOPS);
	private final JCheckBoxLanguage limitVLoopsCB = new JCheckBoxLanguage(StringConstants.LIMIT_V_LOOPS);
	private final JTextField limitVRLoopsTF = new JTextField(TEXT_FIELD_SIZE);
	private final JTextField limitVLoopsTF = new JTextField(TEXT_FIELD_SIZE);
	private final JTextField limitSearchTimeTF = new JTextField(TEXT_FIELD_SIZE);

	private final JLabel limitVRLoopsL = new JLabelLanguage(StringConstants.LIMIT_VR_LOOPS);
	private final JLabel limitVLoopsL = new JLabelLanguage(StringConstants.LIMIT_V_LOOPS);

	private final JLabel limitSearchTimeL = new JLabelLanguage(StringConstants.MAX_SEARCH_TIME_MINUTES);

	private final JCheckBoxLanguage override = new JCheckBoxLanguage(StringConstants.OVERRIDE_LOOP_COUNT);
	private final JTextField overrideTF = new JTextField(TEXT_FIELD_SIZE);
	
	private final JCheckBoxLanguage limitLongestLoopsCB = new JCheckBoxLanguage(StringConstants.LIMIT_LONGEST_LOOPS);
	private final JTextField limitLongestLoopsTF = new JTextField(TEXT_FIELD_SIZE);
	
	private final JCheckBoxLanguage limitLoopsPerFileCB = new JCheckBoxLanguage(StringConstants.LIMIT_LOOPS_PER_FILE);
	private final JTextField limitLoopsPerFileTF = new JTextField(TEXT_FIELD_SIZE);


	private String savedLoopSize = DEFAULT_LOOP_SIZE;

	private String savedMaxSearchTime = DEFAULT_LOOP_SIZE;
	private String savedMaxVRLimit = "10";
	private String savedMaxVLimit = "10";
	private String savedMaxLongestLimit = "10";
	private String savedLoopsPerFileLimit = "100000";

	
	private String defaultLoopsPerFileLimit = "1000000";
	//	 NumberFormat format = NumberFormat.getInstance();
	//	 NumberFormatter formatter = new NumberFormatter(format);
	//	 final JFormattedTextField field = new JFormattedTextField(formatter);


	public LoopAnalysisPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);

		create();
	}

	private void create() {
		
//		setPreferredSize(new Dimension(600, 250));
//		setMinimumSize(new Dimension(600, 250));

		maxLoopSizeL.setLabelFor(maxLoopSize);

		analyze.setEnabled(true);
		hyperlinkLoopsCB.setEnabled(true);
		limitLoopsPerFileCB.setEnabled(true);
		analyzeIVV.setEnabled(false);

		maxLoopSize.setEnabled(true);
		variableRelationships.setEnabled(true);
		variableLoops.setEnabled(true);
		override.setEnabled(false);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		maxLoopSizeL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		analyze.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		analyzeIVV.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		maxLoopSize.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		variableRelationships.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		variableLoops.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		hyperlinkLoopsCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		override.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		limitVLoopsCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitVRLoopsCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitLongestLoopsCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitLoopsPerFileCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitSearchTimeL.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitVRLoopsTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitLongestLoopsTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitLoopsPerFileTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitVLoopsTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		limitSearchTimeTF.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		summaryReportRB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		detailedReportRB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		writeLoopsToFileCB.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		analyze.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		hyperlinkLoopsCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		variableRelationships.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		variableLoops.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		limitVRLoopsCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		limitLongestLoopsCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		limitLoopsPerFileCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		override.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});
		
		limitVLoopsCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		summaryReportRB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		detailedReportRB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		writeLoopsToFileCB.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				setDependencies();
			}
		});

		detailedReportRB.setSelected(true);

		loadMap();

		setValues();

		setLayout(new GridBagLayout());

		GridBagConstraints c = new GridBagConstraints();

		int row = 0;
		c.fill = GridBagConstraints.NONE;

		c.anchor = GridBagConstraints.WEST;
		c.weightx = 1.0;
		c.weighty = .10;

		c.gridwidth = 1;
		c.gridy = row++;
		c.gridx = 0;
		add(analyze, c);
		
		c.insets = new Insets(0,20,0,0);

		c.gridwidth = 1;
		c.gridy = row++;
		c.gridx = 0;
		add(analyzeIVV, c);

		c.gridy = row++;
		add(limitSearchTimeL,c);
		
		c.gridx = 0;
		limitSearchTimeTF.setInputVerifier(new IntegerVerifier(
				"Maximum Search Time",1,Integer.MAX_VALUE));
		limitSearchTimeTF.setPreferredSize(new SdmDimension(75,25));
		limitSearchTimeTF.setMinimumSize(new SdmDimension(75,25));
		c.gridy = row++;
		add(limitSearchTimeTF, c);

		c.fill = GridBagConstraints.HORIZONTAL;
		


		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = 1;
		c.gridy = row++;
		c.gridx = 0;
		add(maxLoopSizeL, c);

		c.fill = GridBagConstraints.NONE;
		maxLoopSize.setPreferredSize(new SdmDimension(75,20));
		maxLoopSize.setMinimumSize(new SdmDimension(75,20));
		c.gridy = row++;
		c.gridx = 0;
		add(maxLoopSize, c);
		c.fill = GridBagConstraints.HORIZONTAL;

		c.gridy = row++;
		c.gridx = 0;
		add(writeLoopsToFileCB, c);

		//				c.gridy = row++;
		//				c.gridx = 0;
		//				add(readLoopsToFileCB, c);

		/*
		 * this is to configure loops processing for detailed or summary reporting
		 */

		ButtonGroup group = new ButtonGroup();
		group.add(detailedReportRB);
		group.add(summaryReportRB);

		c.gridy = row++;
		c.gridx = 0;
		add(summaryReportRB, c);

		c.gridy = row++;
		c.gridx = 0;
		add(detailedReportRB, c);

		// create a panel with title for detailed reports
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		detailedReportsPanel.setLayout(new GridBagLayout());
		
		String t = language.getText(StringConstants.DETAILED_REPORTS);

		detailedReportsPanel.setBorder(BorderFactory.createTitledBorder(null, 
				language.getText(StringConstants.DETAILED_REPORTS), TitledBorder.LEFT , 
				TitledBorder.TOP,  language.getFont(Font.BOLD,SDMDocFont.MEDIUM) , Color.black));

		int rowDR = 0;

		c.gridwidth = 1;
		c.gridy = rowDR++;
		c.gridx = 0;
		detailedReportsPanel.add(hyperlinkLoopsCB, c);
		c.gridx = 1;
		detailedReportsPanel.add(limitLoopsPerFileCB, c);
		c.gridx = 2;
		limitLoopsPerFileTF.setInputVerifier(new IntegerVerifier(
				"Limit Loops per File",1,1000000));
		detailedReportsPanel.add(limitLoopsPerFileTF, c);

		if (!InformationManagers.getInstance().isLimitLoopProcessing()) {
			c.gridy = rowDR++;
			c.gridx = 0;
			detailedReportsPanel.add(variableRelationships, c);
			c.gridx = 1;
			detailedReportsPanel.add(limitVRLoopsCB, c);
			c.gridx = 2;
			limitVRLoopsTF.setInputVerifier(new IntegerVerifier(
					"Limit Loops per Variable Pairs",1,1000));
			detailedReportsPanel.add(limitVRLoopsTF, c);


			c.gridy = rowDR++;
			c.gridx = 0;
			detailedReportsPanel.add(variableLoops, c);
			c.gridx = 1;
			detailedReportsPanel.add(limitVLoopsCB, c);
			c.gridx = 2;
			limitVLoopsTF.setInputVerifier(new IntegerVerifier(
					"Limit Loops per Variable",1,1000));
			detailedReportsPanel.add(limitVLoopsTF, c);

			c.gridy = rowDR++;
			c.gridx = 0;
			detailedReportsPanel.add(override, c);
			
//			c.gridx = 1;
//			detailedReportsPanel.add(limitLongestLoopsCB, c);
//			c.gridx = 2;
//			limitLongestLoopsTF.setInputVerifier(new IntegerVerifier(
//					"Limit Longest Loops Variable Relationship",1,1000000));
//			detailedReportsPanel.add(limitLongestLoopsTF, c);

		}

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 3;
		c.gridy = row;
		c.gridx = 0;
		add(detailedReportsPanel, c);


	}

	class GreaterThan2Verifier extends InputVerifier {
		public boolean verify(JComponent input) {
			JTextField tf = (JTextField) input;
			String maxSize = tf.getText();
			if (!Parser.isInteger(maxSize)) {
				displayError(maxSize);
				return false;
			}
			if (Integer.parseInt(maxSize) > 1) {
				return true;
			} else {
				displayError(maxSize);
				return false;
			}
		}

		public void displayError(String maxSize) {
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			JOptionPane.showMessageDialog(null,
					language.getText(StringConstants.MAXIMUM_LOOP_LENGTH_MUST_BE_AN_INTEGER)+" > 1",
					language.getText(StringConstants.INVALID_MAXIMUM_LOOP_LENGTH),
					JOptionPane.ERROR_MESSAGE);
		}
	}

	public void changeLanguage() {
		super.changeLanguage();

		analyze.setText(StringConstants.ANALYZE_LOOPS);
		hyperlinkLoopsCB.setText(StringConstants.HYPERLINK_LOOP);
		analyzeIVV.setText(StringConstants.ANALYZE_LOOPS_WITH_IVV);
		variableRelationships.setText(StringConstants.VARIABLE_RELATIONSHIPS);
		variableLoops.setText(StringConstants.VARIABLE_LOOPS);
		override.setText(StringConstants.OVERRIDE_LOOP_COUNT, 
				NumberFormatter.formatForIntComma(LoopManager.MAX_LOOPS_FOR_POSTPROCESS));
		limitVRLoopsCB.setText(StringConstants.LIMIT_VR_LOOPS);
		// new
		limitLongestLoopsCB.setText(StringConstants.LIMIT_LONGEST_LOOPS);
		limitLoopsPerFileCB.setText(StringConstants.LIMIT_LOOPS_PER_FILE);
		// end new
		limitVLoopsCB.setText(StringConstants.LIMIT_V_LOOPS);
		limitSearchTimeL.setText(StringConstants.MAX_SEARCH_TIME_MINUTES);
		maxLoopSizeL.setText(StringConstants.LOOP_SIZE_MAXIMUM);
		
		summaryReportRB.setText(StringConstants.SUMMARY_REPORTS);
		detailedReportRB.setText(StringConstants.DETAILED_REPORTS);
		writeLoopsToFileCB.setText(StringConstants.WRITE_LOOPS_TO_FILE);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		detailedReportsPanel.setBorder(BorderFactory.createTitledBorder(null, 
				language.getText(StringConstants.DETAILED_REPORTS), TitledBorder.LEFT , 
				TitledBorder.TOP,  language.getFont(Font.BOLD,SDMDocFont.MEDIUM) , Color.black));
	}

	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.LOOP_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
				cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
			} else if (mapNameToTextField.get(var) != null) {
				JTextField tf = mapNameToTextField.get(var);
				tf.setText(options.getOptionValue(var));
			} else if (mapNameToRadioButton.get(var) != null){
				JRadioButtonLanguage rb = mapNameToRadioButton.get(var);
				rb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
			} else {
				System.out.println("ERROR setting values "+var);;
			}
		}
		setDependencies();
	}

	private void setDependencies() {
		

		// mjb try this
		if (maxLoopSize.getText().length() > 0)
			savedLoopSize = maxLoopSize.getText();
		if (limitVRLoopsTF.getText().length() > 0)
			savedMaxVRLimit = limitVRLoopsTF.getText();
		if (limitVLoopsTF.getText().length() > 0)
			savedMaxVLimit = limitVLoopsTF.getText();
		// new
		if (limitLongestLoopsTF.getText().length() > 0)
			savedMaxLongestLimit = limitLongestLoopsTF.getText();
		if (limitLoopsPerFileTF.getText().length() > 0)
			savedLoopsPerFileLimit = limitLoopsPerFileTF.getText();
		// end new
		if (limitSearchTimeTF.getText().length() > 0)
			savedMaxSearchTime = limitSearchTimeTF.getText();


		if (analyze.isSelected()) {
			analyzeIVV.setEnabled(true);
			
			limitSearchTimeTF.setEnabled(true);
			limitSearchTimeTF.setText(savedMaxSearchTime);

//			maxLoopSize.setEnabled(true);
//			maxLoopSize.setInputVerifier(new GreaterThan2Verifier());
			
			maxLoopSize.setEnabled(true);
			maxLoopSize.setText(savedLoopSize);
			maxLoopSize.setInputVerifier(new GreaterThan2Verifier());
			maxLoopSizeL.setText(StringConstants.LOOP_SIZE_MAXIMUM);


			summaryReportRB.setEnabled(true);
			detailedReportRB.setEnabled(true);
			writeLoopsToFileCB.setEnabled(true);
			
			
			if (detailedReportRB.isSelected()) {
				hyperlinkLoopsCB.setEnabled(true);
				variableRelationships.setEnabled(true);
				variableLoops.setEnabled(true);
				override.setEnabled(true);
				limitLoopsPerFileCB.setEnabled(true);



				if (variableRelationships.isEnabled() && variableRelationships.isSelected()) {
					limitVRLoopsCB.setEnabled(true);
					limitVRLoopsTF.setEnabled(true);
					if (limitVRLoopsCB.isSelected()) {
						limitVRLoopsTF.setText(savedMaxVRLimit);
					} else {
						limitVRLoopsTF.setText("");
					}
				} else {
					limitVRLoopsCB.setEnabled(false);
					limitVRLoopsTF.setEnabled(false);
					limitVRLoopsTF.setText("");
				}

				if (variableLoops.isEnabled() && variableLoops.isSelected()) {
					limitVLoopsCB.setEnabled(true);
					limitVLoopsTF.setEnabled(true);
					if (limitVLoopsCB.isSelected())
						limitVLoopsTF.setText(savedMaxVLimit);
					else
						limitVLoopsTF.setText("");
				} else {
					limitVLoopsCB.setEnabled(false);
					limitVLoopsTF.setEnabled(false);
					limitVLoopsTF.setText("");
				}
				// new
				if (override.isEnabled() && override.isSelected()) {
					limitLongestLoopsCB.setEnabled(true);
					limitLongestLoopsTF.setEnabled(true);
					if (limitLongestLoopsCB.isSelected()) {
						limitLongestLoopsTF.setText(savedMaxLongestLimit);
						limitLongestLoopsTF.setEnabled(true);
					} else {
						limitLongestLoopsTF.setText("");
						limitLongestLoopsTF.setEnabled(false);
					}
				} else {
					limitLongestLoopsCB.setEnabled(false);
					limitLongestLoopsTF.setEnabled(false);
					limitLongestLoopsTF.setText("");
				}
				
				if (limitLoopsPerFileCB.isEnabled() ) { //&& limitLoopsPerFileCB.isSelected()) {
					limitLoopsPerFileCB.setEnabled(true);
					limitLoopsPerFileTF.setEnabled(true);
					if (limitLoopsPerFileCB.isSelected()) {
						limitLoopsPerFileTF.setText(savedLoopsPerFileLimit);
						limitLoopsPerFileTF.setEnabled(true);
					} else {
						limitLoopsPerFileTF.setText("");
						limitLoopsPerFileTF.setEnabled(false);
					}
				} else {
					limitLoopsPerFileCB.setEnabled(false);
					limitLoopsPerFileTF.setEnabled(false);
					limitLoopsPerFileTF.setText("");
				}
				// end new
			} else {
				limitVRLoopsCB.setEnabled(false);
				limitVRLoopsTF.setEnabled(false);
				limitVRLoopsTF.setText("");
				limitVLoopsCB.setEnabled(false);
				limitVLoopsTF.setEnabled(false);
				limitVLoopsTF.setText("");
				
				// new
				limitLongestLoopsCB.setEnabled(false);
				limitLongestLoopsTF.setEnabled(false);
				limitLongestLoopsTF.setText("");
				
				limitLoopsPerFileCB.setEnabled(false);
				limitLoopsPerFileTF.setEnabled(false);
				limitLoopsPerFileTF.setText("");
				
				//new end
				hyperlinkLoopsCB.setEnabled(false);
				variableRelationships.setEnabled(false);
				variableLoops.setEnabled(false);
				override.setEnabled(false);
			}

		} else {
			analyzeIVV.setEnabled(false);
			hyperlinkLoopsCB.setEnabled(false);
			//			unlimitedLoopSize.setEnabled(false);
			// only if there is a value
			if (maxLoopSize.getText().length() > 0)
				savedLoopSize = maxLoopSize.getText();
			maxLoopSize.setInputVerifier(null);
			maxLoopSize.setText("");
			maxLoopSize.setEnabled(false);
			variableRelationships.setEnabled(false);
			variableLoops.setEnabled(false);
			override.setEnabled(false);
			//			maxLoopSizeL.setText("");
			maxLoopSizeL.setText(StringConstants.LOOP_SIZE_MAXIMUM);

			limitVRLoopsTF.setText("");
			limitVRLoopsCB.setEnabled(false);
			limitVRLoopsTF.setEnabled(false);

			limitVLoopsTF.setText("");
			limitVLoopsCB.setEnabled(false);
			limitVLoopsTF.setEnabled(false);
			
			limitLoopsPerFileTF.setText("");
			limitLoopsPerFileCB.setEnabled(false);
			limitLoopsPerFileTF.setEnabled(false);

			limitSearchTimeTF.setText("");
			limitSearchTimeTF.setEnabled(false);
			
			summaryReportRB.setEnabled(false);
			detailedReportRB.setEnabled(false);
			writeLoopsToFileCB.setEnabled(false);
		}

	}

	@Override
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.LOOP_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)
				cb.setSelected(value);
		}
	}

	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.LOOP_OPTIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb != null)  {
				if (cb.isSelected()) {
					options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
				} else {
					options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
				}
			} else if (mapNameToTextField.get(var) != null) {
				JTextField tf = mapNameToTextField.get(var);
				if (tf.getText().length() > 0) {
					options.setValue(var, options.getOptionType(var), tf.getText());
				} else {
					if (var.equals(StringConstants.MAX_LIMIT_LOOPS_PER_FILE)) {
						options.setValue(var, options.getOptionType(var), defaultLoopsPerFileLimit);
					} else {
						options.setValue(var, options.getOptionType(var), DEFAULT_LOOP_SIZE);
					}
				}
			} else {
				JRadioButton rb = mapNameToRadioButton.get(var);
				if (rb.isSelected()) {
					options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
				} else {
					options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
				}
			}
		}
		return true;
	}

	private void loadMap() {

		mapNameToTextField.put(StringConstants.LOOP_SIZE_MAXIMUM, maxLoopSize);

		mapNameToTextField.put(StringConstants.MAX_LIMIT_VR_LOOPS, limitVRLoopsTF);
		mapNameToTextField.put(StringConstants.MAX_LIMIT_V_LOOPS, limitVLoopsTF);
		mapNameToTextField.put(StringConstants.MAX_SEARCH_TIME_MINUTES, limitSearchTimeTF);
		mapNameToTextField.put(StringConstants.MAX_LIMIT_LONGEST_LOOPS, limitLongestLoopsTF);
		mapNameToTextField.put(StringConstants.MAX_LIMIT_LOOPS_PER_FILE, limitLoopsPerFileTF);

		//		mapNameToCheckBox.put(StringConstants.LOOP_SIZE_AUTO_DISCOVER, unlimitedLoopSize);
		mapNameToCheckBox.put(StringConstants.ANALYZE_LOOPS, analyze);
		mapNameToCheckBox.put(StringConstants.HYPERLINK_LOOP, hyperlinkLoopsCB);
		mapNameToCheckBox.put(StringConstants.ANALYZE_LOOPS_WITH_IVV, analyzeIVV);
		mapNameToCheckBox.put(StringConstants.VARIABLE_RELATIONSHIPS, variableRelationships);
		mapNameToCheckBox.put(StringConstants.VARIABLE_LOOPS, variableLoops);
		mapNameToCheckBox.put(StringConstants.OVERRIDE_LOOP_COUNT, override);

		mapNameToCheckBox.put(StringConstants.LIMIT_VR_LOOPS, limitVRLoopsCB);
		mapNameToCheckBox.put(StringConstants.LIMIT_V_LOOPS, limitVLoopsCB);
		
		mapNameToCheckBox.put(StringConstants.LIMIT_LONGEST_LOOPS, limitLongestLoopsCB);
		mapNameToCheckBox.put(StringConstants.LIMIT_LOOPS_PER_FILE, limitLoopsPerFileCB);
		
		mapNameToCheckBox.put(StringConstants.WRITE_LOOPS_TO_FILE, writeLoopsToFileCB);
		mapNameToRadioButton.put(StringConstants.SUMMARY_REPORTS, summaryReportRB);
		mapNameToRadioButton.put(StringConstants.DETAILED_REPORTS, detailedReportRB);
		

	}
}
