package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.Arrays;
import java.util.Comparator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.BoxLayout;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

public class LoopLengthAnalysisToolAvailableResultsPanel extends JPanel {

	//Create the combo box, select item at index 4.
	//Indices start at 0, so 4 specifies the pig.
	JComboBox<String> availableResultsList;

	String mdlFile = "";
	
	Pattern p1;
	Pattern p2;
	Pattern p3;
	
	JLabel titleLabel;
	String title;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public LoopLengthAnalysisToolAvailableResultsPanel(String title) {
		
		this.title = title;
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		availableResultsList = new JComboBox<String>();
		
		availableResultsList.setSize(350, 25);
		availableResultsList.setPreferredSize(new SdmDimension(350, 25));
		availableResultsList.setBackground(Color.WHITE);
		availableResultsList.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
		
		setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		
		titleLabel = new JLabelLanguage(title);
		titleLabel.setFont(language.getFont(Font.BOLD, SDMDocFont.MEDIUM));
		add(titleLabel);
		titleLabel.setAlignmentX(CENTER_ALIGNMENT);
		add(availableResultsList);
		
		setEnabled(false);
		
		String loopCount = "^\\(\\d+\\)$";
		p1 = Pattern.compile(loopCount);
		
		String timeStep = "^\\(\\d{4}\\.\\d{2}\\.\\d{2}\\. \\d{2}.\\d{2}.\\d{2}\\)$";
		p2 = Pattern.compile(timeStep);
		
		String loopCountTimeStep = "^\\(\\d+\\)\\(\\d{4}\\.\\d{2}.\\d{2}\\. \\d{2}\\.\\d{2}\\.\\d{2}\\)$";
		p3 = Pattern.compile(loopCountTimeStep);
	
//		availableResultsList.addActionListener(new ActionListener() {
//			@Override
//			public void actionPerformed(ActionEvent event) {
//				setMdlSpecific(mdlFile);
//			}
//		});
		
	}
	
	public void changeLanguage() {

		titleLabel.setFont(SDMDocLanguageSpecific.getInstance().getFont(Font.PLAIN, 14));
		titleLabel.setText(title);
	}
	
	public String getSelected() {
		return (String) availableResultsList.getSelectedItem();
	}
	
	public void setMdlSpecific(String mdlFile) {
		
		if (mdlFile == null || mdlFile.trim().length() == 0 )
			return;

		this.mdlFile = mdlFile;
		availableResultsList.removeAllItems();
		File mdl = new File(mdlFile);
		FileOrganizer fo = new FileOrganizer(false, mdl.getParent(), mdlFile);
		String model = fo.getModelname();
		// need to check the directory associated with this model
		File parentt = mdl.getParentFile();
		File parent = null;
		try {
			parent = new File(parentt.getCanonicalPath()+"/"+model+"_SDMDoc");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return;
		}
		int addedCount = 0;
		String[] dirList = sortByModTime(parent, parent.list());
		for (String s : dirList) {
			System.out.println("Checking "+parent+"/"+s);
			if (new File(parent+"/"+s).isDirectory()) {
				if ((s.equals(model+ "_DOC_GRAPH_") || 
						matchesLoopTimePatterns(model, s)) &&
						(hasPolarity(getArrowPolarityFileName(new File(parent+"/"+s)))||
								hasPolarity(getArrowPolarityFileNameWithoutPolarity(new File(parent+"/"+s))))) {
					availableResultsList.addItem(s);
					addedCount++;
				}
		}
	}
		if (addedCount > 0)
			availableResultsList.setEnabled(true);
	}
	
	private boolean hasPolarity(String file) {
		return new File(file).exists();
	}
	
	private String getArrowPolarityFileName(File docGraph) {
		
		return docGraph.getAbsolutePath()+"/structure/arrowPolarityFromEquations.tab";
	}
	
	private String getArrowPolarityFileNameWithoutPolarity(File docGraph) {
		
		return docGraph.getAbsolutePath()+"/arrowPolarity.tab";
	}

	
	public String[] sortByModTime(final File parent, String[] dirs) {
		
		if (dirs == null)
			return new String[0];
		
		Arrays.sort(dirs, new Comparator<String>() {
		    @Override
		    public int compare(String o1, String o2) {
		    	File f1 = new File(parent.getPath()+"/"+o1);
		    	File f2 = new File(parent.getPath()+"/"+o2);
		    	long diff = createdOrLastModified(f1.getPath()) - createdOrLastModified(f2.getPath());
		        return diff == 0l ? 0 : diff > 0l ? -1 : 1;
		    }
		});
		return dirs;
	}
	
	public long createdOrLastModified(String s) {
		BasicFileAttributes attr;
		try {
			Path file = new File(s).toPath();
			attr = Files.readAttributes(file, BasicFileAttributes.class);
			return Math.max(attr.lastModifiedTime().toMillis(), attr.creationTime().toMillis());
		} catch (IOException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			return 0l;
		}
	}
	
	private boolean matchesLoopTimePatterns(String modelName, String aDir) {
		
		// world2(30)(2017.05.02. 14.49.38)_DOC_GRAPH_
		
		String remain = aDir.replace(modelName,"").replace("_DOC_GRAPH_", "");
		
		Matcher m = p1.matcher(remain);
		if (m.find())
			return true;
		
		m = p2.matcher(remain);
		if (m.find())
			return true;

		m = p3.matcher(remain);
		if (m.find())
			return true;
		
		return false;
	}

}

