package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class LoopLengthAnalysisToolPanel extends JPanel {
	
	static Logger log = Logger.getLogger(LoopLengthAnalysisToolPanel.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final int TEXT_FIELD_SIZE = 60;
	private static final int NUMBER_FIELD_SIZE = 15;
	private static String FC_DIR_FILE = "data/ChooserDir.txt";

	private static final String CHECK_DEFAULT = "Check";

	private JLabel modelFileLabel = new JLabelLanguage(
			StringConstants.VENSIM_MODEL_FILE_NAME);
	
	private JLabel startingLengthLabel = new JLabelLanguage(
			StringConstants.STARTING_LENGTH);
	private JLabel endingLengthLabel = new JLabelLanguage(
			StringConstants.ENDING_LENGTH);

	private JTextField modelFileText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField startingLengthText = new JTextField(NUMBER_FIELD_SIZE);
	private JTextField endingLengthText = new JTextField(NUMBER_FIELD_SIZE);

	private JButtonLanguage modelFileButton = new JButtonLanguage(StringConstants.BROWSE);

	private JButton runAnalysisButton; //  = new JButton(StringConstants.MAKE_COPIES);
	LoopDiscoverWorker ldw = null;

	private JDialog parent;
	private String title;
	
	MessageFrame mf;
	
	// LoopLengthAnalysisToolAvailableResultsPanel(UserSelectedOptions uso, String title)
	private LoopLengthAnalysisToolAvailableResultsPanel available = new 
			LoopLengthAnalysisToolAvailableResultsPanel(StringConstants.AVAILABLE_FOR_REUSE);

	public LoopLengthAnalysisToolPanel(JDialog parent) {

		this.parent = parent;
		this.title = StringConstants.LOOP_LENGTH_ANALYSIS;

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));
		
		runAnalysisButton = new JButtonLanguage(StringConstants.RUN_LOOP_LENGTH_ANALYSIS);
		modelFileLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		startingLengthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		endingLengthLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		modelFileText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		startingLengthText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		endingLengthText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		modelFileButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		runAnalysisButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		createLayout();
		createListeners();
		setDependencies();

	}
	
	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));
		
		modelFileLabel.setText(StringConstants.VENSIM_MODEL_FILE_NAME);
		startingLengthLabel.setText(StringConstants.STARTING_LENGTH);
		endingLengthLabel.setText(StringConstants.ENDING_LENGTH);
		runAnalysisButton.setText(language.getText(StringConstants.RUN_LOOP_LENGTH_ANALYSIS));
	}
	
	public void removeMessageFrame() {
		if (ldw != null) {
			ldw.cancel(true);
			ldw = null;
//			if (mf != null) {
//				mf.disposeMessageFrame();
//				mf = null;
//			}
		}
	}

	private void createLayout() {
		setLayout(new GridBagLayout());
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		modelFileLabel.setFont(language.getFont(Font.BOLD, 12));
//		startingLengthLabel.setFont(language.getFont(Font.BOLD, 12));
//		endingLengthLabel.setFont(language.getFont(Font.BOLD, 12));
		
		startingLengthText.setHorizontalAlignment(JTextField.CENTER);
		endingLengthText.setHorizontalAlignment(JTextField.CENTER);
		
		runAnalysisButton.setEnabled(true);

		int row = -1;

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;

		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(modelFileLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(modelFileText, c);
		c.gridx = 1;
		c.weightx = 0.0;
		add(modelFileButton, c);
		
		// available
		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(available, c);
//		c.fill = GridBagConstraints.HORIZONTAL;
//		c.gridy = ++row;
//		add(modelFileText, c);

		c.weightx = 1.0;
		int labelRow = row;
		c.gridy = ++row;
		
//		c.gridx = 0;
//		c.fill = GridBagConstraints.CENTER;
//		add(startingLengthLabel, c);
//		c.fill = GridBagConstraints.NONE;
//		c.gridy = ++row;
//		add(startingLengthText, c);
		
//		row = labelRow;
		c.gridy = ++row; // ++row
		c.gridx = 0; // 0
		c.fill = GridBagConstraints.CENTER;
		add(endingLengthLabel, c);
		c.fill = GridBagConstraints.NONE;
		c.gridy = ++row;
		add(endingLengthText, c);

		c.fill = GridBagConstraints.NONE;

		c.gridwidth = 1;

		c.fill = GridBagConstraints.CENTER;

			c.gridy = ++row;
			c.gridx = 0;
			add(runAnalysisButton, c);

	}
	
	private void addIfNotThere(List<String> list, String s) {
		if (!list.contains(s))
			list.add(s);
	}

	private void createListeners() {
		final String c1 = CHECK_DEFAULT;
		createBrowserListener(modelFileButton, modelFileText, "mdl", c1);
		

		runAnalysisButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				final SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

				boolean ok = true;
				List<String> errorMessages = new ArrayList<String>();
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				int startingLength = -1;
				int endingLength = -1;
				
				try {
					startingLength = 2; // Integer.valueOf( startingLengthText.getText());
				} catch (NumberFormatException e1) {
					addIfNotThere(errorMessages, language.getText(StringConstants.INVALID_STARTING_LENGTH));
					ok = false;
				}
				
				try {
					endingLength = Integer.valueOf( endingLengthText.getText());
				} catch (NumberFormatException e1) {
					addIfNotThere(errorMessages, language.getText(StringConstants.INVALID_ENDING_LENGTH));
					ok = false;
				}
				
				if (startingLength < 1) {
					addIfNotThere(errorMessages, language.getText(StringConstants.INVALID_STARTING_LENGTH));
					ok = false;
				}
				
				if (endingLength >= 500) {
					addIfNotThere(errorMessages, language.getText(StringConstants.INVALID_ENDING_LENGTH));
					ok = false;
				} 
				
				File mdl = 	new File(modelFileText.getText());
				if (!mdl.exists()) {
					ok = false;
					addIfNotThere(errorMessages, language.getText(StringConstants.SPECIFIED_SYSTEM_DYNAMICS_MODEL_NOT_FOUND));
				} else {
					File polarity = new File(getArrowPolarityFileName());
					if (!polarity.exists()) {
						ok = false;
						addIfNotThere(errorMessages, language.getText(StringConstants.ASSOCIATED_ARROW_POLARITY_FILE_NOT_FOUND));
					}
				}
				
				
				if (!ok) {
					JOptionPane
					.showMessageDialog(
							null,
							getErrorMessages(errorMessages),
							language.getText(StringConstants.ERROR),        // title
							JOptionPane.ERROR_MESSAGE);
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				
				mf = new MessageFrame("LoopLengthAnalysis");
				
				mf.addTimeStamp();
				
//				final LoopDiscovery ld = new LoopDiscovery(
//						getArrowPolarityFileName(), startingLengthText.getText(), endingLengthText.getText(), mf);
				
				try {
					mf.setBlipMode(true);
					final MessageProcessor msgProc = mf;
					
					ldw = new LoopDiscoverWorker(getArrowPolarityFileName(), 
							startingLengthText.getText(), endingLengthText.getText(), msgProc);
					ldw.execute();
//					SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
//						@Override
//						protected Void doInBackground() throws Exception {
//							try {
//								final LoopDiscovery ld = new LoopDiscovery(
//										getArrowPolarityFileName(), startingLengthText.getText(), endingLengthText.getText(), mf);
//								ld.initialize();
//								ld.doit();
//								setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
//								JOptionPane
//										.showMessageDialog(
//												null,
//												language.getText(StringConstants.PROCESSING_COMPLETE),
//												language.getText(StringConstants.PROCESSING_COMPLETE),
//												JOptionPane.INFORMATION_MESSAGE);
//							} catch (Exception e) {
//								String errMsg = e.toString();
//								setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
//								
//								log.log(Level.FATAL, "Internal Error Detected", e);
//								JOptionPane.showMessageDialog(null, errMsg,
//										language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
//										JOptionPane.ERROR_MESSAGE);
//								e.printStackTrace();
//								// TODO Auto-generated catch block
//								e.printStackTrace();
//							}
//								
//								return null;
//
//						}
//						
//
//					};
//
//
//					worker.execute();
					
				} catch (Exception e) {

					String errMsg = e.toString();
					String myMsg = "\n\n"+
							language.getText(StringConstants.JAVA_ERROR_MESSAGES)+"\n\n";
					log.log(Level.FATAL, "Internal Error Detected", e);
					mf.addMessage(errMsg + "\n");
					mf.setBlipMode(false);
					JOptionPane.showMessageDialog(null, errMsg + myMsg,
							language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
							JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
					mf.setBlipMode(false);
					
				}
//				try {
//					ld.initialize();
//					ld.doit();
//					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
//					JOptionPane
//							.showMessageDialog(
//									null,
//									language.getText(StringConstants.PROCESSING_COMPLETE),
//									language.getText(StringConstants.PROCESSING_COMPLETE),
//									JOptionPane.INFORMATION_MESSAGE);
//				} catch (Exception e) {
//					String errMsg = e.toString();
//					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
//					
//					log.log(Level.FATAL, "Internal Error Detected", e);
//					JOptionPane.showMessageDialog(null, errMsg,
//							language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
//							JOptionPane.ERROR_MESSAGE);
//					e.printStackTrace();
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
			}
		});

	}
	
	private String getErrorMessages(List<String> errorMessages) {
		StringBuilder sb = new StringBuilder();
		
		for (int i = 0; i < errorMessages.size(); i++) {
			if (i > 0)
				sb.append("\n");
			sb.append(errorMessages.get(i));
		}
		
		return sb.toString();
		
	}
	
	private String getArrowPolarityFileName() {
		
		File mdl = new File(modelFileText.getText());
		File parent = mdl.getParentFile();
		String modelname = mdl.getName().replace(".mdl", "").replace(".MDL", "").
				replace(".xmile", "").replace(".XMILE", "");
		String selectedOutputDirectory = available.getSelected();
//		String fn = parent.getAbsolutePath()+"/"+modelname+"_SDMDoc/"+modelname+"_DOC_GRAPH_/structure/arrowPolarityFromEquations.tab";
		String fn = parent.getAbsolutePath()+"/"+modelname+"_SDMDoc/"+selectedOutputDirectory+"/structure/arrowPolarityFromEquations.tab";
		File withPolarity = new File(fn);
		if (withPolarity.exists()) {
			return fn;
		}
		fn = parent.getAbsolutePath()+"/"+modelname+"_SDMDoc/"+selectedOutputDirectory+"/arrowPolarity.tab";
//		File noPolarity = new File(fn);
//		if (noPolarity.exists())
		return fn;
	}

	private void createBrowserListener(/*final*/ JButtonLanguage button,
			/*final*/ JTextField textField, /*final*/ String fileType,
			/*final*/  String checkDefault) {
		final JButtonLanguage fbutton = button;
		final String ffileType = fileType;
		final JTextField ftextField = textField;
		final String fcheckDefault = new String(checkDefault);
		fbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter(ffileType
						.toUpperCase() + " Files", ffileType.toLowerCase());
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(0, 200);

				String savedValue = ftextField.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					 saveFCDir(dir);
					String path = sf.getAbsolutePath();
					ftextField.setText(path);
					available.setMdlSpecific(path);
				} else {
					ftextField.setText(savedValue);
					if (savedValue.length() == 0) {
					}
				}
				setDependencies();
			}
		});
	}


	private void setDependencies() {
//		runAnalysisButton.setEnabled(true);
		
			if (modelFileText.getText().length() > 0
					&& startingLengthText.getText().length() > 0
					&& endingLengthText.getText().length() > 0) {
				runAnalysisButton.setEnabled(true);
				

			}		
	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void setValues(boolean value) {
	}
	
	public MessageProcessor getMessageFrame() {
		return mf;
	}

}
