package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.awt.Dimension;
import java.awt.Font;
import java.io.BufferedWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;

public class MessageFrame extends JFrame implements MessageProcessor {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final String BLIP = "#";
	private static final long BLIP_PAUSE = 3000L;
	
	private static final int MAX_WIDTH = 2000;
	private static final int MAX_HEIGHT = 1000;
	
	private JFrame masterFrame;
//	private JEditorPane editorPane;
	private JTextPane editorPane;
	
	private boolean blipMode = false;
	private boolean blipModeSave = false;
	private int blipStart = 0;
	private Thread blipThread;
	
	private int numBlip = 0;
	private int numBlipNL = 0;
	
	private String logfileTag = "NoTagProvided";
	private BufferedWriter bw = null;
	
	public MessageFrame(String logfileTag) {
		this.logfileTag = logfileTag;
		System.out.println("logfile tag: "+this.logfileTag);
		create(20, 20, 700, 900);
		createLog();
		showPanel();
	}
	
	public MessageFrame(String logfileTag, int xLoc, int yLoc) {
		
		this.logfileTag = logfileTag;
		System.out.println("logfile tag: "+this.logfileTag);
		create(xLoc, yLoc, 700, 900);
		createLog();
		hidePanel();
	}
	
	public MessageFrame(String logfileTag, int xLoc, int yLoc, int width, int height) {
		
		this.logfileTag = logfileTag;
		System.out.println("logfile tag: "+this.logfileTag);
		create(xLoc, yLoc, width, height);
		createLog();
		hidePanel();
	}
	
	public void create(int xLoc, int yLoc) {
		
	}
	
	public void createLog() {
		bw = Utilities.openFileForWriting("log/msgLog_"+logfileTag);
	}
	
	public void closeLog() {
		try {
			if (bw != null)
				bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void setCaretPosition(int pos) {
		editorPane.setCaretPosition(pos);
	}
	
	public void viewTop() {
		setCaretPosition(0);
	}
	
	public void create(int xLoc, int yLoc, int width, int height) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		//Create and set up the window.
		final JFrame frame = new JFrame(language.getText(StringConstants.SDM_DOC_TOOL)+"/Java "+
				language.getText(StringConstants.VERSION)+StandaloneGUI.version+" "+language.getText(StringConstants.SYSTEM_MESSAGES));
		masterFrame = frame;
		
		//		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setPreferredSize(new SdmDimension(Math.min(width, MAX_WIDTH), Math.min(MAX_HEIGHT, height)));
		frame.setLocation (xLoc, yLoc) ;

		
		editorPane = new JTextPane();
		
		DefaultCaret caret = (DefaultCaret)editorPane.getCaret();
		caret.setUpdatePolicy(DefaultCaret.ALWAYS_UPDATE);
		
		editorPane.setEditable(false);
		JScrollPane jsp = new JScrollPane(editorPane);
		jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		frame.getContentPane().add(jsp, "Center");
		
		editorPane.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		

		frame.pack();
	}
	
	public void setFontSize(int size) {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		editorPane.setFont(language.getFont(Font.PLAIN, size));
	}
	
	public void addMsgs(List<String> list) {
		for (String text: list) {
			addMsg(text+"\n");
		}
	}
	
	
	public void addMsg(String text) {
		try {
			
		      Document doc = editorPane.getDocument();
		      doc.insertString(doc.getLength(), text, null);
		      if (!text.equals(BLIP) && !text.equals(BLIP+"\n")) {
		    	  System.out.print(text);
		      }
		   } catch(BadLocationException exc) {
		      exc.printStackTrace();
		   }
		if (bw != null) {
			try {
				if (!text.equals(BLIP) && !text.equals(BLIP+"\n")) {
					bw.append(text);
					bw.flush();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	

	}
	
	public void blip() {
		if (blipMode) {
			numBlip++;
			if (numBlip % 30 == 0) {
				addMsg(BLIP+"\n");
				numBlipNL++;
			} else {
				addMsg(BLIP);
			}
		}
	}
	
	public void addTimeStamp() {
		String timeStamp = new Date().toString();
		addMsg("\n#####  "+timeStamp+"  #####");
		
	}
	
	public void addMessage(String text) {

		blipModeSave = blipMode;
		// if in blip mode, do a clear first
		if (blipMode) {
			blipClear();
		}
		
		String pattern = "HH:mm:ss";
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat(pattern);

		String date = simpleDateFormat.format(new Date());


		addMsg("\n"+date+" | "+text);
		blipMode = blipModeSave;
		// need to define the new blipStart
		if (blipMode) {
			Document doc = editorPane.getDocument();
			blipStart = doc.getLength();
		}
		
		pause(20L);
		if (!masterFrame.isVisible())
			showPanel();
		pause(20L);

	}
	
	public void addMessageNP(String text) {
		
		addMsg(text);
		
//		pause(250L);
//		editorPane.revalidate();
//	      editorPane.repaint();
	}
	
	public void addMessageNPNoView(String text) {
		addMsg(text);
		this.setVisible(false);
	}
	
	public void pause(long t) {
		try {
		    Thread.sleep(t);                 //1000 milliseconds is one second.
		} catch(InterruptedException ex) {
		    Thread.currentThread().interrupt();
		}
	}
	
	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		masterFrame.setTitle(language.getText(StringConstants.SYSTEM_MESSAGES));
	}
	
	public void disposeMessageFrame() {
//		this.dispose();
		hidePanel();
	}
	
	public void showPanel() {
		masterFrame.setVisible(true);
	}
	
	public void hidePanel() {
		masterFrame.setVisible(false);
	}
	public boolean isBlipMode() {
		return blipMode;
	}
	public void setBlipMode(boolean blipMode) {
		
		if (!blipMode && !this.blipMode)
			return;
		
		this.blipMode = blipMode;
		
		Document doc = editorPane.getDocument();
		if (blipMode) {
			
			if (blipThread != null)
				blipThread.interrupt();
			
			blipStart = doc.getLength();
			
			numBlip = 0;
			numBlipNL = 0;
			
			blipThread = new Thread(new Runnable() {
			
			        public void run() {
			            try {
			                while (!Thread.currentThread().isInterrupted()) {
			                    Thread.sleep(BLIP_PAUSE);
			                    blip();
			                }
			            } catch (InterruptedException e) {
//			                Thread.currentThread().interrupt();
			            }
			        }
			    });
			    blipThread.start();
			
		} else {
			if (blipThread != null)
				blipThread.interrupt();
			blipClear();
			blipThread = null;
		}
	}
	
	public void blipClear() {
		Document doc = editorPane.getDocument();
		int len = doc.getLength() - blipStart - numBlipNL;
		numBlip = 0;
		numBlipNL = 0;
		try {
			doc.remove(blipStart, len);
		} catch (BadLocationException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
		}
	}

	@Override
	public void endTask() {
		
		blipModeSave = blipMode;
		// if in blip mode, do a clear first
		if (blipMode) {
			blipClear();
		}
		addMsg("\n");
		blipMode = blipModeSave;
		// need to define the new blipStart
		if (blipMode) {
			Document doc = editorPane.getDocument();
			blipStart = doc.getLength();
		}
		
		if (!masterFrame.isVisible())
			showPanel();
		pause(30L);
		
	}


}
