package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.ModelCopier;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class ModelCopierPanel extends JPanel {
	
	static Logger log = Logger.getLogger(ModelCopierPanel.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final int TEXT_FIELD_SIZE = 60;
	private static String FC_DIR_FILE = "data/ChooserDir.txt";

	private static final String GENERATE_DOCX = "GenerateDocx";
	private static final String GENERATE_VENSIM = "GenerateVensim";

	private static final String CHECK_DEFAULT = "Check";
	private static final String DO_NOT_CHECK_DEFAULT = "DoNotCheck";

	private JLabel originalModelFileLabel = new JLabelLanguage(
			StringConstants.ORIGINAL_VENSIM_MODEL_FILE_NAME);
	private JLabel originalExcelFileLabel = new JLabelLanguage(
			StringConstants.ORIGINAL_EXCEL_DATA_FILE_NAME);
	private JLabel numberOfCopiesLabel = new JLabelLanguage(
			StringConstants.NUMBER_OF_COPIES);

	private JTextField originalModelFileText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField originalExcelFileText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField numberOfCopiesText = new JTextField(TEXT_FIELD_SIZE);

	private JButtonLanguage originalModelFileButton = new JButtonLanguage(StringConstants.BROWSE);
	private JButtonLanguage originalExcelFileButton = new JButtonLanguage(StringConstants.BROWSE);

	private JButton makeCopiesButton; //  = new JButton(StringConstants.MAKE_COPIES);

	private JDialog parent;
	private String title;

	public ModelCopierPanel(JDialog parent) {

		this.parent = parent;
		this.title = StringConstants.MODEL_COPIER;

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		originalModelFileLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		originalExcelFileLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		numberOfCopiesLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		originalModelFileText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		originalExcelFileText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		numberOfCopiesText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		originalModelFileButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		originalExcelFileButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));




		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));
		
		makeCopiesButton = new JButtonLanguage(StringConstants.MAKE_COPIES);
		makeCopiesButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		createLayout();
		createListeners();
		setDependencies();

	}
	
	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));
		
		originalModelFileLabel.setText(StringConstants.ORIGINAL_VENSIM_MODEL_FILE_NAME);
		originalExcelFileLabel.setText(StringConstants.ORIGINAL_EXCEL_DATA_FILE_NAME);
		numberOfCopiesLabel.setText(StringConstants.NUMBER_OF_COPIES);
		makeCopiesButton.setText(language.getText(StringConstants.MAKE_COPIES));
		
	}

	private void createLayout() {
		setLayout(new GridBagLayout());
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		originalModelFileLabel.setFont(language.getFont(Font.BOLD, 12));
//		numberOfCopiesLabel.setFont(language.getFont(Font.BOLD, 12));
//		originalExcelFileLabel.setFont(language.getFont(Font.BOLD, 12));

		int row = -1;

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;

		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(originalModelFileLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(originalModelFileText, c);
		c.gridx = 1;
		c.weightx = 0.0;
		add(originalModelFileButton, c);

		c.weightx = 1.0;
		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(originalExcelFileLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(originalExcelFileText, c);
		c.gridx = 1;
		c.weightx = 1.0;
		add(originalExcelFileButton, c);
		
		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(numberOfCopiesLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(numberOfCopiesText, c);

		c.fill = GridBagConstraints.NONE;

		c.gridwidth = 1;

		c.fill = GridBagConstraints.CENTER;

			c.gridy = ++row;
			c.gridx = 0;
			add(makeCopiesButton, c);

	}

	private void createListeners() {
		final String c1 = CHECK_DEFAULT;
		createBrowserListener(originalModelFileButton, originalModelFileText, "mdl", c1);
		final String c2 = DO_NOT_CHECK_DEFAULT;
		createBrowserListener(originalExcelFileButton, originalExcelFileText, "xlsx", c2);
		

		makeCopiesButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				int numCopies = -1;
				try {
					numCopies = Integer.valueOf( numberOfCopiesText.getText());
				} catch (NumberFormatException e1) {
					JOptionPane
					.showMessageDialog(
							null,
							language.getText(StringConstants.INVALID_COPY_COUNT),
							language.getText(StringConstants.INVALID_COPY_COUNT),
							JOptionPane.ERROR_MESSAGE);
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				if (numCopies < 1 || numCopies >= 1000) {
					JOptionPane
					.showMessageDialog(
							null,
							language.getText(StringConstants.INVALID_COPY_COUNT),
							language.getText(StringConstants.INVALID_COPY_COUNT),
							JOptionPane.ERROR_MESSAGE);
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				ModelCopier copier = new ModelCopier(
						originalModelFileText.getText(), originalExcelFileText.getText(), numCopies);
				try {
					copier.makeCopies();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					JOptionPane
							.showMessageDialog(
									null,
									language.getText(StringConstants.COPIES_CREATED),
									language.getText(StringConstants.COPIES_CREATED),
									JOptionPane.INFORMATION_MESSAGE);
				} catch (Exception e) {
					String errMsg = e.toString();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					
					log.log(Level.FATAL, "Internal Error Detected", e);
					JOptionPane.showMessageDialog(null, errMsg,
							language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
							JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});

	}

	private void createBrowserListener(/*final*/ JButtonLanguage button,
			/*final*/ JTextField textField, /*final*/ String fileType,
			/*final*/  String checkDefault) {
		final JButtonLanguage fbutton = button;
		final String ffileType = fileType;
		final JTextField ftextField = textField;
		final String fcheckDefault = new String(checkDefault);
		fbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter(ffileType
						.toUpperCase() + " Files", ffileType.toLowerCase());
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(0, 200);

				String savedValue = ftextField.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					 saveFCDir(dir);
					String path = sf.getAbsolutePath();
					ftextField.setText(path);
				} else {
					ftextField.setText(savedValue);
					if (savedValue.length() == 0) {
					}
				}
				setDependencies();
			}
		});
	}


	private void setDependencies() {
		makeCopiesButton.setEnabled(true);
		
			if (originalModelFileText.getText().length() > 0
					&& originalExcelFileText.getText().length() > 0
					&& numberOfCopiesText.getText().length() > 0) {
				makeCopiesButton.setEnabled(true);
				

			}		
	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void setValues(boolean value) {
	}

}
