package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JSeparator;

public class ModelInformationPanel extends SDMOptionsPanel {
	
	JCheckBoxLanguage totalNumberOfVariables = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_VARIABLES);
	JCheckBoxLanguage totalNumberOfStateVariables = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES);
	JCheckBoxLanguage totalNumberOfStocks = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_STOCKS);
	
	JCheckBoxLanguage totalNumberOfExogenousVariables = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES);
	JCheckBoxLanguage totalNumberOfEndogenousVariables = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES);
	
	CheckboxAssociatedTextField totalNumberOfEndogenousVariablesText = new CheckboxAssociatedTextField(totalNumberOfEndogenousVariables, 
			StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH);
	
	
	JCheckBoxLanguage totalNumberOfFeedbackLoops = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV);
	JCheckBoxLanguage totalNumberOfFeedbackLoopsWithIVV = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV);
	
	JCheckBoxLanguage totalNumberOfCausalLinks = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_CAUSAL_LINKS);
	JCheckBoxLanguage totalNumberOfRateToRateLinks = new JCheckBoxLanguage(StringConstants.RATE_TO_RATE_LINKS);
	JCheckBoxLanguage totalNumberOfUnits = new JCheckBoxLanguage(StringConstants.UNITS_NAME);
	
	JCheckBoxLanguage totalNumberOfMacros = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_MACROS);
	JCheckBoxLanguage totalNumberOfUsingMacros = new JCheckBoxLanguage(StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS);
	JCheckBoxLanguage variablesWithSourceInformation = new JCheckBoxLanguage(StringConstants.VARIABLES_WITH_SOURCE_INFORMATION);
	JCheckBoxLanguage variablesWithDimensionlessUnits = new JCheckBoxLanguage(StringConstants.DIMENSIONLESS_UNIT_VARIABLES);
	JCheckBoxLanguage variablesWithoutValueRange = new JCheckBoxLanguage(StringConstants.VALUE_RANGE_SPECIFIED);
	JCheckBoxLanguage functionSensitivityParameters = new JCheckBoxLanguage(StringConstants.FUNCTION_SENSITIVITY_PARAMETERS);
	JCheckBoxLanguage dataLookupTableds = new JCheckBoxLanguage(StringConstants.DATA_LOOKUP_TABLES);
	JCheckBoxLanguage timeUnit = new JCheckBoxLanguage(StringConstants.TIME_UNIT);
	JCheckBoxLanguage initialTime = new JCheckBoxLanguage(StringConstants.INITIAL_TIME);
	JCheckBoxLanguage finalTime = new JCheckBoxLanguage(StringConstants.FINAL_TIME);
	JCheckBoxLanguage reportedTimeInterval = new JCheckBoxLanguage(StringConstants.REPORTED_TIME_INTERVAL);
	JCheckBoxLanguage timeStep = new JCheckBoxLanguage(StringConstants.TIME_STEP);
	JCheckBoxLanguage modelIsFullyFormulated = new JCheckBoxLanguage(StringConstants.MODEL_IS_FULLY_FORMULATED);
	JCheckBoxLanguage modelerDefinedGroups = new JCheckBoxLanguage(StringConstants.MODELER_DEFINED_GROUPS);
	
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public ModelInformationPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);
		
		loadMap();
		
		add(totalNumberOfVariables);
		add(totalNumberOfStateVariables);
		add(totalNumberOfStocks);
		add(totalNumberOfExogenousVariables);
		add(totalNumberOfEndogenousVariables);
		add(totalNumberOfEndogenousVariablesText);
		add(totalNumberOfFeedbackLoops);
		add(totalNumberOfFeedbackLoopsWithIVV);
		add(totalNumberOfCausalLinks);
		add(totalNumberOfRateToRateLinks);
		add(totalNumberOfUnits);
		
		
		add(totalNumberOfMacros);
		add(totalNumberOfUsingMacros);
		add(new JSeparator(JSeparator.HORIZONTAL));
		
		add(variablesWithSourceInformation);
		add(variablesWithDimensionlessUnits);
		add(variablesWithoutValueRange);
		add(functionSensitivityParameters);
		add(dataLookupTableds);
		add(new JSeparator(JSeparator.HORIZONTAL));
		
		add(timeUnit);
		add(initialTime);
		add(finalTime);
		add(reportedTimeInterval);
		add(timeStep);
		add(new JSeparator(JSeparator.HORIZONTAL));
		
		add(modelIsFullyFormulated);
		add(modelerDefinedGroups);
		
		setFonts();
		
		totalNumberOfEndogenousVariables.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				totalNumberOfEndogenousVariablesText.setDependencies();
			}
		});
		
		setValues();
	}
	
	public void setFont(JCheckBoxLanguage cb) {
		cb.setFont(SDMDocLanguageSpecific.getInstance().getFont(Font.PLAIN,  SDMDocFont.MEDIUM));
	}
	
	public void setFonts() {
		setFont(totalNumberOfVariables);
		setFont(totalNumberOfStateVariables);
		setFont(totalNumberOfStocks);
		setFont(totalNumberOfExogenousVariables);
		setFont(totalNumberOfEndogenousVariables);
		totalNumberOfEndogenousVariablesText.setFonts(
				SDMDocLanguageSpecific.getInstance().getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		setFont(totalNumberOfFeedbackLoops);
		setFont(totalNumberOfFeedbackLoopsWithIVV);
		setFont(totalNumberOfCausalLinks);
		setFont(totalNumberOfRateToRateLinks);
		setFont(totalNumberOfUnits);
		
		
		setFont(totalNumberOfMacros);
		setFont(totalNumberOfUsingMacros);
		
		setFont(variablesWithSourceInformation);
		setFont(variablesWithDimensionlessUnits);
		setFont(variablesWithoutValueRange);
		setFont(functionSensitivityParameters);
		setFont(dataLookupTableds);
		
		
		setFont(timeUnit);
		setFont(initialTime);
		setFont(finalTime);
		setFont(reportedTimeInterval);
		setFont(timeStep);
		
		setFont(modelIsFullyFormulated);
		setFont(modelerDefinedGroups);
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		 totalNumberOfVariables.setText(StringConstants.TOTAL_NUMBER_OF_VARIABLES);
		 totalNumberOfStateVariables.setText(StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES);
		 totalNumberOfStocks.setText(StringConstants.TOTAL_NUMBER_OF_STOCKS);
		 
		 totalNumberOfExogenousVariables.setText(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES);
		 totalNumberOfEndogenousVariables.setText(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES);
		 totalNumberOfEndogenousVariablesText.changeLanguage();
		 
		 
		 totalNumberOfFeedbackLoops.setText(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV);
		 totalNumberOfFeedbackLoopsWithIVV.setText(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV);
		
		 totalNumberOfCausalLinks.setText(StringConstants.TOTAL_NUMBER_OF_CAUSAL_LINKS);
		 totalNumberOfUnits.setText(StringConstants.UNITS_NAME);
		 totalNumberOfRateToRateLinks.setText(StringConstants.NUM_RATE_TO_RATE_LINKS);
		
		 totalNumberOfMacros.setText(StringConstants.TOTAL_NUMBER_OF_MACROS);
		 totalNumberOfUsingMacros.setText(StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS);
		 variablesWithSourceInformation.setText(StringConstants.VARIABLES_WITH_SOURCE_INFORMATION);
		 variablesWithDimensionlessUnits.setText(StringConstants.DIMENSIONLESS_UNIT_VARIABLES);
		 variablesWithoutValueRange.setText(StringConstants.VALUE_RANGE_SPECIFIED);
		 functionSensitivityParameters.setText(StringConstants.FUNCTION_SENSITIVITY_PARAMETERS);
		 dataLookupTableds.setText(StringConstants.DATA_LOOKUP_TABLES);
		 timeUnit.setText(StringConstants.TIME_UNIT);
		 initialTime.setText(StringConstants.INITIAL_TIME);
		 finalTime.setText(StringConstants.FINAL_TIME);
		 reportedTimeInterval.setText(StringConstants.REPORTED_TIME_INTERVAL);
		 timeStep.setText(StringConstants.TIME_STEP);
		 modelIsFullyFormulated.setText(StringConstants.MODEL_IS_FULLY_FORMULATED);
		 modelerDefinedGroups.setText(StringConstants.MODELER_DEFINED_GROUPS);
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.MODEL_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			
			if (options.getOptionValue(var) == null)
				System.out.println("BRKPT "+var);
			
			cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
		totalNumberOfEndogenousVariablesText.setValues(options);
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.MODEL_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
		}
		 return totalNumberOfEndogenousVariablesText.saveValues(options);
	}
	
	private void loadMap() {
		
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_VARIABLES, totalNumberOfVariables);
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES, totalNumberOfStateVariables);
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_STOCKS, totalNumberOfStocks);
		
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES, totalNumberOfExogenousVariables);
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES, totalNumberOfEndogenousVariables);
		
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV, totalNumberOfFeedbackLoops);
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV, totalNumberOfFeedbackLoopsWithIVV);
		
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_CAUSAL_LINKS, totalNumberOfCausalLinks);
		mapNameToCheckBox.put(StringConstants.UNITS_NAME, totalNumberOfUnits);
		mapNameToCheckBox.put(StringConstants.RATE_TO_RATE_LINKS, totalNumberOfRateToRateLinks);
		
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_MACROS,totalNumberOfMacros);
		mapNameToCheckBox.put(StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS,totalNumberOfUsingMacros);
		mapNameToCheckBox.put(StringConstants.VARIABLES_WITH_SOURCE_INFORMATION, variablesWithSourceInformation);
		mapNameToCheckBox.put(StringConstants.DIMENSIONLESS_UNIT_VARIABLES, variablesWithDimensionlessUnits);
		mapNameToCheckBox.put(StringConstants.VALUE_RANGE_SPECIFIED, variablesWithoutValueRange);
		mapNameToCheckBox.put(StringConstants.FUNCTION_SENSITIVITY_PARAMETERS, functionSensitivityParameters);
		mapNameToCheckBox.put(StringConstants.DATA_LOOKUP_TABLES,dataLookupTableds);
		mapNameToCheckBox.put(StringConstants.TIME_UNIT, timeUnit);
		mapNameToCheckBox.put(StringConstants.INITIAL_TIME, initialTime);
		mapNameToCheckBox.put(StringConstants.FINAL_TIME, finalTime);
		mapNameToCheckBox.put(StringConstants.REPORTED_TIME_INTERVAL, reportedTimeInterval);
		mapNameToCheckBox.put(StringConstants.TIME_STEP,timeStep);
		mapNameToCheckBox.put(StringConstants.MODEL_IS_FULLY_FORMULATED, modelIsFullyFormulated);
		mapNameToCheckBox.put(StringConstants.MODELER_DEFINED_GROUPS, modelerDefinedGroups);
		
	}
	
	@Override
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.MODEL_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}
