package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Font;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;

public class NamingOptionsPanel extends JPanel {
	
	JCheckBoxLanguage includeLoopSize = new JCheckBoxLanguage(StringConstants.INCLUDE_LOOP_SIZE);
	JCheckBoxLanguage includeTimeStamp = new JCheckBoxLanguage(StringConstants.INCLUDE_TIME_STAMP);
	
	UserSelectedOptions uso;
	String title;
	JLabel titleLabel;
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public NamingOptionsPanel(UserSelectedOptions uso, String title) {
		this.uso = uso;
		this.title = title;
		
		loadMap();
		titleLabel = new JLabelLanguage(title);
		add(titleLabel);
		add(includeLoopSize);
		add(includeTimeStamp);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		titleLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		includeLoopSize.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		includeTimeStamp.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		
		setValues();
	}
	
	public void changeLanguage() {
		
		includeLoopSize.setText(StringConstants.INCLUDE_LOOP_SIZE);
		includeTimeStamp.setText(StringConstants.INCLUDE_TIME_STAMP);
		titleLabel.setText(title);
	}
	
	public void setValues() {
		
		for (String var : UserSelectedOptions.NAMING_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (uso != null)
				cb.setSelected(uso.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public void saveValues() {
		
		for (String var : UserSelectedOptions.NAMING_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected()) {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.TRUE);
			} else {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.FALSE);
			}
		}
		
		uso.save();
	}
	
	private void loadMap() {

		mapNameToCheckBox.put(StringConstants.INCLUDE_LOOP_SIZE,includeLoopSize);
		mapNameToCheckBox.put(StringConstants.INCLUDE_TIME_STAMP, includeTimeStamp);
	}
	
	public void setValues(boolean value) {
		
		for (String var : UserSelectedOptions.NAMING_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}

