package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;

public class OptionsButtonPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private OptionsFrame optionsFrame;
	private JFrameWithValueSetter otherFrame;
	private JButton selectAllButton;
	private JButton selectNoneButton;
	private JButton saveButton;
	private JButton quitButton;
	
	public OptionsButtonPanel(final OptionsFrame optionsFrame, final JFrameWithValueSetter otherFrame) {
		this.optionsFrame = optionsFrame;
		this.otherFrame = otherFrame;
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
        selectAllButton = new JButtonLanguage(StringConstants.SELECT_ALL);
        selectAllButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        selectAllButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
        
        selectAllButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	otherFrame.setAllValues(true);
            }
        });
        
        
        selectNoneButton = new JButtonLanguage(StringConstants.SELECT_NONE);
        selectNoneButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        selectNoneButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        selectNoneButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	otherFrame.setAllValues(false);
            }
        });
        
        
        quitButton = new JButtonLanguage(StringConstants.QUIT);
        quitButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        quitButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        quitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	otherFrame.setVisible(false);
            }
        });
        
        saveButton = new JButtonLanguage(StringConstants.SAVE);
        saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        saveButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        saveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	if (optionsFrame.saveButton())
            		otherFrame.setVisible(false);
            }
        });
        
		createLayout();
		
		
		
		
	}
	
	public OptionsButtonPanel(final OptionsFrame optionsFrame) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
        selectAllButton = new JButtonLanguage(StringConstants.SELECT_ALL);
        selectAllButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        
        selectAllButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        selectAllButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	optionsFrame.setAllValues(true);
            }
        });
        
        
        selectNoneButton = new JButtonLanguage(StringConstants.SELECT_NONE);
        selectNoneButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        selectNoneButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        selectNoneButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	optionsFrame.setAllValues(false);
            }
        });
        
        
		this.optionsFrame = optionsFrame;
		

        
        quitButton = new JButtonLanguage(StringConstants.QUIT);
        quitButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        quitButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        quitButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	optionsFrame.quitButton();
            }
        });
        
        saveButton = new JButtonLanguage(StringConstants.SAVE);
        saveButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        saveButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
       
        
        saveButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent event) {
            	if (optionsFrame.saveButton())
            		optionsFrame.hidePanel();
            }
        });
        
//        createLayout();
        GridLayout grid = new GridLayout(0,2);
		setLayout(grid);
		grid.setHgap(20);
		grid.setVgap(20);
		add(saveButton);
		add(quitButton);
        
//        GridBagLayout grid = new GridBagLayout();
//        setLayout(grid);
//        GridBagConstraints c = new GridBagConstraints();
//        
//        c.insets = new Insets(0,0,0,0);  //top padding
//        c.fill = GridBagConstraints.HORIZONTAL;
//        c.weightx = 1.0;
//        c.weighty = 1.0;
//       
//        
//        c.gridwidth = 1;
//        c.gridx = 0;
//        c.gridy = 0;
////       add(selectAllButton,c);
//       c.gridx = 1;
////       add(selectNoneButton,c);
//       c.gridx = 1;
//       add(saveButton,c);
//       c.gridx = 2;
//       add(quitButton,c);
		
	}
	
	private void createLayout() {
		 GridBagLayout grid = new GridBagLayout();
	        setLayout(grid);
	        GridBagConstraints c = new GridBagConstraints();
	        
	        c.insets = new Insets(0,0,0,0);  //top padding
	        c.fill = GridBagConstraints.HORIZONTAL;
	        c.weightx = 1.0;
	        c.weighty = 1.0;
	       
	        
	        c.gridwidth = 1;
	        c.gridx = 0;
	        c.gridy = 0;
	       add(selectAllButton,c);
	       c.gridx = 1;
	       add(selectNoneButton,c);
	       c.gridx = 2;
	       add(saveButton,c);
	       c.gridx = 3;
	       add(quitButton,c);
	        
	}
	
	public void changeLanguage() {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		selectAllButton.setText(StringConstants.SELECT_ALL);
		selectNoneButton.setText(StringConstants.SELECT_NONE);
		saveButton.setText(StringConstants.SAVE);
		quitButton.setText(StringConstants.QUIT);
	}

}
