package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

public class OptionsFrame extends SDMDocJFrame {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6837134685220107395L;
	
	private SDMDocJFrame frame;
	private UserSelectedOptions userSelectedOptions;
	private UserSelectedOptions savedUserSelectedOptions;
	
	private ModelInformationPanel mip;
	private VariableInformationPanel vip;
	private ArrowInformationPanel aip;
	private WarningsPanel wp;
	private PotentialOmissionsPanel po;
	
	private LoopAnalysisPanel lap;
	
	private JButton genDocButton;
	private JButton browseButton;
	
	private GraphOutputOptionsPanel goop;
	
	private OptionsButtonPanel obp;
	
	private boolean genDocButtonEnabled;
	
	
	// Alternate Structure
	
	private JFrameWithValueSetter miFrame;
	private JFrameWithValueSetter viwFrame;
	private JFrameWithValueSetter poaialFrame;
	private JFrameWithValueSetter gaoobFrame;
	
	private OptionsButtonPanel miObp;
	private OptionsButtonPanel viwObp;
	private OptionsButtonPanel poaialObp;
	private OptionsButtonPanel gaoobObp;
	
	private MultiLineButtonLanguage modelInformationButton;
	private MultiLineButtonLanguage variableInformationWarningsButton;
	private MultiLineButtonLanguage potentialOmissionsArrowInformationAnalyzeLoopsButton;
	private MultiLineButtonLanguage graphAndOutputOptionsButton;
	
	
	public OptionsFrame(UserSelectedOptions userSelectedOptions, JButtonLanguage genDocButton,  JButtonLanguage browseButton,
			String helpFile) {
		
//		super(helpFile);
		this.userSelectedOptions = userSelectedOptions;
		savedUserSelectedOptions = new UserSelectedOptions(userSelectedOptions);
		this.genDocButton = genDocButton;
		this.browseButton = browseButton;
//		create();
		createAlternate(helpFile);
	}
	
	public void createAlternate(String helpFile) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		//Create and set up the window.
		frame = new SDMDocJFrame(language.getText(StringConstants.MODEL_ASSESSMENT_RESULTS_OPTIONS), helpFile);
		frame.setPreferredSize(new SdmDimension(600, 250)); // 550
		frame.setLocation (50, 200) ;
		
		frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener( new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent we) {
            	quitButton();
            }
        } );
		
		frame.addInformationMenu(frame);
		frame.setJMenuBar(frame.myMenubar);
		
//		frame.add(frame.myMenubar);
		
		modelInformationButton =  new MultiLineButtonLanguage();
		modelInformationButton.addLine(StringConstants.MODEL_INFORMATION);
		modelInformationButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		variableInformationWarningsButton = new MultiLineButtonLanguage();
		variableInformationWarningsButton.addLine(StringConstants.VARIABLE_INFORMATION);
		variableInformationWarningsButton.addLine(StringConstants.WARNINGS);
		
		potentialOmissionsArrowInformationAnalyzeLoopsButton = new MultiLineButtonLanguage();
		potentialOmissionsArrowInformationAnalyzeLoopsButton.addLine(StringConstants.POTENTIAL_OMISSIONS);
		potentialOmissionsArrowInformationAnalyzeLoopsButton.addLine(StringConstants.ARROW_INFORMATION);
//		potentialOmissionsArrowInformationAnalyzeLoopsButton.addLine(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS);
		potentialOmissionsArrowInformationAnalyzeLoopsButton.addLine(StringConstants.ANALYZE_LOOPS);
				
		graphAndOutputOptionsButton = new MultiLineButtonLanguage();
		graphAndOutputOptionsButton.addLine(StringConstants.GRAPH_AND_OUTPUT_OPTIONS);
		
		createButtonListeners();
		createOptionFrames();
		
		JPanel multiplePanels = new JPanel();
		GridBagLayout gridMP = new GridBagLayout();
		multiplePanels.setLayout(gridMP);
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		
		
		JPanel optionButtonPanel = new JPanel();

//		GridBagLayout grid = new GridBagLayout();
		GridLayout grid = new GridLayout(0,2);
		optionButtonPanel.setLayout(grid);
		
		//Set up the horizontal gap value
        grid.setHgap(10);
        //Set up the vertical gap value
        grid.setVgap(10);
		
		obp = new OptionsButtonPanel(this);
		c.fill = GridBagConstraints.BOTH;
		c.gridwidth = 1;
		
		optionButtonPanel.add(modelInformationButton);
		optionButtonPanel.add(variableInformationWarningsButton);
		optionButtonPanel.add(potentialOmissionsArrowInformationAnalyzeLoopsButton);
		optionButtonPanel.add(graphAndOutputOptionsButton);
		
		c.gridheight = 1;
		c.weightx = 1.0;
		c.weighty = 0.9;
		c.gridx = 0;
		c.gridy = 0;
		multiplePanels.add(optionButtonPanel, c);
		c.gridy = 1;
		c.weighty = 0.1;
		c.gridheight = 1;
		
		c.insets = new Insets(20,20,20,20);
		
		multiplePanels.add(obp, c);
		

		frame.getContentPane().add(multiplePanels);
		frame.pack();
//		frame.setVisible(true);
	}
	
	private void createOptionFrames() {
		mip = new ModelInformationPanel(this, StringConstants.MODEL_INFORMATION);
		vip = new VariableInformationPanel(this, StringConstants.VARIABLE_INFORMATION);
		wp = new WarningsPanel(this,StringConstants.WARNINGS);
		po = new PotentialOmissionsPanel(this, StringConstants.POTENTIAL_OMISSIONS);
		aip = new ArrowInformationPanel(this, StringConstants.ARROW_INFORMATION);
		lap = new LoopAnalysisPanel(this, StringConstants.ANALYZE_LOOPS);
		goop = new GraphOutputOptionsPanel(this, StringConstants.GRAPH_AND_OUTPUT_OPTIONS);
		
		createMiFrame();
		createViwFrame();
		createPoaialFrame();
		createGaoobFrame();
	}
	
	private void createMiFrame() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		miFrame = new JFrameWithValueSetter(language.getText(StringConstants.MODEL_INFORMATION),"ModelInformationOptionWindow.png");
		miFrame.setPreferredSize(new SdmDimension(600, 850));
		miFrame.setLocation(100, 200);
		
		miFrame.addInformationMenu(miFrame);
		
		JPanel panel = new JPanel();
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		mip.setValues();
		panel.add(mip, c);
		miFrame.addToSet(mip);
		miObp = new OptionsButtonPanel(this, miFrame);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 1;
		panel.add(miObp, c);
		
		miFrame.getContentPane().add(panel);
		miFrame.pack();
	}
	
	private void createViwFrame() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		viwFrame = new JFrameWithValueSetter(language.getText(StringConstants.VARIABLE_INFORMATION) + ", " +
				language.getText(StringConstants.WARNINGS), "VariableInformationWarning.png"
		);
		
		viwFrame.setPreferredSize(new SdmDimension(600, 700));
		viwFrame.setLocation(150, 350);
		
		viwFrame.addInformationMenu(viwFrame);
		
		JPanel panel = new JPanel();
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		panel.add(vip, c);
		viwFrame.addToSet(vip);

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 1;
		panel.add(wp,c);
		viwFrame.addToSet(wp);
		
		viwObp = new OptionsButtonPanel(this, viwFrame);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 2;
		panel.add(viwObp, c);
		
		viwFrame.getContentPane().add(panel);
		viwFrame.pack();

	}
	
	private void createPoaialFrame() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		poaialFrame = new JFrameWithValueSetter(language.getText(StringConstants.POTENTIAL_OMISSIONS) +", " +
				language.getText(StringConstants.ARROW_INFORMATION) +", "+
				language.getText(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS) +", "+
				language.getText(StringConstants.ANALYZE_LOOPS), "PotentialOmissionsArrowLoops.png");
		
		
		poaialFrame.setPreferredSize(new SdmDimension(800,800)); // new Dimension(700, 550) 550 y
		poaialFrame.setLocation(200, 200);
		
		poaialFrame.addInformationMenu(poaialFrame);
		poaialFrame.setJMenuBar(poaialFrame.myMenubar);
		
		JPanel panel = new JPanel();
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		panel.add(po, c);
		poaialFrame.addToSet(po);

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 1; // c.gridy++;
		panel.add(aip, c);
		poaialFrame.addToSet(aip);
		
		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 3; // c.gridy++;
		panel.add(lap, c);
		poaialFrame.addToSet(lap);
		
		poaialObp = new OptionsButtonPanel(this, poaialFrame);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 4; // c.gridy++;
		panel.add(poaialObp, c);
		
		poaialFrame.getContentPane().add(panel);
		poaialFrame.pack();

	}
	
	private void createGaoobFrame() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		gaoobFrame = new JFrameWithValueSetter(language.getText(StringConstants.GRAPH_AND_OUTPUT_OPTIONS),
				"GraphOptions.png");
		gaoobFrame.setPreferredSize(new SdmDimension(800, 375));
		gaoobFrame.setLocation(250, 450);
		
		gaoobFrame.addInformationMenu(gaoobFrame);
		gaoobFrame.setJMenuBar(gaoobFrame.myMenubar);
		
		JPanel panel = new JPanel();
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.BOTH;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1.0;
		c.weighty = 1.0;

		c.gridheight = 1;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 0;
		mip.setValues();
		panel.add(goop, c);
		gaoobFrame.addToSet(goop);
		
		gaoobObp = new OptionsButtonPanel(this, gaoobFrame);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		c.gridx = 0;
		c.gridy = 1;
		panel.add(gaoobObp, c);
		
		gaoobFrame.getContentPane().add(panel);
		gaoobFrame.pack();
	}
	
	
	private void createButtonListeners() {
		
		modelInformationButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				miFrame.setVisible(true);
			}
		});

		variableInformationWarningsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				viwFrame.setVisible(true);
			}
		});
		
		potentialOmissionsArrowInformationAnalyzeLoopsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				poaialFrame.setVisible(true);
			}
		});
		
		graphAndOutputOptionsButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				gaoobFrame.setVisible(true);
			}
		});
	}
	
//	public void create() {
//		
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		
//		//Create and set up the window.
//		frame = new SDMDocJFrame(language.getText(StringConstants.MODEL_ASSESSMENT_RESULTS_OPTIONS));
//		//		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//		frame.setPreferredSize(new SdmDimension(1200, 600));
//		frame.setLocation (0, 200) ;
//		frame.add(myMenubar);
//		addInformationMenu(frame);
//
//		JPanel panel = new JPanel();
//
//		GridBagLayout grid = new GridBagLayout();
//		panel.setLayout(grid);
//		GridBagConstraints c = new GridBagConstraints();
//		c.fill = GridBagConstraints.HORIZONTAL;
//
////		JLabel label = new JLabel("Stuff goes here");
////		label.setFont(new Font("Arial", Font.BOLD, 18));
////		label.setPreferredSize(new Dimension(100, 100));
////		panel.add(label,c);
//		
//		mip = new ModelInformationPanel(this, StringConstants.MODEL_INFORMATION);
//		
//		c.anchor = GridBagConstraints.FIRST_LINE_START;
//		c.weightx = 1.0;
//		c.weighty = 1.0;
//
//		c.gridheight = 3;
//		c.gridwidth = 1;
//		c.gridx = 0;
//		c.gridy = 0;
//		panel.add(mip, c);
//		
//		vip = new VariableInformationPanel(this, StringConstants.VARIABLE_INFORMATION);
//
//		c.gridheight = 1;
//		c.gridwidth = 1;
//		c.gridx = 1;
//		c.gridy = 0;
//		panel.add(vip, c);
//		
//		wp = new WarningsPanel(this,StringConstants.WARNINGS);
//
//		c.gridheight = 2;
//		c.gridwidth = 1;
//		c.gridx = 1;
//		c.gridy = 1;
//		panel.add(wp,c);
//		
//
//		
//		po = new PotentialOmissionsPanel(this, StringConstants.POTENTIAL_OMISSIONS);
//
//		c.gridheight = 1;
//		c.gridwidth = 1;
//		c.gridx = 2;
//		c.gridy = 0;
//		panel.add(po, c);
//		
//		aip = new ArrowInformationPanel(this, StringConstants.ARROW_INFORMATION);
//
//		c.gridwidth = 1;
//		c.gridx = 2;
//		c.gridy = c.gridy++;
//		panel.add(aip, c);
//	
//		
//		lap = new LoopAnalysisPanel(this, StringConstants.ANALYZE_LOOPS);
//		c.gridwidth = 1;
//		c.gridx = 2;
//		c.gridy = c.gridy++;
//		panel.add(lap, c);
//		
//		
//		
//		goop = new GraphOutputOptionsPanel(this, StringConstants.GRAPH_AND_OUTPUT_OPTIONS);
//		c.gridwidth = 1;
//		c.gridx = 0;
//		c.gridy = c.gridy++;
//		panel.add(goop, c);
//		
//
//
//		
//		obp = new OptionsButtonPanel(this);
//		c.fill = GridBagConstraints.HORIZONTAL;
//		c.gridwidth = 1;
//		c.gridx = 1;
//		c.gridy = c.gridy++;
//		panel.add(obp, c);
//
//		frame.getContentPane().add(panel);
//		frame.pack();
////		frame.setVisible(true);
//	}
	
	public void changeLanguage() {
		frame.setTitle(SDMDocLanguageSpecific.getInstance().getText(StringConstants.MODEL_ASSESSMENT_RESULTS_OPTIONS));
		frame.changeLanguage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.MODEL_ASSESSMENT_RESULTS_OPTIONS));
		mip.changeLanguage();
		vip.changeLanguage();
		aip.changeLanguage();
		wp.changeLanguage();
		po.changeLanguage();
		goop.changeLanguage();
		lap.changeLanguage();
		obp.changeLanguage();
		
		modelInformationButton.changeLanguage();
		variableInformationWarningsButton.changeLanguage();
		potentialOmissionsArrowInformationAnalyzeLoopsButton.changeLanguage();
		graphAndOutputOptionsButton.changeLanguage();
		
		miObp.changeLanguage();
		viwObp.changeLanguage();
		poaialObp.changeLanguage();
		gaoobObp.changeLanguage();
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		miFrame.setTitle(language.getText(StringConstants.MODEL_INFORMATION));
//		
//		poaialFrame.setTitle(language.getText(StringConstants.POTENTIAL_OMISSIONS) +", " +
//				language.getText(StringConstants.ARROW_INFORMATION) +", "+
//				language.getText(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS) +", "+
//				language.getText(StringConstants.ANALYZE_LOOPS));
//		
//		gaoobFrame.setTitle(language.getText(StringConstants.GRAPH_AND_OUTPUT_OPTIONS));
//		
//		viwFrame.setTitle(language.getText(StringConstants.VARIABLE_INFORMATION) + ", " +
//				language.getText(StringConstants.WARNINGS)
//		);
		
		miFrame.changeLanguage(language.getText(StringConstants.MODEL_INFORMATION));
		
		poaialFrame.changeLanguage(language.getText(StringConstants.POTENTIAL_OMISSIONS) +", " +
				language.getText(StringConstants.ARROW_INFORMATION) +", "+
				language.getText(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS) +", "+
				language.getText(StringConstants.ANALYZE_LOOPS));
		
		gaoobFrame.changeLanguage(language.getText(StringConstants.GRAPH_AND_OUTPUT_OPTIONS));
		
		viwFrame.changeLanguage(language.getText(StringConstants.VARIABLE_INFORMATION) + ", " +
				language.getText(StringConstants.WARNINGS)
		);
//		private JFrame miFrame;
//		private JFrame viwFrame;
//		private JFrame poaialFrame;
//		private JFrame gaoobFrame;
		
		
	}
	
	public void showPanel() {
		
		genDocButtonEnabled = genDocButton.isEnabled();
		genDocButton.setEnabled(false);
		browseButton.setEnabled(false);
		savedUserSelectedOptions = new UserSelectedOptions(userSelectedOptions);
		mip.setValues();
		vip.setValues();
		aip.setValues();
		wp.setValues();
		
		po.setValues();
		lap.setValues();
		goop.setValues();
		frame.setVisible(true);
	}
	
	public void hidePanel() {
		genDocButton.setEnabled(genDocButtonEnabled);
		browseButton.setEnabled(true);

		miFrame.setVisible(false);
		viwFrame.setVisible(false);
		poaialFrame.setVisible(false);
		gaoobFrame.setVisible(false);

		frame.setVisible(false);
	}
	
	public boolean saveMIP() {
		return mip.saveValues();
	}
	
	public boolean saveVIW() {
		boolean success = true;
		success = success && vip.saveValues();
		success = success && wp.saveValues();
		return success;
	}
	public boolean  savePOAIPLAP() {
		boolean success = true;
		success = success && aip.saveValues();
		success = success && po.saveValues();
		
		success = success && lap.saveValues();
		return success;
	}
	
	public boolean saveGOOP() {
		return goop.saveValues();
	}
	
	public boolean saveButton() {
		// on save, this becomes official
		boolean success = true;
		success = success && saveMIP();
		success = success && saveVIW();
		success = success && savePOAIPLAP();
		success = success && saveGOOP();
		
		if (!success)
			return false;
		
		savedUserSelectedOptions.copyFrom(userSelectedOptions);
		userSelectedOptions.save();
		mip.setValues();
		vip.setValues();
		aip.setValues();
		wp.setValues();
		po.setValues();
		
		goop.setValues();
		lap.setValues();
//		hidePanel();
		
		return true;
	}
	
	public void quitButton() {
		// on quit, restore to previous value
		userSelectedOptions.copyFrom(savedUserSelectedOptions);
		
		// need to restore to saved values
		mip.setValues();
		vip.setValues();
		aip.setValues();
		wp.setValues();
		po.setValues();
	
		goop.setValues();
		
		hidePanel();
	}

	public UserSelectedOptions getUserSelectedOptions() {
		return userSelectedOptions;
	}
	
	public void setMIPValues(boolean value) {
		mip.setValues(value);
	}
	
	public void setVIWValues(boolean value) {
		vip.setValues(value);
		wp.setValues(value);
	}
	public void setPOAIPLAPValues(boolean value) {
		aip.setValues(value);
		po.setValues(value);
	
		lap.setValues();
	}
	public void setGOOPValues(boolean value) {
		goop.setValues();
	}
	
	public void setAllValues(boolean value) {
		setMIPValues(value);
		setVIWValues(value);
		setPOAIPLAPValues(value);
		setGOOPValues(value);
	}
	

}
