package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Dimension;
import java.awt.Font;
import java.util.LinkedHashMap;
import java.util.Map;

public class PotentialOmissionsPanel extends SDMOptionsPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	JCheckBoxLanguage unused = new JCheckBoxLanguage(StringConstants.UNUSED_VARIABLES);
	JCheckBoxLanguage supplementary = new JCheckBoxLanguage(StringConstants.SUPPLEMENTARY_VARIABLES);
	JCheckBoxLanguage supplementaryUsed = new JCheckBoxLanguage(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED);
	JCheckBoxLanguage complexVariable = new JCheckBoxLanguage(StringConstants.COMPLEX_VARIABLE);
	JCheckBoxLanguage complexStock = new JCheckBoxLanguage(StringConstants.COMPLEX_STOCK);
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	
	public PotentialOmissionsPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);

		loadMap();
	
		add(unused);
		add(supplementary);
		add(supplementaryUsed);
		add(complexVariable);
		add(complexStock);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		unused.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		supplementary.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		supplementaryUsed.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		complexVariable.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		complexStock.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		setPreferredSize(new SdmDimension(700, 460));
		
		setValues();
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		unused.setText(StringConstants.UNUSED_VARIABLES);
		supplementary.setText(StringConstants.SUPPLEMENTARY_VARIABLES);
		supplementaryUsed.setText(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED);
		complexVariable.setText(StringConstants.COMPLEX_VARIABLE);
		complexStock.setText(StringConstants.COMPLEX_STOCK);
		
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.POTENTIAL_OMISSIONS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.POTENTIAL_OMISSIONS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
		}
		return true;
	}
	
	private void loadMap() {
		
		mapNameToCheckBox.put(StringConstants.UNUSED_VARIABLES, unused);
		mapNameToCheckBox.put(StringConstants.SUPPLEMENTARY_VARIABLES, supplementary);
		mapNameToCheckBox.put(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, supplementaryUsed);
		mapNameToCheckBox.put(StringConstants.COMPLEX_VARIABLE,complexVariable);
		mapNameToCheckBox.put(StringConstants.COMPLEX_STOCK, complexStock);
		
		
	}
	
	@Override
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.POTENTIAL_OMISSIONS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}
