package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.awt.Font;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JPanel;

public class ReuseOptionsPanel extends JPanel {
	
	JCheckBoxLanguage reuseViews = new JCheckBoxLanguage(StringConstants.REUSE_VIEW_DIAGRAM);
	JCheckBoxLanguage reuseClass = new JCheckBoxLanguage(StringConstants.REUSE_CLASS);
	JCheckBoxLanguage reuseThumbnails = new JCheckBoxLanguage(StringConstants.REUSE_THUMBNAILS);
	JCheckBoxLanguage reusePolarities = new JCheckBoxLanguage(StringConstants.REUSE_POLARITIES);
	JCheckBoxLanguage reuseLoops = new JCheckBoxLanguage(StringConstants.REUSE_LOOPS);
	
	UserSelectedOptions uso;
	FileOrganizer privateFileOrganizer;
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public ReuseOptionsPanel(UserSelectedOptions uso, String title) {
		this.uso = uso;
		
		loadMap();
//		JLabel t = new JLabel(title+":");
//		add(t);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		reuseViews.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		reuseClass.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		reuseThumbnails.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		reusePolarities.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		reuseLoops.setFont(language.getFont(Font.PLAIN, SDMDocFont.SMALL));
		
		add(reuseViews);
		add(reuseClass);
		add(reuseThumbnails);
		add(reusePolarities);
		add(reuseLoops);
		
		setValues();
	}
	
	public void setMdlSpecific(String mdlFile, String dir) {
		setValues();
		if (mdlFile == null || dir == null || mdlFile.length() == 0 || dir.length() == 0)
			return;
		File mdl = new File(mdlFile);
		privateFileOrganizer = new FileOrganizer(false, mdl.getParent(), mdlFile /*, uso*/);
		privateFileOrganizer.setDocGraphDirectory(
				mdl.getParent()+"/"+
		privateFileOrganizer.getModelname()+"_SDMDOC"+"/"+dir);
		Reuse reuse = InformationManagers.getInstance().getReuse();
		reuse.setDocGraphDirectory(privateFileOrganizer.getDocGraphDirectory());
		for (String var : UserSelectedOptions.REUSE) {
			setEnabledSelected(mdl, privateFileOrganizer, var);
		}
	}
	
	public void setEnabledSelected(File mdl, FileOrganizer fo, String var) {
		if (reuseEligible(mdl, fo, var)) {
			mapNameToCheckBox.get(var).setEnabled(true);
			mapNameToCheckBox.get(var).setSelected(uso.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public boolean reuseEligible(File mdl, FileOrganizer fo, String var) {

		boolean eligible = false;
		long mdlLastModified = mdl.lastModified(); 
		long otherLastModified;
		File f;
		String[] files;
		if (var.equals(StringConstants.REUSE_VIEW_DIAGRAM)) {
			File dir = new File(fo.getViewsDirectory());
			if (!dir.exists()) {
				return false;
			} else {
				files = dir.list();
				for (String s : files) {
					if (s.startsWith("__VIEW") &&
							s.endsWith(".png")) {
						otherLastModified = createdOrLastModified(fo.getViewsDirectory()+"/"+s);
						return  otherLastModified > mdlLastModified;
					}
				}
			}

		} else if (var.equals(StringConstants.REUSE_CLASS)) {
			File classFile = new File(fo.getJavaClassFile());
			if (!classFile.exists()) {
				return false;
			} else {
				otherLastModified = createdOrLastModified(fo.getJavaClassFile());
				return otherLastModified > mdlLastModified;
			}
		} else if (var.equals(StringConstants.REUSE_THUMBNAILS)) {
			File dir = new File(fo.getThumbnailsDirectory());
			if (!dir.exists()) {
				return false;
			} else {
				files = dir.list();
				for (String s : files) {
					if (!s.startsWith("__VIEW") &&
							s.endsWith(".png")) {
						otherLastModified = createdOrLastModified(fo.getThumbnailsDirectory()+"/"+s);
						return otherLastModified > mdlLastModified;
					}
				}
			}
		} else if (var.equals(StringConstants.REUSE_POLARITIES)) {
			File dir = new File(fo.getStructureDirectory());
			if (!dir.exists()) {
				return false;
			} else {
				otherLastModified = createdOrLastModified(fo.getStructureDirectory()+"/arrowPolarityFromEquationsNoIVV.tab");
				return otherLastModified > mdlLastModified;
			}
		} 
		else if (var.equals(StringConstants.REUSE_LOOPS)) {
			File dir = new File(fo.getDocGraphDirectory());
			if (!dir.exists()) {
				return false;
			} else {
				otherLastModified = createdOrLastModified(fo.getDocGraphDirectory()+"/cyclesNoIVV.dat");
				return otherLastModified > mdlLastModified;
			}
		} 
		return eligible;
	}
	
	public void changeLanguage() {
		
		reuseViews.setText(StringConstants.REUSE_VIEW_DIAGRAM);
		reuseClass.setText(StringConstants.REUSE_CLASS);
		reuseThumbnails.setText(StringConstants.REUSE_THUMBNAILS);
		reusePolarities.setText(StringConstants.REUSE_POLARITIES);
		reuseLoops.setText(StringConstants.REUSE_LOOPS);
	}
	
	public void setValues() {
		
		// this is used for initial setting and if the mdl filename is cleared
		for (String var : UserSelectedOptions.REUSE) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (uso != null) {
				if (uso.getOptionValue(var) != null) {
//					cb.setSelected(uso.getOptionValue(var).equals(UserSelectedOptions.TRUE));
					cb.setSelected(false);
					cb.setEnabled(false);
				} else { 
					System.out.println("DEBUG...");
				}
			}
		}
	}
	
	public void saveValues() {
		
		for (String var : UserSelectedOptions.REUSE) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected()) {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.TRUE);
			} else {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.FALSE);
			}
		}
		
		uso.save();
		push();
	}
	
	private void loadMap() {

		mapNameToCheckBox.put(StringConstants.REUSE_VIEW_DIAGRAM,reuseViews);
		mapNameToCheckBox.put(StringConstants.REUSE_CLASS, reuseClass);
		mapNameToCheckBox.put(StringConstants.REUSE_THUMBNAILS, reuseThumbnails);
		mapNameToCheckBox.put(StringConstants.REUSE_POLARITIES, reusePolarities);
		mapNameToCheckBox.put(StringConstants.REUSE_LOOPS, reuseLoops);
			
	}
	
	public void setValues(boolean value) {
		
		for (String var : UserSelectedOptions.REUSE) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}
	
	public void push() {
		Reuse reuse = InformationManagers.getInstance().getReuse();
		reuse.setClassFile(uso.getOptionValue(StringConstants.REUSE_CLASS).equals(UserSelectedOptions.TRUE));
		reuse.setViews(uso.getOptionValue(StringConstants.REUSE_VIEW_DIAGRAM).equals(UserSelectedOptions.TRUE));
		reuse.setThumbnails(uso.getOptionValue(StringConstants.REUSE_THUMBNAILS).equals(UserSelectedOptions.TRUE));
		reuse.setPolarity(uso.getOptionValue(StringConstants.REUSE_POLARITIES).equals(UserSelectedOptions.TRUE));
		reuse.setLoops(uso.getOptionValue(StringConstants.REUSE_LOOPS).equals(UserSelectedOptions.TRUE));
		if (privateFileOrganizer != null)
			reuse.setDocGraphDirectory(privateFileOrganizer.getDocGraphDirectory());
	}
	
	public long createdOrLastModified(String s) {
		BasicFileAttributes attr;
		try {
			Path file = new File(s).toPath();
			attr = Files.readAttributes(file, BasicFileAttributes.class);
			return Math.max(attr.lastModifiedTime().toMillis(), attr.creationTime().toMillis());
		} catch (IOException e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			return 0l;
		}
	}

}

