package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.MDLProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;
import gov.anl.gss.sdm.server.model.xmile.XmileReader;

import java.awt.Component;
import java.awt.Cursor;
import java.awt.Desktop;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingWorker;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

public class StandaloneGUI extends SDMDocJFrame {
	
	public final static String version = "1.4.7";
	public final static boolean THREE_D_ENABLED = false;
//	public final static boolean REPAST_SIMPHONY = false;
	
	public final static String[] RECOGNIZED_FILE_TYPES = {"mdl", "xmile", "stmx"}; 
	
	static Logger log = Logger.getLogger(StandaloneGUI.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1893237669846913860L;
	
	private StandaloneGUI thisFrame = null;
	private SDMDocJFrame actualFrame = null;
	
	private MessageFrame messageFrame = null;
	private MessageFrame errorFrame = null;
	
	

	private static String FC_DIR_FILE = "data/ChooserDir.txt";
	private static String FEEDBACK_URL = "http://www.gss.anl.gov/system-dynamics-model-documentation-tool-sdm-doc/";
	
	private MDLProcessor sdmDocHtml = null;
	
	private String preselectedMDL = null;
	private String logfileTag = "NoTagProvided";
	
//	private NamingOptionsPanel namingOptionsPanel;
//	private ReuseOptionsPanel reuseOptionsPanel;
//	private AvailableResultsPanel availableResultsPanel;
	
	public StandaloneGUI(String args) {
		super(args);
		thisFrame = this;
			this.preselectedMDL = args;
		
	}
	
	public StandaloneGUI(String[] args) {
		super(args);
		thisFrame = this;
		if (args.length > 0) {
			this.logfileTag = args[0];
			System.out.println("MF: "+this.logfileTag);
			for (int i = 0 ; i < args.length; i++) {
				System.out.println("MF: arg "+args[i]);
			}
		}
		if (args.length > 1) {
			this.preselectedMDL = args[1];
		}
		
	}
	
	public StandaloneGUI() {
		super();
		thisFrame = this;
	}
	
	public static boolean threeDEnabled() {
		return THREE_D_ENABLED;
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event-dispatching thread.
	 */
	/* private static */void createAndShowGUI() {

		
		
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (UnsupportedLookAndFeelException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		System.out.println("-cp "+System.getProperty("java.class.path"));
		
		SDMDocLanguageSpecific.initialize();

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		UIManager.put("OptionPane.messageFont", new Font(SDMDocFont.FONT, Font.PLAIN, SDMDocFont.MEDIUM));
		UIManager.put("OptionPane.buttonFont", new Font(SDMDocFont.FONT, Font.PLAIN, SDMDocFont.MEDIUM));
		
		final JTextField statusbar = new JTextField("");
		statusbar.setAutoscrolls(true);

		//

		// Create and set up the window.
//		final JFrame frame = thisFrame;
		
		final SDMDocJFrame frame = new SDMDocJFrame(
				language.getText(StringConstants.SDM_DOC_TOOL) + "/Java "
						+ language.getText(StringConstants.VERSION) + version
						+ (System.getProperty("sun.arch.data.model").equals("32") ? " (x86) " : " (x64) ")
						+ " -- " + language.getDisplayName(), "MainWindow1800.png");
		frame.setPreferredSize(new SdmDimension(900, 350)); // 750, 300
		
//		frame.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		if (preselectedMDL == null) {
			frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE); // JFrame.EXIT_ON_CLOSE
		} else {
			frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);	// HIDE_ON_CLOSE
		}
		
		frame.addWindowListener(new WindowAdapter() {
			  public void windowClosing(WindowEvent we) {
				  if (InformationManagers.getInstance() != null && InformationManagers.getInstance().getUserSelectedOptions() != null)
						InformationManagers.getInstance().getUserSelectedOptions().close();
				  closeMessageFrameLogs();
				  	System.exit(0);
			  }
			});
		

		JPanel panel = new JPanel();

		final JButtonLanguage genDocButton;
//		genDocButton = new JButton(
//				language.getText(StringConstants.GENERATE_DOCUMENT));
		
		genDocButton = new JButtonLanguage(StringConstants.GENERATE_DOCUMENT);

		final JButtonLanguage fileChooser;

		fileChooser = new JButtonLanguage(StringConstants.BROWSE);
		// The options panel

		UserSelectedOptions uso = new UserSelectedOptions("data/SDM-Doc_UserOptions.tab" /* load/save to file */);
		
		final OptionsFrame optionsFrame = new OptionsFrame(uso, genDocButton, fileChooser, "ModelOptionsPanel.png");
		final AboutFrame aboutFrame = new AboutFrame();
		final CodeGenerationOptionsFrame cgOptionsFrame = new CodeGenerationOptionsFrame(uso);
		
		final NamingOptionsPanel namingOptionsPanel;
		final ReuseOptionsPanel reuseOptionsPanel;
		final AvailableResultsPanel availableResultsPanel;
		
		namingOptionsPanel = new NamingOptionsPanel(uso, StringConstants.NAMING_OPTIONS);
		
		reuseOptionsPanel = new ReuseOptionsPanel(uso, StringConstants.REUSE);
		availableResultsPanel = new AvailableResultsPanel(uso, StringConstants.AVAILABLE_FOR_REUSE, this, reuseOptionsPanel);		
		
		GridBagLayout grid = new GridBagLayout();
		panel.setLayout(grid);
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;

		final JFrame thisFrame = this;

//		final JMenuBar menubar = new JMenuBar();

		final JMenuLanguage file = new JMenuLanguage(StringConstants.FILE);
		final JMenuLanguage lang = new JMenuLanguage(StringConstants.LANGUAGE);
		final JMenuLanguage tools = new JMenuLanguage(StringConstants.TOOLS);
		
		file.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		lang.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		tools.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		final JMenuItemLanguage extractCommentsMenuItem = new JMenuItemLanguage(
				StringConstants.EXTRACT_COMMENTS);
		tools.add(extractCommentsMenuItem);
		extractCommentsMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		extractCommentsMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				CommentCleanerDialog ccf = new CommentCleanerDialog(thisFrame,
						StringConstants.COMMENT_CLEANER,
						StringConstants.EXTRACT_COMMENTS, "ExtractComments.png");
			}
		});

		final JMenuItemLanguage mergeCommentsMenuItem = new JMenuItemLanguage(
				StringConstants.MERGE_COMMENTS);
		tools.add(mergeCommentsMenuItem);
		mergeCommentsMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		mergeCommentsMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				CommentCleanerDialog ccf = new CommentCleanerDialog(thisFrame,
						StringConstants.COMMENT_CLEANER,
						StringConstants.MERGE_COMMENTS, "MergeComments.png");
			}
		});
		
		
		  final JMenuItemLanguage extractVariableNamesMenuItem = new JMenuItemLanguage(
				StringConstants.EXTRACT_VARIABLE_NAMES);
		tools.add(extractVariableNamesMenuItem);
		extractVariableNamesMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		extractVariableNamesMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				VariableNameTranslatorDialog ccf = new VariableNameTranslatorDialog(thisFrame,
						StringConstants.VARIABLE_NAMES_TRANSLATOR,
						StringConstants.EXTRACT_VARIABLE_NAMES, "ExtractVariables.png");
			}
		});

		
		final JMenuItemLanguage translateVariableNamesMenuItem = new JMenuItemLanguage(
				StringConstants.REPLACE_VARIABLE_NAMES);
		tools.add(translateVariableNamesMenuItem);
		translateVariableNamesMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		translateVariableNamesMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				VariableNameTranslatorDialog ccf = new VariableNameTranslatorDialog(thisFrame,
						StringConstants.VARIABLE_NAMES_TRANSLATOR,
						StringConstants.REPLACE_VARIABLE_NAMES, "ReplaceVariables.png");
			}
		});
		
		final JMenuItemLanguage adjacencyToMapMenuItem = new JMenuItemLanguage(
				StringConstants.ADJACENCY_TO_MAP);
		tools.add(adjacencyToMapMenuItem);
		adjacencyToMapMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		adjacencyToMapMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				AdjacencyToMapDialog ccf = new AdjacencyToMapDialog(thisFrame,
						StringConstants.ADJACENCY_TO_MAP, "AdjMatrixToMap.png");
			}
		});
		
		final JMenuItemLanguage makeCopiesMenuItem = new JMenuItemLanguage(
				StringConstants.GENERATE_MODEL_COPIES);
		tools.add(makeCopiesMenuItem);
		makeCopiesMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		makeCopiesMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				ModelCopierDialog ccf = new ModelCopierDialog(thisFrame, StringConstants.GENERATE_MODEL_COPIES,
						"ModelCopies.png");
			}
		});
		
		final JMenuItemLanguage loopAnalysisMenuItem = new JMenuItemLanguage(
				StringConstants.RUN_LOOP_LENGTH_ANALYSIS);
		tools.add(loopAnalysisMenuItem);
		loopAnalysisMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		loopAnalysisMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LoopLengthAnalysisToolDialog ccf = new LoopLengthAnalysisToolDialog(thisFrame, StringConstants.RUN_LOOP_LENGTH_ANALYSIS,
						"LoopLengthAnalysis.png");
			}
		});
		 

		final JMenuItemLanguage exitMenuItem = new JMenuItemLanguage(
				StringConstants.EXIT);
		
		exitMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		exitMenuItem.setToolTipText(language
				.getText(StringConstants.EXIT_APPLICATION));

		exitMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				if (preselectedMDL == null) {
				} else {
					thisFrame.setVisible(false);
				}
			
				if (InformationManagers.getInstance() != null && InformationManagers.getInstance().getUserSelectedOptions() != null)
					InformationManagers.getInstance().getUserSelectedOptions().close();
				closeMessageFrameLogs();
				System.exit(0);
				
			}
		});

		file.add(exitMenuItem);

		final JMenuItemLanguage optionsMenuItem = new JMenuItemLanguage(StringConstants.OPTIONS);
		optionsMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		// optionsMenuItem.setToolTipText(language.getText("SelectUserOptions"));
		optionsMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				optionsFrame.changeLanguage();
				optionsFrame.update(null);
				optionsFrame.showPanel();
			}
		});
		
		file.add(optionsMenuItem);
		
		// cgOptionsFrame
		final JMenuItemLanguage cgOptionsMenuItem = new JMenuItemLanguage(
				/*StringConstants.OPTIONS*/ StringConstants.CODE_GENERATION_OPTIONS);

		// optionsMenuItem.setToolTipText(language.getText("SelectUserOptions"));
		cgOptionsMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				cgOptionsFrame.changeLanguage();
				cgOptionsFrame.update(null);
				cgOptionsFrame.showPanel();
			}
		});
		
		// removed code generation options from menu
		
//		file.add(cgOptionsMenuItem);

		genDocButton.setBounds(50, 60, 80, 30);
		genDocButton.setAlignmentX(Component.CENTER_ALIGNMENT);

		genDocButton.setEnabled(false);

		final JLabel label = new JLabelLanguage(
				StringConstants.DEFINITION_INPUT_FILE);
		label.setFont(language.getFont(Font.BOLD, 18));

		final JMenuLanguage help = new JMenuLanguage(language.getText(StringConstants.HELP));
		help.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		final JMenuItemLanguage aboutMenuItem = new JMenuItemLanguage(
				StringConstants.ABOUT);
		final JMenuItemLanguage feedbackMenuItem = new JMenuItemLanguage(
				StringConstants.FEEDBACK);
		final JMenuItemLanguage acknowledgementsMenuItem = new JMenuItemLanguage(
				StringConstants.ACKNOWLEDGEMENTS);
		
		aboutMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		feedbackMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		acknowledgementsMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		// aboutMenuItem.setToolTipText(language.getText("About"));
		aboutMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {

				aboutFrame.changeLanguage();
				aboutFrame.update(null);
				aboutFrame.showPanel();

				// changeLanguage(frame, file, exitMenuItem, optionsMenuItem,
				// help, aboutMenuItem, genDocButton, label, fileChooser);

			}
		});

		feedbackMenuItem.setToolTipText(StringConstants.FEEDBACK);
		feedbackMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific
						.getInstance();
				JOptionPane
				.showMessageDialog(
						null,
						
						language.getText(StringConstants.FEEDBACK_EMAIL)+
						"\n\n                 Ignacio Martinez-Moyano  martinez-moyano@anl.gov",
						language.getText(StringConstants.FEEDBACK),
						JOptionPane.INFORMATION_MESSAGE);

//				BrowserControl.displayURL(FEEDBACK_URL);
				// http://www.dis.anl.gov/feedback/sys_dyn_model.php?tname=your%20name&temail=your%20email&tphone=&tlocation=&tversion=5.0
			}
		});

		final AcknowledgementsFrame af = new AcknowledgementsFrame();

		acknowledgementsMenuItem.setToolTipText(language
				.getText(StringConstants.ACKNOWLEDGEMENTS));
		acknowledgementsMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				af.showPanel();
			}
		});

		help.add(aboutMenuItem);
		help.add(feedbackMenuItem);
		help.add(acknowledgementsMenuItem);

		frame.myMenubar.add(file);
		frame.myMenubar.add(lang);
		frame.myMenubar.add(tools);
		frame.myMenubar.add(help);
		
		if (frame.getHelpFile() != null) {
			addInformationMenu(frame);
			
		} 

//		frame.setJMenuBar(myMenubar);

		genDocButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific
						.getInstance();
//				final MDLProcessor sdmDoc = null;
				boolean internalError = false;
				
				namingOptionsPanel.saveValues();
				reuseOptionsPanel.saveValues();

				if (statusbar.getText().length() == 0) {
					JOptionPane.showMessageDialog(null,
							language.getText(StringConstants.PLEASE_SPECIFY_SYSTEM_DYNAMICS_MODEL_FILENAME),
							language.getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
				} else if (!new File(statusbar.getText()).exists()) {
					JOptionPane.showMessageDialog(null,
							language.getText(StringConstants.SPECIFIED_SYSTEM_DYNAMICS_MODEL_NOT_FOUND),
							language.getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
				} else {
					try {
						
						frame.setCursor(Cursor
								.getPredefinedCursor(Cursor.WAIT_CURSOR));
						messageFrame = new MessageFrame(logfileTag);
						
						System.out.println("GUI: logfileTag <"+logfileTag+">");
						
						messageFrame.addTimeStamp();
						
						String originalModelFile = statusbar.getText();
						String modelFile = statusbar.getText();
						
						boolean xmileOrigin = false;
						
						if (Utilities.isOfXmlOrigin(modelFile)) {
						
//						if (Utilities.stringEndsWith(modelFile, "\\.[xX][mM][iI][lL][eE]$")) {
							xmileOrigin = true;
							messageFrame.addMessage(" ### "+language.getText(StringConstants.TRANSLATING)+" "
									+ new File(modelFile).getName()
									+ " ###\n");
							XmileReader xmileReader = new XmileReader(modelFile);
							xmileReader.readAndTranslate();
//							modelFile = modelFile.replaceAll("\\.[xX][mM][iI][lL][eE]$","_xmile.mdl");
							modelFile = Utilities.getConvertedFileName(modelFile);
						}
						
						

						messageFrame.addMessage(" ### "
								+ new File(modelFile).getName()
								+ " ###\n");

						messageFrame.addMessage(language
								.getText(StringConstants.CONSTRUCTOR) + "...\n");

						messageFrame.setBlipMode(true);
						
						errorFrame = new MessageFrame(logfileTag.replaceAll(".txt$","_error.txt"),600,200);
						errorFrame.setVisible(false);
						errorFrame.addMessageNPNoView("### "+StringConstants.ERROR_MESSAGES+" ###");
						errorFrame.hidePanel();
						
						String obj = Utilities.makeLegalJavaName(new File(modelFile).getName().replace(".MDL", "").replace(".mdl", "").toUpperCase());
						
						String root = new File(statusbar.getText()).getParent();

						final MDLProcessor sdmDoc = new MDLProcessor(originalModelFile, modelFile, obj, root, messageFrame, errorFrame, optionsFrame.getUserSelectedOptions());
						
//						sdmDoc.setInformationManagers(InformationManagers.getInstance());
//						sdmDoc.setXmileOrigin(xmileOrigin);
						
						
						sdmDocHtml = sdmDoc;
						messageFrame.setBlipMode(false);
						messageFrame.addMessage(language
								.getText(StringConstants.EXECUTING) + "...\n");
						
						

						try {
							messageFrame.setBlipMode(true);
							final MessageProcessor msgProc = messageFrame;

							SwingWorker<Void, Void> worker = new SwingWorker<Void, Void>() {
								@Override
								protected Void doInBackground() throws Exception {

									
									try {
										String out = sdmDoc.getHTMLFileName();
										File f = new File(out);
										if (f.exists())
											f.delete();
										sdmDoc.process(false);
									} catch (Exception e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									String out = sdmDoc.getHTMLFileName();
										if (!new File(out).exists()) {
										JOptionPane.showMessageDialog(null,
												SDMDocLanguageSpecific.getInstance().
												getText(StringConstants.HTML_REPORT_NOT_GENERATED),
												SDMDocLanguageSpecific.getInstance().
												getText(StringConstants.GENERATE_DOCUMENT), JOptionPane.ERROR_MESSAGE);
									} else {
										if (Desktop.isDesktopSupported()) {
											msgProc.addMessage(SDMDocLanguageSpecific.getInstance().
													getText(StringConstants.OPENING_HTML_IN_BROWSER));
											try {
												Desktop.getDesktop().browse(
														new File(out).toURI());
											} catch (IOException e) {
												// TODO Auto-generated catch block
												e.printStackTrace();
											}
										}
									} 
										
										return null;

								}
								

							};


							worker.execute();
							
						} catch (Exception e) {

							String errMsg = e.toString();
							String myMsg = "\n\n"+
									language.getText(StringConstants.JAVA_ERROR_MESSAGES)+"\n\n";
							log.log(Level.FATAL, "Internal Error Detected", e);
							messageFrame.addMessage(errMsg + "\n");
							messageFrame.setBlipMode(false);
							JOptionPane.showMessageDialog(null, errMsg + myMsg,
									language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
									JOptionPane.ERROR_MESSAGE);
							e.printStackTrace();
							internalError = true;
							messageFrame.setBlipMode(false);
							
						}
					} finally {
						frame.setCursor(Cursor
								.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					}
				}
			}
		});

		fileChooser.setBounds(50, 60, 80, 30);
		fileChooser.setAlignmentX(Component.CENTER_ALIGNMENT);

		// Create a file chooser that opens up as an Open dialog
		fileChooser.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter("Model Files", RECOGNIZED_FILE_TYPES);
//						"mdl", "xmile", "stmx");
//				FileFilter filter = new FileNameExtensionFilter("Model Files",
//						"mdl");
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(0, 200);

				String savedValue = statusbar.getText();

				int option = chooser.showOpenDialog(fileChooser);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					saveFCDir(dir);
					String path = sf.getAbsolutePath();
					genDocButton.setEnabled(true);
					statusbar.setText(path);
					availableResultsPanel.setMdlSpecific(statusbar.getText());
					reuseOptionsPanel.setMdlSpecific(statusbar.getText(), availableResultsPanel.getSelected());
				} else {
					statusbar.setText(savedValue);
					if (savedValue.length() == 0)
						genDocButton.setEnabled(false);
					availableResultsPanel.setMdlSpecific(statusbar.getText());
					if (statusbar.getText().trim().length() == 0)
						reuseOptionsPanel.setValues();
					else
						reuseOptionsPanel.setMdlSpecific(statusbar.getText(), availableResultsPanel.getSelected());
				}
			}
		});
		
		if (preselectedMDL != null) {
			statusbar.setText(preselectedMDL);
			genDocButton.setEnabled(true);
			fileChooser.setEnabled(false);
			availableResultsPanel.setMdlSpecific(statusbar.getText());
			reuseOptionsPanel.setMdlSpecific(statusbar.getText(), availableResultsPanel.getSelected());
			
		}

		JMenuItemLanguage langRegMenuItem = new JMenuItemLanguage(
				StringConstants.REGISTER_LANGUAGE);

		langRegMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageRegistrationDialog lrf = new LanguageRegistrationDialog(
						thisFrame, "LanguageRegistration.png");
				// lrf.setVisible(true);
				initializeLanguageMenu(frame, file, exitMenuItem,
						optionsMenuItem, acknowledgementsMenuItem, lang, help,
						aboutMenuItem, feedbackMenuItem, genDocButton, label,
						fileChooser, optionsFrame, cgOptionsFrame,  af, tools, 
						extractCommentsMenuItem, mergeCommentsMenuItem,
						namingOptionsPanel,
						reuseOptionsPanel,
						availableResultsPanel,
						extractVariableNamesMenuItem, 
						translateVariableNamesMenuItem,
						adjacencyToMapMenuItem,
						makeCopiesMenuItem,
						cgOptionsMenuItem,
						loopAnalysisMenuItem, information
						);
			}
		});
		
		
		JMenuItemLanguage langAdminMenuItem = new JMenuItemLanguage(
				StringConstants.LANGUAGE_ADMINISTRATION);
		
		langAdminMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageAdministrationDialog lad = new LanguageAdministrationDialog(
						thisFrame);
				// lrf.setVisible(true);
				initializeLanguageMenu(frame, file, exitMenuItem,
						optionsMenuItem, acknowledgementsMenuItem, lang, help,
						aboutMenuItem, feedbackMenuItem, genDocButton, label,
						fileChooser, optionsFrame, cgOptionsFrame,  af, tools, 
						extractCommentsMenuItem, mergeCommentsMenuItem,
						namingOptionsPanel,
						reuseOptionsPanel,
						availableResultsPanel,
						extractVariableNamesMenuItem, 
						translateVariableNamesMenuItem,
						adjacencyToMapMenuItem,
						makeCopiesMenuItem,
						cgOptionsMenuItem,
						loopAnalysisMenuItem, information);
			}
		});
		
		
		JMenuItemLanguage langTranslationMenuItem = new JMenuItemLanguage(
				StringConstants.LANGUAGE_TRANSLATION_REPORT);
		
		langTranslationMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageTranslationReportDialog lrf = new LanguageTranslationReportDialog(
						thisFrame);
				// lrf.setVisible(true);
				initializeLanguageMenu(frame, file, exitMenuItem,
						optionsMenuItem, acknowledgementsMenuItem, lang, help,
						aboutMenuItem, feedbackMenuItem, genDocButton, label,
						fileChooser, optionsFrame, cgOptionsFrame,  af, tools, 
						extractCommentsMenuItem, mergeCommentsMenuItem,
						namingOptionsPanel,
						reuseOptionsPanel,
						availableResultsPanel,
						extractVariableNamesMenuItem, 
						translateVariableNamesMenuItem,
						adjacencyToMapMenuItem,
						makeCopiesMenuItem,
						cgOptionsMenuItem,
						loopAnalysisMenuItem, information);
			}
		});
		
		lang.add(langAdminMenuItem);
		lang.add(langTranslationMenuItem);
		lang.add(langRegMenuItem);
		
		langAdminMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		langTranslationMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		langRegMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		

		initializeLanguageMenu(frame, file, exitMenuItem, optionsMenuItem,
				acknowledgementsMenuItem, lang, help, aboutMenuItem,
				feedbackMenuItem, genDocButton, label, fileChooser,
				optionsFrame, cgOptionsFrame, af, tools, extractCommentsMenuItem, mergeCommentsMenuItem,
				namingOptionsPanel,
				reuseOptionsPanel,
				availableResultsPanel,
				extractVariableNamesMenuItem, translateVariableNamesMenuItem,adjacencyToMapMenuItem,
				makeCopiesMenuItem,
				cgOptionsMenuItem,
				loopAnalysisMenuItem, information);

		c.insets = new Insets(0, 0, 0, 0); // top padding
		c.fill = GridBagConstraints.CENTER;
		c.weightx = 1.0;
		c.weighty = 1.0;
		// c.fill = GridBagConstraints.HORIZONTAL;

		c.gridwidth = 3;
		c.gridx = 1;
		c.gridy = 0;
		panel.add(label, c);

		c.gridwidth = 3;
		c.gridx = 1;
		c.gridy = 1;
		panel.add(statusbar, c);
		statusbar.setPreferredSize(new SdmDimension(660, 25));
		statusbar.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 2;
		if (preselectedMDL == null) {
		panel.add(fileChooser, c);
		fileChooser.setPreferredSize(new SdmDimension(200, 25));
		fileChooser.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		}

		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 3;
		panel.add(genDocButton, c);
		genDocButton.setPreferredSize(new SdmDimension(200, 25));
		genDocButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 4;
		panel.add(namingOptionsPanel, c);
		
		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 5;
		panel.add(availableResultsPanel, c);
		
		c.gridwidth = 1;
		c.gridx = 2;
		c.gridy = 6;
		panel.add(reuseOptionsPanel, c);

		frame.getContentPane().add(panel);

		// Display the window.
		frame.pack();
		frame.setVisible(true);
		frame.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		runAnyTestCode();
	}
	
	private void runAnyTestCode() {
//		List<String> col1 = new ArrayList<String>();
//		List<String> col2 = new ArrayList<String>();
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		for (String s : StringConstants.ALL_STRING_CONSTANTS) {
//			col1.add(s);
//			col2.add(language.getText(s));
//		}
//		CommentProcessor cp = new CommentProcessor();
//		cp.createTable(col1, col2);
	}
	
//	private void addInformationMenu(SDMDocJFrame frame) {
//		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		menubar.add(Box.createHorizontalGlue());
//		final JMenuLanguage information = new JMenuLanguage(StringConstants.INFORMATION);
//		information.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
//		final JMenuItemLanguage screen = new JMenuItemLanguage(StringConstants.SCREEN);
//		screen.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
//		information.add(screen);
//		
//		final SDMDocJFrame ff = frame;
//		
//		screen.addActionListener(new ActionListener() {
//			@Override
//			public void actionPerformed(ActionEvent event) {
//				ImageDisplay disp = new ImageDisplay(ff.getHelpFile());
//			}
//		});
//		
//		menubar.add(information);
//	}
	
	private void closeMessageFrameLogs() {
		if (messageFrame != null)
			messageFrame.closeLog();
		if (errorFrame != null)
			errorFrame.closeLog();
	}
	


	private void initializeLanguageMenu(SDMDocJFrame frame, JMenuLanguage file,
			JMenuItemLanguage exitMenuItem, JMenuItemLanguage optionsMenuItem,
			JMenuItemLanguage acknowledgementsMenuItem, JMenuLanguage lang, JMenuLanguage help,
			JMenuItemLanguage aboutMenuItem, JMenuItemLanguage feedbackMenuItem,
			JButtonLanguage genDocButton, JLabel screenTitle, JButtonLanguage fileChooser,
			OptionsFrame optionsFrame, CodeGenerationOptionsFrame cgOptionsFrame, AcknowledgementsFrame af, JMenuLanguage tools, 
			JMenuItemLanguage extractCommentsMenuItem, JMenuItemLanguage mergeCommentsMenuItem,
			NamingOptionsPanel namingOptionsPanel,
			ReuseOptionsPanel reuseOptionsPanel,
			AvailableResultsPanel availableResultsPanel,
			JMenuItemLanguage extractVariableNamesMenuItem, JMenuItemLanguage translateVariableNamesMenuItem,
			JMenuItemLanguage adjacencyToMapMenuItem,
			JMenuItemLanguage makeCopiesMenuItem,
			JMenuItemLanguage cgOptionsMenuItem,
			JMenuItemLanguage runLoopAnalysisMenuItem, JButtonLanguage information) {
		final SDMDocJFrame fframe = frame;
		final JMenuLanguage ffile = file;
		
		final JMenuLanguage ftools = tools;
		final JMenuItemLanguage fextractCommentsMenuItem = extractCommentsMenuItem;
		final JMenuItemLanguage fmergeCommentsMenuItem = mergeCommentsMenuItem;
		
		final JMenuItemLanguage fextractVariableNamesMenuItem = extractVariableNamesMenuItem;
		final JMenuItemLanguage ftranslateVariableNamesMenuItem = translateVariableNamesMenuItem;
		final JMenuItemLanguage fadjacencyToMapMenuItem = adjacencyToMapMenuItem;
		final JMenuItemLanguage fmakeCopiesMenuItem = makeCopiesMenuItem;
		
		final JMenuItemLanguage fexitMenuItem = exitMenuItem;
		final JMenuItemLanguage foptionsMenuItem = optionsMenuItem;
		final JMenuLanguage flang = lang;
		final JMenuLanguage fhelp = help;
		final JMenuItemLanguage faboutMenuItem = aboutMenuItem;
		final JMenuItemLanguage ffeedbackMenuItem = feedbackMenuItem;
		final JMenuItemLanguage facknowledgementsMenuItem = acknowledgementsMenuItem;
		
		final JMenuItemLanguage fcgOptionsMenuItem = cgOptionsMenuItem;
		final JMenuItemLanguage frunLoopAnalysisMenuItem = runLoopAnalysisMenuItem;
		
		final JButtonLanguage fgenDocButton = genDocButton;
		final JLabel fscreenTitle = screenTitle;
		final JButtonLanguage ffileChooser = fileChooser;
		final JButtonLanguage finformation = information;
		final OptionsFrame foptionsFrame = optionsFrame;
		final CodeGenerationOptionsFrame fcgOptionsFrame = cgOptionsFrame;
		final AcknowledgementsFrame faf = af;
		
		final NamingOptionsPanel fnamingOptionsPanel = namingOptionsPanel;
		final ReuseOptionsPanel freuseOptionsPanel = reuseOptionsPanel;
		final AvailableResultsPanel favailableResultsPanel = availableResultsPanel;

		lang.removeAll();

		JMenuItemLanguage langRegMenuItem = new JMenuItemLanguage(
				StringConstants.REGISTER_LANGUAGE);
		final JFrame thisFrame = this;
		langRegMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageRegistrationDialog lrf = new LanguageRegistrationDialog(
						thisFrame, "LanguageRegistration.png");
				// lrf.setVisible(true);
				initializeLanguageMenu(fframe, ffile, fexitMenuItem,
						foptionsMenuItem, facknowledgementsMenuItem, flang,
						fhelp, faboutMenuItem, ffeedbackMenuItem,
						fgenDocButton, fscreenTitle, ffileChooser,
						foptionsFrame, fcgOptionsFrame, faf, ftools, fextractCommentsMenuItem, fmergeCommentsMenuItem,
						fnamingOptionsPanel,
						freuseOptionsPanel,
						favailableResultsPanel,
						fextractVariableNamesMenuItem, ftranslateVariableNamesMenuItem,
						fadjacencyToMapMenuItem,
						fmakeCopiesMenuItem, fcgOptionsMenuItem, frunLoopAnalysisMenuItem, finformation);
				
			}
		});
		
		
		JMenuItemLanguage langAdminMenuItem = new JMenuItemLanguage(
				StringConstants.LANGUAGE_ADMINISTRATION);
		
		langAdminMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageAdministrationDialog lad = new LanguageAdministrationDialog(
						thisFrame, "LanguageAdministration.png");
				// lrf.setVisible(true);
				initializeLanguageMenu(fframe, ffile, fexitMenuItem,
						foptionsMenuItem, facknowledgementsMenuItem, flang,
						fhelp, faboutMenuItem, ffeedbackMenuItem,
						fgenDocButton, fscreenTitle, ffileChooser,
						foptionsFrame, fcgOptionsFrame, faf, ftools, fextractCommentsMenuItem, fmergeCommentsMenuItem,
						fnamingOptionsPanel,
						freuseOptionsPanel,
						favailableResultsPanel,
						fextractVariableNamesMenuItem, ftranslateVariableNamesMenuItem,
						fadjacencyToMapMenuItem,
						fmakeCopiesMenuItem,
						fcgOptionsMenuItem,
						frunLoopAnalysisMenuItem, finformation);
			}
		});
		
		
		JMenuItemLanguage langTranslationMenuItem = new JMenuItemLanguage(
				StringConstants.LANGUAGE_TRANSLATION_REPORT);
		
		langTranslationMenuItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent event) {
				LanguageTranslationReportDialog lrf = new LanguageTranslationReportDialog(
						thisFrame, "LanguageTranslationReport.png");
				// lrf.setVisible(true);
				initializeLanguageMenu(fframe, ffile, fexitMenuItem,
						foptionsMenuItem, facknowledgementsMenuItem, flang,
						fhelp, faboutMenuItem, ffeedbackMenuItem,
						fgenDocButton, fscreenTitle, ffileChooser,
						foptionsFrame, fcgOptionsFrame, faf, ftools, fextractCommentsMenuItem, fmergeCommentsMenuItem,
						fnamingOptionsPanel,
						freuseOptionsPanel,
						favailableResultsPanel,
						fextractVariableNamesMenuItem, ftranslateVariableNamesMenuItem,
						fadjacencyToMapMenuItem,
						fmakeCopiesMenuItem,
						fcgOptionsMenuItem,
						frunLoopAnalysisMenuItem, finformation);
			}
		});
		
		lang.add(langAdminMenuItem);
		lang.add(langTranslationMenuItem);
		lang.add(langRegMenuItem);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		langAdminMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		langTranslationMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		langRegMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		for (String l : SDMDocLanguageSpecific.getKnownDisplayNames()) {
			JMenuItemSpecificLanguage langMenuItem = new JMenuItemSpecificLanguage(l);
			final String thisLang = langMenuItem.getLanguage();
			langMenuItem.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent event) {

					// here we assign the new language to be used
					boolean setAsDefault = true;
					SDMDocLanguageSpecific.getInstance(thisLang, setAsDefault, thisFrame);

					SDMDocLanguageSpecific language = SDMDocLanguageSpecific
							.getInstance();
					System.out.println("language select menu item " + thisLang
							+ " instance = " + language.getDisplayName());

					changeLanguage(fframe, ffile, fexitMenuItem,
							foptionsMenuItem, facknowledgementsMenuItem, flang,
							fhelp, faboutMenuItem, ffeedbackMenuItem,
							fgenDocButton, fscreenTitle, ffileChooser,
							foptionsFrame, fcgOptionsFrame, faf, ftools, 
							fextractCommentsMenuItem, fmergeCommentsMenuItem,
							fnamingOptionsPanel,
							freuseOptionsPanel,
							favailableResultsPanel,
							fextractVariableNamesMenuItem, 
							ftranslateVariableNamesMenuItem,
							fadjacencyToMapMenuItem,
							fmakeCopiesMenuItem,
							fcgOptionsMenuItem,
							frunLoopAnalysisMenuItem,
							finformation);

				}
			});
			lang.add(langMenuItem);
			langMenuItem.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		}
	}

	public void actionPerformed(ActionEvent e) {
		if ("disable".equals(e.getActionCommand())) {

		} else {

		}
	}
	
	public static void viaSDMUI(String file) {
		final StandaloneGUI gui = new StandaloneGUI(file);
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}
	
	public static void importSDMDoc(String file) {
		final StandaloneGUI gui = new StandaloneGUI(file);
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}

	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		
		String log4jConfPath = "./data/log4j.properties";
		PropertyConfigurator.configure(log4jConfPath);
		StaticSwitches.initialize();
		SDMDocFont.initialize();
		
		for (int i = 0; i < args.length; i++)
			System.out.println("*** "+i+" "+args[i]);
			
		final StandaloneGUI gui = new StandaloneGUI(args);
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				gui.createAndShowGUI();
			}
		});
	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void changeLanguage(SDMDocJFrame frame, JMenuLanguage file,
			JMenuItemLanguage exitMenuItem, JMenuItemLanguage optionsMenuItem,
			JMenuItemLanguage acknowledgementsMenuItem, JMenuLanguage lang, JMenuLanguage help,
			JMenuItemLanguage aboutMenuItem, JMenuItemLanguage feedbackMenuItem,
			JButtonLanguage genDocButton, JLabel screenTitle, JButtonLanguage fileChooser,
			OptionsFrame optionsFrame, CodeGenerationOptionsFrame cgOptionsFrame, 
			AcknowledgementsFrame af, JMenuLanguage tools, 
			JMenuItemLanguage extractCommentsMenuItem, JMenuItemLanguage mergeCommentsMenuItem,
			NamingOptionsPanel namingOptionsPanel,
			ReuseOptionsPanel reuseOptionsPanel,
			AvailableResultsPanel availableResultsPanel,
			JMenuItemLanguage extractVariableNamesMenuItem,
			JMenuItemLanguage translateVariableNamesMenuItem,
			JMenuItemLanguage adjacencyToMapMenuItem,
			JMenuItemLanguage makeCopiesMenuItem,
			JMenuItemLanguage cgOptionsMenuItem,
			JMenuItemLanguage loopAnalysisMenuItem,
			JButtonLanguage information
			) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		System.out.println("changeLanguage displayName "
				+ language.getDisplayName());

		screenTitle.setFont(language.getFont(Font.BOLD, SDMDocFont.LARGE));
		fileChooser.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		genDocButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		initializeLanguageMenu(frame, file, exitMenuItem, optionsMenuItem,
				acknowledgementsMenuItem, lang, help, aboutMenuItem,
				feedbackMenuItem, genDocButton, screenTitle, fileChooser,
				optionsFrame, cgOptionsFrame, af, tools, extractCommentsMenuItem, mergeCommentsMenuItem,
				namingOptionsPanel,
				reuseOptionsPanel,
				availableResultsPanel,
				extractVariableNamesMenuItem,
				translateVariableNamesMenuItem,
				adjacencyToMapMenuItem,
				makeCopiesMenuItem,
				cgOptionsMenuItem,
				loopAnalysisMenuItem, information
				);

		feedbackMenuItem.setText(StringConstants.FEEDBACK);
		lang.setText(StringConstants.LANGUAGE);

		frame.setTitle(language.getText(StringConstants.SDM_DOC_TOOL)
				+ "/Java " + language.getText(StringConstants.VERSION) + version
				+ " -- " + language.getDisplayName());
		frame.changeLanguage(language.getText(StringConstants.SDM_DOC_TOOL)
				+ "/Java " + language.getText(StringConstants.VERSION) + version
				+ " -- " + language.getDisplayName());
		
		frame.setFont(language.getFont(Font.PLAIN, SDMDocFont.LARGE));

		file.setText(StringConstants.FILE);

		exitMenuItem.setText(StringConstants.EXIT);
		exitMenuItem.setToolTipText(StringConstants.EXIT_APPLICATION);

		optionsMenuItem.setText(StringConstants.OPTIONS);
		optionsMenuItem.setToolTipText(StringConstants.SELECT_USER_OPTIONS);
		
		cgOptionsMenuItem.setText(StringConstants.CODE_GENERATION_OPTIONS);
		loopAnalysisMenuItem.setText(StringConstants.RUN_LOOP_LENGTH_ANALYSIS);
		
		mergeCommentsMenuItem.setText(StringConstants.MERGE_COMMENTS);
		extractCommentsMenuItem.setText(StringConstants.EXTRACT_COMMENTS);
		
		extractVariableNamesMenuItem.setText(StringConstants.EXTRACT_VARIABLE_NAMES);
		translateVariableNamesMenuItem.setText(StringConstants.REPLACE_VARIABLE_NAMES);
		adjacencyToMapMenuItem.setText(StringConstants.ADJACENCY_TO_MAP);
		makeCopiesMenuItem.setText(StringConstants.MAKE_COPIES);
		tools.setText(StringConstants.TOOLS);

		help.setText(StringConstants.HELP);

		aboutMenuItem.setText(StringConstants.ABOUT);
//		aboutMenuItem.setToolTipText(StringConstants.ABOUT);

		acknowledgementsMenuItem.setText(StringConstants.ACKNOWLEDGEMENTS);
//		acknowledgementsMenuItem.setToolTipText(StringConstants.ACKNOWLEDGEMENTS);

//		genDocButton.setText(language.getText(StringConstants.GENERATE_DOCUMENT));
		genDocButton.setText(StringConstants.GENERATE_DOCUMENT);
		screenTitle.setText(StringConstants.DEFINITION_INPUT_FILE);
		
		if (information != null)
			information.setText(StringConstants.INFORMATION);

		fileChooser.setText(StringConstants.BROWSE);

		af.changeLanguage();
		
		namingOptionsPanel.changeLanguage();
		reuseOptionsPanel.changeLanguage();
		availableResultsPanel.changeLanguage();

		frame.update(frame.getGraphics());
		optionsFrame.changeLanguage();
		optionsFrame.update(optionsFrame.getGraphics());
		


	}
	

}
