package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Dimension;
import java.awt.Font;
import java.util.LinkedHashMap;
import java.util.Map;

public class VariableInformationPanel extends SDMOptionsPanel {
	
	JCheckBoxLanguage description = new JCheckBoxLanguage(StringConstants.DESCRIPTION);
	JCheckBoxLanguage metedata = new JCheckBoxLanguage(StringConstants.META_DATA);
	JCheckBoxLanguage presentInViews = new JCheckBoxLanguage(StringConstants.PRESENT_IN_VIEWS);
	JCheckBoxLanguage usedBy = new JCheckBoxLanguage(StringConstants.USED_BY);
	JCheckBoxLanguage numberOfFeedbackLoops = new JCheckBoxLanguage(StringConstants.FEEDBACK_LOOPS);
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();

	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public VariableInformationPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);
		
		setPreferredSize(new SdmDimension(180, 140));

		loadMap();
		
		add(description);
		add(metedata);
		add(presentInViews);
		add(usedBy);
		add(numberOfFeedbackLoops);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		description.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		metedata.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		presentInViews.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		usedBy.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		numberOfFeedbackLoops.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		setValues();
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		description.setText(StringConstants.DESCRIPTION);
		metedata.setText(StringConstants.META_DATA);
		presentInViews.setText(StringConstants.PRESENT_IN_VIEWS);
		usedBy.setText(StringConstants.USED_BY);
		numberOfFeedbackLoops.setText(StringConstants.FEEDBACK_LOOPS);
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.VARIABLE_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.VARIABLE_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
		}
		return true;
	}
	
	private void loadMap() {
		

		mapNameToCheckBox.put(StringConstants.DESCRIPTION, description);
		mapNameToCheckBox.put(StringConstants.META_DATA, metedata);
		mapNameToCheckBox.put(StringConstants.PRESENT_IN_VIEWS, presentInViews);
		mapNameToCheckBox.put(StringConstants.USED_BY,usedBy);
		mapNameToCheckBox.put(StringConstants.FEEDBACK_LOOPS, numberOfFeedbackLoops);
		
		
	}
	
	
	public void setValues(boolean value) {
		for (String var : UserSelectedOptions.VARIABLE_INFORMATION_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}
