package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.VariableNameLanguageTranslator;

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.border.TitledBorder;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

public class VariableNameTranslatorPanel extends JPanel {
	
	static Logger log = Logger.getLogger(VariableNameTranslatorPanel.class.getSimpleName());

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private static final int TEXT_FIELD_SIZE = 60;
	private static String FC_DIR_FILE = "data/ChooserDir.txt";

	private static final String GENERATE_DOCX = "GenerateDocx";
	private static final String GENERATE_VENSIM = "GenerateVensim";

	private static final String CHECK_DEFAULT = "Check";
	private static final String DO_NOT_CHECK_DEFAULT = "DoNotCheck";
	
	JCheckBoxLanguage initializeTranslationCheckBox = new JCheckBoxLanguage(StringConstants.INITIALIZE_TRANSLATION);
	UserSelectedOptions uso;
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();

	private JLabel originalLabel = new JLabelLanguage(
			StringConstants.ORIGINAL_VENSIM_FILE);
	private JLabel extractedLabel = new JLabelLanguage(
			StringConstants.EXTRACTED_VARIABLE_NAMES_FILE);
	private JLabel mergedLabel = new JLabelLanguage(
			StringConstants.REPLACED_VARIABLE_NAMES_VENSIM_FILE);

	private JTextField originalText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField extractedText = new JTextField(TEXT_FIELD_SIZE);
	private JTextField mergedText = new JTextField(TEXT_FIELD_SIZE);

	private JButtonLanguage originalBrowseButton = new JButtonLanguage(StringConstants.BROWSE);
	private JButtonLanguage extractedBrowseButton = new JButtonLanguage(StringConstants.BROWSE);
	private JButtonLanguage mergedBrowseButton = new JButtonLanguage(StringConstants.BROWSE);

	private JButtonLanguage extractButton = new JButtonLanguage(StringConstants.EXTRACT_VARIABLE_NAMES);
	private JButtonLanguage translateButton = new JButtonLanguage(StringConstants.REPLACE_VARIABLE_NAMES);

	private JDialog parent;
	private String title;
	private boolean extractMode = true;
	private boolean translateMode;

	public VariableNameTranslatorPanel(JDialog parent, String title, String mode) {

		this.parent = parent;
		this.title = title;
		if (mode.equals(StringConstants.REPLACE_VARIABLE_NAMES))
			extractMode = false;
		translateMode = !extractMode;

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		initializeTranslationCheckBox.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		originalLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		extractedLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mergedLabel.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		originalText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		extractedText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mergedText.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		originalBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		extractedBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		mergedBrowseButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));

		extractButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		translateButton.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));


		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, SDMDocFont.MEDIUM), Color.black));

		createLayout();
		createListeners();
		setDependencies();
		
		loadMap();
		
		setValues();

	}

	private void createLayout() {
		setLayout(new GridBagLayout());
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
//		originalLabel.setFont(language.getFont(Font.BOLD, 12));
//		mergedLabel.setFont(language.getFont(Font.BOLD, 12));
//		extractedLabel.setFont(language.getFont(Font.BOLD, 12));

		int row = -1;

		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1.0;
		c.weighty = 1.0;
		c.gridwidth = 1;

		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(originalLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(originalText, c);
		c.gridx = 1;
		c.weightx = 0.0;
		add(originalBrowseButton, c);

		c.weightx = 1.0;
		c.gridy = ++row;
		c.gridx = 0;
		c.fill = GridBagConstraints.CENTER;
		add(extractedLabel, c);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridy = ++row;
		add(extractedText, c);
		c.gridx = 1;
		c.weightx = 1.0;
		add(extractedBrowseButton, c);
		

		if (translateMode) {
			c.weightx = 1.0;
			c.gridy = ++row;
			c.gridx = 0;
			c.fill = GridBagConstraints.CENTER;
			add(mergedLabel, c);
			c.fill = GridBagConstraints.HORIZONTAL;
			c.gridy = ++row;
			add(mergedText, c);
			c.gridx = 1;
			c.weightx = 0.0;
			add(mergedBrowseButton, c);
		} else {
			c.weightx = 1.0;
			c.gridy = ++row;
			c.gridx = 0;
			c.fill = GridBagConstraints.CENTER;
			add(initializeTranslationCheckBox, c);
		}

		c.fill = GridBagConstraints.NONE;

		c.gridwidth = 1;

		c.fill = GridBagConstraints.CENTER;

		if (extractMode) {
			c.gridy = ++row;
			c.gridx = 0;
			add(extractButton, c);
		}
		if (translateMode) {
			c.gridy = ++row;
			c.gridx = 0;
			add(translateButton, c);
		}

	}

	private void createListeners() {
		final String c1 = CHECK_DEFAULT;
		createBrowserListener(originalBrowseButton, originalText, "mdl", c1);
		final String c2 = DO_NOT_CHECK_DEFAULT;
		createBrowserListener(extractedBrowseButton, extractedText, "docx", c2);
		createBrowserListener(mergedBrowseButton, mergedText, "mdl", c2);

		extractButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {

				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				VariableNameLanguageTranslator vnlt = new VariableNameLanguageTranslator(GENERATE_DOCX,
						originalText.getText(), extractedText.getText(), initializeTranslationCheckBox.isSelected());
				try {
					vnlt.execute();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					JOptionPane
							.showMessageDialog(
									null,
									language.getText(StringConstants.VARIABLE_NAMES_HAVE_BEEN_EXTRACTED),
									language.getText(StringConstants.EXTRACT_VARIABLE_NAMES_FROM_VENSIM_FILE),
									JOptionPane.INFORMATION_MESSAGE);
				} catch (Exception e) {
					String errMsg = e.toString();
					setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
					
					log.log(Level.FATAL, "Internal Error Detected", e);
					JOptionPane.showMessageDialog(null, errMsg,
							language.getText(StringConstants.INTERNAL_ERROR_DETECTED),
							JOptionPane.ERROR_MESSAGE);
					e.printStackTrace();
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		});

		translateButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				
				SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				
				saveValues();
				
				VariableNameLanguageTranslator vnlt = new VariableNameLanguageTranslator(GENERATE_VENSIM,
						originalText.getText(), extractedText.getText(),
						mergedText.getText());
				vnlt.execute();
				setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
				JOptionPane
						.showMessageDialog(
								null,
								language.getText(StringConstants.VARIABLE_NAMES_HAVE_BEEN_REPLACED),
								language.getText(StringConstants.REPLACE_VARIABLE_NAMES_WITH_VENSIM_FILE),
								JOptionPane.INFORMATION_MESSAGE);
			}
		});

	}

	private void createBrowserListener(/*final*/ JButtonLanguage button,
			/*final*/ JTextField textField, /*final*/ String fileType,
			/*final*/  String checkDefault) {
		final JButtonLanguage fbutton = button;
		final String ffileType = fileType;
		final JTextField ftextField = textField;
		final String fcheckDefault = new String(checkDefault);
		fbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				FileFilter filter = new FileNameExtensionFilter(ffileType
						.toUpperCase() + " Files", ffileType.toLowerCase());
				JFileChooser chooser = new JFileChooser(loadFCDir());
				chooser.setLocale(SDMDocLanguageSpecific.getInstance()
						.getLocale());
				chooser.setFileFilter(filter);
				chooser.setMultiSelectionEnabled(false);
				chooser.setLocation(0, 200);

				String savedValue = ftextField.getText();

				int option = chooser.showOpenDialog(null);
				if (option == JFileChooser.APPROVE_OPTION) {
					File sf = chooser.getSelectedFile();
					String dir = sf.getParent();
					String filelist = sf.getName();
					 saveFCDir(dir);
					String path = sf.getAbsolutePath();
					ftextField.setText(path);
				} else {
					ftextField.setText(savedValue);
					if (savedValue.length() == 0) {
					}
				}
				setDefaults(fbutton, ftextField, fcheckDefault);
				setDependencies();
			}
		});
	}

	private void setDefaults(JButtonLanguage fbutton, JTextField textField, String checkDefaults) {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		if (!checkDefaults.equals(CHECK_DEFAULT))
			return;
		String orig = originalText.getText();
		if (orig.length() == 0)
			return;

		File aFile;

		String defaultExtracted = orig.replace(".MDL", ".docx").replace(".mdl",
				".docx");
		aFile = new File(defaultExtracted);
		if (!translateMode && fbutton.equals(originalBrowseButton) && aFile.exists()) {
			
			UIManager.put("OptionPane.yesButtonText",language
					.getText(StringConstants.YES));
			   UIManager.put("OptionPane.noButtonText", language
						.getText(StringConstants.NO));

			int dialogButton = JOptionPane.YES_NO_OPTION;
			int reply = JOptionPane
					.showConfirmDialog(
							null,
							language.getText(
											StringConstants.EXTRACTED_VARIABLE_NAMES_FILE_EXISTS_AND_WILL_BE_OVERWRITTEN)
									+ "\n"
									+ SDMDocLanguageSpecific.getInstance()
											.getText(StringConstants.OVERWRITE)+"?",
							language.getText(
									StringConstants.WARNING), dialogButton);

			if (reply == JOptionPane.YES_OPTION) {
				extractedText.setText(defaultExtracted);
			} else {
				extractedText.setText("");
			}
		} else {
			extractedText.setText(defaultExtracted);
		}

		if (translateMode) {
			String defaultMerged = orig.replace(".MDL", "_NEW.MDL").replace(
					".mdl", "_NEW.MDL");
			aFile = new File(defaultMerged);
			if (aFile.exists()) {
				int dialogButton = JOptionPane.YES_NO_OPTION;
				int reply = JOptionPane
						.showConfirmDialog(
								null,
								language.getText(
												StringConstants.REPLACED_VARIABLE_NAMES_FILE_EXISTS_AND_WILL_BE_OVERWRITTEN)
										+ "\n"
										+ language.getText(
														StringConstants.OVERWRITE)+"?",
								SDMDocLanguageSpecific.getInstance().getText(
										StringConstants.WARNING), dialogButton);

				if (reply == JOptionPane.YES_OPTION) {
					mergedText.setText(defaultMerged);
				} else {
					mergedText.setText("");
				}
			} else {
				mergedText.setText(defaultMerged);
			}
		}

	}

	private void setDependencies() {
		extractButton.setEnabled(false);
		translateButton.setEnabled(false);
		if (translateMode) {
			if (originalText.getText().length() > 0
					&& extractedText.getText().length() > 0
					&& mergedText.getText().length() > 0) {
				extractButton.setEnabled(true);
				translateButton.setEnabled(true);

			}
		}
		if (extractMode) {
			if (originalText.getText().length() > 0
					&& extractedText.getText().length() > 0) {
				extractButton.setEnabled(true);
			}
		}
	}

	public void changeLanguage() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		setBorder(BorderFactory.createTitledBorder(null,
				language.getText(title), TitledBorder.LEFT, TitledBorder.TOP,
				language.getFont(Font.BOLD, 12), Color.black));

		originalLabel.setText(SDMDocLanguageSpecific.getInstance().getText(
				StringConstants.ORIGINAL_VENSIM_FILE));
		extractedLabel.setText(SDMDocLanguageSpecific.getInstance().getText(
				StringConstants.EXTRACTED_VARIABLE_NAMES_FILE));
		mergedLabel.setText(SDMDocLanguageSpecific.getInstance().getText(
				StringConstants.REPLACED_VARIABLE_NAMES_VENSIM_FILE));

		originalBrowseButton.setText(SDMDocLanguageSpecific.getInstance()
				.getText(StringConstants.BROWSE));
		extractedBrowseButton.setText(SDMDocLanguageSpecific.getInstance()
				.getText(StringConstants.BROWSE));
		mergedBrowseButton.setText(SDMDocLanguageSpecific.getInstance()
				.getText(StringConstants.BROWSE));

		extractButton.setText(SDMDocLanguageSpecific.getInstance().getText(
				StringConstants.EXTRACT_COMMENTS));
		translateButton.setText(SDMDocLanguageSpecific.getInstance().getText(
				StringConstants.REPLACE_VARIABLE_NAMES));

	}

	private String loadFCDir() {

		String dir = ".";

		File file = new File(FC_DIR_FILE);
		if (!file.exists()) {
			BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
			try {
				bw.append(dir);
				bw.append("\n");
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		BufferedReader br = Utilities.openForRead(FC_DIR_FILE);
		for (String line : Utilities.getAsList(br)) {
			dir = line;
		}

		return dir;

	}

	private void saveFCDir(String dir) {
		BufferedWriter bw = Utilities.openFileForWriting(FC_DIR_FILE);
		try {
			bw.append(dir + "\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public void setValues() {
		
		for (String var : UserSelectedOptions.VARIABLE_NAME_TRANSLATIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (uso != null)
				cb.setSelected(uso.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public void saveValues() {
		
		for (String var : UserSelectedOptions.VARIABLE_NAME_TRANSLATIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected()) {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.TRUE);
			} else {
				if (uso != null)
					uso.setValue(var, uso.getOptionType(var), UserSelectedOptions.FALSE);
			}
		}
		
		if (uso != null)
			uso.save();
	}
	
	private void loadMap() {

		mapNameToCheckBox.put(StringConstants.INITIALIZE_TRANSLATION,initializeTranslationCheckBox);
	}
	
	public void setValues(boolean value) {
		
		for (String var : UserSelectedOptions.VARIABLE_NAME_TRANSLATIONS) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}
