package gov.anl.gss.sdm.server.model.documentor.gui;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.awt.Font;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.JSeparator;

public class WarningsPanel extends SDMOptionsPanel {
	
	JCheckBoxLanguage undocumentedEquations = new JCheckBoxLanguage(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES);
	JCheckBoxLanguage equationsWithEmbeddedData = new JCheckBoxLanguage(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA);
	JCheckBoxLanguage equationsWithUnitErrorsWarnings = new JCheckBoxLanguage(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS);
	JCheckBoxLanguage variablesNotInAnyView = new JCheckBoxLanguage(StringConstants.VARIABLES_NOT_IN_ANY_VIEW);
//	JCheckBoxLanguage incompletelyDefinedSubscriptedVariables = new JCheckBoxLanguage(StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES);
	JCheckBoxLanguage nonmonotonicLookupFunctions = new JCheckBoxLanguage(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS);
	JCheckBoxLanguage cascadingLookupFunctions = new JCheckBoxLanguage(StringConstants.CASCADING_LOOKUP_FUNCTIONS);
	JCheckBoxLanguage nonzeroEndSlope = new JCheckBoxLanguage(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS);
	JCheckBoxLanguage ifThenElse = new JCheckBoxLanguage(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS);
	JCheckBoxLanguage minMax = new JCheckBoxLanguage(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS);
	JCheckBoxLanguage stepPulse = new JCheckBoxLanguage(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS);
	
	
//	JCheckBoxLanguage vpmOutdated = new JCheckBox("Alert Me When VPM May Be Outdataded");
//	JCheckBoxLanguage newerDocVersion = new JCheckBox("Alert Mw When a Newer Version of SDM-Doc is Available");
	
	JCheckBoxLanguage ignoreEmbedded01 = new JCheckBoxLanguage(StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1);
	
	private Map<String, JCheckBoxLanguage> mapNameToCheckBox = new LinkedHashMap<String, JCheckBoxLanguage>();
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -5684112206610764840L;

	public WarningsPanel(OptionsFrame optionsFrame, String title) {
		super(optionsFrame, title);
		
		loadMap();
		
		add(undocumentedEquations);
		add(equationsWithEmbeddedData);
		add(equationsWithUnitErrorsWarnings);
		add(variablesNotInAnyView);
//		add(incompletelyDefinedSubscriptedVariables);
		add(nonmonotonicLookupFunctions);
		add(cascadingLookupFunctions);
		add(nonzeroEndSlope);
		add(ifThenElse);
		add(minMax);
		add(stepPulse);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		undocumentedEquations.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		equationsWithEmbeddedData.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		equationsWithUnitErrorsWarnings.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		variablesNotInAnyView.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		nonmonotonicLookupFunctions.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		cascadingLookupFunctions.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		nonzeroEndSlope.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		ifThenElse.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		minMax.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		stepPulse.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
	
		add(new JSeparator(JSeparator.HORIZONTAL));
	
//		add(vpmOutdated);
//		add(newerDocVersion);
//		
//		add(new JSeparator(JSeparator.HORIZONTAL));
		
		ignoreEmbedded01.setFont(language.getFont(Font.PLAIN, SDMDocFont.MEDIUM));
		
		add(ignoreEmbedded01);
		
		setValues();
	}
	
	public void changeLanguage() {
		super.changeLanguage();
		undocumentedEquations.setText(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES);
		equationsWithEmbeddedData.setText(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA);
		equationsWithUnitErrorsWarnings.setText(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS);
		variablesNotInAnyView.setText(StringConstants.VARIABLES_NOT_IN_ANY_VIEW);
//		incompletelyDefinedSubscriptedVariables.setText(StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES);
		nonmonotonicLookupFunctions.setText(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS);
		cascadingLookupFunctions.setText(StringConstants.CASCADING_LOOKUP_FUNCTIONS);
		nonzeroEndSlope.setText(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS);
		ifThenElse.setText(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS);
		minMax.setText(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS);
		stepPulse.setText(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS);
		ignoreEmbedded01.setText(StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1);
	}
	
	public void setValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.WARNINGS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(options.getOptionValue(var).equals(UserSelectedOptions.TRUE));
		}
	}
	
	public boolean saveValues() {
		UserSelectedOptions options = optionsFrame.getUserSelectedOptions();
		for (String var : UserSelectedOptions.WARNINGS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			if (cb.isSelected())
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.TRUE);
			else
				options.setValue(var, options.getOptionType(var), UserSelectedOptions.FALSE);
		}
		return true;
	}
	
	private void loadMap() {

		mapNameToCheckBox.put(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES,undocumentedEquations);
		mapNameToCheckBox.put(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, equationsWithEmbeddedData);
		mapNameToCheckBox.put(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, equationsWithUnitErrorsWarnings);
		mapNameToCheckBox.put(StringConstants.VARIABLES_NOT_IN_ANY_VIEW, variablesNotInAnyView);
//		mapNameToCheckBox.put(StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES, incompletelyDefinedSubscriptedVariables);
		mapNameToCheckBox.put(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,nonmonotonicLookupFunctions);
		mapNameToCheckBox.put(StringConstants.CASCADING_LOOKUP_FUNCTIONS, cascadingLookupFunctions);
		mapNameToCheckBox.put(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, nonzeroEndSlope);
		mapNameToCheckBox.put(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, ifThenElse);
		mapNameToCheckBox.put(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, minMax);
		mapNameToCheckBox.put(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,stepPulse);
		mapNameToCheckBox.put(StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1, ignoreEmbedded01);


	}
	
	@Override
	public void setValues(boolean value) {
		
		for (String var : UserSelectedOptions.WARNINGS_CHOICES) {
			JCheckBoxLanguage cb = mapNameToCheckBox.get(var);
			cb.setSelected(value);
		}
	}

}

