package gov.anl.gss.sdm.server.model.documentor.gui;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

public class WebsiteVersion {

	private static final String LOCATOR = "Java version (<b>ver ";
	private static final String SPLITTER = "Java version \\(<b>ver ";


	private String websiteVersion;
	private boolean valid = true;



	public WebsiteVersion() {
		retrieveWebsiteVersion();
	}

	public boolean isNewerThan(String localVersion) {
		boolean outOfDate = false;

		String[] webParts = websiteVersion.split("\\.");
		String[] localParts = localVersion.split("\\.");

		Integer[] webNums = stringToNumber(webParts);
		Integer[] localNums = stringToNumber(localParts);

		for (int i = 0; i < webNums.length; i++) {
			if (webNums[i] > localNums[i]) {
				outOfDate = true;
			}
			if (webNums[i] < localNums[i]) {
				outOfDate = false;
				break;
			}
		}

		return outOfDate;
	}

	public Integer[] stringToNumber(String[] s) {
		Integer[] i = new Integer[s.length];
		for (int j = 0; j < i.length; j++)
			i[j] = Integer.parseInt(s[j]);

		return i;
	}

	public void retrieveWebsiteVersion() {

		try {
			// Create a URL for the desired page
			URL url = new URL("http://tools.systemdynamics.org/sdm-doc/");       

			// Read all the text returned by the server
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
			String str;
			while ((str = in.readLine()) != null) {
				if (str.contains(LOCATOR)) {
					try {
						websiteVersion = str.split(SPLITTER)[1].split(",")[0];
					} catch (Exception e) {
						// TODO Auto-generated catch block
						valid = false;
						e.printStackTrace();
					}
					System.out.println(str);
					if (valid)
						System.out.println("websiteVersion: "+websiteVersion);
					else
						System.out.println("Website Version could not be determined");
				}
				// str is one line of text; readLine() strips the newline character(s)
			}
			in.close();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			valid = false;
		} catch (IOException e) {
			valid = false;
			// TODO Auto-generated catch block
			e.printStackTrace();
		}


	}

	public static void main(String[] args) {
		WebsiteVersion wv = new WebsiteVersion();
		if (wv.isValid()) {
			System.out.println("ood: "+wv.isNewerThan("1.0.1"));
			System.out.println("ood: "+wv.isNewerThan("0.9.0"));
			System.out.println("ood: "+wv.isNewerThan("1.0"));
		}

	}

	/**
	 * @return the websiteVersion
	 */
	public String getWebsiteVersion() {
		return websiteVersion;
	}

	/**
	 * @return the valid
	 */
	public boolean isValid() {
		return valid;
	}
}
