package gov.anl.gss.sdm.server.model.documentor.plots;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStream;

import javax.imageio.ImageIO;

import org.apache.log4j.Logger;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;


public class LookupPlot /* extends ApplicationFrame */ {
static Logger log = Logger.getLogger(LookupPlot.class.getSimpleName());
	
	
	Color[] colors = {Color.red, Color.black,  Color.blue, Color.green, Color.magenta, Color.cyan, Color.orange, Color.pink};
	
	private String fileName;
	private String fileNameRelative;
	
	 boolean addedHighlight = false;

	/**
	 * Creates a new lookup plot.
	 *
	 * @param title  the frame title.
	 */
	public LookupPlot(String title) {

		final XYDataset dataset = createDataset();
		final JFreeChart chart = createChart(dataset, "Demo", "X", "Y");
		final ChartPanel chartPanel = new ChartPanel(chart);
		chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));

		OutputStream out = null;
		try {
			out = new FileOutputStream(new File("lookupTable.png"));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		writeAsPNG( chart, out, 500,  270 );

	}
	
	public LookupPlot(String title, Equation equation, String modelFile) {
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		final XYSeriesCollection dataset = new XYSeriesCollection();
		
		 addedHighlight = false;
		
		XYData xyData = new XYData(equation);
		if (xyData.isValid()) {
			xyData.addSeriesToDataset(dataset);

			// do not add highlight point if outside of range of data

			if (options.getOptionValue(StringConstants.HIGHLIGHT_POINT).equals(UserSelectedOptions.TRUE)) {
				addedHighlight = true;
				double nonZeroX = Double.parseDouble(options.getOptionValue(StringConstants.HIGHLIGHT_POINT_X));
				double nonZeroY = Double.parseDouble(options.getOptionValue(StringConstants.HIGHLIGHT_POINT_Y));

				if (nonZeroX >= xyData.getMinXRange() && nonZeroY >= xyData.getMinYRange() &&
						nonZeroX <= xyData.getMaxXRange() && nonZeroY <= xyData.getMaxYRange()) {

					XYSeries xh = new XYSeries("xh");

					xh.add(nonZeroX, Math.min(xyData.getMinY(), 0.0));
					xh.add(nonZeroX, nonZeroY);

					XYSeries yh = new XYSeries("yh");

					yh.add(Math.min(xyData.getMinX(), 0.0), nonZeroY);
					yh.add(nonZeroX, nonZeroY);

					dataset.addSeries(xh);
					dataset.addSeries(yh);
				} else {
					addedHighlight = false;
				}
			}

			String t = equation.getOriginalLhs().replace(StringConstants.RSSD_LOOKUP, "");
			String x = "X";
			String y = "Y";
			final JFreeChart chart = createChart(dataset, "", x, y);
			final ChartPanel chartPanel = new ChartPanel(chart);
			chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));

			OutputStream out = null;
			try {
				out = new FileOutputStream(new File(getPlotFilename(modelFile, equation)));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			writeAsPNG( chart, out, 500,  270 );
		}

	}
	
	private String getPlotFilename(String modelFile, Equation equation) {
		
//		String outputDir = InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName();
//		String outputDirRelative = InformationManagers.getInstance().getModelManager().getOutputDirectoryName();
		
		String outputDir = InformationManagers.getInstance().getFileOrganizer().getThumbnailsDirectory();
		String outputDirRelative = InformationManagers.getInstance().getFileOrganizer().getRelativeThumbnailsDirectory();
		
		(new File(outputDir)).mkdirs();
		
		String plotFile = Utilities.generateNumberedFileName("plot") + ".png";
		fileName = outputDir + "/" + plotFile;
		fileNameRelative = outputDirRelative + "/" + plotFile;
		return fileName;
	}

	/**
	 * Creates a sample dataset.
	 * 
	 * @return a sample dataset.
	 */
	private XYDataset createDataset() {

		final XYSeries series2 = new XYSeries("Second");
		series2.add(1.0, 5.0);
		series2.add(2.0, 7.0);
		series2.add(3.0, 6.0);
		series2.add(4.0, 8.0);
		series2.add(5.0, 4.0);
		series2.add(6.0, 4.0);
		series2.add(7.0, 2.0);
		series2.add(8.0, 1.0);

		final XYSeriesCollection dataset = new XYSeriesCollection();

		dataset.addSeries(series2);


		return dataset;

	}

	/**
	 * Creates a chart.
	 * 
	 * @param dataset  the data for the chart.
	 * 
	 * @return a chart.
	 */
	private JFreeChart createChart(final XYDataset dataset, String title, String xaxisLabel, String yaxisLabel) {

		// create the chart...
		final JFreeChart chart = ChartFactory.createXYLineChart(
				title,      // chart title
				xaxisLabel,                      // x axis label
				yaxisLabel,                      // y axis label
				dataset,                  // data
				PlotOrientation.VERTICAL,
				false,                     // include legend
				true,                     // tooltips
				false                     // urls
				);

		// NOW DO SOME OPTIONAL CUSTOMISATION OF THE CHART...
		chart.setBackgroundPaint(Color.white);

		// get a reference to the plot for further customisation...
		final XYPlot plot = chart.getXYPlot();
		plot.setBackgroundPaint(Color.lightGray);
		//    plot.setAxisOffset(new Spacer(Spacer.ABSOLUTE, 5.0, 5.0, 5.0, 5.0));
		plot.setDomainGridlinePaint(Color.white);
		plot.setRangeGridlinePaint(Color.white);

		final XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
		renderer.setSeriesLinesVisible(0, true);
//		renderer.setSeriesShapesVisible(1, true);
		
		Stroke dash = new BasicStroke(
		        2.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND,
		        1.0f, new float[] {6.0f, 6.0f}, 0.0f
		    );
		
		int seriesCount = dataset.getSeriesCount();
		if (seriesCount > 2) {
			for (int i = 0; i < seriesCount; i++)
				renderer.setSeriesPaint(i, colorSelector(i));
			
			if (addedHighlight) {
				renderer.setSeriesPaint(seriesCount-2, Color.black);
				renderer.setSeriesPaint(seriesCount-1, Color.black);

				renderer.setSeriesStroke(seriesCount-2,dash);
				renderer.setSeriesStroke(seriesCount-1,dash);
			}
		} else {
			renderer.setSeriesPaint(0, Color.red);
			renderer.setSeriesPaint(1, Color.black);
			renderer.setSeriesPaint(2, Color.black);

			renderer.setSeriesStroke(1,dash);
			renderer.setSeriesStroke(2,dash);
		}
		
		plot.setRenderer(renderer);

		// change the auto tick unit selection to integer units only...
		final NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());
		// OPTIONAL CUSTOMISATION COMPLETED.

		return chart;

	}
	
	public Color colorSelector(int i) {
		return colors[i % 7];
	}

	/**
	 * Starting point for the demonstration application.
	 *
	 * @param args  ignored.
	 */
	public static void main(final String[] args) {

		final LookupPlot demo = new LookupPlot("SDM-Doc Lookup Table");
	}

	public void writeAsPNG( JFreeChart chart, OutputStream out, int width, int height )
	{
		try
		{
			BufferedImage chartImage = chart.createBufferedImage( width, height, null);
			ImageIO.write( chartImage, "png", out );
		}
		catch (Exception e)
		{
			log.debug(e);
		}
	}

	public String getFileName() {
		return fileName;
	}

	public String getFileNameRelative() {
		return fileNameRelative;
	} 
}
