package gov.anl.gss.sdm.server.model.documentor.plots;

import java.awt.BasicStroke;
import java.awt.Color;
import java.text.DecimalFormat;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYDataset;

public class SimpleLinePlot {

	private JFreeChart chart;

	/**
	 * Creates a new demo.
	 *
	 * @param title  the frame title.
	 */
	public SimpleLinePlot(XYDataset data) {
		chart = createChart(data);
	}


	/**
	 * Creates a chart.
	 *
	 * @param dataset  the data for the chart.
	 *
	 * @return a chart.
	 */
	private static JFreeChart createChart(XYDataset dataset) {

//		XYDataset dataset = createDataset(data);

		// create the chart...
		JFreeChart chart = ChartFactory.createXYLineChart(
				"",      // chart title
				"",                      // x axis label
				"",                      // y axis label
				dataset,                  // data
				PlotOrientation.VERTICAL,
				false,                     // include legend
				false,                     // tooltips
				false                     // urls
				);

		// get a reference to the plot for further customisation...
		XYPlot plot = (XYPlot) chart.getPlot();
		plot.setDomainPannable(true);
		plot.setRangePannable(true);
		XYLineAndShapeRenderer renderer	= (XYLineAndShapeRenderer) plot.getRenderer();
		renderer.setBaseShapesVisible(false);
		renderer.setBaseShapesFilled(false);
		
		
		 int ds = dataset.getSeriesCount();
		 for (int series = 0; series < ds; series++)
			 renderer.setSeriesStroke(series, new BasicStroke(1.0f));
		
		
		plot.setDomainGridlinesVisible(false);
		plot.setRangeGridlinesVisible(false);
		
		plot.setRangeGridlinePaint(Color.black);
		plot.setDomainGridlinePaint(Color.black);
		
		plot.setBackgroundPaint(Color.white);

		plot.setDomainZeroBaselineVisible(true);
        plot.setRangeZeroBaselineVisible(true);
   

		// change the auto tick unit selection to integer units only...
		NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
		rangeAxis.setVisible(true);
		
		   DecimalFormat df=new DecimalFormat();
		   df.applyPattern("##0.#####E0");

		   //these lines do not fix the bug, just change label from 0.0000000 to 0E0
		   rangeAxis.setAutoRangeMinimumSize(1e-10);
		   rangeAxis.setNumberFormatOverride(df);
		
		ValueAxis valueAxis = (ValueAxis) plot.getDomainAxis();
		valueAxis.setVisible(true);

		return chart;
	}

	public JFreeChart getChart() {
		return chart;
	}
}
