package gov.anl.gss.sdm.server.model.documentor.plots;

import gov.anl.gss.sdm.server.model.analysis.loops.VariableLoopCount;
import gov.anl.gss.sdm.server.model.bridge.BridgeModelResults;
import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.ExtendedMemoryDatabase;
import gov.anl.gss.sdm.server.model.support.MemoryMapperInterface;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.StreamlinedMemoryMapper;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;
import gov.anl.gss.sdm.server.model.translator.ArrayManager;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

public class ThumbnailPlots {
	
	public final static int HARDCODED_PLOT_COUNT_LIMIT = 1000;
	
	private List<BridgeModelResults> results = new ArrayList<BridgeModelResults>();
	private String directory;
	private String model;
	private MemoryMapperInterface memoryMapper;
	private ExtendedMemoryDatabase extendedMemory;
	private ArrayManager arrayManager;
	private NamedSubscriptManager namedSubscriptManager;
	
	public ThumbnailPlots(String model, List<String> logFiles,  FileOrganizer fileOrganizer, String runName) {
//	directory = fileOrganizer.getDocGraphDirectory();
		
		directory = fileOrganizer.getThumbnailsDirectory();
		
		this.model = model;

			namedSubscriptManager = InformationManagers.getInstance().getNamedSubscriptManager();
			arrayManager = InformationManagers.getInstance().getArrayManager();
			memoryMapper = new StreamlinedMemoryMapper(fileOrganizer.getMapperFile(), new MessageGobbler());
			extendedMemory = new ExtendedMemoryDatabase(fileOrganizer.getMapperFile(), fileOrganizer.getRunsDirectory()+"/"+runName+"HSQLDB");
			extendedMemory.loadRunParameters();
			UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
			memoryMapper.getVariablesOrderedByType();
			if (!options.isSelected(StringConstants.ANALYZE_LOOPS)) {
				createThumbnails(Utilities.asList(InformationManagers.getInstance().getEquationManager().getLhsVariables()));
			} else {
				createThumbnails(getThumbnailOrder());
			}
			extendedMemory.shutdownDatabase();
	}
	
	public ThumbnailPlots(String model, List<String> logFiles,  FileOrganizer fileOrganizer ) {
		this(model, logFiles, fileOrganizer, "baseline" );


	}
	
	private List<String> getThumbnailOrder() {
		List<String> als = new ArrayList<String>();
		List<VariableLoopCount> al = new ArrayList<VariableLoopCount>();
		al.addAll(InformationManagers.getInstance().getAnalysisManager().getLoopManager().gatherVariableLoopCountInformation());
		try {
			Collections.sort(al, new Comparator<VariableLoopCount>() {
				@Override
				public int compare(VariableLoopCount arg0, VariableLoopCount arg1) {
					try {
//						if (arg0.getNumLoops() == arg1.getNumLoops()) {
//							if (arg0.getEquation().getTypeSortValue() == arg1.getEquation().getTypeSortValue()) {
//								return arg0.getVariable().compareTo(arg1.getVariable());
//							} else {
//								return arg0.getEquation().getTypeSortValue() < arg1.getEquation().getTypeSortValue() ? -1 : 1;
//							}
//						} else {
//							return -1 * (arg0.getNumLoops() < arg1.getNumLoops() ? -1 : 1);
//						}
						
						if (arg0.getEquation().getTypeSortValue() == arg1.getEquation().getTypeSortValue()) {
							if (arg0.getNumLoops() == arg1.getNumLoops()) {
								return arg0.getVariable().compareTo(arg1.getVariable());
							} else {
								return -1 * (arg0.getNumLoops() < arg1.getNumLoops() ? -1 : 1);
							}
						} else {
							return arg0.getEquation().getTypeSortValue() < arg1.getEquation().getTypeSortValue() ? -1 : 1;
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for (VariableLoopCount vlc : al)
			als.add(vlc.getVariable());
		return als;
	}
	
	private void createThumbnails(List<String> originalVariables) {
		int  max = HARDCODED_PLOT_COUNT_LIMIT;
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.LIMIT_THUMBNAIL_COUNT)) {
			try {
				max = Integer.parseInt(options.getOptionValue(StringConstants.THUMBNAIL_MAXIMUM));
			} catch (NumberFormatException e) {
				max = 100;
			}
		}

		max = Math.min(max, HARDCODED_PLOT_COUNT_LIMIT);

		double[] timeVector = extendedMemory.getTimeVector(); // getVarVector("Time");
		double[] emvtv;

		Reuse reuse = InformationManagers.getInstance().getReuse();

		int count = 0;

		for (String ovar : originalVariables) {
			
			if (!memoryMapper.isValidVariable(ovar))
				continue;
			String gvar = Utilities.getNameArgument(ovar);
			if (gvar == null)
				System.out.println("Thumbnail: "+ovar);
			if (gvar.toLowerCase().contains(StringConstants.RSSD_LOOKUP.toLowerCase()))
				continue;
			Equation eqn1 = InformationManagers.getInstance().getEquationManager().getEquationFor(ovar);
			if (eqn1 == null) {
				System.out.println("eqn1 is null for variable "+gvar);
				continue;
			}
			if (eqn1.isDefinesLookup())
				continue;
			int seriesIndex = 0;
			if (++count > max)
				break;
			//  directory + "/"+ Utilities.makeLegal(var) + ".png";
			//  here either copy or create
			if (reuse.isThumbnails()) {
				File thumbFile = new File(genFileName(gvar));
				if (reuse.getDocGraphDirectory().equals(directory)) {

					if (thumbFile.exists())
						continue;
				} else {
					File existing = new File(reuse.getThumbnailsdirectory()+ "/"+ /*Utilities.makeLegalJavaName(gvar)*/ ovar + ".png");
					try {
						FileUtils.copyFile(existing, thumbFile);
					} catch (IOException e) {
						XYSeriesCollection dataset = new XYSeriesCollection();
						dataset.addSeries(createSeries(gvar+seriesIndex, extendedMemory.getVarTimeVector(gvar) /*extendedMemory.getVarVector(var)*/,timeVector));
						// need to create it
						ThumbnailPlot tc = new ThumbnailPlot(dataset, genFileName(gvar));
						tc.generate();
					}
					continue;
				}

			} 	
			XYSeriesCollection dataset = new XYSeriesCollection();

			Equation eqn = InformationManagers.getInstance().getEquationManager().getEquationFor(ovar);

			if (eqn.isHasLHSArrayReference()) {
				String[] subs = namedSubscriptManager.getExpandedSubscripts(eqn.getLhsArrayReference());
				String t = memoryMapper.getVarType(gvar);
				int index = memoryMapper.getIndex(gvar);
//				System.out.println("Number of Expanded subscripts: "+subs.length+" type: "+t+" index: "+index);
				for (int i = 0; i < subs.length; i++) {
					if (i > 2)
						break;
					int ind = index + i;
					emvtv = extendedMemory.getVarTimeVectorByIndex(t, ind);
					dataset.addSeries(createSeries(gvar+i, emvtv /*extendedMemory.getVarVector(var)*/,timeVector));
				}
					// need to create it
					ThumbnailPlot tc = new ThumbnailPlot(dataset, genFileName(gvar));
					tc.generate();
//				}
			} else {
				emvtv = extendedMemory.getVarTimeVector(ovar);
				if (emvtv != null) {
					dataset.addSeries(createSeries(gvar+seriesIndex, emvtv /*extendedMemory.getVarTimeVector(var)*/ /*extendedMemory.getVarVector(var)*/,timeVector));
					// need to create it
					ThumbnailPlot tc = new ThumbnailPlot(dataset, genFileName(gvar));
					tc.generate();
					if (StaticSwitches.WRITE_RESULTS)
						writeResultsFile(gvar, emvtv,timeVector);
				}
			}
		}
	}
	
	private void writeResultsFile(String gvar, double[] dataVector, double[] timeVector) {
		BufferedWriter bw = Utilities.openFileForWriting(genResultsFileName(gvar));
		try {
			for (int i = 0; i < timeVector.length; i++) {
				bw.append(Utilities.getOriginalNameForGV(gvar)+"\t"+timeVector[i]+"\t"+ dataVector[i]+"\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private XYSeries createSeries(String variable, double[] dataVector, double[] timeVector) {
		
		XYSeries series = new XYSeries(variable);
		
		for (int i = 0; i < timeVector.length; i++) {
			series.add(timeVector[i], dataVector[i]);
		}
		
		return series;
	}
	
	private void createThumbnailsBMR(List<String> variables) {
		List<Double> timeVector = results.get(0).getTimeVector();
		
		int countLimiter = 0;
		
		for (String var : variables) {
			int seriesIndex = 0;
			if (countLimiter++ > 100)
				break;
			XYSeriesCollection dataset = new XYSeriesCollection();
			for (BridgeModelResults bmr : results) {
				dataset.addSeries(createSeries(var+seriesIndex, bmr.getValues(var),timeVector));
				seriesIndex++;
			}
			
			ThumbnailPlot tc = new ThumbnailPlot(dataset, genFileName(var));
			tc.generate();
		}
	}
	
	private XYSeries createSeries(String variable, List<Double> dataVector, List<Double> timeVector) {
		
		XYSeries series = new XYSeries(variable);
		
		for (int i = 0; i < timeVector.size(); i++) {
			series.add(timeVector.get(i), dataVector.get(i));
		}
		
		return series;
	}
	
	private String genFileName(String gvar) {


//		return directory + "/"+ Utilities.makeLegalJavaName(var)+ ".png";
		return directory + "/"+ Utilities.makeLegalFilename(Utilities.getOriginalNameForGV(gvar))+ ".png";
	}
	
	private String genResultsFileName(String gvar) {
//		return directory+"/results/"+ Utilities.makeLegalJavaName(var) + ".tab";
//		return directory+"/results/"+ Utilities.makeLegalFilename(Utilities.getOriginalNameForGV(gvar)) + ".tab";
		return InformationManagers.getInstance().getFileOrganizer().getResultsDirectory()+ "/"+Utilities.makeLegalFilename(Utilities.getOriginalNameForGV(gvar)) + ".tab";
	}
	
}
