package gov.anl.gss.sdm.server.model.documentor.plots;

import gov.anl.gss.sdm.server.model.translator.Equation;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.mutable.MutableDouble;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;

public class XYData {
	
	List<XYDataOneLine> lines = new ArrayList<XYDataOneLine>();
	
	List<Double> x = new ArrayList<Double>();
	List<Double> y = new ArrayList<Double>();
	MutableDouble minXRange = new MutableDouble();
	MutableDouble minYRange = new MutableDouble();
	
	MutableDouble maxXRange = new MutableDouble();
	MutableDouble maxYRange = new MutableDouble();
	
	private boolean valid;
	
	public XYData(Equation equation) {
		valid = equation.createXYData(x, y, minXRange, maxXRange, minYRange, maxYRange);
	}
	
	public XYDataset getAsXYDataset() {
		final XYSeries series = new XYSeries("lookup");
		for (int i = 0; i < x.size(); i++) {
			series.add(x.get(i), y.get(i));
		}
		

		final XYSeriesCollection dataset = new XYSeriesCollection();

		dataset.addSeries(series);


		return dataset;

	}
	
	public XYSeries getAsXYSeries() {
		final XYSeries series = new XYSeries("lookup");
		for (int i = 0; i < x.size(); i++) {
			series.add(x.get(i), y.get(i));
		}
		
		return series;

	}
	
	public XYSeries getAsXYSeries(int s) {
		final XYSeries series = new XYSeries("lookup "+s);
		int jstart = s * x.size();
		for (int i = 0; i < x.size(); i++) {
			series.add(x.get(i), y.get(jstart++));
		}
		
		return series;

	}
	
	public void addSeriesToDataset(XYSeriesCollection dataset) {
		for (int s = 0; s < getNumberOfSeries(); s++) {
			dataset.addSeries(getAsXYSeries(s));
		}
	}
	
	
	public double getMinX() {
		return x.get(0);
	}
	
	public double getMinY() {
		return y.get(0);
	}

	public List<Double> getX() {
		return x;
	}

	public List<Double> getY() {
		return y;
	}

	public double getMinXRange() {
		return minXRange.doubleValue();
	}

	public double getMinYRange() {
		return minYRange.doubleValue();
	}

	public double getMaxXRange() {
		return maxXRange.doubleValue();
	}

	public double getMaxYRange() {
		return maxYRange.doubleValue();
	}
	
	public int getNumberOfSeries() {
		return y.size() / x.size();
	}

	/**
	 * @return the valid
	 */
	public boolean isValid() {
		return valid;
	}

}
