package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.commons.lang3.StringEscapeUtils;

public class Anchors {
	
	public static final String NL = "\n";
	public static final String DEF_HEADER = NL+"<a name='";
	public static final String DEF_FOOTER = "'>";
	
	public static final String REF_HEADER = NL+"<a href='#";
	public static final String FILE_REF_HEADER = NL+"<a href='";
	public static final String NEW_WINDOW = "target=\"_blank\"";
	public static final String REF_FOOTER = "'>";
	
	public static final String REF_FOOTER2 = "</a>";
	
	public static final String SINGLE_QUOTE = "SQ";
	public static final String LEFT_PAREN = "LP";
	public static final String RIGHT_PAREN = "RP";
	
	public static int dump = 0;
	
	private static Map<String, String> anchors = new LinkedHashMap<String, String>();
	private static Map<String, String> anchorsColors = new LinkedHashMap<String, String>();
	
	public static void register(String pos) {
		// do not allow quotes in anchors unless they are escaped
		String posNoQuote = pos.replaceAll("^\"", "").replaceAll("\"$", "");
		
		String anchor =  createAnchorName(posNoQuote);
		anchors.put(pos, anchor);
		anchors.put(posNoQuote, anchor);
	}
	
	public static void registerColor(String pos) {
		
		// do not allow quotes in anchors unless they are escaped
				String posNoQuote = pos.replaceAll("^\"", "").replaceAll("\"$", "");
				
				String anchor =  createAnchorName(posNoQuote);
				anchorsColors.put(pos, anchor);
				anchorsColors.put(posNoQuote, anchor);
				
//		anchorsColors.put(pos, createAnchorName(pos));
	}
	
	public static String createAnchorName(String pos) {
		
		// don't know if single quotes or parens cause problems, so get right of both
		
		String t = new String(pos);
		
		return t.replace(" ", "").replace("'", SINGLE_QUOTE)
				.replace("(", LEFT_PAREN)
				.replace(")", RIGHT_PAREN);
	}
	
	// <A name='XXX'>
	public static String getAnchorDefinition(String pos) {
		if (anchors.containsKey(pos)) {
			return DEF_HEADER+StringEscapeUtils.escapeHtml4(anchors.get(pos))+DEF_FOOTER;
		} else {
			register(pos);
			return getAnchorDefinition(pos);
		}
	}
	
	public static String getAnchorDefinitionNoEscape(String pos) {
		

		if (anchors.containsKey(pos)) {
			return DEF_HEADER+anchors.get(pos)+DEF_FOOTER;
		} else {
			register(pos);
			return getAnchorDefinitionNoEscape(pos);
		}
	}
	
	
	public static String getAnchorReferenceNoEscape(String pos) {
		if (anchors.containsKey(pos)) {
			return REF_HEADER+anchors.get(pos)+REF_FOOTER+pos.replace("-VIEW-", "").replace("-MODULE-", "").
					replace("-GROUP-", "").replace("-TYPE-", "").replace("-ALL-", "")+REF_FOOTER2;
		} else {
			register(pos);
			return getAnchorReferenceNoEscape(pos);
		}
	}
	
	public static String getFileAnchorReference(String file, String pos) {
		StringBuilder sb = new StringBuilder();
		sb.append("<a href='");
		sb.append(StringEscapeUtils.escapeHtml4(file));
		sb.append("#");
		sb.append(StringEscapeUtils.escapeHtml4(createAnchorName(pos)));
		sb.append("'>");
		sb.append(StringEscapeUtils.escapeHtml4(pos));
		sb.append("</a>");
		return sb.toString();
	}
	
	public static String getFileAnchorReference(String file, String pos, String display) {
		StringBuilder sb = new StringBuilder();
		sb.append("<a href='");
		sb.append(StringEscapeUtils.escapeHtml4(file));
		sb.append("#");
		sb.append(StringEscapeUtils.escapeHtml4(createAnchorName(pos)));
		sb.append("'>");
		sb.append(StringEscapeUtils.escapeHtml4(display));
		sb.append("</a>");
		return sb.toString();
	}
	
	// <A href='#xxx'>
	public static String getAnchorReference(String pos) {
		if (anchors.containsKey(pos)) {
			return REF_HEADER+
					StringEscapeUtils.escapeHtml4(anchors.get(pos))+
					REF_FOOTER+
					StringEscapeUtils.escapeHtml4(Parser.removeQuotes(pos).replace("-VIEW-", "").
					replace("-MODULE-", "").
					replace("-GROUP-", "").
					replace("-TYPE-", "").
					replace("-ALL-", ""))+
					REF_FOOTER2;
		} else {
			register(pos);
			return getAnchorReference(pos);
		}
	}
	
	public static String getAnchorReferenceWithColor(String pos) {
		if (anchorsColors.containsKey(pos)) {
			return REF_HEADER+StringEscapeUtils.escapeHtml4(anchorsColors.get(pos))+"' style='color: #FFFFFF"+
		REF_FOOTER+StringEscapeUtils.escapeHtml4(Parser.removeQuotes(pos).replace("-VIEW-", "").replace("-MODULE-", "").
					replace("-GROUP-", "").replace("-TYPE-", "").replace("-ALL-", ""))+REF_FOOTER2;
		} else {
			registerColor(pos);
			return getAnchorReferenceWithColor(pos);
		}
	}
	
	public static String getFileReference(String file, String display) {
			return FILE_REF_HEADER+StringEscapeUtils.escapeHtml4(file)+REF_FOOTER+display+REF_FOOTER2;
	}
	
	public static String getAnchorReference(String forAnchor, String variable) {
		if (anchors.containsKey(forAnchor)) {
			return REF_HEADER+StringEscapeUtils.escapeHtml4(anchors.get(forAnchor))+REF_FOOTER+
					StringEscapeUtils.escapeHtml4(Parser.removeQuotes(variable))+REF_FOOTER2;
		} else {
			register(forAnchor);
			return getAnchorReference(forAnchor, variable);
		}
	}
	
	public static String getAnchorReferenceNoEscape(String forAnchor, String variable) {
		if (anchors.containsKey(forAnchor)) {
			return REF_HEADER+anchors.get(forAnchor)+REF_FOOTER+
					Parser.removeQuotes(variable)+REF_FOOTER2;
		} else {
			register(forAnchor);
			return getAnchorReferenceNoEscape(forAnchor, variable);
		}
	}

}
