package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.ArrowPolaritySelector;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.List;

public class ArrowListTable implements GetXML {
	
	protected static final String TITLE_MARKER = "$$$TITLE$$$";
	protected static final String NAME_MARKER = "$$$NAME$$$";
	protected static final String NUM_MARKER = "$$$NUM$$$";
	protected static final String TOP_MARKER = "$$$TOP$$$";

	protected static final String HEADER_START = "<br/><hr style='height:1px; color:Blue'/>\n" +
			"$$$NAME$$$\n" +
			"$$$TOP$$$" ;
	
	protected static final String HEADER_END = 
			"<h3>$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)</h3>\n" +
			"<table  class='sample'>\n";
	
	protected static final String TITLE = "$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)";
	protected static final String CAUSE = "##Cause##";
	protected static final String EFFECT = "##Effect##";
	protected static final String POLARITY = "##Polarity##";
	
//	"<table align='center'  class='sample'>\n";
	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
	
	protected static final String COLUMN_HEADERS = "<tr>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Cause##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Effect##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Polarity##</center></b></td>\n" +
			"</tr>\n";
	
	protected String title;
	
	protected List<ArrowListTableRow> arrows;
	
	protected ArrowPolaritySelector arrowPolaritySelector;
	
	protected PolarityResults polarityResults;
	
	protected ArrowListTable(PolarityResults polarityResults, ArrowPolaritySelector arrowPolaritySelector) {
		this.polarityResults = polarityResults;
		this.title = arrowPolaritySelector.getSelectionCriteria();
		this.arrowPolaritySelector = arrowPolaritySelector;
	}
	
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	public List<List<List<String>>> getTableData() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		selectVariables();
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		TableUtilities.addTitle(table,
			TITLE.replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(arrows.size())));
		
			List<List<String>> row = TableUtilities.createRow();
			
			TableUtilities.addRow(table, row);
			
			TableUtilities.addColumn(
					row, TableUtilities.createColumn(language.getTextHtml(StringConstants.CAUSE)));
			TableUtilities.addColumn(
					row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EFFECT)));
			TableUtilities.addColumn(
					row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POLARITY)));
					
			for (ArrowListTableRow aRow : arrows) {
				TableUtilities.addRow(table, aRow.getRowData());
			}
		
		return table;
	}
	
	public String getXML() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		selectVariables();
		String alpha = "";
		StringBuilder sb = new StringBuilder();
		
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(arrowPolaritySelector.getInternalName())).
					replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
			
			if (arrows.size() > 0) {
				QuickLinkArrowAlphabet qla = new QuickLinkArrowAlphabet(arrows, "List"+arrowPolaritySelector.getInternalName());
				sb.append(qla.getXML());
			}
			
			sb.append(HEADER_END.replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(arrows.size())));
				
			sb.append(COLUMN_HEADERS
					.replace("##Cause##", language.getTextHtml(StringConstants.CAUSE))
					.replace("##Effect##", language.getTextHtml(StringConstants.EFFECT))
					.replace("##Polarity##", language.getTextHtml(StringConstants.POLARITY))
					);
			for (ArrowListTableRow row : arrows) {
				
				String listMarker = "$$$LISTNAME$$$";
				String list = "";
				// href='#
				String letter = row.getXML().split("href='#")[1].substring(0, 1);
//			if (variableSelector.getInternalName().equals(VariableNames.VARIABLES_LIST)) {
					if (!letter.toUpperCase().equals(alpha)) {
						list = Anchors.getAnchorDefinition("List"+arrowPolaritySelector.getInternalName()+letter.toUpperCase());
						alpha = letter.toUpperCase();
					}
//			}
				
				sb.append(row.getXML().replace(listMarker, list));
			}

			sb.append(FOOTER);
			sb.append(BREAK);
			//		sb.append(SEPARATOR);
			
			UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
			if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
				List<List<List<String>>> table = getTableData();
				String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+arrowPolaritySelector.getInternalName()+".tab";
				FlatFileWriter.writeFile(fn, table, "\t");
			}
		
		return sb.toString();
	}
	
	protected void selectVariables() {
		arrows = new ArrayList<ArrowListTableRow>();
		
		for (ArrowPolarity arrow : polarityResults.getArrows()) {
			if (arrowPolaritySelector.isSelect(arrow))
				arrows.add(new ArrowListTableRow(arrow));
		}
	}
	
}
