package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.GroupsCounter;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.ModelVariablesInformation;
import gov.anl.gss.sdm.server.model.documentor.ModulesCounter;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TypeCounter;
import gov.anl.gss.sdm.server.model.documentor.ViewsCounter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.ArrowPolaritySelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.ArrowPolaritySelectorNegative;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.ArrowPolaritySelectorPositive;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.ArrowPolaritySelectorUnknown;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorCascadingLookup;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorComplexity;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorComplexityStock;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorDataLookup;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorDmnlUnits;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorEmbeddedData;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorEndogenous;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorExogenous;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorFunctionSensitivityParameter;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorIfThenElse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorLevel;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorMinMax;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNonZeroEnd;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNonmonotonic;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorNotInView;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSourceInformation;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorState;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorStepPulse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSupplementary;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorSupplementaryUsed;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUndocumented;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUnitErrorsWarnings;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUnused;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorUsingMacros;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelectorValueRangeSpecified;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class Document implements GetXML {
	
	private Documentor documentor;
	
	private DocumentHeader documentHeader;
	private DocumentFooter documentFooter;
	
	private VariableListTable undocumented;
	private VariableListTable supplementary;
	private VariableListTable ifThenElse;
	private VariableListTable minMax;
	private VariableListTable complexity;
	private VariableListTable complexityStock;
	private VariableListTable state;
	private VariableListTable notInView;
	private VariableListTable unitErrorsWarnings;
	
	private VariableListTable inOutLinks;
	
	private VariableListTable supplementaryUsed;
	private VariableListTable unused;
	private VariableListTable level;
	private VariableListTable embeddedData;
	private VariableListTable nonmonotonic;
	private VariableListTable nonZeroEnd;
	private VariableListTable cascading;
	private VariableListTable stepPulse;
	
	private VariableListTable allVariables;
	
	private VariableListTable feedbackLoops;
	private VariableListTable exogenous;
	private VariableListTable endogenous;
	
	private VariableListTableExtraColumns sources;
	private VariableListTable dmnl;
	private VariableListTable valueRange;
	private VariableListTable sensivities;
	private VariableListTable dataLookups;
	private VariableListTable usingMacros;
	
	private ArrowListTable positivePolarityArrows;
	private ArrowListTable negativePolarityArrows;
	private ArrowListTable unknownPolarityArrows;
	
	private MacroTable macroTable;
	private UnitsTable unitsTable;
	private RateToRateTable rateToRateTable;
	
	private ViewVariablesBarTable viewVariablesBarTable;
	
	private QuickLink quickLink;
	private QuickLinkAlphabet quickLinkAlphabetDetail;
	private QuickLinkAlphabet quickLinkAlphabetList;
	
	
	private List<VariableListTableMultiValueColumns> modelVariables = new ArrayList<VariableListTableMultiValueColumns>();
	
	private VariableTable variableTable;
	
	private VariableSelector variableSelectorUndocumented;
	private VariableSelector variableSelectorSupplementary;
	private VariableSelector variableSelectorIfThenElse;
	private VariableSelector variableSelectorMinMax;
	private VariableSelector variableSelectorInOutLinks;
	private VariableSelector variableSelectorComplexity;
	private VariableSelector variableSelectorCompletxityStock;
	private VariableSelector variableSelectorState;
	private VariableSelector variableSelectorNotInView;
	private VariableSelector variableSelectorUnitErrorsWarnings;
	
	private VariableSelector variableSelectorSupplementaryUsed;
	private VariableSelector variableSelectorUnused;
	private VariableSelector variableSelectorLevel;
	private VariableSelector variableSelectorEmbeddedData;
	private VariableSelector variableSelectorNonMonotonic;
	private VariableSelector variableSelectorNonZeroEnd;
	private VariableSelector variableSelectorCascading;
	private VariableSelector variableSelectorStepPulse;
	
	private VariableSelector variableSelectorAll;
	
	private VariableSelector variableSelectorFeedbackLoops;
	private VariableSelector variableSelectorExogenous;
	private VariableSelector variableSelectorEndogenous;
	
	private VariableSelector variableSelectorSourceInformation;
	private VariableSelector variableSelectorDmnl;
	private VariableSelector variableSelectorValueRangeSpecified;
	private VariableSelector variableSelectorSensitivity;
	private VariableSelector variableSelectorDataLookup;
	
	private VariableSelector variableSelectorUsingMacros;
	
	private ArrowPolaritySelector arrowPolaritySelectorPositve;
	private ArrowPolaritySelector arrowPolaritySelectorNegative;
	private ArrowPolaritySelector arrowPolaritySelectorUnknown;
	
	private SingleSectionTable sst;
	
	private TypesTable typesTable;
	private TypeCounter typeCounter;
	
	private GroupingTable viewsTable;
	private ViewsCounter viewsCounter;
	
	private GroupingTable groupsTable;
	private GroupsCounter groupsCounter;
	
	private GroupingTable modulesTable;
	private ModulesCounter modulesCounter;
	
	private GroupingTable variableTypesTable;
	
	private MultiSectionTable modelInformation;
	private MultiSectionTable warnings;
	private MultiSectionTable possibleOmissions;
	

	private Writer writer;
	
	private MultiTableHeader multiTableHeader;
	
	private String sourceFile;
	private String modelName;
	private MessageProcessor mf;
	
	private Map<String, Equation> equations;
	
	public Document(Documentor documentor, String modelName, String sourceFile, Map<String, Equation> equations) {
		this.documentor = documentor;
		this.modelName = modelName;
		this.sourceFile = sourceFile;
		this.equations = equations;
		this.mf = documentor.getMessageProcessor();
		
		
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		mf.addMessage(language.getText(StringConstants.COUNTING_AND_BUILDING_TABLES));

		// this is the detailed list information
		for (ModelVariablesInformation mvi : documentor.getModelVariablesInformation()) {
			modelVariables.add(new VariableListTableMultiValueColumns(mvi, mvi.getInternalName(), mvi.getGroupingType(), mvi.getGroupingName(), 
					mvi.size(), documentor.displayModule(), documentor.displayThumbNail()));
		}
		
		

		typeCounter = documentor.getTypeCounter();
		typesTable = new TypesTable(language.getTextHtml(StringConstants.VARIABLE_TYPES), typeCounter, documentor.displayModule());

		viewsCounter = documentor.getViewsCounter();
		viewsTable = new GroupingTable(language.getTextHtml(StringConstants.VIEWS), viewsCounter, StringConstants.QUICK_LINK_NAME_VIEWS);

		groupsCounter = documentor.getGroupsCounter();
		groupsTable = new GroupingTable(language.getTextHtml(StringConstants.GROUPS), groupsCounter, StringConstants.QUICK_LINK_NAME_GROUPS);

		modulesCounter = documentor.getModulesCounter();
		modulesTable = new GroupingTable(language.getTextHtml(StringConstants.MODULES), modulesCounter, StringConstants.QUICK_LINK_NAME_MODULES);

		documentHeader = new DocumentHeader(modelName);
		documentFooter = new DocumentFooter(sourceFile);

		variableSelectorAll = new VariableSelector(language.getTextHtml(StringConstants.ALL_VARIABLES), StringConstants.ALL_VARIABLES);
		allVariables = new VariableListTable(equations, variableSelectorAll, documentor.displayModule());

		variableSelectorSupplementaryUsed = new VariableSelectorSupplementaryUsed(language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED), 
				StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED);
		supplementaryUsed = new VariableListTable(equations, variableSelectorSupplementaryUsed, documentor.displayModule());

		variableSelectorUnused = new VariableSelectorUnused(language.getTextHtml(StringConstants.UNUSED_VARIABLES), StringConstants.UNUSED_VARIABLES);
		unused = new VariableListTable(equations, variableSelectorUnused, documentor.displayModule());

		variableSelectorLevel = new VariableSelectorLevel(language.getTextHtml(StringConstants.STOCK_VARIABLES), StringConstants.STOCK_VARIABLES);
		level = new VariableListTable(equations, variableSelectorLevel, documentor.displayModule());

		variableSelectorEmbeddedData = new VariableSelectorEmbeddedData(language.getTextHtml(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA), StringConstants.EQUATIONS_WITH_EMBEDDED_DATA);
		embeddedData = new VariableListTable(equations, variableSelectorEmbeddedData, documentor.displayModule());

		variableSelectorNonMonotonic = new VariableSelectorNonmonotonic(language.getTextHtml(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS), StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS);
		nonmonotonic = new VariableListTable(equations, variableSelectorNonMonotonic, documentor.displayModule());

		variableSelectorNonZeroEnd = new VariableSelectorNonZeroEnd(language.getTextHtml(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS), StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS);
		nonZeroEnd = new VariableListTableExtraColumns(equations, variableSelectorNonZeroEnd, VariableListTableRowExtraColumns.NONZERO, documentor);

		variableSelectorCascading = new VariableSelectorCascadingLookup(language.getTextHtml(StringConstants.CASCADING_LOOKUP_FUNCTIONS),StringConstants.CASCADING_LOOKUP_FUNCTIONS);
		cascading = new VariableListTable(equations, variableSelectorCascading, documentor.displayModule());

		variableSelectorStepPulse = new VariableSelectorStepPulse(language.getTextHtml(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS), StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS);
		stepPulse = new VariableListTable(equations, variableSelectorStepPulse, documentor.displayModule());

		variableSelectorUndocumented = new VariableSelectorUndocumented(language.getTextHtml(StringConstants.UNDOCUMENTED_VARIABLES), StringConstants.UNDOCUMENTED_VARIABLES);
		undocumented = new VariableListTable(equations, variableSelectorUndocumented, documentor.displayModule());


		variableSelectorSupplementary = new VariableSelectorSupplementary(language.getTextHtml(StringConstants.SUPPLEMENTARY_VARIABLES),StringConstants.SUPPLEMENTARY_VARIABLES);
		supplementary = new VariableListTable(equations, variableSelectorSupplementary, documentor.displayModule());

		variableSelectorIfThenElse = new VariableSelectorIfThenElse(language.getTextHtml(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS), StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS);
		ifThenElse = new VariableListTable(equations, variableSelectorIfThenElse, documentor.displayModule());

		variableSelectorMinMax = new VariableSelectorMinMax(language.getTextHtml(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS), StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS);
		minMax = new VariableListTable(equations, variableSelectorMinMax, documentor.displayModule());
		
		variableSelectorInOutLinks = new VariableSelector(language.getTextHtml(StringConstants.IN_OUT_LINKS),StringConstants.IN_OUT_LINKS);
		inOutLinks = new VariableListTableExtraColumns(equations, variableSelectorInOutLinks, VariableListTableRowExtraColumns.IN_OUT_LINKS, documentor);

		variableSelectorComplexity = new VariableSelectorComplexity(language.getTextHtml(StringConstants.COMPLEX_VARIABLE),StringConstants.COMPLEX_VARIABLE);
		complexity = new VariableListTableExtraColumns(equations, variableSelectorComplexity, VariableListTableRowExtraColumns.COMPLEXITY, documentor);

		variableSelectorCompletxityStock = new VariableSelectorComplexityStock(language.getTextHtml(StringConstants.COMPLEX_STOCK),StringConstants.COMPLEX_STOCK);
		complexityStock = new VariableListTable(equations, variableSelectorCompletxityStock, documentor.displayModule());

		variableSelectorState = new VariableSelectorState(language.getTextHtml(StringConstants.STATE_VARIABLES), StringConstants.STATE_VARIABLES);
		state = new VariableListTable(equations, variableSelectorState, documentor.displayModule());

		variableSelectorNotInView = new VariableSelectorNotInView(language.getTextHtml(StringConstants.VARIABLES_NOT_IN_ANY_VIEW), StringConstants.VARIABLES_NOT_IN_ANY_VIEW);
		notInView = new VariableListTable(equations, variableSelectorNotInView, documentor.displayModule());

		variableSelectorUnitErrorsWarnings = new VariableSelectorUnitErrorsWarnings(language.getTextHtml(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS), StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS);
		unitErrorsWarnings = new VariableListTableExtraColumns(equations, variableSelectorUnitErrorsWarnings, VariableListTableRowExtraColumns.UNITS, documentor);

		variableSelectorFeedbackLoops = new VariableSelector(language.getTextHtml(StringConstants.FEEDBACK_LOOPS),StringConstants.FEEDBACK_LOOPS);
		feedbackLoops = new VariableListTableExtraColumns(equations, variableSelectorFeedbackLoops, VariableListTableRowExtraColumns.FEEDBACK, documentor);
		
		variableSelectorExogenous = new VariableSelectorExogenous(language.getTextHtml(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS),StringConstants.EXOGENOUS_VARIABLE_ANALYSIS);
		exogenous = new VariableListTableExtraColumns(equations, variableSelectorExogenous, VariableListTableRowExtraColumns.EXOGENOUS, documentor);
		
		variableSelectorEndogenous = new VariableSelectorEndogenous(language.getTextHtml(StringConstants.ENDOGENOUS_VARIABLE_ANALYSIS),StringConstants.ENDOGENOUS_VARIABLE_ANALYSIS);
		endogenous = new VariableListTableExtraColumns(equations, variableSelectorEndogenous, VariableListTableRowExtraColumns.ENDOGENOUS, documentor);

		variableSelectorSourceInformation = new VariableSelectorSourceInformation(language.getTextHtml(StringConstants.VARIABLES_WITH_SOURCE_INFORMATION),StringConstants.VARIABLES_WITH_SOURCE_INFORMATION);
		sources = new VariableListTableExtraColumns(equations, variableSelectorSourceInformation, VariableListTableRowExtraColumns.SOURCES, documentor);

		variableSelectorDmnl = new VariableSelectorDmnlUnits(language.getTextHtml(StringConstants.VARIABLES_WITH_DIMENSIONLESS_UNITS),StringConstants.DIMENSIONLESS_UNIT_VARIABLES);
		dmnl = new VariableListTable(equations, variableSelectorDmnl, documentor.displayModule());
		
		variableSelectorValueRangeSpecified = new VariableSelectorValueRangeSpecified(language.getTextHtml(StringConstants.VALUE_RANGE_SPECIFIED),StringConstants.VALUE_RANGE_SPECIFIED);
		valueRange = new VariableListTable(equations, variableSelectorValueRangeSpecified, documentor.displayModule());


		variableSelectorSensitivity = new VariableSelectorFunctionSensitivityParameter(language.getTextHtml(StringConstants.FUNCTION_SENSITIVITY_PARAMETERS),StringConstants.FUNCTION_SENSITIVITY_PARAMETERS);
		sensivities = new VariableListTable(equations, variableSelectorSensitivity, documentor.displayModule());

		variableSelectorDataLookup = new VariableSelectorDataLookup(language.getTextHtml(StringConstants.DATA_LOOKUP_TABLES),StringConstants.DATA_LOOKUP_TABLES);
		dataLookups = new VariableListTable(equations, variableSelectorDataLookup, documentor.displayModule());

		variableSelectorUsingMacros = new VariableSelectorUsingMacros(language.getTextHtml(StringConstants.USING_MACROS),StringConstants.USING_MACROS);
		usingMacros = new VariableListTable(equations, variableSelectorUsingMacros, documentor.displayModule());
		
		arrowPolaritySelectorPositve = new ArrowPolaritySelectorPositive(language.getTextHtml(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS),StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS);
		positivePolarityArrows = new ArrowListTable(documentor.getAnalysisManager().getPolarityResultsNoIVV(), arrowPolaritySelectorPositve);
		
		arrowPolaritySelectorNegative = new ArrowPolaritySelectorNegative(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS),StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS);
		negativePolarityArrows = new ArrowListTable(documentor.getAnalysisManager().getPolarityResultsNoIVV(), arrowPolaritySelectorNegative);
		
		arrowPolaritySelectorUnknown = new ArrowPolaritySelectorUnknown(language.getTextHtml(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS),StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS);
		unknownPolarityArrows = new ArrowListTable(documentor.getAnalysisManager().getPolarityResultsNoIVV(), arrowPolaritySelectorUnknown);
		
		macroTable = new MacroTable();
		unitsTable = new UnitsTable();
		rateToRateTable = new RateToRateTable();

		viewVariablesBarTable = new ViewVariablesBarTable(documentor.getViewVariables());

		quickLink = new QuickLink(documentor);
		
		quickLinkAlphabetDetail = new QuickLinkAlphabet(documentor, "Detail");


	}
	
	private void finish() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		modelInformation = new MultiSectionTable(language.getTextHtml(StringConstants.MODEL_INFORMATION), documentor.getModelInformation());
		warnings = new MultiSectionTable(language.getTextHtml(StringConstants.WARNINGS), documentor.getWarnings());
		possibleOmissions = new MultiSectionTable(language.getTextHtml(StringConstants.POTENTIAL_OMISSIONS), documentor.getPotentialOmissions());
		multiTableHeader = new MultiTableHeader();
	}
	
	
	public boolean getXML(BufferedWriter sb) {
		finish();
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		
//		StringBuilder sb = new StringBuilder();
		try {
			sb.append(documentHeader.getXML().replace("$$$ADEF$$$", Anchors.getAnchorDefinition("alttop")));
			
			sb.append(quickLink.getXML());
			sb.append(multiTableHeader.getXML().replace("$$$ADEF$$$", Anchors.getAnchorDefinition("alttop")));
			
			sb.append(modelInformation.getXML());
			
			sb.append(warnings.getXML());
			
			
			if (!options.isConfiguredForMap())
				sb.append(possibleOmissions.getXML());
			
			sb.append(spacer());
			
			sb.append(typesTable.getXML());
			
			sb.append(viewsTable.getXML());
			
			sb.append(groupsTable.getXML());
			if (documentor.displayModule())
				sb.append(modulesTable.getXML());
			
			// All Variable detail
			
			sb.append(quickLinkAlphabetDetail.getXML());
			
			for (VariableListTableMultiValueColumns mvi : modelVariables) {
				mf.addMessage(mvi.getGroupingName());
				if (mvi.getGroupingName().equalsIgnoreCase("smooth")) {
					mvi.setExtra("("+typeCounter.getSmooth()+"/"+typeCounter.getSmoothTotal()+")");
				}
				if (mvi.getGroupingName().equalsIgnoreCase("delay")) {
					mvi.setExtra("("+typeCounter.getDelay()+"/"+typeCounter.getDelayTotal()+")");
				}
				if (mvi.getXML(sb)) {
					
				} else {
					
				}
			}
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
	
		
//		public static final String[] WARNINGS_CHOICES = {
//			"Undocumented Equations", "Equations with Embedded Data", "Equations With Unit Errors or Warnings", "Variables Not in Any View", 
//			"Incompletely Defined Subscripted Variables", "Nonmonotonic Lookup Functions",
//			"Cascading (Chained) Lookup Functions", "Non-Zero End Sloped Lookup Functions", "Equations with \"IF THEN ELSE functions\"", 
//			"Equations with \"MIN\" or \"MAX\" Functions", "Equations with \"STEP\", \"PULSE\", or Related Functions",
//			"Ignore Embedded Constants of 0 or 1"
//		};
//
//		public static final String[] POTENTIAL_OMISSIONS_CHOICES = {
//			"Unused Variables", "Supplementary Variables", "Supplementary Variables Being Used", 
//			"Complex Variable Formulations (Richardson's Rule)", "Complex Stock Formulations"
		
		
		InformationManagers.getInstance().getUnitsManager().report(null);
		appendToXML("", sb, allVariables );
		appendToXML("", sb, inOutLinks );
		appendToXML(StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES, sb, undocumented );
		appendToXML(StringConstants.SUPPLEMENTARY_VARIABLES, sb, supplementary );
		appendToXML(StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, sb, supplementaryUsed );
		appendToXML(StringConstants.UNUSED_VARIABLES, sb, unused );
		appendToXML(StringConstants.TOTAL_NUMBER_OF_STOCKS, sb, level );
		
		appendToXML(StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, sb, embeddedData );
		appendToXML(StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS, sb, nonmonotonic );
		appendToXML(StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, sb, nonZeroEnd );
		appendToXML(StringConstants.CASCADING_LOOKUP_FUNCTIONS, sb, cascading );
		appendToXML(StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS, sb, stepPulse );
		appendToXML(StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, sb, ifThenElse );
		appendToXML(StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, sb, minMax );
//		appendToXML(StringConstants.IN_OUT_LINKS, sb, inOutLinks );
		
		appendToXML(StringConstants.COMPLEX_VARIABLE, sb, complexity );
		appendToXML(StringConstants.COMPLEX_STOCK, sb, complexityStock );
		appendToXML(StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES, sb, state );
		
		appendToXML(StringConstants.VARIABLES_WITH_SOURCE_INFORMATION, sb, sources );
		appendToXML(StringConstants.DIMENSIONLESS_UNIT_VARIABLES, sb, dmnl );
		appendToXML(StringConstants.VALUE_RANGE_SPECIFIED, sb, valueRange );
		appendToXML(StringConstants.FUNCTION_SENSITIVITY_PARAMETERS, sb, sensivities );
		appendToXML(StringConstants.DATA_LOOKUP_TABLES, sb, dataLookups );
		
		appendToXML(StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS, sb, usingMacros );
		
		appendToXML(StringConstants.VARIABLES_NOT_IN_ANY_VIEW, sb, notInView );
		appendToXML(StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, sb, unitErrorsWarnings );
		
		if (options.isSelected(StringConstants.UNITS_USED)) {
			appendToXML(StringConstants.UNITS_NAME, sb, unitsTable );
		}
		
		try {
			sb.append(Anchors.getAnchorDefinition(StringConstants.FEEDBACK_LOOPS_WITH_IVV));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		appendToXML(StringConstants.FEEDBACK_LOOPS, sb, feedbackLoops );
		
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES)) {
			appendToXML(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES, sb, exogenous );
//			appendToXML(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES, sb, endogenous );
		}
		
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES)) {
			appendToXML(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES, sb, endogenous );
		}
		
		appendToXML(StringConstants.MACROS, sb, macroTable );
		
		appendArrowPolarity(sb);
		try {
			sb.append(Anchors.getAnchorDefinition(StringConstants.QUICK_LINK_NAME_ARROW_POLARITY));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			sb.append(Anchors.getAnchorDefinition(StringConstants.CAUSAL_LINKS));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		appendToXML(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS, sb, positivePolarityArrows );
		appendToXML(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS, sb, negativePolarityArrows );
		appendToXML(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS, sb, unknownPolarityArrows );
		
		
			appendToXML(StringConstants.RATE_TO_RATE_LINKS, sb, rateToRateTable );
		
		
		
		
		try {
			sb.append(viewVariablesBarTable.getXML());
			sb.append(documentor.getViewVariables().getXML());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (options.isSelected(StringConstants.VARIABLE_RELATIONSHIPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
			LoopManager loopManager = InformationManagers.getInstance().getAnalysisManager().getLoopManager();
			String someHTML = loopManager.getVariableRelationships().getXML();
			loopManager.writeVariableRelationships(someHTML);
		}
		
		if (options.isSelected(StringConstants.VARIABLE_LOOPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
			LoopManager loopManager = InformationManagers.getInstance().getAnalysisManager().getLoopManager();
			String someHTML = loopManager.getVariableLoops().getXML();
			loopManager.writeVariableLoops(someHTML);
		}
		
		
		try {
			sb.append(documentFooter.getXML());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return true;
	}
	
	private String spacer() {
		return "</br>";
	}
	
	private void appendArrowPolarity(/*StringBuilder*/ BufferedWriter sb) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS) ||
				options.isSelected(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS) ||
				options.isSelected(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS)) {
			try {
				sb.append(Anchors.getAnchorDefinition(StringConstants.QUICK_LINK_NAME_ARROW_POLARITY));
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}
	
	private void appendToXML(String option, /*StringBuilder*/ BufferedWriter sb, GetXML table) {
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (option.equals("") || options.isSelected(option) || option.equals(StringConstants.MACROS)) {
		
				try {
					sb.append(table.getXML());
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			
		} else {
//			System.out.println("skip append to xml "+option+" selected? "+options.isSelected(option));
		}
		
	}

	
	public String getXML() {
		// TODO Auto-generated method stub
		return "";
		
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
