package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.StandaloneGUI;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.attribute.BasicFileAttributeView;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileTime;
import java.text.DateFormat;
import java.util.Date;
import java.util.List;

public class DocumentFooter implements GetXML {
	
	private final static String HEADER = "<hr style='height:1px; color:Blue'/>\n" +
			"<div align='right' style='font-size:small;'>\n" +
			"<span style='font-family:Microsoft Sans Serif; font-size:x-small;'>\n" +
			"##SourceFile##: ";
	
	private final static String FOOTER = "\n" +
			"<br/>##Report Created on## $$$TIMESTAMP$$$" +
			"<br/><a href='http://www.gss.anl.gov/system-dynamics-model-documentation-tool-sdm-doc/'>" +
//			"<br/><a href='http://www.gss.anl.gov/feedback/sys_dyn_model.php?tname=your%20name&temail=your%20email&tphone=&tlocation=&tversion=5.0'>"
			"##SDMDocTool##</a> ##Version## "  +StandaloneGUI.version +
			"<br/><a href='http://www.gss.anl.gov'>##GlobalSecuritySciencesDivision##</a>\n" +
			"<br/><a href='http://www.anl.gov'>##ArgonneNationalLaboratory##</a>\n" +
			"</span>\n" +
			"</div>\n" +
			"</body>\n" +
			"</html>\n";
	
	private String sourceFile;
	
	public DocumentFooter(String sourceFile) {
		this.sourceFile = sourceFile;
	}
	
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	public String getXML() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		DateFormat df = DateFormat.getDateInstance(DateFormat.LONG, SDMDocLanguageSpecific.getInstance().getLocale());
		String formattedDate = df.format( new Date());
		formattedDate = new Date().toString();
		
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER.replace("##SourceFile##", language.getTextHtml(StringConstants.SOURCE_FILE)));
		sb.append(sourceFile+"&nbsp;("+fileModTime(sourceFile)+")");
		sb.append(FOOTER.replace("$$$TIMESTAMP$$$", formattedDate)
				.replace("##SDMDocTool##", language.getTextHtml(StringConstants.SDM_DOC_TOOL))
				.replace("##GlobalSecuritySciencesDivision##", "Global Security Sciences Division")
				.replace("##Version##", language.getTextHtml(StringConstants.VERSION))
				.replace("##ArgonneNationalLaboratory##", "Argonne National Laboratory") // ReportCreatedOn
				.replace("##Report Created on##", language.getTextHtml(StringConstants.REPORT_CREATED_ON)));
		
		return sb.toString();
	}
	
	private String fileModTime(String file) {
		File f = new File(file);
		Path p = Paths.get(f.getAbsoluteFile().toURI());
		BasicFileAttributes view = null;
		try {
		    view = Files.getFileAttributeView(p,
		                            BasicFileAttributeView.class).readAttributes();
		} catch (IOException e) {
		    // TODO Auto-generated catch block
		    e.printStackTrace();
		}
		FileTime creationTime = view.lastModifiedTime();
		
		return new Date(creationTime.toMillis()).toString();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
