package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.MacroManager;

import java.io.BufferedWriter;
import java.util.List;

public class MacroTable implements GetXML {
	
	protected static final String TITLE_MARKER = "$$$TITLE$$$";
	protected static final String NAME_MARKER = "$$$NAME$$$";
	protected static final String NUM_MARKER = "$$$NUM$$$";
	protected static final String TOP_MARKER = "$$$TOP$$$";
	
	protected static final String DATA_TABLE_HEADER = 
			"$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)";

	protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/>\n" +
			"$$$NAME$$$$$$TOP$$$\n" +
			"<h3>$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)</h3>\n" +
			"<table  class='sample'>\n";
//	"<table align='center'  class='sample'>\n";
	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
	
	protected static final String COLUMN_START = "<td>\n";
	protected static final String COLUMN_END = "</td>\n";
	
	protected static final String ROW_START = "<tr>\n";
	protected static final String ROW_END = "</tr>\n";
	
	protected static final String COLUMN_HEADERS = "<tr>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Name##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Macro Definition##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Expanded Macro Definition##</center></td>\n" +
			"</tr>\n";
	
	protected String title;
	
	
	protected MacroTable() {
	}
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	public String getXML() {
		MacroManager mm = InformationManagers.getInstance().getMacroManager();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.MACROS)).replace(NAME_MARKER, 
				Anchors.getAnchorDefinition(StringConstants.MACROS)).
				replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
				replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(mm.getMacros().size())).replace(TOP_MARKER, 
						Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		sb.append(COLUMN_HEADERS
				.replace("##Name##", language.getTextHtml(StringConstants.NAME))
				.replace("##Macro Definition##", language.getTextHtml(StringConstants.MACRO_DEFINITION))
				.replace("##Expanded Macro Definition##", language.getTextHtml(StringConstants.EXPANDED_MACRO_DEFINITION)));
		
		for (String macroName : mm.getMacros().keySet()) {
			sb.append(ROW_START);
			sb.append(COLUMN_START);
			sb.append(macroName);
			sb.append(COLUMN_END);

			sb.append(COLUMN_START);
			sb.append(mm.getMacros().get(macroName).getOriginalMacroEquation());
			sb.append(COLUMN_END);
			
			sb.append(COLUMN_START);
			sb.append(mm.getMacros().get(macroName).getExpandedMacroEquation());
			sb.append(COLUMN_END);
			
			sb.append(ROW_END);
		}
		
		
		sb.append(FOOTER);
		sb.append(BREAK);
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {
			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Macros"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		
//		sb.append(SEPARATOR);
		return sb.toString();
	}
	@Override
	public List<List<List<String>>> getTableData() {
		
		MacroManager mm = InformationManagers.getInstance().getMacroManager();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();

		String titlel;
		

			titlel = DATA_TABLE_HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.MACROS)).replace(NAME_MARKER, 
					Anchors.getAnchorDefinition(StringConstants.MACROS)).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(mm.getMacros().size())).replace(TOP_MARKER, 
							Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP)));
		
		TableUtilities.addTitle(table, titlel);
		
		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NAME)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MACRO_DEFINITION)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EXPANDED_MACRO_DEFINITION)));
		
		for (String macroName : mm.getMacros().keySet()) {
			TableUtilities.addRow(table, mm.getMacros().get(macroName).getRowData());
		}
		
		return table;
	}
}
