package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableRow;
import gov.anl.gss.sdm.server.model.documentor.VariableInformation;
import gov.anl.gss.sdm.server.model.translator.Equation;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class MultiValueMultiColumnTable implements GetXML {

	private static final String HEADER = "<table align='center' class='sample'>";
	private static final String FOOTER = "</table>\n";
	
	private static final String MODULE_HEADER = "<td style='text-align:center'>";
	private static final String MODULE_FOOTER = "</td>\n";
	
	private static final String COLUMN_HEADERS = "<tr>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Module##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><i>Variable Name</i></b></a> and Description</td>\n" +
			"</tr>\n";
	
	private static final String NL = "\n";
	private static final String LHS_HEADER = NL+"<td>";
	private static final String RHS_HEADER = NL+"<br/>";
	private static final String VIEWS_HEADER = NL+"<br/><b>Present in $$$NUMBER$$$ Views:</b><ul>";
	private static final String VIEWS_ITEM_HEADER = NL+"<li>";
//	private static final String USED_BY_HEADER = "<br/><b>Used By:</b><ul>";
	private static final String USED_BY_HEADER = NL+"<b>##UsedBy##</b><ul>";
	private static final String USED_BY_ITEM_HEADER = NL+"<li>";
//	private static final String FEEDBACK_LOOPS_HEADER = "<br/><b>Feedback Loops:</b>";
	private static final String FEEDBACK_LOOPS_HEADER = "NL+<b>##FeedbackLoops##:</b>";
	
	private static final String LHS_FOOTER = "";
	private static final String RHS_FOOTER = "";
	private static final String VIEWS_FOOTER = NL+"</ul>";
	private static final String VIEWS_ITEM_FOOTER = NL+"</li>";
	private static final String USED_BY_FOOTER = NL+"</ul>";
	private static final String USED_BY_ITEM_FOOTER = NL+"</li>";
	private static final String FEEDBACK_LOOPS_FOOTER = NL+"</td></tr>\n";
	
	private String title;
	
	private List<VariableListTableRowMultiValueColumn> variables;
	private TableRow modelVariablesInformation;
	
	public MultiValueMultiColumnTable(TableRow modelVariablesInformation) {
		this.modelVariablesInformation = modelVariablesInformation;
	}
	
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	
	public String getXML() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER);
		sb.append(COLUMN_HEADERS);
		for (List<List<String>> row : modelVariablesInformation.getRows()) {
			// module column
			sb.append(MODULE_HEADER);
			sb.append(row.get(0).get(0));
			sb.append(MODULE_FOOTER);
			// group Column
			sb.append(MODULE_HEADER);
			sb.append(row.get(1).get(0));
			sb.append(MODULE_FOOTER);
			
			// type Column
			sb.append(MODULE_HEADER);
			sb.append(row.get(2).get(0));
			sb.append(MODULE_FOOTER);
			
			int i = 0;
			String item = "";
			
			// lhs
			sb.append(LHS_HEADER);
			sb.append(row.get(3).get(0));
			sb.append(LHS_FOOTER);
			// rhs
			sb.append(RHS_HEADER);
			sb.append(row.get(3).get(1));
			sb.append(RHS_FOOTER);
			
//			List<String> errors = equation.getSemanticMessages();
//			if (errors.size() > 0) {
//				for (String msg : errors) {
//				sb.append("<br/><b>ERROR: "+msg+"</b>");
//				}
//			}
			
			// views
			i = 0;
			int count = 0;
			item = row.get(4).get(i++);
			while (!item.equals(VariableInformation.CELL_SECTION_MARKER)) {
				count++;
				item = row.get(4).get(i++);
			}
			sb.append(VIEWS_HEADER.replace("$$$NUMBER$$$", NumberFormatter.formatForCountNotConditional(count)));
			i = 0;
			item = row.get(4).get(i++);
			while (!item.equals(VariableInformation.CELL_SECTION_MARKER)) {
				sb.append(VIEWS_ITEM_HEADER);
				sb.append(item);
				sb.append(VIEWS_ITEM_FOOTER);
				item = row.get(4).get(i++);
			}
			sb.append(VIEWS_FOOTER);
			// used by
			sb.append(USED_BY_HEADER.replace("##UsedBy##", language.getTextHtml(StringConstants.USED_BY)));
//			i = 0;
			item = row.get(4).get(i++);
			while (!item.equals(VariableInformation.CELL_SECTION_MARKER)) {
				sb.append(USED_BY_ITEM_HEADER);
				sb.append(item);
				sb.append(USED_BY_ITEM_FOOTER);
				item = row.get(4).get(i++);
			}
			sb.append(USED_BY_FOOTER);
			// feedback loops
			sb.append(FEEDBACK_LOOPS_HEADER.replace("##FeedbackLoops##", language.getTextHtml(StringConstants.FEEDBACK_LOOPS)));
			sb.append(row.get(4).get(i));
			sb.append(FEEDBACK_LOOPS_FOOTER);
		}
		
		sb.append(FOOTER);
		return sb.toString();
	}
	
	private List<String> sortByVariableName(Map<String, Equation> equations) {
		List<String> vars = new ArrayList<String>();
		for (String key : equations.keySet())
			vars.add(key);
		Collections.sort(vars);
		return vars;
		
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
