package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

import java.io.BufferedWriter;
import java.util.List;

public class QuickLink implements GetXML {


	protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/><br/>\n" +
			"<table align='center' class='sample'>\n";
	protected static final String FOOTER = "</table><br>\n";
	protected static final String BREAK = "<br/>\n";
	
	protected static final String COLUMN_START = "<td>\n";
	protected static final String COLUMN_END = "</td>\n";
	
	protected static final String ROW_START = "<tr>\n";
	protected static final String ROW_END = "</tr>\n";
	
	private Documentor documentor;
	
	public QuickLink(Documentor documentor) {
		this.documentor = documentor;
	}

	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	public String getXML() {
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		// only a single row
		StringBuilder sb = new StringBuilder();

		sb.append(HEADER);
		sb.append(Anchors.getAnchorDefinition("QuickLink"));
		sb.append(ROW_START);

		
		sb.append(COLUMN_START);
		sb.append("<b>"+language.getTextHtml(StringConstants.QUICK_LINKS)+"</b>");
		sb.append(COLUMN_END);

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_VARIABLES, language.getTextHtml(StringConstants.ALL_VARIABLES)));
		sb.append(COLUMN_END);
		
		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_IN_OUT_LINKS, language.getTextHtml(StringConstants.IN_OUT_LINKS)));
		sb.append(COLUMN_END);
		
//		VariableTypes = Tipos de variable
//				Views = Vistas
//				Groups = Grupos
//				Macros = Subrutinas
//				FeedbackLoops = Ciclos de retroalimentacin

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_VARIABLE_TYPE, language.getTextHtml(StringConstants.VARIABLE_TYPES)));
		sb.append(COLUMN_END);

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_VIEWS, language.getTextHtml(StringConstants.VIEWS)));
		sb.append(COLUMN_END);

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_GROUPS, language.getTextHtml(StringConstants.GROUPS)));
		sb.append(COLUMN_END);

		if (documentor.displayModule()) {
			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_MODULES, language.getTextHtml(StringConstants.MODULES)));
			sb.append(COLUMN_END);
		}
		
		if (options.isSelected(StringConstants.UNITS_USED)) {

			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_UNITS, language.getTextHtml(StringConstants.UNITS_NAME)));
			sb.append(COLUMN_END);
		}

		LoopManager lm = documentor.getAnalysisManager().getLoopManager();
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		
		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_MACROS, language.getTextHtml(StringConstants.MACROS)));
		sb.append(COLUMN_END);

		if (options.isSelected(StringConstants.ANALYZE_LOOPS)) {
			if (options.isSelected(StringConstants.FEEDBACK_LOOPS)) {
				sb.append(COLUMN_START);
				sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_FEEDBACK_LOOPS, language.getTextHtml(StringConstants.FEEDBACK_LOOPS)));
				sb.append(COLUMN_END);
			}

			if (!InformationManagers.getInstance().isLimitLoopProcessing()) {
				if (options.isSelected(StringConstants.HYPERLINK_LOOP)  && options.isSelected(StringConstants.DETAILED_REPORTS)) {

					sb.append(COLUMN_START);
					sb.append(Anchors.getFileReference(lm.getHtmlLoopFileNameNoIVV(), language.getTextHtml(StringConstants.LOOP_LIST_NO_IVV)));
					sb.append(COLUMN_END);


					if (options.isSelected(StringConstants.ANALYZE_LOOPS_WITH_IVV) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
						sb.append(COLUMN_START);
						sb.append(Anchors.getFileReference(lm.getHtmlLoopFileNameWithIVV(), language.getTextHtml(StringConstants.LOOP_LIST_WITH_IVV)));
						sb.append(COLUMN_END);
					}
				}
			}
		}
		
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES)) {

			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_EXOGENOUS, language.getTextHtml(StringConstants.EXOGENOUS_VARIABLE_ANALYSIS)));
			sb.append(COLUMN_END);
//			sb.append(COLUMN_START);
//			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_ENDOGENOUS, language.getTextHtml(StringConstants.ENDOGENOUS)));
//			sb.append(COLUMN_END);
		}
		
		if (options.isSelected(StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES)) {

			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_ENDOGENOUS, language.getTextHtml(StringConstants.ENDOGENOUS_VARIABLE_ANALYSIS)));
			sb.append(COLUMN_END);
		}

		if (options.isSelected(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS) ||
				options.isSelected(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS) ||
				options.isSelected(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS)) {
			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_ARROW_POLARITY, language.getTextHtml(StringConstants.LINK_POLARITY)));
			sb.append(COLUMN_END);
		}

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_NAME_VIEW_VARIABLES, language.getTextHtml(StringConstants.VIEW_SUMMARY)));
		sb.append(COLUMN_END);

		sb.append(COLUMN_START);
		sb.append(Anchors.getAnchorReference(StringConstants.QUICK_LINK_TITLE_VIEW_VARIABLE_PROFILE, language.getTextHtml(StringConstants.VIEW_VARIABLE_PROFILE)));
		sb.append(COLUMN_END);
		
		if (options.isSelected(StringConstants.ANALYZE_LOOPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
			if (options.isSelected(StringConstants.VARIABLE_RELATIONSHIPS)) {
				sb.append(COLUMN_START);
				sb.append(Anchors.getFileReference(lm.getHtmlVariableRelationshipLoopsFileName() /*StringConstants.VARIABLE_RELATIONSHIPS */, language.getTextHtml(StringConstants.VARIABLE_RELATIONSHIPS)));
				sb.append(COLUMN_END);
			}
		}
		
		if (options.isSelected(StringConstants.ANALYZE_LOOPS)) {
			if (options.isSelected(StringConstants.VARIABLE_LOOPS) && options.isSelected(StringConstants.DETAILED_REPORTS)) {
				sb.append(COLUMN_START);
				sb.append(Anchors.getFileReference(lm.getHtmlVariableLoopsFileName() /* StringConstants.VARIABLE_LOOPS */, language.getTextHtml(StringConstants.VARIABLE_LOOPS)));
				sb.append(COLUMN_END);
			}
		}

		sb.append(ROW_END);
		sb.append(FOOTER);

		return sb.toString();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
