package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.VariableInformation;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.translator.Equation;

import java.io.BufferedWriter;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class QuickLinkAlphabet implements GetXML {



//	protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/><br/>\n" +
//			"<table align='center' class='sample'>\n";
//	protected static final String HEADER = "<br/>\n" +
//			"<table align='center' class='sample'>\n";
	
	protected static final String HEADER = "<br/><br/>\n" +
			"<table class='sample'>\n";
	
	protected static final String FOOTER = "</table><br>\n";
//	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	
	protected static final String COLUMN_START = "<td>\n";
	protected static final String COLUMN_END = "</td>\n";
	
	protected static final String ROW_START = "<tr>\n";
	protected static final String ROW_END = "</tr>\n";
	
	protected static final String SPACE = "&nbsp;";
	
	protected static final String ALPHABET = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	
	private Documentor documentor;
	private String marker;
	private Set<String> active;
	
	// Map<String, VariableInformation>()
	
	public QuickLinkAlphabet( Map<String, VariableInformation> eqnMap, String marker) {
		this.marker = marker;
		
		initializeActive(eqnMap);
	}
	
	public QuickLinkAlphabet(List<VariableListTableRow> eqnList, String marker) {
		this.marker = marker;
		initializeActive(eqnList);
	}
	
	public QuickLinkAlphabet(Documentor documentor, String marker) {
		this.documentor = documentor;
		this.marker = marker;
		initializeActive();
	}
	
	private void initializeActive(Map<String, VariableInformation> eqnMap) {
		active = new LinkedHashSet<String>();
		
		for (VariableInformation lte : eqnMap.values()) {
			MutableInteger pos = new MutableInteger();
			Equation eqn = lte.getAnEquation();
			String orig = eqn.getOriginalLhs();
			if (orig.startsWith("\""))
				pos.add(1);
			String letter = orig.substring(pos.value(), pos.value()+1);
			active.add(letter.toUpperCase());
		}
	}
	
	private void initializeActive(List<VariableListTableRow> eqnList) {
		active = new LinkedHashSet<String>();
		
		for (VariableListTableRow lte : eqnList) {
			MutableInteger pos = new MutableInteger();
			Equation eqn = lte.equation;
			String orig = eqn.getOriginalLhs();
			if (orig.startsWith("\""))
				pos.add(1);
			String letter = orig.substring(pos.value(), pos.value()+1);
			active.add(letter.toUpperCase());
		}
	}
	
	private void initializeActive() {
		active = new LinkedHashSet<String>();
		Map<String, Equation> equations = documentor.getEquations();
		
		for (Equation eqn : equations.values()) {
			MutableInteger pos = new MutableInteger();
			String orig = eqn.getOriginalLhs();
			if (orig.startsWith("\""))
				pos.add(1);
			String letter = orig.substring(pos.value(), pos.value()+1);
			active.add(letter.toUpperCase());
		}
	}

	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	public String getXML() {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		// only a single row
		StringBuilder sb = new StringBuilder();

		sb.append(HEADER);
		sb.append(Anchors.getAnchorDefinition("QuickLink"));
		sb.append(ROW_START);

		sb.append(COLUMN_START);
		sb.append("<b>##QuickLinks##:</b>".replace("##QuickLinks##", language.getTextHtml(StringConstants.QUICK_LINKS)));
		sb.append(COLUMN_END);

		MutableInteger pos = new MutableInteger();
		while (pos.value() < ALPHABET.length()) {
			String letter = ALPHABET.substring(pos.value(), pos.value()+1);
			sb.append(COLUMN_START);
			if (active.contains(letter))
			sb.append(SPACE+Anchors.getAnchorReference(marker+letter, letter)+SPACE);
			else
			sb.append(SPACE+letter+SPACE);
			sb.append(COLUMN_END);
			pos.add(1);
		}

		sb.append(ROW_END);
		sb.append(FOOTER);

		return sb.toString();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
