package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.MacroManager;
import gov.anl.gss.sdm.server.model.translator.RateToRateLinkReport;

import java.io.BufferedWriter;
import java.util.List;

public class RateToRateTable implements GetXML {
	
	protected static final String TITLE_MARKER = "$$$TITLE$$$";
	protected static final String NAME_MARKER = "$$$NAME$$$";
	protected static final String NUM_MARKER = "$$$NUM$$$";
	protected static final String TOP_MARKER = "$$$TOP$$$";
	
	protected static final String DATA_TABLE_HEADER = 
			"$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)";

	protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/>\n" +
			"$$$NAME$$$$$$TOP$$$\n" +
			"<h3>$$$TITLE$$$ ($$$NUM$$$ $$$VARIABLES$$$)</h3>\n" +
			"<table  class='sample'>\n";
//	"<table align='center'  class='sample'>\n";
	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
	
	protected static final String COLUMN_START = "<td style='text-align:center'>\n";
	protected static final String COLUMN_END = "</td>\n";
	
	protected static final String ROW_START = "<tr>\n";
	protected static final String ROW_END = "</tr>\n";
	
	protected static final String COLUMN_HEADERS = "<tr>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Tail##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Head##</center></td>\n" +
			"</tr>\n";
	
	protected String title;
	
	
	protected RateToRateTable() {
	}
	
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	public String getXML() {
		RateToRateLinkReport rtr = new RateToRateLinkReport();
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.RATE_TO_RATE_LINKS))
				.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES))
				.replace(NAME_MARKER,Anchors.getAnchorDefinition(StringConstants.RATE_TO_RATE_LINKS)).
				replace(NUM_MARKER,NumberFormatter.formatForCountNotConditional(rtr.getNumRateToRate()))
				.replace(TOP_MARKER, 
						Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		sb.append(COLUMN_HEADERS
				.replace("##Tail##", language.getTextHtml(StringConstants.CAUSE))
				.replace("##Head##", language.getTextHtml(StringConstants.EFFECT)));
		
	
		for (List<String> s : rtr.getTableRows()) {
			sb.append(ROW_START);
			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(s.get(0)));
//			sb.append(zs.get(0));
			sb.append(COLUMN_END);

			sb.append(COLUMN_START);
			sb.append(Anchors.getAnchorReference(s.get(1)));
//			sb.append(s.get(1));
			sb.append(COLUMN_END);
			
			
			sb.append(ROW_END);
		}
		
	
		sb.append(FOOTER);
		sb.append(BREAK);
//		sb.append(SEPARATOR);
		return sb.toString();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		
		RateToRateLinkReport rtr = new RateToRateLinkReport();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();

		String titlel;
		

			titlel = DATA_TABLE_HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.RATE_TO_RATE_LINKS))
					.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES))
					.replace(NAME_MARKER,Anchors.getAnchorDefinition(StringConstants.RATE_TO_RATE_LINKS)).
					replace(NUM_MARKER,NumberFormatter.formatForCountNotConditional(rtr.getNumRateToRate()));
		
		TableUtilities.addTitle(table, titlel);
		
		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NAME)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MACRO_DEFINITION)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EXPANDED_MACRO_DEFINITION)));
		
		for (List<String> s : rtr.getTableRows()) {
			List<List<String>> aRow = TableUtilities.createRow();
			TableUtilities.addColumn(row, TableUtilities.createColumn(s.get(0)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(s.get(1)));
			
			TableUtilities.addRow(table, aRow);
		}
		
		return table;
	}
}
