package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.UnitsManager;

import java.io.BufferedWriter;
import java.util.List;

public class UnitsTable implements GetXML {
	
	protected static final String TITLE_MARKER = "$$$TITLE$$$";
	protected static final String NAME_MARKER = "$$$NAME$$$";
	protected static final String NUM_MARKER = "$$$NUM$$$";
	protected static final String TOP_MARKER = "$$$TOP$$$";
	
	protected static final String DATA_TABLE_HEADER = 
			"$$$TITLE$$$ ($$$NUM$$$)";

	protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/>\n" +
			"$$$NAME$$$$$$TOP$$$\n" +
			"<h3>$$$TITLE$$$ ($$$NUM$$$)</h3>\n" +
			"<table  class='sample'>\n";
//	"<table align='center'  class='sample'>\n";
	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
	
	protected static final String COLUMN_START = "<td>\n";
	protected static final String COLUMN_END = "</td>\n";
	
	protected static final String ROW_START = "<tr>\n";
	protected static final String ROW_END = "</tr>\n";
	
	protected static final String COLUMN_HEADERS = "<tr>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Units##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Alternates##</center></td>\n" +
			"</tr>\n";
	
	protected String title;
	
	
	protected UnitsTable() {
	}
	
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	
	
	public String getXML() {
		UnitsManager mm = InformationManagers.getInstance().getUnitsManager();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.UNITS_NAME)).replace(NAME_MARKER, 
				Anchors.getAnchorDefinition(StringConstants.UNITS_NAME)).
				replace(NUM_MARKER, 
						NumberFormatter.formatForCountNotConditional(mm.getBasicCount()) + "/"+NumberFormatter.formatForCountNotConditional(mm.getCombinedCount()))
						.replace(TOP_MARKER, 
						Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		sb.append(COLUMN_HEADERS
				.replace("##Units##", language.getTextHtml(StringConstants.UNITS_NAME))
				.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
				.replace("##Alternates##", language.getTextHtml(StringConstants.UNITS_ALTERNATES)));
		
	
		for (List<String> s : mm.getTableRows()) {
			sb.append(ROW_START);
			sb.append(COLUMN_START);
			sb.append(s.get(0));
			sb.append(COLUMN_END);

			sb.append(COLUMN_START);
			sb.append(s.get(1));
			sb.append(COLUMN_END);
			
			sb.append(COLUMN_START);
			sb.append(s.get(2));
			sb.append(COLUMN_END);
			
			sb.append(ROW_END);
		}
		
	
		sb.append(FOOTER);
		sb.append(BREAK);
//		sb.append(SEPARATOR);
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+"Units"+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		return sb.toString();
	}

	@Override
	public List<List<List<String>>> getTableData() {
		UnitsManager mm = InformationManagers.getInstance().getUnitsManager();
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		String titlel = DATA_TABLE_HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.UNITS_NAME)).replace(NAME_MARKER, 
				Anchors.getAnchorDefinition(StringConstants.UNITS_NAME)).
				replace(NUM_MARKER, 
						NumberFormatter.formatForCountNotConditional(mm.getBasicCount()) + "/"+NumberFormatter.formatForCountNotConditional(mm.getCombinedCount()));

		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		TableUtilities.addTitle(table, titlel);

		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);

		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NAME)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_ALTERNATES)));

		for (List<String> s : mm.getTableRows()) {

			List<List<String>> aRow = TableUtilities.createRow();
			
			TableUtilities.addColumn(aRow, TableUtilities.createColumn(s.get(0)));
			TableUtilities.addColumn(aRow, TableUtilities.createColumn(s.get(1)));
			TableUtilities.addColumn(aRow, TableUtilities.createColumn(s.get(2)));

			TableUtilities.addRow(table, aRow);

			
		}
		return table;
	}
}
