package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class UserSelectedOptions {

	// "xx",

	private  Map<String, String> optionValues = new LinkedHashMap<String, String>();
	private  Map<String, String> optionTypes= new LinkedHashMap<String, String>();
	
	private List<String> runFiles = new ArrayList<String>();
	
	public static final String TRUE = "true";
	public static final String FALSE = "false";
	
	

	public static final String[] MODEL_INFORMATION_CHOICES = {
		StringConstants.TOTAL_NUMBER_OF_VARIABLES, StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES, 
		StringConstants.TOTAL_NUMBER_OF_STOCKS, 
		StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES, 
		StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES,
		StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_NO_IVV, StringConstants.TOTAL_NUMBER_OF_FEEDBACK_LOOPS_WITH_IVV,
		StringConstants.UNITS_NAME,
		StringConstants.TOTAL_NUMBER_OF_CAUSAL_LINKS, /* VariableNames.NEGATIVE_CAUSAL_MAIN, VariableNames.UNKNOWN_CAUSAL_MAIN, */ 
		StringConstants.RATE_TO_RATE_LINKS, 
		StringConstants.TOTAL_NUMBER_OF_MACROS, StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS,
		StringConstants.VARIABLES_WITH_SOURCE_INFORMATION, StringConstants.DIMENSIONLESS_UNIT_VARIABLES, StringConstants.VALUE_RANGE_SPECIFIED, StringConstants.FUNCTION_SENSITIVITY_PARAMETERS, StringConstants.DATA_LOOKUP_TABLES,
		StringConstants.TIME_UNIT, StringConstants.INITIAL_TIME, StringConstants.FINAL_TIME, StringConstants.REPORTED_TIME_INTERVAL, 
		StringConstants.TIME_STEP, StringConstants.MODEL_IS_FULLY_FORMULATED, StringConstants.MODELER_DEFINED_GROUPS
	};

	public static final String[] VARIABLE_INFORMATION_CHOICES = {
		StringConstants.DESCRIPTION, StringConstants.META_DATA, StringConstants.PRESENT_IN_VIEWS, 
		StringConstants.USED_BY, StringConstants.FEEDBACK_LOOPS
	};
	
	public static final String[] ARROW_INFORMATION_CHOICES = {
		StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS, StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS, StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS
		
	};
	
	public static final String[] NAMING_CHOICES = {
		StringConstants.INCLUDE_LOOP_SIZE, StringConstants.INCLUDE_TIME_STAMP
	};
	
	
	
	public static final String[] VARIABLE_NAME_TRANSLATIONS = {
		StringConstants.INITIALIZE_TRANSLATION
	};
	
	public static final String[] REUSE = {
		StringConstants.REUSE_CLASS, StringConstants.REUSE_VIEW_DIAGRAM,
		StringConstants.REUSE_THUMBNAILS, StringConstants.REUSE_POLARITIES,
		 StringConstants.REUSE_LOOPS
	};

	public static final String[] WARNINGS_CHOICES = {
		StringConstants.NUMBER_OF_UNDOCUMENTED_VARIABLES, StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, StringConstants.VARIABLES_NOT_IN_ANY_VIEW, 
		/* StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES,*/ StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
		StringConstants.CASCADING_LOOKUP_FUNCTIONS, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, 
		StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,
		StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1
	};

	public static final String[] POTENTIAL_OMISSIONS_CHOICES = {
		StringConstants.UNUSED_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, 
		StringConstants.COMPLEX_VARIABLE, StringConstants.COMPLEX_STOCK
	};
	
//	public static final String[] EXOGENOUS = {
//		StringConstants.EXOGENOUS
//	};

	public static final String[] GRAPH_OUTPUT_OPTIONS = {
		StringConstants.WIDTH, StringConstants.HEIGHT, StringConstants.LEFT_MARGIN, StringConstants.BOTTOM_MARGIN, StringConstants.FONT_SIZE,
		StringConstants.VIEW_WIDTH_MAX, StringConstants.HIGHLIGHT_POINT, StringConstants.HIGHLIGHT_POINT_X, StringConstants.HIGHLIGHT_POINT_Y, 
		StringConstants.RULE_THRESHOLD, StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS, StringConstants.INCLUDE_MDL, StringConstants.INCLUDE_THUMBNAILS,
		StringConstants.LIMIT_THUMBNAIL_COUNT, StringConstants.THUMBNAIL_MAXIMUM, StringConstants.GENERATE_3D, StringConstants.MICMAC_ANALYSIS,
		StringConstants.CREATE_FLAT_FILES
	};
	
	
	public static final String[] CODE_GENERATION_OPTIONS = {
			StringConstants.EQUATION_LIMIT,	StringConstants.FLATTEN, StringConstants.COMMENTS,	
			StringConstants.RUNNER,	StringConstants.HASHMAP, StringConstants.NATIVE
	};
	
	public static final String[] LOOP_OPTIONS = {
		StringConstants.LOOP_SIZE_MAXIMUM, /*StringConstants.LOOP_SIZE_AUTO_DISCOVER,*/ StringConstants.ANALYZE_LOOPS, StringConstants.ANALYZE_LOOPS_WITH_IVV, 
			StringConstants.VARIABLE_RELATIONSHIPS, StringConstants.VARIABLE_LOOPS, StringConstants.OVERRIDE_LOOP_COUNT,
			StringConstants.MAX_SEARCH_TIME_MINUTES, StringConstants.LIMIT_VR_LOOPS, StringConstants.LIMIT_V_LOOPS,
			StringConstants.MAX_LIMIT_VR_LOOPS, StringConstants.MAX_LIMIT_V_LOOPS, StringConstants.HYPERLINK_LOOP,
			StringConstants.WRITE_LOOPS_TO_FILE, StringConstants.DETAILED_REPORTS, StringConstants.SUMMARY_REPORTS,
			 StringConstants.LIMIT_LONGEST_LOOPS, StringConstants.MAX_LIMIT_LONGEST_LOOPS,
			 StringConstants.LIMIT_LOOPS_PER_FILE, StringConstants.MAX_LIMIT_LOOPS_PER_FILE
	};

	public static final String[] OTHER_OPTIONS = {
		"xx", "xx", "xx", "xx", "xx",
		"xx", "xx", "xx", "xx"
	};
	
//	public static final String[] EXOGENOUS_OPTIONS = {
//		StringConstants.EXOGENOUS
//	};
	
	public final static String[] NA_FOR_MAP = {
		StringConstants.TOTAL_NUMBER_OF_STATE_VARIABLES, StringConstants.TOTAL_NUMBER_OF_STOCKS,
//		StringConstants.TOTAL_NUMBER_OF_EXOGENOUS_VARIABLES,
//		StringConstants.TOTAL_NUMBER_OF_ENDOGENOUS_VARIABLES,
		StringConstants.TOTAL_NUMBER_OF_MACROS, StringConstants.TOTAL_NUMBER_OF_EQUATIONS_USING_MACROS,
		StringConstants.VARIABLES_WITH_SOURCE_INFORMATION, StringConstants.DIMENSIONLESS_UNIT_VARIABLES, StringConstants.FUNCTION_SENSITIVITY_PARAMETERS, StringConstants.DATA_LOOKUP_TABLES,
		StringConstants.TIME_UNIT, StringConstants.INITIAL_TIME, StringConstants.FINAL_TIME, StringConstants.REPORTED_TIME_INTERVAL, 
		StringConstants.TIME_STEP,
		StringConstants.UNITS_USED,
		StringConstants.EQUATIONS_WITH_EMBEDDED_DATA, StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS, StringConstants.VARIABLES_NOT_IN_ANY_VIEW, 
		/* StringConstants.INCOMPLETELY_DEFINED_SUBSCRIPTED_VARIABLES, */StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
		StringConstants.CASCADING_LOOKUP_FUNCTIONS, StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS, StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS, 
		StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS, StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,
		StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1, StringConstants.UNUSED_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES, StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED, 
		StringConstants.COMPLEX_VARIABLE, StringConstants.COMPLEX_STOCK
		};
	
	private String filename;
	
	private boolean configuredForMap = false;
	private UserSelectedOptions nonMapUserSelectedOptions = null;
	
	private UserSelectedOptions startupUserSelectedOptions = null;
	
	// generates a new object
	public UserSelectedOptions(UserSelectedOptions another) {
		copyFrom(another);
	}
	
	// just copies values
	public void copyFrom(UserSelectedOptions another) {
		optionValues.clear();
		optionTypes.clear();
		filename = another.filename;
		for (String key : another.optionValues.keySet())
			optionValues.put(key, another.optionValues.get(key));
		for (String key : another.optionTypes.keySet())
			optionTypes.put(key, another.optionTypes.get(key));
	}
	
	public UserSelectedOptions(boolean load) {
		
		setDefaultGraph();
		setSelectAllTrue();
		
		if (load) {
			filename = "data/SDM-Doc_UserOptions.tab";
			File file = new File(filename);
			if (file.exists()) {
				load();
			} else {
				System.out.println("Cannot find UserOptions: "+filename);
				setDefaultGraph();
				setSelectAllTrue();
				save();
			}

			startupUserSelectedOptions = new UserSelectedOptions(this);
		}
	}
	
	public UserSelectedOptions(String filename) {
		this.filename = filename;
		load();
	}
	
	public void close() {
		if (configuredForMap) {
			if (nonMapUserSelectedOptions != null) 
				nonMapUserSelectedOptions.save();
			else
				startupUserSelectedOptions.save();
		}
	}
	
	public void print() {
		
		System.out.println("\n### Option Values ###");
		
			for (String key : optionValues.keySet()) {
				
					System.out.print(key);
					System.out.print("\t");
					System.out.print(optionTypes.get(key));
					System.out.print("\t");
					System.out.print(optionValues.get(key));
					System.out.print("\n");
				
			}

	}
	
	public void save() {
		BufferedWriter bw = Utilities.openFileForWriting(filename);
		try {
			for (String key : optionValues.keySet()) {
				
					bw.append(key);
					bw.append("\t");
					bw.append(optionTypes.get(key));
					bw.append("\t");
					bw.append(optionValues.get(key));
					bw.append("\n");
				
			}
			bw.close();
			startupUserSelectedOptions = new UserSelectedOptions(this);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void load() {
		BufferedReader br = Utilities.openForRead(filename);
		List<String> options = Utilities.getAsList(br);
		for (String opt : options) {
			
			String[] parts = opt.split("\\t");
			
			// sometimes int values are passed properly
			if (parts.length == 2) {
				setValue(parts[0], "int", "0");
			} else {
				if (parts[1].equals("null")) {
					if (parts[2].equals("true") || parts[2].equals("false"))
						parts[1] = "boolean";
					else
						parts[1] = "int";
				}
				setValue(parts[0], parts[1], parts[2]);
			}
		}
		UserSelectedOptions uso = this;
		Reuse reuse = InformationManagers.getInstance().getReuse();
		reuse.setClassFile(uso.getOptionValue(StringConstants.REUSE_CLASS).equals(UserSelectedOptions.TRUE));
		reuse.setViews(uso.getOptionValue(StringConstants.REUSE_VIEW_DIAGRAM).equals(UserSelectedOptions.TRUE));
		reuse.setThumbnails(uso.getOptionValue(StringConstants.REUSE_THUMBNAILS).equals(UserSelectedOptions.TRUE));
		reuse.setPolarity(uso.getOptionValue(StringConstants.REUSE_POLARITIES).equals(UserSelectedOptions.TRUE));
		reuse.setLoops(uso.getOptionValue(StringConstants.REUSE_LOOPS).equals(UserSelectedOptions.TRUE));
	}
	
	public void configureForMap(boolean config) {
		if (config) {
			// if already configured, just return
			if (configuredForMap) 
				return;

			// save the current state
			nonMapUserSelectedOptions = new UserSelectedOptions(this);
			
			// set map state
			configuredForMap = true;
			setValue(NA_FOR_MAP, "boolean", FALSE);
			
		} else {
			// if in maop state, need to restore to last nonMap state
			if (configuredForMap) {
				// need to restore
				copyFrom(nonMapUserSelectedOptions);
				configuredForMap = false;
			} else {
				// good to go.
			}
		}
	}
	
	public void configureForXmile() {
//		setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", FALSE);
	}
	
	public void setDefaultGraph() {
		setValue(StringConstants.WIDTH, "int", "572");
		setValue(StringConstants.HEIGHT, "int", "384");
		setValue(StringConstants.LEFT_MARGIN, "int", "64");
		setValue(StringConstants.BOTTOM_MARGIN, "int", "64");
		setValue(StringConstants.FONT_SIZE,"int", "10");
		setValue(StringConstants.VIEW_WIDTH_MAX, "int", "1000");
		setValue(StringConstants.HIGHLIGHT_POINT_X, "int", "1");
		setValue(StringConstants.HIGHLIGHT_POINT_Y, "int", "1");
		setValue(StringConstants.RULE_THRESHOLD, "int", "3");
		setValue(StringConstants.HIGHLIGHT_POINT, "boolean", TRUE);
		setValue(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS, "boolean", TRUE);
		setValue(StringConstants.GENERATE_3D, "boolean", FALSE);
		setValue(StringConstants.INCLUDE_MDL, "boolean", TRUE);
		setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", TRUE);
		setValue(StringConstants.MICMAC_ANALYSIS, "boolean", TRUE);
		setValue(StringConstants.THUMBNAIL_MAXIMUM, "int", "100");
		setValue(StringConstants.LOOP_SIZE_MAXIMUM, "int", "10");
		
		setValue(StringConstants.MAX_LIMIT_V_LOOPS, "int", "50");
		setValue(StringConstants.MAX_LIMIT_VR_LOOPS, "int", "50");
		setValue(StringConstants.MAX_SEARCH_TIME_MINUTES, "int", "5");
		setValue(StringConstants.MAX_LIMIT_LONGEST_LOOPS, "int", "1000");
	}
	
	public void setSelectAllTrue() {
		
		setValue(MODEL_INFORMATION_CHOICES, "boolean", TRUE);
		setValue(VARIABLE_INFORMATION_CHOICES, "boolean", TRUE);
		setValue(ARROW_INFORMATION_CHOICES, "boolean", TRUE);
		setValue(WARNINGS_CHOICES, "boolean", TRUE);
		setValue(POTENTIAL_OMISSIONS_CHOICES, "boolean", TRUE);
//		setValue(EXOGENOUS, "boolean", TRUE);
		setValue(CODE_GENERATION_OPTIONS, "boolean", TRUE);
//		setValue(NAMING_CHOICES, "boolean", TRUE);
		
		setValue(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS, "boolean", TRUE);
		setValue(StringConstants.GENERATE_3D, "boolean", TRUE);
		setValue(StringConstants.INCLUDE_MDL, "boolean", TRUE);
//		setValue(StringConstants.LOOP_SIZE_AUTO_DISCOVER, "boolean", FALSE);
		setValue(StringConstants.ANALYZE_LOOPS, "boolean", TRUE);
		setValue(StringConstants.HYPERLINK_LOOP, "boolean", TRUE);
		setValue(StringConstants.ANALYZE_LOOPS_WITH_IVV, "boolean", TRUE);
		setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", TRUE);
		setValue(StringConstants.MICMAC_ANALYSIS, "boolean", TRUE);
		setValue(StringConstants.LIMIT_THUMBNAIL_COUNT, "boolean", TRUE);
		setValue(StringConstants.VARIABLE_RELATIONSHIPS, "boolean", TRUE);
		setValue(StringConstants.VARIABLE_LOOPS, "boolean", TRUE);
		setValue(StringConstants.OVERRIDE_LOOP_COUNT, "boolean", TRUE);
		
		setValue(StringConstants.LIMIT_V_LOOPS, "boolean", TRUE);
		setValue(StringConstants.LIMIT_VR_LOOPS, "boolean", TRUE);
		setValue(StringConstants.LIMIT_LONGEST_LOOPS, "boolean", TRUE);


		setValue(StringConstants.UNITS_NAME, "boolean", TRUE);
		setValue(StringConstants.UNITS_USED, "boolean", TRUE);
		setValue(StringConstants.RATE_TO_RATE_LINKS, "boolean", TRUE);
	}

	public void setSelectAllFalse() {
		
		setValue(MODEL_INFORMATION_CHOICES, "boolean", FALSE);
		setValue(VARIABLE_INFORMATION_CHOICES, "boolean", FALSE);
		setValue(ARROW_INFORMATION_CHOICES, "boolean", FALSE);
		setValue(WARNINGS_CHOICES, "boolean", FALSE);
		setValue(POTENTIAL_OMISSIONS_CHOICES, "boolean", FALSE);
//		setValue(EXOGENOUS, "boolean", FALSE);
		setValue(CODE_GENERATION_OPTIONS, "boolean", FALSE);
//		setValue(NAMING_CHOICES, "boolean", FALSE);
		
		setValue(StringConstants.CREATE_ZIP_FILE_WITH_ALL_OUTPUT_REPORTS, "boolean", FALSE);
		setValue(StringConstants.GENERATE_3D, "boolean", FALSE);
		setValue(StringConstants.INCLUDE_MDL, "boolean", FALSE);
//		setValue(StringConstants.LOOP_SIZE_AUTO_DISCOVER, "boolean", FALSE);
		setValue(StringConstants.ANALYZE_LOOPS, "boolean", FALSE);
		setValue(StringConstants.HYPERLINK_LOOP, "boolean", FALSE);
		setValue(StringConstants.ANALYZE_LOOPS_WITH_IVV, "boolean", FALSE);
		setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", FALSE);
		setValue(StringConstants.MICMAC_ANALYSIS, "boolean", FALSE);
		setValue(StringConstants.LIMIT_THUMBNAIL_COUNT, "boolean", FALSE);
		setValue(StringConstants.VARIABLE_RELATIONSHIPS, "boolean", FALSE);
		setValue(StringConstants.VARIABLE_LOOPS, "boolean", FALSE);
		setValue(StringConstants.OVERRIDE_LOOP_COUNT, "boolean", FALSE);
		
		setValue(StringConstants.LIMIT_V_LOOPS, "boolean", FALSE);
		setValue(StringConstants.LIMIT_VR_LOOPS, "boolean", FALSE);
		setValue(StringConstants.LIMIT_LONGEST_LOOPS, "boolean", FALSE);


		setValue(StringConstants.UNITS_NAME, "boolean", FALSE);
		setValue(StringConstants.UNITS_USED, "boolean", FALSE);
		setValue(StringConstants.RATE_TO_RATE_LINKS, "boolean", FALSE);
		
	}

	private void setValue(String[] options, String type, String value) {
		for (String opt : options) {
			setValue(opt, type, value);
		}
	}

	public void setValue(String option, String type, String value) {
//		System.out.println("setV opt, type, val "+option+" "+type+" "+value);
		optionValues.put(option, value);
		optionTypes.put(option, type);
	}
	
	public String getOptionType(String option) {
		return optionTypes.get(option);
	}
	
	public String getOptionValue(String option) {
		if (!optionValues.containsKey(option)) {
			optionValues.put(option, "");
		}
		return optionValues.get(option);
	}
	
	public boolean isSelected(String option) {
		if (getOptionValue(option) == null)
			return false;
		return getOptionValue(option).equalsIgnoreCase(TRUE);
	}

	public boolean isConfiguredForMap() {
		return configuredForMap;
	}

	/**
	 * @return the runFiles
	 */
	public List<String> getRunFiles() {
		return runFiles;
	}

	/**
	 * @param runFiles the runFiles to set
	 */
	public void setRunFiles(List<String> runFiles) {
		this.runFiles = runFiles;
	}
}
