package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetRowData;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class VariableListTable implements GetXML {
static Logger log = Logger.getLogger(VariableListTable.class.getSimpleName());
	
	protected final static String[] naForMap = {
			StringConstants.STATE_VARIABLES, StringConstants.STOCK_VARIABLES,
			StringConstants.MACROS, StringConstants.USING_MACROS,
			StringConstants.VARIABLES_WITH_SOURCE_INFORMATION,
			StringConstants.DIMENSIONLESS_UNIT_VARIABLES,
			StringConstants.FUNCTION_SENSITIVITY_PARAMETERS,
			StringConstants.DATA_LOOKUP_TABLES,
			StringConstants.UNDOCUMENTED_VARIABLES,
			StringConstants.EQUATIONS_WITH_EMBEDDED_DATA,
			StringConstants.EQUATIONS_WITH_UNIT_ERRORS_OR_WARNINGS,
			StringConstants.VARIABLES_NOT_IN_ANY_VIEW,
		/*	"Incompletely Defined Subscripted Variables", */
			StringConstants.NONMONOTONIC_LOOKUP_FUNCTIONS,
			StringConstants.CASCADING_LOOKUP_FUNCTIONS,
			StringConstants.NON_ZERO_END_SLOPED_LOOKUP_FUNCTIONS,
			StringConstants.EQUATIONS_WITH_IF_THEN_ELSE_FUNCTIONS,
			StringConstants.EQUATIONS_WITH_MIN_OR_MAX_FUNCTIONS,
			StringConstants.EQUATIONS_WITH_STEP_PULSE_OR_RELATED_FUNCTIONS,
			StringConstants.IGNORE_EMBEDDED_CONSTANTS_OF_0_OR_1,
			StringConstants.UNUSED_VARIABLES,
			StringConstants.SUPPLEMENTARY_VARIABLES,
			StringConstants.SUPPLEMENTARY_VARIABLES_BEING_USED,
			"Complex Variable Formulations (Richardson",
			StringConstants.COMPLEX_STOCK };
	
	protected static final String TITLE_MARKER = "$$$TITLE$$$";
	protected static final String NAME_MARKER = "$$$NAME$$$";
	protected static final String NUM_MARKERV = "$$$NUMV$$$";
	protected static final String NUM_MARKERS = "$$$NUMS$$$";
	protected static final String TOP_MARKER = "$$$TOP$$$";
	protected static final String CLASS_MARKER = "$$$CLASS$$$";

	protected static final String HEADER_START = "<br/><hr style='height:1px; color:Blue'/>\n" +
			"$$$NAME$$$\n" +
			"$$$TOP$$$" ;
	
	// Feb 24, 2020 need to 
	
	protected static final String HEADER_END = 
			"<h3>$$$TITLE$$$ ($$$NUMV$$$ $$$VARIABLES$$$ + $$$NUMS$$$ $$$CONTROLVARIABLES$$$) $$$EXTRA$$$</h3>\n" +
			"<table  class='$$$CLASS$$$'>\n";
	
	protected static final String DATA_TABLE_TITLE = 
			"$$$TITLE$$$ ($$$NUMV$$$ $$$VARIABLES$$$ + $$$NUMS$$$ $$$CONTROLVARIABLES$$$) $$$EXTRA$$$";
	
//	"<table align='center'  class='sample'>\n";
	protected static final String FOOTER = "</table>\n";
	protected static final String BREAK = "<br/>\n";
	protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
	
	protected static final String COLUMN_HEADERS_START = "<tr>\n";
	
	protected static final String DATA_HEADERS_MODULE = "##Module##";
	protected static final String DATA_HEADERS_GROUP = "##Group##";
	protected static final String DATA_HEADERS_TYPE = "##Type##";
	protected static final String DATA_HEADERS_VARIABLE = "##Variable##";
	protected static final String DATA_HEADERS_UNITS = "##Units##";

	protected static final String COLUMN_HEADERS_MODULE = "<td style='vertical-align:bottom; color:Blue'><center>##Module##</center></td>\n";
	protected static final String COLUMN_HEADERS_END = "<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			"</tr>\n";
	
	protected boolean displayModule;
	
	protected String title;
	
	protected List<VariableListTableRow> variables;
	
	protected VariableSelector variableSelector;
	
	protected Map<String, Equation> equations;
	
	protected VariableListTable(Map<String, Equation> equations, VariableSelector variableSelector, boolean displayModule) {
		this.title = variableSelector.getSelectionCriteria();
		this.equations = equations;
		this.variableSelector = variableSelector;
		this.displayModule = displayModule;
	}
	
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	protected List<List<String>> createRow() {
		List<List<String>> row = new ArrayList<List<String>>();
		return row;
	}
	
	protected List<String> createColumn(String s1) {
		List<String> column = new ArrayList<String>();
		column.add(s1);
		return column;
	}
	
	protected void addTitle(List<List<List<String>>> rows, String title) {
		 // this should be an empty table, so row 0, col 0 will contain title
		List<List<String>> row = createRow();
		row.add(createColumn(title));
		rows.add(row);
	}
	
	protected List<String> createColumn(List<String> s1) {
		List<String> column = new ArrayList<String>();
		column.addAll(s1);
		return column;
	}
	
	protected void addColumnHeaders(List<List<List<String>>> rows, List<String> headers) {
		// this shoud be row 1
		List<List<String>> row = createRow();
		row.add(createColumn(headers));
		rows.add(row);
	}
	
	protected List<List<List<String>>> createTable() {
		return new ArrayList<List<List<String>>>();
	}
	


	
	public void addColumn(List<List<String>> row, String text) {
		row.add(createColumn(text));
	}
	
	public List<List<List<String>>> getTableData() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		
		String alpha = "";
		StringBuilder sb = new StringBuilder();
		
		String titlel;
		
		if (needsCondition(title))
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", "");
		else
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", "");
		
		TableUtilities.addTitle(table, titlel);
		
		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);
		if (displayModule) {
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
		}
		
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
		
		
		for (VariableListTableRow vltr : variables) {
			TableUtilities.addRow(table, vltr.getRowData());
		}
		
		
		return table;
	}
	
	public String getXML() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		selectVariables();
		

		
		String alpha = "";
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
				replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
		if (variables.size() > 0) {
			QuickLinkAlphabet qla = new QuickLinkAlphabet(variables, "List"+variableSelector.getInternalName());
			sb.append(qla.getXML());
		}
		
		if (needsCondition(title))
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
		else
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
		
//		if (option.equals(""))
//			sb.append(quickLinkAlphabetList.getXML());
		
		sb.append(COLUMN_HEADERS_START);
		if (displayModule)
			sb.append(COLUMN_HEADERS_MODULE.replace("##Module##", language.getTextHtml(StringConstants.MODULE)));
		sb.append(COLUMN_HEADERS_END.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
				.replace("##Variable##", language.getTextHtml("Variable"))
				.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
				.replace("##Group##", language.getTextHtml(StringConstants.GROUP)));
		for (VariableListTableRow row : variables) {
			
			String listMarker = "$$$LISTNAME$$$";
			String list = "";
			// href='#
			String letter = row.getXML().split("href='#")[1].substring(0, 1);
//			if (variableSelector.getInternalName().equals(VariableNames.VARIABLES_LIST)) {
				if (!letter.toUpperCase().equals(alpha)) {
					list = Anchors.getAnchorDefinition("List"+variableSelector.getInternalName()+letter.toUpperCase());
					alpha = letter.toUpperCase();
				}
//			}
			
			sb.append(row.getXML().replace(listMarker, list));
		}

		sb.append(FOOTER);
		sb.append(BREAK);
		//		sb.append(SEPARATOR);
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+this.variableSelector.getInternalName()+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		
		return sb.toString();
	}
	
	protected void selectVariables() {
		variables = new ArrayList<VariableListTableRow>();
		
		for (String var : InformationManagers.getInstance().getEquationLookup().orderedLHS()) {
			Equation eqn = InformationManagers.getInstance().getEquationManager().getEquationFor(var);
			if (eqn == null) {
				System.out.println("Null Equation: "+var);
				continue;
			}
			if (variableSelector.isSelectSingleEquation(eqn))
				variables.add(new VariableListTableRow(eqn, displayModule));
		}
	}
	
	protected boolean needsCondition(String name) {
		for (String s : naForMap) {
			if (s.startsWith(name))
				return true;
		}
		return false;
	}
	
}
