package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.analysis.graphtheory.ExogenousVariableManager;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

public class VariableListTableExtraColumns extends VariableListTable implements GetXML {
	
	
	protected static final String COLUMN_HEADERS_EXTRA_START = "<tr>\n";
	protected static final String COLUMN_HEADERS_EXTRA_MODULE = "<td style='vertical-align:bottom; color:Blue'><center>##Module##</center></td>\n";
	protected static final String COLUMN_HEADERS_EXTRA_END = "<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$TYPE$$$</center></b></td>\n" +
			"</tr>\n";
	protected static final String COLUMN_HEADERS_EXTRA_FEEDBACK_END = "<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$TYPE$$$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;+&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;-&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;+/- ##Ratio##&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;?&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$TYPE$$$ (IVV)</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;+&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;-&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;+/- ##Ratio##&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;?&nbsp;</center></b></td>\n" +
			"</tr>\n";
	
	protected static final String COLUMN_HEADERS_EXTRA_SOURCES = 
			"<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##DataSource##</center></b></td>\n" +
			"</tr>\n";
	
	protected static final String COLUMN_HEADERS_EXTRA_EXOGENOUS = 
			"<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##VariableNumber##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##DataSource##</center></b></td>\n" +
			"</tr>\n";
	
	protected static final String COLUMN_HEADERS_EXTRA_ENDOGENOUS = 
			"<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			
			"<td style='vertical-align:bottom; color:Blue'><b><center>##VariableNumber##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##InputLinks##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##DirectExogenous##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##% Exogenous##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##IndirectExogenous##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##IndirectMin##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##IndirectMean##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##IndirectMedian##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##IndirectMax##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Unconnected##</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Connected##</center></b></td>\n" +
			"</tr>\n";
	
	protected static final String COLUMN_HEADERS_EXTRA_IN_OUT_LINKS_END = 
			"<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>##Variable##</center></b></td>\n" +
			
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$INOUTCOUNTS$$$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>&nbsp;$$$INOUTRATIO$$$&nbsp;</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$INLINKSBYPOLARITY$$$</center></b></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><center>$$$OUTLINKSBYPOLARITY$$$</center></b></td>\n" +
			"</tr>\n";
	
	protected static final String TYPE_MARKER = "$$$TYPE$$$";
	
	protected String type;
	protected Documentor documentor;
	protected String extraStuff;

	public VariableListTableExtraColumns(Map<String, Equation> equations,
			VariableSelector variableSelector, String type, Documentor documentor) {
		super(equations, variableSelector, documentor.displayModule());
		this.type = type;
		this.documentor = documentor;
	}
	
	@Override
	public void selectVariables() {
		variables = new ArrayList<VariableListTableRow>();
		
		for (String var : InformationManagers.getInstance().getEquationLookup().orderedLHS()) {
			Equation eqn = InformationManagers.getInstance().getEquationManager().getEquationFor(var);
			if (eqn == null) {
				System.out.println("(Selector Missing Equation: "+var);
				continue;
			}
			if (variableSelector.isSelectSingleEquation(eqn))
				variables.add(new VariableListTableRowExtraColumns(eqn, type, documentor));
		}
	}
	
	private String addRichardson(String r) {
		
		String rich = InformationManagers.getInstance().getUserSelectedOptions().getOptionValue(StringConstants.RULE_THRESHOLD);
		
		String newR = r +" ("+ SDMDocLanguageSpecific.getInstance().getTextHtml(StringConstants.RULE_THRESHOLD) + " = "+rich+")";
		return newR;
	}
		
	
	@Override
	public String getXML() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		selectVariables();
		StringBuilder sb = new StringBuilder();
		if (type.equals(VariableListTableRowExtraColumns.IN_OUT_LINKS)) {
			extraStuff = "";
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
				replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample")
					.replace(TITLE_MARKER, title)
					.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_IN_OUT_LINKS_END.replace(TYPE_MARKER, language.getTextHtml(type))
					.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					.replace("$$$INOUTCOUNTS$$$", language.getTextHtml(StringConstants.IN_OUT_COUNTS))
					.replace("$$$INOUTRATIO$$$", language.getTextHtml(StringConstants.IN_OUT_RATIO))
					.replace("$$$INLINKSBYPOLARITY$$$", language.getTextHtml(StringConstants.IN_LINKS_BY_POLARITY))
					.replace("$$$OUTLINKSBYPOLARITY$$$", language.getTextHtml(StringConstants.OUT_LINKS_BY_POLARITY))
					);

		}
		else if (type.equals(VariableListTableRowExtraColumns.COMPLEXITY)) {
			extraStuff = "";
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
				replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample")
					.replace(TITLE_MARKER, addRichardson(title))
					.replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_END.replace(TYPE_MARKER, language.getTextHtml(type))
					.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					);
		} else if (type.equals(VariableListTableRowExtraColumns.UNITS)) {
			extraStuff = "";
				sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));

				sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
						replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
						replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
						replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
						replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
						replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
						replace("$$$EXTRA$$$", "").
						replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
				sb.append(COLUMN_HEADERS_EXTRA_START);
				if (displayModule)
					sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
							.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
							.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
							.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
							.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
							);
				sb.append(COLUMN_HEADERS_EXTRA_END.replace(TYPE_MARKER, language.getTextHtml(/*type)*/ StringConstants.UNITS_NAME))
						.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
				
				// Feedback Loops
				
		} else if (type.equals(VariableListTableRowExtraColumns.FEEDBACK)){
			
			
			String len =  documentor.getAnalysisManager().getLoopManager().getMaximumLoopLengthAsString();
			extraStuff = " "+language.getTextHtml(StringConstants.MAXIMUM_LOOP_LENGTH)+": "+len+" "+
			"["+documentor.getAnalysisManager().getLoopManager().getShortestLoopLengthNoIVV()+","+
			documentor.getAnalysisManager().getLoopManager().getLongestLoopLengthNoIVV()+"] | "+
			"["+documentor.getAnalysisManager().getLoopManager().getShortestLoopLengthWithIVV()+","+
			documentor.getAnalysisManager().getLoopManager().getLongestLoopLengthWithIVV()+"]";
			
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sampleborder").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.LOOPS)).
					replace("+ $$$NUMS$$$ $$$CONTROLVARIABLES$$$", "").
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(
							documentor.getAnalysisManager().getLoopManager().getNumberOfLoopsNoIVV())
							+"|"+
							NumberFormatter.formatForCountNotConditional(documentor.getAnalysisManager().getLoopManager().getNumberOfLoopsWithIVV())).replace("$$$EXTRA$$$", extraStuff));
			
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_FEEDBACK_END.replace(TYPE_MARKER, language.getTextHtml(StringConstants.LOOPS))
					.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					); // type
			
			// Non-Zero
			
		} else if (type.equals(VariableListTableRowExtraColumns.EXOGENOUS)){
			
			ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
//			String extraStuff = " ["+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfExogenousVariables())+"|"+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfEndogenousVariables())+"]";
			extraStuff = "";
			
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", extraStuff));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_EXOGENOUS.replace(TYPE_MARKER, language.getTextHtml(StringConstants.LOOPS))
					.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					.replace("##VariableNumber##", language.getTextHtml(StringConstants.VARIABLE_NUMBER))
					.replace("##DataSource##", language.getTextHtml(StringConstants.DATA_SOURCE))
					); // type
			
			
			// Non-Zero
			
		} else if (type.equals(VariableListTableRowExtraColumns.ENDOGENOUS)){
			
			ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
//			String extraStuff = " ["+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfExogenousVariables())+"|"+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfEndogenousVariables())+"]";
			
			extraStuff = " ("+language.getTextHtml(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+": "+
			InformationManagers.getInstance().getUserSelectedOptions().
				getOptionValue(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+")";
			
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
		
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", extraStuff));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_ENDOGENOUS.replace(TYPE_MARKER, language.getTextHtml(StringConstants.LOOPS))
					.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					.replace("##VariableNumber##", language.getTextHtml(StringConstants.VARIABLE_NUMBER))
					.replace("##InputLinks##", language.getTextHtml(StringConstants.INPUT_LINKS))
					.replace("##DirectExogenous##", language.getTextHtml(StringConstants.DIRECT_EXOGENOUS))
					.replace("##% Exogenous##", language.getTextHtml(StringConstants.PCT_EXOGENOUS))
					.replace("##IndirectExogenous##", language.getTextHtml(StringConstants.INDIRECT_EXOGENOUS))
					.replace("##IndirectMin##", language.getTextHtml(StringConstants.INDIRECT_MINIMUM))
					.replace("##IndirectMean##", language.getTextHtml(StringConstants.INDIRECT_MEAN))
					.replace("##IndirectMedian##", language.getTextHtml(StringConstants.INDIRECT_MEDIAN))
					.replace("##IndirectMax##", language.getTextHtml(StringConstants.INDIRECT_MAXIMUM))
					.replace("##Unconnected##", language.getTextHtml(StringConstants.EXOGENOUS_UNCONNECTED))
					.replace("##Connected##", language.getTextHtml(StringConstants.EXOGENOUS_CONNECTED))
					); // type
			
			// Non-Zero
		
		} else if (type.equals(VariableListTableRowExtraColumns.NONZERO)){
			
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_MODULE.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
						);
			sb.append(COLUMN_HEADERS_EXTRA_END.replace(TYPE_MARKER, language.getTextHtml(type)).replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##Ratio##", language.getTextHtml(StringConstants.RATIO))
					);
		} else if (type.equals(VariableListTableRowExtraColumns.SOURCES)){
			
			sb.append(HEADER_START.replace(NAME_MARKER, Anchors.getAnchorDefinition(variableSelector.getInternalName())).
					replace(TOP_MARKER,  Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
			
			sb.append(HEADER_END.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables)))
					.replace("$$$EXTRA$$$", ""));
			sb.append(COLUMN_HEADERS_EXTRA_START);
			if (displayModule)
				sb.append(COLUMN_HEADERS_EXTRA_SOURCES.replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE))
						.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
						.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
						.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
						.replace("##DataSource##", language.getTextHtml(StringConstants.SOURCES))
						);
			sb.append(COLUMN_HEADERS_EXTRA_END.replace(TYPE_MARKER, language.getTextHtml(type)).replace("##Type##", 
					language.getTextHtml(StringConstants.UNITS_TYPE))
					.replace("##Module##", language.getTextHtml(StringConstants.MODULE))
					.replace("##Variable##", language.getTextHtml(StringConstants.VARIABLE))
					.replace("##Group##", language.getTextHtml(StringConstants.GROUP))
					.replace("##DataSource##", language.getTextHtml(StringConstants.SOURCES))
					);	
		}
		
		String alpha = "";
		int varNum = 1;
		for (VariableListTableRow row : sorted()) {
			
			row.setVariableNumber(varNum++);

			String listMarker = "$$$LISTNAME$$$";
			String list = "";
			String letter;
			// href='#
			if (type.equals(VariableListTableRowExtraColumns.FEEDBACK)) {
				// since these are not sorted by name, we don't have quick links and
				// must put full name
				letter = row.getXML().split("href='#")[1].split("'>")[0];
				
//				if (!letter.toUpperCase().equals(alpha)) {
					
					list = Anchors.getAnchorDefinitionNoEscape("List"+variableSelector.getInternalName()+letter);
					alpha = letter.toUpperCase();
//				}
			} else {
				letter = row.getXML().split("href='#")[1].substring(0, 1);
				//			if (variableSelector.getInternalName().equals(VariableNames.VARIABLES_LIST)) {
				if (!letter.toUpperCase().equals(alpha)) {
					list = Anchors.getAnchorDefinition("List"+variableSelector.getInternalName()+letter.toUpperCase());
					alpha = letter.toUpperCase();
				}
				//			}
			}

			sb.append(row.getXML().replace(listMarker, list));
		}
		
		sb.append(FOOTER);
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+this.variableSelector.getInternalName()+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}
		return sb.toString();
	}
	
	@Override
	public List<List<List<String>>> getTableData() {
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		String titlel;
		
//		selectVariables();
		StringBuilder sb = new StringBuilder();
		if (type.equals(VariableListTableRowExtraColumns.IN_OUT_LINKS)) {
			extraStuff = "";		
			
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, 
						TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.IN_COUNTS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.OUT_COUNTS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.IN_OUT_RATIO)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_IN_LINKS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_IN_LINKS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_IN_LINKS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_OUT_LINKS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_OUT_LINKS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_OUT_LINKS)));
			

		}
		else if (type.equals(VariableListTableRowExtraColumns.COMPLEXITY)) {
			
			extraStuff = "";
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, addRichardson(title)).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.COMPLEXITY)));
			
		} else if (type.equals(VariableListTableRowExtraColumns.UNITS)) {
			extraStuff = "";
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_ALTERNATES)));
				
				// Feedback Loops
				
		} else if (type.equals(VariableListTableRowExtraColumns.FEEDBACK)){
			
			
			String len =  documentor.getAnalysisManager().getLoopManager().getMaximumLoopLengthAsString();
			extraStuff = " "+language.getTextHtml(StringConstants.MAXIMUM_LOOP_LENGTH)+": "+len+" "+
			"["+documentor.getAnalysisManager().getLoopManager().getShortestLoopLengthNoIVV()+","+
			documentor.getAnalysisManager().getLoopManager().getLongestLoopLengthNoIVV()+"] | "+
			"["+documentor.getAnalysisManager().getLoopManager().getShortestLoopLengthWithIVV()+","+
			documentor.getAnalysisManager().getLoopManager().getLongestLoopLengthWithIVV()+"]";
			
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", extraStuff);
			
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NUMBER_OF_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.PCT_OF_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn("+/- "+language.getTextHtml(StringConstants.RATIO)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NUMBER_OF_FEEDBACK_LOOPS)+" (IVV)"));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.PCT_OF_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn("+/- "+language.getTextHtml(StringConstants.RATIO)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNKNOWN_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			

				// Non-Zero
			
		} else if (type.equals(VariableListTableRowExtraColumns.EXOGENOUS)){
			
			ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
//			String extraStuff = " ["+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfExogenousVariables())+"|"+
//					NumberFormatter.formatForCountNotConditional(evm.getNumberOfEndogenousVariables())+"]";
			extraStuff = "";
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EXOGENOUS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE_NUMBER)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.DATA_SOURCE)));

			// Non-Zero
			
		} else if (type.equals(VariableListTableRowExtraColumns.ENDOGENOUS)){
			
			ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();

			extraStuff = " ("+language.getTextHtml(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+": "+
			InformationManagers.getInstance().getUserSelectedOptions().
				getOptionValue(StringConstants.MAXIMUM_ENDOGENOUS_PATH_LENGTH)+")";		
			
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			
			TableUtilities.addTitle(table, titlel);
			
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			
			
			if (displayModule) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}

			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE_NUMBER)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INPUT_LINKS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.DIRECT_EXOGENOUS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.PCT_EXOGENOUS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INDIRECT_EXOGENOUS)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INDIRECT_MINIMUM)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INDIRECT_MEAN)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INDIRECT_MEDIAN)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.INDIRECT_MAXIMUM)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EXOGENOUS_UNCONNECTED)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.EXOGENOUS_CONNECTED)));
			
			// Non-Zero
		
		} else if (type.equals(VariableListTableRowExtraColumns.NONZERO)){
			
			extraStuff = "";		
			
			titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
					replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
					replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
					replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
					replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
					replace("$$$EXTRA$$$", "");
			TableUtilities.addTitle(table, titlel);
			
			List<List<String>> row = TableUtilities.createRow();
			TableUtilities.addRow(table, row);
			if (displayModule) {
				TableUtilities.addColumn(row, 
						TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
			}
			
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NON_ZERO)));
		} else if (type.equals(VariableListTableRowExtraColumns.SOURCES)){
				
				extraStuff = "";		
				
				titlel = DATA_TABLE_TITLE.replace(CLASS_MARKER, "sample").replace(TITLE_MARKER, title).
						replace("$$$VARIABLES$$$", language.getTextHtml(StringConstants.VARIABLES)).
						replace("$$$CONTROLVARIABLES$$$", language.getTextHtml(StringConstants.CONTROL_VARIABLES)).
						replace(NUM_MARKERV, NumberFormatter.formatForCountNotConditional(Parser.countNonSystemVariables(variables))).
						replace(NUM_MARKERS, NumberFormatter.formatForCountNotConditional(Parser.countSystemVariables(variables))).
						replace("$$$EXTRA$$$", "");
				
				TableUtilities.addTitle(table, titlel);
				
				List<List<String>> row = TableUtilities.createRow();
				TableUtilities.addRow(table, row);
				if (displayModule) {
					TableUtilities.addColumn(row, 
							TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
				}
				
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.SOURCES)));
		}
		
		int varNum = 1;
		for (VariableListTableRow vltr : sorted()) {
//			vltr.setVariableNumber(varNum++);
			TableUtilities.addRow(table, vltr.getRowData());
		}
		
		
		return table;
	}
	
	private List<VariableListTableRow> sorted() {
		List<VariableListTableRow> al = new ArrayList<VariableListTableRow>();
		List<VariableListTableRowExtraColumns> alx = new ArrayList<VariableListTableRowExtraColumns>();
		
		
		if (type.equals(VariableListTableRowExtraColumns.UNITS)) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
						return o1.getVariable().compareToIgnoreCase(o2.getVariable());
				}
			});
			
			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
		} else if (type.equals(VariableListTableRowExtraColumns.NONZERO)) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
					return o1.getVariable().compareToIgnoreCase(o2.getVariable());

				}
			});
			
			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
		} else if (type.equals(VariableListTableRowExtraColumns.FEEDBACK)) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
					if (o2.getSortField().intValue() == o1.getSortField().intValue())
						return o1.getVariable().compareToIgnoreCase(o2.getVariable());
					return o2.getSortField().compareTo(o1.getSortField());

				}
			});

			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
		} else if (type.equals(VariableListTableRowExtraColumns.EXOGENOUS) ||
				type.equals(VariableListTableRowExtraColumns.ENDOGENOUS)) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
//					if ((o1 == null || o2 == null || o2.getSortField() == null || o1.getSortField() == null ) && 
//							type.equals(VariableListTableRowExtraColumns.ENDOGENOUS))
//						System.out.println("HERE");
					
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
					if (o2.getSortField().intValue() == o1.getSortField().intValue())
						return o1.getVariable().compareToIgnoreCase(o2.getVariable());
					return o2.getSortField().compareTo(o1.getSortField());

				}
			});

			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
		} else if (type.equals(VariableListTableRowExtraColumns.IN_OUT_LINKS) ) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					// 4/11/2017 - sort high to low change 
//					if (o2.getSortField().intValue() == o1.getSortField().intValue()) {
//						return o1.getVariable().compareToIgnoreCase(o2.getVariable());
//					}
//					return o1.getSortField().compareTo(o2.getSortField());
					
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
					if (o1.getSortField().intValue() == o2.getSortField().intValue()) {
						return o2.getVariable().compareToIgnoreCase(o1.getVariable());
					}
					return o2.getSortField().compareTo(o1.getSortField());


				}
			});

			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
			
		} else if (type.equals(VariableListTableRowExtraColumns.COMPLEXITY) ) {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;
					
					if (o2.getSortField().intValue() == o1.getSortField().intValue()) {
//						System.out.println("cmp: {"+o1.getVariable()+"|"+o1.getSortField().intValue()+"} {"+o2.getVariable()+"|"+o2.getSortField().intValue()+"} -> "+o1.getVariable() +" and "+o2.getVariable());
						return o1.getVariable().compareToIgnoreCase(o2.getVariable());
					}
//					System.out.println("cmp: {"+o1.getVariable()+"|"+o1.getSortField().intValue()+"} {"+o2.getVariable()+"|"+o2.getSortField().intValue()+"} -> "+o1.getSortField().intValue() +" and "+o2.getSortField().intValue());
					return o1.getSortField().compareTo(o2.getSortField());

				}
			});

			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
			
		} else {
			for (VariableListTableRow ltr : variables) {
				alx.add((VariableListTableRowExtraColumns) ltr);
			}
			
			Collections.sort(alx, new Comparator<VariableListTableRowExtraColumns>() {
				@Override
				public int compare(VariableListTableRowExtraColumns o1, VariableListTableRowExtraColumns o2) {
					int initialPartition = Utilities.systemVariableCompare(o1.getVariable(), o2.getVariable());
					if (initialPartition != 0)
						return initialPartition;

					return o2.getSortField().compareTo(o1.getSortField());

				}
			});
			
			for (VariableListTableRowExtraColumns ltrec : alx) {
				al.add((VariableListTableRow) ltrec);
			}
			return al;
		}
		
		
	}

}
