package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableRow;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.List;

import org.apache.commons.lang3.StringEscapeUtils;

public class VariableListTableMultiValueColumns implements GetXML {

	private static final String DATA_TABLE_HEADER = 
					"($$$TYPE$$$) $$$NAME$$$ ($$$NUM$$$ ##variables##)";


	private static final String HEADER = "<table class='$$$CLASS$$$'>" +
			   "<tr><td style='text-align:center; font-size:8px;'><a href='#top'><b>##TOP##</b></a></td>" +
					"<td colspan=3 style='background:blue;color:White; font-size:large'>$$$ADEF$$$<b>($$$TYPE$$$) $$$NAME$$$ ($$$NUM$$$ ##variables##) ##EXTRA##</td></tr><tr>\n";
	
	private static final String VIEW_HEADER = "<table class='sample' align='center'>" +
			   "<tr>" +	
			   "<td colspan=1 style='background:blue;color:White; font-size:large'>$$$ADEF$$$<b>($$$TYPE$$$) $$$NAME$$$ ($$$NUM$$$ ##variables##)</td>" +
			   "</tr>\n" +
			   "<tr>" +
			   "<td style='text-align:center; vertical-align:top'><img src='$$$OUTPUT_DIR$$$/$$$LEGAL_NAME$$$.png'/></td>" + 
			   "</tr>\n" +
			   "</table>\n";

	private static final String FOOTER = "</table>\n";
	
	private static final String CLASS_MARKER = "$$$CLASS$$$";
	private static final String TYPE_MARKER = "$$$TYPE$$$";
	private static final String NAME_MARKER = "$$$NAME$$$";
	private static final String NUM_MARKER = "$$$NUM$$$";
	private static final String ANCHOR_DEF_MARKER = "$$$ADEF$$$"; // <a name='view0'/>
	private static final String THUMBNAIL_MARKER = "$$$THUMBNAIL$$$";
	
	
	private static final String COLUMN_HEADERS_START = "<tr>\n";
	private static final String COLUMN_HEADERS_MODULE = "<td style='vertical-align:bottom; color:Blue'><center>##Module##</center></td>\n";
	private static final String COLUMN_HEADERS_END = "<td style='vertical-align:bottom; color:Blue'><center>##Group##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><center>##Type##</center></td>\n" +
			"<td style='vertical-align:bottom; color:Blue'><b><i>##VariableNameAndDescription##</i></b></td>\n";
	private static final String COLUMN_HEADERS_THUMBNAIL = "<td style='vertical-align:bottom; color:Blue'><center>##Thumbnail##</center></td>\n";
	private static final String COLUMN_HEADERS_END_END = "</tr>\n";
	
	private String title;
	private String internalName;
	private String groupingName;
	private String groupingType;
	private int numVariables;
	
	
	private String extra = "";
	
	private List<VariableListTableRowMultiValueColumn> variables;
	private TableRow modelVariablesInformation;
	
	private boolean displayModule;
	private boolean displayThumbnail;
	
	public VariableListTableMultiValueColumns(TableRow modelVariablesInformation, String internalName, String groupingType, String groupingName, int numVariables, 
			boolean displayModule, boolean displayThumbnail) {
		this.modelVariablesInformation = modelVariablesInformation;
		this.groupingName = groupingName;
		this.numVariables = numVariables;
		this.groupingType = groupingType;
		this.displayModule = displayModule;
		this.displayThumbnail = displayThumbnail;
		if (groupingName.equals("Lookup"))
			this.displayThumbnail = false;
		this.internalName = internalName;
	}
	
	@Override
	public String getXML() {
		return null;
	}
	
	@Override
	public boolean getXML(BufferedWriter bw) {

		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		String alpha = "";
		StringBuilder sb = new StringBuilder();
		
		try {
		
		// for views, we want to insert the model diagram for this view
		
			if (groupingType.equals("View")) {
				
//				String outputDirRelative = Utilities.escapePound(StringEscapeUtils.escapeHtml4(InformationManagers.getInstance().getModelManager().getOutputDirectoryName()));
				String outputDirRelative = Utilities.escapePound(StringEscapeUtils.escapeHtml4(InformationManagers.getInstance().getFileOrganizer().getRelativeViewsDirectory()));
				
				bw.append(VIEW_HEADER.replace(NAME_MARKER, StringEscapeUtils.escapeHtml4(groupingName)).replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(numVariables)).
						replace(ANCHOR_DEF_MARKER, Anchors.getAnchorDefinition("-"+internalName+"-"+groupingName)).replace(TYPE_MARKER, groupingType)
						.replace("##variables##", StringConstants.VARIABLES).replace("$$$OUTPUT_DIR$$$", outputDirRelative).replace("$$$LEGAL_NAME$$$", "__VIEW__"+Utilities.makeLegalJavaName(groupingName)));
			}
			
			bw.append(HEADER.replace(CLASS_MARKER, "sample")
					.replace(NAME_MARKER, StringEscapeUtils.escapeHtml4(groupingName))
					.replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(numVariables))
					.replace(ANCHOR_DEF_MARKER, Anchors.getAnchorDefinition("-"+internalName+"-"+groupingName))
					.replace(TYPE_MARKER, groupingType)
					.replace("##TOP##", language.getTextHtml(StringConstants.TOP))
					.replace("##EXTRA##",extra)
					.replace("##variables##", language.getTextHtml(StringConstants.VARIABLES)));

			//		QuickLinkAlphabet quickLinkAlphabet = new QuickLinkAlphabet(modelVariablesInformation, "Detail"+groupingType+"-"+groupingName);


			bw.append(COLUMN_HEADERS_START);
			if (displayModule) {
				bw.append(COLUMN_HEADERS_MODULE.replace("##Module##", language.getTextHtml(StringConstants.MODULE)));
			}
			bw.append(COLUMN_HEADERS_END.replace("##Group##", language.getTextHtml(StringConstants.GROUP)).
					replace("##Type##", language.getTextHtml(StringConstants.UNITS_TYPE)).replace("##VariableNameAndDescription##", language.getTextHtml(StringConstants.VARIABLE_NAME_AND_DESCRIPTION)));
			if (displayThumbnail) {
				bw.append(COLUMN_HEADERS_THUMBNAIL.replace("##Thumbnail##", language.getTextHtml(StringConstants.THUMBNAIL)));
			}
			bw.append(COLUMN_HEADERS_END_END);

			for (List<List<String>> row : modelVariablesInformation.getRows()) {
				int col = 2;
				if (row.size() > 3)
					col = 3;
				String orig = row.get(col).get(0);
				MutableInteger pos = new MutableInteger();
				String letter = extractName(orig, pos);
				String detailMarker = "$$$DETAILNAME$$$";
				String detail = "";
				if (internalName.equalsIgnoreCase("ALL")) {
					if (!letter.toUpperCase().equals(alpha.toUpperCase())) {
						detail = Anchors.getAnchorDefinition("Detail"+letter.toUpperCase());
						alpha = letter.toUpperCase();
					}
				}

				sb.append(row.get(0).get(0));
				sb.append(row.get(1).get(0));
				sb.append(row.get(2).get(0).replace(detailMarker, detail));
				if (row.size() > 3)
					sb.append(row.get(3).get(0).replace(detailMarker, detail));
				sb.append("\n");
			}
//			sb.append(FOOTER);
			
				bw.append(sb.toString());
				bw.append(FOOTER);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		if (options.isSelected(StringConstants.CREATE_FLAT_FILES)) {

			List<List<List<String>>> table = getTableData();
			//		System.out.println(table);
			String fn = InformationManagers.getInstance().getFileOrganizer().getFlatFileTablesDirectory()+"/"+internalName+"_"+groupingName+".tab";
			FlatFileWriter.writeFile(fn, table, "\t");
		}

		return true;
	}
	
	private String extractName(String s, MutableInteger pos) {
		String letter;
		
		String[] parts = s.split("'>");
		int ptr = 0;
		if (parts[1].startsWith("\""))
			ptr = 1;
		letter = Parser.characterAt(ptr, parts[1]);
		
		return letter;
	}

	public String getExtra() {
		return extra;
	}

	public void setExtra(String extra) {
		this.extra = extra;
	}

	public String getGroupingName() {
		return groupingName;
	}

	@Override
	public List<List<List<String>>> getTableData() {
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();

		//		if (groupingType.equals("View")) {
		//			
		//			String outputDirRelative = InformationManagers.getInstance().getModelManager().getOutputDirectoryName();
		//			
		//			sb.append(VIEW_HEADER.replace(NAME_MARKER, StringEscapeUtils.escapeHtml4(groupingName)).replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(numVariables)).
		//					replace(ANCHOR_DEF_MARKER, Anchors.getAnchorDefinition("-"+internalName+"-"+groupingName)).replace(TYPE_MARKER, groupingType)
		//					.replace("##variables##", StringConstants.VARIABLES).replace("$$$OUTPUT_DIR$$$", outputDirRelative).replace("$$$LEGAL_NAME$$$", "__VIEW__"+Utilities.makeLegalJavaName(groupingName)));
		//		}

		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		List<List<List<String>>> table = TableUtilities.createFlatFileTable();
		String titlel;


		//		($$$TYPE$$$) $$$NAME$$$ ($$$NUM$$$ ##variables##)
		titlel = (DATA_TABLE_HEADER
				.replace(NAME_MARKER, StringEscapeUtils.escapeHtml4(groupingName))
				.replace(NUM_MARKER, NumberFormatter.formatForCountNotConditional(23))
				.replace(TYPE_MARKER, groupingType)
				.replace("##variables##", language.getTextHtml(StringConstants.VARIABLES)));
		TableUtilities.addTitle(table, titlel);

		List<List<String>> row = TableUtilities.createRow();
		TableUtilities.addRow(table, row);
		if (displayModule) {
			TableUtilities.addColumn(row, 
					TableUtilities.createColumn(language.getTextHtml(StringConstants.MODULE)));
		}

		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.GROUP)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS_TYPE)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE_NUMBER)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.VARIABLE)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.UNITS)));
		TableUtilities.addColumn(row, TableUtilities.createColumn(language.getText(StringConstants.EQUATION)));


		if (options.isSelected(StringConstants.DESCRIPTION))
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.DESCRIPTION)));
		if (options.isSelected("Meta")) {
			TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.SOURCE)));
			if (options.isSelected(StringConstants.PRESENT_IN_VIEWS))
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.PRESENT_IN_VIEWS)));
			if (options.isSelected(StringConstants.USED_BY))
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.USED_BY)));
			if (options.isSelected(StringConstants.FEEDBACK_LOOPS)) {
//				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NUMBER_OF_FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.PCT_OF_FEEDBACK_LOOPS)));

				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.POSITIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));

				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MIN_LENGTH_FEEDBACK_LOOPS)));
				TableUtilities.addColumn(row, TableUtilities.createColumn(language.getTextHtml(StringConstants.NEGATIVE_POLARITY_MAX_LENGTH_FEEDBACK_LOOPS)));
			}

			for (List<List<String>> datarow : modelVariablesInformation.getDataTableRows()) {
				TableUtilities.addRow(table, datarow);
			}


		}

		return table;
	}

}
