package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetRowData;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.util.List;

import org.apache.commons.lang3.StringEscapeUtils;

public class VariableListTableRow implements GetXML, GetRowData {
	
	protected static final String HEADER = "<tr>\n";
	protected static final String FOOTER = "</tr>\n";
	
	protected static final String ROW_HEADER = "<td style='text-align:center'>";
	protected static final String ROW_FOOTER = "</td>";
	
	protected static final String VARIABLE_HEADER = "<td>";
	protected static final String VARIABLE_FOOTER = "</td>";
	
	protected String module;
	protected String group;
	protected String type;
	protected String variable;
	protected String units;
	protected String rawunits;
	
	protected Equation equation;
	protected boolean displayModule;
	
	protected int variableNumber = 0;
	
	public VariableListTableRow(Equation equation, boolean displayModule) {
		this.equation = equation;
		module = equation.getModule();
		group = equation.getGroup();
//		type = "#" + equation.getVariableNumber() + "<br/>" + equation.getType();
		type = equation.getType();
		variable = equation.getOriginalLhs();
		units =HTMLConstants.RED_START+ " ("+StringEscapeUtils.escapeHtml4(equation.getUnits())+")"+HTMLConstants.COLOR_END;
		rawunits = equation.getUnits();
		this.displayModule = displayModule;
	}
	
	@Override
	public boolean getXML(BufferedWriter sb) {
		return true;
	}
	
	public List<List<String>> getRowData() {
		List<List<String>> row = TableUtilities.createRow();
		if (displayModule)
			TableUtilities.addColumn(row, TableUtilities.createColumn(module));
		TableUtilities.addColumn(row, TableUtilities.createColumn(group));
		TableUtilities.addColumn(row, TableUtilities.createColumn(type));
		TableUtilities.addColumn(row, TableUtilities.createColumn(variable+" ("+rawunits+")"));
		return row;
	}
	
	public String getXML() {
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER);
		if (displayModule)
			sb.append(ROW_HEADER+module+ROW_FOOTER);
		sb.append(ROW_HEADER+"$$$LISTNAME$$$"+group+ROW_FOOTER);
		sb.append(ROW_HEADER+type+ROW_FOOTER);
//		sb.append(VARIABLE_HEADER+Anchors.getAnchorReference(InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs()))+units+VARIABLE_FOOTER);
//		
		sb.append(VARIABLE_HEADER+Anchors.getAnchorReference(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(equation.getLhs()))+units+VARIABLE_FOOTER);
		
		sb.append(FOOTER);
		return sb.toString();
	}

	public String getVariable() {
		return variable;
	}

	public Integer getVariableNumber() {
		return variableNumber;
	}

	public void setVariableNumber(int variableNumber) {
		this.variableNumber = variableNumber;
	}

	@Override
	public List<List<List<String>>> getTableData() {
		// TODO Auto-generated method stub
		return null;
	}

}
