package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import java.util.List;

import gov.anl.gss.sdm.server.model.analysis.graphtheory.ExogenousVariableManager;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.ExogenousVariableRow;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.analysis.polarity.VariableComplexity;
import gov.anl.gss.sdm.server.model.analysis.polarity.VariableComplexityData;
import gov.anl.gss.sdm.server.model.documentor.Documentor;
import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.HTMLConstants;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.TableUtilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

public class VariableListTableRowExtraColumns extends VariableListTableRow implements GetXML {
	
	public static final String UNITS = "Units";
	public static final String COMPLEXITY = "Complexity";
	public static final String FEEDBACK = "Feedback";
	public static final String NONZERO = "NonZero";
	public static final String IN_OUT_LINKS = "InOutLinks";
	public static final String EXOGENOUS = "Exogenous";
	public static final String ENDOGENOUS = "Endogenous";
	public static final String SOURCES = "Sources";
	
	public static final String COLUMN_SEPARATOR = "###CS###";
	
	protected String extra = "";
	protected String extraNoHTML = "";
	protected Documentor documentor;
	protected Integer sortField;
	protected String type;
	

	public VariableListTableRowExtraColumns(Equation equation, String type, Documentor documentor) {
		
		super(equation, documentor.displayModule());
		this.type = type;
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		this.documentor = documentor;
		if (type.equals(UNITS)) {
			for (String s : equation.getUnitsInconsistencyReport()) {    // getUnitsInconsistencyReport   getUnitsMessages
				if (s == null || s.equals("null"))
					continue;
				if (!(extra.endsWith("Units: ") || extra.length() == 0))
					extra = extra + "<br/>";
				extra += " "+Parser.removeQuotes(s); // s.replace("\"", "");
			}
		} else if (type.equals(NONZERO)) {
			
			
			boolean left = equation.getEquationCharacteristics().getNonZeroLeft().value();
			boolean right = equation.getEquationCharacteristics().getNonZeroRight().value();
			if (left && right) {
				extra = language.getTextHtml(StringConstants.BOTH);
			} else if (left) {
				extra = language.getTextHtml(StringConstants.LEFT);
			} else {
				extra = language.getTextHtml(StringConstants.RIGHT);
			}
			extraNoHTML = extra;
		} else if (type.equals(IN_OUT_LINKS)) {
			// need access to In Out Links
			VariableComplexityData vcd = equation.getComplexityData();
			extra = colorCodedLinkDetail(vcd);
			extraNoHTML = colorCodedLinkDetailNoHTML(vcd);
			sortField = vcd != null ? vcd.getIn()+vcd.getOut() : 0;
//			extra = equation.getComplexityData();
		} else if (type.equals(COMPLEXITY)) {
			// need access to VariableComplexityData
			extra = equation.getComplexFormulationCount();
			extraNoHTML = extra;
			sortField = Integer.parseInt(extra);
//			extra = equation.getComplexityData();
		} else if (type.equals(FEEDBACK)) {
			LoopManager lm = documentor.getAnalysisManager().getLoopManager();
			String screenName = InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs());
			
		
			extra = getFeedbackExtraNoIVV(screenName) + COLUMN_SEPARATOR + getFeedbackExtraWithIVV(screenName);
			extraNoHTML = getFeedbackExtraNoIVVNoHTML(screenName) + COLUMN_SEPARATOR + getFeedbackExtraWithIVVNoHTML(screenName);
			sortField = lm.getNumberOfLoopsWith(screenName);
		} else if (type.equals(EXOGENOUS)) {
			ExogenousVariableManager evm = documentor.getAnalysisManager().getExogenousVariableManager();
//			String screenName = InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs());
			String screenName = equation.getVariableName(); 

			extra = getExogenousExtra(screenName);
			extraNoHTML = extra;
			sortField = exogenousSort(screenName);
		} else if (type.equals(ENDOGENOUS)) {
			ExogenousVariableManager evm = documentor.getAnalysisManager().getExogenousVariableManager();
			String screenName = InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs());

			screenName = equation.getVariableName();
			extra = getEndogenousExtra(screenName);
			extraNoHTML = getEndogenousExtraNoHTML(screenName);
			sortField = exogenousSort(screenName);
			
		} else if (type.equals(SOURCES)) {
			ExogenousVariableManager evm = documentor.getAnalysisManager().getExogenousVariableManager();
			String screenName = InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs());

			screenName = equation.getVariableName();
			extra = getSourcesExtra(screenName);
			extraNoHTML = extra;
			sortField = exogenousSort(screenName);
			
		} else {
			extra = "";
			extraNoHTML = extra;
		}
	}
	
	private String colorCodedLinkDetail(VariableComplexityData vcd) {
		VariableComplexity vc = InformationManagers.getInstance().getAnalysisManager().getVariableComplexity();
		vc.determineSizing();
		
		int width = 2;
		
		String inOut;
		String in;
		String out;
		String ratio;
		
		if (vcd == null) {
			inOut = "&nbsp;("+/*HTMLConstants.BOLD_START +*/
					HTMLConstants.BLACK_START+expandToWidth(0, width)+"|"+HTMLConstants.COLOR_END +
					expandToWidth(0, width)+HTMLConstants.COLOR_END+/*HTMLConstants.BOLD_END+*/")";
			ratio = "&nbsp;&nbsp;"+/*HTMLConstants.BOLD_START + */ 
					HTMLConstants.BLACK_START+expandToWidth(NumberFormatter.formatForRatio((1.0/0.0)), width)+
					HTMLConstants.COLOR_END+/*HTMLConstants.BOLD_END+*/"";
			in = "&nbsp;&nbsp;&nbsp;"+
					HTMLConstants.BLUE_START+expandToWidth(0, width)+HTMLConstants.COLOR_END+
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.RED_START+expandToWidth(0, width)+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+expandToWidth(0, width)+HTMLConstants.COLOR_END+"";
			
			out = "&nbsp;&nbsp;&nbsp;"+
					HTMLConstants.BLUE_START+expandToWidth(0, width)+HTMLConstants.COLOR_END+
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.RED_START+expandToWidth(0, width)+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+expandToWidth(0, width)+HTMLConstants.COLOR_END+"";
		} else {
			inOut = "&nbsp;"+/*HTMLConstants.BOLD_START +*/
					HTMLConstants.BLACK_START+expandToWidth(vcd.getIn(), width)+"&nbsp;|&nbsp;"+HTMLConstants.COLOR_END +
					expandToWidth(vcd.getOut(), width)+HTMLConstants.COLOR_END+/*HTMLConstants.BOLD_END+*/"";
			ratio = "&nbsp;&nbsp;"+/*HTMLConstants.BOLD_START +*/
					HTMLConstants.BLACK_START+expandToWidth(NumberFormatter.formatForRatio(vcd.getInOutRatio()), width)+
					HTMLConstants.COLOR_END+/*HTMLConstants.BOLD_END+*/"";
			in = "&nbsp;&nbsp;&nbsp;"+
					HTMLConstants.BLUE_START+expandToWidth(vcd.getInPos(), width)+HTMLConstants.COLOR_END+
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.RED_START+expandToWidth(vcd.getInNeg(), width)+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+expandToWidth(vcd.getInUnk(), width)+HTMLConstants.COLOR_END+"";

			out = "&nbsp;&nbsp;&nbsp;"+
					HTMLConstants.BLUE_START+expandToWidth(vcd.getOutPos(), width)+HTMLConstants.COLOR_END+
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.RED_START+expandToWidth(vcd.getOutNeg(), width)+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+"|"+HTMLConstants.COLOR_END +
					HTMLConstants.BLACK_START+expandToWidth(vcd.getOutUnk(), width)+HTMLConstants.COLOR_END+"";
		}
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = inOut+COLUMN_SEPARATOR+ratio+COLUMN_SEPARATOR+in+COLUMN_SEPARATOR+out;

		return extra;
	}
	
	private String colorCodedLinkDetailNoHTML(VariableComplexityData vcd) {
		VariableComplexity vc = InformationManagers.getInstance().getAnalysisManager().getVariableComplexity();
		vc.determineSizing();
		
		int width = 2;
		
		String inOut;
		String in;
		String out;
		String ratio;
		
		if (vcd == null) {
			inOut = 
					expandToWidthNoHTML(0, width)+"|"+
					expandToWidthNoHTML(0, width);
			ratio = 
					expandToWidthNoHTML(NumberFormatter.formatForRatio((1.0/0.0)), width);
			
			in = 
					expandToWidthNoHTML(0, width)+
					"|"
					+expandToWidthNoHTML(0, width)
					+"|"+
					expandToWidthNoHTML(0, width);
			
			out = 
					expandToWidthNoHTML(0, width)+
					"|"+
					expandToWidthNoHTML(0, width)+
					"|"+
					expandToWidthNoHTML(0, width);
		} else {
			inOut = 
					expandToWidthNoHTML(vcd.getIn(), width)+"|"+
					expandToWidthNoHTML(vcd.getOut(), width);
			ratio = expandToWidthNoHTML(NumberFormatter.formatForRatio(vcd.getInOutRatio()), width);
			in = 
					expandToWidthNoHTML(vcd.getInPos(), width)+
					"|"+
					expandToWidthNoHTML(vcd.getInNeg(), width)+
					"|"+
					expandToWidthNoHTML(vcd.getInUnk(), width);

			out = 
					expandToWidthNoHTML(vcd.getOutPos(), width)+
					"|"+
					expandToWidthNoHTML(vcd.getOutNeg(), width)+
					"|"+
					expandToWidthNoHTML(vcd.getOutUnk(), width);
		}
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = inOut+COLUMN_SEPARATOR+ratio+COLUMN_SEPARATOR+in+COLUMN_SEPARATOR+out;

		return extra;
	}
	
	private int exogenousSort(String screenName) {
		
		ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
		ExogenousVariableRow evr = evm.getExogenousVariableRow(screenName);
		if (evr == null || evr.isExogenous())
			return 10000000;
		return (int) (evr.getDirectPercentage() * 1000);
		
	}
	
	private String getSourcesExtra(String screenName) {
		// num inputs, num direct inputs, % direct inputs, num indirect, min path, mean path, median path, max path
		StringBuilder sb = new StringBuilder();
		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(screenName);
		
		sb.append(eqn.getSource());

		return sb.toString();
	}
	
	private String getExogenousExtra(String screenName) {
		// num inputs, num direct inputs, % direct inputs, num indirect, min path, mean path, median path, max path
		StringBuilder sb = new StringBuilder();
		ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
		ExogenousVariableRow evr = evm.getExogenousVariableRow(screenName);
		
		if (evr == null) {
			System.out.println("No evr for "+screenName);
		}
		
		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(screenName);
		
//		sb.append(evr.getVariableType());
		sb.append(getVariableNumber());
		sb.append(COLUMN_SEPARATOR);
		if (eqn.isGetExternalData()) {
			sb.append(SDMDocLanguageSpecific.getInstance().getText(StringConstants.EXTERNAL));
		} else if (evr.getNumDirectExogenous() == evr.getNumDirectInputs() && evr.getNumDirectExogenous() > 0) {
			sb.append(evr.getNumDirectInputs()+" "+SDMDocLanguageSpecific.getInstance().getText(StringConstants.EXOGENOUS));
		} else {
			sb.append(SDMDocLanguageSpecific.getInstance().getText(StringConstants.HARDCODED));
		}

		return sb.toString();
	}
	
	private String getEndogenousExtra(String screenName) {
		// num inputs, num direct inputs, % direct inputs, num indirect, min path, mean path, median path, max path
		StringBuilder sb = new StringBuilder();
		ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
		ExogenousVariableRow evr = evm.getExogenousVariableRow(screenName);
		
//		sb.append(evr.getVariableType());
		sb.append(getVariableNumber());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumDirectForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumDirectExogenousForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getExogenousPercentForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumIndirectForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMinLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMeanLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMedianLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMaxLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getReferenceNoPath());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getReferencePath());
		return sb.toString();
	}
	
	private String getEndogenousExtraNoHTML(String screenName) {
		// num inputs, num direct inputs, % direct inputs, num indirect, min path, mean path, median path, max path
		StringBuilder sb = new StringBuilder();
		ExogenousVariableManager evm = InformationManagers.getInstance().getAnalysisManager().getExogenousVariableManager();
		ExogenousVariableRow evr = evm.getExogenousVariableRow(screenName);
		
//		sb.append(evr.getVariableType());
		sb.append(getVariableNumber());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumDirectForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumDirectExogenousForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getExogenousPercentForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getNumIndirectForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMinLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMeanLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMedianLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getIndirectMaxLengthForHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getReferenceNoPathNoHTML());
		sb.append(COLUMN_SEPARATOR);
		sb.append(evr == null ? "?" : evr.getReferencePathNoHTML());
		return sb.toString();
	}
	
	private String getFeedbackExtraWithIVV(String screenName) {
		LoopManager lm = documentor.getAnalysisManager().getLoopManager();
		
		ColumnSizing columnSizing = documentor.getColumSizing();
		
		long minPos = lm.getMinimumLengthLoopSizeIVV(screenName, "+");
		long minNeg = lm.getMinimumLengthLoopSizeIVV(screenName, "-");
		long minPrb = lm.getMinimumLengthLoopSizeIVV(screenName, "?");
		long maxPos = lm.getMaximumLengthLoopSizeIVV(screenName, "+");
		long maxNeg = lm.getMaximumLengthLoopSizeIVV(screenName, "-");
		long maxPrb = lm.getMaximumLengthLoopSizeIVV(screenName, "?");
		
		String posRange = "&nbsp;["+expandToWidth(minPos, columnSizing.getPosRangeMinSize())+","+expandToWidth(maxPos, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.COLOR_END;
		String negRange = "&nbsp;["+expandToWidth(minNeg, columnSizing.getPosRangeMinSize())+","+expandToWidth(maxNeg, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.COLOR_END;
		String prbRange = "&nbsp;["+expandToWidth(minPrb, columnSizing.getPrbRangeMinSize())+","+expandToWidth(maxPrb, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.BOLD_END;
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = expandToWidth(NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWithIVV(screenName)), columnSizing.getLoopCountMaxSize()) +
				HTMLConstants.SPACE+"("+expandToWidth(lm.getPercentageCountsIVV(screenName), columnSizing.getPercentageMaxSize())+"%)" + COLUMN_SEPARATOR +
				HTMLConstants.BLUE_START+expandToWidth(lm.getPercentagePositiveCountsIVV(screenName), columnSizing.getLoopCountMaxSize())+posRange + HTMLConstants.COLOR_END+ COLUMN_SEPARATOR +
				HTMLConstants.RED_START+expandToWidth(lm.getPercentageNegativeCountsIVV(screenName), columnSizing.getLoopCountMaxSize())+negRange + HTMLConstants.COLOR_END + COLUMN_SEPARATOR +
				lm.getPositiveNegativeRatioWithIVV(screenName)+ COLUMN_SEPARATOR +
				HTMLConstants.BOLD_START+expandToWidth(lm.getPercentageProblemCountsIVV(screenName), columnSizing.getLoopCountMaxSize()) + HTMLConstants.BOLD_END + prbRange;

		return extra;
	}
	
	private String getFeedbackExtraWithIVVNoHTML(String screenName) {
		LoopManager lm = documentor.getAnalysisManager().getLoopManager();
		
		ColumnSizing columnSizing = documentor.getColumSizing();
		
		long minPos = lm.getMinimumLengthLoopSizeIVV(screenName, "+");
		long minNeg = lm.getMinimumLengthLoopSizeIVV(screenName, "-");
		long minPrb = lm.getMinimumLengthLoopSizeIVV(screenName, "?");
		long maxPos = lm.getMaximumLengthLoopSizeIVV(screenName, "+");
		long maxNeg = lm.getMaximumLengthLoopSizeIVV(screenName, "-");
		long maxPrb = lm.getMaximumLengthLoopSizeIVV(screenName, "?");
		
		String posRange = "["+minPos+","+maxPos+"]";
		String negRange = "["+minNeg+","+maxNeg+"]";
		String prbRange = "["+minPrb+","+maxPrb+"]";
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWithIVV(screenName))+
				"("+lm.getPercentageCountsIVV(screenName)+"%)" + COLUMN_SEPARATOR +
				lm.getPercentagePositiveCountsIVV(screenName)+posRange + COLUMN_SEPARATOR +
				lm.getPercentageNegativeCountsIVV(screenName)+negRange + COLUMN_SEPARATOR +
				lm.getPositiveNegativeRatioWithIVV(screenName)+ COLUMN_SEPARATOR +
				lm.getPercentageProblemCountsIVV(screenName)+ prbRange;

		return extra;
	}
	
	private String getFeedbackExtraNoIVV(String screenName) {
		LoopManager lm = documentor.getAnalysisManager().getLoopManager();
		
		ColumnSizing columnSizing = documentor.getColumSizing();
		
		long minPos = lm.getMinimumLengthLoopSize(screenName, "+");
		long minNeg = lm.getMinimumLengthLoopSize(screenName, "-");
		long minPrb = lm.getMinimumLengthLoopSize(screenName, "?");
		long maxPos = lm.getMaximumLengthLoopSize(screenName, "+");
		long maxNeg = lm.getMaximumLengthLoopSize(screenName, "-");
		long maxPrb = lm.getMaximumLengthLoopSize(screenName, "?");
		
		String posRange = "&nbsp;["+expandToWidth(minPos, columnSizing.getPosRangeMinSize())+","+expandToWidth(maxPos, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.COLOR_END;
		String negRange = "&nbsp;["+expandToWidth(minNeg, columnSizing.getPosRangeMinSize())+","+expandToWidth(maxNeg, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.COLOR_END;
		String prbRange = "&nbsp;["+expandToWidth(minPrb, columnSizing.getPrbRangeMinSize())+","+expandToWidth(maxPrb, columnSizing.getPosRangeMaxSize())+"]"+HTMLConstants.BOLD_END;
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = expandToWidth(NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWith(screenName)), columnSizing.getLoopCountMaxSize()) +
				HTMLConstants.SPACE+"("+expandToWidth(lm.getPercentageCounts(screenName), columnSizing.getPercentageMaxSize())+"%)" + COLUMN_SEPARATOR +
				HTMLConstants.BLUE_START+expandToWidth(lm.getPercentagePositiveCounts(screenName), columnSizing.getLoopCountMaxSize())+posRange + HTMLConstants.COLOR_END+ COLUMN_SEPARATOR +
				HTMLConstants.RED_START+expandToWidth(lm.getPercentageNegativeCounts(screenName), columnSizing.getLoopCountMaxSize())+negRange + HTMLConstants.COLOR_END + COLUMN_SEPARATOR +
				lm.getPositiveNegativeRatio(screenName)+ COLUMN_SEPARATOR +
				HTMLConstants.BOLD_START+expandToWidth(lm.getPercentageProblemCounts(screenName), columnSizing.getLoopCountMaxSize()) + HTMLConstants.BOLD_END + prbRange;

		return extra;
	}
	
	private String getFeedbackExtraNoIVVNoHTML(String screenName) {
		LoopManager lm = documentor.getAnalysisManager().getLoopManager();
		
		ColumnSizing columnSizing = documentor.getColumSizing();
		
		long minPos = lm.getMinimumLengthLoopSize(screenName, "+");
		long minNeg = lm.getMinimumLengthLoopSize(screenName, "-");
		long minPrb = lm.getMinimumLengthLoopSize(screenName, "?");
		long maxPos = lm.getMaximumLengthLoopSize(screenName, "+");
		long maxNeg = lm.getMaximumLengthLoopSize(screenName, "-");
		long maxPrb = lm.getMaximumLengthLoopSize(screenName, "?");
		
		String posRange = "["+minPos+","+maxPos+"]";
		String negRange = "["+minNeg+","+maxNeg+"]";
		String prbRange = "["+minPrb+","+maxPrb+"]";
		
		// return "&nbsp;("+pct+"%)";
		
		String extra = NumberFormatter.formatForCountNotConditional(lm.getNumberOfLoopsWith(screenName)) + 
				"("+lm.getPercentageCounts(screenName)+"%)"+ COLUMN_SEPARATOR +
				lm.getPercentagePositiveCounts(screenName)+ posRange +COLUMN_SEPARATOR +
				lm.getPercentageNegativeCounts(screenName)+negRange + COLUMN_SEPARATOR +
				lm.getPositiveNegativeRatio(screenName)+ COLUMN_SEPARATOR +
				lm.getPercentageProblemCounts(screenName)+ prbRange;

		return extra;
	}
	
	private String expandToWidth(String value, long width) {
		long len = value.length();
		long toAdd = width - len;
		if (toAdd == 0)
			return value;
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < toAdd; i++) {
			sb.append(HTMLConstants.SPACE);
		}
		return sb.append(value).toString();
		
	}
	
	private String expandToWidthNoHTML(String value, long width) {
		long len = value.length();
		long toAdd = width - len;
		if (toAdd == 0)
			return value;
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < toAdd; i++) {
			sb.append(" ");
		}
		return sb.append(value).toString();
		
	}
	
	private String expandToWidth(long ivalue, long width) {
		String value = NumberFormatter.formatForCountNotConditional(ivalue);
		long len = value.length();
		long toAdd = width - len;
		if (toAdd == 0)
			return value;
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < toAdd; i++) {
			sb.append(HTMLConstants.SPACE);
		}
		return sb.append(value).toString();
		
	}
	
	private String expandToWidthNoHTML(long ivalue, long width) {
		String value = NumberFormatter.formatForCountNotConditional(ivalue);
		long len = value.length();
		long toAdd = width - len;
		if (toAdd == 0)
			return value;
		StringBuilder sb = new StringBuilder();
		for (int i = 0; i < toAdd; i++) {
			sb.append(" ");
		}
		return sb.append(value).toString();
		
	}
	
	
	@Override
	public String getXML() {
		StringBuilder sb = new StringBuilder();
		sb.append(HEADER);
		if (displayModule)
			sb.append(ROW_HEADER+module+ROW_FOOTER);
		sb.append(ROW_HEADER+"$$$LISTNAME$$$"+group+ROW_FOOTER);
		sb.append(ROW_HEADER+getTranslatedType()+ROW_FOOTER);
		sb.append(VARIABLE_HEADER+Anchors.getAnchorReference(InformationManagers.getInstance().getEquationLookup().getScreenName(equation.getLhs()))+units+VARIABLE_FOOTER);
		if (extra.contains(COLUMN_SEPARATOR)) {
			for (String col : extra.split(COLUMN_SEPARATOR)) {
				sb.append(ROW_HEADER+col+ROW_FOOTER);			}
		} else {
			sb.append(ROW_HEADER+extra+ROW_FOOTER);
		}
		sb.append(FOOTER);
		return sb.toString();
	}
	
	@Override
	public List<List<String>> getRowData() {
		
		List<List<String>> row = TableUtilities.createRow();
		if (displayModule)
			TableUtilities.addColumn(row, TableUtilities.createColumn(module));
		TableUtilities.addColumn(row, TableUtilities.createColumn(group));
		TableUtilities.addColumn(row, TableUtilities.createColumn(getTranslatedType()));
		TableUtilities.addColumn(row, TableUtilities.createColumn(variable+" ("+rawunits+")"));
		
		if (extraNoHTML.contains(COLUMN_SEPARATOR)) {
			for (String col : extraNoHTML.split(COLUMN_SEPARATOR)) {
				TableUtilities.addColumn(row, TableUtilities.createColumn(col));
				
				}
		} else {
			TableUtilities.addColumn(row, TableUtilities.createColumn(extraNoHTML));
		}
		return row;
	}
	
	private String getTranslatedType() {
		
		if (type.equals(StringConstants.FEEDBACK) || 
				type.equals(StringConstants.COMPLEXITY) ||
		type.equals(StringConstants.EXOGENOUS) ||
		type.equals(StringConstants.ENDOGENOUS)) {
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			return language.getTextHtml(type);
		} else if (type.equals(StringConstants.IN_OUT_LINKS)) {
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			return language.getTextHtml(StringConstants.IN_OUT_LINKS_TYPE);
		} else if (type.equals(StringConstants.NON_ZERO)) {
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			return language.getTextHtml(StringConstants.NON_ZERO);
		} else if (type.equals(StringConstants.UNITS)) {
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			return language.getTextHtml(StringConstants.UNITS_NAME);
		} else {
			return type;
		}
		
	}

	public Integer getSortField() {
		return sortField;
	}
	
	@Override
	public void setVariableNumber(int variableNumber) {
		if (type.equals(StringConstants.EXOGENOUS) || type.equals(StringConstants.ENDOGENOUS)) {
			extra = extra.replaceFirst("0#", variableNumber+"#");
			extraNoHTML = extraNoHTML.replaceFirst("0#", variableNumber+"#");
		}
		this.variableNumber = variableNumber;
	}
}
