package gov.anl.gss.sdm.server.model.documentor.xmlwriter;

import gov.anl.gss.sdm.server.model.documentor.GetXML;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.ViewVariables;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.MacroManager;

import java.io.BufferedWriter;
import java.text.DecimalFormat;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class ViewVariablesBarTable implements GetXML {
		
		protected static final String TITLE_MARKER = "$$$TITLE$$$";
		protected static final String NAME_MARKER = "$$$NAME$$$";
		protected static final String NUM_MARKER = "$$$NUM$$$";
		protected static final String TOP_MARKER = "$$$TOP$$$";

		protected static final String HEADER = "<br/><hr style='height:1px; color:Blue'/>\n" +
				"$$$NAME$$$$$$TOP$$$\n" +
				"<h3>$$$TITLE$$$</h3>\n" +
				"<table  class='sample'>\n";
//		"<table align='center'  class='sample'>\n";
		protected static final String FOOTER = "</table>\n";
		protected static final String BREAK = "<br/>\n";
		protected static final String SEPARATOR = "<hr style='height:1px; color:Blue'/>";
		
		protected static final String COLUMN_START = "<td>\n";
		protected static final String COLUMN_END = "</td>\n";
		
		protected static final String ROW_START = "<tr>\n";
		protected static final String ROW_END = "</tr>\n";
		
		protected static final String COLUMN_HEADERS = "<tr>\n" +
				"<td style='vertical-align:bottom; color:Blue'><center>##View##</center></td>\n" +
				"<td style='vertical-align:bottom; color:Blue'><center>##VariableProfile##</center></td>\n" +
				"</tr>\n";
		
//		protected static final String BAR_COLUMN_START = "<td style='text-align:center;color:White;'><b><font style='background-color:Blue'>";
		protected static final String BAR_COLUMN_START = "<td><b><font style='background-color:Blue'>";
		protected static final String BAR_COLUMN_END = "</td>\n";
		
		protected static final String BAR_COLUMN_START_END = "</font></b>";
		
		private static final String BAR = "&nbsp;";
		
		
		protected String title;
		
		protected ViewVariables viewVariables;
		protected int numVariables;
		protected Map<String, Integer> viewVariablesCount;
		protected Map<String, Double> viewVariablesPercent;
		
		
		protected ViewVariablesBarTable(ViewVariables viewVariables) {
			this.viewVariables = viewVariables;
			extractData();
		}
		
		private void extractData() {
			numVariables = InformationManagers.getInstance().getEquationManager().getVariables().size();
			viewVariablesCount = new LinkedHashMap<String, Integer>();
			viewVariablesPercent = new LinkedHashMap<String, Double>();
			
			for (String view : viewVariables.getViewVariables().keySet()) {
				int varsInView = viewVariables.getViewVariables().get(view).size();
				
				viewVariablesCount.put(view, varsInView);
				double pct = 100* ((double) varsInView/ (double) numVariables);
				viewVariablesPercent.put(view, pct);
			}
		}
		
		@Override
		public boolean getXML(BufferedWriter sb) {
			return true;
		}
		
		
		
		public String getXML() {
			
			SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
			
			MacroManager mm = InformationManagers.getInstance().getMacroManager();
			StringBuilder sb = new StringBuilder();
			sb.append(HEADER.replace(TITLE_MARKER, language.getTextHtml(StringConstants.VIEW_VARIABLE_PROFILE)).replace(NAME_MARKER, Anchors.getAnchorDefinition(StringConstants.VIEW_VARIABLE_PROFILE)).
					replace(TOP_MARKER, Anchors.getAnchorReference("top", language.getTextHtml(StringConstants.TOP))));
			sb.append(COLUMN_HEADERS.replace("##View##", language.getTextHtml(StringConstants.VIEW)).
					replace("##VariableProfile##", language.getTextHtml(StringConstants.VIEW_VARIABLE_PROFILE)));
			
			for (String view : viewVariables.getViews()) {
				sb.append(ROW_START);
				sb.append(COLUMN_START);
				sb.append(Anchors.getAnchorReference("-VIEW-"+view));
				sb.append(COLUMN_END);

				int numPerPercent = 2;
				sb.append(BAR_COLUMN_START);
				int numBlanks = viewVariablesPercent.get(view).intValue() * numPerPercent;
				
				for (int i = 0; i < numBlanks; i++) {
					sb.append(BAR);
				}
				sb.append(BAR_COLUMN_START_END);
				DecimalFormat form = new DecimalFormat("###.#");
				String pct = form.format(viewVariablesPercent.get(view));
				sb.append(BAR+BAR+NumberFormatter.formatForCountNotConditional(viewVariablesCount.get(view))+language.getTextHtml(StringConstants.VARIABLES)+" ("+pct+"%)");
				sb.append(BAR_COLUMN_END);
				sb.append(ROW_END);
			}
			
			
			sb.append(FOOTER);
			sb.append(BREAK);
//			sb.append(SEPARATOR);
			return sb.toString();
		}

		@Override
		public List<List<List<String>>> getTableData() {
			// TODO Auto-generated method stub
			return null;
		}
	}
