package gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector;

import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.LinkedHashSet;
import java.util.Set;

public class VariableSelector {
	
	public static final String SENSITIVITY = "[Function Sensitivity Parameter:";
	public static final String SOURCE = "[Source:";
	public static final String LOOKUP = "[Data Lookup:";
	
	private String selectionCriteria;
	private String internalName;
	protected Set<String> uniqueVariables = new LinkedHashSet<String>();
	
	public VariableSelector(String selectionCriteria, String internalName) {
		this.selectionCriteria = selectionCriteria;
		this.internalName = internalName;
	}
	

	protected boolean isUnique(String vn) {
		if (uniqueVariables.contains(vn)) {
			return false;
		}
		uniqueVariables.add(vn);
		return true;
	}
	
	public boolean isSelectMultipleEquation(Equation equation) {
		// this allows multiple equations to be processed
		if (equation.isAutoGenerated() || 
				equation.isDefinesSubscript() || 
				!isSyntacticallyCorrect(equation) /* ||
				isControlGroup(equation)*/ ) {
			return false;
		} else {
			return true;
		}
	}
	
	public boolean isSelectSingleEquation(Equation equation) {
		if (equation.isAutoGenerated() || 
				equation.isDefinesSubscript() || 
				!isSyntacticallyCorrect(equation) /* ||
				isControlGroup(equation) */ ) {
			return false;
		} else {
			return isUnique(equation.getVariableName());
		}
	}

	public String getSelectionCriteria() {
		return selectionCriteria;
	}

	public String getInternalName() {
		return internalName;
	}
	
	public boolean ignore(Equation equation) {
		

		if (isDefinesSubscript(equation))
			return true;
		if (/*isControlGroup(equation) || */
				isAutogenerated(equation))
			return true;
		return false;
	}
	
	public boolean isWithLookup(Equation equation) {
		return Parser.isWithLookup(equation.getLhs());
	}
	
	public boolean isControlGroup(Equation equation) {
		return equation.isControlGroup();
	}
	
	public boolean isDefinesSubscript(Equation equation) {
		return equation.isDefinesSubscript();
	}
	
	public boolean isAutogenerated(Equation equation) {
		return equation.isAutoGenerated();
	}
	
	public boolean isSyntacticallyCorrect(Equation equation) {
		return equation.isSyntacticallyCorrect();
	}

//	public boolean isNAorTime(String s) {
//		if (s == null)
//			return false;
//		return s.toUpperCase().equals("TIME") || s.toUpperCase().equals("NAREPLACEMENT");
//	}
}
