package gov.anl.gss.sdm.server.model.powersim;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.ModelAnalyzer;
import gov.anl.gss.sdm.server.model.translator.SystemDynamicsObjectManager;
import gov.anl.gss.sdm.server.model.xmile.utilities.IfThenElseConverter;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.List;

public class PowersimModel {

	public static final String TEXT = "Text";
	public static final String XLSX = "Xlsx";

	private String name;
	private PowersimModelDescription modelDescription;
	private List<String> mdlFormat;

	private String inputFile;
	private String sheet;
	private PowersimTextParser textParser;
	private PowersimXlsxParser xlsxParser;
	private String format;
	
	private List<Equation> equations;
	private List<String> mdlContents;
	
	// Name, Variable Type, Unit, Dimensions, Definition, Value, Documentation, Note

	public PowersimModel(String inputfile, String format, String sheet) {
		this(inputfile, format);
		this.sheet = sheet;
		this.format = XLSX;
	}

	public PowersimModel(String inputfile, String format) {
		this.inputFile = inputfile;
		this.format = format;

	}

	public static void main(String[] args) {
		PowersimModel m;
		if (args.length == 2) {
			m = new PowersimModel(args[0], args[1]);
		} else {
			m = new PowersimModel(args[0], args[1], args[2]);
		}
		m.process();
	}
	
	private void process() {
		setModelDescription(new PowersimModelDescription(load()));
//		print();
		parse();
		writeVensimToFile("c:/MDLBATCH/powersim.mdl");
	}
	
	public void writeVensimToFile(String file) {
		BufferedWriter bw = Utilities.openFileForWritingUTF8(file);
		List<String> vensim = mdlContents;
		try {
			
			bw.append("{UTF-8}\n");
			bw.append("********************************************************\n");
			bw.append("\t"+"powerSim"+"\n");
			bw.append("********************************************************\n");
			bw.append("\t|\n");
			
			for (String s : vensim) {
				String eqn = s.replace("\n", " ");
				if (isIfThenElse(eqn)) {
					IfThenElseConverter itec = new IfThenElseConverter(eqn);
//				System.out.println(itec.getOriginalEquation());
					bw.append(itec.getConvertedEquation()+"\n");
				} else {
					bw.append(s+"\n");
				}
			}
			bw.append("\\\\\\---/// Sketch information - do not modify anything except names\n");
			bw.append("V300  Do not put anything below this section - it will be ignored\n");
			bw.append("*View 1\n");
			bw.append("$192-192-192,0,Times New Roman|12||0-0-0|0-0-0|0-0-255|-1--1--1|-1--1--1|96,96,100,0\n");
			bw.append("///---\\\\\\\n");
			
//			vensim = asVensimViews();
//			for (String s : vensim) {
//				bw.append(s+"\n");
//			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private boolean isIfThenElse(String s) {
		return false;
	}
	
	private void print() {
		int rowNumber = 0;
		List<String> row;
		 while ((row = modelDescription.getRow(rowNumber)) != null) {
			System.out.println(rowNumber+", "+row);
			rowNumber++;
		}
	}

	private boolean parse() {

		if (format.equals(TEXT))
			return parseText();
		else
			return parseXlsx();

	}

	private boolean parseText() {
//		TextParser parser = new TextParser();
//		mdlFormat = parser.parse(modelDescription);
//		name = parser.getName();
		return true;
	}

	private boolean parseXlsx() {
		xlsxParser = new PowersimXlsxParser(modelDescription);
		mdlContents = xlsxParser.parseSource();
		SystemDynamicsObjectManager sdObjectManager = new SystemDynamicsObjectManager();

		ModelAnalyzer analyzer = new ModelAnalyzer(mdlContents);
		return true;
	}

	private List<List<String>> load() {

		if (format.equals(TEXT))
			return loadText();
		else
			return loadXlsx();
	}

	private List<List<String>> loadText() {

		//		originalText = Utilities.getAsList(inputFile);

		return null;
	}

	private List<List<String>> loadXlsx() {

		PowersimXlsxReader rdr = new PowersimXlsxReader(inputFile);
		rdr.loadSheet(sheet);
		return rdr.getData();
	}

	public PowersimModelDescription getModelDescription() {
		return modelDescription;
	}

	public void setModelDescription(PowersimModelDescription modelDescription) {
		this.modelDescription = modelDescription;
	}

}