package gov.anl.gss.sdm.server.model.powersim;

import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class PowersimTextParser {
	
	private static final Set<String> MAINMODEL = new LinkedHashSet<String>();
	private static final Set<String> LEVEL_1_TYPES = new LinkedHashSet<String>();
	private static final Set<String> LEVEL_2_TYPES = new LinkedHashSet<String>();
	
	static {
		MAINMODEL.add("mainmodel");

		LEVEL_1_TYPES.add("const");
		LEVEL_1_TYPES.add("aux");
		LEVEL_1_TYPES.add("level"); 
		LEVEL_1_TYPES.add("range");
		LEVEL_1_TYPES.add("unit");

		LEVEL_2_TYPES.add("autotype");
		LEVEL_2_TYPES.add("autounit"); 
		// LEVEL_2_TYPES.add("autodef"); 
		LEVEL_2_TYPES.add("init");
		LEVEL_2_TYPES.add("def"); 
		LEVEL_2_TYPES.add("doc"); 
		LEVEL_2_TYPES.add("dim"); 
		LEVEL_2_TYPES.add("permanent"); 
		LEVEL_2_TYPES.add("autodim"); 
		LEVEL_2_TYPES.add("inflow"); 
		LEVEL_2_TYPES.add("outflow");
	}
	
	private String name;
	private List<String> mdlFormat;
	
	private Set<PowersimConstruct> powersimConstructs = new LinkedHashSet<PowersimConstruct>();

	public PowersimTextParser() {
	}
	
	public List<String> parse(List<List<String>> originalText) {
//		
//		List<String> tokens = tokenize(originalText);
//		for (String token : tokens)
//			System.out.println("Token: |"+token+"|");
//		
//		mdlFormat = makeMDL(tokens);
		
		return mdlFormat;
		
	}
	
	private List<String> makeMDL(List<String> tokens) {
		List<String> mdl = new ArrayList<String>();
		
		MutableInteger ptr = new MutableInteger(0);
		name = extractName(tokens, ptr, MAINMODEL);
		while (ptr.value() < tokens.size() && !tokens.get(ptr.value()).equals("}")) {
			PowersimConstruct c = getNextConstruct(tokens, ptr);
			System.out.println(c);
			powersimConstructs.add(c);
		}
		
		ptr.add(1);
		while (ptr.value() < tokens.size() && !tokens.get(ptr.value()).equals("}")) {
			PowersimConstruct c = getNextConstruct(tokens, ptr);
			System.out.println(c);
			powersimConstructs.add(c);
		}
		
		
		return mdl;
	}
	
	public PowersimConstruct getNextConstruct(List<String> tokens, MutableInteger ptr) {
		PowersimConstruct c = new PowersimConstruct(tokens.get(ptr.value()));
		c.setName(extractName(tokens, ptr, LEVEL_1_TYPES));
		extractAttributes(c, tokens, ptr, LEVEL_2_TYPES);
		return c;
	}
	
	public void extractAttributes(PowersimConstruct c, List<String> tokens, MutableInteger ptr, Set<String> keywords) {
		
		MutableBoolean endOfConstruct = new MutableBoolean(false);
		
		// should be pointing to an attribute name
		while(true /*!tokens.get(ptr.value()).equals("}") */) {
//			System.out.println("attr "+tokens.get(ptr.value()));
			String attrib = tokens.get(ptr.valueAndInc());
			String val = getToNextKeyword(tokens, ptr, keywords, endOfConstruct);
			c.addAttribute(attrib, val);
			if (endOfConstruct.value())
				break;
		}
	}
	
	public String getToNextKeyword(List<String> tokens, MutableInteger ptr, Set<String> keywords, MutableBoolean endOfConstruct) {
		StringBuilder sb = new StringBuilder();
		int openbrack = 0;
		int openparen = 0;
		while (ptr.value() < tokens.size() ) {
			
			if (tokens.get(ptr.value()).equals("{")) {
				openbrack++;
				appendToBuffer(sb, tokens.get(ptr.valueAndInc()));
			} else if (tokens.get(ptr.value()).equals("(")) {
				openparen++;
				appendToBuffer(sb, tokens.get(ptr.valueAndInc()));
			} else if (tokens.get(ptr.value()).equals("}")) {
				
				openbrack--;
				if (openbrack <= 0) {
					if (openbrack < 0) {
						endOfConstruct.setValue(true);
					} else {
						appendToBuffer(sb, tokens.get(ptr.value()));
					}
					ptr.add(1);
					if (openparen == 0)
						return sb.toString();
				}
				ptr.add(1);
			} else if (tokens.get(ptr.value()).equals(")")) {
				
				openparen--;
				if (openparen <= 0) {
					if (openparen < 0) {
						endOfConstruct.setValue(true);
					} else {
						appendToBuffer(sb, tokens.get(ptr.value()));
					}
					ptr.add(1);
					if (openbrack == 0)
						return sb.toString();
				}
				ptr.add(1);
			} else if (!keywords.contains(tokens.get(ptr.value()))) {

				appendToBuffer(sb, tokens.get(ptr.valueAndInc()));

			} else {
				break;
			}
		}

		return sb.toString();
	}
	
	public String extractName(List<String> tokens, MutableInteger ptr, Set<String> keywords) {
		StringBuilder sb = new StringBuilder();
		if (keywords.contains(tokens.get(ptr.value()))) {
			ptr.add(1);
			while (!tokens.get(ptr.value()).equals("{")) {
				appendToBuffer(sb, tokens.get(ptr.valueAndInc()));
			}
			// skip the "{" starting
			ptr.add(1);
		}
		
		return sb.toString();
	}
	
	private void appendToBuffer(StringBuilder sb, String text) {
		if (sb.length() > 0)
			sb.append(" ");
		sb.append(text);
	}
	
	private List<String> tokenize(List<String> originalText) {
		StringBuilder sb = new StringBuilder();
		
		for (String s : originalText) {
			sb.append(" "+s);
		}
		
		
		return split(sb.toString());
	}
	
	private List<String> split(String modelExpressions) {
		List<String> tokens = new ArrayList<String>();
		// split string on blanks, but keep quoted strings together
		// units specified as <<...>> should stay together


		MutableInteger position = new MutableInteger(0);

		// break into tokens and determine as much as possible
		// about the equation as we pass through it.
		String token = "";

		Parser.skipWhiteSpace(position, modelExpressions);
		boolean done = false;
		while(!done) {
			Parser.skipWhiteSpace(position, modelExpressions);
			if (!Parser.inRange(position, modelExpressions))
				break;
			String theChar = characterAt(position, modelExpressions);
			if (characterAt(position, modelExpressions).equals("\"") || characterAt(position, modelExpressions).equals("'")) {
				token = theChar+Parser.getQuotedStringStartingAt(position, modelExpressions)+theChar;
//				position.add(1);
			} else if (characterAt(position, modelExpressions).equals("<") && characterAt(position.value()+1, modelExpressions).equals("<")) {
				token = getInlineUnit(position, modelExpressions);
//				position.add(1);
				// check for single character terminators
			} else if (characterAt(position, modelExpressions).equals("[") || 
					characterAt(position, modelExpressions).equals("]") ||
					characterAt(position, modelExpressions).equals("{") ||
					characterAt(position, modelExpressions).equals("}") ||
					characterAt(position, modelExpressions).equals(",")
					) {
				token = characterAt(position, modelExpressions);
				position.add(1);
			} else {
				token = getNonQuotedStringStartingAt(position, modelExpressions);
			} 
			if (token.length() > 0)
				tokens.add(token);
			else
				System.out.println("TOKEN LEN = 0");
			
			
		}

		return tokens;
	}
	
	public String getNonQuotedStringStartingAt(MutableInteger position, String modelExpressions) {
		// gobble up spaces too! (for functions)
		String nonQuotedString = characterAt(position, modelExpressions);
		position.add(1);
		while (Parser.inRange(position, modelExpressions) && 
				/*
				 * Here we list any characters not part of the ordinary terminators that we want to consider as terminators
				 * For example, we need to stop on left and right parens
				 */
				!characterAt(position, modelExpressions).equals(" ") &&
				!characterAt(position, modelExpressions).equals("[") && 
				!characterAt(position, modelExpressions).equals("]") && 
				!characterAt(position, modelExpressions).equals("(") && 
				!characterAt(position, modelExpressions).equals(")") && 
				!characterAt(position, modelExpressions).equals(",") &&
				!characterAt(position, modelExpressions).equals("<") &&
				!characterAt(position, modelExpressions).equals("}") &&
				!characterAt(position, modelExpressions).equals("{")) {
			nonQuotedString += characterAt(position, modelExpressions);
			position.add(1);
		}

		return nonQuotedString.trim();
	}
	
	private String getInlineUnit(MutableInteger position, String line) {
		StringBuilder sb = new StringBuilder("<");
		position.add(1);
		while(!(characterAt(position, line).equals(">") && characterAt(position.value()+1, line).equals(">"))) {
			sb.append(characterAt(position, line));
			position.add(1);
		}
		sb.append(">>");
		position.add(2);
		
		return sb.toString();
	}
	
	private String characterAt(MutableInteger pos, String line) {
		return Parser.characterAt(pos, line);
	}
	
	private String characterAt(int pos, String line) {
		return Parser.characterAt(pos, line);
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the mdlFormat
	 */
	public List<String> getMdlFormat() {
		return mdlFormat;
	}

}
