package gov.anl.gss.sdm.server.model.powersim;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationComponents;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PowersimXlsxParser {
	
	public final static String DOUBLE_SLASH = "//";
	
	PowersimModelDescription modelDescription;
	List<String> row;
	boolean braceIsComment = false;
	
	String name;
	String vType;
	String unit;
	String dimensions;
	String definition;
	String value;
	String documentation;
	String notes;
	
	Map<String, Map<String, Set<String>>> flows = new HashMap<String, Map<String, Set<String>>>();
	Set<String> theLevels = new HashSet<String>();
	Set<String> theFlows = new HashSet<String>();
	
	List<Equation> eqns = new ArrayList<Equation>();
	List<String> mdlContents  = new ArrayList<String>();
	
	public PowersimXlsxParser (PowersimModelDescription modelDescription) {
		this.modelDescription = modelDescription;
	}
	
	public List<String> parseSource() {
		
		// Name, Variable Type, Unit, Dimensions, Definition, Value, Documentation, Note
		int stmt = 1;
		
		while ((row = modelDescription.getRow(stmt)) != null) {
			String tname = "\""+modelDescription.getRowColumn(stmt, "name")+"\"";
			String tvType = modelDescription.getRowColumn(stmt, "variable type");
			if (tvType == null || tvType.equals("")) {
				if (tname.contains(".")) {
					String[] parts = tname.split("\\.");
					String lvlName = Parser.removeQuotes(parts[0]).trim(); // parts[0].replace("\"", "").trim();
					theLevels.add(lvlName);
					String lvlArg = Parser.removeQuotes(parts[1]).trim(); // parts[1].replace("\"", "").trim();
					theFlows.add(lvlArg);
					String lvlFlowType = Parser.removeQuotes(parts[2]).trim(); // parts[2].replace("\"", "").trim();
					if (!flows.containsKey(lvlName)) {
						flows.put(lvlName, new HashMap<String, Set<String>>());
					}
					if (!flows.get(lvlName).containsKey(lvlFlowType)) {
						flows.get(lvlName).put(lvlFlowType, new LinkedHashSet<String>());
					}
					flows.get(lvlName).get(lvlFlowType).add(lvlArg);
				} else {
//					System.out.println("What is convention? "+tname);
				}
			}
			stmt++;
		}
		
		stmt = 1;
		
		while ((row = modelDescription.getRow(stmt)) != null) {
			String noQuoteName = modelDescription.getRowColumn(stmt, "name");
			name = "\""+modelDescription.getRowColumn(stmt, "name")+"\"";
			vType = modelDescription.getRowColumn(stmt, "variable type");
			unit = modelDescription.getRowColumn(stmt, "unit");
			dimensions = modelDescription.getRowColumn(stmt, "dimensions");
			definition = modelDescription.getRowColumn(stmt, "definition");
			value = modelDescription.getRowColumn(stmt, "value");
			documentation = modelDescription.getRowColumn(stmt, "documentation");
			notes = modelDescription.getRowColumn(stmt, "note");
			
			vType = "auxiliary";
			if (theLevels.contains(noQuoteName))
				vType = "level";
			if (theFlows.contains(noQuoteName))
				vType = "flow";
			if (Parser.isNumber(value))
				vType = "constant";
			if (noQuoteName.contains(".in") ||  noQuoteName.contains(".out"))
				vType = "levelFlows";
				
			
			String strippedDefinition = strip(definition, "<<", ">>");
			strippedDefinition = strip(strippedDefinition, "//", "//");

//
//			System.out.println("definition: "+definition);
//			List<String> tokenizedDefinition = Parser.tokenizeMaintainQuotes(definition, braceIsComment);
//			System.out.println("td: "+tokenizedDefinition);

			List<String> equations = new ArrayList<String>();

			strippedDefinition = stripGraphMinMax(strippedDefinition);
			if (vType == null || vType.equals("")) {
				stmt++;
				continue;
			}
			
			if (containsLookupDefinition(strippedDefinition)) {
				equations = getLookupAndReferenceEquations(strippedDefinition);
				for (int i = 0; i < equations.size(); i++) {
					mdlContents.addAll(createRawEquationForLookup(equations.get(i), unit, documentation));
				}
			} else if (vType.equalsIgnoreCase("level")) {
				if (flows.containsKey(Parser.removeQuotes(name))) /* name.replace("\"", ""))) */ {
					mdlContents.addAll(createRawEquationForLevel(name, strippedDefinition, unit, documentation, flows));
				} else {
					System.out.println("Missing flow info for level "+name);
				}
			} else if (!vType.equals("levelFlows")){
				String eqn = name +" = "+strippedDefinition;
//				System.out.println(eqn);
				mdlContents.addAll(createRawEquation(eqn, unit, documentation));
			}
			stmt++;

		}
		
		mdlContents.addAll(createRawEquation("FINAL TIME  = 2100", "Year", "The final time for the simulation"));
		mdlContents.addAll(createRawEquation("INITIAL TIME  = 1900", "Year", "The initial time for the simulation"));
		mdlContents.addAll(createRawEquation("SAVEPER  = 0.25", "Year", "The frequency with which data is stored"));
		mdlContents.addAll(createRawEquation("TIME STEP  = 0.25", "Year", "The time step for the simulation"));
		
		for (int i = 0; i < mdlContents.size(); i++)
			System.out.println(mdlContents.get(i));


		return mdlContents;
	}
	
	public List<String> createRawEquationForLevel(String name, String definition, String unit, String documentation,
			Map<String, Map<String, Set<String>>> flows) {
		List<String> al = new ArrayList<String>();
		StringBuilder sb = new StringBuilder();

		String nameNoQuotes = Parser.removeQuotes(name); // name.replace("\"", "");
		Set<String> ins = flows.get(nameNoQuotes).get("in");
		Set<String> outs = flows.get(nameNoQuotes).get("out");
		sb.append(name+ " = INTEG(");
		int cnt = 0;

		/////////////////////////////////
		if (ins != null) {
			if (ins.size() > 1)
				sb.append("(");
			for (String i : ins) {
				if (cnt++ > 0) {
					sb.append(" + ");
				}
				sb.append("\""+i+"\"");
			}
			if (ins.size() > 1)
				sb.append(")");
			cnt = 0;
		}
		///////////////////////////////

		if (outs != null) {
			if (outs.size() > 0) {
				sb.append("-");
			}
			if (outs.size() > 1)
				sb.append("(");
			for (String i : outs) {
				if (cnt++ > 0) {
					sb.append(" + ");
				}
				sb.append("\""+i+"\"");
			}
			if (outs.size() > 1)
				sb.append(")");
		}

		////////////////////////////

		sb.append(",");
		sb.append(definition);
		sb.append(")");
		al.add(sb.toString());
		al.add("\t~\t"+unit);
		al.add("\t~\t"+(documentation == null ? " " : documentation));
		al.add("\t|");
//		System.out.println(al);
		return al;
	}
	
	public List<String> getLookupAndReferenceEquations(String s) {
//		use this form
//		effect energy tab (
//				 (0.0,0.0), (0.1,0.2), (0.2,0.4),(0.3,0.6), (0.4,0.7), 
//					(0.5,0.8), (0.6,0.85), (0.7,0.9), (0.8,0.95), (0.9,0.98), (1.0,1.0), (1.2,1.02), 
//					(2,1.02) )
		List<String> tokens = Parser.tokenizeMaintainQuotes(s, braceIsComment);
		List<Integer> invocations = findInvocations(tokens);
		List<Integer> dataStarts = findDataStarts(tokens);
		List<Integer> dataEnds = findDataEnds(tokens);
		List<String> looks = buildLookups(tokens, dataStarts, dataEnds);
		if (looks.size() > 1)
			System.out.println("MULTIPLE EQUATIONS");
		return buildEquations(looks, tokens, invocations, dataStarts, dataEnds);
	}
	
	public List<String> buildEquations(List<String> looks, List<String> tokens, 
			List<Integer> invocations, List<Integer> dataStarts, List<Integer> dataEnds) {
		List<String> equations = new ArrayList<String>();
		for (int i = 0; i < looks.size(); i++) {
			String tname = "\""+Parser.removeQuotes(name) /* name.replace("\"", "") */ + (looks.size() > 1 ? i : "")+ StringConstants.RSSD_LOOKUP+"\"";
//			String tname = name.replace("\"", "") + (looks.size() > 1 ? i : "")+ StringConstants.RSSD_LOOKUP;
//			String lookupDef = tname+looks.get(i);
			equations.add(tname+"("+looks.get(i)+")");
			
			int endInvocation = dataStarts.get(i) - 5;
			StringBuilder sb = new StringBuilder();
			
			sb.append(name+" = "+tname);
			for (int j = invocations.get(i)+1; j < endInvocation; j++) {
				sb.append(tokens.get(j));
			}
			sb.append(")");
			equations.add(sb.toString());
		}
		
		
		return equations;
	}
	
	public List<String> buildLookups(List<String> tokens, List<Integer> dataStarts, List<Integer> dataEnds) {
		List<String> looks = new ArrayList<String>();
			for (int i = 0; i < dataStarts.size(); i++) {
				looks.add(buildLookup(tokens, dataStarts.get(i), dataEnds.get(i)));
				if (i == 0)
					break;
			}
		return looks;
	}
	
	public String buildLookup(List<String> tokens, Integer dataStart, Integer dataEnd) {
		StringBuilder sb = new StringBuilder();
		int startXIndex = dataStart-4;
		int deltaXIndex = dataStart-2;
		double x = Double.parseDouble(tokens.get(startXIndex));
		double dx = Double.parseDouble(tokens.get(deltaXIndex));
		
		for (int i = dataStart+1; i < dataEnd; i+=2) {
			if (sb.length() > 0)
				sb.append(",");
			sb.append(x);
			x += dx;
		}
		for (int i = dataStart+1; i < dataEnd; i+=2) {
			if (sb.length() > 0)
				sb.append(",");
			sb.append(tokens.get(i));
		}
		
		
//		for (int i = dataStart+1; i < dataEnd; i+=2) {
//			if (sb.length() > 0)
//				sb.append(",");
//			sb.append("("+x+","+tokens.get(i)+")");
//			x += dx;
//		}
		return sb.toString();
	}
	
	List<Integer> findDataStarts(List<String> tokens) {
		return findMarker(tokens, "{");
	}
	
	List<Integer> findInvocations(List<String> tokens) {
		return findMarker(tokens, new String[] {"graph", "graphcurve", 
				"graphlinas", "graphstep"});
	}
	
	List<Integer> findDataEnds(List<String> tokens) {
		return findMarker(tokens, "}");
	}
	
	List<Integer> findMarker(List<String> tokens, String marker) {
		List<Integer> finds = new ArrayList<Integer>();
		for (int i = 0; i < tokens.size(); i++) {
			if (tokens.get(i).equals(marker))
				finds.add(i);
		}
		return finds;
	}
	
	List<Integer> findMarker(List<String> tokens, String[] marker) {
		List<Integer> finds = new ArrayList<Integer>();
		for (int i = 0; i < tokens.size(); i++) {
			if (contains(marker, tokens.get(i)))
				finds.add(i);
		}
		return finds;
	}
	
	public boolean contains(String[] markers, String token) {
		for (String m : markers) {
			if (m.equals(token.toLowerCase()))
				return true;
		}
		return false;
	}
	
	public boolean containsLookupDefinition(String s) {
		String sl = s.toLowerCase();
		return sl.contains("graph(") || 
				sl.contains("graphcurve(") ||
				sl.contains("graphlinas(") ||
				sl.contains("graphstep(");
	}
	
	public Equation createEquation(String eqn, String unit, String documentation) {
		EquationComponents ec = new EquationComponents(eqn+"~"+unit+"~"+documentation);
		Equation e = new Equation(ec);
		return e;
	}
	
	public List<String> createRawEquation(String eqn, String unit, String documentation) {
		List<String> al = new ArrayList<String>();
		al.add(eqn);
		al.add("\t~\t"+unit);
		al.add("\t~\t"+(documentation == null ? " " : documentation));
		al.add("\t|");
		return al;
	}
	
	public List<String> createRawEquationForLookup(String eqn, String unit, String documentation) {
		List<String> al = new ArrayList<String>();
		if (eqn.contains(StringConstants.RSSD_LOOKUP) && !eqn.contains("=")) {
			String[] parts = eqn.split("\\(");
			al.add(parts[0]+"(");
			al.add(parts[1]);
		} else {
			al.add(eqn);
		}
		al.add("\t~\t"+unit);
		al.add("\t~\t"+(documentation == null ? " " : documentation));
		al.add("\t|");
//		System.out.println(al);
		return al;
	}
	
	public String stripGraphMinMax(String s) {
		
		// for Graph, GraphCurve, GraphLinAs, and GraphStep
		// filter our the min and max values
		
		if (! (s.toLowerCase().contains("graph") && s.contains(DOUBLE_SLASH)))
			return s;
		
		
		return s.replaceAll("//Min:[^;]*;Max:[0-9\\.]*//", "");
		
	}
	
	public List<Integer> locatePatterns(String s , String pattern) {
		List<Integer> locs = new ArrayList<Integer>();
		
		int lastPossibleIndex = s.length() - pattern.length();
		int startIndex = 0;
		int index;
		while (startIndex <= lastPossibleIndex) {
			index = s.indexOf(pattern, startIndex);
			if (index == -1)
				break;
			locs.add(index);
			startIndex = index+pattern.length();
		}	
		return locs;
	}

	public String strip(String s, String startPattern, String endPattern) {
		StringBuilder sb = new StringBuilder();
		if (s == null || !s.contains(startPattern))
			return s;
		List<Integer> starts = locatePatterns(s, startPattern); 
		List<Integer> ends = locatePatterns(s, endPattern);
		
//		System.out.println("s: <"+s+">");
//		System.out.println("starts: "+starts);
//		System.out.println("ends: "+ends);
		
		int start = 0;
		
		for (int i = 0; i < starts.size(); i++) {
			int startSkip = starts.get(i);
			int endSkip = ends.get(i)+endPattern.length()-1;
			
			if (startSkip > start) {
				sb.append(s.substring(start, startSkip));
			}
			
			start = endSkip+1;
			// if this is last grouping check if we need the rest of the line
			if (i == starts.size()-1) {
				if (start < s.length()) {
					sb.append(s.substring(start));
				}
			}
		}
		
		return sb.toString();
	}
}
