package gov.anl.gss.sdm.server.model.structure;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;
import gov.anl.gss.sdm.server.model.analysis.polarity.PolarityResults;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;
import gov.anl.gss.sdm.server.model.translator.View;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class SDObjectManager {
	static Logger log = Logger.getLogger(SDObjectManager.class.getSimpleName());

	public static final String LHS = "LHS";
	public static final String RHS = "RHS";
	public static final String RHS_USE = "RHS_USE";

	private String currentView;

	private Map<String, SDObject> objects;

	private Map<String, Set<String>> usedBy; // key = tail value = head
	private Map<String, Set<String>> uses;	// key head value = tails

	private Map<String, Set<String>> loopUsedBy;
	private Map<String, Set<String>> loopUses;

	private Map<String, Equation> equations;
	private Set<String> rhsVars;
	private Set<String> lhsVars;

	private Map<String, View> graphics;
	private Map<String, String> variableToAnnotated;

	private Map<String, Map<String, Set<String>>> variableReferences;
	private Map<String, Map<String, Set<String>>> stockVariableReferences;


	private String objectName;

	//	public SDObjectManager(Map<String, Equation> equations) {
	//		this();
	//		this.equations = equations;
	//		initializeWithEquations();
	//	}

	public SDObjectManager(Map<String, Equation> equations, Map<String, View> graphics, boolean fullyDefined) {
		this();
		this.equations = equations;
		this.graphics = graphics;
		if (fullyDefined) {
			initializeWithEquations();
		} else {
			initializeWithGraphics();
		}
	}

	public SDObjectManager() {
		usedBy = new LinkedHashMap<String, Set<String>>();
		uses = new LinkedHashMap<String, Set<String>>();
		variableToAnnotated = new LinkedHashMap<String, String>();
		rhsVars = new LinkedHashSet<String>();
		lhsVars = new LinkedHashSet<String>();

		variableReferences = new LinkedHashMap<String, Map<String, Set<String>>>();
		stockVariableReferences = new LinkedHashMap<String, Map<String, Set<String>>>();
	}

	public void broadcastLinks() {

		//		for (String tail : usedBy.keySet()) {
		//			for (String head : usedBy.get(tail)) {
		//				Equation eqnT = InformationManagers.getInstance().getEquationManager().getAnEquationFor(tail);
		//				if (eqnT == null)
		//					continue;
		//				Equation eqnH = InformationManagers.getInstance().getEquationManager().getAnEquationFor(head);
		//				if (eqnH == null)
		//					continue;
		//				eqnH.setNumInputLinks(eqnH.getNumInputLinks()+1);
		//				eqnT.setNumOutputLinks(eqnH.getNumOutputLinks()+1);
		//			}
		//		}

		for (String tail : usedBy.keySet()) {
			for (String head : usedBy.get(tail)) {
				for (Equation eqnT : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(tail)) {
					for (Equation eqnH : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(head)) {
						eqnH.incrementNumInputLinksConditional(tail);
//						eqnT.setNumOutputLinks(eqnH.getNumOutputLinks()+1);
					}
				}
			}
		}
	}

	public List<String[]> getEdges() {

		// private Map<String, Set<String>> usedBy; // key = tail value = head

		List<String[]> edges = new ArrayList<String[]>();
		for (String tail : usedBy.keySet()) {
			for (String head : usedBy.get(tail)) {
				String[] anEdge = new String[2];
				anEdge[0] = new String(tail);
				anEdge[1] = new String(head);
				edges.add(anEdge);
			}
		}

		return edges;
	}

	public void registerObject(SDObject obj) {
		objects.put(obj.getName(), obj);
	}

	public void loadPolarityArrowsFromEquations(PolarityResults polarityResults) {
		//		loadPolarityResultsFromGraphics(polarityResults);

		// uses: key is lhs, value is set of variables on rhs - one per equation
		// usedBy: key is variable value is set of equation LHS in which key is used

		List<String> al = new ArrayList<String>(uses.keySet());
		Collections.sort(al);
		for (String head : al) {
			for (String tail : uses.get(head)) {
				polarityResults.createArrow(tail, head);
			}
		}
	}

	public void loadPolarityResultsFromGraphics(PolarityResults polarityResults) {
		log.debug("... initialize PolarityResults with Graphic objects");

		int numGo = 0;
		int numArrow = 0;

		for (String viewName : graphics.keySet()) {
			View view = graphics.get(viewName);
			for (GraphicObject arrow : view.getGraphicObjects()) {
				numGo++;
				if (arrow.isArrow()) {
					numArrow++;
					GraphicObject to = view.idToGraphic(arrow.getTo());
					GraphicObject from = view.idToGraphic(arrow.getFrom());


					if (to == null || from == null)
						continue;

					if (to.getType().equals(GraphicObject.COMMENT)) {
						System.out.println("Skip To comment arrow");
						continue;
					}
					if (from.getType().equals(GraphicObject.COMMENT)) {
						System.out.println("Skip From comment arrow");
						continue;
					}


					String fname = from.isValve() ? from.getAssociatedVariable().getName() : from.getName();
					String tname = to.isValve() ? to.getAssociatedVariable().getName() : to.getName();
					
					// Becoming a criminal
					if (fname.equals("Becoming a criminal"))
						System.out.println("Becoming a criminal");
					
					String polarityInDiagram = arrow.getPolarity();
					if (!polarityInDiagram.equals(ArrowPolarity.UNKNOWN)) {
						polarityResults.setPolarityForArrow(fname, tname, polarityInDiagram);
						continue;
					}

					Equation feqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(fname);
					Equation teqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(tname);

					if (arrow.isSwapped()) {
						String t = fname;
						Equation tmpEqn = feqn;
						fname = tname;
						feqn = teqn;
						tname = t;
						teqn = tmpEqn;
					}
					if (arrow.isFlowArrow() /*&& ((feqn != null && feqn.isStock()) || (teqn != null && teqn.isStock())) */) {

//					if (arrow.isFlowArrow() && ((feqn != null && feqn.isStock()) || (teqn != null && teqn.isStock()))) {
						// polarity information taken from structure for stocks
						String aPolarity = ArrowPolarity.UNKNOWN;
						if (arrow.isInFlowArrow()) {
							aPolarity = ArrowPolarity.POSITIVE;
						} else if (arrow.isOutFlowArrow()) {
							aPolarity = ArrowPolarity.NEGATIVE;
						}
//						if (feqn != null && feqn.isStock()) {
//							aPolarity = ArrowPolarity.NEGATIVE;
//							if (teqn != null)
//								teqn.setFlow(true);
//						} else if (teqn != null && teqn.isStock()){
//							if (feqn != null)
//								feqn.setFlow(true);
//						}

						if (arrow.isSwapped()) {
							polarityResults.setPolarityForArrow(tname, fname, aPolarity);
						} else {
							polarityResults.setPolarityForArrow(fname, tname, aPolarity);
						}
					} else {
						//						if (arrow.isSwapped()) {
						//							//						arrow.reversePolarity();
						//							polarityResults.setPolarityForArrow(to.isValve() ? to.getAssociatedVariable().getName() : to.getName(),  arrow.getPolarity(),
						//									from.isValve() ? from.getAssociatedVariable().getName() : from.getName() );
						//						} else {
						polarityResults.setPolarityForArrow(from.isValve() ? from.getAssociatedVariable().getName() : 
							from.getName(), to.isValve() ? to.getAssociatedVariable().getName() : to.getName(), arrow.getPolarity());
						//						}
					}

					//					if (arrow.isSwapped())
					//						polarityResults.setPolarityForArrow(to.getName(), from.getName(),  arrow.getPolarity());
					//					else
					//						polarityResults.setPolarityForArrow(from.getName(), to.getName(), arrow.getPolarity());
				}
			}	
		}
		log.debug("numGo: "+numGo+" numArrow: "+numArrow);
		if (SDM_Doc.DUMP_TABULAR_DATA) {
			polarityResults.writeArrowsToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/arrowPolarity.tab");

			polarityResults.writeArrowThicknessToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/arrowThickness.tab");
		}

		if (InformationManagers.getInstance().isSdmuiMode()) {
			polarityResults.writeArrowsToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure"+"/arrowPolarityFromEquationsNoIVV.tab");
			polarityResults.writeArrowThicknessToFile(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName()+"/structure"+"/arrowThickness.tab");
		}
	}

	private void initializeWithGraphics() {

		log.debug("... initialize SDObjectManager with Graphic objects");
		log.debug("... numEquations: "+equations.size());

		System.out.println("... initialize SDObjectManager with Graphic objects");
		System.out.println("... numEquations: "+equations.size());

		// getRHSVariablesForOrdering()

		for (String viewName : graphics.keySet()) {
			View view = graphics.get(viewName);
			for (GraphicObject arrow : view.getGraphicObjects()) {
				if (arrow.isArrow()) {
					GraphicObject to = view.idToGraphic(arrow.getTo());
					GraphicObject from = view.idToGraphic(arrow.getFrom());
					// this is bad, but for now if it is a value arrow and no associated varaible, skip it
					if (to == null || (to.isValve() && to.getAssociatedVariable() == null)) {
//						log.debug("SKIPPING (to) Arrow: to from "+to.getName()+" <- "+from.getName());
						System.out.println("SKIPPING (to) Arrow: to from "); // +to.getName()+" <- "+from.getName());
						continue;
					}
					if (from.isValve() && from.getAssociatedVariable() == null) {
						log.debug("SKIPPING (from) Arrow: to from "+to.getName()+" <- "+from.getName());
						System.out.println("SKIPPING (from) Arrow: to from "+to.getName()+" <- "+from.getName());
						continue;
					}
					//					log.debug("Arrow: to from "+(to.isValve() ? to.getAssociatedVariable().getName() : to.getName())+" <- "+(from.isValve() ? from.getAssociatedVariable().getName() : from.getName()));
					addUse(to.isValve() ? to.getAssociatedVariable().getName() : to.getName(), 
							from.isValve() ? from.getAssociatedVariable().getName() : from.getName());
				}
			}	
		}
	}
	
	public Map<String, Set<String>> getVariableReferences(String var) {
		return variableReferences.get(var);
	}
	
	public Map<String, Set<String>> getStockVariableReferences(String var) {
		return stockVariableReferences.get(var);
	}
	
	private void addLHSVariableReference(String var, String form) {
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		String varOrig = ndtm.getOriginalName(var).replace("!]","]").replace("!,",",");
		String formOrig = ndtm.getOriginalName(form).replace("!]","]").replace("!,",",");
		
		if (varOrig.contains("MINUS"))
			System.out.println("DEBUG Minus1");

		if (!variableReferences.containsKey(varOrig)) {
			variableReferences.put(varOrig, new LinkedHashMap<String, Set<String>>());
			variableReferences.get(varOrig).put(LHS, new LinkedHashSet<String>());
			variableReferences.get(varOrig).put(RHS, new LinkedHashSet<String>());
			variableReferences.get(varOrig).put(RHS_USE, new LinkedHashSet<String>());
		}
		variableReferences.get(varOrig).get(LHS).add(formOrig);
		
//		System.out.println("VarRefLHS: <"+varOrig+"> <"+formOrig+">");
	}
	
	private void addRHSUseVariableReference(String rhsReference) {

		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
//		String rhsReferenceVariable = Parser.getVariableName(rhsReference);
		String rhsReferenceVariable = Parser.getVariableName(ndtm.getOriginalName(rhsReference).replace("!]","]").replace("!,",","));
		String rhsReferenceT = /*Parser.getVariableName(*/ndtm.getOriginalName(rhsReference).replace("!]","]").replace("!,",",")/*)*/;
		
		if (rhsReferenceVariable.contains("MINUS")) {
			System.out.println("DEBUG Minus2 "+rhsReferenceVariable);
		}
		
		
		if (!variableReferences.containsKey(rhsReferenceVariable)) {
			variableReferences.put(rhsReferenceVariable, new LinkedHashMap<String, Set<String>>());
			variableReferences.get(rhsReferenceVariable).put(LHS, new LinkedHashSet<String>());
			variableReferences.get(rhsReferenceVariable).put(RHS, new LinkedHashSet<String>());
			variableReferences.get(rhsReferenceVariable).put(RHS_USE, new LinkedHashSet<String>());
		}
		
//		System.out.println("RHSUse: <"+rhsReferenceVariable+"> <"+rhsReference+">");
		
		variableReferences.get(rhsReferenceVariable).get(RHS_USE).add(rhsReferenceT.replace("!]","]").replace("!,",","));
	}
	
	private void addRHSVariableReference(String var, String form) {

		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		String varOrig = ndtm.getOriginalName(var).replace("!]","]").replace("!,",",");
		String formOrig = ndtm.getOriginalName(form).replace("!]","]").replace("!,",",");
		
		if (varOrig.contains("MINUS"))
			System.out.println("DEBUG Minus3");
		
		
		if (!variableReferences.containsKey(varOrig)) {
			variableReferences.put(varOrig, new LinkedHashMap<String, Set<String>>());
			variableReferences.get(varOrig).put(LHS, new LinkedHashSet<String>());
			variableReferences.get(varOrig).put(RHS, new LinkedHashSet<String>());
			variableReferences.get(varOrig).put(RHS_USE, new LinkedHashSet<String>());
		}
		variableReferences.get(varOrig).get(RHS).add(formOrig.replace("!]","]").replace("!,",","));
//		System.out.println("VarRefRHS: <"+varOrig+"> <"+formOrig.replace("!]","]").replace("!,",",")+">");
	}
	
	private void addVariableReferences(Equation eqn) {
		// keep track of the form of all variable references
		
		// what does this really accomplish?
		
		String lhsVar = Parser.removeAnnotation(eqn.getLhs());
		if (ArrayReference.isArrayReference(lhsVar)) {
			ArrayReference ar = new ArrayReference(lhsVar);
			lhsVar = ar.getArrayName();
		}
		
//		System.out.println("VarRefEqn: "+eqn.getVensimEquationOnly());
		
		addLHSVariableReference(lhsVar, Parser.removeAnnotation(eqn.getLhs()));
		

		if (!eqn.isDefinesLookup()) {

			for (String v : eqn.getRHSVariablesForObjectManager()) { // eqn.getRHSVariablesForOrdering()) {
				String var = v;
				if (ArrayReference.isArrayReference(v)) {
					ArrayReference ar = new ArrayReference(v);
					var = ar.getArrayName();
				}
				
				addRHSVariableReference(Parser.removeAnnotation(var), Parser.removeAnnotation(eqn.getLhs())); // Parser.removeAnnotation(lhsVar));
//				addRHSVariableReference(Parser.removeAnnotation(lhsVar), Parser.removeAnnotation(var));
				
				addRHSUseVariableReference(Parser.removeAnnotation(v));
			}
		}
	}

	private void initializeWithEquations() {

		log.debug("... initialize SDObjectManager");
		log.debug("... numEquations: "+equations.size());

		ArrayReference ar = null;

		// getRHSVariablesForOrdering()
		
		int numEquations = equations.keySet().size();
		int numConsidered = 0;

		for (String lhs : equations.keySet()) {
			// only assignments need to be considered

			Equation eqn = equations.get(lhs);
			
			if (!eqn.isDefinesSubscript())
				addVariableReferences(eqn);

			if (!eqn.isAssignment() && !eqn.isDefinesLookup()) {
				continue;
			}
			if (eqn.isArrayInitialization())
				continue;
			if (eqn.isDefinesLookup())
				continue;
			if (eqn.isGetExternalData())
				continue;

			
			numConsidered++;
			String rhs;
			String lhst;
			lhst = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));

			Set<String> rhsVars = eqn.getRHSVariables();
			if (rhsVars.size() > 0) {
				for (String rhst : rhsVars) {
					rhs = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhst));

					addUse(lhst, rhs);

					variableToAnnotated.put(rhs, InformationManagers.getInstance().getEquationLookup().getLhsName(rhs));
				} 
			} else {
				lhsVars.add(lhs);
			}
			variableToAnnotated.put(lhst, lhs);
		}

		if (SDM_Doc.DUMP_TABULAR_DATA) {

			dumpToFile(Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), "equationVariables.tab"));
			dumpVariableReferencesToFile(Utilities.openFileForWriting(
					InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName(), "variableReferences.tab"));
		}

	}
	
	public Set<String> getVariables() {
		return new LinkedHashSet<String>(variableReferences.keySet());
	}
	
	public Set<String> getLhsVariables() {
		Set<String> lhsSet =  new LinkedHashSet<String>();
		for (String v : variableReferences.keySet()) {
			lhsSet.addAll(variableReferences.get(v).get(LHS));
		}
		
		
		return lhsSet;
	}
	
	public void dumpVariableReferencesToFile(BufferedWriter bw) {
		
		// private Map<String, Map<String, Set<String>>> variableReferences;
		
		for (String var : variableReferences.keySet()) {
			
			Map<String, Set<String>> refs = variableReferences.get(var);
			Set<String> lhsRefs = refs.get(LHS);
			Set<String> rhsRefs = refs.get(RHS);
			Set<String> rhsUseRefs = refs.get(RHS_USE);
			
			try {
				for (String s : Utilities.ordered(lhsRefs)) {
					bw.append(var+"\t"+LHS+"\t"+s+"\n");
				}
				
				for (String s : Utilities.ordered(rhsRefs)) {
					bw.append(var+"\t"+RHS+"\t"+s+"\n");
				}
				
				for (String s : Utilities.ordered(rhsUseRefs)) {
					bw.append(var+"\t"+RHS_USE+"\t"+s+"\n");
				}
				
				
				
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void dumpToFile(BufferedWriter bw) {
		try {
			bw.append("LHS Vars\n");
			List<String> al = new ArrayList<String>(lhsVars);
			Collections.sort(al);
			for (String s : al) {
				bw.append(s+" lhs\n");
			}
			bw.append("\nRHS Vars\n");
			al = new ArrayList<String>(rhsVars);
			Collections.sort(al);
			for (String s : al) {
				bw.append(s+" rhs\n");
			}

			bw.append("\nVar\tusedBy\n");
			al = new ArrayList<String>(usedBy.keySet());
			Collections.sort(al);
			for (String s : al) {
				bw.append(s+" used by\n");
				for (String b : usedBy.get(s)) {
					bw.append("\t"+b+"\n");
				}
			}

			bw.append("\nVar\tuses\n");
			al = new ArrayList<String>(uses.keySet());
			Collections.sort(al);
			for (String s : al) {
				bw.append(s+" uses\n");
				for (String b : uses.get(s)) {
					bw.append("\t"+b+"\n");
				}
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public Map<String, Set<String>> getAllInitialValueVariables() {
		Map<String, Set<String>> allIVV = new LinkedHashMap<String, Set<String>>();
		for (Equation eqn : equations.values()) {
			if (eqn.isStock()) {

				for (String ivv : toOriginalName(eqn.getFunctionInitialVariables())) {

					boolean ignore = false;

					// this same ivv can be used in the non initial value expression
					// if so, don't load this

					for (String ivv2 : toOriginalName(eqn.getNonFunctionInitialVariables())) {
						if (ivv.equals(ivv2))
							ignore = true;
					}

					if (!ignore) {
						if (!allIVV.containsKey(ivv)) {
							allIVV.put(ivv, new LinkedHashSet<String>());
						}

						allIVV.get(ivv).add(eqn.getOriginalLhs());
					}
				}
			} else if (eqn.isInitial()) {
				if (!allIVV.containsKey(eqn.getOriginalLhs())) {
					allIVV.put(eqn.getOriginalLhs(), new LinkedHashSet<String>());
				}
				allIVV.get(eqn.getOriginalLhs()).add(eqn.getOriginalLhs());
			}
		} 
		return allIVV;

	}

	public void initializeForLoopDiscovery() {
		loopUsedBy = copyMap(usedBy);
		loopUses = copyMap(uses);
	}

	public Map<String, Set<String>> copyMap(Map<String, Set<String>> map) {
		Map<String, Set<String>> copy = new LinkedHashMap<String, Set<String>>();

		for (String key : map.keySet()) {
			copy.put(key, new LinkedHashSet<String>());
			for (String val : map.get(key)) {
				copy.get(key).add(val);
			}

		}

		return copy;
	}

	//	public Set<String> getUsedByNotInitialValue(String variable) {
	//
	//		Set<String> aStringSet = usedBy.get(variable);
	//		
	//		if (aStringSet == null) {
	//			return new LinkedHashSet<String>();
	//		}
	//		
	//		Set<String> aStringSetNotInitialValue = new LinkedHashSet<String>(aStringSet);
	//		
	//		for (String usedLHS : aStringSet) {
	//			// get this equation
	//			Equation eqn = equations.get(variableToAnnotated.get(usedLHS));
	//			if (eqn != null) {
	//				if (eqn.isStock()) {
	//					// fiv contains the variables used as initial values for this eqn
	//					// if out variable is used as iv, then remove it
	//					Set<String> fiv = toOriginalName(eqn.getFunctionInitialVariables());
	//					if (fiv.contains(variable))
	//						aStringSetNotInitialValue.remove(variable);
	//				} else {
	//					return aStringSet;
	//				}
	//			} else {
	//				log.debug("Something wrong");
	//			}
	//		}
	//		
	//		return aStringSetNotInitialValue;
	//	}

	private Set<String> toOriginalName(Set<String> aSet) {
		Set<String> set = new LinkedHashSet<String>();

		for (String annotated : aSet) {
			String orig = 
					Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(annotated));
			set.add(orig);
		}
		return set;
	}

	public Set<String> getUsedBy(String variable) {

		Set<String> aStringSet = usedBy.get(variable);
		if (aStringSet != null)
			return aStringSet;
		else {
			aStringSet = usedBy.get(Parser.removeQuotes(variable)); // variable.replace("\"", ""));
			if (aStringSet != null)
				return aStringSet;
			return new LinkedHashSet<String>();
		}
	}

	public Set<String> getUses(String variable) {

		Set<String> aStringSet = uses.get(variable);
		if (aStringSet != null)
			return aStringSet;
		else {
			return new LinkedHashSet<String>();
		}
	}

	public Set<String> getLoopUsedBy(String variable) {

		Set<String> aStringSet = loopUsedBy.get(variable);
		if (aStringSet != null)
			return aStringSet;
		else {
			return new LinkedHashSet<String>();
		}
	}

	public Set<String> getLoopUses(String variable) {

		Set<String> aStringSet = loopUses.get(variable);
		if (aStringSet != null)
			return aStringSet;
		else {
			return new LinkedHashSet<String>();
		}
	}

	public void addVariable(String variable) {

		if (!usedBy.containsKey(variable))
			usedBy.put(variable, new LinkedHashSet<String>());
	}

	public void addUse(String lhs, String rhs) {

		//		System.out.println("addUse: l,r "+lhs+" ,"+ rhs);
		if (lhs.equals("0") || rhs.equals("0"))
			System.out.println("Zero?");

		// don't allow self reference
		if (rhs.equals(lhs))
			return;

		String newRHS = new String(rhs);
		if (rhs.contains(StringConstants.RSSD_LOOKUP) || rhs.startsWith("lookup.")) {
			newRHS = newRHS.replace("lookup.", "");
		}
		
		// mjb 8/8/21
		
		
		newRHS = Parser.removeQuotes(newRHS);

		lhsVars.add(lhs);
		rhsVars.add(rhs);

		addVariable(newRHS);

		usedBy.get(newRHS).add(lhs);
		if (!uses.containsKey(lhs))
			uses.put(lhs, new LinkedHashSet<String>());
		uses.get(lhs).add(newRHS);

	}

	public List<String> getUnused() {
		// variables that appear only on lhs
		List<String> al = new ArrayList<String>();
		for (String lhs : lhsVars) {
			if (!rhsVars.contains(lhs))
				al.add(lhs);
		}
		return al;
	}

	public List<String> getUndefined() {
		// variables that appear only on rhs
		List<String> al = new ArrayList<String>();
		for (String rhs : rhsVars) {
			if (!lhsVars.contains(rhs))
				al.add(rhs);
		}
		return al;
	}

	public List<String> getStockAndAuxiliaryVariables() {

		// used only for loop discovery

		List<String> al = new ArrayList<String>();
		// only variables on lhs 
		for (String s : lhsVars) {
			// if this invokes initial, it cannot be part of loop
			if (equations.get(s).isInitial())
				continue;
			// must use something else
			if (usedBy.get(s) != null && usedBy.get(s).size() > 0)
				al.add(s);
		}
		return al;
	}

	public List<String> getStockAndAuxiliaryVariablesForLoops() {
		List<String> al = new ArrayList<String>();
		// only variables on lhs 
		for (String s : lhsVars) {
			// must use something else
			if ((usedBy.get(s) != null && usedBy.get(s).size() > 0) && 
					uses.get(s) != null && uses.get(s).size() > 0)
				al.add(s);
		}
		return al;
	}

	public Set<String> getAllKnowVariables() {
		Set<String> aSet = new LinkedHashSet<String>();
		// only variables on lhs 
		aSet.addAll(lhsVars);
		aSet.addAll(rhsVars);
		Set<String> toRemove = new LinkedHashSet<String>();
		for (String var : aSet) {
			if (!equations.containsKey(var)) {
				toRemove.add(var);
			} else {
				if(!equations.get(var).isAssignment()) {
					toRemove.add(var);
				}
			}
		}
		aSet.removeAll(toRemove);
		return aSet;
	}

	public void setEquations(Map<String, Equation> equations) {
		this.equations = equations;
		initializeWithEquations();
	}

	public String getCurrentView() {
		return currentView;
	}

	public void setCurrentView(String currentView) {
		this.currentView = currentView;
	}

	public Map<String, Set<String>> getUses() {
		return uses;
	}

	public Map<String, Set<String>> getUsedBy() {
		return usedBy;
	}

	/**
	 * @return the loopUsedBy
	 */
	public Map<String, Set<String>> getLoopUsedBy() {
		return loopUsedBy;
	}

	/**
	 * @return the loopUses
	 */
	public Map<String, Set<String>> getLoopUses() {
		return loopUses;
	}

	/**
	 * @return the equations
	 */
	public Map<String, Equation> getEquations() {
		return equations;
	}

	/**
	 * @return the objectName
	 */
	public String getObjectName() {
		return objectName;
	}

	/**
	 * @param objectName the objectName to set
	 */
	public void setObjectName(String objectName) {
		this.objectName = objectName;
	}

	public Map<String, Map<String, Set<String>>> getVariableReferences() {
		return variableReferences;
	}


}
