/**
 * 
 */
package gov.anl.gss.sdm.server.model.support;

import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 * Contains information on an array structure
 * 
 * @author bragen
 *
 */
public class ArrayDefinition {

	private String name;
	private Map<Integer, Set<String>> subscriptReferences;

	public ArrayDefinition(String name, int numDimensions) {
		this.name = name;
		subscriptReferences = new LinkedHashMap<Integer, Set<String>>();
		for (int i = 0; i < numDimensions; i++) {
			subscriptReferences.put(i, new LinkedHashSet<String>());
		}
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void addReference(int pos, String subscript) {
		if (subscriptReferences == null ||
				subscriptReferences.get(pos) == null) {
			subscriptReferences.put(pos, new LinkedHashSet<String>());
//			System.out.println("ArrayDefinition: Needed to expand "+name+" added pos "+pos+" for subscript "+subscript);
		}
		subscriptReferences.get(pos).add(subscript);
	}

	public Set<String> getSubscriptReferencesForPos(int pos) {
		return subscriptReferences.get(pos);
	}

	public int getNumDimensions() {
		return subscriptReferences.size();
	}

	public boolean usesSubscript(String subscript) {
		boolean uses = false;
		for (Set<String> subs : subscriptReferences.values()) {
			if (subs.contains(subscript))
				return true;
		}

		return uses;
	}
}
