/**
 * 
 */
package gov.anl.gss.sdm.server.model.support;


//import gov.anl.rssd.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

/**
 * @author bragen
 *
 */
public class ArrayReference {
	
static Logger log = Logger.getLogger(ArrayReference.class.getSimpleName());

	private String arrayName;
	private List<String> subscripts;
	private String originalReference;


	public String getOriginalReference() {
		return originalReference;
	}
	
	public String toString() {
		return "<"+arrayName+"><"+subscripts+"><"+originalReference+">";
	}



	public ArrayReference(String reference) {
		originalReference = reference;
		arrayName = extractArrayName(reference);
		subscripts = extractSubscripts(reference);
	}
	
	

	public String getVensimReference() {
		StringBuilder sb = new StringBuilder();

		sb.append(arrayName);
		sb.append("[");
		int i = 0;
		for (String s : subscripts) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}
		sb.append("]");
		return sb.toString();
	}
	
	public String getVensimSubscriptReference() {
		StringBuilder sb = new StringBuilder();

		sb.append("[");
		int i = 0;
		for (String s : subscripts) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}
		sb.append("]");
		return sb.toString();
	}
	
	   public static List<String> expand(String s) {
	    	List<String> al = new ArrayList<String>();
	    	if (!ArrayReference.isArrayReference(s)) {
	    		al.add(s);
	    		return al;
	    		
	    	}
	    	
	    	ArrayReference ar = new ArrayReference(s.replace("!", ""));
	    	for (SubscriptCombination outerSub : InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(ar.getSubscriptsAsArray())) {
				// Build an array reference based on the subscript value
				String expandedLHS = "array."+ar.getArrayName()+ "[" + outerSub.getSubscriptValue() + "]";
				al.add(expandedLHS);
	    	}
	    	return al;
	    }

	/* No References */
	public String generateLHSHeader(String valueVariable) {

		StringBuilder code = new StringBuilder();
		//	if (!Translator.useNativeDataTypes) {
		// X[ns1,ns2, external]
		// generates
		//for (SubscriptCombination outerSub : getSubscriptValueCombinations("ns1", "ns2", "external")) {
		code.append("for (SubscriptCombination outerSub : getSubscriptValueCombinations(");
		int s = 0;
		for (String sub : subscripts) {
			if (s++ > 0)
				code.append(",");
			code.append(Parser.quoted(sub)); // "\""+sub+"\"");
			//	    }
		}
		code.append(")) {\n");

		//	} else {
		//	   
		//	    int subNum = 0;
		//	    for (String sub : subscripts) {
		//		code.append("for (int outer"+subNum+" = 0; outer"+subNum+" < "+NamedSubscriptManager.getNumIndexFor(sub)+"; outer"+subNum+"++) {\n");
		//		subNum++;
		//	    }
		//	}
		return code.toString(); 

	}

	public boolean hasRangeSubscript() {

		for (String sub : subscripts)
			if (isRangeSubscript(sub))
				return true;
		return false;
	}

	/* No References */
	public String generateLHSFooter(String valueVariable) {
		StringBuilder code = new StringBuilder();

		//	if (!Translator.useNativeDataTypes) {
		code.append("arraySetValue(\""+arrayName+"\", outerSub.getSubscriptValue(), "+valueVariable+");\n");
		code.append("}\n");
		//		} else {
		//		    code .append(NativeDataTypeManager.getLegalName(arrayName));
		//		 
		//		    for (int subNum = 0; subNum < subscripts.size(); subNum++) {
		//			code.append("[outer"+subNum+"]");
		//		    }
		//		    code.append(" = "+valueVariable+";\n");
		//		    for (String sub : subscripts) {
		//			code.append("}\n");
		//		    }
		//		}

		return code.toString();
	}

	/* No References */
	public String generateRHSImplementation() {
		StringBuilder code = new StringBuilder();
		// represents pop[race,state!]
		// generates arrayValueOf("pop",concatAsSubscript(outerSub.getSubscriptValue("race"), rangeSub.getSubscriptValue("state")))
		//	if (!Translator.useNativeDataTypes) {
		code.append("arrayValueOf(\""+arrayName+"\",concatAsSubscript(");
		int subNum = 0;
		for (String sub : subscripts) {
			if (subNum++ > 0)
				code.append(",");
			if (isRangeSubscript(sub)) {
				code.append("rangeSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
			} else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript
				code.append("outerSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
			} else {
				code.append(Parser.quoted(sub.replace("!", "")));
			}

		}

		code.append("))");
		//	} else {
		//	    code.append(NativeDataTypeManager.getLegalName(arrayName));
		//		int subNum = 0;
		//		for (String sub : subscripts) {
		//		    if (isRangeSubscript(sub)) {
		//			code.append("rangeSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
		//		    } else if (NamedSubscriptManager.isNamedSubscript(sub)) { // if not a named subscript
		//			code.append("outerSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
		//		    } else {
		//			code.append("\""+sub.replace("!", "")+"\"");
		//		    }
		//		   
		//		}
		//		
		//		code.append("))");
		//	}
		return code.toString();
	}

	/* No References */
	public String generateRHSName() {
		StringBuilder code = new StringBuilder();
		// represents pop[race,state!]
		// generates arrayValueOf("pop",concatAsSubscript(outerSub.getSubscriptValue("race"), rangeSub.getSubscriptValue("state")))

		//	code.append("\""+arrayName+"[\"+concatAsSubscript{AR.generateRHSName}(");
		code.append("\""+arrayName+"[\"+concatAsSubscript(");
		int subNum = 0;
		for (String sub : subscripts) {
			if (subNum++ > 0)
				code.append(",");
			if (isRangeSubscript(sub)) {
				code.append("rangeSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
			} else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript
				code.append("outerSub.getSubscriptValue(\""+sub.replace("!", "")+"\")");
			} else {
				code.append(Parser.quoted(sub.replace("!", "")));
			}

		}

		code.append("+\"]\")");
		return code.toString();
	}
	
	public static String deannotate(String ar) {
		return ar.replace("array.", "").replace("lookup.", "");
	}
	public static String annotate(String ar) {
		if (!isArrayReference(ar)) {
			return ar;
		}
		if (ar.startsWith("array."))
			return ar;
		return "array." + ar;
	}
	
	public static Set<String> annotate(Set<String> ar) {
		Set<String> al = new LinkedHashSet<String>();
		for (String a : ar) {
			al.add(annotate(a));
		}
		return al;
	}
	
	public static boolean containsRangeSubscript(ArrayReference ar) {
		for (String s : ar.getSubscripts()) {
			if (isRangeSubscript(s))
				return true;
		}
		return false;
	}
	
	public static String getReferenceWithoutRangeSubscript(ArrayReference ar) {
		return asCompleteArrayReference(ar.getOriginalReference());
	}
	
	public static String asCompleteArrayReference(String token) {
		ArrayReference tar = new ArrayReference(token);
		NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();
		List<String> newSubs = new ArrayList<String>();
		for (String subr : tar.getSubscripts()) {
			String s = subr.replace("!", "");
			if (nsm.isNamedSubscript(s)) {
				newSubs.add(s);
			} else {
				newSubs.add(nsm.getNamedSubscriptWithElement(s));
			}
		}
		tar.setSubscripts(newSubs);
		return tar.getArrayName() + "["+tar.getSubscript()+"]";
	}
	
	public static boolean isPartialArrayReference(String token) {
		if (!ArrayReference.isArrayReference(token)) {
			return false;
		}
		
		ArrayReference tar = new ArrayReference(token);
		NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();
		
		for (String subr : tar.getSubscripts()) {
			String s = subr.replace("!", "");
			if (!nsm.isNamedSubscript(s)) {
				return true;
			}
		}
		return false;
	}

	public static boolean isArrayReference(String token) {
		
		if (Parser.isQuotedString(token)) {
			return false;
		}
		// treat this as a literal, what about actual
		// variables have " in them?
		if ((token.startsWith("\"array.") && token.endsWith("\"")) || (token.startsWith("stringConcat(\"memory.") && token.endsWith("\")")))
			return false;
		if ((token.startsWith("array.") || token.startsWith("lookup.")  ) && token.contains("[") && token.contains("]") ||
				token.contains("[") && token.contains("]"))	
			return true;
		else
			return false;
	}

	public static boolean isRangeSubscript(String subscript) {
		if (subscript.endsWith("!"))
			return true;
		else
			return false;
	}

	public static String extractArrayName(String reference) {
		return reference.split("\\[")[0].replace("array.","").replace("lookup.","").trim();
	}

	
	   public String getOriginalSubscript() {
	    	return "["+getSubscript()+"]";
	    }
	   
	public List<String> extractSubscripts(String reference) {
		List<String> subscripts = new ArrayList<String>();
		
		String[] verify = reference.split("\\[");
		if (verify.length < 2) {
//			System.out.println("ArrayReference: no subscripts for extractSubscripts (Might be ok): "+reference);
			return subscripts;
		}

		for (String sub : reference.split("\\[")[1].split("]")[0].split(","))
			subscripts.add(sub.trim());
		return subscripts;
	}

	/* No References */
	public String[] getNonRangeSubscriptsAsArray() {
		List<String> al = getNonRangeSubscripts();
		return al.toArray(new String[al.size()]);
	}

	public List<String> getNonRangeSubscripts() {
		List<String> ranges = new ArrayList<String>();
		for (String sub : subscripts)
			if (!sub.contains("!"))
				ranges.add(sub);

		return ranges;
	}

	public String[] getRangeSubscriptsAsArray() {
		List<String> al = getRangeSubscripts();
		return al.toArray(new String[al.size()]);
	}

	public List<String> getRangeSubscripts() {
		List<String> ranges = new ArrayList<String>();
		for (String sub : subscripts)
			if (sub.contains("!"))
				ranges.add(sub);

		return ranges;
	}

	public List<String> getRangeSubscriptsWithDimension() {
		List<String> ranges = new ArrayList<String>();
		int dim = 0;
		for (String sub : subscripts)
			if (sub.contains("!"))
				ranges.add(sub.replace("!","")+"###"+dim);
		dim++;

		return ranges;
	}

	/* No References */
	public String[] getRangeSubscriptsNamesAsArray() {
		List<String> al = getRangeSubscriptsNames();
		return al.toArray(new String[al.size()]);
	}

	public List<String> getRangeSubscriptsNames() {
		List<String> ranges = new ArrayList<String>();
		for (String sub : subscripts)
			if (sub.contains("!"))
				ranges.add(sub.replace("[", "").replace("]", "").replace("!", ""));

		return ranges;
	}

	public List<String> getUniqueRangeSubscriptsNames() {
		List<String> ranges = new ArrayList<String>();
		for (String sub : subscripts)
			if (sub.contains("!")) {
				String newSub = sub.replace("[", "").replace("]", "").replace("!", "");
				if (!ranges.contains(newSub))
					ranges.add(newSub);
			}

		return ranges;
	}

	public String getSubscriptsAsMethodParameters() {
		StringBuilder sb = new StringBuilder();
		for (String sub : subscripts) {
			if (sb.length() > 0)
				sb.append(",");
			sb.append(Parser.quoted(sub)); // "\""+sub+"\"");
		}
		return sb.toString();
	}

	public String getArrayName() {
		return arrayName;
	}

	public void setArrayName(String arrayName) {
		this.arrayName = arrayName;
	}

	public List<String> getSubscripts() {
		return subscripts;
	}

	public String[] getSubscriptsAsArray() {
		String[] s = new String[subscripts.size()];
		int i = 0;
		for (String sub : subscripts)
			s[i++] = sub;
		return s;
	}

	public void setSubscripts(List<String> subscripts) {
		this.subscripts = subscripts;
	}

	//    public Set<String> expand() {
		//	Set<String> al = new LinkedHashSet<String>();
	//	
	//	for (SubscriptCombination sc : ArrayManager.getSubscriptValueCombinations(this.getSubscriptsAsArray())) {
	////	    al.add(arrayName+ "["+sc.getSubscriptValue()+"]");
	//	    
	//	    al.add("array."+arrayName+ "["+sc.getSubscriptValue()+"]");
	//	}
	//	
	//	return al;
	//    }
	//    
	//    public Set<String> expandRawSubscript() {
	//	Set<String> al = new LinkedHashSet<String>();
	//	
	//	for (SubscriptCombination sc : ArrayManager.getSubscriptValueCombinations(this.getSubscriptsAsArray())) {
	////	    al.add(arrayName+ "["+sc.getSubscriptValue()+"]");
	//	    
	//	    al.add("["+sc.getSubscriptValue()+"]");
	//	}
	//	
	//	return al;
	//    }

	public String getSubscript() {
		StringBuilder sb = new StringBuilder();
		int i = 0;
		for (String s : subscripts) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}


		return sb.toString();
	}
	
	public String getWithQuotedArrayName() {
		StringBuilder sb = new StringBuilder();
		sb.append(Parser.quoted(arrayName)); // "\""+arrayName+"\"");
		sb.append("[");
		int i = 0;
		for (String s : subscripts) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}
		sb.append("]");
		return sb.toString();
	}
	
	public String getFullyQuoted() {
		StringBuilder sb = new StringBuilder();
		sb.append(Parser.quoted(arrayName)); // "\""+arrayName+"\"");
		sb.append("[");
		int i = 0;
		for (String s : subscripts) {
			if (i++ > 0)
				sb.append(",");
			sb.append(Parser.quoted(s)); // "\""+s+"\"");
		}
		sb.append("]");
		return sb.toString();
	}
}
