package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.docx.reader.DocxReader;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;

import org.docx4j.jaxb.Context;
import org.docx4j.openpackaging.exceptions.Docx4JException;
import org.docx4j.openpackaging.exceptions.InvalidFormatException;
import org.docx4j.openpackaging.packages.WordprocessingMLPackage;
import org.docx4j.wml.BooleanDefaultTrue;
import org.docx4j.wml.CTBorder;
import org.docx4j.wml.HpsMeasure;
import org.docx4j.wml.ObjectFactory;
import org.docx4j.wml.P;
import org.docx4j.wml.R;
import org.docx4j.wml.RPr;
import org.docx4j.wml.STBorder;
import org.docx4j.wml.Tbl;
import org.docx4j.wml.TblBorders;
import org.docx4j.wml.TblPr;
import org.docx4j.wml.TblWidth;
import org.docx4j.wml.Tc;
import org.docx4j.wml.TcPr;
import org.docx4j.wml.Text;
import org.docx4j.wml.Tr;

public class CommentProcessor {
	
	/**
	 * Thanks to: http://blog.iprofs.nl/2012/09/06/creating-word-documents-with-docx4j/
	 */
	
	/*
	 * Tool writes/reads a docx file with variable names and comments
	 */
	
	private String inStringFile;
	private String inDocxFile;
	private String outDocxFile;
	
	ObjectFactory factory;
	WordprocessingMLPackage wordMLPackage = null;
	
	
	private List<String> variableComments;
	
	public CommentProcessor() {
		factory = Context.getWmlObjectFactory();
	}
	
	public CommentProcessor(List<String> variableComments, String outDocxFile) {
		this();
		this.variableComments = variableComments;
		this.outDocxFile = outDocxFile;
	}
	
	public CommentProcessor(String inDocxFile) {
		this();
		this.inDocxFile = inDocxFile;
	}
	
	
	public CommentProcessor(String inStringFile, String inDocxFile, String outDocxFile) {
		this();
		this.inStringFile = inStringFile;
		this.inDocxFile = inDocxFile;
		this.outDocxFile = outDocxFile;
	}
	
	
    public static void main(String[] args) {
    	CommentProcessor cp = new CommentProcessor(args[0], args[1], args[2]);
    	cp.docxToString();
//    	cp.processStringToDocx();
    }
    
    public List<String> docxToString() {
    	List<String> text = loadDocx();
    	
    	text = squeezeBlanks(text);
    	return text;
   	
    }
    
    public List<String> squeezeBlanks(List<String> text) {
    	List<String> textSqueezed = new ArrayList<String>();
    	for (String s : text) {
    		String y = new String(s);
    		while (y.contains("\t")) {
    			y = y.replace("\t", " ");
    		}
    		while (y.contains("  ")) {
    			y = y.replace("  ", " ");
    		}
//    		y = y.replace("\t", " ");
    		textSqueezed.add(y);
    	}
    	return textSqueezed;
    }
    
    public void stringToDocx() {
    	writeDocx(processTextForOutput(variableComments));
    }
    
    private List<String> processTextForOutput(List<String> text) {
    	List<String> processedText = new ArrayList<String>();
    	
    	StringBuilder sb = new StringBuilder();
    	
    	int i = 0;
    	
    	while (i < text.size()) {
    		String line = text.get(i).replace(">>>", "");
    		
    		// beginning of a new varaible/comment combo
    		// grab tag and variable name
    		if (line.startsWith("_Variable_")) {
    			
    			processedText.add(line);
    			i++;
    			processedText.add(text.get(i++).replace(">>>", ""));
    			
    			// start of a comment
    			// tag, concat all text, tag
    		} else if (line.startsWith("_CommentStart_")) {
    			processedText.add(text.get(i++));
    			sb.setLength(0);
    			int cLine = 0;
    			line = text.get(i++).replace(">>>", "");
    			while(!line.equals("_CommentEnd_")) {
    				if (cLine++ > 0)
    					sb.append(" ");
    				sb.append(line);
    				line = text.get(i++).replace(">>>", "");
    			}
    			processedText.add(sb.toString());
    			processedText.add(line);
    		} else {
    			
    		}
    	}
    	
    	
    	return processedText;
    }
    
    private void writeDocx(List<String> text) {
    	
    	
    	
		try {
			wordMLPackage = WordprocessingMLPackage.createPackage();
		} catch (InvalidFormatException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	
    	
    	for (String t : text) {
    		wordMLPackage.getMainDocumentPart().addParagraphOfText(t);
    	}
    	
    	try {
			wordMLPackage.save(new java.io.File(outDocxFile));
		} catch (Docx4JException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	
    }
    
    public void createTable(List<String> col1, List<String> col2) {

    	try {
    		wordMLPackage = WordprocessingMLPackage.createPackage();
    	} catch (InvalidFormatException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}

    	Tbl t = factory.createTbl();

    	addBorders(t);

    	Tr tableRow = factory.createTr();
    	addStyledTableCell(tableRow,  "Key", true, "18", 300);
    	addStyledTableCell(tableRow,  "Translation", true, "18", 300);
    	t.getContent().add(tableRow);
    	for (int i = 0; i < col1.size(); i++) {
    		tableRow = factory.createTr();

    		addStyledTableCell(tableRow,  col1.get(i), false, "11", 300);
    		addStyledTableCell(tableRow,  col2.get(i), false, "11", 300);

    		t.getContent().add(tableRow);
    	}


    	wordMLPackage.getMainDocumentPart().addObject(t);

    	try {
    		wordMLPackage.save(new java.io.File("TestTable.docx"));
    	} catch (Docx4JException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}
    }
    
    
    /**
     *  In this method we create a table cell,add the styling and add the cell to
     *  the table row.
     */
    private  void addStyledTableCell(Tr tableRow, String content,
                        boolean bold, String fontSize, int width) {
        Tc tableCell = factory.createTc();
        addStyling(tableCell, content, bold, fontSize);
        tableRow.getContent().add(tableCell);
        
       
//        tableCell.getContent().add(
//            wordMLPackage.getMainDocumentPart().createParagraphOfText(
//                content));
 
        if (width > 0) {
            setCellWidth(tableCell, width);
        }
        
        
        
    }
 
    /**
     *  This is where we add the actual styling information. In order to do this
     *  we first create a paragraph. Then we create a text with the content of
     *  the cell as the value. Thirdly, we create a so-called run, which is a
     *  container for one or more pieces of text having the same set of
     *  properties, and add the text to it. We then add the run to the content
     *  of the paragraph.
     *  So far what we've done still doesn't add any styling. To accomplish that,
     *  we'll create run properties and add the styling to it. These run
     *  properties are then added to the run. Finally the paragraph is added
     *  to the content of the table cell.
     */
    private  void addStyling(Tc tableCell, String content,
                    boolean bold, String fontSize) {
        P paragraph = factory.createP();
 
        Text text = factory.createText();
        text.setValue(content);
 
        R run = factory.createR();
        run.getContent().add(text);
 
        paragraph.getContent().add(run);
 
        RPr runProperties = factory.createRPr();
        if (bold) {
            addBoldStyle(runProperties);
        }
 
        if (fontSize != null && !fontSize.isEmpty()) {
            setFontSize(runProperties, fontSize);
        }
 
        run.setRPr(runProperties);
 
        tableCell.getContent().add(paragraph);
    }
 
    /**
     *  In this method we're going to add the font size information to the run
     *  properties. First we'll create a half-point measurement. Then we'll
     *  set the fontSize as the value of this measurement. Finally we'll set
     *  the non-complex and complex script font sizes, sz and szCs respectively.
     */
    private  void setFontSize(RPr runProperties, String fontSize) {
        HpsMeasure size = new HpsMeasure();
        size.setVal(new BigInteger(fontSize));
        runProperties.setSz(size);
        runProperties.setSzCs(size);
    }
 
    /**
     *  In this method we'll add the bold property to the run properties.
     *  BooleanDefaultTrue is the Docx4j object for the b property.
     *  Technically we wouldn't have to set the value to true, as this is
     *  the default.
     */
    private  void addBoldStyle(RPr runProperties) {
        BooleanDefaultTrue b = new BooleanDefaultTrue();
        b.setVal(true);
        runProperties.setB(b);
    }
    
	private  void addBorders(Tbl table) {
        table.setTblPr(new TblPr());
        CTBorder border = new CTBorder();
        border.setColor("auto");
        border.setSz(new BigInteger("4"));
        border.setSpace(new BigInteger("0"));
        border.setVal(STBorder.SINGLE);
 
        TblBorders borders = new TblBorders();
        borders.setBottom(border);
        borders.setLeft(border);
        borders.setRight(border);
        borders.setTop(border);
        borders.setInsideH(border);
        borders.setInsideV(border);
        table.getTblPr().setTblBorders(borders);
    }
	
	 /**
     *  In this method we create a cell and add the given content to it.
     *  If the given width is greater than 0, we set the width on the cell.
     *  Finally, we add the cell to the row.
     */
    private  void addTableCellWithWidth(Tr row, String content, int width){
        Tc tableCell = factory.createTc();
        tableCell.getContent().add(
            wordMLPackage.getMainDocumentPart().createParagraphOfText(
                content));
 
        if (width > 0) {
            setCellWidth(tableCell, width);
        }
        row.getContent().add(tableCell);
    }
 
    /**
     *  In this method we create a table cell properties object and a table width
     *  object. We set the given width on the width object and then add it to
     *  the properties object. Finally we set the properties on the table cell.
     */
    private  void setCellWidth(Tc tableCell, int width) {
        TcPr tableCellProperties = new TcPr();
        TblWidth tableWidth = new TblWidth();
        tableWidth.setW(BigInteger.valueOf(width));
        tableCellProperties.setTcW(tableWidth);
        tableCell.setTcPr(tableCellProperties);
    }
    
    private  void addTableCell(Tr tableRow, String content) {
        Tc tableCell = factory.createTc();
        tableCell.getContent().add(
            wordMLPackage.getMainDocumentPart().createParagraphOfText(content));
        tableRow.getContent().add(tableCell);
        }
   
    
    private void print(List<String> text) {
    	for (String t : text)
    		System.out.println(t);
    	
    }
    
    private List<String> loadDocx() {		
		DocxReader reader = new DocxReader(inDocxFile);
		return reader.getDocText();
	}


	public String getInStringFile() {
		return inStringFile;
	}


	public void setInStringFile(String inStringFile) {
		this.inStringFile = inStringFile;
	}


	public String getInDocxFile() {
		return inDocxFile;
	}


	public void setInDocxFile(String inDocxFile) {
		this.inDocxFile = inDocxFile;
	}


	public String getOutDocxFile() {
		return outDocxFile;
	}


	public void setOutDocxFile(String outDocxFile) {
		this.outDocxFile = outDocxFile;
	}


	public List<String> getVariableComments() {
		return variableComments;
	}


	public void setVariableComments(List<String> variableComments) {
		this.variableComments = variableComments;
	}

}
