package gov.anl.gss.sdm.server.model.support;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class EquationDependencyProcessor {
	
	public static List<VariableLevel> masterList = new ArrayList<VariableLevel>();
	
	class VariableLevel {
		
		public String lhs;
		public String variable;
		public int level;
		public VariableLevel next = null;;
		public VariableLevel child = null;
		
		public VariableLevel(String lhs, String variable, int level) {
			this.variable = variable;
			this.level = level;
			this.lhs = lhs;
			System.out.println("new VL: "+this.toString());
			EquationDependencyProcessor.masterList.add(this);
		}
		
		public String level() {
			return Integer.toString(level)+ "("+lhs+")";
		}
		public String toString() {
			return variable + " " + level + "("+lhs+")";
		}
	}
	
	// output/initialRequiresExpandedLhsExpandedRHS.txt
	
//	LHS: <array.Adjusted Temperature change from preindustrial[Deterministic]>
//		RHS <Temp Adjustment for SLR>
//		RHS <array.Temperature change from preindustrial[Deterministic]>

	private Map<String, Set<String>> dependencies = new LinkedHashMap<String, Set<String>>();
	private Set<String> processed = new LinkedHashSet<String>();

	public EquationDependencyProcessor(String file) {
		load(file);
	}
	
//	private Map<String, List<Double>> rssdValues;
//	private Map<String, List<Double>> vensimValues;
	
	public void showDependencyValues(String varToTrack, Map<String, List<Double>> rssdValues, 
			Map<String, List<Double>> vensimValues) {
		VariableLevel root = buildDependencyTree(varToTrack);
//		List<VariableLevel> al = toList(root);
		for (VariableLevel vl : masterList) {
			System.out.println(vl.toString());
		}
	}
	
	public List<VariableLevel> toList(VariableLevel root) {
		List<VariableLevel> al = new ArrayList<VariableLevel>();
		addToList(root, al);
		return al;
	}
	
	public void addToList(VariableLevel vl, List<VariableLevel> al) {
		if (vl == null)
			return;
		al.add(vl);
		addToList(vl.child, al);
		addToList(vl.next, al);
		while (vl.next != null) {
			vl = vl.next;
			addToList(vl, al);
		}
			
	}
	
	public VariableLevel buildDependencyTree(String varToTrack) {
		VariableLevel root = new VariableLevel("", varToTrack, 1);
		expand(root);
		return root;
	}

	public void expand(VariableLevel vl) {
		// add children
		VariableLevel prev = null;
		VariableLevel newVL = null;

		int proc = 0;
		if (processed.contains(vl.variable))
			return;
		processed.add(vl.variable);
		if (!dependencies.containsKey(vl.variable))
			return;
		for (String rhs : dependencies.get(vl.variable)) {
			if (proc++ == 0) {
				newVL = new VariableLevel(vl.variable, rhs, vl.level+1);
				vl.child = newVL;
				prev = newVL;
				expand(newVL);
			} else {
				newVL = new VariableLevel(vl.variable, rhs, vl.level+1);
				prev.next = newVL;
				expand(newVL);
			}
		}

		// add next
	}

	public void load(String file) {
		BufferedReader logReader = openForRead(file);
		load(logReader);
		try {
			logReader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void load(BufferedReader reader) {

		String aLine = null;
		
		String lhs = "";
		String rhs = "";

		try {
			aLine = reader.readLine();
			while (aLine != null) {
				if (aLine.startsWith("LHS:")) {
					lhs = aLine.split("<")[1].replace(">", "").replace("array.", "");
					dependencies.put(lhs, new LinkedHashSet<String>());
				} else {
					rhs = aLine.split("<")[1].replace(">", "").replace("array.", "");
					dependencies.get(lhs).add(rhs);
				}
				aLine = reader.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static BufferedReader openForRead(String filename) {

		BufferedReader fileReader = null;

		// open the file for reading
		try {
			fileReader = new BufferedReader (new FileReader(new File(filename)));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return fileReader;
	}

}
