package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.support.database.SDModelRunDatabase;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class ExtendedMemoryDatabase {

	private MemoryMapperInterface mapper;
	private String file;
	private String mapperFile;
	
	private double timeStep;
	
	
	private SDModelRunDatabase hsqldb;
	
	private Map<String, List<Double>> rssdValues = new LinkedHashMap<String, List<Double>>();
	
	public ExtendedMemoryDatabase(String mapperFile, String file) {
		this.mapper = new StreamlinedMemoryMapper(mapperFile, new MessageGobbler());
		this.file = file;
		this.mapperFile = mapperFile;
		initializeDBForRead();
	}

	
	public ExtendedMemoryDatabase(String mapperFile, String file, double timeStep) {
		this.mapper = new StreamlinedMemoryMapper(mapperFile, new MessageGobbler());
		this.file = file;
		this.mapperFile = mapperFile;
		
		this.timeStep = timeStep;
		initializeNewDB();
	}

	public ExtendedMemoryDatabase(MemoryMapperInterface mapper, String file, double timeStep) {
		this.mapper = mapper;
		this.file = file;
		this.mapperFile = mapper.getMapperFile();
		this.timeStep = timeStep;
		initializeNewDB();
	}
	
	public void setRunParameters(double initialTime, double finalTime, double dt, double savper) {
		hsqldb.setRunParameters(initialTime, finalTime, dt, savper);
	}
	
	public void shutdownDatabase() {
		hsqldb.shutdown();
	}
	
	public void reinitialize() {
		hsqldb.reinitialize();
	}
	
	public void writeTabFileVarRow(String file) {
		BufferedWriter bw = Utilities.openFileForWriting(file);

		List<String> sortedKeys = new ArrayList<String>();
		sortedKeys.add("Time");
		sortedKeys.addAll(mapper.getVariableMemoryOrder());

		try {
			for (String key : sortedKeys) {
				bw.append(key);
				bw.append("\t");
				bw.append(mapper.getDataStoreAndOffset(key));

				int numVals = rssdValues.get(sortedKeys.get(0)).size();
				for (int r = 0 ; r< numVals; r++) {

					bw.append("\t");
					bw.append(Double.toString(rssdValues.get(key).get(r)));

					
				}
				bw.append("\n");
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeTabFile(String file) {
		BufferedWriter bw = Utilities.openFileForWriting(file);

		List<String> sortedKeys = new ArrayList<String>();
		sortedKeys.add("Time");
		sortedKeys.addAll(mapper.getVariableMemoryOrder());
		
		try {
			int i = 0;
			for (String key : sortedKeys) {
				if (i++ > 0)
					bw.append("\t");
				bw.append(mapper.getDataStoreAndOffset(key));
			}
			bw.append("\n");
			 i = 0;
			for (String key : sortedKeys) {
				if (i++ > 0)
					bw.append("\t");
				bw.append(key);
			}
			bw.append("\n");

			int numVals = rssdValues.get(sortedKeys.get(0)).size();
			for (int r = 0 ; r< numVals; r++) {
				i = 0;
				for (String key : sortedKeys) {
					if (i++ > 0)
						bw.append("\t");
					bw.append(Double.toString(rssdValues.get(key).get(r)));
				}
				bw.append("\n");
			}


			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeVMem(double time, double[] vMem) {
		hsqldb.writeVMem(time, vMem);
	}
	
	public void writeLookupValues(String var, double[][] values) {
		hsqldb.writeLookupValues(var, values);
	}
	
	public void writeCMem(double time, double[] cMem) {
		hsqldb.writeCMem(time, cMem);
	}
	
	public void initializeNewDB() {
		hsqldb = new SDModelRunDatabase(file);
		hsqldb.initializeNewRunDatabase(mapperFile);
		hsqldb.load();
	}
	
	public void initializeDBForRead() {
		hsqldb = new SDModelRunDatabase(file);
		hsqldb.load();
		
	}
	
	public void backstepToTime(double time, double[] cMem, double vMem[]) {
		hsqldb.backstepToTime(time, cMem, vMem);
	}

	private void writeMapper() {
		String mapperFile = file.replace(".bat", ".map");
		
		try {
			BufferedWriter bw = Utilities.openFileForWriting(mapperFile);
			for (String rec : mapper.getMapperRecords()) {
				bw.append(rec+"\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void loadDBValues() {

		List<String> vars = mapper.getVariableMemoryOrder();
		for (String var : vars) {
			List<Double> al = new ArrayList<Double>();
			rssdValues.put(var, al);
			
			for (double v : getVarTimeVector(var)) {
				al.add(v);
			}
		}		
	}
	
	public void loadRunParameters() {
		hsqldb.loadRunParameters();
	}
	
	public double[] getTimeVector() {
		return hsqldb.getTimeVector();
	}
	
	public double[] getVarTimeVectorByIndex(String type, int index) {
		return hsqldb.getVarTimeVectorByIndex(type, index+1);
	}
	
	public double[] getVarTimeVector(String var) {
		return hsqldb.getVarTimeVector(var);
	}
	
	public double[] getVarTimeVector(String var, double time) {
		return hsqldb.getVarTimeVector(var, time);
	}
	
	public double[] getVarsValue(String[] vars, double time) {
		return hsqldb.getVarsValue(vars, time);
	}
	
	public double getVarAtTime(String var, double time) {
		return hsqldb.getVarAtTime(var, time);
	}
	
	public void close() {
		shutdownDatabase();
	}

	public Map<String, List<Double>> getRssdValues() {
		return rssdValues;
	}

	public MemoryMapperInterface getMapper() {
		return mapper;
	}
}
