package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.ArrayManager;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Parser;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class MemoryMapper implements MemoryMapperInterface {

	public static final int MAX_ELEMENTS_FOR_RUNNING = 10000;
	public static final String CONSTANT = "C";
	public static final String DYNAMIC = "D";
	
	public static final String STOCK = "S";
	public static final String FLOW = "F";
	public static final String AUXILIARY = "A";
	public static final String LOOKUP = "L";
	
	private Map<String, MutableInteger> sizeByType = new LinkedHashMap<String, MutableInteger>();

	private Map<String, Equation> equations;
	
	private Map<String, String> eqnLHS = new LinkedHashMap<String, String>();
	
	private Map<String, String> varType = new LinkedHashMap<String, String>();
	private Map<String, String> eqnType = new LinkedHashMap<String, String>();
	
	private List<String> sortedLHS;
	private List<String> expandedSortedLHS = new ArrayList<String>();

	private List<String> evaluationOrderOneTime = new ArrayList<String>();
	private List<String> evaluationOrderRepeated = new ArrayList<String>();

	public List<String> getExpandedSortedLHS() {
		if (expandedSortedLHS == null)
			generateExpandedSortedLHS();
		return expandedSortedLHS;
	}

	// these need to be mapped for Constant and Dynamic
	private Map<String, Map<String, Integer>> offset = new LinkedHashMap<String, Map<String, Integer>>();
	private Map<String, Map<Integer, String>> reverseIndex = new LinkedHashMap<String, Map<Integer, String>>();
	private Map<String, Map<String, Integer>> numElements = new LinkedHashMap<String, Map<String, Integer>>();
	
	private Map<String, MutableInteger> currentOffset = new LinkedHashMap<String, MutableInteger>();
	private Map<String, MutableInteger> currentNumElements = new LinkedHashMap<String, MutableInteger>();
	private Map<String, Map<String, Integer>> expandedIndex = new LinkedHashMap<String, Map<String, Integer>>();
	
	private String mapperFile;
	private MessageProcessor info;

	NativeDataTypeManager ndtm = InformationManagers.getInstance()
			.getNativeDataTypeManager();
	NamedSubscriptManager nsm = InformationManagers.getInstance()
			.getNamedSubscriptManager();
	
	public MemoryMapper(MessageProcessor info) {
	// deal with multiple referenced to same array
		this.info = info;
		
		sizeByType.put(CONSTANT, new MutableInteger(0));
		sizeByType.put(DYNAMIC, new MutableInteger(0));
		
		currentOffset.put(CONSTANT, new MutableInteger(0));
		currentOffset.put(DYNAMIC, new MutableInteger(0));
		
		currentNumElements.put(CONSTANT, new MutableInteger(0));
		currentNumElements.put(DYNAMIC, new MutableInteger(0));
		
		offset.put(CONSTANT, new LinkedHashMap<String, Integer>());
		offset.put(DYNAMIC, new LinkedHashMap<String, Integer>());
		
		reverseIndex.put(CONSTANT, new LinkedHashMap<Integer, String>());
		reverseIndex.put(DYNAMIC, new LinkedHashMap<Integer, String>());
		
		numElements.put(CONSTANT, new LinkedHashMap<String, Integer>());
		numElements.put(DYNAMIC, new LinkedHashMap<String, Integer>());
		
		expandedIndex.put(CONSTANT, new LinkedHashMap<String, Integer>());
		expandedIndex.put(DYNAMIC, new LinkedHashMap<String, Integer>());
	}

	public MemoryMapper(String mapperFile, MessageProcessor info) {
		this(info);
		this.mapperFile = mapperFile;
		loadMapper();
	}

	public MemoryMapper(Map<String, Equation> equations, String mapperFile, MessageProcessor info) {
		this(info);
	

		Set<String> processed = new LinkedHashSet<String>();

		this.equations = equations;
		sortedLHS = new ArrayList<String>();
		int numEqn = equations.size();
		int interval = Math.max(numEqn / 10, 1);
		int num = 1;
		int pct = 10;
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String s : equations.keySet()) {
			
//			if (Parser.isSystemVariable(s)) {
//				System.out.println("SYSTEM VAR");
//			}
			
			
			Equation eqn = equations.get(s);
			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				pct += 10;
			}
			num++;
			
			if (eqn.isDefinesSubscript())
				continue;
			if (eqn.isDefinesLookup())
				continue;
			if (eqn.isStock()) {
				eqnType.put(common(s), STOCK);
				eqnType.put(common(screenName(s)), STOCK);
			} else if (eqn.isFlow()) {
				eqnType.put(common(s), FLOW);
				eqnType.put(common(screenName(s)), FLOW);
			} else {
				eqnType.put(common(s), AUXILIARY);
				eqnType.put(common(screenName(s)), AUXILIARY);
			}

			if (ArrayReference.isArrayReference(s)) {
				ArrayReference ar = new ArrayReference(s);
				String arrayName = ar.getArrayName();
				if (processed.contains(arrayName))
					continue;
				processed.add(arrayName);
				varType.put(common(arrayName), eqn.isOneTime() ? CONSTANT : DYNAMIC);
				
			}
			eqnLHS.put(screenName(s), s);
			sortedLHS.add(screenName(s));
			
			String lhsAsInMdl = eqn.getLhsAsInMdl();
			String lhs = eqn.getLhs();
			String sn = screenName(s);
			
			if (screenName(s).equalsIgnoreCase("Time"))
				varType.put(common(screenName(s)), DYNAMIC);
			else			
				varType.put(common(screenName(s)), eqn.isOneTime() ? CONSTANT : DYNAMIC);
			
		}
		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
		
		this.mapperFile = mapperFile;
		sortedLHS = sortIgnoreCase(sortedLHS);
		info.addMessage(language.getText(StringConstants.DETERMINING_MEMORY_REQUIREMENTS));
		determineMemoryRequirements();
		info.addMessage(language.getText(StringConstants.EXPANDING_LHS_OF_EQUATIONS));
		if (StaticSwitches.LIMIT_LHS_EXPANSION) {
			
		} else {
			info.addMessage(language.getText(StringConstants.EXPANDING_LHS_OF_EQUATIONS));
			generateExpandedSortedLHS();
		}
	}
	
	public Map<String, Map<String, Integer>> getVariableStorageMap() {
		return numElements;
	}

	private void generateExpandedSortedLHS() {
		
		expandedSortedLHS = new ArrayList<String>();
		
		Map<String, MutableInteger> index = new LinkedHashMap<String, MutableInteger>();
		index.put(CONSTANT, new MutableInteger(0));
		index.put(DYNAMIC, new MutableInteger(0));
		int numEqn = equations.size();
		int interval = Math.max(numEqn / 10, 1);
		int num = 1;
		int pct = 10;
		
		int count = 0;
		for (String lhs : sortedLHS) {
			String eLHS = eqnLHS.get(lhs);
			Equation eqn = equations.get(eLHS);
			if (!ArrayReference.isArrayReference(eLHS)) {
				count++;
			} else {
				ArrayReference ar = new ArrayReference(eLHS);
				count += 	 InformationManagers.getInstance()
						.getArrayManager()
						.countExpandRawSubscript(ar);
			}
		}
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		info.addMessage(language.getText(StringConstants.TOTAL_EXPANDED_COUNT_FOR)+" "+numEqn+" "+
				language.getText(StringConstants.EQUATIONS)+": "+count+"\n");
		
		for (String lhs : sortedLHS) {
			String eLHS = eqnLHS.get(lhs);
			Equation eqn = equations.get(eLHS);
			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				pct += 10;
			}
			num++;
			
			if (!ArrayReference.isArrayReference(eLHS)) {
				expandedSortedLHS.add(eLHS);
			} else {

				ArrayReference ar = new ArrayReference(eLHS);

				Set<String> al = InformationManagers.getInstance()
						.getArrayManager()
						.expandRawSubscript(ar);
				varType.put(common(ar.getArrayName()), getVarType(eLHS) );
				eqnType.put(common(ar.getArrayName()), getEqnType(eLHS) );
				
				
				for (String outerSub : al) {
					// Build an array reference based on the subscript value
//					String subr = "["
//							+ /* outerSub.getSubscriptValue() */outerSub + "]";
					expandedSortedLHS.add(ar.getArrayName() + outerSub);
					varType.put(common(ar.getArrayName() + outerSub), getVarType(eLHS)); 
					eqnType.put(common(ar.getArrayName() + outerSub), getEqnType(eLHS) );
				}
			}
		}
		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
		
		num = 1;
		pct = 10;
		
			
		numEqn = expandedSortedLHS.size();
		interval = Math.max(numEqn / 10,1);
		info.addMessage(language.getText(StringConstants.SAVING_EXPANDED_LHS_EQUATION_DEFINITION));
		for (String s : expandedSortedLHS) {
//			String vt = varType.get(common(s));
//			expandedIndex = new LinkedHashMap<String, Map<String, Integer>>();
//			Map<String, Integer> m = expandedIndex.get(varType.get(common(s)));
//			index = new LinkedHashMap<String, MutableInteger>();
//			MutableInteger m2 = index.get(getVarType(s));
			expandedIndex.get(varType.get(common(s))).put(s, index.get(getVarType(s)).value());
			index.get(getVarType(s)).add(1);
			
			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				
				pct += 10;
			}
			num++;
		}
		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
	}

	private List<String> generateExpandedLHS(String lhs) {
		List<String> al = new ArrayList<String>();
		if (!ArrayReference.isArrayReference(lhs)) {
			al.add(lhs);
		} else {
			ArrayReference ar = new ArrayReference(lhs);

			List<String> al2 = InformationManagers.getInstance()
					.getArrayManager().getAlphaSubscripts(ar.getArrayName(), 0);

			for (/* SubscriptCombination */String outerSub : al2 /*
																 * InformationManagers
																 * .
																 * getInstance()
																 * .
																 * getArrayManager
																 * ().
																 * getSubscriptValueCombinations
																 * (ar.
																 * getSubscriptsAsArray
																 * ())
																 */) {
				// Build an array reference based on the subscript value
				String subr = "[" + /* outerSub.getSubscriptValue() */outerSub
						+ "]";
				al.add(ar.getArrayName() + subr);
			}

			// ArrayReference ar = new ArrayReference(lhs);
			//
			// for (SubscriptCombination outerSub :
			// InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(ar.getSubscriptsAsArray()))
			// {
			// // Build an array reference based on the subscript value
			// String subr = "[" + outerSub.getSubscriptValue() + "]";
			// al.add(ar.getArrayName()+subr);
			// }
		}

		return al;

	}

//	public int getExpandedIndex(String s) {
//		return expandedIndex.get(s);
//	}

	public List<String> getVariables() {
		return expandedSortedLHS;
	}
	
	public List<String> getStocks() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : sortedLHS) {
			if (eqnType.get(common(s)).equals(STOCK))
				ordered.add(screenName(s));
		}
		
		return ordered;
	}
	
	@Override
	public List<String> getLookups() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : sortedLHS) {
			if (eqnType.get(common(s)).equals(LOOKUP))
				ordered.add(screenName(s));
		}
		
		return ordered;
	}

	
	public List<String> getVariablesOrderedByType() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(STOCK))
				ordered.add(s);
		}
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(FLOW))
				ordered.add(s);
		}
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(AUXILIARY))
				ordered.add(s);
		}
		
		return ordered;
	}

	private void loadMapper() {
		List<String> lines = Utilities.getAsList(mapperFile);
		String[] parts = Parser.splitQuoted(lines.get(0), "\t");
		currentOffset.put(CONSTANT, new MutableInteger(Integer.parseInt(parts[0])));
		currentNumElements.put(CONSTANT, new MutableInteger(Integer.parseInt(parts[1])));
		
		currentOffset.put(DYNAMIC, new MutableInteger(Integer.parseInt(parts[2])));
		currentNumElements.put(DYNAMIC, new MutableInteger(Integer.parseInt(parts[3])));
		
		int num = Integer.parseInt(lines.get(1));
		int i;
		int start;
		
		// variable types
		for (i = 2; i < 2 + num; i++) {
			parts = Parser.splitQuoted(lines.get(i), "\t");
			varType.put(common(parts[1]), parts[0]);
			eqnType.put(common(parts[1]), parts[2]);
		}

		start = i;
		int numExp = Integer.parseInt(lines.get(start++));
		
		// expanded LHS index
		for (i = start; i < start + numExp; i++) {
			parts = Parser.splitQuoted(lines.get(i), "\t");

			expandedIndex.get(parts[0]).put(parts[1], Integer.parseInt(parts[2]));
			expandedSortedLHS.add(parts[1]);
			numElements.get(getVarType(parts[1])).put(parts[1], 1);
		}

		start = i;
		int sLHS = Integer.parseInt(lines.get(start++));
		sortedLHS = new ArrayList<String>();
		for (i = start; i < start + sLHS; i++) {
			parts = Parser.splitQuoted(lines.get(i), "\t");

			sortedLHS.add(parts[0]);
			eqnLHS.put(parts[0], parts[1]);
			offset.get(getVarType(parts[0])).put(parts[0], Integer.parseInt(parts[2]));
			reverseIndex.get(getVarType(parts[0])).put(Integer.parseInt(parts[2]), parts[0]);
			numElements.get(getVarType(parts[0])).put(parts[0], Integer.parseInt(parts[3]));
			eqnType.put(common(parts[0]), parts[4]);
			eqnType.put(common(parts[1]), parts[4]);
			// n.add(s + "\t" + eqnLHS.get(s) + "\t" + offset.get(s) + "\t" +
			// numElements.get(s));
		}

		start = start + sLHS;
		int numOneTime = Integer.parseInt(lines.get(start++));
		for (i = start; i < start + numOneTime; i++) {
			evaluationOrderOneTime.add(lines.get(i));
		}

		start = start + numOneTime;
		int numRepeat = Integer.parseInt(lines.get(start++));
		for (i = start; i < start + numRepeat; i++) {
			evaluationOrderRepeated.add(lines.get(i));
		}

	}

	public List<String> getVariableMemoryOrder() {
		// List<Integer> keys = new ArrayList<Integer>(reverseIndex.keySet());
		// Collections.sort(keys);
		// List<String> ordered = new ArrayList<String>();
		// for (Integer i : keys) {
		// ordered.add(reverseIndex.get(i));
		// }

		return this.expandedSortedLHS;
	}

	public List<String> getVariableExecutionOrder() {
		// List<Integer> keys = new ArrayList<Integer>(reverseIndex.keySet());
		// Collections.sort(keys);
		List<String> ordered = new ArrayList<String>();
		// for (Integer i : keys) {
		// ordered.add(reverseIndex.get(i));
		// }

		ordered.addAll(evaluationOrderOneTime);
		ordered.addAll(evaluationOrderRepeated);
		return ordered;
	}

	public void writeMapper(boolean badEvalOrder) {

		try {
			BufferedWriter bw = Utilities.openFileForWriting(mapperFile);
			bw.append(currentOffset.get(CONSTANT).value() + "\t" + currentNumElements.get(CONSTANT).value() + "\t" 
					+ currentOffset.get(DYNAMIC).value() + "\t" + currentNumElements.get(DYNAMIC).value() + "\t"
					+ mapperFile + "\t"+sizeByType.get(CONSTANT).value()+"\t"+sizeByType.get(DYNAMIC) +
					"\n");
			
			bw.append(varType.size() + "\n");
			
			for (String s : varType.keySet()) {
				bw.append(getVarType(s)+ "\t" + s + "\t"+eqnType.get(common(s))+"\n");
			}
			
			if (!StaticSwitches.LIMIT_LHS_EXPANSION) {
				bw.append(expandedSortedLHS.size() + "\n");

				for (String s : expandedSortedLHS) {
					bw.append(getVarType(s)+ "\t" + s + "\t" + expandedIndex.get(getVarType(s)).get(s) + "\n");
				}

				bw.append(getMapperRecords().size() + "\n");

				for (String rec : getMapperRecords()) {
					bw.append(rec + "\n");
				}

				bw.append(evaluationOrderOneTime.size() + "\n");
				for (String s : evaluationOrderOneTime) {
					bw.append(s + "\n");
				}

				bw.append(evaluationOrderRepeated.size() + "\n");
				for (String s : evaluationOrderRepeated) {
					bw.append(s + "\n");
				}
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// test out hsqldb stuff
//		SDModelRunDatabase runDB = new SDModelRunDatabase("database/testHSQLDB");
//		runDB.initializeNewRunDatabase(mapperFile);
//		runDB.printVariables();
//		runDB.shutdown();
		
	}
	
	public List<List<String>> getVariableMapRecords() {
		List<List<String>> records = getVariableMapRecords(CONSTANT);
		records.addAll(getVariableMapRecords(DYNAMIC));
		return records;
	}
	
	public List<List<String>> getVariableMapRecords(String type) {
		List<List<String>> records = new ArrayList<List<String>>();
		for (String s : numElements.get(type).keySet()) {
			List<String> record = new ArrayList<String>();
			record.add(s);
			record.add(type);
			if (offset.get(getVarType(s)).get(s) == null) {
				record.add(Integer.toString(expandedIndex.get(getVarType(s)).get(s)));
			} else {
				record.add(Integer.toString(offset.get(getVarType(s)).get(s)));
			}
			record.add(Integer.toString(numElements.get(getVarType(s)).get(s)));
			records.add(record);
		}
		return records;
	}
	

	public List<String> getMapperRecords() {
		List<String> records = new ArrayList<String>();
		for (String s : sortedLHS) {
			records.add(s + "\t" + eqnLHS.get(s) + "\t" + offset.get(getVarType(s)).get(s) + "\t"
					+ numElements.get(getVarType(s)).get(s)+"\t"+eqnType.get(common(s)));
		}
		return records;
	}

	private List<String> sortIgnoreCase(List<String> list) {
		List<String> sorted = new ArrayList<String>();
		List<String> temp = new ArrayList<String>();
		Map<String, String> lowerToOrig = new LinkedHashMap<String, String>();
		for (String s : list) {
			lowerToOrig.put(s.toLowerCase(), s);
			temp.add(s.toLowerCase());
		}
		Collections.sort(temp);
		for (String s : temp) {
			sorted.add(lowerToOrig.get(s));
		}
		return sorted;
	}
	
	private String getType(Equation eqn) {
		if (eqn.isOneTime() && !eqn.getLhs().equals("Time"))
			return CONSTANT;
		else
			return DYNAMIC;
	}

	public void determineMemoryRequirements() {
		
		int numEqn = equations.size();
		int interval = Math.max(numEqn / 10, 1);
		int num = 1;
		int pct = 10;
			
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String lhs : sortedLHS) {
			
			Equation eqn = equations.get(lhs);
			

			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				
				pct += 10;
			}
			num++;
			
			if (eqn == null && ArrayReference.isArrayReference(lhs)) {
				eqn = equations.get("array." + lhs);
			}
			if (eqn.isDefinesSubscript())
				continue;
			
			String type = getType(eqn);
			
			String screenName = screenName(lhs);
			if (eqn.isDefinesLookup() || eqn.isDefinesLookupGetXls()) {

				int numCombo;
				// let's put both in
				offset.get(type).put(screenName, currentOffset.get(type).value());
				if (ArrayReference.isArrayReference(lhs)) {
					ArrayReference ar = new ArrayReference(lhs);
					offset.get(type).put(new ArrayReference(lhs).getArrayName(),
							currentOffset.get(type).value());
					numCombo = nsm
							.getNumSubscriptCombinations(new ArrayReference(lhs))
							* eqn.getVectorSpaceRequirements();
					ArrayManager am = InformationManagers.getInstance()
							.getArrayManager();
					numCombo = am
							.getAlphaSubscripts(ar.getArrayName(), 0).size();
					for (int dim = 1; dim < am.getNumDimensions(ar.getArrayName()); dim++) {
						numCombo *= am
								.getAlphaSubscripts(ar.getArrayName(), dim).size();
					}
				} else {
					numCombo = eqn.getVectorSpaceRequirements();
				}
				numElements.get(type).put(screenName, numCombo);
				currentOffset.get(type).add(numCombo);
				currentNumElements.get(type).add(numCombo);
				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(numCombo);
			} else if (ArrayReference.isArrayReference(lhs)) {
				ArrayReference ar = new ArrayReference(lhs);
				// let's put both in
				offset.get(type).put(screenName, currentOffset.get(type).value());
				offset.get(type).put(new ArrayReference(lhs).getArrayName(),
						currentOffset.get(type).value());
				int numCombo = nsm
						.getNumSubscriptCombinations(new ArrayReference(lhs))
						* eqn.getVectorSpaceRequirements();
				ArrayManager am = InformationManagers.getInstance()
						.getArrayManager();
				numCombo = am
						.getAlphaSubscripts(ar.getArrayName(), 0).size();
				for (int dim = 1; dim < am.getNumDimensions(ar.getArrayName()); dim++) {
					numCombo *= am
							.getAlphaSubscripts(ar.getArrayName(), dim).size();
				}
				numElements.get(type).put(screenName, numCombo);
				currentOffset.get(type).add(numCombo);
				currentNumElements.get(type).add(numCombo);
				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(numCombo);
			} else {
				// scalar
				offset.get(type).put(screenName, currentOffset.get(type).value());
				currentOffset.get(type).add(1);
				numElements.get(type).put(screenName, 1);
				currentNumElements.get(type).add(1);
				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(1);
			}
		}

		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
		
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();

		if (!options.isSelected(StringConstants.LIMIT_THUMBNAIL_COUNT) &&

		(currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value()) > 
			MemoryMapper.MAX_ELEMENTS_FOR_RUNNING) {
			InformationManagers
					.getInstance()
					.getModelManager()
					.setRunnable(
							false,
							"Too many elements for thumbnails "
									+ currentNumElements);
		}
	}

	private String screenName(String lhs) {
		return ndtm.getOriginalNameRemovingAnnotation(lhs).replace("!", "");
	}

	public String getOffset(String lhs) {
		String sn = (screenName(lhs));
		String type = getVarType(lhs);
		
			if (offset.get(type) == null) {
//				System.out.println("offset type missing for "+lhs+" "+sn+" varType = "+varType.get(common(sn)));
				return "-1";
			}
		if (offset.get(type).containsKey(sn)) {
			return offset.get(varType.get(common(sn))).get(sn).toString();
		}
		// is this an array reference with terminal subscripts?
		if (ArrayReference.isArrayReference(sn)) {
			String next = new ArrayReference(sn).getArrayName();
			if (offset.get(type).containsKey(next))
				return offset.get(type).get(new ArrayReference(sn).getArrayName())
						.toString();
		}
		System.out.println("Confused in Memory Map looking for " + lhs
				+ " screen name: " + sn);
		if (sn == null)
			return "0";
		else
			return "-9999";
	}
	
	public String getVarType(String lhs) {
		
		String type = null;
		String arg = screenName(ArrayReference.deannotate(new String(lhs))).replace("!", "");
		
		if (!varType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		if (!varType.containsKey(common(arg))) {

		} else {
			type = varType.get(common(arg));
		}
		
		return type;
	}
	
	public String getEqnType(String lhs) {
		
		String type = null;
		String arg = screenName(ArrayReference.deannotate(new String(lhs))).replace("!", "");
		
		if (!eqnType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		if (!eqnType.containsKey(common(arg))) {

		} else {
			type = eqnType.get(common(arg));
		}
		
		return type;
	}
	
	public String getDataStoreAndOffset(String lhs) {
		return getDataStore(lhs)+"["+getIndex(lhs)+"]";
	}
	
	public String getDataStore(String lhs) {
		
		String arg = screenName(ArrayReference.deannotate(new String(lhs)));
		if (!varType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}

		
		String type = varType.get(common(arg));
		
		if (type == null)
			return "cvMem";
		
		if (type.equals(CONSTANT))
			return "cMem";
		else
			return "vMem";
	}

	public int getIndex(String lhs) {
		
		String aName = common(screenName(lhs));
		String varT = varType.get(aName);
		if (varT == null) {
			System.out.println("varT NULL");
		}

		return expandedIndex.get(varT).get(lhs);
	}

	public String getNumElements(String lhs) {
		return numElements.get(screenName(lhs)).toString();
	}

	public String getSizeRequirement() {
		return Integer.toString(currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value());
	}
	
	public int getNumElementsForType(String type) {
		return currentNumElements.get(type).value();
	}

	public int getNumElements() {
		return currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value();
	}

	public void report() {
		System.out.println("+++ Memory Map +++");
		System.out.println("Size: " + getSizeRequirement());
		for (String s : sortedLHS) {
			System.out.println("lhs: " + s + " offset: " + offset.get(s)
					+ " elements: " + numElements.get(s) + " eqnLHS: "
					+ eqnLHS.get(s));
		}

		// for (String s : expandedSortedLHS) {
		String s = expandedSortedLHS.get(expandedSortedLHS.size() - 1);
		System.out
				.println("Final ExpIndex: " + s + ": " + expandedIndex.get(s));
		// }
	}

	public List<String> getEvaluationOrderOneTime() {
		return evaluationOrderOneTime;
	}

	public void setEvaluationOrderOneTime(List<String> evaluationOrderOneTime) {
		
		Set<String> processed = new LinkedHashSet<String>();
		for (String lhs : evaluationOrderOneTime) {
			
			if (ArrayReference.isArrayReference(lhs)) {
				ArrayReference ar = new ArrayReference(lhs);
				String arrayName = ar.getArrayName();
				if (processed.contains(arrayName))
					continue;
				processed.add(arrayName);
			}
			
			List<String> gg = generateExpandedLHS(lhs);
			
			
			this.evaluationOrderOneTime.addAll(gg);
		}
	}

	public List<String> getEvaluationOrderRepeated() {

		return evaluationOrderRepeated;
	}

	public void setEvaluationOrderRepeated(List<String> evaluationOrderRepeated) {
		
		Set<String> processed = new LinkedHashSet<String>();
		for (String lhs : evaluationOrderRepeated) {
			
			if (ArrayReference.isArrayReference(lhs)) {
				ArrayReference ar = new ArrayReference(lhs);
				String arrayName = ar.getArrayName();
				if (processed.contains(arrayName))
					continue;
				processed.add(arrayName);
			}
			this.evaluationOrderRepeated.addAll(generateExpandedLHS(lhs));
		}

	}

	public String getMapperFile() {
		return mapperFile;
	}

	public Map<String, MutableInteger> getSizeByType() {
		return sizeByType;
	}

	public Map<String, Equation> getEquations() {
		return equations;
	}

	public Map<String, String> getEqnLHS() {
		return eqnLHS;
	}

	public Map<String, String> getVarType() {
		return varType;
	}

	public List<String> getSortedLHS() {
		return sortedLHS;
	}

	public Map<String, Map<String, Integer>> getOffset() {
		return offset;
	}

	public Map<String, Map<Integer, String>> getReverseIndex() {
		return reverseIndex;
	}

	public Map<String, MutableInteger> getCurrentOffset() {
		return currentOffset;
	}

	public Map<String, MutableInteger> getCurrentNumElements() {
		return currentNumElements;
	}

	public Map<String, Map<String, Integer>> getExpandedIndex() {
		return expandedIndex;
	}

	public NativeDataTypeManager getNdtm() {
		return ndtm;
	}

	public NamedSubscriptManager getNsm() {
		return nsm;
	}
	
	public String common(String var) {
//		if (InformationManagers.getInstance().getVariableNamePatternGenerator().isPattern(var)) {
//			return var;
//		} else {
//			return InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(var);
//		}
		
		return var;
	}

	@Override
	public List<List<String>> getLookupMapRecords() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public List<List<String>> getLookupMapRecords(String type) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean isValidVariable(String var) {
		// TODO Auto-generated method stub
		return true;
	}


	/*
	 * List<String> evaluationOrderOneTime = new ArrayList<String>();
	 * List<String> evaluationOrderRepeated = new ArrayList<String>();
	 */

}
