package gov.anl.gss.sdm.server.model.support;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.log4j.Logger;

public class NameConverter {
static Logger log = Logger.getLogger(NameConverter.class.getSimpleName());
	private Map<String, String> rssdToVensim = new LinkedHashMap<String, String>();
	private Map<String, String> vensimToRssd = new LinkedHashMap<String, String>();
	
	public NameConverter(String file) {
		loadNames(file);
	}
	
	public void loadNames(String file) {
		BufferedReader logReader = openForRead(file);
		 loadNames(logReader);
	}
	
	public void loadNames(BufferedReader reader) {

		String aLine = null;

		try {
			aLine = reader.readLine();
			aLine = reader.readLine();
			while (aLine != null) {
				String[] values = aLine.split(",");
				if (values.length > 2) {
					for (int i = 2; i < values.length; i++) {
						values[1] += ", " + values[i];
					}
				} 
				rssdToVensim.put(values[1], values[0]);
				vensimToRssd.put(values[0], values[1]);

//				log.debug(values[0]+" <> "+values[1]);
				aLine = reader.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String getVensimName(String name) {
		if (!name.contains("[")) {
			return rssdToVensim.get(name);
		} else {
			String[] parts = name.split("\\[");
			if (rssdToVensim.get(parts[0]) == null)
				return null;
			else
				return rssdToVensim.get(parts[0]) + "["+parts[1];
		}
			
	}
	
	public String getRssdName(String name) {
		
		if (!name.contains("[")) {
			return vensimToRssd.get(name);
		} else {
			String[] parts = name.split("\\[");
			if (vensimToRssd.get(parts[0]) == null)
				return null;
			else
				return vensimToRssd.get(parts[0]) + "["+parts[1];
		}
	}
	
    public static BufferedReader openForRead(String filename) {
    	
    	BufferedReader fileReader = null;

    	// open the file for reading
    	try {
    	    fileReader = new BufferedReader (new FileReader(new File(filename)));
    	} catch (FileNotFoundException e) {
    	    // TODO Auto-generated catch block
    	    e.printStackTrace();
    	}
    	
    	return fileReader;
        }
}
