/**
 * 
 */
package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.translator.Node;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author bragen
 *
 */
public class NamedSubscriptManager {

	private  Map<String, NamedSubscriptDefinition> namedSubscripts;
	private  Map<String, String> namedSubscriptWithElementCache;
	private  Map<String, List<String>> valuesForCache;
	private  Map<String, Integer> numIndexForCache;
	private  Map<String, Integer> numSubscriptCombinationsCache;
	private  Map<String, String[]> expandedSubscriptsCache;
	
	private Map<String, Node> subscriptToNode;
	

	public NamedSubscriptManager() {
		namedSubscripts = new LinkedHashMap<String, NamedSubscriptDefinition>();
		namedSubscriptWithElementCache = new LinkedHashMap<String, String>();
		valuesForCache = new LinkedHashMap<String, List<String>>();
		numIndexForCache = new LinkedHashMap<String, Integer>();
		numSubscriptCombinationsCache = new LinkedHashMap<String, Integer>();
		expandedSubscriptsCache = new LinkedHashMap<String, String[]>();
		
		subscriptToNode = new LinkedHashMap<String,Node>();
	}
	
	public String getNamedSubscriptsForMemoryMapper() {
		StringBuilder sb = new StringBuilder();
		sb.append(namedSubscripts.size()+"\n");
		for (String key : Utilities.ordered(namedSubscripts.keySet())) {
			sb.append(key+":");
			NamedSubscriptDefinition nsd = namedSubscripts.get(key);
			for (String s : nsd.getValues()) {
				sb.append("\t"+s);
				
			}
			sb.append("\n");
		}
		
		return sb.toString();
	}
	
	public void addMappedSubscriptAsNamed(NamedSubscriptDefinition nsd) {
		if (!namedSubscripts.containsKey(nsd.getName())) {
			namedSubscripts.put(nsd.getName(), nsd);
		}
	}
	
	public Set<String> getDefiningSubscripts(String missing, Set<String> have) {
		
		// we return the elements of have that are needed to define missing
		// these values must be within have
		// if missing is a terminal subscript, we should return a single value
		// if it is a namedSubscript, then we may need to return multiple values to
		// define missing in its entirety
		
		// this is the subscript for which we need lhs subscript
		String subscript = new ArrayReference(missing).getSubscript();
		String arrayName = new ArrayReference(missing).getArrayName();
		
		// extract the subscripts from the full array reference
		Set<String> haves = new LinkedHashSet<String>();
		for (String ref : have) {
			haves.add(new ArrayReference(ref).getSubscript());
		}
		
		
		Set<String> defining = new LinkedHashSet<String>();
		Set<String> expandedSubscript = new LinkedHashSet<String>();
		
		// if the missing subscript is a named subscript, we need to get all the terminal
		// subscripts for this named subscript
		if (isNamedSubscript(subscript)) {
			expandedSubscript.addAll(getValuesFor(subscript));
		} else {
			expandedSubscript.add(subscript);
		}
		
		// if there 
		for (String subr : expandedSubscript) {
			defining.addAll(getNamedSubscriptsWithElement(subr, haves));
		}
		
		Set<String> intersection = Utilities.intersect(haves, defining);
		
		return combineArrayWithSubscripts(arrayName, intersection);
	}
	
	public Set<String> combineArrayWithSubscripts(String arrayName, Set<String> intersection) {
		Set<String> set = new LinkedHashSet<String>();
		for (String s : intersection) {
			set.add(arrayName+"["+s+"]");
		}
		return set;
	}
	
	public Set<String> getNamedSubscriptsWithElement(String element, Set<String> haves) {
		Set<String> nswe = new LinkedHashSet<String>();
			for (String key : namedSubscripts.keySet()) {
				if (namedSubscripts.get(key).getValues().contains(element)) {
					nswe.add(key);
				} 
			}
			if (haves.contains(element))
				nswe.add(element);
			return nswe;
	}
	
	public String getNamedSubscriptWithElement(String element) {
		if (namedSubscriptWithElementCache.containsKey(element)) {
			return namedSubscriptWithElementCache.get(element);
		} else {
			for (String key : namedSubscripts.keySet()) {
				if (namedSubscripts.get(key).getValues().contains(element)) {
					namedSubscriptWithElementCache.put(element,key);
					return key;
				} 
			}
			namedSubscriptWithElementCache.put(element, null);
			return namedSubscriptWithElementCache.get(element);
		}

	}

//	public  void subscriptDefinition(String subscriptName, String...subscriptNames) {
//
//		// Is this the first time we have seen arrays?
//		if (!namedSubscripts.containsKey(subscriptName)) {
//			namedSubscripts.put(subscriptName, new NamedSubscriptDefinition(subscriptName, subscriptNames));
//		}
//	}

	public  void subscriptDefinition(String subscriptName, List<String> subscriptNames) {

		// Is this the first time we have seen arrays?
		if (!namedSubscripts.containsKey(subscriptName)) {
			namedSubscripts.put(subscriptName, new NamedSubscriptDefinition(subscriptName, subscriptNames));
			System.out.println("NS: "+subscriptName+" -> "+subscriptNames);
//			buildSubscriptTree(namedSubscripts.get(subscriptName));
		}
	}
	
	private void buildSubscriptTree(NamedSubscriptDefinition subDef) {
		
		String name = subDef.getName();
		List<String> values = subDef.getValues();
		List<Node> valueNodes = new ArrayList<Node>();
		Node nameNode;
		Node valueNode;
		
		// note that we may be adding to an existing tree structure
		
		if (!subscriptToNode.containsKey(name)) {
			subscriptToNode.put(name, new Node(name));
		}
		nameNode = subscriptToNode.get(name);
		
		for (int i = 0; i < values.size(); i++) {
			String subr = values.get(i);
			if (!subscriptToNode.containsKey(subr)) {
				subscriptToNode.put(subr, new Node(subr));
			}
			valueNode = subscriptToNode.get(subr);
			valueNodes.add(valueNode);
		}
		
		Node prev = valueNodes.get(0);
		nameNode.setChild(prev);
		prev.setParent(nameNode);
		
		Node node = null;
		
		for (int nptr = 1; nptr < values.size(); nptr++) {
			prev = valueNodes.get(nptr-1);
			node = valueNodes.get(nptr);
			
			prev.setNext(node);
			node.setPrevious(prev);
			node.setParent(nameNode);
		}
		
	}
	
	public List<Node> getRootNodes() {
		List<Node> rootNodes = new ArrayList<Node>();
		for (Node n : subscriptToNode.values()) {
			if (n.getParent() == null) {
				rootNodes.add(n);
				Node.printTreeToConsole(n);
			}
		}
		return rootNodes;
	}
	
	public List<String> getNamedSubscripts() {
		return new ArrayList<String>(namedSubscripts.keySet());
	}

	public  List<String> getValuesFor(String subscript) {
		String subNoBang = subscript.replace("!", "");
		if (valuesForCache.containsKey(subNoBang)) {
			return valuesForCache.get(subNoBang);
		} else {
			List<String> al = new ArrayList<String>();
			if (!isNamedSubscript(subNoBang)) {
				al.add(subscript);
			} else {
				for (String val : namedSubscripts.get(subNoBang).getValues()) {
					al.addAll(getValuesFor(val));
				}
			}
			valuesForCache.put(subNoBang, al);
			return al;
		}
	}

	public  int getNumIndexFor(String subscript) {
		String subNoBang = subscript.replace("!", "");
		if (numIndexForCache.containsKey(subNoBang)) {
			return numIndexForCache.get(subNoBang);
		} else {
			int numIndex = getValuesFor(subNoBang).size();
			numIndexForCache.put(subNoBang, numIndex);
			return numIndex;
		}
	}

	public  int getIndexFor(String namedSubscript, String subscript) {
		return getValuesFor(namedSubscript).indexOf(subscript);
	}

	public  boolean isNamedSubscript(String subscript) {
		if (namedSubscripts.containsKey(subscript.replace("!", "")))
			return true;
		else
			return false;
	}

	public  boolean hasNamedSubscript(ArrayReference arrayReference) {
		for (String subscript : arrayReference.getSubscripts()) {
			if (namedSubscripts.containsKey(subscript))
				return true;
		}
		return false;

	}

	public  int getNumSubscriptCombinations(ArrayReference arrayReference) {
		String key = arrayReferenceKey(arrayReference);
		if (numSubscriptCombinationsCache.containsKey(key)) {
			return numSubscriptCombinationsCache.get(key);
		} else {
			int numSubscriptCombinations = 1;
			List<Integer> exp = new ArrayList<Integer>();
			for (String subscript : arrayReference.getSubscripts()) {
				if (namedSubscripts.containsKey(subscript))
					exp.add(getValuesFor(subscript).size());
			}

			for (Integer s : exp) {
				numSubscriptCombinations *= s.intValue();
			}
			numSubscriptCombinationsCache.put(key, numSubscriptCombinations);

			return numSubscriptCombinations;
		}

	}
	
	public  String[] getExpandedSubscripts(ArrayReference arrayReference) {
		String key = arrayReferenceKey(arrayReference);
		if (expandedSubscriptsCache.containsKey(key)) {
			return expandedSubscriptsCache.get(key);
		} else {
			List<String> exp = new ArrayList<String>();
			for (String subscript : arrayReference.getSubscripts()) {
				if (namedSubscripts.containsKey(subscript))
					exp.addAll(getValuesFor(subscript));
			}
			int i = 0;
			String[] expSub = new String[exp.size()];
			for (String s : exp) {
				expSub[i++] = s;
			}
			expandedSubscriptsCache.put(key, expSub);
			return expSub;
		}

	}

	public  void dumpMappings(BufferedWriter bw, String separator) {

		try {
			bw.append("Name,Value,Index\n");
			for (String name : namedSubscripts.keySet()) {
				NamedSubscriptDefinition nsd = namedSubscripts.get(name);
				int index = 0;
				for (String value : nsd.getValues()) {
					bw.append(name+separator+value+separator+(index++)+"\n");
				}
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
	}

	public  int getIndex(String name, String value) {
		if (namedSubscripts.containsKey(name)) {
			return namedSubscripts.get(name).getValues().indexOf(value);
		} else {
			return -1;
		}
	}
	
	private String arrayReferenceKey(ArrayReference arrayReference) {
		StringBuilder sb = new StringBuilder();
		for (String subscript : arrayReference.getSubscripts()) {
			sb.append("#"+subscript);
		}
		
		return sb.toString();
	}
	

}
