package gov.anl.gss.sdm.server.model.support;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.math3.analysis.interpolation.LinearInterpolator;
import org.apache.commons.math3.analysis.polynomials.PolynomialSplineFunction;
import org.apache.log4j.Logger;

public class SDFunctions {
static Logger log = Logger.getLogger(SDFunctions.class.getSimpleName());

    private Map<String, SmoothFunction> smoothFunctions = new LinkedHashMap<String, SmoothFunction>();
    private Map<String, DelayFunction> delayFunctions = new LinkedHashMap<String, DelayFunction>();
    private Map<String, Lookup> lookups = new LinkedHashMap<String, Lookup>();
    private Map<String, Double> activeInitials = new LinkedHashMap<String, Double>();
    
    private Map<String, Trend> trends = new LinkedHashMap<String, Trend>();
    
    private Map<String, Integer> elmCounts = new LinkedHashMap<String, Integer>();
    
    private boolean leeaMode = false;
 

	public void setLeeaMode(boolean leeaMode) {
		this.leeaMode = leeaMode;
	}

	private int obCount = 0;
    private static int MAX_OB = 300;

    private SDModel model;

    public SDFunctions(SDModel model) {
	this.model = model;
    }
    
    public SDFunctions() {
    	this.model = null;
    }
    
    public void backstep() {
    	for (SmoothFunction sf : smoothFunctions.values()) {
    		sf.backstep();
    	}
    	
    	for (DelayFunction df : delayFunctions.values()) {
    		df.backstep();
    	}
    }
    
    public void registerSubscript(String subr, int numElements) {
    	elmCounts.put(subr, numElements);
    }
    
    public Double LOOKUPBACKWARDS(double[][] lookupArray, double x) {
    	Double yVal = null;
    	int xval = 0;
    	int yval = 1;
    	int len = lookupArray[xval].length;
    	// out of range
    	if (x < lookupArray[0][0]) {
    		return lookupArray[1][0];
    	}

    	if (x > lookupArray[xval][len - 1]) {
    		return lookupArray[yval][len - 1];
    	}

    	int pos = 0;
    	for (int i = 0; i < len; i++) {
    		if (lookupArray[xval][i] == x) {
    			return lookupArray[yval][i];
    		} else if (x > lookupArray[xval][i]) {
    			return lookupArray[xval][i-1];
    		}
    	}
    	return lookupArray[xval][len-1];
    }
    
    public Double LOOKUPFORWARDS(double[][] lookupArray, double x) {
    	Double yVal = null;
    	int xval = 0;
    	int yval = 1;
    	int len = lookupArray[xval].length;
    	// out of range
    	if (x < lookupArray[0][0]) {
    		return lookupArray[1][0];
    	}

    	if (x > lookupArray[xval][len - 1]) {
    		return lookupArray[yval][len - 1];
    	}

    	int pos = 0;
    	for (int i = 0; i < len-1; i++) {
    		if (lookupArray[xval][i] == x) {
    			return lookupArray[yval][i];
    		} else if (x > lookupArray[xval][i] && x < lookupArray[xval][i+1]) {
    			return lookupArray[xval][i+1];
    		}
    	}
    	return lookupArray[xval][len-1];
    }

    
    
    public Double LOOKUP(double[][] lookupArray, double x) {
	Double yVal = null;
	int xval = 0;
	int yval = 1;
	
	if (lookupArray == null) {
		System.out.println("Missing lookup table");
	}
	
	int len = lookupArray[xval].length;
	// out of range
	if (x < lookupArray[0][0]) {
		if (++obCount <= MAX_OB) {
			log.debug("(func) WARNING: Below Out of range access to lookup: "+ x+" range: "+lookupArray[0][0]+" - "+lookupArray[xval][len - 1]);
//			log.debug("Size: "+lookupArray.length+" x "+lookupArray[0].length);
		}
	    return lookupArray[1][0];
	}

	if (x > lookupArray[xval][len - 1]) {
		if (++obCount <= MAX_OB) {
			log.debug("(func) WARNING: Above Out of range access to lookup: "+ x+" range: "+lookupArray[0][0]+" - "+lookupArray[xval][len - 1]);
			System.out.println("(func) WARNING: Above Out of range access to lookup: "+ x+" range: "+lookupArray[0][0]+" - "+lookupArray[xval][len - 1]);
//			log.debug("Size: "+lookupArray.length+" x "+lookupArray[0].length);
		}
	    return lookupArray[yval][len - 1];
	}

	int pos = 0;
	for (int i = 0; i < len; i++) {
	    if (lookupArray[xval][i] == x) {
		return lookupArray[yval][i];
	    } else if (x < lookupArray[xval][i]) {
		pos = i - 1;
		break;
	    }
	}

	if (lookupArray[xval][pos] == lookupArray[xval][pos + 1])
	    return null;
	yVal = lookupArray[yval][pos] * (x - lookupArray[xval][pos + 1])
		/ (lookupArray[xval][pos] - lookupArray[xval][pos + 1])
		+ lookupArray[yval][pos + 1] * (x - lookupArray[xval][pos])
		/ (lookupArray[xval][pos + 1] - lookupArray[xval][pos]);

	return yVal;
    }
  
    
    private double slope(double x1, double y1, double x2, double y2) {
    	    return (y2 - y1) / (x2 - x1); 
    }

    public Double LOOKUPEXTRAPOLATEBACKWARDS(double[][] lookupArray, double x) {
    	int xval = 0;
    	int yval = 1;
    	double slope = slope(lookupArray[xval][0], lookupArray[yval][0],
    			lookupArray[xval][1], lookupArray[yval][1]);
    	return (lookupArray[xval][0] - x) * slope;
    }

    public Double LOOKUPEXTRAPOLATEFORWARDS(double[][] lookupArray, double x) {
    	int xval = 0;
    	int yval = 1;
    	int len = lookupArray[xval].length;
    	double slope = slope(lookupArray[xval][len-2], lookupArray[yval][len-2],
    			lookupArray[xval][len-1], lookupArray[yval][len-1]);
    	return (x - lookupArray[xval][len-1]) * slope;
    }
    
    public Double LOOKUPEXTRAPOLATE(double[][] lookupArray, double x) {
    	int xval = 0;
    	int len = lookupArray[xval].length;
    	// out of range
    	if (x < lookupArray[0][0]) {
    		return LOOKUPEXTRAPOLATEBACKWARDS(lookupArray, x);
    	}

    	if (x > lookupArray[xval][len - 1]) {
    		return LOOKUPEXTRAPOLATEFORWARDS(lookupArray, x);
    	}

    	return LOOKUP(lookupArray, x);
    }
    

    public void ADDLOOKUP(String varName, double...ds ) {
	lookups.put(varName, new Lookup(varName, ds));
	model.saveLookupValues(varName);
    }
    
    public void ADDLOOKUP(String varName, double[] time, double[] values) {
	lookups.put(varName, new Lookup(varName, time, values));
	model.saveLookupValues(varName);
    }

    public void ADDLOOKUPPAIRS(String varName, double minX, double minXY, double maxX, double maxXY, double...ds ) {
	lookups.put(varName, new Lookup(varName,  minX,  minXY,  maxX,  maxXY, ds));
	model.saveLookupValues(varName);
    }
    
    public double[][] ADDLOOKUPNATIVE(String varName, double... ds) {
	lookups.put(varName, new Lookup(varName, ds));
	model.saveLookupValues(varName);
	return lookups.get(varName).getAsArray();
    }
    
    public double[][] ADDLOOKUPNATIVE(String varName, double[] time, double[] values) {
	lookups.put(varName, new Lookup(varName, time, values));
	model.saveLookupValues(varName);
	return lookups.get(varName).getAsArray();
    }

    public double[][] ADDLOOKUPPAIRSNATIVE(String varName, int numValues, double minX,
    		double minXY, double maxX, double maxXY, double... ds) {
    	lookups.put(varName, new Lookup(varName, minX, minXY, maxX, maxXY, ds));
    	model.saveLookupValues(varName);
    	return lookups.get(varName).getAsArray();
    }
    
    public double[][] getLookupValues(String varName) {
    	return lookups.get(varName).getAsArray();
    }

//    public double INTEG(String varName, double currentValue, double time, double timeStep, double val, double initial) {
//	if (model.getCurrentTime() <= model.getINITIALTIME()) {
//	    return initial;
//	} else {
//		double newVal = currentValue + (val * model.getTIMESTEP());
//	    return newVal;
//	}
//    }
    
    public double INTEG(double currentValue, double val, double initial) {
	if (model.isInitialization()) {
	    return initial;
	} else {
		double newVal = currentValue + (val * model.getTIMESTEP());
	    return newVal;
	}
    }


    public double IFTHENELSE(String varName,   double currentValue, double time, double timeStep, boolean test, double arg1, double arg2) {
	if (test)
	    return arg1;
	else
	    return arg2;

    }
    
    public double IFTHENELSE(boolean test, double arg1, double arg2) {
	if (test)
	    return arg1;
	else
	    return arg2;

    }

    public double NOP(String varName,  double currentValue, double time, double timeStep) {
	return currentValue;
    }



    public double MIN(double arg1, double arg2) {
	return arg1 < arg2 ? arg1 : arg2;
    }

    public double MAX(double arg1, double arg2) {
	return arg1 > arg2 ? arg1 : arg2;
    }
    public double ZIDZ(double arg1, double arg2) {
	return arg2 == 0.0 ? 0.0 : arg1 / arg2;
    }

    public double XIDZ(double arg1, double arg2, double arg3) {
	return arg2 == 0.0 ? arg3 : arg1 / arg2;
    }
    public double INITIAL(double value, double time, double timeStep, double arg1) {
    	if (model.getCurrentTime() == model.getINITIALTIME()) {
    		return arg1;
    	}
    	return value;
    }
    public double INITIAL(double arg1) {
    	
    	return arg1;
    }
    
    public double REINITIAL(double value, double time, double arg1) {
    	if (model.getCurrentTime() == model.getINITIALTIME()) {
    		return arg1;
    	}
    	return value;
    }
    
    public double STEP(double time, double arg1, double arg2) {
	if (time < arg2) {
	    return 0.0;
	} else {
	    return arg1;
	}

    }

    public double PULSE(String varName,  double currentValue,double time, double timeStep, double start, double width) {
	if (time >= start && time < start + width) {
	    return 1.0;
	} else {
	    return 0.0;
	}
    }

    public double MODULO(double arg1, double arg2) {
	return arg1 % arg2;
    }

    public double DELAY1(String varName, double currentValue, double time, double timeStep,double arg1,double arg2) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new Delay1(this, varName, arg2));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2);
    }
    public double DELAY1I(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new Delay1(this, varName, arg2));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3);
    }


    public double DELAY3(String varName, double currentValue,double time, double timeStep,double arg1,double arg2) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new DelayN(this, varName, arg2));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, 3, 3);
    }
    public double DELAY3I(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new DelayN(this, varName, arg2));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3, 3);
    }

    public double DELAYN(String varName,  double currentValue, double time, double timeStep, double arg1, double arg2, double arg3, double arg4) {

	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new DelayN(this, varName, arg2));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3, arg4);
    }

    public double SMOOTH(String varName,  double arg1, double arg2) {

	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    smoothFunctions.put(varName, new Smooth(this, varName, arg2));
	}
	SmoothFunction smooth = smoothFunctions.get(varName);	
	return smooth.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2);

    }

    public double SMOOTHI(String varName,  double arg1, double arg2, double arg3) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    smoothFunctions.put(varName, new Smooth(this, varName, arg2));
	}
	SmoothFunction smooth = smoothFunctions.get(varName);	
	return smooth.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3);
    }

    public double SMOOTH3(String varName, double arg1,double arg2) {

//	return SMOOTH(varName, currentValue, time, timeStep, arg1, arg2);
//	//	if (model.getCurrentTime() == model.getINITIALTIME()) {
//	//	    smoothFunctions.put(varName, new SmoothN(this, varName, arg2));
//	//	}
//	//	SmoothFunction smooth = smoothFunctions.get(varName);	
//	//	return smooth.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, 3);
//
//    }
    	return 0.0;
    }
    public double SMOOTH3I(String varName, double arg1,double arg2,double arg3) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    smoothFunctions.put(varName, new SmoothN(this, varName, arg2));
	}
	SmoothFunction smooth = smoothFunctions.get(varName);	
	return smooth.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3, 3);
    }




    public double SMOOTHN(String varName,   double arg1, double arg2, double arg3, double arg4) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    smoothFunctions.put(varName, new SmoothN(this, varName, arg2));
	}
	SmoothFunction smooth = smoothFunctions.get(varName);	
	return smooth.getValue(model.getCurrentTime(), model.getTIMESTEP(), arg1, arg2, arg3, arg4);
    }



    public double GAME(double arg1) {
	return arg1;
    }

    public double SIN(double arg1) {
	return Math.sin(arg1);
    }

    public double RAMP(String varName,  double currentValue, double time, double timeStep, double slope, double startTime, double endTime) {
	if (time < startTime) {
	    return 0.0;
	} else if (time < endTime) {
	    return slope*(time - startTime);
	} else {
	    return slope*(endTime - startTime);
	}
    }

    public double GETDATAMAX(String varName,  double currentValue, double time, double timeStep, double arg1, double arg2, double arg3) {
	notImplemented();
	return arg1;
    }

    public double TREND(String varName,  double currentValue, double time, double timeStep, double arg1, double arg2, double arg3) {
    	if (!trends.containsKey(varName)) {
    		trends.put(varName, new Trend(this, arg1, arg2, arg3));
    	}
    	
	return trends.get(varName).getNext(arg1, arg2);
    }

    public double SAMPLEIFTRUE(String varName,  double currentValue, double time, double timeStep, boolean arg1, double arg2, double arg3) {
//	notImplemented();
    	
    	System.out.println("SAMPLEIFTRUE("+varName+", "+currentValue+", "+time+", "+timeStep+", "+arg1+", "+arg2+", "+arg3+")");
	return arg2;
    }

    protected void notImplemented() {
	ArrayList<String> al = null; //  = new ArrayList<String>();
	String x = al.get(10);
    }

    public double ABS(double arg1) {

	return Math.abs(arg1);
    }
    public double ACTIVEINITIAL(String varName, double currentValue, double time, double timeStep,double arg1,double arg2) {
    	if (model.isInitialization()) {
    		activeInitials.put(varName, arg2);
    		return arg2;
    	}
    	if (model.getCurrentTime() == model.getINITIALTIME()) {
    		if (activeInitials.containsKey(varName)) {
    			System.out.println("ActiveInitial difference: "+varName+" "+activeInitials.get(varName).doubleValue()+" "+arg1+"  "+
    					(activeInitials.get(varName).doubleValue()-arg1));
    		} else {
    			System.out.println("ActiveInitial difference: "+varName+" Never Called in initialization");
    		}
    	}
    	return arg1;
    }
    public double ALLOCATEAVAILABLE(double arg1,double arg2,double arg3) {
//	notImplemented();
	return arg1;
    }
    public double ALLOCATEBYPRIORITY(double arg1,double arg2,double arg3,double arg4,double arg5) {
//	notImplemented();
	return arg1;
    }
    public double ALLOC(double arg1,double arg2,double arg3,double arg4) {
	notImplemented();
	return arg1;
    }
    public double ARCCOS(double arg1) {
	return Math.acos(arg1);
    }
    public double ARCSIN(double arg1) {
	return Math.asin(arg1);
    }
    public double ARCTAN(double arg1) {
	return Math.atan(arg1);
    }
    public double COS(double arg1) {
	return Math.cos(arg1);
    }
    public double COSH(double arg1) {
	return Math.cosh(arg1);
    }
    public double CUMULATE(String varName, double currentValue, double time, double timeStep,double arg1) {

	notImplemented();
	return arg1;
    }
    public double CUMULATEF(String varName, double currentValue, double time, double timeStep,double arg1) {

	notImplemented();
	return arg1;
    }
    public double DELAYBATCH(String varName, double currentValue,double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
	notImplemented();
	return arg1;
    }
    public double DELAYCONVEYOR(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
	notImplemented();
	return arg1;
    }
    public double DELAYFIXED(String varName, double currentValue, double time, double timeStep,double input,double delayTime,double initialValue) {
	if (model.getCurrentTime() == model.getINITIALTIME()) {
	    delayFunctions.put(varName, new DelayFixed(this, varName, timeStep, delayTime, initialValue));
	}
	DelayFunction delay = delayFunctions.get(varName);	
	return delay.getValue(model.getCurrentTime(), model.getTIMESTEP(), input);
    }
    public double DELAYINFORMATION(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	notImplemented();
	return arg1;
    }
    public double DELAYMATERIAL(String varName, double currentValue,double time, double timeStep,double arg1,double arg2,double arg3,double arg4) {
	notImplemented();
	return arg1;
    }

    public double DELAYPROFILE(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5) {
	notImplemented();
	return arg1;
    }


    public double DELAYP(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	notImplemented();
	return arg1;
    }
    public double DEMANDATPRICE(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	notImplemented();
	return arg1;
    }
    public double DEPRECIATEBYSCHEDULE(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5) {
	notImplemented();
	return arg1;
    }
    public double DEPRECIATESTRAIGHTLINE(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4) {
	notImplemented();
	return arg1;
    }
    public double ELMCOUNT(String subscript) {
	return (double) elmCounts.get(subscript).intValue();
    }
    public double ELMCOUNT(double n) {
    	return n;
        }
    public double EXP(double arg1) {
	return Math.exp(arg1);
    }
    public double FINDZERO(String varName, double currentValue, double time, double timeStep,double arg1) {
	notImplemented();
	return arg1;
    }
    public double FORECAST(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3) {
	notImplemented();
	return arg1;
    }

    public double GAMMALN(String varName, double currentValue, double time, double timeStep,double arg1) {
	notImplemented();
	return arg1;
    }
    public double GET123CONSTANTS(String varName, double currentValue, double time, double timeStep,String arg1,String arg2,String arg3) {
	notImplemented();
	return -1.0;
    }
    public double GET123DATA(String varName, double currentValue, double time, double timeStep,String arg1,String arg2,String arg3,String arg4) {
	notImplemented();
	return -1.0;
    }
    public double GETDATAATTIME(String varName, double currentValue, double time, double timeStep,double arg1,double arg2) {
	notImplemented();
	return arg1;
    }
    public double GETDATABETWEENTIMES(/* String varName, double currentValue, double time, double timeStep, */ String ts, double forTime ,double mode) {

	// ts can be an array reference, 
	if (model.getTimeSeriesData().hasTimeSeriesFor(ts)) {
	    return model.getTimeSeriesData().getDataBetweenTimes(ts, forTime, mode);
	} else {
	    return 0.0;
	}
    }
    public double GETDATAFIRSTTIME(String varName, double currentValue, double time, double timeStep,double arg1) {
	notImplemented();
	return arg1;
    }
    public double GETDATALASTTIME(String varName, double currentValue, double time, double timeStep,double arg1) {
	notImplemented();
	return arg1;
    }
    public double GETDATATOTALPOINTS(String varName, double currentValue, double time, double timeStep,double mode) {

	notImplemented();
	return mode;
    }



    public int getRowNumberFromCellAddress(String cell) {
	return Utilities.getRowFromCellAddress(cell);
    }

    public int getColumnNumberFromCellAddress(String cell) {
	return Utilities.getColumnFromCellAddress(cell);
    }





    public double INTEGER(double arg1,double arg2) {
	notImplemented();
	return arg1;
    }
    public double INTEGER(double arg1) {
	return (int) arg1;
    }
    public double INTERNALRATEOFRETURN(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4) {
	notImplemented();
	return arg1;
    }
    public double INVERTMATRIX(String varName, double currentValue, double time, double timeStep,double arg1,double arg2) {
	notImplemented();
	return arg1;
    }
    public double LN(double arg1) {
	return Math.log(arg1);
    }
    public double LOG(double arg1,double arg2) {
	return Math.log(arg2) / Math.log(arg1);
    }

    public double POWER(double arg1,double arg2) {
	return Math.pow(arg1, arg2);
    }

    public double PULSETRAIN(String varName, double currentValue, double time, double timeStep,
    		double initialTime,double width,double interval,double endTime) {
    	if (time < initialTime || time >= endTime)
    		return 0;
    	BigDecimal t = new BigDecimal(time - initialTime);
    	BigDecimal r = t.remainder(new BigDecimal(interval));
    	BigDecimal d = new BigDecimal(width-timeStep);
    	if (r.doubleValue() < d.doubleValue())
    		return 1.0;
    	else
    		return 0.0;
    
    }

    public double RANDOMBETA(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6,double arg7) {
	notImplemented();
	return arg1;
    }
    public double RANDOMBINOMIAL(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6,double arg7) {
	notImplemented();
	return arg1;

    }
    public double RANDOMEXPONENTIAL(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5) {
	notImplemented();
	return arg1;
    }
    public double RANDOMGAMMA(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
	notImplemented();
	return arg1;
    }
    public double RANDOMLOOKUP(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
	notImplemented();
	return arg1;
    }
    public double RANDOMNORMAL(double arg1,double arg2,double arg3,double arg4,double arg5) {
	notImplemented();
	return arg1;
    }
//    public double RANDOMPOISSON(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
//	notImplemented();
//	return arg1;
//    }
    public double RANDOMTRIANGULAR(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
	notImplemented();
	return arg1;
    }
   

    public double SINH(double arg1) {
	return Math.sinh(arg1);
    }




    public double SQRT(double arg1) {
	return Math.sqrt(arg1);
    }

    public  double TAN(double arg1) {
	return Math.tan(arg1);
    }
    public double TANH(double arg1) {
	return Math.tanh(arg1);
    }

    public static boolean isInteger( String input )
    {
	try
	{
	    Integer.parseInt( input );
	    return true;
	}
	catch( Exception e)
	{
	    return false;
	}
    }
    
    /**
     * @param string
    
     * @return
     */
    public double VDMLOOKUP(String string) {
    	return model.getData().valueOf(string);
    }
    
    /**
     * @param string
    
     * @return
     */
    public double EXTERNAL_DATA(String string) {
    	return model.getData().valueOf(string);
    }

    /**
     * @param string
     * @param arrayValueOf
     * @param time
     * @param timeStep
     * @param valueOf
     * @return
     */
    public double VDMLOOKUP(String string, double arrayValueOf,
	    double time, double timeStep, double valueOf) {


	return model.getData().valueOf(string);
    }
    
    /**
     * @param string
     * @param arrayValueOf
     * @param time
     * @param timeStep
     * @param valueOf
     * @return
     */
    public double VDMLOOKUP(String string, double arrayValueOf,
	    double time, double timeStep, double[] valueOf) {


	return model.getData().valueOf(string);
    }
    
    /**
     * @param string
     * @param arrayValueOf
     * @param time
     * @param timeStep
     * @param valueOf
     * @return
     */
    public double VDMLOOKUP(String string, double arrayValueOf,
	    double time, double timeStep, double[][] valueOf) {


	return model.getData().valueOf(string);
    }

    /**
     * @param string
     * @param arrayValueOf
     * @param time
     * @param timeStep
     * @param sarray
     * @param vexp
     * @param mval
     * @param operation
     * @param err
     * @return
     */
    public double VECTORSELECT(String string, double arrayValueOf,
	    double time, double timeStep, double[] sarray, double[] vexp,
	    double mval, double operation, double err) {
	
	double result = 0.0;
	int numIncluded = 0;
	
	// set initial value of result to appropriate value for operation
	
	if (operation == 0.0 || operation == 4.0) {
	    result= 0.0;
	} else if (operation == 1.0) {
	    result = 1.0;
	} else if (operation == 2.0) {
	    result = Double.MAX_VALUE;
	} else if (operation == 3.0) {
	    result = Double.MIN_VALUE;
	}
	
	// perform the requested operation
	
	for (int i = 0; i < sarray.length; i++) {
	    if (sarray[i] == 1.0) {
		numIncluded++;
		if (operation == 0.0 || operation == 4.0) {
		    result += vexp[i];
		} else if (operation == 1.0) {
		    result *= vexp[i];
		} else if (operation == 2.0) {
		    result = Math.min(result, vexp[i]);
		} else if (operation == 3.0) {
		    result = Math.max(result, vexp[i]);
		}
	    }
	}
	
	// check the error conditions
	// need to figure out how to handle these error conditions
	
	if (err == 0.0) {
	    // do nothing
	} else if (err == 1.0) {
	    if (numIncluded == 0) {
		// raise error
	    }
	} else if (err == 2.0) {
	    if (numIncluded > 1) {
		// raise error
	    }
	} else if (err == 3.0) {
	    if (numIncluded == 0 || numIncluded > 1) {
		// raise error
	    }
	}
	
	// finally return the result
	
	if (operation == 4.0) {
	    return result / (double) numIncluded;
	} else {
	    return result;
	}
    }
    
    public double VECTORELMMAP(String string, double arrayValueOf,
	    double time, double timeStep, double[] _da1,  double e) {
	
	// c implementation
	/*
	
	double VECTORELMMAP(char *string, double arrayValueOf,double time, double timeStep, *double _da1,  double e) {
		
		return _da1[(int)e];
	}
	*/

	
	
	// TODO Auto-generated method stub
	return 0.0;
    }
    
    public double VECTORLOOKUP(double[] yvec, double x, double xmin, double xmax, double mode) {
    	
    	double[] xvec = new double[yvec.length];
    	double inc = (xmax - xmin)/(yvec.length-1);
    	xvec[0] = xmin;
    	xvec[xvec.length-1] = xmax;
    	for (int i = 1; i < xvec.length-2; i++) {
    		xvec[i] = xvec[i-1] + inc;
    	}
    	
    	int slot = 0;
    	
    	if (mode == 0.0) {  // a normal lookup function determining y from x.

    		// return linear interpolation of (x,y) on xi
    		LinearInterpolator li = new LinearInterpolator();
    		PolynomialSplineFunction psf = li.interpolate(xvec, yvec);
    		double yi = psf.value(x);


    	} else if (mode == 1.0) { // 1 the equivalent of LOOKUP AREA
    		
    	} else if (mode == 2.0) { // 2 the equivalent of  LOOKUP BACKWARD
    		
    		slot = findSlot(xvec, x);
    		slot = slot < 0 ? 0 : slot == Integer.MAX_VALUE ? xvec.length-1 : slot;
    		
    	} else if (mode == 3.0) { // 3 the equivalent of LOOKUP EXTRAPOLATE
    		
    	} else if (mode == 4.0) { // 4 the equivalent of LOOKUP FORWARD
    		
    		slot = findSlot(xvec, x);
    		slot = slot < 0 ? 0 : slot == Integer.MAX_VALUE ? xvec.length-1 : slot;
    		
    	} else if (mode == 5.0) { // 5 the equivalent of LOOKUP INVERT
    		
    	} else if (mode == 6.0) { // 6 the equivalent of LOOKUP SLOPE
    		
    	} else if (mode == 7.0) { // 7 the equivalent of LOOKUP AREA with extrapolation

    	} else if (mode == 8.0) { // 8 the equivalent of LOOKUP INVERT with extrapolation
    		
    	} else if (mode == 9.0) { // 9 the equivalent of LOOKUP SLOPE with extrapolation
    		
    	} else {
    		return Double.NaN;
    	}
    	
    	return 0.0;
    }
    
    private int findSlot(double[] xvec, double x) {
    	if (x < xvec[0])
    		return -1;
    	
    	int slot = 0;
    	
    	while (x < xvec[slot])
    		slot++;
    	if (x > xvec[slot])
    		return Integer.MAX_VALUE;
    	else
    		return slot;
    	
    }
    
    public double[] VECTORREORDER(double[] vector, double[] order) {
    	double[] newVector = new double[vector.length];
    	
    	for (int i = 0; i < order.length; i++) {
    		newVector[i] = vector[(int) order[i]];
    	}
    	
    	return newVector;
    }
    
    public double[] VECTORSORTORDER(String string, double arrayValueOf,
	    double time, double timeStep, double[] _da1, double e) {
	
	// use a bubble sort for the ordering -- based on assumption that vectors to be
	// sorted are relatively short
	
	// first make a copy of the original vector and index vector
	double[] _da1_copy = new double[_da1.length];
	double[] index = new double[_da1.length];
	for (int i = 0; i < _da1.length; i++) {
	    _da1_copy[i] = _da1[i];
	    index[i] = i;
	}
	
	// now sort both copy and index arrays
	
	int i = 0, j = 0;
	double tempv, tempi;
	 
	  for (i = (_da1_copy.length - 1); i > 0; i--)
	  {
	    for (j = 1; j <= i; j++)
	    {
	      if ((e > 0.0 && _da1_copy[j-1] > _da1_copy[j]) ||
		      (e <= 0.0 && _da1_copy[j-1] < _da1_copy[j]))
	      {
		  // swap data values
	        tempv = _da1_copy[j-1];
	        _da1_copy[j-1] = _da1_copy[j];
	        _da1_copy[j] = tempv;
	        
	        // swap index values
	        tempi = index[j-1];
	        index[j-1] = index[j];
	        index[j] = tempi;
	      }
	    }
	  }

	
	
	return index;
    }
    
    public double[] VECTORRANK(double [] vector,double direction) {
    	double[] rank = new double[vector.length];
    	
    	Map<String, List<Integer>> index = new LinkedHashMap<String, List<Integer>>();
    	Map<String, Double> value = new LinkedHashMap<String, Double>();
    	List<Double> forSorting = new ArrayList<Double>();
    	
    	for (int i = 0; i < vector.length; i++) {
    		String d = Double.toString(vector[i]);
    		
    		if (!index.containsKey(d))
    			index.put(d, new ArrayList<Integer>());
    		
    		index.get(d).add(i);
    		value.put(d, vector[i]);
    		forSorting.add(vector[i]);
    	}
    	
    	Collections.sort(forSorting, Collections.reverseOrder(null));
    	for (int i = 0; i < forSorting.size(); i++) {
    		String d = Double.toString(vector[i]);
    		List<Integer> indexes = index.get(d);
    		int ind = indexes.get(0);
    		indexes.remove(0);
    		rank[ind] = i+1;
    	}
    	
    	
    	return rank;
    }

	public SDModel getModel() {
		return model;
	}
	
	   
    public boolean isLeeaMode() {
		return leeaMode;
	}
    
    
    /*
     * These are for xmile processing
     * PI
TIME
DT
STARTTIME
SELF
STOPTIME
INIT

     */
    
    public double PI() {
    	return Math.PI;
    }
    
    public double TIME() {
    	return model.getCurrentTime();
    }
    
    public double DT() {
    	return model.getTIMESTEP();
    }
    
    public double STARTTIME() {
    	return model.getINITIALTIME();
    }
    
    public double STOPTIME() {
    	return model.getFINALTIME();
    }
    
    public double SELF() {
    	return Math.PI;
    }
    
    public double INIT(double var) {
    	return var;
    }
    
    public double XMILEPULSE2(String varName,  double currentValue,double time, double timeStep, double start, double width) {
    	return PULSE(varName,  currentValue,time, timeStep, start, width);
 
    }
    
    public double XMILEPULSE3(String varName, double currentValue, double time, double timeStep,
    		double initialTime,double width,double interval) {
    	return PULSETRAIN(varName, currentValue, time, timeStep,
    		initialTime,width,interval,model.getFINALTIME());
    	
    }
  
}
