package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.math3.distribution.TriangularDistribution;
import org.apache.log4j.Logger;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import cern.jet.random.Beta;
import cern.jet.random.Binomial;
import cern.jet.random.Exponential;
import cern.jet.random.Gamma;
import cern.jet.random.Normal;
import cern.jet.random.Poisson;
import cern.jet.random.Uniform;
import cern.jet.random.engine.MersenneTwister;

public class SDFunctionsWithXLSColt extends SDFunctions {
	static Logger log = Logger.getLogger(SDFunctionsWithXLSColt.class.getSimpleName());

	private Map<String, XSSFWorkbook> workbooks = new LinkedHashMap<String, XSSFWorkbook>();

	private Uniform uniformDistribution = null;
	private Beta betaDistribution = null;
	private Binomial binomialDistribution = null;
	private Exponential exponentialDistribution = null;
	private Gamma gammaDistribution = null;
	private Normal normalDistribution = null;
	private Poisson poissonDistribution = null;
	private TriangularDistribution triangularDistribution = null;
	
	private static String [] FUNCTIONS_REQUIERING_SDMODEL = {
		"ACTIVEINITIAL", "DELAY1", "DELAY1I", "DELAY3", "DELAY3I",
		"DELAYFIXED", "DELAYN", "GETDATABETWEENTIMES", "INITIAL",
		"INTEG", "REINITIAL", "SMOOTH", "SMOOTH3I", "SMOOTHI", "SMOOTHN"
	};

	public static boolean containsNoPolarityFunction(String equation) {
		for (String func : FUNCTIONS_REQUIERING_SDMODEL) {
			if (equation.contains("sdFunctions."+func))
				return true;
		}
		
		return false;
	}

	public SDFunctionsWithXLSColt(SDModel model) {
		super(model);
		// TODO Auto-generated constructor stub
	}

	public SDFunctionsWithXLSColt() {

		super();
	}
	
	public void closeWorkbooks() {
		for (XSSFWorkbook wb : workbooks.values()) {
			try {
				wb.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	public XSSFWorkbook getWorkbook(String name) {
		//		System.out.println("GetWorkbook: "+name);
		if (!workbooks.containsKey(name)) {
			workbooks.put(name, WorkbookUtils.openWorkbook(name));
			//	        System.out.println("GetWorkbook: Opening");
		}
		//	    System.out.println("GetWorkbook: Returning");
		return workbooks.get(name);
	}

	public double GETXLSCONSTANTS(String book,String tab, String cell) {
		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);

		return getNumericDataInCell(sheet, cell);
	}
	
	public List<String> GETXLSSUBSCRIPT(String book,String tab, String firstCell, String lastCell, String prefix) {
		int firstRow;
		int firstCol;
		int lastRow;
		int lastCol;
		
		List<String> subs = null;
		File f = new File(book);
		if (!f.exists())
			return subs;
		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);
		if (sheet == null)
			return subs;
		
		if (!Utilities.isValidCellAddress(firstCell)) {
			return subs;
		}
		firstRow = Utilities.getRowFromCellAddress(firstCell);
		firstCol = Utilities.getColumnFromCellAddress(firstCell);
		
		if (Utilities.isValidCellAddress(lastCell)) {
			lastRow = Utilities.getRowFromCellAddress(lastCell);
			lastCol = Utilities.getColumnFromCellAddress(lastCell);
		} else {
			if (Utilities.isLetter(Parser.characterAt(0, lastCell))) {
				lastRow = firstRow;
				lastCol = firstCol;
			} else if (Utilities.isDigit(Parser.characterAt(0, lastCell))) {
				lastRow = firstRow;
				lastCol = firstCol;
			} else {
				return subs;
			}
		}
		
		subs = new ArrayList<String>();
		
		for (int row = firstRow; row <= lastRow; row++) {
			for (int col = firstCol; col <= lastCol; col++) {
				String aSub = getDataInCellAsString(sheet, row, col);
				if (aSub != null && aSub.length() > 0) {
					if (subs.size() > 0)
						subs.add(",");
					subs.add(prefix+aSub);
				}
			}
		}
		
		return subs;
	}

	public double[] GETXLSCONSTANTS(String book,String tab, String cell, int numRows, int numColumns) {
		double[] fromXls = new double[numRows*numColumns];
		
		boolean transpose = cell.endsWith("*");
		String tcell = cell.replace("*", "");
		int startRow =  getRowNumberFromCellAddress(tcell);
		int startCol =  getColumnNumberFromCellAddress(tcell);
		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);
		int pos = 0;
		
		int nRows = numRows;
		int nCols = numColumns;
		
		if (transpose) {
			nRows = numColumns;
			nCols = numRows;
		}
		
		for (int r = startRow; r < startRow + nRows; r++) {
			for (int c = startCol; c < startCol + nCols; c++) {
				fromXls[pos++] = getNumericDataInCell(sheet, r, c);
			}
		}

		return fromXls;

	}

	public double GETXLSCONSTANTS(String varName, double currentValue, double time, double timeStep, String book,String tab,String cell) {
		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);
		return getNumericDataInCell(sheet, cell);

	}
	
	public double NUMBER(double n) {
		return n;
	}
	
    private double getNumericDataInCell(Cell aCell) {
    	try {
    		if (aCell.getCellType() == CellType.STRING) {
    			String s = aCell.getStringCellValue();
    			//    		 System.out.println("Value "+s);
    			return -1.0;
    			//String
    		} else {

    			double val = aCell.getNumericCellValue();
    			return val;
    		}
    	} catch (Exception e) {
    	    // TODO Auto-generated catch block
    	    return -1.0;
    	}
    	
    }
    
	private String getDataInCellAsString(XSSFSheet sheet, int row, int col) {
		Iterator<Row> rowIt = sheet.rowIterator();
		int currentRow = 0;
		int currentColumn = 0;
		Row aRow = null;
		Cell aCell = null;
		while (rowIt.hasNext()) {
			aRow = rowIt.next();
			currentRow = aRow.getRowNum()+1;
			if (currentRow == row)
				break;
		}
		if (currentRow < row)
			return "BAD ROW getDataInCellAsString "+row+","+col;
		Iterator<Cell> cellIt = aRow.cellIterator();
		while (cellIt.hasNext()) {
			aCell = cellIt.next();
			currentColumn = aCell.getColumnIndex()+1;
			if (currentColumn == col)
				break;
		}

		if (currentColumn < col)
			return "BAD COL getDataInCellAsString "+row+","+col;

		try {
			if (aCell.getCellType() == CellType.STRING) {
				String s = aCell.getStringCellValue();
				return s;
				//String
			} else {

				double val = aCell.getNumericCellValue();
				return Double.toString(val);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return "BAD TYPE getDataInCellAsString "+row+","+col;
		}

	}
    


	private static double getNumericDataInCell(XSSFSheet sheet, int row, int col) {
		Iterator<Row> rowIt = sheet.rowIterator();
		int currentRow = 0;
		int currentColumn = 0;
		Row aRow = null;
		Cell aCell = null;
		while (rowIt.hasNext()) {
			aRow = rowIt.next();
			currentRow = aRow.getRowNum()+1;
			if (currentRow == row)
				break;
		}
		if (currentRow < row)
			return -1.0;
		Iterator<Cell> cellIt = aRow.cellIterator();
		while (cellIt.hasNext()) {
			aCell = cellIt.next();
			currentColumn = aCell.getColumnIndex()+1;
			if (currentColumn == col)
				break;
		}

		if (currentColumn < col)
			return -1.0;

		try {

			if (aCell.getCellType() == CellType.STRING) {
				String s = aCell.getStringCellValue();
//						  System.out.println("Value "+s);
				return -1.0;
				//String
			} else {

				double val = aCell.getNumericCellValue();
				return val;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return -1.0;
		}

	}

	private double getNumericDataInCell(XSSFSheet sheet, String cell) {
		return getNumericDataInCell(sheet, getRowNumberFromCellAddress(cell), getColumnNumberFromCellAddress(cell));
	}
	
    private double[] getValuesInRow(Row aRow, int cellIndex) {
    	List<Double> timeList = new ArrayList<Double>();
    	Iterator<Cell> cellIt = aRow.cellIterator();
    	
    		// find starting cell
    		while (cellIt.hasNext()) {
    			Cell aCell = cellIt.next();
//    			cellIndex = aCell.getColumnIndex()+1;
    			if (aCell.getColumnIndex()+1 == cellIndex) {
    				// this is the starting cell for data
    				timeList.add(getNumericDataInCell(aCell));
    				// process to last, contiguous cell
    				while (cellIt.hasNext()) {
    					aCell = cellIt.next();
    					if (aCell.getColumnIndex()+1 == cellIndex +1) {
    						timeList.add(getNumericDataInCell(aCell));
    						cellIndex++;
    					} else {
    						return asArray(timeList);
    					}
    				}
    				return asArray(timeList);
    			}
    		}
    		return asArray(timeList);
    	
    }
    
private double[] getValuesInColumn(Row aRow, Iterator<Row> rowIt, int cellIndex) {
	List<Double> timeList = new ArrayList<Double>();
	Iterator<Cell> cellIt = aRow.cellIterator();
	int rowIndex = 0;
	while (cellIt.hasNext()) {
		Cell aCell = cellIt.next();
//		cellIndex = aCell.getColumnIndex()+1;
		if (aCell.getColumnIndex()+1 == cellIndex) {
			// this is the starting cell for data
			timeList.add(getNumericDataInCell(aCell));
		}
	}
	while (rowIt.hasNext()) {
		Row aRow2 = rowIt.next();
		if (aRow2.getRowNum()+1 == rowIndex+1) {
			rowIndex++;
			while (cellIt.hasNext()) {
    			Cell aCell = cellIt.next();
    			cellIndex = aCell.getColumnIndex()+1;
    			if (aCell.getColumnIndex()+1 == cellIndex) {
    				timeList.add(getNumericDataInCell(aCell));
    			}
    		}
		} else {
			return asArray(timeList);
		}
	}
	return asArray(timeList);
    }
    


	private static double[] getTimeValues(XSSFSheet sheet, String timeRowOrCol, int startRow, int startCol) {
		List<Double> timeList = new ArrayList<Double>();
		double[] timeArray = null;
		
		
		/*
		 *     	boolean byRow = isInteger(timeRowOrCol);
    	Row aRow = null;
    	int row;

    	// locate starting row
    	if (byRow)
    		row = Integer.parseInt(timeRowOrCol);
    	else
    		row = getColumnNumberFromCellAddress(timeRowOrCol+"1");
    	Iterator<Row> rowIt = sheet.rowIterator();
    	while (rowIt.hasNext()) {
    		aRow = rowIt.next();
    		if (aRow.getRowNum()+1 == row) {
    			break;
    		}
    	}
    	
    	if (byRow) {
    		return getValuesInRow(aRow, getColumnNumberFromCellAddress(cell));
    	} else {
    		return getValuesInColumn(aRow, rowIt, getColumnNumberFromCellAddress(cell));
    	}
		 */
		
		

		boolean byRow = isInteger(timeRowOrCol);
		double previousValue =-1.0;
		int row;
		int col;
		double value;

		if (byRow) {
			row = Integer.parseInt(timeRowOrCol);
			col = startCol;
			value = getNumericDataInCell(sheet, row, col++);
			while(value >= 0.0 && value > previousValue) {
				timeList.add(value);
				previousValue = value;
				value = getNumericDataInCell(sheet, row, col++);
			}
		} else {
			row = startRow;
			col = Utilities.convertColumnToNumber(timeRowOrCol);
			value = getNumericDataInCell(sheet, row++, col);
			while(value >= 0.0 && value > previousValue) {
				//    			log.debug("Time: "+(row-1)+" value = "+value);
				timeList.add(value);
				previousValue = value;
				value = getNumericDataInCell(sheet, row++, col);
			}
		}

		timeArray = new double[timeList.size()];
		int pos = 0;
		for (double d : timeList)
			timeArray[pos++] = d;
		return timeArray;
	}
	
	private double[] asArray(List<Double> list) {
    	double[] array = new double[list.size()];
    	for (int i = 0; i < list.size(); i++) {
    		array[i] = list.get(i).doubleValue();
    	}
    	return array;
    }
	
    private double[] getDataValues(XSSFSheet sheet, String timeRowOrCol, String cell) {
    	
       	boolean byRow = isInteger(timeRowOrCol);
    	Row aRow = null;
    	
    	int row = Utilities.getRowFromCellAddress(cell);
    	int col = getColumnNumberFromCellAddress(cell);
    	
    	String contents;

    	// locate starting row and column
    	if (byRow) {
    		// this should be ok. We'll need to verify starting column
//    		row = Integer.parseInt(timeRowOrCol);
    		col = getColumnNumberFromCellAddress(cell);
    		// get the contents of this cell as string
    		contents = getDataInCellAsString(sheet, row, col);
    		if (!Parser.isNumber(contents)) {
    			// try the next row as starting point
    			col += 1;
    			contents = getDataInCellAsString(sheet, row, col);
    			if (!Parser.isNumber(contents)) {
        			// try the next row as starting point
        			System.out.println("Bad data values");
        			return null;
        		}
    		}
    	} else {
    		// first, get Row from cell
    		row = getRowNumberFromCellAddress(cell);
    		col = getColumnNumberFromCellAddress(timeRowOrCol+"1");
    		// get the contents of this cell as string
    		contents = getDataInCellAsString(sheet, row, col);
    		if (!Parser.isNumber(contents)) {
    			// try the next row as starting point
    			row += 1;
    			contents = getDataInCellAsString(sheet, row, col);
    			if (!Parser.isNumber(contents)) {
        			// try the next row as starting point
        			System.out.println("Bad data values");
        			return null;
        		}
    		}
    	}

    	// locate starting row
    	
    	
    	Iterator<Row> rowIt = sheet.rowIterator();
    	while (rowIt.hasNext()) {
    		aRow = rowIt.next();
    		if (aRow.getRowNum()+1 == row) {
    			break;
    		}
    	}
    	
    	if (byRow) {
    		return getValuesInRow(aRow, getColumnNumberFromCellAddress(cell));
    	} else {
    		return getValuesInColumn(aRow, rowIt, getColumnNumberFromCellAddress(cell));
    	}
    }
    
    public TimeSeriesInstance GETXLSDATA(String varName, double currentValue, double time, double timeStep, String book,String tab,String timeRowOrCol,String cell) {
		
    	TimeSeriesInstance tsi = null;

    	XSSFWorkbook wb = getWorkbook(book);
    	XSSFSheet sheet = wb.getSheet(tab);

    	double[] timeValues = getTimeValues(sheet, timeRowOrCol, cell);

    	double[] fromXls = getDataValues(sheet, timeRowOrCol, cell);

    	tsi = new TimeSeriesInstance(timeValues, fromXls);

    	return tsi;
}
    
    private double[] getTimeValues(XSSFSheet sheet, String timeRowOrCol, String cell) {
    	
    	// example: GET XLS DATA( 'All-Data-Monthly E.xlsx', 'PP-Analysis-Monthly-Totals', 'A', 'Q2' )
    	// 'A' tells us time values are in Column A
    	// 'Q2' tells us the data starts in row 2, but in this example actually points to variable name not value
    	// this should also tell us to start looking for time in row 2
    	// try looking to see if numeric or not...  // getDataInCellAsString(sheet, int row, int col)
    	
    	boolean byRow = isInteger(timeRowOrCol);
    	
    	Row aRow = null;
    	int row;
    	int col;
    	String contents;

    	// locate starting row and column
    	if (byRow) {
    		// this should be ok. We'll need to verify starting column
    		row = Integer.parseInt(timeRowOrCol);
    		col = getColumnNumberFromCellAddress(cell);
    		// get the contents of this cell as string
    		contents = getDataInCellAsString(sheet, row, col);
    		if (!Parser.isNumber(contents)) {
    			// try the next row as starting point
    			col += 1;
    			contents = getDataInCellAsString(sheet, row, col);
    			if (!Parser.isNumber(contents)) {
        			// try the next row as starting point
        			System.out.println("Bad time values");
        			return null;
        		}
    		}
    	} else {
    		// first, get Row from cell
    		row = getRowNumberFromCellAddress(cell);
    		col = getColumnNumberFromCellAddress(timeRowOrCol+"1");
    		// get the contents of this cell as string
    		contents = getDataInCellAsString(sheet, row, col);
    		if (!Parser.isNumber(contents)) {
    			// try the next row as starting point
    			row += 1;
    			contents = getDataInCellAsString(sheet, row, col);
    			if (!Parser.isNumber(contents)) {
        			// try the next row as starting point
        			System.out.println("Bad time values");
        			return null;
        		}
    		}
    	}
    	Iterator<Row> rowIt = sheet.rowIterator();
    	while (rowIt.hasNext()) {
    		aRow = rowIt.next();
    		if (aRow.getRowNum()+1 == row) {
    			break;
    		}
    	}
    	
    	if (byRow) {
    		return getValuesInRow(aRow, col);
    	} else {
    		return getValuesInColumn(aRow, rowIt, col);
    	}
    }
    
//	public double[] GETXLSLOOKUPS(String book,String tab, String cell, int numRows, int numColumns) {
//		double[] fromXls = new double[numRows*numColumns];
//		
//		boolean transpose = cell.endsWith("*");
//		String tcell = cell.replace("*", "");
//		int startRow =  getRowNumberFromCellAddress(tcell);
//		int startCol =  getColumnNumberFromCellAddress(tcell);
//		XSSFWorkbook wb = getWorkbook(book);
//		XSSFSheet sheet = wb.getSheet(tab);
//		int pos = 0;
//		
//		int nRows = numRows;
//		int nCols = numColumns;
//		
//		if (transpose) {
//			nRows = numColumns;
//			nCols = numRows;
//		}
//		
//		for (int r = startRow; r < startRow + nRows; r++) {
//			for (int c = startCol; c < startCol + nCols; c++) {
//				fromXls[pos++] = getNumericDataInCell(sheet, r, c);
//			}
//		}
//
//		return fromXls;
//
//	}
    
    public double[][][] GETXLSLOOKUPS(String book,String tab,String timeRowOrCol,String cell, int numLookups) {

    	//    	public double[][] getAsArray() {
    	//    	    double[][] look = new double[2][];
    	//    	    look[0] = xVals;
    	//    	    look[1] = yVals;
    	//    	    
    	//    	    return look;
    	//    	}

    	double[][][] table = new double[numLookups][][];
    	XSSFWorkbook wb = getWorkbook(book);
    	XSSFSheet sheet = wb.getSheet(tab);

    	boolean byRow = isInteger(timeRowOrCol);

    	int startRow =  getRowNumberFromCellAddress(cell);
    	int startCol =  getColumnNumberFromCellAddress(cell);

    	double[] timeValues = getTimeValues(sheet, timeRowOrCol, startRow, startCol);
    	int num = timeValues.length;

    	int r = 0;
    	int c = 0;

    	for (int look = 0; look < numLookups; look++) {

    		table[look] = new double[2][];
    		double[] fromXls = new double[num];

    		int pos = 0;
    		if (byRow) {
    			r = startRow+look; 
    			for ( c = startCol; c < startCol + num; c++) {
    				fromXls[pos++] = getNumericDataInCell(sheet, r, c);
    			}

    		} else {
    			c = startCol + num; 
    			for (r = startRow; r < startRow + timeValues.length; r++) {
    				fromXls[pos++] = getNumericDataInCell(sheet, r, c);
    			}

    		}
    		
    		for (int x = 0; x < num; x++) {
    			table[look][0][x] = timeValues[x];
    		}
    		
    		for (int x = 0; x < num; x++) {
    			table[look][1][x] = fromXls[x];
    		}
    	}

    	return table;

    }
    
    public double[][] GETXLSLOOKUPS(String book,String tab,String timeRowOrCol,String cell) {

    	double[][] table = new double[2][];
    	XSSFWorkbook wb = getWorkbook(book);
    	XSSFSheet sheet = wb.getSheet(tab);

    	boolean byRow = isInteger(timeRowOrCol);

    	int startRow =  getRowNumberFromCellAddress(cell);
    	int startCol =  getColumnNumberFromCellAddress(cell);

    	double[] timeValues = getTimeValues(sheet, timeRowOrCol, startRow, startCol);
    	int num = timeValues.length;

    	double[] fromXls = new double[num];

    	int pos = 0;
    	if (byRow) {
    		int r = startRow; 
    		for (int c = startCol; c < startCol + num; c++) {
    			fromXls[pos++] = getNumericDataInCell(sheet, r, c);
    		}

    	} else {
    		int c = startCol; 
    		for (int r = startRow; r < startRow + num; r++) {
    			fromXls[pos++] = getNumericDataInCell(sheet, r, c);
    		}

    	}
    	table[0] = timeValues;
    	table[1] = fromXls;;

    	return table;

    }
    	

	public TimeSeriesInstance GETXLSDATA(String book,String tab,String timeRowOrCol,String cell) {

		TimeSeriesInstance tsi = null;

    	XSSFWorkbook wb = getWorkbook(book);
    	XSSFSheet sheet = wb.getSheet(tab);

    	double[] timeValues = getTimeValues(sheet, timeRowOrCol, cell);

    	double[] fromXls = getDataValues(sheet, timeRowOrCol, cell);

    	tsi = new TimeSeriesInstance(timeValues, fromXls);

    	return tsi;
	}
	
	public TimeSeriesInstance GETDIRECTDATA(String book,String tab,String timeRowOrCol,String cell) {

		TimeSeriesInstance tsi = null;

    	XSSFWorkbook wb = getWorkbook(book);
    	XSSFSheet sheet = wb.getSheet(tab);

    	double[] timeValues = getTimeValues(sheet, timeRowOrCol, cell);

    	double[] fromXls = getDataValues(sheet, timeRowOrCol, cell);

    	tsi = new TimeSeriesInstance(timeValues, fromXls);

    	return tsi;
	}

//	public double GETXLSDATA(String varName, double currentValue, double time, double timeStep, String book,String tab,String timeRowOrCol,String cell) {
//
//		// this needs to return
//
//		notImplemented();
//		return -999.0;
//	}
	
	public static TimeSeriesInstance GETXLSLOOKUPS(XSSFWorkbook wb, String tab, String timeRowOrCol, int startRow, int startCol, int numRows, int numColumns) {
		return GETXLSDATA(wb, tab, timeRowOrCol, startRow, startCol , numRows, numColumns);
	}
	
	public static TimeSeriesInstance GETXLSDATA(XSSFWorkbook wb, String tab, String timeRowOrCol, int startRow, int startCol, int numRows, int numColumns) {

		TimeSeriesInstance tsi = null;

		XSSFSheet sheet = wb.getSheet(tab);
		if (sheet == null)
			return tsi;

		// for now, assume there is only a single subscript being applied
		// numCols in this case is the number of colums or rows that contain data points
		// still need to determine the number of time values in the series

		boolean byRow = isInteger(timeRowOrCol);

		double[] timeValues = getTimeValues(sheet, timeRowOrCol, startRow, startCol);

		double[] fromXls = new double[numRows*numColumns*timeValues.length];

		int pos = 0;
		if (byRow) {
			for (int r = startRow; r < startRow + numRows; r++) {
				for (int c = startCol; c < startCol + timeValues.length; c++) {
					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		} else {
			for (int c = startCol; c < startCol + numColumns; c++) {
				for (int r = startRow; r < startRow + timeValues.length; r++) {

					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		}

		tsi = new TimeSeriesInstance(timeValues, fromXls);

		return tsi;
	}

	public TimeSeriesInstance GETXLSLOOKUPS(String book, String tab, String timeRowOrCol, int startRow, int startCol, int numRows, int numColumns) {
		return GETXLSDATA(book, tab, timeRowOrCol, startRow, startCol , numRows, numColumns);
	}

	public TimeSeriesInstance GETXLSLOOKUPS(String book, String tab, String timeRowOrCol, String cell, int numRows, int numColumns) {
		return GETXLSDATA(book, tab, timeRowOrCol, cell, numRows, numColumns);
	}

	public TimeSeriesInstance GETXLSDATA(String book, String tab, String timeRowOrCol, int startRow, int startCol, int numRows, int numColumns) {

		TimeSeriesInstance tsi = null;

		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);
		if (sheet == null)
			return tsi;

		// for now, assume there is only a single subscript being applied
		// numCols in this case is the number of colums or rows that contain data points
		// still need to determine the number of time values in the series

		boolean byRow = isInteger(timeRowOrCol);

		double[] timeValues = getTimeValues(sheet, timeRowOrCol, startRow, startCol);

		double[] fromXls = new double[numRows*numColumns*timeValues.length];

		int pos = 0;
		if (byRow) {
			for (int r = startRow; r < startRow + numRows; r++) {
				for (int c = startCol; c < startCol + timeValues.length; c++) {
					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		} else {
			for (int c = startCol; c < startCol + numColumns; c++) {
				for (int r = startRow; r < startRow + timeValues.length; r++) {

					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		}

		tsi = new TimeSeriesInstance(timeValues, fromXls);

		return tsi;
	}

	public TimeSeriesInstance GETXLSDATA(String book, String tab, String timeRowOrCol, String cell, int numRows, int numColumns) {

		TimeSeriesInstance tsi = null;

		XSSFWorkbook wb = getWorkbook(book);
		XSSFSheet sheet = wb.getSheet(tab);

		// for now, assume there is only a single subscript being applied
		// numCols in this case is the number of colums or rows that contain data points
		// still need to determine the number of time values in the series

		boolean byRow = isInteger(timeRowOrCol);

		int startRow =  getRowNumberFromCellAddress(cell);
		int startCol =  getColumnNumberFromCellAddress(cell);

		double[] timeValues = getTimeValues(sheet, timeRowOrCol, startRow, startCol);

		double[] fromXls = new double[numRows*numColumns*timeValues.length];

		int pos = 0;
		if (byRow) {
			for (int r = startRow; r < startRow + numRows; r++) {
				for (int c = startCol; c < startCol + timeValues.length; c++) {
					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		} else {
			for (int c = startCol; c < startCol + numColumns; c++) {
				for (int r = startRow; r < startRow + timeValues.length; r++) {

					fromXls[pos++] = getNumericDataInCell(sheet, r, c);
				}
			}
		}

		tsi = new TimeSeriesInstance(timeValues, fromXls);

		return tsi;
	}

	public double RANDOM01() {
		return Math.random();
	}
	
	public double RANDOMPOISSON(String varName, double currentValue, double time, double timeStep,double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
		if (poissonDistribution == null) {
			poissonDistribution = new Poisson(arg3, new MersenneTwister());
		}
		return poissonDistribution.nextDouble();
	    }

	public double RANDOMUNIFORM(String varName,  double currentValue, double time, double timeStep, double arg1, double arg2, double arg3) {
		if (uniformDistribution == null) {
			uniformDistribution = new Uniform(arg1, arg2, (int) arg3);
		}
		return uniformDistribution.nextDouble();
	}

	public double RANDOMNORMAL(double arg1, double arg2, double arg3, double arg4, double arg5) {
		if (normalDistribution == null) {
//			normalDistribution = new Normal(arg1, arg2, new MersenneTwister());
			normalDistribution = new Normal(arg3, arg4, new MersenneTwister());
		}
		double next = normalDistribution.nextDouble();
		while (next < arg1 || next > arg2) 
			next = normalDistribution.nextDouble();
		return next;
	}

	public double RANDOMTRIANGULAR(double arg1,double arg2,double arg3,double arg4,double arg5,double arg6) {
		if (triangularDistribution == null) {
			triangularDistribution = new TriangularDistribution(arg3, arg4, arg5);
		}
		return triangularDistribution.sample();
	}

	public double DOMINANTPATH(String varArgMarker, double tpm,  double...ds) {

		double path = 1.0;
		double dominantPath = 0.0;
		double maxAbsValue = Double.MIN_VALUE;
		boolean increasing = tpm >= 0;

		for (double d : ds) {
			if (increasing) {
				if (d >= 0.0 && Math.abs(d) > maxAbsValue) {
					maxAbsValue = Math.abs(d);
					dominantPath = path;
				}
			} else {
				if (d < 0.0 && Math.abs(d) > maxAbsValue) {
					maxAbsValue = Math.abs(d);
					dominantPath = path;
				}
			}
			path += 1.0;
		}

		return dominantPath;

	}
	
	public double QUANTUM(double arg1,double arg2) {
		
		if (arg2 <= 0.0)
			return arg1;
		
		int q = (int) (arg2 * (int) (arg1/arg2));
		
		return (double) q;

	}
	
	public double GETSUBSCRIPTVALUE(String subr) {
		return 0.0;
	}
	
    
	public void dumpSheet(XSSFWorkbook wb, String sheet, BufferedWriter bw) {
		try {

			XSSFSheet aSheet = wb.getSheet(sheet);


			Iterator<Row> rowIt = aSheet.rowIterator();
			while (rowIt.hasNext()) {
				Row aRow = rowIt.next();
				Iterator<Cell> cellIt = aRow.cellIterator();
				while (cellIt.hasNext()) {
					Cell aCell = cellIt.next();
					bw.append(sheet+",");
					bw.append((aRow.getRowNum()+1)+","+(aCell.getColumnIndex()+1)+",");
					
					int rowNum = aRow.getRowNum()+1;
					int colNum = aCell.getColumnIndex()+1;


					bw.append(sheet+"_R"+(aRow.getRowNum()+1)+"C"+(aCell.getColumnIndex()+1)+",");
					if (aCell.getCellType() == CellType.STRING ) {
						bw.append("\""+aCell.getRichStringCellValue().toString()+"\",");
						bw.append("Constant");

					} else if ( 
							aCell.getCellType() == CellType.FORMULA && 
							aCell.getCachedFormulaResultType() == CellType.STRING) {
						bw.append("\""+aCell.getRichStringCellValue().toString()+"\",");
						String theFormula = aCell.getCellFormula();
						if (theFormula.startsWith("CELL"))
							theFormula = "CELL(address reference)";
						bw.append("\""+theFormula+"\"");
					} else if ( 
							aCell.getCellType() == CellType.FORMULA && 
							aCell.getCachedFormulaResultType() == CellType.NUMERIC) {
						bw.append("\""+aCell.getNumericCellValue()+"\",");
						String theFormula = "PROBLEM!";
							try {
								theFormula = aCell.getCellFormula();
							} catch (Exception e) {
								// TODO Auto-generated catch block
//								e.printStackTrace();
							}
							
						if (theFormula.startsWith("CELL"))
							theFormula = "CELL(address reference)";
						bw.append("\""+theFormula+"\"");
						
					} else {
						bw.append("\""+Double.toString(aCell.getNumericCellValue())+"\",");
						bw.append("Constant");
					}
					bw.append("\n");
					bw.flush();
				}
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}


}
