package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.apache.log4j.Logger;


public class SDModelWithPropertiesVDM_Native extends SDModel{
	static Logger log = Logger.getLogger(SDModelWithPropertiesVDM_Native.class.getSimpleName());
	
	protected static final double INITIAL_CONSTANT_VALUE = Double.MIN_VALUE;
	
	private boolean printGetSet = false;
	
	private Set<String> SKIP_LOGGING = new LinkedHashSet<String>();

	public static Properties PROPERTIES = new Properties();
	private static String DEFAULT_PROPERTIES = "DefaultRunner.properties";
	private static String MISC_DIRECTORY = "miscOutput/";
	private static String SUBSCRIPT_FILE = "SubscriptSpace_";
	
	protected Map<String, Double> time0DataSets = new LinkedHashMap<String, Double>(); 

	protected LoggerJava logger = new LoggerJava();
	
	protected boolean resaveConstants = false;
	protected boolean reinitialize = false;

	protected String name;

	public static String SEPARATOR = "\t";
	public static final boolean trace = false;
	private String lastVar = "";


	protected VDM vdm = null;
	
	public double[] cMem;
	public double[] vMem;
	
	public ExtendedMemoryDatabase extendedMemory;
	
	// get rid of these

//	protected Map<String, Double> mapMemory = new LinkedHashMap<String, Double>();
//	protected Map<String, List<Double>> mapMemoryHistory = new LinkedHashMap<String, List<Double>>();
//	protected boolean mapMemoryHistoryInitialized = false;
//	protected Map<String, double[][]> mapMemoryLookup = new LinkedHashMap<String, double[][]>();
//	protected Map<String, double[]> mapMemoryVector = new LinkedHashMap<String, double[]>();
	
	// END get rid of these
	
	
	
	protected List<Double> timeVector = new ArrayList<Double>();
	protected Map<String, Map<Integer, List<String>>> allocatedIndicies = new LinkedHashMap<String, Map<Integer, List<String>>>();

	public SDModelWithPropertiesVDM_Native(String name) {
		super(name, null);
	}
	public SDModelWithPropertiesVDM_Native(String name, String[] args) {
		super(name, args);

		if (args != null && args.length > 0)
			loadProperties(args[0]);
		else
			loadProperties();
		
		SKIP_LOGGING.add("TIME_STEP");
		SKIP_LOGGING.add("NAREPLACEMENT");
		SKIP_LOGGING.add("time");
		SKIP_LOGGING.add("Time");
		
		sdFunctions = new SDFunctionsWithXLSColt(this);
		message = new MessageJava();
		results = new ResultsReporterJava();
		timeSeriesData.setNativeDataTypes(true);
		
		OUTPUT_DIRECTORY = PROPERTIES.getProperty("outputDirectory");
		DATA_DIRECTORY = PROPERTIES.getProperty("dataDirectory");

		this.name = name;

		extendedMemory = new ExtendedMemoryDatabase(
					args[0], // "C:/Users/bragen/Desktop/SDM-UI/workspace/Models/world2/world2.map",
					args[1], // "C:/Users/bragen/Desktop/SDM-UI/workspace/Models/world2/world2_DOC_GRAPH_/runs/baselineHSQLDB",
					Double.parseDouble(args[2]) // 0.25
					);



	}
	
	public void reinitializeConstantsMemory() {
		int size = cMem.length;
		cMem = new double[size];
		for (int i = 0; i < cMem.length; i++)
			cMem[i] = INITIAL_CONSTANT_VALUE;
	}
	
	public void initializeConstantsMemory(int size) {
		System.out.println("### Constants Memory size = "+size+" ###");
		cMem = new double[size];
		for (int i = 0; i < cMem.length; i++)
			cMem[i] = INITIAL_CONSTANT_VALUE;
	}
	
	public double initializeConstant(int index, double value) {
		if (cMem[index] == INITIAL_CONSTANT_VALUE) {
//			System.out.println("resetting "+index+" to "+value+" currently "+cMem[index]);
			cMem[index] = value;
			
		} else {
//			System.out.println("Not resetting "+index+" to "+value+" currently "+cMem[index]);
		}
		return value;
	}
	
	public boolean debugValue(String lhs, double value) {
		System.out.println("DV: ("+currentTime+") "+lhs+ " = "+NumberFormatter.formatForSmall(value));
		boolean bad = false;
		if (Double.isNaN(value) || Double.isInfinite(value)) {
			bad = true;
			System.out.println(" >>>> "+lhs+" = "+NumberFormatter.formatForSmall(value)+" -> Bad");
		}
		return bad;
	}
	
//	public double[] getVarsValue(String[] vars, double time) {
//		return extendedMemory.
//	}
//	public double[][] getVarsValue(String[] vars, double starttime, double endtime);
//	public double[][] getVarsValue(String[] vars);
	
	public void setRunParameters(double initialTime, double finalTime, double dt, double savper) {
		extendedMemory.setRunParameters(initialTime, finalTime, dt, savper);
	}


	@Override
	public void execute() {

		log.debug("SDModelWithPropertiesVDM_Native: execute()");

		//            logger.setLogFile(name+"Log.csv");

		Initializer.initialize();
		initializeVDM();
		if (vdm != null)
			vdm.advanceTime();

		oneTime();
		int tick = 0;
		for (double time = getINITIALTIME(); time <= getFINALTIME(); time += getTIMESTEP()) {
			data.setCurrentTime(time);
			currentTime = time;
			Synchronizer.synchronize(currentTime, getTIMESTEP());
			repeated(time, getTIMESTEP());
			reportTimeStep(time);

			if (dumpMemory) {
				// dump memory at the end of each time tick
				String filename = "_memoryDump_"+(tick++)+".csv";
				results.writeReport(RunnerConstants.OUTPUT_DIRECTORY+name+filename, data);
			}

			if (vdm != null)
				vdm.advanceTime();
		}
		//    	    results.writeReport(RunnerConstants.OUTPUT_DIRECTORY+name+"_sdReport.csv", data);
		log.debug("SDModelWithPropertiesVDM_Native: logger.close()");

		logger.close();
	}

	public void endSimulation() {
		log.debug("### End of Simulation Processing ###");
		finish();
	}

	public void finish() {
		log.debug("SDModelWithPropertiesVDM_Native: logger.close()");
		logger.close();
		sdFunctions.closeWorkbooks();

		// convert log file to tabular form

		LogConverter logConverter = new LogConverter();
		logConverter.convert(new String[] {name+"Log.tab", MISC_DIRECTORY+SUBSCRIPT_FILE+name+".csv", name+"Tablular.tab"});

	}

	//        protected void logit(String var, double time, double value) {
	//        	
	//        	BigDecimal bd = new BigDecimal(value);
	//        	bd = bd.round(new MathContext(6));
	//        	double d = bd.doubleValue();
	//        	logger.log(var+","+time+","+d);
	////            logger.log(var+","+time+","+value);
	//        }


	protected void logit(String var, double[] history) {
		StringBuilder sb = new StringBuilder();
		sb.append(var);
		for (double d : history) {
			sb.append(SEPARATOR);
			if (Double.isNaN(d) || Double.isInfinite(d)) {
				sb.append(d);
			} else {
				BigDecimal bd = new BigDecimal(d);
				bd = bd.round(new MathContext(7));
				double db = bd.doubleValue();
				sb.append(bd);
			}
		}
		
		logger.log(sb.toString());
	}
	
	
	protected void logit(String var, String time, String value) {
		logger.log(var+SEPARATOR+time+SEPARATOR+value);
	}
	protected void logit(String var, double time, double value, double savper) {
		int t = (int) (time/savper);
//		var = null;
		double remainder = (time - (((double) t) * savper));
		if (remainder == 0.0) {
//			if (trace) {
//				String v = var.split("\\[")[0];
//				if (!v.equals(lastVar))
//					log.debug("log: "+var);
//				lastVar = v;
//			}
			if (Double.isNaN(value) || Double.isInfinite(value)) {
				logger.log(var.replace("memory.", "")+SEPARATOR+time+SEPARATOR+value);
			} else {
				BigDecimal bd = new BigDecimal(value);
				bd = bd.round(new MathContext(7));
				double d = bd.doubleValue();
				logger.log(var.replace("memory.", "")+SEPARATOR+time+SEPARATOR+d);
			}
		}
	}

	protected void logitVector(String var, double time, int length, double[] value) {
		logger.log(var+SEPARATOR+time+SEPARATOR+value);
	}

	public boolean loadProperties(String file) {
		File props = new File(file);
		if (props.exists()) {
			try {
				PROPERTIES.load(new FileInputStream(props));
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			return true;
		} else {
			return false;
		}
	}

	public boolean loadProperties() {
		return loadProperties(DEFAULT_PROPERTIES);
	}

	private void initializeVDM() {
		String vdmFile = SDModelWithPropertiesVDM_Native.PROPERTIES.getProperty("vdmfile");
		if (vdmFile.length() > 0) {
			vdm = new VDM(this);
			vdm.setFile(vdmFile);
			vdm.loadFromFile();
		}


	}

	public void done() {
		
		if (logger != null) {
			logger.close();
		}
		
		if (extendedMemory != null) {
			extendedMemory.close();
		}
			
	}
	
	private void getHistoryToPrint(double[] historyToPrint, List<Double> history) {
		int index = 0;
		int arrayIndex = 0;
		for (double time = initialTime; time <= finalTime; time +=dt) {
			// this may be a partial run, stop short if that is the case
			if (index >= history.size())
				break;
			int t = (int) (time/saveper);
			double remainder = (time - (((double) t) * saveper));
			if (remainder == 0.0) {
				historyToPrint[arrayIndex++] = history.get(index);
			}
			index++;
		}
	}
	
//	public void logMemoryOnceByRow() {
//		// called at the end of repeated (one call per dt)
//		int numVals = (int) ((finalTime - initialTime) / dt) +1;
//		List<String> al = new ArrayList<String>(mapMemory.keySet());
//		Collections.sort(al);
//		
//		double[] historyToPrint;
//		List<Double> history;
//		
//		history = mapMemoryHistory.get("Time");
//		
//		historyToPrint = new double[numVals];
//		getHistoryToPrint(historyToPrint, history);
//		logit("Time", historyToPrint);	
//		
//		for (String key : al) {
//			// some variables that we do not want to log
//			if (SKIP_LOGGING.contains(key))
//				continue;
//			history = mapMemoryHistory.get(key);
//			historyToPrint = new double[numVals];
//			getHistoryToPrint(historyToPrint, history);
//			logit(key, historyToPrint);			
//		}
//	}

//	public void logMemoryOnce() {
//
//		// called at the end of repeated (one call per dt)
//
//		List<String> al = new ArrayList<String>(mapMemory.keySet());
//		Collections.sort(al);
//
//		int index = 0;
//		int numVals = (int) ((finalTime - initialTime) / dt) +1;
//		for (double time = initialTime; time <= finalTime; time +=dt) {
//			for (String key : al) {
//				// some variables that we do not want to log
//				if (SKIP_LOGGING.contains(key))
//					continue;
//				List<Double> history = mapMemoryHistory.get(key);
//
//				if (index < history.size())
//					logit(key, time, mapMemoryHistory.get(key).get(index), saveper);
//				//			} else {
//				//				logit(key, time, mapMemoryHistory.get(key).get(0), saveper);
//				//			}
//			}
//			index++;
//		}
//	}
	
//	public void initializeMemoryMapHistory() {
//		
//		List<String> al = new ArrayList<String>(mapMemory.keySet());
//		for (String key : al) {
//			if (!mapMemoryHistory.containsKey(key)) 
//				mapMemoryHistory.put(key, new ArrayList<Double>());
//			mapMemoryHistory.get(key).add(mapMemory.get(key));
//		}
//		
//		mapMemoryHistoryInitialized = true;
//		logit("Variable", "Time", "Value");
//		logMemoryToHistory();
//		
//	}
	public void logMemoryToHistory() {
//		
//		// called at the end of repeated (one call per dt)
//		
//		List<String> al = new ArrayList<String>(mapMemory.keySet());
//		for (String key : al) {
////			mapMemoryHistory.get(key).add(mapMemory.get(key));
//			logit(key, currentTime, mapMemory.get(key), saveper);
//		}
//		
////		System.out.println("logMemoryToHistory currentTime = "+currentTime + " "+
////				mapMemoryHistory.get("Time").size());
	}
	public double[] getForTimeStep(String var) {
		double[] vals = new double[(int) (1/this.saveper)];
//		List<Double> v = mapMemoryHistory.get(var);
//		for (int i = 0; i < vals.length; i++) {
//			if (v.size() == 1) {
//				vals[i] = v.get(0);
//			} else {
//				vals[i] = v.get(v.size() - vals.length + i);
//			}
//		}
		return vals;
	}
	// setViaSDMUI
	
	public void setViaSDMUI(String var, double value) {
		
		MemoryMapperInterface mm = extendedMemory.getMapper();
		int index = mm.getIndex(var);
		System.out.println("setViaSDMUI: "+var+" "+value+" "+mm.getVarType(var)+" "+index);
		setVectorMemory(var, value);
		if (time == getINITIALTIME()) {
			reinitialize = true;
			time0DataSets.put(var, value);
//			extendedMemory.reinitialize();
//			this.sdmuiExecuteStep1();
//			this.sdmuiExecuteStep2();
		} else {
			resaveConstants = true;
		}
	}
	
	public void reinitializeRun() {
		extendedMemory.reinitialize();
		reinitializeConstantsMemory();
		for (String var : time0DataSets.keySet()) {
			setVectorMemory(var, time0DataSets.get(var).doubleValue());
		}
		
		this.sdmuiExecuteStep1();
		this.sdmuiExecuteStep2();
		reinitialize = false;
	}
	
	public void set(String var, double value) {
		setVectorMemory(var, value);
	}
	
	public void setVectorMemory(String var, double value) {
		MemoryMapperInterface mm = extendedMemory.getMapper();
		int index = mm.getIndex(var);
		if (mm.getVarType(var).equals(MemoryMapper.CONSTANT)) {
			
			cMem[index] = value;
		} else {
			
			vMem[index] = value;
		}
	}

	public double get(String var, double time) {
		return getVectorMemory(var, time);
//		if (mapMemoryHistory.containsKey(var)) {
//			if (mapMemoryHistory.get(var).size() == 1) {
//				return mapMemoryHistory.get(var).get(0);
//			} else {
//				int index = mapTimeToIndex(time);
//				return mapMemoryHistory.get(var).get(index);
//			}
//		} else {
//			return 0.0;
//		}
	}
	
	public double[] get(String[] vars, double time) {
		return getVectorMemory(vars, time);
	}
	
	public double[] getVectorMemory(String[] vars, double time) {
		double d[] = extendedMemory.getVarsValue(vars, time);
		return d;
	}
	
	public double getVectorMemory(String var, double time) {
		double d = extendedMemory.getVarAtTime(var, time);
		return d;
	}
	
	protected int mapTimeToIndex(double time) {
		for (int i = 0; i < getTimeVector().size(); i++)
			if (getTimeVector().get(i).doubleValue() == time)
				return i;
		return 0;
	}
	
	public List<Double> getTimeVector() {

		if (timeVector.size() > 0) {
			return timeVector;
		} else {
			double st = initialTime;
			double ft = finalTime;
			double sav = dt;

			for (double t = st; t <= ft; t += sav) {
				int tm = (int) (t/sav);
				double remainder = (t - (((double) tm) * sav));
				if (remainder == 0.0) {
					timeVector.add(t);
				}
			}
			return timeVector;
		}

	}
//	public void setNolog(String var, double value) {
//		mapMemory.put(var, value);
//	}
//
//	public void set(String var, double value[][]) {
//		mapMemoryLookup.put(var, value);
//	}
//	
//	public void set(String var, double value[]) {
////		mapMemoryLookup.put(var, value);
//	}
//
	public double get(String var) {
		return getVectorMemory(var);
//		if (mapMemory.containsKey(var)) {
////			if (printGetSet)
////				System.out.println("Get "+var+" = "+mapMemory.get(var));
//			return mapMemory.get(var);
//		} else {
////			System.out.println("## Init ## "+var);
//			setNolog(var, 0.0);
//			return get(var);
//		}
	}
	
	public double getVectorMemory(String var) {
		MemoryMapperInterface mm = extendedMemory.getMapper();
		int index = mm.getIndex(var);
		if (mm.getEqnType(var).equals(MemoryMapper.CONSTANT)) {
			return cMem[index];
		} else {
			return vMem[index];
		}
	}

//	public double[][] getLookup(String var) {
//		return mapMemoryLookup.get(var);
//	}
//	
//	public double[] getVector(String var) {
//		return mapMemoryVector.get(var);
//	}

	public void loadSubscriptDefinition(BufferedReader br) {
		if (br == null) return;
		List<String> mapInfo = Utilities.getAsList(br);
		mapInfo.remove(0);
		
		Map<String, Set<String>> subscripts = new LinkedHashMap<String, Set<String>>();
		for (String line : mapInfo) {
			String[] p = line.split("\t");
			
			// Name, Value, Index
			String name = p[0];
			String value = p[1];
			
			if (!subscripts.containsKey(name))
				subscripts.put(name, new LinkedHashSet<String>());
			
			subscripts.get(name).add(value);			
		}
		
		for (String name : subscripts.keySet()) {
			sdFunctions.registerSubscript(name, subscripts.get(name).size());
		}
	}
	
	public void loadSubscriptMap(BufferedReader br) {
		if (br == null) return;
		List<String> mapInfo = Utilities.getAsList(br);
		mapInfo.remove(0);
		for (String line : mapInfo) {
			String[] p = line.split("\t");
			if (!allocatedIndicies.containsKey(p[0])) {
				allocatedIndicies.put(p[0], new LinkedHashMap<Integer, List<String>>());
			}
			Map<Integer, List<String>> array = allocatedIndicies.get(p[0]);
			int subscriptIndex = Integer.parseInt(p[1]);
			if (!array.containsKey(subscriptIndex)) {
				array.put(subscriptIndex, new ArrayList<String>());
			}
			List<String> subs = array.get(subscriptIndex);
			subs.add(p[2]);
		}
	}

	public String asExpandedArrayRef(String array, int ... integers) {
//		Map<String, Map<Integer, List<String>>> allocatedIndicies = new LinkedHashMap<String, Map<Integer, List<String>>>();
		int numDimensions = allocatedIndicies.get(array).size();
		StringBuilder sb = new StringBuilder(array);
		sb.append("[");
		for (int i = 0; i < numDimensions; i++) {
			int index = integers[i];
			if (i>0) sb.append(",");
			sb.append(""+allocatedIndicies.get(array).get(i).get(index)+"");
		}
		sb.append("]");
		return sb.toString();
	}
	
	public void sdmuiExecute(String runName, String logDir) {
		sdmuiExecuteStep1(runName, logDir);
		sdmuiExecuteStep2();
		while (currentTime < finalTime) {
			stepMap();
		}
		done();
	}
	
	public void sdmuiExecuteStep1() {
		initialization = true;
		oneTime();
		initialization = false;
	}

	public void sdmuiExecuteStep1(String runName, String logDir) {
		if (logger != null)
			logger.close();
		logger = new LoggerJava();
		logger.setLogFile(logDir+"/"+runName+".tab");
		initialization = true;
		oneTime();
		initialization = false;
	}
	public void sdmuiExecuteStep2() {
		System.out.println("sdmuiExecuteStep2 in");
		repeated(time, timeStep);
		System.out.println("sdmuiExecuteStep2 out");
//		initializeMemoryMapHistory();
	}
	
	public void stepMap() {
		double timeStep = dt;
		for (int i = 0; i < (int) (1/dt); i++) {
			double time = currentTime + timeStep;
			set("Time", time);
			currentTime = time;
			repeated(time, timeStep);
		}
	}
	
	public void backstepMap() {
		
		for (int i = 0; i < (int) (1/dt); i++) {
			backstep();
		}
		resetMemoryFromHistory();
		currentTime = currentTime - 1;
		time = time - 1;
	}
	
	public double getCurrentTime() {
		return currentTime;
	}
	
	public void backstepExtendedMemory(double time) {
		
		double backTime = currentTime - time;
		int numDT = (int) (backTime/dt);
		for (int i = 0; i < numDT; i++) {
			sdFunctions.backstep();
		}
		
		extendedMemory.backstepToTime(time, cMem, vMem);
		
		currentTime = time;
		this.time = time;
	}
	
	private void backstep() {
//		List<Double> al = mapMemoryHistory.get("Time");
//		int index = al.size()-1;
//		for (String var : mapMemoryHistory.keySet()) {
//			List<Double> al2 = mapMemoryHistory.get(var);
//			if (index >= al2.size()) {
//				// this is a constant value
////				System.out.println(" backstep "+var+" "+al2.size());
//			} else {
//				al2.remove(index);
//			}
//		}
		sdFunctions.backstep();
	}
//	
	private void resetMemoryFromHistory() {
//		int index = mapMemoryHistory.get("Time").size()-1;
//		for (String var : mapMemoryHistory.keySet()) {
//			List<Double> al2 = mapMemoryHistory.get(var);
//			if (index >= al2.size()) {
//				setNolog(var, al2.get(0));
//			} else {
//				setNolog(var, al2.get(index));
//			}
//		}
	}
	
	public double[] getValuesUpToTime(String var, double time) {
		return getValuesUpToTimeVectorMemory(var, time);
	}
	
	public double[] getValuesUpToTimeVectorMemory(String var, double time) {
		double[] v = extendedMemory.getVarTimeVector(var, time);		
		return v;
		
		
	}
	
	public double[] genVector(double[] vectorMemory, int start, int length) {
		double[] vector = new double[length];
		int v = 0;
		for (int i = start; i < start + length; i++) {
			vector[v++] = vectorMemory[i];
		}
		return vector;
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][] matrix) {
		int v = start;
		int numRow = matrix.length;
		int numCol = matrix[0].length;
		for (int r = 0; r < numRow; r++) {
			for (int c = 0; c < numCol; c++) {
			vectorMemory[v++] = matrix[r][c];
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					vectorMemory[v++] = matrix[d1][d2][d3];
				}
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					for (int d4 = 0; d4 < matrix[0][0][0].length; d4++) {
						vectorMemory[v++] = matrix[d1][d2][d3][d4];
					}
				}
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					for (int d4 = 0; d4 < matrix[0][0][0].length; d4++) {
						for (int d5 = 0; d5 < matrix[0][0][0][0].length; d5++) {
							vectorMemory[v++] = matrix[d1][d2][d3][d4][d5];
						}
					}
				}
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][][][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					for (int d4 = 0; d4 < matrix[0][0][0].length; d4++) {
						for (int d5 = 0; d5 < matrix[0][0][0][0].length; d5++) {
							for (int d6 = 0; d6 < matrix[0][0][0][0][0].length; d6++) {
								vectorMemory[v++] = matrix[d1][d2][d3][d4][d5][d6];
							}
						}
					}
				}
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][][][][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					for (int d4 = 0; d4 < matrix[0][0][0].length; d4++) {
						for (int d5 = 0; d5 < matrix[0][0][0][0].length; d5++) {
							for (int d6 = 0; d6 < matrix[0][0][0][0][0].length; d6++) {
								for (int d7 = 0; d7 < matrix[0][0][0][0][0][0].length; d7++) {
								vectorMemory[v++] = matrix[d1][d2][d3][d4][d5][d6][d7];
								}
							}
						}
					}
				}
			}
		}
	}
	
	public void storeMatrix(double[] vectorMemory, int start, double[][][][][][][][] matrix) {
		int v = start;

		for (int d1 = 0; d1 < matrix.length; d1++) {
			for (int d2 = 0; d2 < matrix[0].length; d2++) {
				for (int d3 = 0; d3 < matrix[0][0].length; d3++) {
					for (int d4 = 0; d4 < matrix[0][0][0].length; d4++) {
						for (int d5 = 0; d5 < matrix[0][0][0][0].length; d5++) {
							for (int d6 = 0; d6 < matrix[0][0][0][0][0].length; d6++) {
								for (int d7 = 0; d7 < matrix[0][0][0][0][0][0].length; d7++) {
									for (int d8 = 0; d8 < matrix[0][0][0][0][0][0][0].length; d8++) {
								vectorMemory[v++] = matrix[d1][d2][d3][d4][d5][d6][d7][d8];
									}
								}
							}
						}
					}
				}
			}
		}
	}
	
	
	
	public void storeVector(double[] vectorMemory, int start, double[] vector) {
		int i = 0;
		for (int v = start; v < start + vector.length; v++) {
			vectorMemory[v] = vector[i++];
		}
	}
	
	public double[][] genMatrix(double[] vectorMemory, int start, int numRow, int numCol) {
		double[][] matrix = new double[numRow][numCol];
		int i = start;
		for (int r = 0; r < numRow; r++) {
			for (int c = 0; c < numCol; c++) {
				matrix[r][c] = vectorMemory[i++];
			}
		}
		return matrix;
	}
	
	@Override
	public void saveLookupValues(String var) {
		extendedMemory.writeLookupValues(var, sdFunctions.getLookupValues(var));
	}
	
	protected void saveMemory() {
		
//		System.out.println("saveMemory: "+time);
		
		if (time == getINITIALTIME() || resaveConstants) {
			extendedMemory.writeCMem(time, cMem);
			resaveConstants = false;
		}
		
		if (time == getINITIALTIME()) {
			extendedMemory.writeVMem(time, vMem);
		} else if (time % saveper == 0) {
			extendedMemory.writeVMem(time, vMem);
		}
	}
	public boolean isReinitialize() {
		return reinitialize;
	}


}
