package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.ArrayManager;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.NativeDataTypeManager;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class StreamlinedMemoryMapper  implements MemoryMapperInterface {

	public static final int MAX_ELEMENTS_FOR_RUNNING = 10000;
	public static final String CONSTANT = "C";
	public static final String DYNAMIC = "D";
	
	public static final String STOCK = "S";
	public static final String FLOW = "F";
	public static final String AUXILIARY = "A";
	public static final String LOOKUP = "L";
	
	public static final String ARRAY = "AR";
	public static final String SCALAR = "SC";
	
	private Map<String, MutableInteger> sizeByType = new LinkedHashMap<String, MutableInteger>();

	private Map<String, Equation> equations;
	
	private Map<String, StreamlinedMemoryMapperRecord> dataRecords = 
			new HashMap<String, StreamlinedMemoryMapperRecord>();
	private Map<String, StreamlinedMemoryMapperSubscriptDefinition> subscripts = 
			new HashMap<String, StreamlinedMemoryMapperSubscriptDefinition>();
	
	private Map<String, String> eqnLHS = new LinkedHashMap<String, String>();
	
	private Map<String, String> varType = new LinkedHashMap<String, String>();
	private Map<String, String> eqnType = new LinkedHashMap<String, String>();
	
	private List<String> sortedLHS = new ArrayList<String>();
	private List<String> expandedSortedLHS = new ArrayList<String>();

	private Map<String, Integer> evaluationOrderOneTime = new HashMap<String, Integer>();
	private List<String> evaluationOrderOneTimeList = null;
	private Map<String, Integer> evaluationOrderRepeated = new HashMap<String, Integer>();
	private List<String> evaluationOrderRepeatedList = null;

	// these need to be mapped for Constant and Dynamic
	private Map<String, Map<String, Integer>> offset = new LinkedHashMap<String, Map<String, Integer>>();
	private Map<String, Map<Integer, String>> reverseIndex = new LinkedHashMap<String, Map<Integer, String>>();
	private Map<String, Map<String, Integer>> numElements = new LinkedHashMap<String, Map<String, Integer>>();
	
	private Map<String, MutableInteger> currentOffset = new LinkedHashMap<String, MutableInteger>();
	private Map<String, MutableInteger> currentNumElements = new LinkedHashMap<String, MutableInteger>();
	private Map<String, Map<String, Integer>> expandedIndex = new LinkedHashMap<String, Map<String, Integer>>();
	
	private String mapperFile;
	private MessageProcessor info;

	NativeDataTypeManager ndtm = InformationManagers.getInstance()
			.getNativeDataTypeManager();
	NamedSubscriptManager nsm = InformationManagers.getInstance()
			.getNamedSubscriptManager();
	ArrayManager am = InformationManagers.getInstance().getArrayManager();
	
	public StreamlinedMemoryMapper(MessageProcessor info) {
	// deal with multiple referenced to same array
		this.info = info;
		
		sizeByType.put(CONSTANT, new MutableInteger(0));
		sizeByType.put(DYNAMIC, new MutableInteger(0));
		
		currentOffset.put(CONSTANT, new MutableInteger(0));
		currentOffset.put(DYNAMIC, new MutableInteger(0));
		
		currentNumElements.put(CONSTANT, new MutableInteger(0));
		currentNumElements.put(DYNAMIC, new MutableInteger(0));
		
		offset.put(CONSTANT, new LinkedHashMap<String, Integer>());
		offset.put(DYNAMIC, new LinkedHashMap<String, Integer>());
		
		reverseIndex.put(CONSTANT, new LinkedHashMap<Integer, String>());
		reverseIndex.put(DYNAMIC, new LinkedHashMap<Integer, String>());
		
		numElements.put(CONSTANT, new LinkedHashMap<String, Integer>());
		numElements.put(DYNAMIC, new LinkedHashMap<String, Integer>());
		
		expandedIndex.put(CONSTANT, new LinkedHashMap<String, Integer>());
		expandedIndex.put(DYNAMIC, new LinkedHashMap<String, Integer>());
	}

	public StreamlinedMemoryMapper(String mapperFile, MessageProcessor info) {
		this(info);
		this.mapperFile = mapperFile;
		loadMapper();
	}

	public StreamlinedMemoryMapper(Map<String, Equation> equations, String mapperFile, MessageProcessor info) {
		this(info);
	

		Set<String> processed = new LinkedHashSet<String>();

		this.equations = equations;
		int numEqn = equations.size();
		int interval = Math.max(numEqn / 10, 1);
		int num = 1;
		int pct = 10;
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String s0 : equations.keySet()) {
			
			String s = ArrayReference.deannotate(s0);
			if (ArrayReference.isArrayReference(s)) {
				s = new ArrayReference(s).getArrayName();
			}
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(s); // equations.get(s);
			if (eqn == null) {
				System.out.println("NULL: "+s);
				continue;
			}
			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				pct += 10;
			}
			num++;
			
			if (eqn.isDefinesSubscript())
				continue;
			if (eqn.isDefinesLookup()) {
				eqnType.put(common(s), LOOKUP);
				eqnType.put(common(screenName(s)), LOOKUP);
			} else if (eqn.isStock()) {
				eqnType.put(common(s), STOCK);
				eqnType.put(common(screenName(s)), STOCK);
			} else if (eqn.isFlow()) {
				eqnType.put(common(s), FLOW);
				eqnType.put(common(screenName(s)), FLOW);
			} else {
				eqnType.put(common(s), AUXILIARY);
				eqnType.put(common(screenName(s)), AUXILIARY);
			}

			if (eqn.isHasLHSArrayReference()) {
				ArrayReference ar = eqn.getLhsArrayReference();
				String arrayName = ar.getArrayName();
				if (processed.contains(arrayName))
					continue;
				processed.add(arrayName);
				varType.put(common(arrayName), eqn.isOneTime() ? CONSTANT : DYNAMIC);
				
			}
			eqnLHS.put(screenName(s), s);
			sortedLHS.add(screenName(s));

			String sn = screenName(s);
			
			if (screenName(s).equalsIgnoreCase("Time"))
				varType.put(common(screenName(s)), DYNAMIC);
			else			
				varType.put(common(screenName(s)), eqn.isOneTime() ? CONSTANT : DYNAMIC);
			
			// create our record

			StreamlinedMemoryMapperRecord smmdr = new StreamlinedMemoryMapperRecord(this,
					common(s), varType.get(common(s)), eqnType.get(common(s)), 
					(eqn.isHasLHSArrayReference() ? ARRAY : SCALAR), -1, -1, -1,
					(eqn.isHasLHSArrayReference() ? eqn.getLhsArrayReference().getSubscripts() : null));
			dataRecords.put(sn,  smmdr);
		}
		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
		
		this.mapperFile = mapperFile;
		sortedLHS = sortIgnoreCase(sortedLHS);
		info.addMessage(language.getText(StringConstants.DETERMINING_MEMORY_REQUIREMENTS));
		determineMemoryRequirements();
		updateRecordsWithRequirements();
		info.addMessage(language.getText(StringConstants.EXPANDING_LHS_OF_EQUATIONS));
		generateExpandedSortedLHS();
	}
	
	private void generateExpandedSortedLHS() {
		// expandedSortedLHS
		
	}

	private void updateRecordsWithRequirements() {
		for (String lhs : sortedLHS) {
			StreamlinedMemoryMapperRecord smmr = dataRecords.get(lhs);
			if (!smmr.getVariableType().equals(LOOKUP)) {
				smmr.setOffset(offset.get(smmr.getConstantDynamic()).get(lhs));
			}
				smmr.setNumElements(numElements.get(smmr.getConstantDynamic()).get(lhs));
		}
	}
	
	public Map<String, Map<String, Integer>> getVariableStorageMap() {
		return numElements;
	}


	public List<String> getVariables() {
		return expandedSortedLHS;
	}
	
	public List<String> getStocks() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : sortedLHS) {
			if (eqnType.get(common(s)).equals(STOCK))
				ordered.add(screenName(s));
		}
		
		return ordered;
	}
	
	@Override
	public List<String> getLookups() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : sortedLHS) {
			if (eqnType.get(common(s)).equals(LOOKUP))
				ordered.add(screenName(s));
		}
		
		return ordered;
	}
	
	public List<String> getVariablesOrderedByType() {
		List<String> ordered = new ArrayList<String>();
		
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(STOCK))
				ordered.add(s);
		}
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(FLOW))
				ordered.add(s);
		}
		for (String s : expandedSortedLHS) {
			if (eqnType.get(common(s)).equals(AUXILIARY))
				ordered.add(s);
		}
		
		return ordered;
	}

	private void loadMapper() {
		List<String> lines = Utilities.getAsListCharset(mapperFile);
		
		String[] parts = Parser.splitQuoted(lines.get(0), "\t");
		currentOffset.put(CONSTANT, new MutableInteger(Integer.parseInt(parts[0])));
		currentNumElements.put(CONSTANT, new MutableInteger(Integer.parseInt(parts[1])));
		
		currentOffset.put(DYNAMIC, new MutableInteger(Integer.parseInt(parts[2])));
		currentNumElements.put(DYNAMIC, new MutableInteger(Integer.parseInt(parts[3])));
		
		int num = Integer.parseInt(lines.get(1)); // number of subscripts
		int ptr = 2;
		for (int i = 0; i < num; i++) {
			StreamlinedMemoryMapperSubscriptDefinition smmsd = new StreamlinedMemoryMapperSubscriptDefinition(lines.get(ptr++));
			subscripts.put(smmsd.getName(), smmsd);
		}
		
		while (ptr < lines.size()) {
			StreamlinedMemoryMapperRecord smmr = new StreamlinedMemoryMapperRecord(this, lines.get(ptr++));
			dataRecords.put(smmr.getVariableName(), smmr);
		}
		
		populateFromDataRecords();
	}
	
	private void populateFromDataRecords() {
		int i;
		int start;
		
		for (String lhs : dataRecords.keySet()) {
			StreamlinedMemoryMapperRecord smmr = dataRecords.get(lhs);
			varType.put(lhs, smmr.getConstantDynamic());
			eqnType.put(lhs, smmr.getVariableType());
			offset.get(smmr.getConstantDynamic()).put(lhs, smmr.getOffset());
			reverseIndex.get(smmr.getConstantDynamic()).put(smmr.getOffset(), lhs);
			numElements.get(smmr.getConstantDynamic()).put(lhs, smmr.getNumberOfElements());
			if (smmr.isOneTime())
				evaluationOrderOneTime.put(lhs, smmr.getEvaluationOrder());
			else
				evaluationOrderRepeated.put(lhs, smmr.getEvaluationOrder());
			
//			expandedIndex
//			expandedSortedLHS
//			sortedLHS
		}

	}

	public List<String> getVariableMemoryOrder() {
		// List<Integer> keys = new ArrayList<Integer>(reverseIndex.keySet());
		// Collections.sort(keys);
		// List<String> ordered = new ArrayList<String>();
		// for (Integer i : keys) {
		// ordered.add(reverseIndex.get(i));
		// }

		return this.expandedSortedLHS;
	}

	public List<String> getVariableExecutionOrder() {
		// List<Integer> keys = new ArrayList<Integer>(reverseIndex.keySet());
		// Collections.sort(keys);
		List<String> ordered = new ArrayList<String>();
		// for (Integer i : keys) {
		// ordered.add(reverseIndex.get(i));
		// }

//		ordered.addAll(evaluationOrderOneTime);
//		ordered.addAll(evaluationOrderRepeated);
		return ordered;
	}

	public void writeMapper(boolean badEvalOrder) {

		try {
			BufferedWriter bw = Utilities.openFileForWritingCharset(mapperFile, SDMDocLanguageSpecific.getInstance().getFileCharSet());
			bw.append(currentOffset.get(CONSTANT).value() + "\t" + currentNumElements.get(CONSTANT).value() + "\t" 
					+ currentOffset.get(DYNAMIC).value() + "\t" + currentNumElements.get(DYNAMIC).value() + "\t"
					+ mapperFile + "\t"+sizeByType.get(CONSTANT).value()+"\t"+sizeByType.get(DYNAMIC) +
					"\n");
			
			bw.append(nsm.getNamedSubscriptsForMemoryMapper());
			
			for (StreamlinedMemoryMapperRecord smmr : dataRecords.values()) {
				bw.append(smmr.getDataRecordForMemoryMapper());
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public List<List<String>> getLookupMapRecords() {
		List<List<String>> records = getLookupMapRecords(LOOKUP);
		return records;
	}
	
	public List<List<String>> getVariableMapRecords() {
		List<List<String>> records = getVariableMapRecords(CONSTANT);
		records.addAll(getVariableMapRecords(DYNAMIC));
//		records.addAll(getVariableMapRecords(LOOKUP));
		return records;
	}
	
	public List<List<String>> getVariableMapRecords(String type) {
		List<List<String>> records = new ArrayList<List<String>>();
		for (String s : numElements.get(type).keySet()) {
			List<String> record = new ArrayList<String>();
			record.add(s);
//			record.add(Utilities.makeLegalJavaName(s)); // 3/13/2020
			record.add(type);
			if (offset.get(getVarType(s)).get(s) == null) {
				record.add(Integer.toString(expandedIndex.get(getVarType(s)).get(s)));
			} else {
				record.add(Integer.toString(offset.get(getVarType(s)).get(s)));
			}
			record.add(Integer.toString(numElements.get(getVarType(s)).get(s)));
			records.add(record);
		}
		return records;
	}
	
	public List<List<String>> getLookupMapRecords(String type) {
		List<List<String>> records = new ArrayList<List<String>>();
		for (String lhs : eqnType.keySet()) {
			List<String> record = new ArrayList<String>();
			if (eqnType.get(lhs).equals(type)) {
				record.add(lhs);
				record.add(type);
				//			if (offset.get(getVarType(lhs)).get(lhs) == null) {
				//				record.add(Integer.toString(expandedIndex.get(getVarType(lhs)).get(lhs)));
				//			} else {
				//				record.add(Integer.toString(offset.get(getVarType(lhs)).get(lhs)));
				//			}
				record.add(Integer.toString(numElements.get(getVarType(lhs)).get(lhs)));
				records.add(record);
			}
		}
		return records;
	}
	

	public List<String> getMapperRecords() {
		List<String> records = new ArrayList<String>();
		for (String s : sortedLHS) {
			records.add(s + "\t" + eqnLHS.get(s) + "\t" + offset.get(getVarType(s)).get(s) + "\t"
					+ numElements.get(getVarType(s)).get(s)+"\t"+eqnType.get(common(s)));
		}
		return records;
	}

	private List<String> sortIgnoreCase(List<String> list) {
		List<String> sorted = new ArrayList<String>();
		List<String> temp = new ArrayList<String>();
		Map<String, String> lowerToOrig = new LinkedHashMap<String, String>();
		for (String s : list) {
			lowerToOrig.put(s.toLowerCase(), s);
			temp.add(s.toLowerCase());
		}
		Collections.sort(temp);
		for (String s : temp) {
			sorted.add(lowerToOrig.get(s));
		}
		return sorted;
	}
	
	private String getType(Equation eqn) {
		if (eqn.isOneTime() && !eqn.getLhs().equals("Time"))
			return CONSTANT;
		else
			return DYNAMIC;
	}

	public void determineMemoryRequirements() {
		
		int numEqn = equations.size();
		int interval = Math.max(numEqn / 10, 1);
		int num = 1;
		int pct = 10;
			
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		for (String lhs : sortedLHS) {
			
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs); // )equations.get(lhs);
			
			if (num % interval == 0 && pct < 100) {
				info.addMessage(language.getText(StringConstants.PROGRESS)+": "+pct+"% "+
						language.getText(StringConstants.OF)+" "+numEqn);
				pct += 10;
			}
			num++;
			
			if (eqn == null && ArrayReference.isArrayReference(lhs)) {
				eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs); // equations.get("array." + lhs);
			}
			if (eqn.isDefinesSubscript())
				continue;
			
			String type = getType(eqn);
			
			String screenName = screenName(lhs);
			if (eqn.isDefinesLookup() || eqn.isDefinesLookupGetXls()) {

				int numCombo;
				// let's put both in
//				offset.get(type).put(screenName, currentOffset.get(type).value());
				if (ArrayReference.isArrayReference(lhs)) {
					ArrayReference ar = new ArrayReference(lhs);
//					offset.get(type).put(new ArrayReference(lhs).getArrayName(),
//							currentOffset.get(type).value());
					numCombo = nsm
							.getNumSubscriptCombinations(new ArrayReference(lhs))
							* eqn.getVectorSpaceRequirements();
					ArrayManager am = InformationManagers.getInstance()
							.getArrayManager();
					numCombo = am
							.getAlphaSubscripts(ar.getArrayName(), 0).size();
					for (int dim = 1; dim < am.getNumDimensions(ar.getArrayName()); dim++) {
						numCombo *= am
								.getAlphaSubscripts(ar.getArrayName(), dim).size();
					}
				} else {
					numCombo = eqn.getVectorSpaceRequirements();
				}
				numElements.get(type).put(screenName, numCombo/2);
//				currentOffset.get(type).add(numCombo);
//				currentNumElements.get(type).add(numCombo);
//				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(numCombo);
			} else if (eqn.isHasLHSArrayReference()) {
				ArrayReference ar = eqn.getLhsArrayReference();
				// let's put both in
				offset.get(type).put(screenName, currentOffset.get(type).value());
				offset.get(type).put(eqn.getLhsAsInMdl(),
						currentOffset.get(type).value());
				int numCombo = nsm
						.getNumSubscriptCombinations(ar)
						* eqn.getVectorSpaceRequirements();
				ArrayManager am = InformationManagers.getInstance()
						.getArrayManager();
				numCombo = am
						.getAlphaSubscripts(ar.getArrayName(), 0).size();
				for (int dim = 1; dim < am.getNumDimensions(ar.getArrayName()); dim++) {
					numCombo *= am
							.getAlphaSubscripts(ar.getArrayName(), dim).size();
				}
				numElements.get(type).put(screenName, numCombo);
				currentOffset.get(type).add(numCombo);
				currentNumElements.get(type).add(numCombo);
				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(numCombo);
			} else {
				// scalar
				offset.get(type).put(screenName, currentOffset.get(type).value());
				currentOffset.get(type).add(1);
				numElements.get(type).put(screenName, 1);
				currentNumElements.get(type).add(1);
				sizeByType.get(eqn.isOneTime() ? CONSTANT : DYNAMIC).add(1);
			}
		}

		info.addMessage(language.getText(StringConstants.PROGRESS)+": 100% "+
				language.getText(StringConstants.OF)+" "+numEqn);
		
		UserSelectedOptions options = InformationManagers.getInstance()
				.getUserSelectedOptions();

		if (!options.isSelected(StringConstants.LIMIT_THUMBNAIL_COUNT) &&

		(currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value()) > 
			StreamlinedMemoryMapper.MAX_ELEMENTS_FOR_RUNNING) {
			InformationManagers
					.getInstance()
					.getModelManager()
					.setRunnable(
							false,
							"Too many elements for thumbnails "
									+ currentNumElements);
		}
	}

	private String screenName(String lhs) {
		return ndtm.getOriginalNameRemovingAnnotation(lhs).replace("!", "");
	}

	public String getOffset(String lhs) {
		String sn = (screenName(lhs));
		String type = getVarType(lhs);
		
			if (offset.get(type) == null) {
//				System.out.println("offset type missing for "+lhs+" "+sn+" varType = "+varType.get(common(sn)));
				return "-1";
			}
		if (offset.get(type).containsKey(sn)) {
			return offset.get(varType.get(common(sn))).get(sn).toString();
		}
		// is this an array reference with terminal subscripts?
		if (ArrayReference.isArrayReference(sn)) {
			String next = new ArrayReference(sn).getArrayName();
			if (offset.get(type).containsKey(next))
				return offset.get(type).get(new ArrayReference(sn).getArrayName())
						.toString();
		}
		System.out.println("Confused in Memory Map looking for " + lhs
				+ " screen name: " + sn);
		if (sn == null)
			return "0";
		else
			return "-9999";
	}
	
	public String getVarType(String lhs) {
		
		String type = null;
		String arg = screenName(ArrayReference.deannotate(new String(lhs))).replace("!", "");
		
		if (!varType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		if (!varType.containsKey(common(arg))) {

		} else {
			type = varType.get(common(arg));
		}
		
		return type;
	}
	
	public String getEqnType(String lhs) {
		
		String type = null;
		String arg = screenName(ArrayReference.deannotate(new String(lhs))).replace("!", "");
		
		if (!eqnType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		if (!eqnType.containsKey(common(arg))) {

		} else {
			type = eqnType.get(common(arg));
		}
		
		return type;
	}
	
	public String getDataStoreAndOffset(String lhs) {
		return getDataStore(lhs)+"["+getIndex(lhs)+"]";
	}
	
	public String getDataStore(String lhs) {
		
		String arg = screenName(ArrayReference.deannotate(new String(lhs)));
		if (!varType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		String type = varType.get(common(arg));
		
		if (type == null)
			return "cvMem";
		
		if (type.equals(CONSTANT))
			return "cMem";
		else
			return "vMem";
	}

	public int getIndex(String lhs) {
		String sn = common(screenName(lhs));
		
		String type = null;
		String arg = screenName(ArrayReference.deannotate(new String(lhs))).replace("!", "");
		
		if (!varType.containsKey(common(arg))) {
			if (ArrayReference.isArrayReference(arg)) {
				arg = new ArrayReference(arg).getArrayName();
			}
		}
		
		if (!varType.containsKey(common(arg))) {

		} else {
			type = varType.get(common(arg));
		}
		
//		String vt = varType.get(sn);
		
//		Map<String, Integer> m = expandedIndex.get(vt);
		Map<String, Integer> m = offset.get(type);
		return m.get(arg);
	}

	public String getNumElements(String lhs) {	
		String sn = screenName(lhs);
		return numElements.get(getVarType(sn)).get(sn).toString();
	}

	public String getSizeRequirement() {
		return Integer.toString(currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value());
	}
	
	public int getNumElementsForType(String type) {
		return currentNumElements.get(type).value();
	}

	public int getNumElements() {
		return currentNumElements.get(CONSTANT).value() + currentNumElements.get(DYNAMIC).value();
	}

	public void report() {
		System.out.println("+++ Memory Map +++");
		System.out.println("Size: " + getSizeRequirement());
		for (String s : sortedLHS) {
			System.out.println("lhs: " + s + " offset: " + offset.get(s)
					+ " elements: " + numElements.get(s) + " eqnLHS: "
					+ eqnLHS.get(s));
		}

		// for (String s : expandedSortedLHS) {
		String s = expandedSortedLHS.get(expandedSortedLHS.size() - 1);
		System.out
				.println("Final ExpIndex: " + s + ": " + expandedIndex.get(s));
		// }
	}

	public String getMapperFile() {
		return mapperFile;
	}

	public Map<String, MutableInteger> getSizeByType() {
		return sizeByType;
	}

	public Map<String, Equation> getEquations() {
		return equations;
	}

	public Map<String, String> getEqnLHS() {
		return eqnLHS;
	}

	public Map<String, String> getVarType() {
		return varType;
	}

	public List<String> getSortedLHS() {
		return sortedLHS;
	}

	public Map<String, Map<String, Integer>> getOffset() {
		return offset;
	}

	public Map<String, Map<Integer, String>> getReverseIndex() {
		return reverseIndex;
	}

	public Map<String, MutableInteger> getCurrentOffset() {
		return currentOffset;
	}

	public Map<String, MutableInteger> getCurrentNumElements() {
		return currentNumElements;
	}

	public Map<String, Map<String, Integer>> getExpandedIndex() {
		return expandedIndex;
	}

	public NativeDataTypeManager getNdtm() {
		return ndtm;
	}

	public NamedSubscriptManager getNsm() {
		return nsm;
	}
	
	public String common(String var) {
//		if (InformationManagers.getInstance().getVariableNamePatternGenerator().isPattern(var)) {
//			return var;
//		} else {
//			return InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(var);
//		}
		
		return var;
	}

	@Override
	public List<String> getEvaluationOrderOneTime() {
		if (evaluationOrderOneTimeList != null) {
			return evaluationOrderOneTimeList;
		} else {
			evaluationOrderOneTimeList = new ArrayList<String>();
			List<VariableOrder> al = new ArrayList<VariableOrder>();
			for (String key : evaluationOrderOneTime.keySet()) {
				al.add(new VariableOrder(key, evaluationOrderOneTime.get(key)));
			}
			Collections.sort(al);
			for (VariableOrder vo : al)
				evaluationOrderOneTimeList.add(vo.getVariable());
			return evaluationOrderOneTimeList;
		}
	}

	@Override
	public void setEvaluationOrderOneTime(List<String> evaluationOrderOneTime) {
		// This lhs contains subscripts -- need to strip
		evaluationOrderOneTimeList = evaluationOrderOneTime;
		
		for (int i=0; i < evaluationOrderOneTime.size(); i++) {
			String var = evaluationOrderOneTimeList.get(i);
			String origVar = var;
			if (ArrayReference.isArrayReference(var)) {
				var = ArrayReference.extractArrayName(var);
			}
			StreamlinedMemoryMapperRecord smmr = dataRecords.get(var);
			if (smmr == null) {
				System.out.println("No SMMR for origVar: "+origVar+" var: "+var);
			} else {
				smmr.setEvaluationOrder(i);
			}
		}
		
	}

	@Override
	public List<String> getEvaluationOrderRepeated() {
		if (evaluationOrderRepeatedList != null) {
			return evaluationOrderRepeatedList;
		} else {			
			evaluationOrderRepeatedList = new ArrayList<String>();
			List<VariableOrder> al = new ArrayList<VariableOrder>();
			for (String key : evaluationOrderRepeated.keySet()) {
				al.add(new VariableOrder(key, evaluationOrderRepeated.get(key)));
			}
			Collections.sort(al);
			for (VariableOrder vo : al)
				evaluationOrderRepeatedList.add(vo.getVariable());
			return evaluationOrderRepeatedList;
		}
	}

	@Override
	public void setEvaluationOrderRepeated(List<String> evaluationOrderRepeated) {
		evaluationOrderRepeatedList = evaluationOrderRepeated;
		
		for (int i=0; i < evaluationOrderRepeated.size(); i++) {
			String var = evaluationOrderRepeated.get(i);
			if (ArrayReference.isArrayReference(var))
				var = ArrayReference.extractArrayName(var);
			StreamlinedMemoryMapperRecord smmr = dataRecords.get(var);
			if (smmr == null) {
				System.out.println("NO SMMR FOR REPEATED: "+var);
			} else {
			smmr.setEvaluationOrder(i);
			}
		}
		
		
	}

	@Override
	public List<String> getExpandedSortedLHS() {
		// TODO Auto-generated method stub
		return new ArrayList<String>();
	}

	@Override
	public boolean isValidVariable(String var) {
		return getVarType(var) != null;
	}


}
