package gov.anl.gss.sdm.server.model.support;

import java.util.ArrayList;
import java.util.List;

public class StreamlinedMemoryMapperRecord {

	private String variableName;			//	0
	private String constantDynamic;			//	1
	private String variableType;			//	2
	private String scalarArray;				//	3
	private int offset;						//	4
	private int numElements;				//	5
	private int evaluationOrder;			//	6
	private List<String> subscripts;		//	7
	private StreamlinedMemoryMapper mapper;
	
	public StreamlinedMemoryMapperRecord(StreamlinedMemoryMapper mapper) {
		this.mapper = mapper;
	}
	public StreamlinedMemoryMapperRecord(StreamlinedMemoryMapper mapper,
			String variableName,
			String constantDynamic,
			String variableType,
			String scalarArray,
			int offset,
			int numElements,
			int evaluationOrder,
			List<String> subscripts) {
		this(mapper);
		this.variableName = variableName;
		this.constantDynamic = constantDynamic;
		this.variableType = variableType;
		this.scalarArray = scalarArray;
		this.offset = offset;
		this.numElements = numElements;
		this.evaluationOrder = evaluationOrder;
		this.subscripts = subscripts;
	}
			
	
	public StreamlinedMemoryMapperRecord(StreamlinedMemoryMapper mapper, String dataRecord) {
		this(mapper);
		parseDataRecord(dataRecord);
	}
	
	public int getNumberOfElements() {
		return numElements;
	}
	
	private void parseDataRecord(String dataRecord) {
		String[] parts = dataRecord.split("\t");
		variableName = parts[0];
		constantDynamic = parts[1];
		variableType = parts[2];
		scalarArray = parts[3];
		offset = Integer.parseInt(parts[4]);
		numElements = Integer.parseInt(parts[5]);
		evaluationOrder = Integer.parseInt(parts[6]);
		if (isArray()) {
			subscripts = new ArrayList<String>();
			for (int i = 7; i < parts.length; i++)
				subscripts.add(parts[i]);
		}
	}
	
	public String toString() {
		return getDataRecordForMemoryMapper();
	}
	
	public String getDataRecordForMemoryMapper() {
		StringBuilder sb = new StringBuilder();
		sb.append(variableName+"\t");
		sb.append(constantDynamic+"\t");
		sb.append(variableType+"\t");
		sb.append(scalarArray+"\t");
		sb.append(offset+"\t");
		sb.append(numElements+"\t");
		sb.append(evaluationOrder);
		if (isArray()) {
			for (String s : subscripts) {
				sb.append("\t"+s);
			}
		}
		sb.append("\n");
		return sb.toString();
	}
	
	public boolean isArray() {
		return scalarArray.equals(StreamlinedMemoryMapper.ARRAY);
	}
	
	public boolean isScalar() {
		return scalarArray.equals(StreamlinedMemoryMapper.SCALAR);
	}
	
	public boolean isConstant() {
		return constantDynamic.equals(StreamlinedMemoryMapper.CONSTANT);
	}
	
	public boolean isDynamic() {
		return constantDynamic.equals(StreamlinedMemoryMapper.DYNAMIC);
	}
	
	public boolean isStock() {
		return variableType.equals(StreamlinedMemoryMapper.STOCK);
	}
	
	public boolean isFlow() {
		return variableType.equals(StreamlinedMemoryMapper.FLOW);
	}
	
	public boolean isAuxiliary() {
		return variableType.equals(StreamlinedMemoryMapper.AUXILIARY);
	}
	
	public boolean isLookup() {
		return variableType.equals(StreamlinedMemoryMapper.LOOKUP);
	}
	
	public boolean isOneTime() {
		return isConstant() || isLookup();
	}
	
	public boolean isRepeated() {
		return isDynamic();
	}

	public String getVariableName() {
		return variableName;
	}

	public String getConstantDynamic() {
		return constantDynamic;
	}

	public String getVariableType() {
		return variableType;
	}

	public String getScalarArray() {
		return scalarArray;
	}

	public int getOffset() {
		return offset;
	}

	public int getEvaluationOrder() {
		return evaluationOrder;
	}

	public List<String> getSubscripts() {
		return subscripts;
	}

	public StreamlinedMemoryMapper getMapper() {
		return mapper;
	}
	public void setVariableName(String variableName) {
		this.variableName = variableName;
	}
	public void setConstantDynamic(String constantDynamic) {
		this.constantDynamic = constantDynamic;
	}
	public void setVariableType(String variableType) {
		this.variableType = variableType;
	}
	public void setScalarArray(String scalarArray) {
		this.scalarArray = scalarArray;
	}
	public void setOffset(int offset) {
		this.offset = offset;
	}
	public void setEvaluationOrder(int evaluationOrder) {
		this.evaluationOrder = evaluationOrder;
	}
	public void setSubscripts(List<String> subscripts) {
		this.subscripts = subscripts;
	}
	public void setMapper(StreamlinedMemoryMapper mapper) {
		this.mapper = mapper;
	}
	public int getNumElements() {
		return numElements;
	}
	public void setNumElements(int numElements) {
		this.numElements = numElements;
	}
	
}
