package gov.anl.gss.sdm.server.model.support;

import java.io.BufferedReader;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class SystemDynamicsModel extends SDModelWithPropertiesVDM_Native {
	
	private Map<String, Double> memory = new LinkedHashMap<String, Double>();
	private Map<String, double[][]> memoryLookup = new LinkedHashMap<String, double[][]>();
	private Map<String, Map<Integer, List<String>>> allocatedIndicies = new LinkedHashMap<String, Map<Integer, List<String>>>();


	public SystemDynamicsModel(String name) {
		super(name);
		// TODO Auto-generated constructor stub
	}
	
	public void loadSubscriptMap(BufferedReader br) {
		List<String> mapInfo = Utilities.getAsList(br);
		mapInfo.remove(0);
		for (String line : mapInfo) {
			String[] p = line.split("\t");
			if (!allocatedIndicies.containsKey(p[0])) {
				allocatedIndicies.put(p[0], new LinkedHashMap<Integer, List<String>>());
			}
			Map<Integer, List<String>> array = allocatedIndicies.get(p[0]);
			int subscriptIndex = Integer.parseInt(p[1]);
			if (!array.containsKey(subscriptIndex)) {
				array.put(subscriptIndex, new ArrayList<String>());
			}
			List<String> subs = array.get(subscriptIndex);
			subs.add(p[2]);
		}
	}

	public String asExpandedArrayRef(String array, int ... integers) {
		Map<String, Map<Integer, List<String>>> allocatedIndicies = new LinkedHashMap<String, Map<Integer, List<String>>>();
		int numDimensions = allocatedIndicies.get(array).size();
		StringBuilder sb = new StringBuilder(array);
		sb.append("[");
		for (int i = 0; i < numDimensions; i++) {
			int index = integers[i];
			if (i>0) sb.append(",");
			sb.append(""+allocatedIndicies.get(array).get(i).get(index)+"");
		}
		sb.append("]");
		return sb.toString();
	}
	public void set(String var, double value) {
		memory.put(var, value);
		logit(var, currentTime, value, saveper);
	}

	public void setNolog(String var, double value) {
		memory.put(var, value);
	}

	public void set(String var, double value[][]) {
		memoryLookup.put(var, value);
	}

	public double get(String var) {
		return memory.get(var);
	}
	
	public double get(String var, double time) {
		return memory.get(var);
	}

	public double[][] getLookup(String var) {
		return memoryLookup.get(var);
	}

	@Override
	public double getINITIALTIME() {
		return memory.get("INITIAL TIME");
	}
	@Override
	public double getFINALTIME() {
		return memory.get("FINAL TIME");
	}
	@Override
	public double getTIMESTEP() {
		return memory.get("TIME STEP");
	}

}
