package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.StandaloneGUI;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

public class Utilities {
	static Logger log = Logger.getLogger(Utilities.class.getSimpleName());

	/*
	 * return name.replace(" ", "_").replace("/", "_").replace("?", "_")
	 * .replace(":", "_").replace("-", "_").replace("&", "_").replace(".", "_")
	 * .replace("(", "_").replace(")", "_") .replace("(", "_").replace(")",
	 * "_").replace(".", "_").replace(",", "_").replace("'", "").replace("*",
	 * "_");
	 */
	private static String BLANK =  "_"; // "_BLANK_"; // "_"; //
	private static String SLASH = "_SLASH_";
	private static String QMARK = "_QMARK_";
	private static String COLON = "_COLON_";
	private static String MINUS = "_MINUS_";
	private static String AMPERSAND = "_AMPERSAND_";
	private static String DOT = "_DOT_";
	private static String LPAREN = "_LPAREN_";
	private static String RPAREN = "_RPAREN_";
	private static String COMMA = "_COMMA_";
	private static String STAR = "_STAR_";
	private static String SQUOTE = "_SQUOTE_";
	private static String SCOLON = "_SCOLON_";
	private static String HASH = "_HASH_";

	private static String GENERATED_VAR = "GV";
	private static int GENERATED_ID = 0;

	private static int num = 1;

	private static Map<String, String> generated_var_map_orig_to_gv = new LinkedHashMap<String, String>();
	private static Map<String, String> generated_var_map_gv_to_orig = new LinkedHashMap<String, String>();

	public static boolean writeGeneratedVarMap(String file) {
		BufferedWriter bw = openFileForWritingUTF8(file);
		try {
			for (String s : ordered(generated_var_map_orig_to_gv.keySet())) {

				bw.append(s + "\t");
				bw.append(generated_var_map_orig_to_gv.get(s) + "\n");
			}
			bw.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}

	public static Set<String> minus(Set<String> set1, Set<String> set2) {
		Set<String> set = new LinkedHashSet<String>(set1);
		set.removeAll(set2);
		return set;
	}

//	public static String annotate(String s) {
//
//		if (!ArrayReference.isArrayReference(s))
//			return s;
//		else
//			return "array." + s;
//	}

	// public static Set<String> intersect(Set<String> set1, Set<String> set2) {
	// Set<String> newSet = new LinkedHashSet<String>();
	// for (String str : set1)
	// if (set2.contains(str))
	// newSet.add(str);
	// return newSet;
	// }

	public static List<String> getAsListUTF8(String filename) {
		try {
			return getAsList(new BufferedReader(new InputStreamReader(
					new FileInputStream(filename), "UTF8")));
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return new ArrayList<String>();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return new ArrayList<String>();
		}
	}

	public static boolean isOfXmlOrigin(String fileName) {

		String lc = fileName.toLowerCase();
		for (String fileType : StandaloneGUI.RECOGNIZED_FILE_TYPES) {
			if (fileType.equals("mdl"))
				continue;
			if (lc.endsWith("." + fileType))
				return true;
		}
		return false;
	}

	public static String getConvertedFileName(String fileName) {
		if (isXmileFileType(fileName))
			return fileName
					.replaceAll("\\.[xX][mM][iI][lL][eE]$", "_xmile.mdl");
		if (isStmxFileType(fileName))
			return fileName.replaceAll("\\.[sS][tT][mM][xX]$", "_stmx.mdl");
		else
			return fileName;
	}

	public static boolean isXmileFileType(String file) {
		if (file.toLowerCase().endsWith("xmile"))
			return true;
		else
			return false;
	}

	public static boolean isStmxFileType(String file) {
		if (file.toLowerCase().endsWith("stmx"))
			return true;
		else
			return false;
	}

	public static boolean stringEndsWith(String s, String notation) {

		// String notation = "[nN][oO][rR][mM][aA][lL] *\\( *[^,]+, *[^,)]+\\)";
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(s);
		boolean found = false;
		while (m.find()) {
			String not = m.group();
			return true;
		}
		return found;

	}

	public static List<String> ordered(Set<String> aSet) {
		return sortIgnoreCase(aSet);
	}

	public static String escapePound(String filename) {
		String xxx = filename.replace("#", "&#35;");
		return filename.replace("#", "&#35;");
	}

	public static boolean fileExists(String filename) {
		File f = new File(filename);
		return f.exists();
	}

	public static List<String> loadDirectSubscripts(List<String> equationTokens) {
		List<String> subscriptTokens = new ArrayList<String>();
		String file = equationTokens.get(4).replace("'", "");
		String tab = equationTokens.get(6).replace("'", "");
		String firstCell = equationTokens.get(8).replace("'", "");
		String lastCell = equationTokens.get(10).replace("'", "");
		String prefix = equationTokens.get(12).replace("'", "");

		if (tab.length() == 0)
			tab = ",";

		int column = getColumnFromCellAddress(firstCell) - 1;

		List<String> raw = Utilities.getAsList(InformationManagers
				.getInstance().getFileOrganizer().getModelDirectory()
				+ "/" + file);

		for (String t : raw) {
			String subr = t;

			if (t.contains(tab)) {
				subr = t.split(tab)[column];
			}

			if (subscriptTokens.size() > 0)
				subscriptTokens.add(",");
			subscriptTokens.add(subr);
		}
		return subscriptTokens;
	}

	public static List<String> getAsListCharset(String filename) {
		File f = new File(filename);
		System.out.println(f.getAbsolutePath());
		if (!f.exists()) {
			return new ArrayList<String>();
		}
		return getAsList(openForReadCharset(filename));
	}

	public static List<String> getAsList(String filename) {
		File f = new File(filename);
		System.out.println(f.getAbsolutePath());
		if (!f.exists()) {
			return new ArrayList<String>();
		}
		return getAsList(openForRead(filename));
	}

	public static List<String> getAsList(BufferedReader reader) {

		List<String> fileContents = new ArrayList<String>();
		String aLine = null;

		String lhs = "";
		String rhs = "";

		try {
			aLine = reader.readLine();
			while (aLine != null) {
				fileContents.add(aLine);
				aLine = reader.readLine();
			}
			reader.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return fileContents;
	}

	public static void writeToFileUTF8(String file, List<String> al) {
		try {
			Writer out = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(file), "UTF-8"));
			try {
				for (String s : al) {
					out.write(s + "\r\n");
				}

			} finally {
				out.close();
			}
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void printToConsole(Set<String> al) {
		printToConsole(ordered(al));
	}

	public static void printToConsole(List<String> al) {
		for (String s : al)
			System.out.println("ptc: " + s);
	}

	public static void writeToFile(String file, Set<String> al) {
		writeToFile(file, ordered(al));
	}

	public static void writeToFile(String file, List<String> al) {
		try {
			BufferedWriter bw = openFileForWritingUTF8(file);
			for (String s : al) {
				bw.append(s + "\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static String getOriginalNameForGV(String gv) {
		if (generated_var_map_gv_to_orig.containsKey(gv)) {
			return generated_var_map_gv_to_orig.get(gv);
		} else {
			return gv;
		}
	}

	public static BufferedWriter openFileForWriting(String directory,
			String filename) {
		return openFileForWritingUTF8(directory + "/" + filename);
	}

	public static BufferedWriter openFileForWriting(String filename) {
		BufferedWriter report = null;
		// System.out.println("openFileForWriting(String filename): "+filename);
		// for (StackTraceElement t : Thread.currentThread().getStackTrace()) {
		// System.out.println(t);
		// }
		try {
			File aFile = new File(filename);
			if (aFile.getParentFile() != null)
				aFile.getParentFile().mkdirs();
			report = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(aFile, false), "UTF-8"));
		} catch (FileNotFoundException | UnsupportedEncodingException e) {
			log.debug(filename + " FileNotFoundException");
			e.printStackTrace();
		}

		return report;
	}

	public static BufferedWriter openFileForWritingCharset(String filename,
			String charset) {
		BufferedWriter report = null;
		// System.out.println("openFileForWriting(String filename, String charset)): "+filename+", "+charset);
		// for (StackTraceElement t : Thread.currentThread().getStackTrace()) {
		// System.out.println(t);
		// }
		try {
			File aFile = new File(filename);
			if (aFile.getParentFile() != null)
				aFile.getParentFile().mkdirs();
			report = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(filename), charset));
		} catch (FileNotFoundException e) {
			log.debug(filename + " FileNotFoundException");
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return report;
	}

	public static BufferedWriter openFileForWritingUTF8(String filename) {
		BufferedWriter report = null;

		try {
			File aFile = new File(filename);
			if (aFile.getParentFile() != null)
				aFile.getParentFile().mkdirs();
			report = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(filename), "UTF-8"));
		} catch (FileNotFoundException e) {
			log.debug(filename + " FileNotFoundException");
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return report;
	}

	// BufferedReader in = new BufferedReader(new InputStreamReader(stream,
	// encoding));
	// Reader in = new InputStreamReader(new FileInputStream("file"), "UTF-8"));

	public static BufferedReader openForReadCharset(String filename) {

		BufferedReader fileReader = null;

		File f = new File(filename);
		if (!f.exists())
			return fileReader;

		// open the file for reading
		try {
			fileReader = new BufferedReader(new InputStreamReader(
					new FileInputStream(filename), "UTF-8"));
		} catch (FileNotFoundException | UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return fileReader;
	}

	public static BufferedReader openForRead(String filename) {

		return openForReadCharset(filename);

		// BufferedReader fileReader = null;
		//
		// File f = new File(filename);
		// if (!f.exists())
		// return fileReader;
		//
		// // open the file for reading
		// try {
		// fileReader = new BufferedReader (new FileReader(new File(filename)));
		// } catch (FileNotFoundException e) {
		// // TODO Auto-generated catch block
		// e.printStackTrace();
		// }
		//
		// return fileReader;
	}

	public static int getRowFromCellAddress(String cellAddress) {
		int row = 1;
		String rc = cellAddress;
		if (cellAddress.contains("!")) {
			rc = cellAddress.split("!")[1];
		}
		int rowPos = getFirstNumberPosition(rc);
		row = Integer.parseInt(rc.substring(rowPos));
		return row;
	}

	public static int getColumnFromCellAddress(String cellAddress) {
		int column = 1;
		String rc = cellAddress;
		if (cellAddress.contains("!")) {
			rc = cellAddress.split("!")[1];
		}
		int rowPos = getFirstNumberPosition(rc);
		column = convertColumnToNumber(rc.substring(0, rowPos));

		return column;
	}

	public static int convertColumnToNumber(String column) {
		int num = 0;
		int pos = 0;

		char[] chars = column.toCharArray();
		for (int i = chars.length - 1; i >= 0; i--) {
			num += (Character.getNumericValue(chars[i]) - 9)
					* Math.pow(26, pos++);
		}
		return num;
	}

	private static int getFirstNumberPosition(String address) {
		MutableInteger pos = new MutableInteger(0);
		while (inRange(address, pos)
				&& Character
						.isLetter(characterAt(address, pos).toCharArray()[0]))
			pos.add(1);
		return pos.value();
	}

	private static boolean inRange(String equation, MutableInteger position) {
		if (position.value() <= equation.length() - 1)
			return true;
		else
			return false;
	}

	public static boolean isLetter(String aChar) {
		return aChar.matches("[a-zA-Z]");
	}

	public static boolean isDigit(String aChar) {
		return aChar.matches("[0-9]");
	}

	public static boolean isValidCellAddress(String address) {
		// Valid is considered letter+digit+
		boolean expectingLetter = true;
		boolean expectingDigit = false;
		boolean foundLetter = true;
		boolean foundDigit = false;
		boolean valid = true;
		MutableInteger position = new MutableInteger();
		while (inRange(address, position)) {
			String aChar = characterAt(address, position);
			if (isLetter(aChar)) {
				if (expectingLetter) {
					foundLetter = true;
				} else {
					valid = false;
				}

			} else if (isDigit(aChar)) {
				if (expectingDigit
						|| (expectingLetter && !expectingDigit && foundLetter)) {
					expectingDigit = true;
					expectingLetter = false;
					foundDigit = true;
				} else {
					valid = false;
				}

			} else {
				valid = false;
			}

			position.add(1);
		}

		return valid;
	}

	private static String characterAt(String equation, MutableInteger position) {
		return equation.substring(position.value(), position.value() + 1);
	}

	public static String asString(List<String> al) {
		StringBuilder sb = new StringBuilder("[");
		int i = 0;
		for (String s : al) {
			if (i++ > 1)
				sb.append(", ");
			sb.append(s);
		}
		sb.append("]");
		return sb.toString();
	}

	public static void resetNumberedFile() {
		num = 0;
	}

	public static String generateUniquePlotFileName() {
		String filename = "";
		long millis = System.currentTimeMillis();
		filename = "plot_" + millis;
		return filename;
	}

	public static String generateNumberedFileName() {
		String filename = "";
		filename = "file_" + num++;
		return filename;
	}

	public static String generateNumberedFileName(String file) {
		String filename = "";
		filename = file + "_" + num++;
		return filename;
	}

	public static String getNextNumberedFileName(String file) {
		String filename = "";
		filename = file + "_" + num;
		return filename;
	}

	public static void writeToFile(String file, String message) {
		try {
			BufferedWriter bw = openFileForWritingUTF8(file);
			bw.append(message);
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static void writeToFileCharset(String file, String message) {
		try {
			BufferedWriter bw = openFileForWritingCharset(file,
					SDMDocLanguageSpecific.getInstance().getFileCharSet());
			bw.append(message);
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public static String[] asArray(List<String> al) {
		String[] a = new String[al.size()];
		for (int i = 0; i < a.length; i++)
			a[i] = al.get(i);
		return a;
	}

	public static String getNameArgument(String s) {

		String name;

		boolean hasSdml = false;
		if (s.endsWith(StringConstants.RSSD_LOOKUP)) {
			name = s.replace(StringConstants.RSSD_LOOKUP, "");
			hasSdml = true;
		} else {
			name = s;
		}

		if (nonAscii(name)) {
			if (generated_var_map_orig_to_gv.containsKey(name))
				return generated_var_map_orig_to_gv.get(name)
						+ (hasSdml ? StringConstants.RSSD_LOOKUP : "");
			else
				return s;
		} else {
			return s;
		}
	}

	public static boolean nonAscii(String s) {
		for (int i = 0; i < s.length(); i++)
			if (s.codePointAt(0) > 127)
				return true;
		return false;
	}

	public static String getNextGeneratedName() {
		if (generated_var_map_orig_to_gv.size() == 0)
			GENERATED_ID = 0;
		return GENERATED_VAR + (GENERATED_ID++);
	}

	public static String revertFromLegalJavaName(String tname) {
		String className = tname.replace(SLASH, "/")
				.replace(QMARK, "?").replace(COLON, ":").replace(SCOLON, ";")
				.replace(SQUOTE, "'").replace(MINUS, "-")
				.replace(AMPERSAND, "&").replace(DOT, ".").replace(LPAREN, "(")
				.replace(RPAREN, ")").replace(HASH, "#").replace(COMMA, ",").replace(BLANK, " ");

		return className;
	}

	public static String makeLegalJavaName(String tname) {

		String name;

		boolean hasSdml = false;
		if (tname.endsWith(StringConstants.RSSD_LOOKUP)) {
			name = tname.replace(StringConstants.RSSD_LOOKUP, "");
			hasSdml = true;
		} else {
			name = tname;
		}

		if (nonAscii(name)) {
			if (generated_var_map_orig_to_gv.containsKey(name))
				return generated_var_map_orig_to_gv.get(name)
						+ (hasSdml ? StringConstants.RSSD_LOOKUP : "");
			String generatedName = getNextGeneratedName();
			generated_var_map_orig_to_gv.put(name, generatedName);
			generated_var_map_gv_to_orig.put(generatedName, name);
			return generatedName + (hasSdml ? StringConstants.RSSD_LOOKUP : "");
		}

		String className = name.replace(" ", BLANK).replace("/", SLASH)
				.replace("?", QMARK).replace(":", COLON).replace(";", SCOLON)
				.replace("'", SQUOTE).replace("-", MINUS)
				.replace("&", AMPERSAND).replace(".", DOT).replace("(", LPAREN)
				.replace(")", RPAREN).replace(".", DOT).replace("#", HASH)
				.replace(",", COMMA);

		if (className.length() < 1) {
			System.out.println("no name");
		}

		if (isDigit(className.substring(0, 1))) {
			className = "_" + className;
		}

		return className + (hasSdml ? StringConstants.RSSD_LOOKUP : "");
	}

	// public static String makeLegalFilename(String name) {
	// return name.replace(" ", "_").replace("/", "_").replace("?", "_")
	// .replace(":", "_").replace("-", "_").replace("&", "_").replace(".", "_")
	// .replace("(", "_").replace(")", "_")
	// .replace("(", "_").replace(")", "_").replace(".", "_").replace(",",
	// "_").replace("'", "").replace("*",
	// "_").replaceAll("[^\\x0A\\x0D\\x20-\\x7E]", "");
	// }

	public static String makeLegalFilename(String name) {
		return name.replace(" ", "_").replace("/", "_").replace("?", "_")
				.replace(":", "_").replace("-", "_").replace("&", "_")
				.replace(".", "_").replace("(", "_").replace(")", "_")
				.replace("(", "_").replace(")", "_").replace(".", "_")
				.replace(",", "_").replace("'", "").replace("*", "_")/*
																	 * .replaceAll
																	 * (
																	 * "[^\\x0A\\x0D\\x20-\\x7E]"
																	 * , "")
																	 */;
	}

	public static List<String> sortIgnoreCase(Set<String> set) {
		List<String> ordered = new ArrayList<String>();
		Map<String, String> ignoreCase = new LinkedHashMap<String, String>();
		for (String mixed : set) {
			ignoreCase.put(mixed.toLowerCase(), mixed);
		}
		List<String> toSort = new ArrayList<String>(ignoreCase.keySet());
		Collections.sort(toSort);

		for (String key : toSort) {
			ordered.add(ignoreCase.get(key));
		}
		return ordered;
	}

	public static int systemVariableCompare(String v0, String v1) {
		boolean isSys0 = Parser.isSystemVariable(v0);
		boolean isSys1 = Parser.isSystemVariable(v1);
		if (!isSys0 && isSys1) {
			return -1;
		} else if (isSys0 && !isSys1) {
			return 1;
		} else {
			return 0;
		}
	}

	public static List<Double> asList(double[] array) {
		List<Double> al = new ArrayList<Double>();
		for (double dbl : array)
			al.add(dbl);
		return al;
	}

	public static List<String> asList(String[] array) {
		List<String> al = new ArrayList<String>();
		for (String dbl : array)
			al.add(dbl);
		return al;
	}

	public static List<String> asList(Set<String> set) {
		List<String> al = new ArrayList<String>();
		for (String dbl : set)
			al.add(dbl);
		return al;
	}

	public static String convertModelNameToClassName(String mdlFile) {
		File f = new File(mdlFile);
		String mdl = f.getName();
		String model = mdl.substring(0, mdl.lastIndexOf(".")).trim();
		String className = makeLegalJavaName(model.toUpperCase());

		return className;
	}

	public static Set<String> intersect(Set<String> set1, Set<String> set2) {
		Set<String> inter = union(set1, set2);
		inter.retainAll(set1);
		inter.retainAll(set2);
		return inter;

	}

	public static Set<String> union(Set<String> set1, Set<String> set2) {
		Set<String> set = new LinkedHashSet<String>(set1);
		set.addAll(set2);
		return set;
	}

	public static String idToName(String id) {
		return InformationManagers.getInstance().getEquationManager()
				.getVariableNameForID(id);
	}

	public static String idToName(int id) {
		return InformationManagers.getInstance().getEquationManager()
				.getVariableNameForID(id);
	}

	public static String nameToID(String name) {
		return InformationManagers.getInstance().getEquationManager()
				.getVariableIDForName(name);
	}

	public static List<String> idToName(List<String> id) {
		List<String> asName = new ArrayList<String>();
		for (String i : id)
			asName.add(Utilities.idToName(i));
		return asName;
	}

	public static List<String> nameToID(List<String> name) {
		List<String> asID = new ArrayList<String>();
		for (String n : name)
			asID.add(Utilities.idToName(n));
		return asID;
	}

	public static List<String> toVerbose(List<String> mdlContents) {
		List<String> verbose = new ArrayList<String>();

		// some line will be "verbose" already. Find those that are not and
		// convert
		// continuation will already have been removed

		for (String aLine : mdlContents) {
			String[] parts = aLine.split(" ~");
			if (parts.length == 1) {
				verbose.add(aLine);
			} else if (aLine.endsWith("~~|")) {
				String s = aLine.replace("~~|", "");
				verbose.add(s);
				verbose.add("\t~~|");
			} else {
				int i = 0;
				for (String s : parts) {
					if (i++ > 0) {
						verbose.add("\t~ " + s);
					} else {
						verbose.add(s);
					}
				}
			}
		}

		return verbose;
	}

	public static List<String> removeContinuation(List<String> mdlContents) {
		List<String> sansContinuation = new ArrayList<String>();
		StringBuilder completeLine = new StringBuilder();
		int ptr = 0;
		boolean concat = false;

		boolean tabbedArray = false;

		while (ptr < mdlContents.size()) {

			// get a line from the file

			String line = mdlContents.get(ptr++);

			if (!tabbedArray) {
				tabbedArray = line.contains("TABBED ARRAY");
			} else {
				tabbedArray = !line.contains(")");
			}

			boolean hasContinuation = line.endsWith("\\")
					&& !line.startsWith("///");
			;
			// does this line need to be added to previous line(s)?

			if (concat) {
				if (tabbedArray) {
					if (!(completeLine.toString().endsWith("\t") || line
							.startsWith("/t"))) {
						completeLine.append("\t");
					}
				}
				completeLine.append(hasContinuation ? line.replaceAll("\\\\$",
						"") : line);
			} else {
				completeLine.setLength(0);
				if (tabbedArray) {
					if (!(completeLine.toString().endsWith("\t") || line
							.startsWith("/t"))) {
						completeLine.append("\t");
					}
				}
				completeLine.append(hasContinuation ? line.replaceAll("\\\\$",
						"") : line);
			}
			if (!hasContinuation) {
				sansContinuation.add(completeLine.toString());
				// tabbedArray = false;
			}
			concat = hasContinuation;

		}

		return sansContinuation;
	}

	public static boolean matchesPattern(String string, String pattern) {
		String notation = pattern;
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(string);
		return m.find();
	}

}
