package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.HashMap;
import java.util.Map;

public class VariableNamePatternGenerator {
	
	private static final String START_GROUP = "[";
	private static final String END_GROUP = "]";
	private static final String SPACE = " ";
	private static final String UNDERSCORE = "_";
	private static final String NEWLINE = "\\n";
	private static final String TAB = ""; // "\\t";
	private static final String WHITE_SPACE = SPACE + UNDERSCORE + NEWLINE+TAB;
	
	private static final String PATTERN_TAG = "<<Pat>>";
	
	private  Map<String, String> patternToVar = new HashMap<String, String>();
	
	public VariableNamePatternGenerator() {
		
	}
	
	public String getPatternFor(String var) {
		StringBuilder sb = new StringBuilder();
		
		for (int pos = 0; pos < var.length(); pos++) {
			String aChar = Parser.characterAt(pos, var);
			sb.append(START_GROUP);
			if (isWhiteSpace(aChar)) {
				sb.append(WHITE_SPACE);
			} else {
				if (aChar.toLowerCase().equals(aChar.toUpperCase()))
					sb.append(aChar);
				else
					sb.append(aChar.toLowerCase()+aChar.toUpperCase());
			}
			sb.append(END_GROUP);
		}
		String pattern = PATTERN_TAG+sb.toString();
		patternToVar.put(pattern, var);
		return pattern;
	}
	
	public String getVarForPattern(String pattern) {
		if (patternToVar.containsKey(pattern)) {
			return patternToVar.get(pattern);
		} else {
			if (isPattern(pattern)) {
				return pattern;
			} else {
				return pattern;
			}
		}
	}
	
	private boolean isWhiteSpace(String aChar) {
		if (aChar.equals(SPACE) ||
				aChar.equals(UNDERSCORE) ||
				aChar.equals(NEWLINE))
			return true;
		else
			return false;
	}
	
	public boolean isPattern(String pat) {
		if (pat == null) {
			System.out.println("pat is null");
		}
		return pat.startsWith(PATTERN_TAG);
	}

}
