package gov.anl.gss.sdm.server.model.support;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;

public class VensimExtendedMemoryComparer {

	private VensimLogProcessor vlp;
	private ExtendedMemoryLogProcessor emlp;
	private String extendedMemoryFile;
	private HeadTailMap headTailMap;
	private List<Double> time;
	
	private List<String> varsToAnalyze = new ArrayList<String>();
	
	private boolean addTime1DiffVars = false;

	public VensimExtendedMemoryComparer(String[] args /*String vensimFile, String mapFile, String extendedMemoryFile*/) {
		this.extendedMemoryFile = args[2];
		vlp = new VensimLogProcessor();
		vlp.loadLog(args[0]);
		emlp = new ExtendedMemoryLogProcessor(args[1], extendedMemoryFile);
		time = emlp.getValuesForVariable("Time");
		
		File f = new File(extendedMemoryFile);
		headTailMap = new HeadTailMap(f.getParent()+"/../structure/"+"headTailMap"+".tab");
		
		for (int i = 3; i < args.length; i++) {
			varsToAnalyze.add(args[i]);
		}
		
		if (args.length < 4)
			addTime1DiffVars = true;
		
	}

	public boolean same() {

		time = emlp.getValuesForVariable("Time");
		int timeValues = time.size();
		
	

		boolean isSame = true;
		int totalBadCount = 0;
		for (int t = 0; t < timeValues; t++) {
			
			if (t > 1)
				break;
			System.out.println("--- time ---");
			int badCount = 0;
			if (totalBadCount > 1250)
				break;
			for (String var : emlp.getVariables()) {
				if (var.equals("NAREPLACEMENT"))
					continue;
				if (var.equals("SAVEPER"))
					continue;
				List<Double> vensimValues = vlp.getValuesForVariable(var);
				List<Double> rssdValues = emlp.getValuesForVariable(var);

				double vv;
				double rv;
				

				if (vensimValues != null) {

					if (vensimValues.size() == 1) {
						vv = vensimValues.get(0).doubleValue();
					} else {
						vv = vensimValues.get(t).doubleValue();
					}
					rv = rssdValues.get(t);
					

//					if (Math.abs(rv) < 1.0 || Math.abs(rv) < 1.0)
//						continue;
					if (Math.abs(vv-rv)/vv > 0.10f) {
						isSame = false;
						totalBadCount++;
						System.out.println("var: "+var+" t: "+t+" v: "+vv+" r: "+rv);
						if (t < 5 && addTime1DiffVars && varsToAnalyze.size() < 1)
							varsToAnalyze.add(var);
							
						if (badCount++ > 50) {
							System.out.println(">50");
							break;
						}
					} else {

					}
				} else {
					System.out.println("No Vensim Values for "+var);
				}
			}
		}
		
		if (isSame)
			System.out.println("Results are the same");
		else
			System.out.println("!!!!!!!!! Results differ");
		return isSame;
	}
	
	private void merge() {
		File f = new File(extendedMemoryFile);
		BufferedWriter bw = Utilities.openFileForWriting(f.getParent()+"/../Excel/"+"MERGED"+".tab");
		
		List<Double> time = emlp.getValuesForVariable("Time");
		int timeValues = time.size();
		
		writeTabbed(bw, "Time", time);
	
			try {
				for (String var : emlp.getVariables()) {
					if (var.equals("NAREPLACEMENT"))
						continue;
					if (var.equals("SAVEPER"))
						continue;
					List<Double> vensimValues = vlp.getValuesForVariable(var);
					List<Double> rssdValues = emlp.getValuesForVariable(var);
					
					writeTabbed(bw, var+"(R)", rssdValues);
					writeTabbed(bw, var+"(V)", vensimValues);
				}
				
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

	}
	
	private void writeTabbed(BufferedWriter bw, String var, List<Double> list) {
		try {
			bw.append(var);
			for (Double v : list) {
				bw.append("\t");
				bw.append(format6(v));
			}
			bw.append("\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String format6(double v) {
		if (Double.isNaN(v))
			return "NaN";
		if (Double.isInfinite(v))
			return "Infinity";
		MathContext round3SigFig = new MathContext(5,RoundingMode.HALF_UP);
//		System.out.println((new BigDecimal(v)).round(round3SigFig));
		String pct = (new BigDecimal(v)).round(round3SigFig).toPlainString();
		return pct;
	}
	
	private void analyzeVariables() {
		for (String var : varsToAnalyze) {
			analyzeVariable(var, 0, /*time.size()-1*/ 40 , new LinkedHashSet<String>(), true, 0);
		}
	}
	
	private void analyzeVariable(String var, int timeStart, int timeEnd, HashSet<String> visited, boolean toplevel, int level) {
		List<Double> vensimValues = vlp.getValuesForVariable(var);
		List<Double> rssdValues = emlp.getValuesForVariable(var);
		
		
		
		int start = Math.max(0, timeStart);
		visited.add(var+"{"+start+"}");
		
		int timeValues = time.size();
		for (int t = start; t < timeEnd; t++) {
			double vv;
			double rv;
			
			if (vensimValues != null) {

				if (vensimValues.size() == 1) {
					vv = vensimValues.get(0).doubleValue();
				} else {
					vv = vensimValues.get(t).doubleValue();
				}
				rv = rssdValues.get(t);
				double diff = Math.abs(vv-rv)/vv;
				if (diff > 0.01f) {
					System.out.println(indent(level)+"var: "+var+" t: "+t+" v: "+vv+" r: "+rv+"  DIFF: "+diff);
					for (String v : headTailMap.getRHSVars(var)) {
						if (!visited.contains(v+"{"+(t-1)+"}"))
							analyzeVariable(v, t-1, t, visited, false, level+1);
					}
//					System.out.println(indent(level)+"### Term "+var+"{"+t+"}"+" @ t = "+t);
					if (!toplevel)
						return;
				} else {
//					System.out.println(indent(level)+"#OK# var: "+var+" t: "+t+" v: "+vv+" r: "+rv);
				}
				
			}
			
		}
	}
	
	public String indent(int i) {
		StringBuilder sb = new StringBuilder();
		for (int j = 0; j < i; j++) {
			sb.append("  ");
		}
		return sb.toString();
	}


	public static void main(String[] args) {
		
		// "c:/mdl/FRACC.map" "c:/mdl/FRACC_DOC_GRAPH_/runs/baselineHSQLDB"
		
		// "c:/MDL/arms2_DOC_GRAPH_/runs/Current.tab" "c:/MDL/arms2.map" "c:/MDL/ARMS2_DOC_GRAPH_/runs/baselineExtended.tab"
		String arg1 = args[0];
		String arg2 = args[1];
		String arg3 = args[2];

		VensimExtendedMemoryComparer stj = new VensimExtendedMemoryComparer(args /*arg1, arg2, arg3*/);
		stj.merge();
		boolean isSame = stj.same();
		stj.analyzeVariables();
	}


}
