package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class VensimLogProcessor {
	
	Map<String, List<Double>> vensimValues;
	
	public VensimLogProcessor() {
		vensimValues = new LinkedHashMap<String, List<Double>>();
	}
	
	public Map<String, List<Double>> loadLog(String file) {
		BufferedReader logReader = Utilities.openForRead(file);
		 return loadLog(logReader);
	}
	
	public List<Double> getValuesForVariable(String var) {
		return vensimValues.get(var);
	}
	
	public Map<String, List<Double>> loadLog(BufferedReader reader) {
		
		String aLine = null;
		try {
			aLine = reader.readLine();
			String[] vars = aLine.split("\t");
			for (int i = 0; i < vars.length; i++) {
				String var = vars[i];
				vars[i] =  Parser.removeQuotes(var); // var.replace("\"", "");
				vensimValues.put( Parser.removeQuotes(var) /* var.replace("\"", "") */ , new ArrayList<Double>());
			}
			aLine = reader.readLine();
			while (aLine != null) {
				int i = 0;
				String[] values = aLine.split("\t");
				for (String value : values) {
					String v = vars[i++];
					if (value.length() == 0)
						continue;
					double val = -999.99;
					try {
						val = Double.parseDouble(value);
					} catch (NumberFormatException e) {
						// TODO Auto-generated catch block
//						e.printStackTrace();
					}
					vensimValues.get(v).add(val);
				}
				aLine = reader.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
		return vensimValues;
	}
	
	public void print() {
		List<String> orderedVensimVars = new ArrayList<String>();
		for (String k : vensimValues.keySet())
			orderedVensimVars.add(k);
		Collections.sort(orderedVensimVars);
		for (String v : orderedVensimVars) {
			System.out.println("Vensim: "+v+" " +vensimValues.get(v).get(0));
		}
	}
}
