package gov.anl.gss.sdm.server.model.support;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class VensimRSSDComparer {
	
	private RSSDLogProcessor rssd;
	private VensimLogProcessor vensim;
	private String rssdLogFile;
	private String rssdSubscriptFile;
	private String vensimLogFile;
	private String comparisonFile;
	
	NameConverter nameConverter;
	EquationDependencyProcessor dependecies = 
		new EquationDependencyProcessor("output/initialRequiresExpandedLhsExpandedRHS.txt");
	
	String varToTrack = "Equilibrium Temperature[TwoC]";
	
	private Map<String, List<Double>> rssdValues;
	private Map<String, List<Double>> vensimValues;
	private Map<String, List<Double>> diffs = new LinkedHashMap<String, List<Double>>();
	private Map<String, List<Double>> pctDiffs = new LinkedHashMap<String, List<Double>>();
	
	public VensimRSSDComparer(String[] args) {
		rssdLogFile = args[0];
		rssdSubscriptFile = args[1];
		vensimLogFile = args[2];
		comparisonFile = args[3];
		nameConverter = new NameConverter(rssdSubscriptFile.replace("SubscriptSpace", "LegalNames"));
	}
	
    public static void main(String[] args) {
    	
    	VensimRSSDComparer vensimRSSDComparer = new VensimRSSDComparer(args);
    	vensimRSSDComparer.compare();
        }
	
	public void compare() {
		RSSDLogProcessor rSSDLogProcessor = new RSSDLogProcessor(nameConverter);
		rssdValues = rSSDLogProcessor.loadLog(rssdLogFile, rssdSubscriptFile);
//		rSSDLogProcessor.print();
		VensimLogProcessor vensimLogProcessor = new VensimLogProcessor();
		vensimValues = vensimLogProcessor.loadLog(vensimLogFile);
//		vensimLogProcessor.print();
		
		
		
		List<String> orderedVensimVars = new ArrayList<String>();
		for (String k : vensimValues.keySet())
			orderedVensimVars.add(k);
		Collections.sort(orderedVensimVars);
		
		List<String> orderedRSSDVars = new ArrayList<String>();
		for (String k : rssdValues.keySet())
			orderedRSSDVars.add(k);
		Collections.sort(orderedRSSDVars);
		
//		for (int i = 0; i < orderedRSSDVars.size(); i++) {
//			System.out.println(orderedRSSDVars.get(i)+" , "+
//					(i >= orderedVensimVars.size() ? " None " : orderedVensimVars.get(i)));
//		}
		
//		for (int i = 0; i < orderedRSSDVars.size(); i++) {
//			
//			if (nameConverter.getVensimName(orderedRSSDVars.get(i)) == null)
//				System.out.println("No Vensim Name: "+orderedRSSDVars.get(i));
//		}
//		
//		for (int i = 0; i < orderedVensimVars.size(); i++) {
//			if (nameConverter.getRssdName(orderedVensimVars.get(i)) == null)
//				System.out.println("No RSSD Name: "+orderedVensimVars.get(i));
//		}
//		
//		boolean cutShort = true;
//		
//		if(cutShort)
//			return;
//		
		for (String var : orderedVensimVars) {
			diffs.put(var, new ArrayList<Double>());
			pctDiffs.put(var, new ArrayList<Double>());
			List<Double> r = null;
			r = rssdValues.get(nameConverter.getRssdName(var));
			if (r == null)
				r = rssdValues.get(var);
			List<Double> v = vensimValues.get(var);
			List<Double> d = diffs.get(var);
			List<Double> pd = pctDiffs.get(var);
			
			if (r == null || v == null) {
				System.out.println("huh? "+var+" "+nameConverter.getRssdName(var));
				continue;
			}
			for (int i = 0; i < v.size(); i++) {
				if (i < r.size()) {
					d.add(r.get(i) - v.get(i));
					if (v.get(i) == 0.0) {
						if (r.get(i) - v.get(i) == 0.0) {
							pd.add(0.0);
						} else {
							pd.add(99999999.0);
						}
					} else {
					pd.add(100.0*((r.get(i) - v.get(i))/v.get(i)));
					}
				} else {
					d.add(-999.99);
					pd.add(-999.99);
				}
			}
		}
//		dependecies.buildDependencyTree(varToTrack);
		dependecies.showDependencyValues(varToTrack, rssdValues, vensimValues);
		write();
		writeTrack();
	}
	
	public void writeTrack() {
		BufferedWriter bw = Utilities.openFileForWriting("output/track.txt");
		
		writeListTrack(bw, "Variable", "Level", vensimValues.get("Time"), "Source");
		for (EquationDependencyProcessor.VariableLevel vl : EquationDependencyProcessor.masterList) {
			if (rssdValues.get(nameConverter.getRssdName(vl.variable)) != null)
				writeListTrack(bw, vl.variable, vl.level(), rssdValues.get(nameConverter.getRssdName(vl.variable)), "RSSD");
			else
				writeListTrack(bw, vl.variable, vl.level(), rssdValues.get(vl.variable), "RSSD");
			writeListTrack(bw, vl.variable, vl.level(), vensimValues.get(vl.variable), "Vensim");
			writeListTrack(bw, vl.variable, vl.level(), diffs.get(vl.variable), "Diff");
			writeListTrack(bw, vl.variable, vl.level(), pctDiffs.get(vl.variable), "pctDiff");
		}
		
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void write() {
		BufferedWriter bw = Utilities.openFileForWriting(comparisonFile);
		List<String> orderedVars = new ArrayList<String>();
		for (String k : vensimValues.keySet())
			orderedVars.add(k);
		Collections.sort(orderedVars);
		// first write
		
		writeList(bw, "Variable", vensimValues.get("Time"), "Source");
		for (String v : orderedVars) {
			if (rssdValues.get(nameConverter.getRssdName(v)) != null)
				writeList(bw, v, rssdValues.get(nameConverter.getRssdName(v)), "RSSD");
			else
				writeList(bw, v, rssdValues.get(v), "RSSD");
			writeList(bw, v, vensimValues.get(v), "Vensim");
			writeList(bw, v, diffs.get(v), "Diff");
			writeList(bw, v, pctDiffs.get(v), "pctDiff");
		}
		
		try {
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeList(BufferedWriter bw, String var, List<Double> al, String tag) {
	
		
		try {
			bw.append(tag);
			bw.append("/");
			bw.append(var);
		
			
			if (al == null) {
				bw.append("\n");
				return;
			}
			
			for (Double d : al) {
				bw.append("\t");
				bw.append(Double.toString(d));
			}
			bw.append("\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void writeListTrack(BufferedWriter bw, String var, String level, List<Double> al, String tag) {
	
		
		try {
			bw.append(tag);
			bw.append("/");
			bw.append(var);
			bw.append("\t");
			bw.append(level);
		
			
			if (al == null) {
				bw.append("\n");
				return;
			}
			
			for (Double d : al) {
				bw.append("\t");
				bw.append(Double.toString(d));
			}
			bw.append("\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
}
