package gov.anl.gss.sdm.server.model.support;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class VensimRSSDLogMerger {
	
	private Map<String, String> vensimValues = new LinkedHashMap<String, String>();
	private Map<String, String> rssdValues = new LinkedHashMap<String, String>();
	private Map<String, List<String>> diffValues = new LinkedHashMap<String, List<String>>();
	
	private String vensimFile;
	private String rssdFile;
	private String vensimRssdFile;
	
	public VensimRSSDLogMerger(String[] args) {
		vensimFile = args[0];
		rssdFile = args[1];
		vensimRssdFile = args[2];
	}
	
	public void load() {
		for (String line : Utilities.getAsList(vensimFile)) {
			String var =  Parser.removeQuotes(line.split("\t")[0]); // line.split("\t")[0].replace("\"", "");
			vensimValues.put(var, line);
		}
		for (String line : Utilities.getAsList(rssdFile)) {
			String var = line.split("\t")[0];
			rssdValues.put(var, line);
		}
		
		for (String var : rssdValues.keySet()) {
			double vensim;
			double rssd;
			double diff;
			if (!vensimValues.containsKey(var)) {
				System.out.println("Vensim Missing Var: "+var);
			}
			String[] v = vensimValues.get(var).split("\t");
			String[] r = rssdValues.get(var).split("\t");
			diffValues.put(var, new ArrayList<String>());
			
			for (int t = 1; t < r.length; t++) {
				int vt = t;
				if (r.length > v.length)
					vt = 1;
				vensim = Double.parseDouble(v[vt]);
				
				rssd = Double.parseDouble(r[t]);
				if (rssd == 0.0 && vensim == 0.0)
					diff = 1.0;
				else
					diff = vensim/rssd;
				diffValues.get(var).add(Double.toString(diff));
			}
			
		}
		
		List<String> vars = new ArrayList<String>(rssdValues.keySet());
		Collections.sort(vars);
		
		BufferedWriter bw = Utilities.openFileForWriting(vensimRssdFile);
		
		try {
			bw.append("software\tvariable\n");
			for (String var : vars) {
				bw.append("Vensim\t");
				bw.append(vensimValues.get(var));
				bw.append("\n");
				bw.append("RSSD\t");
				bw.append(rssdValues.get(var));
				bw.append("\n");
				bw.append("diff\t"+var);
				for (String v : diffValues.get(var)) {
					bw.append("\t"+v);
				}
				bw.append("\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void merge() {
		
	}
	
	public static void main(String[] args) {

		VensimRSSDLogMerger merger = new VensimRSSDLogMerger(args);
		merger.load();
		merger.merge();
	}

}
