package gov.anl.gss.sdm.server.model.support;


import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.mutable.MutableBoolean;

public class Verify {

	protected List<String> errorMessages;

	public Verify(List<String> errorMessages) {
		this.errorMessages = errorMessages;
	}

	public void verifyValue(MutableBoolean correct, List<List<String>> table,
			int row, int col, String value2) {
		if (verifyCell(table, row, col)) {
			String value1 = table.get(row).get(col);
			if (!value1.equals(value2)) {
				correct.setValue(false);
				errorMessages.add("Table Value: Expecting <" + value2
						+ "> found <" + value1 + ">");
			}
		} else {
			correct.setValue(false);
		}

	}

	public void verifyValueNotEmpty(MutableBoolean correct,
			List<List<String>> table, int row, int col) {
		if (verifyCell(table, row, col)) {
			String value1 = table.get(row).get(col);
			if (value1.trim().length() <= 0) {
				correct.setValue(false);
			}
		} else {
			correct.setValue(false);
		}
	}

	public void verifyValueInList(MutableBoolean correct,
			List<List<String>> table, int row, int col,
			List<List<String>> dbTable, int position) {
		boolean found = false;
		if (verifyCell(table, row, col)) {
			String value = table.get(row).get(col);
			for (List<String> r : dbTable) {
				if (r.get(position).equals(value)) {
					found = true;
				}
			}

			if (!found) {
				correct.setValue(false);
				errorMessages.add("Table Value: Expecting one of <" + getColumn(dbTable,0)
						+ "> found <" + value + ">");
			}
		} else {
			correct.setValue(false);
		}
	}
	
	public String getColumn(List<List<String>> dbTable, int position) {
		StringBuilder sb = new StringBuilder();
		
			for (List<String> r : dbTable) {
				sb.append("<"+r+">");
			}
			return sb.toString();
	}
			

	public void verifyRows(MutableBoolean correct, List<List<String>> table,
			int size) {

		if (table.size() != size) {
			correct.setValue(false);
			errorMessages.add("Incorrect Table Row Count: Expecting <" + size
					+ "> found <" + table.size() + ">");
		}

	}

	public void verifyRowsNotZero(MutableBoolean correct,
			List<List<String>> table) {

		if (table.size() < 2) {
			correct.setValue(false);
			errorMessages
					.add("Incorrect Table Row Count: Expecting at least 1 data found <"
							+ (table.size() - 1) + ">");
		}

	}

	public void verifyNumber(MutableBoolean correct, List<List<String>> table,
			int row, int col) {
		if (verifyCell(table, row, col)) {
			if (!isNumber(table.get(row).get(col))) {
				correct.setValue(false);
				errorMessages
						.add("Incorrect Table Row Count: Expecting at least 1 data found <"
								+ (table.size() - 1) + ">");
			}
		} else {
			correct.setValue(false);
		}

	}
	
	public static String getTableRow(List<List<String>> table, int row) {
		StringBuilder sb = new StringBuilder();
		
		for (String col : table.get(row)) {
			sb.append("<"+col+">");
		}
		
		return sb.toString();
	}

	public boolean verifyCell(List<List<String>> table, int row, int col) {
		boolean cellExists = false;

		if (row < table.size()) {
			if (col < table.get(row).size()) {
				cellExists = true;
			}
		}
		if (!cellExists) {
			errorMessages.add("Table Cell: (" + row + "," + col
					+ ") does not exist");
		}

		return cellExists;
	}
	
	public static boolean verifyCSV(String s) {
		// verify that this is a comma separated list of integers
		boolean verified = true;

		String cellValue = s;
		List<String> tokens = tokenize(cellValue);
		for (String i : tokens) {
			if (!isInteger(i))
				verified = false;
		}

		return verified;
	}
	
	public static boolean verifyCSV(List<List<String>> table, int row, int col) {
		// verify that this is a comma separated list of integers		
		String cellValue = getCellValue(table, row, col).trim();
		return verifyCSV(cellValue);		
	}
	
	public String[] splitCSV(MutableBoolean correct, List<List<String>> table,
			int row, int col) {
		if (!verifyCell(table, row, col)) {
			return new String[] {};
		}
		return table.get(row).get(col).split(",");
	}

	public static boolean isNumber(String token) {
		if (isInteger(token) || isReal(token))
			return true;
		else
			return false;
	}

	public static  boolean isInteger(String input) {
		try {
			Integer.parseInt(input);
			return true;
		} catch (Exception e) {
			return false;
		}
	}

	public static boolean isReal(String input) {
		if (input.equals(":NA:"))
			return true;
		try {
			Double.parseDouble(input);
			return true;
		} catch (Exception e) {
			return false;
		}
	}
	
	public static String getCellValue(List<List<String>> table, int row, int col) {
		return new String(table.get(row).get(0));
	}
	
	private static List<String> tokenize(String s) {
		MutableInteger pos = new MutableInteger();
		StringBuilder sb = new StringBuilder();
		List<String> tokens = new ArrayList<String>();
		
		while(inRange(s, pos)) {
			String character = characterAt(s, pos);
			if (character.equals(",")) {
				tokens.add(sb.toString().trim());
				sb.setLength(0);
			} else {
				sb.append(character);
			}
			pos.add(1);
			if (!inRange(s, pos)) {
				tokens.add(sb.toString().trim());
			}
		}
		
		
		return tokens;
	}
	
	public static String characterAt(String str, MutableInteger pos) {
		String s =  str.substring(pos.value(), pos.value()+1);
		return str.substring(pos.value(), pos.value()+1);
	}
	
	public static boolean inRange(String str, MutableInteger pos) {
		if (pos.value() <= str.length()-1)
			return true;
		else
			return false;
	}

}
