package gov.anl.gss.sdm.server.model.support.database;

import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.analysis.loops.RelationshipLoop;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

public class SDMDocDatabase {

	/**
	 * This is the gateway to the SDM-Doc/SDM-UI results database
	 * 
	 * Supported Task:
	 * 
	 * database creation based on .map file
	 * database update
	 * database access
	 */

	private String dbFile;
	private DBConnection dbConn;
	
	private PreparedStatement psInsertNoIVVLoopLinks;
	private PreparedStatement psInsertNoIVVLoop;
	
	private PreparedStatement psInsertWithIVVLoopLinks;
	private PreparedStatement psInsertWithIVVLoop;

	public SDMDocDatabase(String dbFile) {
		this.dbFile = dbFile;
		try {
			dbConn = new DBConnection(dbFile);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
	}
	
	public void initializeNewSDMDocDatabase() {
		createModelStructureTables();
		
		makePrepared();
	}
	

	public void load() {

	}

	public void createModelStructureTables() {
		createUses();
		createUsedBy();
		createModelVariables(); 
		createLoopsNoIVV(); 
		createLoopsNoIVVLinks(); 
		createLoopsWithIVV();
		createLoopsWithIVVLinks();
		createArrows();
		createArrowPolarity();
		createSubscriptDefinition(); 
		createSubscriptValues();
		createSubscriptUse();
		createModelMeta();

		createViews();
		createViewVariables();
		createVariableCharacteristics(); 

	}

	public void writeLoops(List<Loop> loops, String type) {
		if (type.equals(LoopManager.NO_IVV)) {
			writeNoIVVLoops(loops);
		} else {
			writeWithIVVLoops(loops);
		}
	}

	public void writeNoIVVLoops(List<Loop> loops) {
		for (Loop loop : loops) {
			insertNoIVVLoop(loop);
		}
	}

	public void writeWithIVVLoops(List<Loop> loops) {
		for (Loop loop : loops) {
			insertWithIVVLoop(loop);
		}
	}


	
	public void exec(String sql) {
		 try {
			dbConn.update(sql);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	

	public void createUses() {
		exec("drop table Uses If Exists");
		exec("CREATE Cached TABLE Uses ( id INTEGER IDENTITY, str_col VARCHAR(256), num_col INTEGER)");
	}

	public void createUsedBy() {
		exec("drop table UsedBy If Exists");
		exec("CREATE Cached TABLE UsedBy ( id INTEGER IDENTITY, VariableID INTEGER, UsesID INTEGER)");
	}
	public void createModelVariables() {
		exec("drop table Variables If Exists");
		exec("CREATE Cached TABLE Variables ( id INTEGER IDENTITY, VariableID INTEGER, Variable VARCHAR(256), "
				+ "Comment VARCHAR(1024), Units VARCHAR(256), Type VARCHAR(3))");
	}
	public void createLoopsNoIVV() {
		exec("drop table LoopsNoIVV If Exists");
		exec("CREATE Cached TABLE LoopsNoIVV ( id INTEGER IDENTITY, LoopIdent VARCHAR(32), Length INTEGER, Polarity VARCHAR(2))");
	}
	public void createLoopsNoIVVLinks() {
		exec("drop table LoopsNoIVVLinks If Exists");
		exec("CREATE Cached TABLE LoopsNoIVVLinks ( id INTEGER IDENTITY, LoopID INTEGER, INDEX INTEGER, VariableID INTEGER)");
	}
	public void createLoopsWithIVV() {
		exec("drop table LoopsWithIVV If Exists");
		exec("CREATE Cached TABLE LoopsWithIVV ( id INTEGER IDENTITY, LoopIdent VARCHAR(32), Length INTEGER, Polarity VARCHAR(2))");
	}
	public void createLoopsWithIVVLinks() {
		exec("drop table LoopsWithIVVLinks If Exists");
		exec("CREATE Cached TABLE LoopsWithIVVLinks ( id INTEGER IDENTITY, LoopID INTEGER, INDEX INTEGER, VariableID INTEGER)");
	}
	public void createArrows() {
		exec("drop table Arrows If Exists");
		exec("CREATE Cached TABLE Arrows ( id INTEGER IDENTITY, TailID INTEGER, HeadID INTEGER)");
	}
	public void createArrowPolarity() {
		exec("drop table ArrowPolarity If Exists");
		exec("CREATE Cached TABLE ArrowPolarity ( id INTEGER IDENTITY, ArrowID INTEGER, Polarity VARCHAR(2))");
	}
	public void createSubscriptDefinition() {
		exec("drop table SubscriptDefinition If Exists");
		exec("CREATE Cached TABLE SubscriptDefinition ( id INTEGER IDENTITY, Subscript VARCHAR(256))");
	}
	public void createSubscriptValues() {
		exec("drop table SubscriptValues If Exists");
		exec("CREATE Cached TABLE SubscriptValues ( id INTEGER IDENTITY, SubscriptID INTEGER, Index INTEGER, SubscriptValue VARCHAR(256))");
	}
	public void createSubscriptUse() {
		exec("drop table SubscriptUse If Exists");
		exec("CREATE Cached TABLE SubscriptUse ( id INTEGER IDENTITY, SubscriptID INTEGER, VariableID INTEGER, DimensionIndex INTEGER)");
	}
	public void createModelMeta() {
		exec("drop table ModelMeta If Exists");
		exec("CREATE Cached TABLE ModelMeta ( id INTEGER IDENTITY, MdlFile VARCHAR(256), IsMap VARCHAR(2), "
				+ "Runnable VARCHAR(2), InitialTime VARCHAR(256), FinalTime VARCHAR(256), TimeStep VARCHAR(256), "
				+ "SavePer VARCHAR(256), TimeUnits VARCHAR(256))");
	}
	
	public void createViews() {
		exec("drop table Views If Exists");
		exec("CREATE Cached TABLE Views ( id INTEGER IDENTITY, ViewName VARCHAR(256))");
	}
	public void createViewVariables() {
		exec("drop table ViewVariables If Exists");
		exec("CREATE Cached TABLE ViewVariables ( id INTEGER IDENTITY, ViewID INTEGER, VariableID INTEGER)");
	}
	public void createVariableCharacteristics() {
		exec("drop table VariableCharacteristics If Exists");
		exec("CREATE Cached TABLE VariableCharacteristics ( id INTEGER IDENTITY, VariableID INTEGER, UsesMacro VARCHAR(2), "
				+ "SourceInformation VARCHAR(2), DmlsUnits VARCHAR(2), MinMaxValues VARCHAR(2), FunctionSensitivityParameters VARCHAR(2), "
				+ "DataLookup VARCHAR(2), Undocumented VARCHAR(2), EmbeddedData VARCHAR(2), InView VARCHAR(2), NonmonotonicLookup VARCHAR(2), "
				+ "CascadingLookup VARCHAR(2), NonZeroEndSlopedLookup VARCHAR(2), IfThenElse VARCHAR(2), MinMax VARCHAR(2), "
				+ "StepPulseRelated VARCHAR(2), UnitErrorsOrWarnings VARCHAR(2), Used VARCHAR(2), Supplementary VARCHAR(2), "
				+ "SupplementaryUsed VARCHAR(2), ComplexVariable VARCHAR(2), ComplexStock VARCHAR(2))");
	}

	public String getDbFile() {
		return dbFile;
	} 
	
	public void shutdown() {
		if (dbConn == null)
			return;
		try {
			dbConn.shutdown();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void insertNoIVVLoop(Loop loop) {
		
//		CREATE Cached TABLE LoopsNoIVV ( id INTEGER IDENTITY, LoopIdent VARCHAR(32), Length INTEGER, Polarity VARCHAR(2))
//		sql = "insert into LoopsNoIVV(LoopIdent, Length, Polarity) values( ?, ?, ?)";
//		psInsertNoIVVLoop = dbConn.prepareStatement(sql);
//		"Select id from LoopsNoIVV where LoopIdent = '"+loop.getLoopIdentifierHTML()+"'";
		
		try {
			psInsertNoIVVLoop.setString(1, loop.getLoopIdentifierHTML());
			psInsertNoIVVLoop.setInt(2, loop.getSize());
			psInsertNoIVVLoop.setString(3, loop.getLoopPolarity());
			
			System.out.println("NoIVVLinks: "+loop.getLoopIdentifierHTML()+" "+loop.getSize()+" "+loop.getLoopPolarity());
			
			psInsertNoIVVLoop.executeUpdate();
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		
		String sql = "Select id from LoopsNoIVV where LoopIdent = '"+loop.getLoopIdentifierHTML()+"'";
		int id = DBUtilities.getSingleIntegerValue(dbConn, sql);
		for (int i = 0; i < loop.getSize(); i++) {
			try {
				
//				sql = "insert into LoopsNoIVVLinks(LoopID, INDEX, VariableID) values( ?, ?, ?)";
//				psInsertNoIVVLoopLinks = dbConn.prepareStatement(sql);
				
				psInsertNoIVVLoopLinks.setInt(1, id);
				psInsertNoIVVLoopLinks.setInt(2, i);
				psInsertNoIVVLoopLinks.setInt(3, Integer.parseInt(loop.getLoop().get(i)));
				
				System.out.println("NoIVVLinks: "+id+" "+i+" "+Integer.parseInt(loop.getLoop().get(i)));
				
				psInsertNoIVVLoopLinks.executeUpdate();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public void insertWithIVVLoop(Loop loop) {
		
		try {
			psInsertWithIVVLoop.setString(1, loop.getLoopIdentifierHTML());
			psInsertWithIVVLoop.setInt(2, loop.getSize());
			psInsertWithIVVLoop.setString(3, loop.getLoopPolarity());
			psInsertWithIVVLoop.executeUpdate();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		int id = DBUtilities.getSingleIntegerValue(dbConn, "Select id from LoopsNoIVV where LoopIdent = '"+loop.getLoopIdentifierHTML()+"'");
		for (int i = 0; i < loop.getSize(); i++) {
			try {
				psInsertWithIVVLoopLinks.setInt(1, id);
				psInsertWithIVVLoopLinks.setInt(2, i);
				psInsertWithIVVLoopLinks.setInt(3, Integer.parseInt(loop.getLoop().get(i)));
				psInsertWithIVVLoopLinks.executeUpdate();
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	private void makePrepared() {
		String sql;
		sql = "insert into LoopsNoIVV(LoopIdent, Length, Polarity) values( ?, ?, ?)";
		psInsertNoIVVLoop = dbConn.prepareStatement(sql);
		
		sql = "insert into LoopsNoIVVLinks(LoopID, INDEX, VariableID) values( ?, ?, ?)";
		psInsertNoIVVLoopLinks = dbConn.prepareStatement(sql);
		
		sql = "insert into LoopsWithIVV(LoopIdent, Length, Polarity) values( ?, ?, ?)";
		psInsertWithIVVLoop = dbConn.prepareStatement(sql);
		
		sql = "insert into LoopsWithIVVLinks(LoopID, INDEX, VariableID) values( ?, ?, ?)";
		psInsertWithIVVLoopLinks = dbConn.prepareStatement(sql);

	}
	
	public Set<Loop> getNoIVVLoopsContaining(String varA, String numA) {
		Set<Loop> loops = new LinkedHashSet<Loop>();
		
			Statement st1 = null;
			ResultSet rs1 = null;

			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				
//					exec("CREATE Cached TABLE LoopsNoIVV ( id INTEGER IDENTITY, LoopIdent VARCHAR(32), Length INTEGER, Polarity VARCHAR(2))");
//					exec("CREATE Cached TABLE LoopsNoIVVLinks ( id INTEGER IDENTITY, LoopID INTEGER, INDEX INTEGER, VariableID INTEGER)");
				
				String sql = "Select a.LoopID, b.LoopIdent "
						+ "from LoopsNoIVVLinks a, LoopsNoIVV b "
						+ "where VariableID = "+numA+" and a.LoopID = b.ID";
				rs1 = st1.executeQuery(sql);
				loops = loadNoIVVLoops(rs1);
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		
		return loops;
	}
	
	public Set<Loop> loadNoIVVLoops(ResultSet rs1) throws SQLException {
		
		Set<Loop> loops = new LinkedHashSet<Loop>();
		
		// first find the loops that are in common
		for (; rs1.next(); ) {
			int loopID = rs1.getInt(1);
			String loopIdent = rs1.getString(2);
			
			Statement st1 = null;
			ResultSet rs2 = null;
			
			// now get all the links in the loop

			List<String> aLoopList = new ArrayList<String>();
			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select VariableID from LoopsNoIVVLinks where LoopID = "+loopID+" order by INDEX";
				rs2 = st1.executeQuery(sql);
				for (; rs2.next(); ) {
					aLoopList.add(rs2.getString(1));
				}
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			// create the loop Object
			Loop aLoop = new Loop(aLoopList);
			aLoop.setLoopIdentifier(loopIdent);
			// Loop loop, int distAB, int distBA
			loops.add(aLoop);
		}
		return loops;
	}
	
	public int getMaxSizeNoIVVLoop() {
		return DBUtilities.getSingleIntegerValue(dbConn, "Select max(length) from LoopsNoIVV");
	}
	
	public int getMaxSizeWithIVVLoop() {
		return DBUtilities.getSingleIntegerValue(dbConn, "Select max(length) from LoopsWithIVV");
	}
	
	public Set<Loop> getLoopsOfLength(int length) {
		Set<Loop> loops = new LinkedHashSet<Loop>();
		
			Statement st1 = null;
			ResultSet rs1 = null;

			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select a.LoopID, b.LoopIdent "
						+ "from LoopsNoIVVLinks a, LoopsNoIVV b "
						+ "where b.length = "+length+" and a.LoopID = b.ID";
				rs1 = st1.executeQuery(sql);
				loops = loadWithIVVLoops(rs1);
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		
		return loops;
	}
	
	public Set<Loop> getWithIVVLoopsContaining(String varA, String numA) {
		Set<Loop> loops = new LinkedHashSet<Loop>();
		
			Statement st1 = null;
			ResultSet rs1 = null;

			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select a.LoopID, b.LoopIdent "
						+ "from LoopsWithIVVLinks a, LoopsWithIVV b "
						+ "where VariableID = "+numA+" and a.LoopID = b.ID";
				rs1 = st1.executeQuery(sql);
				loops = loadWithIVVLoops(rs1);
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		
		return loops;
	}
	
	public Set<Loop> loadWithIVVLoops(ResultSet rs1) throws SQLException {
		
		Set<Loop> loops = new LinkedHashSet<Loop>();
		
		// first find the loops that are in common
		for (; rs1.next(); ) {
			int loopID = rs1.getInt(1);
			String loopIdent = rs1.getString(2);
			
			Statement st1 = null;
			ResultSet rs2 = null;
			
			// now get all the links in the loop

			List<String> aLoopList = new ArrayList<String>();
			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select VariableID from LoopsWithIVVLinks where LoopID = "+loopID+" order by INDEX";
				rs2 = st1.executeQuery(sql);
				for (; rs2.next(); ) {
					aLoopList.add(rs2.getString(1));
				}
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			// create the loop Object
			Loop aLoop = new Loop(aLoopList);
			aLoop.setLoopIdentifier(loopIdent);
			// Loop loop, int distAB, int distBA
			loops.add(aLoop);
		}
		return loops;
	}
	
	
		
//		exec("CREATE Cached TABLE LoopsNoIVV ( id INTEGER IDENTITY, LoopIdent VARCHAR(32), Length INTEGER, Polarity VARCHAR(2))");
//		exec("CREATE Cached TABLE LoopsNoIVVLinks ( id INTEGER IDENTITY, LoopID INTEGER, INDEX INTEGER, VariableID INTEGER)");
	
	public Set<RelationshipLoop> getRelationshipLoops(String varA, String varB, String numA, String numB) {
		Set<RelationshipLoop> loops = new LinkedHashSet<RelationshipLoop>();
		
			Statement st1 = null;
			ResultSet rs1 = null;
			

			//        exec("CREATE Cached TABLE Variables ( id INTEGER IDENTITY, VariableName VARCHAR(256), "
			//				+ "VariableType VARCHAR(2), VariableIndex INTEGER, numElements INTEGER)");

			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select a.LoopID, a.VariableID, a.index, b.VariableID, b.index, c.LoopIdent "
						+ "from LoopsNoIVVLinks a, LoopsNoIVVLinks b,  LoopsNoIVV c "
						+ "where a.LoopID = b.LoopID and a.VariableID = "+numA+" and b.variableID = "+numB+" " 
						+ "and c.id = a.LoopID";
				rs1 = st1.executeQuery(sql);
				loops = loadVariables(rs1);
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			
		
		return loops;
	}
	
	public Set<RelationshipLoop> loadVariables(ResultSet rs1) throws SQLException {
		
		Set<RelationshipLoop> loops = new LinkedHashSet<RelationshipLoop>();
		
		// first find the loops that are in common
		for (; rs1.next(); ) {
			int loopID = rs1.getInt(1);
			int varAID = rs1.getInt(2);
			int varAIndex = rs1.getInt(3);
			int varBID = rs1.getInt(4);
			int varBIndex = rs1.getInt(5);
			String loopIdent = rs1.getString(6);
			
			Statement st1 = null;
			ResultSet rs2 = null;
			
			// now get all the links in the loop

			List<String> aLoopList = new ArrayList<String>();
			st1 = dbConn.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
			try {
				
				String sql = "Select VariableID from LoopsNoIVVLinks where id = "+loopID+" order by INDEX";
				rs2 = st1.executeQuery(sql);
				for (; rs2.next(); ) {
					aLoopList.add(rs2.getString(1));
				}
				st1.close();
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			// create the loop Object
			Loop aLoop = new Loop(aLoopList);
			aLoop.setLoopIdentifier(loopIdent);
			// Loop loop, int distAB, int distBA
			loops.add(new RelationshipLoop(aLoop, dist(aLoopList.size(), varAIndex, varBIndex), 
					dist(aLoopList.size(), varBIndex, varAIndex)));
		}
		return loops;
	}
	
	private int dist(int size, int indexA, int indexB) {
		int d;
		if (indexA < indexB) {
			d = indexB - indexA;
		} else {
			d = size - indexA + indexB;
		}
		return d;
	}
	
	public int getNumLoopsWithPolarity(String polarity) {
		return DBUtilities.getSingleIntegerValue(dbConn, "select count(ID) from LoopNoIVV where Polarity = '"+polarity+"'");
	}

}
