package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.analysis.graphtheory.AdjacencyMatrix;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.AdjacencyMatrixToMap;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.KeyValue;
import gov.anl.gss.sdm.server.model.analysis.graphtheory.Position;
import gov.anl.gss.sdm.server.model.analysis.loops.GraphCyclesThread;
import gov.anl.gss.sdm.server.model.analysis.loops.GraphCyclesThreadMinimal;
import gov.anl.gss.sdm.server.model.analysis.loops.Loop;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.awt.Font;
import java.awt.FontMetrics;
import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

public class AdjacencyToMap {
	
	public static final int TEXTPOS_INSIDE = 0;
	public static final int TEXTPOS_ABOVE = 3; // 3 - not flipped
	public static final int TEXTPOS_BELOW = 1; // 1 - not flipped
	public static final int TEXTPOS_LEFT = 2;
	public static final int TEXTPOS_RIGHT = 4;
	
	public static final int FORMAT_CIRCLE = 0;
	public static final int FORMAT_ARC = 1;
	public static final int FORMAT_RECTANGLE = 2;
	
	public int ADJUST_SIZE = 20;

	private String matrixFile;
	private String mapFile;
	
	private int format;
	
	// for arcs and rectangles
	private List<Position> locsLeft;
	private List<Position> locsRight;
	
	// for rectangles
	private List<Position> locsAbove;
	private List<Position> locsBelow;
	
	// for circles
	private List<Position> locsCircle;
	
	private int aboveCount = 0;
	private int belowCount = 0;
	
	private int numLeft = 0;
	private int numAbove = 0;
	private int numBelow = 0;
	private int numRight = 0;
	private int numCircle = 0;
	
	private int numPerArcLeft = 0;
	private int numPerArcRight = 0;
	
	private int circleRadius;
	private int arcsRadius;
	private int rectangleWidth;
	private double circleThreshhold;
	private double arcsArcDegrees;
	
	private Map<Integer, String> arrowMapping = new LinkedHashMap<Integer, String>();
	
	private Map<Integer, List<List<String>>> byLoopLength = 
			new LinkedHashMap<Integer, List<List<String>>>();
	private int minimumLoopLength = Integer.MAX_VALUE;
	private int maximumLoopLength = Integer.MIN_VALUE;
	
	// this will be the ordered list of positions to be used
	List<Position> orderedLocs = new ArrayList<Position>();;
	
	private AdjacencyMatrixToMap adjMat = new AdjacencyMatrixToMap();
	
	boolean byGroup;
	List<String> groups;
	List<String> shapeColors;
	List<String> textColors;
	boolean byIn; 
	boolean byOut; 
	String groupFile;
	boolean byLength; 
	boolean bySpecific;
	boolean byUpTo;
	int maxlen;
	int intergroupGap;
	
	int shapeSize;
	int arrowWidth;
	
//	int RADIUS = 700;
	int nVars = 0;

	public AdjacencyToMap(String file1, String file2, int format, 
			int circleRadius,int arcsRadius,int rectangleWidth, double circleThreshhold, int intergroupGap, 
			double arcDegrees,boolean byGroup, List<String> groups, List<String> shapeColors, 
			List<String> textColors, boolean byIn,
			boolean byOut,String groupFile, boolean byLength, int maxlen,boolean specific,
			boolean byUpTo, int shapeSize, int arrowWidth) {

		matrixFile = file1;
		mapFile = file2;
		this.format = format;

		this.circleRadius = circleRadius;
		this.arcsRadius = arcsRadius;
		this.rectangleWidth = rectangleWidth;
		this.circleThreshhold = circleThreshhold;
		this.arcsArcDegrees = arcDegrees;

		this.byGroup = byGroup;
		this.groups = groups;
		this.shapeColors = shapeColors;
		this.textColors = textColors;
		this.byIn = byIn; 
		this.byOut = byOut; 
		this.groupFile = groupFile;
		this.byLength = byLength; 
		this.bySpecific = specific;
		this.byUpTo = byUpTo;
		this.maxlen = maxlen;
		this.intergroupGap = intergroupGap;
		this.shapeSize = shapeSize;
		this.arrowWidth = arrowWidth;

		arrowMapping.put(1, "0");
		arrowMapping.put(2, "12");
		arrowMapping.put(3, "13");
		arrowMapping.put(4, "14");
		arrowMapping.put(5, "15");
		arrowMapping.put(6, "16");

	}


	public static void main(String[] args) {
		AdjacencyToMap cleaner = null;

		cleaner = new AdjacencyToMap(args[0], args[1], 0, 500, 500, 500, 10.0, 0, 30.0, 
				false, null, null, null, false, false, "", false, 4, false, false,0,2);
		if (cleaner != null)
			cleaner.execute();
	}

	public void execute() {


		nVars = adjMat.loadFromFile(matrixFile);
		adjMat.setGroupFile(groupFile);
		adjMat.loadGroupFile(groupFile, groups, shapeColors, textColors);
		writeMapFile(mapFile);

		if (byGroup) {

			

			if (byIn) {
				for (String g : groups) { 
					writeMapByCharacteristics(mapFile, false,g,"In", -1);
//				writeMapByCharacteristics(mapFile, false,"Manage","In", -1);  
//				writeMapByCharacteristics(mapFile, false,"Connect","In",-1);  
//				writeMapByCharacteristics(mapFile, false,"Distribute","In", -1);  
//				writeMapByCharacteristics(mapFile, false,"Supply","In", -1);  
				}
			}
			if (byOut) {
				for (String g : groups) {
					writeMapByCharacteristics(mapFile, false,g,"Out", -1);
//				writeMapByCharacteristics(mapFile, false,"Manage","Out", -1);  
//				writeMapByCharacteristics(mapFile, false,"Connect","Out", -1);  
//				writeMapByCharacteristics(mapFile, false,"Distribute","Out", -1);  
//				writeMapByCharacteristics(mapFile, false,"Supply","Out", -1);  
				}
				 
			}
		}
		
		if (byLength) {

			// now generate loop length dependent maps

			GraphCyclesThreadMinimal gc = null;
			Thread graphCycleThreadMinimal = null;

			//		UserSelectedOptions options = InformationManagers.getInstance()
			//				.getUserSelectedOptions();
			//		if (options == null) {
			//			options = new UserSelectedOptions(true);
			//			InformationManagers.getInstance().setUserSelectedOptions(options);
			//		}
			//
			//		int maxLen = Integer.parseInt(options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM));

			gc = new GraphCyclesThreadMinimal(getEdges(), maxlen);
			graphCycleThreadMinimal = gc;
			graphCycleThreadMinimal.start();
			graphCycleThreadMinimal.setPriority(Thread.MIN_PRIORITY);
			boolean haveCycles = false;
			boolean done = false;
			while (!done) {
				if (!graphCycleThreadMinimal.isAlive()) {
					if (gc.getCycles() != null) {
						haveCycles = true;
					}
					done = true;
				} else {
					try {
						Thread.sleep(5000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
			}

			if (!haveCycles) {
				System.out.println("Failed to find any loops");
			} else {
				for (List<String> cycle : gc.getCycles()) {
					//				System.out.println("loops: "+cycle);
					int s = cycle.size();

					minimumLoopLength = Math.min(s, minimumLoopLength);
					maximumLoopLength = Math.max(s, maximumLoopLength);

					if (!byLoopLength.containsKey(s)) {
						byLoopLength.put(s, new ArrayList<List<String>>());
					}
					byLoopLength.get(s).add(cycle);
				}
			}

			for (int length = minimumLoopLength; length <= maximumLoopLength; length++) {
				if (bySpecific)
					writeMapByCharacteristics(mapFile, true,"","", length, bySpecific, false); 
				
				if (byUpTo)
					writeMapByCharacteristics(mapFile, true,"","", length, false, true); 
			}
		}
	}
	
	public List<String[]> getEdges() {
		
		// private Map<String, Set<String>> usedBy; // key = tail value = head
		
		List<String[]> edges = new ArrayList<String[]>();
		for (String head : adjMat.getVariablesOrdered()) {
			if (Parser.isSystemVariable(head))
				continue;
			for (String tail : adjMat.getPredecessor().get(head)) {
				if (Parser.isSystemVariable(tail))
					continue;
				String[] anEdge = new String[2];
				anEdge[0] = new String(tail);
				anEdge[1] = new String(head);
				edges.add(anEdge);
			}
		}
		
		return edges;
	}
	
	public boolean writeMapFile(String file) {
		return writeMapByCharacteristics(file, true,"","", -1);  // Manage
	}
	
	public AdjacencyMatrix buildMatrixFromCharcteristics(AdjacencyMatrixToMap full, 
			boolean allGroups, String group, String inOut, int length, boolean bySpecific) {

		AdjacencyMatrix m = new AdjacencyMatrix();
		for (String var : full.getSuccessor().keySet()) {
			m.getSuccessor().put(var, new LinkedHashSet<String>());
			m.getPredecessor().put(var, new LinkedHashSet<String>());
		}

		if (length <= 0) {

			for (String var : full.getSuccessor().keySet()) {
				for (String s : full.getSuccessor().get(var)) {
					boolean add = false;
					if (allGroups) {
						add = true;
					} else {
						if (full.getGroupForVariable(var).equals(group) && inOut.equals("Out")) {
							add = true;
						} else if (full.getGroupForVariable(s).equals(group) && inOut.equals("In")) {
							add = true;
						}
					}
					if (add) {
						m.addSuccessor(var, s);
					} else {
						System.out.println("CHAR: skipping "+var+" ->  "+s);
					}
				}

			}
		} else {
			int start = 0;
			int end = length;
			if (bySpecific) {
				start = length;
			}
			for (int l = start; l <= length; l++) {
				if (byLoopLength.containsKey(l)) {
					for (List<String> loop : byLoopLength.get(l)) {
						for (int ptr = 0; ptr < loop.size()-1; ptr++) {
							m.addSuccessor(loop.get(ptr), loop.get(ptr+1));
						}
						m.addSuccessor(loop.get(loop.size()-1), loop.get(0));
					}
				}
			}
		}

		return m;
	}
	
	public boolean writeMapByCharacteristics(String file, boolean allGroups, String group, String inOut, int length) {
		return writeMapByCharacteristics(file, allGroups,group,inOut,length,false,false);
	}
	
	public boolean writeMapByCharacteristics(String file, boolean allGroups, String group, String inOut, int length,
			boolean bySpecific, boolean byUpTo) {
		
		String annotatedFile = null;
		
		if (allGroups) {
			if (length > 0) {
				if (bySpecific)
					annotatedFile = file.replace("_ALL.mdl", "_ALL-l-"+length+".mdl");
				else
					annotatedFile = file.replace("_ALL.mdl", "_ALL-u-"+length+".mdl");
			} else {
				annotatedFile = file;
			}
		} else {
			annotatedFile = file.replace(".mdl", "-"+group+"-"+inOut+".mdl").replace("_ALL", "");
		}
		
		// add gap and threshold to file name
		String gt = "-gtsa-"+intergroupGap+"-"+(int) circleThreshhold+"-"+shapeSize+"-"+arrowWidth;
		annotatedFile = annotatedFile.replace(".mdl",gt+".mdl");
		
		System.out.println(annotatedFile);
		
		// create a new adjacency matrix based on characteristics
		AdjacencyMatrix am = buildMatrixFromCharcteristics(adjMat, allGroups, group, inOut, length, bySpecific);
		

		// for graphics calculations
		GraphicsHelper gh = new GraphicsHelper();
		int numVars = 0;
		BufferedWriter bw = Utilities.openFileForWritingUTF8(annotatedFile);
		try {
			bw.append("{UTF-8}\n");
			
			// all variables will always be present

			// variable and number of outlinks
			List<KeyValue> al = new ArrayList<KeyValue>();
			
			for (String var : adjMat.getVariablesOrdered()) {
				if (Parser.isSystemVariable(var))
					continue;
				al.add(new KeyValue(var, am.getRowCounts(var)));
			}

			numVars = al.size();

			// assign vensim IDs based on the order in which they'll be written to the file
			Map<String, Integer> vensimIDs = new HashMap<String, Integer>();
			int vensimID = 1;
			for (KeyValue kv : al) {
				vensimIDs.put(kv.getKey(), vensimID++);
			}
			
			int id = vensimIDs.size()+1;

			// write the equations to the mdl file
			for (KeyValue kv : al) {
				String var = kv.getKey();
				if (Parser.isSystemVariable(var))
					continue;
				StringBuilder sb = new StringBuilder();
				sb.append("\" "+var+" \""+" = A FUNCTION OF( ");
				int num = 0;
				for (String s : am.getPredecessor().get(var)) {
					if (num++ > 0)
						sb.append(",\\\n");
					sb.append("\" "+s+" \"");
				}
				sb.append(")\n\t~\t\n\t~\t\t|\n\n");
				bw.append(sb.toString());
			}
			bw.append("********************************************************\n");
			bw.append(".Control\n");
			bw.append("****************************************************~\n");
			bw.append("Simulation Control Parameters\n");
			bw.append("\t|\n\n");

			bw.append("FINAL TIME  = 100\n\t~\tMonth\n\t~\tThe final time for the simulation.\n\t\t|\n\n");
			bw.append("INITIAL TIME  = 0\n\t~\tMonth\n\t~\tThe initial time for the simulation.\n\t\t|\n");
			bw.append("SAVEPER  = TIME STEP\n\t~\tMonth [0,?]\n\t~\tThe frequency with which output is stored.\n\t\t|\n\n");
			bw.append("TIME STEP  = 1\n\t~\tMonth [0,?]\n\t~\tThe final time for the simulation.\n\t\t|\n\n");

			bw.append("\\\\\\---/// Sketch information - do not modify anything except names\n");
			bw.append("V300  Do not put anything below this section - it will be ignored\n");
			bw.append("*View 1\n");
			bw.append("$192-192-192,0,Calibri Light|16|B|0-0-0|0-0-0|0-0-255|-1--1--1|-1--1--1|96,96,5,0\n");

			//			bw.append("$192-192-192,0,Times New Roman|12||0-0-0|0-0-0|0-0-255|-1--1--1|-1--1--1|96,96,100,0\n");

			// 10,1,Stock,662,381,40,20,3,3,0,0,0,0,0,0
			// 1,3,5,1,4,0,0,22,0,0,0,-1--1--1,,1|(578,381)|

			// create a map with locations for all variables

			Map<String, Position> locations = new HashMap<String, Position>();



			if (format == FORMAT_ARC) {
				getVariableArcPositions(numVars, arcsRadius);
			} else if (format == FORMAT_CIRCLE) {
				getVariableCirclePositions(numVars, circleRadius);
			} else {
				getVariableRectanglePositions(numVars, rectangleWidth);
			}

			int i = 0;
			String firstVar = al.get(0).getKey();
			String processingGroup = adjMat.getGroupForVariable(firstVar);
			for (KeyValue kv : al) {
				if (Parser.isSystemVariable(kv.getKey()))
					continue;
				if (format == FORMAT_CIRCLE) {
					String thisGroup = adjMat.getGroupForVariable(kv.getKey());
					if (!thisGroup.equals(processingGroup)) {
						i += intergroupGap;
						processingGroup = thisGroup;
					}

					if (i >= orderedLocs.size()) {
						System.out.println("Not Enough Locations");
					}
				}
				locations.put(kv.getKey(), orderedLocs.get(i));
				i += 1;
			}
			Map<String, Position> relocatedNamePositions = null;

			locations = shift(locations);
			
			if (format == FORMAT_CIRCLE) {

				relocatedNamePositions = createRelocatedNamePositionsCircle(locations);
				addRelocatedNamePositionsToLocations(relocatedNamePositions, locations);
				locations = shift(locations);
			}

			
			int count = 1;

			for (KeyValue kv : al) {
				//			for (Integer col : adjMat.getColumnToName().keySet()) {
				if (Parser.isSystemVariable(kv.getKey()))
					continue;

				String name = kv.getKey();

				//				List<String> split = splitWords(name, 17);
				//				System.out.println("Split: "+split);

				int myID = vensimIDs.get(name);

				int col = 0;
				col = adjMat.getNameToColumn().get(name);

				int x = (int) Double.parseDouble(locations.get(name).getX());
				int y = (int) Double.parseDouble(locations.get(name).getY());
				int w = gh.getWidth(name)/2;
				w = 5;
				int h = gh.getHeight(); // +heightAdjustment(locations.get(name));

				int shape = 8; // clear box
				if (format == FORMAT_RECTANGLE) {
					shape = 3;
					w = 2*gh.getHeight();
				}

				if (format == FORMAT_ARC || format == FORMAT_CIRCLE) {
					shape = 2;
					w = shapeSize; // 5; // 15
					h = shapeSize; // 5; // 15
				}

				//				10,39,"44.0 Fuel Refining and Processing Fuels", 1172,135 , 12,12 ,2,3,0,2, 3 ,0,0,0,-1--1--1,0-0-0,|0||255-255-255,0,0,0,0,0,0

				int hasf = 7;
				String shapeColor = null;
				String textColor = null;
				shapeColor = adjMat.getShapeColorForVariable(name);
				textColor = adjMat.getTextColorForVariable(name);

				if (requiresNameRelocation(locations.get(name))) {
					// this is where we change font color to white
					bw.append("10,"+myID+","+"\" "+name+" \""+","+x+","+y+","+w+","+h+","+
							shape+",3,0,"+hasf+","+locations.get(name).getTextPos()+",0,0,0,"+shapeColor+","+shapeColor+",|0||"+"255-255-255"+",0,0,0,0,0,0\n");
				} else {
					bw.append("10,"+myID+","+"\" "+name+" \""+","+x+","+y+","+w+","+h+","+shape+",3,0,"+hasf+","+
							locations.get(name).getTextPos()+",0,0,0,"+shapeColor+","+shapeColor+",|16||"+textColor+",0,0,0,0,0,0\n");
				}
			}
			for (KeyValue kv : al) {
				String name = kv.getKey();

				int myID = vensimIDs.get(name);
				int col = adjMat.getNameToColumn().get(name);
				
				String color = null;
				color = adjMat.getShapeColorForVariable(name);
				
				String effectiveWidth = arrowMapping.get(arrowWidth);
				
				for (String ar : am.getRowSet(adjMat.getColumnToName().get(col))) {
					bw.append("1,"+(id++)+","+myID+","+
							vensimIDs.get(ar)+",0,0,0,"+effectiveWidth+",3,64,0,"+color+",|0||-1--1--1,1|"+midPoint(locations, name, ar)+"|\n");
				}
			}
			for (KeyValue kv : al) {
				String name = kv.getKey();
				if (requiresNameRelocation(locations.get(name))) {
					// this is where we write out comment
					String reloc = name + "RELOC";
					int x = Integer.parseInt(locations.get(reloc).getX());
					int y = Integer.parseInt(locations.get(reloc).getY());
					int w = shapeSize;
					
					String color = adjMat.getTextColorForVariable(name);
					String white = "255-255-255";
					int hasf = 7;
					int h = 1;

					bw.append("12,"+(id++)+",0,"+x+","+y+","+w+","+h+
							",8,7,0,"+hasf+","+locations.get(reloc).getTextPos()+",0,0,0,"+white+","+white+",|16||"+color+",0,0,0,0,0,0\n");
					bw.append(name+"\n");
				}
			}
			bw.append("///---\\\\\\\n");


			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
	}
	
	public void addRelocatedNamePositionsToLocations(Map<String, Position> relocatedNamePositions, 
			Map<String, Position> locations) {
		
		for (String n : relocatedNamePositions.keySet()) {
			locations.put(n+"RELOC", relocatedNamePositions.get(n));
		}
		
	}
	
	public boolean requiresNameRelocation(Position position) {
//		return false;
		return position.getHeightAdjustment() != 0;
		
	}
	
	public Map<String, Position> createRelocatedNamePositionsCircle(Map<String, Position> locations) {
		
		
		int numAbove = 0;
		int numBelow = 0;
		
		int maxYAbove = Integer.MAX_VALUE;
		int maxYBelow = Integer.MIN_VALUE;
		
		Position peakAbove = null;
		Position peakBelow = null;
		
		String nameBelow = null;
		String nameAbove = null;
		
		for (String name : locations.keySet()) {
			Position pos = locations.get(name);
			if (!requiresNameRelocation(pos))
				continue;
			if (pos.isTextAbove()) {
				numAbove++;
				
				// these are actually smaller Ys for peaks
				if (Integer.parseInt(pos.getY()) < maxYAbove) {
					maxYAbove = Integer.parseInt(pos.getY());
					peakAbove = pos;
					nameAbove = name;
				}
			} else {
				numBelow++;
				
				if (Integer.parseInt(pos.getY()) > maxYBelow) {
					maxYBelow = Integer.parseInt(pos.getY());
					peakBelow = pos;
					nameBelow = name;
				}
			}
		}
		
		Map<String, Position> relocatedNames = new LinkedHashMap<String, Position>();
		
		List<String> namesRequiringRelocation = new ArrayList<String>();
		for (String name : locations.keySet()) {
			Position pos = locations.get(name);
			if (!requiresNameRelocation(pos))
				continue;
			namesRequiringRelocation.add(name);
			
			relocatedNames.put(name, createRelocatedNamePosition(name, pos, numAbove, numBelow, peakAbove, peakBelow));

		}
//		relocatedNames = relocateNames(namesRequiringRelocation, locations,
//				numAbove, numBelow, peakAbove, peakBelow, nameAbove, nameBelow );
		
		
		return relocatedNames;
	}
	
	public Map<String, Position> relocateNames(List<String> namesRequiringRelocation, Map<String, Position> locations,
			int numAbove, int numBelow, Position peakAbove, Position peakBelow, String nameAbove, String nameBelow) {
		Map<String, Position> relocatedNames = new LinkedHashMap<String, Position>();
		
		List<String> above = new ArrayList<String>();
		List<String> below = new ArrayList<String>();
		
		// need to process above and below separately
		
		// process below
		
		for (String name : namesRequiringRelocation) {
			Position p = locations.get(name);
			if (p.equals(peakBelow))
				continue;
			if (locations.get(name).getTextPos() == TEXTPOS_BELOW) {
				below.add(name);
			}
		}
		
		Collections.sort(below, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				if (arg0.length() < arg1.length())
					return -1;
				else if (arg0.length() > arg1.length())
					return 1;
				else return 0;
			}
		});
		
		System.out.println("BELOW: "+below);
		
		int v = 0;
		int adjustment = 10; // 25;
		
		for (String name : below) {
			Position position = locations.get(name);

			relocatedNames.put(name, new Position(position.getX(), Integer.toString(peakBelow.getYasInt()+adjustment*v), 
					position.getTextPos(), position.getHeightAdjustment()));
			v++;

		}
		
		relocatedNames.put(nameBelow, new Position(peakBelow.getX(), Integer.toString(peakBelow.getYasInt()+adjustment*v), 
				peakBelow.getTextPos(), peakBelow.getHeightAdjustment()));
		v++;
		
		// process above
		
		for (String name : namesRequiringRelocation) {
			Position p = locations.get(name);
			if (p.equals(peakAbove))
				continue;
			if (locations.get(name).getTextPos() == TEXTPOS_ABOVE) {
				above.add(name);
			}
		}
		
		Collections.sort(above, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				if (arg0.length() < arg1.length())
					return -1;
				else if (arg0.length() > arg1.length())
					return 1;
				else return 0;
			}
		});
		
		System.out.println("ABOVE: "+above);
		
		v = 0;
		
		for (String name : above) {
			Position position = locations.get(name);

			relocatedNames.put(name, new Position(position.getX(), Integer.toString(peakAbove.getYasInt()-adjustment*v), 
					position.getTextPos(), position.getHeightAdjustment()));
			v++;

		}
		
		relocatedNames.put(nameAbove, new Position(peakAbove.getX(), Integer.toString(peakAbove.getYasInt()-adjustment*v), 
				peakAbove.getTextPos(), peakAbove.getHeightAdjustment()));
		v++;
		
		
		return relocatedNames;
	}
	
	public Position createRelocatedNamePosition(String name, Position position, int numAbove, int numBelow,
			Position peakAbove, Position peakBelow) {
		
		int adjustment = 0;
		int slotHeight = 1; // 50
		int fixed = 12; // 5;
		
		int index = position.getHeightAdjustment();
		int distanceToPeakAdjustment = 0;
		int peakIndex = 0;
		
		
		if (position.getTextPos() == TEXTPOS_BELOW) {
			
			// bottom of circle - generated clockwise -> right to left
			peakIndex = Math.abs(peakBelow.getHeightAdjustment());
			index = Math.abs(index);
			if (peakIndex > index) {
				distanceToPeakAdjustment = peakIndex - (peakIndex - index);
			} else if (peakIndex < index) {
				distanceToPeakAdjustment = peakIndex - (index - peakIndex);
			} else {
				distanceToPeakAdjustment = 0;
			}
			
//			index = numBelow - Math.abs(index) + 1;
			
			if (position.getXasInt() < peakBelow.getXasInt()) {
				// left of peak
				adjustment = fixed + (slotHeight * distanceToPeakAdjustment);
			} else if (position.getXasInt() > peakBelow.getXasInt()) {
				// right of peak
//				adjustment = fixed+ (slotHeight/2 + (index-peakIndex-1)*slotHeight);
				adjustment = fixed+ (slotHeight * distanceToPeakAdjustment);
			} else if (position.getXasInt() == peakBelow.getXasInt()) {
				// processing peak
				adjustment = fixed + (slotHeight * peakIndex);
			}
//			adjustment += 1;
		} else {
			// above clockwise -> left to right index accurate
			peakIndex = Math.abs(peakAbove.getHeightAdjustment());
			index = Math.abs(index);
			
			if (peakIndex > index) {
				distanceToPeakAdjustment = peakIndex - (peakIndex - index);
			} else if (peakIndex < index) {
				distanceToPeakAdjustment = peakIndex - (index - peakIndex);
			} else {
				distanceToPeakAdjustment = 0;
			}
			
			if (position.getXasInt() < peakAbove.getXasInt()) {
				// left of peak
				adjustment = -1 * (fixed + (slotHeight * distanceToPeakAdjustment));
			} else if (position.getXasInt() > peakAbove.getXasInt()) {
				// right of peak
//				adjustment = -1 * (fixed + (slotHeight/2 + (index-peakIndex-1)*slotHeight));
				adjustment = -1 * (fixed + (slotHeight * distanceToPeakAdjustment));
			} else if (position.getXasInt() == peakAbove.getXasInt()) {
				// processing peak
				adjustment = -1 * (fixed + (slotHeight * (peakIndex)));
			}
//			adjustment -= 1;
		}
		System.out.println("adj: name"+name+" orig y "+position.getY()+" adjust "+adjustment+" index: "+index+" peakIndex = "+peakIndex+" distAdj = "+distanceToPeakAdjustment);
		
		
		
		return new Position(position.getX(), Integer.toString(Integer.parseInt(position.getY())+adjustment), 
				position.getTextPos(), position.getHeightAdjustment());
		
	}
	
	public int heightAdjustment(Position pos) {
		if (pos.getHeightAdjustment() == 0)
			return 0;
		
		int numRequiringAdjustment;
		int thisOne;
		int heightAdjustment = pos.getHeightAdjustment();
		
		int adjust = 0;
		
		if (heightAdjustment < 0) {
			// this is the below ones
			numRequiringAdjustment = Math.abs(belowCount);
			thisOne = Math.abs(heightAdjustment);
		} else {
			// the above ones
			numRequiringAdjustment = aboveCount;
			thisOne = heightAdjustment;
		}
		
		double t1 = -1.0;
		double t2 = -1.0;
		
		// determine peak
		double peak = numRequiringAdjustment/2 + 1;
		if (thisOne <= peak) {
			adjust = thisOne * ADJUST_SIZE;;
			System.out.println(pos+" peak = "+peak+" thisOne = "+thisOne+" adjust: "+adjust);
		} else {
			t1 = (peak - (thisOne - peak));
			t2 = ADJUST_SIZE - new GraphicsHelper().getHeight()/2;
			adjust = (int) (t1 * t2);
			System.out.println(pos+" peak = "+peak+" thisOne = "+thisOne+" t1,t2 "+t1+","+t2+" adjust: "+adjust);
		}
		
		// 1 2 3 4 5 -> 1 2 3 2 1
		// 1 2 3 4 5 6 -> 1 2 3 4 3 2
		
		System.out.println(pos+" t1,t2 "+t1+","+t2+"  adjust: "+adjust);
		
		return adjust;
	}
	
	public Map<String, Position> shift(Map<String, Position> locations) {

		double xShift = 0.0;
		double yShift = 0.0;
		
		Map<String, Position> newLocations = new HashMap<String, Position>();

		double maxY = Double.NEGATIVE_INFINITY;
		for (String loc : locations.keySet()) {

			Position p1 = locations.get(loc);
			double x1 = Double.parseDouble(p1.getX());
			if (x1 < 0.0) {
				xShift = Math.max(Math.abs(x1), xShift);
			}
			double y1 = Double.parseDouble(p1.getY());
			if (y1 < 0.0) {
				yShift = Math.max(Math.abs(y1), yShift);
			}
		}

		for (String loc : locations.keySet()) {
			
			Position p1 = locations.get(loc);
			Position shifted = applyShift(p1, xShift, yShift);
			newLocations.put(loc, shifted);

		}
		
		return newLocations;
	}
	
	public Position applyShift(Position p, double xShift, double yShift) {
		int x1 = (int) (Double.parseDouble(p.getX()) + xShift);
		int y1 = (int) (Double.parseDouble(p.getY()) + yShift);
		
		return new Position(Integer.toString(x1), Integer.toString(y1),p.getTextPos(),p.getHeightAdjustment());
	}
	
	public String midPoint(Map<String, Position> locations, String n1, String n2) {
		
		Position p1 = locations.get(n1);
		Position p2 = locations.get(n2);
		double x1 = Double.parseDouble(p1.getX());
		double x2 = Double.parseDouble(p2.getX());
		
		double y1 = Double.parseDouble(p1.getY());
		double y2 = Double.parseDouble(p2.getY());
		
		double x = (x1+x2)/2.0;
		double y = (y1+y2)/2.0;
		if (Math.abs(x) < .0001)
			x = 0.0;
		if (Math.abs(y) < .0001)
			y = 0.0;
		
		return "("+(int) x+
		","+ (int) y+
		")";
	}
	
	// 10,1,Stock,662,381,40,20,3,3,0,0,0,0,0,0
	// 1,3,5,1,4,0,0,22,0,0,0,-1--1--1,,1|(578,381)|
	
	public void getVariableCirclePositions(int numberOfPositions, int radius) {
		/*
		 * for i = 1 to n
		 *  a = 360 / n * i
		 *	x = math.cos(math.radians(a)) * r
		 *	y = math.sin(math.radians(a)) * r
		 *
		 */
		
		
		
		locsCircle = new ArrayList<Position>();
		
		int textpos = 0;
		double transitionDegrees = circleThreshhold; // 10.0;
		
		aboveCount = 0;
		belowCount = 0;
		
		int numGroups = groups.size();
		int emptyPositions = intergroupGap * numGroups;
		double angle = 360.0 / (numberOfPositions+emptyPositions);
		
		for (int varNum = 1; varNum <= numberOfPositions+emptyPositions; varNum++) {
			
			int heightAdjustment = 0;

			double ang = angle * (varNum-1);
			int x = (int) (Math.cos(Math.toRadians(ang)) * radius);
			int y = (int) (Math.sin(Math.toRadians(ang)) * radius);
			
			if (ang >= 0.0 && ang <= 90 - transitionDegrees) {
				textpos = TEXTPOS_RIGHT;
			} else if (ang > 90 - transitionDegrees && ang <= 90 + transitionDegrees) {
				textpos= TEXTPOS_BELOW; // ABOVE
				aboveCount++;
				heightAdjustment = aboveCount;
			} else if (ang > 90 +  transitionDegrees && ang <= 270 - transitionDegrees) {
				textpos= TEXTPOS_LEFT;
			} else if (ang > 270 - transitionDegrees && ang <= 270 + transitionDegrees) {
				textpos= TEXTPOS_ABOVE; // BELOW
				--belowCount;
				heightAdjustment = belowCount;
			} else {
				textpos= TEXTPOS_RIGHT;
			}

//			System.out.println(varNum+","+x+","+y+" angle: "+ang+" textpos "+textpos+","+heightAdjustment);
			locsCircle.add(new Position(Integer.toString(x),Integer.toString(y), textpos, heightAdjustment));
		}
		
		orderedLocs.addAll(locsCircle);
		
		return;
	}
	
	public void getVariableRectanglePositions(int numberOfPositions, int width) {
		
		locsLeft = new ArrayList<Position>();
		locsRight = new ArrayList<Position>();
		locsAbove = new ArrayList<Position>();
		locsBelow = new ArrayList<Position>();
		
		GraphicsHelper gh = new GraphicsHelper();
		
// determine how many variables will be on each side
		
		int numSides = (int) (numberOfPositions * .72);
		if (numSides % 2 != 0) {
			numSides += 1;
		}
		
		int numLeft = numSides/2;
		int numRight = numLeft;
		
		int numRemaining = numberOfPositions - numSides;
		int numTop = numRemaining/2;
		int numBottom = numRemaining - numTop;
		
		// now determine spacing 
		
		int yGap = 25;
		
		int heightPer = gh.getHeight();
		int widthPer = 2*heightPer;
		int totalHeight = (heightPer + yGap) * numLeft;
		
		int xIndent =  2*widthPer;
		int remainingWidth = width - 2*xIndent;
		
		int widthPerTop = remainingWidth / (numTop-1);
		int widthPerBottom = remainingWidth / (numBottom-1);
		
		int xGapTop = (width - widthPerTop * numTop)/(numTop+1);		// 5;
		int xGapBottom = (width - widthPerBottom * numBottom)/(numBottom+1);		// 5;
		
		
		
		int xStart = 0;
		int yStart = yGap;
		
		int x = xStart;
		int y = yStart+heightPer;
		
		// top to bottom
		
		for (int i = 0; i < numLeft; i++) {
			locsLeft.add(new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_LEFT, 0));
			y += yGap + heightPer;
		}
		
		// bottom left to right
		// y value already set from above
		// here we want a fixed indent for all four corners
		x += 2*widthPer; // xGapBottom + widthPerBottom;
		
		for (int i = 0; i < numBottom; i++) {
			locsBelow.add(new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_BELOW, 0));
			x += widthPerBottom;
		}
		
		// bottom up
		
//		x -= xGapTop + widthPerTop;
		x = x - widthPerBottom + xIndent;
		y -= yGap +heightPer;
		
//		y -= yGap;
		for (int i = 0; i < numRight; i++) {
			locsRight.add(new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_RIGHT, 0));
			y -= yGap + heightPer;
		}
		
		// top right to left
		x -= xIndent; // xGapBottom + widthPerBottom;
//		y -= yGap; //  + heightPer;
		for (int i = 0; i < numRight; i++) {
			locsAbove.add(new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_ABOVE, 0));
			x -= widthPerTop;
		}
		
		orderedLocs.addAll(locsLeft);
		orderedLocs.addAll(locsBelow);
		orderedLocs.addAll(locsRight);
		orderedLocs.addAll(locsAbove);
		
		return;
	}
	
	public void getVariableArcPositions(int numberOfPositions, int radius) {
		
		locsLeft = new ArrayList<Position>();
		locsRight = new ArrayList<Position>();
		
		// start at -75 -> 0 -> 75
		// 105 -> 255
		
		// arc len = 150
		// have 2 arcs
		
		double arcDegrees = arcsArcDegrees; // 120; // 150
		
		double offset = (180.0 - arcDegrees)/2;
		
		double startLeft = 90.0 + offset;
		double startRight = 90.0 - offset;
		
		numPerArcLeft = numberOfPositions / 2;
		double degreesLeft = arcDegrees / numPerArcLeft;
		
		
		numPerArcRight = numberOfPositions - numPerArcLeft;
		double degreesRight = arcDegrees / numPerArcRight;
		
		double ang = startLeft;
		
		for (int i = 0; i < numPerArcLeft; i++) {
			int x = (int) (Math.cos(Math.toRadians(ang)) * radius);
			int y = (int) (Math.sin(Math.toRadians(ang)) * radius);

			System.out.println(x+","+y+" angle: "+ang);
			
			locsLeft.add( new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_LEFT, 0));
			ang += degreesLeft;
		}
		
		ang = startRight;
		for (int i = 0; i < numPerArcRight; i++) {
			int x = (int) (Math.cos(Math.toRadians(ang)) * radius);
			int y = (int) (Math.sin(Math.toRadians(ang)) * radius);

			System.out.println(x+","+y+" angle: "+ang);
			
			locsRight.add( new Position(Integer.toString(x),Integer.toString(y), TEXTPOS_RIGHT, 0));
			ang -= degreesRight;
		}
		
		orderedLocs.addAll(locsLeft);
		orderedLocs.addAll(locsRight);
		
		return;
		
	}
	
	List<String> splitWords(String text, int wid){
		List<String> wordStrings=new ArrayList<String>();
		if(text==null)
			return wordStrings;
		String[] words = text.split(" ");
//		String[] words = new String[1];
//		words[0] = text;
		String line = "";

		if(words.length==0)
			return wordStrings;

		boolean complete=true;
		FontMetrics metrics= new GraphicsHelper().getFm();
		
		String testLine=words[0];
		line=testLine;
		int maxWidth=wid*2;
		for(int n = 1; n < words.length; n++) {
			testLine = line+" "+words[n];

			  int testWidth=metrics.stringWidth(testLine);

			if (testWidth > maxWidth) {
				// have to stop it before adding it
				wordStrings.add(line);
				line = words[n];
			}
			else {
				line = testLine;
			}

		}
		wordStrings.add(line);

		return wordStrings;
	}
	


}
