package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.bridge.SDMBridge;
import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.io.File;

import org.apache.log4j.PropertyConfigurator;

public class BatchGenRun {
	/**
	 * Run a specific model as many times as needed -- see main() 
	 */
	
	private FileOrganizer fileOrganizer;
	private UserSelectedOptions options;
	private File mdlFile;
	private String mdl;
	private String runName;
	
	private SDMBridge bridge;
	
	
	public BatchGenRun(String mdl, String scenario) {
		
		/*
		 * Initialize the system based on the specified mdl file
		 */
		
		SDMDocLanguageSpecific.initialize();
		this.mdl = mdl;
		mdlFile = new File(mdl);
		options = new UserSelectedOptions(true);
		this.runName = scenario;
		
		// make sure these options are turned off - loop counts don't matter and time stamp
		// will be based on current time and not the the original SDM-Doc was executed
		options.setValue(StringConstants.INCLUDE_LOOP_SIZE, "boolean", UserSelectedOptions.FALSE);
		options.setValue(StringConstants.INCLUDE_TIME_STAMP, "boolean", UserSelectedOptions.FALSE);
		
		// this initializes all the various files that can be referenced during the execution
		fileOrganizer = new FileOrganizer(false, mdlFile.getParent(), mdl, options);
	}
	
	public boolean genRun() {
		boolean res = runModel(runName);

		if (!res) {
			System.out.println("No Successful Run. Try generate and run");
			if (genCode()) {
				res = runModel(runName);
				if (res) {
					System.out.println("Gencode() Success using new classes");
				} else {
					System.out.println("Gencode() success but run fail");
				}

				return res;
			} else {
				System.out.println("Gencode() failure");
				return false;
			}
		} else {
			System.out.println("Success using existing classes");
			return true;
		}
	}
	
	public double getValue(String var) {
		System.out.println("Result Variable: "+bridge.getCurrentValue(var));
		return bridge.getCurrentValue(var);
	}
	
	private boolean genCode() {
		
		try {
			String obj = Utilities.makeLegalJavaName((mdlFile).getName().replace(".MDL", "").replace(".mdl", "").toUpperCase());
			
			String root = mdlFile.getParent();

			final MDLProcessor sdmDoc = new MDLProcessor(mdl, mdl, obj, root, new MessageGobbler(), 
					new MessageGobbler(), options);
			sdmDoc.setGenDoc(true);
			sdmDoc.setBatchExecution(true);
			sdmDoc.process(false);
			return true;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		
	}
	
	public boolean runModel(String runName) {
		// create the model/data bridge
		bridge = new SDMBridge(fileOrganizer);
		
		// specify the one to run
		bridge.selectModel(fileOrganizer.getModelname());
		
		// can we instantiate?
		if (bridge.createNewRun(runName)) {
			
			// initialize model and set initial values based on mdl file initial values
			bridge.startExecution();
			
			// run it to the end as specified in mdl file model settings
			bridge.runTo(bridge.getEndTime());
			
			// these are the recognized time steps
//			System.out.println(bridge.getTimeVector());
			
			// close the database and terminate the bridge
			bridge.done();
			return true;
		} else {
			return false;
		}
	}
	
    public static void main(String[] args) {
    	
    	String log4jConfPath = "./data/log4j.properties";
		PropertyConfigurator.configure(log4jConfPath);
		StaticSwitches.initialize();
    	
    	String mdlFile = args[0];
    	String runName = "baseline";
    	if (args.length > 1)
    		runName = args[1];
    	
    	BatchGenRun runner = new BatchGenRun(mdlFile, runName);
    	runner.genRun();
    }

}
