package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Map;

public class BatchRunner {
	
	private String filesToProcessFile;
	private String optionsFile;
	
	private Map<String, String> optionValues = new LinkedHashMap<String, String>();
	
	public BatchRunner(String filesToProcessFile, String optionsFile) {
		this.filesToProcessFile = filesToProcessFile;
		this.optionsFile = optionsFile;
		
		SDMDocLanguageSpecific.initialize();
	
		processOptions();
	}
	
	private void processOptions() {
		
		for (String line : Utilities.getAsList(optionsFile)) {
			String[] fields = line.split("=");
			optionValues.put(fields[0].trim(), fields[1].trim());
		}
		
	}
	
	private void execute() {

		for (String line : Utilities.getAsList(filesToProcessFile)) {
			String mdl = line.split("\t")[0];
			String object = line.split("\t")[1].toUpperCase();
			
			
			try {
				MDLProcessor mdlProc = new MDLProcessor(mdl, mdl, object, optionValues.get("SystemRoot"));
				mdlProc.getSdmProc().setBatchExecution(true);
				mdlProc.process(false);
			} catch (Exception e) {
				writeErrorToFile(mdl, e);
				// TODO Auto-generated catch block
				System.out.println("ERROR! "+mdl);
				e.printStackTrace();
			}
		}
	}
	
	public void writeErrorToFile(String mdl, Exception e) {
		try {
			FileWriter fw = new FileWriter("batchErrors.txt",true); //the true will append the new data
		    fw.write("\n*** ERROR! "+mdl+"\n");
			for (StackTraceElement ste : e.getStackTrace()) {
				fw.write(ste.toString()+"\n");
			}
			fw.close();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
	}
	
    public static void main(String[] args) {
    	
    	BatchRunner runner = new BatchRunner(args[0], args[1]);
    	runner.execute();
    }

}
