package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelManager;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.Logger;

public class FileOrganizer {
	
	private static final String SRC_CLASS_DIRECTORY = "srcgen";
	private final static String PACKAGE = "gov.anl.gss.sdm.server.simulation.models";
	
	static Logger log = Logger.getLogger(FileOrganizer.class.getSimpleName());

	private String rootDirectory;
	private String modelDirectory;
	private String modelsDirectory;
	private String dynamicDirectory;
	private String docGraphDirectory;
	private String docGraphDirectoryForZip;
	
	private String micmacDirectory;
	private String loopsDirectory;
	private String thumbnailsDirectory;
	private String viewsDirectory;
	private String resultsDirectory;
	
	private String relativeLoopsDirectory;
	private String relativeViewsDirectory;
	private String relativeThumbnailsDirectory;


	private String relativeDocGraphDirectory;
	private String htmlDocGraphDirectory;
	private String srcGenDirectory;
	private String runsDirectory;
	private String flatFileTablesDirectory;

	private String structureDirectory;
	private String vignettesDirectory;
	private String sdmdocDirectory;
	private String mdlFile;
	private String modelname;
	private String taggedModelname;
	
	private String miscDirectory;
	private String outputDirectory;
	
	private String applicationDirectory;
	
	private String htmlFile;
	private String zipFile;
	
	String ls = "";
	String ts = "";
	
	private UserSelectedOptions options;

	private boolean sdmui;
	
	public FileOrganizer(boolean sdmui, String rootDirectory, String mdlFile, 
			UserSelectedOptions options) {
		
		setApplicationDirectory();
		
		this.options = options;
		SimpleDateFormat timeStamp = new SimpleDateFormat("yyyy.MM.dd. HH.mm.ss");
		
		if (options != null) {
			boolean includeLoopSize = options.isSelected(StringConstants.INCLUDE_LOOP_SIZE) && 
					options.isSelected(StringConstants.ANALYZE_LOOPS);
			boolean includeTimeStamp = options.isSelected(StringConstants.INCLUDE_TIME_STAMP);

			if (includeLoopSize) {
				ls = "("+options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM)+")";
			}

			
			if (includeTimeStamp) {
				ts ="("+timeStamp.format(new Date())+")";
			}
		}
		
		
		
		log.debug("FO: <"+sdmui+"><"+rootDirectory+"><"+mdlFile+">");
		this.sdmui = sdmui;
		
		modelname = new File(mdlFile).getName().replace(".mdl", "").replace(".MDL", "").
				replace(".xml", "").replace(".XML", "").replace("&", "_");
		taggedModelname = modelname+ls+ts;

		if (sdmui) {
			deriveSDMUI(rootDirectory.replace("\\", "/"), mdlFile.replace("\\", "/"));
		} else {
			deriveSDMDOC(rootDirectory.replace("\\", "/"), mdlFile.replace("\\", "/"));
		}
		
		InformationManagers.getInstance().setFileOrganizer(this);
	}

	public FileOrganizer(boolean sdmui, String rootDirectory, String mdlFile) {
		setApplicationDirectory();
		log.debug("FO: <"+sdmui+"><"+rootDirectory+"><"+mdlFile+">");
		this.sdmui = sdmui;
		
		modelname = new File(mdlFile).getName().replace(".mdl", "").replace(".MDL", "").
				replace(".xml", "").replace(".XML", "").replace("&", "_");
		
		if (options != null) {
			boolean includeLoopSize = options.isSelected(StringConstants.INCLUDE_LOOP_SIZE) && 
					options.isSelected(StringConstants.ANALYZE_LOOPS);
			boolean includeTimeStamp = options.isSelected(StringConstants.INCLUDE_TIME_STAMP);

			if (includeLoopSize) {
				ls = "("+options.getOptionValue(StringConstants.LOOP_SIZE_MAXIMUM)+")";
			}
			SimpleDateFormat timeStamp = new SimpleDateFormat("yyyy.MM.dd. HH.mm.ss");
			
			if (includeTimeStamp) {
				ts ="("+timeStamp.format(new Date())+")";
			}
		}
		
		taggedModelname = modelname+ls+ts;

		if (sdmui) {
			deriveSDMUI(rootDirectory.replace("\\", "/"), mdlFile.replace("\\", "/"));
		} else {
			deriveSDMDOC(rootDirectory.replace("\\", "/"), mdlFile.replace("\\", "/"));
		}
		
		InformationManagers.getInstance().setFileOrganizer(this);
	}
	
	public void setApplicationDirectory() {
		applicationDirectory = System.getProperty("user.dir");;
	}
	
	public String getRelativeToApplicationDirectory(String file) {
		
		try {
			String relPath = getRelativePath(new File(applicationDirectory), new File(file));
			return relPath;
		} catch (IOException e) {
			
			// if there is no relative path possible then use absolute
			return file;
		}
	}
	
		String getRelativePath(File base, File name) throws IOException  {
		    File parent =base.getParentFile();

		    if (parent == null) {
		        throw new IOException("No common directory");
		    }

		    String bpath = base.getCanonicalPath().replace("\\", "/");
		    String fpath = name.getCanonicalPath().replace("\\", "/");;

		    if (fpath.startsWith(bpath)) {
		    	String v = fpath.substring(bpath.length() + 1);
		        return fpath.substring(bpath.length() + 1);
		    } else {
		    	String v = "../" +getRelativePath(parent, name);
		        return v;
		    }
		
		
	}

	private void deriveSDMUI(String rootDirectory, String mdlFile) {
		
		// this should ne called in an SDMDoc run environment

		ModelManager mm = InformationManagers.getInstance().getModelManager();

		// rootDirectory is "sdmData" directory
		this.rootDirectory = rootDirectory;
		this.modelsDirectory = this.rootDirectory+"/Models";
		this.modelDirectory = this.rootDirectory+"/Models/"+modelname;
		this.dynamicDirectory = this.rootDirectory+"/Dynamic";
		
		this.sdmdocDirectory = modelDirectory;
		this.docGraphDirectory = this.sdmdocDirectory+"/"+modelname+"_DOC_GRAPH_";
		this.docGraphDirectoryForZip = this.sdmdocDirectory+"/"+modelname+"_DOC_GRAPH_";
		
		this.runsDirectory = docGraphDirectory+"/runs";
		
		this.flatFileTablesDirectory = docGraphDirectory+"/FlatFileTables";
		
		this.srcGenDirectory = docGraphDirectory+"/srcgen";
		this.miscDirectory = docGraphDirectory+"/debug";
		this.outputDirectory = docGraphDirectory+"/debug";
		this.structureDirectory = docGraphDirectory+"/structure";
		this.vignettesDirectory = docGraphDirectory+"/vignettes";
		
		this.htmlDocGraphDirectory = modelname+"_DOC_GRAPH_";
		this.relativeDocGraphDirectory = htmlDocGraphDirectory;
		
		this.mdlFile = mdlFile;
		
		this.htmlFile = this.modelDirectory+"/"+modelname+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".html";
		this.zipFile = this.modelDirectory+"/"+modelname+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".zip";


	}
	
	public void updateForModel(String modelname) {
		if (sdmui) {
			this.modelname = modelname;
			taggedModelname = modelname+ls+ts;
			this.modelsDirectory = this.rootDirectory+"/Models";
			this.modelDirectory = this.rootDirectory+"/Models/"+modelname;
			this.dynamicDirectory = this.rootDirectory+"/Dynamic";
			
			this.sdmdocDirectory = modelDirectory;
			this.docGraphDirectory = this.sdmdocDirectory+"/"+modelname+"_DOC_GRAPH_";
			this.docGraphDirectoryForZip = this.sdmdocDirectory+"/"+modelname+"_DOC_GRAPH_";
			
			this.runsDirectory = docGraphDirectory+"/runs";
			this.srcGenDirectory = docGraphDirectory+"/srcgen";
			this.flatFileTablesDirectory = docGraphDirectory+"/FlatFileTables";
			
			this.miscDirectory = docGraphDirectory+"/debug";
			this.outputDirectory = docGraphDirectory+"/debug";
			
			this.structureDirectory = docGraphDirectory+"/structure";
			this.vignettesDirectory = docGraphDirectory+"/vignettes";
			
			this.loopsDirectory = docGraphDirectory+"/loops";
			this.thumbnailsDirectory = docGraphDirectory+"/thumbnails";
			this.viewsDirectory = docGraphDirectory+"/views";
			
			this.htmlDocGraphDirectory = modelname+"_DOC_GRAPH_";
			this.relativeDocGraphDirectory = htmlDocGraphDirectory;
		} else {
			this.modelname = modelname;
			taggedModelname = modelname+ls+ts;
			this.modelDirectory = rootDirectory;
			this.dynamicDirectory = "./Dynamic";
			
			String tts = "";
			String lls = "";
			if (options != null) {
				boolean includeLoopSize = options.isSelected(StringConstants.INCLUDE_LOOP_SIZE) && 
						options.isSelected(StringConstants.ANALYZE_LOOPS);
				boolean includeTimeStamp = options.isSelected(StringConstants.INCLUDE_TIME_STAMP);

				if (includeLoopSize) {
					lls = ls;
				}

				if (includeTimeStamp) {
					tts = ts;
				} 
			}
			
			this.htmlFile = this.modelDirectory+"/"+modelname+"_SDMDoc/" +
					modelname+lls+tts+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".html";
			this.zipFile = this.modelDirectory+"/"+modelname+"_SDMDoc/" +
					modelname+lls+tts+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".zip";
			
			this.docGraphDirectory = this.modelDirectory+"/"+modelname+"_SDMDoc/"+modelname+lls+tts+"_DOC_GRAPH_";
			this.docGraphDirectoryForZip = this.modelDirectory+"/"+modelname+"_SDMDoc";
			this.relativeDocGraphDirectory =  modelname+lls+tts+"_DOC_GRAPH_";
			this.htmlDocGraphDirectory =  modelname+lls+tts+"_DOC_GRAPH_";
			this.runsDirectory = docGraphDirectory+"/runs";
			this.srcGenDirectory = docGraphDirectory+"/srcgen";
			this.structureDirectory = docGraphDirectory+"/structure";
			this.vignettesDirectory = docGraphDirectory+"/vignettes";
			
			this.sdmdocDirectory = null;
			this.modelsDirectory = null;
		}
	}

	private void deriveSDMDOC(String rootDirectory, String mdlFile) {
		
		// root directory is based on mdlFile
		this.rootDirectory = new File(mdlFile).getParent();
				this.modelDirectory = rootDirectory;
		this.dynamicDirectory = "./Dynamic";
		
		
		
		
		String tts = "";
		String lls = "";
		if (options != null) {
			boolean includeLoopSize = options.isSelected(StringConstants.INCLUDE_LOOP_SIZE) && 
					options.isSelected(StringConstants.ANALYZE_LOOPS);
			boolean includeTimeStamp = options.isSelected(StringConstants.INCLUDE_TIME_STAMP);

			if (includeLoopSize) {
				lls = ls;
			}

			
			if (includeTimeStamp) {
				tts = ts;
			} 
		}
		
		this.htmlFile = this.modelDirectory+"/"+modelname+"_SDMDoc/"+
		modelname+lls+tts+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".html";
		this.zipFile = this.modelDirectory+"/"+modelname+"_SDMDoc/"+
		modelname+lls+tts+ "_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".zip";
		
		this.docGraphDirectory = this.modelDirectory+"/"+modelname+"_SDMDoc/"+modelname+lls+tts+"_DOC_GRAPH_";
		this.docGraphDirectoryForZip = this.modelDirectory+"/"+modelname+"_SDMDoc";
		this.relativeDocGraphDirectory =  modelname+lls+tts+"_DOC_GRAPH_";
		
		this.relativeLoopsDirectory =  modelname+lls+tts+"_DOC_GRAPH_/loops";
		this.relativeViewsDirectory =  modelname+lls+tts+"_DOC_GRAPH_/views";
		this.relativeThumbnailsDirectory =  modelname+lls+tts+"_DOC_GRAPH_/thumbnails";
		
		
		this.htmlDocGraphDirectory = modelname+lls+tts+"_DOC_GRAPH_";
		this.runsDirectory = docGraphDirectory+"/runs";
		this.flatFileTablesDirectory = docGraphDirectory+"/FlatFileTables";
		this.srcGenDirectory = docGraphDirectory+"/srcgen";
		this.miscDirectory = docGraphDirectory+"/debug";
		this.outputDirectory = docGraphDirectory+"/debug";
		this.structureDirectory = docGraphDirectory+"/structure";
		this.vignettesDirectory = docGraphDirectory+"/vignettes";
		this.micmacDirectory = docGraphDirectory+"/MICMAC";
		
		this.loopsDirectory = docGraphDirectory+"/loops";
		this.thumbnailsDirectory = docGraphDirectory+"/thumbnails";
		this.viewsDirectory = docGraphDirectory+"/views";
		this.resultsDirectory = docGraphDirectory+"/results";
		
		this.mdlFile = mdlFile;
		this.sdmdocDirectory = null;
		this.modelsDirectory = null;
		
	}
	


	public void print() {
		System.out.println("FO: "+sdmui);
		System.out.println(rootDirectory);
		System.out.println(modelDirectory);
		System.out.println(modelsDirectory);
		System.out.println(dynamicDirectory);
		System.out.println(docGraphDirectory);
		System.out.println(htmlDocGraphDirectory);
		System.out.println(runsDirectory);
		System.out.println(flatFileTablesDirectory);
		System.out.println(srcGenDirectory);
		System.out.println(structureDirectory);
		System.out.println(vignettesDirectory);
		
		System.out.println(loopsDirectory);
		System.out.println(thumbnailsDirectory);
		System.out.println(viewsDirectory);
		System.out.println(resultsDirectory);
		
		System.out.println(sdmdocDirectory);
		System.out.println(mdlFile);
		System.out.println(modelname);
		
		log.debug("FO: "+sdmui);
		log.debug(rootDirectory);
		log.debug(modelDirectory);
		log.debug(modelsDirectory);
		log.debug(dynamicDirectory);
		log.debug(docGraphDirectory);
		log.debug(htmlDocGraphDirectory);
		log.debug(runsDirectory);
		log.debug(srcGenDirectory);
		log.debug(structureDirectory);
		log.debug(vignettesDirectory);
		log.debug(sdmdocDirectory);
		log.debug(mdlFile);
		log.debug(modelname);
	}
	
	public String getMapperFile() {
//			String mapperFile = mdlFile.replace(".mdl", "X#X#X").replace(".MDL", "X#X#X").replace("\\", "/").replace("X#X#X", ".map");
//		String mapperFile = this.modelDirectory+"/"+modelname+"_SDMDoc/"+modelname+".map";
		String mapperFile = this.docGraphDirectory+"/"+modelname+".map";
		return mapperFile;
	}

	public String getRootDirectory() {
		return rootDirectory;
	}

	public String getModelDirectory() {
		return modelDirectory;
	}
	
	public String getHtmlFile() {
		return htmlFile;
	}

	public String getDocGraphDirectory() {
		return docGraphDirectory;
	}
	
	public String getRelativeDocGraphDirectory() {
		return relativeDocGraphDirectory;
	}

	public String getRunsDirectory() {
		return runsDirectory;
	}

	public String getStructureDirectory() {
		return structureDirectory;
	}

	public String getVignettesDirectory() {
		return vignettesDirectory;
	}

	public String getMdlFile() {
		return mdlFile;
	}

	public boolean isSdmui() {
		return sdmui;
	}

	public String getDynamicDirectory() {
		return dynamicDirectory;
	}

	public String getModelsDirectory() {
		return modelsDirectory;
	}

	public String getSdmdocDirectory() {
		return sdmdocDirectory;
	}

	public String getHtmlDocGraphDirectory() {
		return htmlDocGraphDirectory;
	}

	public String getModelname() {
		return modelname;
	}

	public String getZipFile() {
		return zipFile;
	}

	public void setZipFile(String zipFile) {
		this.zipFile = zipFile;
	}
	
	protected String asDirectoryPath(String packageName) {
		return packageName.replace(".", "/");
	}
	
	public String getJavaSrcFile() {
		return getDynamicDirectory()+"/"+SRC_CLASS_DIRECTORY+"/"+ asDirectoryPath(PACKAGE)+"/"+modelname+".java";
	}
	
	public String getJavaSrcFileCompile() {
		return getDynamicDirectory()+"/"+SRC_CLASS_DIRECTORY+"/"+ asDirectoryPath(PACKAGE)+"/"+modelname+".compile.txt";
	}
	
	public String getJavaMemorySrcFile() {
		return getDynamicDirectory()+"/"+SRC_CLASS_DIRECTORY+"/"+ asDirectoryPath(PACKAGE)+"/"+modelname+".java";
	}
	
	public String getJavaMemorySrcFileCompile() {
		return getDynamicDirectory()+"/"+SRC_CLASS_DIRECTORY+"/"+ asDirectoryPath(PACKAGE)+"/"+modelname+".compile.txt";
	}
	
	public String getJavaClassFile() {
		String obj = Utilities.makeLegalJavaName(new File(mdlFile).getName().replace(".MDL", "").replace(".mdl", "").toUpperCase());
		return getDynamicDirectory()+"/"+SRC_CLASS_DIRECTORY+"/"+ asDirectoryPath(PACKAGE)+"/"+obj+".class";
	}

	public void setDocGraphDirectory(String docGraphDirectory) {
		this.docGraphDirectory = docGraphDirectory;
		this.runsDirectory = docGraphDirectory+"/runs";
		this.srcGenDirectory = docGraphDirectory+"/srcgen";
		this.miscDirectory = docGraphDirectory+"/debug";
		this.outputDirectory = docGraphDirectory+"/debug";
		this.structureDirectory = docGraphDirectory+"/structure";
		this.vignettesDirectory = docGraphDirectory+"/vignettes";
		this.micmacDirectory = docGraphDirectory+"/MICMAC";
		this.loopsDirectory = docGraphDirectory+"/loops";
		this.thumbnailsDirectory = docGraphDirectory+"/thumbnails";
		this.viewsDirectory = docGraphDirectory+"/views";
		this.resultsDirectory = docGraphDirectory+"/results";
	}

	public String getMiscDirectory() {
		return miscDirectory;
	}
	
	public String getDebugDirectory() {
		return miscDirectory;
	}

	public String getOutputDirectory() {
		return outputDirectory;
	}

	public String getDocGraphDirectoryForZip() {
		return docGraphDirectoryForZip;
	}

	public void setDocGraphDirectoryForZip(String docGraphDirectoryForZip) {
		this.docGraphDirectoryForZip = docGraphDirectoryForZip;
	}

	public String getSrcGenDirectory() {
		return srcGenDirectory;
	}

	public void setSrcGenDirectory(String srcGenDirectory) {
		this.srcGenDirectory = srcGenDirectory;
	}

	public static String getSrcClassDirectory() {
		return SRC_CLASS_DIRECTORY;
	}

	public String getMicmacDirectory() {
		return micmacDirectory;
	}
	
	public String getMicmacExcelFile() {
		return getMicmacDirectory()+"/"+modelname+"_micmac.xlsx";
	}
	
	public String getMicmacPlotFile() {
		return getMicmacDirectory()+"/"+modelname+"_micmac.png";
	}
	public String getMicmacTextFile() {
		return getMicmacDirectory()+"/"+modelname+"_micmac.txt";
	}
	
	public String getFlatFileTablesDirectory() {
		return flatFileTablesDirectory;
	}
	
	public String getLoopsDirectory() {
		return loopsDirectory;
	}

	public void setLoopsDirectory(String loopsDirectory) {
		this.loopsDirectory = loopsDirectory;
	}

	public String getThumbnailsDirectory() {
		return thumbnailsDirectory;
	}

	public void setThumbnailsDirectory(String thumbnailsDirectory) {
		this.thumbnailsDirectory = thumbnailsDirectory;
	}

	public String getViewsDirectory() {
		return viewsDirectory;
	}

	public void setViewsDirectory(String viewsDirectory) {
		this.viewsDirectory = viewsDirectory;
	}

	public String getRelativeLoopsDirectory() {
		return relativeLoopsDirectory;
	}

	public void setRelativeLoopsDirectory(String relativeLoopsDirectory) {
		this.relativeLoopsDirectory = relativeLoopsDirectory;
	}

	public String getRelativeViewsDirectory() {
		return relativeViewsDirectory;
	}

	public void setRelativeViewsDirectory(String relativeViewsDirectory) {
		this.relativeViewsDirectory = relativeViewsDirectory;
	}

	public String getRelativeThumbnailsDirectory() {
		return relativeThumbnailsDirectory;
	}

	public void setRelativeThumbnailsDirectory(String relativeThumbnailsDirectory) {
		this.relativeThumbnailsDirectory = relativeThumbnailsDirectory;
	}

	public String getResultsDirectory() {
		return resultsDirectory;
	}

	public void setResultsDirectory(String resultsDirectory) {
		this.resultsDirectory = resultsDirectory;
	}

	public String getTaggedModelname() {
		return taggedModelname;
	}
}
