package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.util.ArrayList;
import java.util.List;

public class MDL {
	
	private final static String EQUIVALENCE_START = "22:";

	List<String> mdlContents;
	
	public MDL(String file) {
		mdlContents = Utilities.toVerbose(Utilities.removeContinuation(Utilities.getAsList(file)));
	}

	public MDL(List<String> mdlContents) {
		this.mdlContents = Utilities.toVerbose(Utilities.removeContinuation(mdlContents));
	}
	

	public List<String> getEquationPart() {
		List<String> al = new ArrayList<String>();
		
		for (String aLine : mdlContents) {
			if (aLine.contains("---///"))
				break;
			al.add(aLine);
		}

		return al;
	}

	public List<String> getGraphicsPart() {
		List<String> al = new ArrayList<String>();
		
		boolean keep = false;
		for (String aLine : mdlContents) {
			if (aLine.contains("---///"))
				keep=true;
			if (keep)
				al.add(aLine);
		}

		return al;
	}

	public List<String> getUnitsPart() {
		List<String> al = new ArrayList<String>();
		for (String aLine : mdlContents) {
			if (aLine.startsWith(EQUIVALENCE_START))
				al.add(aLine);
		}

		return al;
	}

	/**
	 * @return the mdlContents
	 */
	public List<String> getMdlContents() {
		return mdlContents;
	}
	
	public List<String> getViewNames() {
		List<String> names = new ArrayList<String>();
		
		List<String> graphics = getGraphicsPart();
		int i = 0;
		while(i < graphics.size()) {
			if (graphics.get(i).contains("---///")) {
				i += 2;
				String view = graphics.get(i);
				names.add(view.substring(1));
			}
			i++;
		}
		
		
		return names;
	}
	
	public List<String> getView(String name) {
		List<String> lines = new ArrayList<String>();
		
		List<String> graphics = getGraphicsPart();
		MutableInteger pos = getViewPosition(name);
		lines.add(graphics.get(pos.valueAndInc()));
		while (pos.value() < graphics.size()) {
		if (graphics.get(pos.value()).contains("---///") || graphics.get(pos.value()).contains("///---"))
				return lines;
			lines.add(graphics.get(pos.valueAndInc()));
		}
		
		return lines;
	}
	
	public MutableInteger getViewPosition(String name) {
		List<String> graphics = getGraphicsPart();
		int i = 0;
		while(i < graphics.size()) {
			if (graphics.get(i).contains("---///")) {
				int j = i + 2;
				String view = graphics.get(j).substring(1);
				if (view.equals(name)) {
					return new MutableInteger(i);
				}
			}
			i++;
		}
		return new MutableInteger(-1);
	}


}
