package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

public class MDLProcessor {
	
	private final static String FORMAT = "MDL";
	private final static String MEMORY = "Native"; // "Native" ? "HashMap"
	private final static String PACKAGE = "gov.anl.gss.sdm.server.simulation.models";

	private SDModelProcessor sdmProc;
//	boolean newMemoryDefinition = true;  // must be false for HashMap, either for Native -- 
	//     Required if Native && Vector functions
	
	private String mdlFile;
	private String objectName;
	private String systemRoot;
	private FileOrganizer fileOrganizer;
	
//	private InformationManagers im;
	
	private boolean xmileOrigin;
	
	private boolean variableNameTranslationMode = false;
	
	public MDLProcessor(String originalModelFile, String mdlFile, String objectName, String systemRoot, MessageProcessor info, MessageProcessor error,
			UserSelectedOptions options) {
		setXmileOrigin(!originalModelFile.equals(mdlFile), originalModelFile);
		fileOrganizer = new FileOrganizer(false, systemRoot, mdlFile, options);
		InformationManagers.getInstance().getModelManager().setFileOrganizer(fileOrganizer);
		InformationManagers.getInstance().setMessageFrame(info);
		this.mdlFile = mdlFile;
		this.objectName = objectName;
		this.systemRoot = systemRoot;
		sdmProc = new SDModelProcessor(FORMAT, mdlFile, PACKAGE, objectName, MEMORY, systemRoot, info, error, options, /*newMemoryDefinition,*/ fileOrganizer);
		
	}
	
	public MDLProcessor(String originalModelFile, String mdlFile, String objectName, String systemRoot) {
		setXmileOrigin(!originalModelFile.equals(mdlFile), originalModelFile);
		UserSelectedOptions options = new UserSelectedOptions(true);
		fileOrganizer = new FileOrganizer(false, systemRoot, mdlFile, options);  // true -> sdmui
		InformationManagers.getInstance().getModelManager().setFileOrganizer(fileOrganizer);
		sdmProc = new SDModelProcessor(FORMAT, mdlFile, PACKAGE, objectName, MEMORY, systemRoot, 
				new MessageGobbler(), new MessageGobbler(), options, /* newMemoryDefinition,*/ fileOrganizer);
		sdmProc.setVariableNameTranslationMode(this.variableNameTranslationMode);
	}
	
	public void setGenDoc(boolean val) {
		sdmProc.setGenDoc(val);
	}
	
	public void setBatchExecution(boolean val) {
		sdmProc.setBatchExecution(val);
	}
	
	public void processSDA(boolean sdmuiLoad) {
		sdmProc.processSDA(sdmuiLoad);
		InformationManagers.getInstance().getExcelManager().closeAllWorkbooks();
	}
	
	public void process(boolean sdmuiLoad) {
		sdmProc.process(sdmuiLoad);
		InformationManagers.getInstance().getExcelManager().closeAllWorkbooks();
	}
	
	public SDModel getSDModel() {
		return sdmProc.getSdModel();
	}
	

	public String getHTMLFileName() {
		String html = InformationManagers.getInstance().getModelManager().getHtmlFileName();
		return html;
	}

	public boolean isXmileOrigin() {
		return xmileOrigin;
	}

	public void setXmileOrigin(boolean xmileOrigin, String originalFile) {
		this.xmileOrigin = xmileOrigin;
		if (xmileOrigin) {
//			sdmProc.getOptions().configureForXmile();
			InformationManagers.getInstance().getModelManager().setRunModel(false);
			InformationManagers.getInstance().getModelManager().setXmileOrigin(true);
			InformationManagers.getInstance().getModelManager().setXmileFileName(originalFile);
		} else {
			InformationManagers.getInstance().getModelManager().setXmileOrigin(false);
		}
	}

	public boolean isVariableNameTranslationMode() {
		return variableNameTranslationMode;
	}

	public void setVariableNameTranslationMode(boolean variableNameTranslationMode) {
		this.variableNameTranslationMode = variableNameTranslationMode;
		sdmProc.setVariableNameTranslationMode(variableNameTranslationMode);
	}

	public SDModelProcessor getSdmProc() {
		return sdmProc;
	}


}
