package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.bridge.SDMBridge;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.io.File;

public class RunRepeater {
	/**
	 * Run a specific model as many times as needed -- see main() 
	 */
	
	private FileOrganizer fileOrganizer;
	private UserSelectedOptions options;
	private File mdlFile;
	
	public RunRepeater(String mdl) {
		
		/*
		 * Initialize the system based on the specified mdl file
		 */
		
		SDMDocLanguageSpecific.initialize();
		mdlFile = new File(mdl);
		options = new UserSelectedOptions(true);
		
		// make sure these options are turned off - loop counts don't matter and time stamp
		// will be based on current time and not the the original SDM-Doc was executed
		options.setValue(StringConstants.INCLUDE_LOOP_SIZE, "boolean", UserSelectedOptions.FALSE);
		options.setValue(StringConstants.INCLUDE_TIME_STAMP, "boolean", UserSelectedOptions.FALSE);
		
		// this initializes all the various files that can be referenced during the execution
		fileOrganizer = new FileOrganizer(false, mdlFile.getParent(), mdl, options);
	}
	
	public boolean runModel(String runName) {
		// create the model/data bridge
		SDMBridge bridge = new SDMBridge(fileOrganizer);
		
		// specify the one to run
		bridge.selectModel(fileOrganizer.getModelname());
		
		// can we instantiate?
		if (bridge.createNewRun(runName)) {
			
			// initialize model and set initial values based on mdl file initial values
			bridge.startExecution();
			
			// modify any variables the are "levers"
			if (runName.equals("run2")) {
				bridge.setValue("Initial TOC Cash", 1.6e+008);
			} else if (runName.equals("run3")) {
				bridge.setValue("Initial TOC Cash", 1.8e+008);
			}
			
			// run it to the end as specified in mdl file model settings
			bridge.runTo(bridge.getEndTime());
			
			// these are the recognized time steps
			System.out.println(bridge.getTimeVector());
			
			// get values for some variables
			String[] varList = new String[] {"Initial TOC Cash", "TOC Cash in Banks"};
			double[][] v = bridge.getValues(runName, varList);
			for (int var = 0; var < varList.length; var++) {
			for (int i = 0; i < v[var].length; i++) {
				System.out.println(" "+v[var][i]);
			}
			}
			
			// close the database and terminate the bridge
			bridge.done();
			return true;
		} else {
			return false;
		}
	}
	
    public static void main(String[] args) {
    	
    	RunRepeater runner = new RunRepeater(args[0]);
    	runner.runModel("run1");
    	runner.runModel("run2");
    	runner.runModel("run3");
    }

}
