package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.bridge.BridgeModelMeta;
import gov.anl.gss.sdm.server.model.bridge.BridgeModelVariable;
import gov.anl.gss.sdm.server.model.documentor.gui.MessageFrame;
import gov.anl.gss.sdm.server.model.documentor.gui.SDMDocFont;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationGraphicValidation;
import gov.anl.gss.sdm.server.model.translator.EquationProcessor;
import gov.anl.gss.sdm.server.model.translator.GraphicsProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.ModelAnalyzer;
import gov.anl.gss.sdm.server.model.translator.ModelManager;
import gov.anl.gss.sdm.server.model.translator.Node;
import gov.anl.gss.sdm.server.model.translator.Reader;
import gov.anl.gss.sdm.server.model.translator.SDM_Doc;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;
import gov.anl.gss.sdm.server.model.translator.SystemDynamicsObjectManager;
import gov.anl.gss.sdm.server.model.translator.TreeTraversal;
import gov.anl.gss.sdm.server.model.translator.View;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.log4j.PropertyConfigurator;
import org.apache.log4j.RollingFileAppender;

public class SDModel {

	private String modelFile;

	private String timeStep = null;
	private String timeUnit = null;
	private String dt = null;
	private String startTime = null;
	private String endTime = null;
	private String savPer = null;
	private String inputFileType = null;
	
	private MDL mdl;
	
	private boolean fullyDefined = false;

	private boolean variableNameTranslationMode = false;

	private EquationProcessor equationProcessor;
	private Map<String, Equation> equations;

	private GraphicsProcessor graphicsProcessor;
	private Map<String, View> graphics;

	private Reader reader;

	private boolean ok = true;

	static Logger log = Logger.getLogger(SDM_Doc.class.getSimpleName());

	public final static String version = "SDModel 1.0";

	static final String LOG_PROPERTIES_FILE = "log4j.properties";

	public static String target = "UNDEFINED";
	public static Properties PROPERTIES = new Properties();

	protected SystemDynamicsObjectManager sdObjectManager;

	protected ModelAnalyzer analyzer;
	
	public ModelAnalyzer getAnalyzer() {
		return analyzer;
	}

	protected String objectName;
	
	protected String rootDirectory;
	
	protected String modelName;


	public String getModelName() {
		return modelName;
	}

	protected static String DEFAULT_PROPERTIES_FILE = "DefaultConverter.properties";

	public SDModel(String modelFile, String inputModelType, String objectName, String rootDirectory, FileOrganizer fileOrganizer, boolean xmileOrigin,
			String xmileFileName) {
		this.modelFile = modelFile;

		this.inputFileType = inputModelType;
		this.objectName = objectName;
		this.rootDirectory = rootDirectory;
		// execute translator through units consistency
		// no code will be generated

		
		reader = new Reader(modelFile);
		InformationManagers.clear();
		InformationManagers.getInstance().getEquationManager().clear();
		Utilities.resetNumberedFile();
//		InformationManagers.getInstance().setSdmuiMode(true);
		InformationManagers.getInstance().getModelManager().setXmileOrigin(xmileOrigin);
		InformationManagers.getInstance().getModelManager().setXmileFileName(xmileFileName);
		InformationManagers.getInstance().setSdmuiRootDirectory(rootDirectory);
		InformationManagers.getInstance().setObjectName(objectName);
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		mm.setModelName(reader.getModelname());
		mm.setFileName(reader.getFilename());
		InformationManagers.getInstance().setFileOrganizer(fileOrganizer);
		
		modelName = reader.getModelname();
//		initializeLogger(mm);
		

		this.loadProperties();
		
		InformationManagers.getInstance().getFunctionManager().load("./data/implementedFunctions.txt");
	}
	
	public boolean loadAndProcess() {
		if (inputFileType.equals("MDL"))
			return loadAndProcessMDL();
		else
			return false;
	}
	
	public boolean loadAndProcessMDL() {
				
		mdl = new MDL(reader.readMDLFile());

		sdObjectManager = new SystemDynamicsObjectManager();
		
		List<String> mdlContents = mdl.getMdlContents();

		analyzer = new ModelAnalyzer(mdlContents);
		if (analyzer.fullyDefined()) {
			fullyDefined = true;
			return fullyDefined(mdlContents);
		} else if (analyzer.fullyConnected()) {
			fullyDefined = false;
			InformationManagers.getInstance().getModelManager().setMap(true);
			return fullyConnected(mdlContents);
		} else {
			// do this for micmac processing 
			InformationManagers.getInstance().getModelManager().setMap(analyzer.isHasMap());
			fullyDefined = false;
			return fullyConnected(mdlContents);
		}
	}
	
	protected boolean fullyDefined(List<String> mdlContents) {
		if (mdlContents == null) {
			ok = false;
			return false;
		}

		// process graphics first since we get screen name information from this portion of the file
		graphicsProcessor = new GraphicsProcessor();
		graphicsProcessor.setVariableNameTranslationMode(variableNameTranslationMode);
		graphics = graphicsProcessor.processGraphics(sdObjectManager, mdlContents);

		equationProcessor = new EquationProcessor();
		equations = equationProcessor.processRawEquations(sdObjectManager, /* analyzer.getVensimEquations()*/ mdlContents);
		InformationManagers.getInstance().getEquationLookup().setEquations(equations);
		InformationManagers.getInstance().getArrayManager().populateArraySubscriptSpace();


		//		Map<String, Equation> fatalErrors = equationProcessor
		//				.getFatalErrors(equations);
		//		errors = fatalErrors.size() > 0;
		//		List<String> messages = new ArrayList<String>();
		//		if (errors) {
		//			printErrors("Fatal", fatalErrors);
		//			generateErrorReport("Fatal Errors", fatalErrors, messages);
		////			return false;
		//		}

		// need to tell equations in which views they are referenced
		for (String v : graphics.keySet()) {
			View view = graphics.get(v);
			for (String var : view.getVariablesInView()) {

				for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(var)) {
					eqn.addView(v);
				}

				//				if (equations.containsKey(var)) {
				//					equations.get(var).addView(v);
				//				} else {
				//					Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(var);
				//					if (eqn == null)
				//						log.debug("No var in equations "+var);
				//					else
				//						eqn.addView(v);
				//				}

				String lookvar = var + StringConstants.RSSD_LOOKUP;
				if (equations.containsKey(lookvar)) {
					equations.get(lookvar).addView(v);
				} else {
					Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(lookvar);
					if (eqn != null)
						eqn.addView(v);
				}

			}
		}

		equationProcessor.checkUsage(equations);

		boolean errors = false;

		Map<String, Equation> syntaxErrors = equationProcessor.getSyntaxErrors(equations);
		errors = syntaxErrors.size() > 0;
		if (errors) {
			printErrors("Syntax", syntaxErrors);
			//			ok = false;   // MJB 5/20/2016
		}

		Map<String, List<String>> syntaxErrorMessages = equationProcessor.getSyntaxErrorMessages(equations);
		boolean haveMessages = syntaxErrorMessages.size() > 0;
		if (haveMessages) {
			for (String key : syntaxErrorMessages.keySet()) {
				System.out.println("### Syntax Messages: "+key+" ###");
				for (String m : syntaxErrorMessages.get(key)) {
					System.out.println(m);
				}
			}
		}

		Map<String, Equation> usageErrors = equationProcessor.getUsageErrors(equations);
		errors = usageErrors.size() > 0;
		if (errors) {
			printErrors("Usage", usageErrors);
			//			ok = false;  // MJB 5/20/2016
		}

		Map<String, Equation> unitsErrors = InformationManagers.getInstance().getUnitsManager().
				performUnitsConsistencyCheck(equations, InformationManagers.getInstance().
						getFileOrganizer().getMiscDirectory()+
						"/unitsConsistencyCheckResultsFile.txt");

		errors = unitsErrors.size() > 0;
//		if (errors) {
//			printErrors("Units", unitsErrors);
//			//			ok = false;
//		}		

		List<EquationGraphicValidation> addedScreenNames = sdObjectManager.validate(equations);

		sdObjectManager.createSystemDynamicsObjectForNonGraphic(addedScreenNames, equations);

		return true;
	}
	
	protected boolean fullyConnected(List<String> mdlContents) {
		if (mdlContents == null) {
			ok = false;
			return false;
		}

		// process graphics first since we get screen name information from this portion of the file
		graphicsProcessor = new GraphicsProcessor();
		graphicsProcessor.setVariableNameTranslationMode(variableNameTranslationMode);
		graphics = graphicsProcessor.processGraphics(sdObjectManager, mdlContents);
		equationProcessor = new EquationProcessor();
		equations = equationProcessor.processRawEquations(sdObjectManager, mdlContents);
		InformationManagers.getInstance().getEquationLookup().setEquations(equations);
		
//		equationProcessor.checkUsage(equations);
		
		// need to tell equations in which views they are referenced
		for (String v : graphics.keySet()) {
			View view = graphics.get(v);
			for (String var : view.getVariablesInView()) {
				if (equations.containsKey(var)) {
					equations.get(var).addView(v);
				} else {
					Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(var);
					if (eqn == null) {
//						ok = false;
						eqn = InformationManagers.getInstance().getEquationLookup().getEquation(var);
//						log.debug("No var in equations "+var);
						System.out.println("No var in equations "+var);
					} else {
						eqn.addView(v);
					}
				}
				String lookvar = var + StringConstants.RSSD_LOOKUP;
				if (equations.containsKey(lookvar)) {
					equations.get(lookvar).addView(v);
				} else {
					Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(lookvar);
					if (eqn != null)
						eqn.addView(v);
				}
			}
		}
		
		List<EquationGraphicValidation> addedScreenNames = sdObjectManager.validate(equations);
		
		sdObjectManager.createSystemDynamicsObjectForNonGraphic(addedScreenNames, equations);
		return true;
	}
	
	public void printErrors(String tag, Map<String, Equation> errors) {
		
		Equation eqn = null;
		int count = 0;
		int maxLength = 0;
		
		Iterator<String> iter = errors.keySet().iterator();
		while (iter.hasNext()) {
			String lhs = iter.next();
			eqn = errors.get(lhs);
			maxLength = Math.max(eqn.getVensimEquation().length(), maxLength);
			count++;
		}
		
		if (count == 0)
			return;
		
		MessageFrame errorsFrame = new MessageFrame("Errors",20, 20, maxLength*8, count*100);
		errorsFrame.setFontSize(SDMDocFont.SMALL);
//		for (String msg : eqn.getSemanticMessages()) {
//			System.out.println("Sem: "+msg);
//			errorsFrame.addMessage(msg);
//		}
		
		iter = errors.keySet().iterator();
		while (iter.hasNext()) {
			String lhs = iter.next();
			eqn = errors.get(lhs);
			System.out.println(tag+" ERROR: "+eqn.getVensimEquationOnly());
			errorsFrame.addMessage(" ERROR: "+eqn.getVensimEquationOnly());
//			System.out.println("Tokens: "+eqn.getTokensOneLine());
			for (String msg : eqn.getSyntaxMessages()) {
				System.out.println("Syn: "+msg);
			}
			for (String msg : eqn.getSemanticMessages()) {
				System.out.println("Sem: "+msg);
				errorsFrame.addMessage(msg);
			}
			for (String msg : eqn.getUnitsMessages()) {
				System.out.println("Units: "+msg);
				errorsFrame.addMessage(msg);
			}

		}
	}
	
	public void generateErrorReport(String title, Map<String, Equation> errors, List<String> messages) {
		Iterator<String> iter = errors.keySet().iterator();
		messages.add("+++ "+title+" +++");
		while (iter.hasNext()) {
			String lhs = iter.next();

			Equation eqn = errors.get(lhs);

			messages.add("Equation:"); // "\n"
			messages.add("\t"+StringEscapeUtils.escapeHtml4(eqn.getVensimEquation().split("~")[0]));

			for (String msg : eqn.getSyntaxMessages()) {
				messages.add(msg);
			}
			for (String msg : eqn.getSemanticMessages()) {
				messages.add(msg);
			}
			for (String msg : eqn.getUnitsInconsistencyReport()) {
				messages.add(msg);
			}
			messages.add("----------");
		}
	}

	/**
	 * @return the ok
	 */
	public boolean isOk() {
		System.out.println("REPORTING isOK = "+ok);
		return ok;
	}

	/**
	 * @return the modelFile
	 */
	public String getModelFile() {
		return modelFile;
	}

	/**
	 * @return the timeStep
	 */
	public String getTimeStep() {
		if (timeStep == null) {
			if (equations != null) {
				Equation eqn = equations.get("TIME STEP");
				if (eqn == null) {
					eqn = equations.get("TIME_STEP");
				}
				Node rhs = TreeTraversal.getRhs(eqn.getTreeRoot());
				timeStep = new String(rhs.getToken());
			} 
		}
		return timeStep;
	}

	/**
	 * @return the timeUnit
	 */
	public String getTimeUnits() {
		if (timeUnit == null) {
			if (equations != null) {
				Equation eqn = equations.get("INITIAL TIME");
				if (eqn == null) {
					eqn = equations.get("INITIAL_TIME");
				}
				timeUnit = new String(eqn.getUnits());
			} 
		}
		return timeUnit;
	}

	/**
	 * @return the dt
	 */
	public String getDt() {
		return getTimeStep();
	}

	/**
	 * @return the startTime
	 */
	public String getStartTime() {
		if (startTime == null) {
			if (equations != null) {
				Equation eqn = equations.get("INITIAL TIME");
				if (eqn == null) {
					eqn = equations.get("INITIAL_TIME");
				}
				Node rhs = TreeTraversal.getRhs(eqn.getTreeRoot());
				startTime = Node.generateExpression(rhs);
			} 
		}
		return startTime;
	}

	/**
	 * @return the endTime
	 */
	public String getEndTime() {
		if (endTime == null) {
			if (equations != null) {
				Equation eqn = equations.get("FINAL TIME");
				if (eqn == null) {
					eqn = equations.get("FINAL_TIME");
				}
				Node rhs = TreeTraversal.getRhs(eqn.getTreeRoot());
				endTime = new String(rhs.getToken());
			} 
		}
		return endTime;
	}
	
	public String getSavPer() {
		if (savPer == null) {
			if (equations != null) {
				Equation eqn = equations.get("SAVEPER");
				Node rhs = TreeTraversal.getRhs(eqn.getTreeRoot());
				savPer = new String(rhs.getToken());
			} 
		}
		return savPer;
	}

	/**
	 * @return the equationProcessor
	 */
	public EquationProcessor getEquationProcessor() {
		return equationProcessor;
	}

	/**
	 * @return the equations
	 */
	public Map<String, Equation> getEquations() {
		return equations;
	}

	/**
	 * @return the graphicsProcessor
	 */
	public GraphicsProcessor getGraphicsProcessor() {
		return graphicsProcessor;
	}

	/**
	 * @return the graphics
	 */
	public Map<String, View> getGraphics() {
		return graphics;
	}

	/**
	 * @return the reader
	 */
	public Reader getReader() {
		return reader;
	}

	public String getVariableUnits(String variable) {
		if (equations.containsKey(variable)) {
			return equations.get(variable).getUnits();
		} else {
			return null;
		}
	}

	public Set<String> getModelLevers() {
		Set<String> levers = new LinkedHashSet<String>();

		for (String var : equations.keySet()) {
			if (equations.get(var).isOneTime())
				levers.add(var);
		}
		return levers;
	}

	public Set<String> getModelOutputs() {
		Set<String> outputs = new LinkedHashSet<String>();

		for (String var : equations.keySet()) {
			if (!equations.get(var).isOneTime())
				outputs.add(var);
		}
		return outputs;
	}

	public SystemDynamicsObjectManager getSdObjectManager() {
		return sdObjectManager;
	}

	public boolean loadProperties() {
		return true; // loadProperties(DEFAULT_PROPERTIES_FILE);
	}

	public boolean loadProperties(String file) {

		try {
			PROPERTIES.load(getClass().getResourceAsStream("/"+file));
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return true;

	}

	private void initializeLogger(ModelManager mm)
	{
		Properties logProperties = new Properties();

		try
		{
			// load our log4j properties / configuration file
			logProperties.load(new FileInputStream(LOG_PROPERTIES_FILE));
			PropertyConfigurator.configure(logProperties);

			Logger rootLogger = Logger.getRootLogger();
			PatternLayout layout = new PatternLayout("%-5p [%c] %m%n");
			RollingFileAppender fileAppender = new RollingFileAppender(layout, mm.getGeneratedOutputDirectoryName()+"/"+"sdmdoc.log");
			fileAppender.setThreshold(Level.ALL);
			fileAppender.setAppend(false);
			fileAppender.activateOptions();
			rootLogger.addAppender(fileAppender);


			log.info("Logging initialized.");
		}
		catch(IOException e)
		{
			throw new RuntimeException("Unable to load logging property " + LOG_PROPERTIES_FILE);
		}
	}

	/**
	 * @return the timeUnit
	 */
	public String getTimeUnit() {
		return timeUnit;
	}

	/**
	 * @return the mdl
	 */
	public MDL getMdl() {
		return mdl;
	}
	
	private boolean isControlVariable(String lhs) {
		String[] controls = {"time", "time step", "timestep", "time_step", "initial time", "initial_time",
				"final time", "final_time", "saveper"};
		for (String no : controls) {
			if (lhs.equalsIgnoreCase(no))
				return true;
		}
		return false;
	}
	
	public void createModelVariablesFiles(String structureDir, String vignetteDir, String runsDir) {
		
		if (StaticSwitches.WRITE_BRIDGE_FILES) {

			String file = structureDir+"/variables.tab";
			BufferedWriter bw = Utilities.openFileForWriting(file);

			String vfile = vignetteDir+"/baseline.tab";
			BufferedWriter bwv = Utilities.openFileForWriting(vfile);

			String lfile = structureDir+"/locations.tab";
			BufferedWriter bwl = Utilities.openFileForWriting(lfile);



			int pos = 1;
			List<String> forOrdering = new ArrayList<String>(equations.keySet());
			Collections.sort(forOrdering);
			for (String key : forOrdering) {
				Equation eqn = equations.get(key);
				eqn.markFlows();
			}
			try {
				for (String key : forOrdering) {
					Equation eqn = equations.get(key);
					if (isControlVariable(eqn.getLhs()))
						continue;
					if (eqn.getLhs().contains(StringConstants.RSSD_LOOKUP))
						continue;
					if (eqn.getLhs().contains("NAREPLACEMENT"))
						continue;
					if (eqn.isDefinesLookup())
						continue;

					BridgeModelVariable mv = new BridgeModelVariable(eqn);
					mv.saveWithType(bw);
					mv.saveAsVignette(bwv, pos++);
					bw.append("\n");
					bwv.append("\n");
				}

				for (View v : graphics.values()) {
					v.writeLocations(bwl);
				}

				bw.close();
				bwv.close();
				bwl.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
		// create custom graphs if in vensim file
		
		if (StaticSwitches.WRITE_BRIDGE_FILES) {
			createCustomGraphs(structureDir);

			String template = "data/ScenarioDocumentationTemplate.docx";
			String helpFile = vignetteDir+"/baseline.docx";
			try {
				FileUtils.copyFile(new File(template), new File(helpFile));

			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

		}
			// copy the mdl file to the model directory

			try {
				FileUtils.copyFileToDirectory(new File(modelFile), new File(structureDir));
				FileUtils.copyFileToDirectory(new File(modelFile), new File(structureDir).getParentFile());
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		
		
		// create the vignettes and runs directories
		if (StaticSwitches.WRITE_BRIDGE_FILES) {
			Utilities.writeToFile(vignetteDir+"/readme.txt", "Vignette files will be placed here\n");
		}
		Utilities.writeToFile(runsDir+"/readme.txt", "Run results files will be placed here\n");
		
		if (StaticSwitches.WRITE_BRIDGE_FILES) {
			if (!this.isFullyDefined()) {
				// need to build tail and head maps from arrows and not equations
				Map<String, Set<String>> headTailMap = new LinkedHashMap<String, Set<String>>();
				Map<String, Set<String>> tailHeadMap = new LinkedHashMap<String, Set<String>>();
				graphicsProcessor.buildHeadsAndTails(headTailMap, tailHeadMap);
				SDModelCodeGenerator.writeMapToFile(structureDir+"/headTailMap.tab", headTailMap);
				SDModelCodeGenerator.writeMapToFile(structureDir+"/tailHeadMap.tab", tailHeadMap);
			}
		}
		
		System.out.println("SDModel "+structureDir+"/");
		BridgeModelMeta meta = new BridgeModelMeta(structureDir+"/", objectName);
		meta.setMdlFile(new File(modelFile).getName());
		meta.setMap(!this.isFullyDefined());
		meta.setRunnable(this.isFullyDefined());
		meta.setDt(getDt());
		meta.setInitialTime(getStartTime());
		meta.setFinalTime(getEndTime());
		meta.setSaveper(getSavPer());
		meta.setTimeUnits(getTimeUnits());
		meta.save();
	}
	
	private int numCustomGraphs(List<String> graphics) {
		int num = 0;
		int ptr = 0;
		while (ptr < graphics.size()) {
			String line = graphics.get(ptr++);
			if (line.toLowerCase().startsWith(":graph")) {
				num++;
			}
		}
		return num;
	}
	
	private int graphNPointer(List<String> graphics, int n) {
		int num = 0;
		int ptr = 0;
		while (ptr < graphics.size()) {
			String line = graphics.get(ptr);
			if (line.toLowerCase().startsWith(":graph")) {
				num++;
				if (num == n)
					return ptr;
			}
			ptr++;
		}
		return -1;
	}
	
	private int graphNTitle(List<String> graphics, int n) {
		int ptr = graphNPointer(graphics, n);
		while (ptr < graphics.size()) {
			String line = graphics.get(ptr);
			if (line.toLowerCase().startsWith(":graph")) {
				return -1;
			} else if (line.toLowerCase().startsWith(":title")) {
				return ptr;
			}
			ptr++;
		}
		return -1;
	}
	
//	public void createCustomGraphs(String structureDir) {
//		List<SDModelCustomGraph> graphs = new ArrayList<SDModelCustomGraph>();
//		List<String> graphics = mdl.getGraphicsPart();
//		int ptr = 0;
//		while (ptr < graphics.size()) {
//			String line = graphics.get(ptr++);
//			if (line.toLowerCase().startsWith(":graph")) {
//				while (ptr < graphics.size() && !graphics.get(ptr).toLowerCase().startsWith(":title ")) {
//					ptr++;
//				}
//				String title = graphics.get(ptr).split(" ", 2)[1].trim();
//				SDModelCustomGraph cg = new SDModelCustomGraph(title);
//				graphs.add(cg);
//				while (++ptr < graphics.size()) {
//					if (graphics.get(ptr).toLowerCase().startsWith(":var ")) {
//						cg.addVariable(graphics.get(ptr).split(" ", 2)[1].trim());
//					}
//					if (graphics.get(ptr).toLowerCase().startsWith(":graph ")) {
//						break;
//					}
//				}
//			}
//		}
//		if (graphs.size() > 0) {
//			String file = structureDir+"/customGraphs.tab";
//			BufferedWriter bw = Utilities.openFileForWriting(file);
//			try {
//				for (SDModelCustomGraph cg : graphs) {
//					bw.append(cg.getFileRecord());
//				}
//				bw.close();
//			} catch (IOException e) {
//				// TODO Auto-generated catch block
//				e.printStackTrace();
//			}
//			
//			// now lets create this for the baseline scenario
//			
//			String baselineFile = structureDir+"/../vignettes/baseline_customGraphs.tab";
//			try {
//				FileUtils.copyFile(new File(file), new File(baselineFile));
//				
//			} catch (IOException e1) {
//				// TODO Auto-generated catch block
//				e1.printStackTrace();
//			}
//		}
//	}
	
	public void createCustomGraphs(String structureDir) {
		List<SDModelCustomGraph> graphs = new ArrayList<SDModelCustomGraph>();
		List<String> graphics = mdl.getGraphicsPart();
		int numGraphs = numCustomGraphs(graphics);
		int ptr = 0;
		String title;
		for (int n = 1; n <= numGraphs; n++) {
			ptr = graphNPointer(graphics, n);
			int titl = graphNTitle(graphics, n);
			if (titl > 0) {
				title = graphics.get(titl).split(" ", 2)[1].trim();
			} else {
				title = "No Title";
			}
			SDModelCustomGraph cg = new SDModelCustomGraph(title);
			graphs.add(cg);
			int end = graphics.size();
			if (n < numGraphs)
				end = graphNPointer(graphics, n+1);


			while (ptr < end) {
				if (graphics.get(ptr).toLowerCase().startsWith(":var ")) {
					cg.addVariable(graphics.get(ptr).split(" ", 2)[1].trim());
				}
				ptr++;
			}
		}
			
		
		if (graphs.size() > 0) {
			String file = structureDir+"/customGraphs.tab";
			BufferedWriter bw = Utilities.openFileForWriting(file);
			try {
				for (SDModelCustomGraph cg : graphs) {
					bw.append(cg.getFileRecord());
				}
				bw.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			// now lets create this for the baseline scenario
			
			String baselineFile = structureDir+"/../vignettes/baseline_customGraphs.tab";
			try {
				FileUtils.copyFile(new File(file), new File(baselineFile));
				
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}
	}

	/**
	 * @return the fullyDefined
	 */
	public boolean isFullyDefined() {
		return fullyDefined;
	}
	
	public boolean isMap() {
		return !fullyDefined;
	}

	public boolean isVariableNameTranslationMode() {
		return variableNameTranslationMode;
	}

	public void setVariableNameTranslationMode(boolean variableNameTranslationMode) {
		this.variableNameTranslationMode = variableNameTranslationMode;
	}

}
