package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.ProcessBuilder.Redirect;

public class SDModelCompiler extends SDModelTool {
	
//	private static final String CMD = "jdk1.8.0_45/bin/javac";
	
	private static final String CMD = "javac";
	
	private static final String CLASSPATH = "-cp";
	
	private static final String UTF8 = "UTF-8";
	private static final String ENCODING = "-encoding";
	
	String javaCompileFile;
	MessageProcessor info;
	
	
	public SDModelCompiler(String javaCompileFile, MessageProcessor info) {
		this.javaCompileFile = javaCompileFile;
		this.info = info;
	}
	
	public void javac(String classPath, String srcFile) {
		ProcessBuilder pb;
		pb = new ProcessBuilder(CMD, CLASSPATH, classPath, srcFile);
		File log = new File(javaCompileFile);
	
		if (log.exists())
			log.delete();
		 pb.redirectErrorStream(true);
		 pb.redirectOutput(Redirect.appendTo(log));
		
		try {
			Process tr = pb.start();
			try {
				tr.waitFor();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			ok = true;
		} catch (IOException e) {
			// TODO Auto-generated catch block
			ok = false;
			System.out.println("Compile failed");
			e.printStackTrace();
		}
	}
	
	
	public void compile(String classPath, String srcFile) {
		
		

		String[] javacArgs = new String[] {
				"-classpath", classPath, ENCODING, UTF8, 
				"-d", "./Dynamic/" + FileOrganizer.getSrcClassDirectory(),
				srcFile };
		PrintWriter out = null;
		try {
			out = new PrintWriter( new FileWriter( srcFile.replace(".java", ".compile.txt") ) );
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			ok = false;
			System.out.println("Couldn't open C:/temp/out.txt");
			return;
		}

		int errorCode = -1;
		try {
			errorCode = com.sun.tools.javac.Main.compile(javacArgs, out);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		System.out.println("Class Directory: "+new File(srcFile).getParent());

		System.out.println(StringConstants.COMPILE_ERROR_CODE+": "+ errorCode);
		info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.COMPILE_ERROR_CODE)+": "+ errorCode);
		ok = errorCode == 0 ? true : false;
		return;
	}

}
