package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.VariableNumberGenerator;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationProcessor;
import gov.anl.gss.sdm.server.model.translator.GraphicsProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.SystemDynamicsObjectManager;
import gov.anl.gss.sdm.server.model.translator.View;

import java.util.Map;

public class SDModelDocumentor extends SDModelTool {

	private SDModel sdModel;

	private EquationProcessor equationProcessor;
	private Map<String, Equation> equations;
	private GraphicsProcessor graphicsProcessor;
	private Map<String, View> graphics;
	protected SystemDynamicsObjectManager sdObjectManager;
	
	protected UserSelectedOptions userSelectedOptions;
	protected SDMDocLanguageSpecific lang;
	
	private boolean fullyDefined;
	
	private boolean sdmuiMode = false;
	private String sdmuiRootDirectory;
	
	private String objectName;
	
	AnalysisManager analysisManager;
	
	private MessageProcessor info;
	
	public SDModelDocumentor(SDModel sdModel, MessageProcessor info, UserSelectedOptions userSelectedOptions) {
		equationProcessor = sdModel.getEquationProcessor();
		equations = sdModel.getEquations();
		graphicsProcessor = sdModel.getGraphicsProcessor();
		graphics = sdModel.getGraphics();
		sdObjectManager = sdModel.getSdObjectManager();
		this.fullyDefined = sdModel.isFullyDefined();
		this.userSelectedOptions = userSelectedOptions;
		InformationManagers.getInstance().setUserSelectedOptions(userSelectedOptions);
		this.info = info;
		
		
		
		SDMDocLanguageSpecific.initialize();
		lang = SDMDocLanguageSpecific.getInstance();

	}
	
	public void processPolarity(String objectName) {
			processFullyDefinedPolarity(objectName);
	}
	
	public void processLeea(String objectName) {
		processFullyDefinedLeea(objectName);
}
	
	public void process(String objectName) {
		if (isFullyDefined()) {
			processFullyDefined(objectName);
		} else {
			processFullyConnected(objectName);
		}
	}
	
	protected void processFullyConnected(String objectName) {
		analysisManager = new AnalysisManager(sdObjectManager, equations, graphics, false /* not fully defined */, info, objectName);
		InformationManagers.getInstance().setAnalysisManager(analysisManager);
		new VariableNumberGenerator();
    	analysisManager.analyzeFullyConnected();
    	Utilities.writeGeneratedVarMap(InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory()+"/generatedVarMap.tab");
//    	info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.GENERATING_DOCUMENT));
	}

	protected void processFullyDefined(String objectName) {
		analysisManager = new AnalysisManager(sdObjectManager, equations, graphics, true /* fully defined */, info, objectName);
		InformationManagers.getInstance().setAnalysisManager(analysisManager);
		analysisManager.setSdmuiMode(sdmuiMode);
		analysisManager.setSdmuiRootDirectory(sdmuiRootDirectory);
		new VariableNumberGenerator();
		analysisManager.analyzeFullyDefinedPolarity();
    	analysisManager.analyzeFullyDefined();
    	Utilities.writeGeneratedVarMap(InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory()+"/generatedVarMap.tab");
	}
	
	protected void processFullyDefinedPolarity(String objectName) {
		
//    	analysisManager.analyzeFullyDefinedPolarity();
	}
	
	protected void processFullyDefinedLeea(String objectName) {
		
    	analysisManager.analyzeFullyDefinedLeea();
	}
	
	public void generateHTML() {
		
		info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.GENERATING_DOCUMENT));
		info.setBlipMode(true);
    	analysisManager.document(getProjectLocation());
    	info.setBlipMode(false);
	}
	
	protected String getProjectLocation() {
		return "output";
	}

	protected String asDirectoryPath(String packageName) {
		return packageName.replace(".", "/");
	}

	/**
	 * @return the sdModel
	 */
	public SDModel getSdModel() {
		return sdModel;
	}

	/**
	 * @param sdModel the sdModel to set
	 */
	public void setSdModel(SDModel sdModel) {
		this.sdModel = sdModel;
	}

	/**
	 * @return the equationProcessor
	 */
	public EquationProcessor getEquationProcessor() {
		return equationProcessor;
	}

	/**
	 * @param equationProcessor the equationProcessor to set
	 */
	public void setEquationProcessor(EquationProcessor equationProcessor) {
		this.equationProcessor = equationProcessor;
	}

	/**
	 * @return the equations
	 */
	public Map<String, Equation> getEquations() {
		return equations;
	}

	/**
	 * @param equations the equations to set
	 */
	public void setEquations(Map<String, Equation> equations) {
		this.equations = equations;
	}

	/**
	 * @return the graphicsProcessor
	 */
	public GraphicsProcessor getGraphicsProcessor() {
		return graphicsProcessor;
	}

	/**
	 * @param graphicsProcessor the graphicsProcessor to set
	 */
	public void setGraphicsProcessor(GraphicsProcessor graphicsProcessor) {
		this.graphicsProcessor = graphicsProcessor;
	}

	/**
	 * @return the graphics
	 */
	public Map<String, View> getGraphics() {
		return graphics;
	}

	/**
	 * @param graphics the graphics to set
	 */
	public void setGraphics(Map<String, View> graphics) {
		this.graphics = graphics;
	}

	/**
	 * @return the sdObjectManager
	 */
	public SystemDynamicsObjectManager getSdObjectManager() {
		return sdObjectManager;
	}

	/**
	 * @param sdObjectManager the sdObjectManager to set
	 */
	public void setSdObjectManager(SystemDynamicsObjectManager sdObjectManager) {
		this.sdObjectManager = sdObjectManager;
	}

	/**
	 * @return the fullyDefined
	 */
	public boolean isFullyDefined() {
		return fullyDefined;
	}

	/**
	 * @param fullyDefined the fullyDefined to set
	 */
	public void setFullyDefined(boolean fullyDefined) {
		this.fullyDefined = fullyDefined;
	}

	/**
	 * @return the objectName
	 */
	public String getObjectName() {
		return objectName;
	}

	/**
	 * @param objectName the objectName to set
	 */
	public void setObjectName(String objectName) {
		this.objectName = objectName;
	}

	/**
	 * @return the sdmuiMode
	 */
	public boolean isSdmuiMode() {
		return sdmuiMode;
	}

	/**
	 * @param sdmuiMode the sdmuiMode to set
	 */
	public void setSdmuiMode(boolean sdmuiMode) {
		this.sdmuiMode = sdmuiMode;
	}

	/**
	 * @return the sdmuiRootDirectory
	 */
	public String getSdmuiRootDirectory() {
		return sdmuiRootDirectory;
	}

	/**
	 * @param sdmuiRootDirectory the sdmuiRootDirectory to set
	 */
	public void setSdmuiRootDirectory(String sdmuiRootDirectory) {
		this.sdmuiRootDirectory = sdmuiRootDirectory;
	}
}
