package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.support.database.ModelRunDatabaseVerifier;
import gov.anl.gss.sdm.server.model.support.database.SDMDocDatabase;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.SystemUtils;

public class SDModelProcessor extends SDModelTool {
	
	public static Properties PROPERTIES = new Properties();
	
	private static final boolean GEN_CODE = true;
	private boolean genDoc = true;
	private boolean batchExecution = false;
	
	private static final String SRC_CLASS = FileOrganizer.getSrcClassDirectory();
	
//	boolean newMemoryDefinition;
	
	boolean sdmuiLoad = false;
	
	private SDModel sdModel;
	private SDModelCodeGenerator sdModelCodeGenerator;
	private SDModelDocumentor sdModelDocumentor;
	private SDModelRunner sdModelRunner;
	private SDModelCompiler sdModelCompiler;
	
	private String packageName;
	private String objectName;
	private String inputFileType;
	private String mdlFile;
	private String memoryImplementation;
	private String rootDirectory;
	
	private String docGraphDirectory;
	private String modelDirectory;
	
	private MessageProcessor info;
	private MessageProcessor error;
	private UserSelectedOptions options;
	
	private FileOrganizer fileOrganizer;
	
	private boolean variableNameTranslationMode = false;

	private Object codeGenerationFailure;
	
	public SDModelProcessor(String inputFileType, String mdlFile, String packageName, String objectName, 
			String memoryImplementation, String rootDirectory, MessageProcessor info, MessageProcessor error,
			UserSelectedOptions options, /*boolean newMemoryDefinition,*/ FileOrganizer fileOrganizer) {
		this(inputFileType, mdlFile, packageName, objectName, memoryImplementation, 
				rootDirectory, info, error, options, /*newMemoryDefinition,*/ false, fileOrganizer);
	}
	
	public SDModelProcessor(String inputFileType, String mdlFile, String packageName, String objectName, 
			String memoryImplementation, String rootDirectory, MessageProcessor info, MessageProcessor error,
			UserSelectedOptions options, /* boolean newMemoryDefinition,*/ boolean sdmuiLoad,  FileOrganizer fileOrganizer) {
		
		this.inputFileType = inputFileType;
		this.mdlFile = mdlFile;
		this.packageName = packageName;
		this.objectName = objectName;
		this.rootDirectory = rootDirectory;
		
		this.memoryImplementation = memoryImplementation;
//		this.newMemoryDefinition = newMemoryDefinition;
		this.info = info;
		this.error = error;
		this.options = options;
		this.sdmuiLoad = sdmuiLoad;
		
		this.fileOrganizer = fileOrganizer;
	
	}
	
	private void createModelDirectoryStructure(String objectName) {

		fileOrganizer.print();
		// make sure that the model directory exists
		new File(fileOrganizer.getModelDirectory()).mkdirs();
		new File(fileOrganizer.getRunsDirectory()).mkdirs();
		new File(fileOrganizer.getSrcGenDirectory()).mkdirs();
		new File(fileOrganizer.getStructureDirectory()).mkdirs();
		
		new File(fileOrganizer.getViewsDirectory()).mkdirs();
		new File(fileOrganizer.getLoopsDirectory()).mkdirs();
		new File(fileOrganizer.getThumbnailsDirectory()).mkdirs();
		if (fileOrganizer.isSdmui()) {
			new File(fileOrganizer.getVignettesDirectory()).mkdirs();
			new File(fileOrganizer.getSdmdocDirectory()).mkdirs();
		}
		
		// create the SDM-Doc database
		
		if (!batchExecution) 
			ModelRunDatabaseVerifier.purge(fileOrganizer.getRunsDirectory());
		
		SDMDocDatabase db = new SDMDocDatabase(fileOrganizer.getStructureDirectory()+"/"+objectName+"_HSQLDB");
		InformationManagers.getInstance().getDatabaseManager().setDb(db);
		db.initializeNewSDMDocDatabase();
		
		
	}
	
	public void process(boolean sdmuiLoad) {

//		System.out.println("process(boolean sdmuiLoad)");
//		options.print();
		this.sdmuiLoad = sdmuiLoad;
		
		Reuse reuse = InformationManagers.getInstance().getReuse();
		
		sdModel = new SDModel(mdlFile, inputFileType, objectName, rootDirectory, fileOrganizer,InformationManagers.getInstance().getModelManager().isXmileOrigin(),
				InformationManagers.getInstance().getModelManager().getXmileFileName());
		sdModel.setVariableNameTranslationMode(variableNameTranslationMode);
		
		InformationManagers.getInstance().setReuse(reuse);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		info.addMessage(language.getText(StringConstants.REUSE_VIEW_DIAGRAM)+":  "+reuse.isViews());
		info.addMessage(language.getText(StringConstants.REUSE_CLASS)+":  "+reuse.isClassFile());
		info.addMessage(language.getText(StringConstants.REUSE_THUMBNAILS)+":  "+reuse.isThumbnails());
		info.addMessage(language.getText(StringConstants.REUSE_POLARITIES)+":  "+reuse.isPolarity());
		
		String modelName = InformationManagers.getInstance().getModelManager().getModelName();
		if (sdmuiLoad) {
			InformationManagers.getInstance().getFileOrganizer().
			updateForModel(InformationManagers.getInstance().
					getModelManager().getModelName());
		}
		
		InformationManagers.getInstance().setSdmuiMode(sdmuiLoad);
		
		
		
		docGraphDirectory = fileOrganizer.getDocGraphDirectory();
		modelDirectory = rootDirectory;
		
		String javaSrcFile = null;
		String javaCompileFile = null;

		String javaMemorySrcFile = null;
		String javaMemoryCompileFile = null;

		
		if (!isVariableNameTranslationMode()) {


			if (sdmuiLoad) {
				System.out.println("FileUtils.copyFileToDirectory");
				try {
					FileUtils.copyFileToDirectory(new File(mdlFile), new File(fileOrganizer.getModelDirectory()));
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			System.out.println("POST FileUtils.copyFileToDirectory");
			System.out.println("BEFORE createModelDirectoryStructure(objectName);");

			createModelDirectoryStructure(objectName);

			System.out.println("AFTER createModelDirectoryStructure(objectName);");

			// these files are relative to the installed software
			javaSrcFile =fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/"+objectName+".java";
			javaCompileFile = fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/"+objectName+".compile.txt";

			javaMemorySrcFile =fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/Memory"+objectName+".java";
			javaMemoryCompileFile = fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/Memory"+objectName+".compile.txt";

		}
		info.addMessage(language.getText(StringConstants.OPERATING_SYSTEM)+": "+System.getProperty("os.name"));
		
		InformationManagers.getInstance().setUserSelectedOptions(options);
		info.addMessage(language.getText(StringConstants.LOAD_AND_PROCESS));
		boolean lAndP = sdModel.loadAndProcess();
		
	//  Here is where we return if doing translation
		if (isVariableNameTranslationMode())
			return;

		
		
		if (sdModel.isOk()) {
			System.out.println("MODEL IS OK in SDModelProcessor.process()");
			sdModel.createModelVariablesFiles(fileOrganizer.getStructureDirectory(),
					fileOrganizer.getVignettesDirectory(), fileOrganizer.getRunsDirectory());
		} else {
			System.out.println("MODEL IS NOT OK in SDModelProcessor.process()");		}
		InformationManagers.getInstance().getMappedSubscriptManager().makeConsistent();

		boolean useExistingClassFile = InformationManagers.getInstance().getReuse().isClassFile();
		boolean useExistingThumbnails = InformationManagers.getInstance().getReuse().isThumbnails();
		
		
		
		boolean generateThumbnails = InformationManagers.getInstance().getModelManager().isRunModel();
		generateThumbnails = generateThumbnails && options.isSelected(StringConstants.INCLUDE_THUMBNAILS);
		generateThumbnails = generateThumbnails && !useExistingThumbnails; 
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails && options.isSelected(StringConstants.INCLUDE_THUMBNAILS);
		
		if (!genDoc) {
			useExistingClassFile = true;
			generateThumbnails = false;
		}
		// look head to turn off thumbnails if they are unavailable
		if (!InformationManagers.getInstance().getModelManager().isRunnable()) {
			//			options.setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", "false");
			InformationManagers.getInstance().getModelManager().setRunModel(false);
		} else {
			InformationManagers.getInstance().getModelManager().setRunModel(generateThumbnails);
		}

		

		
		System.out.println("isRunnable: "+InformationManagers.getInstance().getModelManager().isRunnable());
		System.out.println("runModel: "+InformationManagers.getInstance().getModelManager().isRunModel());
		System.out.println("useExistingClassFile: "+InformationManagers.getInstance().getReuse().isClassFile());
		System.out.println("useExistingThumbnails: "+InformationManagers.getInstance().getReuse().isThumbnails());
		System.out.println("generateThumbnails: "+generateThumbnails);
		System.out.println("batchExecution: "+batchExecution);
		
//		info.addMessage("OS: "+System.getProperty("os.name"));
		
		if ((sdModel.isMap() || sdModel.isOk()) && genDoc /*&& !sdmuiLoad*/) {
			sdModelDocumentor = new SDModelDocumentor(sdModel, info, options);
			sdModelDocumentor.setSdmuiMode(true);
			sdModelDocumentor.setSdmuiRootDirectory(rootDirectory);
			sdModelDocumentor.process(objectName);
			sdModelDocumentor.processPolarity(objectName);
		}
		
		
		if ((!sdModel.isMap() /* && options.isSelected(StringConstants.INCLUDE_THUMBNAILS)*/ ) || sdmuiLoad) {
			boolean runnable = InformationManagers.getInstance().getModelManager().isRunnable();
			boolean isRunModel = InformationManagers.getInstance().getModelManager().isRunModel();
			
			
			if (InformationManagers.getInstance().getModelManager().isRunnable() && 
					(InformationManagers.getInstance().getModelManager().isRunModel() || batchExecution )) {
				
				// close any workbooks to avoid contention
				InformationManagers.getInstance().getExcelManager().closeAllWorkbooks();

				if (useExistingClassFile) {
					// runner
					
					// don't need to generate code, but we will not need to run the model is we are 
					// resusing thumbnails and link polarity but we will need to copy over the database
					
					if (reuse.isThumbnails() && reuse.isPolarity()) {
						String fromDir = InformationManagers.getInstance().getReuse().getDocGraphDirectory();
						String toDir = InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName();
						File fromF = new File(fromDir);
						File toF = new File(toDir);
						try {
							if (!fromF.getCanonicalPath().equals(toF.getCanonicalPath())) {
								
								try {
									
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.data"), 
												new File(toDir+"/runs/baselineHSQLDB.data"));
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.properties"), 
												new File(toDir+"/runs/baselineHSQLDB.properties"));
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.script"), 
												new File(toDir+"/runs/baselineHSQLDB.script"));
									
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					} else {

					SDModelRunner sdmRunner = new SDModelRunner();
					info.addMessage(SDMDocLanguageSpecific.getInstance().
							getText(StringConstants.EXECUTING_MODEL_TO_GENERATE_BASELINE_RUN));
					
					if (!sdmRunner.run(modelName, rootDirectory, "baseline", /*newMemoryDefinition,*/ generateThumbnails,
							fileOrganizer, info)) {
						info.addMessage(SDMDocLanguageSpecific.getInstance().
								getText(StringConstants.COULD_NOT_CREATE_THUMBNAILS));
						InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
					} else {
						InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(true);
					}
					}

				} else {

					if (lAndP && sdModel.isFullyDefined() && sdModel.isOk()) {
						info.addMessage(language.getText(StringConstants.BEGIN_PROCESSING_EQUATIONS));
						sdModelCodeGenerator = new SDModelCodeGenerator(sdModel, packageName, objectName, 
								rootDirectory, /* memoryImplementation,*/ javaSrcFile, info, /* newMemoryDefinition, */
								fileOrganizer);
						List<String> messages = new ArrayList<String>();
						if (sdModelCodeGenerator.isValid(messages) && GEN_CODE) {
							codeGenerationFailure = sdModelCodeGenerator.process();
						} else {
							if (GEN_CODE) {
								info.addMessage("Model Equations Invalid -- Code not generated");;
								for (String m : messages) {
									info.addMessage(m);
								}
							}
						}
					}

					//			if (InformationManagers.getInstance().getModelManager().isRunnable()) {
					//				if (!sdModel.isMap() && options.isSelected(StringConstants.INCLUDE_THUMBNAILS) || sdmuiLoad) {
					if (!sdModel.isMap() && (InformationManagers.getInstance().getModelManager().isRunModel() /* && 
						!InformationManagers.getInstance().getModelManager().isXmileOrigin()*/) || sdmuiLoad || batchExecution) {

//						info.addMessage("Creating Thumbnails\n");
						info.setBlipMode(true);


						if (sdModelCodeGenerator.isOk() && sdModel.isFullyDefined() && sdModel.isOk()) {
							// compiler

							info.addMessage(language.getText(StringConstants.COMPILING_MODEL_MEMORY));
							boolean sdmCompilerMemoryOK = true;
							SDModelCompiler sdmCompilerMemory; 
							if (memoryImplementation.equals("Native")) {
								sdmCompilerMemory = new SDModelCompiler(javaMemoryCompileFile, info);
								sdmCompilerMemory.compile("./SDM-Doc.jar",javaMemorySrcFile);
								sdmCompilerMemoryOK = sdmCompilerMemory.isOk();
								if (sdmCompilerMemoryOK) {
									//								File f = new File(javaMemoryCompileFile);
									//								if (f.length() == 0l) {
									//									f.delete();
									//								}
								} else {
									InformationManagers.getInstance().getModelManager().
									setRunnable(false, language.getText(StringConstants.JAVA_COMPILE_ERRORS));
									info.addMessage(language.getText(StringConstants.JAVA_COMPILE_ERRORS));
									InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
								}
							}

							//						File thisDir = new File(".");
							//						for (String s : thisDir.list())
							//							System.out.println("FILE: "+s);

							SDModelCompiler sdmCompiler = new SDModelCompiler(javaCompileFile, info);
//							info.addMessage("OS: "+System.getProperty("os.name"));
							String separator = ":";
							if (SystemUtils.IS_OS_WINDOWS)
								separator = ";";
							info.addMessage(language.getText(StringConstants.COMPILING_MODEL_EQUATIONS));
							sdmCompiler.compile("./SDM-Doc.jar"+separator+fileOrganizer.getDynamicDirectory()+"/"+
									FileOrganizer.getSrcClassDirectory(),javaSrcFile);

							// runner
							boolean compiled = sdmCompiler.isOk();
							if (compiled) {
								if (compiled && sdmCompilerMemoryOK) {
									// lets copy source code into output directory for user
									try {
										FileUtils.copyFileToDirectory(new File(javaSrcFile), 
												new File(InformationManagers.getInstance().
														getFileOrganizer().getSrcGenDirectory()));
									} catch (IOException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									try {
										FileUtils.copyFileToDirectory(new File(javaMemorySrcFile), 
												new File(InformationManagers.getInstance().
														getFileOrganizer().getSrcGenDirectory()));
									} catch (IOException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									if (!batchExecution) {
										SDModelRunner sdmRunner = new SDModelRunner();
										
										info.addMessage(SDMDocLanguageSpecific.getInstance().
											getText(StringConstants.EXECUTING_MODEL_TO_GENERATE_BASELINE_RUN));
										if (!sdmRunner.run(modelName/*objectName*/, rootDirectory, "baseline", /*newMemoryDefinition,*/ generateThumbnails,
												fileOrganizer, info)) {
											info.addMessage(SDMDocLanguageSpecific.getInstance().
													getText(StringConstants.COULD_NOT_CREATE_THUMBNAILS));
											InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
										} else {
											InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(true);
										}
									}
								}
							} else {
								InformationManagers.getInstance().getModelManager().setRunnable(false, 
										SDMDocLanguageSpecific.getInstance().getText(StringConstants.JAVA_COMPILE_ERRORS));
								info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.JAVA_COMPILE_ERRORS));
								InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
							}
						}
					}
				}
				info.setBlipMode(false);
			} else {
				
					info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.MODEL_IS_NOT_RUNNABLE));
					info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.REASONS)+":\n"+
					InformationManagers.getInstance().getModelManager().getRunnableReason());
					
					boolean tooBig = true;
				
				// couldn't run model for some reason, but let's try to generate code
					if (!tooBig) {
				try {
					if (lAndP && sdModel.isFullyDefined() && sdModel.isOk()) {
						
						sdModelCodeGenerator = new SDModelCodeGenerator(sdModel, packageName, objectName, 
								rootDirectory, /* memoryImplementation,*/ javaSrcFile, info, /*newMemoryDefinition, */
								fileOrganizer);
						List<String> messages = new ArrayList<String>();
						if (sdModelCodeGenerator.isValid(messages) && GEN_CODE) {
							info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.GENERATING_CODE));
							sdModelCodeGenerator.process();
						}
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
					}

//				if (options.isSelected(StringConstants.INCLUDE_THUMBNAILS)) {
//					String reason = InformationManagers.getInstance().getModelManager().getRunnableReason();
//					info.addMessage("Model Unrunable. Reason: \n"+reason+"\n");
//				} else {
//					info.addMessage("Model not run -- Thumbnails not requested\n");
//				}

				// need to turn off thumbnails if requested
				//				options.setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", "false");
				InformationManagers.getInstance().getModelManager().setRunModel(false);

			}
		}
		
		if (sdModel.isOk() && genDoc /*&& !sdmuiLoad*/) {
//			sdModelDocumentor.processLeea(objectName);   // 9/23/2019 for now until xmile stuff is working
			
//			sdModelDocumentor.processLeea(objectName); 
			
		}
			
			
			
			if (sdModel.isOk() && genDoc /*&& !sdmuiLoad*/) {
				sdModelDocumentor.generateHTML();
				info.addMessage(language.getText(StringConstants.HTML_REPORT_FILE_GENERATED));
				

			}
			
			// done with the sdm-doc database
			InformationManagers.getInstance().getDatabaseManager().shutdownDatabase();
			if (InformationManagers.getInstance() != null && 
					InformationManagers.getInstance().getAnalysisManager() != null && 
					InformationManagers.getInstance().getAnalysisManager().getLoopManager() != null)
				InformationManagers.getInstance().getAnalysisManager().getLoopManager().releaseLoopStorage();
		

	}
	
	public void processSDA(boolean sdmuiLoad) {

//		System.out.println("process(boolean sdmuiLoad)");
//		options.print();
		this.sdmuiLoad = sdmuiLoad;
		
		Reuse reuse = InformationManagers.getInstance().getReuse();
		
		sdModel = new SDModel(mdlFile, inputFileType, objectName, rootDirectory, fileOrganizer,InformationManagers.getInstance().getModelManager().isXmileOrigin(),
				InformationManagers.getInstance().getModelManager().getXmileFileName());
		sdModel.setVariableNameTranslationMode(variableNameTranslationMode);
		
		InformationManagers.getInstance().setReuse(reuse);
		
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		
		info.addMessage(language.getText(StringConstants.REUSE_VIEW_DIAGRAM)+":  "+reuse.isViews());
		info.addMessage(language.getText(StringConstants.REUSE_CLASS)+":  "+reuse.isClassFile());
		info.addMessage(language.getText(StringConstants.REUSE_THUMBNAILS)+":  "+reuse.isThumbnails());
		info.addMessage(language.getText(StringConstants.REUSE_POLARITIES)+":  "+reuse.isPolarity());
		
		String modelName = InformationManagers.getInstance().getModelManager().getModelName();
		if (sdmuiLoad) {
			InformationManagers.getInstance().getFileOrganizer().
			updateForModel(InformationManagers.getInstance().
					getModelManager().getModelName());
		}
		
		InformationManagers.getInstance().setSdmuiMode(sdmuiLoad);
		
		
		
		docGraphDirectory = fileOrganizer.getDocGraphDirectory();
		modelDirectory = rootDirectory;
		
		String javaSrcFile = null;
		String javaCompileFile = null;

		String javaMemorySrcFile = null;
		String javaMemoryCompileFile = null;

		
		if (!isVariableNameTranslationMode()) {


			if (sdmuiLoad) {
				System.out.println("FileUtils.copyFileToDirectory");
				try {
					FileUtils.copyFileToDirectory(new File(mdlFile), new File(fileOrganizer.getModelDirectory()));
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			System.out.println("POST FileUtils.copyFileToDirectory");
			System.out.println("BEFORE createModelDirectoryStructure(objectName);");

			createModelDirectoryStructure(objectName);

			System.out.println("AFTER createModelDirectoryStructure(objectName);");

			// these files are relative to the installed software
			javaSrcFile =fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/"+objectName+".java";
			javaCompileFile = fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/"+objectName+".compile.txt";

			javaMemorySrcFile =fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/Memory"+objectName+".java";
			javaMemoryCompileFile = fileOrganizer.getDynamicDirectory()+"/"+SRC_CLASS+"/"+ asDirectoryPath(packageName)+"/Memory"+objectName+".compile.txt";

		}
		info.addMessage(language.getText(StringConstants.OPERATING_SYSTEM)+": "+System.getProperty("os.name"));
		
		InformationManagers.getInstance().setUserSelectedOptions(options);
		info.addMessage(language.getText(StringConstants.LOAD_AND_PROCESS));
		boolean lAndP = sdModel.loadAndProcess();
		
	//  Here is where we return if doing translation
		if (isVariableNameTranslationMode())
			return;

		
		
		if (sdModel.isOk()) {
			sdModel.createModelVariablesFiles(fileOrganizer.getStructureDirectory(),
					fileOrganizer.getVignettesDirectory(), fileOrganizer.getRunsDirectory());
		}

		boolean useExistingClassFile = InformationManagers.getInstance().getReuse().isClassFile();
		boolean useExistingThumbnails = InformationManagers.getInstance().getReuse().isThumbnails();
		
		boolean generateThumbnails = InformationManagers.getInstance().getModelManager().isRunModel();
		generateThumbnails = generateThumbnails && options.isSelected(StringConstants.INCLUDE_THUMBNAILS);
		generateThumbnails = generateThumbnails && !useExistingThumbnails; 
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.POSITIVE_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.NEGATIVE_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails || options.isSelected(StringConstants.FUNCTION_BASED_POLARITY_CAUSAL_LINKS);
		generateThumbnails = generateThumbnails && options.isSelected(StringConstants.INCLUDE_THUMBNAILS);
		// look head to turn off thumbnails if they are unavailable
		if (!InformationManagers.getInstance().getModelManager().isRunnable()) {
			//			options.setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", "false");
			InformationManagers.getInstance().getModelManager().setRunModel(false);
		} else {
			InformationManagers.getInstance().getModelManager().setRunModel(generateThumbnails);
		}

		

		
		System.out.println("isRunnable: "+InformationManagers.getInstance().getModelManager().isRunnable());
		System.out.println("runModel: "+InformationManagers.getInstance().getModelManager().isRunModel());
		System.out.println("useExistingClassFile: "+InformationManagers.getInstance().getReuse().isClassFile());
		System.out.println("useExistingThumbnails: "+InformationManagers.getInstance().getReuse().isThumbnails());
		System.out.println("generateThumbnails: "+generateThumbnails);
		System.out.println("batchExecution: "+batchExecution);
		
//		info.addMessage("OS: "+System.getProperty("os.name"));
		
		if (sdModel.isOk() && genDoc /*&& !sdmuiLoad*/) {
			sdModelDocumentor = new SDModelDocumentor(sdModel, info, options);
			sdModelDocumentor.setSdmuiMode(true);
			sdModelDocumentor.setSdmuiRootDirectory(rootDirectory);
			sdModelDocumentor.process(objectName);
//			sdModelDocumentor.processPolarity(objectName);
		}
		
		
		if ((!sdModel.isMap() /* && options.isSelected(StringConstants.INCLUDE_THUMBNAILS)*/ ) || sdmuiLoad) {
			boolean runnable = InformationManagers.getInstance().getModelManager().isRunnable();
			boolean isRunModel = InformationManagers.getInstance().getModelManager().isRunModel();
			
			
			if (InformationManagers.getInstance().getModelManager().isRunnable() && 
					(InformationManagers.getInstance().getModelManager().isRunModel() || batchExecution )) {
				
				// close any workbooks to avoid contention
				InformationManagers.getInstance().getExcelManager().closeAllWorkbooks();

				if (useExistingClassFile) {
					// runner
					
					// don't need to generate code, but we will not need to run the model is we are 
					// resusing thumbnails and link polarity but we will need to copy over the database
					
					if (reuse.isThumbnails() && reuse.isPolarity()) {
						String fromDir = InformationManagers.getInstance().getReuse().getDocGraphDirectory();
						String toDir = InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName();
						File fromF = new File(fromDir);
						File toF = new File(toDir);
						try {
							if (!fromF.getCanonicalPath().equals(toF.getCanonicalPath())) {
								
								try {
									
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.data"), 
												new File(toDir+"/runs/baselineHSQLDB.data"));
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.properties"), 
												new File(toDir+"/runs/baselineHSQLDB.properties"));
										FileUtils.copyFile(new File(fromDir+"/runs/baselineHSQLDB.script"), 
												new File(toDir+"/runs/baselineHSQLDB.script"));
									
								} catch (IOException e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}
							}
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					} else {

					SDModelRunner sdmRunner = new SDModelRunner();
					info.addMessage(SDMDocLanguageSpecific.getInstance().
							getText(StringConstants.EXECUTING_MODEL_TO_GENERATE_BASELINE_RUN));
					
					if (!sdmRunner.run(modelName, rootDirectory, "baseline", /*newMemoryDefinition,*/ generateThumbnails,
							fileOrganizer, info)) {
						info.addMessage(SDMDocLanguageSpecific.getInstance().
								getText(StringConstants.COULD_NOT_CREATE_THUMBNAILS));
						InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
					} else {
						InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(true);
					}
					}

				} else {

					if (lAndP && sdModel.isFullyDefined() && sdModel.isOk()) {
						info.addMessage(language.getText(StringConstants.BEGIN_PROCESSING_EQUATIONS));
						sdModelCodeGenerator = new SDModelCodeGenerator(sdModel, packageName, objectName, 
								rootDirectory, /* memoryImplementation,*/ javaSrcFile, info, /*newMemoryDefinition,*/
								fileOrganizer);
						List<String> messages = new ArrayList<String>();
						if (sdModelCodeGenerator.isValid(messages) && GEN_CODE) {
							codeGenerationFailure = sdModelCodeGenerator.process();
						} else {
							if (GEN_CODE) {
								info.addMessage("Model Equations Invalid -- Code not generated");;
								for (String m : messages) {
									info.addMessage(m);
								}
							}
						}
					}

					//			if (InformationManagers.getInstance().getModelManager().isRunnable()) {
					//				if (!sdModel.isMap() && options.isSelected(StringConstants.INCLUDE_THUMBNAILS) || sdmuiLoad) {
					if (!sdModel.isMap() && (InformationManagers.getInstance().getModelManager().isRunModel() /* && 
						!InformationManagers.getInstance().getModelManager().isXmileOrigin()*/) || sdmuiLoad || batchExecution) {

//						info.addMessage("Creating Thumbnails\n");
						info.setBlipMode(true);


						if (sdModelCodeGenerator.isOk() && sdModel.isFullyDefined() && sdModel.isOk()) {
							// compiler

							info.addMessage(language.getText(StringConstants.COMPILING_MODEL_MEMORY));
							boolean sdmCompilerMemoryOK = true;
							SDModelCompiler sdmCompilerMemory; 
							if (memoryImplementation.equals("Native")) {
								sdmCompilerMemory = new SDModelCompiler(javaMemoryCompileFile, info);
								sdmCompilerMemory.compile("./SDM-Doc.jar",javaMemorySrcFile);
								sdmCompilerMemoryOK = sdmCompilerMemory.isOk();
								if (sdmCompilerMemoryOK) {
									//								File f = new File(javaMemoryCompileFile);
									//								if (f.length() == 0l) {
									//									f.delete();
									//								}
								} else {
									InformationManagers.getInstance().getModelManager().
									setRunnable(false, language.getText(StringConstants.JAVA_COMPILE_ERRORS));
									info.addMessage(language.getText(StringConstants.JAVA_COMPILE_ERRORS));
									InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
								}
							}

							//						File thisDir = new File(".");
							//						for (String s : thisDir.list())
							//							System.out.println("FILE: "+s);

							SDModelCompiler sdmCompiler = new SDModelCompiler(javaCompileFile, info);
//							info.addMessage("OS: "+System.getProperty("os.name"));
							String separator = ":";
							if (SystemUtils.IS_OS_WINDOWS)
								separator = ";";
							info.addMessage(language.getText(StringConstants.COMPILING_MODEL_EQUATIONS));
							sdmCompiler.compile("./SDM-Doc.jar"+separator+fileOrganizer.getDynamicDirectory()+"/"+
									FileOrganizer.getSrcClassDirectory(),javaSrcFile);

							// runner
							boolean compiled = sdmCompiler.isOk();
							if (compiled) {
								if (compiled && sdmCompilerMemoryOK) {
									// lets copy source code into output directory for user
									try {
										FileUtils.copyFileToDirectory(new File(javaSrcFile), 
												new File(InformationManagers.getInstance().
														getFileOrganizer().getSrcGenDirectory()));
									} catch (IOException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									try {
										FileUtils.copyFileToDirectory(new File(javaMemorySrcFile), 
												new File(InformationManagers.getInstance().
														getFileOrganizer().getSrcGenDirectory()));
									} catch (IOException e1) {
										// TODO Auto-generated catch block
										e1.printStackTrace();
									}
									if (!batchExecution) {
										SDModelRunner sdmRunner = new SDModelRunner();
										
										info.addMessage(SDMDocLanguageSpecific.getInstance().
											getText(StringConstants.EXECUTING_MODEL_TO_GENERATE_BASELINE_RUN));
										if (!sdmRunner.run(modelName/*objectName*/, rootDirectory, "baseline", /*newMemoryDefinition, */ generateThumbnails,
												fileOrganizer, info)) {
											info.addMessage(SDMDocLanguageSpecific.getInstance().
													getText(StringConstants.COULD_NOT_CREATE_THUMBNAILS));
											InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
										} else {
											InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(true);
										}
									}
								}
							} else {
								InformationManagers.getInstance().getModelManager().setRunnable(false, 
										SDMDocLanguageSpecific.getInstance().getText(StringConstants.JAVA_COMPILE_ERRORS));
								info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.JAVA_COMPILE_ERRORS));
								InformationManagers.getInstance().getModelManager().setModelSuccessfullyExecuted(false);
							}
						}
					}
				}
				info.setBlipMode(false);
			} else {
				
					info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.MODEL_IS_NOT_RUNNABLE));
					info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.REASONS)+":\n"+
					InformationManagers.getInstance().getModelManager().getRunnableReason());
					
					boolean tooBig = true;
				
				// couldn't run model for some reason, but let's try to generate code
					if (!tooBig) {
				try {
					if (lAndP && sdModel.isFullyDefined() && sdModel.isOk()) {
						
						sdModelCodeGenerator = new SDModelCodeGenerator(sdModel, packageName, objectName, 
								rootDirectory, /* memoryImplementation, */ javaSrcFile, info, /* newMemoryDefinition, */
								fileOrganizer);
						List<String> messages = new ArrayList<String>();
						if (sdModelCodeGenerator.isValid(messages) && GEN_CODE) {
							info.addMessage(SDMDocLanguageSpecific.getInstance().getText(StringConstants.GENERATING_CODE));
							sdModelCodeGenerator.process();
						}
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
					}

//				if (options.isSelected(StringConstants.INCLUDE_THUMBNAILS)) {
//					String reason = InformationManagers.getInstance().getModelManager().getRunnableReason();
//					info.addMessage("Model Unrunable. Reason: \n"+reason+"\n");
//				} else {
//					info.addMessage("Model not run -- Thumbnails not requested\n");
//				}

				// need to turn off thumbnails if requested
				//				options.setValue(StringConstants.INCLUDE_THUMBNAILS, "boolean", "false");
				InformationManagers.getInstance().getModelManager().setRunModel(false);

			}
		}
		
		if (sdModel.isOk() && genDoc /*&& !sdmuiLoad*/) {
//			sdModelDocumentor.processLeea(objectName);   // 9/23/2019 for now until xmile stuff is working
			
//			sdModelDocumentor.processLeea(objectName);   // rogelio stuff
			
		}
			
			
			
			if (sdModel.isOk() && genDoc /*&& !sdmuiLoad*/) {
				sdModelDocumentor.generateHTML();
				info.addMessage(language.getText(StringConstants.HTML_REPORT_FILE_GENERATED));
				

			}
			
			// done with the sdm-doc database
			InformationManagers.getInstance().getDatabaseManager().shutdownDatabase();
			if (InformationManagers.getInstance() != null && 
					InformationManagers.getInstance().getAnalysisManager() != null && 
					InformationManagers.getInstance().getAnalysisManager().getLoopManager() != null)
				InformationManagers.getInstance().getAnalysisManager().getLoopManager().releaseLoopStorage();
		

	}
	
//    public static void main(String[] args) {
//    	
//    	SDModelProcessor sdmProc = new SDModelProcessor(args[0], args[1], args[2], args[3], args[4], args[5],
//    		new MessageGobbler(), new MessageGobbler(), new UserSelectedOptions(true), false, true);
//    	sdmProc.process(true);
//    }

	/**
	 * @return the sdModel
	 */
	public SDModel getSdModel() {
		return sdModel;
	}

	/**
	 * @return the sdModelCodeGenerator
	 */
	public SDModelCodeGenerator getSdModelCodeGenerator() {
		return sdModelCodeGenerator;
	}

//	/**
//	 * @return the destinationDirectory
//	 */
//	public String getDestinationDirectory() {
//		return destinationDirectory;
//	}

	/**
	 * @return the packageName
	 */
	public String getPackageName() {
		return packageName;
	}

	/**
	 * @return the objectName
	 */
	public String getObjectName() {
		return objectName;
	}

	/**
	 * @return the inputFileType
	 */
	public String getInputFileType() {
		return inputFileType;
	}
	
	protected String asDirectoryPath(String packageName) {
		return packageName.replace(".", "/");
	}

	public UserSelectedOptions getOptions() {
		return options;
	}

	public void setOptions(UserSelectedOptions options) {
		this.options = options;
	}

	public boolean isVariableNameTranslationMode() {
		return variableNameTranslationMode;
	}

	public void setVariableNameTranslationMode(boolean variableNameTranslationMode) {
		this.variableNameTranslationMode = variableNameTranslationMode;
	}

	public boolean isGenDoc() {
		return genDoc;
	}

	public void setGenDoc(boolean genDoc) {
		this.genDoc = genDoc;
	}


	public boolean isBatchExecution() {
		return batchExecution;
	}

	public void setBatchExecution(boolean batchExecution) {
		this.batchExecution = batchExecution;
	}

}
