package gov.anl.gss.sdm.server.model.tools;

import gov.anl.gss.sdm.server.model.documentor.MessageGobbler;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;

import java.util.ArrayList;
import java.util.List;

public class SdToJava {
	
	private static final boolean GEN_CODE = true;
	private static final boolean GEN_DOC = true;
	
	private SDModel sdModel;
	private SDModelCodeGenerator sdModelCodeGenerator;
	private SDModelDocumentor sdModelDocumentor;
	
//	protected String destinationDirectory;
	private String packageName;
	private String objectName;
	private String inputFileType;
	private String mdlFile;
	private String memoryImplementation;
	private String rootDirectory;
	
	public SdToJava(String inputFileType, String mdlFile, String packageName, String objectName, 
			String memoryImplementation, String rootDirectory) {
		
		this.inputFileType = inputFileType; 
		this.mdlFile = mdlFile;
		this.packageName = packageName;
		this.objectName = objectName;
		this.rootDirectory = rootDirectory;
		
		this.memoryImplementation = memoryImplementation;
	
	}
	
	public void process() {

		sdModel = new SDModel(mdlFile, inputFileType, objectName, rootDirectory, new FileOrganizer(false, rootDirectory, mdlFile), false, null);

		if (sdModel.loadAndProcess() && sdModel.isFullyDefined() && sdModel.isOk()) {
			sdModelCodeGenerator = new SDModelCodeGenerator(sdModel, packageName, objectName, rootDirectory, memoryImplementation,
					new MessageGobbler(), /*false, */  null);
			List<String> messages = new ArrayList<String>();
			if (sdModelCodeGenerator.isValid(messages) && GEN_CODE) {
				sdModelCodeGenerator.process();
			}
		}

		if (sdModel.isOk()) {
			sdModel.createModelVariablesFiles("", "", "");
		}
		
		if (sdModel.isOk() && GEN_DOC) {
			sdModelDocumentor = new SDModelDocumentor(sdModel, new MessageGobbler(), new UserSelectedOptions(true));
			sdModelDocumentor.setSdmuiMode(true);
			sdModelDocumentor.setSdmuiRootDirectory(rootDirectory);
			sdModelDocumentor.process(objectName);
			
		}
	}
	
    public static void main(String[] args) {
    	
    	SdToJava stj = new SdToJava(args[0], args[1], args[2], args[3], args[4], args[5]);
    	stj.process();
    }

	/**
	 * @return the sdModel
	 */
	public SDModel getSdModel() {
		return sdModel;
	}

	/**
	 * @return the sdModelCodeGenerator
	 */
	public SDModelCodeGenerator getSdModelCodeGenerator() {
		return sdModelCodeGenerator;
	}

//	/**
//	 * @return the destinationDirectory
//	 */
//	public String getDestinationDirectory() {
//		return destinationDirectory;
//	}

	/**
	 * @return the packageName
	 */
	public String getPackageName() {
		return packageName;
	}

	/**
	 * @return the objectName
	 */
	public String getObjectName() {
		return objectName;
	}

	/**
	 * @return the inputFileType
	 */
	public String getInputFileType() {
		return inputFileType;
	}

}
