package gov.anl.gss.sdm.server.model.translator;

/**
 * Used only for code generation of Native Data Types
 */


import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MemoryMapperInterface;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

/**
 * @author bragen
 *
 */
public class ArrayReferenceNative {
static Logger log = Logger.getLogger(ArrayReferenceNative.class.getSimpleName());

    private String arrayName;
    private List<String> subscripts;
    private Equation equation;
    private boolean lookup = false;
    private boolean vector = false;
    
    private String originalReference;
    
    
    public String getOriginalReference() {
		return originalReference;
	}

	public boolean isVector() {
		return vector;
	}

	public void setVector(boolean vector) {
		this.vector = vector;
	}

	public ArrayReferenceNative(String reference, Equation equation) {
    	if (reference.startsWith("lookup"))
    		lookup = true;
    	originalReference = reference;
	arrayName = extractArrayName(reference);
	subscripts = extractSubscripts(reference);
	this.equation = equation;
	
    }
    
    public ArrayReferenceNative(String reference) {
    	originalReference = reference;
	arrayName = extractArrayName(reference);
	subscripts = extractSubscriptsNative(reference);	
    }
    
    public ArrayReferenceNative(ArrayReference ar, Equation equation) {
    	originalReference = ar.getOriginalReference();
	arrayName = new String(ar.getArrayName());
	subscripts = new ArrayList<String> (ar.getSubscripts());
	this.equation = equation;
    }
    
    public String generateLHSHeader(String valueVariable) {
	
	StringBuilder code = new StringBuilder();
	EquationArrayReferenceStructure ears = equation.getEars();
//	code.append("// getOuter ArrayReferenceNative 50\n");
	code.append(ears.getOuterLoops());
	return code.toString();
	
//	StringBuilder code = new StringBuilder();
//	
//	// define the temporary arrays that we'll need
//	EquationArrayReferenceStructure ears = equation.getEars();
//	
//	
//	// move this code to ears.
//	// need to maintain info between ArrayRefs -> this is tied to ears within equation?
//	List<String> packedInfo = ears.getOuterArraySubscriptDimensionPacked();
//	
//	
//	Map<String, List<String>> uniqueBySubscript = new LinkedHashMap<String, List<String>>();
//	
//	// first determine if we need multiple index arrays
//	for (String packed : packedInfo) {
//	    String[] info = packed.split("###");
//	    if (!uniqueBySubscript.containsKey(info[1])) {
//		uniqueBySubscript.put(info[1], new ArrayList<String>());
//	    }
//	    List<String> bySybscript = uniqueBySubscript.get(info[1]);
//	    String indicies = ArrayManager.getIndicies(info[0], Integer.parseInt(info[2]), info[1]);
//	    if (!bySybscript.contains(indicies))
//		bySybscript.add(indicies);
//	}
//	
//	// create index arrays only when needed
//	for (String packed : packedInfo) {
//	    String[] info = packed.split("###");
//	    if (uniqueBySubscript.get(info[1]).size() > 1) {
//	    code.append("int[] indexArray_"+CodeGenerator.getLegalName(info[0]+"_"+info[1]+"_"+info[2])+"_ = new int[] {"+
//		    ArrayManager.getIndicies(info[0], Integer.parseInt(info[2]), info[1])+"};\n");
//	    }
//	}
//	
//
//	    int subNum = 0;
//	    for (String sub : subscripts) {
//		List<String> unique = uniqueBySubscript.get(sub);
//		if (unique.size() > 1) {
//		    code.append("for (int outer"+subNum+" = 0; outer"+subNum+" < "+NamedSubscriptManager.getNumIndexFor(sub)+"; outer"+subNum+"++) {\n");
//		} else {
//		    code.append("for (int outer"+subNum+" :  new int[] {"+
//		    ArrayManager.getIndicies(arrayName, subNum, sub)+"} {\n");
//		}
//		subNum++;
//	    }
//	
//	return code.toString(); 

    }
    
    public boolean hasRangeSubscript() {
	
	for (String sub : subscripts)
	    if (isRangeSubscript(sub))
		return true;
	return false;
    }
    
    public String generateLHSFooter(String valueVariable, /*boolean nativeMemoryImplementation,*/ /*boolean newMemoryDefinition,*/ boolean DEBUG_VALUES) {
    	StringBuilder code = new StringBuilder();

//    	if (nativeMemoryImplementation) {
    		// Assignment
    		EquationArrayReferenceStructure ears = equation.getEars();
//    		if (newMemoryDefinition) {
    			if (!equation.isVectorResult()) {
    				code.append(ears.getLHSassignmentVectorMemory());
    				code.append(" = "+valueVariable+";\n");
    			} else {
    				code.append(ears.getLHSassignmentVectorMemoryVector());
    				code.append(valueVariable+");\n");
    			}
//    		} else {
//    			code.append(ears.getLHSassignment());
//    			code.append(" = "+valueVariable+";\n");
//    		}
    		
    		
//    		if (!newMemoryDefinition) {
//
//    			// Logging
//    			//	code.append("/* generateLHSFooter */\n");
//    			code.append("/* log1 */logit(");
//    			code.append(ears.getLHSassignmentName());
//    			code.append(",");
//    			code.append("time,");
//    			code.append(valueVariable+",memory.get_SAVEPER());\n");
//    		}
    		
    		if (DEBUG_VALUES) {
    			
    			Node lhs = equation.getTreeRoot().getChild();
    			String assignment = ears.getLHSassignmentVectorMemory(); // InformationManagers.getInstance().getMemoryMapper().getDataStore(lhs.getToken())+"["+
    					// InformationManagers.getInstance().getMemoryMapper().getOffset(lhs.getToken()) +"]";
				code.append("debugValue(\""+assignment+"\", "+assignment+");\n");
			}

    		// closing braces
    		for (String sub : subscripts) {
    			code.append("}\n");
    		}
//    	} else {
//    		// Assignment
//    		EquationArrayReferenceStructure ears = equation.getEars();
//    		code.append("set("+ears.getLHSassignmentName());
//    		code.append(", "+valueVariable+");\n");
//
//    		// Logging
//    		//	code.append("/* generateLHSFooter */\n");
////    		code.append("/* log1 */logit(");
////    		code.append(ears.getLHSassignmentName());
////    		code.append(",");
////    		code.append("time,");
////    		code.append(valueVariable+",memory.get_SAVEPER());\n");
//
//    		// closing braces
//    		for (String sub : subscripts) {
//    			code.append("} // ARN\n");
//    		}
//    	}


    	return code.toString();
    }
    
    
    public String generateRHSImplementation() {
	StringBuilder code = new StringBuilder();
	// pop[race,state!]
	// generates arrayValueOf("pop",concatAsSubscript(outerSub.getSubscriptValue("race"), rangeSub.getSubscriptValue("state")))
	// generates range loops
	// generates pop[outer0][range0]
	
	EquationArrayReferenceStructure ears = equation.getEars();
	

	    code.append(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName));
	    
	    // need to process this is the same fashio as lhs so that we get outer0*12 + outer1 construct
//	    goober
		int subNum = 0;
		for (String sub : subscripts) {
		    if (isRangeSubscript(sub)) {
			code.append("["+equation.getEars().getRangeSubscriptForImplementation(arrayName, sub, subNum)+"]");
		    } else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript
			code.append("["+equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum)+"]");
		    } else {
			// This may to be referenced through an index array even if a terminal
			// this needs to be checked first. If not then check if terminal subscript

			if (equation.getEars().requiresIndirection(arrayName, sub, subNum)) {
			    code.append("["+equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum)+"]");
			} else {
			    if (InformationManagers.getInstance().getArrayManager().isTerminalSubscript(arrayName, sub, subNum)) {
				code.append("["+InformationManagers.getInstance().getArrayManager().getTerminalValue(arrayName, sub, subNum)+"]"); 
			    } else {
				code.append("\n// PROBLEM: "+arrayName+" "+sub+" "+subNum+"\n");
				code.append("["+sub.replace("!", "")+"]"); 
//				String makeError = null;
//				log.debug("// goober "+makeError.toLowerCase());
			    }
			}
		    }
		    subNum++;
		   
		}
	
	return code.toString();
}
    
    public String getVectorReference() {
    	StringBuilder code = new StringBuilder();
    	MemoryMapperInterface memoryMapper = InformationManagers.getInstance().getMemoryMapper();
    	String originalLHS = equation.getOriginalLhs();
    	String offset = memoryMapper.getOffset(originalLHS);
//    	code.append("vectorMemory["+offset);
    	code.append(memoryMapper.getDataStore(originalLHS)+"["+offset);
    	
    	// form: X[a,b,c] = a*size(b)*size(c) + b*size(c) + c;
    	
    	int numSubscripts = getSubscripts().size();
    	
    	for (int dim = 0; dim < numSubscripts; dim++) {
    		
    		// if we are not working with the last subscript, then we must multiply by the sizes
    		// of the rest of the subscripts
    		
    		// if 0 < 2
    		if (dim < numSubscripts -1 ) {
    			code.append("+outer"+dim);
    			int i = 1;
    			// while 1, 2 < 3
    			while (dim+i < numSubscripts) {
    				String sub1 = getSubscripts().get(dim+i);
    				String data = InformationManagers.getInstance().getArrayManager().
    						getIndiciesSorted(getArrayName(), dim+i, sub1);
    				int num = data.split(",").length;

    				// more dimensions to come, sho need to multiply by next subs size
    				code.append("*"+num);
    				i++;
    			}
    		} else {
				code.append("+outer"+dim);
			}
    	}
    	
    	code.append("]");
    	return code.toString();
    }
    
    public String generateRHSImplementationVectorMemory() {
	StringBuilder code = new StringBuilder();
	
	EquationArrayReferenceStructure ears = equation.getEars();
		// should this just be the arrayName?

		String offset = InformationManagers.getInstance().getMemoryMapper().getOffset(originalReference);
		
		// We can have two possible implementations:
		// 1) Simple scalar address as in: vectorMemory[2648+outer0* 12+outer1]
		// 2) or a vector: genVector(vectorMemory, 2648 +outer0*12, outer1.length)
		//
		
		if (vector) {
//			code.append("genVector(vectorMemory, "+offset);
			code.append("genVector("+InformationManagers.getInstance().getMemoryMapper().getDataStore(originalReference)+", "+offset);
		} else {
//			code.append("vectorMemory["+offset);
			code.append(InformationManagers.getInstance().getMemoryMapper().getDataStore(originalReference)+"["+offset);
		}
	    
		int subNum = 0;
		int numSubscripts = subscripts.size();
    	
    	for (int dim = 0; dim < numSubscripts; dim++) {
		
			boolean includeSubrName = true;
			boolean lastSubr = subNum == subscripts.size()-1;
			StringBuilder timesSize = new StringBuilder();
			String sub = subscripts.get(dim);
			String sub1;
			String data;
			int num;
			
			if (dim < subscripts.size()-1) {
//				code.append("+outer"+dim);
//				sub1 = subscripts.get(dim+1);
//				data = InformationManagers.getInstance().getArrayManager().getIndiciesSorted(getArrayName(), dim+1, sub1);
//				int num = data.split(",").length;
//				timesSize = "*"+num;
				int i = 1;
				while (dim+i < numSubscripts) {
    				sub1 = getSubscripts().get(dim+i);
    				data = InformationManagers.getInstance().getArrayManager().
    						getIndiciesSorted(getArrayName(), dim+i, sub1);
    				num = data.split(",").length;

    				// more dimensions to come, sho need to multiply by next subs size
    				timesSize.append("*"+num);
    				i++;
    			}
			} else {  // this is only for vector operations
				sub1 = subscripts.get(subNum);
				data = InformationManagers.getInstance().getArrayManager().getIndiciesSorted(getArrayName(), dim, sub1);
				num = data.split(",").length;
				if (vector) {
					timesSize.append(Integer.toString(num));
					includeSubrName = false;
				} else {
					timesSize.setLength(0);
				}
			}
			if (isRangeSubscript(sub)) {
				if (vector) {
					if (lastSubr) {
						// this is end argument
						code.append(", "+(includeSubrName ? equation.getEars().getRangeSubscriptForImplementation(arrayName, sub, subNum) : "") +timesSize.toString());
					} else {
						code.append("+"+(includeSubrName ? equation.getEars().getRangeSubscriptForImplementation(arrayName, sub, subNum) : "")+timesSize.toString());
					}
				
				} else {
					code.append("+"+equation.getEars().getRangeSubscriptForImplementation(arrayName, sub, subNum)+timesSize.toString());
				}
			} else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) {
				if (vector) {
					if (lastSubr) {
						code.append(", "+(includeSubrName ? equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum) : "")+timesSize.toString());
					} else {
						code.append("+"+(includeSubrName ? equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum) : "")+timesSize.toString());
					}
				} else {
					code.append("+"+equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum)+timesSize.toString());
				}
			} else {
				// This may to be referenced through an index array even if a terminal
				// this needs to be checked first. If not then check if terminal subscript

				if (equation.getEars().requiresIndirection(arrayName, sub, subNum)) {
					code.append("+("+equation.getEars().getSubscriptForImplementation(arrayName, sub, subNum)+timesSize.toString()+")");
				} else {
					if (InformationManagers.getInstance().getArrayManager().isTerminalSubscript(arrayName, sub, subNum)) {
						code.append("+"+InformationManagers.getInstance().getArrayManager().getTerminalValue(arrayName, sub, subNum)); 
					} else {
						code.append("\n// PROBLEM: "+arrayName+" "+sub+" "+subNum+"\n");
						code.append("["+sub.replace("!", "")+"]"); 
						//				String makeError = null;
						//				log.debug("// goober "+makeError.toLowerCase());
					}
				}
			}
			subNum++;

		}
		if (vector) {
			code.append(")");
		} else {
			code.append("]");
		}
	
	return code.toString();
}
    
    public String asMapGet() {

    	if (lookup)
    		return "getLookup("+generateRHSOriginalName()+")";
    	else if (vector)
    		return "getVector("+generateRHSOriginalName()+")";
    	else
    		return "get("+generateRHSOriginalName()+")";
}
    
    public String asMapName() {
    	
	return generateRHSOriginalName();
}
    
    public String generateRHSOriginalName() {
    	
    	
	StringBuilder code = new StringBuilder();
	// pop[race,state!]
	// generates arrayValueOf("pop",concatAsSubscript(outerSub.getSubscriptValue("race"), rangeSub.getSubscriptValue("state")))
	// generates range loops
	// generates pop[outer0][range0]
//	code.append("/* origName  */ ");
	code.append("asExpandedArrayRef(");
	code.append("\"");
//	    code.append(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName));
	    code.append(InformationManagers.getInstance().getNativeDataTypeManager().
	    		getOriginalName(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName)));
	    code.append("\"");
	    
		int dimension = 0;
		for (String sub : subscripts) {
		    if (isRangeSubscript(sub)) {
		    	code.append(",range"+equation.getEars().getRangeSubscriptNumber(sub));
		    } else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript
		    	code.append(",outer"+equation.getEars().getOuterSubscriptNumber(sub));
		    } else {
		    	String index = InformationManagers.getInstance().getArrayManager().getNumericIndexForSubscript(arrayName, dimension, sub);
		    	code.append(","+index);   // need a lookup to determine terminal value!
		    }
		    dimension++;
		   
		}
		code.append(")");
	
//		log.debug("RHS Name: "+code.toString());
	return code.toString();
    }
    
    public String generateRHSName() {
	
	
	StringBuilder code = new StringBuilder();
	// pop[race,state!]
	// generates arrayValueOf("pop",concatAsSubscript(outerSub.getSubscriptValue("race"), rangeSub.getSubscriptValue("state")))
	// generates range loops
	// generates pop[outer0][range0]
//	code.append("/* generateRHSName  */ ");
//	if (!equation.isNativeMemoryImplementation()) {
//	code.append("asExpandedArrayRef(");
//	code.append("\"");
//	    code.append(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName));
////	    code.append(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName)));
//	    code.append("\"");
//	    int dimension = 0;
//		for (String sub : subscripts) {
//		    if (isRangeSubscript(sub)) {
//			code.append("range"+equation.getEars().getRangeSubscriptNumber(sub));
//		    } else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript
//			code.append("outer"+equation.getEars().getOuterSubscriptNumber(sub));
//		    } else {
//			
//			String index = InformationManagers.getInstance().getArrayManager().getNumericIndexForSubscript(arrayName, dimension, sub);
//	    	code.append(","+index);   // need a lookup to determine terminal value!
//		    }
//		    dimension++;
//		}
//		code.append(")");
//	} else {
		// stringConcat("investment effectiveness1","[",intToString(outer0),"]")
		code.append("stringConcat(");
		code.append("\"");
		    code.append(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName));
		    code.append("\",\"[\"");
		    int dimension = 0;
			for (String sub : subscripts) {
			    if (isRangeSubscript(sub)) {
			    	code.append("range"+equation.getEars().getRangeSubscriptNumber(sub));
			    } else if (InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(sub)) { // if not a named subscript outer0
			    	code.append(",intToString(");
			    	code.append("outer"+equation.getEars().getOuterSubscriptNumber(sub));
			    	code.append(")");
			    } else {
			    	String index = InformationManagers.getInstance().getArrayManager().getNumericIndexForSubscript(arrayName, dimension, sub);
			    	code.append(","+index);   // need a lookup to determine terminal value!
			    }
			    dimension++;
			}
			code.append(",\"]\")");
//	}
	
//		log.debug("RHS Name: "+code.toString());
	return code.toString();
    }
    
    public static boolean isArrayReference(String token) {
	if (token.startsWith("array.") && token.contains("[") && token.contains("]") ||
		token.contains("[") && token.contains("]"))	
	    return true;
	else
	    return false;
    }
    
    public static boolean isRangeSubscript(String subscript) {
	if (subscript.endsWith("!"))
	    return true;
	else
	    return false;
    }
    
    private String extractArrayName(String reference) {
	return reference.split("\\[")[0].replace("array.","").replace("lookup.", "").trim();
    }
    
    private List<String> extractSubscriptsNative(String reference) {
	
	List<String> subscripts = new ArrayList<String>();
	String[] subs = reference.split("\\[");
	String sub = "";
	for (int i = 1; i < subs.length; i++) {
	    sub += "["+subs[i];
	}
	subscripts.add(sub.trim());
	return subscripts;
    }
    
    private List<String> extractSubscripts(String reference) {
	
	List<String> subscripts = new ArrayList<String>();
	for (String sub : reference.split("\\[")[1].split("]")[0].split(","))
	    subscripts.add(sub.trim());
	return subscripts;
    }
    
    public String[] getNonRangeSubscriptsAsArray() {
	List<String> al = getNonRangeSubscripts();
	return al.toArray(new String[al.size()]);
    }
    
    public List<String> getNonRangeSubscripts() {
	List<String> ranges = new ArrayList<String>();
	for (String sub : subscripts)
	    if (!sub.contains("!"))
		ranges.add(sub);
	
	return ranges;
    }
    
    public String[] getRangeSubscriptsAsArray() {
	List<String> al = getRangeSubscripts();
	return al.toArray(new String[al.size()]);
    }
    
    public List<String> getRangeSubscripts() {
	List<String> ranges = new ArrayList<String>();
	for (String sub : subscripts)
	    if (sub.contains("!"))
		ranges.add(sub);
	
	return ranges;
    }
    
    public String[] getRangeSubscriptsNamesAsArray() {
	List<String> al = getRangeSubscriptsNames();
	return al.toArray(new String[al.size()]);
    }
    
    public List<String> getRangeSubscriptsNames() {
	List<String> ranges = new ArrayList<String>();
	for (String sub : subscripts)
	    if (sub.contains("!"))
		ranges.add(sub.replace("[", "").replace("]", "").replace("!", ""));
	
	return ranges;
    }
    
    public String getSubscriptsAsMethodParameters() {
	StringBuilder sb = new StringBuilder();
	for (String sub : subscripts) {
	    if (sb.length() > 0)
		sb.append(",");
	    sb.append("\""+sub+"\"");
	}
	return sb.toString();
    }

    public String getArrayName() {
        return arrayName;
    }

    public void setArrayName(String arrayName) {
        this.arrayName = arrayName;
    }

    public List<String> getSubscripts() {
        return subscripts;
    }
    
    public String[] getSubscriptsAsArray() {
	String[] s = new String[subscripts.size()];
	int i = 0;
	for (String sub : subscripts)
	    s[i++] = sub;
        return s;
    }

    public void setSubscripts(List<String> subscripts) {
        this.subscripts = subscripts;
    }
    
//    public Set<String> expand() {
//	Set<String> al = new LinkedHashSet<String>();
//	
//	for (SubscriptCombination sc : ArrayManager.getSubscriptValueCombinations(this.getSubscriptsAsArray())) {
////	    al.add(arrayName+ "["+sc.getSubscriptValue()+"]");
//	    
//	    al.add("array."+arrayName+ "["+sc.getSubscriptValue()+"]");
//	}
//	
//	return al;
//    }
//    
//    public Set<String> expandRawSubscript() {
//	Set<String> al = new LinkedHashSet<String>();
//	
//	for (SubscriptCombination sc : ArrayManager.getSubscriptValueCombinations(this.getSubscriptsAsArray())) {
////	    al.add(arrayName+ "["+sc.getSubscriptValue()+"]");
//	    
//	    al.add("["+sc.getSubscriptValue()+"]");
//	}
//	
//	return al;
//    }
    
    public String getSubscript() {
	StringBuilder sb = new StringBuilder();
	int i = 0;
	for (String s : subscripts) {
	    if (i++ > 0)
		sb.append(",");
	    sb.append(s);
	}
	
	
	return sb.toString();
    }
}
