package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.gui.MessageFrame;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MemoryMapper;
import gov.anl.gss.sdm.server.model.support.MemoryMapperInterface;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CodeGenerator {
	
	public static final boolean DEBUG_VALUES = false;

	private static int EQUATION_LIMIT = 100; // 100;
	private int currentEquationNumber = 0;
	private int currentMethodNumber;
	
	private int errorMessageLocation = 40;

	private static Set<String> indexArraysPacked = new LinkedHashSet<String>();

	private static boolean flatten = true; // true
	private static boolean useRSParameters = false;
//	private String target; // one of "Java" or "C"
//	private String memoryImplementation; // one of "Native" or "HashMap"  boolean nativeMemory()
//	private boolean repastSimphony = false; // isRepastSimphony()
//	private boolean rsHybrid = false; // isRsHybrid()
	private boolean includeComments = true; // true
	private boolean generateRunner = true;  // false
	
	private boolean codeGenerationFailure = false;
	
//	private boolean newMemoryDefinition;

	private List<String> evaluationOrderOneTime;
	private List<String> evaluationOrderRepeated;
	private Map<String, Equation> equations;

	private BufferedWriter sourceCode;

	private String start = "INITIAL_TIME";
	private String end = "FINAL_TIME";
	private String step = "TIME_STEP";

	private String tStep = "1.0";

	private int doubleUsed = 0;
	private int intUsed = 0;

	private int booleanUsed = 0;

	private int stringUsed = 0;

	private int maxTempInt = -1;
	private int maxTemp = -1;
	private int maxBoolean = -1;
	private int maxString = -1;

	private int nextValueVariable = 0;

	private String objectName;

	private String srcDir;
	private String packageName;
	
	private FileOrganizer fileOrganizer;


	// configuration for generated code




	private Set<String> outerSubscripts = new LinkedHashSet<String>();

	private Equation currentGenerate;

	private boolean currentHasLhsArrayReference = false;
	
	private MemoryMapperInterface memoryMapper;

	public CodeGenerator(String srcDir, 
			List<String> evaluationOrderOneTime, 
			List<String> evaluationOrderRepeated, 
			Map<String, Equation> equations, 
			String objectName, 
			String packageName,
//			String target,
//			String memoryImplementation,
//			boolean newMemoryDefinition,
			FileOrganizer fileOrganizer) {
		this.srcDir = srcDir;
		this.evaluationOrderOneTime = evaluationOrderOneTime;
		this.evaluationOrderRepeated = evaluationOrderRepeated;
		this.equations = equations;
		this.objectName = objectName;
//		this.target = target;
		this.packageName = packageName;
//		this.memoryImplementation = memoryImplementation;
//		this.newMemoryDefinition = newMemoryDefinition;
		this.fileOrganizer= fileOrganizer;
		memoryMapper = InformationManagers.getInstance().getMemoryMapper();

		if (!equations.containsKey("INTITIAL_TIME")) {
			if (equations.containsKey("INITIALTIME"))
				start = "INITIALTIME";
			else
				start = "nonexistant";
		}
		if (!equations.containsKey("FINAL_TIME")) {
			if (equations.containsKey("FINALTIME"))
				end = "FINALTIME";
			else
				end = "nonexistant";
		}
		if (!equations.containsKey("TIME_STEP")) {
			if (equations.containsKey("TIMESTEP"))
				step = "TIMESTEP";
			else
				step = "TIME_STEP";
		}
	}


	public void openFile() {
//		if (isGenerateJava()) {
//			sourceCode = Utilities.openFileForWriting(srcDir+objectName+".java");
//			sourceCode = Utilities.openFileForWritingCharset(srcDir+objectName+".java", SDMDocLanguageSpecific.getInstance().getFileCharSet());
			sourceCode = Utilities.openFileForWritingUTF8(srcDir+objectName+".java");
//		} else if (isGenerateC()) {
//			sourceCode = Utilities.openFileForWriting(srcDir+objectName+".c");
//		} else {
//			sourceCode = null;
//		}
	}

	public void closeFile() {
		try {
			sourceCode.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public boolean generateCode(ActiveInitialProcessor aip) {

		//	if (translator.isUnitsConsistency())
		//	    performUnitsConsistencyCheck();

//		if (isGenerateC() || isGenerateJava()) {
			openFile();


			// create the "runner" class for this model
			if (/* isGenerateJava() && */ generateRunner) {
				runner(srcDir);
			}

			// create the header code for the model
			// includes imports and constructor
//			if (isGenerateJava()) {
				startObject(sourceCode);
//			} else {
//				startObjectC(sourceCode);
//				InformationManagers.getInstance().getNativeDataTypeManager().generateArrayDeclarationC(sourceCode);	
//			}


			// write the oneTime method that is used to initial the model components
			codeGenerationFailure = onetime(sourceCode) || codeGenerationFailure;

			boolean notYet = false;
			// do required initializtion iff there is an ACTIVE INITIAL used
			if (aip != null && !notYet) {
				aip.write(this, sourceCode);
			}


			// generate the methods that are called repeatedly as time progresses
			codeGenerationFailure = repeated(sourceCode) || codeGenerationFailure;

			// update any time series values
//			if (isGenerateJava()) {
				timeSeriesReferences(sourceCode);
//			}

			// finally close out the object
//			if (isGenerateJava()) 
				endObject(sourceCode);
//			else
//				endObjectC(sourceCode);

			closeFile();
//		}
		return codeGenerationFailure;
	}

	private void startObject(BufferedWriter bw) {

		// create the header information for the object

		try {

			bw.append("package "+packageName+";\n\n");

			// imports
			bw.append("// imports\n\n");

			// to facilitate switching between JavaScript and Java implementations, we have
			// refactored the code to contain language specific classes
//			if (isRepastSimphony()) {
//				bw.append("import "+ReaderConstants.SUPPORT+".SDModelWithPropertiesVDM_Native"+";\n");
//			
//				bw.append("import repast.simphony.engine.schedule.ISchedule;\n");
//				bw.append("import repast.simphony.engine.schedule.ScheduledMethod;\n");
//				bw.append("import repast.simphony.engine.environment.RunEnvironment;\n");
//				bw.append("import repast.simphony.parameter.Parameters;\n\n");
//				bw.append("import repast.simphony.systemdynamics.bridge.SDMUIExecutable;\n");
//				bw.append("import "+ReaderConstants.SUPPORT+".Utilities"+";\n\n");
//			} else {

				bw.append("import gov.anl.gss.sdm.server.model.support"+".SDModelWithPropertiesVDM_Native"+";\n");
				bw.append("import gov.anl.gss.sdm.server.model.bridge.SDMUIExecutable;\n");
				bw.append("import gov.anl.gss.sdm.server.model.support"+".TimeSeriesInstance"+";\n");
				bw.append("import gov.anl.gss.sdm.server.model.support"+".TimeSeriesData"+";\n");
				bw.append("import gov.anl.gss.sdm.server.model.support"+".Utilities"+";\n");
				
//				if (nativeMemory()) {
					bw.append("import gov.anl.gss.sdm.server.simulation.models.Memory"+objectName+";\n\n");
//					if (newMemoryDefinition) {
						bw.append("import gov.anl.gss.sdm.server.model.support"+".ExtendedMemoryDatabase"+";\n");
//					}
//				} else {
//					bw.append("\n");
//				}
//			}
			// constructor
			bw.append("public class "+objectName+" extends SDModelWithPropertiesVDM_Native implements SDMUIExecutable {\n\n");

//			if (nativeMemory()) {
				bw.append("private Memory"+objectName+" memory;\n");
//			} 
			bw.append("public "+objectName+"(String name, String[] args) {\n");
			bw.append("super(name, args);\n");
			
//			if (target.equals(ReaderConstants.JAVA)) {
				String value;
//				if (nativeMemory()) {
//					if (newMemoryDefinition) {
						int vectorSize = memoryMapper.getNumElements();
//						bw.append("vectorMemory = new double["+vectorSize+"];\n");
						bw.append("vMem = new double["+memoryMapper.getNumElementsForType(MemoryMapper.DYNAMIC)+"];\n");
//						bw.append("cMem = new double["+memoryMapper.getNumElementsForType(MemoryMapper.CONSTANT)+"];\n");
						bw.append("initializeConstantsMemory("+memoryMapper.getNumElementsForType(MemoryMapper.CONSTANT)+");\n");
//					}
					
					bw.append("memory = new Memory"+objectName+"();\n\n\n");
					String mapperFile = fileOrganizer.getMapperFile();
					String extendedMemoryFile = fileOrganizer.getRunsDirectory()+"/baselineHSQLDB";
					value = equations.get("TIME STEP").getRHSConstantValue();
//					if (newMemoryDefinition) {
//						bw.append("extendedMemory = new ExtendedMemoryDatabase("+
//								"\""+mapperFile+"\"," +
//								"\""+extendedMemoryFile+"\","+
//								value+
//								");\n");
//					}
//				} 

				value = equations.get("SAVEPER").getRHSConstantValue();
				if (value != null ) {
					bw.append("saveper = "+value+";\n");
				} else {
					// if no saveper, it should be checked in vensim checkbox to use time step
					value = equations.get("TIME STEP").getRHSConstantValue();
					if (value != null) {
						bw.append("saveper = "+value+";\n");
					} else {
						// force a compilation error in java
						// should set a flag that code generation failed
						bw.append("saveper = "+value+";\n");
					}
				}

				value = equations.get("TIME STEP").getRHSConstantValue();
				bw.append("dt = "+value+";\n");

				value = equations.get("INITIAL TIME").getRHSConstantValue();
				bw.append("initialTime = "+value+";\n");
				
//				if (newMemoryDefinition) {
					String offset = memoryMapper.getOffset("Time");
//					bw.append("   vectorMemory["+offset+"] = "+value+";\n");
					bw.append("   vMem["+offset+"] = "+value+";\n");
//				}
				

				value = equations.get("FINAL TIME").getRHSConstantValue();
				bw.append("finalTime = "+value+";\n");
				bw.append("currentTime = initialTime;\n");
				bw.append("set(\"Time\", initialTime);\n");
				bw.append("set(\"TIME STEP\", dt);\n");
				bw.append("set(\"FINAL TIME\", finalTime);\n");
				bw.append("set(\"INITIAL TIME\", initialTime);\n");
				bw.append("set(\"SAVEPER\", saveper);\n");
//				bw.append("logit(\"Variable\", \"Time\", \"Value\");\n");
				
				
				
//			}

			bw.append("}\n\n");
//			bw.append("public "+objectName+"(String name, String[] args) {\n");
//			bw.append("super(name, args);\n\n");

			// perform some initialization based on language

			

//			bw.append("}\n\n");
			bw.flush();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void startObjectC(BufferedWriter bw) {

		// create the header information for the object

		try {
			bw.append("#include <stdio.h>\n");
			bw.append("#include <stdlib.h>\n");
			bw.append("#include <stdarg.h>\n");
			bw.append("#include <stdbool.h>\n");
			bw.append("#include <string.h>\n\n");
			bw.append("#include <float.h>\n\n");
			bw.append("#include <math.h>\n\n");
			//	    bw.append("#include <support.h>\n\n");
			bw.append("#include \"support.h\"\n\n");


			bw.append("#include \"vensimSupport.h\"\n\n");
			bw.append("#include \"memory"+objectName+".h\"\n");

			bw.append("void repeated(double time, double timeStep);\n");

			if (ActiveInitialProcessor.requiresProcessing(equations))
				bw.append("void initialize();\n");

			//	    bw.append("int main(void) {\n");
			//	    bw.append("}\n\n");

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private boolean onetime(BufferedWriter bw) {

		Map<String, String> initialValues = new LinkedHashMap<String, String>();
		String indent = "    ";
		String statement = null;
		
		Set<String> processed = new LinkedHashSet<String>();
		
		try {
//			if (isGenerateJava()) {
				bw.append("public ");
//			}
			bw.append("void oneTime0() {\n\n");
			bw.append(" System.out.println(\"oneTime\");\n");
			bw.append("timeStep = getTIMESTEP();\n");
			bw.append("time = getINITIALTIME();\n");

//			if (isGenerateJava() && isRepastSimphony()) {
//				bw.append("Parameters params = RunEnvironment.getInstance().getParameters();\n");
//			}


			// NOTE: only "2" and "3" seem to be used, Need to figure out this "GAME" thing
			for (String lhs : evaluationOrderOneTime) {
				
				

				if (lhs.equalsIgnoreCase("time") || lhs.equalsIgnoreCase("timestep"))
					continue;
				Equation equationForThisLHS = InformationManagers.getInstance().getEquationManager().getEquationFor(lhs); // InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs);
				
				if (equationForThisLHS == null) {
//					System.out.println("NO EQUATION FOR LHS: "+lhs+" IS THIS FROM AN ARTIFICIAL EQUATION FOR ORDERING?");
					continue;
//					equationForThisLHS = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs);
//					if (equationForThisLHS == null)
//						System.out.println("How can this be? "+lhs);
				}
				

				processed.add(lhs);
				String debugStatement = "";
				
//				if (equationForThisLHS.getVensimEquationOnly().contains("XLS CONSTANTS") ||
//					equationForThisLHS.getVensimEquationOnly().contains("XLS LOOKUP"))
//					System.out.println("GET XLS");
				
				// no matter what the equation says, this is the order to use!
//				if (equations.get(lhs).isOneTime()) { 
					currentEquationNumber++;
					if (currentEquationNumber > EQUATION_LIMIT) {
						currentEquationNumber = 0;
						currentMethodNumber++;
						resetLimits();
						bw.append("}\n\n");
//						if (isGenerateJava())
							bw.append("protected ");
						bw.append("void oneTime"+currentMethodNumber+"() {\n\n");
						bw.append(" timeStep = getTIMESTEP();\n");
						bw.append(" time = getINITIALTIME();\n");
						//			 bw.append("getINITIALTIME() + (getTickCount() - 1.0) * timeStep;\n");
//						if (isGenerateJava() && isRepastSimphony())
//							bw.append("Parameters params = RunEnvironment.getInstance().getParameters();\n");
					} if (!/*InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs)*/equationForThisLHS.isOneTime() || 
							equationForThisLHS.isGame() || 
							equationForThisLHS.isUsesInitialFunction()) {
						generateCode(equationForThisLHS, /*InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs),*/ bw, DEBUG_VALUES);
					} else if (/*InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */ equationForThisLHS.getCleanEquation().contains("=")) {

						// Need to deal with arrays
						if (/* InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */ equationForThisLHS.isHasLHSArrayReference()  || 
								/* InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */ equationForThisLHS.isUsesTimeSeries() ||
								equationForThisLHS.isDefinesLookupGetXls()) {
							// TODO: test that this is indeed a initializer 
							if (includeComments)
								bw.append(equationForThisLHS.getUnitsAndComment());
							bw.append("{\n");
							bw.append(/*scrub( *//* InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */ 
									equationForThisLHS.generateArrayConstantsInitialization(/*nativeMemory()*/ /*, 
									newMemoryDefinition*/ ))/*)*/;
							bw.append("}\n");
						} else {
							if (includeComments)
								bw.append(equationForThisLHS.getUnitsAndComment());
							String[] bothSides = /* InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */ equationForThisLHS.getCleanEquation().split("=", 2);
							if (bothSides[1].startsWith("sdFunctions.GETXLS")) {
								ModelManager mm = InformationManagers.getInstance().getModelManager();
								FileOrganizer fo = InformationManagers.getInstance().getFileOrganizer();
								bothSides[1] = bothSides[1].replace("(\"", "(\""+fo.getRelativeToApplicationDirectory(mm.getModelDirectory())+"/");
							}
							if (InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]).contains(step))
								tStep = forceDouble(bothSides[1]);

//							if (!isGenerateJava()) {
//								statement = indent+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0])+" = "+forceDouble(bothSides[1])+"; // 2;\n" +
//										"/* A */ logit(\""+bothSides[0]+"\", 0.0,"+forceDouble(bothSides[1])+",memory.get_SAVEPER());\n"; // 2\n";
//							} else {
								//				    bw.append("/* oneTime */\n");
//								if (isRepastSimphony()) {
////									if (nativeMemory()) {
//										statement = indent+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0])+
//												" = (Double) params.getValue(\""+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]).replace("memory.", "")+
//												"\",memory.get_SAVEPER()); // 2;\n" +
//												"/* B */ logit(\""+bothSides[0]+"\", getINITIALTIME(), (Double) params.getValue(\""+
//												InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]).replace("memory.", "")+"\"),memory.get_SAVEPER());\n"; // 2\n";
////									} else {
////										statement = indent+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0])+
////												" = (Double) params.getValue(\""+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]).replace("memory.", "")+
////												"\",memory.get_SAVEPER()); // 2;\n" +
////												"/* C */ logit(\""+bothSides[0]+"\", getINITIALTIME(), (Double) params.getValue(\""+
////												InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]).replace("memory.", "")+"\"),memory.get_SAVEPER());\n"; // 2\n";
////									}
//								} else {

//									if (nativeMemory()) {
//										if (newMemoryDefinition) {
											String offset = memoryMapper.getOffset(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]));
//											statement = indent + "vectorMemory["+offset+"]" +""+" = "+forceDouble(bothSides[1])+"; // 2a;\n";
//											statement = indent + memoryMapper.getDataStore(bothSides[0])+"["+offset+"]" +""+" = "+forceDouble(bothSides[1])+"; // 2a;\n";
											statement = "initializeConstant("+offset+" , "+forceDouble(bothSides[1])+")";
											debugStatement = "\"" +"initializeConstant("+offset+" , "+forceDouble(bothSides[1])+")\"";
											
//										} else {
//										statement = indent+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0])+""+" = "+forceDouble(bothSides[1])+"; // 2;\n" +
//												"/* A */ logit(\""+bothSides[0]+"\", 0.0,"+forceDouble(bothSides[1])+",memory.get_SAVEPER());\n"; // 2\n";
//										}
//									} else {
//										statement = indent+"set(\""+InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(bothSides[0])+"\" , "+forceDouble(bothSides[1])+"); // 2;\n";
//										//    											+ "log(\""+bothSides[0]+"\", 0.0,"+forceDouble(bothSides[1])+");\n"; // 2\n";
//									}
//								}
//							}

							initialValues.put(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(bothSides[0]), forceDouble(bothSides[1]));
							String scrubbed = statement; // scrub(statement);
							bw.append(scrubbed+";\n");
							String s = "debugValue("+debugStatement+", "+scrubbed+");\n";
							if (DEBUG_VALUES) {
								bw.append(s);
							}
						}
					} else if (equationForThisLHS.isDefinesLookup()){

						String lhSide = equationForThisLHS.getLhs();
						if (ArrayReference.isArrayReference(lhSide)) {
							// need to get the assigned index for this
							ArrayReference ar = new ArrayReference(lhSide);
							//				getTerminalValue(String arrayName, String subscriptName, int dimension)
//							if (nativeMemory()) {
								statement = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhSide) + 
										"["+InformationManagers.getInstance().getArrayManager().getTerminalValue(ar.getArrayName(), ar.getSubscripts().get(0), 0)+ "]" 
										+ " = " + equationForThisLHS.getCleanEquation()+"; // 3\n";
//							} else {
//								statement = "set(\""+InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhSide) + 
//										"["+InformationManagers.getInstance().getArrayManager().getTerminalValue(ar.getArrayName(), ar.getSubscripts().get(0), 0)+ "]" 
//										+ "\", " + equationForThisLHS.getCleanEquation()+"); // 3\n";
//							}
						} else {
							// if this is a lookup, it must be a scalar, but we have marked at array. for other processing
							// want both storeMatrix and as previous in array
//							if (nativeMemory()) {
//								if (this.newMemoryDefinition) {
//									statement = "storeMatrix(vectorMemory, ";
//									statement += memoryMapper.getOffset(lhSide)+", ";
//									statement += eqn.getCleanEquation()+"); // 3n\n";
									statement = "";
//								} 
									statement += InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhSide.replace("array.", "")) + " = " + equationForThisLHS.getCleanEquation()+"; // 3\n" ;
								
//							} else {
//								statement = "set(\""+InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhSide.replace("array.", "")) + "\", " + equationForThisLHS.getCleanEquation()+"); // 3\n";
//							}
						}
						if (includeComments)
							bw.append(equationForThisLHS.getUnitsAndComment());
						bw.append("{\n");
						bw.append(/*scrub(*/statement)/*)*/;
						bw.append("}\n");

					} else {
						if (!equationForThisLHS.isDefinesSubscript()) {
							statement = indent + /* InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs) */equationForThisLHS.getCleanEquation()+"; // 3\n";
							if (includeComments)
								bw.append(equationForThisLHS.getUnitsAndComment());
							bw.append("{\n");
							bw.append(/*scrub(*/ statement) /*)*/;
							bw.append("}\n");
						}
					}
//				} else if (equations.get(lhs).isHasInitialValue() && 
//						initialValueInitialized(lhs)) {
//					System.out.println("(1) ############### This code is reachable #####################");
//					if (equations.get(lhs).isHasLHSArrayReference()) {
//
//					} else {
//
//						//Need to deal wirh arrays
//
//						statement = indent + "setValue(\""+lhs+"\","+getIntialValueVariable(lhs)+"); // 4\n";
//						if (includeComments)
//							bw.append(eqn.getUnitsAndComment());
//						bw.append("{\n");
//						bw.append(scrub(statement));
//						bw.append("}\n");
//					}
//				}

			}

			bw.append("}\n\n");
			bw.flush();

//			if (isGenerateJava())
				bw.append("public ");
			bw.append("void oneTime() {\n\n");
			
			bw.append("setRunParameters( initialTime,  finalTime, dt, saveper);\n");
//			bw.append("initialization = true;\n");
//			if (nativeMemory()) {
				bw.append(/*scrub( */ "memory.TIME_STEP = "+tStep+";\n") /*)*/ ;
				
//			} else {
//				bw.append(scrub("set(\"TIME_STEP\" , "+tStep+");\n"));
//				
//				// these files are now under the model location
//				String outputDirectory = InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName();
//				
//				String subFile = outputDirectory+ "/structure/"+"subscriptSpace.tab";
//				bw.append("loadSubscriptMap(Utilities.openForRead(\""+subFile+"\"));\n");
//				subFile = outputDirectory + "/structure/"+"subscriptDefinition.tab";
//				bw.append("loadSubscriptDefinition(Utilities.openForRead(\""+subFile+"\"));\n");
//			}

			for (int i = 0; i <= currentMethodNumber; i++) {
				bw.append("   oneTime"+i+"();\n");
			}
			bw.append("currentTime = initialTime;\n");
//			if (!newMemoryDefinition) {
//				bw.append("initializeMemoryMapHistory();\n");
//			}
//			if (newMemoryDefinition) {
//				bw.append("   extendedMemory.write(vectorMemory);\n");
//			}
//			bw.append("initialization = false;\n");
			bw.append("}\n\n");
			bw.flush();

//			if (isRepastSimphony()) {
//				String ScenarioDirectory = /* translator.getDestinationDirectory() */ "scenario" +"/" + objectName + ".rs/";
//				RepastSimphonyEnvironment.generateParametersXml(Utilities.openFileForWriting(ScenarioDirectory+"parameters.xml"), objectName, initialValues);
//			}


		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return false;   // i.e. no failure

	}

	private String forceDouble(String rhs) {



		// this is a constants or an expression consisting of only numbers and operators
		// we need to make sure that the expression will be recognized as a double in java

		return Parser.forceDouble(rhs);
	}
	private boolean repeated(BufferedWriter bw) {
		try {

//			if (isGenerateJava() && isRepastSimphony()){ 
//				bw.append("@ScheduledMethod(");
//				bw.append("start = 1,");
//				bw.append("interval = 1,");
//				bw.append("shuffle = true)\n");
//				bw.append("public ");
//
//				bw.append("public void step() {\n");
//				bw.append("ISchedule schedule = repast.simphony.engine.environment.RunEnvironment\n");
//				bw.append(".getInstance().getCurrentSchedule();\n");
//				bw.append(scrub("double timeStep = dt;\n"));
//				bw.append(scrub("double time = getINITIALTIME() + (schedule.getTickCount() - 1.0) * timeStep;\n"));
//
////				if (nativeMemory()) {
//					bw.append(scrub("memory.Time = time;\n"));
////				} else {
////					bw.append(scrub("set(\"Time\", time);\n"));
////				}
//
//				bw.append(scrub("repeated(time, timeStep);\n"));
//
//				if (isGenerateJava())
//					bw.append(scrub("currentTime = time;\n"));
//				bw.append("}\n\n");
//			} else {
				// Need C implementation!
//			}

//			if (isGenerateJava()) {
				bw.append("public ");
//			}
			bw.append("void repeated0(double time, double timeStep) {\n\n");

			currentMethodNumber = 0;
			int numPass = 2;
			for (int pass = 0; pass < numPass; pass++) {

				for (String lhs : evaluationOrderRepeated) {

					Equation equationForThisLHS = InformationManagers.getInstance().getEquationManager().getEquationFor(lhs);  //  InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs);
					
					if (equationForThisLHS == null) {
//						System.out.println("NO EQUATION FOR LHS: "+lhs+" IS THIS FROM AN ARTIFICIAL EQUATION FOR ORDERING?");
						continue;
					}

					boolean skip = false;

						if (pass == 0) {
							if ( equationForThisLHS.isStock() ||  equationForThisLHS.isOneTime())
								skip = true;
						} else {
							if (equationForThisLHS.isOneTime() || !equationForThisLHS.isStock())
								skip = true;
						}

						if (skip)
							continue;

						currentEquationNumber++;
						if (currentEquationNumber > EQUATION_LIMIT) {
							currentEquationNumber = 0;
							currentMethodNumber++;
							resetLimits();
							bw.append("}\n\n");
//							if (isGenerateJava())
								bw.append("protected ");
							bw.append("void repeated"+currentMethodNumber+"(double time, double timeStep) {\n\n");
						}

						if (equationForThisLHS.isVdmLookup()) {
							bw.append("/*\n");
							bw.append(" * This is automatically processed\n");
							bw.append(" * Included for documentation purposes\n");
							
						}
						//		    bw.append("{ /* repeated */ \n");
						if (equationForThisLHS.isVdmLookup()) {
							bw.append("*/\n");
						} else {
						resetCounters();
						codeGenerationFailure =  generateCode(equationForThisLHS, bw, DEBUG_VALUES) || codeGenerationFailure;
						
						}
				}
				if (/*isGenerateJava() && */ pass == 0) {
//					if (newMemoryDefinition) {
						bw.append("    saveMemory();\n");
//					} 
				}
			}
			// will always have an open method
			bw.append("}\n\n");


//			if (isGenerateJava())
				bw.append("public ");
			bw.append("void repeated(double time, double timeStep) {\n\n");
//			if (isGenerateJava()) {
				bw.append("  if (time % 1.0 == 0)\n");
				bw.append("  message.println(\"repeated: \"+time+\" \"+timeStep);\n");
				bw.append("  data.setCurrentTime(time);\n");
				bw.append("  this.time = time;\n");
				bw.append("  setValue(\"Time\", time);\n");
				bw.append("  timeSeriesData.advanceTime(data, time);\n");
				
				bw.append("  updateTimeSeriesReferences(time);\n");
				
				// need to make sure that the time variable storedin vector memory is set
//				if (newMemoryDefinition) {
					String offset = memoryMapper.getOffset("Time");
					bw.append("   vMem["+offset+"] = time;\n");
//				}
				
//			}

			for (int i = 0; i <= currentMethodNumber; i++) {
				bw.append("   repeated"+i+"(time, timeStep);\n");
			}
			
//			if (isGenerateJava()) {
//				if (newMemoryDefinition) {
////					bw.append("  if (time % 1.0 == 0)\n");
////					bw.append("  extendedMemory.write(vectorMemory);\n");
//				} else {
//					bw.append("   if (mapMemoryHistoryInitialized)\n");
//					bw.append("       logMemoryToHistory();\n");
//				}
//			}


			bw.append("}\n\n");
			
			bw.append("public void runTo(double time) {\n");
			bw.append("for (double t = initialTime+dt; t <= finalTime; t += dt) {\n");
			bw.append("repeated(t, getTIMESTEP());\n");
			bw.append("}\n");
			bw.append("}\n");
			bw.flush();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return codeGenerationFailure;

	}

	private void timeSeriesReferences(BufferedWriter bw) {
		try {

			bw.append("protected void updateTimeSeriesReferences(double time) {\n\n");
			
			// evaluationOrderOneTime
			
			List<String> al = new ArrayList<String>(evaluationOrderOneTime);
			al.addAll(evaluationOrderRepeated);

			for (String lhs : al) {

				Equation equationForThisLHS = InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs);  //  InformationManagers.getInstance().getEquationManager().getAnEquationFor(lhs);
				if (equationForThisLHS.isUsesTimeSeries()) {
					
					if (equationForThisLHS.isHasLHSArrayReference()) {
						bw.append("{\n");
						bw.append(equationForThisLHS.getEars().getOuterLoops());

//						if (nativeMemory()) {
							bw.append(equationForThisLHS.getEars().getLHSassignment());
							bw.append(" = data.arrayValueOf(\""+equationForThisLHS.getEars().getLhsArrayReference().getArrayName()+"\", ");
							for (int i = 0; i < equationForThisLHS.getEars().getOuterClosingCount(); i++) {
								if (i > 0)
									bw.append("+");
								bw.append("\"[\"+outer"+i+"+\"]\"");
							}
							bw.append(");\n");
							//		    bw.append("/* timeSeriesReferences */\n");
							bw.append("logit("+equationForThisLHS.getEars().getLHSassignmentName()+",time,"+equationForThisLHS.getEars().getLHSassignment()+",memory.get_SAVEPER());\n");
//						} else {
//
//							//    					memory.Instability_State_Exogenous[outer0][outer1] = data.arrayValueOf("Instability State Exogenous", "["+outer0+"]"+"["+outer1+"]");
//							//    					logit(asExpandedArrayRef("Instability State Exogenous",outer0,outer1),time,memory.Instability_State_Exogenous[outer0][outer1]);
//
//							bw.append("set("+equationForThisLHS.getEars().getLHSassignmentName());
//							bw.append(", data.arrayValueOf(\""+equationForThisLHS.getEars().getLhsArrayReference().getArrayName()+"\", ");
//							for (int i = 0; i < equationForThisLHS.getEars().getOuterClosingCount(); i++) {
//								if (i > 0)
//									bw.append("+");
//								bw.append("\"[\"+outer"+i+"+\"]\"");
//							}
//							bw.append("));\n");
//
//						}

						for (int i = 0; i < equationForThisLHS.getEars().getOuterClosingCount(); i++) {
							bw.append("}\n");
						}
						bw.append("}\n");

					}else {

						// SCALAR UPDATE TIME SERIES
						bw.append(equationForThisLHS.getTimeSeriesUpdate());
					}
					
				}
			}
			// will always have an open method
			bw.append("}\n\n");

			bw.flush();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void flattenTree(Node treeRoot) {
		int level = 0;
		if (treeRoot == null)
			return;
		flattenTree(treeRoot.getChild(), level);
	}

	private void flattenTree(Node node, int level) {
		if (node == null)
			return;

		//	    System.out.println(">>>>> "+level+"\n"+node.getInfo());

		flattenTree(node.getChild(), level+1);
		// check all children one level down, if any of them are terminal and not placeholders
		// pass their equation tail up to this level and terminate (although we may
		// recursively work our way back up but that will require some more thought)
		//	    if (node.getToken().startsWith(">"))
		//		System.out.println(">");

		Node nextLevel = node.getChild();
		while (nextLevel != null) {

			// added isDeleted and removed isPlaceHolder

			if ( /*isLeaf(nextLevel) &&*/ !nextLevel.isPlaceHolder()  && isEligibleToFlatten(nextLevel)) {

				flatten(node, nextLevel);

				nextLevel.setPlaceHolder(true);
				nextLevel.setDeleted(true); // simulated delete
			}

			nextLevel = nextLevel.getNext();
		}

		//	    if (!endThisBranch)
		//		flattenTree(node.getChild(), level+1);
		flattenTree(node.getNext(), level);
	}

	private boolean isEligibleToFlatten(Node node) {
		if (node.getGeneratedCodeHead().length() == 0 &&
				node.getGeneratedCodeElse().length() == 0 &&
				node.getGeneratedCodeTail().length() > 0)
			return true;
		else
			return false;
	}

	private void flatten(Node node, Node child) {
		String childVar = child.getResultsVariable();

		String childEqn = "("+   child.getGeneratedCodeTail().toString().split("=",2)[1].replace(";", "").trim()+")";

		String flattened = "";
		if (containsChildVar(childVar, node.getGeneratedCodeTail().toString())) {
			flattened = replaceChildVar(childVar, childEqn, node.getGeneratedCodeTail().toString());
			node.setGeneratedCodeTail(new StringBuilder(flattened));
		} else if (containsChildVar(childVar, node.getGeneratedCodeHead().toString())) {
			flattened = replaceChildVar(childVar, childEqn, node.getGeneratedCodeHead().toString());
			node.setGeneratedCodeHead(new StringBuilder(flattened));
		} else if (containsChildVar(childVar, node.getGeneratedCodeElse().toString())) {
			flattened = replaceChildVar(childVar, childEqn, node.getGeneratedCodeElse().toString());
			node.setGeneratedCodeElse(new StringBuilder(flattened));
		} 

	}

	private String replaceChildVar(String childVar, String childEqn, String code) {

		// the occurance of this variable can be as a argument to a method or as
		// a stand-alone variable in an expression
		String replaced = code.replace(childVar+",", childEqn+",").
				replace(childVar+" ", childEqn+" ").
				replace(childVar+")", childEqn+")").
				replace(childVar+";", childEqn+";").
				replace(childVar+")", childEqn+")");
		return replaced;
	}

	private boolean containsChildVar(String childVar, String code) {

		String[] terminators = new String[] {" ", ",", ";", ")"};
		for (String term : terminators) {
			if (code.contains(childVar+term))
				return true;
		}
		return false;
	}

	public void printTree(Node treeRoot) {
		int level = 0;
		if (treeRoot == null)
			return;
		System.out.println("Level "+level + " token "+treeRoot.getToken() + "\n["+treeRoot.getInfo()+"]");
		printTree(treeRoot.getChild(), 1);
	}

	public void generateTemps(Node treeRoot) {
		StringBuilder sb = new StringBuilder();
		if (treeRoot != null) 
			sb.append(generateTemps(treeRoot.getChild(), 1));
		if (sb.length() > 0) {
			addTempsToLHS(treeRoot, sb.toString());
		}
		return;
	}

	private void addTempsToLHS(Node treeRoot, String temps) {

		Node lhs = treeRoot.getChild();

		StringBuilder newHeader = new StringBuilder();
		newHeader.append(temps);
		newHeader.append(lhs.getGeneratedCodeHead());
		lhs.setGeneratedCodeHead(newHeader);


	}

	public String generateTemps(Node node, int level) {
		StringBuilder sb = new StringBuilder();
		if (node != null) {
			if (!node.isPlaceHolder())
				sb.append(printTemp(node));
			sb.append(generateTemps(node.getChild(), level+1));
			sb.append(generateTemps(node.getNext(), level));
		}
		return sb.toString();
	}

	public String printTemp(Node node) {

		StringBuilder sb = new StringBuilder();

		if (node.getResultsVariable() != null) {

			if (node.getResultsVariable().startsWith("_b")) {

//				if (!isGenerateJava())
//					sb.append("bool "+node.getResultsVariable()+" = false;\n");
//				else
					sb.append("boolean "+node.getResultsVariable()+" = false;\n");
			} else if (node.getResultsVariable().startsWith("_t")) {
				// need to declare lookups differently
				if (node.getToken().startsWith("lookup.")) {
					sb.append("double[][] "+node.getResultsVariable()+";\n");
				}else if (node.getToken().startsWith("sdFunctions.VECTORRANK") ||
						node.getToken().startsWith("sdFunctions.VECTORSORTORDER") ||
						node.getToken().startsWith("sdFunctions.VECTORREORDER") ||
						// this checks for first arg to function call requiring a vector
						(node.getParent() != null &&
							node.getPrevious() == null && node.getParent().getToken().startsWith("sdFunctions.VECTORRANK"))) {
					sb.append("double[] "+node.getResultsVariable()+";\n");
				} else {
					sb.append("double "+node.getResultsVariable()+" = 0.0;\n");
				}
			} else if (node.getResultsVariable().startsWith("_s")) {
//				if (!isGenerateJava())
//					sb.append("char* "+node.getResultsVariable()+" = (char *) malloc(100);\n");
//				else
					sb.append("String "+node.getResultsVariable()+" = \"\";\n");
			}
		}

		return sb.toString();
	}

	private void printTree(Node node, int level) {
		if (node == null)
			return;
		System.out.println("Level "+level + " token "+node.getToken() + "["+node.getInfo()+"]");
		printTree(node.getChild(), level+1);
		printTree(node.getNext(), level);
	}

	private String convertRangeFunction(Node node, Map<Node, String> rhsStatements, String valueVariable) {
		//	    this is where I need to make vector select work, I think
		StringBuilder code = new StringBuilder();
		// code gets translated into a single value

		EquationArrayReferenceStructure ears = currentGenerate.getEars();

		String type = getRangeFunctionType(node);

		Map<String, Integer> rangeIndex = new LinkedHashMap<String, Integer>();

		// first determine the subscripts that range in the function call
		// it is possible that there are none!
		List<Node> arrayReferenceNodes = getArrayReferences(node);
		List<String> rangeSubscripts = new ArrayList<String>();
		int index = 0;
		for (Node arn : arrayReferenceNodes) {
			ArrayReference ar = new ArrayReference(arn.getToken());
			for (String rangeSub : ar.getRangeSubscripts()) {
				String subscript = rangeSub;
				if (!rangeIndex.containsKey(subscript)) {
					rangeIndex.put(subscript, index++);
				}
				rangeSubscripts.add(rangeSub.replace("!", ""));
			}
		}

		// now alter range index ??????????????
		for (Node arn : arrayReferenceNodes) {
			ArrayReference ar = new ArrayReference(arn.getToken());
		}

		// define the loop
		//	    for (SubscriptCombination sub : getSubscriptValueCombinations("race", "state")) {
		//		value += arrayValueOf("population", concatAsSubscript(sub.getSubscriptValue("race"),sub.getSubscriptValue("state"));
		//	}
		String tVar = getNextDouble();
		if (type.equals("VMAX")) { 
			code.append("double "+tVar+" = -Double.MAX_VALUE;\n");
		}
		if (type.equals("VMIN")) {
			code.append("double "+tVar+" = Double.MAX_VALUE;\n");
		}

		boolean addRangeBracket = false;


		//	    code.append("/* CG 868 */\n");
		code.append(ears.getRangeLoops(arrayReferenceNodes));
		addRangeBracket = true;

		//		// this is the number of unique range subscripts that appear on the RHS
		//		int numRange = ears.getRHSrangeSubscripts().size();
		//		List<String> rangeSubs = ears.getRHSrangeSubscripts();
		//		// these are array + "###" + range subscript + "###" + dimension triples 
		//		int numIndexArrays = ears.getRHSarrayRangeSubscripts().size();
		//		
		//		// define the temporary arrays that we'll need
		//		for (String packedInfo : ears.getRHSarrayRangeSubscripts()) {
		//		    String[] info = packedInfo.split("###");
		//		    code.append("int[] indexArray_"+getLegalName(info[0]+"_"+info[1]+"_"+info[2])+"_ = new int[] {"+
		//			    ArrayManager.getIndicies(info[0], Integer.parseInt(info[2]), info[1])+"};\n");
		//		}
		//		
		//		
		//		for ( int pos = 0; pos < numRange; pos++) {
		//		    code.append("// loop for range variable "+rangeSubs.get(pos)+"\n");
		//		    code.append("for (int range"+pos+" = 0; range"+pos+" < "+
		//			    NamedSubscriptManager.getValuesFor(rangeSubs.get(pos).replace("!", "")).size()+"; range"+pos+"++) { // need to figure out closing }\n");
		//		}



		if (type.equals("VMAX")) {
			code.append("if ("+tVar+" < ");

			Node expressionRoot = node.getChild().getNext().getNext().getNext().getNext();

			//	    String exp = generateExpression(expressionRoot,  rhsStatements);
			String exp = generateExpression(expressionRoot);
			code.append(exp);
			code.append(") {\n");
			code.append(tVar+" = "+exp+";");
			code.append("\n}\n");
			code.append("\n}\n");
			if (addRangeBracket)
				code.append("} /* Range Bracket */\n");
			code.append(valueVariable+" = "+tVar+";\n");

		} else if (type.equals("VMIN")) {
			code.append("if ("+tVar+" > ");

			Node expressionRoot = node.getChild().getNext().getNext().getNext().getNext();

			//	    String exp = generateExpression(expressionRoot,  rhsStatements);
			String exp = generateExpression(expressionRoot);
			code.append(exp);
			code.append(") {\n");
			code.append(tVar+" = "+exp+";");
			code.append("\n}\n");
			code.append("\n}\n");
			if (addRangeBracket)
				code.append("} /* Range Bracket */\n");
			code.append(valueVariable+" = "+tVar+";\n");
		} else {

			code.append(valueVariable);
			code.append(" "+(type.equals("SUM") ? "+" : "*")+"= (\n");

			// generate the code as specified in arg # 5 of the range function
			// note that this can be a complex tree for the computation

			//	    Node expressionRoot = node.getChild().getNext().getNext().getNext().getNext();

			Node expressionRoot = TreeTraversal.getFunctionArgument(node, 1);

			code.append(generateExpression(expressionRoot));
			//	    code.append(generateExpression(expressionRoot,  rhsStatements));
			code.append(");\n");

			for (int i = 0; i < ears.getRangeClosingCount(arrayReferenceNodes); i++)
				code.append("}\n");

			if (addRangeBracket)
				code.append("} /* Range Bracket */\n");

		}



		// now that we've generated code. alter the tree

		node.setToken(valueVariable);
		node.setChild(null);

		return code.toString();
	}

	public static String getLegalName(String s ) {
		// cleanup the string for use as a portion of a variable/array name
		String legal = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(s.replace("!", ""));
		return legal.replace("memory.", "");
	}

	private String generateExpression(Node node, Map<Node, String> rhsStatements) {

		StringBuilder sb = new StringBuilder();

		if (node == null)
			return "";

		if (isLeaf(node)) {
			if (rhsStatements.containsKey(node)) {

				return rhsStatements.get(node);
			} else {
				return node.getToken();
			}
		}
		if (node.getChild() != null) {
			sb.append(generateExpression(node.getChild(), rhsStatements));
		}
		sb.append(node.getToken());
		if (node.getChild() != null && node.getChild().getNext() != null) {
			sb.append(generateExpression(node.getChild().getNext(), rhsStatements));
		}

		return sb.toString();

	}

	private String getValueVariable() {
		if (nextValueVariable == 52)
			System.out.println("_vv 52");
		String vv = "_vv"+nextValueVariable;
		nextValueVariable++;
		return vv;
	}
	private List<Node> getArrayReferences(Node node) {
		List<Node> al = new ArrayList<Node>();

		if (node == null)
			return al;

		if (node.getToken().startsWith("array.")) {
			al.add(node);
		}

		// assume for now that there are no nested references

		List<Node> more = getArrayReferencesChild(node.getChild());
		if (more.size() > 0)
			al.addAll(more);

		//	more = getArrayReferences(node.getNext());
		//	if (more.size() > 0)
		//	    al.addAll(more);

		return al;
	}

	private List<Node> getArrayReferencesChild(Node node) {
		List<Node> al = new ArrayList<Node>();

		if (node == null)
			return al;

		if (node.getToken().startsWith("array.")) {
			al.add(node);
		}

		// assume for now that there are no nested references

		List<Node> more = getArrayReferencesChild(node.getChild());
		if (more.size() > 0)
			al.addAll(more);

		more = getArrayReferencesChild(node.getNext());
		if (more.size() > 0)
			al.addAll(more);

		return al;
	}
	private Node getLHS(Node node) {
		return node.getChild();
	}

	private boolean isArrayReference(Node node) {
		return ArrayReference.isArrayReference(node.getToken());
	}

	private boolean isRangeFunctionNode(Node node) {
		if (node.getToken().startsWith("sdFunctions.SUM") || 
				node.getToken().startsWith("sdFunctions.PROD") ||
				node.getToken().startsWith("sdFunctions.VMIN") ||
				node.getToken().startsWith("sdFunctions.VMAX"))
			return true;
		else
			return false;
	}

	private String getRangeFunctionType(Node node) {
		if (node.getToken().startsWith("sdFunctions.SUM"))
			return "SUM";
		else if (node.getToken().startsWith("sdFunctions.PROD"))
			return "PROD";
		else if (node.getToken().startsWith("sdFunctions.VMIN"))
			return "VMIN";
		else if (node.getToken().startsWith("sdFunctions.VMAX"))
			return "VMAX";
		else 
			return "???";
	}

	private List<Node> getRangeFunctionReferences(Node node) {
		List<Node> al = new ArrayList<Node>();

		if (node == null)
			return al;

		if (isRangeFunctionNode(node)) {
			al.add(node);
		}

		// assume for now that there are no nested references

		List<Node> more = getRangeFunctionReferences(node.getChild());
		if (more.size() > 0)
			al.addAll(more);

		more = getRangeFunctionReferences(node.getNext());
		if (more.size() > 0)
			al.addAll(more);

		return al;
	}


	private String generateModifiedCode(Node node) {
		StringBuilder sb = new StringBuilder();

		if (node == null)
			return "";

		if (!node.getInfo().equals("@"))
			return node.getInfo();

		if (Parser.isOperator(node.getToken()) || Parser.isBooleanOperator(node.getToken())){
			if (node.getToken().equals("^")) {
				sb.append("Math.pow(");
				sb.append(generateModifiedCode(node.getChild()));
				sb.append(",");
				sb.append(generateModifiedCode(node.getChild().getNext()));
				sb.append(");\n");
			} else {
				// boolean operator
				sb.append(generateModifiedCode(node.getChild()));
				sb.append(node.getToken());
				sb.append(generateModifiedCode(node.getChild().getNext()));
			}

		} else if (Parser.isFunctionInvocation(node.getToken())){
			// function invocation

			sb.append(node.getToken()+"(");
			Node n = node.getChild();
			sb.append(generateModifiedCode(n));
			n = n.getNext();
			while(n != null) {
				sb.append(",");
				sb.append(generateModifiedCode(n));
				n = n.getNext();
			}

			sb.append(")");
		} else {
			// the leaf node
			sb.append(node.getToken());
		}


		return sb.toString();
	}

	public boolean generateCode(Equation equation, BufferedWriter bw, boolean DEBUG_VALUES) {


		resetCounters();

		ModelManager mm = InformationManagers.getInstance().getModelManager();

		currentGenerate = equation;
		// This is intended only for "repeated" statements

		if (!equation.isTreeCodeGenerated()) {

			EquationArrayReferenceStructure ears = null;
			if((equation.isHasLHSArrayReference() || equation.isHasRHSArrayReference())) {
				ears = equation.getEars();
			}

			if (!generateLHScode(equation, ears)) {
				return true;  // TODAY false
			}
			
			if (!generateRHScode(equation, ears)) {
				return true;
			}


			Node root = equation.getTreeRoot();

			if (flatten && !equation.isUsesRangeFunction()) {
				flattenTree(root);
				if (!equation.isIte() && !equation.containsVectorFunction()) {
					Node lhs = TreeTraversal.getLhs(root);
					Node rhs = TreeTraversal.getRhs(root);
//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							if (rhs.getGeneratedCodeTail().toString().split(" = ").length > 1) {
								// lhs tail contains:
								// assignment to variable
								
								// any closing }
								String[] lhsTail = lhs.getGeneratedCodeTail().toString().split("\n");
//								String logit = lhsTail[1];
								String lhPart = lhsTail[0].split("=")[0].trim();

								String rhPart = rhs.getGeneratedCodeTail().toString().split(" = ")[1];
//								logit = logit.replace(rhs.getResultsVariable(), lhPart);
								// reconstruct lhs tail
								lhs.getGeneratedCodeTail().setLength(0);
								lhs.getGeneratedCodeTail().append(lhPart + " = " + rhPart).append("\n");
								// any closing parens that need to be added?
								for (int i = 1; i < lhsTail.length; i++) {
									lhs.getGeneratedCodeTail().append(lhsTail[i]+"\n");
								}
								rhs.setPlaceHolder(true);
							}
//						} else {
//							if (rhs.getGeneratedCodeTail().toString().split(" = ").length > 1) {
//								// lhs tail contains:
//								// assignment to variable
//								// log statement
//								// any closing }
//								String[] lhsTail = lhs.getGeneratedCodeTail().toString().split("\n");
//								String logit = lhsTail[1];
//								String lhPart = lhsTail[0].split("=")[0].trim();
//
//								String rhPart = rhs.getGeneratedCodeTail().toString().split(" = ")[1];
//								logit = logit.replace(rhs.getResultsVariable(), lhPart);
//								// reconstruct lhs tail
//								lhs.getGeneratedCodeTail().setLength(0);
//								lhs.getGeneratedCodeTail().append(lhPart + " = " + rhPart).append(logit).append("\n");
//								// any closing parens that need to be added?
//								for (int i = 2; i < lhsTail.length; i++) {
//									lhs.getGeneratedCodeTail().append(lhsTail[i]+"\n");
//								}
//								rhs.setPlaceHolder(true);
//							}
//						}
//					} else {
//
//						// form: memoryWrite("xxx", ...); where ... is any expression
//						// lhpart = memoryWrite("xxx"
//						// rhpart = ...
//
//						String[] lhsTail = lhs.getGeneratedCodeTail().toString().split("\n");
//
//						String statement = lhsTail[0];
//
//						// if arrays, have to worry about generatedCodeHead!
//
//						String resultsVar;
//						String expression;
//						if (!equation.isUsesRangeFunction()) {
//							String[] rhPart = rhs.getGeneratedCodeTail().toString().split(" = ");
//							resultsVar = rhPart[0];
//							expression = rhPart[1].replace(";\n", "");
//							// reconstruct lhs tail
//							lhs.getGeneratedCodeTail().setLength(0);
//							//						lhs.getGeneratedCodeTail().append(lhPart + " = " + rhPart).append(logit).append("\n");
//							lhs.getGeneratedCodeTail().append(statement.replace(resultsVar, expression)+"\n");
//
//
//							// any closing parens that need to be added?
//							for (int i = 1; i < lhsTail.length; i++) {
//								lhs.getGeneratedCodeTail().append(lhsTail[i]+"\n");
//							}
//						}
//						rhs.setPlaceHolder(true);
//
//					}
				}
			}
			generateTemps(root);

			equation.setTreeCodeGenerated(true);
			
				
		}

		// There are a couple of instances in which we actually need to rewrite
		// some of the code that we have already written
		if (equation.requiresPostGenerationProcessing()) {
			postGenerationProcessing(equation);
		}


		//		    printTree(root);

		writeGeneratedCode(equation, bw, DEBUG_VALUES);
//		if (equation.isStock()) {
//			equation.inlineInteg();
//			equation.setTreeCodeGenerated(false);
//		}
		return false; // i.e. no failure
	}

	private void postGenerationProcessing(Equation equation) {
		//	System.out.println("##### Going In #####");
		//	printTree(equation.getTreeRoot());
//		if (equation.isHasVectorSortOrder()) {
//			postGenerationProcessingVectorSortOrder(equation);
//		} else if (equation.isHasVectorElmMap()) {
//			postGenerationProcessingVectorElmMap(equation);
//		}

		//	System.out.println("##### Going Out #####");
		//	printTree(equation.getTreeRoot());

	}

	private void postGenerationProcessingVectorSortOrder(Equation equation) {


		// THIS NEEDS TO CHANGE. WE CANNOT PASS THE ORIGINAL VARIABLE TO VSO
		// MUST CREATE A NEW TEMP ARRAY

		Node root = equation.getTreeRoot();
		Node lhs = root.getChild();
		Node rhs = lhs.getNext();

		String resultsVariable = rhs.getResultsVariable();

		// gather each line of code in the head into an array
		String[] linesOfCode = lhs.getGeneratedCodeHead().toString().split("\n");
		int vectorLength = 0;
		int keepToLine = 0;
		String removedLoopVar = "";

		// original int* outer0_index = newIntArray(10,0, 1, 2, 3, 4, 5, 6, 7, 8, 9);
		// now      int outer0_index[7] = { 0, 1, 2, 3, 4, 5, 6 };
		// back up to the line that creates the last "outer" loop
		for (int i = linesOfCode.length-1; i >= 0; i--) {
			String code = linesOfCode[i];
			if (code.contains("newIntArray(")) {
				// get the first arg to newIntArray
				//			String vLen = code.split("Array\\(")[1].split(",")[0];

				//get the length of the vector
				String vLen = code.split("]")[0].split("\\[")[1];
				vectorLength = Integer.parseInt(vLen);

				keepToLine = i;
				// get "outer0" portion of line
				//			removedLoopVar = code.split("=")[0].replace("int* ", "").replace("int[] ", "").replace("_index ", "");
				removedLoopVar = code.split("\\[")[0].split(" ")[1].replace("_index", "");
				break;
			}
		}

		String scalarDeclaration = "double "+resultsVariable+" = 0.0";
		String vectorDeclaration = "";
//		if (!isGenerateJava()){
			//	    vectorDeclaration = "double "+resultsVariable+"["+vectorLength+"];";
			vectorDeclaration = "double *"+resultsVariable+/* " = newDoubleArray("+vectorLength+ ");";*/ ";";
//		} else {
//			vectorDeclaration = "double[] "+resultsVariable+" = new double["+vectorLength+"];";
//		}

		StringBuilder newHeader = new StringBuilder();

		for (int i = 0; i < keepToLine; i++) {
			if (linesOfCode[i].contains(scalarDeclaration)) {
				newHeader.append(vectorDeclaration);
			} else {
				newHeader.append(linesOfCode[i]);
			}
			newHeader.append("\n");
		}

		lhs.setGeneratedCodeHead(newHeader);

		// need to remove the "}" associated with the removed outer loop

		StringBuilder newTail = new StringBuilder();
		linesOfCode = lhs.getGeneratedCodeTail().toString().split("\n");

		for (int i = 0; i < linesOfCode.length-1; i++) {
			if (linesOfCode[i].contains("["+removedLoopVar+"]")) {
				newTail.append(linesOfCode[i].replace("["+removedLoopVar+"]", ""));
			} else if (linesOfCode[i].contains("logit"))  {
				String newLog = linesOfCode[i].replace("logit", "logitVector").replace(resultsVariable, vectorLength+","+resultsVariable).replace(removedLoopVar, "0");
				newTail.append(newLog);
			} else {
				newTail.append(linesOfCode[i]);
			}
			newTail.append("\n");
		}

		lhs.setGeneratedCodeTail(newTail);

		// need to remove the removeLoopVar passed in to Vector Sort Order funtion
		// next to last argument
		// editing RHS tail

		StringBuilder rhsNewTail = new StringBuilder();
		linesOfCode = rhs.getGeneratedCodeTail().toString().split("\n");

		for (int i = 0; i < linesOfCode.length; i++) {
			if (linesOfCode[i].contains("["+removedLoopVar+"]")) {
				int occurrence = 0;
				String[] stuff = linesOfCode[i].split(",");
				for (int s = 0; s < stuff.length; s++) {
					if (s > 0){
						rhsNewTail.append(",");
					}
					if (stuff[s].contains("["+removedLoopVar+"]")) {
						occurrence++;
						if (occurrence == 1) {
							// specifying the "value" to func
							rhsNewTail.append(stuff[s].replace(removedLoopVar, "0"));
						} else if (occurrence == 2) {

							rhsNewTail.append(stuff[s].replace("["+removedLoopVar+"]", ""));
						} else {
							rhsNewTail.append(stuff[s]);
						}
					} else if (stuff[s].contains("intToString("+removedLoopVar+")")) {

						rhsNewTail.append(stuff[s].replace(removedLoopVar, "0"));

					} else {
						rhsNewTail.append(stuff[s]);
					}
				}

			} else {
				rhsNewTail.append(linesOfCode[i]);
			}
			rhsNewTail.append("\n");
		}
		rhs.setGeneratedCodeTail(rhsNewTail);
	}

	private void postGenerationProcessingVectorElmMap(Equation equation) {

		/*
	_t0 = VECTORELMMAP(
		stringConcat("sorted_target_active[",intToString(outer0),"][",intToString(outer1),"]"),
		sorted_target_active[outer0][outer1],
		time,
		timeStep,
		(Target_is_Active[outer0][0]),    <<< needs to be (&Target_is_Active[outer0][0]),
		(Target_Order[outer0][outer1]));



		 */

		// if (Translator.target.equals(ReaderConstants.JAVA))

		Node root = equation.getTreeRoot();
		Node lhs = root.getChild();
		Node rhs = lhs.getNext();

		String zeroOffset = "0";
		String[] invocation = rhs.getGeneratedCodeTail().toString().split(",");
		int index = invocation.length-2;
//		if (isGenerateJava()) {
			// NOTE! This [0] doesn't need to be 0!
			// 
			zeroOffset = getZeroOffset(invocation[index]);
			invocation[index] = invocation[index].replace("["+zeroOffset+"]", "");
//		} else {
//			invocation[index] = "&" + invocation[index];
//		}

		index = invocation.length-1;
		invocation[index] = invocation[index].replace("(", "("+zeroOffset+"+");

		StringBuilder sb = new StringBuilder();
		int i = 0;
		for (String s : invocation) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}
		rhs.setGeneratedCodeTail(sb);

	}

	private String getZeroOffset(String invocation) {
		String zeroOffset = "";
		String notation = "\\[[0-9]+\\]\\)";
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(invocation);
		boolean found = m.find();
		String fnd = m.group();
		zeroOffset = fnd.replace("[", "").replace("])", "");
		return zeroOffset;
	}

	private boolean generateLHScode(Equation equation, EquationArrayReferenceStructure ears) {
		// generate the code for the LHS
		// nodes will tell child nodes into which variable
		// to place their result
		

		Node lhs = null;
		String resultsVariable;
		
		Node root = equation.getTreeRoot();

		lhs = root.getChild();
		if (lhs == null) {
//			InformationManagers.getInstance().getMessageFrame().addMessage(
			new MessageFrame(equation.getLhs(),errorMessageLocation,errorMessageLocation,700,200).addMessage(
					"An internal error has occurred processing this equation:\n"+
					"    "+equation.getVensimEquationOnly()+"\n"+
							"Model cannot be executed");
			InformationManagers.getInstance().getModelManager().setRunnable(false,
					"An internal error has occurred processing this equation:\n"+
					"    "+equation.getVensimEquationOnly()+"\n"+
					"Model cannot be executed");
			errorMessageLocation += 20;
			return false;
		}
		Node rhs = lhs.getNext();

		resultsVariable = getNextDouble();
		rhs.setResultsVariable(resultsVariable);



		if (!isArrayReference(lhs)) {

			currentHasLhsArrayReference = false;
			outerSubscripts.clear();

			// one of two forms:
			// A = B
			// generates:
			// _d = RHS head

			// setValue(A, _d) tail

			// ====================================================== Here ==================================

//			if (nativeMemory()) {
//				if (newMemoryDefinition) {
					// vector index access
//					lhs. getGeneratedCodeTail().append(
//							"vectorMemory["+memoryMapper.getOffset(lhs.getToken()) +"]"+
//							" = "+resultsVariable+";\n");
					lhs. getGeneratedCodeTail().append(
							memoryMapper.getDataStore(lhs.getToken())+"["+memoryMapper.getOffset(lhs.getToken()) +"]"+
							" = "+resultsVariable+";\n");
//					if (DEBUG_VALUES) {
//						String assignment = memoryMapper.getDataStore(lhs.getToken())+"["+memoryMapper.getOffset(lhs.getToken()) +"]";
//						lhs. getGeneratedCodeTail().append("debugValue(\""+assignment+"\", "+assignment+");\n");
//					}
//				} else {
//					lhs. getGeneratedCodeTail().append(InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs.getToken())+" = "+resultsVariable+";\n");
//					lhs.getGeneratedCodeTail().append("logit(\""+lhs.getToken()+"\",time,"+resultsVariable+",memory.get_SAVEPER());\n");
//				}
//			} else {
//				// InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName
//				lhs.getGeneratedCodeTail().append("set(\""+InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs.getToken())+"\", "+resultsVariable+");\n");
//				//	    	lhs.getGeneratedCodeTail().append("/* mw */ "+"logit(\""+lhs.getToken()+"\",time,"+resultsVariable+");\n");
//			}
			//		lhs.getGeneratedCodeTail().append("/* generateLHScode */\n");


		} else {

			ArrayReferenceNative ar = new ArrayReferenceNative(lhs.getToken(), equation);
			currentHasLhsArrayReference = true;
			outerSubscripts.addAll(ar.getSubscripts());
			lhs.getGeneratedCodeHead().append(ar.generateLHSHeader(resultsVariable));
			lhs.getGeneratedCodeTail().append(ar.generateLHSFooter(resultsVariable, /*nativeMemory(), */ /*newMemoryDefinition,*/ DEBUG_VALUES));
		}
		return true;

	}

	private boolean generateRHScode(Equation equation, EquationArrayReferenceStructure ears) {
		// generate the code for the RHS

		Node node = equation.getTreeRoot().getChild().getNext(); // -> this is RHS

		return generateRHSCode(node);

	}

	private void printNode(Node node, boolean traverseChildSiblings) {
		if (node == null) {
			return;
		}

		if (Equation.isITENode(node)) {

			// skip placeholders
			Node condition = node.getChild(); // .getNext().getNext().getNext().getNext();
			Node thenExpression = condition.getNext();
			Node elseExpression = thenExpression.getNext();

			// print the condition Expression

			printNode(condition, false);
			printNodeCodeHead(node);
			printNode(thenExpression, false);
			printNodeCodeElse(node);
			printNode(elseExpression, false);
			printNodeCodeTail(node);


		} else {
			printNode(node.getChild(), true);
			if (traverseChildSiblings) 
				printNode(node.getNext(), true);
			printNodeCode(node);
		}
	}

	private void printNodeCode(Node node) {
		try {
			if (!node.isPlaceHolder()) {
				if (node.getGeneratedCodeHead().length() > 0) {
					sourceCode.append(/*scrub(*/node.getGeneratedCodeHead().toString()) /*)*/;
				}

				if (node.getGeneratedCodeElse().length() > 0) {
					sourceCode.append(/*scrub(*/ node.getGeneratedCodeElse().toString()) /*)*/ ;
				}

				if (node.getGeneratedCodeTail().length() > 0) {
					sourceCode.append(/* scrub(*/ node.getGeneratedCodeTail().toString()) /*)*/;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private void printNodeCodeHead(Node node) {
		try {
			if (!node.isPlaceHolder()) {
				if (node.getGeneratedCodeHead().length() > 0) {
					sourceCode.append(/*scrub( */ node.getGeneratedCodeHead().toString()) /*)*/;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private void printNodeCodeElse(Node node) {
		try {
			if (!node.isPlaceHolder()) {

				if (node.getGeneratedCodeElse().length() > 0) {
					sourceCode.append(/*scrub(*/ node.getGeneratedCodeElse().toString()) /*)*/;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	private void printNodeCodeTail(Node node) {
		try {
			if (!node.isPlaceHolder()) {
				if (node.getGeneratedCodeTail().length() > 0) {
					sourceCode.append(/* scrub( */ node.getGeneratedCodeTail().toString()) /*)*/ ;
				}
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void writeGeneratedCode(Equation equation, BufferedWriter bw, boolean DEBUG_VALUES) {
		// traverse the tree and write the code to file

		try {
			Node root = equation.getTreeRoot();

			Node lhs = root.getChild();
			Node rhs = lhs.getNext();
			if (includeComments)
				bw.append(equation.getUnitsAndComment());

			// only when needed
			boolean group = false;
			if (lhs.getGeneratedCodeHead().length() > 0)
				group = true;
			if (group)
				bw.append("{ // group open\n");
			bw.append(/* scrub( */ lhs.getGeneratedCodeHead().toString()) /*)*/;
			boolean traverseChildSiblings = true;
			printNode(rhs, traverseChildSiblings);  // print the right hand side
			bw.append(/* scrub( */ lhs.getGeneratedCodeTail().toString()) /*)*/;
			
			if (DEBUG_VALUES) {
				String assignment = /*scrub( */ lhs.getGeneratedCodeTail().toString().split("\n")[0] /*)*/ .split("=", 2)[0].trim();
				bw.append("debugValue(\""+assignment+"\", "+assignment+");\n");
			}
			

			// only when needed
			if (group)
				bw.append("} // group close\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private String timeGuard(String exp) {
//		if (nativeMemory())
			return "(memory.time == 0.0 ? 0.0 : "+exp+" )";
//		else
//			return "(memory.time == 0.0 ? 0.0 : "+exp+" )";
	}


	// this seems to work OK without Array References -- need to incorporate and then pass all
	// equations through this.
	// code will be stored in the nodes for later printing -- where we will need to worry about ordering
	// 

	private boolean generateRHSCode(Node node) {

		if (node == null)
			return true;

		StringBuilder theHead;
		StringBuilder theTail;
		StringBuilder theElse;

		theHead = node.getGeneratedCodeHead();
		theElse = node.getGeneratedCodeElse();
		theTail = node.getGeneratedCodeTail();


		if (!node.isPlaceHolder()) {

			// this node has already been assigned the variable into which to store it's value;

			if (isRangeFunctionNode(node)) {
				FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(node.getToken());
				// ignore the place holders
				Node n = node.getChild();
				int numPh = 0;
				if (fd.isRequiresName())
					numPh++;
				if (fd.isRequiresTime())
					numPh++;
				if (fd.isRequiresTimeStep())
					numPh++;
				if (fd.isRequiresValue())
					numPh++;
				for (int i = 0; i < numPh; i++) {
					n.setPlaceHolder(true);
					n = n.getNext();
				}
				// we will in fact ignore the fifth as well as we are inlining this code
				// make sure our result varaible is initialized
				theHead.append(node.getResultsVariable()+" = 0.0;\n");
				theHead.append(convertRangeFunction(node,genRHSStatements(node), node.getResultsVariable()));

				n.setPlaceHolder(true);
				// create the loop over range variables
				// sum/prod of values into results
			} else  if (node.getToken().equals("sdFunctions.IFTHENELSE")) {
				// need to specify the dest var for cond, true, and false
				Node condition = getCONDITION(node);
				String conditionVariable = getNextBoolean();
				condition.setResultsVariable(conditionVariable);

				Node thenNode = getTHEN(node);
				String thenVariable = getNextDouble();
				thenNode.setResultsVariable(thenVariable);		

				Node elseNode = getELSE(node);
				String elseVariable = getNextDouble();
				elseNode.setResultsVariable(elseVariable);

				// the first four arguments are placeholders and should not be processed
//				Node n = node.getChild();
//				for (int i = 0; i < 4; i++) {
//					n.setPlaceHolder(true);
//					n = n.getNext();
//				}

				//generate head and tail code

				theHead.append("if ("+conditionVariable+") {\n");

				boolean rvDouble = node.getResultsVariable().startsWith("_t");
				boolean thenVariableDouble = thenVariable.startsWith("_t");
				boolean elseVariableDouble = thenVariable.startsWith("_t");
				boolean rvBoolean = !rvDouble;
				boolean thenVariableBoolean = !thenVariableDouble;
				boolean elseVariableBoolean = !elseVariableDouble;

				// we may need to cast double values to boolean
				String cast = "";
				if (rvBoolean && thenVariableDouble) {
					cast = " == 1 ";
				}

				theElse.append(node.getResultsVariable()+" = "+thenVariable+cast+";\n");
				theElse.append("} else {\n");

				cast = "";
				if (rvBoolean && elseVariableDouble) {
					cast = " == 1 ";
				}

				theTail.append(node.getResultsVariable()+" = "+elseVariable+cast+";\n");
				theTail.append("}\n");

			} else if (Parser.isOperator(node.getToken()) || Parser.isBooleanOperator(node.getToken())){

				String variable1 = "";
				String variable2 = "";
				// the operator is either unary or binary if not boolean
				// when working with boolean operators, will need to look ahead to determine if child
				// nodes are boolean or doubles
				if (!Parser.isBooleanOperator(node.getToken())) {
					variable1 = getNextDouble();
					Node left = node.getChild();
					left.setResultsVariable(variable1);

					if (left.getNext() == null) {
						// unary operator 
						theTail.append(node.getResultsVariable()+" = "+node.getToken().replace("_", "-")+" "+variable1+";\n");
					} else {
						// binary operator
						variable2 = getNextDouble();
						Node right = left.getNext();
						right.setResultsVariable(variable2);
						if (node.getToken().equals("^")) {
							theTail.append(node.getResultsVariable()+" = Math.pow("+variable1+","+variable2+");\n");
						} else {
							theTail.append(node.getResultsVariable()+" = "+variable1+" "+node.getToken()+" "+variable2+";\n");
						}
					}
				} else {
					// working with either a logical operator or a relational operator as root
					// convert from Vensim operator to its Java equivalent
					if (node.getToken().equals(":AND:"))
						node.setToken("&&");
					if (node.getToken().equals(":OR:"))
						node.setToken("||");
					if (node.getToken().equals(":NOT:"))
						node.setToken("!");

					if (Parser.isLogicalOperator(node.getToken())) {
						variable1 = getNextBoolean();
						variable2 = getNextBoolean();
					} else { // relational operator
						variable1 = getNextDouble();
						variable2 = getNextDouble();
					}

					if (Parser.isBinaryOperator(node.getToken())) { 
						node.getChild().setResultsVariable(variable1);
						node.getChild().getNext().setResultsVariable(variable2);
						theTail.append(node.getResultsVariable()+" = "+variable1+" "+node.getToken()+" "+variable2+";\n");
					} else {
						node.getChild().setResultsVariable(variable1);
						theTail.append(node.getResultsVariable()+" = "+node.getToken()+" ("+variable1+");\n");
					}
				}
			} else if (node.getToken().equals("sdFunctions.VECTORSELECT")){
				String suffix1 = getNextInt();
				String suffix2 = getNextInt();
				ArrayReference ar1 = null;
				ArrayReference ar2 = null;
				Node arg1 = node.getChild().getNext().getNext().getNext().getNext();
				Node arg2 = arg1.getNext();

				// these should be arrays and their range subscripts show be the same
				if (ArrayReference.isArrayReference(arg1.getToken())) {
					ar1 = new ArrayReference(arg1.getToken());
				}
				if (ArrayReference.isArrayReference(arg2.getToken())) {
					ar2 = new ArrayReference(arg2.getToken());
				}

				if (ar1 == null) {
					System.out.println("sdFunctions.VECTORSELECT ar1 = null");
				}



				List<String> range = ar1.getRangeSubscriptsNames();





//				if (!isGenerateJava())
//					theHead.append("double _da"+suffix1+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");
//				else
					theHead.append("double[] _da"+suffix1+" = new double"+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");

				theHead.append("int _i"+suffix1+" = 0;\n");
				// need a range loop
				ArrayReferenceNative ar1n = new ArrayReferenceNative(arg1.getToken(), currentGenerate);
				EquationArrayReferenceStructure ears = currentGenerate.getEars();
				List<Node> arrayReferenceNodes = getArrayReferences(node);
				//		    theHead.append("/* CG 1571 */\n");
				//		    theHead.append("{\n");
				theHead.append(ears.getRangeLoops(arrayReferenceNodes));
				theHead.append("_da"+suffix1+"[_i"+suffix1+"++] = "+ar1n.generateRHSImplementation()+";\n");
				for (int i = 0; i < ears.getRangeClosingCount(); i++)
					theHead.append("}\n");
				theHead.append("}\n");

				if (ar2 != null) {
					range = ar2.getRangeSubscriptsNames();


//					if (!isGenerateJava())
//						theHead.append("double _da"+suffix2+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");
//					else
						theHead.append("double[] _da"+suffix2+" = new double"+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");

					theHead.append("int _i"+suffix2+" = 0;\n");
					// need a range loop
					ar1n = new ArrayReferenceNative(arg2.getToken(), currentGenerate);

					ears = currentGenerate.getEars();
					arrayReferenceNodes = getArrayReferences(node);
					theHead.append(ears.getRangeLoops(arrayReferenceNodes));

					theHead.append("_da"+suffix2+"[_i"+suffix2+"++] = "+ar1n.generateRHSImplementation()+";\n");
					for (int i = 0; i < ears.getRangeClosingCount(); i++)
						theHead.append("}\n");
					theHead.append("}\n");

				} else {
					// here we have an arithmetic expression that evaluates to an array of the same length
					// as the _da1 array
					// for example:

					//		        VECTOR SELECT(
					//		        	Path resource definition[EnergyPath!,Resource],
					//		        	Energy to meet demand[EnergyPath!]/Efficiency[EnergyPath!],
					//		        	0,
					//		        	VSSUM,
					//		        	VSERRATLEASTONE)

					// We want to generate the child, the node, the childs next repeatedly

//					if (!isGenerateJava())
//						theHead.append("double _da"+suffix2+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");
//					else
						theHead.append("double[] _da"+suffix2+" = new double"+"["+InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))+"];\n");

					theHead.append("int _i"+suffix2+" = 0;\n");
					// need a range loop
					// arg2 is not an array reference -- that is why we are in this portion of the code.
					// that means it is some form of expression

					ears = currentGenerate.getEars();
					arrayReferenceNodes = getArrayReferences(node);
					theHead.append(ears.getRangeLoops(arrayReferenceNodes));

					// we need to generate the code based on a tree with this argument as the root.
					//			ArrayReferenceNative ar1n = new ArrayReferenceNative(arg2.getToken(), currentGenerate);

					// generateExpression (currentGenerate, node);
					theHead.append("_da"+suffix2+"[_i"+suffix2+"++] = "+generateExpression(arg2 /*, genRHSStatements(node) */)+";\n");   // here no genRHSS

					for (int i = 0; i < ears.getRangeClosingCount(); i++)
						theHead.append("}\n");
					theHead.append("}\n");

					arg2.setPlaceHolder(false);
				}

				// start build the function call

				// current

				// _t0 = sdFunctions.VECTORSELECT(stringConcat("memory.Aggregated_CO2_FF_emissions[",intToString(outer0),"]",NULL),
				//	memory.Aggregated_CO2_FF_emissions[outer0],time,timeStep,_da0,_da1,_t1,_t2,_t3

				// convert to

				// _t0 = VECTORSELECT(7,_da0,_da1,(VSSUM),(VSERRATLEASTONE));
				// _t0 = VECTORSELECT(7,_da0,_da1,(VSSUM),(VSERRATLEASTONE));
				// VECTOR SELECT(sarray,vexp,mval,act,err)
				// double VECTORSELECT(int length, double* _da1, double* _da2, double operation, double err)
				// where is length available?
				// InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0))

				boolean useAlternateImplementation = true;

				theTail.append(node.getResultsVariable()+" = ");
				theTail.append(node.getToken()+"(");

				Node n = node.getChild();

				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				String lhs = currentGenerate.getLhs();
				String valueArg;
				String nameArg;
				if (ArrayReference.isArrayReference(lhs)) {

					valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
					nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();


				} else {
					valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);

					nameArg = "\""+Utilities.getNameArgument(lhs)+"\"";
				}

				if (useAlternateImplementation) {

					theTail.append(InformationManagers.getInstance().getNamedSubscriptManager().getNumIndexFor(range.get(0)));
					theTail.append(",");

					// the first four arguments are placeholders and should not be processed
					// 1st and second arguments need to be updated if they refer to arrays as subscript
					// definition is created as we go and may referenced them before defined

					int numSkip = 4;
					for (int i = 0; i < numSkip; i++) {

						if (i == 1)
						{
							n.setToken(valueArg);
						} else if (i == 0) {
							n.setToken(nameArg);
						}

						n.setPlaceHolder(true);
						n = n.getNext();
					}

					// These two parameters need to be arrays that are generated in separate equations.
					// model this after how we generate code for it then else



					String name = "_da1";
					//			n.setResultsVariable(name);
					//			theTail.append(name);


					nameArg = "_da"+suffix1;
					n.setToken(nameArg);
					n.setPlaceHolder(true);
					theTail.append(nameArg);
					n = n.getNext();
					theTail.append(",");
					nameArg = "_da"+suffix2;
					n.setPlaceHolder(true);
					n.setToken(nameArg);
					theTail.append(nameArg);
					n = n.getNext();

					int param = 1;

					while(n != null) {
						if (param != 1)
							theTail.append(",");

						String t = getNextDouble();
						n.setResultsVariable(t);
						if (param != 1)
							theTail.append(t);
						n = n.getNext();
						param = 2;
					}

				} else {


					// the first four arguments are placeholders and should not be processed
					// 1st and second arguments need to be updated if they refer to arrays as subscript
					// definition is created as we go and may referenced them before defined

					int numSkip = 4;
					for (int i = 0; i < numSkip; i++) {
						if (i > 0)
							theTail.append(",");
						if (i == 1)
						{
							n.setToken(valueArg);
						} else if (i == 0) {
							n.setToken(nameArg);
						}
						theTail.append(n.getToken());
						n.setPlaceHolder(true);
						n = n.getNext();
					}

					// These two parameters need to be arrays that are generated in separate equations.
					// model this after how we generate code for it then else

					theTail.append(",");

					String name = "_da1";
					//			n.setResultsVariable(name);
					//			theTail.append(name);


					nameArg = "_da"+suffix1;
					n.setToken(nameArg);
					n.setPlaceHolder(true);
					theTail.append(nameArg);
					n = n.getNext();
					theTail.append(",");
					nameArg = "_da"+suffix2;
					n.setPlaceHolder(true);
					n.setToken(nameArg);
					theTail.append(nameArg);
					n = n.getNext();

					while(n != null) {

						theTail.append(",");

						String t = getNextDouble();
						n.setResultsVariable(t);
						theTail.append(t);
						n = n.getNext();
					}
				}


				theTail.append(");\n");


			} else if (node.getToken().equals("sdFunctions.SAMPLEIFTRUE")){
				theTail.append(node.getResultsVariable()+" = ");
				theTail.append(node.getToken()+"(");

				Node n = node.getChild();

				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				String lhs = currentGenerate.getLhs();
				String valueArg;
				String nameArg;
				if (ArrayReference.isArrayReference(lhs)) {

//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementationVectorMemory();
//						} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
//						}
//					} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).asMapGet();
//					}
//					if (nativeMemory()) {
						nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();
//					} else {
//						nameArg = new ArrayReferenceNative(lhs, currentGenerate).asMapName();	
//					}


				} else {

//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							String offset = memoryMapper.getOffset(lhs);
//							valueArg = "vectorMemory["+offset+"]";
							valueArg = memoryMapper.getDataStore(lhs)+"["+offset+"]";
//						} else {
//							valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);
//						}
//					} else {
//						valueArg = asGet(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));
//					}
//					nameArg = "\""+lhs+"\"";
					nameArg = "\""+Utilities.getNameArgument(lhs)+"\"";
				}


				// the first four arguments are placeholders and should not be processed
				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				for (int i = 0; i < 4; i++) {
					if (i > 0)
						theTail.append(",");
					if (i == 1)
					{
						n.setToken(valueArg);
					} else if (i == 0) {
						n.setToken(nameArg);
					}
					theTail.append(n.getToken());
					n.setPlaceHolder(true);
					n = n.getNext();
				}

				// need to deal with GET DATA BETWEEN TIMES as one of its parameters to be
				// passed is a string rather than a double!

				theTail.append(",");

				String b = getNextBoolean();
				n.setResultsVariable(b);
				theTail.append(b);
				n = n.getNext();

				while(n != null) {

					theTail.append(",");

					String t = getNextDouble();
					n.setResultsVariable(t);
					theTail.append(t);
					n = n.getNext();
				}


				theTail.append(");\n");
			} else if (node.getToken().equals("sdFunctions.GETDATABETWEENTIMES")){

				//		    /*
				//			Equation: RS PFC at start= INITIAL(GET DATA BETWEEN TIMES(RS PFC, Other emissions cut start time , 0))
				//			Units:tons/Year
				//			Comment: None Provided
				//		*/
				//		_t0 = sdFunctions.INITIAL(
				//			"RS PFC at start",
				//			valueOf("RS PFC at start"),
				//			time,
				//			timeStep,
				//				(sdFunctions.GETDATABETWEENTIMES(
				//					"RS PFC at start",
				//					valueOf("RS PFC at start"),
				//					time,
				//					timeStep,
				//					("RS PFC"),
				//					(valueOf("Other emissions cut start time")),
				//					(0))));
				//		setValue("RS PFC at start", _t0);

				theTail.append(node.getResultsVariable()+" = ");
				theTail.append(node.getToken()+"(");

				Node n = node.getChild();

				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				String lhs = currentGenerate.getLhs();
				String valueArg;
				String nameArg;
				
				if (ArrayReference.isArrayReference(lhs)) {

//					if (nativeMemory()) {
						valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
//					} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).asMapGet();
//					}
//					if (nativeMemory()) {
						nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();
//					} else {
//						nameArg = new ArrayReferenceNative(lhs, currentGenerate).asMapName();	
//					}


				} else {

//					if (nativeMemory()) {
						valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);
//					} else {
//						valueArg = asGet(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));
//					}
//					nameArg = "\""+lhs+"\"";
					nameArg = "\""+Utilities.getNameArgument(lhs)+"\"";
				}

//				valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);
//				nameArg = "\""+lhs+"\"";


				//		if (ArrayReference.isArrayReference(lhs)) {
				//		    valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
				//		    nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();
				//		} else {
				//		    valueArg = NativeDataTypeManager.getLegalName(lhs);
				//		    nameArg = "\""+lhs+"\"";
				//		}


				// the first four arguments are placeholders and should not be processed
				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				//		for (int i = 0; i < 4; i++) {
				//		    if (i > 0)
				//			theTail.append(",");
				//		    if (i == 1)
				//		    {
				//			n.setToken(valueArg);
				//		    } else if (i == 0) {
				//			n.setToken(nameArg);
				//		    }
				//		    theTail.append(n.getToken());
				//		    n.setPlaceHolder(true);
				//		    n = n.getNext();
				//		}

				// need to deal with GET DATA BETWEEN TIMES as one of its parameters to be
				// passed is a string rather than a double!

				//		theTail.append(",");

				String name = getNextString();
				n.setResultsVariable(name);
				theTail.append(name);

				String aRef = n.getToken();

				//		if (ArrayReference.isArrayReference(aRef)) {
				//		    nameArg = new ArrayReference(aRef).generateRHSName();
				//		} else {
				//		    nameArg = generateRHSName(aRef);
				//		}

				//		}   NEED TO CHECK THIS!


				n.setToken(nameArg);
				n = n.getNext();

				while(n != null) {

					theTail.append(",");

					String t = getNextDouble();
					n.setResultsVariable(t);
					theTail.append(t);
					n = n.getNext();
				}


				theTail.append(");\n");
			} else if (Parser.isFunctionInvocation(node.getToken())){


				FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(node.getToken());

				int numFunctionArgs = fd.getNumArgsAll();
				boolean providesInitialValue = fd.isSuppliesInitialValue();
				int processArgNum = 0;

				theTail.append(node.getResultsVariable()+" = ");
				theTail.append(node.getToken()+"(");

				Node n = node.getChild();

				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				String lhs = currentGenerate.getLhs();
				String valueArg;
				String nameArg;
				
				// if ELMCOUNT else  n is arg
				if (node.getToken().startsWith("sdFunctions.ELMCOUNT")) {
					// specialCase
					String subr = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(n.getToken());
					NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();
					int numSubs = nsm.getValuesFor(subr).size();
//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							valueArg = Integer.toString(numSubs);
//						} else {
//							valueArg = Integer.toString(numSubs);
//						}
//					} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).asMapGet();
//					}
//					if (nativeMemory()) {
						nameArg = subr;
//					} else {
//						nameArg = new ArrayReferenceNative(lhs, currentGenerate).asMapName();	
//					}

				} else if (ArrayReference.isArrayReference(lhs)) {

//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementationVectorMemory();
//						} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
//						}
//					} else {
//						valueArg = new ArrayReferenceNative(lhs, currentGenerate).asMapGet();
//					}
//					if (nativeMemory()) {
						nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();
//					} else {
//						nameArg = new ArrayReferenceNative(lhs, currentGenerate).asMapName();	
//					}


				} else {

//					if (nativeMemory()) {
//						if (newMemoryDefinition) {
							String offset = memoryMapper.getOffset(lhs);
//							valueArg = "vectorMemory["+offset+"]";
							valueArg = memoryMapper.getDataStore(lhs)+"["+offset+"]";
//						} else {
//							valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);
//						}
//					} else {
//						valueArg = asGet(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));
//					}
//					nameArg = "\""+lhs+"\"";
					nameArg = "\""+Utilities.getNameArgument(lhs)+"\"";
				}


				// the first four arguments are placeholders and should not be processed
				// 1st and second arguments need to be updated if they refer to arrays as subscript
				// definition is created as we go and may referenced them before defined

				boolean addComma = false;

				int numTransfer = 0;
				if (fd.isRequiresName()) {
					numTransfer++;
					n.setToken(nameArg);
					addComma = true;
					processArgNum++;
					theTail.append(n.getToken());
					n.setPlaceHolder(true);
					n = n.getNext();
				}
				if (fd.isRequiresValue() || fd.getExternalName().equals("ELMCOUNT")) {
					
					numTransfer++;
					n.setToken(valueArg);
					if (addComma) 
						theTail.append(",");
					addComma = true;
					processArgNum++;
					theTail.append(n.getToken());
					n.setPlaceHolder(true);
					n = n.getNext();
				}
				if (fd.isRequiresTime()) {
					numTransfer++;
					if (addComma) 
						theTail.append(",");
					addComma = true;
					processArgNum++;
					theTail.append(n.getToken());
					n.setPlaceHolder(true);
					n = n.getNext();
				}
				if (fd.isRequiresTimeStep()) {
					numTransfer++;
					if (addComma) 
						theTail.append(",");
					addComma = true;
					processArgNum++;
					theTail.append(n.getToken());
					n.setPlaceHolder(true);
					n = n.getNext();
				}
				//
				//			if (node.getToken().startsWith("sdFunctions.LOOKUP"))
				//				numTransfer = 0;

				//			for (int i = 0; i < numTransfer; i++) {
				//				processArgNum++;
				//				if (i > 0)
				//					theTail.append(",");
				//				if (i == 1)
				//				{
				//					n.setToken(valueArg);
				//				} else if (i == 0) {
				//					n.setToken(nameArg);
				//				}
				//				theTail.append(n.getToken());
				//				n.setPlaceHolder(true);
				//				n = n.getNext();
				//			}

				// need to deal with GET DATA BETWEEN TIMES as one of its parameters to be
				// passed is a string rather than a double!

				int cnt = 0;
				
				// this is horrible!
				if (node.getToken().equals("sdFunctions.ELMCOUNT"))
						n=null;

				while(n != null) {
					if (numTransfer > 0 || (numTransfer == 0 && cnt > 0))
						theTail.append(",");
					cnt++;

					String t;
//					if (node.getToken().equals("sdFunctions.ELMCOUNT") && !nativeMemory()) {
//						t = getNextString();
//					} else {
						t  = getNextDouble();
//					}
					n.setResultsVariable(t);
					if (!providesInitialValue) {
						theTail.append(t);
					} else {
						if ((processArgNum < numFunctionArgs-1) && !currentGenerate.isActiveInitial()) {
//							if (nativeMemory()) {
//								theTail.append("(time == getINITIALTIME() ? 0.0 : "+t+" )");
								theTail.append(t);
//							} else {
//								//							theTail.append("(get(\"time\") == getINITIALTIME() ? 0.0 : "+t+" )");
//								theTail.append(t);
//							}
						} else {
							theTail.append(t);
						}
					}
					processArgNum++;
					n = n.getNext();
				}


				theTail.append(");\n");

			} else {
				
				boolean isLookup = node.getToken().startsWith("lookup."); //  || currentGenerate.getVensimEquationOnly().contains("Lookup");
				
				// Either SCALAR or ARRAY Reference
				// the leaf node
				if (isArrayReference(node) ) { // those that will evaluate to a double
					if (node.getResultsVariable() != null) {
						if (node.getResultsVariable().startsWith("_s")) {
							theTail.append(node.getResultsVariable()+" = "+node.getToken()+";\n");
						} else {

//							if (nativeMemory()) {
								ArrayReferenceNative arn = new ArrayReferenceNative(node.getToken(), currentGenerate);
								if (node.getParent().getToken().startsWith("sdFunctions.VECTORRANK"))
									arn.setVector(true);
								if (/*newMemoryDefinition  &&*/ !isLookup) {
									
									theTail.append(node.getResultsVariable()+" = "+
											arn.generateRHSImplementationVectorMemory()+";\n");
								} else {
								theTail.append(node.getResultsVariable()+" = "+
										arn.generateRHSImplementation()+";\n");
								}
//							} else {
//								// might be a vector get (or lookup) or scalar, lookup and scalar handled in asMapGet
//								ArrayReferenceNative arn = new ArrayReferenceNative(node.getToken(), currentGenerate);
//								if (node.getParent().getToken().startsWith("sdFunctions.VECTORRANK"))
//									arn.setVector(true);
//								theTail.append(node.getResultsVariable()+" = "+arn.asMapGet()+";\n");
//							}

						}
					} else {
						System.out.println("PANIC!");
//						if (nativeMemory()) {
//							theTail.append(node.getResultsVariable()+" = "+new ArrayReferenceNative(node.getToken(), currentGenerate).generateRHSImplementation()+";\n");
//						} else {
//							theTail.append(node.getResultsVariable()+" = "+new ArrayReferenceNative(node.getToken(), currentGenerate).asMapGet()+";\n");
//						}

					}
				} else {
					// Note: we may actually attempt to assign a double to a boolean (nested ITE)
					// check the results variable -- it will be of the proper type

					if (node.getResultsVariable() == null) {
						System.out.println("HUH?");

					}

					boolean resultTypeDouble = node.getResultsVariable().startsWith("_t");
					boolean resultTypeBoolean = node.getResultsVariable().startsWith("_b");
					boolean resultString = node.getResultsVariable().startsWith("_s");

					String castToBoolean = "";
					if (resultTypeBoolean) {
						castToBoolean = " == 1";
					}

					isLookup = node.getToken().startsWith("lookup."); //  || currentGenerate.getVensimEquationOnly().contains("Lookup");
					

//					if (hashmapMemory()) {
//						if (resultTypeDouble || resultTypeBoolean) {
//
//
//
//							String v = InformationManagers.getInstance().getNativeDataTypeManager().
//									getOriginalName(node.getToken());
//
//							if (Parser.isNumber(v)) {
//								theTail.append(node.getResultsVariable()+
//										" = "+v+castToBoolean+";\n");
//							} else {
//								if (isLookup) {
//									theTail.append(node.getResultsVariable()+ " = "+asGetLookup(v+castToBoolean)+";\n");
//								} else {
//									if (node.getParent().getToken().startsWith("sdFunctions.VECTORRANK"))
//										theTail.append(node.getResultsVariable()+ " = "+asGetVector(v+castToBoolean)+";\n");
//									else
//									theTail.append(node.getResultsVariable()+ " = "+asGet(v+castToBoolean)+";\n");
//								}
//							}
//
//						} else {
//							String v = node.getToken();
//							if (Parser.isNumber(v)) {
//								theTail.append(node.getResultsVariable()+" = "+node.getToken()+";\n");
//							} else {
//								if (isLookup) {
//									theTail.append(node.getResultsVariable()+" = "+asGetLookup(node.getToken())+";\n");
//								} else {
//									// this is a horrible hack, please fix this? don't know if this will always work!
//									// MJB TEST!
//									if (v.startsWith("asExpandedArrayRef")) {
//										theTail.append(node.getResultsVariable()+" = "+node.getToken()+";\n");
//									} else	if (v.startsWith("memory.")) {
//											theTail.append(node.getResultsVariable()+" = "+asString(node.getToken())+";\n");
//									} else {
//										theTail.append(node.getResultsVariable()+" = "+asGet(node.getToken())+";\n");
//									}
//								}
//							}
//
//						} // end res
//					} else {

						if (resultTypeDouble || resultTypeBoolean) {

							

							if (/*newMemoryDefinition && */ !isLookup) {
								// what if this is a subscript being used as a variable?
								String origName = InformationManagers.getInstance().getNativeDataTypeManager().
										getOriginalName(node.getToken());
								if (InformationManagers.getInstance().getArrayManager().isNamedSubscript(origName)) {
									// MJB 4/12/16 This needs to be changed to lookup proper subscript
									String vLHS = currentGenerate.getOriginalLhs();
									String arrayName = new ArrayReference(vLHS).getArrayName();
									int pos = InformationManagers.getInstance().getArrayManager().getSubscriptPosition(arrayName, origName);
									theTail.append(node.getResultsVariable()+" = "+"outer"+pos+castToBoolean+";\n");
								} else if (!Parser.isNumber(node.getToken())) {
									
									// this is a hack -- need to check for all possible vensim variables
									String t= node.getToken();
									if (t.toUpperCase().equals("TIME") || t.toUpperCase().equals("MEMORY.TIME")) {
										theTail.append(node.getResultsVariable()+
												" = "+"time"+castToBoolean+";\n");
									} else {
										String offset = memoryMapper.getOffset(InformationManagers.getInstance().getNativeDataTypeManager().
												getLegalName(node.getToken()));
										//								String v = "vectorMemory["+offset+"]";
										String v = memoryMapper.getDataStore(InformationManagers.getInstance().getNativeDataTypeManager().
												getLegalName(node.getToken()))+"["+offset+"]";
										theTail.append(node.getResultsVariable()+
												" = "+v+castToBoolean+";\n");
									}
								} else {
									String v = InformationManagers.getInstance().getNativeDataTypeManager().
											getLegalName(node.getToken());
									theTail.append(node.getResultsVariable()+
											" = "+v+castToBoolean+";\n");
								}
							} else {
								
								// need a genMatrix
								
								String v = InformationManagers.getInstance().getNativeDataTypeManager().
										getLegalName(node.getToken());
//								theTail.append(node.getResultsVariable()+" = genMatrix(vectorMemory, "+memoryMapper.getOffset(v)+", row, col);\n");
								
								
								theTail.append(node.getResultsVariable()+
										" = "+v+castToBoolean+";\n");
							}

						} else {
							String v = node.getToken();

							theTail.append(node.getResultsVariable()+" = "+node.getToken()+";\n");


						} // end res

					
				}
			}
		}

		if (node.getChild() != null) 
			generateRHSCode(node.getChild());
		if (node.getNext() != null) 
			generateRHSCode(node.getNext());
		return true;
	}
	
	private String asString(String originalName) {
		
		return "\""+InformationManagers.getInstance().getNativeDataTypeManager().getOriginalNameRemovingAnnotation(originalName)+"\"";
	}

	private String asGet(String originalName) {
		return "get(\""+originalName+"\")";
	}
	
	private String asGetVector(String originalName) {
		return "getVector(\""+originalName+"\")";
	}

	private String asGetLookup(String originalName) {
		return "getLookup(\""+originalName+"\")";    	
	}


	private String generateExpression(Node node) {
		StringBuilder sb = new StringBuilder();
		node.setPlaceHolder(true);

		// node is the root node of an expression
		// if an operator:
		// left child
		// operator
		// right child
		if (Parser.isArithmeticOperator(node.getToken())) {
			sb.append(generateExpression(node.getChild()));
			sb.append(node.getToken());
			sb.append(generateExpression(node.getChild().getNext()));
			return sb.toString();
			// if an array reference, generate rhs
		} else if (ArrayReference.isArrayReference(node.getToken())) {

//			if(nativeMemory()) {
//				if (newMemoryDefinition) {
					sb.append(new ArrayReferenceNative(node.getToken(), currentGenerate).generateRHSImplementationVectorMemory());
//				} else {
//					sb.append(new ArrayReferenceNative(node.getToken(), currentGenerate).generateRHSImplementation());
//				}
//			} else {
//				sb.append(new ArrayReferenceNative(node.getToken(), currentGenerate).asMapGet());
//			}

			return sb.toString();
		} else if (node.getToken().startsWith("sdFunctions.LOOKUP")) {
			sb.append("/* MJB Requires dealing with LOOKUP access!*/");

			sb.append("   ");
			sb.append(generateLOOKUP(node));
			//	    if (Translator.useNativeDataTypes) 
			//		sb.append(new ArrayReferenceNative(node.getToken(), currentGenerate).generateRHSImplementation());
			//	    else
			//		sb.append(new ArrayReference(node.getToken()).generateRHSImplementation());
			return sb.toString();
			// else this should just be variable
		} else {
			//			sb.append(node.getToken());

			String valueArg;

			
//				if (newMemoryDefinition) {
					String offset = memoryMapper.getOffset(node.getToken());
					//					valueArg = "vectorMemory["+offset+"]";
					valueArg = memoryMapper.getDataStore(node.getToken())+"["+offset+"]";
//				} else {
//					valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(node.getToken());
//				}
			
			sb.append(valueArg);
		}

		return sb.toString();
	}

	private String generateLOOKUP(Node node) {
		StringBuilder theTail = new StringBuilder();
		theTail.append(node.getToken()+"(");

		Node n = node.getChild();

		// 1st and second arguments need to be updated if they refer to arrays as subscript
		// definition is created as we go and may referenced them before defined

		String lhs = currentGenerate.getLhs();
		String valueArg;
		String nameArg;
		if (ArrayReference.isArrayReference(lhs)) {

			valueArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSImplementation();
			nameArg = new ArrayReferenceNative(lhs, currentGenerate).generateRHSName();


		} else {

			valueArg = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(lhs);

//			nameArg = "\""+lhs+"\"";
			nameArg = "\""+Utilities.getNameArgument(lhs)+"\"";
		}


		// the first four arguments are placeholders and should not be processed
		// 1st and second arguments need to be updated if they refer to arrays as subscript
		// definition is created as we go and may referenced them before defined

		int numTransfer = 4;

		if (node.getToken().startsWith("sdFunctions.LOOKUP"))
			numTransfer = 0;

		int cnt = 0;

		while(n != null) {
			if (numTransfer == 4 || (numTransfer == 0 && cnt > 0))
				theTail.append(",");
			cnt++;

			String t = getNextDouble();
			n.setResultsVariable(t);
			theTail.append(generateExpression(n));
			n = n.getNext();
		}


		theTail.append(")\n");



		return theTail.toString();
	}

	private String generateRHSName(String valueOfRef) {
		return valueOfRef.replace("valueOf(", "").replace(")", "");
	}

	private Node getCONDITION(Node node) {

		// skip the first four and return # 5
		return node.getChild(); //.getNext().getNext().getNext().getNext();
	}

	private Node getTHEN(Node node) {
		// skip the first four and return # 6
		return node.getChild().getNext(); // .getNext().getNext().getNext().getNext();

	}

	private Node getELSE(Node node) {
		// skip the first four and return # 7
		return node.getChild().getNext().getNext(); // .getNext().getNext().getNext().getNext();

	}

	//	private List<Node> getIFTHENELSE(Node node) {
	//	    List<Node> al = new ArrayList<Node>();
	//	    
	//	    if (node == null)
	//		return al;
	//	    
	//	    if (node.getToken().startsWith("sdFunctions.IFTHENELSE")) {
	//		al.add(node);
	//	    }
	//	    
	//	    List<Node> more = getIFTHENELSE(node.getChild());
	//	    if (more.size() > 0)
	//		al.addAll(more);
	//	    
	//	    more = getIFTHENELSE(node.getNext());
	//	    if (more.size() > 0)
	//		al.addAll(more);
	//	    
	//	    return al;
	//	}

	private boolean isLeaf(Node node) {
		// this should never happen, I think
		if (node == null)
			return true;

		if (node.getChild() == null)
			return true;
		//	    else if (node.getChild().isDeleted())
		//		return true;
		//	    else
		return false;
	}

	private boolean isTerminal(Node node) {

		if (node == null)
			return true;

		if (node.getToken().startsWith("sdFunctions.IFTHENELSE"))
			return false;

		return isTerminal(node.getChild()) && isTerminal(node.getNext());
	}


	//	private void execute(BufferedWriter bw) {
	//		
	//		try {
	//			bw.append("public void execute(BufferedWriter bw) {\n\n");
	//			bw.append("    oneTime();\n");
	//			
	//			bw.append("    for (double time = getINITIALTIME(); time <= getFINALTIME(); time += getTIMESTEP()) {\n" +
	//				"         Data.setCurrentTime(time);\n" +
	//				"         currentTime = time;\n" +
	//				"         repeated(time, getTIMESTEP());\n" +
	//				"         reportTimeStep(bw, time);\n");
	//			
	//			bw.append("    }\n");
	//			
	//			bw.append("}\n\n");
	//			bw.flush();
	//			
	//		} catch (IOException e) {
	//			// TODO Auto-generated catch block
	//			e.printStackTrace();
	//		}
	//		
	//	}

//	private void report(BufferedWriter bw) {
//		try {
//
//			bw.append("public BufferedWriter report(String filename) {\n\n");
//			bw.append("    if (!trace.isTrace() || report == null)\n");
//			bw.append("       return null;\n");
//			bw.append("  BufferedWriter bw = null;\n");
//			bw.append("  try {\n");
//			bw.append("    bw = Utilities.openFileForWriting(filename);\n\n");
//
//			ArrayList<String> sortOrder = new ArrayList<String>(evaluationOrderOneTime);
//			sortOrder.addAll(evaluationOrderRepeated);
//			Collections.sort(sortOrder);
//
//			// generate the column headers
//			bw.append("    bw.append(\"time\");");
//			for (String lhs : sortOrder) {
//				if (!equations.get(lhs).getCleanEquation().contains("="))
//					continue;
//				if (equations.get(lhs).isHasLHSArrayReference()) {
//
//				} else {
//					if (equations.get(lhs).isRepeated()) {
//						//					bw.append("    bw.append(\",\");\n");
//						bw.append("    bw.append(\","+lhs+"\");\n");
//					} else {
//						//					bw.append("    bw.append(\",\");\n");
//						bw.append("    bw.append(\","+lhs+"\");\n");
//					}
//				}
//			}
//			bw.append("    bw.append(\"\\n\");\n");
//
//			bw.append("    bw.flush();\n");
//
//
//			bw.append("  } catch (IOException e) {\n");
//			bw.append("    e.printStackTrace();\n");
//			bw.append("  }\n");
//			bw.append("    return bw;\n\n");
//			bw.append("  }\n");
//			// generate the data
//
//			bw.append("public void reportTimeStep(double time) {\n");
//			bw.append("    if (!trace.isTrace() || report == null)\n");
//			bw.append("       return;\n");
//			bw.append("    int i = (int) ((currentTime - getINITIALTIME())/getTIMESTEP());\n");
//
//			// this should be based on SAVPER
//
//			bw.append("BufferedWriter bw = report;\n");
//			bw.append("  try {\n");
//			bw.append("    bw.append(Double.toString(currentTime));\n");
//			for (String lhs : sortOrder) {
//
//				if (!equations.get(lhs).getCleanEquation().contains("="))
//					continue; if (equations.get(lhs).isHasLHSArrayReference()) {
//
//					} else {
//						bw.append("    bw.append(\",\"+Double.toString(valueOf(\""+lhs+"\", i)));\n");    // get "lhs"_history.get(i);
//					}
//
//			}
//			bw.append("    bw.append(\"\\n\");\n");
//			bw.append("    i++;\n");
//
//			//			bw.append("}\n\n");
//			bw.append("    bw.flush();\n");
//			//			bw.append("    bw.close();\n");
//			bw.append("  } catch (IOException e) {\n");
//			bw.append("    e.printStackTrace();\n");
//			bw.append("  }\n");
//			bw.append("}\n\n");
//			bw.flush();
//
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//
//	}

	public void runner(String srcDir) {

		BufferedWriter bw = Utilities.openFileForWritingCharset(srcDir+objectName+"_runner.java", 
				SDMDocLanguageSpecific.getInstance().getFileCharSet());

		try {


			bw.append("package "+packageName+";\n\n");


			bw.append("public class "+objectName+"_runner  {\n");
			bw.append("public "+objectName+"_runner(String name, String[] args) {}\n");
			bw.append("public static void main(String[] args) {\n");
			bw.append("new "+objectName+"(\""+objectName+"\", args).sdmuiExecute(\"run1\", \"somedir\");\n");
			bw.append("}\n");

			bw.append("}\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void endObject(BufferedWriter bw) {
		try {

//			if (nativeMemory()) {
				bw.append("public Memory"+objectName+" getMemory() {\n");
				bw.append("return memory;\n");
				bw.append("}\n\n");
//			} else {
//
//			}

//			if (target.equals(ReaderConstants.JAVA)) {
//				bw.append("@Override\n");
//				bw.append("public double getINITIALTIME() {\n");
//				if (nativeMemory()) {
//					bw.append("return memory.getINITIALTIME();\n");
//				} else {
//					bw.append("return get(\"INITIAL TIME\");\n");
//				}
//				bw.append("}\n");
//				bw.append("@Override\n");
//				bw.append("public double getFINALTIME() {\n");
//				if (nativeMemory()) {
//					bw.append("return memory.getFINALTIME();\n");
//				} else {
//					bw.append("return get(\"FINAL TIME\");\n");
//				}
//				bw.append("}\n");
//				bw.append("@Override\n");
//				bw.append("public double getTIMESTEP() {\n");
//				if (nativeMemory()) {
//					bw.append("return memory.getTIMESTEP();\n");
//				} else {
//					bw.append("return get(\"TIME STEP\");\n");
//				}
//				bw.append("}\n");
//			}


			bw.append("}\n\n");
			bw.flush();
			//			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void endObjectC(BufferedWriter bw) {
		try {

			// don't think we need this.
			//	    bw.append("}\n\n");
			bw.flush();
			//			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private String getNextBoolean() {
		String next = "_b"+ booleanUsed;
		if (booleanUsed > this.maxBoolean) {
			maxBoolean = booleanUsed;
			//	    try {
			//		if (Translator.target.equals(ReaderConstants.C))
			//		    sourceCode.append("bool "+next+" = false;\n");
			//		else
			//		    sourceCode.append("boolean "+next+" = false;\n");
			//	    } catch (IOException e) {
			//		// TODO Auto-generated catch block
			//		e.printStackTrace();
			//	    }
		}
		booleanUsed++;
		return  next;
	}

	// write temps later not npw.
	// traverse tree and if not placeholder, generate the code for results variable.
	private String getNextDouble() {
		String next = "_t"+ doubleUsed;
		if (doubleUsed > this.maxTemp) {
			maxTemp = doubleUsed;
			//	    try {
			//		sourceCode.append("double "+next+" = 0.0;\n");
			//	    } catch (IOException e) {
			//		// TODO Auto-generated catch block
			//		e.printStackTrace();
			//	    }
		}
		doubleUsed++;
		return next;
	}

	private String getNextInt() {

		String next = /* "_i"+ */ Integer.toString(intUsed);
		if (intUsed > this.maxTempInt) {
			maxTempInt = intUsed;
			//	    try {
			//		sourceCode.append("int "+next+" = 0;\n");
			//	    } catch (IOException e) {
			//		// TODO Auto-generated catch block
			//		e.printStackTrace();
			//	    }
		}
		intUsed++;
		return next;
	}

	private String getNextString() {
		String next = "_s"+ stringUsed;
		if (stringUsed > this.maxString) {
			maxString = stringUsed;
			//	    try {
			//		sourceCode.append("String "+next+" = \"\";\n");
			//	    } catch (IOException e) {
			//		// TODO Auto-generated catch block
			//		e.printStackTrace();
			//	    }
		}
		stringUsed++;
		return next;
	}

	private void resetCounters() {
		resetLimits();
		booleanUsed = 0;
		doubleUsed = 0;
		stringUsed = 0;
		intUsed = 0;


	}

	private void resetLimits() {
		maxTemp = -1;
		maxTempInt = -1;
		maxBoolean = -1;
		maxString = -1;
		indexArraysPacked.clear();
	}

	public static boolean indexDefined(String pack) {
		return indexArraysPacked.contains(pack);
	}

	public static void defineIndex(String pack) {
		indexArraysPacked.add(pack);
	}


	private Map<Node, String> genRHSStatements(Node node) {
		List<Node> arrayReferenceNodes = getArrayReferences(node);

		Map<Node, String> rhsStatements = new LinkedHashMap<Node, String>();

		Node lhsNode = null;;
		lhsNode = getLHS(node);

		for (Node n : arrayReferenceNodes) {
			if (!n.equals(lhsNode)) {

				rhsStatements.put(n, new ArrayReferenceNative(n.getToken(), currentGenerate).generateRHSImplementation());

			}
		}

		return rhsStatements;
	}

	public BufferedWriter getSourceCode() {
		return sourceCode;
	}

	public void setSourceCode(BufferedWriter sourceCode) {
		this.sourceCode = sourceCode;
	}

	public static String scrub(String lineOfCode) {
		if (isGenerateJava())
			return lineOfCode;
		else if (isGenerateC())
			return lineOfCode.replace("memory.", "").
					replace("sdFunctions.", "").
					replace("array.", "").
					replace("lookup.", "").
					replace("Double.MAX_VALUE", "DBL_MAX").
					replace("Double.MIN_VALUE", "DBL_MIN").
					replace("/* holder1 */", "1").
					replace("/* holder2 */", "1").
					replace("schedule.", "").
					replace("Math.pow", "pow");
		else
			return lineOfCode;

	}

	private static boolean isGenerateC() {
		// TODO Auto-generated method stub
		return false;
	}


	private static boolean isGenerateJava() {
		// TODO Auto-generated method stub
		return true;
	}


	public static String scrubC(String lineOfCode) {
		return lineOfCode.replace("memory.", "").
				replace("sdFunctions.", "").
				replace("array.", "").
				replace("lookup.", "").
				replace("Double.MAX_VALUE", "DBL_MAX").
				replace("Double.MIN_VALUE", "DBL_MIN").
				replace("/* holder1 */", "1").
				replace("/* holder2 */", "1").
				replace("schedule.", "").
				replace("Math.pow", "pow");

	}



//	private boolean isGenerateC() {
//		return target.equals("C");
//	}
//
//	private boolean isGenerateJava() {
//		return !target.equals("C");
//	}

//	private boolean nativeMemory() {
//		return memoryImplementation.equalsIgnoreCase("Native");
//	}

//	private boolean hashmapMemory() {
//		return !nativeMemory();
//	}

	public void performUnitsConsistencyCheck() {
		//	UnitsManager.performUnitsConsistencyCheck(evaluationOrder, equations, "./ConsistencyResults.xml");
	}

//
//	/**
//	 * @return the repastSimphony
//	 */
//	public boolean isRepastSimphony() {
//		return repastSimphony;
//	}
//
//
//	/**
//	 * @return the rsHybrid
//	 */
//	public boolean isRsHybrid() {
//		return rsHybrid;
//	}
//




}
