package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.CommentProcessor;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class CommentCleaner {

	static Logger log = Logger.getLogger(SDM_Doc.class.getSimpleName());
	
	private final static String START_TAG = "\t~";
	private final static String EQN_END_TAG = "\t|";
	private final static String EQN_END_TAG2 = START_TAG+"\t"+EQN_END_TAG;
	private final static String GRAPHICS_START = "\\\\\\---/// Sketch information";
	private final static String MACRO_START = ":MACRO:";
	private final static String MACRO_END = ":END OF MACRO:";
	private final static String STARS = "******";
	
	private final static int MAX_COMMENT_LENGTH = 80; // the point at which we continue a single line comment
	
	private final static String SUPPLEMENTARY = ":SUPPLEMENTARY";
	
	private final static String COMMENT_START = "\t~\t";
	
	private final static String LINE_START = "\t\t";
	
	private final static String VARIABLE = "_Variable_";
	private final static String START_COMMENT = "_CommentStart_";
	private final static String END_COMMENT = "_CommentEnd_";
	
	

	private Map<String, List<String>> variableCommentsMap = new LinkedHashMap<String, List<String>>();
	
	
	private List<String> mdlContents;

	private String docxFile;
	private String vensimInputFile;
	private String vensimOutputFile;
	private boolean generateDocx;
	
	private boolean groupBy = false;

	public CommentCleaner(String mode, String file1, String file2, boolean groupBy) {
		
			generateDocx = true;
			vensimInputFile = file1;
			docxFile = file2;
			this.groupBy = groupBy;
	}
	
	public CommentCleaner(String mode, String file1, String file2, String file3) {

			generateDocx = false;
			vensimInputFile = file1;
			docxFile = file2;
			vensimOutputFile = file3;
	}


	public static void main(String[] args) {
		CommentCleaner cleaner = null;
		if (args.length == 3) {
			cleaner = new CommentCleaner(args[0], args[1], args[2], false);
		} else {
			cleaner = new CommentCleaner(args[0], args[1], args[2], args[3]);
		}
		if (cleaner != null)
			cleaner.execute();
	}

	public boolean execute() {

		initialize();

		if (generateDocx)
			return executeGenerateDocx();
		else
			return executeGenerateVensim();
	}

	private boolean executeGenerateDocx() {
		
		// read and parse mdl file
		mdlContents = Utilities.getAsListUTF8(vensimInputFile);
		mdlContents = removeContinuation(mdlContents);
		
//		print(mdlContents);
		
		return generateDocx();

	}

	private boolean executeGenerateVensim() {
		
		// read and parse mdl file
		mdlContents = Utilities.getAsListUTF8(vensimInputFile);
//		mdlContents = removeContinuation(mdlContents);
//		print(mdlContents);
		
		return generateVensim();
	}

	private void initialize() {



	}
	
	private List<String> removeContinuation(List<String> withContinuation) {
		List<String> noContinuation = new ArrayList<String>();
		
		MutableInteger ptr = new MutableInteger();
		
		while (ptr.value() < mdlContents.size()) {
			String line = getLine(ptr, mdlContents);
			noContinuation.add(line);
			ptr.add(1);
		}
		
		return noContinuation;
	}
	
	
	protected boolean generateDocx() {
		
		List<String> forDocx = extractComments();

		CommentProcessor cp = new CommentProcessor(forDocx, docxFile);
		cp.stringToDocx();
		return true;
	}
	
	private List<String> extractComments() {
		List<String> forDocx = new ArrayList<String>();
		
		MutableInteger ptr = new MutableInteger();
		
		while (ptr.value() < mdlContents.size()) {
			String line = getLine(ptr, mdlContents); // mdlContents.get(ptr.value());
			
			if (line.startsWith(GRAPHICS_START)) {
//				print(forDocx);
//				return forDocx;
				return groupBy();
			} else if (line.startsWith(MACRO_START)) {
				skipToLineStartingWith(MACRO_END, ptr, mdlContents);
			} else if (line.startsWith(STARS)) {
				skipToNextBlock(ptr, mdlContents);
			} else if (line.startsWith("{UTF")) {
				ptr.add(1);
			} else if (line.trim().equals("")) {
					ptr.add(1);
			} else {

				String varName = line;

				if (line.contains(":=")) {
					varName = line.split(":=")[0].trim();
				} else if (line.contains("=")) {
					varName = Parser.extractLHS(line);
				}  else if (line.contains("(")) {
					varName = line.split("\\(")[0].trim();
				} else if (line.endsWith(":")) {
					varName = line.replace(":","").trim();
				}
				
				List<String> comments = extractComment(ptr, mdlContents);
				variableCommentsMap.put(varName, comments);

				skipToNextBlock(ptr, mdlContents);
			}
			
		}
		

//		print(forDocx);
		return groupBy();
//		return forDocx;
		
	}
	
	private List<String> sortVariableNames(List<String> vars) {
		
		Map<String, String> nonQuotedOrigMap = new LinkedHashMap<String, String>();
		Map<String, String> origNonQuotedMap = new LinkedHashMap<String, String>();
		
		for (String var : vars) {
			String noQuotes =  Parser.removeQuotes(var); // var.replace("\"", "");
			nonQuotedOrigMap.put(noQuotes.toLowerCase(), var);
			origNonQuotedMap.put(var, noQuotes);
		}
		
		List<String> sortedVars = new ArrayList<String>(nonQuotedOrigMap.keySet());
		
		List<String> sorted = new ArrayList<String>();
		
		Collections.sort(sortedVars);
		
		for (String v : sortedVars) {
			sorted.add(nonQuotedOrigMap.get(v));
		}
		
		
		return sorted;
	}
	
	private List<String> groupBy() {
		List<String> forDocx = new ArrayList<String>();
		List<String> vars = new ArrayList<String>(variableCommentsMap.keySet());
		
		Collections.sort(vars);
		vars = sortVariableNames(vars);
		
		for (int pass = 0; pass < 2; pass++) {
			for (int i = 0; i < vars.size(); i++) {
				String varName = vars.get(i);
				boolean add = false;
				List<String> comments = variableCommentsMap.get(varName);
				if (pass == 0) {
					// if groupBy, only those with comments
					if (groupBy && comments.size() > 0)
						add = true;
				} else {
					// if groupBy, only those without comments or
					// if !groupBy
					if (groupBy && comments.size() == 0 ||
							!groupBy)
						add = true;
				}
				if (add) {
					forDocx.add(VARIABLE);
					forDocx.add(varName);
					forDocx.add(START_COMMENT);
					comments = variableCommentsMap.get(varName);

						forDocx.addAll(comments);

					forDocx.add(END_COMMENT);
				}
			}
		}

		return forDocx;
	}
	
	private void skipToLineStartingWith(String text, MutableInteger ptr, List<String> mdl, List<String> newMdl) {
		while (!mdl.get(ptr.value()).equals(text)) {
			newMdl.add(mdl.get(ptr.value()));
				ptr.add(1);
		}
		ptr.add(1);
	}
	
	private void skipToLineStartingWith(String text, MutableInteger ptr, List<String> mdlContents) {
		while (!mdlContents.get(ptr.value()).equals(text)) {
			ptr.add(1);
		}
		ptr.add(1);
	}
	
	private void skipToNextBlock(MutableInteger ptr, List<String> mdl, List<String> newMdl) {
		// ptr is pointing to something we don't care about
		// simply skip to the first non-blank line after EQN_END_TAG
//		System.out.println("SKIP");
		try {
			while (ptr.value() < mdl.size() && !mdl.get(ptr.value()).equals(EQN_END_TAG) &&
					 !mdl.get(ptr.value()).startsWith(EQN_END_TAG2) ) {
//				System.out.println(mdl.get(ptr.value()));
				newMdl.add(mdl.get(ptr.value()));
				ptr.add(1);
			}
			// this is \t~
			if (ptr.value() >= mdl.size())
				return;
			newMdl.add(mdl.get(ptr.value()));
//			System.out.println(mdl.get(ptr.value()));
			ptr.add(1);
			if (ptr.value() >= mdl.size())
				return;
//			System.out.println(mdl.get(ptr.value()));
			newMdl.add(mdl.get(ptr.value()));
			ptr.add(1);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void skipToNextBlock(MutableInteger ptr, List<String> mdlContents) {
		// ptr is pointing to something we don't care about
		// simply skip to the first non-blank line after EQN_END_TAG
		while (!mdlContents.get(ptr.value()).equals(EQN_END_TAG) &&
				 !mdlContents.get(ptr.value()).startsWith(EQN_END_TAG2)) {
			ptr.add(1);
		}
		// this is \t~
		ptr.add(1);
//		if (!mdlContents.get(ptr.value()).trim().equals(""))
//			System.out.println("HUH? "+mdlContents.get(ptr.value()));
		ptr.add(1);
	}
	
	protected boolean generateVensim() {
		// get variable names and comments
		// replace existing comments
		// write vensim files
		
		variableCommentsMap = loadCleanComments();
		addContinuationToComments();
		List<String> newMdl = merge(mdlContents, variableCommentsMap);
		
		print(newMdl);
		
		Utilities.writeToFileUTF8(vensimOutputFile, newMdl);
		
		return true;
	}
	
	private void addContinuationToComments() {
		
		for (String key : variableCommentsMap.keySet()) {
			List<String> comments = variableCommentsMap.get(key);
			
			List<String> withContinuation = addContinuationToComments(comments);
			variableCommentsMap.put(key, withContinuation);
		}
		
	}
	
	private List<String> addContinuationToComments(List<String> comments) {
		List<String> withContinuation = new ArrayList<String>();
		for (String c : comments) {
			if (c.length() < MAX_COMMENT_LENGTH) {
				withContinuation.add(c);
			} else {
				withContinuation.addAll(split(c));
			}
		}
		
		return withContinuation;
	}
	
	private List<String> split(String comment) {
		List<String> splitComment = new ArrayList<String>();
		
		MutableInteger start = new MutableInteger();
		MutableInteger end = new MutableInteger(comment.length());
		
		while (end.value()-1 >= start.value()) {
			splitComment.add(getChunk(comment, start, end));
		}
		
		return splitComment;
	}
	
	private String getChunk(String s, MutableInteger start, MutableInteger end) {
		
		int numchars = Math.min(MAX_COMMENT_LENGTH, end.value() - start.value());
		String chunk = s.substring(start.value(), start.value() + numchars);
		start.add(numchars);
		return chunk + (start.value() >= end.value() ? "" : "\\");
	}
	
	private Map<String, List<String>> loadCleanComments() {
		Map<String, List<String>> cleanComments = new LinkedHashMap<String, List<String>>();
		CommentProcessor cp = new CommentProcessor(docxFile);
		List<String> comments = cp.docxToString();
//		print(comments);
		
		List<String> processedText = null;
    	
    	int i = 0;
		
		while (i < comments.size()) {
    		String line = comments.get(i).replace(">>>", "");
    		
    		// beginning of a new varaible/comment combo
    		// grab tag and variable name
    		if (line.startsWith("_Variable_")) {    			
    			i++;
    			String varName = comments.get(i++).replace(">>>", "");
    			processedText = new ArrayList<String>();
    			cleanComments.put(varName, processedText);
    			// start of a comment
    			// tag, concat all text, tag
    		} else if (line.startsWith("_CommentStart_")) {
    			i++;
    			line = comments.get(i++).replace(">>>", "");
    			while(!line.equals("_CommentEnd_")) {
    				processedText.add(line);
    				line = comments.get(i++).replace(">>>", "");
    			}
    		} else {
//    			System.out.println("Huh?");
    			i++;
    		}
    	}
		
		return cleanComments;
	}

	
	private void print(List<String> mdlContents) {
//		for (String s : mdlContents)
//			System.out.println("<"+s+">");
	}
	
	private List<String> merge(List<String> originalMdl, Map<String, List<String>> clean) {
		List<String> newMdl = new ArrayList<String>(originalMdl);
		
		for (String v : clean.keySet()) {
			newMdl = replaceComment(newMdl, v, clean.get(v));
		}
		
		return newMdl;
	}
	
	private List<String> extractComment(MutableInteger ptr, List<String> mdl) {
		List<String> newMdl = new ArrayList<String>();
		String oldLine;
		
		// we are pointing at the line that contains the variable name
		
		// now locate the start of the comment for this variable
		// start tag is \t~
		// first start tag for units
		// then start tag for comments
		// then either \t| to terminate equation
		// or \t~ to terminate comment
		// note: if the comment line ends with a \, concatenate with next line
		// if not it is a new line
		
		// consume the equation
		
		oldLine = mdl.get(ptr.value());
		
		while(!oldLine.startsWith(START_TAG)) {
			ptr.add(1);
			oldLine = mdl.get(ptr.value());
		}
		
		// we have located the units
		// consume the units
		// this can be one line or more -- need to peek at next line
		String nextLine = mdl.get(ptr.value()+1);
		if (nextLine.startsWith(START_TAG)) {
			// this is a one line unit
			// point to beginning of comment
			ptr.add(1);
		} else {
			// multiple lines -- consume them
			ptr.add(1);
			oldLine = mdl.get(ptr.value());
			while(!oldLine.startsWith(START_TAG)) {
				ptr.add(1);
				oldLine = mdl.get(ptr.value());
			}
		}
		
		// ptr points to the beginning of the comment
		
//		ptr.add(1);
//		oldLine = mdl.get(ptr.value());
//		while(!oldLine.startsWith(START_TAG)) {
//			ptr.add(1);
//			oldLine = mdl.get(ptr.value());
//		}
		
		// grab the comment
		
		oldLine = getLine(ptr, mdl); // new String(oldLine.replace(COMMENT_START, ""));
		
		// NOTE: we may have START_TAG,START_TAG:SUMMPLEMENTARY -> which is no comment and
		
		if (oldLine.startsWith(COMMENT_START+COMMENT_START)) {
			return newMdl;
		}
		
		// oldLine can be a terminator if no comment has been specified
		
		if (!oldLine.startsWith(EQN_END_TAG2)) {

			newMdl.add(oldLine.replace(COMMENT_START, ""));
			ptr.add(1);
			oldLine = getLine(ptr, mdl);

			while(!oldLine.startsWith(START_TAG) && !oldLine.startsWith(EQN_END_TAG)  && !oldLine.startsWith(EQN_END_TAG2)) {
				newMdl.add(oldLine.replace(LINE_START, ""));
				ptr.add(1);
				oldLine = getLine(ptr, mdl);
			}
		}
		
		return newMdl;
	}
	
	private String getLine(MutableInteger ptr, List <String> mdl) {
		// ptr pointing to current line
		// return with pointer set to the last line number associated with this line (i.e. the last continued line)
		StringBuilder sb = new StringBuilder();

		String line = mdl.get(ptr.value());
		if (line.endsWith("\\")) {

			while (line.endsWith("\\")) {
				sb.append(line.replaceAll("\\\\$", ""));
				ptr.add(1);
				line = mdl.get(ptr.value());
			}
		}
		if (line.startsWith("\t\t")) {
			line = line.replaceFirst("\t\t", "");
		}
		sb.append(line);
//		ptr.add(1);
		return sb.toString();
	}
	
	private List<String> replaceComment(List<String> mdl, String lhs, List<String> comments) {
		List<String> newMdl = new ArrayList<String>();
		
		
//		System.out.println("lhs: "+lhs);
//		for (String c : comments) {
//			System.out.println("   "+c);
//		}
		
		// first we need to locate the lhs
		// NOTE: The lhs is the complete variable name. Due to continuation, the variable name could
		// be spread across multiple lines.
		
		MutableInteger oldPtr = new MutableInteger();
		int newPtr = 0;
		MutableInteger ptr = new MutableInteger();
		
		// get started 
		// transfer until we get to lhs -- oldPtr will point to the start of the equation
		String oldLine = mdl.get(oldPtr.value());
		ptr.setValue(oldPtr.value());
		String effectiveOldLine = getLine(ptr, mdl);
		
		// this needs more intelligence
		// must be able to jump to end of equations
		
		boolean done = false;
		
		while (!done) {
			// if macro definition just transfer
			if (effectiveOldLine.startsWith(MACRO_START)) {
				skipToLineStartingWith(MACRO_END, oldPtr, mdl, newMdl);
			} else if (effectiveOldLine.startsWith(STARS)) {
				skipToNextBlock(oldPtr, mdl, newMdl);
			} else if (effectiveOldLine.startsWith("{UTF")) {
				newMdl.add(effectiveOldLine);
				oldPtr.add(1);
			} else if (effectiveOldLine.trim().equals("")) {
				newMdl.add(effectiveOldLine);
				oldPtr.add(1);
			} else {

				String varName = effectiveOldLine;

				if (effectiveOldLine.contains(":=")) {
					varName = effectiveOldLine.split(":=")[0].trim();
				} else if (effectiveOldLine.contains("=")) {
					varName = Parser.extractLHS(effectiveOldLine); // effectiveOldLine.split("=")[0].trim();
				}  else if (effectiveOldLine.contains("(")) {
					varName = effectiveOldLine.split("\\(")[0].trim();
				} else if (effectiveOldLine.endsWith(":")) {
					varName = effectiveOldLine.replace(":","").trim();
				}

				if (varName.equals(lhs))
					break;
				skipToNextBlock(oldPtr, mdl, newMdl);
				
			}

			if (oldPtr.value() < mdl.size()) {
			oldLine = mdl.get(oldPtr.value());
			ptr.setValue(oldPtr.value());
			effectiveOldLine = getLine(ptr, mdl);
			} else {
				// we are at end of file -- passed through all the graphics
				return newMdl;
			}
			//			
		}
		

		
		// we have located lhs, so transfer that to new 
		newMdl.add(oldLine);

		// oldPtr points to the beginning of the equation specification
		// this line may be continued,
		// have equation on 1 or more following lines
		// or gave the entire equation on this line
		
		// ========================================================================================
		oldLine = mdl.get(oldPtr.incAndValue());
		while(!oldLine.startsWith(START_TAG)) {
			newMdl.add(oldLine);
//			System.out.println("Add: "+newMdl.get(newMdl.size()-1));
			oldLine = mdl.get(oldPtr.incAndValue());
		}
		
		// we have located the units
		// consume the units
		// this can be one line or more -- need to peek at next line
		String nextLine = mdl.get(oldPtr.value()+1);
		if (nextLine.startsWith(START_TAG)) {
			// this is a one line unit
			// point to beginning of comment
			
			// Note there may be an empty comment with a :SUPPLEMENTARY tag
			
			if (!oldLine.equals(EQN_END_TAG)) {
				newMdl.add(oldLine);
//				System.out.println("Add: "+newMdl.get(newMdl.size()-1));
			}
			oldPtr.add(1);
			oldLine = nextLine;
		} else {
			// multiple lines -- consume them
			
			oldLine = mdl.get(oldPtr.incAndValue());
			while(!oldLine.startsWith(START_TAG)) {
				newMdl.add(oldLine);
				
//				System.out.println("Add: "+newMdl.get(newMdl.size()-1));
				oldLine = mdl.get(oldPtr.incAndValue());
			}
		}
		
		boolean supplementary = false;
		if (oldLine.contains(SUPPLEMENTARY)) {
			supplementary = true;
		}
		
		boolean endedEquation = false;
		if (comments.size() == 1 && comments.get(0).length() == 0 && !supplementary) {
			newMdl.add(EQN_END_TAG2);
//			System.out.println("Add: "+newMdl.get(newMdl.size()-1));
			endedEquation = true;
		} else if (comments.size() == 1 && comments.get(0).length() == 0 && supplementary) {
			newMdl.add(COMMENT_START+COMMENT_START+SUPPLEMENTARY);
//			System.out.println("Add: "+newMdl.get(newMdl.size()-1));
			newMdl.add(EQN_END_TAG);
//			System.out.println("Add: "+newMdl.get(newMdl.size()-1));
			endedEquation = true;
		} else {

			// we have located the start of the comment
			if (comments.size() == 0) {
				comments.add("");
			}
			newMdl.add("\t~\t"+comments.get(newPtr++));
//			System.out.println("Add: "+newMdl.get(newMdl.size()-1));
//			if (newMdl.get(newMdl.size()-1).contains("LINK TO DATA AND MODEL RATIONALE")) {
//				System.out.println("oldPtr "+oldPtr);
//			}
			while (newPtr < comments.size()) {
				newMdl.add("\t\t"+comments.get(newPtr++));
//				System.out.println("Add: "+newMdl.get(newMdl.size()-1));
//				if (newMdl.get(newMdl.size()-1).contains("LINK TO DATA AND MODEL RATIONALE")) {
//					System.out.println("oldPtr "+oldPtr);
//				}
			}
			
			
		}

		// skip the old comment
		//		oldLine = mdl.get(++oldPtr);
//		while(!oldLine.startsWith(START_TAG) && !oldLine.startsWith(EQN_END_TAG)  && !oldLine.startsWith(EQN_END_TAG2)) {
		while(!oldLine.startsWith(EQN_END_TAG)  && !oldLine.startsWith(EQN_END_TAG2)) {

			if (oldLine.startsWith(COMMENT_START) && oldLine.contains(SUPPLEMENTARY))
				if (!endedEquation)
					newMdl.add(COMMENT_START+SUPPLEMENTARY);
			oldLine = mdl.get(oldPtr.incAndValue());
		}
		
		// look ahead to see if
		if (!endedEquation)
			newMdl.add(EQN_END_TAG);

		// transfer the rest of the old mdl file
		
		// problem is: oldPtr not pointing past end of equation.

		while(oldPtr.incAndValue() < mdl.size()) {
			newMdl.add(mdl.get(oldPtr.value()));

			
		}
		
		return newMdl;
	}
}
