package gov.anl.gss.sdm.server.model.translator;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class EquationComponents {
static Logger log = Logger.getLogger(EquationComponents.class.getSimpleName());

	private static final String END_OF_EQN_MARKER = "~\t";
	
	private boolean hasEquation = false;
	private boolean hasUnits = false;
	private boolean hasComment = false;
	private boolean hasMacroInvocation = false;
	private boolean supplementary = false;
	private boolean sectionMarker = false;
	private boolean parsable = false;
	private boolean equation = false;
	private boolean connected = true;
	
	private boolean xmileToMdlTranslation = false;
	
	private boolean typeString = false;
	
	private boolean viewStartMarker = false;
	private boolean graphicsEndMarker = false;
	
	private String vensimEquation;
	private String unalteredVensimEquation;
	
	private String equationPart = "";
	private String unitsPart = "";
	private String commentPart = "";
	private String supplementaryPart = "";
	
	public EquationComponents(String equation, boolean xmile2mdl) {
		this(equation);
		this.xmileToMdlTranslation = xmile2mdl;
	}
	
	public EquationComponents(String equation) {

	
//		log.debug("EC: "+equation);
//		System.out.println("EC: "+equation);
		this.vensimEquation = equation;
		unalteredVensimEquation = new String(vensimEquation);
		
		if (equation.toUpperCase().contains("GET XLS LOOKUPS")) {
			List<String> tokens = Parser.tokenizeMaintainSingleQuotes(equation.replaceAll(END_OF_EQN_MARKER+"$", ""));
			if (!tokens.contains("=")) {
				vensimEquation = alterGetXlsLookups(tokens)+END_OF_EQN_MARKER;
			}
		}
		analyze();
	}
	
	private String alterGetXlsLookups(List<String> tokens) {
		
		List<String> altered = new ArrayList<String>();
		int i;
		for (i = 0; i < tokens.size()-1;i++) {
			if (tokens.get(i+1).equals("GET XLS LOOKUPS")) {
				altered.add("=");
			} else {
				altered.add(tokens.get(i));
			}
		}
		
		return Parser.concatenateList(altered);
	}
	
	private void analyze() {

		clean();
		equationPart = "";
		unitsPart = "";
		commentPart = "";
		supplementaryPart = "";
		
		
		// equation consists of three parts
		String[] eqn = vensimEquation.split("~");
		equationPart = eqn[0].trim();
//		log.debug("EqnPart: "+equationPart);
		if  (eqn.length > 1) {
			unitsPart = eqn[1].trim();
//			log.debug("unitsPart: "+unitsPart);
		}
		if  (eqn.length > 2) {
			commentPart = eqn[2].trim();
//			log.debug("commentPart: "+commentPart);
		}
		if  (eqn.length > 3) {
			supplementaryPart = eqn[3].trim();
//			log.debug("supplementaryPart: "+supplementaryPart);
		}
		
		// the real work
		
		if (vensimEquation.startsWith("***")) {
			sectionMarker = true;
		} else if (vensimEquation.startsWith("///---\\\\\\")) {
			graphicsEndMarker = true;
		} else if (vensimEquation.startsWith("\\\\\\---///")) {
			viewStartMarker = true;
		} else {
			parsable = true;
			if (vensimEquation.contains("=")) {
				equation = true;
				hasEquation = true;
				if (vensimEquation.contains("A FUNCTION OF")) {
					hasEquation = false;
					
				} 
					// don't know if this is a correct assumption or not
					String integ = vensimEquation.replace(" ", "").toLowerCase();
					if (integ.contains("integ(,)"))
							hasEquation = false;
				
			} else {
				hasEquation = true;
			}
			if (unitsPart.length() > 0)
				hasUnits = true;
			if (commentPart.length() > 0)
				hasComment = true;
			if (supplementaryPart.contains(":SUPPLEMENTARY")) {
				supplementary = true;
//				log.debug("SUPPLEMENTARY"+vensimEquation);
			}
		}
	}
	
	private void clean() {
		// do some initial cleanup of equation
				if (InformationManagers.getInstance().getMacroManager().containsMacroInvocation(vensimEquation)) {
					vensimEquation = (InformationManagers.getInstance().getMacroManager().expand(vensimEquation)).replaceAll("\t", "");
					hasMacroInvocation = true;
				} else {
					vensimEquation = vensimEquation.replaceAll("\t", "");
				}

				// if any subscript ranges have blanks before the "!", get rid of them
				if (vensimEquation.contains("!")) {
					vensimEquation = vensimEquation.replaceAll(" *!", "!");
				}
				// is this a string assignment?
				if (vensimEquation.contains(":IS:")) {
//					vensimEquation = vensimEquation.replace(":IS:", "=");
					typeString = true;
				}

				// reference predefined variables?
				if (vensimEquation.contains(":NA:")) {
					vensimEquation = vensimEquation.replaceAll(":NA:", "NAREPLACEMENT");
				}

	}
	
	public boolean isMarker() {
		return graphicsEndMarker || viewStartMarker || sectionMarker;
	}

	public boolean isHasEquation() {
		return hasEquation;
	}

	public boolean isHasUnits() {
		return hasUnits;
	}

	public boolean isHasComment() {
		return hasComment;
	}

	public boolean isSupplementary() {
		return supplementary;
	}

	public String getVensimEquation() {
		return vensimEquation;
	}

	public boolean isHasMacroInvocation() {
		return hasMacroInvocation;
	}

	public String getEquationPart() {
		return equationPart;
	}

	public String getUnitsPart() {
		return unitsPart;
	}

	public String getCommentPart() {
		return commentPart;
	}

	public String getSupplementaryPart() {
		return supplementaryPart;
	}

	public boolean isSectionMarker() {
		return sectionMarker;
	}

	public boolean isParsable() {
		return parsable;
	}

	public boolean isEquation() {
		return equation;
	}

	public boolean isViewStartMarker() {
		return viewStartMarker;
	}

	public boolean isGraphicsEndMarker() {
		return graphicsEndMarker;
	}

	public boolean isConnected() {
		return connected;
	}

	public boolean isTypeString() {
		return typeString;
	}

	public boolean isXmileToMdlTranslation() {
		return xmileToMdlTranslation;
	}

	public void setXmileToMdlTranslation(boolean xmileToMdlTranslation) {
		this.xmileToMdlTranslation = xmileToMdlTranslation;
	}

}
