package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.ArrayReference;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

public class EquationLookup {

	private Map<String, Equation> equations;
	
	private Map<String, String> screenNameToLhs;
	private Map<String, String> lhsToScreenName;
	
	private List<String> orderedLhs = null;
	public List<String> orderedVariables = null;
	
	public void setEquations(Map<String, Equation> eqns) {
		equations = eqns;
		screenNameToLhs = new LinkedHashMap<String, String>();
		lhsToScreenName = new LinkedHashMap<String, String>();
		
		for (String key : equations.keySet()) {
			
			screenNameToLhs.put(Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(key)), key);
			lhsToScreenName.put(key, Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(key)));
		}
	}
	
	public Equation getEquation(String screenName) {
		
		String effectiveScreenName = new String(screenName);
		
		Equation eqn = InformationManagers.getInstance().getEquationManager().
				asSingleton(InformationManagers.getInstance().getEquationManager().getAllEquationsFor(effectiveScreenName));
		if (eqn == null) {
			return InformationManagers.getInstance().getEquationManager().
					asSingleton(InformationManagers.getInstance().getEquationManager().getAllEquationsFor("\""+effectiveScreenName+"\""));
		}
		return eqn;
		
//		if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(effectiveScreenName) == null) {
//			// we may have the variable with quotes
//			effectiveScreenName = "\"" + screenName + "\"";
//			if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(effectiveScreenName) == null) {
//				return null;
//			}
//		}
//		
//		if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(effectiveScreenName).size() == 0)
//			return null;
//		Iterator<Equation> anEqn = InformationManagers.getInstance().getEquationManager().getAllEquationsFor(effectiveScreenName).iterator();
//		Equation eqn = anEqn.next();
//		return eqn;
//		if (!screenNameToLhs.containsKey(screenName))
//			return null;
//		return equations.get(screenNameToLhs.get(screenName));
	}
	
	public String getScreenName(String lhsName) {
		return ArrayReference.extractArrayName(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhsName));
//		return lhsName;
//		if (!lhsToScreenName.containsKey(lhsName))
//			return lhsName;
//		else
//			return  Parser.removeQuotes(lhsToScreenName.get(lhsName)); // .replace("\"", "");
	}
	
	public String getLhsName(String screenName) {
		if (!screenNameToLhs.containsKey(screenName))
			return screenName;
		else
			return screenNameToLhs.get(screenName);
	}
	
	public List<String> orderedLHS() {
		if (orderedLhs != null)
			return orderedLhs;
		orderedLhs = ordered(InformationManagers.getInstance().getEquationManager().orderedLHS());
		return orderedLhs;
	}
	
	public List<String> orderedVariables() {
		if (orderedVariables != null) {
			return orderedVariables;
		}
		orderedVariables = ordered(InformationManagers.getInstance().getEquationManager().orderedVariables());
		return orderedVariables;
	}

	public Map<String, Equation> getEquations() {
		return equations;
	}
	
	public List<String> ordered(List<String> vars) {
		List<String> orderedListOfVariables = new ArrayList<String>();
		Map<String, String> lowerCaseKeyMap = new LinkedHashMap<String, String>();
		for (String var : vars) {
			String key =  Parser.removeQuotes(var) /*.replace("\"", "")*/ .replace("array.", "").toLowerCase();
			lowerCaseKeyMap.put(key, var);
		}
		List<String> lowerCaseKeysList = new ArrayList<String>(lowerCaseKeyMap.keySet());
//		Collections.sort(lowerCaseKeysList);
		
		try {
			Collections.sort(lowerCaseKeysList, new Comparator<String>() {
				@Override
				public int compare(String arg0, String arg1) {
					try {
						// first separate non-control and control
						boolean isSys0 = Parser.isSystemVariable(arg0);
						boolean isSys1 = Parser.isSystemVariable(arg1);
						if (!isSys0 && isSys1) {
							return -1;
						} else if (isSys0 && !isSys1) {
							return 1;
						} else {
							return arg0.compareToIgnoreCase(arg1);
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
				}
			});
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		for (String lcKey : lowerCaseKeysList)
			orderedListOfVariables.add(lowerCaseKeyMap.get(lcKey));
		
		return orderedListOfVariables;
	}

}
