package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class EquationOrderProcessor {

	String miscDirectory;

	public EquationOrderProcessor(String miscDirectory) {
		this.miscDirectory = miscDirectory;
	}
	
	public EquationOrderProcessor() {
//		this("output");
		if (InformationManagers.getInstance().getFileOrganizer() != null && 
				InformationManagers.getInstance().getFileOrganizer().getMiscDirectory().length() > 0) {
			miscDirectory = InformationManagers.getInstance().getFileOrganizer().getMiscDirectory();
		} else {
			miscDirectory = "output";
		}
	}
	


	protected ArrayList<String> determineEvaluationOrder(Map<String, Equation> equations, 
			boolean considerInitialValues, String fileNameHead, boolean printResults, VariableDependencyProcessor variableDependencyProcessor) {

		// all realLHS that still require processing.
		List<String> allRealLHS = new ArrayList<String>();

		// expandedLhsMap maps an expandedLHS to its higher level lhs subscript from which
		// it is expanded. This map remains intact in the algorithm.
		
		Map<String, String> expandedLhsToRealLhsMap = variableDependencyProcessor.getExpandedLhsToRealLhsMap();

		// lhsExpandedMap maps a realLHS to its expandedLHSs. This map remains intact in the
		// algorithm
		
		Map<String, HashSet<String>> realLhsToExpandedLhsMap = variableDependencyProcessor.getRealLhsToExpandedLhsMap();

		// lhsExpandedNotInitialized starts as a copy of lhsExpandedMap. As expandedLHSs become available
		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
		// the key which means that we do not need to consider this realLHS for processing.
		
		Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized = variableDependencyProcessor.getRealLhsToExpandedLhsNotInitialized();

		// requiresExpanded maps expandedLHS to set of rhs expanded variables. Set elements are removed
		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
		// becomes empty.
		
		Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS = variableDependencyProcessor.getRequiresExpandedLhsExpandedRHS();
		

		// all realLHS that still require processing.
		List<String> allExpLHS = new ArrayList<String>();
		
		// this is the order in which we will evaluate the equations
		ArrayList<String> evaluationOrder = new ArrayList<String>();

		// for each equation:
		// get the set of variables that are required
		// then walk through the set repeatedly removing those with no requirements after they have been met

		// for each RHS set in requiresExpandedLhsExpandedRHS, need to add the eqn LHS of the rhs Item

//		if (printResults) {
//			dumpRequires(fileNameHead+"realLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
//			dumpRequires2(fileNameHead+"expandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
//			boolean startChain = false; // want this to be false for production
//			dumpRequiresExpanded(fileNameHead+"initialRequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRHS, startChain); // HerelhsExpandedNotInitialized
//			dumpRequiresExpanded(fileNameHead+"initialRealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
//		}

		// allLHS needs to be reset
		allRealLHS.clear();
		
		List<String> removeMe = new ArrayList<String>();
		
		// HACK: all of a sudden, time and timeStep are passing through to here. They must be removed
		for (String key : realLhsToExpandedLhsNotInitialized.keySet()) {
			Set<String> hs = realLhsToExpandedLhsNotInitialized.get(key);
			hs.remove("time"); hs.remove("Time"); hs.remove("timeStep");
			if (key.equalsIgnoreCase("time") || key.equalsIgnoreCase("timestep"))
				removeMe.add(key);
		}
		for (String k : removeMe)
			realLhsToExpandedLhsNotInitialized.remove(k);
		
		removeMe.clear();
		
		for (String key : requiresExpandedLhsExpandedRHS.keySet()) {
			Set<String> hs = requiresExpandedLhsExpandedRHS.get(key);
			hs.remove("time"); hs.remove("Time"); hs.remove("timeStep");
			if (key.equalsIgnoreCase("time") || key.equalsIgnoreCase("timestep"))
				removeMe.add(key);
		}
		for (String k : removeMe)
			requiresExpandedLhsExpandedRHS.remove(k);

		

		// processing is slightly different after first pass through the equations
		boolean done = false;
		int loopCount = 0;
		
		// the set of real LHS not yet initialized
		allRealLHS.addAll(realLhsToExpandedLhsNotInitialized.keySet());
		allExpLHS.addAll(requiresExpandedLhsExpandedRHS.keySet());
		
		int startSize = allExpLHS.size();
		int endSize = 0;
		
		while (!done) {

			// this will contain LHS's that have been assigned a value
			ArrayList<String> toRemoveFromRequires = null;

			System.out.println("####################### Loop ############################### Exp NumToProcess "+allExpLHS.size()+
					" eqn remaining "+realLhsToExpandedLhsNotInitialized.size());

			toRemoveFromRequires = new ArrayList<String>();

			for (String expandedLHS : allExpLHS) {
				String realLHS = toRealLHS(expandedLHS, expandedLhsToRealLhsMap);
				
				// this may have already been ordered due to chaining of data available
				if (evaluationOrder.contains(realLHS) || evaluationOrder.contains(expandedLHS))
					continue;
				
//				toRemoveFromRequires = new ArrayList<String>();

					Equation eqn = equations.get(realLHS);
					if (eqn == null) 
						System.out.println("EOP: Equation is null for realLHS: "+realLHS);
					//  check for constants or those with no remaining requirements
					// as above, but also check if an initial value is available
					
					if (/* requiresExpandedLhsExpandedRHS.get(expandedLHS) == null ||  */
							(requiresExpandedLhsExpandedRHS.get(expandedLHS) != null 
							&& (requiresExpandedLhsExpandedRHS.get(expandedLHS).size() == 0))) {  // was requires (requiresExpanded) MJB 9/12
						//					System.out.println("Free: "+expandedLHS);
						toRemoveFromRequires.add(expandedLHS);
//						removeRequirement(1,toRemoveFromRequires, requiresExpandedLhsExpandedRHS, realLhsToExpandedLhsNotInitialized, 
//								expandedLhsToRealLhsMap, evaluationOrder, equations);
					} 
			}
			
			removeRequirement(1,toRemoveFromRequires, requiresExpandedLhsExpandedRHS, realLhsToExpandedLhsNotInitialized, 
					expandedLhsToRealLhsMap, evaluationOrder, equations);
			endSize =  allExpLHS.size();

			if (endSize < startSize) {
				System.out.println("Making Progress");
				loopCount = 0;
			} else {
				loopCount++;
				System.out.println("No Progress "+loopCount);
			}
			startSize = endSize;
			if (loopCount < 5) {
				// continue trying
				if (realLhsToExpandedLhsNotInitialized.size() == 0) {
					System.out.println("Successfully ordered all equations");
					done  = true;
				} else {
					// still more to do
					System.out.println("Remaining Equations: "+realLhsToExpandedLhsNotInitialized.size());
					allExpLHS.clear();
					allExpLHS.addAll(requiresExpandedLhsExpandedRHS.keySet());
				}
			} else {
				System.out.println("Not making progress. Terminating due to LoopCount: "+loopCount);
				System.out.println("Cannot order these equations:");

				for (String lhs : realLhsToExpandedLhsNotInitialized.keySet()) {
					System.out.println("\n%%%%%%");
					System.out.println(">>> lhs <"+lhs+"> ");
					if (equations.get(lhs) == null)
						System.out.println("NO EQUATION EXISTS FOR: "+lhs);
					else
						System.out.println(equations.get(lhs).getVensimEquationOnly());
					printRequires(lhs, realLhsToExpandedLhsNotInitialized, requiresExpandedLhsExpandedRHS);

					done = true;
				}
				break;
			}
		} // while (!done)

		if (requiresExpandedLhsExpandedRHS.size() > 0)
			analyzeBlockage(requiresExpandedLhsExpandedRHS);
		boolean endChain = true;
		dumpRequiresExpanded(fileNameHead+"postOrderrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRHS, endChain); // Here
		dumpRequiresExpanded(fileNameHead+"postrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, endChain);

		evaluationOrder.addAll(realLhsToExpandedLhsNotInitialized.keySet());

		analyzeEvaluationOrder(evaluationOrder, equations, realLhsToExpandedLhsMap, expandedLhsToRealLhsMap, fileNameHead);
		return evaluationOrder;

	}
	
	private String toRealLHS(String lhs, Map<String, String> expandedLhsToRealLhsMap) {
		if (expandedLhsToRealLhsMap.containsKey(lhs)) 
			return expandedLhsToRealLhsMap.get(lhs);
		else
			return lhs;
	}

	protected void analyzeEvaluationOrder(List<String> evaluationOrder, Map<String, Equation> equations,
			Map<String, HashSet<String>> lhsExpandedMap, Map<String, String> expandedLhsMap, String fileNameHead) {

		boolean orderProblem = false;
		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileNameHead+"evaluationOrder.txt");
		HashMap<String, Integer> initializationLine = new LinkedHashMap<String, Integer>();
		for (int i = 0; i < evaluationOrder.size(); i++) {
			initializationLine.put(evaluationOrder.get(i), i);
			String lhs = evaluationOrder.get(i);
			//	    System.out.println("<"+lhs+">");

			if (expandedLhsMap.containsKey(lhs)) {
				String real = expandedLhsMap.get(lhs);
				initializationLine.put(real, i);
			}

			if (lhsExpandedMap.containsKey(lhs)) {
				for (String exp : lhsExpandedMap.get(lhs)) {
					initializationLine.put(exp, i);
				}
			}
		}

		try {
			for (int i = 0; i < evaluationOrder.size(); i++) {
				String lhs = evaluationOrder.get(i);

				if (equations.get(lhs) == null) {
					bw.append(" EQN: "+i+" ##### NULL eqn for LHS "+lhs+" #####\n");
					continue;
				}

				bw.append(" EQN: "+i+" "+lhs+" = "+equations.get(lhs).getCleanEquation()+"\n");
				bw.append(" EQN: "+i+" "+lhs+" = "+equations.get(lhs).getVensimEquationOnly()+"\n");

			}
			for (int i = 0; i < evaluationOrder.size(); i++) {
				String lhs = evaluationOrder.get(i);

				if (equations.get(lhs) == null) {
					bw.append("\n"+i);
					bw.append(" EQN:  ##### NULL eqn for LHS "+lhs+" #####");
					continue;
				}

				bw.append("\n"+i);
				bw.append(" EQN: "+equations.get(lhs).getVensimEquationOnly()+"\n");
				bw.append(" LHS: "+lhs+"\n");
				bw.flush();
				HashSet<String> rhsVars = null;
				
				if (equations.get(lhs).isAssignment()) {
					rhsVars = (HashSet<String>) equations.get(lhs).getRHSVariablesForOrdering(false); // Expanded();
					if (equations.get(lhs).isStock() || equations.get(lhs).isActiveInitial() || equations.get(lhs).isSampleIfTrue()) {
						rhsVars = (HashSet<String>) equations.get(lhs).getFunctionInitialVariables();
					} 
				
				}
//				if (equations.get(lhs).isRepeated())
//					rhsVars = equations.get(lhs).getRHSVariables();

				boolean problem = false;
				boolean orderingProblem = false;
				boolean missingProblem = false;

				if (rhsVars == null || rhsVars.size() == 0) {
					bw.append("     None\n");
					problem = false;
					orderingProblem = false;
					missingProblem = false;
				} else {
					int r = 0;
					for (String rhs1 : rhsVars) {
						String rhs = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhs1);
						if (rhs.equalsIgnoreCase("Time"))
							continue;
						if (rhs.equalsIgnoreCase("TimeStep"))
							continue;

						// need to contend with array references  30Sep

						if (initializationLine.get(rhs) != null) {
							r = initializationLine.get(rhs);

							if (i < r && !equations.get(lhs).isOrderedWithInitialValue() && !equations.get(lhs).isDefinesLookup()) {
								orderingProblem = true;
								bw.append("     <"+rhs+"> "+r + (i < r ? " <<<<<<<<" : "")+" "+equations.get(lhs).isOrderedWithInitialValue()+"\n");
							} else {
								bw.append("     <"+rhs+"> "+r + " "+ /* equations.get(lhs).isOrderedWithInitialValue()+*/ "\n");
							}
						} else {
							// this might be a array assignment and we alooking at an expanded LHS
							// convert back to the realLHS
							String rLHS = expandedLhsMap.get(rhs);
							if (initializationLine.containsKey(rLHS)) {
								r = initializationLine.get(rLHS);
								bw.append("     <"+rLHS+"> "+r + (i < r ? " <<<<<<<<" : "")+"\n");
								if (i < r && !equations.get(rLHS).isOrderedWithInitialValue()  && !equations.get(lhs).isDefinesLookup()) 
									orderingProblem = true;
							} else {
								rLHS = "array."+rhs;
								if (initializationLine.containsKey(rLHS)) {
									r = initializationLine.get(rLHS);
									bw.append("     <"+rLHS+"> "+r + (i < r ? " <<<<<<<<" : "")+"\n");
									if (i < r && !equations.get(rLHS).isOrderedWithInitialValue()) 
										orderingProblem = true;
								} else {
									bw.append(" >>>> initializtion line missing for <"+rhs+"> try individuals\n");
									missingProblem = false;
									// if the rhs is an array reference to a "group"
									// see if the individual array refers have initialization lines
									if ((rhs.contains("!") || !ArrayReference.isArrayReference(lhs)) &&
											ArrayReference.isArrayReference(rhs)) {


										ArrayReference rhsArrayReference = new ArrayReference(rhs.replace("!",""));
										for (String expandedRHS : ArrayReference.expand(rhs)) {
											//									for (SubscriptCombination subrCombo : InformationManagers.getInstance().getArrayManager().
											//											getSubscriptValueCombinations(rhsArrayReference.getSubscriptsAsArray())) {
											//										// Build an array reference based on the subscript value
											//										String expandedRHS = "array."+rhsArrayReference.getArrayName()+ "[";
											//										int subn = 0;
											//										for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
											//											if (subn++ > 0) {
											//												expandedRHS += ",";
											//											}
											//											expandedRHS += subrCombo.getSubscriptValue(subscript);										 
											//										}
											//										expandedRHS += "]";
											if (!initializationLine.containsKey(expandedRHS)) {
												bw.append(" >>>> initializtion line missing for individual <"+expandedRHS+">\n");
												missingProblem = true;
											} else {
												r = initializationLine.get(expandedRHS);
												bw.append(" >>>> initializtion line FOUND for individual <"+expandedRHS+"> "+r+"\n");
											}
										}


									} else 	if (ArrayReference.isArrayReference(lhs)) {
										if (!ArrayReference.isArrayReference(rhs)) {
											//										System.out.println("Why do I think that this should be an AR");
											continue;
										}
										ArrayReference rhsArrayReference = new ArrayReference(rhs.replace("!",""));
										ArrayReference lhsArrayReference = new ArrayReference(lhs);
										for (String expandedRHS : ArrayReference.expand(rhs)) {
											//									for (SubscriptCombination subrCombo : InformationManagers.getInstance().getArrayManager().
											//											getSubscriptValueCombinations(lhsArrayReference.getSubscriptsAsArray())) {
											//										// Build an array reference based on the subscript value
											//										String expandedRHS = "array."+rhsArrayReference.getArrayName()+ "[";
											//										int subn = 0;
											//										for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
											//											if (subn++ > 0) {
											//												expandedRHS += ",";
											//											}
											//											expandedRHS += subrCombo.getSubscriptValue(subscript);										 
											//										}
											//										expandedRHS += "]";
											if (!initializationLine.containsKey(expandedRHS)) {
												bw.append(" >>>> initializtion line missing for individual <"+expandedRHS+">\n");
												missingProblem = true;
											} else {
												r = initializationLine.get(expandedRHS);
												bw.append(" >>>> initializtion line FOUND for individual <"+expandedRHS+"> "+r+"\n");
											}
										}
									} else {}
								}
							}
						}
						// lookups will be defined in oneTime()
						if (equations.get(lhs).getVensimEquationOnly().contains("sdFunctions.LOOKUP"))
							problem = false;
						if (i > r /*&& (equations.get(lhs).getCleanEquation().contains("INTEG") ||
				equations.get(lhs).getCleanEquation().contains("SMOOTHI")) */) {
							problem = false;
						}
					}
				}
				if (problem) {
					bw.append("######### P R O B L E M #########\n");
					orderProblem = true;
				}

				if (orderingProblem) {
					bw.append("######### ORDERING P R O B L E M #########\n");
					orderProblem = true;
				}

				if (missingProblem) {
					bw.append("######### MISSING P R O B L E M #########\n");
					orderProblem = true;
				}
			}
			bw.flush();
			bw.close();


			if (orderProblem) {
				System.out.println(SDMDocLanguageSpecific.getInstance().getText(StringConstants.EVALUATION_ORDER_PROBLEMS));
			} else {
				System.out.println(SDMDocLanguageSpecific.getInstance().getText(StringConstants.CLEAN_EVALUATION_ORDER));
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void printRequires(String realLHS, Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized,
			Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS) {
		System.out.println(">>>>>>>>>>>>>>>>>>>>> Requires Start >>>>>>>>>>>>>>>>>>>>>>>>>>>>");
		Set<String> hs = realLhsToExpandedLhsNotInitialized.get(realLHS);
		for (String expLHS : hs) {
			System.out.println("     expLHS: "+expLHS);
			for (String rhs : requiresExpandedLhsExpandedRHS.get(expLHS)) {
				System.out.println("          expRHS: "+rhs);
			}
		}
		System.out.println(">>>>>>>>>>>>>>>>>>>>>>>>Requires End>>>>>>>>>>>>>>>>>>>>>>>>>");
	}
	
	protected void removeRequirement(int level, ArrayList<String> toRemoveFromRequires, 
			Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS,
			Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized,
			Map<String, String> expandedLhsToRealLhsMap,
			ArrayList<String> evaluationOrder,
			Map<String, Equation> equations) {

		// these are the only realLHS that we can remove on this pass
		
		if (toRemoveFromRequires.size() == 0)
			return;

		Set<String> candidateRealLHS = new LinkedHashSet<String>();
		List<String> newlyFreed = new ArrayList<String>();

		for (String toRemoveExpLHS : toRemoveFromRequires) {
			if (expandedLhsToRealLhsMap.containsKey(toRemoveExpLHS)) {
				candidateRealLHS.add(expandedLhsToRealLhsMap.get(toRemoveExpLHS));
			}
		}

		for (String toRemoveExpLHS : toRemoveFromRequires) {
			// have two LHS: toRemove and realLHS NOTE: toRemove could = realLHS
			String realLHS = toRealLHS(toRemoveExpLHS, expandedLhsToRealLhsMap);

			// we've determined that this array reference can take place
			// remove it from requiresExpanded
			// this does not added to evaluation order until we know that all other references to this array have been processed

			// we have identified this expLHS as having no more requires in requiresExpandedLhsExpandedRHS
			// by removing this key, we will not process this in the future
			requiresExpandedLhsExpandedRHS.remove(toRemoveExpLHS);
			if (realLhsToExpandedLhsNotInitialized.get(realLHS) != null) {
				realLhsToExpandedLhsNotInitialized.get(realLHS).remove(toRemoveExpLHS);
				if (realLhsToExpandedLhsNotInitialized.get(realLHS).size() == 0) {
					if (realLHS.contains("array_"))
						System.out.println("array_");
					evaluationOrder.add(realLHS);
					realLhsToExpandedLhsNotInitialized.remove(realLHS);
//					System.out.println("Schedule"+level+": "+realLHS);
					if (equations.get(realLHS) != null && equations.get(realLHS).isStock())
						equations.get(realLHS).setOrderedWithInitialValue(true);
				}
			} else {
				System.out.println("EOP: Internal error realLhsToExpandedLhsNotInitialized realLHS: "+realLHS+" toRemoveExpLHS: "+toRemoveExpLHS);
			}

		}
		for (String toRemoveExpLHS : toRemoveFromRequires) {
			newlyFreed.clear();
			String realLHS = toRealLHS(toRemoveExpLHS, expandedLhsToRealLhsMap);
			// now check the RHS data structures, removing all occurrences
			newlyFreed.addAll(removeFromRHS(toRemoveExpLHS, requiresExpandedLhsExpandedRHS)); // "RHS" of equation
			newlyFreed.addAll(removeFromRHS(realLHS, requiresExpandedLhsExpandedRHS));
//			for (String free : newlyFreed) {
//				ArrayList<String> al = new ArrayList<String>();
//				al.add(free);
//				removeRequirement(level+1,al, 
//						requiresExpandedLhsExpandedRHS,
//						realLhsToExpandedLhsNotInitialized,
//						expandedLhsToRealLhsMap,
//						evaluationOrder,
//						equations);
//			}
 
		}

	}
	
	protected void removeRequirementOriginal(ArrayList<String> toRemoveFromRequires, 
			Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS,
			Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized,
			Map<String, String> expandedLhsToRealLhsMap,
			ArrayList<String> evaluationOrder,
			Map<String, Equation> equations) {

		// these are the only realLHS that we can remove on this pass

		Set<String> candidateRealLHS = new LinkedHashSet<String>();
		List<String> newlyFreed = new ArrayList<String>();

		for (String toRemoveExpLHS : toRemoveFromRequires) {
			if (expandedLhsToRealLhsMap.containsKey(toRemoveExpLHS)) {
				candidateRealLHS.add(expandedLhsToRealLhsMap.get(toRemoveExpLHS));
			}
		}

		for (String toRemoveExpLHS : toRemoveFromRequires) {
			// have two LHS: toRemove and realLHS NOTE: toRemove could = realLHS
			String realLHS = toRealLHS(toRemoveExpLHS, expandedLhsToRealLhsMap);

			// we've determined that this array reference can take place
			// remove it from requiresExpanded
			// this does not added to evaluation order until we know that all other references to this array have been processed

			// we have identified this expLHS as having no more requires in requiresExpandedLhsExpandedRHS
			// by removing this key, we will not process this in the future
			requiresExpandedLhsExpandedRHS.remove(toRemoveExpLHS);
			
			if (realLhsToExpandedLhsNotInitialized.get(realLHS) != null) {
				realLhsToExpandedLhsNotInitialized.get(realLHS).remove(toRemoveExpLHS);
			} else {
				System.out.println("EOP: Internal error realLhsToExpandedLhsNotInitialized realLHS: "+realLHS+" toRemoveExpLHS: "+toRemoveExpLHS);
			}


			// now check the RHS data structures, removing all occurrences
			newlyFreed.addAll(removeFromRHS(toRemoveExpLHS, requiresExpandedLhsExpandedRHS)); // "RHS" of equation

			// at this point, we need to see if all array subscripts have been achieved. Then we can add to evaluationOrder

			// once again, to remove can be a realLHS or expLHS
			// cast to its parent realLHS and get the set of expLHS 


/* Start Skip */			
			Set<String> expLHSNotInit = realLhsToExpandedLhsNotInitialized.get(realLHS);

			// the the exp/real LHS exists in the set 
			if (expLHSNotInit != null) {
				// if the size is now 0, we can add the realLHS to the evaluation order list
				if (expLHSNotInit.size() == 0) {  
					if (!evaluationOrder.contains(realLHS) && candidateRealLHS.contains(realLHS)) {
						evaluationOrder.add(realLHS);
//						System.out.println("Schedule1: "+realLHS);
						if (equations.get(realLHS) != null && equations.get(realLHS).isStock())
							equations.get(realLHS).setOrderedWithInitialValue(true);
						// never need to process this again
						realLhsToExpandedLhsNotInitialized.remove(realLHS);
						// so we need to remove it from RHS datastructures as this is the last chance we will have
						newlyFreed.addAll(removeFromRHS(realLHS, requiresExpandedLhsExpandedRHS));
					}

				} 
			} 
//			else {
//				// real realLHSToExpLHS either was null or toRemove wasn't in there
//				if (!evaluationOrder.contains(realLHS)) {
//					evaluationOrder.add(realLHS);
//					System.out.println("EOP: ### Internal Problem Schedule2: "+realLHS);
//					if (equations.get(realLHS) != null && equations.get(realLHS).isStock())
//						equations.get(realLHS).setOrderedWithInitialValue(true);
//					// and again remove from to process set and from rhs data structures
//					realLhsToExpandedLhsNotInitialized.remove(realLHS);
//					removeFromRHS(realLHS, /* requiresLhsScalarExpandedRhs, */ requiresExpandedLhsExpandedRHS);
//				}
//			}
/* End Skip */

			//			} else {
			//				// we remove from requiresLhsScalarExpandedRhs and then check requires and requiresExpanded
			//				// this will always be a non array LHS
			//				requiresLhsScalarExpandedRhs.remove(toRemove);
			//				System.out.println("Schedule3: "+toRemove);
			//				evaluationOrder.add(toRemove);
			//
			//				removeFromRHS(toRemove, requiresLhsScalarExpandedRhs, requiresExpandedLhsExpandedRHS);
			//			}


		}

		//		// have any of our candidates been freed by all these operations
		//		List<String> al = new ArrayList<String>();
		//		for (String key : requiresLhsScalarExpandedRhs.keySet()) {
		//			if (requiresLhsScalarExpandedRhs.get(key).size() == 0)
		//				al.add(key);
		//		}
		//		for (String key : requiresExpandedLhsExpandedRHS.keySet()) {
		//			if (requiresExpandedLhsExpandedRHS.get(key).size() == 0)
		//				al.add(key);
		//		}


		//		
		//		System.out.println("##### Candidate LHS Set size = "+candidateLHS.size());
//				System.out.println("##### newlyFreed set size = "+newlyFreed.size());
		//		


//				for (String lhs : newlyFreed) {
//							String realLHS = toRealLHS(lhs, expandedLhsToRealLhsMap);
//					if (!candidateRealLHS.contains(realLHS))
//						continue;
//					// now that we have removed from RHS, make a quick pass and see if we can schedule
//					
//					if (realLhsToExpandedLhsNotInitialized.get(realLHS) != null) {
//						if (realLhsToExpandedLhsNotInitialized.get(realLHS).size() == 0) {
//							
//							realLhsToExpandedLhsNotInitialized.remove(realLHS);
//							if (!evaluationOrder.contains(realLHS)) {
//								System.out.println("############################### Schedule4: "+realLHS);
//								evaluationOrder.add(realLHS);
//								if (equations.get(realLHS).isStock())
//									equations.get(realLHS).setOrderedWithInitialValue(true);
//							}
//						}
//					} else {
//						
//						if (!evaluationOrder.contains(realLHS)) {
//							System.out.println("############################### Schedule5: "+realLHS+" not initialized is null");
//							evaluationOrder.add(realLHS);
//							if (equations.get(realLHS).isStock())
//								equations.get(realLHS).setOrderedWithInitialValue(true);
//						}
//					}
//					
//				}
//


	}
	
	protected List<String> removeFromRHS(String lhs, Map<String, HashSet<String>> requiresExpandedLhsExpandedRhs) {
		
		List<String> newlyFreed = new ArrayList<String>();
		newlyFreed.addAll(removeFromRHS(lhs, requiresExpandedLhsExpandedRhs, "requiresExpandedLhsExpandedRhs"));
		return newlyFreed;
	}



	protected void removeRequirementOld(ArrayList<String> toRemoveFromRequires, Map<String, HashSet<String>> requiresLhsScalarExpandedRhs,
			Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS,
			Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized,
			Map<String, String> expandedLhsToRealLhsMap,
			ArrayList<String> evaluationOrder) {



		System.out.println("removeRequirement # "+toRemoveFromRequires.size());

		for (String toRemove : toRemoveFromRequires) {

			if (ArrayReference.isArrayReference(toRemove)) {

				// we've determined that this array reference can take place
				// remove it from requiresExpanded
				// this does not added to evaluation order until we know that all other references to this array have been processed

				if (requiresExpandedLhsExpandedRHS.remove(toRemove) == null) { // "LHS" of equation
					System.out.println("Bad Remove from requiresExpanded: "+toRemove);

				}

				if (realLhsToExpandedLhsNotInitialized.remove(toRemove) == null) { // "LHS" of equation
					//			System.out.println("Bad Remove from lhsExpandedNotInitialized: "+toRemove);

				}
				removeFromRHS(toRemove, requiresLhsScalarExpandedRhs, requiresExpandedLhsExpandedRHS); // "RHS" of equation

				// at this point, we need to see if all array subscripts have been achieved. Then we can add to evaluationOrder

				String realLHS = expandedLhsToRealLhsMap.get(toRemove);
				HashSet<String> ehs = realLhsToExpandedLhsNotInitialized.get(realLHS);
				if (ehs != null && ehs.contains(toRemove)) {
					ehs.remove(toRemove);

					if (ehs.size() == 0) {
						if (!evaluationOrder.contains(realLHS))
							evaluationOrder.add(realLHS);
						realLhsToExpandedLhsNotInitialized.remove(realLHS);
						removeFromRHS(realLHS, requiresLhsScalarExpandedRhs, requiresExpandedLhsExpandedRHS);

					} else {

					}
				} else if (ehs == null) {
					if (!evaluationOrder.contains(realLHS))
						evaluationOrder.add(realLHS);
					else
						System.out.println("Already in Order: "+realLHS);
					realLhsToExpandedLhsNotInitialized.remove(realLHS);
					removeFromRHS(realLHS, requiresLhsScalarExpandedRhs, requiresExpandedLhsExpandedRHS);
				}


			} else {
				// we remove from requires and then check requires and requiresExpanded
				// this will always be a non array LHS
				requiresLhsScalarExpandedRhs.remove(toRemove);
				if (!evaluationOrder.contains(toRemove))
					evaluationOrder.add(toRemove);
				removeFromRHS(toRemove, requiresLhsScalarExpandedRhs, requiresExpandedLhsExpandedRHS);
			}
		}
	}

	protected void removeFromRHS(String lhs, Map<String, HashSet<String>> requiresLhsScalarExpandedRhs, 
			Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS) {

		removeFromRHS(lhs, requiresLhsScalarExpandedRhs, "requires");
		removeFromRHS(lhs, requiresExpandedLhsExpandedRHS, "requiresExpanded");
	}

	protected List<String> removeFromRHS(String variableToRemove, Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS, String tag) {
		
		List<String> newlyFreed = new ArrayList<String>();
		
		// these will remove realLHS and expLHS where appropriate

		for (String expLHS : requiresExpandedLhsExpandedRHS.keySet()) {
			HashSet<String> hs = requiresExpandedLhsExpandedRHS.get(expLHS);
			if (hs != null && hs.contains(variableToRemove)) {
				hs.remove(variableToRemove);
				if (hs.size() == 0) {
					newlyFreed.add(expLHS);
				}

			}
		}
		return newlyFreed;
	}

	protected void dumpRequires(String fileName, Map<String, HashSet<String>> requires) {

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);

		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
				for (String r : sortSet(requires.get(lhs))) {
					bw.append("     RHS <"+r+">\n");
				}
			}
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void dumpRequires2(String fileName, Map<String, String> requires) {

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);

		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
				String r = requires.get(lhs);
				bw.append("     RHS <"+r+">\n");
			}

			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}




	protected void analyzeBlockage(Map<String, HashSet<String>> requiresExpandedLhsExpandedRhs) {

		int minBlock = Integer.MAX_VALUE;

		ArrayList<String> allExpLHS = new ArrayList<String>(requiresExpandedLhsExpandedRhs.keySet());
		Collections.sort(allExpLHS);

		Map<String, MutableInteger> counts = new LinkedHashMap<String, MutableInteger>();

		for (String expLHS : allExpLHS) {
			int level = 1;
			HashSet<String> loop = new LinkedHashSet<String>();

			getLoopVariable(level, expLHS, requiresExpandedLhsExpandedRhs, loop, new LinkedHashSet<String>());					

			for (String l : loop){ 
//				System.out.println("Found loopVar: "+l);
				if (!counts.containsKey(l)) {
					counts.put(l, new MutableInteger());
				}
				counts.get(l).add(1);
			}
		}


		for (String var : counts.keySet()) {
			if (counts.get(var).value() > 0 && counts.get(var).value() < minBlock) {
				minBlock = counts.get(var).value();
			}
		}

		Set<String> singles = new LinkedHashSet<String>();
		for (String var : counts.keySet()) {
			if (counts.get(var).value() == minBlock) {
				singles.add(var);
			}
		}

		System.out.println("### Blockers ### minBlock = "+minBlock+" ###");
		for (String single : singles) {
			System.out.println("Blocker -> "+single);
			for (String s : requiresExpandedLhsExpandedRhs.get(single))
				System.out.println("   Waiting -> "+s);
		}
	}
	
	protected void getLoopVariable(int level, String expLHS, Map<String, HashSet<String>> requiresExpandedLhsExpandedRhs, 
			HashSet<String> loop, HashSet<String> alreadyPrinted) {

		if (level > 50) {
			return;
		}

		for (String expRHS : sortSet(requiresExpandedLhsExpandedRhs.get(expLHS))) {
			if (loop.contains(expRHS))
				continue;
			if (alreadyPrinted.contains(expRHS)) {
				loop.add(expRHS);
//				System.out.println("Found Loop Var: "+expRHS);
				continue;
			}

			alreadyPrinted.add(expRHS);
			getLoopVariable(level+1, expRHS, requiresExpandedLhsExpandedRhs, loop, alreadyPrinted);
		}
		return;

	}
	
	protected void dumpRequiresExpanded(String fileName, Map<String, HashSet<String>> requires, boolean printChain) {

		//	boolean printChain = true;

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);


		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
				int level = 1;
				if (printChain) {
					printChain(bw, level, lhs, requires, new LinkedHashSet<String>());
				} else {
					if (requires.get(lhs) != null) {
						for (String r : sortSet(requires.get(lhs))) {
							bw.append("     RHS <"+r+">\n");
						}
					}
				}
			}
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void printChain(BufferedWriter bw, int level, String lhs, Map<String, HashSet<String>> requires, HashSet<String> alreadyPrinted) {

		if (level > 50) {
			try {
				bw.append("Terminating recursion\n");
				return;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		StringBuilder blanks = new StringBuilder();
		for (int i = 0; i < level; i++) {
			blanks.append("  ");
		}

		for (String r : sortSet(requires.get(lhs))) {
			if (alreadyPrinted.contains(r)) {
				try {
					bw.append("                      ### looping on "+r+"\n");
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				continue;
			}
			try {
				bw.append(blanks.toString()+"RHS <"+r+"> \n");
				alreadyPrinted.add(r);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			printChain(bw, level+1, r, requires, alreadyPrinted);
		}

	}


	protected List<String> sortSet(HashSet<String> set) {
		ArrayList<String> al = new ArrayList<String>();
		if (set == null)
			return al;
		Iterator<String> iter = set.iterator();
		while (iter.hasNext()) {
			String aString = iter.next();
			if (aString == null) {
				System.out.println("Huh? sortSet");
				continue;
			}
			al.add(aString);
		}

		Collections.sort(al);

		return al;
	}

	protected boolean hasInitialValue(boolean considerInitialValues, String lhs, String expandedLHS, Map<String, 
			HashSet<String>> requiresExpanded, Map<String, HashSet<String>> lhsExpandedNotInitialized, Map<String, Equation> equations) {
		boolean initialized = false;

		Equation eqn = equations.get(lhs);
		// functions that support initial values
		if (containsInitialValueFunction(equations.get(lhs))) {

			initialized = true;
			Set<String> initializationVariables = equations.get(lhs).getFunctionInitialVariables();

			for (String var : initializationVariables) {

				String v = removeValueOf(var).replaceAll("\"", "");
				// the requires
				v = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(v);

				if (Parser.isNumber(v))
					continue;
				if (ArrayReference.isArrayReference(v)) {
					if (lhsExpandedNotInitialized.containsKey(v) && lhsExpandedNotInitialized.get(v).size() > 0 ) {
						initialized = false;
						//						continue;
					}

				} else /* scalar */ {
					if (lhsExpandedNotInitialized.containsKey(v) && lhsExpandedNotInitialized.get(v).size() > 0 ) {
						initialized = false;
					}
				}
				//		else if (requires.containsKey(v)) {
				//		    initialized = false;
				//		} else {
				//		    System.out.println("Not in requires: "+v);
				//		}
			}
		} 



		if (considerInitialValues) {
			if (!equations.get(lhs).isOrderedWithInitialValue()) {	
				equations.get(lhs).setOrderedWithInitialValue(initialized);
			}
			return initialized;
		} else {
			// do not consider initial values if it is a stock or ACTIVE INITIAL
			if (equations.get(lhs).isStock() || equations.get(lhs).isActiveInitial() || equations.get(lhs).isSampleIfTrue())
				return false;
			if (!equations.get(lhs).isOrderedWithInitialValue()) {	
				equations.get(lhs).setOrderedWithInitialValue(initialized);
			}

			return initialized;
		}
	}

	protected boolean hasInitialValue(boolean considerInitialValues, String lhs, Map<String, HashSet<String>> requires, Map<String, Equation> equations) {
		boolean initialized = false;

		Equation eqn = equations.get(lhs);

		if (eqn == null) {
			System.out.println("hasIV no eqn: "+lhs);
			return false; 
		}

		if (!considerInitialValues) {
			if (eqn.isStock() || eqn.isActiveInitial() || eqn.isSampleIfTrue())
				return false;
		}

		// functions that support initial values
		if (containsInitialValueFunction(equations.get(lhs))) {

			// HERE -- get a list of the initialization variables in the statement and check each of them

			//		if (Equation.isNumber(var))
			//			return true;
			//		if (requires.containsKey(var))
			//			return false;

			initialized = true;
			Set<String> initializationVariables = equations.get(lhs).getFunctionInitialVariables();

			for (String var : initializationVariables) {

				String v = removeValueOf(var).replaceAll("\"", "");
				// the requires
				v = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(v);

				if (Parser.isNumber(v))
					continue;
				else if (requires.containsKey(v)) {
					initialized = false;
				} else {
					//					System.out.println("Not in requires: "+v+" "+equations.get(lhs).getVensimEquationOnly());
				}
			}
		}
		if (!equations.get(lhs).isOrderedWithInitialValue()) {	
			equations.get(lhs).setOrderedWithInitialValue(initialized);
		}
		return initialized;
	}

	protected String removeValueOf(String var) {
		String retVar = var.replace("valueOf(", "").replaceAll("\"", "").replaceAll("\\)", "");
		return retVar;
	}

	protected boolean containsInitialValueFunction(Equation equation) {
		// want to traverse tree (via tokens)
		for (String token : equation.getTokens()) {
			if (Parser.isFunctionInvocation(token)) {
				FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(token);
				if (fd.isSuppliesInitialValue()) {
					return true;
				}
			} 
		}
		return false;
	}
	
	protected String[] removeBang(String[] inList) {
		String[] outList = new String[inList.length];
		for (int i = 0; i < inList.length; i++) {
			outList[i] = inList[i].replace("!", "");
		}
		return outList;
	}
}
