package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class EquationProcessor {
static Logger log = Logger.getLogger(EquationProcessor.class.getSimpleName());

    public static HashSet<String> rhsvariables = new LinkedHashSet<String>(); // code generation
    public static HashSet<String> functions = new LinkedHashSet<String>(); // none
    public static HashSet<String> lookups = new LinkedHashSet<String>(); // tokenize
    public static HashSet<String> subscripts = new LinkedHashSet<String>();
    public static Map<String, List<String>> subscriptValues = new LinkedHashMap<String, List<String>>();
    
    private static final String FUNCTION_OF = "A FUNCTION OF";
    
    private SystemDynamicsObjectManager sdObjectManager;
    
    private List<String> duplicateLHS = new ArrayList<String>();
    
    boolean processingMacroCode = false;
    
    private String currentModule;
    private String currentGroup;
    
    private List<String> groupOrder = new ArrayList<String>();
    private Map<String, String> groupText = new HashMap<String, String>();
    
    public EquationProcessor() {
    }

    public EquationProcessor(boolean processingMacroCode) {
	this.processingMacroCode = processingMacroCode;
    }
    
    public void checkUsage(Map<String, Equation> equations) {
    	for (Equation eqn : equations.values()) {
    		eqn.checkUsage(equations);
    	}
    }
    
    public Map<String, Equation> getFatalErrors(Map<String, Equation> equations) {
   	 Map<String, Equation> fatalErrors = new LinkedHashMap<String, Equation>();
   	 Iterator<String> iter = equations.keySet().iterator();
   	 while (iter.hasNext()) {
   		 String lhs = iter.next();
   		 Equation eqn = equations.get(lhs);
   		 if (eqn.isFatal())
   			 fatalErrors.put(lhs, eqn);
   	 }
   	 return fatalErrors;
   	
   }
    
    public Map<String, List<String>> getSyntaxErrorMessages(Map<String, Equation> equations) {
    	Map<String, List<String>> errorMessages = new LinkedHashMap<String, List<String>>();
   	 Iterator<String> iter = equations.keySet().iterator();
   	 while (iter.hasNext()) {
   		 String lhs = iter.next();
   		 Equation eqn = equations.get(lhs);
   		 if (!eqn.isSyntacticallyCorrect())
   			 errorMessages.put(lhs, eqn.getSyntaxMessages());
   	 }
   	 return errorMessages;
    }
    
    public Map<String, Equation> getSyntaxErrors(Map<String, Equation> equations) {
    	 Map<String, Equation> syntaxErrors = new LinkedHashMap<String, Equation>();
    	 Iterator<String> iter = equations.keySet().iterator();
    	 while (iter.hasNext()) {
    		 String lhs = iter.next();
    		 Equation eqn = equations.get(lhs);
    		 if (!eqn.isSyntacticallyCorrect())
    			 syntaxErrors.put(lhs, eqn);
    	 }
    	 return syntaxErrors;
    	
    }
    
    public Map<String, Equation> getUsageErrors(Map<String, Equation> equations) {
   	 Map<String, Equation> usageErrors = new LinkedHashMap<String, Equation>();
   	 Iterator<String> iter = equations.keySet().iterator();
   	 while (iter.hasNext()) {
   		 String lhs = iter.next();
   		 Equation eqn = equations.get(lhs);
   		 if (!eqn.isUsageCorrect())
   			 usageErrors.put(lhs, eqn);
   	 }
   	 return usageErrors;
   	
   }
    
    public HashMap<String, Equation> processMap(SystemDynamicsObjectManager sdObjectManager, List<String> vensimEquations/* , String units */) {
    	HashMap<String, Equation> equations = new LinkedHashMap<String, Equation>();
    	this.sdObjectManager = sdObjectManager;

    	// Need this here -- note that we explicity set Time = time in repeated loop during execution
    	
    	for (String eqn : vensimEquations) {
    		String lhs = "";
    		if (eqn.contains("="))
    			lhs = Parser.extractLHS(eqn);
    		EquationComponents mapEquationComponents = new EquationComponents(eqn);
    		if (eqn.contains(FUNCTION_OF)) {
    			equations.put(lhs, new Equation(mapEquationComponents, true));
    		} else {
    			equations.put(lhs, new Equation(mapEquationComponents));
    		}
    	}
    	
    	EquationComponents timeEquationComponents = new EquationComponents("Time=0.0~"+equations.get(TranslatorConstants.TIME_STEP).getUnits().split("\\[")[0]+
    			"~This is autogenerated12");
    	Equation timeEquation = new Equation(timeEquationComponents);
    	timeEquation.setAutoGenerated(true);
    	timeEquation.setIncludeInTranslation(false);
    	equations.put(timeEquation.getLhs(), timeEquation);
//    	EquationComponents naEquationComponents = new EquationComponents("NAREPLACEMENT = 0.0000001~any~This is autogenerated13");
    	EquationComponents naEquationComponents = new EquationComponents("NAREPLACEMENT = Double.MIN_VALUE~any~This is autogenerated13");
    	Equation naEquation = new Equation(naEquationComponents);
    	naEquation.setAutoGenerated(true);
    	naEquation.setIncludeInTranslation(false);
    	equations.put(naEquation.getLhs(), naEquation);

    	return equations;
    }
    public Map<String, Equation> processRawEquations(SystemDynamicsObjectManager sdObjectManager, List<String> rawEquations/* , String units */) {
    	HashMap<String, Equation> equations = new LinkedHashMap<String, Equation>();
    	this.sdObjectManager = sdObjectManager;
    	
//    	if (InformationManagers.getInstance().getSystemModel() == null) {
//    		log.debug("System Model is null - processRawEquations needs system model");
//    	}

    	
    	processRawEquations(equations, sdObjectManager,  rawEquations);
    	
    	if (!processingMacroCode) {
    		// Need this here -- note that we explicity set Time = time in repeated loop during execution
    		Equation ts = equations.get(TranslatorConstants.TIME_STEP);
    		if (ts == null) {
    			ts = equations.get(TranslatorConstants.TIME_STEP.replace(" ", "_"));
    		}
    		
        	EquationComponents timeEquationComponents = new EquationComponents("Time=0.0~"+ts.getUnits().split("\\[")[0]+
        			"~This is autogenerated14");
    		Equation timeEquation = new Equation(timeEquationComponents);
    		timeEquation.setAutoGenerated(true);
    		timeEquation.setIncludeInTranslation(false);
    		equations.put(timeEquation.getLhs(), timeEquation);
    		
//    		EquationComponents naEquationComponents = new EquationComponents("NAREPLACEMENT = 0.0000001~any~This is autogenerated15");
    		EquationComponents naEquationComponents = new EquationComponents("NAREPLACEMENT = Double.MIN_VALUE~any~This is autogenerated15");
    		Equation naEquation = new Equation(naEquationComponents);
    		naEquation.setAutoGenerated(true);
    		naEquation.setIncludeInTranslation(false);
    		equations.put(naEquation.getLhs(), naEquation);

    	}
    	return equations;
    }

    public HashMap<String, Equation> processRawEquations(HashMap<String, Equation> equations, 
    		SystemDynamicsObjectManager sdObjectManager, List<String> rawEquations) {
    	

    	String vensimEquation;
    	String aLine;
    	
    	currentGroup = InformationManagers.getInstance().getModelManager().getModelName();
    	currentModule = "Default";

    	// read all equations until we hit "\\\---///"

    	// this loop extends to the botton of the method

    	MutableInteger linePtr = new MutableInteger(0);

    	while (true) {
    		
    		boolean mustBalance = true;
    		
    		String initialLineTerminator = "";
    		String insertedEqual = "";
    		if (linePtr.value() >= rawEquations.size())
    			return equations;
    		aLine = rawEquations.get(linePtr.valueAndInc());
    	
    		// skip over empty lines prior to the start of an equation
    		while (aLine.length() == 0) {
    			if (linePtr.value() >= rawEquations.size() && processingMacroCode) {
    				return equations;
    			}
    			if (linePtr.value() >= rawEquations.size()) {
    				return equations;
    			}
    			aLine = rawEquations.get(linePtr.valueAndInc());
    			
    			
    		}
    		
			if (aLine.startsWith("{")) {
//				System.out.println("{ break");
				while (!aLine.endsWith("}")) {
					if (linePtr.value() >= rawEquations.size()) {
						return equations;
					}
					aLine = rawEquations.get(linePtr.valueAndInc());

				}
				continue;
			}
			
//			System.out.println("EQN: "+aLine);
       
    		// if the first line of the equation is continued onto additional lines
    		// we want to capture all lines and concatenate as a single line
    		if (aLine.contains("TABBED")) {

    			//		Sample Tabbed array equation:
    			//		We will translate this into a "normal" array initialization
    			//		Aggregated Definition[COP,Aggregated Regions]=1,0,0;1,0,0;...;0,0,1
    			//		Aggregated Definition[COP,Aggregated Regions]=TABBED ARRAY(
    			//			1	0	0
    			//			1	0	0
    			//			0	1	0
    			//			0	1	0
    			//			1	0	0
    			//			0	1	0
    			//			0	0	1
    			//			)
    			//			~	dmnl
    			//			~	Tabbed array to define to which aggregated region each of the COP blocs \
    			//				belongs.
    			//			|

    			aLine = aLine.split("=", 2)[0]+"=";
    			String anotherLine;
    			boolean newDataRecord = true;
    			anotherLine = rawEquations.get(linePtr.valueAndInc()).trim();
    			while (!anotherLine.endsWith(")")) {

    				// first check if this is a multi equation
    				// if so need to treat different

    				if (anotherLine.endsWith(") ~~|")) {
    					aLine += anotherLine.replace(") ", "");
    				} else	if (anotherLine.contains("TABBED")) {
        					aLine += anotherLine.replace("TABBED ARRAY(", "");
        					newDataRecord = true;
    				} else {
    					
    					// this is a continued line
    					if (anotherLine.endsWith("\\")) {
    						aLine += anotherLine.replace("\\", "").replace("\t", ",");
    					} else {

    						// this is a new line
    					String[] values = anotherLine.split("\t");
    					int vPtr = 0;
    					for (String value : values) {
    						String tval = value;
    						if (tval.equals("\\"))
    							continue;
    						if (value.endsWith("\\")) {
    							tval = value.replace("\\", "");
    						}
    						if (vPtr++ > 0)
    							aLine += ",";
    						aLine += tval;


    					}
    					aLine += ";";
    				}
    				}
    				anotherLine = rawEquations.get(linePtr.valueAndInc()).trim();
    			}

//    			System.out.println("TABBED: "+aLine);
    		} else {
    			// or unbalanced parens
    			if (aLine.endsWith("\\") /* || unbalancedParens(aLine)*/) {
    				String concat = aLine.replaceAll("\\\\$", ""); // .replace("\\", "");
    				while (aLine.endsWith("\\") /* || unbalancedParens(aLine) */) {
    					aLine = rawEquations.get(linePtr.valueAndInc());
    					concat += aLine.replaceAll("\\\\$", ""); // .replace("\\", "");
    				}
    				aLine = concat;
    				
    			}
    		}

    		// what can we learn from initial line?
    		boolean isAssignment = false, isLookup = false, isVdmLookup = false, isSubscriptDefinition = false,  unsure = true, isMulti = false;
    		boolean isMacroDefinition = false;
    		
//    		System.out.println("RAW: "+aLine);

    		// is it an assignment statement?
    		// assignment has both left and right sides
    		if (aLine.contains("=")) {
    			isAssignment = true;
    			unsure = false;
    			initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());

    			// ordering is important. Much check for macro before since macro definition
    			// meets criteria of non-quoted colon
    		} else if (aLine.contains(":MACRO:")) {
    			unsure = false;
    			isMacroDefinition = true;
    			initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());
    			// check if we have a subscript definition
    		} else if (Parser.containsNonquotedColon(aLine)
    				|| aLine.contains("<->")) { 
    			// nonQuotedColon
    			isSubscriptDefinition = true;
    			unsure = false;
    			initialLineTerminator = ":";
    			initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());
    			// how about a lookup?
    		
    			// is it a multiline with first line a vdmlookup?
    		} else if (aLine.endsWith("~~|")) {
    			// and multi-equation also
    			
    			isMulti = true;
    			unsure = false;
    			initialLineTerminator = "~~|";
    			// need to add vdmlookup invocation in this case
    			String lhs = aLine.replace("~~|", "").trim();
    			if (!aLine.contains("(")) {
    				isVdmLookup = true;
    				aLine = lhs + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(lhs)+"\")"+initialLineTerminator;
    			}
    		} else if ( Parser.matchesInlineLookup(aLine) || Parser.matchesInlineLookup(aLine+rawEquations.get(linePtr.value()))) { // /*aLine.endsWith("(") */ {
    			// Note: there cannot be an = in statement, but that is caught
    			// in initial comparison
//    			String nextLine = rawEquations.get(linePtr.value());
//    			List<String> toks = Parser.tokenize(aLine+nextLine);
    			
    			isLookup = true;
    			unsure = false;
    			initialLineTerminator = "(";
    		} else {
    			// if not an assignment &&
    			// not a subscript definition &&
    			// not a lookup definition &&
    			// not a multi-equation vmdlookup

    			// may be a single equation vdmlookup
    			// or a "special" assignment that doesn't have an =
    			// need to look at the next line to determine

    			unsure = true;

    		}

    		// this is the beginning of the graphics, return as we do not do anything with the graphics
    		if (aLine.contains("---///")) {
//    			markGenerated(equations);
//    			printVariableTypes(equations);
    			return equations;
    		}

    		// process macros independently
    		if (isMacroDefinition) {
    			//		System.err.println("Macros are currently not supported -- EXITING");
    			//		System.exit(1);
    			processMacro(aLine, rawEquations, linePtr);
    		} else {

    			// This is were we will need to deal with TABBED ARRAYS to read in properly.
    			// Just convert into ArrayInitialization

    			// start constructing the entire vensim equation
    			vensimEquation = new String(aLine); // .replace("\\", "");
    			int additionalLineCount = 0;

    			// Now process the rest of the Variable Equation package
    			// read until we hit the stand-alone "|"
    			while (true && linePtr.value() < rawEquations.size()) {

    				insertedEqual = "";

    				// start with the "second" line of the equation package
    				aLine = rawEquations.get(linePtr.valueAndInc());
    				
    				if (aLine.startsWith("~\t"))
    					mustBalance = false;

    				// is this a continued line? if so read it in in its entirety
    				if (aLine.endsWith("\\") || (mustBalance && unbalancedParens(aLine))) {
    					String concat = aLine.replaceAll("\\\\$", "");   // MJB Sep302019
    					while (aLine.endsWith("\\") || (mustBalance && unbalancedParens(concat))) {
    						if (linePtr.value() >= rawEquations.size())
    							System.out.println("DEBUG Aug27");
    						aLine = rawEquations.get(linePtr.valueAndInc());
    						if (aLine.startsWith("~\t"))
    	    					mustBalance = false;
    						
    						concat += aLine.replaceAll("\\\\$", "");
    					}
    					unsure = false;
    					aLine = concat;
    					
    				}

    				additionalLineCount++;
    				

    				// check for validity and termination conditions

    				if (aLine == null) {
    					System.err.println("Unexpected EOF!!!");
    				}
    				if (aLine.contains("---///")) {
    					System.err.println("Found graphics marker in wrong place!");
    				}
    				// are we finished with the package?
    				if (aLine.endsWith("|") && !aLine.endsWith("~~|")) { // used to be \t| the trim() on readLine() has thrown this off
    					break;
    				}

    				// Note that each equation in a multi-equation package could be a different type!
    				// so we need to repeat the logic from above. This should really go into a separate
    				// method that will deal with everything 
    				//					This needs work


    				// we could determine at this point that this is actually a multi-equation package
    				if (aLine.endsWith("~~|")) {
    					isMulti = true;
    				}
    				if (isMulti) {
    					if (isVdmLookup) {
    						String lhs = aLine.replace("~~|", "").trim();
    						aLine = lhs + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(lhs)+"\")"+initialLineTerminator;
    					}

    				} else {
    					// special processing based on what we have from first line

    					if (unsure) {
    						// may be a single equation vdmlookup
    						// or a "special" assignment that doesn't have an =
    						// need to look at the next line to determine
    						if (aLine.startsWith("~")) {
    							isVdmLookup = true;
    							unsure = false;
    							// need to modify current vensimEquation
    							// if this is an array reference, need to add array.
    							//						if (vensimEquation.contains("["))
    							//						    vensimEquation = "array." + vensimEquation;
    							vensimEquation = vensimEquation + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(vensimEquation)+"\")";
    						} else {
    							// this should be a number and thus an assignment
    							isAssignment = true;
    							unsure = false;
    							if (!vensimEquation.startsWith("***"))
    								insertedEqual = " = ";
    						}

    					} else {

    					}

    				}
    				
    				if (vensimEquation.startsWith("***") && insertedEqual.length() > 0) {
    					log.debug("I added = to a group");
    				}

    				vensimEquation = vensimEquation + insertedEqual + aLine; // .replace("\\", "");

    			}
    			
    			// vensimEquation is equation text
    			if (!vensimEquation.toUpperCase().contains(":THE CONDITION:")  && !vensimEquation.toUpperCase().contains(":IMPLIES:") && 
    					!vensimEquation.toUpperCase().contains(":TEST INPUT:")){

    			// we now have the equation text
    			// skip "equations" that begin with "*" -- section markers
    			if (!vensimEquation.startsWith("***")){
    				
    				// trap any equations requiring special processing prior to creating
    				List<String> eqnToProcess = new ArrayList<String>();
    				//				    eqnToProcess.add(vensimEquation);
    				eqnToProcess = splitMultipleEquations(vensimEquation);
    				for (String aVensimEquation : eqnToProcess) {
    				
    					
    					if (aVensimEquation.contains("WITH LOOKUP") /* || isLookup */) {
    						processWithLookup(aVensimEquation, equations);
    					} else {
			
    						EquationComponents equationComponents = new EquationComponents(aVensimEquation);
    						Equation equation;
    						
    						if (!equationComponents.isHasEquation()) {
    			    			equation =  new Equation(equationComponents, true);
    			    		} else {
    			    			equation = new Equation(equationComponents);
    			    		}
    						
    						
//    						Equation equation = new Equation(equationComponents);
    						if (equation.isAbort()) {
    							log.debug("ParsingAbort "+equation.getVensimEquation());
    						}
    						equation.setGroup(currentGroup);
    						equation.setModule(currentModule);
    						
    						if (!equation.isSyntacticallyCorrect() || equation.isAbort()) {
    							// even if it is not correct put into equation set
    							if (!isDuplicate(equation.getLhs(), equations)) {
    								equations.put(equation.getLhs(), equation);
    							} else {
    								log.debug("Not saved due to duplicate (1) "+equation.getLhs());
    							}
    						} else {

    							if (equation.getLhs() == null) {
    								log.debug("Equation with no LHS: "+aVensimEquation);
    								if (!isDuplicate(equation.getLhs(), equations))
    									equations.put(equation.getLhs(), equation);
    							} else {
    								if (!isDuplicate(equation.getLhs(), equations)) {
    									equations.put(equation.getLhs(), equation);

    									String screenName = SystemDynamicsObjectManager.getScreenName(equation.getLhs());
    									if (equation.isAssignment() || equation.isDefinesLookup() || equation.isDefinesSubscript() || equation.isJustMap())
    										sdObjectManager.addEquation(screenName, equation);
    								} else {
    									log.debug("Not saved due to duplicate (2) "+equation.getLhs());
    								}
    							}
    						}
    					}
    				}
    			} else {
//    						    log.debug("Skipping <"+vensimEquation+">");
    				currentGroup = vensimEquation.split("~")[0].replace("*", "");
    				groupOrder.add(currentGroup);
    				groupText.put(currentGroup, vensimEquation);
    				log.debug("Setting currentGroup: <"+currentGroup+">");
    			}
    		}
    		}
    	}	
    	
 
    	

    }
    
    private boolean unbalancedParens(String s) {
    	
    	MutableInteger pos = new MutableInteger(0);
    	int parenCount = 0;
    	while (Parser.inRange(pos, s)) {
    		if (Parser.characterAt(pos, s).equals("(")) {
    			parenCount++;
    		} else if (Parser.characterAt(pos, s).equals(")")) {
    			parenCount--;
    		}
    		pos.add(1);
    			
    	}
    	
    	return parenCount != 0;
    }
    
    private void printVariableTypes(HashMap<String, Equation> equations) {
    	log.debug("###printVariableTypes###");
    	for (String key : equations.keySet()) {
    		// subscript definitions have no type in this context
    		if (equations.get(key).isDefinesSubscript())
    			continue;
    		log.debug(equations.get(key).getVensimEquationOnly());
//    		log.debug(equations.get(key).getVariableType());
    	}
    }
    
    private void markGenerated(HashMap<String, Equation> equations) {
//   	for (String lhs : new String[] {"InitialTime", "FinalTime", "TimeStep", "Savper", "INITIAL_TIME", "FINAL_TIME", "TIME_STEP", "SAVPER" ,"INITIAL TIME", "FINAL TIME", "TIME STEP", "SAVEPER"}) {
   		for (String lhs : new String[] {"Savper",  "SAVPER" ,"SAVEPER"}) {
    		if (equations.containsKey(lhs))
    			equations.get(lhs).setAutoGenerated(true);
    	}
    }
    
    private boolean isDuplicate(String lhs, Map<String, Equation> equations) {
    	if (equations.containsKey(lhs)) {
    		this.duplicateLHS.add(lhs);
    		log.debug("Duplicate LHS found: "+lhs+" "+equations.get(lhs).getVensimEquationOnly());
    		return true;
    	} else {
    		return false;
    	}
    }



    private List<String> splitMultipleEquations(String vensimEquation) {
	List<String> eqn = new ArrayList<String>();
//	log.debug("CHECKING FOR MULTIPLE EQUATION");
	if (vensimEquation.contains("~~|")) {
		
//		log.debug("FOUND MULTIPLE EQUATION");

	    // split multi-equation definition into multiple single equation definitions

	    String[] parts = vensimEquation.split("~\t", 2);
	    String[] commands = parts[0].split("~~\\|");
	    String units = "\t~\t" + parts[1];

	    for (String cmd : commands) {

		eqn.add(cmd + units);
	    }


	    return eqn;
	} else {
	    eqn.add(vensimEquation);
	    return eqn;
	}
    }

    private void processMacro(String eqn, List<String> rawEquations, MutableInteger linePtr) {

	if (!InformationManagers.getInstance().getMacroManager().readMacroAndProcess(eqn, rawEquations, linePtr)) {
	    log.debug("Bad Macro");;
	}

    }

    private void processWithLookup(String eqn, HashMap<String, Equation> equations) {
    	//		"Op: Effect of Fatigue on Capacity"= WITH LOOKUP (
    	//		"Op: Fatigue Normalized",
    	//			([(0,0)-(4,2)],(0,1.25),(0.35,1.2),(0.65,1.12),(1,1),(1.35,0.85),(1.65138,0.692982)\
    	//			,(2,0.5),(2.5,0.25),(3,0.1),(4,0.1) ))


    	// NOTE: if the LHS contains an array reference, we want to name the lookup table without the 
    	// array reference being included

    	// lhs
    	
    	String[] parts = eqn.split("~")[0].split("=", 2);
    	boolean withQuotes = false;
    	if (parts[0].contains("\""))
    		withQuotes = true;
    	String l = "";

    	String lhsArrayReference = "";

    	if (parts[0].contains("[")) {
    		String[] withAR = parts[0].split("\\[");
    		parts[0] = withAR[0];
    		lhsArrayReference = "["+withAR[1];
    	}


    	l = parts[0].replaceAll("\"", "");
    	String newl = l + StringConstants.RSSD_LOOKUP;
    	if (withQuotes)
    		newl = "\"" + newl + "\"";
    	
    	String[] extra = eqn.split("~");
    	String comment = "";
    	String units = "";
    	if (extra.length > 1)
    		units = extra[1];
    	if (extra.length > 2)
    		comment = extra[2];

    	// if the rhs contains an array reference, then we cannot
    	// split the whole thing on a ",". 
    	// change the subscript commas to "###" and then convert back

    	// need to scan the rhs changing any "," in an array reference to "###"
    	// there can be any number
    	
    	MutableInteger rhsPos = new MutableInteger();
    	String arg = extractWithLookupArgument(parts[1], rhsPos);
    	
    	String newr = parts[1].replace(arg+",", "");
    	
    	// clean newr
    	newr = newr.replace("WITH LOOKUP", "");
    	MutableInteger pos = new MutableInteger();
    	Parser.skipWhiteSpace(pos, newr);
    	newr = newr.substring(pos.value()+1, newr.length()-1);
    	
//    	WITH LOOKUP (([(-6,-2)-(6,2)],(-5,0.75),(-4,0.8),(-3,0.85),(-2,0.9),(-1,0.95),(0,1),(1,1.05),(2,1.1),(3,1.15),(4,1.2),(5,1.25) ))~	Dmnl
    	
    	// should be looking at our comma (arg separator)

    	StringBuilder rhsBuf = new StringBuilder();
    	parts[1] = alterArrayReferenceCommas(parts[1]);

    	newr = newr.substring(0, newr.length());

    	EquationComponents equationComponents = new EquationComponents(newl+newr+"~"+units+"~"+comment);
    	Equation equation = new Equation(equationComponents);
    	equation.setOriginalVensimEquation(eqn);
    	equation.setIncludeInTranslation(true);
    	equation.setModule(currentModule);
    	equation.setGroup(currentGroup);
    	if (equation.getLhs() == null) {
    		log.debug("Equation with no LHS: "+newl+newr);
    	} else {

    		String screenName = SystemDynamicsObjectManager.getScreenName(equation.getLhs());
    		sdObjectManager.addEquation(screenName, equation);
    		equations.put(equation.getLhs(), equation);
    	}
    	equation.setAutoGenerated(true);
    	// now reconstruct the original with a lookup reference

    	String newEqn = parts[0]+lhsArrayReference + " = "+ newl+"("+arg+")";
    	// need to grab units and comment from original rhs
    	
    	equationComponents = new EquationComponents(newEqn+"~"+units+"~"+comment);
    	equation = new Equation(equationComponents);
    	equation.setIncludeInTranslation(false);
    	equation.setModule(currentModule);
    	equation.setGroup(currentGroup);
    	if (equation.getLhs() == null) {
    		log.debug("Equation with no LHS: "+newEqn);
    	} else {
    		if (!isDuplicate(equation.getLhs(), equations)) {
    			equations.put(equation.getLhs(), equation);
    			
//    			log.debug("WithLookup: "+equation.getVensimEquationOnly());

    			String screenName = SystemDynamicsObjectManager.getScreenName(equation.getLhs());
    			sdObjectManager.addEquation(screenName, equation);
    		}
    	}
    }
    
    private String extractWithLookupArgument(String rhs, MutableInteger pos) {
    	StringBuilder sb = new StringBuilder();
    	String s = new String(rhs);
    	// first remove with lookup
    	s = s.replace("WITH LOOKUP", "");
    	pos.setValue(0);
    	Parser.skipWhiteSpace(pos, s);
    	// we are now pointing at "(" so skips
    	pos.add(1);
    	Parser.skipWhiteSpace(pos, s);
    	// now pointing to first char of argument
    	// grab until we find argument separator
    	sb.append(extractExpression(pos, s));
    	
    	return sb.toString();
    }
    
    private String extractExpression(MutableInteger pos, String s) {
    	MutableInteger savePos = new MutableInteger(pos);
    	StringBuilder sb = new StringBuilder();
    	
    	// the argument separator is our end marker
    	while(Parser.inRange(pos, s)) {
    		String c = Parser.characterAt(pos, s);
    		if (Parser.isFunctionArgumentSeparator(c)) {
    			return sb.toString();
    		} else if (c.equals("(")) {
    			sb.append(c);
    			String e = extractTo(s, ")", pos);
    			if (e == null) {
    				pos.setValue(savePos.value());
    				return null;
    			} else {
    				sb.append(e);
    			}
    		} else if (c.equals("[")) {
    			sb.append(c);
    			String e = extractTo(s, "]", pos);
    			if (e == null) {
    				pos.setValue(savePos.value());
    				return null;
    			} else {
    				sb.append(e);
    			}
    		} else {
    			sb.append(c);
    		}
    		pos.add(1);
    	}
    	
    	// if we reach, error
    	pos.setValue(savePos.value());
		return null;
    	
    }
    
    private String extractTo(String s, String endMarker, MutableInteger pos) {
    	MutableInteger savePos = new MutableInteger(pos);
    	String startMarker = Parser.characterAt(pos, s);
    	int openCount = 1;
    	// by definition, pos points to first occurence of startMarker (i.e."(" or "[" or "{"
    	pos.add(1);
    	
    	StringBuilder sb = new StringBuilder();
    	while(Parser.inRange(pos, s)) {
    		String c = Parser.characterAt(pos, s);
    		if (c.equals(startMarker)) {
    			openCount++;
    		} else if (c.equals(endMarker)) {
    			openCount--;
    			if (openCount == 0) {
    				sb.append(c);
    				return sb.toString();
    			}
    		}
    		sb.append(c);
    		pos.add(1);
    		
    	}
    	
    	// if fall through, no match, restore pos and return null
    	pos.setValue(savePos.value());
    	return null;
    }
    
    private void processWithLookupORIG(String eqn, HashMap<String, Equation> equations) {
    	//		"Op: Effect of Fatigue on Capacity"= WITH LOOKUP (
    	//		"Op: Fatigue Normalized",
    	//			([(0,0)-(4,2)],(0,1.25),(0.35,1.2),(0.65,1.12),(1,1),(1.35,0.85),(1.65138,0.692982)\
    	//			,(2,0.5),(2.5,0.25),(3,0.1),(4,0.1) ))


    	// NOTE: if the LHS contains an array reference, we want to name the lookup table without the 
    	// array reference being included

    	// lhs
    	String[] parts = eqn.split("=", 2);
    	boolean withQuotes = false;
    	if (parts[0].contains("\""))
    		withQuotes = true;
    	String l = "";

    	String lhsArrayReference = "";

    	if (parts[0].contains("[")) {
    		String[] withAR = parts[0].split("\\[");
    		parts[0] = withAR[0];
    		lhsArrayReference = "["+withAR[1];
    	}


    	l = parts[0].replaceAll("\"", "");
    	String newl = l + StringConstants.RSSD_LOOKUP;
    	if (withQuotes)
    		newl = "\"" + newl + "\"";

    	// if the rhs contains an array reference, then we cannot
    	// split the whole thing on a ",". 
    	// change the subscript commas to "###" and then convert back

    	// need to scan the rhs changing any "," in an array reference to "###"
    	// there can be any number

    	StringBuilder rhsBuf = new StringBuilder();
    	parts[1] = alterArrayReferenceCommas(parts[1]);

    	String[] rhs = parts[1].split(",", 2);
    	String newr = rhs[1].trim();
    	newr = newr.substring(0, newr.length());

    	EquationComponents equationComponents = new EquationComponents(newl+newr);
    	Equation equation = new Equation(equationComponents);
    	if (equation.getLhs() == null) {
    		log.debug("Equation with no LHS: "+newl+newr);
    	} else {

    		String screenName = SystemDynamicsObjectManager.getScreenName(equation.getLhs());
    		sdObjectManager.addEquation(screenName, equation);
    		equations.put(equation.getLhs(), equation);
    	}
    	equation.setAutoGenerated(true);
    	// now reconstruct the original with a lookup reference

    	String newEqn = parts[0]+lhsArrayReference + "="+newl+rhs[0].replaceAll("WITH LOOKUP", "").replaceAll("###", ",")+")";
    	// need to grab units and comment from original rhs
    	String[] extra = rhs[1].split("~");
    	String comment = "";
    	String units = "";
    	if (extra.length > 1)
    		units = extra[1];
    	if (extra.length > 2)
    		comment = extra[2];
    	equationComponents = new EquationComponents(newEqn+"~"+units+"~"+comment);
    	equation = new Equation(equationComponents);
    	if (equation.getLhs() == null) {
    		log.debug("Equation with no LHS: "+newEqn);
    	} else {
    		if (!isDuplicate(equation.getLhs(), equations)) {
    			equations.put(equation.getLhs(), equation);
    			
//    			log.debug("WithLookup: "+equation.getVensimEquationOnly());

    			String screenName = SystemDynamicsObjectManager.getScreenName(equation.getLhs());
    			sdObjectManager.addEquation(screenName, equation);
    		}
    	}
    }

    private String alterArrayReferenceCommas(String rhs) {
	StringBuilder sb = new StringBuilder();
	boolean inArrayReference = false;
	boolean doneChecking = false;
	MutableInteger position = new MutableInteger(0);

	while (Parser.inRange(rhs, position)) {
	    if (Parser.characterAt(rhs, position).equals("[") && !doneChecking) {
		inArrayReference = true;
		sb.append(Parser.characterAt(rhs, position));
	    } else if (Parser.characterAt(rhs, position).equals("]")&& !doneChecking) {
		inArrayReference = false;
		sb.append(Parser.characterAt(rhs, position));
	    } else if (Parser.characterAt(rhs, position).equals(",")&& !doneChecking) {
		if (inArrayReference) {
		    inArrayReference = false;
		    sb.append("###");
		} else {
		    sb.append(",");
		    doneChecking = true;
		}
	    } else {
		sb.append(Parser.characterAt(rhs, position));
	    }
	    position.add(1);
	}

	return sb.toString();
    }

	public List<String> getDuplicateLHS() {
		return duplicateLHS;
	}

	public Map<String, String> getGroupText() {
		return groupText;
	}

	public void setGroupText(Map<String, String> groupText) {
		this.groupText = groupText;
	}

	public List<String> getGroupOrder() {
		return groupOrder;
	}
	
	public void addToHeadOfGroupOrder(String s) {
		List<String> al = new ArrayList<String>();
		al.add(s);
		al.addAll(groupOrder);
		groupOrder = al;
	}

	public void setGroupOrder(List<String> groupOrder) {
		this.groupOrder = groupOrder;
	}
}
