package gov.anl.gss.sdm.server.model.translator;

public class ExpressionGenerator {
	
	public static String generateExpression(Node node) {
		return generateExpression(node, true);
	}
	
	public static String generateExpression(Node node, boolean translateToSDName) {
		StringBuilder sb = new StringBuilder();

		if (node != null) {


			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child  
			if (Parser.isArithmeticOperator(node.getToken()) || Parser.isEqualSign(node.getToken())
					|| Parser.isRelationalOperator(node.getToken())) {
				if (!Parser.isEqualSign(node.getToken()))
					sb.append("(");
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(generateExpression(TreeTraversal.getLhs(node), translateToSDName)); // node.getChild()
					sb.append(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(node.getToken()));
					sb.append(generateExpression(TreeTraversal.getRhs(node), translateToSDName)); // node.getChild().getNext()
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(generateExpression(node.getChild(), translateToSDName)); // this is really a LHS location, but RHS of unary
				}
				if (!Parser.isEqualSign(node.getToken()))
					sb.append(")");
				return sb.toString();

			} else if (Parser.isFunctionInvocation(node.getToken())) {
				sb.append(generateFunctionCall(node, translateToSDName));
			} else {
				if (translateToSDName)
					sb.append(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(node.getToken()));
				else
					sb.append(node.getToken());
			}

		}

		return sb.toString();
	}
	
	public static String generateFunctionCall(Node node, boolean translateToSDName) {
		StringBuilder sb = new StringBuilder();
		
		if (node != null) {
			// create the entire invocation
			// node is the function name
			sb.append(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(node.getToken()));
			sb.append("(");
			int i = 0;
			
			Node arg = TreeTraversal.getFunctionArgument(node, 1);
			while (arg != null) {
				if (i++ > 0)
					sb.append(",");
				sb.append(generateExpression(arg, translateToSDName));
				arg = arg.getNext();
			}
			
			sb.append(")");
			
		}
		
		return sb.toString();
	}

}
