package gov.anl.gss.sdm.server.model.translator;


public class FunctionDescription {

	private static final String FIELD_SEPARATOR = "\t";
	public static final String SCALAR = "scalar";
	public static final String VECTOR = "vector";

	private String externalName;
	private String internalName;
	private boolean supported = false;
	private String javaObject;
	private boolean requiresName = false;
	private boolean requiresValue = false;
	private boolean requiresTime = false;
	private boolean requiresTimeStep = false;
	private String returnType;
	private String returnUnits;
	private boolean suppliesInitialValue = false;
	private int initialValueArg;
	private int minNumArgs;
	private int maxNumArgs;
	private String returnDimension;

	public String getReturnDimension() {
		return returnDimension;
	}

	private String[] argUnits;
	private String[] argDimensions;



	private boolean variableLengthArguments = false;
	private boolean variableLengthNoLimit = false;

	public FunctionDescription(String description) {
		parse(description);
	}

	private void parse(String description) {
		String[] fields = description.split(FIELD_SEPARATOR);
		
		int field = 0;
		
		setExternalName(fields[field++]);
		setInternalName(fields[field++]);
		setSupported(fields[field++].equals("Y"));
		setJavaObject(fields[field++]);
		setRequiresName(fields[field++].equals("Y"));
		setRequiresValue(fields[field++].equals("Y"));
		setRequiresTime(fields[field++].equals("Y"));
		setRequiresTimeStep(fields[field++].equals("Y"));
		setReturnType(fields[field++]);
		setReturnDimension(fields[field++].toLowerCase().equals(SCALAR) ? SCALAR : VECTOR);
		setReturnUnits(fields[field++]);
		setSuppliesInitialValue(fields[field++].equals("Y"));
		if (fields[field-1].equals("Y")) {
			setInitialValueArg(Integer.parseInt(fields[field++]));
		} else {
			field++;
			setInitialValueArg(-1);
		}
		setMinNumArgs(Integer.parseInt(fields[field++]));
		setMaxNumArgs(Integer.parseInt(fields[field++]));
		if (!isVariableLengthNoLimit()) {
			argUnits = new String[getMaxNumArgs()];
			argDimensions = new String[getMaxNumArgs()];
			int pos = 0;
			int tField = field;
			for (int i = tField; i < tField + getMaxNumArgs(); i++) {
				String unitDim = fields[field++];
				argDimensions[pos] = unitDim.contains("/v") ? VECTOR : SCALAR;
				argUnits[pos++] = unitDim.replace("/v", "");
				
			}
		} else {
			argUnits = new String[getMinNumArgs()];
			argDimensions = new String[getMaxNumArgs()];
			int pos = 0;
			int tField = field;
			for (int i = tField; i < tField + getMinNumArgs(); i++) {
				String unitDim = fields[field++];
				argDimensions[pos] = unitDim.contains("/v") ? VECTOR : SCALAR;
				argUnits[pos++] = unitDim.replace("/v", "");
			}
		}
	}

	//    public String getAlternateReturnUnit(int funcReturnIndex) {
	//	
	//    }

	public boolean returnsVector() {
		return returnDimension.equals(VECTOR);
	}
	
	public boolean returnsScalar() {
		return returnDimension.equals(SCALAR);
	}
	
	public int getNumberOfAddedArguments() {
		int n = 0;
		if (isRequiresName())
			n++;
		if (isRequiresTime())
			n++;
		if (isRequiresTimeStep())
			n++;
		if (isRequiresValue())
			n++;
		return n;
	}

	public void setReturnDimension(String returnDimension) {
		this.returnDimension = returnDimension;
	}

	public String getExternalName() {
		return externalName;
	}
	public void setExternalName(String externalName) {
		this.externalName = externalName;
	}
	public String getInternalName() {
		return internalName;
	}
	public void setInternalName(String internalName) {
		this.internalName = internalName;
	}
	public boolean isSupported() {
		return supported;
	}
	public void setSupported(boolean supported) {
		this.supported = supported;
	}
	public String getJavaObject() {
		return javaObject;
	}
	public void setJavaObject(String javaObject) {
		this.javaObject = javaObject;
	}
	public boolean isRequiresName() {
		return requiresName;
	}
	public void setRequiresName(boolean requiresName) {
		this.requiresName = requiresName;
	}
	public boolean isRequiresValue() {
		return requiresValue;
	}
	public void setRequiresValue(boolean requiresValue) {
		this.requiresValue = requiresValue;
	}
	public boolean isRequiresTime() {
		return requiresTime;
	}
	public void setRequiresTime(boolean requiresTime) {
		this.requiresTime = requiresTime;
	}
	public boolean isRequiresTimeStep() {
		return requiresTimeStep;
	}
	public void setRequiresTimeStep(boolean requiresTimeStep) {
		this.requiresTimeStep = requiresTimeStep;
	}
	public String getReturnType() {
		return returnType;
	}

	public String getAlternateReturnUnits() {
		StringBuilder sb = new StringBuilder();
		int definedReturnIndex = Integer.parseInt(returnUnits.replace("arg", ""))-1;
		for (int i = 0; i < minNumArgs; i++) {
			if (i == definedReturnIndex)
				continue;
			if (argUnits[i].equals(argUnits[definedReturnIndex])) {
				sb.append(",");
				sb.append("arg"+(i+1));
			}
		}

		return sb.toString();
	}

	public boolean requiresUnitExpression() {

		if (returnUnits.contains("/") || returnUnits.contains("*")) {
			return true;
		} else {
			return false;
		}
	}

	public static String getFunctionReturn(String func) {
		String funcAndReturn = func.replace("sdFunctions.", "");
		if (funcAndReturn.contains("<")) {
			return funcAndReturn.split("<")[1].split(">")[0];
		} else {
			return "";
		}
	}

	public static int getFunctionReturnIndex(String loadedFunctionName) {
		String funcReturn = getFunctionReturn(loadedFunctionName);
		if (funcReturn.contains("/") || funcReturn.contains("*")) {
			return -1;
		}
		if (funcReturn.startsWith("arg")) {
			return Integer.parseInt(funcReturn.replace("arg", ""))-1;
		}
		return -2;
	}

	public static String getFunctionName(String loadedFunctionName) {
		String funcAndReturn = loadedFunctionName.replace("sdFunctions.", "");
		if (funcAndReturn.contains("<")) {
			return funcAndReturn.split("<")[0];
		} else {
			return funcAndReturn;
		}
	}

	public void setReturnType(String returnType) {
		this.returnType = returnType;
	}
	public String getReturnUnits() {
		return returnUnits /* +getAlternateReturnUnits() */;
	}
	public void setReturnUnits(String returnUnits) {
		this.returnUnits = returnUnits;
	}
	public int getNumArgs() {
		return minNumArgs;
	}

	public int firstActualArgPosition() {
		int n = 0;
		if (isRequiresName()) n++;
		if (isRequiresValue()) n++;
		if (isRequiresTime()) n++;
		if (isRequiresTimeStep()) n++;
		return n;
	}

	public int getNumArgsAll() {
		int n = minNumArgs;
		if (isRequiresName()) n++;
		if (isRequiresValue()) n++;
		if (isRequiresTime()) n++;
		if (isRequiresTimeStep()) n++;
		return n;
	}

	public void setMinNumArgs(int numArgs) {
		// variable length signified with a negative min number of args
		if (numArgs < 0) {
			setVariableLengthArguments(true);

			// force these this

			setRequiresName(true);

		}
		this.minNumArgs = Math.abs(numArgs);
	}
	public String[] getArgUnits() {
		return argUnits;
	}
	public void setArgUnits(String[] argUnits) {
		this.argUnits = argUnits;
	}

	public boolean isSuppliesInitialValue() {
		return suppliesInitialValue;
	}

	public void setSuppliesInitialValue(boolean suppliesInitialValue) {
		this.suppliesInitialValue = suppliesInitialValue;
	}

	public boolean isVariableLengthArguments() {
		return variableLengthArguments;
	}

	public void setVariableLengthArguments(boolean variableLengthArguments) {
		this.variableLengthArguments = variableLengthArguments;
	}

	public int getMaxNumArgs() {
		return maxNumArgs;
	}

	public void setMaxNumArgs(int maxNumArgs) {
		if (maxNumArgs < 0)
			setVariableLengthNoLimit(true);
		this.maxNumArgs = Math.abs(maxNumArgs);
	}

	public int getMinNumArgs() {
		return minNumArgs;
	}

	public boolean isVariableLengthNoLimit() {
		return variableLengthNoLimit;
	}

	public void setVariableLengthNoLimit(boolean variableLengthNoLimit) {
		this.variableLengthNoLimit = variableLengthNoLimit;
	}
	
	public String[] getArgDimensions() {
		return argDimensions;
	}

	public void setArgDimensions(String[] argDimensions) {
		this.argDimensions = argDimensions;
	}
	
	public String getArgDimensions(int arg) {
		return argDimensions[arg];
	}

	public int getInitialValueArg() {
		return initialValueArg;
	}

	public void setInitialValueArg(int initialValueArg) {
		this.initialValueArg = initialValueArg;
	}
}
