package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class FunctionManager {
static Logger log = Logger.getLogger(FunctionManager.class.getSimpleName());
	

	private Map<String, FunctionDescription> allFunctions;
	private String functionFile;

	public FunctionManager() {
		allFunctions = new LinkedHashMap<String, FunctionDescription>();
	}

//	public void load(String file) {
//		functionFile = file;
//		load();
//	}
	
	 public void load(String file) {
		    try {
		    	
		    	log.info("OPENING "+file);
		    	log.info("LOCATION: "+new File(file).getAbsolutePath());
		    	
		      load(new FileInputStream(file));
		    } catch (IOException ex) {
		      ex.printStackTrace();
		    }
		  }


	public OperationResult validateFunctionReference(Map<String, Equation> equations, Equation equation, MutableInteger pos, MutableBoolean lhs) {
		OperationResult or = new OperationResult();
		List<String> tokens = equation.getTokens();
		String token = tokens.get(pos.value());
		FunctionDescription fd = getDescription(token);
		if (fd != null)
			return or;


		or.setErrorMessage("Function not registered "+token);

		return or;
	}

	public void load(InputStream in) {
		BufferedReader fileReader = null;

		String aLine;
		// open the file for reading
		
			fileReader = new BufferedReader (new InputStreamReader(in));
		

		try {
			aLine = fileReader.readLine(); // header record
			aLine = fileReader.readLine();
			while (aLine != null) {

				// force all functions to upper case

				FunctionDescription fd = new FunctionDescription(aLine);
				allFunctions.put(fd.getExternalName().toUpperCase(), fd);
				allFunctions.put(clean(fd.getExternalName().toUpperCase()), fd);
				allFunctions.put(fd.getInternalName().toUpperCase(), fd);
				
				allFunctions.put(fd.getExternalName().toUpperCase().replace(" ", "_"), fd);
				allFunctions.put(clean(fd.getExternalName().toUpperCase()).replace(" ", "_"), fd);
				allFunctions.put(fd.getInternalName().toUpperCase().replace(" ", "_"), fd);
				aLine = fileReader.readLine();
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
	}

	public boolean isVensimFunction(String function) {

		return allFunctions.containsKey(function.replace("sdFunctions.", "").toUpperCase());
	}

	public FunctionDescription getDescription(String function) {

		if (allFunctions.get(function.replace("sdFunctions.", "").toUpperCase()) == null) {
			printFunctions();
			return null;
		}

		return allFunctions.get(function.replace("sdFunctions.", "").toUpperCase());
	}

	private void printFunctions() {
		List<String> funcs = new ArrayList<String>();
		for (String key : allFunctions.keySet())
			funcs.add(key);
		Collections.sort(funcs);
		for (String func : funcs) 
			log.debug("Function: "+func);
	}

	public boolean canCheckUnitsConsistency(Equation teqn) {

		boolean canCheck = true;
		String eqn = teqn.getVensimEquationOnly();

		// some functions do not allow unit consistency checks
		// if this equation contains one of them, we'll just skip it.
		for (String functionName : allFunctions.keySet()) {
			FunctionDescription fd = allFunctions.get(functionName);
			if ((fd.getReturnUnits().equals("NA") && teqn.callsFunction(functionName)) ||
					(functionName.equalsIgnoreCase("LOOKUP") && teqn.callsFunction(functionName)))
				return false;
		}

		return canCheck;

	}

	private String clean(String s) {
		return  Parser.removeQuotes(s).replace(" ", "")
				.replace(":", "_")
				.replace("\t", "")
				.replace("&", "_")
				.replace("-", "_")
//				.replace("\"", "")
				.replace("(", "_")
				.replace(")", "_")
				.replace("'", "");
	}

}
