package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.analysis.polarity.ArrowPolarity;

import java.io.BufferedWriter;
import java.io.IOException;

import org.apache.log4j.Logger;

public class GraphicObject {
static Logger log = Logger.getLogger(GraphicObject.class.getSimpleName());
    
    public final static long mask0 = 1 << 0;
    public final static long mask1 = 1 << 1;
    public final static long mask2 = 1 << 2;
    public final static long mask3 = 1 << 3;
    public final static long mask4 = 1 << 4;
    public final static long mask5 = 1 << 5;
    public final static long mask6 = 1 << 6;
    public final static long mask7 = 1 << 7;
    public final static long mask8 = 1 << 8;
    
    public static final String ARROW = "1";
    public static final String VARIABLE = "10";
    public static final String VALVE = "11";
    public static final String COMMENT = "12";
    public static final String BITMAP = "30";
    public static final String METAFILE = "31";
    public static final String RATE = "99";
    
    public static final String CLOUD = "48";
    
    public static final String CLOUD_NAME = "C_L_O_U_D_";
    public static final String RATE_NAME = "R_A_T_E_";
    
    private static int CREATE_ID = 100000;
    private static final String VARIABLE_CREATE = "10,$$$ID$$$,$$$NAME$$$,688,450,95,27,40,3,0,0,-1,0,0,0";
    
    private static final String ID_MARKER = "$$$ID$$$";
    private static final String NAME_MARKER = "$$$NAME$$$";
    
    public static final String POSITIVE_POLARITY = "43";
    public static final String NEGATIVE_POLARITY = "45";
    
    private boolean autoGenerated = false;

    private View view;

    private String rawObject;
    
    private boolean recognized = true;

    private String type;
    private String id;
    private String uid;

    // Arrows
    private String from;
    private String to;
    private String shape; // Variables
    private String hidden; // Variables
    private String polarity;
    private String thickness;
    private String hasFont; // Variables
    private String delayType;
    private String reserved;
    private String color;
    private String font; // Variables
    private String numPoints;
    private String pointList;

    // Variables
    private String name;
    private String x;
    private String y;
    private String width;
    private String height;
    //    private String shape;
    private String bits;
    //    private String hidden;
    //    private String hasFont;
    private String textPos;
    private String boxWidth;
    private String nav1;
    private String nav2;
    private String boxColor;
    private String fillColor;
    //    private String font;
    
    private boolean swapped = false;
    
    private String additionalText;
    private SystemDynamicsObjectManager sdObjectManager;
    
    private GraphicObject associatedVariable;

    public GraphicObject(SystemDynamicsObjectManager sdObjectManager, View view) {
    	this.view = view;
    	this.sdObjectManager = sdObjectManager;
        }

    public GraphicObject(SystemDynamicsObjectManager sdObjectManager, View view, String rawObject) {
	this.rawObject = new String(rawObject);
	this.view = view;
	this.sdObjectManager = sdObjectManager;
	if (rawObject.contains("ExternalSplit"))
		System.out.println("ExternalSplit");
	parse();
    }

    public void parse() {
    	String[] fields = Parser.splitQuoted(rawObject, ",");
    	type = fields[0];
    	
        	
    	id = fields[1];
    	if (type.equals("1"))
    		parseAsArrow(fields);
    	else if (type.equals("10"))
    		parseAsVariable(fields);
    	else if (type.equals("11"))
    		parseAsValve(fields);
    	else if (type.equals("12"))
    		parseAsComment(fields);
    	else if (type.equals("30"))
    		parseAsBitmap(fields);
    	else if (type.equals("31"))
    		parseAsMetafile(fields);
    	else
    		recognized = false;
    }
    
    public void writeLocation(BufferedWriter bw) {
    	if (!type.equals(VARIABLE) && !type.equals(RATE))
    		return;
    	try {
			bw.append(name+"\t"+view.getName()+"\t"+x+"\t"+y+"\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
    public boolean isValve() {
    	return type.equals("11");
    }
    
    public boolean isVariable() {
    	return type.equals("10");
    }
    
    public boolean isRate() {
    	return type.equals("99");
    }
    
    public boolean isArrow() {
    	return type.equals("1");
    }
    
    public boolean isComment() {
    	return type.equals("12");
    }
    
    public boolean isBitmap() {
    	return type.equals("30");
    }
    
    public boolean isMetafile() {
    	return type.equals("31");
    }
    
    public boolean isCloud() {
    	return type.equals("48");
    }

    private void parseAsArrow(String[] fields) {

    	// Arrows
    	from = fields[2];
    	to = fields[3];
    	shape = fields[4];
    	hidden = fields[5];
    	polarity = fields[6];
    	thickness = fields[7];
    	hasFont = fields[8];
    	delayType = fields[9];
    	reserved = fields[10];
    	color = fields[11];
    	font = fields[12];
    	// Need to handle the various formats that are possible


    	numPoints = reconstructNumPoints(fields, 13);
    	pointList = reconstructPointlist(fields, 13);;
    }
    
    private String reconstructNumPoints(String [] fields, int index) {
	String[] result = fields[index].split("\\|");
	return result[0];
    }
    
    private String reconstructPointlist(String [] fields, int index) {
	StringBuilder sb = new StringBuilder();
	
	// first reconstruct original string
	for (int i = index; i < fields.length; i++) {
	    if (i != index)
		sb.append(",");
	    sb.append(fields[i]);
	}
	// 3|(360,198)|(446,198)|(446,143)|
	String[] f = sb.toString().split("\\|");
	sb.setLength(0);
	for (int i = 1; i < f.length; i++) {
		sb.append("|");
		sb.append(f[i]);
	}
	sb.append("|");
	return sb.toString();
    }

    private void parseAsVariable(String[] fields) {
	name = fields[2]; // mjb 8/4/21 .replace("\"", "");
	x = fields[3];
	y = fields[4];
	width = fields[5];
	height = fields[6];
	shape = fields[7];
	bits = fields[8];
	if (bitwiseAnd(bits, mask2))
	    additionalText = new String(view.getNextRawObject());
	hidden = fields[9];
	hasFont = fields[10];
	textPos = fields[11];
	boxWidth = fields[12];
	nav1 = fields[13];
	nav2 = fields[14];
	if (nav1.equals("253") && nav2.equals("253"))
	    additionalText = new String(view.getNextRawObject());
	if (!hasFont.equals("0")) {
	    boxColor = fields[15];
	    fillColor = fields[16];
	    font = fields[17];
	}
    }

    private void parseAsValve(String[] fields) {
	name =  Parser.removeQuotes(fields[2]); // .replace("\"", "");
	x = fields[3];
	y = fields[4];
	width = fields[5];
	height = fields[6];
	shape = fields[7];
	bits = fields[8];
	hidden = fields[9];
	hasFont = fields[10];
	textPos = fields[11];
	boxWidth = fields[12];
	nav1 = fields[13];
	nav2 = fields[14];
	if (!hasFont.equals("0")) {
	    boxColor = fields[15];
	    fillColor = fields[16];
	    font = fields[17];
	}
	
	// is it always connect to a variable?
	additionalText = new String(view.peekNextRawObject());
	
	if (additionalText.startsWith(VALVE)) {
		String newID = Integer.toString(CREATE_ID++);;
		String newName = RATE_NAME+newID;
		String newRaw = VARIABLE_CREATE.replace(ID_MARKER, newID).replace(NAME_MARKER, newName);
		GraphicObject go = new GraphicObject(sdObjectManager, view, newRaw);
		go.setAutoGenerated(true);
		associatedVariable = go;
		go.setType(GraphicObject.RATE);
		go.setAssociatedVariable(this);
	} else {

		GraphicObject go = new GraphicObject(sdObjectManager, view, additionalText);
		if (go.getType().equals(VARIABLE)) {
			additionalText = new String(view.getNextRawObject());
			go = new GraphicObject(sdObjectManager, view, additionalText);
			associatedVariable = go;
			go.setType(GraphicObject.RATE);
			go.setAssociatedVariable(this);
		} else {
			// need to create a new variable which will be our rate variable
			// swap id's with the valve since the arrows point to the valve rather than rate as usual
			String newID = Integer.toString(CREATE_ID++);;
			String newName = RATE_NAME+newID;
			String newRaw = VARIABLE_CREATE.replace(ID_MARKER, newID).replace(NAME_MARKER, newName);
			go = new GraphicObject(sdObjectManager, view, newRaw);
			go.setAutoGenerated(true);
			associatedVariable = go;
			go.setType(GraphicObject.RATE);
			go.setAssociatedVariable(this);
		}
	}

	
    }

    private void parseAsComment(String[] fields) {
	name =  Parser.removeQuotes(fields[2]); // .replace("\"", "");
	if (name.equals(CLOUD)) {
		name = CLOUD_NAME+id;
		type = CLOUD;
	}
	x = fields[3];
	y = fields[4];
	width = fields[5];
	height = fields[6];
	shape = fields[7];
	bits = fields[8];
	if (bitwiseAnd(bits, mask2))
	    additionalText = new String(view.getNextRawObject());	
	hidden = fields[9];
	hasFont = fields[10];
	textPos = fields[11];
	boxWidth = fields[12];
	nav1 = fields[13];
	nav2 = fields[14];
	if (!hasFont.equals("0") && fields.length == 18) {
	    boxColor = fields[15];
	    fillColor = fields[16];
	    font = fields[17];
	}
    }

    private void parseAsBitmap(String[] fields) {
	name =  Parser.removeQuotes(fields[2]); // .replace("\"", "");
	x = fields[3];
	y = fields[4];
	width = fields[5];
	height = fields[6];
	shape = fields[7];
	bits = fields[8];
	hidden = fields[9];
	hasFont = fields[10];
	textPos = fields[11];
	boxWidth = fields[12];
	nav1 = fields[13];
	nav2 = fields[14];
	if (!hasFont.equals("0")) {
	    boxColor = fields[15];
	    fillColor = fields[16];
	    font = fields[17];
	}
	// additionalText = new String(view.getNextRawObject());
    }

    private void parseAsMetafile(String[] fields) {
	name =  Parser.removeQuotes(fields[2]); // .replace("\"", "");
	x = fields[3];
	y = fields[4];
	width = fields[5];
	height = fields[6];
	shape = fields[7];
	bits = fields[8];
	hidden = fields[9];
	hasFont = fields[10];
	textPos = fields[11];
	boxWidth = fields[12];
	nav1 = fields[13];
	nav2 = fields[14];
	if (!hasFont.equals("0")) {
	    boxColor = fields[15];
	    fillColor = fields[16];
	    font = fields[17];
	}
	additionalText = new String(view.getNextRawObject());
    }
    
    public void print() {
	
    	System.out.println(">>>>");
    	System.out.println("    ID: "+id);
	
	if (type.equals("1")) {
	    System.out.println("    Type: "+"Arrow"+
	    "    Name: "+name+
	    "    From: "+from+
	    "    To: "+to+
	    "    NumPoints: "+numPoints+
	    "    PointList: "+pointList+
	    "    Shape: "+shape+" Swapped: "+swapped);
	} else if (type.equals("10")) {
		System.out.println("    Type: "+"Variable");
		System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Shape: "+shape);
	} else if (type.equals("11")) {
	    System.out.println("    Type: "+"Valve");
	    System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Shape: "+shape);
	    System.out.println("    Assoc: "+associatedVariable.getId());
	} else if (type.equals("12")) {
	    System.out.println("    Type: "+"Comment");
	    System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Comment: "+additionalText);
	    System.out.println("    Shape: "+shape);
	}   else if (type.equals("48")) {
		System.out.println("    Type: "+"Cloud");
		System.out.println("    Name: "+name);
		System.out.println("    X: "+x);
		System.out.println("    Y: "+y);
		System.out.println("    Width: "+width);
		System.out.println("    Height: "+height);
		System.out.println("    Comment: "+additionalText);
		System.out.println("    Shape: "+shape);
	} else if (type.equals("30")) {
	    System.out.println("    Type: "+"BitMap");
	    System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Shape: "+shape);
	} else if (type.equals("31")) {
	    System.out.println("    Type: "+"Metafile");
	    System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Shape: "+shape);
	} else if (type.equals("99")) {
	    System.out.println("    Type: "+"Rate");
	    System.out.println("    Name: "+name);
	    System.out.println("    X: "+x);
	    System.out.println("    Y: "+y);
	    System.out.println("    Width: "+width);
	    System.out.println("    Height: "+height);
	    System.out.println("    Shape: "+shape);
	}
    }
    
    public boolean isInfluenceArrow() {
    	return type.equals(ARROW) && (shape.equals("0") ||shape.equals("1"));
    }
    
    public boolean isFlowArrow() {
    	return type.equals(ARROW) && (shape.equals("4") ||shape.equals("100") ||shape.equals("68") || shape.equals("36"));
    }
    
    public boolean isInFlowArrow() {
    	return isFlowArrow() && (shape.equals("100") || shape.equals("36"));
    }
    
    public boolean isOutFlowArrow() {
    	return isFlowArrow() && (shape.equals("4")  ||shape.equals("68"));
    }
    
    public boolean isInputOutput() {
	if (nav1 == null || nav2 == null)
	    return false;
	return (nav1.equals("253") && nav2.equals("253"));
    }
    
    private boolean bitwiseAnd(String value, long mask) {
	long val = Long.parseLong(value);
	long result = val & mask;
	return result > 0;
    }

    public View getView() {
        return view;
    }

    public String getRawObject() {
        return rawObject;
    }

    public String getType() {
        return type;
    }
    
    public String getTypeAsString() {
        return view.translateCodeToString(type);
    }

    public String getId() {
        return id;
    }

    public String getFrom() {
        return from;
    }

    public String getTo() {
        return to;
    }

    public String getShape() {
        return shape;
    }

    public String getHidden() {
        return hidden;
    }

    public String getPolarity() {
    	if (polarity.equals(NEGATIVE_POLARITY))
    		return ArrowPolarity.NEGATIVE;
    	else if (polarity.equals(POSITIVE_POLARITY))
    		return ArrowPolarity.POSITIVE;
    	else
    		return ArrowPolarity.UNKNOWN;
    }
    
    public void reversePolarity() {
    	if (polarity.equals(NEGATIVE_POLARITY))
    		polarity = POSITIVE_POLARITY;
    	else if (polarity.equals(POSITIVE_POLARITY))
    		polarity = NEGATIVE_POLARITY;
    }

    public String getThickness() {
        return thickness;
    }

    public String getHasFont() {
        return hasFont;
    }

    public String getDelayType() {
        return delayType;
    }

    public String getReserved() {
        return reserved;
    }

    public String getColor() {
        return color;
    }

    public String getFont() {
        return font;
    }

    public String getNumPoints() {
        return numPoints;
    }

    public String getPointList() {
        return pointList;
    }

    public String getName() {
        return name;
    }

    public String getX() {
        return x;
    }

    public String getY() {
        return y;
    }

    public String getWidth() {
        return width;
    }

    public String getHeight() {
        return height;
    }

    public String getBits() {
        return bits;
    }

    public String getTextPos() {
        return textPos;
    }

    public String getBoxWidth() {
        return boxWidth;
    }

    public String getNav1() {
        return nav1;
    }

    public String getNav2() {
        return nav2;
    }

    public String getBoxColor() {
        return boxColor;
    }

    public String getFillColor() {
        return fillColor;
    }

    public String getAdditionalText() {
        return additionalText;
    }

    public GraphicObject getAssociatedVariable() {
        return associatedVariable;
    }

	public void setType(String type) {
		this.type = type;
	}

	public void setAssociatedVariable(GraphicObject associatedVariable) {
		this.associatedVariable = associatedVariable;
	}

	public void setTo(String to) {
		this.to = to;
	}
	
	public String toString() {
		return "id="+id+" type="+type+" name="+name;
	}

	public boolean isAutoGenerated() {
		return autoGenerated;
	}

	public void setAutoGenerated(boolean autoGenerated) {
		this.autoGenerated = autoGenerated;
	}

	public boolean isSwapped() {
		return swapped;
	}

	public void setSwapped(boolean swapped) {
		this.swapped = swapped;
	}

	public void setPolarity(String polarity) {
		this.polarity = polarity;
	}
	
	public boolean isPositivePolarity() {
		return polarity.equals(POSITIVE_POLARITY);
	}
	
	public boolean isNegativePolarity() {
		return polarity.equals(NEGATIVE_POLARITY);
	}
	
	public boolean isUnknownPolarity() {
		return !isNegativePolarity() && !isPositivePolarity();
	}
	
	public void setPositivePolarity() {
		polarity = POSITIVE_POLARITY;
	}
	
	
	public void setNegativePolarity() {
		polarity = NEGATIVE_POLARITY;
	}

	/**
	 * @return the sdObjectManager
	 */
	public SystemDynamicsObjectManager getSdObjectManager() {
		return sdObjectManager;
	}

	/**
	 * @param sdObjectManager the sdObjectManager to set
	 */
	public void setSdObjectManager(SystemDynamicsObjectManager sdObjectManager) {
		this.sdObjectManager = sdObjectManager;
	}

	/**
	 * @param view the view to set
	 */
	public void setView(View view) {
		this.view = view;
	}

	/**
	 * @param rawObject the rawObject to set
	 */
	public void setRawObject(String rawObject) {
		this.rawObject = rawObject;
	}

	/**
	 * @param id the id to set
	 */
	public void setId(String id) {
		this.id = id;
	}

	/**
	 * @param from the from to set
	 */
	public void setFrom(String from) {
		this.from = from;
	}

	/**
	 * @param shape the shape to set
	 */
	public void setShape(String shape) {
		this.shape = shape;
	}

	/**
	 * @param hidden the hidden to set
	 */
	public void setHidden(String hidden) {
		this.hidden = hidden;
	}

	/**
	 * @param thickness the thickness to set
	 */
	public void setThickness(String thickness) {
		this.thickness = thickness;
	}

	/**
	 * @param hasFont the hasFont to set
	 */
	public void setHasFont(String hasFont) {
		this.hasFont = hasFont;
	}

	/**
	 * @param delayType the delayType to set
	 */
	public void setDelayType(String delayType) {
		this.delayType = delayType;
	}

	/**
	 * @param reserved the reserved to set
	 */
	public void setReserved(String reserved) {
		this.reserved = reserved;
	}

	/**
	 * @param color the color to set
	 */
	public void setColor(String color) {
		this.color = color;
	}

	/**
	 * @param font the font to set
	 */
	public void setFont(String font) {
		this.font = font;
	}

	/**
	 * @param numPoints the numPoints to set
	 */
	public void setNumPoints(String numPoints) {
		this.numPoints = numPoints;
	}

	/**
	 * @param pointList the pointList to set
	 */
	public void setPointList(String pointList) {
		this.pointList = pointList;
	}

	/**
	 * @param name the name to set
	 */
	public void setName(String name) {
		this.name = name;
	}

	/**
	 * @param x the x to set
	 */
	public void setX(String x) {
		this.x = x;
	}

	/**
	 * @param y the y to set
	 */
	public void setY(String y) {
		this.y = y;
	}

	/**
	 * @param width the width to set
	 */
	public void setWidth(String width) {
		this.width = width;
	}

	/**
	 * @param height the height to set
	 */
	public void setHeight(String height) {
		this.height = height;
	}

	/**
	 * @param bits the bits to set
	 */
	public void setBits(String bits) {
		this.bits = bits;
	}

	/**
	 * @param textPos the textPos to set
	 */
	public void setTextPos(String textPos) {
		this.textPos = textPos;
	}

	/**
	 * @param boxWidth the boxWidth to set
	 */
	public void setBoxWidth(String boxWidth) {
		this.boxWidth = boxWidth;
	}

	/**
	 * @param nav1 the nav1 to set
	 */
	public void setNav1(String nav1) {
		this.nav1 = nav1;
	}

	/**
	 * @param nav2 the nav2 to set
	 */
	public void setNav2(String nav2) {
		this.nav2 = nav2;
	}

	/**
	 * @param boxColor the boxColor to set
	 */
	public void setBoxColor(String boxColor) {
		this.boxColor = boxColor;
	}

	/**
	 * @param fillColor the fillColor to set
	 */
	public void setFillColor(String fillColor) {
		this.fillColor = fillColor;
	}

	/**
	 * @param additionalText the additionalText to set
	 */
	public void setAdditionalText(String additionalText) {
		this.additionalText = additionalText;
	}

	/**
	 * @return the uid
	 */
	public String getUid() {
		return uid;
	}

	/**
	 * @param uid the uid to set
	 */
	public void setUid(String uid) {
		this.uid = uid;
	}
	
	public void writeToCsvFile(BufferedWriter bw) {
			try {
				bw.append(type); bw.append("\t");
				bw.append(id); bw.append("\t");
				bw.append(uid); bw.append("\t");

				// Arrows
				bw.append(from); bw.append("\t");
				bw.append(to); bw.append("\t");
				bw.append(shape); bw.append("\t"); // Variables
				bw.append(hidden); bw.append("\t"); // Variables
				bw.append(polarity); bw.append("\t");
				bw.append(thickness); bw.append("\t");
				bw.append(hasFont); bw.append("\t"); // Variables
				bw.append(delayType); bw.append("\t");
				bw.append(reserved); bw.append("\t");
				bw.append(color); bw.append("\t");
				bw.append(font); bw.append("\t"); // Variables
				bw.append(numPoints); bw.append("\t");
				bw.append(pointList); bw.append("\t");

				// Variables
				bw.append(name); bw.append("\t");
				bw.append(x); bw.append("\t");
				bw.append(y); bw.append("\t");
				bw.append(width); bw.append("\t");
				bw.append(height); bw.append("\t");
   
				bw.append(bits); bw.append("\t");
 
				bw.append(textPos); bw.append("\t");
				bw.append(boxWidth); bw.append("\t");
				bw.append(nav1); bw.append("\t");
				bw.append(nav2); bw.append("\t");
				bw.append(boxColor); bw.append("\t");
				bw.append(fillColor); bw.append("\t");
				
				bw.append(""+swapped); bw.append("\t");
				
				bw.append(additionalText); bw.append("\t");
				
				bw.append(""+associatedVariable); bw.append("\n");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	}

	public boolean isRecognized() {
		return recognized;
	}
}
