package gov.anl.gss.sdm.server.model.translator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class GraphicsProcessor {
static Logger log = Logger.getLogger(GraphicsProcessor.class.getSimpleName());
	
	private final static String EQUIVALENCE_START = "22:";
    
    private SystemDynamicsObjectManager sdObjectManager;
    private Map<String, View> viewMap = new LinkedHashMap<String, View>();
    
    private boolean variableNameTranslationMode = false;
    
    private List<String> extraStuff = new ArrayList<String>();
    
    private int generatedViewNameID;
    
    Map<String, Set<String>> headTailMap = new LinkedHashMap<String, Set<String>>();
	Map<String, Set<String>> tailHeadMap = new LinkedHashMap<String, Set<String>>();

    public GraphicsProcessor() {
	InformationManagers.getInstance().setGraphicsProcessor(this);
	generatedViewNameID = 1;
    }
    
	public List<String[]> getEdges() {
		
		// private Map<String, Set<String>> usedBy; // key = tail value = head
		
		List<String[]> edges = new ArrayList<String[]>();
		for (String tail : tailHeadMap.keySet()) {
			for (String head : tailHeadMap.get(tail)) {
				String[] anEdge = new String[2];
				anEdge[0] = new String(tail);
				anEdge[1] = new String(head);
				edges.add(anEdge);
			}
		}
		
		return edges;
	}
    
    public Map<String, View> processGraphics(SystemDynamicsObjectManager sdObjectManager, List<String> mdlContents) {

	
	
	this.sdObjectManager = sdObjectManager;

	// first skip to the beginning of the 
	int graphRecord = 0;

	for (String aLine : mdlContents) {
	    if (aLine.contains("---///"))
		break;
	    graphRecord++;
	}
	
	if (graphRecord >= mdlContents.size() /* || InformationManagers.getInstance().getModelManager().isXmileOrigin() */)
		return new HashMap<String, View>();

	int viewNumber = 0;
	
	String aLine = "";
	String graphicsMarker = mdlContents.get(graphRecord++);	

	while(true) {
		List<String> forViewImage = new ArrayList<String>();
		// if there are no graphics, we are pointing to EOF marker checkfor it
		if (graphRecord >= mdlContents.size()) {
			if (!aLine.endsWith("---\\\\\\")) {
				System.out.println("Error in locating beginning/end of graphic data");
			}
			break;
		}
		String next = mdlContents.get(graphRecord);
	    if (aLine.endsWith("---\\\\\\") || next.endsWith("---\\\\\\")) {
	    	break;
	    }
	    
	    forViewImage.add("Beginning of View Definition");
	    forViewImage.add(mdlContents.get(graphRecord));
	    String versionCode = mdlContents.get(graphRecord++);
	    String viewName = mdlContents.get(graphRecord).substring(1);
	    if (viewName.length() == 0) {
	    	viewName = "GenName"+generatedViewNameID++;
	    	forViewImage.add(mdlContents.get(graphRecord++)+viewName);
	    } else {
	    	forViewImage.add(mdlContents.get(graphRecord++));
	    }
	   
	    forViewImage.add(mdlContents.get(graphRecord));
	    String viewDefaultFont = mdlContents.get(graphRecord++);
	    
	    View aView = new View(viewName, versionCode, viewDefaultFont);
	    viewMap.put(viewName, aView);
	    viewNumber++;
	    aLine = mdlContents.get(graphRecord++);
	    List<String> objects = new ArrayList<String>();
	    while(!aLine.startsWith("\\\\\\---///") && !aLine.startsWith("///---\\\\\\")) {
	    	
	    	objects.add(aLine);
	    	forViewImage.add(aLine);
	    	if (graphRecord >= mdlContents.size()) {
	    		log.debug("Graphics processor bad data read");
	    	}
	    	aLine = mdlContents.get(graphRecord++);

	    }
	    
	    aView.setRawObjects(objects);
	    aView.parse(sdObjectManager);
	    
	    if (!isVariableNameTranslationMode()) {
	    		aView.generateImage(forViewImage);
	    }
	    
	}

    // what's left
	
	// InformationManagers.getInstance().getUnitsManager().addEquivalence(equiv[0], equiv[1]);
	
	UnitsManager unitsManager = InformationManagers.getInstance().getUnitsManager();
	
	// Dimensionless, Dmnl, Fraction, Nil, 1
//	mdlContents.add("Dmnl, Dimensionless,  Fraction, Nil, 1");
    
    while (graphRecord < mdlContents.size()) {
    	aLine = mdlContents.get(graphRecord++);
    	extraStuff.add(aLine);
    	if (aLine.startsWith(EQUIVALENCE_START)) {
    		String[] equiv = aLine.replace(EQUIVALENCE_START, "").split(",");
    		String effectiveUnit = equiv[0].trim();
    		unitsManager.addEquivalence(effectiveUnit, effectiveUnit);
    		for (int i = 1; i < equiv.length; i++) {
    			unitsManager.addEquivalence(equiv[i].trim(), effectiveUnit);
    		}
    	}
    	
    }
    

    
    unitsManager.finalize();
	return viewMap;
    }
    
    public void buildHeadsAndTails(Map<String, Set<String>> headTailMap,
    		Map<String, Set<String>> tailHeadMap) {
    	
    	this.headTailMap = headTailMap;
    	this.tailHeadMap = tailHeadMap;
    	
    	// construct arrow maps
    	for (View view : viewMap.values()) {
    		view.buildHeadsAndTails(headTailMap, tailHeadMap);
    	}
    }
    
	  public Map<String, GraphicObject> getFlows(View view) {
		  Map<String, GraphicObject> flows = new LinkedHashMap<String, GraphicObject>();
		  
		  for (GraphicObject go : view.getGraphicObjects()) {
			  if (go.isValve()) {
				  go.setName(go.getAssociatedVariable().getName());
				  flows.put(go.getId(), go);
			  }
		  }
		  return flows;
	  }
	  
	  public Map<String, GraphicObject> getRates(View view) {
		  Map<String, GraphicObject> rates = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isRate())
			  rates.put(go.getId(), go);
		  }
		  return rates;
	  }
	  
	  public Map<String, GraphicObject> getClouds(View view) {
		  Map<String, GraphicObject> clouds = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isCloud())
			  clouds.put(go.getId(), go);
		  }
		  return clouds;
	  }
	  
	  public Map<String, GraphicObject> getLinks(View view) {
		  Map<String, GraphicObject> links = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isArrow())
			  links.put(go.getId(), go);
	  }
		  return links;
	  }
	  
	  public Map<String, GraphicObject> getVariables(View view) {
		  Map<String, GraphicObject> variables = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isVariable())
			  variables.put(go.getId(), go);
		  }
		  return variables;
	  }
	  public Map<String, GraphicObject> getComments(View view) {
		  Map<String, GraphicObject> comments = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isComment())
			  comments.put(go.getId(), go);
		  }
		  return comments;
	  }
	  public Map<String, GraphicObject> getBitmaps(View view) {
		  Map<String, GraphicObject> bitmaps = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isBitmap())
			  bitmaps.put(go.getId(), go);
		  }
		  return bitmaps;
	  }
	  public Map<String, GraphicObject> getMetaFiles(View view) {
		  Map<String, GraphicObject> metafiles = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
		  if (go.isMetafile())
			  metafiles.put(go.getId(), go);
		  }
		  return metafiles;
	  }
	  
	  public Map<String, GraphicObject> getAllGraphicObjects(View view) {
		  Map<String, GraphicObject> metafiles = new LinkedHashMap<String, GraphicObject>();
		  for (GraphicObject go : view.getGraphicObjects()) {
			  metafiles.put(go.getId(), go);
		  }
		  return metafiles;
	  }
	  
	  public GraphicObject getAssociatedLink(GraphicObject aCloud, Map<String, GraphicObject> links) {
		  String id = aCloud.getId();
		  for (GraphicObject link : links.values()) {
			  if (link.getTo().equals(id)) {
				  return link;
			  }
		  }
		  return null;
		  
	  }
	  
	  public GraphicObject getAssociatedFlow(GraphicObject link, Map<String, GraphicObject> flows) {
		  String id = link.getFrom();
		  for (GraphicObject flow : flows.values()) {
			  if (flow.getId().equals(id)) {
				  return flow;
			  }
		  }
		  return null;
	  }

	public boolean isVariableNameTranslationMode() {
		return variableNameTranslationMode;
	}

	public void setVariableNameTranslationMode(boolean variableNameTranslationMode) {
		this.variableNameTranslationMode = variableNameTranslationMode;
	}

	public List<String> getExtraStuff() {
		return extraStuff;
	}

	public void setExtraStuff(List<String> extraStuff) {
		this.extraStuff = extraStuff;
	}

	public Map<String, View> getViewMap() {
		return viewMap;
	}
}
