package gov.anl.gss.sdm.server.model.translator;

//import repast.simphony.systemdynamics.sdmodel.SystemModel;
import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.analysis.EquationManager;
import gov.anl.gss.sdm.server.model.analysis.loops.LoopManager;
import gov.anl.gss.sdm.server.model.analysis.polarity.DataBridge;
import gov.anl.gss.sdm.server.model.documentor.MessageProcessor;
import gov.anl.gss.sdm.server.model.documentor.VariableInformationFactory;
import gov.anl.gss.sdm.server.model.documentor.gui.MessageFrame;
import gov.anl.gss.sdm.server.model.documentor.gui.Reuse;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.ExcelManager;
import gov.anl.gss.sdm.server.model.support.MappedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.MemoryMapperInterface;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.SubscriptManager;
import gov.anl.gss.sdm.server.model.support.VariableNamePatternGenerator;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;

public class InformationManagers {

  private static InformationManagers instance = null;

  private ArrayManager arrayManager; // >
  private UnitsManager unitsManager; // >
  private FunctionManager functionManager; // >
  private SystemDynamicsObjectManager systemDynamicsObjectManager; // not static
  private MappedSubscriptManager mappedSubscriptManager; // >
  private NamedSubscriptManager namedSubscriptManager; // >
  private NativeDataTypeManager nativeDataTypeManager; // >
  private SubscriptManager subscriptManager; // empty
  private MacroManager macroManager;
  private MessageManager messageManager;
  private ModelManager modelManager;
  private DatabaseManager databaseManager;
  private GraphicsProcessor graphicsProcessor;
  private VariableManager variableManager;
  private LoopManager loopManager;
  
  public VariableManager getVariableManager() {
	return variableManager;
}

public void setVariableManager(VariableManager variableManager) {
	this.variableManager = variableManager;
}

private ExcelManager excelManager;
  private EquationManager equationManager;
  private EquationLookup equationLookup;
  
  private VariableInformationFactory variableInformationFactory;
  
  private ModelAnalyzer modelAnalyzer;
  
  public ModelAnalyzer getModelAnalyzer() {
	return modelAnalyzer;
}

public void setModelAnalyzer(ModelAnalyzer modelAnalyzer) {
	this.modelAnalyzer = modelAnalyzer;
}

private VariableNamePatternGenerator variableNamePatternGenerator;
  
  private DataBridge dataBridge;
  
  private FileOrganizer fileOrganizer;
  
  private Reuse reuse;
  
  private boolean limitLoopProcessing = false;

  private MessageProcessor messageFrame;
  
  private MemoryMapperInterface memoryMapper;
  

  public MemoryMapperInterface getMemoryMapper() {
	  return memoryMapper;
  }

  public void setMemoryMapper(MemoryMapperInterface memoryMapper) {
	  this.memoryMapper = memoryMapper;
  }

private AnalysisManager analysisManager;
  
  private ErrorManager errorManager;
  
  private SDObjectManager sdObjectManager;
  
  private UserSelectedOptions userSelectedOptions;
  
  private boolean sdmuiMode = false;
  private String sdmuiRootDirectory;
  private String objectName;
  
//  private SystemModel systemModel;

  public static InformationManagers getInstance() {
	 
    if (instance == null) {
      instance = new InformationManagers();
    }
   
    return instance;
  }
  
  public static void clear() {
	   instance = null;
	  }
 

  public InformationManagers() {
    arrayManager = new ArrayManager();
    unitsManager = new UnitsManager();
    functionManager = new FunctionManager();
    systemDynamicsObjectManager = new SystemDynamicsObjectManager();
    mappedSubscriptManager = new MappedSubscriptManager();
    namedSubscriptManager = new NamedSubscriptManager();
    nativeDataTypeManager = new NativeDataTypeManager();
    subscriptManager = new SubscriptManager();
    macroManager = new MacroManager();
    messageManager = new MessageManager();
    modelManager = new ModelManager();
    sdObjectManager = new SDObjectManager();
    databaseManager  = new DatabaseManager();
    errorManager = new ErrorManager();
    variableManager = new VariableManager();
    dataBridge = null;
    reuse = new Reuse();
    variableNamePatternGenerator = new VariableNamePatternGenerator();
    excelManager = new ExcelManager();
    equationManager = new EquationManager();
    equationLookup = new EquationLookup();
    variableInformationFactory = new VariableInformationFactory();
    
  }
  
  public String getMemoryMapperFileName() {
	  return getFileOrganizer().getMapperFile();
  }
  
  public String getStructureDatabaseName() {
	  return getDatabaseManager().getDb().getDbFile();
  }
  
  public String getRunDatabaseName() {
	  return getFileOrganizer().getRunsDirectory()+"/baselineHSQLDB";
  }
  
  public DataBridge setDataBridge(String dbFile) {
	  dataBridge = new DataBridge(dbFile);
	  return dataBridge;
  }
  
  public String getObjectName() {
	  return objectName;
  }
  
  public ModelManager getModelManager() {
	    return modelManager;
	  }

  public ArrayManager getArrayManager() {
    return arrayManager;
  }

  public UnitsManager getUnitsManager() {
    return unitsManager;
  }

  public FunctionManager getFunctionManager() {
    return functionManager;
  }

  public SystemDynamicsObjectManager getSystemDynamicsObjectManager() {
    return systemDynamicsObjectManager;
  }

  public MappedSubscriptManager getMappedSubscriptManager() {
    return mappedSubscriptManager;
  }

  public NamedSubscriptManager getNamedSubscriptManager() {
    return namedSubscriptManager;
  }

  public NativeDataTypeManager getNativeDataTypeManager() {
    return nativeDataTypeManager;
  }

  public SubscriptManager getSubscriptManager() {
    return subscriptManager;
  }

  public MacroManager getMacroManager() {
    return macroManager;
  }

  public MessageManager getMessageManager() {
    return messageManager;
  }

//public SystemModel getSystemModel() {
//	return systemModel;
//}
//
//public void setSystemModel(SystemModel systemModel) {
//	this.systemModel = systemModel;
//}

public SDObjectManager getSdObjectManager() {
	return sdObjectManager;
}

public UserSelectedOptions getUserSelectedOptions() {
	return userSelectedOptions;
}

public void setUserSelectedOptions(UserSelectedOptions userSelectedOptions) {
	this.userSelectedOptions = userSelectedOptions;
	
	reuse.setClassFile(userSelectedOptions.getOptionValue(StringConstants.REUSE_CLASS).equals(UserSelectedOptions.TRUE));
	reuse.setViews(userSelectedOptions.getOptionValue(StringConstants.REUSE_VIEW_DIAGRAM).equals(UserSelectedOptions.TRUE));
	reuse.setThumbnails(userSelectedOptions.getOptionValue(StringConstants.REUSE_THUMBNAILS).equals(UserSelectedOptions.TRUE));
	reuse.setPolarity(userSelectedOptions.getOptionValue(StringConstants.REUSE_POLARITIES).equals(UserSelectedOptions.TRUE));
	reuse.setLoops(userSelectedOptions.getOptionValue(StringConstants.REUSE_LOOPS).equals(UserSelectedOptions.TRUE));
}

/**
 * @return the errorManager
 */
public ErrorManager getErrorManager() {
	return errorManager;
}

/**
 * @return the analysisManager
 */
public AnalysisManager getAnalysisManager() {
	return analysisManager;
}

/**
 * @param analysisManager the analysisManager to set
 */
public void setAnalysisManager(AnalysisManager analysisManager) {
	this.analysisManager = analysisManager;
}

/**
 * @return the sdmuiMode
 */
public boolean isSdmuiMode() {
	return sdmuiMode;
}

/**
 * @param sdmuiMode the sdmuiMode to set
 */
public void setSdmuiMode(boolean sdmuiMode) {
	System.out.println("setSdmuiMode: "+false);
	this.sdmuiMode = sdmuiMode;
	modelManager.setSdmuiMode(sdmuiMode);
}

public void setSdmuiRootDirectory(String rootDirectory) {
	this.sdmuiRootDirectory = rootDirectory;
	modelManager.setSdmuiRootDirectory(rootDirectory);
}

/**
 * @param objectName the objectName to set
 */
public void setObjectName(String objectName) {
	this.objectName = objectName;
}

/**
 * @return the sdmuiRootDirectory
 */
public String getSdmuiRootDirectory() {
	return sdmuiRootDirectory;
}

public FileOrganizer getFileOrganizer() {
	return fileOrganizer;
}

public void setFileOrganizer(FileOrganizer fileOrganizer) {
	this.fileOrganizer = fileOrganizer;
}

public DatabaseManager getDatabaseManager() {
	return databaseManager;
}

public void setDatabaseManager(DatabaseManager databaseManager) {
	this.databaseManager = databaseManager;
}

public boolean isLimitLoopProcessing() {
	return limitLoopProcessing;
}

public void setLimitLoopProcessing(boolean limitLoopProcessing) {
	this.limitLoopProcessing = limitLoopProcessing;
}

public GraphicsProcessor getGraphicsProcessor() {
	return graphicsProcessor;
}

public void setGraphicsProcessor(GraphicsProcessor graphicsProcessor) {
	this.graphicsProcessor = graphicsProcessor;
}

//public VariableManager getVariableManager() {
//	return variableManager;
//}

public void setSdObjectManager(SDObjectManager sdObjectManager) {
	this.sdObjectManager = sdObjectManager;
}

public Reuse getReuse() {
	return reuse;
}

public static void setInstance(InformationManagers instance) {
	InformationManagers.instance = instance;
}

public DataBridge getDataBridge() {
	return dataBridge;
}

public VariableNamePatternGenerator getVariableNamePatternGenerator() {
	return variableNamePatternGenerator;
}

public void setVariableNamePatternGenerator(
		VariableNamePatternGenerator variableNamePatternGenerator) {
	this.variableNamePatternGenerator = variableNamePatternGenerator;
}

public void setReuse(Reuse reuse) {
	this.reuse = reuse;
}

public MessageProcessor getMessageFrame() {
	if (messageFrame == null) {
		messageFrame = new MessageFrame("InformationManager",20, 20, 700, 50);
	}
	return messageFrame;
}

public void setMessageFrame(MessageProcessor messageFrame) {
	this.messageFrame = messageFrame;
}

public ExcelManager getExcelManager() {
	return excelManager;
}

public EquationManager getEquationManager() {
	return equationManager;
}

public EquationLookup getEquationLookup() {
	return equationLookup;
}

public VariableInformationFactory getVariableInformationFactory() {
	return variableInformationFactory;
}

public LoopManager getLoopManager() {
	return loopManager;
}

public void setLoopManager(LoopManager loopManager) {
	this.loopManager = loopManager;
}

}
