package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class ModelAnalyzer {
static Logger log = Logger.getLogger(ModelAnalyzer.class.getSimpleName());
	
	private final static String FUNCTION_OF = "A FUNCTION OF";

	private List<String> rawEquations;
	private boolean hasEquation = false;
	private boolean hasMap = false;
	private boolean hasEmpty = false;
	
	List<String> vensimEquations = new ArrayList<String>();

	public ModelAnalyzer(List<String> rawEquations) {
		this.rawEquations = rawEquations;
		analyzeContent();
		InformationManagers.getInstance().setModelAnalyzer(this);
	}

	public void analyzeContent() {
		buildVensimEquations(vensimEquations);
		for (String eqn : vensimEquations) {
			// we want to determine if this is a specified model or simply a map
			if (eqn.contains(FUNCTION_OF)) {
				String[] parts = eqn.split(FUNCTION_OF);
				if (parts[1].replace(" ", "").startsWith("()")) {
					hasEmpty = true;
				} else {
					hasMap = true;
				}
			} else {
				hasEquation = true;
			}
		}
		
		if (hasMap || hasEmpty) {
			InformationManagers.getInstance().getModelManager().
			setRunnable(false, "Map = "+hasMap+" Empty Equation = "+hasEmpty);
		}
	}
	
	public boolean fullyConnected() {
		return hasMap && !hasEmpty;
	}
	
	public boolean fullyDefined() {
		return hasEquation && !hasMap && !hasEmpty;
	}
	
	private boolean isTerseFormat() {
		
		return rawEquations.get(0).endsWith("|");
		
	}
	
	public List<String> buildVensimEquationsTerseFormat(List<String> vensimEquations) {
		for (String s : rawEquations) {
			if (s.length() > 0)
				vensimEquations.add(s);
		}
		
		return vensimEquations;
	}

	public List<String> buildVensimEquations(List<String> vensimEquations) {

		String vensimEquation;
		String aLine;
		
		if(isTerseFormat()) {
			return buildVensimEquationsTerseFormat(vensimEquations);
		}

		// this loop extends to the botton of the method

		MutableInteger linePtr = new MutableInteger(0);

		while (true && linePtr.value() < rawEquations.size()) {
			String initialLineTerminator = "";
			String insertedEqual = "";
			aLine = rawEquations.get(linePtr.valueAndInc());
			

			// skip over empty lines prior to the start of an equation
			while (aLine.length() == 0) {
				if (linePtr.value() >= rawEquations.size()) {
					return vensimEquations;
				}
				aLine = rawEquations.get(linePtr.valueAndInc());

			}
			
			
			if (aLine.startsWith("{") ) {
				System.out.println("{ break");
				while (!aLine.endsWith("}")) {
					if (linePtr.value() >= rawEquations.size()) {
						return vensimEquations;
					}
					aLine = rawEquations.get(linePtr.valueAndInc());

				}
				continue;
			}


			// if the first line of the equation is continued onto additional lines
			// we want to capture all lines and concatenate as a single line
			if (aLine.contains("TABBED")) {

				//		Sample Tabbed array equation:
				//		We will translate this into a "normal" array initialization
				//		Aggregated Definition[COP,Aggregated Regions]=1,0,0;1,0,0;...;0,0,1
				//		Aggregated Definition[COP,Aggregated Regions]=TABBED ARRAY(
				//			1	0	0
				//			1	0	0
				//			0	1	0
				//			0	1	0
				//			1	0	0
				//			0	1	0
				//			0	0	1
				//			)
				//			~	dmnl
				//			~	Tabbed array to define to which aggregated region each of the COP blocs \
				//				belongs.
				//			|

				aLine = aLine.split("=", 2)[0]+"=";
				String anotherLine;
				anotherLine = rawEquations.get(linePtr.valueAndInc()).trim();
				while (!anotherLine.endsWith(")")) {

					String[] values = anotherLine.split("\t");
					int vPtr = 0;
					for (String value : values) {
						if (vPtr++ > 0)
							aLine += ",";
						aLine += value;


					}
					aLine += ";";
					anotherLine = rawEquations.get(linePtr.valueAndInc()).trim();
				}


			} else {
				if (aLine.endsWith("\\")) {
					String concat = aLine.replaceAll("\\\\$", "");
					while (aLine.endsWith("\\")) {
						aLine = rawEquations.get(linePtr.valueAndInc());
						concat += aLine.replaceAll("\\\\$", "");
					}
					aLine = concat;

				}
			}

			// what can we learn from initial line?
			boolean isAssignment = false, isLookup = false, isVdmLookup = false, isSubscriptDefinition = false,  unsure = true, isMulti = false;
			boolean isMacroDefinition = false;

			// is it an assignment statement?
			// assignment has both left and right sides
			if (aLine.contains("=")) {
				isAssignment = true;
				unsure = false;
				initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());

				// ordering is important. Much check for macro before since macro definition
				// meets criteria of non-quoted colon
			} else if (aLine.contains(":MACRO:")) {
				unsure = false;
				isMacroDefinition = true;
				initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());
				// check if we have a subscript definition
			} else if (Parser.containsNonquotedColon(aLine)
					|| aLine.contains("<->")) { 
				// nonQuotedColon
				isSubscriptDefinition = true;
				unsure = false;
				initialLineTerminator = ":";
				initialLineTerminator = aLine.substring(aLine.length()-1, aLine.length());
				// how about a lookup?
			} else if (aLine.endsWith("(")) {
				// Note: there cannot be an = in statement, but that is caught
				// in initial comparison
				isLookup = true;
				unsure = false;
				initialLineTerminator = "(";
				// is it a multiline with first line a vdmlookup?
			} else if (aLine.endsWith("~~|")) {
				// and multi-equation also
				isVdmLookup = true;
				isMulti = true;
				unsure = false;
				initialLineTerminator = "~~|";
				// need to add vdmlookup invocation in this case
				String lhs = aLine.replace("~~|", "").trim();
				//				    if (lhs.contains("["))
				//					    lhs += "array.";
				aLine = lhs + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(lhs)+"\")"+initialLineTerminator;
			} else {
				// if not an assignment &&
				// not a subscript definition &&
				// not a lookup definition &&
				// not a multi-equation vmdlookup

				// may be a single equation vdmlookup
				// or a "special" assignment that doesn't have an =
				// need to look at the next line to determine

				unsure = true;

			}


			// this is the beginning of the graphics, return as we do not do anything with the graphics
			if (aLine.contains("---///")) {
				return vensimEquations;
			}

			// process macros independently
			if (isMacroDefinition) {
				//		System.err.println("Macros are currently not supported -- EXITING");
				//		System.exit(1);
				processMacro(aLine, rawEquations, linePtr);
			} else {

				// This is were we will need to deal with TABBED ARRAYS to read in properly.
				// Just convert into ArrayInitialization

				// start constructing the entire vensim equation
				vensimEquation = new String(aLine).replace("\\", "");

				// Now process the rest of the Variable Equation package
				// read until we hit the stand-alone "|"
				while (true && linePtr.value() < rawEquations.size()) {

					insertedEqual = "";

					// start with the "second" line of the equation package
					aLine = rawEquations.get(linePtr.valueAndInc());


					// is this a continued line? if so read it in in its entirety
					if (aLine.endsWith("\\")) {
						String concat = aLine.replaceAll("\\\\$", "");
						while (aLine.endsWith("\\")) {
							if (linePtr.value() > rawEquations.size())
								System.out.println("OVERFLOW");
							aLine = rawEquations.get(linePtr.valueAndInc());
							concat += aLine.replaceAll("\\\\$", "");
						}
						aLine = concat;

					}

					// check for validity and termination conditions

					if (aLine == null) {
						System.err.println("Unexpected EOF!!!");
					}
					if (aLine.contains("---///")) {
						System.err.println("Found graphics marker in wrong place!");
					}
					// are we finished with the package?
					if (aLine.endsWith("|") && !aLine.endsWith("~~|")) { // used to be \t| the trim() on readLine() has thrown this off
						break;
					}

					// Note that each equation in a multi-equation package could be a different type!
					// so we need to repeat the logic from above. This should really go into a separate
					// method that will deal with everything 
					//					This needs work


					// we could determine at this point that this is actually a multi-equation package
					if (aLine.endsWith("~~|")) {
						isMulti = true;
					}
					if (isMulti) {
						if (isVdmLookup) {
							String lhs = aLine.replace("~~|", "").trim();
							aLine = lhs + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(lhs)+"\")"+initialLineTerminator;
						}

					} else {
						// special processing based on what we have from first line

						if (unsure) {
							// may be a single equation vdmlookup
							// or a "special" assignment that doesn't have an =
							// need to look at the next line to determine
							if (aLine.startsWith("~")) {
								isVdmLookup = true;
								unsure = false;
								// need to modify current vensimEquation
								// if this is an array reference, need to add array.
								//						if (vensimEquation.contains("["))
								//						    vensimEquation = "array." + vensimEquation;
								vensimEquation = vensimEquation + " = "+Parser.externalDataMethod()+"(\""+Parser.removeAllQuotes(vensimEquation)+"\")";
							} else {
								// this should be a number and thus an assignment
								isAssignment = true;
								unsure = false;
								if (!vensimEquation.startsWith("***"))
									insertedEqual = " = ";
							}

						} else {

						}

					}

					if (vensimEquation.startsWith("***") && insertedEqual.length() > 0) {
						log.debug("I added = to a group");
					}

					vensimEquation = vensimEquation + insertedEqual + aLine.replace("\\", "");

				}

				// we now have the equation text
				// skip "equations" that begin with "*" -- section markers
				if (!vensimEquation.startsWith("***")) {

					// trap any equations requiring special processing prior to creating
					List<String> eqnToProcess = new ArrayList<String>();
					//				    eqnToProcess.add(vensimEquation);
					eqnToProcess = splitMultipleEquations(vensimEquation);
					for (String aVensimEquation : eqnToProcess) {
						vensimEquations.add(aVensimEquation);
					}
				} 
			}
		}	
		return vensimEquations;
	}

	private List<String> splitMultipleEquations(String vensimEquation) {
		List<String> eqn = new ArrayList<String>();
		//	log.debug("CHECKING FOR MULTIPLE EQUATION");
		if (vensimEquation.contains("~~|")) {

			//		log.debug("FOUND MULTIPLE EQUATION");

			// split multi-equation definition into multiple single equation definitions
			String[] parts;
			String[] commands;
			String units;

			parts = vensimEquation.split("~\t", 2);
			commands = parts[0].split("~~\\|");
			units = "\t~\t" + parts[1];
			
			// only subscripted varaibles can have multiple equations
			// if we find a scalar that has multiple equations, keep only the first
			// one
			
			if (!commands[0].contains("[")) {
				eqn.add(commands[0] + units);
			} else {
				for (String cmd : commands) {
					eqn.add(cmd + units);
				}
			}
			return eqn;
		} else {
			eqn.add(vensimEquation);
			return eqn;
		}
	}

	private void processMacro(String eqn, List<String> rawEquations, MutableInteger linePtr) {

		if (!InformationManagers.getInstance().getMacroManager().readMacroAndProcess(eqn, rawEquations, linePtr)) {
			log.debug("Bad Macro");;
		}
	}

	public boolean isHasEquation() {
		return hasEquation;
	}

	public boolean isHasMap() {
		return hasMap;
	}

	public boolean isHasEmpty() {
		return hasEmpty;
	}

	public List<String> getVensimEquations() {
		return vensimEquations;
	}

}
