package gov.anl.gss.sdm.server.model.translator;

import java.util.Map;

import org.apache.log4j.Logger;

public class ModelComponents {
static Logger log = Logger.getLogger(ModelComponents.class.getSimpleName());

	private Map<String, Equation> equations;
	
	private boolean complete = true;
	private boolean connected = true;
	private boolean equationsComplete = true;
	private boolean unitsComplete = true;
	
	public ModelComponents(Map<String, Equation> equations) {
		this.equations = equations;
		analyze();
		
		printAnalysis();
	}
	
	private void analyze() {
		for (Equation eqn : equations.values()) {
		
			
			EquationComponents ec = eqn.getEquationComponents();
			
			if (eqn.isDefinesSubscript())
				continue;
			
			if (!ec.isHasEquation()) {
				equationsComplete = false;
			}
			if (!ec.isHasUnits()) {
				unitsComplete = false;
			}
			if (!ec.isConnected())
				connected = false;
		}
	}
	
	private void printAnalysis() {
		log.debug("### Model Components Analysis ###\n");
		log.debug("Complete: "+complete);
		log.debug("Connected: "+connected);
		log.debug("EquationsComplete: "+equationsComplete);
		log.debug("UnitsComplete: "+unitsComplete);
		
		log.debug("\nCausal Trees: "+canAnalyzeCausalTrees());
		log.debug("Loops: "+canAnalyzeLoops());
		log.debug("Polarity: "+canAnalyzePolarity());
		log.debug("Generate Code: "+canGenerateCode());
	}
	
	public boolean canAnalyzeLoops() {
		return connected;
	}
	
	public boolean canAnalyzeCausalTrees() {
		return connected;
	}
	
	public boolean canAnalyzePolarity() {
		return isComplete();
	}
	
	public boolean canGenerateCode() {
		return isComplete();
	}
	
	public boolean isComplete() {
		complete = connected && equationsComplete /* && unitsComplete */;
		return complete;
	}

	public Map<String, Equation> getEquations() {
		return equations;
	}


	public boolean isConnected() {
		return connected;
	}

	public boolean isEquationsComplete() {
		return equationsComplete;
	}

	public boolean isUnitsComplete() {
		return unitsComplete;
	}
}
