package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.apache.commons.io.FileUtils;

public class ModelCopier {
	
	private String mdlFile = "";
	private String xlsxFile = "";
	private int numCopies;
	private List<String> masterCopy;
	private String mdlDirectory;
	private String xlsxDirectory;
	private String ts;
	
	public ModelCopier(String mdlFile, String xlsxFile, int numCopies) {
		/*
			#!/bin/sh

			filename="Model Definition"
			vensim=${1}
			finalnum=${2}
			num=1
			while [ $num -le $finalnum ]
			do
        		extended=$num
        		while [ `echo $extended | wc -c` -lt 4 ]
        			do
                		extended=0${extended}
        			done
				cat "${1}.mdl" | sed -e"s#${filename}.xlsx#${filename}_${extended}.xlsx#" > "${1}_${extended}.mdl"
				num=`expr $num + 1`
			done
		 */
		this.mdlFile = mdlFile;
		this.xlsxFile = xlsxFile;
		this.numCopies = numCopies;
		masterCopy = Utilities.getAsList(mdlFile);
		mdlDirectory = new File(mdlFile).getParent();
		xlsxDirectory = new File(xlsxFile).getParent();
		SimpleDateFormat timeStamp = new SimpleDateFormat("yyyy.MM.dd. HH.mm.ss");
		ts ="("+timeStamp.format(new Date())+")";
		
	}
	
	public void makeCopies() {
		int n = 1;
		while (n <= numCopies) {
			String extended = leadingZeros(n, 3);
			makeCopy(extended);
			n++;
		}
	}
	
	private String copiesDir(String modelName) {
		
		return modelName + "_Copies "+ts;
	}
	
	private void makeCopy(String extended) {
		String model = mdlFile.replaceAll("\\.[mM][dD][lL]$", "");
		String modelName = new File(mdlFile).getName().replaceAll("\\.[mM][dD][lL]$", "");
		String newModel = new File(model + "_" + extended +".mdl").getName();
		String newModelPath = mdlDirectory + "/" +copiesDir(modelName) + "/" + newModel;
		
		
//		String excel = xlsxFile.replaceAll("\\.[xX][lL][sS][xX]$", "");
		String excelName = new File(xlsxFile).getName();
		String excelNameNoExtension = excelName.replaceAll("\\.[xX][lL][sS][xX]$", "");
		String newExcel = new File(excelNameNoExtension+ "_" + modelName + "_" + extended +".xlsx").getName();
		String newExcelPath = xlsxDirectory + "/" +copiesDir(modelName) + "/" + newExcel;
		
		BufferedWriter bw = Utilities.openFileForWriting(newModelPath);
		try {
			for (String line : masterCopy) {
					bw.append(updateXlsxFile(line, excelName, newExcel));
					bw.append("\n");
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		makeXlsxCopy(xlsxFile, newExcelPath);
		
	}
	
	public String updateXlsxFile(String line, String excelName, String newExcel) {
		return line.replace(excelName, newExcel);
	}
	
	private void makeXlsxCopy(String existing, String newFile) {
		try {
			FileUtils.copyFile(new File(existing), new File(newFile));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private String leadingZeros(int num, int numPlaces) {
		String withLeadingZeros = Integer.toString(num);
		while (withLeadingZeros.length() < 3) {
			withLeadingZeros = "0" + withLeadingZeros;
		}
		return withLeadingZeros;
		
	}

}
