package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class ModelManager {
	
	public  static final String GRAPH_DIR = "_DOC_GRAPH_";
	public  static final String TOOL_OUTPUT_DIR = "_SDMDoc";
	
	private String fileName;
	private String modelName;
	private String generatedOutputDirectoryName = null;
	
	private boolean xmileOrigin;
	private String xmileFileName;
	
	private boolean map = false;
	
	private boolean runnable = true; // assume runnable if we find something, change
	private List<String> runnableReason = new ArrayList<String>();
	
	private FileOrganizer fileOrganizer;
	
	private boolean requiresExpandedSubscriptOrdering = false;

	private boolean sdmuiMode = false;
	private String sdmuiRootDirectory;
	
	private String mapperFile;
	
	private boolean runModel;
	
	private boolean modelSuccessfullyExecuted = false;
	
	public String getMapperFile() {
//		if (fileOrganizer == null) {
//			if (mapperFile == null)
//				mapperFile = fileName.replace(".mdl", "X#X#X").replace(".MDL", "X#X#X").replace("\\", "/").replace("X#X#X", ".map");
//			return mapperFile;
//		} else {
//			return fileOrganizer.getMapperFile();
//		}
		return InformationManagers.getInstance().getFileOrganizer().getMapperFile();
	}

	public ModelManager() {
	
	}
	
	public ModelManager(String fileName, String modelName) {
		this.fileName = fileName;
		setModelName(modelName);
	}
	
	public String getToolOutputDirectoryName() {
		return modelName + TOOL_OUTPUT_DIR;
	}

	public String getFileName() {
		return fileName;
	}
	
	public String getModelDirectory() {
		File f = new File(fileName);
		
		return f.getParent().replace("\\", "/");
	}

	public String getModelName() {
		return modelName;
	}

	public void setFileName(String fileName) {
		this.fileName = fileName;
	}
	
	public String getMdlName() {
		return new File(getFileName()).getName();
	}

	public void setModelName(String modelName) {
		this.modelName = modelName.replace("&", "_");
	}
	
	public String getOutputDirectoryName() {
//		return modelName+GRAPH_DIR;
		return InformationManagers.getInstance().getFileOrganizer().getRelativeDocGraphDirectory();
		
	}
	
	public String getGeneratedOutputDirectoryName() {
		return InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectory();
	}
	
	public String getGeneratedOutputDirectoryNameForZip() {
		return InformationManagers.getInstance().getFileOrganizer().getDocGraphDirectoryForZip();
	}
	
	public String getHtmlFileName() {
		String outputFileName;
		if (!sdmuiMode) {
			return InformationManagers.getInstance().getFileOrganizer().getHtmlFile();

		} else {
			outputFileName =InformationManagers.getInstance().getFileOrganizer().getModelDirectory()+"/"+
					modelName+"_"+SDMDocLanguageSpecific.getInstance().getDisplayLanguage()+".html";
		}
		return outputFileName;

	}
	
	public String getZipFileName() {
		return getHtmlFileName().replace(".html",".zip");
	}

	/**
	 * @return the sdmuiMode
	 */
	public boolean isSdmuiMode() {
		return sdmuiMode;
	}

	/**
	 * @param sdmuiMode the sdmuiMode to set
	 */
	public void setSdmuiMode(boolean sdmuiMode) {
		this.sdmuiMode = sdmuiMode;
	}

	/**
	 * @return the sdmuiRootDirectory
	 */
	public String getSdmuiRootDirectory() {
		return sdmuiRootDirectory;
	}

	/**
	 * @param sdmuiRootDirectory the sdmuiRootDirectory to set
	 */
	public void setSdmuiRootDirectory(String sdmuiRootDirectory) {
		this.sdmuiRootDirectory = sdmuiRootDirectory;
	}
	
	public boolean isRunnable() {
		return runnable;
	}

	public void setRunnable(boolean runnable) {
		this.runnable = runnable;
	}
	
	public void setRunnable(boolean runnable, String runnableReason) {
		this.runnable = runnable;
		if (!this.runnableReason.contains(runnableReason))
			this.runnableReason.add(runnableReason);
	}

	public String getRunnableReason() {
		
		// this is hokey, but for now...
		for (String r :  runnableReason) {
			if (r.startsWith("Too many elements for thumbnails"))
				return r+"\n";
		}
		
		if (!isRunModel()) {
			this.runnableReason.add("Model run not requested");
		}
		
		StringBuilder sb = new StringBuilder();
		for (String r :  runnableReason) {
			sb.append(r+"\n");
		}
		
		return sb.toString();
	}

	public FileOrganizer getFileOrganizer() {
		return fileOrganizer;
	}

	public void setFileOrganizer(FileOrganizer fileOrganizer) {
		this.fileOrganizer = fileOrganizer;
	}

	public boolean isRequiresExpandedSubscriptOrdering() {
//		System.out.println("### requiresExpandedSubscriptOrdering = "+requiresExpandedSubscriptOrdering+" ###");
		return requiresExpandedSubscriptOrdering;
	}

	public void setRequiresExpandedSubscriptOrdering(
			boolean requiresExpandedSubscriptOrdering) {
		this.requiresExpandedSubscriptOrdering = requiresExpandedSubscriptOrdering;
	}

	public boolean isMap() {
		return map;
	}

	public void setMap(boolean map) {
		this.map = map;
	}

	public boolean isRunModel() {
		return runModel;
	}

	public void setRunModel(boolean runModel) {
		this.runModel = runModel;
	}

	public boolean isXmileOrigin() {
		return xmileOrigin;
	}

	public void setXmileOrigin(boolean xmileOrigin) {
		this.xmileOrigin = xmileOrigin;
//		if (xmileOrigin) {
//			if (!this.runnableReason.contains("Model specified in XMILE"))
//				this.runnableReason.add("Model specified in XMILE");
//		}
	}

	public boolean isModelSuccessfullyExecuted() {
		return modelSuccessfullyExecuted;
	}

	public void setModelSuccessfullyExecuted(boolean modelSuccessfullyExecuted) {
		this.modelSuccessfullyExecuted = modelSuccessfullyExecuted;
	}

	public String getXmileFileName() {
		return xmileFileName;
	}

	public void setXmileFileName(String xmileFileName) {
		this.xmileFileName = xmileFileName;
	}

}
