package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

//import repast.simphony.data2.AggregateOp;
//import repast.simphony.data2.engine.DataSetDescriptor;
//import repast.simphony.engine.schedule.ScheduleParameters;

//import com.thoughtworks.xstream.XStream;

public class NativeDataTypeManager {
static Logger log = Logger.getLogger(NativeDataTypeManager.class.getSimpleName());

	private  static final String[] SYSTEM_INITIAL = {"INITIAL TIME", "INITIALTIME", "INTIAL_TIME"};
	private  static final String[] SYSTEM_FINAL = {"FINAL TIME", "FINALTIME", "FINAL_TIME"};
	private  static final String[] SYSTEM_TIMESTEP = {"TIME STEP", "TIMESTEP", "TIME_STEP"};
	private  static final String[] NAREPLACEMENT = {"NAREPLACEMENT"};
	private  static final String[] SAVEPER = {"SAVEPER"};
	
	private static Set<String> SYSTEM_VARIABLES = new LinkedHashSet<>();
	
	static {
		for (String s : SYSTEM_INITIAL)
			SYSTEM_VARIABLES.add(s);
		for (String s : SYSTEM_FINAL)
			SYSTEM_VARIABLES.add(s);
		for (String s : SYSTEM_TIMESTEP)
			SYSTEM_VARIABLES.add(s);
		for (String s : NAREPLACEMENT)
			SYSTEM_VARIABLES.add(s);
		for (String s : SAVEPER)
			SYSTEM_VARIABLES.add(s);
	}

	public  Set<String> scalars;
	// key = Vensim name, value = Java name
	public  Map<String, String> legal;
	public  Map<String, String> original;
	public  Map<String, String> dataType;
	// key = Vensim name
	public  Map<String, NativeArray> arrays;
	
	private int genMemCount = 0;

//	public  Set<ArrayDeclaration> arrayDeclarations = new LinkedHashSet<ArrayDeclaration>();
	
	public  Map<String, ArrayDeclaration> arrayDeclarations = new LinkedHashMap<String, ArrayDeclaration>();
	
	private int getterFile = 1;

	public NativeDataTypeManager() {
		scalars = new LinkedHashSet<String>();
		// key = Vensim name, value = Java name
		legal = new LinkedHashMap<String, String>();
		original = new LinkedHashMap<String, String>();
		dataType = new LinkedHashMap<String, String>();
		// key = Vensim name
		arrays = new LinkedHashMap<String, NativeArray>();

//		arrayDeclarations = new LinkedHashSet<ArrayDeclaration>();
		arrayDeclarations = new LinkedHashMap<String, ArrayDeclaration>();
	}
	
	public  OperationResult validateScalarReference(Map<String, Equation> equations, Equation equation, MutableInteger pos, MutableBoolean lhs) {
		OperationResult or = new OperationResult();
		String token = equation.getTokens().get(pos.value());
		String original = getOriginalName(token);
		
		if (scalars.contains(original))
			return or;
		
		if (arrays.containsKey(original)) {
			if (InformationManagers.getInstance().getArrayManager().isUsedAsLookup(original)) {
				return or;
			}
			or.setErrorMessage("Array referenced as Scalar - "+token+" - original name "+original);
			return or;
		}
		
		if (equations.containsKey(original))
			return or;
		
		or.setErrorMessage("Scalar not found - "+token+" - original name "+original);
		return or;
		
	}
	
	public NativeArray getNativeArray(String array) {
		if (arrays.containsKey(array))
			return arrays.get(array);
		else {
			String noAnnot = ArrayReference.deannotate(array);
			if (arrays.containsKey(noAnnot))
				return arrays.get(noAnnot);
			else
				return null;
		}
	}
	
	public  void addVariableAsArray(Equation equation, String variable, String dataType) {

		this.dataType.put(variable, dataType);
		addArray(equation, variable, dataType);
	}

	public  void addVariable(Equation equation, String variable, String dataType) {
		
		if (variable.startsWith("lookup."))
			System.out.println("LOOKUP");

		this.dataType.put(variable, dataType);
		if (ArrayReference.isArrayReference(variable))
			addArray(equation, variable, dataType);
		else
			addScalar(variable, dataType);
	}

	private  void addScalar(String scalar, String dataType) {
		if (arrays.containsKey(scalar)) {
			log.debug("#####Attempting to add scalar already defined as array: "+scalar);
			log.debug("Using array definition");
			return;
		}

		scalars.add(scalar);
		String legalized = legalize(scalar);
		legal.put(scalar, legalized);
		legal.put(legalized, legalized);
		addOriginalName(legalized, scalar);
	}

	private  void addArray(Equation equation, String arrayRef, String dataType) {
		
		//
		
		ArrayReference ar = new ArrayReference(arrayRef);
		String name = ar.getArrayName();
		NativeArray na = new NativeArray(name);

		if (scalars.contains(name)) {
			log.debug("Attempting to add array already defined as scalar: "+name);
			log.debug("Remove from scalars");
			scalars.remove(name);
		}

		arrays.put(name, na);
		String legalized = legalize(name);
		legal.put(name, legalized);
		legal.put(legalized, legalized);
		addOriginalName(legalized, name);
		int lookupHiddenDimensions = 0;
		if (equation != null && equation.isDefinesLookupGetXls())
			lookupHiddenDimensions = 2;
		na.setNumDimensions(ar.getSubscripts().size()+lookupHiddenDimensions);
		int s = 0;
		for (String subscript : ar.getSubscripts()) {
			na.setDimensionName(s++, subscript);
		}
	}

	private  String legalize(String name) {
		
		
		String legalName = null;
		legalName = Utilities.makeLegalJavaName(name);
		try {
			if (Parser.isNumber(Parser.characterAt(legalName, new MutableInteger(0))))
				legalName = "_" + legalName;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if (legal.containsValue(legalName)) {
			String col = collision(legalName);
			legal.put(name, col);
			addOriginalName(col, name);
		} else {
			legal.put(name, legalName);
			addOriginalName(legalName, name);
		}

		return legalName;
	}
	
	public static String getAsJavaLocalVariable(String var) {
		
//		log.debug("getAsJavaLocalVariable "+var);
		
		// remove annotations that may have been added (e.g. memory. lookup. etc)
		
		return var.replace("memory.", "").replace("lookup.", "");
		
	}

	private  String collision(String name) {
		String aPossible = name + "_";
		while (legal.containsValue(aPossible)) {
			aPossible += "_";
		}
		return aPossible;
	}
	
	public String quoteIfNecessary(String vensimVariable) {
		return quoteIfNecessary(vensimVariable, false);
	}
		public String quoteIfNecessary(String vensimVariable, boolean force) {
			if (vensimVariable.startsWith("\"") && vensimVariable.endsWith("\""))
				return vensimVariable;

		// allow the operators to pass through
		if (Parser.isOperator(vensimVariable) || (vensimVariable.length() == 1 && Parser.isTerminator(vensimVariable.charAt(0))) ||
				(vensimVariable.length() == 1 && vensimVariable.equals("[")) ||
				(vensimVariable.length() == 1 && vensimVariable.equals("]"))) {
			//			log.debug("========== quoted in/out (operator) ///"+vensimVariable+"/// "+vensimVariable);
			return vensimVariable;
		}
		
		boolean isVar = false;
		if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(vensimVariable) != null && 
				InformationManagers.getInstance().getEquationManager().getAllEquationsFor(vensimVariable).size() >= 1) {
			isVar = true;
		}
		
		if (!isVar) {
			//			log.debug("========== quoted in/out (operator) ///"+vensimVariable+"/// "+vensimVariable);
			return vensimVariable;
		}
		
		if (ArrayReference.isArrayReference(vensimVariable)) {
			ArrayReference ar = new ArrayReference(vensimVariable);
			return ar.getWithQuotedArrayName(); // ar.getFullyQuoted();
		}

		if (Parser.startsWithNumber(vensimVariable) || vensimVariable.contains("/") ||
				vensimVariable.contains("=") ||
				vensimVariable.contains(":") ||
				vensimVariable.contains("-") ||
				vensimVariable.contains(",") ||
				vensimVariable.contains("&") ||
				vensimVariable.contains("%") ||
				vensimVariable.contains("+") ||
				vensimVariable.contains("*") ||
				vensimVariable.contains("$") ||
				vensimVariable.contains("<") ||
				vensimVariable.contains(">") ||
				vensimVariable.contains(".") ||
				vensimVariable.contains("(") ||
				vensimVariable.contains(")") ||
				force ) {
			String quoted = "\""+ vensimVariable + "\"";

//			if (ArrayReference.isArrayReference(vensimVariable)) {
//				ArrayReference ar = new ArrayReference(vensimVariable);
//				return ar.getWithQuotedArrayName(); // ar.getFullyQuoted();
//			} else {
				if (quoted.contains("[")) {
					quoted = quoted.replace("]\"", "]").replace("[", "\"[");
				}

				//			log.debug("========== quoted in/out ///"+vensimVariable+"/// "+quoted);
				return quoted;
//			}
		} else {
			//			log.debug("========== quoted in/out (no change)///"+vensimVariable+"/// "+vensimVariable);
			return vensimVariable;
		}

	}
	
	public String quoteIfNecessary(String originalVariable, String translatedVariable) {
		


		// allow the operators to pass through
		if (Parser.isOperator(translatedVariable) || (translatedVariable.length() == 1 && Parser.isTerminator(translatedVariable.charAt(0))) ||
				(translatedVariable.length() == 1 && translatedVariable.equals("[")) ||
				(translatedVariable.length() == 1 && translatedVariable.equals("]"))) {
			//			log.debug("========== quoted in/out (operator) ///"+vensimVariable+"/// "+vensimVariable);
			return translatedVariable;
		}
		
		boolean isVar = false;
		if (InformationManagers.getInstance().getEquationManager().getAllEquationsFor(originalVariable) != null && InformationManagers.getInstance().getEquationManager().getAllEquationsFor(originalVariable).size() >= 1) {
			isVar = true;
		}
		
		if (!isVar) {
			//			log.debug("========== quoted in/out (operator) ///"+vensimVariable+"/// "+vensimVariable);
			return translatedVariable;
		}
		
		if (translatedVariable.trim().length() == 0)
			return originalVariable;
		
		// if we start with a double quote, just return it
		if (Parser.characterAt(0, translatedVariable).equals("\""))
			return translatedVariable;

		if (Parser.startsWithNumber(translatedVariable) || translatedVariable.contains("/") ||
				translatedVariable.contains("=") ||
				translatedVariable.contains(":") ||
				translatedVariable.contains("-") ||
				translatedVariable.contains(",") ||
				translatedVariable.contains("&") ||
				translatedVariable.contains("%") ||
				translatedVariable.contains("+") ||
				translatedVariable.contains("*") ||
				translatedVariable.contains(".") ||
				translatedVariable.contains("(") ||
				translatedVariable.contains(")") ) {
			String quoted = "\""+ translatedVariable + "\"";

			if (ArrayReference.isArrayReference(translatedVariable)) {
				ArrayReference ar = new ArrayReference(translatedVariable);
				return ar.getFullyQuoted();
			} else {
				if (quoted.contains("[")) {
					quoted = quoted.replace("]\"", "]").replace("[", "\"[");
				}

				//			log.debug("========== quoted in/out ///"+vensimVariable+"/// "+quoted);
				return quoted;
			}
		} else {
			//			log.debug("========== quoted in/out (no change)///"+vensimVariable+"/// "+vensimVariable);
			return translatedVariable;
		}

	}
	
	public  String getOriginalNameRemovingAnnotation(String anAnnotatedVariable) {
		String without = new String(anAnnotatedVariable);
		if (anAnnotatedVariable.startsWith("array."))
			without =  anAnnotatedVariable.replace("array.", "");
		if (anAnnotatedVariable.startsWith("lookup."))
			without =  anAnnotatedVariable.replace("lookup.", "");
		if (anAnnotatedVariable.startsWith("memory."))
			without =  anAnnotatedVariable.replace("memory.", "");
		else
			without =  anAnnotatedVariable;
		return getOriginalName(without);
		
	}
	
	public Set<String> getOriginalNames(Set<String> vars) {
		Set<String> o = new LinkedHashSet<String>();
		for (String var : vars) {
			o.add(getOriginalNameRemovingAnnotation(var));
		}
		
		return o;
		
	}

	public  String getOriginalName(String legalized) {
		
		String nameToReturn = "";
		
		String minus = Utilities.revertFromLegalJavaName(legalized);
		if (minus.contains("MINUS")) {
			System.out.println("MINUS: "+legalized);
			minus = Utilities.revertFromLegalJavaName(legalized);
		}
		
		if (legalized.startsWith("array."))
			return legalized.replace("array.", "");
		if (legalized.startsWith("lookup."))
			return legalized.replace("lookup.", "");
		
		String s = legalized.replace("memory.", "").replace("lookup.", "");
		
		
		if (original.containsKey(s)) {
			nameToReturn =  original.get(s);
		} else{ 
			nameToReturn =  legalized;
		}
		
		// force lookup change
//		return nameToReturn.replace(StringConstants.RSSD_LOOKUP, "");
		return nameToReturn;
	}
	
	public  String getOriginalNameForSubstitution(String legalized) {
		if (legalized.startsWith("array."))
			return legalized;
		String s = legalized.replace("memory.", "").replace("lookup.", "");
		if (original.containsKey(s))
			return original.get(s);
		else
			return legalized;
	}
	
	public  String getOriginalNameQuotedIfNecessary(String legalized) {
		return getOriginalNameQuotedIfNecessary(legalized, false);
	}
	
	public  String getOriginalNameQuotedIfNecessary(String legalized, boolean force) {
		if (Parser.isNumber(legalized))
			return legalized;
		if (Parser.isArithmeticOperator(legalized))
			return legalized;
		if (Parser.isRelationalOperator(legalized)) {
			if (legalized.equals("=="))
				return "=";
			else
				return legalized;
		}
		if (legalized.startsWith("array."))
			return quoteIfNecessary(legalized.replace("array.", ""), force);
		String s = legalized.replace("memory.", "").replace("lookup.", "");
		if (original.containsKey(s))
			return quoteIfNecessary(original.get(s), force);
		else
			return quoteIfNecessary(legalized, force);
	}
	
//	public  String getOriginalNameQuoted(String legalized) {
//		return getOriginalNameQuoted(legalized, false);
//	}
//	
//	public  String getOriginalNameQuoted(String legalized, boolean force) {
//		if (Parser.isNumber(legalized))
//			return legalized;
//		if (Parser.isArithmeticOperator(legalized))
//			return legalized;
//		if (Parser.isRelationalOperator(legalized)) {
//			if (legalized.equals("=="))
//				return "=";
//			else
//				return legalized;
//		}
//		if (legalized.startsWith("array."))
//			return quoteIfNecessary(legalized.replace("array.", ""), force);
//		String s = legalized.replace("memory.", "").replace("lookup.", "");
//		if (original.containsKey(s))
//			return quoteIfNecessary(original.get(s), force);
//		else
//			return quoteIfNecessary(legalized, force);
//	}

	public  String getLegalName(String variable) {
		return getLegalName(null, variable);
	}

	public  String getLegalName(Equation equation, String variable) {
		String referenceType = "memory.";
		if (variable.contains("memory."))
			referenceType = "memory.";
		else if (variable.contains("lookup."))
			referenceType = "lookup.";
		String in = variable.replace("memory.", "").replace("lookup.", "");
		if (ArrayReference.isArrayReference(in)) {
			in = new ArrayReference(in).getArrayName();
//			log.debug("Array Reference: "+variable+" /// "+in);
		}
		if (Parser.isInteger(in) || Parser.isReal(in)) {
			return in;
		}
		String name = legal.get(in);
		if (name == null) {
			if (Parser.isInteger(in) || Parser.isReal(in)) {
				return in;
			}
			// we haven't seen this yet
			if (in.equals("array")) {
				log.debug("First getLegalName Legal -> <"+in+"><"+name+">");
				Thread.dumpStack();
			}

			addVariable(equation, variable, "double");
		}
		name = legal.get(in);
		if (name == null) {
			if (in.equals("array")) {
				log.debug("SEcond Legal -> <"+in+"><"+name+">");
				Thread.dumpStack();
			}
			// we haven't seen this yet

		}

		if (SDM_Doc.target.equals(ReaderConstants.JAVA)) {
			return "memory." + name;
		} else if (SDM_Doc.target.equals(ReaderConstants.C)) {
			return "memory." + name;
		} else {
			return "memory." + name;
		}
	}

	public  void generateMemoryJava(BufferedWriter bw, String objectName, String packageName /*, boolean newMemoryDefinition*/) {

		try {
			// class header information

			//	    bw.append("package "+ReaderConstants.PACKAGE+";\n\n");
			bw.append("package "+packageName+";\n\n");
			bw.append("public class Memory"+objectName+" {\n");
			List<String> al;
			// scalars
//			if (!newMemoryDefinition) {
//				al = new ArrayList<String>();
//				al.addAll(scalars);
//				Collections.sort(al);
//				for (String scalar : al) {
//
//					bw.append("public "+dataType.get(scalar)+" "+legal.get(scalar)+";\n");
//					//		bw.append("public  double "+legal.get(scalar)+";\n");
//					//		bw.append("List<Double> "+legal.get(scalar)+"_history = new ArrayList<Double>();\n");
//				}
//			} else {
				bw.append("public  double "+"TIME_STEP"+";\n");
				bw.append("public  double "+"FINAL_TIME"+";\n");
				bw.append("public  double "+"INITIAL_TIME"+";\n");
				bw.append("public  double "+"SAVEPER"+";\n");
//			}

			// arrays
			
			ArrayManager am = InformationManagers.getInstance().getArrayManager();

			al = new ArrayList<String>();
//			al.addAll(arrays.keySet());
			al.addAll(am.getArraysUsedAsLookup());
			Collections.sort(al);
			for (String array : al) {
				
				if ( /*!newMemoryDefinition || (newMemoryDefinition && */ am.isUsedAsLookup(array) /*)*/ ) {

					bw.append("public double");
					//		bw.append("public  double");
					NativeArray na = arrays.get(array);
					if (na == null) {
						System.out.println("NO NATIVE ARRAY: "+array);
						continue;
					}
					for (int i = 0; i < na.getNumDimensions(); i++)
						bw.append("[]");
					bw.append(" "+legal.get(array)+";\n");
				}
			}

			// constructor

			bw.append("public Memory"+objectName+"() {\n");

			for (String array : al) {
				if (! /*(!newMemoryDefinition || (newMemoryDefinition && */ am.isUsedAsLookup(array))/* ))*/ {
					continue;
				}
				
				int dim = 0;

				NativeArray na = arrays.get(array);
				if (na == null) {
					System.out.println("NO NATIVE ARRAY for "+array);
					continue;
				}
				boolean declare = false;
				for (String subscript : na.getDimensionNames()) {
					String numInd = Integer.toString(InformationManagers.getInstance().getArrayManager().getNumIndicies(array, dim));
					if (!numInd.equals("0"))
						declare = true;
					dim++;
				}
				if (!declare)
					continue;
				bw.append(legal.get(array)+" = new double");
				dim = 0;
				for (String subscript : na.getDimensionNames()) {
					String numInd = Integer.toString(InformationManagers.getInstance().getArrayManager().getNumIndicies(array, dim));
					if (numInd.equals("0"))
						numInd = "/* "+subscript+" */";
					bw.append("["+numInd+"]");
					dim++;
				}
				bw.append(";\n");

			}

			bw.append("}\n\n");

//			generateDataSetGetters(objectName, translator);

			// HERE!!! these three getters need to be customized to what variables are use!
			
			
			bw.append("public double get_SAVEPER() {\n");
			bw.append("return SAVEPER;\n");
			bw.append("}\n");

			bw.append("public double getINITIALTIME() {\n");
			bw.append("return "+getInitialTimeVariable()+";\n");
			bw.append("}\n");

			bw.append("public double getFINALTIME() {\n");
			bw.append("return "+getFinalTimeVariable()+";\n");
			bw.append("}\n");

			bw.append("public double getTIMESTEP() {\n");
			bw.append("return "+getTimeStepVariable()+";\n");
			bw.append("}\n");

			// class tail
			bw.append("}\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	public  void generateMemoryC(BufferedWriter bw, String objectName, 
			ActiveInitialProcessor aip, boolean requiresActiveInitialProcessing) {
		
		// until I can figure out what is screwing up
		genMemCount++;

		try {
			
			bw.append("#ifndef ClearnLibOSX_memoryCLearn_h\n");
			bw.append("#define ClearnLibOSX_memoryCLearn_h\n");

			bw.append("#include \"vensimSupport.h\"\n");


			// scalars
			List<String> al = new ArrayList<String>();
			al.addAll(scalars);
//			if (aip != null) {
//				for (String lhs : aip.getVariablesToDeclare()) {
//					if (scalars.contains(lhs)) {
//						al.add(lhs+ActiveInitialProcessor.INITIAL_VALUE_MARKING);
////						al.add(ActiveInitialProcessor.makeInitialVariable(lhs));
//					}
//				}
//			}
			Collections.sort(al);
			for (String scalar : al) {
				boolean isInitialValue = false;
				String effectiveScalar;
				String realScalar;
				if (scalar.endsWith(ActiveInitialProcessor.INITIAL_VALUE_MARKING)) {
					effectiveScalar = scalar;
					isInitialValue = true;
					realScalar = scalar.replace(ActiveInitialProcessor.INITIAL_VALUE_MARKING, "");
				} else {
					effectiveScalar = scalar;
					realScalar = scalar;
				}
				
//				System.out.println("realScalar: "+realScalar+" effectiveScalar: "+effectiveScalar);
				
				// note thata we want this to fail because we will not be referencing a lookup directly for a value
				if (InformationManagers.getInstance().getArrayManager().isUsedAsLookup(effectiveScalar)) {
					bw.append(CodeGenerator.scrubC("LOOKUP_t *"));
					//		    int n = ArrayManager.getNumDimensions(scalar);
					//		    for (int i = 0; i < 2; i++)
					//			bw.append("*");
				} else {
					String t = "double";
					if (dataType.get(realScalar).equals("String"))
						t = "char[256]";
					bw.append(CodeGenerator.scrubC(t+" "));
				}
				bw.append(CodeGenerator.scrubC(legal.get(realScalar)+
						(isInitialValue ? ActiveInitialProcessor.INITIAL_VALUE_MARKING : "")+";\n"));
			}

			// arrays

			al = new ArrayList<String>();
			al.addAll(arrays.keySet());
			if (aip != null) {
				for (String lhs : aip.getVariablesToDeclare()) {
					if (arrays.containsKey(lhs)) {
						if (!al.contains(lhs+ActiveInitialProcessor.INITIAL_VALUE_MARKING))
							al.add(lhs+ActiveInitialProcessor.INITIAL_VALUE_MARKING);
					}
				}
			}
			Collections.sort(al);

			for (String array : al) {
				boolean isInitialValue = false;
				String effectiveArray;
				String realArray;
				if (array.contains(ActiveInitialProcessor.INITIAL_VALUE_MARKING)) {
					effectiveArray = array;
					realArray = /* effectiveArray  = */ array.replace(ActiveInitialProcessor.INITIAL_VALUE_MARKING, "");
					isInitialValue = true;
				} else {
					realArray = array;
					effectiveArray = array;
				}

//				System.out.println("realArray: "+realArray+" effectiveArray: "+effectiveArray);
				
				int dim = 0;

				String arrayType;
				
				// need to worry about initial values for stock variables .replace("ACTIVEINITIAL", "_initialValue");

				
				if (InformationManagers.getInstance().getArrayManager().isUsedAsLookup(effectiveArray)) {
					bw.append(CodeGenerator.scrubC("LOOKUP_t *"));
					arrayType = "LOOKUP_t";
				} else {
					bw.append(CodeGenerator.scrubC("double "));
					arrayType = "double";
				}

				NativeArray na = arrays.get(realArray);
				dim = 0;
				int numDimensions = na.getDimensionNames().length;
				if (arrayType.equals("LOOKUP_t"))
					numDimensions -= 2;
				if (numDimensions <= 0) {
					bw.append(""); // "*"
				} else {
					Integer[] dimSize = new Integer[numDimensions];

					for (String subscript : na.getDimensionNames()) {
						String numInd = Integer.toString(InformationManagers.getInstance().getArrayManager().getNumIndicies(realArray, dim));
						//		    if (!numInd.equals("0")) {
						bw.append("*");
						dimSize[dim] = Integer.parseInt(numInd);
						//		    } else {
						//			System.out.println("brkpt");
						//		    }

						dim++;
						if (dim == numDimensions)
							break;
					}

					ArrayDeclaration ad;

					if ((aip == null && genMemCount == 1 && !requiresActiveInitialProcessing) || 
							(aip != null && genMemCount > 1) && requiresActiveInitialProcessing) {



						if (isInitialValue) {
							ad = new ArrayDeclaration(CodeGenerator.scrubC(legal.get(realArray)+ActiveInitialProcessor.INITIAL_VALUE_MARKING), 
									arrayType, dimSize.length, dimSize);
							if (!arrayDeclarations.containsKey(ad.getName())) {
								arrayDeclarations.put(ad.getName(), ad);
//								System.out.println("genMemCount->>> "+genMemCount+" "+ad.getName());
							}
						} else {
							ad = new ArrayDeclaration(CodeGenerator.scrubC(legal.get(realArray)), arrayType, dimSize.length, dimSize);
							if (!arrayDeclarations.containsKey(ad.getName())) {
								arrayDeclarations.put(ad.getName(), ad);
//								System.out.println("genMemCount->>> "+genMemCount+" "+ad.getName());
							}
						}
					}


				}
				if (isInitialValue) {
					bw.append(CodeGenerator.scrubC(legal.get(realArray)+ActiveInitialProcessor.INITIAL_VALUE_MARKING));
				} else {
					bw.append(CodeGenerator.scrubC(legal.get(realArray)));
				}

				//		na = arrays.get(array);
				//		dim = 0;
				//		for (String subscript : na.getDimensionNames()) {
				//		    String numInd = Integer.toString(ArrayManager.getNumIndicies(array, dim));
				//		    if (!numInd.equals("0")) {
				//			bw.append(CodeGenerator.scrub("["+numInd+"]"));
				//		    }
				//		    
				//		    dim++;
				//		}
				bw.append(";\n");

			}
			bw.append("#endif\n");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	



	
	public  void generateArrayDeclarationC(BufferedWriter bw/* , String objectName, Translator translator */) {

		try {
			bw.append("void declareArrays() {\n");

			List<ArrayDeclaration> sortedAD = new ArrayList<ArrayDeclaration>();
			for (String key : arrayDeclarations.keySet()) {
				ArrayDeclaration ad = arrayDeclarations.get(key);;
				sortedAD.add(ad);
//				bw.append("/* "+ad.getName()+" */\n");
			}
			
			Collections.sort(sortedAD, new Comparator<ArrayDeclaration>() {
				@Override
				public int compare(ArrayDeclaration o1, ArrayDeclaration o2) {
					
					return o1.getName().compareTo(o2.getName());

				}
			});
			
			
			int alreadyDefined = 0;
			for (ArrayDeclaration ad : sortedAD) {
				
				int currentR = 1;


				// NOTE: Currently implemented for only 2 dimensions

				// double x[3];  ==>
				//		double *x;
				//		x = (double *) malloc(3 * sizeof(double));
				// double x[3][4]; ==>
				//		double **x;
				//		x = (double *) malloc(3 * sizeof(double));
				//		int r;
				//		for (r = 0; r < 3; r++) { x[r] = (double *) malloc(4 * sizeof(double)); }
				//
				// bw.append(ad.getName()+" = ("+ad.getType()+"*) malloc("+ad.getNumDimensions()+" * sizeof("+ad.getType()+"*));\n");

				//	        if (ad.getType().equals("LOOKUP_t")) {
				//	            bw.append(ad.getName()+" = ("+ad.getType()+"*) malloc("+ad.getNumDimensions()+" * sizeof("+ad.getType()+"*));\n");
				//	            continue;
				//	        }



				bw.append(ad.getName()+" = ("+ad.getType());
				if (ad.getType().equals("LOOKUP_t"))
					bw.append("*");


				for (int i = 0; i < ad.getNumDimensions(); i++)
					bw.append("*");

				bw.append(") malloc("+ad.getDimensionSize()[0]+" * sizeof("+ad.getType());

				if (ad.getNumDimensions() > 1 || ad.getType().equals("LOOKUP_t"))
					bw.append(" *));\n");
				else
					bw.append("));\n");
				if (ad.getType().equals("LOOKUP_t"))
					continue;
				for (int dim = 1; dim < ad.getNumDimensions(); dim++) {
					if (currentR > alreadyDefined) {
						bw.append("int r"+currentR+";\n");
						alreadyDefined = currentR;
					}
					bw.append("for (r"+currentR+" = 0; r"+currentR+" < "+ad.getDimensionSize()[dim-1]+"; r"+currentR+"++) {\n"); // was dim
					bw.append(ad.getName()+"[r"+currentR+"] = ("+ad.getType()+"*) malloc("+ad.getDimensionSize()[dim]+" * sizeof("+ad.getType()+"));\n");
					currentR++;

				}
				for (int dim = 1; dim < ad.getNumDimensions(); dim++) {
					bw.append("}\n");
				}

			}

			bw.append("}\n\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}



//	private  void generateDataSetGetters(String objectName, Translator translator) {
//		
//		int maxGetterCount = 500; // 500
//		int getterCount = 0;
//		
//		
//		BufferedWriter bw = null;
//
//		// Create a DataSetDescriptor
//
//		DataSetDescriptor dsd = new DataSetDescriptor("DS1", DataSetDescriptor.DataSetType.AGGREGATE);
//		dsd.setIncludeRandomSeed(true);
//		dsd.setIncludeTick(true);
//		dsd.setScheduleParameters(ScheduleParameters.createRepeating(1, 1));
//		
////		dsd.addAggregateMethodDataSource(id, className, methodName, aggType);
////		dsd.addAggregateMethodDataSource("ID", translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, "getID", AggregateOp.SUM);
//
////		dsd.addMethodDataSource("ID", translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, "getID");
////		dsd.setSourceType(translator.getPackageName()+".MemoryGetter"+objectName);
//		getterCount++;
//		
////		dsd.addMethodDataSource("ID", translator.getPackageName()+".Memory"+objectName, "getID");
////		dsd.setSourceType(translator.getPackageName()+".Memory"+objectName);
//		
//		String SourceDirectory = translator.getSourceDirectory() + translator.asDirectoryPath(translator.getPackageName())+ "/";
//		writeMemoryGetterBaseClass(SourceDirectory, translator.getPackageName(), objectName);
//		bw = Translator.openReport(SourceDirectory+"MemoryGetter"+getterFile+"_"+objectName+".java");
//		writeMemoryGetterHeader(bw, "MemoryGetter"+getterFile+"_"+objectName, objectName, translator.getPackageName(), getterFile-1);
//		
//		try {
//			bw.append("public String "+"getID"+"() {\n");
//			bw.append("return \""+objectName+"\";\n");
//			bw.append("}\n");
//		} catch (IOException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
//
//		// scalars
//		try {
//			List<String> al = new ArrayList<String>();
//			al.addAll(scalars);
//			Collections.sort(al);
//			for (String scalar : al) {
//				
//				if (getterCount >= maxGetterCount) {
//					bw.append("}\n");
//					bw.close();
//					getterFile++;
//					getterCount = 0;
//					bw = Translator.openReport(SourceDirectory+"MemoryGetter"+getterFile+"_"+objectName+".java");
//					writeMemoryGetterHeader(bw, "MemoryGetter"+getterFile+"_"+objectName, objectName, translator.getPackageName(), getterFile-1);
//				}
//
//				String getter = "get_"+legal.get(scalar);
//				String original = scalar;
//
////				dsd.addMethodDataSource(original, translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, getter);
//				dsd.addAggregateMethodDataSource(original, translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, getter, AggregateOp.SUM);
////				dsd.setSourceType(translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName);
//				
//				getterCount++;
//
//				bw.append("public double get_"+legal.get(scalar)+"() {\n");
//				bw.append("return memory."+legal.get(scalar)+";\n");
//				bw.append("}\n");
//				//		bw.append("public  double "+legal.get(scalar)+";\n");
//				//		bw.append("List<Double> "+legal.get(scalar)+"_history = new ArrayList<Double>();\n");
//			}
//
//			// arrays
//
//			// NOTE: we do not want to generate getters for any lookup tables
//
//			al = new ArrayList<String>();
//			al.addAll(arrays.keySet());
//			Collections.sort(al);
//			for (String array : al) {
//
//				if (EquationProcessor.lookups.contains(array)) {
//					continue;
//				}
//
//				Map<Integer, List<Integer>> indexValueMap = new LinkedHashMap<Integer, List<Integer>>();
//				for (int dimension = 0; dimension < InformationManagers.getInstance().getArrayManager().getNumDimensions(array); dimension++) {
//					indexValueMap.put(dimension, new ArrayList<Integer>());
//					for (int index = 0; index < InformationManagers.getInstance().getArrayManager().getNumIndicies(array, dimension); index++) {
//						indexValueMap.get(dimension).add(index);
//					}
//				}
//
//				// total number of dimensions in the array
//				int numDimensions = InformationManagers.getInstance().getArrayManager().getNumDimensions(array);
//
//				List<StringBuilder> methodName = new ArrayList<StringBuilder>();
//				List<StringBuilder> bodySubscript = new ArrayList<StringBuilder>();
//
//				// compute the number of combinations of indicies
//				int numToGenerate = 1;
//				for (int dimension = 0; dimension < InformationManagers.getInstance().getArrayManager().getNumDimensions(array); dimension++) {
//					numToGenerate *= indexValueMap.get(dimension).size();
//				}
//				for (int i = 0; i < numToGenerate; i++) {
//					methodName.add(new StringBuilder());
//					bodySubscript.add(new StringBuilder());
//				}
//
//				// for each dimension compute 
//				for (int dimension = 0; dimension < InformationManagers.getInstance().getArrayManager().getNumDimensions(array); dimension++) {
//					int numPer = 1;
//					for (int dim = dimension+1; dim < InformationManagers.getInstance().getArrayManager().getNumDimensions(array); dim++) {
//						numPer *= indexValueMap.get(dim).size();
//					}
//					int ptr = 0;
//
//					while (ptr < numToGenerate) {
//
//						for (int subr = 0; subr < indexValueMap.get(dimension).size(); subr++) {
//							for (int j = 0; j < numPer; j++) {
//								StringBuilder sb = bodySubscript.get(ptr);
//								sb.append("["+subr+"]");
//								StringBuilder sb2 = methodName.get(ptr);
//								ptr++;
//								if (sb2.length() == 0) {
//									sb2.append("$__$");  // this represents "["
//								} else {
//									sb2.append("$_$");   // this represents ","
//								}
//								sb2.append(InformationManagers.getInstance().getArrayManager().getVensimSubscript(array, dimension, subr));
//							}
//						}
//					}
//
//				}
//
//
//
//
//				for (int i = 0; i < bodySubscript.size(); i++) {
//
//					String getter = "get_"+legal.get(array)+makeLegal(methodName.get(i).toString().trim());
//					String original = array;
//					String vensim = asVensim(original+makeLegal(methodName.get(i).toString().trim()));
//					
//					if (getterCount >= maxGetterCount) {
//						bw.append("}\n");
//						bw.close();
//						getterFile++;
//						getterCount = 0;
//						bw = Translator.openReport(SourceDirectory+"MemoryGetter"+getterFile+"_"+objectName+".java");
//						writeMemoryGetterHeader(bw, "MemoryGetter"+getterFile+"_"+objectName, objectName, translator.getPackageName(),getterFile-1);
//					}
//
////					dsd.addMethodDataSource(vensim, translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, getter);
//					
////					dsd.setSourceType(translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName);
//					
//					getterCount++;
//
//					// Start
//					int dim = 0;
//					NativeArray na = arrays.get(array);
//					StringBuilder squareBrackets = new StringBuilder();
//
//					for (String subscript : na.getDimensionNames()) {
//						String numInd = Integer.toString(InformationManagers.getInstance().getArrayManager().getNumIndicies(array, dim));
//						if (numInd.equals("0"))
//							squareBrackets.append("[]");
//						dim++;
//					}
//					// End
//					if (squareBrackets.toString().length() > 0)
//						continue;
//					dsd.addAggregateMethodDataSource(vensim, translator.getPackageName()+".MemoryGetter"+getterFile+"_"+objectName, getter, AggregateOp.SUM);
//					bw.append("public double"+squareBrackets.toString()+" "+getter+"() {\n");
//					bw.append("return memory."+legal.get(array)+bodySubscript.get(i).toString()+";\n");
//					bw.append("}\n");
//				}
//
//
//
//
//			}
//			//		 dsd.addMethodDataSource("ID", objectName+".Memory"+objectName, "getID");
//			//			dsd.setSourceType(objectName+".Memory"+objectName);
//			
//			bw.append("}\n");
//			bw.close();
//
//			
//
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//
//		String ScenarioDirectory = translator.getScenarioDirectory();
//		
//		dsd.setScheduleParameters(repast.simphony.engine.schedule.ScheduleParameters.createRepeating(1, 1, 
//				repast.simphony.engine.schedule.PriorityType.LAST ));
//
//		serialize(dsd, ScenarioDirectory+"repast.simphony.action.data_set_0.xml");
//	}
	
	private void writeMemoryGetterBaseClass(String sourceDirectory, String packageName, String objectName) {
		BufferedWriter bw = SDM_Doc.openReport(sourceDirectory+"MemoryGetter"+objectName+".java");
		
		try {
			bw.append("package "+packageName+";\n\n");
			bw.append("public class MemoryGetter"+objectName+" {\n");
			bw.append("protected Memory"+objectName+" memory;\n");
			bw.append("public MemoryGetter"+objectName+"(Memory"+objectName+" memory) {\n");
			bw.append("this.memory = memory;\n");
			bw.append("}");
			bw.append("}");
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void writeMemoryGetterHeader(BufferedWriter bw, String newObjectName, String objectName, String packageName, int previous) {

		try {
			bw.append("package "+packageName+";\n\n");
			if (previous == 0) {
				bw.append("public class "+newObjectName+" extends MemoryGetter"+objectName+"{\n");
			} else {
				bw.append("public class "+newObjectName+" extends MemoryGetter"+objectName+"{\n");
//				bw.append("public class "+newObjectName+" extends MemoryGetter"+previous+"_"+objectName+"{\n");
			}

			bw.append("public "+newObjectName+"(Memory"+objectName+ " memory) {\n");
			
			bw.append("super(memory);\n\n");
			bw.append("}\n\n");
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

//	private  void serialize(DataSetDescriptor dsd, String toFile) {
//		XStream xstream = new XStream();
//		//	xstream.registerConverter(new FastMethodConverter(xstream));
//		try {
//			xstream.toXML(dsd, new FileWriter(toFile));
//		} catch (IOException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
//	}

	private  String asVensim(String s) {
		String vensim = s.replace("$__$", "[");
		vensim = vensim.replace("$_$", ",");
		return vensim + "]";
	}

	private  String getInitialTimeVariable() {
		
		Iterator<String> iter = scalars.iterator();
		while (iter.hasNext()) {
			String s = iter.next();
			for (String si : SYSTEM_INITIAL) {
				if (si.equalsIgnoreCase(s)) {
					return Utilities.makeLegalJavaName(s);
				}
			}
		}
		
		return "ERROR_INITIAL";
	}

	private  String getFinalTimeVariable() {
		
		Iterator<String> iter = scalars.iterator();
		while (iter.hasNext()) {
			String s = iter.next();
			for (String si : SYSTEM_FINAL) {
				if (si.equalsIgnoreCase(s)) {
					return Utilities.makeLegalJavaName(s);
				}
			}
		}
		
		return "ERROR_FINAL";
	}

	private  String getTimeStepVariable() {
		Iterator<String> iter = scalars.iterator();
		while (iter.hasNext()) {
			String s = iter.next();
			for (String si : SYSTEM_TIMESTEP) {
				if (si.equalsIgnoreCase(s)) {
					return Utilities.makeLegalJavaName(s);
				}
			}
		}
		
		
		return "ERROR_TIMESTEP";
	}

	public  void dumpLegalNames(BufferedWriter bw) {

		try {
			bw.append("Name,Legal Name\n");
			for (String name : legal.keySet()) {
				bw.append(name+","+legal.get(name)+"\n");
			}

			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}	
	}

	public  String getDataType(String var) {
		return dataType.get(var);
	}
	
    public String getLegalNameWithSubscripts(Equation equation, String variable) {
	String subscript = "";
	String referenceType = "memory.";
	if (variable.contains("memory."))
	    referenceType = "memory.";
	else if (variable.contains("lookup."))
	    referenceType = "lookup.";
	else if (variable.contains("array."))
	    referenceType = "array.";
	String in = variable.replace("memory.", "").replace("lookup.", "");
	if (ArrayReference.isArrayReference(in)) {
	    in = new ArrayReference(in).getArrayName();
	    subscript = "[" + variable.split("\\[")[1];
	}
	String name = legal.get(in);
	if (name == null) {
	    if (Parser.isInteger(in) || Parser.isReal(in))
		return in;
	    // we haven't seen this yet
	    log.debug("getLegalNameWithSubscripts Legal -> <"+in+"><"+name+">");
	    addVariable(equation, variable, "double");
	}
	name = legal.get(in);
	if (name == null) {
	    // we haven't seen this yet
	    log.debug("Legal -> <"+in+"><"+name+">");
	}

	if (SDM_Doc.target.equals(ReaderConstants.JAVA)) {
	    return referenceType + name + subscript;
	} else if (SDM_Doc.target.equals(ReaderConstants.C)) {
	    return "memory." + name;
	} else {
	    return "memory." + name;
	}
    }
    
    public boolean isScalar(String variable) {
    	return scalars.contains(variable);
    }
    
    public boolean isArray(String variable) {
    	return arrays.containsKey(variable);
    }

	public int getGetterFile() {
		return getterFile;
	}
	
	public void addOriginalName(String legalized, String name) {
		original.put(legalized, name);
	}
	
	public static boolean isSystemVariable(String var) {
//		private  static final String[] SYSTEM_INITIAL = {"INITIAL TIME", "INITIALTIME", "INTIAL_TIME"};
//		private  static final String[] SYSTEM_FINAL = {"FINAL TIME", "FINALTIME", "FINAL_TIME"};
//		private  static final String[] SYSTEM_TIMESTEP = {"TIME STEP", "TIMESTEP", "TIME_STEP"};
//		private  static final String[] NAREPLACEMENT = {"NAREPLACEMENT"};
//		private  static final String[] SAVEPER = {"SAVEPER"};
		return SYSTEM_VARIABLES.contains(var.toUpperCase());
		
	}

}
