package gov.anl.gss.sdm.server.model.translator;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;

public class Node {
static Logger log = Logger.getLogger(Node.class.getSimpleName());
    
    private Node parent = null;
    private Node child = null;
    private Node next = null;
    private Node previous = null;
    private String token;
    
    private boolean placeHolder = false;
    private boolean deleted = false;
    
    private String resultsVariable = null;
    private StringBuilder generatedCodeHead = new StringBuilder();
    private StringBuilder generatedCodeTail = new StringBuilder();
    private StringBuilder generatedCodeElse = new StringBuilder();
    
    private String replacedToken = "@";
    
    private boolean initialized = false;
    
    public Node(Node anotherNode) {
    	// make an exact copy;
    	this.parent = anotherNode.parent;
    	this.child = anotherNode.child;
    	this.next = anotherNode.next;
    	this.previous = anotherNode.previous;
    	this.token = new String(anotherNode.token);
    }
    
    public Node(String token) {
	this.token = token;
    }
    

    public Node getParent() {
        return parent;
    }

    public void setParent(Node parent) {
        this.parent = parent;
    }

    public Node getChild() {
        return child;
    }

    public void setChild(Node child) {
        this.child = child;
    }

    public boolean isInitialized() {
        return initialized;
    }

    public void setInitialized(boolean initialized) {
        this.initialized = initialized;
    }

    public Node getNext() {
        return next;
    }

    public void setNext(Node next) {
        this.next = next;
    }

    public Node getPrevious() {
        return previous;
    }

    public void setPrevious(Node previous) {
        this.previous = previous;
    }

    public String getToken() {
        return token;
    }
    
    public String getUnits() {
        return InformationManagers.getInstance().getUnitsManager().getUnits(token);
    }

    public void setToken(String token) {
        this.token = token;
    }

    public String getInfo() {
    	StringBuilder sb = new StringBuilder();
    	String spacing = "      ";
    	sb.append(spacing+"token: "+token+"\n");
    	sb.append(spacing+"units: "+getUnits()+"\n");
    	sb.append(spacing+"placeHolder: "+(placeHolder ? "True" : "False")+"\n");
    	sb.append(spacing+"results variable: "+resultsVariable+"\n");
    	sb.append(spacing+"head: "+generatedCodeHead+"\n");
    	sb.append(spacing+"else: "+generatedCodeElse+"\n");
    	sb.append(spacing+"tail: "+generatedCodeTail+"\n");
    	sb.append(spacing+"parent: "+parent+"\n");
    	sb.append(spacing+"child: "+child+"\n");
    	sb.append(spacing+"next: "+next+"\n");
    	sb.append(spacing+"previous: "+previous+"\n");

    	return sb.toString(); // replacedToken;
    }
    
    public void print(String title) {
    	log.debug("### "+title+" ###");
    	printLink("This", this);
    	printLink("Parent", getParent());
    	printLink("Child", getChild());
    	printLink("Next", getNext());
    	printLink("Previous", getPrevious());
    }
    
    public void printLink(String tag, Node node) {
    	
    	log.debug(tag+" "+(node == null ? "NULL" : node.getToken()));
    }
    
    public String getInfo(String blanks) {
	StringBuilder sb = new StringBuilder();
	
	sb.append(blanks+"token: "+token+"\n");
	sb.append(blanks+"units: "+InformationManagers.getInstance().getUnitsManager().getUnits(token)+"\n");
	sb.append(blanks+"placeHolder: "+(placeHolder ? "True" : "False")+"\n");
	sb.append(blanks+"results variable: "+resultsVariable+"\n");
	sb.append(blanks+"head: "+generatedCodeHead+"\n");
	sb.append(blanks+"else: "+generatedCodeElse+"\n");
	sb.append(blanks+"tail: "+generatedCodeTail+"\n");
        return sb.toString(); // replacedToken;
    }
    
    public String getInfoTerse(String blanks) {
	StringBuilder sb = new StringBuilder();
	
	sb.append(blanks+"token: "+token+"\n");
	sb.append(blanks+"units: "+InformationManagers.getInstance().getUnitsManager().getUnits(token)+"\n");
        return sb.toString(); // replacedToken;
    }
    
    public void setDoNotPrune(boolean value) {
    	placeHolder = value;
    	if (!value)
    		return;
    	Node p = getParent();
    	if (p != null)
    		p.setDoNotPrune(value);
    }
    
    public boolean isDoNotPrune() {
    	return placeHolder;
    }

    public void setReplacedToken(String replacedToken) {
        this.replacedToken = replacedToken;
    }

    public StringBuilder getGeneratedCodeHead() {
        return generatedCodeHead;
    }

    public void setGeneratedCodeHead(StringBuilder generatedCode) {
        this.generatedCodeHead = generatedCode;
    }

    public StringBuilder getGeneratedCodeTail() {
        return generatedCodeTail;
    }

    public void setGeneratedCodeTail(StringBuilder generatedCodeTail) {
        this.generatedCodeTail = generatedCodeTail;
    }

    public String getResultsVariable() {
        return resultsVariable;
    }

    public void setResultsVariable(String resultsVariable) {
        this.resultsVariable = resultsVariable;
    }

    public StringBuilder getGeneratedCodeElse() {
        return generatedCodeElse;
    }

    public void setGeneratedCodeElse(StringBuilder generatedCodeElse) {
        this.generatedCodeElse = generatedCodeElse;
    }

    public boolean isPlaceHolder() {
        return placeHolder;
    }

    public void setPlaceHolder(boolean placeHolder) {
        this.placeHolder = placeHolder;
    }

    public boolean isDeleted() {
        return deleted;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }
    
    public boolean hasChild() {
    	return child != null;
    }
    
    public boolean hasParent() {
    	return parent != null;
    }
    
    public boolean hasNext() {
    	return next != null;
    }
    
    public boolean hasPrevious() {
    	return previous != null;
    }
    
    public boolean isTerminal() {
	if (child == null)
	    return true;
	else
	    return false;
    }
    
    public String toString() {
    	return token;
    }
    
    public static String expand(Node node) {
    	StringBuilder sb = new StringBuilder();
    	expand(node, sb);
    	return sb.toString();
    }
	
	private static void expand(Node node, StringBuilder sb) {
	    if (node == null)
	    	return;
	    expand(node.getChild(), sb);
	    String vensimVar = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(node.getToken());
	    sb.append(vensimVar);
	    expand(node.getNext(), sb);
	}
	
	public static boolean addParens = true;
	
	public static String lparen(Node node) {
		return node.getChild() != null && addParens ? "(" : "";
	}
	
	public static String rparen(Node node) {
		return node.getChild() != null  && addParens ? ")" : "";
	}
	
	public static String generateExpression(Node node) {
		StringBuilder sb = new StringBuilder();
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		FunctionManager fm = InformationManagers.getInstance().getFunctionManager();

		if (node != null) {

			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child
			
			if (Parser.isFunctionInvocation(node)) {
				// we need to get back to original Function name
				FunctionDescription fd = fm.getDescription(node.getToken());
				sb.append(fd.getExternalName()+"(");
				int argNum = 0;
				Node arg = TreeTraversal.getFunctionArgument(node, 1);
				while (arg != null) {
					if (argNum++ > 0)
						sb.append(",");
					sb.append(lparen(arg));
					sb.append(generateExpression(arg));
					sb.append(rparen(arg));
					arg = arg.getNext();
				}
				sb.append(")");
			} else if (Parser.isArithmeticOperator(node.getToken()) || Parser.isEqualSign(node.getToken()) 
					|| Parser.isRelationalOperator(node.getToken())|| Parser.isLogicalOperator(node.getToken())) {
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(lparen(TreeTraversal.getLhs(node)));
					sb.append(generateExpression(TreeTraversal.getLhs(node)));
					sb.append(rparen(TreeTraversal.getLhs(node)));
					sb.append(ndtm.getOriginalNameQuotedIfNecessary(node.getToken(), true));
					sb.append(lparen(TreeTraversal.getRhs(node)));
					sb.append(generateExpression(TreeTraversal.getRhs(node)));
					sb.append(rparen(TreeTraversal.getRhs(node)));
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(lparen(node.getChild()));
					sb.append(generateExpression(node.getChild()));
					sb.append(rparen(node.getChild()));
				}
				return sb.toString();

			} else {
				sb.append(ndtm.getOriginalNameQuotedIfNecessary(node.getToken(), true));
			}

		}
		return sb.toString();
	}
	
	public String generateExpression() {
		
		Node node = this;
		StringBuilder sb = new StringBuilder();
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();

		if (node != null) {

			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child
			
			if (Parser.isArithmeticOperator(node.getToken()) || Parser.isEqualSign(node.getToken())) {
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(generateExpression(TreeTraversal.getLhs(node)));
					sb.append(ndtm.getOriginalNameQuotedIfNecessary(node.getToken()));
					sb.append(generateExpression(TreeTraversal.getRhs(node)));
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(generateExpression(node.getChild()));
				}
				return sb.toString();

			} else {
				sb.append(ndtm.getOriginalNameQuotedIfNecessary(node.getToken()));
			}

		}
		return sb.toString();
	}
	
	public static void printPaths(BufferedWriter bw, Node node) {
		List<String> al = new ArrayList<>();
		printPaths(bw, node, 0, al);
	}
	
	public static void getConnected(Node node, Map<String, Set<String>> hasPath) {
		List<String> al = new ArrayList<>();
		getConnected(node, 0, al, hasPath);
	}
	
	public static void getConnected(Node node, int level, List<String> al, Map<String, Set<String>> hasPath) {
		if (node == null) {
			return;
		}
		al.add(node.getToken());
//		System.out.println((al.size()-1)+",\""+al.get(0)+"\""+","+"\""+al.get(al.size()-1)+"\","+al+"\n");
		if (al.size() > 1) {
		String exo = al.get(0);
		
			String endo = al.get(al.size()-1);
			if (!hasPath.containsKey(endo)) {
				hasPath.put(endo, new LinkedHashSet<String>());
			}
			hasPath.get(endo).add(exo);
		}
		

		getConnected(node.getChild(), level+1, new ArrayList<String>(al), hasPath);
		getConnected(node.getNext(), level, new ArrayList<String>(al), hasPath);
	}
	
	private static void printPaths(BufferedWriter bw, Node node, int level, List<String> al) {
		if (node == null) {
			return;
		}
		al.add(node.getToken());

		if (bw != null) {
			try {
				bw.append((al.size()-1)+",\""+al.get(0)+"\""+","+"\""+al.get(al.size()-1)+"\","+al+"\n");
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		printPaths(bw, node.getChild(), level+1, new ArrayList<String>(al));
		printPaths(bw, node.getNext(), level, new ArrayList<String>(al));
	}
	
	public static void printTreeToConsole(Node node) {
		printTreeToConsole(node, 0);
	}
	
	
	private static void printTreeToConsole(Node node, int level) {
	    if (node == null)
		return;
	    String blanks="";
	    for (int i = 0; i < level; i++)
		blanks += "  ";
//	    System.out.println(blanks+"<"+level + "> "+node.getToken()+" no Prune: "+node.isDoNotPrune());
	    System.out.println(blanks+"<"+level + "> "+node.getToken());
	    printTreeToConsole(node.getChild(), level+1);
	    printTreeToConsole(node.getNext(), level);
	}
	
	public static void printTree(Node node) {
	    printTree(node, 0);
	}
	
	
	private static void printTree(Node node, int level) {
	    if (node == null)
		return;
	    String blanks="";
	    for (int i = 0; i < level; i++)
		blanks += "  ";
	    log.debug(blanks+"<"+level + "> "+node.getToken()+" no Prune: "+node.isDoNotPrune());
	    printTree(node.getChild(), level+1);
	    printTree(node.getNext(), level);
	}
	
	public static List<String> extractVariables(Node node) {
		List<String> al = new ArrayList<String>();
		extractVariables(node, 0, al);
		return al;
	}
	
	
	private static void extractVariables(Node node, int level, List<String> al) {
	    if (node == null)
	    	return;
	    String token = node.getToken();
	    
	    if (!Parser.isNumber(token) && 
	    		!Parser.isOperator(token)&& 
	    		!al.contains(token) &&
	    		!token.startsWith("\"") &&
	    		!token.startsWith("sdFunc"))
	    	al.add(token);
	    
	    extractVariables(node.getChild(), level+1, al);
	    if (level != 0)
	    	extractVariables(node.getNext(), level, al);
	}
	
	////////////////////////
	
	public static List<String> extractChildNames(Node node) {
		List<String> al = new ArrayList<String>();
		if (node != null) {
			Node aNode = node.getChild();
			while (aNode != null) {
				al.add(aNode.getToken());
				aNode = aNode.getNext();
			}
		}
		return al;
	}
	
	public static List<Node> extractChildNodes(Node node) {
		List<Node> al = new ArrayList<Node>();
		if (node != null) {
			Node aNode = node.getChild();
			while (aNode != null) {
				al.add(aNode);
				aNode = aNode.getNext();
			}
		}
		return al;
	}
	
	public static void pruneNode(Node node) {
		// remove the node from the tree structure wherever it may be

		if (node.isDoNotPrune())
			return;
		
		// if parent is null -- root node -- no can do
		if (node.getParent() == null) {
			log.debug("Cannot remove root node");
			return;
		}
		
		// is this the child of the parent?
		if (node.getParent().getChild().equals(node)) {
			Node myParent = node.getParent();
			Node myNext = node.getNext();
			myParent.setChild(myNext);
			if (myNext != null)
				myNext.setPrevious(node.getPrevious());
			node.setChild(null);
			node.setNext(null);
			node.setParent(null);
			
			return;
		} else {
			// must be in the next chain of the parent's child
			// need to worry about next and previous
			Node myPrevious = node.getPrevious();
			Node myNext = node.getNext();
			if (myPrevious != null) {
				myPrevious.setNext(myNext);
			}
			if (myNext != null) {
				myNext.setPrevious(myPrevious);
			}
			node.setChild(null);
			node.setNext(null);
			node.setParent(null);
			
			return;
		}
	}
	
	public static void replaceNode(Node oldNode, Node newNode) {
		Node oldParent = oldNode.getParent();
		Node oldPrevious = oldNode.getPrevious();
		Node oldNext = oldNode.getNext();
		
		// Parent relationships
		if (oldParent.getChild().equals(oldNode))
			oldParent.setChild(newNode);
		newNode.setParent(oldParent);
		
		// sibling relationships
		if (oldPrevious != null)
		oldPrevious.setNext(newNode);
		if (oldNext != null)
		oldNext.setPrevious(newNode);
		
		newNode.setPrevious(oldPrevious);
		newNode.setNext(oldNext);
	}
	
	public static void printAsTree(Node node) {
	    printAsTree(node, 0);
	}
	
	public static void printAsTree(Node node, int level) {
	    if (node == null)
		return;
	    String blanks="";
	    for (int i = 0; i < level; i++)
		blanks += "    ";
	    System.out.println(blanks+"Level "+level + " token "+node.getToken() + 
		    "   child: "+(node.getChild() != null ? node.getChild().getToken() : "NULL") +
		    "   next: " +(node.getNext() != null ? node.getNext().getToken() : "NULL") + 
		    "   parent: " +(node.getParent() != null ? node.getParent() : "NULL"));
	    printAsTree(node.getChild(), level+1);
	    printAsTree(node.getNext(), level);
	}
	
	public Node addChildren(Set<String> children) {
		Node childNode = null;
		Node currentNode = null;
		int count = 0;
		for (String child : children) {
			if (count++ == 0) {
				childNode = new Node(child);
				currentNode = childNode;
			} else {
				currentNode.setNext(new Node(child));
				currentNode.getNext().setPrevious(currentNode);
				currentNode = currentNode.getNext();
			}
		}
		child = childNode;
		childNode.setParent(this);
		return childNode;
	}

}
