package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.VariableListTableRow;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

public class Parser {
static Logger log = Logger.getLogger(Parser.class.getSimpleName());

	private static char[] terminators = { '+', '-', '*', '/', '(', ')', '=', ',', '^', 
		'`', '>', '<', ':', ';', '{', '}'};
	private static HashSet<Character> terminatorSet = new LinkedHashSet<Character>();
	
	private static String[] inlineLookupCharacters = {"(", ")", "[", "]" , ",", "-"};
	

	public static final String INTERNAL_UNARY_MINUS = "_";

	static {

		for (char c : terminators) {
			terminatorSet.add(c);
		}
	}
	
	public static String removeAnnotation(String s ) {
		return s.replace("memory.", "").replace("lookup.", "").replace("array.", "").replace("sdFunctions.", "");
	}
	
	public static String concatenateList(List<String> al) {
		StringBuilder sb = new StringBuilder();
		for (String s : al)
			sb.append(s);
		return sb.toString();
	}
	
	public static String quoted(String s) {
		if (isQuotedString(s)) {
			return s;
		} else {
			if (s.contains("\"")) {
				s = s.replace("\"", "\\");
			}
			return "\""+s+"\"";
		}
	}
	
	public static String getVariableName(String lhs) {
		if (ArrayReference.isArrayReference(lhs))
			return Parser.removeAllQuotes(Parser.removeAnnotation(Parser.removeAnySubscripts(lhs)));
		else
			return Parser.removeAllQuotes(lhs);
	}
	
	public static boolean isIfThenElse(String eqn) {
		boolean ite = false;
		List<String> wordtokens = new ArrayList<String>();
		List<String> tokens = tokenize(eqn.toLowerCase(), false, false, true, false, true);
		for (String t : tokens) {
			String c = t.replace("\t", "");
			String[] splt = c.split(" ");
			if (splt.length > 1) {
				for (String w : splt) {
					if (w.length() > 0)
						wordtokens.add(w);
				}
			} else {
				wordtokens.add(c);
			}
		}
		
		ite = wordtokens.contains("if") && wordtokens.contains("then") && wordtokens.contains("else");
	//  will need to work about nested if then else
	//	IF-THEN-ELSE -> .1* (IF (Policy_switch=1) THEN Progressive_discipline ELSE Status_quo)
	//  .1 * IF THEN ELSE((Policy_switch=1), Progressive_discipline, Progressive_discipline)

		return ite;
	}
	
	public static String removeAnySubscripts(String s) {
		if (ArrayReference.isArrayReference(s))
			return new ArrayReference(s).getArrayName();
		else
			return s;
	}
	
	public static boolean containsAnnotation(String s ) {
		return s.contains("memory.")|| s .contains("lookup.") || s.contains("array.")|| s.contains("sdFunctions.");
	}

	public static String translateUnaryOperator(String u) {
		if (u.equals(INTERNAL_UNARY_MINUS))
			return "-";
		else
			return u;
	}
	
	public static boolean isWithLookup(String token) {
		return token.endsWith(StringConstants.RSSD_LOOKUP);
	}

	public static boolean isOperator(String token) {
		return isArithmeticOperator(token) ||
				isLogicalOperator(token) || isRelationalOperator(token) ||
				token.equals("=");
	}

	public static boolean isArithmeticOperator(String token) {
		if (token.equals("+") ||
				token.equals("-") ||
				token.equals("*") ||
				token.equals("/") ||
				token.equals("_") ||
				token.equals("^") 
				)
			return true;
		else
			return false;
	}

	public static boolean isBinaryOperator(String token) {
		if (!token.equals("_") && !token.equals("!") && !token.equals(":NOT:"))
			return true;
		else
			return false;
	}

	public static boolean isEqualSign(String token) {
		return token.equals("=");
	}

	public static boolean isUnaryOperator(String token) {
		return ! isBinaryOperator(token);
	}

	public static boolean isBooleanOperator(String token) {
		return isLogicalOperator(token) || isRelationalOperator(token);
	}
	public static boolean isLogicalOperator(String token) {
		if (
				token.equals(":OR:") ||
				token.equals(":NOT:") ||
				token.equals(":AND:") ||
				token.equals("||") ||
				token.equals("!") ||
				token.equals("&&")
				)
			return true;
		else
			return false;
	}
	public static boolean isRelationalOperator(String token) {
		if (token.equals("==") ||
				token.equals("<>") ||
				token.equals(">") ||
				token.equals(">=") ||
				token.equals("<") ||
				token.equals("<=")
				)
			return true;
		else
			return false;
	}

	public static String getFunctionName(String functionInvocation) {
		return functionInvocation.replace("sdFunctions.", "");
	}
	
	public static String getFunctionName(Node n) {
		return getFunctionName(n.getToken());
	}

	public static boolean isFunctionInvocation(String token) {
		if (token == null)
			return false;
		if (token.startsWith("sdFunction"))
			return true;
		else
			return false;
	}
	
	public static boolean isFunctionInvocation(Node n) {
		return isFunctionInvocation(n.getToken());
	}
	


	public static boolean isFunctionArgumentSeparator(String token) {
		if (token.equals(","))
			return true;
		else
			return false;
	}
	
	public static boolean isLeftSquareBracket(String token) {
		if (token.equals("["))
			return true;
		else
			return false;
	}

	public static boolean isRightSquareBracket(String token) {
		if (token.equals("]"))
			return true;
		else
			return false;
	}

	public static boolean isLeftParen(String token) {
		if (token.equals("("))
			return true;
		else
			return false;
	}

	public static boolean isRightParen(String token) {
		if (token.equals(")"))
			return true;
		else
			return false;
	}
	
	public static boolean isVerticalBar(String token) {
		if (token.equals("|"))
			return true;
		else
			return false;
	}
	
	public static boolean isPercent(String token) {
		if (token.equals("%"))
			return true;
		else
			return false;
	}
	
	public static boolean isLong( String input )
	{
		try
		{
			Long.parseLong( input );
			return true;
		}
		catch( Exception e)
		{
			return false;
		}
	}
	
	public static int toIntegerWholeNumber ( String input ) {
		
		int whole;
		
		try {
			whole = Integer.parseInt(input);
			return whole;
		} catch (NumberFormatException e) {
			try {
				whole = (int) Double.parseDouble(input);
				return whole;
			} catch (NumberFormatException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
				return Integer.MAX_VALUE;
			}
		}
		
	}

	public static boolean isInteger( String input )
	{
		
		if (input == null)
			return false;
		// really want to know if this is a whole number
		try
		{
			Integer.parseInt( input );
			return true;
		}
		catch( Exception e)
		{
			try {
				double d = Double.parseDouble(input);
				int i = (int) d;
				if ((double) i == d)
				return true;
			} catch (NumberFormatException e1) {
				return false;
			}
			return false;
		}
	}
	
	public static boolean matchesVensimVariablePattern(String v) {
		if (isQuotedString(v))
			return true;
		if (isWord(v))
			return true;
		return false;
	}
	
	public static boolean matchesInlineLookup(String s) {
		
		boolean match = true;
		List<String> tokens = tokenize(s);
		if (tokens.size() == 0)
			return false;
		if (tokens.size() == 1)
			return false;
		if (tokens.contains("="))
			return false;
		if (!matchesVensimVariablePattern(tokens.get(0)))
			return false;
		if (!tokens.get(1).equals("("))
			return false;
		match = matchesLookupDataSpecification(tokens, 2);
		return match;
	}
	
	public static boolean matchesLookupDataSpecification(List<String> tokens, int startPos) {
		// inlineLookupCharacters
		
		for (int i = startPos; i < tokens.size(); i++) {
			String token = tokens.get(i);
			if (!isNumber(token) && !isInlineLookupCharacter(token)) {
				return false;
			}
		}
		return true;
	}
	
	public static boolean isInlineLookupCharacter(String token) {
		for (int i = 0; i < inlineLookupCharacters.length; i++) {
			if (token.equals(inlineLookupCharacters[i]))
				return true;
		}
		return false;
	}

	public static boolean isReal( String input )
	{
		if (input == null)
			return false;
		if (input.equals(":NA:"))
			return true;
		try
		{
			Double.parseDouble( input );
			return true;
		}
		catch( Exception e)
		{
			return false;
		}
	}
	
	public static boolean isStaticReal( String input )
	{
		if (input == null)
			return false;
		if (input.equals("Double.MIN_VALUE"))
			return true;
		try
		{
			Double.parseDouble( input );
			return true;
		}
		catch( Exception e)
		{
			return false;
		}
	}

	public static boolean isNumber(String token) {
		if (isInteger(token) || isReal(token) || isStaticReal(token))
			return true;
		else
			return false;
	}

	public static boolean isWord(String expression) {

		MutableInteger ptr = new MutableInteger(0);

		while (inRange(expression, ptr)) {
			String aChar = characterAt(expression, ptr);

			// just added "-" for hyphenated words 5/22/2014

			if (!Character.isLetterOrDigit(aChar.charAt(0)) &&  !Character.isWhitespace(aChar.charAt(0)) &&
					!aChar.equals("_") && !aChar.equals(",") && !aChar.equals("'") && !aChar.equals("-") && !aChar.equals("+")
					&& !aChar.equals(":")&& !aChar.equals("(")&& !aChar.equals(")") && !aChar.equals("&") && !aChar.equals(".")
					&& !aChar.equals(">")&& !aChar.equals("<")&& !aChar.equals("%")
					&& !aChar.equals("/")) {
				return false;
			}

			ptr.add(1);

		}


		return true;
	}
	
	public static  boolean isVariableReference(String token) {
		if (token.startsWith("\"")
				|| token.startsWith("memory.")
				|| token.startsWith("array."))
			return true;
		else
			return false;
	}

	public static boolean isQuotedString(String s) {
		return (s.startsWith("\"") && s.endsWith("\"")) ||
				(s.startsWith("'") && s.endsWith("'"));
	}
	
	public static boolean isTimeVariable(String s) {
		return s.equalsIgnoreCase("time");
	}
	
	public static String removeAllQuotes(String s) {
	
			return s.replace("\"", "");
	}
	
	public static String removeQuotes(String s) {
		if (!isQuotedString(s))
			return s;
		else
			return s.replaceFirst("\"", "").replaceAll("\"$", "").replace("\\\"", "\"");
	}
	
	public static boolean isNaReplacement(String s) {
		if (s == null)
			return false;
		return s.equalsIgnoreCase("nareplacement");
	}

	public static boolean isSystemVariable(String s) {	
		return s.equalsIgnoreCase("time") || s.equalsIgnoreCase("timestep") || s.equalsIgnoreCase("saveper") ||
				s.equalsIgnoreCase("start time") || s.equalsIgnoreCase("final time")  ||
				s.equalsIgnoreCase("initial time") || s.equalsIgnoreCase("initial_time")  ||
				s.equalsIgnoreCase("time step") || s.equalsIgnoreCase("time_step")  ||
				s.equalsIgnoreCase("start_time") || s.equalsIgnoreCase("final_time") ||
				s.equalsIgnoreCase("NAREPLACEMENT") || isCloudVariable(s);
	}
	public static boolean isCloudVariable(String s) {	
		return s.startsWith("C_L_O_U_D");
	}
	
	public static int countSystemVariables(List<VariableListTableRow> vars) {
		int count = 0;
		for (VariableListTableRow s : vars)
			if (isSystemVariable(s.getVariable()))
				count++;
		
		return count;
	}
	
	public static int countNonSystemVariables(List<VariableListTableRow> vars) {
		int count = 0;
		for (VariableListTableRow s : vars)
			if (!isSystemVariable(s.getVariable()))
				count++;
		
		return count;
	}
	
	public static String externalDataMethod() {
//		if (StandaloneGUI.REPAST_SIMPHONY)
//			return "EXTERNAL_DATA";
//		else
			return "EXTERNAL_DATA";
	}


	public static String forceDouble(String expression) {
		List<String> tokens = new ArrayList<String>();
		MutableInteger ptr = new MutableInteger(0);

		// Must not change quoted strings

		boolean inQuote = false;

		if (expression.contains(externalDataMethod()))
			return expression;

		while (inRange(expression, ptr)) {
			String aChar = characterAt(expression, ptr);
			if (aChar.equals("\""))
				inQuote = !inQuote;
			if (!inQuote && startMemoryReference(expression, ptr)) {
				String memoryRef = getMemoryReferenceStartingAt(expression, ptr);
				tokens.add(memoryRef);
			} else {

				if (!inQuote && (Character.isDigit(aChar.charAt(0)) || aChar.equals("."))) {
					String number = getNumberStartingAt(expression, ptr);
					if (!number.contains("."))
						number += ".0";
					tokens.add(number);
				} else {
					tokens.add(aChar);
					ptr.add(1);
				}
			}

		}

		String forced = "";
		for (String s : tokens) {
			forced += s;
		}

		return forced;
	}

	private static boolean startMemoryReference(String expression, MutableInteger ptr) {
		if (characterAt(ptr.value(), expression).equals("m") &&
				characterAt(ptr.value()+1, expression).equals("e") &&
				characterAt(ptr.value()+2, expression).equals("m") &&
				characterAt(ptr.value()+3, expression).equals("o") &&
				characterAt(ptr.value()+4, expression).equals("r") &&
				characterAt(ptr.value()+5, expression).equals("y") &&
				characterAt(ptr.value()+6, expression).equals(".") )
			return true;
		else
			return false;
	}

	private static String getMemoryReferenceStartingAt(String equation, MutableInteger position) {
		String memoryReference = "";

		// gather characters until the , and return trim()
		while (inRange(equation, position) && !characterAt(equation, position).equals(",")) {
			memoryReference += characterAt(equation, position);
			position.add(1);
		}

		return memoryReference;
	}

	public static String characterAt(String equation, MutableInteger position) {
		
		try {
			return equation.substring(position.value(), position.value()+1);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}

	private static String getNumberStartingAt(String equation, MutableInteger position) {
		String number = "";

		// unary minus?

		if (characterAt(equation, position).equals("-")) {
			number="-";
			position.add(1);
		}
		while (inRange(equation, position) && (Character.isDigit(characterAt(equation, position).charAt(0)) ||
				characterAt(equation, position).equals("."))) {
			number += characterAt(equation, position);
			position.add(1);
		}

		return number;
	}

	public static boolean inRange(String equation, MutableInteger position) {
		if (position.value() <= equation.length()-1)
			return true;
		else
			return false;
	}

	public static  int getRowFromCellAddress(String cellAddress) {
		int row = 1;
		String rc = cellAddress;
		if (cellAddress.contains("!")) {
			rc = cellAddress.split("!")[1];
		}
		int rowPos = getFirstNumberPosition(rc);
		row = Integer.parseInt(rc.substring(rowPos));
		return row;
	}

	public static  String getColumnLettersFromCellAddress(String cellAddress) {
		int column =  1;
		String rc = cellAddress;
		if (cellAddress.contains("!")) {
			rc = cellAddress.split("!")[1];
		}
		int rowPos = getFirstNumberPosition(rc);
		return rc.substring(0, rowPos);
	}

	public static  int getColumnFromCellAddress(String cellAddress) {
		int column =  1;
		String rc = cellAddress;
		if (cellAddress.contains("!")) {
			rc = cellAddress.split("!")[1];
		}
		int rowPos = getFirstNumberPosition(rc);
		column = convertColumnToNumber(rc.substring(0, rowPos));

		return column;
	}

	public static  int convertColumnToNumber(String column) {
		int num = 0;
		int pos = 0;

		char[] chars = column.toCharArray();
		for (int i = chars.length-1; i >=0; i--) {
			num += (Character.getNumericValue(chars[i]) - 9) * Math.pow(26, pos++);
		}
		return num;
	}

	private static int getFirstNumberPosition(String address) {
		MutableInteger pos = new MutableInteger(0);
		while (inRange(address, pos) && Character.isLetter(characterAt(address, pos).toCharArray()[0]))
			pos.add(1);
		return pos.value();
	}

	public static boolean containsNonquotedColon(String line) {
		boolean colon = false;
		boolean inQuote = false;
		MutableInteger position = new MutableInteger(0);
		while (inRange(line, position)) {
			if (characterAt(line, position).equals("\"")) 
				inQuote = !inQuote;
			if (characterAt(line, position).equals(":") && !colon) 
				colon = !inQuote;
			position.add(1);
		}

		return colon;
	}
	
	public static String getRestOfStringStartingAt(MutableInteger position, String equation) {
		StringBuilder string = new StringBuilder();
		while (inRange(position, equation)) {
			string.append(characterAt(position, equation));
			position.add(1);
		}

			return string.toString().trim();
		}


	public static String getStringStartingAt(MutableInteger position, String equation) {
		String string = "";

		if (inRange(position, equation) && characterAt(position, equation).equals("\"")) {
			return getQuotedStringStartingAt(position, equation);
		} else {
			while (inRange(position, equation) && (Character.isLetter(characterAt(position, equation).charAt(0)) ||
					Character.isDigit(characterAt(position, equation).charAt(0)) || 
					characterAt(position, equation).equals(" ") || 
					characterAt(position, equation).equals("\"") ||
					characterAt(position, equation).equals("!"))) {
				string += characterAt(position, equation);
				position.add(1);
			}

			return string.trim();
		}
	}
	
	public static String getNumberWithOptionalRangeStartingAt(MutableInteger position, String equation) {
		String number = "";

		// unary minus?

		if (characterAt(position, equation).equals("-")) {
			number="-";
			position.add(1);
		}
		while (inRange(position, equation) && (Character.isDigit(characterAt(position, equation).charAt(0)) ||
				characterAt(position, equation).equals(":") ||
				characterAt(position, equation).equals("."))) {
			number += characterAt(position, equation);
			position.add(1);
		}

		return number;
	}

	public static String getNumberStartingAt(MutableInteger position, String equation) {
		String number = "";

		// unary minus?

		if (characterAt(position, equation).equals("-")) {
			number="-";
			position.add(1);
		}
		while (inRange(position, equation) && (Character.isDigit(characterAt(position, equation).charAt(0)) ||
				characterAt(position, equation).equals("."))) {
			number += characterAt(position, equation);
			position.add(1);
		}

		return number;
	}
	
	public static boolean isEquivalentString(String s1, String s2) {
		String s1nb = s1.replace(" ", "");
		String s2nb = s2.replace(" ", "");
		return s1nb.equalsIgnoreCase(s2nb);
	}
	
	public static void skipToDelimiter(MutableInteger position, String equation, String delimitor) {
		// gobble up spaces too! (for functions)
		while (inRange(position, equation) && !characterAt(position, equation).equals(delimitor)) {
			position.add(1);
			
		}
		
	}
	
	public static void skipToEndDelimiter(MutableInteger position, String equation, String delimitor) {
		// gobble up spaces too! (for functions)
		while (inRange(position, equation) && !characterAt(position, equation).equals(delimitor)) {
			position.add(1);
			if (inRange(position, equation)) {
					String theChar = characterAt(position, equation);
			}
		}
		// point past comma or end of line
//			position.add(1);
	}
	
	public static String getStringStartingAtWithNonQuotedDelimitor(MutableInteger position, String equation, String[] delimitor) {
		// gobble up spaces too! (for functions)
		boolean inQuote = false;
		String returnString = "";
		while (inRange(position, equation)) {
			String charAt = characterAt(position, equation);
			//				!characterAt(position, equation).equals(delimitor)) {

			if (charAt.equals("\"")) {
				inQuote = !inQuote;
				returnString += charAt;
				position.add(1);
			} else if (isDelimitor(charAt, delimitor) && !inQuote) {
//				position.add(1);
				return returnString;
			} else {
				returnString += charAt;
				position.add(1);
			}

		}
		// point past comma or end of line
		//	position.add(1);
		return returnString.trim();
	}
	
	public static boolean isDelimitor(String charAt, String[] delimitors) {
		for (String del : delimitors) {
			if (charAt.equals(del))
				return true;
		}
		
		return false;
		
	}

	public static String getNonQuotedStringStartingAtWithDelimitor(MutableInteger position, String equation, String delimitor) {
		// gobble up spaces too! (for functions)
		String nonQuotedString = "";
		while (inRange(position, equation) && !characterAt(position, equation).equals(delimitor)) {
			nonQuotedString += characterAt(position, equation);
			position.add(1);
		}
		// point past comma or end of line
		//	position.add(1);
		return nonQuotedString.trim();
	}
	
	public static String getNonBlankStringStartingAt(MutableInteger position, String equation, boolean allowColon) {
		// gobble up spaces too! (for functions)
		String nonQuotedString = "";
		while (inRange(position, equation) && 
				/*
				 * Here we list any characters not part of the ordinary terminators that we want to consider as terminators
				 * For example, we need to stop on left and right parens
				 */
				!characterAt(position, equation).equals("[") && 
				!characterAt(position, equation).equals("]") && 
				!characterAt(position, equation).equals(" ") && 
				!characterAt(position, equation).equals("\t") && 
				(
				Character.isLetterOrDigit(characterAt(position, equation).charAt(0)) ||
				characterAt(position, equation).equals("#") ||
				characterAt(position, equation).equals(".") ||
				characterAt(position, equation).equals("_") ||
				characterAt(position, equation).equals("$") ||
				characterAt(position, equation).equals("!") || // here we allow the ! for subscript ranges normally it's handled elsewhere
				
				(allowColon && characterAt(position, equation).equals(":")) ||
				!isTerminator(characterAt(position, equation).charAt(0))/* ||
				characterAt(position, equation).equals(" ")*/)) {
			nonQuotedString += characterAt(position, equation);
			position.add(1);
		}

		return nonQuotedString.trim();
	}

	public static String getNonQuotedStringStartingAt(MutableInteger position, String equation, boolean allowColon, boolean blankIsDelimitor) {
		// gobble up spaces too! (for functions)
		String nonQuotedString = "";
		while (inRange(position, equation) && 
				/*
				 * Here we list any characters not part of the ordinary terminators that we want to consider as terminators
				 * For example, we need to stop on left and right parens
				 */
				!characterAt(position, equation).equals("[") && 
				!characterAt(position, equation).equals("]") && 
				!(characterAt(position, equation).equals(" ") && blankIsDelimitor) && 
				(
				Character.isLetterOrDigit(characterAt(position, equation).charAt(0)) ||
				characterAt(position, equation).equals("#") ||
				characterAt(position, equation).equals(".") ||
				characterAt(position, equation).equals("_") ||
				characterAt(position, equation).equals("$") ||
				characterAt(position, equation).equals("!") || // here we allow the ! for subscript ranges normally it's handled elsewhere
				
				(allowColon && characterAt(position, equation).equals(":")) ||
				!isTerminator(characterAt(position, equation).charAt(0))
				)) {
			nonQuotedString += characterAt(position, equation);
			position.add(1);
		}

		return nonQuotedString.trim();
	}

	public static String getToMatchingDelimiter(String line, MutableInteger position, String open, String close) {
		// gobble up spaces too! (for functions)
		int numOpen = 1;
		String nonQuotedString = "";
		position.add(1);
		
		while (inRange(position, line)) {
			String ch = characterAt(position, line);
			if (ch.equals(open)) {
				numOpen++;
			} else if (ch.equals(close)) {
				numOpen--;
				if (numOpen == 0) {
					// we have what we want
					// point to next position
					position.add(1);
					return nonQuotedString.trim();
				}
			}
			
			nonQuotedString += ch;
			position.add(1);
		}
		// point past comma or end of line
		//	position.add(1);
		return nonQuotedString.trim();
	}

	public static String getQuotedStringStartingAt(MutableInteger position, String equation) {
		String quoteChar = characterAt(position, equation);
		String quotedString = ""; // "\"";
		position.add(1);
		boolean bslash = false;
		while (inRange(position, equation)) {
			String charAt = characterAt(position, equation);

			if (bslash) {
				bslash = false;
				quotedString += charAt;
			} else {
				
				if (charAt.equals("\\")) {
					bslash = true;
					quotedString += charAt;
				} else if (charAt.equals(quoteChar)) { //	else if (charAt.equals("\""))
					break;	
				} else {
					quotedString += charAt;
				}
			}
			position.add(1);
		}

		//		while (inRange(position, equation) && !characterAt(position, equation).equals(quoteChar)) {
		//			quotedString += characterAt(position, equation);
		//			position.add(1);
		//		}


		//	    quotedString += "\"";
		position.add(1);

		return quotedString;
	}
	
	public static String getQuotedStringStartingAt(MutableInteger position, String equation, boolean maintainQuote) {
		String quoteChar = characterAt(position, equation);
		String quotedString = ""; // "\"";
		position.add(1);
		boolean bslash = false;
		while (inRange(position, equation)) {
			String charAt = characterAt(position, equation);

			if (bslash) {
				bslash = false;
				quotedString += charAt;
			} else {
				
				if (charAt.equals("\\")) {
					bslash = true;
					quotedString += charAt;
				} else if (charAt.equals(quoteChar)) { //	else if (charAt.equals("\""))
					break;	
				} else {
					quotedString += charAt;
				}
			}
			position.add(1);
		}

		//		while (inRange(position, equation) && !characterAt(position, equation).equals(quoteChar)) {
		//			quotedString += characterAt(position, equation);
		//			position.add(1);
		//		}


		//	    quotedString += "\"";
		position.add(1);

		if (maintainQuote)
			return "\""+quotedString+"\"";
		return quotedString;
	}

	public static String characterAt(int position, String equation) {
		if (position > equation.length()-1)
			return null;
		String s = equation.substring(position, position+1);
		return equation.substring(position, position+1);
	}

	public static String characterAt(MutableInteger position, String equation) {
		String s =  equation.substring(position.value(), position.value()+1);
		return equation.substring(position.value(), position.value()+1);
	}

	public static String beyondPosition(MutableInteger position, String equation) {
		return equation.substring(position.value()+1);
	}

	public static void skipWhiteSpace(MutableInteger position, String equation, String delimiter) {
		while (inRange(position, equation) && (!characterAt(position, equation).equals(delimiter) && Character.isWhitespace(characterAt(position, equation).charAt(0))))
			position.add(1);
	}
	
	public static void skipWhiteSpace(MutableInteger position, String equation) {
		while (inRange(position, equation) && Character.isWhitespace(characterAt(position, equation).charAt(0)))
			position.add(1);
	}
	
	public static void skipWhiteSpaceNL(MutableInteger position, String equation) {
		while (inRange(position, equation) && Character.isWhitespace(characterAt(position, equation).charAt(0)) && !characterAt(position, equation).equals("\n"))
			position.add(1);
	}
	
	public static void skipWhiteSpaceNLTAB(MutableInteger position, String equation) {
		while (inRange(position, equation) && Character.isWhitespace(characterAt(position, equation).charAt(0)) && !characterAt(position, equation).equals("\n")
				&& !characterAt(position, equation).equals("\t"))
			position.add(1);
	}
	
	public static String grabWhiteSpace(MutableInteger position, String equation) {
		StringBuilder ws = new StringBuilder();
		while (inRange(position, equation) && Character.isWhitespace(characterAt(position, equation).charAt(0))) {
			ws.append(characterAt(position, equation));
			position.add(1);
		}
		
		return ws.toString();
	}

	public static boolean inRange(MutableInteger position, String equation) {
		if (position.value() <= equation.length()-1)
			return true;
		else
			return false;
	}

	public static String[] splitQuoted(String record, String delimitor) {
		MutableInteger position = new MutableInteger(0);
		List<String> tokens = new ArrayList<String>();

		// break into tokens comma separated, but allow quoted strings
		// with commans
		String token = "";

		skipWhiteSpace(position, record, delimitor);
		boolean done = false;
		while(!done) {

			// get to the beginning of the next token, making sure that
			// we are still in range
			skipWhiteSpace(position, record, delimitor);
			if (!inRange(position, record))
				break;

			// get the first character of the token (note that it can be the delimiter ","
			String theChar = characterAt(position, record);
			if (characterAt(position, record).equals(delimitor)) {
				tokens.add("");
				position.add(1);
				// if we hit a quote, find it's corresponding close quote
			} else if (characterAt(position, record).equals("\"") || characterAt(position, record).equals("'")) {
				token = getQuotedStringStartingAt(position, record, true);   // maintain quote
				tokens.add(token);
				skipWhiteSpace(position, record, delimitor);

				if (inRange(position, record) && !characterAt(position, record).equals(delimitor))
					log.debug("Not Delimiter(1)! "+characterAt(position, record));

				position.add(1);
			} else {
				token = getNonQuotedStringStartingAtWithDelimitor(position, record, delimitor);
				tokens.add(token);
				skipWhiteSpace(position, record, delimitor);

				if (inRange(position, record) && !characterAt(position, record).equals(delimitor))
					log.debug("Not Delimiter(2)! "+characterAt(position, record));

				position.add(1);
			}

		}

		String[] split = new String[tokens.size()];
		int i = 0;
		for (String s : tokens)
			split[i++] = s;
		return split;
	}
	
	public static boolean startsWithNumber(String s) {
		if (Character.isDigit(characterAt(new MutableInteger(0), s).charAt(0)))
			return true;
		else
			return false;
	}
	
	private static String replaceExponentialNotation(String s, Map<String, String> exp) {
		String noExp = new String(s);
		int numExp = 0;
		

		String notation = "[-]?[0-9]?\\.?[0-9]+[Ee][-+]?[0-9]+";
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(s);
		while (m.find()) {
			String not = m.group();
			
				String marker = "EXP_PLACEHOLDER_"+(numExp++);
				noExp = noExp.replaceFirst(not.replace("+", "\\+"), marker);
				exp.put(marker, not);
			
		}

		return noExp;
	}

	
	
	private static List<String> restoreExponentialNotation(List<String> s, Map<String, String> exp) {
		List<String> al = new ArrayList<String>();
		for (String t : s) {
			if (exp.containsKey(t)) {
				al.add(exp.get(t));
			} else {
				al.add(t);
			}
		}
		
		return al;
	}
	
	public static List<String> tokenize(String equation) {
		// public static List<String> tokenize(String equationt, boolean convertUnaryMinus, boolean maintainQuotes, boolean braceIsComment, boolean combineUnaryMinus) {

		return tokenize(equation, false, false, true, false, false);
	}
	
	public static List<String> tokenizeMaintainSingleQuotes(String equation) {
		return tokenizeMaintainSingleQuotes(equation, false, true);
	}
	
	public static List<String> tokenizeMaintainQuotes(String equation, boolean braceIsComment) {
		return tokenize(equation, true, true, braceIsComment, true, false);
	}
	
	public static List<String> tokenizeMaintainQuotes(String equation) {
		return tokenize(equation, false, true, true, false, false);
	}
	
	public static List<String> tokenize(String equationt, boolean convertUnaryMinus) {
		return tokenize(equationt, convertUnaryMinus, false, true, false, false);
	}
	public static List<String> tokenize(String equationt, boolean convertUnaryMinus, boolean maintainQuotes, 
			boolean braceIsComment, boolean combineUnaryMinus, boolean blankIsDelimiter) {

		Map<String, String> exp = new HashMap<String, String>();
		String equation = replaceExponentialNotation(equationt, exp);

		MutableInteger position = new MutableInteger(0);
		List<String> tokens = new ArrayList<String>();

		// break into tokens and determine as much as possible
		// about the equation as we pass through it.
		String token = "";
		String unaryMinus = "";
		
		int lastPosition = -1;

		skipWhiteSpace(position, equation);
		boolean done = false;
		while(!done) {
			skipWhiteSpace(position, equation);
			if (!inRange(position, equation))
				break;
			if (position.value() == lastPosition) {
				System.out.println("Parser: tokenize() stuck at position "+lastPosition);
				System.out.println(equation);
				String theChar = characterAt(position, equation);
				System.out.println(theChar);
				
//				Equation x = null;
//				if (x.equals("NULL"))
//					System.out.println("Terminate");
				
				break;
			}
			
			lastPosition = position.value();

			String theChar = characterAt(position, equation);
			if (braceIsComment && theChar.equals("{")) {
				skipToEndDelimiter(position, equation, "}");
				position.add(1);
			}
			
			skipWhiteSpace(position, equation);
			if (!inRange(position, equation))
				break;

			if (characterAt(position, equation).equals("\"")) {

				String quote = characterAt(position, equation);
				token = getQuotedStringStartingAt(position, equation);
				if (maintainQuotes) {
					token = quote+token+quote;
				}
				
			} else if (characterAt(position, equation).equals("'")) {

					String quote = characterAt(position, equation);
					token = getQuotedStringStartingAt(position, equation);
//					if (maintainQuotes) {
						token = quote+token+quote;
//					}


			} else if (Character.isLetter(characterAt(position, equation).charAt(0)) || 
					characterAt(position, equation).equals("$")) {
				boolean allowColon = false;
				token = unaryMinus+ getNonQuotedStringStartingAt(position, equation, allowColon, blankIsDelimiter);

			} else if (Character.isDigit(characterAt(position, equation).charAt(0)) ||
					characterAt(position, equation).equals(".")) {
				token = unaryMinus+ getNumberStartingAt(position, equation);
			} else if (characterAt(position, equation).equals(":")) {
				// need to know if this is an operator or the terminating character of a subscript name
				if (startOfOperator(position, equation)) {
					token = getBooleanOperatorStartingAt(position, equation);
				} else {
					token = ":";
					position.add(1);
				}
			} else if (characterAt(position, equation).equals("|")) {
				token = "|";
				position.add(1);
			} else if (characterAt(position, equation).equals("~")) {
				token = "~";
				position.add(1);
			} else if (isTerminator(characterAt(position, equation).charAt(0))) {
				// special cases:
				// if "-" need to check if unary and attach to next token
				// if "=" need to change to "=="
				// if "<" need to check to "<>"
				// if "+" need to check if this is a unary + (previous token = "("
				if (characterAt(position, equation).equals("-")) {
					// there may not be a previous token need to check
					if (tokens.size()-1 >= 0) {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							if (convertUnaryMinus) {
								if (combineUnaryMinus) {
									position.add(1);
									if (Character.isDigit(characterAt(position, equation).charAt(0)) ||
											characterAt(position, equation).equals(".")) {
										token = "-"+ getNumberStartingAt(position, equation);
										position.add(-1);
									} else {
										position.add(-1);
									}
								} else {
									token = "_"; // unary minus
								}
							} else {
								token = "-";
							}
						} else {
							token = "-";
						}
					} else {
						if (convertUnaryMinus) {
							if (combineUnaryMinus) {
								position.add(1);
								if (Character.isDigit(characterAt(position, equation).charAt(0)) ||
										characterAt(position, equation).equals(".")) {
									token = "-"+ getNumberStartingAt(position, equation);
									position.add(-1);
								} else {
									position.add(-1);
									
								}
							} else {
								token = "_"; // unary minus
							}
						} else {
							token = "-";
						}
					}

				} else if (characterAt(position, equation).equals("+")) {
					if (tokens.size()-1 < 0) {
						token = "+";
					} else {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							token = "+"; // discard unary plus will this work?
						} else {
							token = "+";
						}
					}
				} else if (characterAt(position, equation).equals("=")) {
					token = "="; // used to be "==" with no add
				} else if (characterAt(position, equation).equals("{")) {  // here only if processing a powersim model
					token = "{"; // used to be "==" with no add
				} else if (characterAt(position, equation).equals("}")) {
					token = "}"; // used to be "==" with no add
				} else if (characterAt(position, equation).equals("<")) {
					if (characterAt(position.value()+1, equation).equals(">")) {
						token = "<>";
						position.add(1);
					} if (characterAt(position.value()+1, equation).equals("=")) {
						token = "<=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}
				}  else if (characterAt(position, equation).equals(">")) {  // look ahead for "=" i.e. != "<>"
					if (characterAt(position.value()+1, equation).equals("=")){
						token = ">=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}  
				} else {
					token = characterAt(position, equation);
				}
				position.add(1);
				//  || characterAt(position, equation).equals("!")
			
			} else if (characterAt(position, equation).equals("[") || characterAt(position, equation).equals("]")) {
				token = characterAt(position, equation);
				position.add(1);
			}
			
			// this is a look ahead
			skipWhiteSpace(position, equation);
			if (inRange(position, equation)) {
				if (characterAt(position, equation).equals("!")) {
					token += characterAt(position, equation);
					position.add(1);
				}
			}
			if (token.length() == 0) {
				log.debug("Null Token");
				log.debug(equation);
				log.debug("position: "+position.value());
				return tokens;
			} else {
				tokens.add(token);
			}

		}



		return restoreExponentialNotation(tokens, exp);
	}
	
	public static List<String> tokenizeMaintainSingleQuotes(String equationt, boolean convertUnaryMinus, boolean maintainQuotes) {

		Map<String, String> exp = new HashMap<String, String>();
		String equation = replaceExponentialNotation(equationt, exp);

		MutableInteger position = new MutableInteger(0);
		List<String> tokens = new ArrayList<String>();
		

		// break into tokens and determine as much as possible
		// about the equation as we pass through it.
		String token = "";
		String unaryMinus = "";
		
		int lastPosition = -1;

		skipWhiteSpace(position, equation);
		boolean done = false;
		while(!done) {
			skipWhiteSpace(position, equation);
			if (!inRange(position, equation))
				break;
			if (position.value() == lastPosition) {
				System.out.println("Parser: tokenizeMaintainSingleQuotes() stuck at position "+lastPosition);
				System.out.println(equation);
				String theChar = characterAt(position, equation);
				System.out.println(theChar);
				
//				Equation x = null;
//				if (x.equals("NULL"))
//					System.out.println("Terminate");
				
				break;
			}
			
			lastPosition = position.value();

			String theChar = characterAt(position, equation);
			
			if (theChar.equals("{")) {
				skipToEndDelimiter(position, equation, "}");
				position.add(1);
			}
			
			skipWhiteSpace(position, equation);
			if (!inRange(position, equation))
				break;

			if (characterAt(position, equation).equals("\"") || characterAt(position, equation).equals("'")) {

				String quote = characterAt(position, equation);
				token = getQuotedStringStartingAt(position, equation);
				if (maintainQuotes && quote.equals("'")) {
					token = quote+token+quote;
				}


			} else if (Character.isLetter(characterAt(position, equation).charAt(0)) || 
					characterAt(position, equation).equals("$")) {
				boolean allowColon = false;
				token = unaryMinus+ getNonQuotedStringStartingAt(position, equation, allowColon, false);

			} else if (Character.isDigit(characterAt(position, equation).charAt(0)) ||
					characterAt(position, equation).equals(".")) {
				token = unaryMinus+ getNumberStartingAt(position, equation);
			} else if (characterAt(position, equation).equals(":")) {
				// need to know if this is an operator or the terminating character of a subscript name
				if (startOfOperator(position, equation)) {
					token = getBooleanOperatorStartingAt(position, equation);
				} else {
					token = ":";
					position.add(1);
				}
			} else if (characterAt(position, equation).equals("|")) {
				token = "|";
				position.add(1);
			} else if (isTerminator(characterAt(position, equation).charAt(0))) {
				// special cases:
				// if "-" need to check if unary and attach to next token
				// if "=" need to change to "=="
				// if "<" need to check to "<>"
				// if "+" need to check if this is a unary + (previous token = "("
				if (characterAt(position, equation).equals("-")) {
					// there may not be a previous token need to check
					if (tokens.size()-1 >= 0) {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							if (convertUnaryMinus)
								token = "_"; // unary minus
							else
								token = "-";
						} else {
							token = "-";
						}
					} else {
						if (convertUnaryMinus)
							token = "_"; // unary minus
						else
							token = "-"; // unary minus
					}

				} else if (characterAt(position, equation).equals("+")) {
					if (tokens.size()-1 < 0) {
						token = "+";
					} else {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							token = "+"; // discard unary plus will this work?
						} else {
							token = "+";
						}
					}
				} else if (characterAt(position, equation).equals("=")) {
					token = "="; // used to be "==" with no add

				} else if (characterAt(position, equation).equals("<")) {  // look ahead for ">" i.e. != "<>"
					if (characterAt(position.value()+1, equation).equals(">")) {
						token = "<>";
						position.add(1);
					} else if (characterAt(position.value()+1, equation).equals("=")) {
						token = "<=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}
				}  else if (characterAt(position, equation).equals(">")) {  // look ahead for "=" i.e. != "<>"
					if (characterAt(position.value()+1, equation).equals("=")){
						token = ">=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}  
				} else {
					token = characterAt(position, equation);
				}
				position.add(1);
				//  || characterAt(position, equation).equals("!")
			
			} else if (characterAt(position, equation).equals("[") || characterAt(position, equation).equals("]")) {
				token = characterAt(position, equation);
				position.add(1);
			}
			
			// this is a look ahead
			skipWhiteSpace(position, equation);
			if (inRange(position, equation)) {
				if (characterAt(position, equation).equals("!")) {
					token += characterAt(position, equation);
					position.add(1);
				}
			}
			if (token.length() == 0) {
				log.debug("Null Token");
				log.debug(equation);
				log.debug("position: "+position.value());
				return tokens;
			} else {
				tokens.add(token);
			}

		}



		return restoreExponentialNotation(tokens, exp);
	}
	
	public static boolean startOfOperator(MutableInteger position, String equation) {
		MutableInteger tpos = new MutableInteger(position);
		tpos.add(1);
		String theRest = ":"+getStringStartingAt(tpos, equation).toUpperCase()+":";
		if (theRest.startsWith(":AND:") || theRest.startsWith(":OR:") || theRest.startsWith(":NOT:")) {
			return true;
		} else {
			return false;
		}
	}
	
	public static List<String> decompose(String equation, boolean isITE, boolean keepQuote) {


		MutableInteger position = new MutableInteger(0);
		List<String> tokens = new ArrayList<String>();
		boolean convertUnaryMinus = false;
		
		boolean processingSubscript = false;
		
		boolean iteEqn = isITE;

		// break into tokens where delimiters are spaces, tabs, newlines, operators,  
		// need to maintain any white space
		String token = "";
		String unaryMinus = "";
		
		int lastPosition = -1;

//		skipWhiteSpace(position, equation);
		boolean done = false;
		while(!done) {
//			skipWhiteSpace(position, equation);
			if (!inRange(position, equation))
				break;
			if (position.value() == lastPosition) {
				System.out.println("Parser: decompose() stuck at position "+lastPosition);
				System.out.println(equation);
				String theChar = characterAt(position, equation);
				System.out.println(theChar);
				
				break;
			}
			
			lastPosition = position.value();

			String theChar = characterAt(position, equation);
			
			if (theChar.equals("{")) {
				skipToEndDelimiter(position, equation, "}");
				position.add(1);
			}
			
			skipWhiteSpaceNLTAB(position, equation);
			if (!inRange(position, equation))
				break;

			if (characterAt(position, equation).equals("\"") || characterAt(position, equation).equals("'")) {
				if (keepQuote)
					token = "\""+getQuotedStringStartingAt(position, equation)+"\"";
				else
					token = getQuotedStringStartingAt(position, equation);
			} else if (characterAt(position, equation).equals("~")) {
				token = "~"; 
				position.add(1);
				iteEqn = false;
			} else if (characterAt(position, equation).equals("\n")) {
				token = "\n"; 
				position.add(1);
			} else if (characterAt(position, equation).equals("\t")) {
				token = "\t"; 
				position.add(1);
			} else if (Character.isLetter(characterAt(position, equation).charAt(0)) || 
					characterAt(position, equation).equals("$")) {
				boolean allowColon = false;
				token = unaryMinus+ getNonBlankStringStartingAt(position, equation, allowColon);
			
			} else if (characterAt(position, equation).equals("|")) {
				token = "|"; 
				position.add(1);
			} else if (Character.isDigit(characterAt(position, equation).charAt(0)) || characterAt(position, equation).equals(".")) {
				if (processingSubscript) {
					token = unaryMinus+ getNumberWithOptionalRangeStartingAt(position, equation);
				} else {
					token = unaryMinus+ getNumberStartingAt(position, equation);
				}
			} else if (characterAt(position, equation).equals(":")) {
				if (iteEqn) {
					token = getBooleanOperatorStartingAt(position, equation);
				} else {
					token = ":"; 
					position.add(1);
				}
			} else if (isTerminator(characterAt(position, equation).charAt(0))) {
				// special cases:
				// if "-" need to check if unary and attach to next token
				// if "=" need to change to "=="
				// if "<" need to check to "<>"
				// if "+" need to check if this is a unary + (previous token = "("
				if (characterAt(position, equation).equals("-")) {
					// there may not be a previous token need to check
					if (tokens.size()-1 >= 0) {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							if (convertUnaryMinus)
								token = "_"; // unary minus
							else
								token = "-";
						} else {
							token = "-";
						}
					} else {
						if (convertUnaryMinus)
							token = "_"; // unary minus
						else
							token = "-"; // unary minus
					}

				} else if (characterAt(position, equation).equals("+")) {
					if (tokens.size()-1 < 0) {
						token = "+";
					} else {
						String previousToken = tokens.get(tokens.size()-1);
						if (previousToken.equals("(") || previousToken.equals("=") || Parser.isOperator(previousToken) || 
								isFunctionArgumentSeparator(previousToken)) {
							token = "+"; // discard unary plus will this work?
						} else {
							token = "+";
						}
					}
				} else if (characterAt(position, equation).equals("=")) {
					token = "="; // used to be "==" with no add
				

				} else if (characterAt(position, equation).equals("<")) {  // look ahead for ">" i.e. != "<>"
					if (characterAt(position.value()+1, equation).equals(">")) {
						token = "<>";
						position.add(1);
					} else if (characterAt(position.value()+1, equation).equals("=")) {
						token = "<=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}
				}  else if (characterAt(position, equation).equals(">")) {  // look ahead for "=" i.e. != "<>"
					if (characterAt(position.value()+1, equation).equals("=")){
						token = ">=";
						position.add(1);
					} else {
						token = characterAt(position, equation);
					}  
				} else {
					token = characterAt(position, equation);
				}
				position.add(1);
				//  || characterAt(position, equation).equals("!")
			
			} else if (characterAt(position, equation).equals("[") || characterAt(position, equation).equals("]")) {
				if (characterAt(position, equation).equals("[")) {
					processingSubscript = true;
				} else {
					processingSubscript = false;
				}
				token = characterAt(position, equation);
				position.add(1);
			} else {
				// do not recognize this character
				// for now, assume a "letter" as part of a name
				token = unaryMinus+ getNonBlankStringStartingAt(position, equation, false);
				position.add(1);
			}
			
//			// this is a look ahead
//			skipWhiteSpace(position, equation);
//			if (inRange(position, equation)) {
//				if (characterAt(position, equation).equals("!")) {
//					token += characterAt(position, equation);
//					position.add(1);
//				}
//			}
			if (token.length() == 0) {
				log.debug("Null Token");
				log.debug(equation);
				log.debug("position: "+position.value());
				return tokens;
			} else {
				tokens.add(token+grabWhiteSpace(position, equation));
			}

		}



		return tokens;
	}

	private static String getBooleanOperatorStartingAt(MutableInteger position, String equation) {
		String booleanOperator = ":";
		position.add(1);
		while (inRange(position, equation) && !characterAt(position, equation).equals(":")) {
			booleanOperator += characterAt(position, equation);
			position.add(1);
		}
		booleanOperator += ":";
		position.add(1);

		return booleanOperator;
	}

	public static  boolean isTerminator(char c) {
		if (terminatorSet.contains(c))
			return true;
		else
			return false;
	}

	public static String extractLHS(String equation) {
		if (!containsUnquotedEqual(equation))
			return "";
		MutableInteger posEqual = getUnquotedEqualPosition(equation);
		MutableInteger pos = new MutableInteger(0);
		StringBuilder sb = new StringBuilder();
		while (pos.value() < posEqual.value()) {
			sb.append(characterAt(pos, equation));
			pos.add(1);
		}
		return sb.toString().trim();
	}

	public static String extractRHS(String equation) {
		MutableInteger pos = getUnquotedEqualPosition(equation);
		pos.add(1);
		StringBuilder sb = new StringBuilder();
		while (inRange(pos, equation)) {
			sb.append(characterAt(pos, equation));
			pos.add(1);
		}
		return sb.toString().trim();
//		return equation.split("=",2)[1].trim();
	}
	
	public static boolean containsUnquotedEqual(String equation) {
		MutableInteger position = new MutableInteger(0);
		boolean inQuote = false;
		
		while (inRange(position, equation)) {
			String theChar = characterAt(position, equation);
			if (theChar.equals("=")) {
				if (!inQuote)
					return true;
				
			} else if (theChar.equals("\"")) {
				inQuote = !inQuote;
			}
			position.add(1);
		}

		return false;
	}
	
	public static MutableInteger getUnquotedEqualPosition(String equation) {
		MutableInteger position = new MutableInteger(0);
		boolean inQuote = false;
		
		while (inRange(position, equation)) {
			String theChar = characterAt(position, equation);
			if (theChar.equals("=")) {
				if (!inQuote)
					return position;
				
			} else if (theChar.equals("\"")) {
				inQuote = !inQuote;
			}
			position.add(1);
		}

		return new MutableInteger(-1);
	}

	public static String noSubscripts(String s) {
		
		if (ArrayReference.isArrayReference(s)) {
			return new ArrayReference(s).getArrayName();
//		if (!s.contains("["))
//			return s;
		
		}

		return s;
	}
	
	public static String noSubscriptValues(String s) {
		if (!s.contains("["))
			return s;
		boolean inSubscript = false;
		String noSV = "";
		MutableInteger pos = new MutableInteger(0);
		while (inRange(pos, s)) {
			if (characterAt(pos, s).equals("[")) {
				inSubscript = true;
				noSV += "[";
				
			} else if (characterAt(pos, s).equals("]")) {
				inSubscript = false;
				noSV += "]";
			} else {
				if (!inSubscript) {
					noSV += characterAt(pos, s);
				}
			}
			pos.add(1);
		}

		return noSV;
	}


	public static List<String> extractFunctionAndArguments(String s, int funcStart) {
		List<String> arguments = new ArrayList<String>();
		
		MutableInteger pos = new MutableInteger(funcStart);
		StringBuilder sb = new StringBuilder();
		
		// first grab function
		String function = "";
		while (inRange(pos, s)) {
			String achar = characterAt(pos, s);
			if (achar.equals("(")) {
				function = sb.toString();
				break;
			} else {
				sb.append(achar);
			}
			pos.add(1);
		}
		arguments.add(function);
		// skip past paren
		pos.add(1);
		
		int openParens = 0;
		sb.setLength(0);
		// now just keep grabbing arguments
		while (inRange(pos, s)) {
			String achar = characterAt(pos, s);
			if (achar.equals("(")) {
				// a paren within argument
				openParens++;
				sb.append(achar);
			} else if (achar.equals(")")) {
				openParens--;
				if (openParens >= 0) {
					// this is a closing paren in an argument
					sb.append(achar);
				} else {
					// this is closing paren for function
					arguments.add(sb.toString());
					return arguments;
				}
			} else if (achar.equals(",")) {
				if (openParens > 0) {
					// this is a closing paren in an argument
					sb.append(achar);
				} else {
					// this is closing paren for function
					arguments.add(sb.toString());
					sb.setLength(0);

				}

			} else {
				sb.append(achar);
			}
			pos.add(1);
		}
		
		
		return arguments;
	}
	
	public static int functionStart(String s, String function, int startPos) {
		
//		String notation = "[iI][nN][tT] *\\(.*\\)";
//		Pattern p = Pattern.compile(notation);
//		Matcher m = p.matcher(s);
//	
//		while (m.find()) {
//			String not = m.group();
//			String integer = not.replaceFirst("[iI][nN][tT] *(", "INTEGER(");
//			System.out.println("INT IN: "+s);
//			s = s.replace(not, integer);
//			System.out.println("INT OUT: "+s);
//		}
		
		String sUpper = s.toUpperCase();
		String fUpper = function.toUpperCase();

		String notation = fUpper+" *\\(.*\\)";
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(sUpper);
		if (m.find())
			return m.start();

				return -1;
	}
}
