package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.structure.SDObjectManager;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class RateToRateLinkReport {
	
	private List<String[]> edges;
	private InformationManagers im;
	private SDObjectManager vensimEquationManager;
	private List<String> withEdges;
	private Map<String, List<String>> rateToRateMap = new HashMap<String, List<String>>();
	
	private int numRateToRate = 0;
	
	public RateToRateLinkReport() {
		
		im = InformationManagers.getInstance();
		vensimEquationManager = im.getSdObjectManager();
		
		if (!im.getModelManager().isMap())
			edges = vensimEquationManager.getEdges();
		else
			edges = InformationManagers.getInstance().getGraphicsProcessor().getEdges();
		process();
	}
	
    public void process() {
    	
    	List<List<String>> al = new ArrayList<List<String>>();
    	
    	List<String> allVars = new ArrayList<String>(InformationManagers.getInstance().getEquationManager().getVariables()); //  vensimEquationManager.getAllKnowVariables());
    	List<String> allFlows = new ArrayList<String>();
    	
    	for (String var : allVars) {
    		Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
    		if (eqn == null || !eqn.isFlow())
    			continue;
    		allFlows.add(Parser.removeQuotes(var));
    	}
    	
    	// grab the rate to rate edges
    	
    	for (String[] edge : edges) {
    		if (allFlows.contains(edge[0]) && allFlows.contains(edge[1])) {
    			if (!rateToRateMap.containsKey(edge[0]))
    				rateToRateMap.put(edge[0], new ArrayList<String>());
    			List<String> l = rateToRateMap.get(edge[0]);
    			if (!l.contains(edge[1]))
    				l.add(edge[1]);
    		}
    	}
    	
    	// order them
    	withEdges = new ArrayList<String>(rateToRateMap.keySet());
    	Collections.sort(withEdges);
    	for (String v : withEdges) {
    		List<String> l = rateToRateMap.get(v);
    		numRateToRate += l.size();
    		Collections.sort(l);
    	}
    }
    
    public List<List<String>> getTableRows() {
    	List<List<String>> al = new ArrayList<List<String>>();
    	for (String t : withEdges) {
    		
    		for (String h : rateToRateMap.get(t)) {
    			List<String> e = new ArrayList<String>();
    			e.add(t);
    			e.add(h);
    			al.add(e);
    		}
    	}
    	
    	return al;
    }

	public int getNumRateToRate() {
		return numRateToRate;
	}


}
