package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.xmile.XmileModel;
import gov.anl.gss.sdm.server.model.xmile.XmileReader;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class Reader {
static Logger log = Logger.getLogger(Reader.class.getSimpleName());

    private String filename;
    private String modelname;
    private boolean xmileFormat = false;
    private boolean mdlFormat = false;
    
    // for mdl file
    private List<String> mdlContents;
    
    // for XMILE
    private XmileModel xmileModel;

    public Reader() {

    }

    public Reader(String filename) {
	this();
	this.filename = filename;
	if (filename.toLowerCase().endsWith("mdl"))
		mdlFormat = true;
	else
		xmileFormat = true;
	
	modelname = new File(filename).getName().replace(".mdl", "").replace(".MDL", "").
			replace(".xmile", "").replace(".XMILE", "");
    }
    
    public void readModelFile() {
    	if (mdlFormat) {
    		readMDLFile();
    	} else {
    		XmileReader rdr = new XmileReader(filename);
    		xmileModel = rdr.loadModel();
    	}
    }

    public List<String> readMDLFile() {
    	List<String> mdlContents = new ArrayList<String>();

    	BufferedReader fileReader = null;

    	String aLine;
    	// open the file for reading
    	try {
    		FileInputStream fstream = new FileInputStream(filename);
    		DataInputStream in = new DataInputStream(fstream);
    		try {
    			fileReader = new BufferedReader(new InputStreamReader(in,"UTF-8"));
    		} catch (UnsupportedEncodingException e) {
    			// TODO Auto-generated catch block
    			e.printStackTrace();
    		}

    	} catch (FileNotFoundException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}

    	// should have {UTF-8} as first line
    	try {
    		aLine = fileReader.readLine();
    		if (!aLine.contains("{UTF-8}")) {
    			log.debug("Bad initial record: "+aLine);
    			log.debug("Will try to comtinue");
    			boolean plusTab = false;
    			if (aLine.endsWith("\t"))
    				plusTab = true;
    			mdlContents.add(aLine.trim()+(plusTab ? "\t" : ""));
    			//		return null;
    		}

    	} catch (IOException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}

    	try {
    		aLine = fileReader.readLine();
    		while (aLine != null) {
    			boolean plusTab = false;
    			if (aLine.endsWith("\t"))
    				plusTab = true;
    			//		System.out.println("aLine1: "+aLine+(plusTab ? "\t" : ""));

    			mdlContents.add(aLine.trim()+(plusTab ? "\t" : ""));
    			aLine = fileReader.readLine();
    		}
    		fileReader.close();
    	} catch (IOException e) {
    		// TODO Auto-generated catch block
    		e.printStackTrace();
    	}	

    	return mdlContents;
    }

	public String getFilename() {
		return filename;
	}

	public String getModelname() {
		return modelname;
	}

	/**
	 * @return the log
	 */
	public static Logger getLog() {
		return log;
	}

	/**
	 * @return the xmileFormat
	 */
	public boolean isXmileFormat() {
		return xmileFormat;
	}

	/**
	 * @return the mdlFormat
	 */
	public boolean isMdlFormat() {
		return mdlFormat;
	}

	/**
	 * @return the mdlContents
	 */
	public List<String> getMdlContents() {
		return mdlContents;
	}

	/**
	 * @return the xmileModel
	 */
	public XmileModel getXmileModel() {
		return xmileModel;
	}
}
