package gov.anl.gss.sdm.server.model.translator;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

//import repast.simphony.systemdynamics.sdmodel.InfluenceLink;
//import repast.simphony.systemdynamics.sdmodel.SystemModel;
//import repast.simphony.systemdynamics.sdmodel.Variable;

public class SystemDynamicsObjectManager {
static Logger log = Logger.getLogger(SystemDynamicsObjectManager.class.getSimpleName());

	private Map<String, SystemDynamicsObject> objectsMap;


	public SystemDynamicsObjectManager() {
		objectsMap = new LinkedHashMap<String, SystemDynamicsObject>();
	}



	public void processRSD() {

	}
	
	public void clear() {
		objectsMap.clear();
	}

	
//	public void populate(SystemModel model) {
//		// given an RSD formatted, populate the SDOM
//		clear();
//		
//		// Variables
//		
//		for (Variable variable : model.getVariables()) {
//			String screenName = variable.getName();
//			addSystemDynamicsObject(screenName);
////			addEquation(screenName, variable.getEquation());
//			
//		}
//		// links
//		
//		for (InfluenceLink link : model.getLinks()) {
//			
//		}
//		
//	}

	public List<EquationGraphicValidation> validate(Map<String, Equation> equations) {
		// need to check consistency between equations and graphics objects and arrows
		// the equations are always truth -- augment graphics as required
		
		
//		log.debug("######################" + "VALIDATE" + "######################");

		List<EquationGraphicValidation> validations = new ArrayList<EquationGraphicValidation>();

		for (SystemDynamicsObject sdo : objectsMap.values()) {
			EquationGraphicValidation egv = new EquationGraphicValidation(this, sdo, equations);
			egv.validate();
			if (!egv.isValid())
				validations.add(egv);
		}
//		log.debug("######################" + "VALIDATE" + "######################");
		return validations;
	}

	public void createSystemDynamicsObjectForNonGraphic(List<EquationGraphicValidation> validations, Map<String, Equation> equations) {

		// we have located items that appear in equations, but not in graphics
		// we need to create an associated SystemDynamicsObject and populate with
		// equations and arrows. Graphic Objects will need to be create else where???
		
//		log.debug("######################" + "createSystemDynamicsObjectForNonGraphic" + "######################");

		for (EquationGraphicValidation egv : validations) {
			egv.apply();
		}
//		log.debug("######################" + "createSystemDynamicsObjectForNonGraphic" + "######################");

	}

	public List<Equation> findEquationsWithScreenName(String screenNameE, Map<String, Equation> equations) {
		
			String screenName = getScreenName(screenNameE);  // make sure that we translate to screen name if passed LHS
		List<Equation> list = new ArrayList<Equation>();
		for (String lhs : equations.keySet()) {
			if (getScreenName(lhs).equals(screenName))
				list.add(equations.get(lhs));
		}
		return list;
	}

	public void print() {
		for (SystemDynamicsObject obj : objectsMap.values()) {
//			if (obj.getType().equals(View.VARIABLE) ||
//					obj.getType().equals(View.RATE) ||
//					obj.getType().equals(View.CLOUD) ||
//					obj.getType().equals(View.ARROW))
				obj.print();
		}
	}

	public static String getScreenName(String lhs) {
		// screen name from equation lhs:

		// if subscripted, remove subscripts
		// remove '"'
		
		String screenName = lhs.trim();

		if (!Parser.isQuotedString(lhs)) {

			if (lhs.contains("[")) {
				screenName = lhs.split("\\[")[0].trim();
			}
		}

		return screenName.replace("memory.", "").replace("lookup.", "").replace("array.", ""); // .replace("\"", "");  mjb 8/4/2021
	}

	public List<SystemDynamicsObject> stockVariables() {
		
		// Note: do not work off graphic objects
		// use equations to find 
		
		List<SystemDynamicsObject> stocks = new ArrayList<SystemDynamicsObject>();
		for (SystemDynamicsObject obj : objectsMap.values()) {
//			obj.print();
//			log.debug("SDO: "+obj.getScreenName()+" type: "+obj.getType());
//			if (obj.getType().equals(View.VARIABLE)) {
//				log.debug("Is VARIABLE");
				if (obj.getEquations().size() > 0) {
//					log.debug("Has Equation");
					if (obj.getEquations().get(0).isStock()) {
//						log.debug("Is Stock");
						stocks.add(obj);
					}
				}
//			}
		}

		return stocks;
	}
	
	  public Iterable<String> screenNames() {
		    return objectsMap.keySet();
		  }

	public SystemDynamicsObject getObjectWithName(String screenNameE) {
		String screenName = getScreenName(screenNameE);
		return objectsMap.get(screenName);
	}

	public String getAssociatedVariableFor(String screenNameE) {
		String screenName = getScreenName(screenNameE);
		SystemDynamicsObject sdo = objectsMap.get(screenName);
		return sdo.getAssociatedVariableName();
	}

	public void addSystemDynamicsObject(String screenNameE) {

		String screenName = getScreenName(screenNameE);
		
		if (!objectsMap.containsKey(screenName)) {
			
//			System.out.println("SDOM: Add "+screenName);
			objectsMap.put(screenName, new SystemDynamicsObject(screenName));
		}
	}

	private void addSystemDynamicsObjectPlaceHolder(String screenNameE) {
		String screenName = getScreenName(screenNameE);
//		System.out.println("SDOM: Adding placeholder for: "+screenName);
		objectsMap.put(screenName, new SystemDynamicsObject(screenName));
	}

	public void addGraphicObject(String screenNameE, GraphicObject graphicObject) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
//log.debug("add GO: "+screenName);
		objectsMap.get(screenName).addGraphicObject(graphicObject);
	}

	public void addEquation(String screenNameE, Equation equation) {
		String screenName = getScreenName(screenNameE);
//		log.debug("addEquation: "+screenName+" "+equation.getEquation());
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
		objectsMap.get(screenName).addEquation(equation);
	}

	public void addIncomingArrow(String screenNameE, Arrow incomingArrow) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
		objectsMap.get(screenName).addIncomingArrow(incomingArrow);
	}

	public void addOutgoingArrow(String screenNameE, Arrow incomingArrow) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
		objectsMap.get(screenName).addOutgoingArrow(incomingArrow);
	}

	public List<Equation> getEquations(String screenNameE) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);

		return objectsMap.get(screenName).getEquations();

	}

	public List<GraphicObject> getGraphicObjects(String screenNameE) {
		String screenName = getScreenName(screenNameE);  // make sure that we translate to screen name if passed LHS
		if (!objectsMap.containsKey(screenName)) 
			addSystemDynamicsObjectPlaceHolder(screenName);
		return objectsMap.get(screenName).getGraphicObjects();
	}

	public List<Arrow> getIncomingArrows(String screenNameE) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
		return objectsMap.get(screenName).getIncomingArrows();

	}

	public List<Arrow> getOutgoingArrows(String screenNameE) {
		String screenName = getScreenName(screenNameE);
		if (!objectsMap.containsKey(screenName))
			addSystemDynamicsObjectPlaceHolder(screenName);
		return objectsMap.get(screenName).getOutgoingArrows();
	}

	public List<String> extractStructure(View view) {
		
		List<String> variablesInView = new ArrayList<String>();

		Map<String, GraphicObject> idToGraphicObject = new LinkedHashMap<String, GraphicObject>();
		Map<String, String> idToName= new LinkedHashMap<String, String>();
		Map<String, String> nameToId = new LinkedHashMap<String, String>();
		Map<String, String> nameToType = new LinkedHashMap<String, String>();

		Map<String, List<Arrow>> inArrows = new LinkedHashMap<String, List<Arrow>> ();
		Map<String, List<Arrow>> outArrows  = new LinkedHashMap<String, List<Arrow>> ();
		
		
		
//		log.debug("### Processing View: "+view.getName());

		// map id to object
		for (GraphicObject go : view.getGraphicObjects()) {
			idToGraphicObject.put(go.getId(), go);
		}

		// first grab everything but arrows and populate data structures

		for (GraphicObject go : view.getGraphicObjects()) {
			if (go.isArrow())
				continue;

			String id = go.getId();
			String type = view.translateCodeToString(go.getType());
			String name = go.getName();
			if (id == null || type == null || name == null) {
				System.out.println("Incomplete graphic object: id: "+id+" type: "+type+" name: "+name);
			}
			
//			if (name.contains("[")) {
//				name = name.replace("[", "(").replace("]", ")");
//			}
			
			if (!go.isAutoGenerated()) {
				if (go.isVariable())
					variablesInView.add(name);

				if (go.isRate())
					variablesInView.add(name);
			}

			idToName.put(id, name);
			nameToId.put(name, id);
			nameToType.put(name, type);
		}

		// now extract arrows
		for (GraphicObject arrow : view.getGraphicObjects()) {
			if (!arrow.isArrow())
				continue;
			
			if (idToGraphicObject.get(arrow.getTo()) == null || idToGraphicObject.get(arrow.getTo()) == null) {
				System.out.println("Skipping "+arrow.toString());
				continue;
			}

			// for this arrow, get ID, type (arrow), and name (null)

			String id = arrow.getId();
			String type = view.translateCodeToString(arrow.getType());
			String name = arrow.getName();
			
			// should we even care about this? since name is null!

			idToName.put(id, name);
			nameToId.put(name, id);
			nameToType.put(name, type);

			// get the GO at the head of the arrow
			// if pointing to a valve, set GO to its associated rate
			
			// Note that Vensim stores valve arrow information  
			
			boolean swap = false;
			
			String arrowType = "Unknown";
			String direction = "Unknown";
			
			if (arrow.isInfluenceArrow()) 
				arrowType = Arrow.INFLUENCE;
			if (arrow.isFlowArrow()) 
				arrowType = Arrow.FLOW;
			
			
			// Here is where we need to worry about a valve arrow with no associated Variable
			
			GraphicObject toGraphicObject = idToGraphicObject.get(arrow.getTo());
			
			
			// if arrow is to a valve, point to rate associated with valve
			if (toGraphicObject.isValve()) {
				toGraphicObject = toGraphicObject.getAssociatedVariable();
			}
			
			if (toGraphicObject == null) {
				
			}

			// get the name at the head of the arrow
			String toName = idToName.get(toGraphicObject.getId());

			// get the GO at the tail of the arrow
			// if pointing to a valve, set GO to its associated rate

			GraphicObject fromGraphicObject = idToGraphicObject.get(arrow.getFrom());
			
			if (fromGraphicObject == null) {
				log.debug("EffectiveFromGO == null ");
				arrow.print();
				
				continue;
				
//				for (String s : idToGraphicObject.keySet()) {
//					idToGraphicObject.get(s).print();
//				}
			}
			
			if (fromGraphicObject.isValve()) {
				if (arrow.isInFlowArrow()) {
					swap = true;
					arrow.setSwapped(true);
					// this is actually a flow from stock to valve 
					// my theory is that this should be unknown polarity
					if (arrow.isUnknownPolarity()) {
//						arrow.setNegativePolarity();
					} else {
						log.debug("Unexpected Polarity: "+arrow + " "+arrow.getPolarity());
					}
					
				} else {
					if (arrow.isUnknownPolarity()) {
//						arrow.setPositivePolarity();
					} else {
						log.debug("Unexpected Polarity: "+arrow + " "+arrow.getPolarity());
					}
				}	
				
				if (fromGraphicObject.getAssociatedVariable() == null)  {
//					log.debug("HUH?");
//					go.print();
					
					// if this is no associated variable, then this must be a map and this
					// is pointing off to a cloud, skip it
					
					continue;
				}
				
				fromGraphicObject = fromGraphicObject.getAssociatedVariable();
			}
			
//			log.debug("swap "+swap);

			// get the name at the head of the arrow
			String fromName = idToName.get(fromGraphicObject.getId());
			
			
			if (swap) {
				String tmp = toName;
				toName = fromName;
				fromName = tmp;
			}

			// have lists been initialized?
			if (!inArrows.containsKey(toName))
				inArrows.put(toName, new ArrayList<Arrow>());
			if (!outArrows.containsKey(fromName))
				outArrows.put(fromName, new ArrayList<Arrow>());

			// add arrows to in and out lists as needed
			List<Arrow> in = inArrows.get(toName);
			boolean found = hasArrowWithOtherEnd( in, fromName);
			if (!found) {
				Arrow inArrow = new Arrow(fromName, Arrow.IN, arrowType, swap);
				in.add(inArrow);
//				log.debug(inArrow);
			}

			List<Arrow> out = outArrows.get(fromName);
			found = hasArrowWithOtherEnd( out, toName);
			if (!found) {
				Arrow outArrow = new Arrow(toName, Arrow.OUT, arrowType, swap);
				out.add(outArrow);
//				log.debug(outArrow);
			}
		}

		// now pass this information to the SystemDynamicsOjbectManager;
		for (GraphicObject go : view.getGraphicObjects()) {

			String id = go.getId();
			String name = idToName.get(id);
			String type = nameToType.get(name);

			// arrows are processed separately
			if (go.isArrow())
				continue;

			addSystemDynamicsObject(name);
			addGraphicObject(name, go);
		}


		// we only want arrows that are variable to variable
		// note that we can have indirect connects through valves
		// valves should be the from in these instances?
		// but handle both

		for (String to : inArrows.keySet()) {
			for (Arrow from : inArrows.get(to)) {

				SystemDynamicsObject sdTo = getObjectWithName(to);
				SystemDynamicsObject sdFrom = getObjectWithName(from.getOtherEnd());

				SystemDynamicsObject effectiveTo = sdTo;
				SystemDynamicsObject effectiveFrom = sdFrom;

				if (sdTo != null && sdFrom != null) {
					// MJB 4/10 try this 
					//		    if (sdTo.getType().equals(View.VALVE)) {
					//			effectiveTo = getObjectWithName(getAssociatedVariableFor(sdTo.getScreenName()));
					//		    }
					//		    
					//		    if (sdFrom.getType().equals(View.VALVE)) {
					//			effectiveFrom = getObjectWithName(getAssociatedVariableFor(sdFrom.getScreenName()));
					//		    }
					//		    
					if ((effectiveTo.getType().equals(View.VARIABLE) || effectiveTo.getType().equals(View.RATE) || effectiveTo.getType().equals(View.CLOUD)) && 
							(effectiveFrom.getType().equals(View.VARIABLE) || effectiveFrom.getType().equals(View.RATE) ||  effectiveFrom.getType().equals(View.CLOUD))) {
						addIncomingArrow(effectiveTo.getScreenName(), from);
					} else {
						//			log.debug("Something not right");
					}
				}
			}
		}

		for (String from : outArrows.keySet()) {
			for (Arrow to : outArrows.get(from)) {
				SystemDynamicsObject sdTo = getObjectWithName(to.getOtherEnd());
				SystemDynamicsObject sdFrom = getObjectWithName(from);

				SystemDynamicsObject effectiveTo = sdTo;
				SystemDynamicsObject effectiveFrom = sdFrom;

				if (sdTo != null && sdFrom != null) {
					// MJB 4/10 try this 
					//		    if (sdTo.getType().equals(View.VALVE)) {
					//			effectiveTo = getObjectWithName(getAssociatedVariableFor(sdTo.getScreenName()));
					//		    }
					//		    
					//		    if (sdFrom.getType().equals(View.VALVE)) {
					//			effectiveFrom = getObjectWithName(getAssociatedVariableFor(sdFrom.getScreenName()));
					//		    }

					if ((effectiveTo.getType().equals(View.VARIABLE) || effectiveTo.getType().equals(View.RATE) || effectiveTo.getType().equals(View.CLOUD)) && 
							(effectiveFrom.getType().equals(View.VARIABLE) || effectiveFrom.getType().equals(View.RATE) ||  effectiveFrom.getType().equals(View.CLOUD))) {
						addOutgoingArrow(effectiveFrom.getScreenName(),to);
//						log.debug("addOut: "+to);
					} else {
						//			log.debug("Something not right");
					}
				}
			}
		}
		
//		print();
		return variablesInView;
	}
	
	private boolean hasArrowWithOtherEnd(List<Arrow> in, String name) {
		boolean found = false;
		for (Arrow a : in) {
			if (a.getOtherEnd().equals(name))
				found = true;
		}
		return found;
	}
}
