package gov.anl.gss.sdm.server.model.translator;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

public class TreeTraversal {
	
	public static Node getLhs(Node aNode) {
		// by definition, the lhs is the child of a given node
		return aNode.getChild();
	}
	
	public static Node getRhs(Node aNode) {
		
		if (aNode == null) {
//			System.err.println("NODE INFO NOT AVAILABLE");
			return null;
		}
		
		// by definition, the rhs is the next sibling of the child of a given node
		Node lhs = aNode.getChild();
		if (lhs != null)
			return lhs.getNext();
		else
			return null;
	}
	
	public static Node generateAdditionSequence(Node n) {
		// if there is only one node, return the node
		// if 2 or more nodes, generate the sequence
		// this is used to take the sequence of function args and generate the additions
		
		Node root = new Node(n);
		Node next = root.getNext();
		while (next != null) {
			root = generateAdd(root, next);
			next = next.getNext();
		}
		return root;
	}
	
	public static Node generateAdd(Node arg1, Node arg2) {
		Node root = new Node("+");
		root.setChild(arg1);
		arg1.setNext(arg2);
		
		return root;
	}
	
	public static Node getFunctionArgument(Node functionNode, int argNum) {
		
		if (functionNode == null) {
			System.out.println("functionNode null");
		}
		
		FunctionManager fm = InformationManagers.getInstance().getFunctionManager();
		if (fm == null) {
			System.out.println("functionManager null");
		}
		
		
		// first check if this is indeed a function call
		
		if (!fm.isVensimFunction(functionNode.getToken()))
			return null;
		
		// functionNode is the node with the function name
		// argNum is the index of the argument in the vensim world. i.e. no generated args
		// this index starts at 1
		
		FunctionDescription fd = fm.getDescription(functionNode.getToken());
		
		int currentNode = 1;
		
		// get the first argument to the function
		
		Node node = functionNode.getChild();
		
		if (fd.isRequiresName())
			node = node.getNext();
		
		if (fd.isRequiresValue())
			node = node.getNext();
		
		if (fd.isRequiresTime())
			node = node.getNext();
		
		if (fd.isRequiresTimeStep())
			node = node.getNext();
		
		while (currentNode < argNum) {
			if (node == null)
				return null;
			node = node.getNext();
			currentNode++;
		}
		
		return node;
	}
	
	public static Node getFunctionArgumentIncludingAutoGen(Node functionNode, int argNum) {
		
		FunctionManager fm = InformationManagers.getInstance().getFunctionManager();
		
		// first check if this is indeed a function call
		
		if (!fm.isVensimFunction(functionNode.getToken()))
			return null;
		
		// functionNode is the node with the function name
		// argNum is the index of the argument in the vensim world. i.e. no generated args
		// this index starts at 1
		
		FunctionDescription fd = fm.getDescription(functionNode.getToken());
		
		int currentNode = 1;
		
		// get the first argument to the function
		
		Node node = functionNode.getChild();
		
//		if (fd.isRequiresName())
//			node = node.getNext();
//		
//		if (fd.isRequiresValue())
//			node = node.getNext();
//		
//		if (fd.isRequiresTime())
//			node = node.getNext();
//		
//		if (fd.isRequiresTimeStep())
//			node = node.getNext();
		
		while (currentNode < argNum) {
			if (node == null)
				return null;
			node = node.getNext();
			currentNode++;
		}
		
		return node;
	}
	
	public static List<String> extractVariablesFromChildExcludingFunctions(Node node) {
		List<String> al = new ArrayList<String>();
		if (node.getChild() == null) {
			al.add(node.getToken());
			return al;
		}
		extractVariablesExcludingFunctions(node.getChild(), 0, al);
		extractVariablesExcludingFunctions(node.getChild().getNext(), 0, al);
		return al;
	}
	
	private static void extractVariablesExcludingFunctions(Node node, int level, List<String> al) {
	    if (node == null)
	    	return;
	    
	    // if we are pointing to a function, we want only variable arguments to the function
	    // not the function or generated args to be gathered
	    if (Parser.isFunctionInvocation(node)) {
	    	
	    	// special case, if this is an ITE, we want to excluse arg1
	    	int argNum = 1;
	    	if (Parser.getFunctionName(node).equalsIgnoreCase("IFTHENELSE"))
	    		argNum = 2;
	    	node = TreeTraversal.getFunctionArgument(node, argNum);
	    }
	    if (node == null || node.getToken() == null) {
	    	return;
	    }
	    String token = node.getToken();
	    if (!Parser.isNumber(token) && !Parser.isOperator(token)&& !al.contains(token) 
	    		&& !Parser.isFunctionInvocation(node))
	    	al.add(token);
	    extractVariablesExcludingFunctions(node.getChild(), level+1, al);
	    extractVariablesExcludingFunctions(node.getNext(), level, al);
	}
	
	
	public static List<String> extractVariables(Node node) {
		List<String> al = new ArrayList<String>();
		extractVariables(node, 0, al);
		return al;
	}
	
	
	private static void extractVariables(Node node, int level, List<String> al) {
	    if (node == null)
	    	return;
	    String token = node.getToken();
	    if (!Parser.isNumber(token) && !Parser.isOperator(token)&& !al.contains(token))
	    	al.add(token);
	    extractVariables(node.getChild(), level+1, al);
	    extractVariables(node.getNext(), level, al);
	}
	
	public static Node copyStructure(Node root) {
		Node newRoot = new Node(root.getToken());
		
		copyStructure(root, newRoot);
		return newRoot;
		
	}
	
	private static void copyStructure(Node orig, Node copy) {
		
		// token is set prior to invocation
		// parent
		// child
		// next
		// previous
		
		if (orig.hasChild()) {
			Node origChild = orig.getChild();
			Node copyChild = new Node(origChild.getToken());
			copyChild.setParent(copy);
			copy.setChild(copyChild);
			copyStructure(origChild, copyChild);
		}
		
		if (orig.hasNext()) {
			Node origNext = orig.getNext();
			Node copyNext = new Node(origNext.getToken());
			copy.setNext(copyNext);
			copyNext.setPrevious(copy);
			copyStructure(origNext, copyNext);
		}
		
	}
	
	public static List<Node> getSubTreeAsList(Node node) {
		if (node == null) {
			//		generateRpn();
			//		generateTree();
			return null;
		}
		List<Node> al = new ArrayList<Node>();
		al.add(node);
		traverse(node.getChild(), al);
		return al;
	}
	
	public static List<Node> getTreeAsList(Node node) {
		if (node == null) {
			//		generateRpn();
			//		generateTree();
			return null;
		}
		List<Node> al = new ArrayList<Node>();
		traverse(node, al);
		return al;
	}

	public static int treeSize(Node node) {
		List<Node> al = new ArrayList<Node>();
		traverse(node, al);
		return al.size();
	}
	
	public static void traverse(Node node, List<Node> al) {
		if (node == null)
			return;
		al.add(node);
		traverse(node.getChild(), al);
		traverse(node.getNext(), al);


	}
	
	public static void getVariablesInTree(Node node, Set<String> al) {
		if (node == null)
			return;
		Node aNode = node;
		// looking at a function, need to strip generated arguments and process other args
		if (Parser.isFunctionInvocation(aNode.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(node.getToken());
			aNode = TreeTraversal.getFunctionArgumentIncludingAutoGen(aNode, 1);
			getVariablesInTree(aNode, al);
		} else {
			String token = aNode.getToken();
			if (!Parser.isOperator(token) &&
					!Parser.isQuotedString(token) &&
					!Parser.isNumber(token) &&
					!Parser.isLeftParen(token) &&
					!Parser.isRightParen(token))
				al.add(token);
		}
		
		if (aNode != null)
			getVariablesInTree(aNode.getChild(), al);
// this is where we terminate too soon, we never get through next of original node!
		if (aNode != null) {
//			getRHSVariablesForOrdering(aNode.getNext(), al);
			getVariablesInTree(node.getNext(), al);
		}
		
		return;
	}
		
		public static void getVariablesInFunctionArgument(Node node, Set<String> al) {
			if (node == null)
				return;
			Node aNode = node;
			
			al.add(node.getToken());
			getVariablesInTree(aNode.getChild(), al);
			
			return;

	}
		
		public static Node getFirstFunctionCallInTree(Node node, String function) {
			List<Node> al = new ArrayList<Node>();
			if (node == null)
				return null;
			
			for (Node n : getTreeAsList(node)) {
				if (Parser.isFunctionInvocation(n)) {
					if (function.equals(Parser.getFunctionName(n)))
						return n;
				}
			}
			
			return null; 

		}
		
		public static List<Node> getFunctionCallsInTree(Node node, String function) {
			List<Node> al = new ArrayList<Node>();
			if (node == null)
				return al;
			
			for (Node n : getTreeAsList(node)) {
				if (Parser.isFunctionInvocation(n)) {
					if (function.equals(Parser.getFunctionName(n)))
						al.add(n);
				}
			}
			
			return al; 

		}
		
		public static List<Node> getAllButFunctionCallsInTree(Node node, String function) {
			List<Node> al = new ArrayList<Node>();
			if (node == null)
				return al;
			
			for (Node n : getTreeAsList(node)) {
				if (Parser.isFunctionInvocation(n)) {
					if (!function.equals(Parser.getFunctionName(n)))
						al.add(n);
				}
			}
			
			return al; 

		}
		
		public static List<Node> getAllButFunctionCallsInTree(Node node, Set<String> functions) {
			List<Node> al = new ArrayList<Node>();
			if (node == null)
				return al;
			
			for (Node n : getTreeAsList(node)) {
				if (Parser.isFunctionInvocation(n)) {
					if (!functions.contains(Parser.getFunctionName(n)))
						al.add(n);
				}
			}
			
			return al; 

		}
		
		public static List<Node> getAllFunctionCallsInTree(Node node) {
			List<Node> al = new ArrayList<Node>();
			if (node == null)
				return al;
			
			for (Node n : getTreeAsList(node)) {
				if (Parser.isFunctionInvocation(n)) {
						al.add(n);
				}
			}
			
			return al; 

		}
	
	public static List<Node> getFunctionCallsInTree(Node node, Set<String> functions) {
		List<Node> al = new ArrayList<Node>();
		if (node == null)
			return al;
		
		for (Node n : getTreeAsList(node)) {
			if (Parser.isFunctionInvocation(n)) {
				if (functions.contains(Parser.getFunctionName(n)))
					al.add(n);
			}
		}
		
		return al; 

	}
	
	public static List<Node> getFunctionCallsWithVariablesInTree(Node node, Set<String> functions) {
		List<Node> al = new ArrayList<Node>();
		if (node == null)
			return al;

		System.out.println(getTreeAsList(node));
		for (Node n : getTreeAsList(node)) {
			if (Parser.isFunctionInvocation(n)) {
				if (functions.contains(Parser.getFunctionName(n))) {
					FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(Parser.getFunctionName(n));
					if (extractVariables(n).size() >= fd.getNumberOfAddedArguments()+1) { // i.e. name, value, time, timestep, sdFunction.
						al.add(n);
					} else {
						
					}
				}
			}
		}
		
		return al; 

	}
	
	public static List<Node> getFunctionCallArgs(Node node, Set<String> functions) {
		List<Node> al = new ArrayList<Node>();
		if (node == null)
			return al;
		
		List<Node> nodes = getTreeAsList(node.getChild());
		return nodes;
		
	}
	
	public static Node getLastArgument(Node functionNode) {
		
		FunctionManager fm = InformationManagers.getInstance().getFunctionManager();
		
		// first check if this is indeed a function call
		
		if (!fm.isVensimFunction(functionNode.getToken()))
			return null;

		// get the first argument to the function
		
		Node node = functionNode.getChild();
		
		while(node.getNext() != null) {
			node = node.getNext();
		}
		
		return node;
		
		
	}

}
