package gov.anl.gss.sdm.server.model.translator;


import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

public class UnitsManager {
static Logger log = Logger.getLogger(UnitsManager.class.getSimpleName());
	
	public static String DIMENSIONLESS = "Dmnl";
	
	// keys are store as upper, lower, and mixed case so we can check on lower case for defintion
	public static String[] ALLOWALBE_DIMENSIONLESS = {"dmnl", "dimensionless"};
    
    private  Map<String, String> lhsUnits;
    private  Map<String, String> lhsUnitsRaw;
    private  Map<String, String> unitsEquivalence;
    private Map<String, List<String>> alternatesUsed = new HashMap<String, List<String>>();
    
    private Map<String, SimplifiedUnit> simplifiedUnitsMap = new HashMap<String, SimplifiedUnit>();
    
    List<String> basic = new ArrayList<String>();
	List<String> combined = new ArrayList<String>();
	
    private  UnitConsistencyXMLWriter unitConsistencyXMLWriter;
    
    public UnitsManager() {
    	lhsUnits = new LinkedHashMap<String, String>();
        lhsUnitsRaw = new LinkedHashMap<String, String>();
        unitsEquivalence = new LinkedHashMap<String, String>();
    }
    
    public int getBasicCount() {
    	return basic.size();
    }
    
    public int getCombinedCount() {
    	return combined.size();
    }
    
    public List<List<String>> getTableRows() {
    	List<List<String>> al = new ArrayList<List<String>>();
    	
    	List<String> allUnits = new ArrayList<String>(basic);
    	Collections.sort(allUnits, String.CASE_INSENSITIVE_ORDER);
    	
    	for (String u : allUnits) {
    		List<String> aRow = new ArrayList<String>();
    		al.add(aRow);
    		aRow.add(u);
    		if (isBasic(u)) {
    			aRow.add(SDMDocLanguageSpecific.getInstance().
    					getTextHtml(StringConstants.BASIC));
    		} else {
    			aRow.add(SDMDocLanguageSpecific.getInstance().
    					getTextHtml(StringConstants.COMBINED));
    		}
    		if (alternatesUsed.containsKey(u)) {
    			List<String> alt = new ArrayList<String>();
    			for (String a : alternatesUsed.get(u)) {
    				if (!a.equals(u))
    					alt.add(a);
    			}
    			Collections.sort(alt, String.CASE_INSENSITIVE_ORDER);
    			aRow.add(alt.toString());
    		} else {
    			aRow.add("");
    		}
    	}
    	
    	allUnits = new ArrayList<String>(combined);
    	Collections.sort(allUnits, String.CASE_INSENSITIVE_ORDER);
    	
    	for (String u : allUnits) {
    		List<String> aRow = new ArrayList<String>();
    		al.add(aRow);
    		aRow.add(u);
    		if (isBasic(u)) {
    			aRow.add(SDMDocLanguageSpecific.getInstance().
    					getTextHtml(StringConstants.BASIC));
    		} else {
    			aRow.add(SDMDocLanguageSpecific.getInstance().
    					getTextHtml(StringConstants.COMBINED));
    		}
    		if (alternatesUsed.containsKey(u)) {
    			List<String> alt = new ArrayList<String>();
    			for (String a : alternatesUsed.get(u)) {
    				if (!a.equals(u))
    					alt.add(a);
    			}
    			Collections.sort(alt, String.CASE_INSENSITIVE_ORDER);
    			aRow.add(alt.toString());
    		} else {
    			aRow.add("");
    		}
    	}
    	
    	
    	return al;
    }
    
    
    public void report(BufferedWriter bw) {
//    	  private  Map<String, String> lhsUnits;
//        private  Map<String, String> lhsUnitsRaw;
//        private  Map<String, String> unitsEquivalence;
    	
    	
    	// first determine sets of basic and combined
    	// these all need to be "equivalent" units
    	
//    	for (String lhs : lhsUnits.keySet()) {
    		for (String lhs : lhsUnits.keySet()) {
    		
//    		String units = lhsUnits.get(lhs);
    			if (simplifiedUnitsMap.get(lhs) == null)
    				continue;
    		String units = simplifiedUnitsMap.get(lhs).getUnits();
    		if (units == null) {
    			continue;
    		}
    		if (units.equalsIgnoreCase("any")) {
    			continue;
    		}
    		String effUnits = units;
//    		if (unitsEquivalence.containsKey(units)) {
//    			effUnits = unitsEquivalence.get(units);
//    			if (effUnits == null || effUnits.trim().length() == 0) {
//    				continue;
//    			}
//    			if (!alternatesUsed.containsKey(effUnits)) {
//    				alternatesUsed.put(effUnits, new ArrayList<String>());
//    			}
//    			List<String> al = alternatesUsed.get(effUnits);
//    			if (!al.contains(units))
//    				al.add(units);
//    		}
    		
    		if (isBasic(effUnits)) {
    			if (!basic.contains(effUnits)) {
//    				System.out.println("Basic+ <"+effUnits+"> for lhs: "+lhs);
    				basic.add(effUnits);
    			}
    		} else {
    			if (!combined.contains(effUnits)) {
//    				System.out.println("Combined+ <"+effUnits+"> for lhs: "+lhs);
    				combined.add(effUnits);
    			}
    		}
    	}
    	
//    	System.out.println("#Basic: "+basic.size()+" "+basic);
//    	System.out.println("#Combined: "+combined.size()+" "+combined);
    	
    }
    
    public boolean isCombined(String units) {
    	return new SimplifiedUnit(units).isCombined();
    }
    
    public boolean isBasic(String units) {
    	return !isCombined(units);
    }
    
    public boolean isDmnl(String s) {
    	if (getEffectiveUnits(s).equals(DIMENSIONLESS))
    		return true;
    	else
    		return false;
    }
    
    public  Map<String, Equation> performUnitsConsistencyCheck(Map<String, Equation> equations, String file) {
    	
    	boolean consistent = true;
    	List<String> evaluationOrder = new ArrayList<String>();
    	for (String key : equations.keySet()) {
    		evaluationOrder.add(key);
    	}
    	Map<String, Equation> errors = performUnitsConsistencyCheck(evaluationOrder, equations, file);
    	report(null);
    	return errors;

    }
    
    public  Map<String, Equation> performUnitsConsistencyCheck(List<String> evaluationOrder, Map<String, Equation> equations, String file) {
    	boolean consistent = true;
    	Map<String, Equation> errors = new LinkedHashMap<String, Equation>();
    	unitConsistencyXMLWriter = new UnitConsistencyXMLWriter();
    	int equationCount = 0;
    	for (String lhs : evaluationOrder) {

    		Equation eqn = equations.get(lhs);
    		
    	
    		
    		if (!eqn.isAssignment()) {
//    			System.out.println("UC: SKIP No Assign "+eqn.getVensimEquationOnly());
    			continue;
    		}

    		if (!InformationManagers.getInstance().getFunctionManager().canCheckUnitsConsistency(eqn)) {
//    			System.out.println("UC: SKIP No Can Do "+eqn.getVensimEquationOnly());
    			continue;
    		}
    		
//    		System.out.println("UC: "+eqn.getVensimEquationOnly());

    		equationCount++;
//    		log.debug("===================================");

    		List<String> units = eqn.getEquationUnits();
    		
    		
    		
    		if (eqn.getUnits() == null || eqn.getUnits().equals("")) {
    			log.debug("UNITS Not Specified: "+eqn.getCleanEquation());
    			eqn.getUnitsMessages().add("UNITS Not Specified");
    			errors.put(eqn.getLhs(), eqn);
    			consistent = false;
    		} else if (!eqn.isArrayInitialization() && !isConsistent(eqn, units)) {
    			log.debug("INCONSISTENT UNITS: "+eqn.getCleanEquation());
    			errors.put(eqn.getLhs(), eqn);
    			consistent = false;
    		} else {
//    			log.debug("!!!YES!!! CONSISTENT UNITS: "+eqn.getCleanEquation());
    		}
    	}
    	unitConsistencyXMLWriter.setEquationCount(equationCount);
    	unitConsistencyXMLWriter.write(file);
    	unitConsistencyXMLWriter.writeReport(file.replace(".xml", ".txt"));
    	return errors;
    }
    
    public  void addEquivalence(String alternateUnit, String effectiveUnit) {
    	unitsEquivalence.put(alternateUnit, effectiveUnit);
    	unitsEquivalence.put(alternateUnit.toUpperCase(), effectiveUnit);
    	unitsEquivalence.put(alternateUnit.toLowerCase(), effectiveUnit);
    }
    
    public  String getEffectiveUnits(String units) {
    	if (units == null) {
    		return units;
    	}
	if (unitsEquivalence.containsKey(units.toLowerCase()))
	    return unitsEquivalence.get(units.toLowerCase());
	else
	    return units;
    }
    
    public  String getUnits(String lhs) {
	
	String units = lhsUnits.get(cleanseLHS(lhs));
	return getEffectiveUnits(units);
    }
    
    public  void addLhsUnits(String lhs, String units) {

    	lhsUnitsRaw.put(lhs, units);

    	String clhs = cleanseLHS(lhs);
    	String cunits = cleanseUnits(units);

    	if (lhsUnits.containsKey(clhs)) 
    		return;

    	lhsUnits.put(clhs, cunits);
    	if (cunits != null && cunits.trim().length() > 0) {
    		SimplifiedUnit su = new SimplifiedUnit(cunits);
    		simplifiedUnitsMap.put(clhs, su);
//    		System.out.println("$$$ "+clhs+" "+simplifiedUnitsMap.get(clhs).getUnits());
    		
    		// is this an alternate?
    		String official = su.getUnits();
    		if (!official.equals(cunits)) {
    			if (!alternatesUsed.containsKey(official))
    				alternatesUsed.put(official, new ArrayList<String>());
    			if (!alternatesUsed.get(official).contains(cunits))
    			alternatesUsed.get(official).add(cunits);
    		}
    	}

    }
    
    public  boolean hasUnits(String lhs) {
	return lhsUnits.containsKey(cleanseLHS(lhs));
    }
    
    public  String cleanseLHS(String lhs) {
    	String lhside = lhs;

    	if (ArrayReference.isArrayReference(lhs)) {
    		lhside = new ArrayReference(lhs).getArrayName();
    	}

    	lhside = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhside);

    	lhside = lhside.replace("memory.", "").replace("lookup.", "").replace("sdFunctions.", "").replace("array.", "");

    	return lhside.replace("memory.", "").replace("lookup.", "").replace("sdFunctions.", "");
    }
    public  String cleanseUnits(String units) {
	if (units == null)
	    return null;
	String u = units;
	
	if (units.contains("[")) {
	    u = units.split("\\[")[0].trim();
	}
	
	// let's remove extra blanks around "/"
	u = u.replaceAll("  */", "/");
	u = u.replaceAll("/  *","/");
	
	return u;
    }
    
    public  void dumpLhsUnits(BufferedWriter bw) {
	List<String> lhs = new ArrayList<String>();
	lhs.addAll(lhsUnits.keySet());
	Collections.sort(lhs);
	for (String l : lhs) {
	    try {
		bw.append("\"");
		bw.append(l);
		bw.append("\"");
		bw.append(",");
		bw.append("\"");
		bw.append(lhsUnits.get(l));
		bw.append("\"");
		bw.append("\n");
	    } catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	}
	try {
	    bw.close();
	} catch (IOException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
    }
    
    public  void dumpLhsUnitsRaw(BufferedWriter bw) {
	List<String> lhs = new ArrayList<String>();
	lhs.addAll(lhsUnitsRaw.keySet());
	Collections.sort(lhs);
	for (String l : lhs) {
	    try {
		bw.append("\"");
		bw.append(l);
		bw.append("\"");
		bw.append(",");
		bw.append("\"");
		bw.append(lhsUnitsRaw.get(l));
		bw.append("\"");
		bw.append("\n");
	    } catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	    }
	}
	try {
	    bw.close();
	} catch (IOException e) {
	    // TODO Auto-generated catch block
	    e.printStackTrace();
	}
    }
    
    public  boolean isConsistent(Equation eqn, List<String> units) {

    	List<String> unitsExpanded = expandUnits(units);
    	unitsExpanded = addFunctionUnits(unitsExpanded);

    	boolean isValid = valid(unitsExpanded, eqn);
    	
//    	if (eqn.isReferencesLookup()) {
//    		for (int i = 0; i < unitsExpanded.size(); i++) {
//    			if (unitsExpanded.get(i).startsWith("sdFunctions.LOOKUP")) {
//    				if (!unitsExpanded.get(i+4).equalsIgnoreCase("dmnl") && ! unitsExpanded.get(i+4).equalsIgnoreCase("dimensionless"))
//    					isValid = false;
//    			}
//    		}
//    	}
    	
    	if (!isValid) {
        	eqn.printTokensOneLine();
        	printUnits(units);
        	log.debug("++++++++++++++++++++++++++++++++");
    		printUnitsIndented(unitsExpanded, eqn);
    	}
    	return isValid;
    }
    
    private  boolean valid(List<String> units, Equation eqn) {
    	
    	
    	// stack based evaluation of units consistency
    	if (units.size() < 3) {
    		return false;
    	}

    	// by definition:	
    	String lhs = units.get(0);

    	if (lhs == null || lhs.equals(""))
    		return false;
    	String equal = units.get(1);
    	List<String> rhs = new ArrayList<String>(units);
    	rhs.remove(0);
    	rhs.remove(0);
    	
    	UnitExpression unitExpression = new UnitExpression(lhs, equal, rhs, eqn);
    	return unitExpression.isValid();
    }
    
    private  List<String> addFunctionUnits(List<String> units) {
	List<String> unitsExpanded = new ArrayList<String>();
	MutableInteger i = new MutableInteger(0);
	while (i.value() < units.size()) {
	    String s = units.get(i.value());
	    if (Parser.isFunctionInvocation(s)) {
		s = s.split("<")[0];
		FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(s);
		if (fd == null)
		    unitsExpanded.add(s+"<NULL>");
		else
		    unitsExpanded.add(s+"<"+fd.getReturnUnits()+">");
	    } else {
		unitsExpanded.add(s);
	    }
	    i.add(1);
	}
	return unitsExpanded;
    }
    
    private  List<String> expandUnits(List<String> units) {
	List<String> unitsExpanded = new ArrayList<String>();
	MutableInteger i = new MutableInteger(0);
	while (i.value() < units.size()) {
	    String s = units.get(i.value());
	    if (Parser.isFunctionInvocation(s)) {
		s = s.split("<")[0];
		i.add(1);
		FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(s);
		unitsExpanded.addAll(expandFunctionUnits(units, i, s));
		
		// should be pointing to the next argument
		for (int p = 0; p < fd.getNumArgs(); p++) {
//		    if (i.value() >= units.size())
//			log.debug("WTF!");
		    String s1 = units.get(i.value());
		    
		    // if null, there is no dimension for this thing
		    // set to "dmnl"
		    
		    if (s1 == null)
		    	s1 = UnitsManager.DIMENSIONLESS;
		    
		    if (Parser.isFunctionInvocation(s1)) {
			s1 = s1.split("<")[0];
			i.add(1);
			FunctionDescription fd1 = InformationManagers.getInstance().getFunctionManager().getDescription(s1);
			unitsExpanded.addAll(expandFunctionUnits(units, i, s1));
		    } else {
			unitsExpanded.add(s1);
			i.add(1);
		    }
		}
		
	    } else {
		unitsExpanded.add(s);
		i.add(1);
	    }
	}
	
	return unitsExpanded;
    }
    
    private  List<String> expandFunctionUnits(List<String> units, MutableInteger i, String functionName) {
	List<String> unitsExpanded = new ArrayList<String>();
	FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(functionName);
	if (fd == null)
	    log.debug("fd is null for "+functionName);
	unitsExpanded.add(functionName);  // function name
	unitsExpanded.add(units.get(i.valueAndInc()));   // left paren
	if (fd.isRequiresName()) {
//	    unitsExpanded.add("NA");
	    i.add(1);
//	    unitsExpanded.add(units.get(i.valueAndInc()));  // separator
	    i.add(1);
	} 
	if (fd.isRequiresValue()) {
//	    unitsExpanded.add(units.get(i.valueAndInc()));
//	    unitsExpanded.add(units.get(i.valueAndInc()));  // separator
	    i.add(2);
	}
	if (fd.isRequiresTime()) {
//	    unitsExpanded.add(Translator.UNITS_PROPERTIES.getProperty("timeUnits"));
	    i.add(1);
//	    unitsExpanded.add(units.get(i.valueAndInc()));  // separator
	    i.add(1);
	}
	if (fd.isRequiresTimeStep()) {
//	    unitsExpanded.add(Translator.UNITS_PROPERTIES.getProperty("timeUnits"));
	    i.add(1);
//	    unitsExpanded.add(units.get(i.valueAndInc()));  // separator
	    i.add(1);
	}
	
	return unitsExpanded;
    }
    
    private  void printUnits(List<String> units) {
	for (String s : units) {
	    log.debug(" \""+s+"\"");
	}
	log.debug("\n");
    }
    
    private  void printUnitsIndented(List<String> units, Equation eqn) {

    	int indent = 0;
    	String indentation = getIndentation(indent);
    	for (String s : units) {
    		if (s == null)
    			s = "null";
    		if (s.equals(")")) {
    			indent--;
    			indentation = getIndentation(indent);
    		}
    		log.debug(indentation+"\""+s+"\"");
    		eqn.getUnitsMessages().add(indentation+"\""+s+"\"");
    		if (s.equals("(")) {
    			indent++;
    			indentation = getIndentation(indent);
    		}
    	}

    }
    
    private  String getIndentation(int indent) {
	StringBuilder sb = new StringBuilder();
	
	for (int i = 0; i < indent; i++)
	    sb.append("   ");
	
	return sb.toString();
	
	
	
    }

    public  UnitConsistencyXMLWriter getUnitConsistencyXMLWriter() {
        return unitConsistencyXMLWriter;
    }
    
    public void finalize() {

    	// need to determine if the user has defined the version of dmnl they will use
    	boolean defineDmnl = true;
    	for (String dmnl : ALLOWALBE_DIMENSIONLESS) {
    		if (unitsEquivalence.containsKey(dmnl)) {
    			defineDmnl = false;
    			// Use the form selected by the user
    			DIMENSIONLESS = unitsEquivalence.get(dmnl);
    			break;
    		}
    	}
    	if (defineDmnl) {
    		
    		// system defaults
    		
    		addEquivalence("Dimensionless", DIMENSIONLESS);
    		addEquivalence("Dmnl", DIMENSIONLESS);
    		addEquivalence("Fraction", DIMENSIONLESS);
    		addEquivalence("Nil", DIMENSIONLESS);
    		addEquivalence("1", DIMENSIONLESS);    		
    	}
    	
//    	log.debug("##### Equivalences #####\n");
//    	List<String> al = new ArrayList<String>(unitsEquivalence.keySet());
//    	Collections.sort(al);
//    	for (String k : al) {
//    		log.debug(k+" -> "+unitsEquivalence.get(k));
//    	}
    
    }

	public List<String> getBasic() {
		return basic;
	}

	public List<String> getCombined() {
		return combined;
	}

}
