package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.analysis.EquationManager;
import gov.anl.gss.sdm.server.model.structure.SDObjectManager;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.SubscriptCombination;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.SDModelCodeGenerator;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class VariableDependencyProcessor {
	
	private static final boolean TRACE_ACTIONS = false;
	
	private Map<String, HashSet<String>> requiresScalarLhsExpandedRhs;
	private Map<String, String> expandedLhsToRealLhsMap;
	private Map<String, HashSet<String>> realLhsToExpandedLhsMap;
	private Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized;
	private Map<String, HashSet<String>> requiresExpandedLhsExpandedRhs;
	private Map<String, HashSet<String>> usesExpandedRhsExpandedLhs;
	private Map<String, Equation> equations;
	
	Map<String, Set<String>> headTailMap = null;
	Map<String, Set<String>> tailHeadMap = null;
	
//	private static boolean IGNORE_EXPANDING = false;
	
	private boolean ignoreCompleteSubscriptProcessing;
	
	private String fileNameHead;
	
	String miscDirectory;
	
	List<String> orderedListOfAvailableVariables = new ArrayList<String>();
	boolean cleanOrderOfAvailableVariables = true;
	
	private static final String STOCK = "S";
	private static final String AUXILIARY = "A";
	private static final String INITIALIZER = "I";
	private static final String CONSTANT_DATA = "C";
		
		
	private Map<String, List<String>> byVariableType = new LinkedHashMap<String, List<String>>();
	

	public VariableDependencyProcessor(Map<String, Equation> equations, String fileNameHead) {
		this.equations = equations;
		this.fileNameHead = fileNameHead;
		
		ignoreCompleteSubscriptProcessing = true; // !InformationManagers.getInstance().getModelManager().
				//		isRequiresExpandedSubscriptOrdering();
		if (InformationManagers.getInstance().getFileOrganizer() != null && 
				InformationManagers.getInstance().getFileOrganizer().getMiscDirectory().length() > 0) {
			miscDirectory = InformationManagers.getInstance().getFileOrganizer().getMiscDirectory();
		} else {
			miscDirectory = "output";
		}
//		if (ignoreCompleteSubscriptProcessing) {
//			ignoreCompleteSubscriptProcessing = rhsLhsVarsSame();
//		}
	}
	
	private boolean rhsLhsVarsSame() {
		boolean same = true;

		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();

		Set<String> allLHS = new LinkedHashSet<String>();
		Set<String> allRHS = new LinkedHashSet<String>();

		for (String realLHS : equations.keySet()) {
			Equation eqn = equations.get(realLHS);

			// we are interested only in certain equation types
			if (eqn.isDefinesSubscript())
				continue;
			if(eqn.isDefinesLookup() ||
					eqn.isArrayInitialization() ||
					eqn.isGetExternalData()) {
				continue;
			}
			allLHS.add(eqn.getOriginalLhs/*QuotedIfNecessary*/());
			for (String rhs : eqn.getRHSVariablesForOrdering()) {
				allRHS.add(ndtm.getOriginalName/*QuotedIfNecessary*/(rhs));
			}
		}

		allRHS.removeAll(allLHS);
		allRHS.remove("time");
		allRHS.remove("timeStep");
		
		Set<String> lookups = new LinkedHashSet<String>();
		for (String s : allRHS) {
			if (s.contains("lookup."))
					lookups.add(s);
		}
		
		allRHS.removeAll(lookups);

		if (allRHS.size() > 0)
			same = false;

		return same;
	}
	
	public void loadForDependency() {
		boolean forOrdering = false;
		if (!StaticSwitches.USE_NEW_FAST_LOAD) {
			load(forOrdering, ignoreCompleteSubscriptProcessing);
		} else {
//			newFastLoad(new StopShortConditionOrdering(), forOrdering);
//			newLoadImplementation();
			newNewLoadImplementation();
		}
	}

	public String loadForOrdering() {
		boolean forOrdering = true; // false
//		return newLoadImplementation();
		return newNewLoadImplementation();
		
//		System.out.println("loadForOrdering: "+StaticSwitches.USE_NEW_FAST_LOAD);
//		
//		if (!StaticSwitches.USE_NEW_FAST_LOAD) {
//			load(forOrdering, ignoreCompleteSubscriptProcessing);
//			return "";
//		} else {
////			newFastLoad(new StopShortConditionOrdering(), forOrdering);
//			return newLoadImplementation();
//		}
	}

	public void loadForActiveInitial() {
		boolean forOrdering = false;
		if (!StaticSwitches.USE_NEW_FAST_LOAD) {
			loadForActiveInitial(ignoreCompleteSubscriptProcessing);
		} else {
//			newFastLoad(new StopShortConditionActiveInitial(), forOrdering);
//			newLoadImplementation();
			newNewLoadImplementation();
		}
	}

	public void loadForActiveInitial(boolean ignoreCompleteSubscriptProcessing) {

		// requiresLhsScalarExpandedRhs maps a non-array realLHS to all rhs variables expanded. This map
		// is modified in the algorithm. The key is removed when the rhs hashset is
		// empty. This signifies that realLHS can be inserted into the evaluation order list.
		requiresScalarLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();

		// expandedLhsToRealLhsMap maps an expandedLHS to its higher level lhs subscript from which
		// it is expanded. This map remains intact in the algorithm.
		expandedLhsToRealLhsMap = new LinkedHashMap<String, String>();

		// realLhsToExpandedLhsMap maps a realLHS to its expandedLHSs. This map remains intact in the
		// algorithm
		realLhsToExpandedLhsMap = new LinkedHashMap<String, HashSet<String>>();

		// realLhsToExpandedLhsNotInitialized starts as a copy of realLhsToExpandedLhsMap. As expandedLHSs become available
		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
		// the key which means that we do not need to consider this realLHS for processing. IS THIS CORRECT?
		realLhsToExpandedLhsNotInitialized = new LinkedHashMap<String, HashSet<String>>();

		// requiresExpandedLhsExpandedRHS maps expandedLHS to set of rhs expanded variables. Set elements are removed
		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
		// becomes empty.
		requiresExpandedLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();


		// all realLHS that still require processing.
		List<String> allExpLHS = new ArrayList<String>();

		// for each equation:
		// get the set of variables that are required
		// then walk through the set repeatedly removing those with no requirements after they have been met

		// first make sure that all mapped subscripts have been defined

		// these are system variables that will be defined at run time

//		requiresExpandedLhsExpandedRHS.put("time", new LinkedHashSet<String>());
//		requiresExpandedLhsExpandedRHS.put("timeStep", new LinkedHashSet<String>());
//
//		realLhsToExpandedLhsNotInitialized.put("time", new LinkedHashSet<String>());
//		realLhsToExpandedLhsNotInitialized.put("timeStep", new LinkedHashSet<String>());
//
//		realLhsToExpandedLhsMap.put("time", new LinkedHashSet<String>());
//		realLhsToExpandedLhsMap.put("timeStep", new LinkedHashSet<String>());
//		
//		requiresExpandedLhsExpandedRHS.put("TIME STEP", new LinkedHashSet<String>());
//		realLhsToExpandedLhsNotInitialized.put("TIME STEP", new LinkedHashSet<String>());
//		realLhsToExpandedLhsMap.put("TIME STEP", new LinkedHashSet<String>());

		for (String realLHS : equations.keySet()) {
			
			// reference to the equations currently being processed
			Equation currentEquation = equations.get(realLHS);

			// this is only for assignment statements
			// maybe array initialization
			if (currentEquation.isDefinesSubscript())
				continue;

			// all equations will have these data structures defined
			// realLHS is the Vensim version of LHS i.e. no subscripts

			realLhsToExpandedLhsMap.put(realLHS, new LinkedHashSet<String>());
			realLhsToExpandedLhsNotInitialized.put(realLHS, new LinkedHashSet<String>());

			allExpLHS.add(realLHS);

			// these will have no dependencies
			// set them up so that they will be made available immediately
			// the stocks are ignored because we have already ordered them

			if(currentEquation.isDefinesLookup() ||
					currentEquation.isArrayInitialization() ||
					currentEquation.isGetExternalData() ||
					currentEquation.isStock() ||
					currentEquation.isData() ||
					currentEquation.isConstant()) {
				// 11/22/2013

				// Note that if not an array references, expandedLHS ends up as realLHS
				if (ignoreCompleteSubscriptProcessing) {
					
					expandedLhsToRealLhsMap.put(realLHS, realLHS);
					realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
//					realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
					requiresExpandedLhsExpandedRhs.put(realLHS, new LinkedHashSet<String>());
				} else {
				for (String expandedLHS : ArrayReference.expand(realLHS)) {
					expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
					realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
//					realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);
					requiresExpandedLhsExpandedRhs.put(expandedLHS, new LinkedHashSet<String>());

				}
				}
				// 11/22/2013

				continue;
			}

			if (!ignoreCompleteSubscriptProcessing && ArrayReference.isArrayReference(realLHS)) {
				ArrayReference lhsArrayReference = new ArrayReference(realLHS);

				// need to check if there are exceptions for this assignment
				// if there are, we need to have access to them so that they can
				// be discard from processing
				List<String> exceptions = new ArrayList<String>();

				if (currentEquation.isHasException()) {
					exceptions = currentEquation.getExceptions();
				}

				// Generate all combinations of realLHS expanded
				for (SubscriptCombination outerSub : InformationManagers.getInstance().getArrayManager().
						getSubscriptValueCombinations(lhsArrayReference.getSubscriptsAsArray())) {

					// Here we will skip any subscript combo values that are defined in another equation within a multi-equation definition
					if (exceptions.contains(outerSub.getSubscriptValue())) {
						System.out.println("Skipping "+"[" + outerSub.getSubscriptValue() + "] for "+realLHS);
						continue;
					}

					// Build an array reference based on the subscript value
					String expandedLHS = "array."+lhsArrayReference.getArrayName()+ "[" + outerSub.getSubscriptValue() + "]";

					requiresExpandedLhsExpandedRhs.put(expandedLHS, new LinkedHashSet<String>());

					// allocate data structures for holding rhs references and mapping from expandedLHS -> realLHS
					expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
					realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
					realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);

					// get the set of all rhsVariable reference
					// these are the tokens stored during parsing. Note that we have the ! still there

					Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrderingActiveInitial();
//					if (forOrdering &&  containsInitialValueFunction(currentEquation) ) {
//						variablesOfInterest = currentEquation.getFunctionInitialVariables();
//					}


					for (String rhsVar : variablesOfInterest) {
						// if this is not an array reference, simply put the variable into the requiresExpanded set
						// there is nothing to expand
						if (!ArrayReference.isArrayReference(rhsVar)) {
							String expRHSOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
							if (!expandedLHS.equals(expRHSOrig)) {
								// pecial case -- if our rhs var is a subscript but used as a variable, do not print error message
								if (!InformationManagers.getInstance().getArrayManager().isNamedSubscript(expRHSOrig))
									requiresExpandedLhsExpandedRhs.get(expandedLHS).add(expRHSOrig);
							}

						} else {
							// if an array access, we must rip apart the array reference and 
							// insert all subscript values based on outerSub current value

							//create an arrayReference for the RHS 
							// recall that we have a LHS arrayReference already defined
							ArrayReference rhsArrayReference = new ArrayReference(rhsVar);

							// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
							// get a list of these combinations

							// need to deal with rhs array like: X[y!, z]
							// currently doesn't seem to do it right

							boolean printit = false;

							String arrayName = null;
							if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
								String[] rhsExpanded;
								if (rhsArrayReference.hasRangeSubscript()) {
									printit = false;
									rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
									if (printit) {
										System.out.println("################ Range sub: "+rhsVar);
										for (String s : rhsExpanded)
											System.out.println(s);
									}
								} else {
									rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
								}

								List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
								for (SubscriptCombination rangeSub : combos){

									arrayName = rhsArrayReference.getArrayName()+"[";
									if (arrayName.startsWith("lookup."))
										arrayName =arrayName.replace("lookup.", "array.");
									else
										arrayName = "array." + arrayName;
									// we now construct an appropriate arrayReference for each combination
									//									arrayName = "array."+rhsArrayReference.getArrayName()+"[";
									int subNum = 0;

									// for each subscript that appears in this rhsArrayReference
									// we need to determine if it is a range subscript or not.
									// If it is, 
									for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
										if (subNum++ > 0)
											arrayName += ",";
										if (ArrayReference.isRangeSubscript(subscript)) {
											arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
										} else {
											arrayName += outerSub.getSubscriptValue(subscript);
										}

									}
									arrayName += "]";
									if (printit)
										System.out.println("Expanded: "+arrayName);
									if (!expandedLHS.equals(arrayName)) {
										
										requiresExpandedLhsExpandedRhs.get(expandedLHS).add(arrayName);
									}

									if (printit)
										System.out.println(expandedLHS+" LR "+arrayName);
									//									}

								}
							} else {
								// With no range subscript, we just copy over the subscript values that appear in the LHS
								arrayName = rhsArrayReference.getArrayName()+"[";
								if (arrayName.startsWith("lookup."))
									arrayName =arrayName.replace("lookup.", "array.");
								else
									arrayName = "array." + arrayName;
								int subNum = 0;

								// for each subscript that appears in this rhsArrayReference
								// we need to determine if it is a range subscript or not.
								// If it is, 
								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
									if (subNum++ > 0)
										arrayName += ",";

									arrayName += outerSub.getSubscriptValue(subscript);

								}
								arrayName += "]";
								if (!expandedLHS.equals(arrayName)) {
									requiresExpandedLhsExpandedRhs.get(expandedLHS).add(arrayName);
								}

							}
						}

					}
				}
			} else {

				requiresExpandedLhsExpandedRhs.put(realLHS, new LinkedHashSet<String>());
				expandedLhsToRealLhsMap.put(realLHS, realLHS);
				realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
				// MJB HERE
				realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
				Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrderingActiveInitial(/*true*/); // Expanded();
//				if (forOrdering &&  containsInitialValueFunction(currentEquation)) {
//					variablesOfInterest = currentEquation.getFunctionInitialVariables();
//				} 

				// Note that there is not an array reference on the LHS, but there can be on the rhs
				// the rhs array reference will require additional work if a range variable appears


				for (String rhsVar : variablesOfInterest) {


					//						System.out.println("VOI: "+realLHS+" "+rhsVar);

					// if this is not an array reference, simply put the variable into the requiresExpanded set
					// there is nothing to expand
					if (!ArrayReference.isArrayReference(rhsVar) || ignoreCompleteSubscriptProcessing) {
						//							InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar));
						String rhsVarOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
						if (rhsVarOrig == null)
							System.out.println("Huh? rhsVarOrig load rhsVar = "+rhsVar);
						if (!realLHS.equals(rhsVarOrig)) {

							
							if (ArrayReference.isPartialArrayReference(rhsVarOrig))
								rhsVarOrig = ArrayReference.asCompleteArrayReference(rhsVarOrig);
							rhsVarOrig = ArrayReference.annotate(rhsVarOrig);
							
							requiresExpandedLhsExpandedRhs.get(realLHS).add(rhsVarOrig.replace("!", ""));
						}

					} else {
						// if an array access, we must rip apart the array reference and 
						// insert all subscript values based on outerSub current value

						//create an arrayReference for the RHS 
						// recall that we have a LHS arrayReference already defined
						ArrayReference rhsArrayReference = new ArrayReference(rhsVar);

						// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
						// get a list of these combinations

						// need to deal with rhs array like: X[y!, z]
						// currently doesn't seem to do it right

						boolean printit = false;

						String arrayName = null;
						if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
							String[] rhsExpanded;
							if (rhsArrayReference.hasRangeSubscript()) {
								printit = false;
								rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
								//									System.out.println("################ Scalar Range sub: "+realLHS+" "+rhsVar);
								//									for (String s : rhsExpanded)
								//										System.out.println(s);
							} else {
								rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
							}

							List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
							for (SubscriptCombination rangeSub : combos){

								// we now construct an appropriate arrayReference for each combination
								arrayName = "array."+rhsArrayReference.getArrayName()+"[";
								int subNum = 0;

								// for each subscript that appears in this rhsArrayReference
								// we need to determine if it is a range subscript or not.
								// If it is, 
								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
									if (subNum++ > 0)
										arrayName += ",";
									if (ArrayReference.isRangeSubscript(subscript)) {
										arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
									} else {
										arrayName += subscript;
									}

								}
								arrayName += "]";
								if (printit)
									System.out.println("Expanded: "+arrayName);
								
								if (!realLHS.equals(arrayName)) {
									requiresExpandedLhsExpandedRhs.get(realLHS).add(arrayName);
								}

								if (printit)
									System.out.println(realLHS+" LR "+arrayName);
								//									}

							}
						} else {
							// this is an array reference where the subscript is terminal
							if (rhsVar == null)
								System.out.println("Huh? load rhsVar");
							if (!realLHS.equals(rhsVar)) {
								requiresExpandedLhsExpandedRhs.get(realLHS).add(rhsVar);
							}
						}

					}
				}

			}

		}

		// for each RHS set in requiresExpandedLhsExpandedRHS, need to add the eqn LHS of the rhs Item
		/* */
		for (String expLHS : requiresExpandedLhsExpandedRhs.keySet()) {
			Set<String> aSetExpRHS = requiresExpandedLhsExpandedRhs.get(expLHS);

			// in either case below, no additional processing required
			if (aSetExpRHS == null || aSetExpRHS.size() == 0)
				continue;

			// capture list of all expRHS that were originally in set
			List<String> aList = new ArrayList<String>();
			aList.addAll(aSetExpRHS);

			// for each of these, determine its realLHS and then add to the set
			// when an expRHS is available, it is removed
			// when the entire equation is available the expRHSrealLHS is freed.
			for (String rhs : aList) {
				String rhsRealLHS = expandedLhsToRealLhsMap.get(rhs);
				if (rhsRealLHS == null) {
					if (rhs.equalsIgnoreCase("time") || rhs.equalsIgnoreCase("timestep")) {

					} else {
						// try this.... 11/22/2013
						if (realLhsToExpandedLhsMap.containsKey(rhs)) {
							// we now track the equation in which the others are defined
							
							aSetExpRHS.add(rhsRealLHS);
						} else {
							// special case -- if our rhs var is a subscript but used as a variable, do not print error message
							if (!InformationManagers.getInstance().getArrayManager().isNamedSubscript(rhs))
								System.out.println("(1) VDP How can this be null in expandedLhsToRealLhsMap: rhs: <"+rhs+"> expLHS <"+expLHS+">");
						}
					}
				} else {
					
//					aSetExpRHS.add(rhsRealLHS);
				}
			}
		}
		/* */
		// ############## WRITE_DEBUG_FILES
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			dumpRequires2(fileNameHead+"AIexpandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
			dumpRequires(fileNameHead+"AIrealLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
			boolean startChain = false; // want this to be false for production
			dumpRequiresExpanded(fileNameHead+"AIinitialrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRhs, startChain); // HerelhsExpandedNotInitialized
			dumpRequiresExpanded(fileNameHead+"AIinitialrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
		}

//		analyzeDependencies(fileNameHead,requiresExpandedLhsExpandedRHS, realLhsToExpandedLhsNotInitialized, expandedLhsToRealLhsMap,
//				realLhsToExpandedLhsMap, equations);


	
	}
	
	public void categorizeVariables(List<String> variables) {
		
		byVariableType.put(STOCK, new ArrayList<String>());
		byVariableType.put(AUXILIARY, new ArrayList<String>());
		byVariableType.put(INITIALIZER, new ArrayList<String>());
		byVariableType.put(CONSTANT_DATA, new ArrayList<String>());
		
		for (String var : variables) {
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(var);
			if (eqn == null) {
				System.out.println("NO EQUATION: "+var);
				continue;
			}
			
			if (eqn.isStock()) {
				if (!byVariableType.get(STOCK).contains(var))
					byVariableType.get(STOCK).add(var);
			}
			if (eqn.isAuxiliary()) {
				if (!byVariableType.get(AUXILIARY).contains(var))
					byVariableType.get(AUXILIARY).add(var);
			}
			if (eqn.isConstant() || eqn.isData()) {
				if (!byVariableType.get(CONSTANT_DATA).contains(var))
				byVariableType.get(CONSTANT_DATA).add(var);
			}
			if (eqn.isStock() || eqn.isActiveInitial()) {
				Set<String> fiv = eqn.getFunctionInitialVariables();
				
				for (String original : InformationManagers.getInstance()
						.getNativeDataTypeManager().getOriginalNames(fiv)) {
					// some constants already processed may also qualify as initializers
					// do not put them in twice
					if (!byVariableType.get(CONSTANT_DATA).contains(original)) {
						if (!byVariableType.get(INITIALIZER).contains(original))
							byVariableType.get(INITIALIZER).add(original);
					}
					
				}
			}
		}
		// since INITIALIZERS are not ordered at this point -- order them
		
		List<String> nonordered = byVariableType.get(INITIALIZER);
		byVariableType.put(INITIALIZER, new ArrayList<String>());
		
		for (String var : variables) {
			if (nonordered.contains(var)) {
				if (!byVariableType.get(INITIALIZER).contains(var))
					byVariableType.get(INITIALIZER).add(var);
			}
		}
	
	}
	
	public String getVariableOrArrayName(String lhs) {
		String aVar = lhs;
		if (ArrayReference.isArrayReference(lhs)) {
			aVar = ArrayReference.extractArrayName(aVar);
		}
		aVar = ArrayReference.deannotate(aVar);
		return aVar;
	}
	
	public String newLoadImplementation() {
		
		System.out.println("\nNew Load Implementation VDP");
		
		Equation eqn = null;
		
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		ArrayManager amgr = InformationManagers.getInstance().getArrayManager();
		
		// this is the true lhs of the equation
		// lhsUsesRhs this maps the actual lhs to the rhs variables for this specific equation
		
		// The use of "equation" in data structure naming indicates that they are lhs of an equation and thus will have subscripts if an array
		// The use of "variable" in data structure naming indicates that they have any subscripts removed leaving just the variable name
		
		Map<String, Set<String>> lhsKeyUsesTheseEquations = new LinkedHashMap<String, Set<String>>();
		Map<String, Set<String>> stockLhsKeyUsesTheseEquations = new LinkedHashMap<String, Set<String>>();
		// this is just a map going the other direction
		Map<String, Set<String>> equationsThatAreUsedByThisEquationLhs = new LinkedHashMap<String, Set<String>>();
		Map<String, Set<String>> stockEquationsThatAreUsedByThisEquationLhs = new LinkedHashMap<String, Set<String>>();
		
		Set<String> allEquationsWaitingForRelease;
		
		lhsKeyUsesTheseEquations.put("time", new LinkedHashSet<String>());
		lhsKeyUsesTheseEquations.put("timeStep", new LinkedHashSet<String>());
		
		dumpVariableReferences("references.txt", InformationManagers.getInstance().getSdObjectManager().getVariableReferences());
		List<String> variables = new ArrayList<String>(InformationManagers.getInstance().getSdObjectManager().getVariables());
		Set<String> terminalCheckCompleted = new LinkedHashSet<String>();
		
		categorizeVariables(variables);
		
		// first make sure that each equation has an empty entry in lhsUsesRhs
		for (String annotatedLHS : equations.keySet()) {
			String equationLhs = ndtm.getOriginalName(annotatedLHS);

			lhsKeyUsesTheseEquations.put(equationLhs, new LinkedHashSet<String>());
			eqn = equations.get(annotatedLHS);
			if (eqn.isStock() || eqn.isActiveInitial()) {
				stockLhsKeyUsesTheseEquations.put(equationLhs, new LinkedHashSet<String>());
			}
		}
		
		int numberOfEquations = 0;
		
		// now populate with rhs information
		
		for (String annotatedLHS : equations.keySet()) {
			
			if (annotatedLHS.startsWith("array.Cigarette smokers["))
				System.out.println("DEBUG cigs");

			// get this down to vensim lhs and then just the variable
			String equationLhs = ndtm.getOriginalName(annotatedLHS);
			String equationLhsVariable = getVariableOrArrayName(equationLhs);
			
			String keyVariable = equationLhsVariable;
			

//			System.out.println("annotated,lhs,lhsVar "+"<"+annotatedLHS+"> <"+lhs+"> <"+lhsVar+">");

			// don't need to worry about defining subscripts
			eqn = equations.get(annotatedLHS);
			if (eqn.isDefinesSubscript())
				continue;
			numberOfEquations++;

			Map<String, Set<String>> refs = InformationManagers.getInstance().getSdObjectManager()
					.getVariableReferences(equationLhsVariable);

			// these are the forms in which this variable appears on lhs and rhs 
			Set<String> keyVariableLhsReferences = refs.get(SDObjectManager.LHS);
			Set<String> keyVariableRhsReferences = refs.get(SDObjectManager.RHS_USE);
			
			Set<String> variablesNeedingEquations;	

			// if more than one lhs, we may need to store extra information in lhsUsesRhs and rhsUsedByLhs
			
			if (!terminalCheckCompleted.contains(equationLhsVariable)) {
				
				// is this an instance of a multi-equation lhs variable?
				if (keyVariableLhsReferences.size() > 1) {
					
					
					// find rhs references that do not have a corresponding lhs reference and
					// create entries
					
					variablesNeedingEquations = Utilities.minus(keyVariableRhsReferences, keyVariableLhsReferences);
					for (String s : variablesNeedingEquations) {
						lhsKeyUsesTheseEquations.put(s, new LinkedHashSet<String>());
//						lhsKeyUsesTheseEquations.get(s).addAll(keyVariableLhsReferences);
						
						// preload equationsThatAreUsedByThisEquationLhs
					}
						
//						for (String anEquationVariableFromARhsReference : keyVariableRhsReferences) {  // these are rhs references
//							
//							// if this anEquationVariableFromARhsReference does not exist in lhsKeyUsesTheseEquations,
//							// then this is the full array access signature ???
//							//
//							// Need all 
//							
//							Equation e = InformationManagers.getInstance().getEquationManager().getAnEquationFor(anEquationVariableFromARhsReference);
//		
//
//							variablesNeedingEquations = e.getRHSVariablesForOrdering(/*true*/);
//							
////							Map<String, Set<String>> refs2 = InformationManagers.getInstance().getSdObjectManager()
////									.getVariableReferences(getVariableOrArrayName(anEquationVariableFromARhsReference));
////
////							// these are the forms in which this variable appears on lhs and rhs 
////							Set<String> keyVariableLhsReferences2 = refs2.get(SDObjectManager.LHS);
////							Set<String> keyVariableRhsReferences2 = refs2.get(SDObjectManager.RHS);
//							
//							for (String s : variablesNeedingEquations) {
//							String v = Parser.removeAnnotation(s).replace("!", "");
//							String justVar = getVariableOrArrayName(v);
//							if (!justVar.equals(keyVariable))
//								continue;
//							if (!lhsKeyUsesTheseEquations.containsKey(v)) {
//								lhsKeyUsesTheseEquations.put(v, new LinkedHashSet<String>());
//								System.out.println("LSH->RHS->LSH not found for <"+v+"> added it as key with RHS "+keyVariableLhsReferences);
////							}
//							/* Set<String> variablesUsedByRhsVariable = */ 
//							lhsKeyUsesTheseEquations.get(v).addAll(keyVariableLhsReferences);
//							}
//						}
//							
//							
////							for (String aVariableUsedByRhsVariable : variablesUsedByRhsVariable) {
////								if (!aVariableUsedByRhsVariable.equals(anEquationVariableFromARhsReference))
////									if (!variablesUsedByRhsVariable.contains(aVariableUsedByRhsVariable))
////										variablesUsedByRhsVariable.add(aVariableUsedByRhsVariable);
////							}
////							
////							
////							for (String aLhsReferenceToKeyVariable : keyVariableLhsReferences) {
////								if (!equationsThatUseThisEquationLhs.containsKey(aLhsReferenceToKeyVariable))
////									equationsThatUseThisEquationLhs.put(aLhsReferenceToKeyVariable, new LinkedHashSet<String>());
////								Set<String> rhsUsedByLhsForLhsRef = equationsThatUseThisEquationLhs.get(aLhsReferenceToKeyVariable);
////								if (!aLhsReferenceToKeyVariable.equals(anEquationVariableFromARhsReference))
////									if (!rhsUsedByLhsForLhsRef.contains(anEquationVariableFromARhsReference))
////										rhsUsedByLhsForLhsRef.add(anEquationVariableFromARhsReference);
////							}
//						}
//					} 
				} 	else {
				}

				terminalCheckCompleted.add(equationLhsVariable);
			}
//			if (!terminalCheckCompleted.contains(equationLhsVariable)) {
//				
//				// is this an instance of a multi-equation lhs variable?
//				if (keyVariableLhsReferences.size() > 1 /* || equationLhsVariablesUsingKeyVariable.size() > 1 */) {
////					if (!setsEqual(keyVariableLhsReferences, keyVariableRhsReferences)) {
//						
//						// in this case, we just add the rhsRef as a LHSref with all actual lhs
//						//
//						// multiple equations act upon this array
//						//
//						// want to search through all equations to find those that have the KeyVariable in its RHS
//						// when we find one, see its Equation matches KeyVariable. If not, we need to verify that this equation has a lhs
//						// in the lhsKeyUsesTheseEquations structure. if it doesn't add one
//						
//						
//						
//						for (String anEquationVariableFromARhsReference : keyVariableRhsReferences) {  // these are rhs references
//							
//							// if this anEquationVariableFromARhsReference does not exist in lhsKeyUsesTheseEquations,
//							// then this is the full array access signature
//							
//							Equation e = InformationManagers.getInstance().getEquationManager().getAnEquationFor(anEquationVariableFromARhsReference);
//		
//
//							variablesOfInterest = e.getRHSVariablesForOrdering(/*true*/);
//							
////							Map<String, Set<String>> refs2 = InformationManagers.getInstance().getSdObjectManager()
////									.getVariableReferences(getVariableOrArrayName(anEquationVariableFromARhsReference));
////
////							// these are the forms in which this variable appears on lhs and rhs 
////							Set<String> keyVariableLhsReferences2 = refs2.get(SDObjectManager.LHS);
////							Set<String> keyVariableRhsReferences2 = refs2.get(SDObjectManager.RHS);
//							
//							for (String s : variablesOfInterest) {
//							String v = Parser.removeAnnotation(s).replace("!", "");
//							String justVar = getVariableOrArrayName(v);
//							if (!justVar.equals(keyVariable))
//								continue;
//							if (!lhsKeyUsesTheseEquations.containsKey(v)) {
//								lhsKeyUsesTheseEquations.put(v, new LinkedHashSet<String>());
//								System.out.println("LSH->RHS->LSH not found for <"+v+"> added it as key with RHS "+keyVariableLhsReferences);
////							}
//							/* Set<String> variablesUsedByRhsVariable = */ 
//							lhsKeyUsesTheseEquations.get(v).addAll(keyVariableLhsReferences);
//							}
//						}
//							
//							
////							for (String aVariableUsedByRhsVariable : variablesUsedByRhsVariable) {
////								if (!aVariableUsedByRhsVariable.equals(anEquationVariableFromARhsReference))
////									if (!variablesUsedByRhsVariable.contains(aVariableUsedByRhsVariable))
////										variablesUsedByRhsVariable.add(aVariableUsedByRhsVariable);
////							}
////							
////							
////							for (String aLhsReferenceToKeyVariable : keyVariableLhsReferences) {
////								if (!equationsThatUseThisEquationLhs.containsKey(aLhsReferenceToKeyVariable))
////									equationsThatUseThisEquationLhs.put(aLhsReferenceToKeyVariable, new LinkedHashSet<String>());
////								Set<String> rhsUsedByLhsForLhsRef = equationsThatUseThisEquationLhs.get(aLhsReferenceToKeyVariable);
////								if (!aLhsReferenceToKeyVariable.equals(anEquationVariableFromARhsReference))
////									if (!rhsUsedByLhsForLhsRef.contains(anEquationVariableFromARhsReference))
////										rhsUsedByLhsForLhsRef.add(anEquationVariableFromARhsReference);
////							}
//						}
////					} 
//				} 	else {
//				}
//
//				terminalCheckCompleted.add(equationLhsVariable);
//			}

			// now capture rhs variables
//			Set<String> variablesOfInterest;			

			variablesNeedingEquations = equations.get(annotatedLHS).getRHSVariablesForOrdering(/*true*/);

			for (String annotatedRhsVar : variablesNeedingEquations) {
				// add rhs varaibles to sets
				Set<String> aSet = lhsKeyUsesTheseEquations.get(equationLhs);
				String rhsVar = ndtm.getOriginalName(annotatedRhsVar).replace("!]","]").replace("!,",",");
				aSet.add(rhsVar);

				if (!equationsThatAreUsedByThisEquationLhs.containsKey(rhsVar))
					equationsThatAreUsedByThisEquationLhs.put(rhsVar,  new LinkedHashSet<String>());
				Set<String> aSet2 = equationsThatAreUsedByThisEquationLhs.get(rhsVar);
				aSet2.add(equationLhs);

				Map<String, Set<String>> ref = InformationManagers.getInstance().getSdObjectManager().
						getVariableReferences(getVariableOrArrayName(rhsVar));

				// these are the forms in which this variable appears on lhs and rhs 
				Set<String> lhsRef = ref.get(SDObjectManager.LHS);
				Set<String> rhsRef = ref.get(SDObjectManager.RHS);
				

				if (lhsRef.size() > 1 || rhsRef.size() > 1) {
					if (!setsEqual(lhsRef, rhsRef)) {
						// in this case, we just add the rhsRef as a LHSref with all actual lhs
						for (String s : rhsRef) {
							if (!lhsKeyUsesTheseEquations.containsKey(s)) {
								
								lhsKeyUsesTheseEquations.put(s, new LinkedHashSet<String>());
							}
							for (String s2 : lhsRef) {
								if (!s2.equals(s))
									lhsKeyUsesTheseEquations.get(s).add(s2);
							}
							for (String s2 : lhsRef) {
								if (!equationsThatAreUsedByThisEquationLhs.containsKey(s2))
									equationsThatAreUsedByThisEquationLhs.put(s2, new LinkedHashSet<String>());
								if (!s2.equals(s))
									equationsThatAreUsedByThisEquationLhs.get(s2).add(s);
							}
						}
					} 
				} 	
			}

			if (equations.get(annotatedLHS).isStock() || equations.get(annotatedLHS).isActiveInitial()) {
				variablesNeedingEquations = equations.get(annotatedLHS).getStockArg1Variables();

				for (String annotatedRhsVar : variablesNeedingEquations) {
					// add rhs varaibles to sets
					Set<String> aSet = stockLhsKeyUsesTheseEquations.get(equationLhs);
					String rhsVar = ndtm.getOriginalName(annotatedRhsVar).replace("!]","]").replace("!,",",");
					if (!rhsVar.equals(equationLhs)) {
						aSet.add(rhsVar);

						if (!stockEquationsThatAreUsedByThisEquationLhs.containsKey(rhsVar))
							stockEquationsThatAreUsedByThisEquationLhs.put(rhsVar,  new LinkedHashSet<String>());
						Set<String> aSet2 = stockEquationsThatAreUsedByThisEquationLhs.get(rhsVar);
						if (!rhsVar.equals(equationLhs))
							aSet2.add(equationLhs);
					}
				}
			}
		}
//		System.out.println("Number of Equations in equations map: "+equations.size());
//		for (String s : Utilities.sortIgnoreCase(equations.keySet())) {
//			Equation e = equations.get(s);
//			if (e.isDefinesSubscript())
//				continue;
//			System.out.println("V# "+e.getVariableNumber()+" E# "+e.getEquationNumber()+" "+s);
//		}
		System.out.println("Number of Equations for ordering: "+numberOfEquations);
		System.out.println("Number of lhsUsesRhs entries: "+lhsKeyUsesTheseEquations.size());
		System.out.println("Number of rhsUsedByLhs entries: "+equationsThatAreUsedByThisEquationLhs.size());
		

//		ArrayList<String> delayedOrdering = new ArrayList<String>();
		
//		System.out.println("lhsUsesRhs: "+lhsKeyUsesTheseEquations);
//		System.out.println("rhsUsedByLhs: "+equationsThatUseThisEquationLhs);
		
		boolean additionalLevel = false;
		dumpUsesUsedByMap("BeforeValidationLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
		dumpUsesUsedByMap("BeforeValidationRhsUsedByLhs.txt", equationsThatAreUsedByThisEquationLhs, additionalLevel);
		
		// this may add some additional info to lhsUsesRhs
		if (!validate(lhsKeyUsesTheseEquations, equationsThatAreUsedByThisEquationLhs)) {
			System.out.println("OOPS!");
		}
		
		
//		dumpUsesUsedByMap("InitialLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
//		dumpUsesUsedByMap("InitialRhsUsedByLhs.txt", equationsThatAreUsedByThisEquationLhs, additionalLevel);
		
//		dumpUsesUsedByMap("StockLhsUsesRhs.txt", stockLhsKeyUsesTheseEquations, additionalLevel);
//		dumpUsesUsedByMap("StockRhsUsedByLhs.txt", stockEquationsThatAreUsedByThisEquationLhs, additionalLevel);
		
		List<String> initializers = byVariableType.get(INITIALIZER);
		
		// can we order these?
		
		// 1) create an initial available list with lhs with empty lhsUsesRhs sets
		for (String key : lhsKeyUsesTheseEquations.keySet()) {
			if (lhsKeyUsesTheseEquations.get(key).size() == 0) {
				orderedListOfAvailableVariables.add(key);
				if (TRACE_ACTIONS)
					System.out.println("Adding initially available: "+key);
				if (initializers.contains(key)) {
					initializers.remove(key);
				}
			}
		}
		String lastInitial = orderedListOfAvailableVariables.get(orderedListOfAvailableVariables.size()-1);
		
		if (TRACE_ACTIONS)
			System.out.println("Initial available list contains "+orderedListOfAvailableVariables.size()+" entries");
		allEquationsWaitingForRelease = new LinkedHashSet<String>(lhsKeyUsesTheseEquations.keySet());
		int ptr = 0;
		
	
		
		while (ptr < orderedListOfAvailableVariables.size()) {
			int numFreed = 0;
			// this is a list of of vars freed by the single variable
			List<String> freed = new ArrayList<String>();

			String availVar = orderedListOfAvailableVariables.get(ptr);
			
			if (TRACE_ACTIONS)
				System.out.println("\nAvailable "+ptr+" "+availVar+"\n");
			allEquationsWaitingForRelease.remove(availVar);
			

			// remove availVar from lhsUsesRhs value and if size goes to zero, append to available list
			// lhsUsesRhs as a "waiting for" dataset
			// rhsUsedByLsh is a "who do I tell I am available"
			Set<String> lhsUsingAvailVar = equationsThatAreUsedByThisEquationLhs.get(availVar);
			if (lhsUsingAvailVar == null) {
				ptr++;
				continue;
			}

			for (String aLHS : lhsUsingAvailVar) {
				Set<String> iUse = lhsKeyUsesTheseEquations.get(aLHS);
				if (iUse.contains(availVar)) {
					iUse.remove(availVar);
//					System.out.println(availVar+" removed from lhsUsesRhs for "+aLHS+" remaining "+iUse.size());
				} else {
					// 2/3 System.out.println("should have availVar "+availVar+" entry in lhsUsesRhs for lhs "+aLHS);
				}

				if (iUse.size() == 0) {
					if (TRACE_ACTIONS)
						System.out.println(">>>>>>>>>>>>>>>"+availVar+" -> "+aLHS+" use size = 0");

					// aLHS is now free. We must update lhsUsesRhs
					if (!orderedListOfAvailableVariables.contains(aLHS)) {
						numFreed++;

						freed.add(aLHS);

						if ((InformationManagers.getInstance().getEquationManager().getAnEquationFor(aLHS).isStock() ||
								InformationManagers.getInstance().getEquationManager().getAnEquationFor(aLHS).isActiveInitial())) {

							// this may be ok if all its required variables have been ordered
							orderedListOfAvailableVariables.add(aLHS);
							if (initializers.contains(aLHS)) {
								initializers.remove(aLHS);
								mergeStockArgs(aLHS, lhsKeyUsesTheseEquations, equationsThatAreUsedByThisEquationLhs, stockLhsKeyUsesTheseEquations, stockEquationsThatAreUsedByThisEquationLhs);
							}
						} else {
							if (TRACE_ACTIONS)
								System.out.println("############Adding "+aLHS+" to available list");
							orderedListOfAvailableVariables.add(aLHS);
							if (initializers.contains(aLHS)) {
								initializers.remove(aLHS);
							}
						}
					} else {

					}
				}
				// we've added one variable to available
				// now lets cascade through
				int fptr = 0;
				while (fptr < freed.size()) {
					String fr = freed.get(fptr++);
					lhsUsingAvailVar = equationsThatAreUsedByThisEquationLhs.get(fr);
					if (lhsUsingAvailVar != null) {
						for ( String faLHS : lhsUsingAvailVar) {
							Set<String> fiUse = lhsKeyUsesTheseEquations.get(faLHS);
							if (fiUse.contains(fr)) {
								if (TRACE_ACTIONS)
									System.out.println("FREED %%%%%%%%%%%%%%% "+fr+ " removed from lhsUsesRhs for "+faLHS+" remaining "+fiUse.size());
								fiUse.remove(fr);
							} 

							if (fiUse.size() == 0) {
								if (!orderedListOfAvailableVariables.contains(faLHS)) {
									if (TRACE_ACTIONS)
										System.out.println("FREED >>>>>>>>>>>>>>>"+fr+" -> "+faLHS+" use size = 0");
									freed.add(faLHS);

									if (TRACE_ACTIONS)
										System.out.println("FREED ############Adding "+faLHS+" to available list");
									orderedListOfAvailableVariables.add(faLHS);
								}
							}
						}
					}
				}
			}
			ptr++;
		}
		
		System.out.println("Number remaining after order: "+allEquationsWaitingForRelease.size());
		System.out.println("Remaining: "+allEquationsWaitingForRelease);
		System.out.println("Number in available: "+orderedListOfAvailableVariables.size());
		
		if (allEquationsWaitingForRelease.size() == 0)
			cleanOrderOfAvailableVariables = true;
		else
			cleanOrderOfAvailableVariables = false; 
		
		additionalLevel = true;
		dumpUsesUsedByMap("FinalLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
		
		// let's look at variable reference counts for what's left
		
		Map<String, MutableInteger> refCounts = new LinkedHashMap<String, MutableInteger>();
		for (String v : lhsKeyUsesTheseEquations.keySet()) {
			refCounts.put(v,  new MutableInteger(0));
		}
		for (String v : lhsKeyUsesTheseEquations.keySet()) {
			Set<String> aSet = lhsKeyUsesTheseEquations.get(v);

			for (String rhsVar : aSet) {
				if (!refCounts.containsKey(rhsVar)) {
					//No ref count for Successful Attacks[a!,p] i.e. not used as key in lhsUsesRhs
					// 2/3 System.out.println("No ref count for "+rhsVar+" i.e. not used as key in lhsUsesRhs");
				} else {
					refCounts.get(rhsVar).add(1);
				}
			}

		}
		
		for (String s : refCounts.keySet()) {
			if (refCounts.get(s).value() > 0)
			System.out.println("Var: "+s+" rhs Count: "+refCounts.get(s));
		}
		
		
		return lastInitial;
//		System.out.println("STOP HERE");
//		Map<String, Set<String>> connections = analyzeLhsUsesRhs(lhsUsesRhs);
//		dumpConnections("Connections.txt", connections);
//		analyzeConnections(connections);
		
	}
	
	public String newNewLoadImplementation() {
		
		System.out.println("\nNew New Load Implementation VDP");
		
		Equation eqn = null;
		
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		EquationManager eqnMgr = InformationManagers.getInstance().getEquationManager();
		
		NamedSubscriptManager nsm = InformationManagers.getInstance().getNamedSubscriptManager();
		
		// this is the true lhs of the equation
		// lhsUsesRhs this maps the actual lhs to the rhs variables for this specific equation
		
		// The use of "equation" in data structure naming indicates that they are lhs of an equation and thus will have subscripts if an array
		// The use of "variable" in data structure naming indicates that they have any subscripts removed leaving just the variable name
		
		Map<String, Set<String>> lhsKeyUsesTheseEquations = new LinkedHashMap<String, Set<String>>();
		Map<String, Set<String>> stockLhsKeyUsesTheseEquations = new LinkedHashMap<String, Set<String>>();
		// this is just a map going the other direction
		Map<String, Set<String>> equationsThatAreUsedByThisEquationLhs = new LinkedHashMap<String, Set<String>>();
		Map<String, Set<String>> stockEquationsThatAreUsedByThisEquationLhs = new LinkedHashMap<String, Set<String>>();
		
		Set<String> allEquationsWaitingForRelease;
		
		lhsKeyUsesTheseEquations.put("time", new LinkedHashSet<String>());
		lhsKeyUsesTheseEquations.put("timeStep", new LinkedHashSet<String>());
		
		dumpVariableReferences("references.txt", InformationManagers.getInstance().getSdObjectManager().getVariableReferences());
		List<String> variables = new ArrayList<String>(InformationManagers.getInstance().getSdObjectManager().getVariables());
		Set<String> terminalCheckCompleted = new LinkedHashSet<String>();
		
		categorizeVariables(variables);
		
		Map<String, Map<String, Set<String>>> refs = InformationManagers.getInstance().getSdObjectManager().getVariableReferences();
		
		int numberOfEquations = 0;
		
		for (String referenceKey : refs.keySet()) {

			// these are the forms in which this variable appears on lhs and rhs 
			// initialize data structure with all equation LHS
			// additional "equations" added later
			
			Set<String> keyVariableLhsReferences = refs.get(referenceKey).get(SDObjectManager.LHS);
			for (String lhsReference : keyVariableLhsReferences) {
				eqn = equations.get(ArrayReference.annotate(lhsReference));
				
				if (eqn == null) {
					System.out.println("MISSING EQN: "+lhsReference);
				}
				if (eqn.isDefinesSubscript())
					continue;
				
//				System.out.println("LHSR1: "+lhsReference);
				lhsKeyUsesTheseEquations.put(lhsReference, new LinkedHashSet<String>());
				if (eqn.isStock() || eqn.isActiveInitial()) {
					stockLhsKeyUsesTheseEquations.put(lhsReference, new LinkedHashSet<String>());
				}
			}
		}
		
		// process each variable
		
		for (String referenceKey : refs.keySet()) {

			// these are the forms in which this variable appears on lhs and rhs 
			Set<String> keyVariableLhsReferences = refs.get(referenceKey).get(SDObjectManager.LHS);
			Set<String> keyVariableRhsUseReferences = refs.get(referenceKey).get(SDObjectManager.RHS_USE);   
			
			Set<String> missingLHS = Utilities.minus(keyVariableRhsUseReferences, keyVariableLhsReferences);
			
			if (missingLHS.size() > 0) {
				System.out.println("Missing for "+referenceKey);
				System.out.println("miss: "+missingLHS);
				System.out.println("have: "+keyVariableLhsReferences);
				for (String missing : missingLHS) {
//					lhsKeyUsesTheseEquations.put(ArrayReference.annotate(missing),new LinkedHashSet<String>());
					System.out.println("LHSR2: "+missing);
					lhsKeyUsesTheseEquations.put(missing,new LinkedHashSet<String>());
					
					// given missing, find actual lhs that defines this subscript
					Set<String> defs = nsm.getDefiningSubscripts(missing, keyVariableLhsReferences);
					lhsKeyUsesTheseEquations.get(missing).addAll(nsm.getDefiningSubscripts(missing, keyVariableLhsReferences));
					
				}
			}
			
			// these are the variables that are found on the rhs of the equation
			Set<String> keyVariableRhsReferences = refs.get(referenceKey).get(SDObjectManager.RHS);
			
			if (referenceKey.equals("Cigarette smokers"))
				System.out.println("DEBUG cigs");
			
			for (String lhsReference : keyVariableLhsReferences) {
				
				eqn = equations.get(ArrayReference.annotate(lhsReference));
				
				if (eqn.isDefinesSubscript())
					continue;
				
				numberOfEquations++;
				Set<String> variablesNeedingEquations;	
				variablesNeedingEquations = eqn.getRHSVariablesForOrdering(/*true*/);
				
				for (String variableOfInterest : variablesNeedingEquations) {
					String oname = Parser.removeAnnotation(ndtm.getOriginalName(variableOfInterest).replace("!]", "]"));
//					System.out.println("LHSR3: "+lhsReference+" -> "+oname);
					lhsKeyUsesTheseEquations.get(lhsReference).add(oname);
					if (!equationsThatAreUsedByThisEquationLhs.containsKey(oname)) {
						equationsThatAreUsedByThisEquationLhs.put(oname, new LinkedHashSet<String>());
					}
//					System.out.println("LHSR4: "+oname+" -> "+lhsReference);
					equationsThatAreUsedByThisEquationLhs.get(oname).add(lhsReference);
				}
				
				if (eqn.isStock() || eqn.isActiveInitial()) {
					variablesNeedingEquations = eqn.getStockArg1Variables();

					for (String variableOfInterest : variablesNeedingEquations) {
						String oname = Parser.removeAnnotation(ndtm.getOriginalName(variableOfInterest));
						// add rhs varaibles to sets
						Set<String> aSet = stockLhsKeyUsesTheseEquations.get(lhsReference);
						String rhsVar = ndtm.getOriginalName(variableOfInterest).replace("!]","]").replace("!,",",");
						if (!rhsVar.equals(variableOfInterest)) {
							aSet.add(rhsVar);

							if (!stockEquationsThatAreUsedByThisEquationLhs.containsKey(rhsVar))
								stockEquationsThatAreUsedByThisEquationLhs.put(rhsVar,  new LinkedHashSet<String>());
							Set<String> aSet2 = stockEquationsThatAreUsedByThisEquationLhs.get(rhsVar);
							if (!rhsVar.equals(lhsReference))
								aSet2.add(lhsReference);
						}
					}
				}
				
			}


		}

		System.out.println("Number of Equations for ordering: "+numberOfEquations);
		System.out.println("Number of lhsUsesRhs entries: "+lhsKeyUsesTheseEquations.size());
		System.out.println("Number of rhsUsedByLhs entries: "+equationsThatAreUsedByThisEquationLhs.size());

		
		boolean additionalLevel = false;
		dumpUsesUsedByMap("BeforeValidationLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
		dumpUsesUsedByMap("BeforeValidationRhsUsedByLhs.txt", equationsThatAreUsedByThisEquationLhs, additionalLevel);
		
		// this may add some additional info to lhsUsesRhs
		if (!validate(lhsKeyUsesTheseEquations, equationsThatAreUsedByThisEquationLhs)) {
			System.out.println("OOPS!");
		}
		
		
		dumpUsesUsedByMap("InitialLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
		dumpUsesUsedByMap("InitialRhsUsedByLhs.txt", equationsThatAreUsedByThisEquationLhs, additionalLevel);
		
		dumpUsesUsedByMap("StockLhsUsesRhs.txt", stockLhsKeyUsesTheseEquations, additionalLevel);
		dumpUsesUsedByMap("StockRhsUsedByLhs.txt", stockEquationsThatAreUsedByThisEquationLhs, additionalLevel);
		
		List<String> initializers = byVariableType.get(INITIALIZER);
		
		// can we order these?
		
		// 1) create an initial available list with lhs with empty lhsUsesRhs sets
		for (String key : lhsKeyUsesTheseEquations.keySet()) {
			if (lhsKeyUsesTheseEquations.get(key).size() == 0) {
				orderedListOfAvailableVariables.add(key);
				if (TRACE_ACTIONS)
					System.out.println("Adding initially available: "+key);
				if (initializers.contains(key)) {
					initializers.remove(key);
				}
			}
		}
		String lastInitial = orderedListOfAvailableVariables.get(orderedListOfAvailableVariables.size()-1);
		
		if (TRACE_ACTIONS)
			System.out.println("Initial available list contains "+orderedListOfAvailableVariables.size()+" entries");
		allEquationsWaitingForRelease = new LinkedHashSet<String>(lhsKeyUsesTheseEquations.keySet());
		int ptr = 0;
		
	
		
		while (ptr < orderedListOfAvailableVariables.size()) {
			int numFreed = 0;
			// this is a list of of vars freed by the single variable
			List<String> freed = new ArrayList<String>();

			String availVar = orderedListOfAvailableVariables.get(ptr);
			
			if (TRACE_ACTIONS)
				System.out.println("\nAvailable "+ptr+" "+availVar+"\n");
			allEquationsWaitingForRelease.remove(availVar);
			

			// remove availVar from lhsUsesRhs value and if size goes to zero, append to available list
			// lhsUsesRhs as a "waiting for" dataset
			// rhsUsedByLsh is a "who do I tell I am available"
			Set<String> lhsUsingAvailVar = equationsThatAreUsedByThisEquationLhs.get(availVar);
			if (lhsUsingAvailVar == null) {
				ptr++;
				continue;
			}

			for (String aLHS : lhsUsingAvailVar) {
				Set<String> iUse = lhsKeyUsesTheseEquations.get(aLHS);
				if (iUse.contains(availVar)) {
					iUse.remove(availVar);
//					System.out.println(availVar+" removed from lhsUsesRhs for "+aLHS+" remaining "+iUse.size());
				} else {
					// 2/3 System.out.println("should have availVar "+availVar+" entry in lhsUsesRhs for lhs "+aLHS);
				}

				if (iUse.size() == 0) {
					if (TRACE_ACTIONS)
						System.out.println(">>>>>>>>>>>>>>>"+availVar+" -> "+aLHS+" use size = 0");

					// aLHS is now free. We must update lhsUsesRhs
					if (!orderedListOfAvailableVariables.contains(aLHS)) {
						numFreed++;

						freed.add(aLHS);

						if ((InformationManagers.getInstance().getEquationManager().getAnEquationFor(aLHS).isStock() ||
								InformationManagers.getInstance().getEquationManager().getAnEquationFor(aLHS).isActiveInitial())) {

							// this may be ok if all its required variables have been ordered
							orderedListOfAvailableVariables.add(aLHS);
							if (initializers.contains(aLHS)) {
								initializers.remove(aLHS);
								mergeStockArgs(aLHS, lhsKeyUsesTheseEquations, equationsThatAreUsedByThisEquationLhs, stockLhsKeyUsesTheseEquations, stockEquationsThatAreUsedByThisEquationLhs);
							}
						} else {
							if (TRACE_ACTIONS)
								System.out.println("############Adding "+aLHS+" to available list");
							orderedListOfAvailableVariables.add(aLHS);
							if (initializers.contains(aLHS)) {
								initializers.remove(aLHS);
							}
						}
					} else {

					}
				}
				// we've added one variable to available
				// now lets cascade through
				int fptr = 0;
				while (fptr < freed.size()) {
					String fr = freed.get(fptr++);
					lhsUsingAvailVar = equationsThatAreUsedByThisEquationLhs.get(fr);
					if (lhsUsingAvailVar != null) {
						for ( String faLHS : lhsUsingAvailVar) {
							Set<String> fiUse = lhsKeyUsesTheseEquations.get(faLHS);
							if (fiUse.contains(fr)) {
								if (TRACE_ACTIONS)
									System.out.println("FREED %%%%%%%%%%%%%%% "+fr+ " removed from lhsUsesRhs for "+faLHS+" remaining "+fiUse.size());
								fiUse.remove(fr);
							} 

							if (fiUse.size() == 0) {
								if (!orderedListOfAvailableVariables.contains(faLHS)) {
									if (TRACE_ACTIONS)
										System.out.println("FREED >>>>>>>>>>>>>>>"+fr+" -> "+faLHS+" use size = 0");
									freed.add(faLHS);

									if (TRACE_ACTIONS)
										System.out.println("FREED ############Adding "+faLHS+" to available list");
									orderedListOfAvailableVariables.add(faLHS);
								}
							}
						}
					}
				}
			}
			ptr++;
		}
		
		System.out.println("Number remaining after order: "+allEquationsWaitingForRelease.size());
		System.out.println("Remaining: "+allEquationsWaitingForRelease);
		System.out.println("Number in available: "+orderedListOfAvailableVariables.size());
		
		if (allEquationsWaitingForRelease.size() == 0)
			cleanOrderOfAvailableVariables = true;
		else
			cleanOrderOfAvailableVariables = false; 
		
		additionalLevel = true;
		dumpUsesUsedByMap("FinalLhsUsesRhs.txt", lhsKeyUsesTheseEquations, additionalLevel);
		
		// let's look at variable reference counts for what's left
		
		Map<String, MutableInteger> refCounts = new LinkedHashMap<String, MutableInteger>();
		for (String v : lhsKeyUsesTheseEquations.keySet()) {
			refCounts.put(v,  new MutableInteger(0));
		}
		for (String v : lhsKeyUsesTheseEquations.keySet()) {
			Set<String> aSet = lhsKeyUsesTheseEquations.get(v);

			for (String rhsVar : aSet) {
				if (!refCounts.containsKey(rhsVar)) {
					//No ref count for Successful Attacks[a!,p] i.e. not used as key in lhsUsesRhs
					// 2/3 System.out.println("No ref count for "+rhsVar+" i.e. not used as key in lhsUsesRhs");
				} else {
					refCounts.get(rhsVar).add(1);
				}
			}

		}
		
		for (String s : refCounts.keySet()) {
			if (refCounts.get(s).value() > 0)
			System.out.println("Var: "+s+" rhs Count: "+refCounts.get(s));
		}
		
		return lastInitial;
//		System.out.println("STOP HERE");
//		Map<String, Set<String>> connections = analyzeLhsUsesRhs(lhsUsesRhs);
//		dumpConnections("Connections.txt", connections);
//		analyzeConnections(connections);
		
	}
	
	
	private void mergeStockArgs(String key, Map<String, Set<String>> lhsUsesRhs,
			Map<String, Set<String>> rhsUsedByLhs,
			Map<String, Set<String>> stockLhsUsesRhs,
			Map<String, Set<String>> stocRrhsUsedByLhs) {
		
		// we know delayedOrdering stocks are ready to go.
		
		
				Set<String> rhs = stockLhsUsesRhs.get(key);
//				if (orderedListOfAvailableVariables.contains(key)) {
//					// can I just remove it from orderedListOfAvailableVariables???
//					System.out.println("How did this become available? "+key);
//				}
				if (rhs != null) {
					for (String r : rhs) {
						// if we haven't already processed it, add it.
						if (!orderedListOfAvailableVariables.contains(r)) {
							Set<String> set = lhsUsesRhs.get(key);
							set.add(r);
							if (!rhsUsedByLhs.containsKey(r))
								rhsUsedByLhs.put(r, new LinkedHashSet<String>());
							set = rhsUsedByLhs.get(r);
							set.add(key);
						}
					}
				}
				orderedListOfAvailableVariables.add(key);
				System.out.println("Merge adds "+key+" to available");
				if (stockLhsUsesRhs.containsKey(key))
					stockLhsUsesRhs.remove(key);
				
			return;
	
		// all the initializers have been ordered
		// we now need to add stock arg1 dependencies to our data structures
		
//		orderedListOfAvailableVariables
		
//		for (String key : stockLhsUsesRhs.keySet()) {
//			Set<String> rhs = stockLhsUsesRhs.get(key);
//			if (orderedListOfAvailableVariables.contains(key)) {
//				// can I just remove it from orderedListOfAvailableVariables???
//				System.out.println("How did this become available? "+key);
//			}
//			for (String r : rhs) {
//				// if we haven't already processed it, add it.
//				if (!orderedListOfAvailableVariables.contains(r)) {
//					lhsUsesRhs.get(key).add(r);
//					if (!rhsUsedByLhs.containsKey(r))
//						rhsUsedByLhs.put(r, new LinkedHashSet<String>());
//					rhsUsedByLhs.get(r).add(key);
//				}
//			}
//		}
		
		
		
	}

	private boolean setsEqual(Set<String> set1, Set<String> set2) {
		Set<String> tempSet1 = new LinkedHashSet<String>(set1);
		Set<String> tempSet2 = new LinkedHashSet<String>(set2);
		
		if (tempSet1.size() == tempSet2.size()) {
			tempSet1.removeAll(tempSet2);
			return tempSet1.size() == 0;
		}
		
		return false;
	}
	
	private boolean validate(Map<String, Set<String>> lhsUsesRhs, Map<String, Set<String>> rhsUsedByLhs) {
		
		// make sure that each rshVar in lhsUsesRhs has an associated lhs in lhsUsesRhs
		boolean valid = true;
		
		Set<String> lhsKeySetExpanded = new LinkedHashSet<String>(); 
		for (String lhs : lhsUsesRhs.keySet()) {
			lhsKeySetExpanded.add(lhs);
//			if (ArrayReference.isArrayReference(lhs)) {
//				lhsKeySetExpanded.add(new ArrayReference(lhs).getArrayName());
//			}
		}
		
		Set<String> rhsKeySetExpanded = new LinkedHashSet<String>(); 
		for (String rhs : rhsUsedByLhs.keySet()) {
			rhsKeySetExpanded.add(rhs.replace("!]", "]"));
//			if (ArrayReference.isArrayReference(rhs)) {
//				rhsKeySetExpanded.add(new ArrayReference(rhs).getArrayName());
//			}
		}
				
		Map<String, Set<String>> lhsUsesRhsToAdd = new LinkedHashMap<String, Set<String>>();
				
		for (String lhs : lhsUsesRhs.keySet()) {
			Set<String> uses = lhsUsesRhs.get(lhs);
			if (uses.size() == 0)
				continue;
			
			String justVariable = null;
			for (String rhs : uses) {
				String rhsKey = rhs.replace("!]", "]");
//				if (ArrayReference.isArrayReference(rhs)) {
//					rhsKey = new ArrayReference(rhs).getArrayName();
//				}
				if (!lhsKeySetExpanded.contains(rhsKey)) {
					if (ArrayReference.isArrayReference(rhsKey)) {
						justVariable = new ArrayReference(rhsKey).getArrayName();
//						for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(justVariable)) {
//							System.out.println(eqn.getVensimEquationOnly());
//						}
						Map<String, Set<String>> ref = InformationManagers.getInstance().getSdObjectManager().
								getVariableReferences(getVariableOrArrayName(rhsKey));

						// these are the forms in which this variable appears on lhs and rhs 
						Set<String> lhsRef = ref.get(SDObjectManager.LHS);
						Set<String> rhsRef = ref.get(SDObjectManager.RHS);
						if (lhsRef.size() > 1) {
//							System.out.println("need to add array init entries: "+rhs+" rhsKey: "+rhsKey+" lhs: "+lhs);
							// rhsKey becomes lhs, all lhs in set enter a fabricated rhs set
							if (!lhsUsesRhsToAdd.containsKey(rhsKey)) {
								lhsUsesRhsToAdd.put(rhsKey, new LinkedHashSet<String>());
								lhsUsesRhsToAdd.get(rhsKey).addAll(lhsRef);
							} else {
//								System.out.println("ALREADY THERE ?need to add array init entries: "+rhs+" rhsKey: "+rhsKey+" lhs: "+lhs);
							}
						} else {
							System.out.println("\n");
							for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(justVariable)) {
								System.out.println(eqn.getVensimEquationOnly());
							}
							System.out.println("#####RSH with no LHS in lhsUsesRhs rhs: "+rhs+" rhsKey: "+rhsKey+" lhs: "+lhs);
							System.out.println("\n");
							valid = false;
						}
					} else {
						System.out.println("\n");
						for (Equation eqn : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(rhsKey)) {
							System.out.println(eqn.getVensimEquationOnly());
						}
						System.out.println("RSH with no LHS in lhsUsesRhs rhs: "+rhs+" rhsKey: "+rhsKey+" lhs: "+lhs);
						System.out.println("\n");
						valid = false;
					}
					
				}
				// is there a corresponding rhsUsedByLhs entry for this variable?
				// this is the mechanism through which variables become available
				
				if (!rhsKeySetExpanded.contains(rhsKey)) {
					System.out.println("RSH with no LHS in rhsUsedByRhs rhs: "+rhs+" rhsKey: "+rhsKey+" lhs: "+lhs);
					valid = false;
				}
			}
		}
		
//		for (String key : lhsUsesRhsToAdd.keySet()) {
//			lhsUsesRhs.put(key, new LinkedHashSet<String>());
//			lhsUsesRhs.get(key).addAll(lhsUsesRhsToAdd.get(key));
//			System.out.println("ADD: "+key+" rhs "+lhsUsesRhsToAdd.get(key));
//			for (String s : lhsUsesRhsToAdd.get(key)) {
//				if (!rhsUsedByLhs.containsKey(s)) {
//					rhsUsedByLhs.put(s, new LinkedHashSet<String>());
//				}
//					rhsUsedByLhs.get(s).add(key);
//			}
//		}
		
		for (String key : lhsUsesRhsToAdd.keySet()) {
			for (String s : lhsUsesRhsToAdd.get(key)) {
				lhsUsesRhs.put(s, new LinkedHashSet<String>());
				lhsUsesRhs.get(s).add(key);
				if (!rhsUsedByLhs.containsKey(s)) {
					rhsUsedByLhs.put(s, new LinkedHashSet<String>());
				}
					rhsUsedByLhs.get(s).addAll(lhsUsesRhsToAdd.get(key));
			}
//			lhsUsesRhs.put(key, new LinkedHashSet<String>());
//			lhsUsesRhs.get(key).addAll(lhsUsesRhsToAdd.get(key));
			System.out.println("ADD: "+key+" rhs "+lhsUsesRhsToAdd.get(key));
//			for (String s : lhsUsesRhsToAdd.get(key)) {
//				if (!rhsUsedByLhs.containsKey(s)) {
//					rhsUsedByLhs.put(s, new LinkedHashSet<String>());
//				}
//					rhsUsedByLhs.get(s).add(key);
//			}
		}
		
		return valid;
	}
	
	private void dumpConnections(String file, Map<String, Set<String>> connections) {
		
		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+file);
		
		try {
			for (String key : connections.keySet()) {
				Set<String> s = connections.get(key);
				for (String key2 : connections.keySet()) {
					if (key2.equals(key))
						continue;
					Set<String> s2 = connections.get(key2);
					Set<String> s3 = intersect(s, s2);
					
					bw.append("\n"+key+" -> "+key2+"\n\n");
					bw.append(getHashsetToDump(key,s));
					bw.append(getHashsetToDump(key2,s2));
					bw.append(getHashsetToDump("Intersect",s3));
				}
			}
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private void analyzeConnections(Map<String, Set<String>> connections) {
		for (String key : connections.keySet()) {
			Set<String> s = connections.get(key);
			for (String key2 : connections.keySet()) {
				if (key2.equals(key))
					continue;
				Set<String> s2 = connections.get(key2);
				Set<String> s3 = intersect(s, s2);
				System.out.println("\n"+key+" -> "+key2+"\n");
				print(key,s);
				print(key2,s2);
				print("Intersect",s3);
			}
		}
	}
	
	private String getHashsetToDump(String s, Set<String> hashset) {
		StringBuilder sb = new StringBuilder();
		sb.append("\n"+s+" <"+hashset.size()+">\n");
		for (String req : hashset) {
			sb.append("   " + req+"\n");
		}
		return sb.toString();
	}
	
	private void print(String s, Set<String> hashset) {
		System.out.println(s+" <"+hashset.size()+">");
		for (String req : hashset) {
			System.out.println("   " + req);
		}
	}
	
	private void expand(String lhs, String s, Map<String, Set<String>> lhsUsesRhs, Map<String, Set<String>> connections) {
		Set<String> set = lhsUsesRhs.get(s);
		for (String s2 : set) {
			if (!connections.get(lhs).contains(s2)) {
				connections.get(lhs).add(s2);
				expand(lhs, s2, lhsUsesRhs, connections);
			}
		}
		
		
	}
	
	private void expand(String lhs, Map<String, Set<String>> lhsUsesRhs, Map<String, Set<String>> connections) {
		Set<String> set = lhsUsesRhs.get(lhs);
		for (String s2 : set) {
			if (!connections.get(lhs).contains(s2)) {
				connections.get(lhs).add(s2);
				expand(lhs, s2, lhsUsesRhs, connections);
			}
		}
		
		
	}
	
	private Map<String, Set<String>> analyzeLhsUsesRhs(Map<String, Set<String>> lhsUsesRhs) {
		
		Map<String, Set<String>> connections = new LinkedHashMap<String, Set<String>>();
		
		for (String lhs : lhsUsesRhs.keySet()) {
			connections.put(lhs, new LinkedHashSet<String>());
			expand(lhs, lhsUsesRhs, connections);
			
		}
		
		Set<String> toRemove = new LinkedHashSet<String>();
		
		for (String s : connections.keySet()) {
			if (connections.get(s).size() == 0)
				toRemove.add(s);
		}
		for (String r : toRemove)
			connections.remove(r);
		return connections;
		
	}
	
	private Set<String> intersect(Set<String> set1, Set<String> set2) {
		Set<String> inter = union(set1, set2);
		inter.retainAll(set1);
		inter.retainAll(set2);
		return inter;


	}

	private Set<String> union(Set<String> set1, Set<String> set2) {
		Set<String> set = new LinkedHashSet<String>(set1);
		set.addAll(set2);
		return set;
	}
	
	private List<Integer> getTerminalSubscripts(ArrayReference arrayReference) {
		
		ArrayManager amgr = InformationManagers.getInstance().getArrayManager();
		List<String> subs = arrayReference.getSubscripts();
		List<Integer> terminalSubs = new ArrayList<Integer>();
		int dim = 0;
		for (String sub : subs) {
			if (amgr.isTerminalSubscript(arrayReference.getArrayName(), sub, dim)) {
				// if there is a rhs reference in which no terminal subscript is used, then we must delay on this variable
				// until all terminals have been
				terminalSubs.add(dim);
				
			}
			dim++;
		}
		return terminalSubs;
	}
	
	public void newFastLoad(StopShortCondition stopShortCondition, boolean loadForOrdering) {
		
		NativeDataTypeManager ndtm = InformationManagers.getInstance().getNativeDataTypeManager();
		// expandedLhsToRealLhsMap maps an expandedLHS to its higher level lhs subscript from which
		// it is expanded. This map remains intact in the algorithm.
		expandedLhsToRealLhsMap = new LinkedHashMap<String, String>();

		// realLhsToExpandedLhsMap maps a realLHS to its expandedLHSs. This map remains intact in the
		// algorithm
		realLhsToExpandedLhsMap = new LinkedHashMap<String, HashSet<String>>();

		// realLhsToExpandedLhsNotInitialized starts as a copy of realLhsToExpandedLhsMap. As expandedLHSs become available
		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
		// the key which means that we do not need to consider this realLHS for processing. IS THIS CORRECT?
		realLhsToExpandedLhsNotInitialized = new LinkedHashMap<String, HashSet<String>>();

		// requiresExpandedLhsExpandedRHS maps expandedLHS to set of rhs expanded variables. Set elements are removed
		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
		// becomes empty.
		LinkedHashMap<String, HashSet<String>> lhsRhsDependency = new LinkedHashMap<String, HashSet<String>>();
		LinkedHashMap<String, HashSet<String>> rhsLhsUses = new LinkedHashMap<String, HashSet<String>>();
//		requiresExpandedLhsExpandedRHS = new LinkedHashMap<String, HashSet<String>>();
//		usesExpandedRhsExpandedLHS = new LinkedHashMap<String, HashSet<String>>();
		
		
		ArrayManager amgr = InformationManagers.getInstance().getArrayManager();
		SDObjectManager equationManager = InformationManagers.getInstance().getSdObjectManager();
		
		lhsRhsDependency.put("time", new LinkedHashSet<String>());
		lhsRhsDependency.put("timeStep", new LinkedHashSet<String>());

		Set<String> terminalCheckCompleted = new LinkedHashSet<String>();

		for (String realLHS0 : equations.keySet()) {
			// for each equation, we need to capture the dependencies i.e. values on the rsh
			// need to be available before the lhs value can be determined
			
			// realLHS0 is the name with annotation e.g. array.armament capacity 1[region]
			// realLHS is the name without annotation as it appears in vensim mdl file e.g. armament capacity 1[region]
			String realLHS = ndtm.getOriginalName(realLHS0);
			
			// create a structure that maps lhs to rhs variables -- String -> Set
			if (!lhsRhsDependency.containsKey(realLHS))
				lhsRhsDependency.put(realLHS, new LinkedHashSet<String>());

			// reference to the equations currently being processed
			Equation currentEquation = equations.get(realLHS0);

			// this is only for assignment statements
			// maybe array initialization
			if (currentEquation.isDefinesSubscript())
				continue;
			
			// when a subscript is a named subscript, it consists of some number of terminal subscripts
			// a terminal subscript cannot be "expanded" anymore i.e. a leaf node on a tree
			// this structure contains only lhs variables. it maps the "expanded version" to the version that
			// appears in the vensim mdl file. If the key is a named subscript, it will map to itself.
			// if the key is a terminal sunscript, it as well as potentially others will map to the named subscript
			// we use the map to itself policy to facilitate the coding of working with the maps
			expandedLhsToRealLhsMap.put(realLHS, realLHS);
			
			// this is a one to many relationship. if a lhs needs to be "expanded" then the value is a set of
			// all possible values that the subscripts can take on. Once again, subscripts map to themselves.
			if (!realLhsToExpandedLhsMap.containsKey(realLHS))
				realLhsToExpandedLhsMap.put(realLHS, new LinkedHashSet<String>());
			realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
			
			// this starts as a copy of realLhsToExpandedLhsMap but changes during execution. When a variable
			// subscript has its requirements met, then we remove the subscript from the value. When the Set of
			// subscripts goes to zero, then the key variable becomes available for use by other variables.
			// once again, we map a variable/subscript to itself.
			if (!realLhsToExpandedLhsNotInitialized.containsKey(realLHS))
				realLhsToExpandedLhsNotInitialized.put(realLHS, new LinkedHashSet<String>());
			realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
			
			// depending on the use of this information, we can at times stop short of full
			// expansion
			if (stopShortCondition.conditionMet(currentEquation))
				continue;

			String aVar = getVariableOrArrayName(realLHS);
			Map<String, Set<String>> refs = InformationManagers.getInstance().getSdObjectManager().getVariableReferences(aVar);
			
			// these are the forms in which this variable appears on lhs and rhs 
			Set<String> lhsRefs = refs.get(SDObjectManager.LHS);
			Set<String> rhsRefs = refs.get(SDObjectManager.RHS);

			// when a terminal subscript appears on the lhs of an equation, we need to
			// handle this differently. All terminql subscripts must be available before the
			// entire array becomes available.
			boolean requiresTerminalProcessing = false;
			String referenceWithoutTerminal = "";
			
			// this is done at the variable level so once we processed a variable regardless
			// of the subscripts, we do not need to check it again since we've completed all
			// processing for it.
			if (!terminalCheckCompleted.contains(aVar)) {
				EquationArrayReferenceStructure ears = currentEquation.getEars();

				if (ears.hasLHSarray()) {
					ArrayReference ar = ears.getLhsArrayReference();
					List<String> subs = ar.getSubscripts();
					List<Integer> terminalSubs = new ArrayList<Integer>();
					int dim = 0;
					for (String sub : subs) {
						if (amgr.isTerminalSubscript(ar.getArrayName(), sub, dim)) {
//							System.out.println("Encountered LHS Terminal Subscript: "+sub+" dim: "+dim+" "+currentEquation);
							// if there is a rhs reference in which no terminal subscript is used, then we must delay on this variable
							// until all terminals have been
							terminalSubs.add(dim);
							
						}
						dim++;
					}

					if (terminalSubs.size() > 0) {
						// need to find the reference that has the non-terminal subscripts
//						if (rhsRefs.size() > 1) {
//							System.out.println("Need to handle");
//						}
						for (String rhsVar0 : rhsRefs) {
							String rhsVar = ndtm.getOriginalName(rhsVar0);
							ArrayReference tar = new ArrayReference(rhsVar);
							List<String> s = tar.getSubscripts();
							dim = 0;
							for (String su : s) {
								if (!amgr.isTerminalSubscript(tar.getArrayName(), su, dim)) {
									if (terminalSubs.contains(dim)) {
										requiresTerminalProcessing = true;
										referenceWithoutTerminal = rhsVar.replace("!]","]").replace("!,", ",");
									}
									dim++;
								}
							}
						}
					}
				}
				if (requiresTerminalProcessing) {
					if (!lhsRhsDependency.containsKey(referenceWithoutTerminal)) {
						lhsRhsDependency.put(referenceWithoutTerminal, new LinkedHashSet<String>());
					}
					if (!realLhsToExpandedLhsMap.containsKey(referenceWithoutTerminal)) {
						realLhsToExpandedLhsMap.put(referenceWithoutTerminal, new LinkedHashSet<String>());
					}
					if (!realLhsToExpandedLhsNotInitialized.containsKey(referenceWithoutTerminal)) {
						realLhsToExpandedLhsNotInitialized.put(referenceWithoutTerminal, new LinkedHashSet<String>());
					}
					for (String r : refs.get(SDObjectManager.LHS)) {
						lhsRhsDependency.get(referenceWithoutTerminal).add(r);
						
						if (!rhsLhsUses.containsKey(r))
							rhsLhsUses.put(r,  new LinkedHashSet<String>());
						rhsLhsUses.get(r).add(referenceWithoutTerminal);
//						expandedLhsToRealLhsMap.put(r, referenceWithoutTerminal);
						realLhsToExpandedLhsMap.get(referenceWithoutTerminal).add(r);
						realLhsToExpandedLhsNotInitialized.get(referenceWithoutTerminal).add(r);
					}

				}
			}
			terminalCheckCompleted.add(aVar);
			// now add the normal dependencies
			Set<String> variablesOfInterest;
			Set<String> variablesOfInterestNotUsed;
			
			if (loadForOrdering) {
				variablesOfInterest = equations.get(realLHS0).getRHSVariablesForOrdering(/*true*/);
				variablesOfInterestNotUsed = equations.get(realLHS0).getRHSVariablesForOrderingActiveInitial();
			} else {
				variablesOfInterest = equations.get(realLHS0).getRHSVariablesForOrderingActiveInitial();
				variablesOfInterestNotUsed = equations.get(realLHS0).getRHSVariablesForOrdering(/*true*/);
			}
			
			for (String rhsVar : variablesOfInterest) {
				HashSet<String> aSet = lhsRhsDependency.get(realLHS);
				String var = ndtm.getOriginalName(rhsVar).replace("!]","]");
				aSet.add(var);
				
				if (!rhsLhsUses.containsKey(var))
					rhsLhsUses.put(var,  new LinkedHashSet<String>());
				HashSet<String> aSet2 = rhsLhsUses.get(var);
				
				aSet2.add(realLHS);
				
			}
		}
		
		requiresExpandedLhsExpandedRhs = lhsRhsDependency;
		usesExpandedRhsExpandedLhs = rhsLhsUses;
		dumpRequires("NewFastLoadLhsRhsDependency.txt", lhsRhsDependency);
		dumpRequires("NewFastLoadRhsLhsUses.txt", usesExpandedRhsExpandedLhs);
		
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			dumpRequires2(fileNameHead+"expandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
			dumpRequires(fileNameHead+"realLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
			boolean startChain = false; // want this to be false for production
			dumpRequiresExpanded(fileNameHead+"initialrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRhs, startChain); // HerelhsExpandedNotInitialized
			dumpRequiresExpanded(fileNameHead+"initialrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
		}

		analyzeDependencies(fileNameHead,requiresExpandedLhsExpandedRhs, realLhsToExpandedLhsNotInitialized, expandedLhsToRealLhsMap,
				realLhsToExpandedLhsMap, equations);
	}
	
	public void fastLoad(boolean forOrdering) {
		
		ArrayManager amgr = InformationManagers.getInstance().getArrayManager();
		SDObjectManager equationManager = InformationManagers.getInstance().getSdObjectManager();

		// requiresLhsScalarExpandedRhs maps a non-array realLHS to all rhs variables expanded. This map
		// is modified in the algorithm. The key is removed when the rhs hashset is
		// empty. This signifies that realLHS can be inserted into the evaluation order list.
//		requiresScalarLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();

		// expandedLhsToRealLhsMap maps an expandedLHS to its higher level lhs subscript from which
		// it is expanded. This map remains intact in the algorithm.
		expandedLhsToRealLhsMap = new LinkedHashMap<String, String>();

		// realLhsToExpandedLhsMap maps a realLHS to its expandedLHSs. This map remains intact in the
		// algorithm
		realLhsToExpandedLhsMap = new LinkedHashMap<String, HashSet<String>>();

		// realLhsToExpandedLhsNotInitialized starts as a copy of realLhsToExpandedLhsMap. As expandedLHSs become available
		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
		// the key which means that we do not need to consider this realLHS for processing. IS THIS CORRECT?
		realLhsToExpandedLhsNotInitialized = new LinkedHashMap<String, HashSet<String>>();

		// requiresExpandedLhsExpandedRHS maps expandedLHS to set of rhs expanded variables. Set elements are removed
		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
		// becomes empty.
		requiresExpandedLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();


		// all realLHS that still require processing.
		List<String> allExpLHS = new ArrayList<String>();

		// for each equation:
		// get the set of variables that are required
		// then walk through the set repeatedly removing those with no requirements after they have been met

		// first make sure that all mapped subscripts have been defined

		// these are system variables that will be defined at run time

//		requiresExpandedLhsExpandedRHS.put("time", new LinkedHashSet<String>());
//		requiresExpandedLhsExpandedRHS.put("timeStep", new LinkedHashSet<String>());

		realLhsToExpandedLhsNotInitialized.put("time", new LinkedHashSet<String>());
		realLhsToExpandedLhsNotInitialized.put("timeStep", new LinkedHashSet<String>());

		realLhsToExpandedLhsMap.put("time", new LinkedHashSet<String>());
		realLhsToExpandedLhsMap.put("timeStep", new LinkedHashSet<String>());
		
		Set<String> terminalCheckCompleted = new LinkedHashSet<String>();

		for (String realLHS : equations.keySet()) {
			
			// reference to the equations currently being processed
			Equation currentEquation = equations.get(realLHS);
			
			// this is only for assignment statements
			// maybe array initialization
			if (currentEquation.isDefinesSubscript())
				continue;
			
			String aVar = getVariableOrArrayName(realLHS);
			Map<String, Set<String>> refs = InformationManagers.getInstance().getSdObjectManager().getVariableReferences(aVar);
			
			boolean requiresTerminalProcessing = false;
			String referenceWithoutTerminal = "";
			if (!terminalCheckCompleted.contains(aVar)) {
				EquationArrayReferenceStructure ears = currentEquation.getEars();

				if (ears.hasLHSarray()) {
					ArrayReference ar = ears.getLhsArrayReference();
					List<String> subs = ar.getSubscripts();
					List<Integer> terminalSubs = new ArrayList<Integer>();
					int dim = 0;
					for (String sub : subs) {
						if (amgr.isTerminalSubscript(ar.getArrayName(), sub, dim)) {
							System.out.println("Encountered LHS Terminal Subscript: "+sub+" dim: "+dim+" "+currentEquation);
							// if there is a rhs reference in which no terminal subscript is used, then we must delay on this variable
							// until all terminals have been
							terminalSubs.add(dim);
							dim++;
						}
					}
					
					if (terminalSubs.size() > 0) {
						// need to find the reference that has the non-terminal subscripts
						Set<String> rhsRefs = refs.get(SDObjectManager.RHS);
						if (rhsRefs.size() > 1) {
							System.out.println("Need to handle");
						}
						for (String rhsVar : rhsRefs) {
							ArrayReference tar = new ArrayReference(rhsVar);
							List<String> s = tar.getSubscripts();
							dim = 0;
							for (String su : s) {
								if (!amgr.isTerminalSubscript(tar.getArrayName(), su, dim)) {
									if (terminalSubs.contains(dim)) {
										requiresTerminalProcessing = true;
										referenceWithoutTerminal = ArrayReference.annotate(rhsVar).replace("!]","]");
									}
									dim++;
								}
							}
						}
					}
				}
			}
			if (requiresTerminalProcessing) {
				if (!requiresExpandedLhsExpandedRhs.containsKey(referenceWithoutTerminal))
					requiresExpandedLhsExpandedRhs.put(referenceWithoutTerminal, new LinkedHashSet<String>());
				for (String r : refs.get(SDObjectManager.LHS)) {
					requiresExpandedLhsExpandedRhs.get(referenceWithoutTerminal).add(r);
				}
				
			}
			terminalCheckCompleted.add(aVar);
			

			

			// all equations will have these data structures defined
			// realLHS is the Vensim version of LHS i.e. no subscripts

			if (!realLhsToExpandedLhsMap.containsKey(realLHS)) {
				realLhsToExpandedLhsMap.put(realLHS, new LinkedHashSet<String>());
				realLhsToExpandedLhsNotInitialized.put(realLHS, new LinkedHashSet<String>());
			}
			allExpLHS.add(realLHS);

			// these will have no dependencies
			// set them up so that they will be made available immediately


			if (!requiresExpandedLhsExpandedRhs.containsKey(realLHS)) {
					requiresExpandedLhsExpandedRhs.put(realLHS, new LinkedHashSet<String>());
			}
				expandedLhsToRealLhsMap.put(realLHS, realLHS);
				realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
				
				realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
				
				Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrdering(/*true*/); // Expanded();

				// Note that there is not an array reference on the LHS, but there can be on the rhs
				// the rhs array reference will require additional work if a range variable appears


				for (String rhsVar : variablesOfInterest) {
					


					//						System.out.println("VOI: "+realLHS+" "+rhsVar);

					// if this is not an array reference, simply put the variable into the requiresExpanded set
					// there is nothing to expand
					if (!ArrayReference.isArrayReference(rhsVar) || ignoreCompleteSubscriptProcessing) {
						
						String rhsVarOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
						
						if (rhsVarOrig == null)
							System.out.println("Huh? rhsVarOrig load rhsVar = "+rhsVar);
						if (!realLHS.equals(rhsVarOrig)) {						
							requiresExpandedLhsExpandedRhs.get(realLHS).add(rhsVarOrig.replace("!", ""));
						}
					}
				}



		}

		// for each RHS set in requiresExpandedLhsExpandedRHS, need to add the eqn LHS of the rhs Item
		/* */

		/* */
		// ############## WRITE_DEBUG_FILES
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			dumpRequires2(fileNameHead+"expandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
			dumpRequires(fileNameHead+"realLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
			boolean startChain = false; // want this to be false for production
			dumpRequiresExpanded(fileNameHead+"initialrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRhs, startChain); // HerelhsExpandedNotInitialized
			dumpRequiresExpanded(fileNameHead+"initialrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
		}

//		analyzeDependencies(fileNameHead,requiresExpandedLhsExpandedRHS, realLhsToExpandedLhsNotInitialized, expandedLhsToRealLhsMap,
//				realLhsToExpandedLhsMap, equations);


	}
	
	public void load(boolean forOrdering, boolean ignoreCompleteSubscriptProcessing) {
		
		ArrayManager amgr = InformationManagers.getInstance().getArrayManager();
		SDObjectManager equationManager = InformationManagers.getInstance().getSdObjectManager();

		// requiresLhsScalarExpandedRhs maps a non-array realLHS to all rhs variables expanded. This map
		// is modified in the algorithm. The key is removed when the rhs hashset is
		// empty. This signifies that realLHS can be inserted into the evaluation order list.
		requiresScalarLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();

		// expandedLhsToRealLhsMap maps an expandedLHS to its higher level lhs subscript from which
		// it is expanded. This map remains intact in the algorithm.
		expandedLhsToRealLhsMap = new LinkedHashMap<String, String>();

		// realLhsToExpandedLhsMap maps a realLHS to its expandedLHSs. This map remains intact in the
		// algorithm
		realLhsToExpandedLhsMap = new LinkedHashMap<String, HashSet<String>>();

		// realLhsToExpandedLhsNotInitialized starts as a copy of realLhsToExpandedLhsMap. As expandedLHSs become available
		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
		// the key which means that we do not need to consider this realLHS for processing. IS THIS CORRECT?
		realLhsToExpandedLhsNotInitialized = new LinkedHashMap<String, HashSet<String>>();

		// requiresExpandedLhsExpandedRHS maps expandedLHS to set of rhs expanded variables. Set elements are removed
		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
		// becomes empty.
		requiresExpandedLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();


		// all realLHS that still require processing.
		List<String> allExpLHS = new ArrayList<String>();

		// for each equation:
		// get the set of variables that are required
		// then walk through the set repeatedly removing those with no requirements after they have been met

		// first make sure that all mapped subscripts have been defined

		// these are system variables that will be defined at run time

//		requiresExpandedLhsExpandedRHS.put("time", new LinkedHashSet<String>());
//		requiresExpandedLhsExpandedRHS.put("timeStep", new LinkedHashSet<String>());

		realLhsToExpandedLhsNotInitialized.put("time", new LinkedHashSet<String>());
		realLhsToExpandedLhsNotInitialized.put("timeStep", new LinkedHashSet<String>());

		realLhsToExpandedLhsMap.put("time", new LinkedHashSet<String>());
		realLhsToExpandedLhsMap.put("timeStep", new LinkedHashSet<String>());
		
		for (String realLHS : equations.keySet()) {
			
			// reference to the equations currently being processed
			Equation currentEquation = equations.get(realLHS);
			
			String aVar = realLHS;
			if (ArrayReference.isArrayReference(realLHS)) {
				aVar = ArrayReference.extractArrayName(aVar);
			}
			
			aVar = ArrayReference.deannotate(aVar);
			
			
			
			EquationArrayReferenceStructure ears = currentEquation.getEars();
			
			if (ears.hasLHSarray()) {
				ArrayReference ar = ears.getLhsArrayReference();
				List<String> subs = ar.getSubscripts();
				int dim = 0;
				for (String sub : subs) {
					if (amgr.isTerminalSubscript(ar.getArrayName(), sub, dim)) {
						System.out.println("Encountered LHS Terminal Subscript: "+sub+" dim: "+dim+" "+currentEquation);
					}
					dim++;
				}
			}
		}

		for (String realLHS : equations.keySet()) {
			
			// reference to the equations currently being processed
			Equation currentEquation = equations.get(realLHS);
			
			String aVar = realLHS;
			if (ArrayReference.isArrayReference(realLHS)) {
				aVar = ArrayReference.extractArrayName(aVar);
			}
			
			aVar = ArrayReference.deannotate(aVar);
			
			Map<String, Set<String>> refs = InformationManagers.getInstance().getSdObjectManager().getVariableReferences(aVar);
			
			// this is only for assignment statements
			// maybe array initialization
			if (currentEquation.isDefinesSubscript())
				continue;

			// all equations will have these data structures defined
			// realLHS is the Vensim version of LHS i.e. no subscripts

			realLhsToExpandedLhsMap.put(realLHS, new LinkedHashSet<String>());
			realLhsToExpandedLhsNotInitialized.put(realLHS, new LinkedHashSet<String>());

			allExpLHS.add(realLHS);

			// these will have no dependencies
			// set them up so that they will be made available immediately

			if(currentEquation.isDefinesLookup() ||
					currentEquation.isArrayInitialization() ||
					currentEquation.isGetExternalData()) {
				// 11/22/2013

				// Note that if not an array references, expandedLHS ends up as realLHS
				if (ignoreCompleteSubscriptProcessing) {

					expandedLhsToRealLhsMap.put(realLHS, realLHS);
					realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
					realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
					requiresExpandedLhsExpandedRhs.put(realLHS, new LinkedHashSet<String>());
				} else {
					for (String expandedLHS : ArrayReference.expand(realLHS)) {
						expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
						realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
						realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);
						requiresExpandedLhsExpandedRhs.put(expandedLHS, new LinkedHashSet<String>());

					}
				}
				// 11/22/2013

				continue;
			}

			if (!ignoreCompleteSubscriptProcessing && ArrayReference.isArrayReference(realLHS)) {
				ArrayReference lhsArrayReference = new ArrayReference(realLHS);

				// need to check if there are exceptions for this assignment
				// if there are, we need to have access to them so that they can
				// be discard from processing
				List<String> exceptions = new ArrayList<String>();

				if (currentEquation.isHasException()) {
					exceptions = currentEquation.getExceptions();
				}

				// Generate all combinations of realLHS expanded
				for (SubscriptCombination outerSub : InformationManagers.getInstance().getArrayManager().
						getSubscriptValueCombinations(lhsArrayReference.getSubscriptsAsArray())) {

					// Here we will skip any subscript combo values that are defined in another equation within a multi-equation definition
					if (exceptions.contains(outerSub.getSubscriptValue())) {
						System.out.println("Skipping "+"[" + outerSub.getSubscriptValue() + "] for "+realLHS);
						continue;
					}

					// Build an array reference based on the subscript value
					String expandedLHS = "array."+lhsArrayReference.getArrayName()+ "[" + outerSub.getSubscriptValue() + "]";

					requiresExpandedLhsExpandedRhs.put(expandedLHS, new LinkedHashSet<String>());

					// allocate data structures for holding rhs references and mapping from expandedLHS -> realLHS
					expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
					realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
					realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);

					// get the set of all rhsVariable reference
					// these are the tokens stored during parsing. Note that we have the ! still there
					


					Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrdering();
//					if (forOrdering &&  containsInitialValueFunction(currentEquation) ) {
//						variablesOfInterest = currentEquation.getFunctionInitialVariables();
//					}
					
					if (variablesOfInterest.contains("Age after renovation"))
						System.out.println("Age after renovation");


					for (String rhsVar : variablesOfInterest) {
						// if this is not an array reference, simply put the variable into the requiresExpanded set
						// there is nothing to expand
						if (!ArrayReference.isArrayReference(rhsVar)) {
							String expRHSOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
							if (!expandedLHS.equals(expRHSOrig)) {
								// special case -- if our rhs var is a subscript but used as a variable, do not print error message
								if (!InformationManagers.getInstance().getArrayManager().isNamedSubscript(expRHSOrig))
									requiresExpandedLhsExpandedRhs.get(expandedLHS).add(expRHSOrig);
							}

						} else {
							// if an array access, we must rip apart the array reference and 
							// insert all subscript values based on outerSub current value

							//create an arrayReference for the RHS 
							// recall that we have a LHS arrayReference already defined
							ArrayReference rhsArrayReference = new ArrayReference(rhsVar);

							// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
							// get a list of these combinations

							// need to deal with rhs array like: X[y!, z]
							// currently doesn't seem to do it right

							boolean printit = false;

							String arrayName = null;
							if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
								String[] rhsExpanded;
								if (rhsArrayReference.hasRangeSubscript()) {
									printit = false;
									rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
									if (printit) {
										System.out.println("################ Range sub: "+rhsVar);
										for (String s : rhsExpanded)
											System.out.println(s);
									}
								} else {
									rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
								}

								List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
								for (SubscriptCombination rangeSub : combos){

									arrayName = rhsArrayReference.getArrayName()+"[";
									if (arrayName.startsWith("lookup."))
										arrayName =arrayName.replace("lookup.", "array.");
									else
										arrayName = "array." + arrayName;
									// we now construct an appropriate arrayReference for each combination
									//									arrayName = "array."+rhsArrayReference.getArrayName()+"[";
									int subNum = 0;

									// for each subscript that appears in this rhsArrayReference
									// we need to determine if it is a range subscript or not.
									// If it is, 
									for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
										if (subNum++ > 0)
											arrayName += ",";
										if (ArrayReference.isRangeSubscript(subscript)) {
											arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
										} else {
											arrayName += outerSub.getSubscriptValue(subscript);
										}

									}
									arrayName += "]";
									if (printit)
										System.out.println("Expanded: "+arrayName);
									if (!expandedLHS.equals(arrayName)) {
										
										requiresExpandedLhsExpandedRhs.get(expandedLHS).add(arrayName);
									}

									if (printit)
										System.out.println(expandedLHS+" LR "+arrayName);
									//									}

								}
							} else {
								// With no range subscript, we just copy over the subscript values that appear in the LHS
								arrayName = rhsArrayReference.getArrayName()+"[";
								if (arrayName.startsWith("lookup."))
									arrayName =arrayName.replace("lookup.", "array.");
								else
									arrayName = "array." + arrayName;
								int subNum = 0;

								// for each subscript that appears in this rhsArrayReference
								// we need to determine if it is a range subscript or not.
								// If it is, 
								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
									if (subNum++ > 0)
										arrayName += ",";

									arrayName += outerSub.getSubscriptValue(subscript);

								}
								arrayName += "]";
								if (!expandedLHS.equals(arrayName)) {
									requiresExpandedLhsExpandedRhs.get(expandedLHS).add(arrayName);
								}

							}
						}

					}
				}
			} else {

				requiresExpandedLhsExpandedRhs.put(realLHS, new LinkedHashSet<String>());
				expandedLhsToRealLhsMap.put(realLHS, realLHS);
				realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
				// MJB HERE
				realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
				Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrdering(/*true*/); // Expanded();
//				if (forOrdering &&  containsInitialValueFunction(currentEquation)) {
//					variablesOfInterest = currentEquation.getFunctionInitialVariables();
//				} 

				// Note that there is not an array reference on the LHS, but there can be on the rhs
				// the rhs array reference will require additional work if a range variable appears


				for (String rhsVar : variablesOfInterest) {
					


					//						System.out.println("VOI: "+realLHS+" "+rhsVar);

					// if this is not an array reference, simply put the variable into the requiresExpanded set
					// there is nothing to expand
					if (!ArrayReference.isArrayReference(rhsVar) || ignoreCompleteSubscriptProcessing) {
						//							InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar));
						String rhsVarOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
						
						if (rhsVarOrig.equals("Age after renovation"))
							System.out.println("Age after renovation");
						if (rhsVarOrig == null)
							System.out.println("Huh? rhsVarOrig load rhsVar = "+rhsVar);
						if (!realLHS.equals(rhsVarOrig)) {
							
							if (ArrayReference.isPartialArrayReference(rhsVarOrig))
								rhsVarOrig = ArrayReference.asCompleteArrayReference(rhsVarOrig);
							rhsVarOrig = ArrayReference.annotate(rhsVarOrig);
							
							requiresExpandedLhsExpandedRhs.get(realLHS).add(rhsVarOrig.replace("!", ""));
						}

					} else {
						// if an array access, we must rip apart the array reference and 
						// insert all subscript values based on outerSub current value

						//create an arrayReference for the RHS 
						// recall that we have a LHS arrayReference already defined
						ArrayReference rhsArrayReference = new ArrayReference(rhsVar);

						// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
						// get a list of these combinations

						// need to deal with rhs array like: X[y!, z]
						// currently doesn't seem to do it right

						boolean printit = false;

						String arrayName = null;
						if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
							String[] rhsExpanded;
							if (rhsArrayReference.hasRangeSubscript()) {
								printit = false;
								rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
								//									System.out.println("################ Scalar Range sub: "+realLHS+" "+rhsVar);
								//									for (String s : rhsExpanded)
								//										System.out.println(s);
							} else {
								rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
							}

							List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
							for (SubscriptCombination rangeSub : combos){

								// we now construct an appropriate arrayReference for each combination
								arrayName = "array."+rhsArrayReference.getArrayName()+"[";
								int subNum = 0;

								// for each subscript that appears in this rhsArrayReference
								// we need to determine if it is a range subscript or not.
								// If it is, 
								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
									if (subNum++ > 0)
										arrayName += ",";
									if (ArrayReference.isRangeSubscript(subscript)) {
										arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
									} else {
										arrayName += subscript;
									}

								}
								arrayName += "]";
								if (printit)
									System.out.println("Expanded: "+arrayName);
								
								if (!realLHS.equals(arrayName)) {
									requiresExpandedLhsExpandedRhs.get(realLHS).add(arrayName);
								}

								if (printit)
									System.out.println(realLHS+" LR "+arrayName);
								//									}

							}
						} else {
							// this is an array reference where the subscript is terminal
							if (rhsVar == null)
								System.out.println("Huh? load rhsVar");
							if (!realLHS.equals(rhsVar)) {
								requiresExpandedLhsExpandedRhs.get(realLHS).add(rhsVar);
							}
						}

					}
				}

			}

		}

		// for each RHS set in requiresExpandedLhsExpandedRHS, need to add the eqn LHS of the rhs Item
		/* */
		for (String expLHS : requiresExpandedLhsExpandedRhs.keySet()) {
			Set<String> aSetExpRHS = requiresExpandedLhsExpandedRhs.get(expLHS);

			// in either case below, no additional processing required
			if (aSetExpRHS == null || aSetExpRHS.size() == 0)
				continue;

			// capture list of all expRHS that were originally in set
			List<String> aList = new ArrayList<String>();
			aList.addAll(aSetExpRHS);

			// for each of these, determine its realLHS and then add to the set
			// when an expRHS is available, it is removed
			// when the entire equation is available the expRHSrealLHS is freed.
			for (String rhs : aList) {
				String rhsRealLHS = expandedLhsToRealLhsMap.get(rhs);
				if (rhsRealLHS == null) {
					if (rhs.equalsIgnoreCase("time") || rhs.equalsIgnoreCase("timestep")) {

					} else {
						// try this.... 11/22/2013
						if (realLhsToExpandedLhsMap.containsKey(rhs)) {
							// we now track the equation in which the others are defined
							
							aSetExpRHS.add(rhsRealLHS);
						} else {
							// special case -- if our rhs var is a subscript but used as a variable, do not print error message
							if (!InformationManagers.getInstance().getArrayManager().isNamedSubscript(rhs))
								System.out.println("(2) VDP How can this be null in expandedLhsToRealLhsMap: rhs: <"+rhs+"> expLHS <"+expLHS+">");
						}
					}
				} else {
					
//					aSetExpRHS.add(rhsRealLHS);
				}
			}
		}
		/* */
		// ############## WRITE_DEBUG_FILES
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			dumpRequires2(fileNameHead+"expandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
			dumpRequires(fileNameHead+"realLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
			boolean startChain = false; // want this to be false for production
			dumpRequiresExpanded(fileNameHead+"initialrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRhs, startChain); // HerelhsExpandedNotInitialized
			dumpRequiresExpanded(fileNameHead+"initialrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
		}

		analyzeDependencies(fileNameHead,requiresExpandedLhsExpandedRhs, realLhsToExpandedLhsNotInitialized, expandedLhsToRealLhsMap,
				realLhsToExpandedLhsMap, equations);


	}
	
	// before trying to bypass array subscript
	
//	public void load(boolean ignoreSubscriptProcessing) {
//
//		// requiresLhsScalarExpandedRhs maps a non-array realLHS to all rhs variables expanded. This map
//		// is modified in the algorithm. The key is removed when the rhs hashset is
//		// empty. This signifies that realLHS can be inserted into the evaluation order list.
//		requiresScalarLhsExpandedRhs = new LinkedHashMap<String, HashSet<String>>();
//
//		// expandedLhsToRealLhsMap maps an expandedLHS to its higher level lhs subscript from which
//		// it is expanded. This map remains intact in the algorithm.
//		expandedLhsToRealLhsMap = new LinkedHashMap<String, String>();
//
//		// realLhsToExpandedLhsMap maps a realLHS to its expandedLHSs. This map remains intact in the
//		// algorithm
//		realLhsToExpandedLhsMap = new LinkedHashMap<String, HashSet<String>>();
//
//		// realLhsToExpandedLhsNotInitialized starts as a copy of realLhsToExpandedLhsMap. As expandedLHSs become available
//		// for reference, they are removed from the HashSet. Once this HashSet becomes empty, we can remove
//		// the key which means that we do not need to consider this realLHS for processing. IS THIS CORRECT?
//		realLhsToExpandedLhsNotInitialized = new LinkedHashMap<String, HashSet<String>>();
//
//		// requiresExpandedLhsExpandedRHS maps expandedLHS to set of rhs expanded variables. Set elements are removed
//		// as they become available for evaluation. The expandedLHS is removed as key when the HashSet
//		// becomes empty.
//		requiresExpandedLhsExpandedRHS = new LinkedHashMap<String, HashSet<String>>();
//
//
//		// all realLHS that still require processing.
//		List<String> allExpLHS = new ArrayList<String>();
//
//		// for each equation:
//		// get the set of variables that are required
//		// then walk through the set repeatedly removing those with no requirements after they have been met
//
//		// first make sure that all mapped subscripts have been defined
//
//		// these are system variables that will be defined at run time
//
//		requiresExpandedLhsExpandedRHS.put("time", new LinkedHashSet<String>());
//		requiresExpandedLhsExpandedRHS.put("timeStep", new LinkedHashSet<String>());
//
//		realLhsToExpandedLhsNotInitialized.put("time", new LinkedHashSet<String>());
//		realLhsToExpandedLhsNotInitialized.put("timeStep", new LinkedHashSet<String>());
//
//		realLhsToExpandedLhsMap.put("time", new LinkedHashSet<String>());
//		realLhsToExpandedLhsMap.put("timeStep", new LinkedHashSet<String>());
//
//		for (String realLHS : equations.keySet()) {
//			
//
//			// reference to the equations currently being processed
//			Equation currentEquation = equations.get(realLHS);
//
//			// this is only for assignment statements
//			// maybe array initialization
//			if (currentEquation.isDefinesSubscript())
//				continue;
//
//			// all equations will have these data structures defined
//			// realLHS is the Vensim version of LHS i.e. no subscripts
//
//			realLhsToExpandedLhsMap.put(realLHS, new LinkedHashSet<String>());
//			realLhsToExpandedLhsNotInitialized.put(realLHS, new LinkedHashSet<String>());
//
//			allExpLHS.add(realLHS);
//
//			// these will have no dependencies
//			// set them up so that they will be made available immediately
//
//			if(currentEquation.isDefinesLookup() ||
//					currentEquation.isArrayInitialization() ||
//					currentEquation.isGetExternalData()) {
//				// 11/22/2013
//
//				// Note that if not an array references, expandedLHS ends up as realLHS
//				for (String expandedLHS : ArrayReference.expand(realLHS)) {
//					expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
//					realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
//					realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);
//					requiresExpandedLhsExpandedRHS.put(expandedLHS, new LinkedHashSet<String>());
//
//				}
//				// 11/22/2013
//
//				continue;
//			}
//
//			if (ArrayReference.isArrayReference(realLHS)) {
//				ArrayReference lhsArrayReference = new ArrayReference(realLHS);
//
//				// need to check if there are exceptions for this assignment
//				// if there are, we need to have access to them so that they can
//				// be discard from processing
//				List<String> exceptions = new ArrayList<String>();
//
//				if (currentEquation.isHasException()) {
//					exceptions = currentEquation.getExceptions();
//				}
//
//				// Generate all combinations of realLHS expanded
//				for (SubscriptCombination outerSub : InformationManagers.getInstance().getArrayManager().
//						getSubscriptValueCombinations(lhsArrayReference.getSubscriptsAsArray())) {
//
//					// Here we will skip any subscript combo values that are defined in another equation within a multi-equation definition
//					if (exceptions.contains(outerSub.getSubscriptValue())) {
//						System.out.println("Skipping "+"[" + outerSub.getSubscriptValue() + "] for "+realLHS);
//						continue;
//					}
//
//					// Build an array reference based on the subscript value
//					String expandedLHS = "array."+lhsArrayReference.getArrayName()+ "[" + outerSub.getSubscriptValue() + "]";
//
//					requiresExpandedLhsExpandedRHS.put(expandedLHS, new LinkedHashSet<String>());
//
//					// allocate data structures for holding rhs references and mapping from expandedLHS -> realLHS
//					expandedLhsToRealLhsMap.put(expandedLHS, realLHS);
//					realLhsToExpandedLhsMap.get(realLHS).add(expandedLHS);
//					realLhsToExpandedLhsNotInitialized.get(realLHS).add(expandedLHS);
//
//					// get the set of all rhsVariable reference
//					// these are the tokens stored during parsing. Note that we have the ! still there
//
//					Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrdering();
////					if (forOrdering &&  containsInitialValueFunction(currentEquation) ) {
////						variablesOfInterest = currentEquation.getFunctionInitialVariables();
////					}
//
//
//					for (String rhsVar : variablesOfInterest) {
//						// if this is not an array reference, simply put the variable into the requiresExpanded set
//						// there is nothing to expand
//						if (!ArrayReference.isArrayReference(rhsVar)) {
//							String expRHSOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
//							if (!expandedLHS.equals(expRHSOrig)) {
//								if (expRHSOrig.contains("[a,p]"))
//									System.out.println("DEBUG requires");
//								requiresExpandedLhsExpandedRHS.get(expandedLHS).add(expRHSOrig);
//							}
//
//						} else {
//							// if an array access, we must rip apart the array reference and 
//							// insert all subscript values based on outerSub current value
//
//							//create an arrayReference for the RHS 
//							// recall that we have a LHS arrayReference already defined
//							ArrayReference rhsArrayReference = new ArrayReference(rhsVar);
//
//							// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
//							// get a list of these combinations
//
//							// need to deal with rhs array like: X[y!, z]
//							// currently doesn't seem to do it right
//
//							boolean printit = false;
//
//							String arrayName = null;
//							if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
//								String[] rhsExpanded;
//								if (rhsArrayReference.hasRangeSubscript()) {
//									printit = false;
//									rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
//									if (printit) {
//										System.out.println("################ Range sub: "+rhsVar);
//										for (String s : rhsExpanded)
//											System.out.println(s);
//									}
//								} else {
//									rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
//								}
//
//								List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
//								for (SubscriptCombination rangeSub : combos){
//
//									arrayName = rhsArrayReference.getArrayName()+"[";
//									if (arrayName.startsWith("lookup."))
//										arrayName =arrayName.replace("lookup.", "array.");
//									else
//										arrayName = "array." + arrayName;
//									// we now construct an appropriate arrayReference for each combination
//									//									arrayName = "array."+rhsArrayReference.getArrayName()+"[";
//									int subNum = 0;
//
//									// for each subscript that appears in this rhsArrayReference
//									// we need to determine if it is a range subscript or not.
//									// If it is, 
//									for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
//										if (subNum++ > 0)
//											arrayName += ",";
//										if (ArrayReference.isRangeSubscript(subscript)) {
//											arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
//										} else {
//											arrayName += outerSub.getSubscriptValue(subscript);
//										}
//
//									}
//									arrayName += "]";
//									if (printit)
//										System.out.println("Expanded: "+arrayName);
//									if (!expandedLHS.equals(arrayName)) {
//										if (arrayName.contains("[a,p]"))
//											System.out.println("DEBUG requires");
//										requiresExpandedLhsExpandedRHS.get(expandedLHS).add(arrayName);
//									}
//
//									if (printit)
//										System.out.println(expandedLHS+" LR "+arrayName);
//									//									}
//
//								}
//							} else {
//								// With no range subscript, we just copy over the subscript values that appear in the LHS
//								arrayName = rhsArrayReference.getArrayName()+"[";
//								if (arrayName.startsWith("lookup."))
//									arrayName =arrayName.replace("lookup.", "array.");
//								else
//									arrayName = "array." + arrayName;
//								int subNum = 0;
//
//								// for each subscript that appears in this rhsArrayReference
//								// we need to determine if it is a range subscript or not.
//								// If it is, 
//								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
//									if (subNum++ > 0)
//										arrayName += ",";
//
//									arrayName += outerSub.getSubscriptValue(subscript);
//
//								}
//								arrayName += "]";
//								if (!expandedLHS.equals(arrayName)) {
//									if (arrayName.contains("[a,p]"))
//										System.out.println("DEBUG requires");
//									requiresExpandedLhsExpandedRHS.get(expandedLHS).add(arrayName);
//								}
//
//							}
//						}
//
//					}
//				}
//			} else {
//
//				requiresExpandedLhsExpandedRHS.put(realLHS, new LinkedHashSet<String>());
//				expandedLhsToRealLhsMap.put(realLHS, realLHS);
//				realLhsToExpandedLhsMap.get(realLHS).add(realLHS);
//				// MJB HERE
//				realLhsToExpandedLhsNotInitialized.get(realLHS).add(realLHS);
//				Set<String> variablesOfInterest = equations.get(realLHS).getRHSVariablesForOrdering(/*true*/); // Expanded();
////				if (forOrdering &&  containsInitialValueFunction(currentEquation)) {
////					variablesOfInterest = currentEquation.getFunctionInitialVariables();
////				} 
//
//				// Note that there is not an array reference on the LHS, but there can be on the rhs
//				// the rhs array reference will require additional work if a range variable appears
//
//
//				for (String rhsVar : variablesOfInterest) {
//
//
//					//						System.out.println("VOI: "+realLHS+" "+rhsVar);
//
//					// if this is not an array reference, simply put the variable into the requiresExpanded set
//					// there is nothing to expand
//					if (!ArrayReference.isArrayReference(rhsVar)) {
//						//							InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar));
//						String rhsVarOrig = InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhsVar);
//						if (rhsVarOrig == null)
//							System.out.println("Huh? rhsVarOrig load rhsVar = "+rhsVar);
//						if (!realLHS.equals(rhsVarOrig)) {
//							if (rhsVarOrig.startsWith("\"") || rhsVarOrig.contains("[a,p]"))
//								System.out.println("DEBUG requires");
//							requiresExpandedLhsExpandedRHS.get(realLHS).add(rhsVarOrig);
//						}
//
//					} else {
//						// if an array access, we must rip apart the array reference and 
//						// insert all subscript values based on outerSub current value
//
//						//create an arrayReference for the RHS 
//						// recall that we have a LHS arrayReference already defined
//						ArrayReference rhsArrayReference = new ArrayReference(rhsVar);
//
//						// some array references may take on multiple values for the eqauation (e.g. SUM(xyz!)
//						// get a list of these combinations
//
//						// need to deal with rhs array like: X[y!, z]
//						// currently doesn't seem to do it right
//
//						boolean printit = false;
//
//						String arrayName = null;
//						if (rhsArrayReference.hasRangeSubscript() || InformationManagers.getInstance().getNamedSubscriptManager().hasNamedSubscript(rhsArrayReference)) {
//							String[] rhsExpanded;
//							if (rhsArrayReference.hasRangeSubscript()) {
//								printit = false;
//								rhsExpanded = removeBang(rhsArrayReference.getRangeSubscriptsAsArray());
//								//									System.out.println("################ Scalar Range sub: "+realLHS+" "+rhsVar);
//								//									for (String s : rhsExpanded)
//								//										System.out.println(s);
//							} else {
//								rhsExpanded = InformationManagers.getInstance().getNamedSubscriptManager().getExpandedSubscripts(rhsArrayReference);
//							}
//
//							List<SubscriptCombination> combos = InformationManagers.getInstance().getArrayManager().getSubscriptValueCombinations(rhsExpanded);
//							for (SubscriptCombination rangeSub : combos){
//
//								// we now construct an appropriate arrayReference for each combination
//								arrayName = "array."+rhsArrayReference.getArrayName()+"[";
//								int subNum = 0;
//
//								// for each subscript that appears in this rhsArrayReference
//								// we need to determine if it is a range subscript or not.
//								// If it is, 
//								for (String subscript : rhsArrayReference.getSubscriptsAsArray()) {
//									if (subNum++ > 0)
//										arrayName += ",";
//									if (ArrayReference.isRangeSubscript(subscript)) {
//										arrayName += rangeSub.getSubscriptValue(subscript.replace("!", ""));
//									} else {
//										arrayName += subscript;
//									}
//
//								}
//								arrayName += "]";
//								if (printit)
//									System.out.println("Expanded: "+arrayName);
//								
//								if (!realLHS.equals(arrayName)) {
//									if (arrayName.contains("[a,p]"))
//										System.out.println("DEBUG requires");
//									requiresExpandedLhsExpandedRHS.get(realLHS).add(arrayName);
//								}
//
//								if (printit)
//									System.out.println(realLHS+" LR "+arrayName);
//								//									}
//
//							}
//						} else {
//							// this is an array reference where the subscript is terminal
//							if (rhsVar == null)
//								System.out.println("Huh? load rhsVar");
//							if (!realLHS.equals(rhsVar)) {
//								if (rhsVar.contains("[a,p]"))
//									System.out.println("DEBUG requires");
//								requiresExpandedLhsExpandedRHS.get(realLHS).add(rhsVar);
//							}
//						}
//
//					}
//				}
//
//			}
//
//		}
//
//		// for each RHS set in requiresExpandedLhsExpandedRHS, need to add the eqn LHS of the rhs Item
//		/* */
//		for (String expLHS : requiresExpandedLhsExpandedRHS.keySet()) {
//			Set<String> aSetExpRHS = requiresExpandedLhsExpandedRHS.get(expLHS);
//
//			// in either case below, no additional processing required
//			if (aSetExpRHS == null || aSetExpRHS.size() == 0)
//				continue;
//
//			// capture list of all expRHS that were originally in set
//			List<String> aList = new ArrayList<String>();
//			aList.addAll(aSetExpRHS);
//
//			// for each of these, determine its realLHS and then add to the set
//			// when an expRHS is available, it is removed
//			// when the entire equation is available the expRHSrealLHS is freed.
//			for (String rhs : aList) {
//				String rhsRealLHS = expandedLhsToRealLhsMap.get(rhs);
//				if (rhsRealLHS == null) {
//					if (rhs.equalsIgnoreCase("time") || rhs.equalsIgnoreCase("timestep")) {
//
//					} else {
//						// try this.... 11/22/2013
//						if (realLhsToExpandedLhsMap.containsKey(rhs)) {
//							// we now track the equation in which the others are defined
//							
//							aSetExpRHS.add(rhsRealLHS);
//						} else {
//							System.out.println("VDP How can this be null in expandedLhsToRealLhsMap: rhs: <"+rhs+"> expLHS <"+expLHS+">");
//						}
//					}
//				} else {
//					
////					aSetExpRHS.add(rhsRealLHS);
//				}
//			}
//		}
//		/* */
//		dumpRequires2(fileNameHead+"expandedLhsToRealLhsMap.txt", expandedLhsToRealLhsMap);
//		dumpRequires(fileNameHead+"realLhsToExpandedLhsMap.txt", realLhsToExpandedLhsMap);
//		boolean startChain = false; // want this to be false for production
//		dumpRequiresExpanded(fileNameHead+"initialrequiresExpandedLhsExpandedRHS.txt", requiresExpandedLhsExpandedRHS, startChain); // HerelhsExpandedNotInitialized
//		dumpRequiresExpanded(fileNameHead+"initialrealLhsToExpandedLhsNotInitialized.txt", realLhsToExpandedLhsNotInitialized, startChain);
//
//		analyzeDependencies(fileNameHead,requiresExpandedLhsExpandedRHS, realLhsToExpandedLhsNotInitialized, expandedLhsToRealLhsMap,
//				realLhsToExpandedLhsMap, equations);
//
//
//	}
	
	protected boolean containsInitialValueFunction(Equation equation) {
		// want to traverse tree (via tokens)
		for (String token : equation.getTokens()) {
			if (Parser.isFunctionInvocation(token)) {
				FunctionDescription fd = InformationManagers.getInstance().getFunctionManager().getDescription(token);
				if (fd.isSuppliesInitialValue()) {
					return true;
				}
			} 
		}
		return false;
	}
	

	
	/**
	 * Returns a list of all stock variables
	 * @return
	 */
	public List<String> getStocks() {
		List<String> al = new ArrayList<String>();
		for(String lhs : equations.keySet()) {
			Equation eqn = equations.get(lhs);
			if (eqn.isStock())
				al.add(lhs);
		}
		return al;
		
	}
	
	protected String[] removeBang(String[] inList) {
		String[] outList = new String[inList.length];
		for (int i = 0; i < inList.length; i++) {
			outList[i] = inList[i].replace("!", "");
		}
		return outList;
	}
	
	public Set<String> getDependencyChain(String variable) {
		Set<String> dependencies = new LinkedHashSet<String>();
		getDependencyChain(variable, dependencies);
//		dependencies.remove(variable);
		return dependencies;
	}
	
	public void getDependencyChain(String variable, Set<String> dependencies) {

		// LHS can be scalar or array, if array, need to expand
		Set<String> expLhs = new LinkedHashSet<String>();
		Set<String> expRhs = new LinkedHashSet<String>();

		// need to check if this is a equation that provides an initial value
		// if so, we only want to add those variables

		Equation eqn = equations.get(variable);
		if (eqn != null) {
			// this lhs maps to an equation, just need to provide one dependency
			expLhs.add(variable);
			if (eqn.getRHSVariables() != null) {
				for (String rhs : eqn.getRHSVariables()) {
					if (eqn.isActiveInitial() || eqn.isStock() || eqn.isHasInitialValue()) {
						System.out.println(variable+" INITIAL VALUE getDC Skip "+rhs);
					} else {
						if (!expRhs.contains(rhs)) {
							expRhs.add(rhs);
							System.out.println(variable+" AddexpRhs1: "+rhs);
						} else {
							System.out.println(variable+" ALREADY THERE AddexpRhs1: "+rhs);
						}

					}
				}
			}

		} else {
			// this lhs does not map to equations
			// must be a subset of a complete array
			// so we will be referencing multiple equations
			// find any equations that map to this arrays name
			for (String k : equations.keySet()) {
				if (!ArrayReference.isArrayReference(k)) {
					continue;
				} else {
					if (ArrayReference.extractArrayName(k).equals(ArrayReference.extractArrayName(variable))) {
						eqn = equations.get(k);
						if (eqn == null) {
							System.out.println(variable+" How could this be null? "+k);
						}
						if (eqn.getRHSVariables() != null) {
							for (String rhs : eqn.getRHSVariables()) {
								if (eqn.isActiveInitial() || eqn.isStock() || eqn.isHasInitialValue()) {
									System.out.println(variable+" INITIAL VALUE getDC Skip "+rhs);
								} else {
									if (!expRhs.contains(rhs)) {
										expRhs.add(rhs);
										System.out.println(variable+" AddexpRhs2: "+rhs);
									} else {
										System.out.println(variable+" ALREADY THERE AddexpRhs2: "+rhs);
									}
								}
						}
					}
					}
				}
			}





			//		if (this.requiresScalarLhsExpandedRhs.containsKey(lhs) &&
			//				requiresScalarLhsExpandedRhs.get(lhs) != null) {
			//			expLhs.add(lhs);
			//			expRhs.addAll(requiresScalarLhsExpandedRhs.get(lhs));
			//		} 
			//		if (this.realLhsToExpandedLhsMap.containsKey(lhs) && 
			//				realLhsToExpandedLhsMap.get(lhs) != null) {
			//			expLhs.addAll(realLhsToExpandedLhsMap.get(lhs));
			//			for (String eLhs : expLhs) {
			//				if (requiresExpandedLhsExpandedRHS.containsKey(eLhs) && 
			//						requiresExpandedLhsExpandedRHS.get(eLhs) != null) {
			//					expRhs.addAll(requiresExpandedLhsExpandedRHS.get(eLhs));
			//				}
			//			}
			//		}
		}
		if (expRhs.size() == 0)
			return;
		for (String eRhs : expRhs) {
			if (dependencies.contains(eRhs)) {
				System.out.println(variable+" NOT AddDependencies: "+eRhs);
				continue;
			}
			System.out.println(variable+" AddDependencies: "+eRhs);
			dependencies.add(eRhs);
			getDependencyChain(eRhs, dependencies);
//			return;
		}
	}
	

	
	
	private boolean hasRhsVariables(Set<String> expLhs) {
		Set<String> rhs;
		for (String lhs : expLhs) {
			rhs = requiresExpandedLhsExpandedRhs.get(lhs);
			if (rhs != null && rhs.size() > 0)
				return true;
		}
		return false;
	}
	
	protected Set<String> getAllRhsDependencies(String lhs) {
		Set<String> dependencies = new LinkedHashSet<String>();
		
		if (ArrayReference.isArrayReference(lhs)) {
			for (String expLHS : ArrayReference.expand(lhs)) {
				getAllRhsDependencies(expLHS, requiresExpandedLhsExpandedRhs, dependencies);
			}
		} else {
			getAllRhsDependencies(lhs, requiresExpandedLhsExpandedRhs, dependencies);
		}

		return dependencies;
	}
	
	protected void getAllRhsDependencies(String lhs, Map<String, HashSet<String>> requires, Set<String> alreadyProcessed) {

		for (String rhs : sortSet(requires.get(lhs))) {
			String aRhs = rhs;

			// this can be a scalar or an expanded LHS, if the latter, get its real LHS to access the equation
			// eqnLHS is the real equation for this lhs
			Equation eqnLHS = equations.get(lhs);

			// if we can't find the equation with this lhs, then it must be an expandedLHS
			// find the equation associated with its unexpanded lhs

			if (eqnLHS == null && expandedLhsToRealLhsMap.containsKey(lhs)) {
				String realLHS = expandedLhsToRealLhsMap.get(lhs);
				eqnLHS = equations.get(realLHS);
			}

			// if null, nothing to work with
			if (eqnLHS == null) {
				System.out.println("No equation available for this lhs cannot apply chain break "+rhs);
				continue;
			} 
			
			// if this equation is one for which we have an initial value, we're done with it
			
			if (eqnLHS.isActiveInitial() || eqnLHS.isStock() || eqnLHS.isSampleIfTrue()) {
				System.out.println("chain break "+rhs+" eqn: "+eqnLHS.getVensimEquationOnly());
				continue;
			} else {
				if (alreadyProcessed.contains(rhs)) {
					continue;
				}
				
				// have a rhs variable that needs to be processed
				// find the equation that generates its value
				
				String realLHS2 = "";
				// what if r is actually a stock variable?
				Equation eqnRHS = equations.get(aRhs);
				if (eqnRHS == null && expandedLhsToRealLhsMap.containsKey(lhs)) {
					realLHS2 = expandedLhsToRealLhsMap.get(aRhs);
					eqnRHS = equations.get(realLHS2);
				}
			

				// if not null, check to see if it's a stock, if it is, we want it to be dependent
				// on stocks initial value, not stock
//				if (eqnRHS != null) {
//					if (eqnRHS.isStock()) {
//						System.out.println("Redirecting Stock Reference "+rhs+" -> "+ActiveInitialProcessor.makeInitialVariable(rhs)+
//								" for equation "+eqnLHS.getVensimEquationOnly());
//						//							aRhs = ActiveInitialProcessor.makeInitialVariable(aRhs);
//					}
//					if (eqnRHS.isActiveInitial()) {
//						System.out.println("Redirecting Active Initial Reference "+rhs+" -> "+ActiveInitialProcessor.makeInitialVariable(rhs)+
//								" for equation "+eqnLHS.getVensimEquationOnly());
//						//							aRhs = ActiveInitialProcessor.makeInitialVariable(aRhs);
//					}
//				} else {
//					System.out.println("Could not find equation for RHS "+realLHS2);
//				}

				alreadyProcessed.add(aRhs);

				// is r is an array, must expand

				if (ArrayReference.isArrayReference(aRhs)) {
					for (String expLHS : ArrayReference.expand(aRhs)) {
						getAllRhsDependencies(expLHS, requiresExpandedLhsExpandedRhs, alreadyProcessed);
					}
				} else {
					getAllRhsDependencies(aRhs, requiresExpandedLhsExpandedRhs, alreadyProcessed);
				}

				getAllRhsDependencies(aRhs, requires, alreadyProcessed);
			}
		}
	}


	protected List<String> sortSet(HashSet<String> set) {
		ArrayList<String> al = new ArrayList<String>();
		if (set == null)
			return al;
		Iterator<String> iter = set.iterator();
		while (iter.hasNext())
			al.add(iter.next());

		Collections.sort(al);

		return al;
	}
	
	protected void dumpRequires(String fileName, Map<String, HashSet<String>> requires) {

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);

		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
				for (String r : sortSet(requires.get(lhs))) {
					bw.append("     RHS <"+r+">\n");
				}
			}
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	protected void dumpRequires2(String fileName, Map<String, String> requires) {

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);

		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
			String r = requires.get(lhs);
					bw.append("     RHS <"+r+">\n");
				}
		
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	protected void dumpUsesUsedByMap(String fileName, Map<String, Set<String>> requires, boolean additionalLevel) {

		//	boolean printChain = true;

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allKey = new ArrayList<String>();

		for (String key : requires.keySet()) {
			if (key == null) {
				System.out.println("Skipping null in requires");
			} else {
				allKey.add(key);
			}
		}

		Collections.sort(allKey);


		try {
			for (String key : allKey) {
				if (requires.get(key).size() > 0) {
					bw.append("KEY: <"+key+">\n");
					for (Equation e : InformationManagers.getInstance().getEquationManager().getAllEquationsFor(key)) {
						bw.append(e.getVensimEquationOnly()+"\n");
					}
					for (String r : requires.get(key)) {
						bw.append("     VALUE <"+r+">\n");
						if (additionalLevel)
							bw.append("                 <"+requires.get(r)+">\n");
					}
				}
			}

			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	// Map<String, Map<String, Set<String>>>
	
	protected void dumpVariableReferences(String fileName, Map<String, Map<String, Set<String>>> refs) {

		//	boolean printChain = true;

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : refs.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);


		try {
			for (String lhs : allLHS) {
				bw.append("### "+lhs+" ###\n");
				bw.append("<"+SDObjectManager.LHS+"> "+getSetAsString(refs.get(lhs).get(SDObjectManager.LHS)));
				bw.append("<"+SDObjectManager.RHS+"> "+getSetAsString(refs.get(lhs).get(SDObjectManager.RHS)));
				bw.append("<"+SDObjectManager.RHS_USE+"> "+getSetAsString(refs.get(lhs).get(SDObjectManager.RHS_USE)));
			}
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private String getSetAsString(Set<String> s) {
		StringBuilder sb = new StringBuilder();
		sb.append("   [ ");
			for (String m : Utilities.ordered(s))
				sb.append("      "+m+", ");
		sb.append("   ]\n");
		return sb.toString();
	}
	
//	private String getSetAsString(Set<String> s) {
//		StringBuilder sb = new StringBuilder();
//		sb.append("   [\n");
//			for (String m : Utilities.ordered(s))
//				sb.append("      "+m+"\n");
//		sb.append("   ]\n");
//		return sb.toString();
//	}


	protected void dumpRequiresExpanded(String fileName, Map<String, HashSet<String>> requires, boolean printChain) {

		//	boolean printChain = true;

		BufferedWriter bw = Utilities.openFileForWriting(miscDirectory+"/"+fileName);
		ArrayList<String> allLHS = new ArrayList<String>();

		for (String lhs : requires.keySet()) {
			if (lhs == null) {
				System.out.println("Skipping null in requires");
			} else {
				allLHS.add(lhs);
			}
		}

		Collections.sort(allLHS);


		try {
			for (String lhs : allLHS) {
				bw.append("LHS: <"+lhs+">\n");
				int level = 1;
				if (printChain) {
					printChain(bw, level, lhs, requires, new LinkedHashSet<String>());
				} else {
					for (String r : sortSet(requires.get(lhs))) {
						bw.append("     RHS <"+r+">\n");
					}
				}
			}
			bw.flush();
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	protected void printChain(BufferedWriter bw, int level, String lhs, Map<String, HashSet<String>> requires, HashSet<String> alreadyPrinted) {

		if (level > 50) {
			try {
				bw.append("Terminating recursion\n");
				return;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		StringBuilder blanks = new StringBuilder();
		for (int i = 0; i < level; i++) {
			blanks.append("  ");
		}

		for (String r : sortSet(requires.get(lhs))) {
			if (alreadyPrinted.contains(r)) {
				try {
					bw.append("                      ### looping on "+r+"\n");
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				continue;
			}
			try {
				bw.append(blanks.toString()+"RHS <"+r+"> \n");
				alreadyPrinted.add(r);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			printChain(bw, level+1, r, requires, alreadyPrinted);
		}

	}
	
	protected void analyzeDependencies(String fileNameHead, Map<String, HashSet<String>> requiresExpandedLhsExpandedRHS, 
			Map<String, HashSet<String>> realLhsToExpandedLhsNotInitialized, 
			Map<String, String> expandedLhsToRealLhsMap, Map<String, HashSet<String>> realLhsToExpandedLhsMap,
			Map<String, Equation> equations) {

		if (InformationManagers.getInstance().getFileOrganizer() != null && 
				InformationManagers.getInstance().getFileOrganizer().getMiscDirectory().length() > 0) {
			miscDirectory = InformationManagers.getInstance().getFileOrganizer().getMiscDirectory();
		} else {
			miscDirectory = "output";
		}
		BufferedWriter bw = null;
		// ############## WRITE_DEBUG_FILES
		if (StaticSwitches.WRITE_DEBUG_FILES) {
			bw = Utilities.openFileForWriting(miscDirectory+"/"+fileNameHead+"dependencyAnalysis.txt");
		}
//		System.out.println("################## Analyze Dependencies #########################");

		Equation eqn;
		String eqnLHS;

		// requiresExpandedLhsExpandedRHS,   realLhsToExpandedLhsNotInitialized
		// make sure that every thing on the RHS of these maps have a corresponding LHS (i.e. key)

		try {
			for (String vensimLHS : equations.keySet()) {
				eqn = equations.get(vensimLHS);

				if (eqn.isDefinesSubscript())
					continue;

				eqnLHS = eqn.getOriginalLhs();

				//				if (eqn.isDefinesLookup() && !ArrayReference.isArrayReference(vensimLHS)) {
				//					eqnLHS = "array."+eqnLHS;
				//				}

				//				bw.append("LHS: "+eqnLHS+"\n");
				if (!realLhsToExpandedLhsMap.containsKey(eqnLHS)) {
					// ############## WRITE_DEBUG_FILES
					if (StaticSwitches.WRITE_DEBUG_FILES) {
						bw.append("     expLHS: MAP MISSING\n");
					}
					System.out.println("### AD Skipping: Missing realLhsToExpandedLhsMap for "+vensimLHS);
					continue;
				}
				boolean continu = false;
				for (String expLHS : realLhsToExpandedLhsMap.get(eqnLHS)) {

					HashSet<String> hs = realLhsToExpandedLhsNotInitialized.get(eqnLHS);

					if (hs == null) {
						System.out.println("### AD Skipping: Missing Set realLhsToExpandedLhsNotInitialized for eqnLHS <"+eqnLHS+"> and expLHS <"+expLHS+">");
						// ############## WRITE_DEBUG_FILES
						if (StaticSwitches.WRITE_DEBUG_FILES) {
							bw.append("     expLHS NOT INIT: MISSING\n");
						}
						continu = true;
					}
					hs = requiresExpandedLhsExpandedRHS.get(expLHS);

					if (hs == null) {
						// ############## WRITE_DEBUG_FILES
						if (StaticSwitches.WRITE_DEBUG_FILES) {
							bw.append("          expRHS: NULL"+"\n");
						}
						System.out.println("### AD Skipping: Missing Set requiresExpandedLhsExpandedRHS for eqnLHS <"+eqnLHS+"> and expLHS <"+expLHS+">");
						continu = true;
					}
					//					if (hs.size() == 0) {
					//						continue;
					//					}
					// ############## WRITE_DEBUG_FILES
					if (StaticSwitches.WRITE_DEBUG_FILES) {
						bw.append("EQN: "+eqn.getVensimEquationOnly()+"\n");

						bw.append("     expLHS: "+expLHS+"\n");
					}
					if (continu)
						continue;
					// ############## WRITE_DEBUG_FILES
					if (StaticSwitches.WRITE_DEBUG_FILES) {
						if (hs.size() == 0) {
							bw.append("          expRHS: SIZE = 0"+"\n");
						}
						for (String rhs : hs) {
							bw.append("          expRHS: "+rhs+"\n");
						}
					}

				}
			}
			if (bw != null)
				bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

//		System.out.println("################## Analyze Dependencies End #########################");
	}
		
	
	private String toRealLHS(String lhs, Map<String, String> expandedLhsToRealLhsMap) {
		if (expandedLhsToRealLhsMap.containsKey(lhs)) 
			return expandedLhsToRealLhsMap.get(lhs);
		else
			return lhs;
	}

//
//	public Map<String, HashSet<String>> getRequiresScalarLhsExpandedRhs() {
//		return requiresScalarLhsExpandedRhs;
//	}

	public Map<String, String> getExpandedLhsToRealLhsMap() {
		return expandedLhsToRealLhsMap;
	}

	public Map<String, HashSet<String>> getRealLhsToExpandedLhsMap() {
		return realLhsToExpandedLhsMap;
	}

	public Map<String, HashSet<String>> getRealLhsToExpandedLhsNotInitialized() {
		return realLhsToExpandedLhsNotInitialized;
	}

	public Map<String, HashSet<String>> getRequiresExpandedLhsExpandedRHS() {
		return requiresExpandedLhsExpandedRhs;
	}

	public Map<String, Equation> getEquations() {
		return equations;
	}

	/**
	 * @return the requiresScalarLhsExpandedRhs
	 */
	public Map<String, HashSet<String>> getRequiresScalarLhsExpandedRhs() {
		return requiresScalarLhsExpandedRhs;
	}
	
	//requiresExpandedLhsExpandedRHS
	
	public Map<String, Set<String>> getTailHeadMap() {
		if (tailHeadMap == null)
			makeHeadsAndTails();
		return tailHeadMap;
	}
	
	public Map<String, Set<String>> getHeadTailMap() {
		if (headTailMap == null)
			makeHeadsAndTails();
		return headTailMap;
	}
	
	private void makeHeadsAndTails() {
		tailHeadMap = new LinkedHashMap<String, Set<String>>();
		headTailMap = new LinkedHashMap<String, Set<String>>();

		for (String lhs : equations.keySet()) {
			// only assignments need to be considered
			Equation eqn = equations.get(lhs);
			if (eqn.isAssignment()) {
//			if (!eqn.isAssignment() && !eqn.isDefinesLookup()) {
//				continue;
//			}
//			if (eqn.isArrayInitialization())
//				continue;
//			if (eqn.isDefinesLookup())
//				continue;

			String rhs;
			String lhst;
			lhst = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(lhs));
			Set<String> rhsVars = eqn.getRhsTokens();
			if (rhsVars.size() > 0) {
				for (String rhst : rhsVars) {
					rhs = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(rhst));


					if (!tailHeadMap.containsKey(rhs))
						tailHeadMap.put(rhs, new LinkedHashSet<String>());
					tailHeadMap.get(rhs).add(lhst);
					
					if (!headTailMap.containsKey(lhs))
						headTailMap.put(lhst, new LinkedHashSet<String>());
					headTailMap.get(lhst).add(rhs);


				} 
			}
			}
		}  // end major loop
		
		String dir = InformationManagers.getInstance().getModelManager().getGeneratedOutputDirectoryName();

		SDModelCodeGenerator.writeMapToFile(dir
				+ "/structure/headTailMap.tab", headTailMap);

					SDModelCodeGenerator.writeMapToFile(dir
				+ "/structure/tailHeadMap.tab", tailHeadMap);
			
	}

	public Map<String, HashSet<String>> getUsesExpandedRhsExpandedLhs() {
		return usesExpandedRhsExpandedLhs;
	}

	public List<String> getOrderedListOfAvailableVariables() {
		return orderedListOfAvailableVariables;
	}

	public boolean isCleanOrderOfAvailableVariables() {
		return cleanOrderOfAvailableVariables;
	}
	

}
