package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.xmile.Model;
import gov.anl.gss.sdm.shared.drawView.SDModelViewImageGenerator;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

public class View {
static Logger log = Logger.getLogger(View.class.getSimpleName());

	public static final String ARROW = "ARROW";
	public static final String VARIABLE = "VARIABLE";
	public static final String VALVE = "VALVE";
	public static final String COMMENT = "COMMENT";
	public static final String BITMAP = "BITMAP";
	public static final String METAFILE = "METAFILE";
	public static final String RATE = "RATE";
	public static final String CLOUD = "CLOUD";
	
	private static final String CLOUD_NAME = "C_L_O_U_D";

	private String name;
	private String versionCode;
	private String viewDefaultFont;
	private List<String> rawObjects;
	private List<GraphicObject> graphicObjects;
	private int currentPtr = 0;

	private Map<String, String> idNameMap;
	
	private Map<String, GraphicObject> idGraphicMap;

	private List<String> variablesInView = new ArrayList<String>();
	
	
	public View(String name, String versionCode, String viewDefaultFont) {
		this.name = name;
		if (name == null || name.length() == 0)
			this.name = "missingViewName";
		this.versionCode = versionCode;
		this.viewDefaultFont = viewDefaultFont;
		graphicObjects = new ArrayList<GraphicObject>();
		idNameMap = new LinkedHashMap<String, String>();
		idGraphicMap = new LinkedHashMap<String, GraphicObject>();

	}
	
	public void buildFromXMILE(Model xmileModel) {
		
		// build the graphic objects piecemeal
		// collect the connects (arrows)
		// and the variables
		
		rawObjects = xmileModel.getAsVensimView();
		
	}

	public void parse(SystemDynamicsObjectManager sdObjectManager) {

//		log.debug("IN PARSE");
		
		int numGo = 0;
		int numArrow = 0;

		while(currentPtr < rawObjects.size()) {
			String raw = rawObjects.get(currentPtr++);
			
			GraphicObject go = new GraphicObject(sdObjectManager, this, raw);
			if (!go.isRecognized())
				continue;
			numGo++;
			if (go.isArrow())
				numArrow++;
			idNameMap.put(go.getId(), go.getName());
			idGraphicMap.put(go.getId(), go);
			graphicObjects.add(go);
			// if this go has an associated variable (can only be valve)
			// need to add to graphic objects as this was processed together
			// with valve
			if (go.isValve()) {
				if (go.getAssociatedVariable() != null) {
					graphicObjects.add(go.getAssociatedVariable());
					idNameMap.put(go.getId(), go.getAssociatedVariable().getName());
					idGraphicMap.put(go.getAssociatedVariable().getId(), go.getAssociatedVariable());
				}  else {
					log.debug("Valve with no associated");
				}
			}
		}
		
		variablesInView = sdObjectManager.extractStructure(this);
		MutableBoolean redirect = new MutableBoolean(false);
		MutableBoolean delete = new MutableBoolean(false);
		processMapIndirection(redirect, delete);
		
		log.debug("Indirection Results: redirect = "+redirect.value()+" delete = "+delete.value());
		
		
	}
	
	public void writeToCsvFile(BufferedWriter bw) {
		for (GraphicObject go : graphicObjects) {
			go.writeToCsvFile(bw);
		}
	}
	
	public void generateImage(List<String> forViewImage) {
		
		ModelManager mm = InformationManagers.getInstance().getModelManager();
		// this is the "new" view image we want
//		String outfile = mm.getGeneratedOutputDirectoryName()+"/__VIEW__"+Utilities.makeLegalJavaName(name)+".png";
		String outfile = InformationManagers.getInstance().getFileOrganizer().getViewsDirectory()+"/__VIEW__"+Utilities.makeLegalJavaName(name)+".png";
		
		// if reusing, it may already exist or needs to be copied from alternate directory
		if (InformationManagers.getInstance().getReuse().isViews()){
			if (new File(outfile).exists())
				return;
			String origView = InformationManagers.getInstance().getReuse().getViewsDirectory()
					+"/__VIEW__"+Utilities.makeLegalJavaName(name)+".png";
			File orgFile = new File(origView);
			if (orgFile.exists()) {
				try {
					FileUtils.copyFile(orgFile, new File(outfile));
				} catch (IOException e) {
					// TODO Auto-generated catch block
//					e.printStackTrace();
					SDModelViewImageGenerator gener = new SDModelViewImageGenerator(name, Utilities.asArray(forViewImage), outfile);
				}
				return;
			}
		}

		SDModelViewImageGenerator gener = new SDModelViewImageGenerator(name, Utilities.asArray(forViewImage), outfile);

	}

	
	public void buildHeadsAndTails(Map<String, Set<String>> headTailMap,
    		Map<String, Set<String>> tailHeadMap) {
		
		System.out.println("Build Heads and Tails: "+name);
		
		int ht = 0;
		int th = 0;
		for (GraphicObject go : graphicObjects) {
			if (go.isArrow()) {
				String tail = idToName(go.getFrom());
				String head = idToName(go.getTo());

				if (tail == null) {
					tail = idToGraphic(go.getFrom()).getName();
				}
				if (head == null) {
					if (idToGraphic(go.getTo()) != null) {
						head = idToGraphic(go.getTo()).getName();
						if (head == null) {
							head = "NOVAR";
						}
					} else {
						head = "NOVAR";
					}

				}

				// need to swap arrow direction on in flows
				//				if (go.isInFlowArrow()) {
				//					String t = tail;
				//					tail = head;
				//					head = t;
				//				}

				if (tail.contains(CLOUD_NAME) ||
						head.contains(CLOUD_NAME))
					continue;



				if (!headTailMap.containsKey(head)) {
					headTailMap.put(head, new LinkedHashSet<String>());
				}
				if (headTailMap.get(head).contains(tail)) {
					//					System.out.println(name+" SAW THIS ALREADY "+tail);
				} else {
					headTailMap.get(head).add(tail);
				}

				if (!tailHeadMap.containsKey(tail)) {
					tailHeadMap.put(tail, new LinkedHashSet<String>());
				}
				if (tailHeadMap.get(tail).contains(head)) {
					//					System.out.println(name+" SAW THIS ALREADY "+head);
				} else {
					tailHeadMap.get(tail).add(head);
				}
			}
		}
	}
	
	private void printGraphicsArrows(String marker) {
		log.debug("##### "+marker+" #####");
		for (GraphicObject go : graphicObjects) {
			if (go.isArrow())
				go.print();
		}
	}
	
	private void printGraphicsState(String marker) {
		log.debug("##### "+marker+" #####");
		for (GraphicObject go : graphicObjects) {
				go.print();
		}
	}
	
	public void writeLocations(BufferedWriter bw) {
		for (GraphicObject go : graphicObjects) {
			if (!go.isArrow())
				go.writeLocation(bw);
		}
	}
	
	private void processMapIndirection(MutableBoolean redirect, MutableBoolean delete) {
		

		// for maps, we must find arrows that go from valve to stock
		// for other arrows to valve, change their dest to the stock (only if no associated variable with valve)
		// note: ghost arrow out variables need to have there polarity reversed
		
		// valves is set of valves for which no associated variable existed
		// we create one for everything to point to.

		Set<String> generatedRateIDs = new LinkedHashSet<String>();
		Set<String> nonAssociatedValves = new LinkedHashSet<String>();
		Map<String, String> toBeSkippedArrows = new LinkedHashMap<String, String>();

		// first grab all valve associated variables that were auto generated
		for (GraphicObject go : graphicObjects) {
			if (go.isValve() && go.getAssociatedVariable().isAutoGenerated()) {
				nonAssociatedValves.add(go.getId());
				generatedRateIDs.add(go.getAssociatedVariable().getId());
				log.debug("genRateID: "+go.getAssociatedVariable().getId()+" "+go.getId());
			}
		}

		// then grab all valve arrows from auto generated
		for (GraphicObject arrow : graphicObjects) {
			if (arrow.isArrow() && nonAssociatedValves.contains(arrow.getFrom())) {
//			if (arrow.isArrow() && generatedRateIDs.contains(arrow.getFrom())) {
				// we only want arrows pointing to a varaible and not a cloud or other such thing
				GraphicObject arrowHead = idGraphicMap.get(arrow.getTo());
				if (arrowHead.isVariable()) {
					toBeSkippedArrows.put(arrow.getFrom(), arrow.getTo());
					log.debug("To Be Skipped: "+arrow.getFrom()+" -> "+arrow.getTo());
				}
			}
		}

		// now redirect arrows to valves
		for (GraphicObject arrow : graphicObjects) {
//			if (arrow.isArrow() && generatedRateIDs.contains(arrow.getTo())) {
				if (arrow.isArrow() && nonAssociatedValves.contains(arrow.getTo())) {
//				log.debug("Redirecting "+go.toString()+" origTo: "+go.getTo()+" newTo: "+valveArrows.get(go.getTo()).getTo());
				GraphicObject generatedTo = idGraphicMap.get(toBeSkippedArrows.get(arrow.getTo()));
				redirect.setValue(true);
				log.debug("Redirecting "+arrow.toString()+" origTo: "+arrow.getTo()+" newTo: "+generatedTo.getId());
	
				arrow.setTo(generatedTo.getId());
				if (generatedTo.isSwapped()) {
					log.debug("Reverse Polarity");
					arrow.reversePolarity();
				}
			}
		}
		
//		log.debug("Removing old valve arrows. Count: "+valveArrows.values().size());
//		for (GraphicObject g : valveArrows.values())
//			g.print();
		for (String id : toBeSkippedArrows.keySet()) {
			GraphicObject toRemove = idGraphicMap.get(id);
			log.debug("toRemove: "+toRemove);
			graphicObjects.remove(toRemove);
			delete.setValue(true);
		}
		
		// finally, remove all nonAssociated valves and references
		List<GraphicObject> allToRemove = new ArrayList<GraphicObject>();
		for (GraphicObject go : graphicObjects) {
			if (nonAssociatedValves.contains(go.getId())) {
				allToRemove.add(go);
				log.debug("toRemove2_1: "+go);
			}
			if (go.isArrow() && (nonAssociatedValves.contains(go.getTo()) )) {
				allToRemove.add(go);
				log.debug("toRemove2_2: "+go);
			}
			if (go.isArrow() &&  nonAssociatedValves.contains(go.getFrom())) {
				allToRemove.add(go);
				log.debug("toRemove2_3: "+go);
			}
		}
		
		if (allToRemove.size() > 0) {
			delete.setValue(true);
			graphicObjects.removeAll(allToRemove);
		}
		return;
	}
	
//	private boolean isMap() {
//		boolean ismap = true;
//		
//		// we will define as map if:
//		// (1) there are only arrows and variables
//		// (2) if valves, they do not have an associated variable
//		
//		for (GraphicObject go : graphicObjects) {
//			if (go.isValve() && go.getAssociatedVariable() != null)
//				ismap = false;
//		}
//		
//		
//		return ismap;
//	}
	
	public GraphicObject idToGraphic(String id) {
		return idGraphicMap.get(id);
	}
	
	public String idToName(String id) {
		return idNameMap.get(id);
	}


	public String translateCodeToString(String numeric) {

		//	public static final String ARROW = "ARROW";
		//	public static final String VARIABLE = "VARIABLE";
		//	public static final String VALVE = "VALVE";
		//	public static final String COMMENT = "COMMENT";
		//	public static final String BITMAP = "BITMAP";
		//	public static final String METAFILE = "METAFILE";

		if (numeric.equals(GraphicObject.ARROW))
			return ARROW;
		else if (numeric.equals(GraphicObject.VARIABLE))
			return VARIABLE;
		else if (numeric.equals(GraphicObject.VALVE))
			return VALVE;
		else if (numeric.equals(GraphicObject.COMMENT))
			return COMMENT;
		else if (numeric.equals(GraphicObject.BITMAP))
			return BITMAP;
		else if (numeric.equals(GraphicObject.METAFILE))
			return METAFILE;
		else if (numeric.equals(GraphicObject.RATE))
			return RATE;
		else if (numeric.equals(GraphicObject.CLOUD))
			return CLOUD;
		else
			return "UNKNOWN";

	}

	public String peekNextRawObject() {
		// error condition?
		String next = rawObjects.get(currentPtr);
		//	currentPtr++;
		return next;

	}

	public String getNextRawObject() {
		// error condition?
		String next = rawObjects.get(currentPtr);
		currentPtr++;
		return next;

	}

	public void print() {
		log.debug("##########################");
		log.debug("View Name: "+name);
		log.debug("View Version: "+versionCode);
		log.debug("View Default Font: "+viewDefaultFont);
		//    	log.debug("Graphic Objects:");
		//    	for (GraphicObject go : graphicObjects)
		//    		go.print();
	}

	public List<String> getRawObjects() {
		return rawObjects;
	}

	public void setRawObjects(List<String> rawObjects) {
		this.rawObjects = rawObjects;
	}

	public String getName() {
		return name;
	}

	public List<GraphicObject> getGraphicObjects() {
		return graphicObjects;
	}

	public List<String> getVariablesInView() {
		return variablesInView;
	}

	public String getViewDefaultFont() {
		return viewDefaultFont;
	}

	public void setViewDefaultFont(String viewDefaultFont) {
		this.viewDefaultFont = viewDefaultFont;
	}

	public Map<String, String> getIdNameMap() {
		return idNameMap;
	}
	

}
