package gov.anl.gss.sdm.server.model.visualization3d;



public class Connection {
	
	public final static String CONNECTION_TAG = "_#_";
	public final static String INFLUENCE = "INFLUENCE";
	public final static String FLOW = "FLOW";
	public final static String HORIZONTAL = "HORIZONTAL";
	public final static String VERTICAL = "VERTICAL";
	public final static String UNKNOWN = "unknown";
	
	public final static String INFLUENCE_COLOR = "blue";
	public final static String FLOW_COLOR = "black";
	
	private final String tail;
	private final String head;
	private final String id;
	
	private Variable tailVariable;
	private Variable headVariable;
	
	
	private  boolean hasArrowHead;
	private  String color;
	
	private String type;
	private String orientation;
	
	// 1,22,8,7,4,0,0,0,0,0,0,-1--1--1,,3|(360,198)|(446,198)|(446,143)|
	private final String pointList;
	
	private double midx, midy, midz;
	
	private double rotx = -Double.MAX_VALUE;
	private double roty = -Double.MAX_VALUE;
	private double rotz = -Double.MAX_VALUE;
	private double rotz2 = -Double.MAX_VALUE;
	
	private DataManager dataManager;
	
	public Connection(DataManager dataManager, String tail, String head, boolean hasArrowHead, String color, String type, String orientation,
			String pointList) {
		this.dataManager = dataManager;
		this.tail = tail;
		this.head = head;
		id = tail + CONNECTION_TAG + head + CONNECTION_TAG + type;
		this.hasArrowHead = hasArrowHead;
		this.color = color;
		this.type = type;
		this.orientation = orientation;
		this.pointList = extractPoint(pointList);
	}
	
	public String extractPoint(String pointList) {
		String[] f = pointList.split("\\|");
		
		if (f.length < 4)
			return f[1];
		else
			return f[2];
		
	}
	
	public void setVariables() {
		this.tailVariable = dataManager.getVariable(tail);
		this.headVariable = dataManager.getVariable(head);
	}

	public String getTail() {
		return tail;
	}

	public String getHead() {
		return head;
	}

	public String getId() {
		return id;
	}
	
	public void adjustCoordinates() {
		// there are none. The midpoints are calculated from adjusted values
	}
	
	public boolean hasPointList() {
		return pointList.length() > 0;
	}
	
	public void computeMidPoint(Variables variables) {
		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double cx1, cy1, cz1;
		double cx2, cy2, cz2;

		if (isInfluence() && hasPointList()) {
			midx = getPointListX()*Visualization3DData.SPREAD_XY - dataManager.getxAdjust();
			midy = (dataManager.getyMax() - (getPointListY()*Visualization3DData.SPREAD_XY)) - dataManager.getyAdjust();

		} else {

			cx1 = variables.getCurrentLocation(t).getAdjustedX();
			cx2 = variables.getCurrentLocation(h).getAdjustedX();
			midx =  (cx1+cx2)/2.0;

			cy1 = variables.getCurrentLocation(t).getAdjustedY();
			cy2 = variables.getCurrentLocation(h).getAdjustedY();
			midy = (cy1+cy2)/2.0;
		}
		
		if (variables.getCurrentLocation(t) == null) 
			System.out.println("CONNECTION NULL");
		
		cz1 = variables.getCurrentLocation(t).getAdjustedZ();
		cz2 = variables.getCurrentLocation(h).getAdjustedZ();
		midz = (cz1+cz2)/2.0;

	}
	
	public void computeRotation(Variables variables) {
//		computeRotationx(variables);
		computeRotationxy(variables);
//		computeRotationSpherical(variables);
//		computeRotationz(variables);
//		computeRotationzx(variables); 
//		computeRotationxyz(variables);
//		computeRotation2(variables); 
	}
	
	public void computeRotationx(Variables variables) {

		Variable h = variables.getVariable(head);

		double rx;

		Location hloc = variables.getCurrentLocation(h);

		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;
		
		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
		} else {
			rx = Math.atan2(z, y)  * 180.0/Math.PI;  // y, z
		}

		rotx = rx;
		roty = 0.0;
		rotz = 0.0;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotationz(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// This is from brian's code

		double atan = 0.0;
		double newx, newy, newz;

		// rotz controls right and left pointing

		// special case seems to work
		if (y == 0.0) {
			if (x > 0) {
				rz = -90.0; // force towards pos x
			} else {
				rz = 90.0; // force towards neg x
			}
		} else {
			atan = (-90 * Math.PI/180.0) + Math.atan2(y, x);
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;  // -90 to get the pointer pointing right rather than up

		}
		
		double thetaDegrees1 = Math.atan2(Math.abs(y), Math.abs(x)) * 180.0/Math.PI;
		double rz2;
		if (x >= 0 && y >=0) {
			rz2 = 0 + thetaDegrees1 - 90; 
		} else if (x >= 0 && y < 0) {
			rz2 = 0 - thetaDegrees1 - 90; 
		} else if (x < 0 && y >=0) {
			rz2 = 180 - thetaDegrees1 - 90; 
		} else if (x < 0 && y < 0) {
			rz2 = 180 + thetaDegrees1 - 90; 
		} else {
			rz2 = 0.0;
		}



		rotx = 0.0;
		roty = 0.0;
		rotz = rz;
		rotz2 = rz2;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz+" "+rz2);
	}
	
	public void computeRotationzx(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// This is from brian's code

		double atan = 0.0;
		double newx, newy, newz;

		// rotz controls right and left pointing

		// special case seems to work
		if (y == 0.0) {
			if (x > 0) {
				rz = -90.0; // force towards pos x
			} else {
				rz = 90.0; // force towards neg x
			}
		} else {
			atan = (-90 * Math.PI/180.0) + Math.atan2(y, x);
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;  // -90 to get the pointer pointing right rather than up

		}


		newy = y*Math.cos(atan) - x*Math.sin(atan);
		newx = x*Math.cos(atan) + y*Math.sin(atan);
		y = newy;
		x = newx;
		
		// this shouldn't affect anything
//		atan = Math.atan2(z, x);
//		ry = Math.atan2(z, x) * 180.0/Math.PI; // 0.0; // 
//
//		newz = z*Math.cos(atan) - x*Math.sin(atan);
//		newx = x*Math.cos(atan) + z*Math.sin(atan);
//		x = newx;
//		z = newy;


		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
		} else {
//			atan = (-90 * Math.PI/180.0) + Math.atan2(y, z);
//			rx = -90 + Math.atan2(y, z)  * 180.0/Math.PI;  // y, z
			
			atan = Math.atan2(y, z);
			rx = Math.atan2(y, z)  * 180.0/Math.PI;  // y, z
		}

		newy = y*Math.cos(atan) - z*Math.sin(atan);
		newz = z*Math.cos(atan) + y*Math.sin(atan);
		y = newy;
		z = newz;

		//		rotx = 180.0 - rx;
		//		roty = 180.0 - ry;
		//		rotz = Math.abs(rz);

		rotx = rx;
		roty = 0.0;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotationxyz(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// This is from brian's code

		double atan = 0.0;
		double newx, newy, newz;
		
		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
		} else {
//			atan = (-90 * Math.PI/180.0) + Math.atan2(y, z);
//			rx = -90 + Math.atan2(y, z)  * 180.0/Math.PI;  // y, z
			
			atan = Math.atan2(y, z);
			rx = Math.atan2(z,y)  * 180.0/Math.PI;  // y, z
		}

		newy = y*Math.cos(atan) - z*Math.sin(atan);
		newz = z*Math.cos(atan) + y*Math.sin(atan);
		y = newy;
		z = newz;

		// rotz controls right and left pointing

		// special case seems to work
		if (y == 0.0) {
			if (x > 0) {
				rz = -90.0; // force towards pos x
			} else {
				rz = 90.0; // force towards neg x
			}
		} else {
			atan = (-90 * Math.PI/180.0) + Math.atan2(y, x);
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;  // -90 to get the pointer pointing right rather than up

		}


		newy = y*Math.cos(atan) - x*Math.sin(atan);
		newx = x*Math.cos(atan) + y*Math.sin(atan);
		y = newy;
		x = newx;
		
		// this shouldn't affect anything
//		atan = Math.atan2(z, x);
//		ry = Math.atan2(z, x) * 180.0/Math.PI; // 0.0; // 
//
//		newz = z*Math.cos(atan) - x*Math.sin(atan);
//		newx = x*Math.cos(atan) + z*Math.sin(atan);
//		x = newx;
//		z = newy;




		//		rotx = 180.0 - rx;
		//		roty = 180.0 - ry;
		//		rotz = Math.abs(rz);

		rotx = rx;
		roty = 0.0;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotationSpherical(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
			ry = 0.0;
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;
		} else {
			SphericalCoordinates coords = new SphericalCoordinates(x, y, z);
			rx = coords.getTheta();
			ry = coords.getPhi();
			rz = 0.0;
		}

		// roty controls right and left pointing


		rotx = rx;
		roty = ry;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotationxy(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
			ry = 0.0;
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;
		} else {
			
			// need to compute x axis rotation (in/out from screen)
			// then y axis rotation
			
			double thetaDegrees1 = Math.atan2(Math.abs(y), Math.abs(z)) * 180.0/Math.PI;
			if (z >= 0 && y >=0) {
				rx = 90 - thetaDegrees1;
			} else if (z >= 0 && y < 0) {
				rx = 90 + thetaDegrees1; 
			} else if (z < 0 && y >=0) {
				rx = -(90 - thetaDegrees1);
			} else if (z < 0 && y < 0) {
				rx = -(90 + thetaDegrees1);
			} else {
				rx = 0.0;
			}
			
			thetaDegrees1 = Math.atan2(Math.abs(z), Math.abs(x)) * 180.0/Math.PI;
			if (z >= 0 && x >=0) {
				ry = 90 - thetaDegrees1;
			} else if (z >= 0 && x < 0) {
				ry = -90 + thetaDegrees1; 
			} else if (z < 0 && x >=0) {
				ry = -90 + thetaDegrees1;
			} else if (z < 0 && x < 0) {
				ry = 90 - thetaDegrees1;
			} else {
				ry = 0.0;
			}

			rz = 0.0;
		}

		// roty controls right and left pointing


		rotx = rx;
		roty = ry;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotationxy2(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);

		double rx;
		double ry;
		double rz;


		Location hloc = variables.getCurrentLocation(h);

		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;

		// rotx  controls in and out of the screen
		if (z == 0.0) {
			// special case for z = 0. if on the same plane, need to set rotation to 0
			rx = 0.0; // z,y
			ry = 0.0;
			rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;
		} else {
//			double thetaDegrees1 = Math.atan2(Math.abs(y), Math.abs(z)) * 180.0/Math.PI;
			double thetaDegrees1 = (Math.atan2(y, z) * 180.0/Math.PI);
//			if (z >= 0 && y >=0) {
//				rx = 90 - thetaDegrees1;
//			} else if (z >= 0 && y < 0) {
//				rx = 90 + thetaDegrees1;
//			} else if (z < 0 && y >=0) {
//				rx = 270 + thetaDegrees1;
//			} else if (z < 0 && y < 0) {
//				rx = 270 - thetaDegrees1;
//			} else {
//				rx = 0.0;
//			}
			
			double atan;
			if (z >= 0 && y >=0) {
				atan = Math.atan2(y, z);
				rx = (Math.atan2(y, z) * 180.0/Math.PI);   // ok
			} else if (z >= 0 && y < 0) {
				atan = Math.atan2(y, -z);
				rx = -(Math.atan2(y, -z) * 180.0/Math.PI);
			} else if (z < 0 && y >=0) {
				atan = Math.atan2(y, -z);
				rx = -(Math.atan2(y, -z) * 180.0/Math.PI);
			} else if (z < 0 && y < 0) {
				atan = Math.atan2(y, z);
				rx = (Math.atan2(y, z) * 180.0/Math.PI);    // ok
			} else {
				atan = 0.0;
				rx = 0.0;
			}
			
			double newz = z*Math.cos(atan) + y*Math.sin(atan);
//			z = newz;
			
//			rx = (Math.atan2(y, z) * 180.0/Math.PI);
			
			//			rx = -90 + Math.atan2(z,y)  * 180.0/Math.PI;  // y, z
			//			ry = Math.atan2(z, x) * 180.0/Math.PI; // 0.0; // 
//			thetaDegrees1 = Math.atan2(Math.abs(z), Math.abs(x)) * 180.0/Math.PI;
//			if (z >= 0 && x >=0) {
//				ry = 90 - thetaDegrees1;
//			} else if (z >= 0 && x < 0) {
//				ry = 270 + thetaDegrees1;
//			} else if (z < 0 && x >=0) {
//				ry = 90 + thetaDegrees1;
//			} else if (z < 0 && x < 0) {
//				ry = 270 - thetaDegrees1;
//			} else {
//				ry = 0.0;
//			}
			
			ry = 180-(Math.atan2(-z, -x) * 180.0/Math.PI);

			rz = 0.0;
		}

		// roty controls right and left pointing


		rotx = rx;
		roty = ry;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public void computeRotation2(Variables variables) {

		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);
		
		double rx;
		double ry;
		double rz;
		
		
		Location hloc = variables.getCurrentLocation(h);
		
		double x = hloc.getAdjustedX() - midx;
		double y = hloc.getAdjustedY() - midy;
		double z = hloc.getAdjustedZ() - midz;
		
		rx = 0.0;
		if (z == 0)
			rx = 0.0;
		else
			rx =  -90 +  Math.atan2(y, z)  * 180.0/Math.PI;
		ry = 0.0;
		rz = -90 + Math.atan2(y, x) * 180.0/Math.PI;
	      
		rotx = rx;
		roty = ry;
		rotz = rz;

		System.out.println(id+" ROT: "+tail+"->"+head+" rot: "+rotx+" "+roty+" "+rotz);
	}
	
	public double aScale(double d) {
		return d;
	}
	

	
	public String asCSV(Variables variables) {
		
		Variable t = variables.getVariable(tail);
		Variable h = variables.getVariable(head);
//		
//		if (t == null || h == null)
//			return null;
//		
		Location hloc = variables.getVarLocation(h);
//		
//		// 180.0/Math.PI
//		
//		
		double x = hloc.getAdjustedX() -  midx;
		double y = hloc.getAdjustedY() -  midy;
		double z = hloc.getAdjustedZ() -  midz;
		
		return tail + "\t" + head + "\t"+ variables.getNameToID().get(tail) + "\t"+ variables.getNameToID().get(head) + 
				"\t" + hasArrowHead + "\t"+ color + "\t" + type + "\t"+ orientation 
				+ "\t"+ midx + "\t"+midy + "\t"+ midz
				+ "\t"+ rotx + "\t"+ roty + "\t"+ rotz 
				+ "\t"+ hloc.getAdjustedX() + "\t"+ hloc.getAdjustedY() + "\t"+ hloc.getAdjustedZ()
				+ "\t"+ x + "\t"+ y + "\t"+ z
				+"\n";
				
				
	}
	
	public double getPointListX() {
		String plx = pointList.split(",")[0].replace("(", "").trim();
		return Double.parseDouble(plx);
	}
	
	public double getPointListY() {
		String ply = pointList.split(",")[1].replace(")", "").trim();
		return Double.parseDouble(ply);
	}
	
	public boolean startsWith(String varName) {
		return tail.equals(varName);
	}
	
	public boolean endsWith(String varName) {
		return head.equals(varName);
	}
	
	public boolean endsWithType(String aType) {
		return false;
	}
	
	public boolean startsWithType(String aType) {
		return false;
	}
	
	public String toString() {
		return id;
	}

	public boolean isHasArrowHead() {
		return hasArrowHead;
	}

	public String getColor() {
		return color;
	}

	public String getPointList() {
		return pointList;
	}

	public String getOrientation() {
		return orientation;
	}

	public void setOrientation(String orientation) {
		this.orientation = orientation;
	}

	public String getType() {
		return type;
	}
	
	public boolean isInfluence() {
		return type.equals(INFLUENCE);
	}
	
	public boolean isFlow() {
		return type.equals(FLOW);
	}
	
	public boolean isHorizontal() {
		return orientation == HORIZONTAL;
	}
	
	public boolean isVertical() {
		return orientation == VERTICAL;
	}

	public double getMidx() {
		return midx;
	}

	public void setMidx(double midx) {
		this.midx = midx;
	}

	public double getMidy() {
		return midy;
	}

	public void setMidy(double midy) {
		this.midy = midy;
	}

	public double getMidz() {
		return midz;
	}

	public void setMidz(double midz) {
		this.midz = midz;
	}

	public double getRotx() {
		return rotx;
	}

	public double getRoty() {
		return roty;
	}

	public double getRotz() {
		return rotz;
	}

	public Variable getTailVariable() {
		return tailVariable;
	}

	public Variable getHeadVariable() {
		return headVariable;
	}

	public void setType(String type) {
		this.type = type;
		
		if (isInfluence()) {
			color = INFLUENCE_COLOR;
		} else {
			color =FLOW_COLOR;
		}
	}

}
