package gov.anl.gss.sdm.server.model.visualization3d;

import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.translator.Arrow;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.GraphicsProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.View;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class ConnectionMap {
	
	private Map<String, Connection> connections;
	private String file;
	private DataManager dataManager;
	
	
	
	public ConnectionMap(String file, DataManager dataManager) {
		connections = new LinkedHashMap<String, Connection>();
		this.file = file;
		this.dataManager = dataManager;
		load();
		
	}
	
	public void setVariables() {
		for (Connection c : connections.values()) {
			c.setVariables();
		}
	}
	
	public void adjustCoordinates() {
		
	}
	
	private void load() {
		
		// we no longer load this information from the tailHeadMap file
		// additional information is required
		
//		for (String line : Utilities.getAsList(file)) {
//			String[] field = line.split("\t");
//			for (int i = 1; i < field.length; i++) {
//				addConnection(field[0], field[i]);
//			}
//		}
		
		// we now generate the connections based on contents of the GrapicObjects
		GraphicsProcessor gp = InformationManagers.getInstance().getGraphicsProcessor();
		Map<String, View> map = gp.getViewMap();
		
		for (View view :map.values()) {
		dataManager.setCurrentView(view.getName());
			
		
//		View view = map.get(dataManager.getCurrentView());
		
		Map<String, GraphicObject> flows = gp.getFlows(view);
		Map<String, GraphicObject> clouds = gp.getClouds(view);
		Map<String, GraphicObject> links = gp.getLinks(view);
		
		Map<String, GraphicObject> allGO = gp.getAllGraphicObjects(view);
		
		// need to create all connections -- defined by links -- need to capture
		// additional information to determine specifics
		//	String tail, String head, boolean hasArrowHead, String color, String type, String orientation,
		//	String pointList
		
		for (GraphicObject obj : links.values()) {
			boolean hasArrowHead = !obj.getShape().equals("100");
			String type = obj.getShape().equals("100") || obj.getShape().equals("4") ? Arrow.FLOW : Arrow.INFLUENCE;
			String color = type.equals(Arrow.INFLUENCE) ? "blue" : "black";
			addConnection(
					allGO.get(obj.getFrom()).getName(),
					allGO.get(obj.getTo()).getName(),
					hasArrowHead,
					color,
					type,
					"",
					obj.getPointList()
					);
		}
		}
	}
	
//	public Connection getFLowConnection(String from, String to) {
//		for (Connection connect : getConnectionsStartingAt(from)) {
//			if (connect.getHead().equals(to))
//				return connect;
//		}
//		return null;
//	}
	
	public Set<Connection> getConnections() {
		Set<Connection> connectionSet = new LinkedHashSet<Connection>(connections.values());
		return connectionSet;
	}
	
	public Set<Connection> getConnectionsStartingAt(String varName) {
		Set<Connection> set = new LinkedHashSet<Connection>();
		
		for (Connection connection : connections.values()) {
			if (connection.startsWith(varName))
				set.add(connection);
		}
		
		return set;
	}
	
	public Set<Connection> getConnectionsEndingAt(String varName) {
		Set<Connection> set = new LinkedHashSet<Connection>();
		
		for (Connection connection : connections.values()) {
			if (connection.endsWith(varName))
				set.add(connection);
		}
		
		return set;
	}
	
	public void addConnection(String tail, String head) {
		Connection c = new Connection(dataManager, tail, head, true, "black", Connection.UNKNOWN, Connection.UNKNOWN,"");
		connections.put(c.getId(), c);
	}
	
	public void addConnection(String tail, String head, boolean hasArrowHead, String color, String type, String orientation,
			String pointList) {
		Connection c = new Connection(dataManager, tail, head, hasArrowHead, color, type, orientation, pointList);
		connections.put(c.getId(), c);
	}
	
	public String getAsCSVString(Variables variables) {
		StringBuilder sb = new StringBuilder();
		
		List<String> ordered = new ArrayList<String>(connections.keySet());
		Collections.sort(ordered);
		
		for (String id : ordered) {
			String csv = connections.get(id).asCSV(variables);
			if (csv != null && !csv.contains("null"))
				sb.append(csv);
		}
		
		return sb.toString();
	}
	
	public void writeConnections(Variables variables, String file) {
//		computeMidPoints(variables);
//		computeRotations(variables);
//		handleOverlaps();
		BufferedWriter bw = Utilities.openFileForWriting(file);
		try {
			bw.append("FromName\tToName\tFromID\tToID\tarrowHead\tcolor\ttype\torientation\tmidx\tmidy\tmidz\trotx\troty\trotz"+
					"\tadjx\tadjy\tadjz\tx\ty\tz\n");
			bw.append(this.getAsCSVString(variables));
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void verifyConnectiontypes() {
		for (Connection c : connections.values()) {
			if (c.isFlow()) {
				// this must be a flow to a stock or to a cloud
				// and tail variable must be flow
				Variable tailVariable = c.getTailVariable();
				Variable headVariable = c.getHeadVariable();
				if (!tailVariable.isFlow() || (!headVariable.isStock() && !headVariable.isCloud())) {
					c.setType(Connection.INFLUENCE);
				}
			}
		}
	}
	
	public void computeMidPoints(Variables variables) {
		for (Connection c : connections.values()) {
			c.computeMidPoint(variables);
		}
	}
	
	public void computeRotations(Variables variables) {
		for (Connection c : connections.values()) {
			c.computeRotation(variables);
		}
	}
	
	public void handleOverlaps() {
		
		// are there multiple connections between these two nodes?
		Set<String> allVars = dataManager.getAllVariableNames();
		for (String tail : allVars) {
			for (String head : allVars) {
				if (tail.equals(head))
					continue;
				Set<Connection> set = dataManager.getConnectionsBetween(tail, head);
				if (set.size() <= 1)
					continue;
				// if both are INFLUENCE
				int influence = 0;
				int flow = 0;
				for (Connection c : set) {
					if (c.isInfluence())
						influence++;
					if (c.isFlow())
						flow++;
				}
				// we want to modify one influence if with flow
				// ideally both if both are influence, but do just one now;
				double deltax = Visualization3DData.OVERLAP_DELTA;
				double deltay = Visualization3DData.OVERLAP_DELTA;
				int numProcessed = 0;
				for (Connection c : set) {
					if (c.isInfluence()) {
						if (numProcessed++ > 0) {
							deltax = -1 * deltax;
							deltay = -1 * deltay;
						}
						if (flow > 0) {
							if (c.isHorizontal()) {
								c.setMidy(c.getMidy()+deltay);
							} else if (c.isVertical()) {
								c.setMidx(c.getMidx()+deltax);
							} else {
								c.setMidy(c.getMidy()+deltay);
								c.setMidx(c.getMidx()+deltax);
							}
						} else {
							c.setMidy(c.getMidy()+deltay);
							c.setMidx(c.getMidx()+deltax);
						}
						numProcessed++;
					}
				}
			}
		}
		
	}

}
