package gov.anl.gss.sdm.server.model.visualization3d;

import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.translator.GraphicsProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.View;

import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

public class DataManager {

	private ConnectionMap connections;
	private Variables variables;
	
	private String currentView;
	
	private double xAdjust;
	private double yAdjust;
	private double zAdjust;
	private double yMax;
	
	public DataManager(ConnectionMap connections, Variables variables) {
		this.connections = connections;
		this.variables = variables;
	}
	
	public DataManager() {
	}
	
	public void adjustVariableCoordinates() {
		variables.adjustCoordinates(this);
	}
	
	public void adjustConnectionCoordinates() {
		connections.adjustCoordinates();
	}
	
	public void determineAdjustments(Map<Variable, MutableInteger> miZ) {
		variables.determineAdjustments(miZ);
	}
	
	public Set<String> getAllVariableNames() {
		Set<String> set = new LinkedHashSet<String>();
		set.addAll(variables.getVariablesMap().keySet());
		
		return set;
	}
	
	public Variable getVariable(String name) {
		return variables.getVariable(name);
	}
	
	public Set<Connection> getConnectionsStartingAt(String name) {
		Set<Connection> s =  connections.getConnectionsStartingAt(name);
		
		if (s != null)
			return s;
		
		return new LinkedHashSet<Connection>();
	}
	
	public Set<Connection> getConnectionsBetween(String tail, String head) {
		Set<Connection> set = new LinkedHashSet<Connection>();
		for (Connection c : connections.getConnections()) {
			
			if ((c.startsWith(tail) && c.endsWith(head)) || 
					(c.startsWith(head) && c.endsWith(tail))) {
				set.add(c);
			}
			
		}
		
		return set;
	}
	
	public void setOrientationForAllConnectionsBetween(String tail, String head, String orientation) {
		for (Connection c : getConnectionsBetween(tail, head)) {
			c.setOrientation(orientation);
		}
	}
	
	public Set<Connection> getConnectionsEndingAt(String name) {
		
		Set<Connection> s =  connections.getConnectionsEndingAt(name);
		
		if (s != null)
			return s;
		
		return new LinkedHashSet<Connection>();
	}

	public void setConnections(ConnectionMap connections) {
		this.connections = connections;
	}

	public void setVariables(Variables variables) {
		this.variables = variables;
	}
	
	public Set<String> getViews() {
		return variables.getViews();
	}
	
	public Set<String> getVarsInView(String view) {
		return variables.getViews();
	}
	
	public Set<Location> getLocationsFor(String var) {
		return variables.getLocationsFor(var);
	}
	
	public Location getOneLocationFor(String var) {
		return variables.getLocationsFor(var).iterator().next();
	}
	
	public void fillConnections() {
		variables.fillConnections();
	}
	
	public void createClouds() {
		GraphicsProcessor gp = InformationManagers.getInstance().getGraphicsProcessor();
		for (View view : gp.getViewMap().values()) {
			setCurrentView(view.getName());
			variables.createClouds();
		}
	}
	
//	public Connection getFLowConnection(String from, String to) {
//		return connections.getFLowConnection(from, to);
//	}
	
	public Set<Variable> getVariablesOfType(String aType) {
		return variables.getVariablesOfType(aType);
	}
	
//	public void addConnection(String tail, String head) {
//		connections.addConnection(tail, head);
//	}
	
	public void addConnection(String tail, String head, boolean hasArrowHead, String color, String type, String orientation,
			String pointList) {
		connections.addConnection(tail, head, hasArrowHead, color, type, orientation, pointList);
		for (Connection c : getConnectionsBetween(tail, head)) {
			c.setVariables();
		}
	}

	public String getCurrentView() {
		return currentView;
	}

	public void setCurrentView(String currentView) {
		this.currentView = currentView;
	}

	public double getxAdjust() {
		return xAdjust;
	}

	public void setxAdjust(double xAdjust) {
		this.xAdjust = xAdjust;
	}

	public double getyAdjust() {
		return yAdjust;
	}

	public void setyAdjust(double yAdjust) {
		this.yAdjust = yAdjust;
	}

	public double getzAdjust() {
		return zAdjust;
	}

	public void setzAdjust(double zAdjust) {
		this.zAdjust = zAdjust;
	}

	public double getyMax() {
		return yMax;
	}

	public void setyMax(double yMax) {
		this.yMax = yMax;
	}
}
