package gov.anl.gss.sdm.server.model.visualization3d;

public class SphericalCoordinates {

	private double x;
	private double y;
	private double z;
	
	private double r;
	private double theta;
	private double phi;
	
	/* 
	The spherical coordinates (r,theta,phi) are related to the Cartesian coordinates (x,y,z) by
	
	r	=	sqrt(x^2+y^2+z^2)	
	(1) theta	=	tan^(-1)(y/x)	
	(2) phi	=	cos^(-1)(z/r)
	
	where r in [0,infty), theta in [0,2pi), and phi in [0,pi], 
		and the inverse tangent must be suitably defined to take the correct quadrant of (x,y) into account. 
	*/
	
	public SphericalCoordinates(double x, double y, double z) {
		this.x = x;
		this.y = y;
		this.z = z;
		computeR();
		computeTheta();
		computePhi();
	}
	
	private void computeR() {
		r = Math.sqrt(x*x + y*y + z*z);
	}
	
	private void computeTheta() {  // rotx?
		theta = Math.atan(y/x);
	}

	private void computePhi() {  // roty?
		phi = Math.acos(z/r);
	}

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}

	public double getZ() {
		return z;
	}
	
	private double toDegrees(double r) {
		return r * 180.0/Math.PI;
	}

	public double getR() {
		return r;
	}

	public double getTheta() {
		return toDegrees(theta);
	}

	public double getPhi() {
		return toDegrees(phi);
	}
}
