package gov.anl.gss.sdm.server.model.visualization3d;

import gov.anl.gss.sdm.server.model.support.MutableInteger;

import java.util.LinkedHashSet;
import java.util.Set;


public class Variable {
	
	private String name;
	private String type;
	
	private DataManager dataManager;
	
	private Set<Connection> inConnections = new LinkedHashSet<Connection>();
	private Set<Connection> outConnections = new LinkedHashSet<Connection>();
	
	private MutableInteger z;
	
	public Variable(DataManager dataManager, String name, String type) {
		this.dataManager = dataManager;
		this.name = name;
		this.type = type;
	}

	public String getName() {
		return name;
	}

	public String getType() {
		return type;
	}
	
	public boolean isOfType(String aType) {
		return type.equals(aType);
	}
	
	public boolean isFlow() {
		return type.equals(Variables.FLOW);
	}
	
	public boolean isStock() {
		return type.equals(Variables.STOCK);
	}
	
	public boolean isAuxiliary() {
		return type.equals(Variables.AUXILIARY);
	}
	
	public boolean isCloud() {
		return type.equals(Variables.CLOUD);
	}
	
	public void fillConnections() {
		inConnections.addAll(dataManager.getConnectionsEndingAt(name));
		outConnections.addAll(dataManager.getConnectionsStartingAt(name));
	}
	
	public int getCountIn() {
		return inConnections.size();
	}
	
	public int getCountOut() {
		return outConnections.size();
	}
	
	public int getCountIn(String aType) {
		int count = 0;
		for (Connection con : inConnections) {
			// the tail must be of type
			if (dataManager.getVariable(con.getTail()).isOfType(aType))
				count++;
		}
		return count;
	}
	
	public int getCountOut(String aType) {
		int count = 0;
		for (Connection con : outConnections) {
			// the head must be of type
			if (dataManager.getVariable(con.getHead()).isOfType(aType))
				count++;
		}
		return count;
	}
	
	
	public int getCountTotal() {
		return getCountIn() + getCountOut();
	}
	
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append(name);
//		sb.append("#####\n");
//		sb.append(name);
//		sb.append(" "+type+"\n");
//		sb.append("# IN #\n");
//		for (Connection con : inConnections) {
//			sb.append("  "+con+"\n");
//		}
//		sb.append("# OUT #\n");
//		for (Connection con : outConnections) {
//			sb.append("  "+con+"\n");
//		}
		
		return sb.toString();
	}
	
	public Set<Variable> getInFromType(String aType) {
		Set<Variable> set = new LinkedHashSet<Variable>();
		for (Connection con : inConnections) {
			// the tail must be of type
			if (dataManager.getVariable(con.getTail()).isOfType(aType))
				set.add(dataManager.getVariable(con.getTail()));
		}
		return set;
	}
	
	public Set<Variable> getOutToType(String aType) {
		Set<Variable> set = new LinkedHashSet<Variable>();
		for (Connection con : outConnections) {
			// the tail must be of type
			if (dataManager.getVariable(con.getHead()).isOfType(aType))
				set.add(dataManager.getVariable(con.getHead()));
		}
		return set;
	}

	public MutableInteger getZ() {
		return z;
	}

	public void setZ(MutableInteger z) {
		if (z.value() == 0)
			System.out.println("ZERO Z");
		this.z = z;
		Set<Location> set =  dataManager.getLocationsFor(name);
		
		if (set != null && set.size() > 0) {
		for (Location l : dataManager.getLocationsFor(name)) {
			if (l != null)
				l.setZ(Integer.toString(z.value()));
		}
		}
	}

	public Set<Connection> getInConnections() {
		return inConnections;
	}

	public Set<Connection> getOutConnections() {
		return outConnections;
	}

}
