package gov.anl.gss.sdm.server.model.visualization3d;

import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.support.database.SDModelRunDatabase;
import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.GraphicsProcessor;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.View;

import java.io.BufferedWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.MathContext;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

public class Variables {
	
	public final static String STOCK = "stock";
	public final static String FLOW = "flow";
	public final static String AUXILIARY = "auxiliary";
	public final static String CLOUD = "cloud";
	
	public final String[] SYSTEM = new String[] {"FINAL TIME", "INITIAL TIME", "NAREPLACEMENT", "SAVEPER", "TIME STEP", "Time"};
	
	private Map<String, Variable> variables;
	private Map<String, Values> values;
	private Map<String, Set<Location>> locations;
	
	private Map<String, Integer> nameToID;
	
	private int id = 0;
	
	
	private String databaseFile;
	private String locationFile;
	private String variablesFile;
	
	private DataManager dataManager;
	
	double[] timeVector;
	boolean okToProceed;
	
	int cloudID = 0;
	
	public Variables(DataManager dataManager, String variablesFile, String locationFile, String databaseFile) {
		this.dataManager = dataManager;
		this.variablesFile = variablesFile;
		this.locationFile = locationFile;
		this.databaseFile =databaseFile;
		
		nameToID = new LinkedHashMap<String, Integer>();
		variables = new LinkedHashMap<String, Variable>();
		values = new LinkedHashMap<String, Values>();
		locations = new LinkedHashMap<String, Set<Location>>();
		okToProceed = load();
	}
	
	public Variable getVariable(String varName) {
		return variables.get(varName);
	}
	
	private boolean load() {
		loadVariables();
		loadLocations();
		
		return loadValues();
//		for (Variable var : variables.values())
//			var.fillConnections();
	}

	
	public void createClouds() {
		
		// need to construct clouds based on grpahic objects
		
		GraphicsProcessor gp = InformationManagers.getInstance().getGraphicsProcessor();
		Map<String, View> map = gp.getViewMap();
		View view = map.get(dataManager.getCurrentView());
		Map<String, GraphicObject> flows = gp.getFlows(view);
		Map<String, GraphicObject> clouds = gp.getClouds(view);
		Map<String, GraphicObject> links = gp.getLinks(view);
		
		for (GraphicObject aCloud : clouds.values()) {
			// find link to/from associated with this cloud -- there can be only one
			GraphicObject link = gp.getAssociatedLink(aCloud, links);
			// now find the flow to or from
			GraphicObject flow = gp.getAssociatedFlow(link, flows);
			
			Variable cloud =  new Variable(dataManager, /* newCloud()*/ aCloud.getName(), CLOUD); //  variables.get(aCloud.getName()); 
			variables.put(cloud.getName(), cloud);
			nameToID.put(cloud.getName(), id++);
			
			Location flowLoc = getVarLocation(dataManager.getVariable(flow.getName()));
			Location cloudLoc = null;
			// get the "other" link from the rate to/from stock
			GraphicObject other = null;
			for (GraphicObject go : links.values()) {
				if (go.getFrom().equals(flow.getId()) && !go.equals(link)) {
					other = go;
					break;
				}
			}
			
			String otherVarName = view.getIdNameMap().get(other.getTo());
			Location stockLoc = getVarLocation(dataManager.getVariable(otherVarName));
			
			// need to determine if flow is left or above stock
			
			boolean horizontal;
			
			double xDiff = stockLoc.getXd() - flowLoc.getXd();
			double yDiff = stockLoc.getYd() - flowLoc.getYd();
			if (Math.abs(xDiff) > Math.abs(yDiff)) {
				horizontal = true;
				if (xDiff > 0) {
					cloudLoc = locateCloud(cloud, null, flowLoc, stockLoc, link.getShape());
				} else {
					cloudLoc = locateCloud(cloud, stockLoc, flowLoc, null, link.getShape());
				}
				dataManager.addConnection(flow.getName(),cloud.getName(),  (link.getShape().equals("100") ? false : true), "black", Connection.FLOW,
						(horizontal  ? Connection.HORIZONTAL : Connection.VERTICAL), link.getPointList());
				dataManager.setOrientationForAllConnectionsBetween(flow.getName(), otherVarName, horizontal ? Connection.HORIZONTAL : Connection.VERTICAL);
			} else {
				horizontal = false;
				if (yDiff > 0) {
					cloudLoc = locateCloud(cloud, null, flowLoc, stockLoc, link.getShape());
				} else {
					cloudLoc = locateCloud(cloud, stockLoc, flowLoc, null, link.getShape());
				}
				dataManager.addConnection(flow.getName(), cloud.getName(), (link.getShape().equals("100") ? false : true), "black", Connection.FLOW,
						(horizontal  ? Connection.HORIZONTAL : Connection.VERTICAL), link.getPointList());
				dataManager.setOrientationForAllConnectionsBetween(flow.getName(), cloud.getName(), horizontal ? Connection.HORIZONTAL : Connection.VERTICAL);
				
			}
			
			Set<Location> set = new LinkedHashSet<Location>();
			set.add(cloudLoc);
			locations.put(cloud.getName(), set);
		}
		
	}
	
	
	public Location locateCloud(Variable cloud, Location leftOrTopStock, Location flow, Location rightOrBottomStock, String shape) {
		// new location can be in any direction
		// first figure out the direction of the existing arrow: left, right, up, down
		// if shape is 100, the arrow is headless so if from flow to stock and 100, the direction is switched
		// if shape is 4, 
		Location loc = null;
		Location tail;
		Location head;
		boolean flowToStock;
		boolean leftToRight;
		int state;
		if (leftOrTopStock == null) {  
			if (shape.equals("100")) {   
				tail = rightOrBottomStock;
				head = flow;
				flowToStock = false;
				leftToRight = false;
				state = 0;
			} else {
				tail = flow;
				head = rightOrBottomStock;
				flowToStock = true;
				leftToRight = true;
				state = 1;
			}
		} else {
			if (shape.equals("100")) {   
				tail = leftOrTopStock;
				head = flow;
				flowToStock = false;
				leftToRight = true;
				state = 2;
			} else {
				tail = flow;
				head = leftOrTopStock;
				flowToStock = true;
				leftToRight = false;
				state = 3;
			}
			
		}
		
		double xDiff = Math.abs(head.getXd() - tail.getXd());
		double yDiff = Math.abs(head.getYd() - tail.getYd());
		
		boolean horizontal = Math.abs(xDiff) > Math.abs(yDiff);
		double xd;
		double yd;
		
		String view = dataManager.getCurrentView();
		if (horizontal) {
			String y;
			
			switch (state) {
			case 0:
				xd = head.getXd() - xDiff;
				y = tail.getY();
				head.setY(y);
				break;
			case 1:
				xd = tail.getXd() - xDiff;
				y = head.getY();
				tail.setY(y);
				break;
			case 2:
				xd = head.getXd() + xDiff;
				y = tail.getY();
				head.setY(y);
				break;
			case 3:
				xd = tail.getXd() + xDiff;
				y = head.getY();
				tail.setY(y);
				break;
			default :
				xd = tail.getXd() + xDiff;
				y = head.getY();
				tail.setY(y);
				break;
			}
			
			loc = new Location(view, Double.toString(xd), y);
		} else {
			String x;
			
			switch (state) {
			case 0:
				yd = head.getYd() - yDiff;
				x = tail.getX();
				head.setX(x);
				break;
			case 1:
				yd = tail.getYd() - yDiff;
				x = head.getX();
				tail.setX(x);
				break;
			case 2:
				yd = head.getYd() + yDiff;
				x = tail.getX();
				head.setX(x);
				break;
			case 3:
				yd = tail.getYd() + yDiff;
				x = head.getX();
				tail.setX(x);
				break;
			default :
				yd = tail.getYd() + yDiff;
				x = head.getX();
				tail.setX(x);
				break;
			}
			
			loc = new Location(view, x, Double.toString(yd));
		}
		
		
		return loc;
		
	}
	
	public Location getVarLocation(Variable flowVar) {
		Location loc = null;
		for (Location l : locations.get(flowVar.getName())) {
			if (l.getView().equals(dataManager.getCurrentView())) {
				loc = l;
				break;
			}
		}
		
		return loc;
	}
	
	public Location getToStockLocation(Set<Connection> set) {
		Location loc = null;
		for (Connection c : set) {
			String var = c.getHead();
			if (variables.get(var).isOfType(STOCK)) {
				loc = getVarLocation(variables.get(var));
				break;
			}
		}
		return loc;
	}
	
	public Location getFromStockLocation(Set<Connection> set) {
		Location loc = null;
		for (Connection c : set) {
			String var = c.getTail();
			if (variables.get(var).isOfType(STOCK)) {
				loc = getVarLocation(variables.get(var));
				break;
			}
		}
		return loc;
	}
	
	public String newCloud() {
		return "Cloud_"+(cloudID++);
	}
	
	public void fillConnections() {
		for (Variable var : variables.values())
			var.fillConnections();
	}
	
	private void loadVariables() {
		
		for (String line : Utilities.getAsList(variablesFile)) {
			String[] field = line.split("\t");
			variables.put(field[0], new Variable(dataManager, field[0], field[6]));
			nameToID.put(field[0], id++);
		}
		
		// make sure we have these in there as they are filtered from the input file
		
		for (String var : SYSTEM) {
			variables.put(var, new Variable(dataManager, var, AUXILIARY));
			nameToID.put(var, id++);
		}
	}
	private void loadLocations() {
		for (String line : Utilities.getAsList(locationFile)) {
			String[] field = line.split("\t");
//			Map<String, Set<Location>> locations;
			if (!locations.containsKey(field[0]))
				locations.put(field[0], new LinkedHashSet<Location>());
			locations.get(field[0]).add(new Location(field[1], field[2], field[3]));
		}
		
	}
	private boolean loadValues() {
		SDModelRunDatabase db = new SDModelRunDatabase(databaseFile);
		boolean ok;
		if (db.dbUsable()) {
			db.load();
			db.loadRunParameters();
			timeVector = db.getTimeVector();
			for (String var : db.getVariables() /*variables.keySet() */) {
				values.put(var, new Values(db.getVarTimeVector(var)));
			}
			ok = true;
		} else {
			ok = false;
		}
		db.shutdown();
		return ok;
	}
	
	public boolean systemVariable(String var) {
		for (String v : SYSTEM) {
			if (v.equals(var))
				return true;
		}
		return false;
	}
	
	public void adjustCoordinates(DataManager dataManager) {
		
		for (String var : variables.keySet()) {
			Variable v = variables.get(var);
			Set<Location> l = locations.get(var);
			if (l != null) {
				for (Location loc : l) {
					if (loc != null) {
						loc.adjustCoordinates(dataManager.getxAdjust(), dataManager.getyAdjust(), dataManager.getzAdjust(), dataManager.getyMax());
					}
				}
			} else {
				// these should be system variables
				if (systemVariable(var)) {
					if (!locations.containsKey(var)) {
						locations.put(var, new LinkedHashSet<Location>());
					}
					Set<Location> s = locations.get(var);
					s.add(new Location(dataManager.getCurrentView(), dataManager.getxAdjust(), 
							dataManager.getyAdjust(), dataManager.getzAdjust()));
//					s.add(new Location(dataManager.getCurrentView(), 0.0, 
//							0.0, 0.0));
				}
			}
		}
	}
	
	public void determineAdjustments(Map<Variable, MutableInteger> miZ) {
		
		double xyMult = Visualization3DData.SPREAD_XY;
		double zMult = Visualization3DData.SPREAD_Z; 
		double yMax;
		double xAdjust = 0.0;
		double yAdjust = 0.0;
		double zAdjust = 0.0;

		
		for (String var : variables.keySet()) {
			Variable v = variables.get(var);
			Set<Location> l = locations.get(var);
			if (l != null) {
				int locCount = 0;
				for (Location loc : l) {
					if (locCount++ > 0)
						break;
					
					if (loc != null) {
						double x = (Double.parseDouble(loc.getX())*xyMult);
						double y = (Double.parseDouble(loc.getY())*xyMult);
						double z = 0;
						if (miZ.containsKey(v)) {
							z = (miZ.get(v).value()*zMult);
						} 
						xAdjust = Math.max(x, xAdjust);
						yAdjust = Math.max(y, yAdjust);
						zAdjust = Math.max(z, zAdjust);
					}
				}
			} 
		}
		
		yMax = yAdjust;

		xAdjust /= 2;
		yAdjust /= 2;
		zAdjust /= 2;
		
		dataManager.setxAdjust(xAdjust);
		dataManager.setyAdjust(yAdjust);
		dataManager.setzAdjust(zAdjust);
		
//		dataManager.setxAdjust(0.0);
//		dataManager.setyAdjust(0.0);
//		dataManager.setzAdjust(0.0);
		
		dataManager.setyMax(yMax);
	}
	
	public void writeVariables(String file, Map<Variable, MutableInteger> miZ) {
		BufferedWriter bw = Utilities.openFileForWriting(file);
		
//		double yMax = dataManager.getyMax();
		
		try {
			bw.append("VID,Name,type,Shape,View,X,Y,Z\n");
			for (String var : variables.keySet()) {
				Variable v = variables.get(var);
				Set<Location> l = locations.get(var);
				if (l != null) {
					int locCount = 0;
				for (Location loc : l) {
					if (locCount++ > 0)
						break;
					if (loc != null) {
						double newx = loc.getAdjustedX();
						double newy = loc.getAdjustedY();
						double newz = loc.getAdjustedZ();
						bw.append(nameToID.get(var)+",");
						
							bw.append(var+","+v.getType()+","+shape(v.getType())+","+loc.getView()+","+newx+","+
									newy+","+newz+"\n");
						
					}
				}
				}
			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public String shape(String type) {
		if (type.equals(AUXILIARY)) {
			return "a-sphere";
		} else if (type.equals(STOCK)) {
			return "a-box";
		} else if (type.equals(FLOW)) {
			return "a-cone";
		} else if (type.equals(CLOUD)) {
			return "a-tetrahedron";
		} else {
			return "a-cone";
		}
	}
	
	private String format(double value) {
		
		try {
			BigDecimal bd = new BigDecimal(value);
			bd = bd.round(new MathContext(7));
			return bd.toPlainString();
		} catch (Exception e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
			if (Double.isNaN(value))
				return "Nan";
			if (Double.isInfinite(value))
				return "Infinite";
			return "BAD double value";
		}
	}
	
	public void writeValuesVector(String file) {
		BufferedWriter bw = Utilities.openFileForWriting(file);
		
		try {
			bw.append("Time");
			for (int i = 0; i < timeVector.length; i++) {
				bw.append(","+format(timeVector[i]));
			}
			bw.append("\n");
			
			for (String var : values.keySet() /* variables.keySet() */) {
				bw.append(var);
				double[] vals = values.get(var).getValues();
				for (int i = 0; i < vals.length; i++) {
					bw.append(","+format(vals[i]));
				}
				bw.append("\n");
			}
			
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	
	public void writeValues(String file) {
		BufferedWriter bw = Utilities.openFileForWriting(file);
		
		try {
			for (String var : values.keySet() /*variables.keySet()*/) {
				double[] vals = values.get(var).getValues();
				for (int i = 0; i < timeVector.length; i++) {
					bw.append(var+","+format(timeVector[i])+","+format(vals[i])+"\n");
				}
			}
			
			for (String var : variables.keySet() /*variables.keySet()*/) {
				if (!var.startsWith("C_L_O_U_D"))
					continue;

				
				for (int i = 0; i < timeVector.length; i++) {
					bw.append(var+","+format(timeVector[i])+","+format(0.0)+"\n");
				}
			}
			
			bw.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public Set<Variable> getVariablesOfType(String aType) {
		Set<Variable> variablesOfType = new LinkedHashSet<Variable>();
		
		for (Variable var : variables.values()) {
			if (var.isOfType(aType))
				variablesOfType.add(var);
		}
		
		return variablesOfType;
	}
	
	public Set<String> getViews() {
		Set<String> views = new LinkedHashSet<String>();
		for (Set<Location> locSet : locations.values()) {
			for (Location loc : locSet) {
				views.add(loc.getView());
			}
		}
		
		return views;
	}
	
	public Set<Variable> getVariablesInView(String view) {
		Set<Variable> vars = new LinkedHashSet<Variable>();
		for (String var : locations.keySet()) {
			Set<Location> locSet = locations.get(var);
			for (Location loc : locSet) {
				if (loc.inView(view))
				vars.add(getVariable(var));
			}
		}
		return vars;
	}
	
	public Set<Location> getLocationsFor(String var) {
		return locations.get(var);
	}

	public Set<Variable> getVariables() {
		Set<Variable> set = new LinkedHashSet<Variable>(variables.values());
		return set;
	}
	
	public Map<String, Variable> getVariablesMap() {
		
		return variables;
	}

	public boolean isOkToProceed() {
		return okToProceed;
	}

	public Map<String, Integer> getNameToID() {
		return nameToID;
	}

	public DataManager getDataManager() {
		return dataManager;
	}
	
	public Location getCurrentLocation(Variable var) {
		for (Location loc : getLocationsFor(var.getName())) {
			if (loc.getView().equals(dataManager.getCurrentView()))
				return loc;
		}
		return null;
	}

	public Map<String, Set<Location>> getLocations() {
		return locations;
	}

}
