package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Dimension extends XmileObject {

//	private Name name;
	private String size;
	private boolean numbered = false;
	private List<String> elements = new ArrayList<String>();

	public Dimension(Node dimensionNode) {

		if (XMLUtilities.hasAttribute(dimensionNode, "name")) {
			name = new Name(XMLUtilities.getAttribute(dimensionNode, "name"));
		} else {
			wellFormed = false;
		}

		if (XMLUtilities.hasAttribute(dimensionNode, "size")) {
			size = XMLUtilities.getAttribute(dimensionNode, "size");
			numbered = true;
			for (int i = 1; i <= Integer.parseInt(size); i++) {
				elements.add("\""+name.getNormalizedName()+"_"+Integer.toString(i)+"\"");
			}
			if (XMLUtilities.hasElementsWithTag(dimensionNode, "elem"))
				wellFormed = false;
		} 
		
		if (XMLUtilities.hasElementsWithTag(dimensionNode, "elem")) {
			for (Element element : XMLUtilities.getElementsWithTag(dimensionNode, "elem")) {
				elements.add(XMLUtilities.getAttribute(element, "name"));
			}
			if (XMLUtilities.hasAttribute(dimensionNode, "size"))
				wellFormed = false;
		} 

	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (name != null)
			name.disperse(xmileModel);
	}

	@Override
	protected void print() {
		System.out.println("// array subscript definition");
		System.out.println(getName()+": "+Utilities.asCsv(elements));
		
	}
	
	public String getEquation() {
		return getName()+": "+Utilities.asCsv(elements)+"~\t~\t|";
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}

	/**
	 * @return the size
	 */
	public String getSize() {
		return size;
	}

	/**
	 * @return the numbered
	 */
	public boolean isNumbered() {
		return numbered;
	}

	/**
	 * @return the elements
	 */
	public List<String> getElements() {
		return elements;
	}
}
