package gov.anl.gss.sdm.server.model.xmile;

import gov.anl.gss.sdm.server.model.translator.Equation;
import gov.anl.gss.sdm.server.model.translator.EquationComponents;

import java.util.LinkedHashMap;
import java.util.Map;

public class EquationBuilder {
	
	private XmileModel xmileModel;
	
	public EquationBuilder(XmileModel xmileModel) {
		this.xmileModel = xmileModel;
	}
	
	public Map<String, Equation> build() {
		Map<String, Equation> equations = new LinkedHashMap<String, Equation>();
		for (Dimension dimension : xmileModel.getDimensions().getDimensions()) {
			EquationComponents ec = new EquationComponents(dimension.getEquation());
			Equation eqn = new Equation(ec);
			equations.put(eqn.getLhs(), eqn);
		}
		for (Model model : xmileModel.getModels()) {
			for (BuildingBlock buildingBlock : model.getVariables().getEquations()) {
				for (String anEquation : buildingBlock.getFullEquationAsList(null)) {
					EquationComponents ec = new EquationComponents(anEquation);
					Equation eqn = new Equation(ec);
					equations.put(eqn.getLhs(), eqn);
				}
			}

//			for (NonApplyToAllArray nata : model.getVariables().getMultiEquations()) {
//				for (String anEquation : nata.getMultiEquations()) {
//					EquationComponents ec = new EquationComponents(anEquation);
//					Equation eqn = new Equation(ec);
//					equations.put(eqn.getLhs(), eqn);
//				}
//			}

			
		}

		SimSpecs simSpecs = xmileModel.getSimSpecs();
		String units = simSpecs.getTimeUnits();

		EquationComponents equationComponents = new EquationComponents("FINAL TIME  = "+simSpecs.getStop()+"~"+units+"~The final time for the simulation. (Autogenerated)");
		Equation equation = new Equation(equationComponents);
		equations.put(equation.getLhs(), equation);

		equationComponents = new EquationComponents("INITIAL TIME  = "+simSpecs.getStart()+"~	"+units+"~The initial time for the simulation. (Autogenerated)");
		equation = new Equation(equationComponents);
		equations.put(equation.getLhs(), equation);

		equationComponents = new EquationComponents("SAVEPER  = TIME STEP~"+units+" [0,?]~	The frequency with which output is stored. (Autogenerated)");
		equation = new Equation(equationComponents);
		equations.put(equation.getLhs(), equation);

		equationComponents = new EquationComponents("TIME STEP  = "+simSpecs.getDt()+"~	"+units+" [0,?]~	The time step for the simulation. (Autogenerated)");
		equation = new Equation(equationComponents);
		equations.put(equation.getLhs(), equation);

		return equations;
	}

}
