package gov.anl.gss.sdm.server.model.xmile;

import org.w3c.dom.Node;

public class GraphicalFunction extends XmileObject {

	private XPoints xpoints;
	private YPoints ypoints;
	private XScale xscale;
	private YScale yscale;

	// optional
	private Name name;
	private String type;

	public GraphicalFunction(Node gfNode) {
		
		if (XMLUtilities.hasElementWithTag(gfNode, "xscale")) {
			XScale xs = new XScale(XMLUtilities.getElementWithTag(gfNode, "xscale"));
			if (xs.isWellFormed())
				xscale = xs;
		}
		
		if (XMLUtilities.hasElementWithTag(gfNode, "yscale")) {
			YScale ys = new YScale(XMLUtilities.getElementWithTag(gfNode, "yscale"));
			if (ys.isWellFormed())
				yscale = ys;
		}
		
		if (XMLUtilities.hasElementWithTag(gfNode, "xpts")) {
			XPoints xp = new XPoints(XMLUtilities.getElementWithTag(gfNode, "xpts"));
			if (xp.isWellFormed())
				xpoints = xp;
		}
		
		if (XMLUtilities.hasElementWithTag(gfNode, "ypts")) {
			YPoints yp = new YPoints(XMLUtilities.getElementWithTag(gfNode, "ypts"));
			if (yp.isWellFormed())
				ypoints = yp;
		}
		
		if (XMLUtilities.hasAttribute(gfNode, "type"))
		 type = XMLUtilities.getAttribute(gfNode, "type");
		
	}
	
	public String getLookupDefinitionEquation(Name name) {
		
		String lookupName = "";
		StringBuilder sb = new StringBuilder(/* getComment() */);
			lookupName = name.getNormalizedName()+"_lookup";
			sb.append(getLookup(lookupName));
		return sb.toString();
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (xpoints != null)
			xpoints.disperse(xmileModel);
		if (ypoints != null)
			ypoints.disperse(xmileModel);
		if (xscale != null)
			xscale.disperse(xmileModel);
		if (yscale != null)
			yscale.disperse(xmileModel);
	}
	
	public String getLookup(String name) {
		StringBuilder sb = new StringBuilder();
		sb.append("(");
		if (xscale != null && yscale != null) {
			sb.append("[("+xscale.getMin()+","+yscale.getMin()+")");
			sb.append("-");
			sb.append("("+xscale.getMax()+","+yscale.getMax()+")],");
		}
		
		if (xpoints != null && ypoints!= null) {
			sb.append(getPairs(xpoints, ypoints));
			sb.append(")");
			return sb.toString();
			
		}
			
		// xpoints should be null
		// need to generate
		
		xpoints = new XPoints(xscale, ypoints.getNumPoints());
		sb.append(getPairs(xpoints, ypoints));
		sb.append(")");
		
		return sb.toString();
	}
	
	private String getPairs(XPoints xpoints, YPoints ypoints) {
		StringBuilder sb = new StringBuilder();
		// there should be the same number of points
		String[] x = xpoints.getXpoints().split(",");
		String[] y = ypoints.getYpoints().split(",");
		if (x.length == y.length) {
			for (int i = 0; i < x.length; i++) {
				if (i > 0)
					sb.append(",");
				sb.append("("+x[i]+","+y[i]+")");
			}
		}

		return sb.toString();
	}

	/**
	 * @return the xpoints
	 */
	public XPoints getXpoints() {
		return xpoints;
	}

	/**
	 * @return the ypoints
	 */
	public YPoints getYpoints() {
		return ypoints;
	}

	/**
	 * @return the xscale
	 */
	public XScale getXscale() {
		return xscale;
	}

	/**
	 * @return the yscale
	 */
	public YScale getYscale() {
		return yscale;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}

	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}

	@Override
	protected void print() {
		System.out.println("Graphical Function:");
		
		xpoints.print();
		ypoints.print();
		xscale.print();
		yscale.print();
		
	}


}
