package gov.anl.gss.sdm.server.model.xmile;

import org.w3c.dom.Node;

public class Header extends XmileObject {
	
	// Required pieces
	private Options options;
	private String vendor;
	private Product product;
	
	// Optional pieces
	private Name name;
	private String version;
	private String caption;
	private Image image;
	private String author;
	private String affiliation;
	private String client;
	private String copyright;
	private Contact contact;
	private String created;
	private String modified;
	private String uuid;
	
	public Header(Node headerNode) {
		// required -- only one
//		if (!XMLUtilities.hasElementWithTag(headerNode, "options")) {
			int elementCount = XMLUtilities.elementCountWithTag(headerNode, "options");
			if (elementCount > 1) {
				System.out.println("ERROR: expecting at most 1 options tag, found "+elementCount);
			wellFormed = false;
			return;
			}
//		}
		if (!XMLUtilities.hasElementWithTag(headerNode, "vendor")) {
			 elementCount = XMLUtilities.elementCountWithTag(headerNode, "vendor");
			System.out.println("ERROR: expecting 1 vendor tag, found "+elementCount);
			wellFormed = false;
			return;
		}
		if (!XMLUtilities.hasElementWithTag(headerNode, "product")) {
			 elementCount = XMLUtilities.elementCountWithTag(headerNode, "product");
			System.out.println("ERROR: expecting 1 product tag, found "+elementCount);
			wellFormed = false;
			return;
		}
		
		product = new Product(XMLUtilities.getElementWithTag(headerNode, "product"));
		if (!product.isWellFormed()) {
			product = null;
			wellFormed = false;
		}
		
		elementCount = XMLUtilities.elementCountWithTag(headerNode, "options");
		if (elementCount == 1) {
			options = new Options(XMLUtilities.getElementWithTag(headerNode, "options"));
			if (!options.isWellFormed()) {
				options = null;
				wellFormed = false;
			}
		}
		
		name = new Name(XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "name")));
		vendor = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "vendor"));
		version = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "version"));
		caption = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "caption"));
		author = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "author"));
		affiliation = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "affiliation"));
		client = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "client"));
		copyright = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "copyright"));
		created = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "created"));
		modified = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "modified"));
		uuid = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(headerNode, "uuid"));
		
		if (XMLUtilities.hasElementWithTag(headerNode, "image")) {
			image = new Image(XMLUtilities.getElementWithTag(headerNode, "image"));
			if (!image.isWellFormed()) {
				image = null;
				wellFormed = false;
			}
		}
		
		if (XMLUtilities.hasElementWithTag(headerNode, "contact")) {
			contact = new Contact(XMLUtilities.getElementWithTag(headerNode, "contact"));
			if (!contact.isWellFormed()) {
				contact = null;
				wellFormed = false;
			}
		}
		
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);

		if (options != null)
			options.disperse(xmileModel);
		
		if (product != null)
			product.disperse(xmileModel);
		if (name != null)
			name.disperse(xmileModel);
		if (image != null)
			image.disperse(xmileModel);
		if (contact != null)
			contact.disperse(xmileModel);
	}
	/**
	 * @return the smile
	 */
	public Options getSmile() {
		return options;
	}
	/**
	 * @return the vendor
	 */
	public String getVendor() {
		return vendor;
	}
	/**
	 * @return the product
	 */
	public Product getProduct() {
		return product;
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}
	/**
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}
	/**
	 * @return the caption
	 */
	public String getCaption() {
		return caption;
	}
	/**
	 * @return the image
	 */
	public Image getImage() {
		return image;
	}
	/**
	 * @return the author
	 */
	public String getAuthor() {
		return author;
	}
	/**
	 * @return the affiliation
	 */
	public String getAffiliation() {
		return affiliation;
	}
	/**
	 * @return the client
	 */
	public String getClient() {
		return client;
	}
	/**
	 * @return the copyright
	 */
	public String getCopyright() {
		return copyright;
	}
	/**
	 * @return the contact
	 */
	public Contact getContact() {
		return contact;
	}
	/**
	 * @return the created
	 */
	public String getCreated() {
		return created;
	}
	/**
	 * @return the modified
	 */
	public String getModified() {
		return modified;
	}
	/**
	 * @return the uuid
	 */
	public String getUuid() {
		return uuid;
	}
	@Override
	protected void print() {
		// TODO Auto-generated method stub
		
	}

	
}
