package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Macro extends XmileObject {
	
	private String filter;
	private Name name;
	
	private List<Parm> params = new ArrayList<Parm>();
	private Eqn equation;

	public Macro(Node macroNode) {

		for (Element element : XMLUtilities.getElementsWithTag(macroNode, "parm")) {
			Parm p = new Parm(element);
			if (p.isWellFormed())
				params.add(p);
			else
				wellFormed = false;
		}
		
		Eqn e = new Eqn("",XMLUtilities.getElementWithTag(macroNode, "eqn"));
		if (e.isWellFormed())
			equation = e;
		else
			wellFormed = false;
		
		filter = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(macroNode, "filter"));
		name = new Name(XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(macroNode, "name")));
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (name != null)
			name.disperse(xmileModel);
		if (equation != null)
			equation.disperse(xmileModel);
		for (Parm p : params)
			p.disperse(xmileModel);
	}

	@Override
	protected void print() {
		System.out.println("Macro:");
		System.out.println("   filter: "+filter);

	}

	/**
	 * @return the filter
	 */
	public String getFilter() {
		return filter;
	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}

	/**
	 * @return the params
	 */
	public List<Parm> getParams() {
		return params;
	}

	/**
	 * @return the equation
	 */
	public Eqn getEquation() {
		return equation;
	}
}
