package gov.anl.gss.sdm.server.model.xmile;

import gov.anl.gss.sdm.server.model.translator.GraphicObject;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;
import org.w3c.dom.Node;


public class Model extends XmileObject {

	private Name name;
	private String run;
	private String password;
	private String namespace;
	private String url;
	
	private static Map<String, String> connectors = new LinkedHashMap<String, String>();
	
	private Level2Display display;
	
	
	private Views views;
	private Behavior behavior;
	private Variables variables;
	private Map<String, BuildingBlock> variableInfo = new LinkedHashMap<String, BuildingBlock>();
	private Map<String, ViewBuildingBlock> viewVariableInfo = new LinkedHashMap<String, ViewBuildingBlock>();
	
	private boolean addModelName = false;
	
	public static int nextID = 1;
	
	List<Object> orderedObjects = new ArrayList<Object>();
	
	public Model(boolean addModelName, Node modelNode) {
		
		name = new Name(XMLUtilities.getAttribute(modelNode, "name"));
		
		this.addModelName =  addModelName;
		
//		if (name.getNormalizedName().length() == 0)
//			this.addModelName = true;
		
		
		if (XMLUtilities.hasElementWithTag(modelNode, "variables")) {
			
			Element varsElement = XMLUtilities.getElementWithTag(modelNode, "variables"); {
				Variables vars = new Variables((this.addModelName ? name.getNormalizedName()+"." : ""),
						varsElement, variableInfo);
					if (vars.isWellFormed())
						variables = vars;
			}
		}
		
		if (XMLUtilities.hasElementWithTag(modelNode, "views")) {
			
			Element viewsElement = XMLUtilities.getElementWithTag(modelNode, "views"); {
				Views vars = new Views((this.addModelName ? name.getNormalizedName()+"." : ""),viewsElement,
						viewVariableInfo);
					if (vars.isWellFormed())
						views = vars;
			}
		}
		
		
		
		if (XMLUtilities.hasElementsWithTag(modelNode, "display")) {
			// lets look only for one with connectors
			for (Element e : XMLUtilities.getElementsWithTag(modelNode, "display")) {
				if (XMLUtilities.hasElementsWithTag(e, "connector")) {
					Level2Display disp = new Level2Display(this.addModelName ? name.getNormalizedName()+"." : "",e);
					if (disp.isWellFormed())
						display = disp;
				}
			}
		}
		

		run = XMLUtilities.getAttribute(modelNode, "run");
		password = XMLUtilities.getAttribute(modelNode, "password");
		namespace = XMLUtilities.getAttribute(modelNode, "namespace");
		url = XMLUtilities.getAttribute(modelNode, "url");
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);

		if (name != null)
			name.disperse(xmileModel);
		if (display != null)
			display.disperse(xmileModel);
		if (views != null)
			views.disperse(xmileModel);
		if (behavior != null)
			behavior.disperse(xmileModel);
		if (variables != null)
			variables.disperse(xmileModel);
	}
	
	public static int resetNextID() {
		nextID = 1;
		return nextID;
	}
	
	private int getNextID() {
		return nextID++;
	}
	
	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}
	/**
	 * @return the run
	 */
	public String getRun() {
		return run;
	}
	/**
	 * @return the password
	 */
	public String getPassword() {
		return password;
	}
	/**
	 * @return the namespace
	 */
	public String getNamespace() {
		return namespace;
	}
	/**
	 * @return the url
	 */
	public String getUrl() {
		return url;
	}

	/**
	 * @return the display
	 */
	public Level2Display getDisplay() {
		return display;
	}
	@Override
	protected void print() {
		System.out.println("Model:");
		
		if (name != null)
			name.print();
		if (variables != null)
			variables.print();
		
		if (behavior != null)
			behavior.print();
		
		if (views != null)
			views.print();
		
//		System.out.println("   run:"+run);
//		System.out.println("   password:"+password);
//		System.out.println("   namespace:"+namespace);
//		System.out.println("   url:"+url);
		
	}

	/**
	 * @return the behavior
	 */
	public Behavior getBehavior() {
		return behavior;
	}
	/**
	 * @return the variables
	 */
	public Variables getVariables() {
		return variables;
	}
	/**
	 * @return the views
	 */
	public Views getViews() {
		return views;
	}
	public List<String> getAsVensimView() {

		
		Map<String, String> nameToID = new LinkedHashMap<String, String>();
		Map<String, String> idToName = new LinkedHashMap<String, String>();
//		loadVensimIDs(nameToID, idToName);
		List<String> vensimRaw = generateVensimRaw(nameToID, idToName);
		return vensimRaw;
	}
	
	private void loadVensimIDs(int pass, View view, Map<String, String> nameToID, Map<String, String> idToName) {
		// assign IDs
		// this is an ordered process:
		// all aux and stocks can be defined
		// all flows (clouds, valves, flows) as needed
		// all arrows/connectors
		
		// variables
		orderedObjects.clear();
		// 2 pass processing
		if (pass == 1) {
			nameToID.clear();
			idToName.clear();
		} 
		
		for (ViewBuildingBlock bb : view.getBuildingBlocks()) {
			if (bb.getType().equals("flow"))
				continue;
			String name = bb.getNormalizedName();
			String pattern = InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(name);
			orderedObjects.add(bb);
//			if (!nameToID.containsKey(pattern)) {
				String sid = Integer.toString(getNextID());
				nameToID.put(pattern, sid);
				idToName.put(sid, pattern);
//			}
		}
		
		// arrows
//		for (View v : views.getViews()) {
			for (Connector c : view.getConnectors()) {
				String name = c.getName();
				String pattern = InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(name);
				orderedObjects.add(c);
//				if (!nameToID.containsKey(pattern)) {
					String sid = Integer.toString(getNextID());
					nameToID.put(pattern, sid);
					idToName.put(sid, pattern);
					
//				}
			}
//		}
		
			// cannot define id until we've created valve!
		for (ViewBuildingBlock bb : view.getBuildingBlocks()) {
			if (!bb.getType().equals("flow"))
				continue;
			String name = bb.getNormalizedName();
			if (pass == 1)
				this.allocateIDsVensimRawFlow(bb, nameToID, idToName);
			String pattern = InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(name);
			orderedObjects.add(bb);
			if (pass == 1 && !nameToID.containsKey(pattern)) {
				String sid = Integer.toString(getNextID());
				nameToID.put(pattern, sid);
				idToName.put(sid, pattern);
			}
		}
		
	}
	
	public void loadConnectors() {
		if (variables.hasModules()) {
			connectors = variables.getModuleConectors();
		}
	}
	
	public List<String> asVensim() {
		List<String> equations = new ArrayList<String>();
//		if (variables.hasModules()) {
//			connectors = variables.getModuleConectors();
//		}
		
		if (variables.getEquations().size() == 0)
			return equations;
		equations.add("********************************************************\n");
		equations.add("\t"+getName()+"\n");
		equations.add("********************************************************\n");
		equations.add("\t|\n");
		for (BuildingBlock bb : variables.getEquations()) {
			
//			if (bb.getNormalizedNameQuoted().startsWith("\"Init_"))
//				System.out.println("DEBUG INIT");
			try {
				addAllNoDups(equations, bb.asVensim(variableInfo), connectors);
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
//		equations.addAll(variables.getModuleEquations());
		return equations;
	}
	
	private void addAllNoDups(List<String> mainList, List<String> toAdd, Map<String, String> connectors) {
		
		for (String eqn : toAdd) {
			String eqnt = eqn.replace("TIME()", "TIME").replace("NaN", "0.0");
			if (!mainList.contains(eqnt)) {
				mainList.add(applyConnectors(eqnt, connectors));
			}
		}
	}
	
	private String applyConnectors(String eqn, Map<String, String> converters) {
		String alteredEqn = new String(eqn);
		if (converters.size() > 0) {
			for (String var : converters.keySet()) {
				if (alteredEqn.contains("\""+var+"\"")) {
					alteredEqn = alteredEqn.replace("\""+var+"\"", "\""+converters.get(var)+"\"");
				}
			}
		}
		
		if (!alteredEqn.equals(eqn)) {
			System.out.println("AC  IN: "+eqn);
			System.out.println("AC OUT: "+alteredEqn);
		}

		return alteredEqn;
	}
	
	public List<String> asVensimView() {
		
		return getAsVensimView();
	}
	
	private List<String> generateVensimRaw(Map<String, String> nameToID, Map<String, String> idToName) {

		List<String> vensimRaw = new ArrayList<String>();

		int viewNum = 1;
		int resetValue = -1;
		for (View v : views.getViews()) {
			if (!v.isHasGraphics())
				continue;
			for (int pass = 1; pass <= 2; pass++) {
				
				if (pass == 1) {
					resetValue = nextID;
				} else {
					nextID = resetValue;
				}
				
			loadVensimIDs(pass, v, nameToID, idToName);
			

			if (pass == 2) {
				vensimRaw.add("\\\\\\---/// Sketch information - do not modify anything except names");
				vensimRaw.add("V300  Do not put anything below this section - it will be ignored");
				vensimRaw.add("*"+(v.getName() == null || v.getName().equals("NONAME") ? getGeneratedViewName() : v.getName()));
				vensimRaw.add("$192-192-192,0,Times New Roman|12||0-0-0|0-0-0|0-0-255|-1--1--1|-1--1--1|72,72,100,0");
			}

			// arrows

			for (Object obj : orderedObjects) {
				if (obj instanceof Connector) {
					Connector c = (Connector) obj;
					List<String> al = this.getAsVensimRaw(c, nameToID, idToName);
					if (pass == 2)
						vensimRaw.addAll(al);
				} else if (obj instanceof ViewBuildingBlock) {
					ViewBuildingBlock bb = (ViewBuildingBlock) obj;
					if (bb.getType().equals(BuildingBlock.FLOW)) {
						List<String> al = this.getAsVensimRawFlow(bb, nameToID, idToName);
						if (pass == 2)
							vensimRaw.addAll(al);
					} else {
						List<String> al = this.getAsVensimRaw(bb, nameToID, idToName, 0, 0);
						if (pass == 2)
							vensimRaw.addAll(al);
					}
				}
			}
			if (pass == 2) {
//			vensimRaw.add("///---\\\\\\");
			}
		}
	}
		


		return vensimRaw;
	}
	
	private String getGeneratedViewName() {
		return "Generated Name "+ XmileModel.generatedViewNameID++;
	}
	
	private List<String> getAsVensimRawFlow(ViewBuildingBlock bb, Map<String, String> nameToID, Map<String, String> idToName) {
		List<String> al = new ArrayList<String>();

		// this consists of the valve, the flow variable, fromStock, toStock, is either stock null then cloud

		//		11,3,1948,569,902,5,6,34,3,0,0,1,0,0,0
		//		10,4,Discovery of Reserves,569,936,64,27,40,3,0,0,-1,0,0,0


		int cloudID;
		int valveID = nextID;

		Set<String> ins = variables.getStocksWithOutflowTo(bb.getName());
		valveID += countValveConnectors(valveID, "100", bb, ins, nameToID, idToName);

		// the outflow from the valve
		Set<String> outs = variables.getStocksWithInflowFrom(bb.getName());
		valveID += countValveConnectors(valveID, "4", bb, outs, nameToID, idToName);

		// the inflow to the valve
		ins = variables.getStocksWithOutflowTo(bb.getName());
		al.addAll(getValveConnectors(valveID, "100", bb, ins, nameToID, idToName));

		// the outflow from the valve
		outs = variables.getStocksWithInflowFrom(bb.getName());
		al.addAll(getValveConnectors(valveID, "4", bb, outs, nameToID, idToName));

		// the valve
		valveID = getNextID();
		String[] fields = new String[15]; // 18
		fields[0] = GraphicObject.VALVE;
		fields[1] = Integer.toString(valveID);
		fields[2] = "48";
		fields[3] = Integer.toString((int) Double.parseDouble(bb.getX()));
		fields[4] = Integer.toString((int) Double.parseDouble(bb.getY()));
		fields[5] = "8";  //
		fields[6] = "8";

		for (int i = 7; i < fields.length; i++)
			fields[i] = "0";  // DC

		fields[7] = "34";
		fields[8] = "3";
		
		fields[11] = "1";
		al.add(asCSV(fields));

		ViewFlow flow = (ViewFlow) bb;

		// the flow associated with the valve
		al.addAll(getAsVensimRaw(bb, nameToID, idToName, 0, 15));
		getNextID();

		return al;
	}
	
	private void allocateIDsVensimRawFlow(ViewBuildingBlock bb, Map<String, String> nameToID, Map<String, String> idToName) {
		
		// the inflow to the valve
		
				Set<String> ins = variables.getStocksWithOutflowTo(bb.getName());
				allocateValveConnectorIDs("100", bb, ins, nameToID, idToName);
				
				// the outflow from the valve
				Set<String> outs = variables.getStocksWithInflowFrom(bb.getName());
				allocateValveConnectorIDs("4", bb, outs, nameToID, idToName);
				int valveID = getNextID();
		// the valve
				
	}
	
	private void allocateValveConnectorIDs(String type, ViewBuildingBlock bb, Set<String> vars, Map<String, String> nameToID, Map<String, String> idToName) {
		if (vars.size() > 0) {
			for (String s : vars) {
				getNextID();
			}
		} else {
			getNextID();
			getNextID();
		}
		
	}
	
	private int countValveConnectors(int valveID, String type, ViewBuildingBlock bb, Set<String> vars, Map<String, String> nameToID, Map<String, String> idToName) {
		int count = 0;
		if (vars.size() > 0) {
			for (String s : vars) {
				count++;
			}
		} else {
			count++;
			count++;
		}
		return count;
	}
	
	private List<String> getValveConnectors(int valveID, String type, ViewBuildingBlock bb, Set<String> vars, Map<String, String> nameToID, Map<String, String> idToName) {
		int cloudID;
		List<String> al = new ArrayList<String>();
		if (vars.size() > 0) {
			for (String s : vars) {
				al.add(getValveArrowConnection(valveID, bb, bb.getName(), s, type, nameToID, idToName));
			}
		} else {
			cloudID = getNextID();
			al.add(getCloud(cloudID, bb, 100, 0));
			al.add(getValveArrowConnection(valveID, bb, bb.getName(), cloudID, type, nameToID, idToName));
		}
		return al;
	}
	
	private List<String> getAsVensimRaw(ViewBuildingBlock bb, Map<String, String> nameToID, 
			Map<String, String> idToName, int xOffset, int yOffset) {
		List<String> al = new ArrayList<String>();
		
		String[] fields = new String[15]; // 18
		
//		// Variables
//		10,8,Oil Intermediate Storage,1049,1017,70,162,3,131,0,0,0,3,0,0
//	    		
//		type = fields[0];
		fields[0] = GraphicObject.VARIABLE;
//		id = fields[1];
//		fields[1] = nameToID.get(bb.getNormalizedName());
		if (!nameToID.containsKey(InformationManagers.getInstance().getVariableNamePatternGenerator().
				getPatternFor(bb.getNormalizedName()))) {
			fields[1] = Integer.toString(getNextID());
		} else { 
			fields[1] = nameToID.get(InformationManagers.getInstance().getVariableNamePatternGenerator().
					getPatternFor(bb.getNormalizedName()));
		}
//		name = fields[2].replace("\"", "");
		
		int xPos = (int) Double.parseDouble(bb.getX())+xOffset;
		int yPos = (int) Double.parseDouble(bb.getY())+yOffset;
		
		fields[2] = bb.getNormalizedName(); // bb.getNormalizedNameQuoted();
		fields[3] = Integer.toString(xPos);
		fields[4] = Integer.toString(yPos);
		
		fields[5] = bb.getWidth();
		fields[6] = bb.getHeight();
		for (int i = 7; i < fields.length; i++)
			fields[i] = "0";  // DC
		
		if (bb.getType().equals("stock")) {
			fields[7] = "3";
		} else if (bb.getType().equals("aux")) {
			fields[7] = "8";
		} else if (bb.getType().equals("flow")) {
			fields[7] = "40";
			
		}
		fields[8] = "3";
//		x = fields[3];
//		y = fields[4];
//		width = fields[5];
//		height = fields[6];
//		shape = fields[7];
//		bits = fields[8];
//		if (bitwiseAnd(bits, mask2))
//		    additionalText = new String(view.getNextRawObject());
//		hidden = fields[9];
//		hasFont = fields[10];
//		textPos = fields[11];
//		boxWidth = fields[12];
//		nav1 = fields[13];
//		nav2 = fields[14];
//		if (nav1.equals("253") && nav2.equals("253"))
//		    additionalText = new String(view.getNextRawObject());
//		if (!hasFont.equals("0")) {
//		    boxColor = fields[15];
//		    fillColor = fields[16];
//		    font = fields[17];
//		}
		
		al.add(asCSV(fields));
		return al;
	}
	
	private String getCloud(int id, ViewBuildingBlock bb, int xOffset, int yOffset) {
		int xPos = (int) Double.parseDouble(bb.getX()) + xOffset;
		int yPos = (int) Double.parseDouble(bb.getY()) + yOffset;
		return "12,"+id+",48,"+xPos+","+yPos+",10,8,0,3,0,0,-1,0,0,0";
	}
	
	private String getValveArrowConnection(int valveID, ViewBuildingBlock bb, String valve, int cloud, String arrowType, Map<String, String> nameToID, Map<String, String> idToName) {
//		StringBuilder sb = new StringBuilder();
//		sb.append(valve);
//		sb.append("->");
//		sb.append(cloud);
//		return sb.toString();
		
		String[] fields = new String[14];
		
//		// Arrows
//		1,44,45,41,100,0,0,22,0,0,0,-1--1--1,,1|(809,398)|
//	    public static final String ARROW = "1";		
//		type = fields[0];
		fields[0] = GraphicObject.ARROW;
//		id = fields[1];
		fields[1] = Integer.toString(getNextID());
//		from = fields[2];
//		fields[2] = nameToID.get(VariableNamePatternGenerator.getPatternFor(valve));
		fields[2] = Integer.toString(valveID);
//		to = fields[3];
		fields[3] = Integer.toString(cloud);
//		shape = fields[4];
		fields[4] = arrowType; // DC
//		hidden = fields[5];
		fields[5] = "0"; // DC
//		polarity = fields[6];
		fields[6] = "0"; // DC
//		thickness = fields[7];
		fields[7] = "22"; // DC
//		hasFont = fields[8];
		fields[8] = "0"; // DC
//		delayType = fields[9];
		fields[9] = "0"; // DC
//		reserved = fields[10];
		fields[10] = "0"; // DC
//		color = fields[11];
		fields[11] = "-1--1--1"; // DC
//		font = fields[12];
		fields[12] = ""; // DC
//		pointList = reconstructPointlist(fields, 13);
//		fields[13] = "1|("+0+","+0+")|"; // DC
		
		// type = 4 is link from stock to valve
		// type = 100 is link from valve to stock
		// points should be 4, 100 in that order
		
		List<Point> pts = bb.getPoints().getPoints();
		int x1 = (int) Double.parseDouble(pts.get(0).getX());
		int y1 = (int) Double.parseDouble(pts.get(0).getY());
		int x2 = (int) Double.parseDouble(pts.get(1).getX());
		int y2 = (int) Double.parseDouble(pts.get(1).getY());
		
		int xPos;
		int yPos;
		
		int valveX = (int) Double.parseDouble(bb.getX());
		int valveY = (int) Double.parseDouble(bb.getY());
		
		boolean horizontal = y1 == y2;
		
		if (arrowType.equals("100")) {
			if (horizontal) {
				yPos = y1;
				xPos = (valveX+x1)/2;
			} else {
				xPos = x1;
				yPos = (valveY+y1)/2;
			}
		} else {
			if (horizontal) {
				yPos = y2;
				xPos = (valveX+x2)/2;
			} else {
				xPos = x2;
				yPos = (valveY+y2)/2;
			}
		}
		fields[13] = "1|("+xPos+","+yPos+")|"; 
		
		return asCSV(fields);
		
	}

	private String getValveArrowConnection(int valveID, ViewBuildingBlock bb, String valve, String stock, String arrowType, Map<String, String> nameToID, Map<String, String> idToName) {
		
//		1,3,5,2,4,0,0,22,0,0,0,-1--1--1,,1|(441,235)|
//		1,4,5,1,100,0,0,22,0,0,0,-1--1--1,,1|(374,235)|

//		StringBuilder sb = new StringBuilder();
//		sb.append(valve);
//		sb.append("->");
//		sb.append(stock);
//		return sb.toString();
		
		String[] fields = new String[14];
		
//		// Arrows
//		1,44,45,41,100,0,0,22,0,0,0,-1--1--1,,1|(809,398)|
//	    public static final String ARROW = "1";		
//		type = fields[0];
		fields[0] = GraphicObject.ARROW;
//		id = fields[1];
		fields[1] = Integer.toString(getNextID());
//		from = fields[2];
//		fields[2] = nameToID.get(VariableNamePatternGenerator.getPatternFor(valve));
		fields[2] = Integer.toString(valveID);
//		to = fields[3];
		fields[3] = nameToID.get(InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(stock));
//		shape = fields[4];
		fields[4] = arrowType; // DC
//		hidden = fields[5];
		fields[5] = "0"; // DC
//		polarity = fields[6];
		fields[6] = "0"; // DC
//		thickness = fields[7];
		fields[7] = "22"; // DC
//		hasFont = fields[8];
		fields[8] = "0"; // DC
//		delayType = fields[9];
		fields[9] = "0"; // DC
//		reserved = fields[10];
		fields[10] = "0"; // DC
//		color = fields[11];
		fields[11] = "-1--1--1"; // DC
//		font = fields[12];
		fields[12] = ""; // DC
//		pointList = reconstructPointlist(fields, 13);
//		fields[13] = "1|("+100+","+100+")|"; // DC   y value  of valve, xvalue of stock
		
		// type = 4 is link from stock to valve
		// type = 100 is link from valve to stock
		// points should be 4, 100 in that order
		
		List<Point> pts = bb.getPoints().getPoints();
//		int x1 = Integer.parseInt(pts.get(0).getX());
//		int y1 = Integer.parseInt(pts.get(0).getY());
//		int x2 = Integer.parseInt(pts.get(1).getX());
//		int y2 = Integer.parseInt(pts.get(1).getY());
		
		int x1 = (int) Double.parseDouble(pts.get(0).getX());
		int y1 = (int) Double.parseDouble(pts.get(0).getY());
		int x2 = (int) Double.parseDouble(pts.get(1).getX());
		int y2 = (int) Double.parseDouble(pts.get(1).getY());
		
		int xPos;
		int yPos;
		
//		int valveX = Integer.parseInt(bb.getX());
//		int valveY = Integer.parseInt(bb.getY());
		
		int valveX = (int) Double.parseDouble(bb.getX());
		int valveY = (int) Double.parseDouble(bb.getY());
		
		boolean horizontal = y1 == y2;
		
		if (arrowType.equals("100")) {
			if (horizontal) {
				yPos = y1;
				xPos = (valveX+x1)/2;
			} else {
				xPos = x1;
				yPos = (valveY+y1)/2;
			}
		} else {
			if (horizontal) {
				yPos = y2;
				xPos = (valveX+x2)/2;
			} else {
				xPos = x2;
				yPos = (valveY+y2)/2;
			}
		}
		fields[13] = "1|("+xPos+","+yPos+")|"; 
		
		return asCSV(fields);
	}

	private List<String> getAsVensimRaw(Connector c, Map<String, String> nameToID, Map<String, String> idToName) {
		
		List<String> al = new ArrayList<String>();
		
		String[] fields = new String[14];
		
//		// Arrows
//		1,44,45,41,100,0,0,22,0,0,0,-1--1--1,,1|(809,398)|
//	    public static final String ARROW = "1";		
//		type = fields[0];
		fields[0] = GraphicObject.ARROW;
//		id = fields[1];
		fields[1] = nameToID.get(InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(c.getName()));
		String[] parts = c.getName().split("->");
//		from = fields[2];
		fields[2] = nameToID.get(InformationManagers.getInstance().getVariableNamePatternGenerator().
				getPatternFor( Parser.removeQuotes(parts[0]))); // .replace("\"", "")));
//		to = fields[3];
		fields[3] = nameToID.get(InformationManagers.getInstance().getVariableNamePatternGenerator().
				getPatternFor( Parser.removeQuotes(parts[1]))); // .replace("\"", "")));
//		shape = fields[4];
		fields[4] = "1"; // DC
//		hidden = fields[5];
		fields[5] = "0"; // DC
//		polarity = fields[6];
		fields[6] = "0"; // DC
//		thickness = fields[7];
		fields[7] = "0"; // DC
//		hasFont = fields[8];
		fields[8] = "0"; // DC
//		delayType = fields[9];
		fields[9] = "64"; // DC
//		reserved = fields[10];
		fields[10] = "0"; // DC
//		color = fields[11];
		fields[11] = "-1--1--1"; // DC
//		font = fields[12];
		fields[12] = ""; // DC
//		pointList = reconstructPointlist(fields, 13);
		if (c.getX().length() > 0 && c.getX().length() > 0) {
			fields[13] = "1|("+c.getX()+","+c.getY()+")|"; // DC
		} else {
			String fromName = idToName.get(fields[2]);
			String toName = idToName.get(fields[3]);
			String x = computeX(fromName, toName);
			String y = computeY(fromName, toName);
			if (x.length() > 0 && y.length() > 0) {
				fields[13] = "1|("+x+","+y+")|"; // DC
			} else {
				fields[13] = "1|("+c.getX()+","+c.getY()+")|"; // DC
			}
		}
		
		al.add(asCSV(fields));
		return al;
	}
	
	private String computeX(String from, String to) {
		
		String patFrom = from;
		String patTo = to;
		ViewBuildingBlock bbFrom = null;
		ViewBuildingBlock bbTo = null;

		if (viewVariableInfo.containsKey(patFrom)) {
			bbFrom = viewVariableInfo.get(patFrom);
		} else {
			return "";
		}
		
		if (viewVariableInfo.containsKey(patTo)) {
			bbTo = viewVariableInfo.get(patTo);
		} else {
			return "";
		}
		
		double xFrom = Double.parseDouble(bbFrom.getX());
		double xTo = Double.parseDouble(bbTo.getX());
		
		return Integer.toString((int) ((xFrom+xTo)/2.0));
	}
	
	private String computeY(String from, String to) {
		
		String patFrom = from;
		String patTo = to;
		ViewBuildingBlock bbFrom = null;
		ViewBuildingBlock bbTo = null;

		if (viewVariableInfo.containsKey(patFrom)) {
			bbFrom = viewVariableInfo.get(patFrom);
		} else {
			return "";
		}
		
		if (viewVariableInfo.containsKey(patTo)) {
			bbTo = viewVariableInfo.get(patTo);
		} else {
			return "";
		}
		
		double yFrom = Double.parseDouble(bbFrom.getY());
		double yTo = Double.parseDouble(bbTo.getY());
		
		return Integer.toString((int) ((yFrom+yTo)/2.0));
	}
	
	private String asCSV(String[] fields) {
		StringBuilder sb = new StringBuilder();
		int i = 0;
		for (String s : fields) {
			if (i++ > 0)
				sb.append(",");
			sb.append(s);
		}
		
		return sb.toString();
	}
	/**
	 * @return the topLevel
	 */
	public boolean isTopLevel() {
		return addModelName;
	}
}
