package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Module extends XmileObject {
	
	private Name name;
	
	private List<Connect> connections = new ArrayList<Connect>();
	private Display display = null;

	public Module(Node moduleNode) {
		
		name = new Name(XMLUtilities.getAttribute(moduleNode, "name"));
		
		if (XMLUtilities.hasElementsWithTag(moduleNode, "connect")) {
			for (Element element : XMLUtilities.getElementsWithTag(moduleNode, "connect")) {
				Connect connect = new Connect(element);
				if (connect.isWellFormed()) {
					connections.add(connect);
//					System.out.println("SHADOW VARIABLE DEFINITION IN Module "+name.getAsAppears());
					connect.print();
				}
			}
		}
		
		if (XMLUtilities.hasElementWithTag(moduleNode, "display")) {
			for (Element element : XMLUtilities.getElementsWithTag(moduleNode, "display")) {
				Display disp = new Display(element);
				if (disp.isWellFormed())
					display = disp;
			}
		}
	}

	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (name != null)
			name.disperse(xmileModel);
		if (display != null)
			display.disperse(xmileModel);
		for (Connect c : connections)
			c.disperse(xmileModel);
	}
	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}
	
	protected List<Connect> getConectors() {
		List<Connect> al = new ArrayList<Connect>();
		for (Connect connect : connections) {
			if (!al.contains(connect))
				al.add(connect);
		}
		return al;
		
	}
	
	protected List<String> getEquations() {
		List<String> al = new ArrayList<String>();
		for (Connect connect : connections) {
			String e = connect.getEquation();
			if (!al.contains(e))
			al.add(e);
		}
		return al;
		
	}

	@Override
	protected void print() {
		System.out.println("Module: "+name.getAsAppears());
		for (Connect connect : connections) {
			connect.print();
		}
		if (display != null)
			display.print();
		
	}

	/**
	 * @return the connections
	 */
	public List<Connect> getConections() {
		return connections;
	}
	
	public Connect getConnection(String moduleName, String varName) {
		if (name.equals(moduleName)) {
			for (Connect c : connections) {
				if (c.getTo().equals(varName))
					return c;
			}
			return null;
		} else {
			return null;
		}
		
	}
}
