package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class NonApplyToAllArray  extends XmileObject{
	
	// this is a multi equation statement
	
	private Name name;
	private Dimensions dimensions;
	private Set<BuildingBlock> equations = new LinkedHashSet<BuildingBlock>();
	private Display display;
	
	boolean wellFormed = true;

	public NonApplyToAllArray(String modelName, Node arraylNode) {
	
		
	for (Element element : XMLUtilities.getElementsWithTag(arraylNode, "stock")) {
		Stock stock = new Stock(modelName, element);
		if (stock.isWellFormed())
			equations.add(stock);
		
	}
	for (Element element : XMLUtilities.getElementsWithTag(arraylNode, "flow")) {
		Flow flow = new Flow(modelName, element);
		if (flow.isWellFormed())
			equations.add(flow);
		
	}
	for (Element element : XMLUtilities.getElementsWithTag(arraylNode, "aux")) {
		Auxiliary auxiliary = new Auxiliary(modelName, element);
		if (auxiliary.isWellFormed())
			equations.add(auxiliary);
		
	}
	
	if (XMLUtilities.hasElementsWithTag(arraylNode, "display")) {
		// lets look only for one with connectors
		for (Element e : XMLUtilities.getElementsWithTag(arraylNode, "display")) {
			
				Display disp = new Display(e);
				if (disp.isWellFormed())
					display = disp;
			}
		}
	
	name = new Name(XMLUtilities.getAttribute(arraylNode, "name"));
	}
	
	public Set<BuildingBlock> getEquations() {
		return equations;
	}
	
	public List<String> getMultiEquations() {
		List<String> al = new ArrayList<String>();
		for (BuildingBlock bb : equations) {
			al.add(name.getNormalizedName()+"["+bb.getSubscript()+"] = "+
				bb.getEquation()+"\t~\t"+(bb.units != null ? bb.units : "")+"\t~\t"+(bb.doc != null ? bb.doc.getComment() : ""));
		}
		
		return al;
	}

	/**
	 * @return the wellFormed
	 */
	public boolean isWellFormed() {
		return wellFormed;
	}

	@Override
	protected void print() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void disperse(XmileModel xmileModel) {
		
		setXmileModel(xmileModel);
		
		if (name != null)
			name.disperse(xmileModel);
		
		if (display != null)
			display.disperse(xmileModel);
		
		if (dimensions != null)
			dimensions.disperse(xmileModel);
		
		for (BuildingBlock bb : equations)
			bb.disperse(xmileModel);
	}
}
