package gov.anl.gss.sdm.server.model.xmile;

import org.w3c.dom.Node;

public class Options extends XmileObject {

	private String version;
	private boolean usesConveyor;
	private boolean usesQueue;
	private boolean usesArrays;
	private String maxDimensions;
	private boolean usesSubmodels;
	
	private boolean usesMacros;
	private boolean usesEventPosters;
	private boolean hasModelView;
	private boolean usesInputs;
	private boolean usesOutputs;
	private boolean usesAnnotations;
	
	private boolean isWellFormed = true;
	
	public Options(Node smileNode) {
		version = XMLUtilities.getAttribute(smileNode, "version");
		usesQueue = XMLUtilities.hasElementWithTag(smileNode, "uses_queue");
		usesArrays = XMLUtilities.hasElementWithTag(smileNode, "uses_arrays");
			
		if (usesArrays) {
			maxDimensions = XMLUtilities.getAttribute(XMLUtilities.getElementWithTag(smileNode, "uses_arrays"), "max_dimensions");
		}
		
		usesSubmodels = XMLUtilities.hasElementWithTag(smileNode, "uses_submodels");
		
		usesConveyor = XMLUtilities.hasElementWithTag(smileNode, "uses_conveyor");
		usesMacros = XMLUtilities.hasElementWithTag(smileNode, "uses_macros");
		usesEventPosters = XMLUtilities.hasElementWithTag(smileNode, "uses_event_posters");
		hasModelView = XMLUtilities.hasElementWithTag(smileNode, "uses_model_view");
		usesInputs = XMLUtilities.hasElementWithTag(smileNode, "uses_inputs");
		usesOutputs = XMLUtilities.hasElementWithTag(smileNode, "uses_outputs");
		usesAnnotations = XMLUtilities.hasElementWithTag(smileNode, "uses_annotations");
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
	}
	
	/**
	 * @return the version
	 */
	public String getVersion() {
		return version;
	}
	/**
	 * @return the usesConveyor
	 */
	public boolean isUsesConveyor() {
		return usesConveyor;
	}
	/**
	 * @return the usesQueue
	 */
	public boolean isUsesQueue() {
		return usesQueue;
	}
	/**
	 * @return the usesArrays
	 */
	public boolean isUsesArrays() {
		return usesArrays;
	}
	/**
	 * @return the maxDimensions
	 */
	public String getMaxDimensions() {
		return maxDimensions;
	}
	/**
	 * @return the usesSubmodels
	 */
	public boolean isUsesSubmodels() {
		return usesSubmodels;
	}
	/**
	 * @return the isWellFormed
	 */
	public boolean isWellFormed() {
		return isWellFormed;
	}

	@Override
	protected void print() {
		System.out.println("Options: ");
		System.out.println("   version: "+version);
		System.out.println("   usesConveyor: "+usesConveyor);
		System.out.println("   usesQueue: "+usesQueue);
		System.out.println("   usesArrays: "+usesArrays);
		System.out.println("   maxDimensions: "+maxDimensions);
		System.out.println("   usesSubmodels: "+usesSubmodels);
		
		System.out.println("   uses_conveyor"+usesConveyor);
		System.out.println("   uses_macros"+usesMacros);
		System.out.println("   uses_event_posters"+usesEventPosters);
		System.out.println("   uses_model_view"+hasModelView);
		System.out.println("   uses_inputs"+usesInputs);
		System.out.println("   uses_outputs"+usesOutputs);
		System.out.println("   uses_annotations"+usesAnnotations);
		
	}

	/**
	 * @return the usesMacros
	 */
	public boolean isUsesMacros() {
		return usesMacros;
	}

	/**
	 * @return the usesEventPosters
	 */
	public boolean isUsesEventPosters() {
		return usesEventPosters;
	}

	/**
	 * @return the hasModelView
	 */
	public boolean isHasModelView() {
		return hasModelView;
	}

	/**
	 * @return the usesInputs
	 */
	public boolean isUsesInputs() {
		return usesInputs;
	}

	/**
	 * @return the usesOutputs
	 */
	public boolean isUsesOutputs() {
		return usesOutputs;
	}

	/**
	 * @return the usesAnnotations
	 */
	public boolean isUsesAnnotations() {
		return usesAnnotations;
	}
}
