package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class SimSpecs extends XmileObject {
	
	// required
	private String start;
	private String stop;
	
	
	// optional
	private String dt = "1";
	private boolean reciprocal = false;
	private String method = "Euler";
	private String timeUnits = "";
	private String pause;
	private String run;
	

	public SimSpecs(Node simSpecNode) {
		if (XMLUtilities.hasAttribute(simSpecNode, "method"))
			method = XMLUtilities.getAttribute(simSpecNode, "method");
		if (XMLUtilities.hasAttribute(simSpecNode, "time_units"))
			timeUnits = XMLUtilities.getAttribute(simSpecNode, "time_units");

		if (!XMLUtilities.hasElementWithTag(simSpecNode, "start")) {
			wellFormed = false;
			error("ERROR: expecting one start tag in sim_spec, found "+
					XMLUtilities.elementCountWithTag(simSpecNode, "start"));
		}

		if (!XMLUtilities.hasElementWithTag(simSpecNode, "stop")) {
			wellFormed = false;
			error("ERROR: expecting one stop tag in sim_spec, found "+
					XMLUtilities.elementCountWithTag(simSpecNode, "stop"));
		}

		start = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(simSpecNode, "start"));
		stop = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(simSpecNode, "stop"));
		if (XMLUtilities.hasElementWithTag(simSpecNode, "dt")) {
			dt = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(simSpecNode, "dt"));
			Element d = XMLUtilities.getElementWithTag(simSpecNode, "dt");
			if (XMLUtilities.hasAttribute(d, "reciprocal")) {
				String recip = XMLUtilities.getAttribute(d, "reciprocal");
				if (recip.equalsIgnoreCase("true"))
					reciprocal = true;
				else
					reciprocal = false;
				
			}
		}
		run = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(simSpecNode, "run"));
		pause = XMLUtilities.getNodeValue(XMLUtilities.getElementWithTag(simSpecNode, "pause"));

	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
	}
	/**
	 * @return the start
	 */
	public String getStart() {
		return start;
	}
	/**
	 * @return the stop
	 */
	public String getStop() {
		return stop;
	}
	/**
	 * @return the dt
	 */
	public String getDt() {
		return dt;
	}
	/**
	 * @return the reciprocal
	 */
	public boolean isReciprocal() {
		return reciprocal;
	}
	/**
	 * @return the method
	 */
	public String getMethod() {
		return method;
	}
	/**
	 * @return the timeUnits
	 */
	public String getTimeUnits() {
		return timeUnits;
	}
	/**
	 * @return the pause
	 */
	public String getPause() {
		return pause;
	}
	/**
	 * @return the run
	 */
	public String getRun() {
		return run;
	}
	/**
	 * @return the wellFormed
	 */
	public boolean isWellFormed() {
		return wellFormed;
	}
	@Override
	protected void print() {
		System.out.println("SimSpecs:");
		System.out.println("   start:"+start);
		System.out.println("   stop:"+stop);
		
		
		// optional
		System.out.println("   dt:"+dt);
		System.out.println("   reciprocal:"+reciprocal);
		System.out.println("   method:"+method);
		System.out.println("   timeUnits:"+timeUnits);
		System.out.println("   pause:"+pause);
		System.out.println("   run:"+run);
		
	}
	
	public List<String> asVensim() {
		List<String> equations = new ArrayList<String>();
		
		equations.add("INITIAL TIME = "+start+"\t~\t"+timeUnits+"\t~\tThe initial time for the simulation\t|");
		equations.add("FINAL TIME = "+stop+"\t~\t"+timeUnits+"\t~\tThe final time for the simulation\t|");
		if (!reciprocal) {
			equations.add("TIME STEP = "+dt+"\t~\t"+timeUnits+"\t~\tThe time step for the simulation\t|");
			equations.add("SAVEPER = "+dt+"\t~\t"+timeUnits+"\t~\tThe saveper for the simulation\t|");
		} else { 
			equations.add("TIME STEP = "+((double) 1 / Integer.parseInt(dt))+"\t~\t"+timeUnits+"\t~\tThe time step for the simulation\t|");
			equations.add("SAVEPER = "+((double) 1 / Integer.parseInt(dt))+"\t~\t"+timeUnits+"\t~\tThe saveper for the simulation\t|");
		}
		
		
		
		return equations;
	}

}
