package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Stock extends BuildingBlock {
	
	protected List<Inflow> inflow = new ArrayList<Inflow>();
	protected List<Outflow> outflow = new ArrayList<Outflow>();
	
	public Stock(String modelName, Node stockNode) {
		super(modelName, stockNode);
		type = BuildingBlock.STOCK;
		boolean hasIn = false;
		boolean hasOut = false;
		
		
//		stocks have elements in which inflow, outflow, and eqn are defined
		
		if (XMLUtilities.hasElementsWithTag(stockNode, "inflow")) {
			hasIn = true;
			for (Element element : XMLUtilities.getElementsWithTag(stockNode, "inflow")) {
				Inflow r = new Inflow(modelName, element);
				if (r.isWellFormed()) {
					inflow.add(r);
				} else {
					error("infow");
				}
			}
		} 
		
		if (XMLUtilities.hasElementsWithTag(stockNode, "outflow")) {
			hasOut = true;
			for (Element element : XMLUtilities.getElementsWithTag(stockNode, "outflow")) {
				Outflow r = new Outflow(modelName, element);
				if (r.isWellFormed()) {
					outflow.add(r);
				} else { 
					error("outfow");
				}
			}
		} 
		
		if (!(hasIn || hasOut))
			error("Missing flows");
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		for (Inflow inf : inflow)
			inf.disperse(xmileModel);
		for (Outflow outf : outflow)
			outf.disperse(xmileModel);
	}

	@Override
	public String getEquation() {
		StringBuilder sb = new StringBuilder(getComment());
		sb.append(name.getNormalizedNameQuoted()+getLhsArrayReference());
		sb.append(" = INTEG(");
		int inCount = inflow.size();
		int outCount = outflow.size();
		int in = 0;
		int out = 0;
		if (inCount > 1)
			sb.append("(");
		for (Inflow i : inflow) {
			if (in++ > 0)
				sb.append("+");
//			sb.append(i.getVariableName());
			sb.append(i.getNormalizedNameQuoted());
		}
		
		if (inCount > 1)
			sb.append(")");
		
		if (outCount > 0)
			sb.append("-");
		
		if (outCount > 1)
			sb.append("(");
		
		for (Outflow o : outflow) {
			if (out++ > 0)
				sb.append("+");
//			sb.append(o.getVariableName());
			sb.append(o.getNormalizedNameQuoted());
		}
		
		if (outCount > 1)
			sb.append(")");
		
		sb.append(",");
		if (eqns!= null && eqns.size() > 0) {
			sb.append(eqns.get(0).getEquation());
		} else {
			if (access != null) {
//				sb.append("SOME SHADOW VARIABLE");
				return "";
			} else {
				error("Missing eqn for generation");
			}
		}
		sb.append(")");
		
		return sb.toString();
	}
	
	protected void error(String s) {
		super.error("Stock: "+s);
	}
	
	public boolean hasInflowFrom(String flow) {
		for (Inflow in : inflow) {
			if (in.getVariableName().equals(flow))
				return true;
		}
		return false;
	}
	
	public boolean hasOutflowTo(String flow) {
		for (Outflow in : outflow) {
			if (in.getVariableName().equals(flow))
				return true;
		}
		return false;
	}

	@Override
	protected void print() {
		// TODO Auto-generated method stub
		
	}

}
