package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Unit extends XmileObject {

	private Name name;
	private String eqn;
	private Set<String> aliases = new LinkedHashSet<String>();
	
	private String units;

	public Unit(Node unitsNode) {
		
		units = XMLUtilities.getNodeValue(unitsNode).replace("-", "_");
		
		if (XMLUtilities.hasAttribute(unitsNode, "name")) {
			name = new Name(XMLUtilities.getAttribute(unitsNode, "name"));
		} else {
			wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(unitsNode, "eqn")) {
			eqn = XMLUtilities.getElementWithTag(unitsNode, "eqn").getNodeValue();
		}
		
		if (XMLUtilities.hasElementsWithTag(unitsNode, "alias")) {
			for (Element element : XMLUtilities.getElementsWithTag(unitsNode, "alias")) {
				aliases.add(XMLUtilities.getNodeValue(element));
			}
		}
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (name != null)
			name.disperse(xmileModel);
	}
	
	@Override
	protected List<String> asVensim() {
		List<String> vensim = new ArrayList<String>();
		if (aliases.size() > 0) {
			StringBuilder sb = new StringBuilder();
			sb.append("22:"+name.getAsAppears());
			for (String s : aliases) {
				sb.append(","+s);
			}
			vensim.add(sb.toString());
		}

		return vensim;
	}

	@Override
	protected void print() {
		System.out.println("Units:");
		if (name != null)
			name.print();
		else
			System.out.println("Name is null");
		
		if (eqn != null)
			System.out.println("Eqn: "+eqn);
		else
			System.out.println("Eqn is null");
		
		
		for (String alias : aliases)
			System.out.println("   Alias: "+alias);

	}

	/**
	 * @return the name
	 */
	public String getName() {
		return name.getAsAppears();
	}

	/**
	 * @return the eqn
	 */
	public String getEqn() {
		return eqn;
	}

	/**
	 * @return the aliases
	 */
	public Set<String> getAliases() {
		return aliases;
	}

	/**
	 * @return the units
	 */
	public String getUnits() {
		return units;
	}

}
