package gov.anl.gss.sdm.server.model.xmile;

import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class Variables extends XmileObject {
	
	private Set<Module> modules = new LinkedHashSet<Module>();
	private Set<BuildingBlock> equations = new LinkedHashSet<BuildingBlock>();
//	private Set<NonApplyToAllArray> multiEquations = new LinkedHashSet<NonApplyToAllArray>();
	
public Variables(String modelName, Node variablesNode, Map<String, BuildingBlock> variableInfo) {
		
		
		
		for (Element element : XMLUtilities.getElementsWithTag(variablesNode, "module")) {
			Module module = new Module(element);
			if (module.isWellFormed()) {
				modules.add(module);
			}
		}
		for (Element element : XMLUtilities.getElementsWithTag(variablesNode, "stock")) {
			Stock stock = new Stock(modelName, element);
			if (stock.isWellFormed()) {
				equations.add(stock);
				variableInfo.put(asInfoKey(stock.getNormalizedName()), stock);
			}
			
		}
		for (Element element : XMLUtilities.getElementsWithTag(variablesNode, "flow")) {
			Flow flow = new Flow(modelName, element);
			if (flow.isWellFormed()) {
				equations.add(flow);
				variableInfo.put(asInfoKey(flow.getNormalizedName()), flow);
			}
			
		}
		for (Element element : XMLUtilities.getElementsWithTag(variablesNode, "aux")) {
			Auxiliary auxiliary = new Auxiliary(modelName, element);
			if (auxiliary.isWellFormed()) {
				equations.add(auxiliary);
				variableInfo.put(asInfoKey(auxiliary.getNormalizedName()), auxiliary);
			}
			
		}
		
//		for (Element element : XMLUtilities.getElementsWithTag(variablesNode, "array")) {
//			NonApplyToAllArray nata = new NonApplyToAllArray(element);
//			if (nata.isWellFormed())
//				multiEquations.add(nata);
//			
//		}
}

private String asInfoKey(String n) {
	return InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(n);
}


public void disperse(XmileModel xmileModel) {
	setXmileModel(xmileModel);
	for (Module x : modules)
		x.disperse(xmileModel);
	for (BuildingBlock x : equations)
		x.disperse(xmileModel);
//	for (NonApplyToAllArray x : multiEquations)
//		x.disperse(xmileModel);
}

	@Override
	protected void print() {
		
		
		for (Module module : modules)
			module.print();
		
		for (BuildingBlock bb : equations) {
			print(bb);
		}
		
//		for (NonApplyToAllArray nata : multiEquations) {
//			for (String bb : nata.getMultiEquations())
//				System.out.println(bb);
//		}
		
	}
	
	protected void print(BuildingBlock bb) {
		if (bb.type.equals("stock"))
			((Stock) bb).print();
		if (bb.type.equals("aux"))
			((Auxiliary) bb).print();
		if (bb.type.equals("flow"))
			((Flow) bb).print();
	}

	/**
	 * @return the modules
	 */
	public Set<Module> getModules() {
		return modules;
	}

	/**
	 * @return the equations
	 */
	public Set<BuildingBlock> getEquations() {
		return equations;
	}
	
	public boolean hasModules() {
		return modules.size() > 0;
	}
	
	public Map<String, String> getModuleConectors() {
		Map<String, String> connectors = new LinkedHashMap<String, String>();
		
//		Need to worry about subscripts for these. Need variableInfo data structure
		
		for (Module m : modules) {
			for (Connect e : m.getConections())
				if (!connectors.containsKey(e.getTo().getNormalizedName()))
					connectors.put(e.getTo().getNormalizedName(), e.getFrom().getNormalizedName());
		}
		
		return connectors;
	}
	
	public List<String> getModuleEquations() {
		List<String> al = new ArrayList<String>();
		
//		Need to worry about subscripts for these. Need variableInfo data structure
		
		for (Module m : modules) {
			for (String e : m.getEquations())
				if (!al.contains(e))
					al.add(e);
		}
		
		return al;
	}

//	/**
//	 * @return the multiEquations
//	 */
//	public Set<NonApplyToAllArray> getMultiEquations() {
//		return multiEquations;
//	}
	
	public Set<String> getStocksWithOutflowTo(String flow) {
		Set<String> stocks = new LinkedHashSet<String>();
		
		for (BuildingBlock bb : equations) {
			if (bb.type.equals("stock")) {
				Stock stock = (Stock) bb;
				if (stock.hasOutflowTo(flow))
					stocks.add(stock.getName());
			}
		}
		
		return stocks;
	}
	
	public Set<String> getStocksWithInflowFrom(String flow) {
		Set<String> stocks = new LinkedHashSet<String>();
		
		for (BuildingBlock bb : equations) {
			if (bb.type.equals("stock")) {
				Stock stock = (Stock) bb;
				if (stock.hasInflowFrom(flow))
					stocks.add(stock.getName());
			}
		}
		
		return stocks;
	}

}
