package gov.anl.gss.sdm.server.model.xmile;

import gov.anl.gss.sdm.server.model.translator.InformationManagers;

import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class View extends XmileObject {
	
	protected Style style;
	protected Set<Connector> connectors = new LinkedHashSet<Connector>();
	protected Set<ViewBuildingBlock> buildingBlocks = new LinkedHashSet<ViewBuildingBlock>();
	protected Set<Alias> aliases = new LinkedHashSet<Alias>();
	
	public View(String modelName, Node viewNode, Map<String, ViewBuildingBlock> viewVariableInfo) {

		if (XMLUtilities.hasElementWithTag(viewNode, "style")) {
			Element element = XMLUtilities.getElementWithTag(viewNode, "style");
			Style s = new Style(element);
			if (s.isWellFormed())
				style = s;
		}
		
		if (XMLUtilities.hasElementsWithTag(viewNode, "connector")) {
			for (Element element : XMLUtilities.getElementsWithTag(viewNode, "connector")) {
				Connector c = new Connector(modelName, element);
				if (c.isWellFormed())
					connectors.add(c);
			}
		}
		
		if (XMLUtilities.hasElementsWithTag(viewNode, "aux")) {
			for (Element element : XMLUtilities.getElementsWithTag(viewNode, "aux")) {
				ViewAuxiliary bb = new ViewAuxiliary(modelName, element);
				
				if (bb.isWellFormed()) {
					buildingBlocks.add(bb);
					viewVariableInfo.put(asInfoKey(bb.getNormalizedName()), bb);
				}
			}
		}
		
		if (XMLUtilities.hasElementsWithTag(viewNode, "stock")) {
			for (Element element : XMLUtilities.getElementsWithTag(viewNode, "stock")) {
				ViewStock bb = new ViewStock(modelName, element);
				if (bb.isWellFormed()) {
					buildingBlocks.add(bb);
					viewVariableInfo.put(asInfoKey(bb.getNormalizedName()), bb);
				}
			}
		}
		
		if (XMLUtilities.hasElementsWithTag(viewNode, "flow")) {
			for (Element element : XMLUtilities.getElementsWithTag(viewNode, "flow")) {
				ViewFlow bb = new ViewFlow(modelName, element);
				if (bb.isWellFormed()) {
					buildingBlocks.add(bb);
					viewVariableInfo.put(asInfoKey(bb.getNormalizedName()), bb);
				}
			}
		}

		if (XMLUtilities.hasElementsWithTag(viewNode, "alias")) {
			for (Element element : XMLUtilities.getElementsWithTag(viewNode, "alias")) {
				Alias a = new Alias(element);
				if (a.isWellFormed())
					aliases.add(a);
			}
		}
	}
	
	private String asInfoKey(String n) {
		return InformationManagers.getInstance().getVariableNamePatternGenerator().getPatternFor(n);
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		
		if (style != null)
			style.disperse(xmileModel);
		
		for (Connector x : connectors)
			x.disperse(xmileModel);
		for (ViewBuildingBlock x : buildingBlocks)
			x.disperse(xmileModel);
		for (Alias x : aliases)
			x.disperse(xmileModel);
	}

	@Override
	protected void print() {

		
		System.out.println("View");
		if (style != null)
			style.print();
		
		for (Connector c : connectors)
			c.print();
		
		for (ViewBuildingBlock vbb : buildingBlocks)
			vbb.print();
		
		for (Alias a : aliases)
			a.print();

	}

	/**
	 * @return the style
	 */
	public Style getStyle() {
		return style;
	}

	/**
	 * @return the connectors
	 */
	public Set<Connector> getConnectors() {
		return connectors;
	}

	/**
	 * @return the buildingBlocks
	 */
	public Set<ViewBuildingBlock> getBuildingBlocks() {
		return buildingBlocks;
	}

	/**
	 * @return the aliases
	 */
	public Set<Alias> getAliases() {
		return aliases;
	}
	
	public boolean isHasGraphics() {
		return connectors.size() > 0 && buildingBlocks.size() > 0;
	}

}
