package gov.anl.gss.sdm.server.model.xmile;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;



public class XmileElement extends XmileObject {
	
	private String subscript;
	private Eqn equation;
	protected GraphicalFunction graphicalFunction;
	
	public XmileElement(String modelName, Node elementNode) {
		
		subscript = XMLUtilities.getAttribute(elementNode, "subscript");
		
		boolean hasGF = false;
		boolean hasEQN = false;
		
		if (XMLUtilities.hasElementWithTag(elementNode, "gf")) {
			hasGF = true;
			Element element = XMLUtilities.getElementWithTag(elementNode, "gf");
				GraphicalFunction r = new GraphicalFunction(element);
				if (r.isWellFormed())
					graphicalFunction = r;
				else
					error("grahical function");
			
		}
		
		if (XMLUtilities.hasElementsWithTag(elementNode, "eqn")) {
			hasEQN = true;
			for (Element element : XMLUtilities.getElementsWithTag(elementNode, "eqn")) {
				Eqn r = new Eqn(modelName, element);
				if (r.isWellFormed())
					equation = r;
				else
					error("bad Eqn");
			}
		} 
		
		if (!hasGF && !hasEQN)
			error("neither GF or EQN provided");
		
		if (hasGF && hasEQN)
			error("both GF or EQN provided");
	}

	@Override
	protected void print() {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
		if (equation != null)
			equation.disperse(xmileModel);
		
	}

	/**
	 * @return the subscript
	 */
	public String getSubscript() {
		return subscript;
	}
	
	public List<String> getSubscriptAsList() {
		List<String> al = new ArrayList<String>();
		for (String s : subscript.split(",")) {
			al.add(s.trim());
		}
		return al;
	}

	/**
	 * @return the equation
	 */
	public String getEquation() {
		try {
			return equation.getEquation();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return null;
		}
	}
	
	public Eqn getEquationAsEqn() {
		return equation;
	}
	
	public String getMultiEquationRHS() {
		return "["+subscript+"] = "+equation.getEquation();
	}

	/**
	 * @return the graphicalFunction
	 */
	public GraphicalFunction getGraphicalFunction() {
		return graphicalFunction;
	}
	
	public String getMultiGfRHS(Name name, Eqn eqn) {
//		return "["+subscript+"]"+eqn.getEquation()+", "+graphicalFunction.getLookupDefinitionEquation(name)/*+")"*/;
		return "["+subscript+"] = WITH LOOKUP("+eqn.getEquation()+", "+graphicalFunction.getLookupDefinitionEquation(name)+")\n"/*+")"*/;
	}

	
//	getLookupDefinitionEquation

}
