package gov.anl.gss.sdm.server.model.xmile;

import java.util.List;



public abstract class XmileObject {
	
	
	protected boolean wellFormed = true;
	protected XmileModel xmileModel;
	
	protected Name name;
	
	
	protected String x;
	protected String y;
	
	protected String width = "50";
	protected String height = "10";
	
	
	protected void message(String msg) {
		System.out.println("MSG: "+msg);
		XMLUtilities.addMessage(msg);
	}
	
	abstract protected void print();
	
	protected List<String> asVensim() {
		return null;
	}
	
	protected void error(String msg) {
		System.out.println(msg);
		XMLUtilities.addErrorMessage("ERR: "+msg);
		wellFormed = false;
	}

	/**
	 * @return the wellFormed
	 */
	public boolean isWellFormed() {
		return wellFormed;
	}
	
	protected String format80(String s) {
		StringBuilder sb = new StringBuilder();
		MutableInteger start = new MutableInteger();
		
		int len = s.length();
		while (start.value() < len) {
			MutableInteger end = new MutableInteger(Math.min(start.value()+75, len-1));
			transfer(sb, s, start, end);
			start.setValue(end.value()+1);
			
		}
		
		return sb.toString();
	}
	
	protected void transfer(StringBuilder sb, String s, MutableInteger start, MutableInteger end) {
		sb.append("\t");
		while (!characterAt(end, s).equals(" ") && end.value() < s.length()-1)
			end.add(1);
		MutableInteger pos = new MutableInteger(start);
		while(pos.value() <= end.value()) {
			sb.append(characterAt(pos, s));
			pos.add(1);
		}
		sb.append("\n");
		
	}
	
	public static String characterAt(MutableInteger position, String string) {
		return string.substring(position.value(), position.value()+1);
	}

	/**
	 * @return the xmileModel
	 */
	public XmileModel getXmileModel() {
		return xmileModel;
	}

	/**
	 * @param xmileModel the xmileModel to set
	 */
	public void setXmileModel(XmileModel xmileModel) {
		this.xmileModel = xmileModel;
	}
	
	abstract public void disperse(XmileModel xmileModel);

	public String getX() {
		return x;
	}

	public String getY() {
		return y;
	}
	
	/**
	 * @return the name
	 */
	public String getName() {
		if (name != null)
			return name.getAsAppears();
		else
			return "NONAME";
	}
	
	/**
	 * @return the name
	 */
	public String getNormalizedName() {
		if (name != null)
			return name.getNormalizedName();
		else
			return "NONAME";
	}

	public String getNormalizedNameQuoted() {
		if (name != null)
			return name.getNormalizedNameQuoted();
		else
			return "NONAME";
	}


	/**
	 * @return the name
	 */
	public Name getNameAsName() {
		return name;
	}

	public String getWidth() {
		return width;
	}

	public void setWidth(String width) {
		this.width = width;
	}

	public String getHeight() {
		return height;
	}

	public void setHeight(String height) {
		this.height = height;
	}

}
