package gov.anl.gss.sdm.server.model.xmile;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.LinkedHashSet;
import java.util.Set;

public class XmileUtilities {
	
	private static Set<String> noArgumentFunctions = new LinkedHashSet<String>();
	
	static {
		noArgumentFunctions.add("PI");
		noArgumentFunctions.add("DT");
		noArgumentFunctions.add("STARTTIME");
		noArgumentFunctions.add("STOPTIME");
		noArgumentFunctions.add("SELF");
		noArgumentFunctions.add("TIME");
		
	}
	
	public static boolean isNonIdentifier(String s) {
		if (Parser.isOperator(s))
			return true;
		if (Parser.isLeftParen(s) || Parser.isRightParen(s))
			return true;
		if (Parser.isLeftSquareBracket(s) || Parser.isRightSquareBracket(s))
			return true;
		if (Parser.isNumber(s))
			return true;
		if (Parser.isFunctionArgumentSeparator(s))
			return true;
		return false;
	}
	
	public static boolean isIdentifier(String s) {
		if (Parser.isQuotedString(s))
			return true;
		String startChar = Parser.characterAt(0, s);
		if (startChar.equals("_") || startChar.equals("$") || Parser.isNumber(startChar))
			return false;
		if (Parser.characterAt(s.length()-1, s).equals("_"))
			return false;
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			boolean allowable = Character.isLetterOrDigit(c) || c == ' ' || c == '_' || c == '$' || c > 127;
			if (!allowable)
				return false;
		}
		return true;
		
	}
	
	public static boolean isNoArgumentFunction(String s) {
		return noArgumentFunctions.contains(s);
	}

}
