package gov.anl.gss.sdm.server.model.xmile.options;

import gov.anl.gss.sdm.server.model.xmile.XMLUtilities;
import gov.anl.gss.sdm.server.model.xmile.XmileModel;
import gov.anl.gss.sdm.server.model.xmile.XmileObject;

import org.w3c.dom.Node;

public class Options extends XmileObject {

	
	private UsesConveyor usesConveyor;
	private UsesQueue usesQueue;
	private UsesArrays usesArrays;
	private UsesSubmodels usesSubmodels;
	private UsesMacros usesMacros;
	private UsesEventPosters usesEventPosters;
	private HasModelView hasModelView;
	private UsesOutputs usesOutputs;
	private UsesInputs usesInputs;
	private UsesAnnotations usesAnnotations;
	
	private String namespace;
	
	private boolean isWellFormed = true;
	
	public Options(Node optionsNode) {
		namespace = XMLUtilities.getAttribute(optionsNode, "namespace");
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_conveyor")) {
			UsesConveyor uc = new UsesConveyor(XMLUtilities.getElementWithTag(optionsNode, "uses_conveyor"));
			if (uc.isWellFormed())
				usesConveyor = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_queue")) {
			UsesQueue uc = new UsesQueue(XMLUtilities.getElementWithTag(optionsNode, "uses_queue"));
			if (uc.isWellFormed())
				usesQueue = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_submodels")) {
			UsesSubmodels uc = new UsesSubmodels(XMLUtilities.getElementWithTag(optionsNode, "uses_submodels"));
			if (uc.isWellFormed())
				usesSubmodels = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_macros")) {
			UsesMacros uc = new UsesMacros(XMLUtilities.getElementWithTag(optionsNode, "uses_macros"));
			if (uc.isWellFormed())
				usesMacros = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "has_model_view")) {
			HasModelView uc = new HasModelView(XMLUtilities.getElementWithTag(optionsNode, "has_model_view"));
			if (uc.isWellFormed())
				hasModelView = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_outputs")) {
			UsesOutputs uc = new UsesOutputs(XMLUtilities.getElementWithTag(optionsNode, "uses_outputs"));
			if (uc.isWellFormed())
				usesOutputs = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_inputsr")) {
			UsesInputs uc = new UsesInputs(XMLUtilities.getElementWithTag(optionsNode, "uses_inputsr"));
			if (uc.isWellFormed())
				usesInputs = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_annotations")) {
			UsesAnnotations uc = new UsesAnnotations(XMLUtilities.getElementWithTag(optionsNode, "uses_annotations"));
			if (uc.isWellFormed())
				usesAnnotations = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_arrays")) {
			UsesArrays uc = new UsesArrays(XMLUtilities.getElementWithTag(optionsNode, "uses_arrays"));
			if (uc.isWellFormed())
				usesArrays = uc;
			else
				wellFormed = false;
		}
		
		if (XMLUtilities.hasElementWithTag(optionsNode, "uses_event_posters")) {
			UsesEventPosters uc = new UsesEventPosters(XMLUtilities.getElementWithTag(optionsNode, "uses_event_posters"));
			if (uc.isWellFormed())
				usesEventPosters = uc;
			else
				wellFormed = false;
		}
	}
	
	public void disperse(XmileModel xmileModel) {
		setXmileModel(xmileModel);
	}

	@Override
	protected void print() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * @return the usesConveyor
	 */
	public UsesConveyor getUsesConveyor() {
		return usesConveyor;
	}

	/**
	 * @return the usesQueue
	 */
	public UsesQueue getUsesQueue() {
		return usesQueue;
	}

	/**
	 * @return the usesArrays
	 */
	public UsesArrays getUsesArrays() {
		return usesArrays;
	}

	/**
	 * @return the usesSubmodels
	 */
	public UsesSubmodels getUsesSubmodels() {
		return usesSubmodels;
	}

	/**
	 * @return the usesMacros
	 */
	public UsesMacros getUsesMacros() {
		return usesMacros;
	}

	/**
	 * @return the usesEventPosters
	 */
	public UsesEventPosters getUsesEventPosters() {
		return usesEventPosters;
	}

	/**
	 * @return the hasModelView
	 */
	public HasModelView getHasModelView() {
		return hasModelView;
	}

	/**
	 * @return the usesOutputs
	 */
	public UsesOutputs getUsesOutputs() {
		return usesOutputs;
	}

	/**
	 * @return the usesInputs
	 */
	public UsesInputs getUsesInputs() {
		return usesInputs;
	}

	/**
	 * @return the usesAnnotations
	 */
	public UsesAnnotations getUsesAnnotations() {
		return usesAnnotations;
	}

	/**
	 * @return the isWellFormed
	 */
	public boolean isWellFormed() {
		return isWellFormed;
	}

	/**
	 * @return the namespace
	 */
	public String getNamespace() {
		return namespace;
	}

}
