package gov.anl.gss.sdm.server.model.xmile.utilities;

import gov.anl.gss.sdm.server.model.translator.Parser;

import java.util.ArrayList;
import java.util.List;
import java.util.Stack;

public class IfThenElseConverter {
	
	public static final String IF = "IF";
	public static final String THEN = "THEN";
	public static final String ELSE = "ELSE";
	
	boolean justCopy = false;
	
	Stack<ITEState> state = new Stack<ITEState>();
	Stack<ITEState> ifStack = new Stack<ITEState>();
	
	int iteLevel = 0;
	
	String originalEquation;
	List<String> decomposed = new ArrayList<String>();
	List<String> converted;
	String convertedEquation;
	
	int currentPosition = 0;
	int openParens = 0;
	
	public IfThenElseConverter(String equation) {
		
		originalEquation = equation;
		List<String> d = Parser.decompose(equation, true, true);
//		System.out.println("ITEC: "+d);
		for (String s : Parser.decompose(equation, true, true)) {
			decomposed.add(s.trim()); 
		}
		converted = new ArrayList<String>();
//		convert();
		convert2(0);
	}
	
	private void convert2(int level) {
		
		positionToIF();
		while (currentPosition <= decomposed.size()-1) {
			String chars = decomposed.get(currentPosition);
			if (!justCopy && chars.trim().toUpperCase().equals(IF)) {
				processIF2();
			} else if (!justCopy && chars.trim().toUpperCase().equals(THEN)) {
				processTHEN2();
			} else if (!justCopy && chars.trim().toUpperCase().equals(ELSE)) {
				processELSE2();
			} else {
				processOther2();
			}
		}
		
		StringBuilder sb = new StringBuilder();
		for (String s : converted) {
			sb.append(s);
		}
		
		convertedEquation = sb.toString();
	}
	
	private void convert() {
		// first position to the start of the IF expression
		positionToIF();
		
		// we will translate IF -> IF THEN ELSE (
		// we will translate THEN -> ,
		// we will translate ELSE -> ,
		// key operation is adding a parent after the ELSE expression
		
		while (currentPosition <= decomposed.size()-1) {
			String chars = decomposed.get(currentPosition);
			if (chars.trim().toUpperCase().equals(IF)) {
				processIF();
			} else if (chars.trim().toUpperCase().equals(THEN)) {
				processTHEN();
			} else if (chars.trim().toUpperCase().equals(ELSE)) {
				processELSE();
			} else {
				processOther();
			}
		}
		
		StringBuilder sb = new StringBuilder();
		for (String s : converted) {
			sb.append(s);
		}
		
		convertedEquation = sb.toString();
		
	}
	
	private void processIF() {
		state.push(new ITEState(IF, ++iteLevel, openParens));
		converted.add("IF THEN ELSE");
		converted.add("(");
		currentPosition++;
	}
	
	private void processTHEN() {
		state.push(new ITEState(THEN, iteLevel, openParens));
		converted.add(",");
		currentPosition++;
	}
	
	private void processELSE() {
		state.push(new ITEState(ELSE, iteLevel, openParens));
		converted.add(",");
		currentPosition++;
	}
	
	private void processOther() {
		String chars = decomposed.get(currentPosition);
		if (chars.trim().equals("~")) {
			justCopy = true;
			//			System.out.println("JUSTCOPY");
		}
		if (chars.equals("(")) {
			openParens++;
		} else if (chars.equals(")")) {
			openParens--;
		}



		if (state.isEmpty()) {
			converted.add(chars);
		} else {
			String currentState = state.peek().getState();
			if (currentState.equals(IF) && (chars.trim().toUpperCase().equals("AND") || chars.trim().toUpperCase().equals("OR") || chars.trim().toUpperCase().equals("NOT"))) {
					converted.add(":"+chars.trim()+":");
			} else {
				if (justCopy) {
				// need to clear stack
					while (!state.isEmpty()) {
						converted.add(")");
						state.pop();  // else
						state.pop();  // then
						state.pop();  // if
						iteLevel--;
					}
					
					converted.add(chars);
				} else {
				converted.add(chars);
				if (currentState.equals(ELSE) && isEndOfExpression()) {
					converted.add(")");
					state.pop();  // else
					state.pop();  // then
					state.pop();  // if
					iteLevel--;
				} 
				}
			}
		}
		currentPosition++;
	}
	
	private void processIF2() {
		state.push(new ITEState(IF, ++iteLevel, openParens));
		ifStack.push(new ITEState(IF, iteLevel, openParens));
		converted.add("IF THEN ELSE");
		converted.add("(");
		currentPosition++;
	}
	
	private void processTHEN2() {
		// then Then is always
		state.push(new ITEState(THEN, iteLevel, openParens));
		converted.add(",");
		currentPosition++;
	}
	
	private void processELSE2() {
		state.push(new ITEState(ELSE, iteLevel, openParens));
		if (nextKeywordIs(ELSE)) {
			iteLevel--;
		}
		converted.add(",");
		currentPosition++;
	}
	
	private boolean hasNextKeyword() {
		int i = currentPosition+1;
		while (i < decomposed.size()-1) {
			String next = decomposed.get(i).trim();
			if (isKeyword(next)) {
				return true;
			}
			
			i++;
		}
		return false;
	}
	
	private boolean nextKeywordIs(String s) {
		int i = currentPosition+1;
		while (i < decomposed.size()-1) {
			String next = decomposed.get(i).trim();
			if (isKeyword(next)) {
				return next.toUpperCase().equals(s);
			}
			
			i++;
		}
		return false;
	}
	
	private boolean isKeyword(String s) {
		return s.toUpperCase().equals(IF) 
			|| s.toUpperCase().equals(THEN)
			|| s.toUpperCase().equals(ELSE);
	}
	
	private void processOther2() {
		String chars = decomposed.get(currentPosition);
		if (chars.trim().equals("~")) {
			justCopy = true;
			//			System.out.println("JUSTCOPY");
		}
		if (chars.trim().equals("(")) {
			openParens++;
		} else if (chars.trim().equals(")")) {
			openParens--;
		}



		if (state.isEmpty()) {
			converted.add(chars);
		} else {
			String currentState = state.peek().getState();
			if (currentState.equals(IF) && (chars.trim().toUpperCase().equals("AND") || chars.trim().toUpperCase().equals("OR") || chars.trim().toUpperCase().equals("NOT"))) {
					converted.add(":"+chars.trim()+":");
			} else {
				if (justCopy) {
				// need to clear stack
					while (!state.isEmpty()) {
						try {
							converted.add(")");
							state.pop();  // else
							state.pop();  // then
							state.pop();  // if
							ifStack.pop();
							iteLevel--;
						} catch (Exception e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					}
					
					converted.add(chars);
				} else {
				converted.add(chars);
				if (currentState.equals(ELSE) && isEndOfExpression()) {
//					converted.add(")");
					state.pop();  // else
					state.pop();  // then
					state.pop();  // if
					ifStack.pop();
					
					while (iteLevel-- > 0) {
						converted.add(")");
					}
				} 
				}
			}
		}
		currentPosition++;
	}
	private boolean isEndOfExpression() {
		// if the current position is end of an expression
		int next = currentPosition+1;
		if (next >= decomposed.size()) {
			return true;
		}
		
		// if pointing to start of comment field yes
		String nextToken = decomposed.get(next);
		if (nextToken.trim().equals("~"))
			return true;

		// if this closes the if
		if (decomposed.get(currentPosition).trim().equals(")") &&
				getCurrentIfParenLevel() == openParens+1)
			return true;
		
		return hasNextKeyword() && nextKeywordIs(ELSE);
		
	}
	
	private int getCurrentIfParenLevel() {
		int pl = 0;
		if (!ifStack.empty()) {
			pl = ifStack.peek().getParenLevel();
		}
		
		return pl;
	}
	
	private void positionToIF() {
		while (currentPosition <= decomposed.size()-1) {
			String chars = decomposed.get(currentPosition);
			if (chars.trim().equals("("))
				openParens++;
			if (chars.trim().equals(")"))
				openParens--;
			if (chars.trim().toUpperCase().equals(IF)) {
				break;
			} else {
				converted.add(chars);
				currentPosition++;
			}
		}
	}

	public String getOriginalEquation() {
		return originalEquation;
	}

	public List<String> getDecomposed() {
		return decomposed;
	}

	public List<String> getConverted() {
		return converted;
	}

	public String getConvertedEquation() {
		return convertedEquation;
	}

}
