package gov.anl.gss.sdm.shared;

import java.io.Serializable;

@SuppressWarnings("serial")
public class TimeTracker implements Serializable{
	double[] times;
	int subSteps;
	int minTime;
	int maxTime;
	int runTime;
	int stopTime;
	int timeOffset;
	String units;
	public TimeTracker(){
		subSteps=0;
	}
	public int getNumTimes(){
		return times.length;
	}
	public double steps(){
		return subSteps+1;
	}
	public double getValueForIndex(int timeIndex){
		return times[timeIndex];
	}
	public double getValueForTime(double time){
		return getValueForIndex(getTimeIndexFor(time));
	}
	public int getTimeIndexFor(double time){
		for(int i=1;i<times.length;i++){
			if(times[i]>time)return i-1;
		}
		return 0;
	}
	public double getTimeForIndex(int index){
		return index/(steps());
	}
	public int getWholeTimeFor(double time){
		return (int)time;
	}
	public int getWholeTimeIndexFor(double time){
		// get the index of the whole step time before this time
		int wholeTime = getWholeTimeFor(time);
		// truncate it
		return getIndexFor(wholeTime);
	}
	public int getIndexFor(double time){
		return (int)((time)*(steps()));
	}
	public double[] getTimes() {
		return times;
	}
	public void initializeTimes(){
		int timeNum = (maxTime-minTime)*(subSteps+1)+1;
		times = new double[timeNum];
		for(int i=minTime;i<timeNum;i++){
			times[i]=getTimeForIndex(i);
		}
	}
	public void setTimes(double[] times) {
		this.times = times;
	}
	public int getSubSteps() {
		return subSteps;
	}
	public void setSubSteps(int subSteps) {
		this.subSteps = subSteps;
	}
	public int getMinTime() {
		return minTime;
	}
	public void setMinTime(int minTime) {
		this.minTime = minTime;
	}
	public int getMaxTime() {
		return maxTime;
	}
	public void setMaxTime(int maxTime) {
		this.maxTime = maxTime;
	}
	public int getRunTime() {
		return runTime;
	}
	public void setRunTime(int runTime) {
		this.runTime = runTime;
	}
	public int getStopTime() {
		return stopTime;
	}
	public void setStopTime(int stopTime) {
		this.stopTime = stopTime;
	}
	public int getTimeOffset() {
		return timeOffset;
	}
	public void setTimeOffset(int timeOffset) {
		this.timeOffset = timeOffset;
	}
	public String getUnits() {
		return units;
	}
	public void setUnits(String units) {
		this.units = units;
	}
	public String getDisplayTime(int timeStep) {
		
		return Integer.toString((int)getAdjustedTime(timeStep));
	}
	public double getAdjustedTime(double time){
		return time+timeOffset;
	}
	public int getSkipIndex(double num){
		int skipTime = (int)(num/steps());
		return skipTime;
	}
	
}