package gov.anl.gss.sdm.shared;

import java.io.Serializable;
import java.util.List;

/**
* @author Mary Ann Bitz
* Argonne National Laboratory
* Global Security Sciences Division 
*/
/**
 * Class to hold one of the variables used in a simulation model
 */
@SuppressWarnings("serial")
public abstract class Variable implements Serializable{
	
	private static final String STOCK = "stock";
	private static final String FLOW = "flow";
	private static final String AUXILIARY = "auxiliary";
	
	protected int position;
	protected String name;
	protected String description;
	protected String units;
	protected String displayName;
	protected String displayUnits;
	protected boolean included;
	
	protected Node usesTree = null;
	protected Node causesTree = null;
	protected List<LoopObject> loops;
	
	protected String type;

	public Variable() {
	}
	public Variable(int position,String name, String description,String units,String displayName, String displayUnits, boolean included,
			String type) {
		this.position=position;
		this.name = name;
		this.description=description;
		this.units=units;
		this.displayName=displayName;
		this.displayUnits = displayUnits;
		this.included=included;
		this.type = type;
	}
	
	public boolean treeComponentsLoaded() {
		return usesTree != null && causesTree != null;
	}
	
	public boolean nameMatches(String s) {
		// match name differently
		String stemp = new String(s).replaceAll("^\"", "").replaceAll("\"$", "");
		String ntemp = new String(name).replaceAll("^\"", "").replaceAll("\"$", "");
		return stemp.equalsIgnoreCase(ntemp);
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = name;
	}
	public String getDescription() {
		return description;
	}
	public void setDescription(String description) {
		this.description = description;
	}
	public String getUnits() {
		if (displayUnits != null && displayUnits.length() > 0)
			return displayUnits;
		if (units != null)
			return units;
		else
			return "";
	}
	public void setUnits(String units) {
		this.units = units;
	}
	
	public String getDisplayName() {
		return displayName;
	}
	public String getDisplayUnits() {
		return displayUnits;
	}
	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}
	
	public void setDisplayUnits(String displayUnits) {
		this.displayUnits = displayUnits;
	}
	
	public boolean isIncluded() {
		return included;
	}
	public void setIncluded(boolean included) {
		this.included = included;
	}

	public String toString(){
		return name;
	}
	public int getPosition() {
		return position;
	}
	public void setPosition(int position) {
		this.position = position;
	}
	public boolean canBeEdited(){
		return false;
	}
	public boolean isStock() {
		if(type==null)return false;
		return type.equals(STOCK);
	}
	public boolean isFlow() {
		if(type==null)return false;
		return type.equals(FLOW);
	}
	public boolean isAuxiliary() {
		if(type==null)return false;
		return type.equals(AUXILIARY);
	}
	/**
	 * @return the type
	 */
	public String getType() {
		return type;
	}
	/**
	 * @param type the type to set
	 */
	public void setType(String type) {
		this.type = type;
	}
	public Node getTree() {
		return usesTree;
	}
	public void setTree(Node tree) {
		this.usesTree = tree;
	}
	public Node getCausesTree() {
		return causesTree;
	}
	public void setCausesTree(Node causesTree) {
		this.causesTree = causesTree;
	}
	public Node getUsesTree() {
		return usesTree;
	}
	public void setUsesTree(Node usesTree) {
		this.usesTree = usesTree;
	}
	public List<LoopObject> getLoops() {
		return loops;
	}
	public void setLoops(List<LoopObject> loops) {
		this.loops = loops;
	}
}
