package gov.anl.gss.sdm.shared.drawView;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Point2D;

public class Arrow extends ModelGraphic{
	protected Integer from;
	protected Integer to;
	protected Integer shape;
	protected Integer hidden;
	protected Integer polarity;
	protected Integer thickness;
	protected Integer hasfont;
	protected Integer delay;

	protected String font;
	protected int x[];
	protected int y[];
	protected int style;
	protected int lineStyle;

	final static int SPECIAL=0;
	final static int SINGLE_LINE=1;
	final static int DOUBLE_LINE=2;
	final static int ARC=1;
	final static int POLYLINE=0;
	final static int PERPENDICULAR=2;
	protected GeneralGraphic fromVariable=null;
	protected GeneralGraphic toVariable=null;
	int fromX,fromY,toX,toY;
	boolean drawHead=true;
	
	boolean antiClockwise;
	boolean switched = false;
	
	public Arrow(ModelDiagram modelView){
		super(modelView);
		color= Color.black;
		thickness=1;
		shape=POLYLINE;
	}
	
	public void shift(int xShift, int yShift) {
		fromX += xShift;
		fromY += yShift;
		
		toX += xShift;
		toY += yShift;
		
		for (int i = 0; i < x.length; i++) {
			x[i] += xShift;
			y[i] += yShift;
		}
	}
	public void setThickness(int code){
		if(code<10){
			style=SPECIAL;
			lineStyle=code;
			thickness=1+ModelView.LINE_THICKNESS_ADJUSTMENT;
		}else{
			if(code>=20){
				thickness=code-19+ModelView.LINE_THICKNESS_ADJUSTMENT;
				style=DOUBLE_LINE;
			}else{
				thickness=code-9+ModelView.LINE_THICKNESS_ADJUSTMENT;
				style=SINGLE_LINE;
			}
		}
	}
	
	public void parseFromStrings(String[] strings){
		try{
			try {
				id= Integer.parseInt(strings[1]);

				from= Integer.parseInt(strings[2]);
				to= Integer.parseInt(strings[3]);
				shape= Integer.parseInt(strings[4]);
				hidden= Integer.parseInt(strings[5]);
				hideLevel=hidden;
				polarity= Integer.parseInt(strings[6]);
				thickness= Integer.parseInt(strings[7]);
				setThickness(thickness);
				hasfont= Integer.parseInt(strings[8]);
				delay= Integer.parseInt(strings[9]);
				//reserved= Integer.parseInt(strings[10]);
				color= ModelView.getColorFrom(strings[11]);
				if(style!=DOUBLE_LINE&&color==null){
					color=modelView.arrowColor;
				}
				if(color==null)color=ModelView.getColorFrom(null);
				font= strings[12];

				// parse the intermediate points
				// first make the correct string
				String coordString = strings[13];

				for(int i=14;i<strings.length;i++)coordString=coordString+","+strings[i];
				String[] coordPoints = coordString.split("\\|");
				if(coordPoints.length>0){
					int num = Integer.parseInt(coordPoints[0]);
					x= new int[num];
					y= new int[num];

					for(int i=0;i<num;i++){
						String coords = coordPoints[i+1];
						coords=coords.substring(1,coords.length()-1);
						// break into two points
						String[] points = coords.split(",");
						if (points.length == 0) {
							return;
						}
						x[i]=Integer.parseInt(points[0]);
						
						y[i]=Integer.parseInt(points[1]);
					}


				}
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}catch(NumberFormatException e){
			return;
		}
	}
	public boolean isValid(){
		return fromVariable!=null && toVariable!=null;
	}
	void drawArrowHead(Graphics2D context,int x1,int y1,int x2,int y2){
		
		int lineLength=10; 
		GeneralPath path = new GeneralPath();
		context.setPaint(color);
		path.moveTo(x2, y2); 
		float angle = (float) Math.toDegrees(Math.atan2(  x2 - x1, y2-y1)); 
		int incx = (int)(Math.sin(((angle + 160) * Math.PI) / 180) * lineLength); 
		int incy = (int)(Math.cos(((angle + 160) * Math.PI) / 180) * lineLength); 
		path.lineTo(x2 + incx, y2 + incy); 
		context.draw(path); 
		int incx2 = (int)(Math.sin(((angle-160) * Math.PI) / 180) * lineLength); 
		int incy2 = (int)(Math.cos(((angle-160) * Math.PI) / 180) * lineLength); 
		path.lineTo(x2 + incx2, y2 + incy2); 
		path.closePath();
		
		context.draw(path);
		context.fill(path);
		
	}
	void drawArrowHeadCurve(Graphics2D context,int x1,int y1,int x2,int y2,double angle2,boolean flag, boolean switched){
		int lineLength=10; 
		GeneralPath path = new GeneralPath();
		context.setPaint(color);
		path.moveTo(x2, y2); 
		float angle4 = (float) Math.toDegrees(Math.atan2(  x2 - h, y2-k))+90; 
	//	System.out.println("angles "+angle+" "+angle3+" "+angle4+" "+flag+" "+switched);
		if(flag==switched)angle4=angle4+180;
		int incx = (int)(Math.sin(((angle4 + 160) * Math.PI) / 180) * lineLength); 
		int incy = (int)(Math.cos(((angle4 + 160) * Math.PI) / 180) * lineLength); 
		path.lineTo(x2 + incx, y2 + incy); 
		context.draw(path); 
		int incx2 = (int)(Math.sin(((angle4-160) * Math.PI) / 180) * lineLength); 
		int incy2 = (int)(Math.cos(((angle4-160) * Math.PI) / 180) * lineLength); 
		path.lineTo(x2 + incx2, y2 + incy2); 
		path.closePath();
		context.draw(path);
		context.fill(path);
		
	}
	double h,k,r,angle1,angle2,angle3;
	
	double a1,a2,a3;

	boolean closeTo(int x1,int x2){
		return (Math.abs(x2-x1)<3);
	}
	public void draw(Graphics2D context){
		
		//if(hideLevel>modelView.hideLevel)return;
		int lastX,lastY;
		int lastToX,lastToY;
		
		GeneralPath path = new GeneralPath();
		context.setPaint(color);
		lastX=fromX;
		lastY=fromY;
		lastToX=toX;
		lastToY=toY;
		//final static int ARC=0;
		//final static int POLYLINE=1;
		//final static int PERPENDICULAR=2;
//		antiClockwise=false;
		switched=false;
		if(shape==ARC && x!=null){
			context.setStroke(new BasicStroke(thickness/2));
			
			/**
			 * angle1 and angle2 are adjusted
			 * a1 and a2 are unadjusted
			 * 
			 * degAngle2 and degAngle1 are used for placement
			 * degA1 and degA2 are used for amount
			 */
			
			double degAngle2 = Math.toDegrees(angle2);
			double degAngle1 = Math.toDegrees(angle1);
//			
//			double degA2 = Math.toDegrees(a2);
//			double degA1 = Math.toDegrees(a1);
//			
//			Arc2D.Double arc = new Arc2D.Double();
//			double amount;
//
//			if (antiClockwise) {
//				amount = Math.abs(degA2-degA1);
//				amount = Math.abs(degAngle2-degAngle1);
//				System.out.println("ANTI");
//				arc.setArcByCenter(h,k,r,-degAngle1,amount,Arc2D.OPEN); // degAngle1
//			} else {
//				amount = Math.abs(degAngle2-degAngle1);
//				amount = Math.abs(degA2-degA1);
//				System.out.println("CLOCKWISE");
//				arc.setArcByCenter(h,k,r,-degAngle1,-amount,Arc2D.OPEN); // degAngle1 amount
//			}
//			context.draw(arc);
			
			
//			if(angle1<angle2){
//				if(angle3<angle1) {
//					antiClockwise=true;
//				} else{
//					if(angle3<angle2) {
//						antiClockwise=false;
//					} else  {
//						antiClockwise=true;
//					}
//				}
//				Arc2D.Double arc = new Arc2D.Double();
//				double amount = Math.abs(degAngle2-degAngle1);
//				//amount=180;
//				if(antiClockwise) {
//					arc.setArcByCenter(h,k,r,-degAngle2,-amount,Arc2D.OPEN);
//					printArcArgs(h,k,r,-degAngle2,-amount,"2 1<2", antiClockwise, switched);
//				} else {
//					arc.setArcByCenter(h,k,r,-degAngle1,-amount,Arc2D.OPEN);
//					printArcArgs(h,k,r,-degAngle1,-amount,"1 1<2", antiClockwise, switched);
//				}
//				context.draw(arc);
//				
//			}else{
//				if(angle3<angle2) {
//					antiClockwise=true;
//				} else{
//					if(angle3<angle1)antiClockwise=false;
//					else antiClockwise=true;
//				}
//				switched=true;
//				
//				Arc2D.Double arc = new Arc2D.Double();
//				double amount = Math.abs(degAngle1-degAngle2);
//				
//				if(antiClockwise) {
//					arc.setArcByCenter(h,k,r,-degAngle1,-amount,Arc2D.OPEN);
//					printArcArgs(h,k,r,-degAngle1,-amount,"1 1>=2", antiClockwise, switched);
//				} else { 
//					arc.setArcByCenter(h,k,r,-degAngle2,-amount,Arc2D.OPEN);
//					printArcArgs(h,k,r,-degAngle1,-amount,"2 1>=2", antiClockwise, switched);
//				}
//				context.draw(arc);
//				
//			}
			
			double amount = Math.abs(degAngle1-degAngle2);;
			double amount1 = Math.abs(degAngle1-degAngle2);
			double amount2 = 360.0 - amount1;
			
			double radians = Math.abs(angle1-angle3) + Math.abs(angle3-angle2);
			if (radians > Math.PI*2)
				radians -= Math.PI*2;
			double radDeg = Math.toDegrees(radians);
			
			double dist13 = GraphicCalculations.distance(fromX, fromY, x[0], y[0]);
			double dist23 = GraphicCalculations.distance(toX, toY, x[0], y[0]);
			double totalDist = dist23+dist13;
			double approx = totalDist/r;
			double approxDeg = Math.toDegrees(approx);
			
			double diff1 = Math.abs(amount1 - approxDeg); // radDeg
			double diff2 = Math.abs(amount2 - approxDeg); // radDeg
			if (diff1 < diff2)
				amount = amount1;
			else
				amount = amount2;
			
			int generationPoint = 0;
			
			
			Point2D fromPoint = new Point(fromX, fromY);
			Point2D toPoint = new Point(toX, toY);
			
			
			if(angle1<angle2){
				if(angle3<angle1) {
					antiClockwise=true;
//					amount = amount2;
				} else{
					if(angle3<angle2) {
						antiClockwise=false;
//						amount = amount1;
					} else  {
						antiClockwise=true;
//						amount = amount2;
					}
				}
				Arc2D.Double arc = new Arc2D.Double();
			
				//amount=180;
				if(antiClockwise) {
					generationPoint = 1;
//					arc.setArcByCenter(h,k,r,/*-degAngle1*/ 90.0,/*-amount*/ -45.0,Arc2D.OPEN);
					arc.setArcByCenter(h,k,r,-degAngle2,-amount,Arc2D.OPEN);
					
				} else {
					generationPoint = 2;
//					arc.setArcByCenter(h,k,r,-degAngle1,/*amount*/ 0.0,Arc2D.OPEN); 
					arc.setArcByCenter(h,k,r,-degAngle1,-amount,Arc2D.OPEN);
					
				}
				context.draw(arc);
				
			}else{
				if(angle3<angle2) {
					antiClockwise=true;
//					amount = amount2;
				} else{
					if(angle3<angle1) {
						antiClockwise=false;
//						amount = amount1;
					} else {
						antiClockwise=true;
//						amount = amount2;
					}
				}
				switched=true;
				
				Arc2D.Double arc = new Arc2D.Double();
				
				if(antiClockwise) {
					
					// Why are these wrong?
					generationPoint = 3;

//					arc.setArcByCenter(h,k,r,/*-degAngle1*/ 90.0,/*-amount*/ -45.0,Arc2D.OPEN);
//					if (amount == amount1)
//						amount = amount2;
//					else
//						amount = amount1;
					arc.setArcByCenter(h,k,r,-degAngle1,-amount,Arc2D.OPEN);
					
				} else { 
					generationPoint = 4;
					arc.setArcByCenter(h,k,r,-degAngle2,-amount,Arc2D.OPEN);
					
//					arc.setArcByCenter(h,k,r,-degAngle1,/*amount*/ 0.0,Arc2D.OPEN);
				}
				context.draw(arc);
				
			}
			
//			System.out.println(id+"\t"+generationPoint+"\t"+from+"\t"+fromX+"\t"+fromY+"\t"+to+"\t"+toX+"\t"+
//					toY+"\t"+x[0]+"\t"+y[0]+"\t"+
//					a1+"\t"+a2+"\t"+a3+"\t"+
//					angle1+"\t"+angle2+"\t"+
//					angle3+"\t"+h+"\t"+k+"\t"+r+"\t"+
//					degAngle1+"\t"+degAngle2+"\t"+
//					antiClockwise+"\t"+switched+"\t"+amount+"\t"+amount1+"\t"+amount2+"\t"+
//					diff1+"\t"+diff2+"\t"+
//					radDeg+"\t"+radians+"\t"+approx);

			lastX=x[0];
			lastY=y[0];
		}else{
			if(style==DOUBLE_LINE){
				context.setStroke(new BasicStroke(1));
				// draw each line segment
				int len=2;
				if(x!=null){
					len=len+x.length;
				}
				int[] pointx=new int[len];
				int[] pointy=new int[len];
				pointx[0]=fromX;
				pointy[0]=fromY;
				if(x!=null){
					for(int i=0;i<x.length;i++){
						pointx[1+i]=x[i];
						pointy[1+i]=y[i];
					}
				}
				pointx[len-1]=toX;
				pointy[len-1]=toY;
				int halfThick = thickness/2;
				// do each segment
				for(int i=0;i<len-1;i++){
					// see if more vertical
					if(Math.abs(pointx[i]-pointx[i+1])<Math.abs(pointy[i]-pointy[i+1])){
						// draw two lines
						path.moveTo(pointx[i]-halfThick, pointy[i]);
						path.lineTo(pointx[i]-halfThick, pointy[i+1]);
						context.draw(path);
						path.moveTo(pointx[i]+halfThick, pointy[i]);
						path.lineTo(pointx[i]+halfThick, pointy[i+1]);
						context.draw(path);
						lastX=pointx[i];
						lastY=pointy[i];
						lastToX=pointx[i];
						lastToY=pointy[i+1];
						
					}else{
						// horizontal
						// draw two lines
						path.moveTo(pointx[i], pointy[i]-halfThick);
						path.lineTo(pointx[i+1], pointy[i]-halfThick);
						context.draw(path);
						path.moveTo(pointx[i], pointy[i]+halfThick);
						path.lineTo(pointx[i+1], pointy[i]+halfThick);
						context.draw(path);
						lastX=pointx[i];
						lastY=pointy[i];
						lastToX=pointx[i+1];
						lastToY=pointy[i];
					}
				}
				
				
			}else{
				context.setStroke(new BasicStroke(thickness/2));
				path.moveTo(fromX, fromY);

				if(x!=null){
					for(int i=0;i<x.length;i++){
						path.lineTo(x[i],y[i]);
						lastX=x[i];
						lastY=y[i];
					}
				}

				path.lineTo(toX, toY);
				
			}
			//if(toVariable instanceof ValveGraphic)drawHead=false;
		}

		// MJB uncomment
		context.draw(path);
	//	path.closePath();
		
			// draw the arrow
		if(drawHead){
			if(shape==ARC){
				drawArrowHeadCurve(context,lastX,lastY,lastToX,lastToY,angle2,antiClockwise,switched);
			} else {
				drawArrowHead(context,lastX,lastY,lastToX,lastToY);
			}
		}
		
		super.draw(context);

	}

	
	public void computeRelations() {
		
		// calculate the positions based on the items it connects
		
		try {
			fromVariable = (GeneralGraphic)modelView.getGraphicFor(from.intValue());
			toVariable = (GeneralGraphic)modelView.getGraphicFor(to.intValue());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(fromVariable==null||toVariable==null)return;

		// make the end points
		int x1 = fromVariable.x;
		int y1=fromVariable.y;
		int x2=toVariable.x;
		int y2=toVariable.y;
		int x3 = fromVariable.x;
		int y3=fromVariable.y;
		int x4=toVariable.x;
		int y4=toVariable.y;

		fromX=x1;
		fromY=y1;
		toX=x2;
		toY=y2;
		
		if(x!=null&&x.length>0){
			// use the middle point in calcs instead
			x3=x[x.length-1];
			y3=y[x.length-1];
			x4=x[0];
			y4=y[0];
		}

		// clip with the box
		Point resultPoint=clipPoint(x1,y1,x4,y4,fromVariable.getMinX(),fromVariable.getMaxX(),fromVariable.getMinY(),fromVariable.getMaxY());
		Point resultPoint2=clipPoint(x2,y2,x3,y3,toVariable.getMinX(),toVariable.getMaxX(),toVariable.getMinY(),toVariable.getMaxY());
		
		if(resultPoint!=null){
			fromX=(int)resultPoint.getX();
			fromY=(int)resultPoint.getY();
		}
		if(resultPoint2!=null){
			toX=(int)resultPoint2.getX();
			toY=(int)resultPoint2.getY();
		}
		// set the range values based on endpoints
		minX=fromX;
		if(minX>toX)minX=toX;
		
		maxX=fromX;
		if(maxX<toX)maxX=toX;
		
		minY=fromY;
		if(minY>toY)minY=toY;
		
		maxY=fromY;
		if(maxY<toY)maxY=toY;
		

		// adjust the range for intermediate points
		if(x!=null){
			for(int i=0;i<x.length;i++){
				if(minX>x[i])minX=x[i];
				if(maxX<x[i])maxX=x[i];
				if(minY>y[i])minY=y[i];
				if(maxY<y[i])maxY=y[i];
			}
		}
		
//		int r = (int)Math.sqrt((x1-x0)*(x1-x0) + (y1-y0)*(y1-y0));
//		int x = x0-r;
//		int y = y0-r;
//		int width = 2*r;
//		int height = 2*r;
//		int startAngle = (int) (180/Math.PI*atan2(y1-y0, x1-x0));
//		int endAngle = (int) (180/Math.PI*atan2(y2-y0, x2-x0));
//		graphics.drawArc(x, y, width, height, startAngle, endAngle);
		
//		if(shape==ARC && x!=null){
//			h = GraphicCalculations.getCircleH(fromX,fromY,x[0],y[0],toX,toY);
//			k = GraphicCalculations.getCircleK(fromX,fromY,x[0],y[0],toX,toY);
//			r = GraphicCalculations.getCircleRadius(fromX,fromY,h,k);
//			
//			angle1 = GraphicCalculations.inverseTan(fromX, fromY, h, k);
//			angle2 = GraphicCalculations.inverseTan(toX, toY, h, k);
//			angle3 = GraphicCalculations.inverseTan(x[0], y[0], h, k);
//			
//
//			a1 = GraphicCalculations.inverseTanNoAdjust(fromX, fromY, h, k);
//			a2 = GraphicCalculations.inverseTanNoAdjust(toX, toY, h, k);
//			a3 = GraphicCalculations.inverseTanNoAdjust(x[0], y[0], h, k);
//		
//		}

		if(shape==ARC && x!=null){
			h = GraphicCalculations.getCircleH(fromX,fromY,x[0],y[0],toX,toY);
			k = GraphicCalculations.getCircleK(fromX,fromY,x[0],y[0],toX,toY);
			r = GraphicCalculations.getCircleRadius(fromX,fromY,h,k);
			
			angle1 = GraphicCalculations.inverseTan(fromX, fromY, h, k);
			angle2 = GraphicCalculations.inverseTan(toX, toY, h, k);
			angle3 = GraphicCalculations.inverseTan(x[0], y[0], h, k);
			

			a1 = GraphicCalculations.inverseTanNoAdjust(fromX, fromY, h, k);
			a2 = GraphicCalculations.inverseTanNoAdjust(toX, toY, h, k);
			a3 = GraphicCalculations.inverseTanNoAdjust(x[0], y[0], h, k);
		
		}
	}
	


	
	
	Point clipPoint(int x1, int y1, int x2,int y2,int minX, int maxX,int minY, int maxY) {
		// clip x1 which is inside the box
		// get the line equation
		// vertical line
		if(x2==x1){
			if(y2<minY){
				return new Point(x2,minY);
			}else{
				return new Point(x2,maxY);
			}
		}
		// horizontal
		if(y2==y1){
			if(x2<minX){
				return new Point(minX,y1);
			}else{
				return new Point(maxX,y1);
			}
		}

		double[] vals = GraphicCalculations.getIntersection(x1, y1, x2,y2,minX, minY,maxX, maxY);
		if(vals==null)return null;
		return new Point((int)vals[0],(int)vals[1]);
		
	}
	
	private String mark(double d) {
		return "<"+d+">";
	}
	private String mark(String d) {
		return "<"+d+">";
	}
	private String mark(boolean d) {
		return "<"+d+">";
	}
	
	private void printArcArgs(double h, double k, double r, double degAngle2, double amount, String angle, boolean anti, boolean switched) {
		System.out.println("setArc "+mark(id)+mark(h)+mark(k)+mark(r)+mark(degAngle2)+mark(amount)+mark(angle)+mark(anti)+mark(switched));
		
	}

	public Integer getThickness() {
		return thickness;
	}

	public void setThicknessAdjustment(int thickness) {
		this.thickness = thickness;
	}
}