package gov.anl.gss.sdm.shared.drawView;

import gov.anl.gss.sdm.server.model.tools.AdjacencyToMap;
import gov.anl.gss.sdm.server.model.tools.GraphicsHelper;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.util.ArrayList;

public abstract class GeneralGraphic extends ModelGraphic{
	/**
	 * 
	 */
	String name;
	int x;
	int y;
	Integer wid;
	Integer hi;
	Integer shape;
	Integer bits;
	Integer hidden;
	Integer hasfont;
	Integer textpos;
	Integer boxWidth;
	Integer nav1;
	Integer nav2;
	Color box;
	Color fill;
	String font;
	int fontSize;
	ArrayList<String> wordStrings;
	boolean hasBoxColor,hasWordColor,hasFillColor,hasFontSize,hasFontAttribs,hasFontFace;
	public static final int RECT=3;
	public static final int CIRCLE=2;
	
	public void parseHasfont(){
		byte b = hasfont.byteValue();
		hasBoxColor = ((hasfont & (1L << 0)) != 0);
		hasWordColor = ((hasfont & (1L << 1)) != 0);
		hasFillColor = ((hasfont & (1L << 2)) != 0);
		hasFontSize = ((hasfont & (1L << 3)) != 0);
		hasFontAttribs = ((hasfont & (1L << 4)) != 0);
		hasFontFace = ((hasfont & (1L << 5)) != 0);
		
	}
	
	public void shift(int xShift, int yShift) {
		x += xShift;
		y += yShift;
	}
	public String getFontFrom(String fontString){
		if(fontString.isEmpty()){
			fontSize=modelView.fontSize;
			return null;
		}

		String[] components = fontString.split("\\|");
		String font="";
		String fontSizeString=null;
		String fontFamily=null;
		String fontStyle=null;
		for(int i=0;i<components.length;i++){
			if(components[i]==null)continue;
			components[i]=components[i].trim();
			if(components[i].length()==0)continue;
			if(components[i].contains("-")){
				color=ModelView.getColorFrom(components[i]);
				if(color==null)
				continue;
			}
			try{
				int num = Integer.parseInt(components[i]);
				this.fontSize=num+ModelView.FONT_ADJUSTMENT;
				fontSizeString=num+"px";
				//font=font+" "+num+"px";
				continue;
			}catch(NumberFormatException e){
			}
			if(components[i].contains(" ")){
				String fontType=null;
				String[] fonts = components[i].split(" ");
				for(String fontPart:fonts){
					if(fontType==null)fontType=fontPart;
					else fontType = fontType+"-"+fontPart;
				}
				if(fontStyle==null){
					fontFamily=fontType;
					continue;
				}
			}
			fontStyle=components[i];
		}
		if(fontStyle!=null)font=fontStyle;
		if(fontSizeString!=null)font=font +" "+fontSize;
		if(fontFamily!=null){
			font=font+" \""+fontFamily+"\"";
		}

		if(fontSize==0){
			fontSize=modelView.fontSize;
		}
		return font;
	}
	void drawText(Graphics2D context,String string,double x, double y){
		context.setStroke(new BasicStroke(1));
		FontMetrics metrics= modelView.context.getFontMetrics(modelView.context.getFont());
		int testWidth=metrics.stringWidth(string);
		justification = CENTER;
		if (textpos == AdjacencyToMap.TEXTPOS_INSIDE) {
			
		} else if (textpos == AdjacencyToMap.TEXTPOS_LEFT) {
			justification = LEFT;
//			justification = RIGHT;
		} else if (textpos == AdjacencyToMap.TEXTPOS_RIGHT) {
			justification = RIGHT;
//			justification = LEFT;
		}
			if(justification==CENTER){
			context.drawString(string, (float)x-testWidth/2, (float)y);
		}else{
			if(justification==LEFT) {
				context.drawString(string, (float)x-wid-testWidth, (float)y);
			} else {
				context.drawString(string, (float)x+wid, (float)y);
			}
		}
	//	context.strokeText(string, (double)x, (double)y);
	}
	void splitWords(String text){
		setFontFor(modelView.context);
		if(text==null)return;
//		String[] words = text.split(" ");
		String[] words = new String[1];
		words[0] = text;
		String line = "";

		if(words.length==0)return;

		boolean complete=true;
		wordStrings=new ArrayList<String>();
		String testLine=words[0];
		line=testLine;
		int maxWidth=wid*2;
		for(int n = 1; n < words.length; n++) {
			testLine = line+" "+words[n];
			Font myFont = new Font(font,Font.PLAIN,fontSize);
			FontMetrics metrics= modelView.context.getFontMetrics(myFont);
			  int testWidth=metrics.stringWidth(testLine);

			if (testWidth > maxWidth) {
				// have to stop it before adding it
				wordStrings.add(line);
				line = words[n];
			}
			else {
				line = testLine;
			}

		}
		wordStrings.add(line);


	}
	final static int LEFT =0;
	final static int CENTER =1;
	final static int RIGHT =2;
	int justification=RIGHT; // CENTER;
//	Graphics2D.TextAlign[] alignments = {Graphics2D.TextAlign.LEFT,Graphics2D.TextAlign.CENTER,Graphics2D.TextAlign.RIGHT};
	public void setFontFor(Graphics2D context){
		context.setFont(new Font(font,Font.PLAIN,fontSize));
	/*	
		if(font!=null){
			// 			System.out.println("font "+font);
			context.setFont(font);
		}else{
			context.setFont(modelView.fontString);
		}
		context.setTextBaseline(TextBaseline.MIDDLE);
		
		context.setTextAlign(alignments[justification]);
		*/
	}
	void drawWords(Graphics2D context){
		if(wordStrings==null)return;
		double posX,posY;

		int lineHeight = fontSize+2;
		posX=x;
		posY= (int)(y - lineHeight*(((double)wordStrings.size()-1)*0.5));
		
		setFontFor(context);
		
		context.setPaint((color==null)? Color.black : color);
		for(int i=0;i<wordStrings.size();i++){
			drawText(context,wordStrings.get(i),posX,posY);
			posY = posY + lineHeight;
		}

	}
	abstract String parseName(String potentialName);
	//n,id,name,x,y,w,h,sh,bits,hid,hasf,tpos,bw,nav1,nav2,box,fill,font
	public GeneralGraphic (ModelDiagram modelView){
		super(modelView);
		shape=0;
	}
	public void parseFromStrings(String[] strings){
		try{
			id= Integer.parseInt(strings[1]);
			name= parseName(strings[2]);
			double dx= Double.parseDouble(strings[3]);
			x = (int) dx;
			
			double dy= Double.parseDouble(strings[4]);
			y = (int) dy;
			wid= Integer.parseInt(strings[5]);
			hi= Integer.parseInt(strings[6]);
			shape= Integer.parseInt(strings[7]);
			bits= Integer.parseInt(strings[8]);
			hidden= Integer.parseInt(strings[9]);
			hideLevel=hidden;
//			if(id==21){
//				System.out.println(" here");
//			}
			hasfont= Integer.parseInt(strings[10]);
			parseHasfont();
			textpos= Integer.parseInt(strings[11]);
			boxWidth= Integer.parseInt(strings[12]);
			nav1= Integer.parseInt(strings[13]);
			nav2= Integer.parseInt(strings[14]);

			byte b = shape.byteValue();
			shape=shape & 0x001F;
			if(strings.length>=16){
				if(hasBoxColor)box=ModelView.getColorFrom(strings[15]);
				if(strings.length>=17){
					if(hasFillColor)fill= ModelView.getColorFrom(strings[16]);
					if(strings.length>=18){
						font= getFontFrom(strings[17]);
					}
				}
			}
			if(font==null){
				fontSize=modelView.fontSize;
				font=fontSize+"px Times-New-Roman";
			}
		}catch(NumberFormatException e){
			return;
		}
	}
	
	public void drawSelected(Graphics2D context){
		// highlight the shape
		if(!fillShape(context,Color.lightGray)){
			fillBoundingBox(context,Color.lightGray);
		}
	}
	
	public boolean drawShape(Graphics2D context,Color color,int lineWid){
		if(color==null)color=modelView.getShapeColor();
		context.setColor((color==null)? Color.black : color);
		context.setStroke(new BasicStroke(lineWid));
		switch(shape){
		case RECT:
			context.drawRect(x-wid,y-hi,wid*2,hi*2);
			return true;
		case CIRCLE:
//			context.drawArc(x-wid, y-hi, wid*2, hi*2, 0,360);
			context.drawArc(x-wid, y-hi, wid*2, hi*2, 0,360);
			return true;
		}
		return false;
	}
	public boolean fillShape(Graphics2D context,Color color){
		if(color==null)color=modelView.getFillColor();
		if(color==null)return true;
		
		context.setColor(color);
		switch(shape){
		case RECT:
			context.fillRect(x-wid,y-hi,wid*2,hi*2);
			return true;
		case CIRCLE:
			context.fillArc(x-wid, y-hi, wid*2, hi*2, 0,360);
			return true;
		}
		return false;
	}
	public void draw(Graphics2D context){
		super.draw(context);
		
		// draw the rectangle if there is one
		fillShape(context,fill);
		drawShape(context,box,1);
		drawWords(context);
		
	}
	public boolean isValid(){
		return (wid>0 && hi >0);
	}

	public void computeRelations() {
		if (wid == null) {
			System.out.println("wid is null");
		}
		
		// if text is positioned to the left or right of the variable, we must account for it
		
		minX=x-wid;
		maxX=x+wid;
		minY=y-hi;
		maxY=y+hi;
		
		if (textpos > 0) {			
			GraphicsHelper gh = new GraphicsHelper(new Font("Times-New-Roman",Font.PLAIN, 12));
			int textWidth = gh.getWidth(name);
			if (textpos == 2) {  // text to left of variable
				minX=x-(wid+textWidth);
			} else if (textpos == 4) {  // text to right of variable
				maxX=x+wid+textWidth;
			}
		}

		if(name!=null)
			splitWords(name);

	}
	public String toString(){
		return ""+name;
	}
	public int getJustification() {
		return justification;
	}
	public void setJustification(int justification) {
		this.justification = justification;
	}

	public int getFontSize() {
		return fontSize;
	}

	public void setFontSize(int fontSize) {
		this.fontSize = fontSize;
	}
}