package gov.anl.gss.sdm.shared.drawView;

public class GraphicCalculations {
	static double area2(double x1, double y1,
			double x2, double y2,
			double x3, double y3) 
	{
		return (x2 - x1) * (y3 - y1) - (x3 - x1) * (y2 - y1);    
	}
	static boolean between(double x1, double y1, 
			double x2, double y2, 
			double x3, double y3) 
	{
		if (x1 != x2) {
			return (x1 <= x3 && x3 <= x2) || (x1 >= x3 && x3 >= x2);   
		}
		else {
			return (y1 <= y3 && y3 <= y2) || (y1 >= y3 && y3 >= y2);   
		}
	}


	/**
	 * Test if the line segment (x1,y1)-&gt;(x2,y2) intersects the line segment 
	 * (x3,y3)-&gt;(x4,y4).
	 *
	 * @param x1 the first x coordinate of the first segment
	 * @param y1 the first y coordinate of the first segment 
	 * @param x2 the second x coordinate of the first segment
	 * @param y2 the second y coordinate of the first segment
	 * @param x3 the first x coordinate of the second segment
	 * @param y3 the first y coordinate of the second segment
	 * @param x4 the second x coordinate of the second segment
	 * @param y4 the second y coordinate of the second segment
	 * @return true if the segments intersect
	 */
	public static boolean linesIntersect(double x1, double y1,
			double x2, double y2,
			double x3, double y3,
			double x4, double y4)
	{
		double a1, a2, a3, a4;

		// deal with special cases
		if ((a1 = area2(x1, y1, x2, y2, x3, y3)) == 0.0) 
		{
			// check if p3 is between p1 and p2 OR
			// p4 is collinear also AND either between p1 and p2 OR at opposite ends
			if (between(x1, y1, x2, y2, x3, y3)) 
			{
				return true;
			}
			else 
			{
				if (area2(x1, y1, x2, y2, x4, y4) == 0.0) 
				{
					return between(x3, y3, x4, y4, x1, y1) 
							|| between (x3, y3, x4, y4, x2, y2);
				}
				else {
					return false;
				}
			}
		}
		else if ((a2 = area2(x1, y1, x2, y2, x4, y4)) == 0.0) 
		{
			// check if p4 is between p1 and p2 (we already know p3 is not
			// collinear)
			return between(x1, y1, x2, y2, x4, y4);
		}

		if ((a3 = area2(x3, y3, x4, y4, x1, y1)) == 0.0) {
			// check if p1 is between p3 and p4 OR
			// p2 is collinear also AND either between p1 and p2 OR at opposite ends
			if (between(x3, y3, x4, y4, x1, y1)) {
				return true;
			}
			else {
				if (area2(x3, y3, x4, y4, x2, y2) == 0.0) {
					return between(x1, y1, x2, y2, x3, y3) 
							|| between (x1, y1, x2, y2, x4, y4);
				}
				else {
					return false;
				}
			}
		}
		else if ((a4 = area2(x3, y3, x4, y4, x2, y2)) == 0.0) {
			// check if p2 is between p3 and p4 (we already know p1 is not
			// collinear)
			return between(x3, y3, x4, y4, x2, y2);
		}
		else {  // test for regular intersection
			return ((a1 > 0.0) ^ (a2 > 0.0)) && ((a3 > 0.0) ^ (a4 > 0.0));
		} 
	}

	static public double[] getIntersection(double x1, double y1, double x2, double y2,double minX,double minY,double maxX,double maxY) {

		double m = (double)(y2-y1)/(double)(x2-x1);
		double b = y1-m*x1;
		int x,y;
		double[] vals= new double[2];
		// see if it intersects the different sides
		//top 
		if(linesIntersect(x1, y1,x2, y2,minX,minY,maxX,minY)){
			//clip with top of rect
			//y=m*x+b;
			vals[0]=(minY-b)/m;
			vals[1]=minY;
			return vals;
		}
		// bottom
		if(linesIntersect(x1, y1,x2, y2,minX,maxY,maxX,maxY)){
			//clip with top of rect
			//y=m*x+b;
			vals[0]=(maxY-b)/m;
			vals[1]=maxY;
			return vals;
		}
		// see if it intersects the different sides
		// left
		if(linesIntersect(x1, y1,x2, y2,minX,minY,minX,maxY)){
			//clip with left of rect
			//y=m*x+b;
			vals[0]=minX;
			vals[1]=b+minX*m;
			return vals;
		}
		// right
		if(linesIntersect(x1, y1,x2, y2,maxX,minY,maxX,maxY)){
			//clip with right of rect
			vals[0]=maxX;
			vals[1]=b+maxX*m;
			return vals;
		}
		return null;

	}
	static public double getCircleH(double a, double b, double c, double d, double e, double f){
		double h = 0.5*((a*a+b*b)*(f-d) + (c*c+d*d)*(b-f) + (e*e+f*f)*(d-b)) / (a*(f-d)+c*(b-f)+e*(d-b));
		return h;
	}
	static public double getCircleK(double a, double b, double c, double d, double e, double f){
		double k = 0.5*((a*a+b*b)*(e-c) + (c*c+d*d)*(a-e) + (e*e+f*f)*(c-a)) / (b*(e-c)+d*(a-e)+f*(c-a)) ;
		return k;
	}
	static public double getCircleRadius(double a, double b, double h, double k){
		double rsquare = (a-h)*(a-h) + (b-k)*(b-k);  
		return Math.sqrt(rsquare);
	}
	static public double inverseTan(double a, double b, double h, double k){
		double angle= Math.atan2(b-k, a-h);
		if(angle<0)angle=angle+2.0*Math.PI;
//		System.out.println(a+", "+b+", "+h+", "+k+"   "+(b-k)+", "+(a-h)+"   "+angle);
	//	if(a<h&&b<k)angle=angle+Math.PI;
	//	else if(a<h&&b>k)angle=angle+Math.PI;
		return angle;
	}
	
	static public double inverseTanNoAdjust(double a, double b, double h, double k){
		double angle= Math.atan2(b-k, a-h);
//		if(angle<0)angle=angle+2.0*Math.PI;
	//	if(a<h&&b<k)angle=angle+Math.PI;
	//	else if(a<h&&b>k)angle=angle+Math.PI;
		return angle;
	}
	static public Double slopeAt(double a, double b, double h, double k){
		double val = Math.atan2(b-k, a-h);
		return val;
	}
	
	static public double distance(double x1, double y1, double x2, double y2) {
		double dist = Math.sqrt((x2-x1)*(x2-x1) + (y2-y1)*(y2-y1));
		return dist;
	}
}
