package gov.anl.gss.sdm.shared.drawView;

import java.awt.Color;
import java.awt.Graphics2D;
import java.util.ArrayList;

abstract public class ModelDiagram{
	public int fontSize=16+ModelView.FONT_ADJUSTMENT;
	public String fontString=fontSize+"px Times-New-Roman";
	//Canvas canvas;
	Graphics2D context;
	int canvasHeight = 800;
	int canvasWidth = 600;
	String modelName;
	int maxX,maxY;
	int minX,minY;
	ModelGraphic currentItem=null;
	int hideLevel=16;
	
	public static Color getColorFrom(String colorString){
		if(colorString==null)return Color.black;
		if(colorString.startsWith("-1"))return null;
		String[] components = colorString.trim().split("-");
		int[] nums = new int[3];
		if(components.length==3){
			for(int i=0;i<3;i++)nums[i]=Integer.parseInt(components[i]);
			return new Color(nums[0],nums[1],nums[2]);
		}
		return Color.black;
	}
	Color bgColor,shapeColor,fillColor,arrowColor,initialArrowColor,textColor;
	public boolean isSelected(ModelGraphic g){
		return g==currentItem;
	}
	public ArrayList<ModelGraphic> graphics;
	public ModelGraphic getGraphicFor(int id){
		return getGraphicFor(graphics,id);
	}
	static public ModelGraphic getGraphicFor(ArrayList<ModelGraphic>graphics,int id){
		for(ModelGraphic graphic:graphics){
			if(graphic.id==id){
				return graphic;
			}
		}
		return null;
	}
	void computeRelations() {
		// do all but arrows
		for(ModelGraphic graphic:graphics){
			if(!(graphic instanceof Arrow)) {
//				if (graphic.id == 42)
//					System.out.println("Not Arrow "+graphic.id);
				try {
					graphic.computeRelations();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
		// then do the arrows
		for(ModelGraphic graphic:graphics){
			if(graphic instanceof Arrow) {
				try {
					graphic.computeRelations();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}
	}
	
	public void resizeCanvas(int wid,int hi){
		// resize the two canvases
	//	canvas.setCoordinateSpaceWidth(wid);
	//	canvas.setCoordinateSpaceHeight(hi);

		canvasWidth=wid;
		canvasHeight=hi;

		// redraw things
	//	clearBackingStore();
	//	mainDraw();
	}
	
	public void drawContents(Graphics2D context){
		// draw box around it
		//context.strokeRect(minX,minY,maxX-minX,maxY-minY);

		for(ModelGraphic graphic:graphics){
			
			if (graphic instanceof GeneralGraphic) {
				GeneralGraphic gg = (GeneralGraphic) graphic;
			}
			
			if(graphic.isValid()&&hideLevel>=graphic.hideLevel) {
				graphic.draw(context);
			}
		}
	}
	public ModelDiagram(String modelName){
		this.modelName=modelName;

		graphics=new ArrayList<ModelGraphic>();
		/*
		canvas = new Canvas();

		canvas.setCoordinateSpaceWidth(canvasWidth);
		canvas.setCoordinateSpaceHeight(canvasHeight);
		backCanvas.setCoordinateSpaceWidth(canvasWidth);
		backCanvas.setCoordinateSpaceHeight(canvasHeight);
*/
	}

	void calcBoundingBox() {
		maxX=0;maxY=0;
		boolean firstTime=true;
		for(ModelGraphic graphic:graphics){
			if(graphic.isValid()){
				if(firstTime){
					minX=graphic.getMinX();
					maxX=graphic.getMaxX();
					minY=graphic.getMinY();
					maxY=graphic.getMaxY();
					firstTime=false;
				}else{
					if(graphic.getMinX()<minX)minX=graphic.getMinX();
					if(graphic.getMaxX()>maxX)maxX=graphic.getMaxX();
					if(graphic.getMinY()<minY)minY=graphic.getMinY();
					if(graphic.getMaxY()>maxY)maxY=graphic.getMaxY();
					if(maxY>10000){
						System.out.println("oops");
					}
				}
			}
		}

		//System.out.println("bounds "+minX+","+minY+" - "+maxX+","+maxY);
	}
	// get object over
	ModelGraphic getObjectOver(double x, double y){
	//	System.out.println("coords "+x+","+y);
		for(ModelGraphic graphic:graphics){
			if(graphic instanceof VariableGraphic){
				// see if in bounding box
				if((graphic.getMinX()<=x && x<=graphic.getMaxX())&&(graphic.getMinY()<=y && y<=graphic.getMaxY())){
					return graphic;
				}
			}
		}
		return null;
	}

	double zoom = 1;
	double totalZoom = 1;
	double offsetX = 0;
	double offsetY = 0;

	public void showWholeArea(){
		calcBoundingBox();
		double sizeX=maxX-minX;
		double sizeY=maxY-minY;
		double scaleX = (double)canvasWidth/sizeX;
		double scaleY = (double)canvasHeight/sizeY;

		totalZoom= scaleX<scaleY ? scaleX : scaleY;
		offsetX = -minX;
		offsetY = -minY;
		

	}
	
	public Color getBgColor() {
		return bgColor;
	}
	public void setBgColor(Color bgColor) {
		this.bgColor = bgColor;
	}
	public Color getShapeColor() {
		return shapeColor;
	}
	public void setShapeColor(Color shapeColor) {
		this.shapeColor = shapeColor;
	}
	public Color getFillColor() {
		return fillColor;
	}
	
	public void setFillColor(Color fillColor) {
		this.fillColor = fillColor;
	}
	public Color getArrowColor() {
		return arrowColor;
	}
	public void setArrowColor(Color arrowColor) {
		this.arrowColor = arrowColor;
	}
	public Color getInitialArrowColor() {
		return initialArrowColor;
	}
	public void setInitialArrowColor(Color initialArrowColor) {
		this.initialArrowColor = initialArrowColor;
	}
	public Color getTextColor() {
		return textColor;
	}
	public void setTextColor(Color textColor) {
		this.textColor = textColor;
	}
	public int getHideLevel() {
		return hideLevel;
	}
	public void setHideLevel(int hideLevel) {
		this.hideLevel = hideLevel;
	}
	public String getCurrentVariable() {
		if(currentItem!=null && currentItem instanceof VariableGraphic)return ((VariableGraphic)currentItem).name;
		return null;
	}
}
