package gov.anl.gss.sdm.shared.drawView;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.UserSelectedOptions;
import gov.anl.gss.sdm.server.model.translator.InformationManagers;
import gov.anl.gss.sdm.server.model.translator.Parser;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;

import net.coobird.thumbnailator.Thumbnails;

public class SDModelViewImageGenerator extends ModelDiagram{
	
	private double SCALE = 1000.0;
	private static final double LARGEST_ADJUSTMENT = 12.0;
	
	private String modelName;
	private String[] viewContents;
	private String imageFile;
	
	public SDModelViewImageGenerator(String modelName, String[] viewContents, String imageFile){
		super(modelName);
		this.modelName = modelName;
		this.viewContents = viewContents;
		this.imageFile = imageFile;
		generate();
	}
	
	private void generate() {
		BufferedImage offImage = new BufferedImage(100, 100,  BufferedImage.TYPE_INT_ARGB); // 100, 100,
		Graphics2D g2 = offImage.createGraphics();
		
		context=g2;
		if(viewContents.length>1){
			parseDefaults(viewContents[3]);
			// get rid of first three strings
			String[] strings = new String[viewContents.length-3];
			for(int i=0;i<strings.length;i++){
				strings[i]=viewContents[i+3];
			}
			parseContents(strings);
		}
		showWholeArea();
		calcBoundingBox();
		
		UserSelectedOptions options = InformationManagers.getInstance().getUserSelectedOptions();
		SCALE = Double.parseDouble(options.getOptionValue(StringConstants.VIEW_WIDTH_MAX));
		
		double xRange = maxX - minX;
		
		double scaling = SCALE / (double) xRange;
		
//		System.out.println("#####################################################################");
//		System.out.println("Name: "+ modelName+ " minX: "+minX+" maxX: "+maxX+" minY: "+minY+" maxY: "+maxY
//				+" scaling: "+scaling);
		
		
//		int xAdjust = -minX;
//		int yAdjust = -minY;
		
		int xAdjust = 0;
		int yAdjust = 0;
		if (minX < 0) {
			xAdjust = -minX;
		}
		if (minY < 0) {
			yAdjust = -minY;
		}
		
//		System.out.println("xAdjust: "+xAdjust+" yAdjust: "+yAdjust);
//		System.out.println("New Buffered Image: "+((int) ((maxX+xAdjust))+100)+" x "+((int) ((maxY+yAdjust))+100));

		// draw it with real image of proper size 
		offImage =
				new BufferedImage((int) ((maxX+xAdjust))+100, (int) ((maxY+yAdjust))+100,     // 100
						BufferedImage.TYPE_INT_ARGB); // TYPE_INT_ARGB
		
		scaling = SCALE / (maxX+xAdjust+150);
		
//		System.out.println("xRange: "+(maxX+xAdjust+100)+" scaling: "+scaling);
		
		adjustFontSizeAndLineThickness(scaling);
		
		
		g2 = offImage.createGraphics();
		
//		if (minX < 0 || minY < 0)
//			g2.translate(minX, -minY);  // -minY to flip?
//		g2.scale(scaling, scaling);
		BufferedImage thumbnail;
		try {
//			thumbnail = Thumbnails.of(offImage)
//			        .scale(0.25)
//			        .asBufferedImage();
//			g2 = thumbnail.createGraphics();
//		} catch (IOException e1) {
//			// TODO Auto-generated catch block
//			e1.printStackTrace();
//		}
		
		context=g2;
		g2.translate(xAdjust, yAdjust+50);
		drawContents(g2);
		
		scaling = 1.0;
		
		thumbnail = Thumbnails.of(offImage)
        .scale(scaling)
        .asBufferedImage();
//		thumbnail = offImage;
		
//		try {
			// retrieve image
			File outputfile = new File(imageFile);
			ImageIO.write(thumbnail, "png", outputfile);
		} catch (IOException e) {

		}
	}

	
	private void parseDefaults(String string) {
		// initArrow
		// Font
		// fontsize
		// fontoptions
		// shapeColor
		// arrowColor
		// fillColor or -1--1--1 for nones
		// background color
		String[] tokens = string.split("\\|");
		if(tokens.length<0)return;
		
		String[] tokenF = tokens[0].split(",");
		if(tokenF.length>2){
			fontString=tokenF[2];
		}
		
		fontSize=Integer.parseInt(tokens[1])+ModelView.FONT_ADJUSTMENT;
		
		textColor=getColorFrom(tokens[3]);
		shapeColor=getColorFrom(tokens[4]);
		arrowColor=getColorFrom(tokens[5]);
		fillColor=getColorFrom(tokens[6]);
		bgColor=getColorFrom(tokens[7]);
		
	}
	
	
	public void parseContents(String[] strings){
		// do the string for the defaults
		
		try {
			// get the string for the model view
			for(int i=0;i<strings.length;i++){
				String string=strings[i];
				
				// try to parse it
//				String[] tokens = string.split(",");
				String[] tokens = Parser.splitQuoted(string, ",");
				if(tokens.length<0)continue;
				switch(tokens[0]){
				case "1":
					Arrow arrow = new Arrow(this);
					arrow.parseFromStrings(tokens);
					graphics.add(arrow);
					break;
				case "10":
					VariableGraphic variable = new VariableGraphic(this);
					variable.parseFromStrings(tokens);
					graphics.add(variable);
					break;
				case "11":
					ValveGraphic variable2 = new ValveGraphic(this);
					variable2.parseFromStrings(tokens);
					graphics.add(variable2);
					break;
				case "12":
					if((i+1)>=strings.length){
						Comment comment = new Comment(this);
						comment.parseFromStrings(tokens);

					}
					Comment comment = new Comment(this);
					comment.parseFromStrings(tokens,(i+1 >= strings.length) ? null : strings[i+1]);

					// see the bit status
					if(comment.hasStringBit()){
						i=i+1;
					}

					
					graphics.add(comment);
					break;
				}
			}
			computeRelations();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}
	
	private void adjustFontSizeAndLineThickness(double scaling) {
		
		if (scaling >= 1.0)
			return;
		
		int adjustment = (int)( (1.0 - scaling) * LARGEST_ADJUSTMENT);
		
		fontSize=18+adjustment;
		fontString=fontSize+"px Times-New-Roman";
		
		for(ModelGraphic graphic:graphics){
			
			if (graphic instanceof GeneralGraphic) {
				GeneralGraphic gg = (GeneralGraphic) graphic;
				gg.setFontSize(Math.min(18, gg.getFontSize()+adjustment));
			}
			
			if (graphic instanceof Arrow) {
				Arrow arrow = (Arrow) graphic;
				arrow.setThicknessAdjustment(Math.min(18, arrow.getThickness()+adjustment));
			}
		}
	}
	

	static String selectedString;
	
}
