package gov.anl.gss.visualization;

import gov.anl.gss.sdm.server.model.documentor.xmlwriter.NumberFormatter;

import java.util.LinkedHashSet;
import java.util.Set;

import org.jfree.data.xy.XYSeries;

public class BoundingBox {
	
	final static int X = 0;
	 final static int Y = 1;

	private double[] lowerLeft = new double[2];
	private double[] lowerRight = new double[2];
	private double[] upperLeft = new double[2];
	private double[] upperRight = new double[2];
	
	private Coordinate coordinate;
	
	private boolean centered = false;
	private double x;
	private double y;
	
	private int numShifts = 0;
	
	private SFEXYPointerAnnotationMetaData meta;
	
	boolean annotationBoundingBox = true;
	
	private int ptr;
	private Set<Integer> ptrs = new LinkedHashSet<Integer>();
	
	public BoundingBox(SFEXYPointerAnnotationMetaData meta, double x,
			double y,
			double width,
			double height,
			int ptr) {
		
		this.meta = meta;
		this.x = x;
		this.y = y;
		this.ptr = ptr;
		ptrs.add(ptr);
		
		this.lowerLeft[X] = x-width/2.0;
		this.lowerRight[X] = x+width/2.0;
		this.upperLeft[X] = x-width/2.0;
		this.upperRight[X] = x+width/2.0;
		
		this.lowerLeft[Y] = y-height/2;
		this.lowerRight[Y] = y-height/2;
		this.upperLeft[Y] = y+height/2;
		this.upperRight[Y] = y+height/2;
		
		coordinate = new Coordinate(x,y, meta.getMinX(), meta.getMaxX(), meta.getMinY(), meta.getMaxY());

	}

	public BoundingBox(
			SFEXYPointerAnnotationMetaData meta, 
			double x,
			double y,
			double[] lowerLeft,
			double[] lowerRight,
			double[] upperLeft,
			double[] upperRight,
			int ptr) {
		
		this.meta = meta;
		this.x = x;
		this.y = y;
		
		this.lowerLeft[X] = lowerLeft[X];
		this.lowerRight[X] = lowerRight[X];
		this.upperLeft[X] = upperLeft[X];
		this.upperRight[X] = upperRight[X];
		
		this.lowerLeft[Y] = lowerLeft[Y];
		this.lowerRight[Y] = lowerRight[Y];
		this.upperLeft[Y] = upperLeft[Y];
		this.upperRight[Y] = upperRight[Y];
		
		this.ptr = ptr;
		ptrs.add(ptr);
		
		coordinate = new Coordinate(x,y, meta.getMinX(), meta.getMaxX(), meta.getMinY(), meta.getMaxY());
	}
	
	public XYSeries getXYSeries() {
		XYSeries series = new XYSeries("bb"+ptr, false, true);
		
		series.add(lowerLeft[X], lowerLeft[Y]);
		series.add(upperLeft[X], upperLeft[Y]);
		series.add(upperRight[X], upperRight[Y]);
		series.add(lowerRight[X], lowerRight[Y]);
		series.add(lowerLeft[X], lowerLeft[Y]);
		
		return series;
	}
	
	public boolean onChart(double minX, double minY, double chartMaxX, double chartMaxY) {
		if (lowerLeft[X] < minX || lowerLeft[Y] < minY)
			return false;
		if (upperRight[X] > chartMaxX || upperRight[Y] > chartMaxY)
			return false;
		return true;
	}
	
	public BoundingBox copy() {
		return new BoundingBox(meta, x,y,lowerLeft, lowerRight, upperLeft, upperRight, ptr);
	}
	
	// need a combine BBs - add to bottom of the bounding box expanding if necessary
	// expand(x,y)
	// shift(x,y)
	
	public void relocate(BoundingBox bb) {
		if (bb == null) 
			System.out.println("?");
		
		double[] shiftValues = meta.getShiftValues(getPtr());
		int xShift = (int) ((bb.getLowerLeft()[0] - lowerLeft[0]) / shiftValues[0]);
		int yShift = (int) ((bb.getLowerLeft()[1] - lowerLeft[1]) / shiftValues[1]);
		
//		System.out.println("relocate "+getPtr()+" xShift: "+xShift+" yShift: "+yShift);
		
		numShifts = 5;
		
		Coordinate a = new Coordinate(lowerLeft[X], lowerLeft[Y], meta.getMinX(), meta.getMaxX(), meta.getMinY(), meta.getMaxY());
		Coordinate b = new Coordinate(bb.getLowerLeft()[X], bb.getLowerLeft()[Y], meta.getMinX(), meta.getMaxX(), meta.getMinY(), meta.getMaxY());
		
//		System.out.println(a+" #TO# "+b);
		
		this.lowerLeft = bb.getLowerLeft();
		this.lowerRight = bb.getLowerRight();
		this.upperLeft = bb.getUpperLeft();
		this.upperRight = bb.getUpperRight();
	}
	
	public double width() {
		return this.lowerRight[X] - this.lowerLeft[X];
	}
	
	public double height() {
		return this.upperLeft[Y] - this.lowerLeft[Y];
	}
	
	public void moveTo(double newx, double newy) {
		shift(newx-x, newy-y);
	}
	
	public void add(BoundingBox bb) {
		ptrs.add(bb.getPtr());
		
		// keep x,y coords of original
		// height will always change width may not
		double xExpand = bb.height();
		double yExpand = bb.width() > width() ? bb.width() - width() : 0.0;
		expand(xExpand, yExpand);
	}
	
	public void annotationShift(double xShift, double yShift) {
		// used to move this bounding box around

		
		this.lowerLeft[X] += xShift;
		this.lowerRight[X] += xShift;
		this.upperLeft[X] += xShift;
		this.upperRight[X] += xShift;
		
		this.lowerLeft[Y] += yShift;
		this.lowerRight[Y] += yShift;
		this.upperLeft[Y] += yShift;
		this.upperRight[Y] += yShift;
		
		x += xShift;
		y += yShift;
		
	}
	
	public void shift(double xShift, double yShift) {
		if (annotationBoundingBox)
			annotationShift(xShift, yShift);
		else
			regularShift(xShift, yShift);
	}
	
	public void regularShift(double xShift, double yShift) {
		// used to move this bounding box around
		
		numShifts++;
		
		this.lowerLeft[X] += xShift;
		this.lowerRight[X] += xShift;
		this.upperLeft[X] += xShift;
		this.upperRight[X] += xShift;
		
		this.lowerLeft[Y] += yShift;
		this.lowerRight[Y] += yShift;
		this.upperLeft[Y] += yShift;
		this.upperRight[Y] += yShift;
		
		resetCandidate();
	}
	
	public int getNumShifts() {
		return numShifts;
	}

	public void resetCandidate() {
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				if (i == j)
					continue;
				meta.getCandidate()[ptr][i][j] = !this.isTooCloseToPlotEdge();
			}
		}
	}
	
	public void expand(double xExpand, double yExpand) {
		this.lowerLeft[X] += xExpand/2.0;
		this.lowerRight[X] += xExpand/2.0;
		this.upperLeft[X] += xExpand/2.0;
		this.upperRight[X] += xExpand/2.0;
		
		this.lowerLeft[Y] += yExpand/2.0;
		this.lowerRight[Y] += yExpand/2.0;
		this.upperLeft[Y] += yExpand/2.0;
		this.upperRight[Y] += yExpand/2.0;
	}
	
	public boolean isTooCloseToPlotEdge() {
		return (lowerLeft[X] < 0.0 || 
				lowerLeft[Y] < 0.0 ||
				upperRight[X] > meta.getMaxX() ||
				upperRight[Y] > meta.getMaxY());
	}
	
	
	public String toString() {
		return "ptr: "+ptr+" c ("+f(x)+","+f(y)+") "+"x ("+f(lowerLeft[X])+" -> "+f(lowerRight[X])+") "+"y ("+f(lowerRight[Y])+" -> "+f(upperRight[Y])+"), h:"+
				NumberFormatter.formatForRatio(height())+", w:"+NumberFormatter.formatForRatio(width());
	}
	
	private String f(double d) {
		return NumberFormatter.formatForSmall(d);
	}
	
	public boolean intersect(BoundingBox other) {
		boolean inter = false;
		
		if (other == null)
			return false;
		inter = cross(other);
		
//		inter = inside(other.getLowerLeft()) ||
//				inside(other.getLowerRight()) ||
//				inside(other.getUpperLeft()) ||
//				inside(other.getUpperRight())  ||
//				inside(other.getLocation()) ;
		
		return inter;
	}
	
	public boolean cross(BoundingBox other) {
		boolean cross = false;
		if (other == null)
			return false;
		
		cross = (lowerLeft[X] < other.getLowerRight()[X]
				&& other.getLowerLeft()[X]  < lowerRight[X] 
				&& lowerLeft[Y] < other.getUpperLeft()[Y] 
						&& other.getLowerLeft()[Y] < upperLeft[Y]);
		return cross;
		
	}
	
	public boolean inside(double[] point) {
		// 
		return point[X] >= lowerLeft[X] &&
				point[X] <= lowerRight[X] &&
				point[Y] >= lowerLeft[Y] &&
				point[Y] <= upperLeft[Y];
	}
	
	public boolean inside(Coordinate coord) {
		return inside(coord.asPoint());
	}
	
	public boolean intersectsWithDataPoint() {
		for (Coordinate coord : meta.getPointCoordinates().values()) {
			if (inside(coord))
				return true;
		}
		return false;
	}
	
	public boolean lineIntersectsBoundingBox(double x1, double y1, double x2, double y2) {
		if (linesIntersect(x1,y1,x2,y2,lowerLeft[X], lowerLeft[Y], upperLeft[X], upperLeft[Y]))
			return true;
		if (linesIntersect(x1,y1,x2,y2,lowerRight[X], lowerRight[Y], upperRight[X], upperRight[Y]))
			return true;
		if (linesIntersect(x1,y1,x2,y2,lowerLeft[X], lowerLeft[Y], lowerRight[X], lowerRight[Y]))
			return true;
		if (linesIntersect(x1,y1,x2,y2,upperLeft[X], upperLeft[Y], upperRight[X], upperRight[Y]))
			return true;
		return false;
	}
	
	public static boolean linesIntersect(double x1, double y1, double x2, double y2, double x3, double y3, double x4, double y4){
	      // Return false if either of the lines have zero length
	      if (x1 == x2 && y1 == y2 ||
	            x3 == x4 && y3 == y4){
	         return false;
	      }
	      // Fastest method, based on Franklin Antonio's "Faster Line Segment Intersection" topic "in Graphics Gems III" book (http://www.graphicsgems.org/)
	      double ax = x2-x1;
	      double ay = y2-y1;
	      double bx = x3-x4;
	      double by = y3-y4;
	      double cx = x1-x3;
	      double cy = y1-y3;

	      double alphaNumerator = by*cx - bx*cy;
	      double commonDenominator = ay*bx - ax*by;
	      if (commonDenominator > 0){
	         if (alphaNumerator < 0 || alphaNumerator > commonDenominator){
	            return false;
	         }
	      }else if (commonDenominator < 0){
	         if (alphaNumerator > 0 || alphaNumerator < commonDenominator){
	            return false;
	         }
	      }
	      double betaNumerator = ax*cy - ay*cx;
	      if (commonDenominator > 0){
	         if (betaNumerator < 0 || betaNumerator > commonDenominator){
	            return false;
	         }
	      }else if (commonDenominator < 0){
	         if (betaNumerator > 0 || betaNumerator < commonDenominator){
	            return false;
	         }
	      }
	      if (commonDenominator == 0){
	         // This code wasn't in Franklin Antonio's method. It was added by Keith Woodward.
	         // The lines are parallel.
	         // Check if they're collinear.
	         double y3LessY1 = y3-y1;
	         double collinearityTestForP3 = x1*(y2-y3) + x2*(y3LessY1) + x3*(y1-y2);   // see http://mathworld.wolfram.com/Collinear.html
	         // If p3 is collinear with p1 and p2 then p4 will also be collinear, since p1-p2 is parallel with p3-p4
	         if (collinearityTestForP3 == 0){
	            // The lines are collinear. Now check if they overlap.
	            if (x1 >= x3 && x1 <= x4 || x1 <= x3 && x1 >= x4 ||
	                  x2 >= x3 && x2 <= x4 || x2 <= x3 && x2 >= x4 ||
	                  x3 >= x1 && x3 <= x2 || x3 <= x1 && x3 >= x2){
	               if (y1 >= y3 && y1 <= y4 || y1 <= y3 && y1 >= y4 ||
	                     y2 >= y3 && y2 <= y4 || y2 <= y3 && y2 >= y4 ||
	                     y3 >= y1 && y3 <= y2 || y3 <= y1 && y3 >= y2){
	                  return true;
	               }
	            }
	         }
	         return false;
	      }
	      return true;
	   }
	
	

	public double[] getLowerLeft() {
		return lowerLeft;
	}

	public double[] getLowerRight() {
		return lowerRight;
	}

	public double[] getUpperLeft() {
		return upperLeft;
	}

	public double[] getUpperRight() {
		return upperRight;
	}

	public int getPtr() {
		return ptr;
	}

	public double getX() {
		return x;
	}

	public double getY() {
		return y;
	}
	
	public double[] getLocation() {
		double[] loc = new double[2];
		loc[X] = x;
		loc[Y] = y;
		return loc;
	}


	public Set<Integer> getPtrs() {
		return ptrs;
	}

	public void setPtrs(Set<Integer> ptrs) {
		this.ptrs = ptrs;
	}

	public void setLowerLeft(double[] lowerLeft) {
		this.lowerLeft = lowerLeft;
	}

	public void setLowerRight(double[] lowerRight) {
		this.lowerRight = lowerRight;
	}

	public void setUpperLeft(double[] upperLeft) {
		this.upperLeft = upperLeft;
	}

	public void setUpperRight(double[] upperRight) {
		this.upperRight = upperRight;
	}

	public void setX(double x) {
		this.x = x;
	}

	public void setY(double y) {
		this.y = y;
	}

	public void setPtr(int ptr) {
		this.ptr = ptr;
	}

	public Coordinate getCoordinate() {
		return coordinate;
	}

	public void setNumShifts(int numShifts) {
		this.numShifts = numShifts;
	}

}
