
package gov.anl.gss.visualization;

/* ---------------------
 * ScatterPlotDemo1.java
 * ---------------------
 * (C) Copyright 2002-2006, by Object Refinery Limited.
 *
 */

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYPointerAnnotation;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.TextAnchor;

/**
 * A demo scatter plot.
 */
public class SFESimpleAnnotatedScatterPlot extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	   private List<List<String>> tableData = new ArrayList<List<String>>();
	   private List<String> tableColumnHeaders = new ArrayList<String>();
	  
	 
	    private JFreeChart chart;
	    
	    private String title;
	
	
	
	
	private final static TextAnchor[][] TEXT_ANCHORS = {{TextAnchor.TOP_LEFT, TextAnchor.TOP_CENTER, TextAnchor.TOP_RIGHT},
		{TextAnchor.CENTER_LEFT, TextAnchor.CENTER, TextAnchor.CENTER_RIGHT},
		{TextAnchor.BOTTOM_LEFT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_RIGHT}};
	
	private final static TextAnchor[][] MAX_TEXT_ANCHORS = 
		{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
		{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
		{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
		};
	
	private final static double ANCHOR_OFFSET = 0.15;
	private double NO_WORRY_X;
	private double NO_WORRY_Y;
	private double NO_WORRY_FRACTION = .20;
	
	
	
	
	private static final Shape circle6 = new Ellipse2D.Double(-24.0, -24.0, 48.0, 48.0);
	private static final Shape circle = new Ellipse2D.Double(-6.0, -6.0, 12.0, 12.0);
	private static final Shape circleSmall = new Ellipse2D.Double(-2.0, -2.0, 4.0, 4.0);
	private static final BasicStroke solidThin = new BasicStroke(0.8f);
	private static final BasicStroke solidMedium = new BasicStroke(1.5f);
	private static final BasicStroke solidThick = new BasicStroke(3.0f);
	
	private static final BasicStroke dashThin = new BasicStroke(0.8f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND, 
			0.0f, new float[] {4.0f},  0.0f);
	
	private static final BasicStroke dashThick = new BasicStroke(3.0f, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_ROUND, 
			0.0f, new float[] {4.0f},  0.0f);
	
	private SFEScatterPlotDataSet sfeDataset;
	
	double maxX, maxY;
	double xTick, yTick;
	String sql;
	Map<Integer, String> seriesType;
	Map<String, Color> colorMap;
    
    public SFESimpleAnnotatedScatterPlot(String title, SFEScatterPlotDataSet sfeDataset, double maxX, double maxY, 
    		Map<Integer, String> seriesType, Map<String, Color> colorMap) {
        super(title);
        this.title = title;
        this.maxX = maxX;
        this.maxY = maxY;
        this.NO_WORRY_X = this.maxX * NO_WORRY_FRACTION;
        this.NO_WORRY_Y = this.maxY * NO_WORRY_FRACTION;
        this.seriesType = seriesType;
        this.colorMap = colorMap;

        JPanel chartPanel = createDemoPanel(sfeDataset);
        chartPanel.setPreferredSize(new java.awt.Dimension(SampleVisualization.PLOT_WIDTH, SampleVisualization.PLOT_HEIGHT));
        setContentPane(chartPanel);
    }

    private JFreeChart createChart(XYSeriesCollection dataset) {
    	String xlab = "";
    	String ylab = "";

    	JFreeChart chart = ChartFactory.createScatterPlot(title,
    			xlab, ylab, dataset, PlotOrientation.VERTICAL, false, false, false);
    	
    	chart.setBorderVisible(false);
    	chart.setBorderStroke(null);

    	XYPlot plot = (XYPlot) chart.getPlot();
    	plot.setNoDataMessage("NO DATA");
    	plot.setDomainZeroBaselineVisible(false);
    	plot.setRangeZeroBaselineVisible(false);
    	plot.setBackgroundPaint(Color.white);
    	plot.setDomainGridlinePaint(Color.white);
    	plot.setRangeGridlinePaint(Color.white);
    	
    	plot.setOutlineVisible(false);

    	XYLineAndShapeRenderer renderer
    	= (XYLineAndShapeRenderer) plot.getRenderer();

    	NextColor nc = new NextColor();
    	for (int i = 0; i < dataset.getSeriesCount()-1; i++) {
    		renderer.setSeriesOutlinePaint(i, Color.black);
    		renderer.setSeriesStroke(i, solidMedium);
    		renderer.setSeriesShape(i, circleSmall);
    		renderer.setUseOutlinePaint(true);
    		renderer.setSeriesItemLabelsVisible(i, false); // true
    		renderer.setSeriesPaint(i,  Color.black); // nc.next()
    		renderer.setSeriesVisibleInLegend(i, false);
    		renderer.setSeriesLinesVisible(i, true);
//    		renderer.setSeriesVisibleInLegend(series, visible);
    		if (seriesType.get(i) != null && seriesType.get(i).equals(SampleVisualization.CONNECTOR)) {
    			renderer.setSeriesLinesVisible(i, true);
    			renderer.setSeriesItemLabelsVisible(i, false);
    			String label = (String) dataset.getSeries(i).getKey();
    			String head = label.split("###")[1].trim();
    			Color c = colorMap.get(head);
    			renderer.setSeriesPaint(i, c);
//    			renderer.setSeriesPaint(i, Color.black);
    		}
    	}
    	
    	int i = dataset.getSeriesCount()-1;
    	renderer.setSeriesOutlinePaint(i, Color.white);
		renderer.setSeriesStroke(i, solidThin);
		renderer.setSeriesShape(i, circleSmall);
		renderer.setUseOutlinePaint(false);
		renderer.setSeriesItemLabelsVisible(i, false); // true
		renderer.setSeriesPaint(i,  Color.black); // nc.next()
		renderer.setSeriesVisibleInLegend(i, false);
		renderer.setSeriesLinesVisible(i, false);
		renderer.setSeriesPaint(i, Color.white);

    	xTick = 0.0; // calcTick(maxX);
    	yTick = 0.0; // calcTick(maxY);

    	NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();
    	domainAxis.setAutoRangeIncludesZero(true);
    	domainAxis.setTickUnit(new NumberTickUnit(xTick));
    	domainAxis.setTickMarksVisible(false);
    	domainAxis.setVisible(false);


    	NumberAxis rangeAxis = (NumberAxis) plot.getRangeAxis();
    	rangeAxis.setAutoRangeIncludesZero(true);
    	rangeAxis.setTickUnit(new NumberTickUnit(yTick));
    	rangeAxis.setTickMarksVisible(false);
    	rangeAxis.setVisible(false);

    	domainAxis.setLowerBound(0.0);
    	
    	maxX = maxX+1;

    	rangeAxis.setLowerBound(0.0);
    	rangeAxis.setUpperBound(maxY+yTick);
    	maxY = maxY+yTick;

    	annotateDataPoints(plot, dataset);

    	return chart;
    }
    
    private double calcTick(double range) {
    	double tick = 0.0;
    	
    	 if (range <= 10)
         	tick = 1.0;
    	 if (range < 5)
    		 tick = .5;
    	 if (range < 2)
    		 tick = .25;
        if (range > 10)
        	tick = 2.0;
        if (range > 20)
        	tick = 5.0;
        if (range > 50)
        	tick = 10;
        if (range > 100)
        	tick = 20;
        if (range > 200)
        	tick = 25;
       
        return tick;
    }
    
    private void annotateDataPoints(XYPlot plot, XYSeriesCollection dataset) {
    	
//    	double[] xValues = new double[dataset.getSeriesCount()];
//    	double[] yValues = new double[dataset.getSeriesCount()];
    	
    	SFEXYPointerAnnotationMetaData meta = new SFEXYPointerAnnotationMetaData(dataset.getSeriesCount(), 0.0, maxX, 0.0, maxY,
    			900.0);
    	
    	int ptr = 0;
//    	for (Object series : dataset.getSeries()) {
    		for (int i = 0; i < dataset.getSeriesCount(); i++) {
    		XYSeries s = dataset.getSeries(i);
    		double x = s.getX(0).doubleValue();
    		double y = s.getY(0).doubleValue();
    		
    		if (seriesType.get(i).equals(SampleVisualization.DATAPOINT_ROW) || 
    				seriesType.get(i).equals(SampleVisualization.DATAPOINT_COL))
    			meta.addAnnotation((String) s.getKey(), x, y);
    		else
    			meta.addAnnotation("", x, y);
    		ptr++;
    	}
    	
    	meta.analyzeLayout();
    	boolean annotate = true;
    	if (meta.percentNoPlaceForText() > 100.0 )
    		annotate = false;

    	ptr = 0;
    	for (int i = 0; i < dataset.getSeriesCount(); i++) {
    		XYSeries s = dataset.getSeries(i);
    		
    		String key = (String) s.getKey();
    		Number x = s.getX(0);
    		Number y = s.getY(0);
    		
//    		TextAnchor anchor = determineTextAnchorPoint(key, key.length(), ptr, xValues, yValues);
    		TextAnchor anchor = meta.determineTextAnchorPoint(ptr);
    		if (anchor == null) {
    			System.out.println("No Place: "+key+" intersects: ");
    			printIntersectors(meta, ptr);
    		}
    		
    		if (seriesType.get(i).equals(SampleVisualization.DATAPOINT_COL)) {
    			anchor = TextAnchor.CENTER_LEFT;
    		} else {
    			anchor = TextAnchor.CENTER_RIGHT;
    		}
    		
    		
    		
    		if (annotate && anchor != null && ptr <= dataset.getSeriesCount() &&  
    				(seriesType.get(i).equals(SampleVisualization.DATAPOINT_ROW) || 
    						seriesType.get(i).equals(SampleVisualization.DATAPOINT_COL)) &&
    				!key.equals("XXX")) {
    			System.out.println("ANNOT: "+key+" "+ptr+" "+anchor);
    			plot.addAnnotation(createAnnotation(key, x.doubleValue(), y.doubleValue(), anchor));
    		}
    		ptr++;
    	}
    }
    

    private void printIntersectors(SFEXYPointerAnnotationMetaData meta, int ptr) {
    	for (String key : meta.getIntersectors().keySet()) {
    		if (key.startsWith(ptr+"#")) {
    			System.out.println("   "+key+" "+meta.getIntersectors().get(key));
    		}
    	}
    }

    
    private List<String> split(String key) {
    	
    	int MAXLEN = 25;
    	List<String> multi = new ArrayList<String>();
    	
    	StringBuilder sb = new StringBuilder();
    	String[] words = key.split(" ");
    	for (String word : words) {
    		if (sb.length() + word.length()+1 >= MAXLEN) {
    			// doesn't fit, flush builder and start
    			multi.add(sb.toString());
    			sb = new StringBuilder();
    			sb.append(word);
    		} else {
    			if (sb.length() > 0)
    				sb.append(" ");
    			sb.append(word);
    		}
    	}
    	
    	if (sb.length() > 0)
    		multi.add(sb.toString());
    	return multi;
    }
    
    private double yOffset(TextAnchor ta) {
    	if (ta == TextAnchor.BOTTOM_CENTER || ta == TextAnchor.BOTTOM_LEFT || ta == TextAnchor.BOTTOM_RIGHT) {
    		return ANCHOR_OFFSET*yTick;
    	} else if (ta == TextAnchor.TOP_CENTER || ta == TextAnchor.TOP_LEFT || ta == TextAnchor.TOP_RIGHT) {
    		return -ANCHOR_OFFSET*yTick;
    	} else if (ta == TextAnchor.CENTER_LEFT) {
    		return 0.0;
    	} else if (ta == TextAnchor.CENTER_RIGHT) {
    		return 0.0;
    	} else {
    		return 0.0;
    	}
    }
    
    private double xOffset(TextAnchor ta) {
    	if (ta == TextAnchor.BOTTOM_CENTER || ta == TextAnchor.BOTTOM_LEFT || ta == TextAnchor.BOTTOM_RIGHT) {
    		return 0.0;
    	} else if (ta == TextAnchor.TOP_CENTER || ta == TextAnchor.TOP_LEFT || ta == TextAnchor.TOP_RIGHT) {
    		return 0.0;
    	} else if (ta == TextAnchor.CENTER_LEFT) {
    		return ANCHOR_OFFSET*xTick;
    	} else if (ta == TextAnchor.CENTER_RIGHT) {
    		return -ANCHOR_OFFSET*xTick;
    	} else {
    		return 0.0;
    	}
    }
    
    private TextAnchor determineTextAnchorPoint(String text, int textLength, int ptr, double[] x, double[] y) {
    	
    	boolean shortLength = textLength <= 20;
    	boolean longLength = !shortLength;
    	
 
    	// allocate computation space
    	double[] xDiff = new double[x.length];
    	double[] yDiff = new double[y.length];
    	
    	// calculate difference in x,y coordinates from ptr to all points
    	for (int i = 0; i < x.length; i++) {
    		xDiff[i] = x[i] - x[ptr];
    		yDiff[i] = y[i] - y[ptr];
    	}
    	

    	double[][] minDistance = new double[][]{{Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE},
    			{Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE},{Double.MAX_VALUE, Double.MAX_VALUE, Double.MAX_VALUE}};
    	System.out.println("\n###Data Point "+x[ptr]+","+y[ptr]+" "+text);
    	
    	for (int i = 0; i < x.length; i++) {
    		int xoff = xDiff[i] < 0.0 ? -1 : xDiff[i] > 0.0 ? 1 : 0;
    		int yoff = yDiff[i] < 0.0 ? 1 : yDiff[i] > 0.0 ? -1 : 0;
    		double dist = distance(x[i], y[i], x[ptr], y[ptr]);
    		
//    		System.out.println(" offset: "+x[i]+"-"+x[ptr]+"="+xDiff[i]+" "+y[i]+"-"+y[ptr]+"="+yDiff[i]+" -> "+xoff+","+yoff);
    		
    		if (dist < minDistance[1+yoff][1+xoff]) {
    			minDistance[1+yoff][1+xoff] = dist;
    		}
    	}
    	
    	
//    	for (int i = 0; i < 3; i++) {
//    		for (int j = 0; j < 3; j++) {
//    			if (minDistance[i][j] == Double.MAX_VALUE)
//    				System.out.print("  XXX");
//    			else
//    				System.out.print("  "+minDistance[i][j]);
//    		}
//    		System.out.print("\n");;
//    	}
    	
    	int iMin = 0;
    	int jMin = 0;
    	double minD = Double.MAX_VALUE;
    	
    	int iMax = 0;
    	int jMax = 0;
    	double maxD = Double.MIN_VALUE;
    	
    	for (int i = 0; i < 3; i++) {
    		for (int j = 0; j < 3; j++) {
    			if (minDistance[i][j] != 0.0 && minDistance[i][j] < minD) {
    				minD = minDistance[i][j];
    				iMin = i;
    				jMin = j;
    			}
    		}
    		
    	}
    	
    	for (int i = 0; i < 3; i++) {
    		for (int j = 0; j < 3; j++) {
    			if (minDistance[i][j] !=Double.MAX_VALUE && minDistance[i][j] > maxD) {
    				maxD = minDistance[i][j];
    				iMax = i;
    				jMax = j;
    			}
    		}
    	}
    	

    	
    	
    	
//    	if (minD > NO_WORRY) {
//    		System.out.println("NO WORRY TEXT ANCHOR["+iMin+"]["+jMin+"] Dist = "+minD);
//
//    		return TextAnchor.CENTER_LEFT;
//    	}
    	
//    	System.out.println("TEXT ANCHOR["+iMin+"]["+jMin+"] Dist = "+minD);
    	System.out.println("TEXT ANCHOR["+iMax+"]["+jMax+"] Dist = "+maxD);
    	
//    	private final static TextAnchor[][] MAX_TEXT_ANCHORS = 
//    		{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
//    		{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
//    		{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
//    		};
    	
//    	return TEXT_ANCHORS[iMin][jMin];
//    	return MAX_TEXT_ANCHORS[iMax][jMax];
    	return  MAX_TEXT_ANCHORS[0][1];
    }
    
    private static double distance(double x1, double y1, double x2, double y2) {
    	return Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
    }
    
    private void addAnnotatedCircle(XYPlot plot, XYSeriesCollection dataset, String label, double x, double y) {
    	
    	 plot.addAnnotation(createAnnotation(label, x, y));
    	
    	 XYLineAndShapeRenderer renderer
         = (XYLineAndShapeRenderer) plot.getRenderer();
    	 
         
         XYSeries series = new XYSeries(label);
         series.add(x, y);
         
         dataset.addSeries(series);
        
        int c1 = dataset.getSeriesCount()-1;
        renderer.setSeriesShape(c1, circle6);
        renderer.setSeriesOutlineStroke(c1, solidThin);
        renderer.setSeriesShapesFilled(c1, true);
        renderer.setSeriesPaint(c1, Color.lightGray);
        renderer.setSeriesItemLabelsVisible(c1, true);
    	renderer.setSeriesShapesVisible(c1, true);
    }
   
    /**
     * Creates a panel for the demo (used by SuperDemo.java).
     *
     * @return A panel.
     */
    public JPanel createDemoPanel(SFEScatterPlotDataSet sfeDataset) {
    	chart = createChart(createScatterData(sfeDataset));
    	return restOfChart(chart);
    }
    
    public JPanel restOfChart(JFreeChart chart) {
        
        ChartPanel chartPanel = new ChartPanel(chart);
        //chartPanel.setVerticalAxisTrace(true);
        //chartPanel.setHorizontalAxisTrace(true);
        // popup menu conflicts with axis trace
        chartPanel.setPopupMenu(null);
       
        chartPanel.setDomainZoomable(true);
        chartPanel.setRangeZoomable(true);
        
        JPanel p = new JPanel();
        p.setLayout(new BoxLayout(p, BoxLayout.LINE_AXIS));
        
//        JPanel annot = new ArrowAnnotationPanel(800);
//        annot.setPreferredSize(new java.awt.Dimension(40, 800));
//        
//        p.add(annot);
//        
//        JPanel arrow = new ArrowPanel(750);
//        arrow.setPreferredSize(new java.awt.Dimension(60, 800));
//       
//        
//        
//        p.add(arrow);
        
        chartPanel.setPreferredSize(new java.awt.Dimension(SampleVisualization.PLOT_WIDTH, SampleVisualization.PLOT_HEIGHT));
        p.add(chartPanel);
        return p;
    }
   

    
    private static XYSeriesCollection  createScatterData(SFEScatterPlotDataSet sfeDataset) {

        return sfeDataset.getScatterData();
    }
    
    private double computeAngle(TextAnchor anchor) {

    	if (anchor == TextAnchor.BOTTOM_LEFT) return 1.75 * Math.PI; 
    	if (anchor == TextAnchor.BOTTOM_CENTER) return 1.5 * Math.PI;  
    	if (anchor == TextAnchor.BOTTOM_RIGHT) return 1.25 * Math.PI; 
    	if (anchor == TextAnchor.CENTER_LEFT) return 0.0 * Math.PI; 
    	if (anchor == TextAnchor.CENTER) return 1.75 * Math.PI;  
    	if (anchor == TextAnchor.CENTER_RIGHT) return Math.PI; 
    	if (anchor == TextAnchor.TOP_LEFT) return 0.25 * Math.PI; 
    	if (anchor == TextAnchor.TOP_CENTER) return 0.50 * Math.PI; 
    	if (anchor == TextAnchor.TOP_RIGHT) return 0.75 * Math.PI; 
    	return 0.0;
    }
    
    private XYPointerAnnotation createAnnotation(String text, double x, double y, TextAnchor anchor) {
    	XYPointerAnnotation annotation;
        Font font = new Font("Arial", Font.PLAIN, SampleVisualization.FONT_SIZE);

        annotation = new XYPointerAnnotation(text, x, y, computeAngle(anchor));
        annotation.setFont(font);
        annotation.setTextAnchor(anchor);
        
        annotation.setTipRadius(10.0);
        annotation.setBaseRadius(30.0);
        annotation.setPaint(Color.black);
        annotation.setArrowLength(0.0);
        
        annotation.setLabelOffset(5.0);

       return annotation;
    }
    
    private  XYTextAnnotation createAnnotation(StringAnnotation stringAnnotation, double x, double y, TextAnchor anchor) {
    	XYTextAnnotation annotation = null;
        Font font = new Font("Arial", Font.PLAIN, SampleVisualization.FONT_SIZE);

        for (String text : stringAnnotation.getText()) {
        annotation = new XYTextAnnotation(text, x, y);
        annotation.setFont(font);
        annotation.setTextAnchor(anchor);
        }
       return annotation;
    }
    
    private  XYTextAnnotation createAnnotation(String text, double x, double y) {
    	XYTextAnnotation annotation;
        Font font = new Font("Arial", Font.PLAIN, 18);

        annotation = new XYTextAnnotation(text, x, y);
        annotation.setFont(font);
        annotation.setTextAnchor(TextAnchor.CENTER);
       return annotation;
    }

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public List<List<String>> getTableData() {
		return tableData;
	}

	public List<String> getTableColumnHeaders() {
		return tableColumnHeaders;
	}

	public JFreeChart getChart() {
		return chart;
	}

	public static TextAnchor[][] getTextAnchors() {
		return TEXT_ANCHORS;
	}

	public static TextAnchor[][] getMaxTextAnchors() {
		return MAX_TEXT_ANCHORS;
	}

	public static double getAnchorOffset() {
		return ANCHOR_OFFSET;
	}

	public double getNO_WORRY_X() {
		return NO_WORRY_X;
	}

	public double getNO_WORRY_Y() {
		return NO_WORRY_Y;
	}

	public double getNO_WORRY_FRACTION() {
		return NO_WORRY_FRACTION;
	}

	public static Shape getCircle6() {
		return circle6;
	}

	public static Shape getCircle() {
		return circle;
	}
	
	public static Shape getCircleSmall() {
		return circleSmall;
	}

	public static BasicStroke getSolidthin() {
		return solidThin;
	}

	public static BasicStroke getSolidthick() {
		return solidThick;
	}

	public static BasicStroke getDashthin() {
		return dashThin;
	}

	public static BasicStroke getDashthick() {
		return dashThick;
	}

	public SFEScatterPlotDataSet getSfeDataset() {
		return sfeDataset;
	}

	public double getMaxX() {
		return maxX;
	}

	public double getMaxY() {
		return maxY;
	}

	public double getxTick() {
		return xTick;
	}

	public double getyTick() {
		return yTick;
	}

	public String getSql() {
		return sql;
	}

	public void setTableData(List<List<String>> tableData) {
		this.tableData = tableData;
	}

	public void setTableColumnHeaders(List<String> tableColumnHeaders) {
		this.tableColumnHeaders = tableColumnHeaders;
	}


}


