package gov.anl.gss.visualization;

import gov.anl.gss.sdm.server.model.analysis.micmac.Annotation;
import gov.anl.gss.sdm.server.model.analysis.micmac.CandidateEvaluation;
import gov.anl.gss.sdm.server.model.analysis.micmac.CandidateLocation;
import gov.anl.gss.sdm.server.model.analysis.micmac.DataPoint;
import gov.anl.gss.sdm.server.model.analysis.micmac.DataPointProximity;
import gov.anl.gss.sdm.server.model.analysis.micmac.DataPointProximitySummary;
import gov.anl.gss.sdm.server.model.analysis.micmac.DistanceDensity;
import gov.anl.gss.sdm.server.model.analysis.micmac.MicMacScatterPlot;
import gov.anl.gss.sdm.server.model.analysis.micmac.MinMaxPair;
import gov.anl.gss.sdm.server.model.analysis.micmac.SlotIterator;
import gov.anl.gss.sdm.server.model.translator.StaticSwitches;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang3.mutable.MutableDouble;
import org.jfree.ui.TextAnchor;

public class SFEXYPointerAnnotationMetaData {

	private static  double CLOSE = 0.01; // 0.001; // 0.15; // 0.15 0.001
	private static final double SIZE = 1200;
	private final static String EDGE = "-1#-1#-1";
	
	private static final double TOO_CLOSE = -1.0;
	private static final int TOO_MANY = Integer.MAX_VALUE;
	
	private static final double SLICE = Math.PI/8;
	
	private List<MinMaxPair> requiresZoom = new ArrayList<MinMaxPair>();
	private Set<String> zooms = new LinkedHashSet<String>();
	

	private final static TextAnchor[][] TEXT_ANCHORS = 
		{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
		{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
		{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
		};
	
	private final static TextAnchor[] SLOT_ANCHORS = {
		TextAnchor.CENTER_LEFT, TextAnchor.CENTER_LEFT, TextAnchor.CENTER_LEFT, TextAnchor.BOTTOM_CENTER,
		TextAnchor.BOTTOM_CENTER, TextAnchor.CENTER_RIGHT, TextAnchor.CENTER_RIGHT, TextAnchor.CENTER_RIGHT,
		TextAnchor.CENTER_LEFT, TextAnchor.CENTER_LEFT, TextAnchor.CENTER_LEFT, TextAnchor.TOP_CENTER,
		TextAnchor.TOP_CENTER, TextAnchor.CENTER_RIGHT, TextAnchor.CENTER_RIGHT, TextAnchor.CENTER_RIGHT
	};
	
//	private final static TextAnchor[][] TEXT_ANCHORS = {
//		{TextAnchor.TOP_LEFT, TextAnchor.TOP_CENTER, TextAnchor.TOP_RIGHT},
//		{TextAnchor.CENTER_LEFT, TextAnchor.CENTER, TextAnchor.CENTER_RIGHT},
//		{TextAnchor.BOTTOM_LEFT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_RIGHT}
//		};

	// these are values as of May 16 -- 10:00pm
//	public final static double HEIGTH_FACTOR = 45.0; // .07
//	public final static double WIDTH_FACTOR = 6.0; // 7.0 5.0 10.0 0.15
//	public final static double ARROW_OFFSET = 0.0001; // 0.02;
	
	// try reducing to see what this does
	// 20, 3 works better but may be too small
	// 25, 4 looks better try 25, 5
	// 25, 5 look good
	
	double HEIGHT_FACTOR = 5.0; // 45.0; // 45.0; // 30.0; // 25, 35, 45
	double WIDTH_FACTOR = 4.7; //  5.0
	public final static double ARROW_OFFSET = 0.0001; // .0001
	
	public final static double BASE_DISTANCE = 20.0;
	
	double DISTANCE_ADJUSTMENT;

	Map<String, Coordinate> annotations = new HashMap<String, Coordinate>();
	Map<Integer, String> annotationIndex = new HashMap<Integer, String>();
	Set<Integer> noPlace = new LinkedHashSet<Integer>();
	
	Map<Integer, List<CandidateLocation>> candidateLocations = new HashMap<Integer, List<CandidateLocation>>();

	private TextAnchor[] selectedAnchor;

	int numAnnotations;

	private double maxX;
	private double maxY;
	private double minX;
	private double minY;
	private double plotMaxX;
	private double plotMaxY;
	private double rangeX;
	private double rangeY;

	private double[] x;
	private double[] y;

	private boolean[][][] candidate;
	
	private int[] iteration;
	
	private BoundingBox[][][] boundingBoxes;
	private BoundingBox[] selectedBoundingBoxes;
	
	private Set<String> usedBoundingBoxes = new LinkedHashSet<String>();
	private Set<BoundingBox> usedAnnotionSpace = new LinkedHashSet<BoundingBox>();
	
	public Set<BoundingBox> getUsedAnnotionSpace() {
		return usedAnnotionSpace;
	}

	private Map<String, Set<String>> intersectors = new HashMap<String, Set<String>>();
	private Map<Integer, Set<Integer>> colocated = new HashMap<Integer, Set<Integer>>();

	int currentAnnotation = 0;
	
	int[][] pointCount = new int[6][6];
	HashSet<Coordinate>[][] pointCoords = new LinkedHashSet[6][6];
	Coordinate[][] centroids = new Coordinate[6][6];
	List<DataPoint> dataPoints = new ArrayList<DataPoint>();
	
	private Map<Integer, Map<Integer, DataPointProximity>> proximity;
	private Map<Integer, List<DataPointProximity>> proximityList;
	private Map<Integer, DataPointProximitySummary> proximitySummaryList;
	private Map<Integer, Coordinate> pointCoordinates;

	public SFEXYPointerAnnotationMetaData(int numAnnotations, double minX, double maxX, double minY, double maxY,
			double distanceAdjustment) {
		this.numAnnotations = numAnnotations;
		this.x = new double[numAnnotations];
		this.y = new double[numAnnotations];
		this.maxX = maxX;
		this.maxY = maxY;
		this.minY = minY;
		this.minX = minX;
		rangeX = maxX - minX;
		rangeY = maxY - minY;
		plotMaxX = maxX + rangeX/5.0;
		plotMaxY = maxY + rangeY/5.0;
		
		this.DISTANCE_ADJUSTMENT = distanceAdjustment;
		
		WIDTH_FACTOR = rangeX/(175.0*MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR);     // ie room for 175 characters decrease denominator to increase width 175 is start
		HEIGHT_FACTOR = rangeX / (30.0*MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR);   // ie room for 50 
		
		candidate = new boolean[numAnnotations][3][3];
		boundingBoxes = new BoundingBox[numAnnotations][3][3];
		selectedBoundingBoxes = new BoundingBox[numAnnotations];
		selectedAnchor = new TextAnchor[numAnnotations];
		iteration = new int[numAnnotations];
		pointCoordinates = new HashMap<Integer, Coordinate>();
		proximity = new HashMap<Integer, Map<Integer, DataPointProximity>>();
		proximityList = new HashMap<Integer, List<DataPointProximity>>();
		proximitySummaryList = new HashMap<Integer, DataPointProximitySummary>();
		initializeCentroids();
		
		double narrow = rangeX * 0.01;
		
		// mark dashed line as annotation
		usedAnnotionSpace.add(new BoundingBox(this, minX, maxY/2.0, narrow, maxY, -1));
		usedAnnotionSpace.add(new BoundingBox(this, maxX, maxY/2.0, narrow, maxY, -2));
		usedAnnotionSpace.add(new BoundingBox(this, maxX/2, minY, maxX , narrow, -3));
		usedAnnotionSpace.add(new BoundingBox(this, maxX/2, maxY, maxX, narrow, -4));
		
		for (int i = 0; i < 6; i++) {
			for (int j = 0; j < 6; j++) {
				pointCoords[i][j] = new LinkedHashSet<Coordinate>();
			}
		}
	}

	public void addAnnotation(String text, double x, double y) {
		this.x[currentAnnotation] = x;
		this.y[currentAnnotation] = y;
		annotationIndex.put(currentAnnotation, text);
		Coordinate coord = new Coordinate(x,y, minX, maxX, minY, maxY);
		usedAnnotionSpace.add(new BoundingBox(this, x, y, getWidth("xx"), getHeight("xx")*.5, -1*(currentAnnotation+10)));
		annotations.put(text, coord);
		pointCoordinates.put(currentAnnotation, coord);
		dataPoints.add(new DataPoint(currentAnnotation, x ,y));
		int coordI = coord.getIndexI();
		int coordJ = coord.getIndexJ();
		pointCount[coordI][coordJ]++;
		pointCoords[coordI][coordJ].add(coord);
		currentAnnotation++;
	}
	
	public List<DataPoint> getOrderedPoints() {
		Collections.sort(dataPoints);
		return dataPoints;
	}
	

	
	public List<CandidateLocation> getLocation(int ptr) {
		return candidateLocations.get(ptr);
	}
	
	public TextAnchor getSlotAnchor(int slot) {
		return SLOT_ANCHORS[slot];
	}

	public void analyzeLayout() {
		
		initializeColocated();
		
		// collect information on a data point and all other data points in the model

		for (int ptr1 = 0; ptr1 < numAnnotations; ptr1++) {
			proximity.put(ptr1, new HashMap<Integer, DataPointProximity>());
			proximityList.put(ptr1, new ArrayList<DataPointProximity>());
			for (int ptr2 = 0; ptr2 < numAnnotations; ptr2++) {
				if (ptr1 == ptr2)
					continue;
				double angle = computeAngle(pointCoordinates.get(ptr1), pointCoordinates.get(ptr2));
				double distance = computeDistance(pointCoordinates.get(ptr1), pointCoordinates.get(ptr2));
				DataPointProximity dpp = new DataPointProximity(ptr1, ptr2, distance, angle);
				proximity.get(ptr1).put(ptr2, dpp);
				proximityList.get(ptr1).add(dpp);
				
//				System.out.println(pointCoordinates.get(ptr1)+" "+pointCoordinates.get(ptr2)+" -> "+angle+" <- "+angle2);
			}
		}
		
		analyzeDensity();
		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			DataPointProximitySummary dpps = analyzeProximity(ptr);
			proximityList.put(ptr, dpps.getOrdered());
			proximitySummaryList.put(ptr,  dpps);
		}
		
		// this sort orders the ptr - should we do "hard" ones or easy? select those with fewest options
		
		List<DataPointProximitySummary> dppsList = new ArrayList<DataPointProximitySummary>();
		dppsList.addAll(proximitySummaryList.values());
		Collections.sort(dppsList);
		
		// pass1
		
		for (int i = 0; i < numAnnotations; i++) {
			int ptr = dppsList.get(i).getPtr();
			CandidateEvaluation[] evals = evaluateAllOption(ptr, dppsList.get(i));
			candidateLocations.put(ptr, chooseLocations(ptr, proximityList.get(ptr), evals));
		}

	}
	
	private int[] adjustSlotCountForBoundaries(int ptr, int[] counts, double[] slotDistance) {
		int[] adjCounts = Arrays.copyOf(counts, counts.length);
		
		if (pointCoordinates.get(ptr).getX() == 0.0 || pointCoordinates.get(ptr).getX() == 1.0) {
			// don't let it use up or down
			adjCounts[3] = TOO_MANY;
			adjCounts[4] = TOO_MANY;
			adjCounts[11] = TOO_MANY;
			adjCounts[12] = TOO_MANY;
		}
			

		if (closeToTopSide(ptr)) {
			for (int s = 0; s < 8; s++) {
				adjCounts[s] = TOO_MANY;
			}
		}
		if (closeToBottomSide(ptr)) {
			for (int s = 8; s < 16; s++) {
				adjCounts[s] = TOO_MANY;
				slotDistance[s] = TOO_CLOSE;
			}
			if (pointCoordinates.get(ptr).getY() == 0.0) {
				adjCounts[7] = TOO_MANY;
				adjCounts[0] = TOO_MANY;
				slotDistance[7] = TOO_CLOSE;
				slotDistance[0] = TOO_CLOSE;
			}
		
		}
		if (closeToLeftSide(ptr)) {
			for (int s = 4; s < 8; s++) {
				adjCounts[s] = TOO_MANY;
				slotDistance[s] = TOO_CLOSE;
			}
			for (int s = 12; s < 15; s++) {
				adjCounts[s] = TOO_MANY;
				slotDistance[s] = TOO_CLOSE;
			}
		}
		if (closeToRightSide(ptr)) {
			for (int s = 0; s < 4; s++) {
				adjCounts[s] = TOO_MANY;
				slotDistance[s] = TOO_CLOSE;
			}
			for (int s = 8; s < 12; s++) {
				adjCounts[s] = TOO_MANY;
				slotDistance[s] = TOO_CLOSE;
			}
		}

		return adjCounts;
	}
	
	private double[][] computeSurroundingDensity(Coordinate coord) {
		double[][] density = new double[3][3];
		
		int myI = coord.getIndexI();
		int myJ = coord.getIndexJ();
		density[1][1] = ((double) pointCount[myI][myJ]) / numAnnotations;
		
		for (int iShift = -1; iShift < 2; iShift++) {
			for (int jShift = -1; jShift < 2; jShift++) {
				if (iShift == 0 && jShift == 0) {
					continue;
				}
					if (myI + iShift < 0 || myI + iShift > 5 ||
							myJ + jShift < 0 || myJ + jShift > 5) {
						density[iShift+1][jShift+1] = 100.0;
					} else {
						density[iShift+1][jShift+1] = ((double) pointCount[myI+iShift][myJ+jShift]) / numAnnotations;
					}
				}
			}
		
		return density;
		
	}

	double[] getLeftToRightDensity(double[][] density) {
		double[] ltr = new double[3];
		ltr[0] = density[0][0] + density[1][0] + density[2][0];
		ltr[1] = density[0][1] + density[1][1] + density[2][1];
		ltr[2] = density[0][2] + density[1][2] + density[2][2];
		
		for (int i = 0; i < 3; i++) {
			while (ltr[i] >= 100.0) {
				ltr[i] -= 100.0;
			}
			if (ltr[i] == 0.0)
				ltr[i] = .001;
		}

		return ltr;
	}
	double[] getTopToBottomDensity(double[][] density) {
		double[] ttb = new double[3];
		ttb[0] = density[0][0] + density[0][1] + density[0][2];
		ttb[1] = density[1][0] + density[1][1] + density[1][2];
		ttb[2] = density[2][0] + density[2][1] + density[2][2];

		for (int i = 0; i < 3; i++) {
			while (ttb[i] >= 100.0) {
				ttb[i] -= 100.0;
			}
			if (ttb[i] == 0.0)
				ttb[i] = .001;
		}
		return ttb;
	}
	
	private CandidateEvaluation[] evaluateAllOption(int ptr, DataPointProximitySummary dppSummary) {
		
		CandidateEvaluation[] evals = new CandidateEvaluation[16];
		Arrays.fill(evals, null);
		
		int[] slotCount = dppSummary.getCount();
		double[] slotDistance = dppSummary.getClosest();
		
		Coordinate coord = pointCoordinates.get(ptr);
		
		double[][] density = computeSurroundingDensity(coord);
		double[] leftToRightDensity = getLeftToRightDensity(density);
		double[] topToBottomDensity = getTopToBottomDensity(density);
		

		double startSize = 45.0 * MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR;
		
		
		for (int s = 0; s < evals.length; s++) {
			evals[s] = new CandidateEvaluation(null, this);
			evals[s].setLeftToRightDensity(getLeftToRightDensity(leftToRightDensity, s));
			evals[s].setTopToBottomDensity(getTopToBottomDensity(topToBottomDensity, s));
		} 

		slotCount = adjustSlotCountForBoundaries(ptr, slotCount, slotDistance);
				
		for (DataPointProximity dpp : dppSummary.getOrdered()) {
			int slot = dpp.getSlot();
			double neighborDensity = getNeighborDensity(density, slot);
			CandidateLocation cl = new CandidateLocation(ptr, pointCoordinates.get(ptr).getX(),
					pointCoordinates.get(ptr).getY(), slot, startSize, getReverseSlotAngle(slot) /*getSlotAngle(slot)*/);
			CandidateEvaluation eval = evals[slot];
			eval.setCandidateLocation(cl);

			if (slotCount[slot] == TOO_MANY ||
					slotDistance[slot] == TOO_CLOSE) {
				eval.setScore(-1.0);
			} else if (slotCount[slot] == 0) {
				eval.setScore((100.0-getEdgeAdjustment(cl))/(neighborDensity == 0.0 ? 0.01 : neighborDensity));
			} else {
				for (double d = 0.1; d < 1.0; d += 0.1) {
					if (slotDistance[slot] <= d) {
						eval.setScore(((100.0-getEdgeAdjustment(cl)) * (1.0 - d)) / (neighborDensity == 0.0 ? 0.1 : neighborDensity));
						break;
					}
					eval.setScore(0.5/(neighborDensity == 0.0 ? 0.1 : neighborDensity));
				}
			}
			evals[slot] = eval;
			eval.getCandidateLocation().setScore(eval.getScore());

		}
		
		// these are slots in which no other points are found
		// these should have highest values
		
		for (int s = 0; s < evals.length; s++) {
			if (evals[s].getCandidateLocation() == null) {
				CandidateLocation cl = new CandidateLocation(ptr, pointCoordinates.get(ptr).getX(),
						pointCoordinates.get(ptr).getY(), s, startSize, getReverseSlotAngle(s));
				CandidateEvaluation eval = evals[s];
				eval.setCandidateLocation(cl);
				double neighborDensity = getNeighborDensity(density, s);
				if (slotCount[s] == 0) {
					eval.setScore((200.0-getEdgeAdjustment(cl))/(neighborDensity == 0.0 ? 0.01 : neighborDensity));
				} else {
					if (slotCount[s] != TOO_MANY)
						System.out.println("How?");
				}
				eval.getCandidateLocation().setScore(eval.getScore());
			}
		}
		return evals;
	}
	
	private double getEdgeAdjustment(CandidateLocation cl) {
		double edgeAdjustment = 0.0;
		DataPoint p = cl.getLocationPoint(DISTANCE_ADJUSTMENT);
		if (p.getX() > maxX || p.getY() > maxY)
			edgeAdjustment = 10.0;
		return edgeAdjustment;
	}
	

	private double getNeighborDensity(double[][] density,  int slot) {
		
		switch (slot) {
		case 0:
		case 8:
			return density[1][2];
		case 9:
		case 10:
			return density[2][2];
		case 11:
		case 12:
			return density[2][1];
		case 13:
		case 14:
			return density[2][0];
		case 15:
		case 7:
			return density[1][0];
		case 5:
		case 6:
			return density[0][0];
		case 4:
		case 3:
			return density[0][1];
		case 2:
		case 1:
			return density[0][2];
		}
		return 100.0;
		


	}
	
	private double getTopToBottomDensity(double[] topToBottomDensity, int slot) {
		switch (slot) {
		case 0:
		case 1:
		case 8:
		case 9:
			return topToBottomDensity[1];
		case 10:
		case 11:
		case 12:
		case 13:
		
			return topToBottomDensity[2];
		case 6:
		case 7:
		case 14:
		case 15:
			return topToBottomDensity[1];
		case 5:
		case 4:
		case 3:
		case 2:
		
			return topToBottomDensity[0];
		}
		return 100.0;
	}
	
	private double getLeftToRightDensity(double[] leftToRightDensity, int slot) {
		switch (slot) {
		
		case 8:
		case 9:
		case 10:
			return leftToRightDensity[2];
		case 11:
		case 12:
			return leftToRightDensity[1];
		case 13:
		case 14:
		case 15:
			return leftToRightDensity[0];
		case 0:
		case 1:
		case 2:
			return leftToRightDensity[2];
		case 3:
		case 4:
			return leftToRightDensity[1];
		case 5:
		case 6:
		case 7:
		
			return leftToRightDensity[0];
		}
		return 100.0;
	}
	
	public List<MinMaxPair> getAllZoom() {
		List<MinMaxPair> l = new ArrayList<MinMaxPair>(requiresZoom);
		requiresZoom.clear();
		return l;
	}
	
	public MinMaxPair getNextZoom() {
		if (requiresZoom.size() == 0)
			return null;
		MinMaxPair next = requiresZoom.get(0);
		requiresZoom.remove(0);
		return next;
	}
	
	private void saveForZoomedGraph(int i, int j) {
		String pair = i+"#"+j;
		if (zooms.contains(pair))
			return;
		zooms.add(pair);
		requiresZoom.add(convertToMinMaxPair(i,j));
	}
	
	private MinMaxPair convertToMinMaxPair(int i, int j) {
		// i is y value with 0 at top -- needs adjustment
		// j is x value left to right
		// range defines the size of a single cell
		double tick = rangeX / 5;
		double yMin = (5-i) * tick;
		double yMax = yMin + tick;
		
		double xMin = j * tick;
		double xMax = xMin + tick;
		
		return new MinMaxPair(xMin, xMax, yMin, yMax);
	}
	
	private List<CandidateLocation> chooseLocations(int ptr, List<DataPointProximity> dppList, CandidateEvaluation[] evals) {
		
		// ArrayList<Element> arrayList = new ArrayList<Element>(Arrays.asList(array));
		List<CandidateLocation> possibleLocations = new ArrayList<CandidateLocation>();
		
		Coordinate coord = pointCoordinates.get(ptr);
		int myI = coord.getIndexI();
		int myJ = coord.getIndexJ();
		double myDensity = ((double) pointCount[myI][myJ]) / numAnnotations;
		double startSize = 45.0 * MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR; // Math.max(45, 10.0 * (1.0+myDensity) + ptr);

		int pc = pointCount[myI][myJ];
		
		if (pc > StaticSwitches.TOO_MANY_POINTS) {
//			System.out.println("TOO MANY: ptr: "+ptr+" pc = "+pc +" in "+myI+","+myJ);
			saveForZoomedGraph(myI, myJ);
			return null;
		}
		
		ArrayList<CandidateEvaluation> evalsList = new ArrayList<CandidateEvaluation>(Arrays.asList(evals));
		Collections.sort(evalsList);
		List<CandidateEvaluation> bestList = new ArrayList<CandidateEvaluation>();
		List<CandidateEvaluation> notQuiteBestList = new ArrayList<CandidateEvaluation>();
		
		double bestScore = evalsList.get(0).getScore();
		int bestCount = 0;
		for (CandidateEvaluation ce : evalsList) {
			if (ce.getScore() == bestScore) {
				bestCount++;
				bestList.add(ce);
			} else {
				if (ce.getScore() > 0.0)
					notQuiteBestList.add(ce);
			}
		}
		
		

		
//		System.out.println("ptr: "+ptr+" best score: "+bestScore+" best count: "+bestCount);
		
		CandidateEvaluation ce = null;
		if (colocated.containsKey(ptr)) {
			
			// colocated with some other points
			
			List<Integer> co = new ArrayList<Integer>(colocated.get(ptr));
			co.add(ptr);
			
			Collections.sort(co);
			
//			if (co.size() <= 16) {
//				for (int i = 0; i < co.size(); i++) {
//					if (co.get(i).intValue() == ptr) {
//						if (i >= evalsList.size())
//							System.out.println("coloc "+ptr+" "+i+" "+co.size()+" "+evalsList.size());
//						ce = evalsList.get(i);
//						System.out.println("Select "+i);
//						ce.getCandidateLocation().setDistance(startSize+i*5);
//						break;
//					}
//				}
//			} else {
				for (int i = 0; i < co.size(); i++) {
					int select = i;
					int iter = i / 16;
					if (co.get(i).intValue() == ptr) {
						if (i >= evalsList.size()) {
//							System.out.println("coloc "+ptr+" "+i+" "+co.size()+" "+evalsList.size());
							
							select = iter % 16;
						}
						ce = evalsList.get(select);
//						System.out.println("Select "+select);
						ce.getCandidateLocation().setDistance(startSize/*+iter*2*/+select*15);
						possibleLocations.add(ce.getCandidateLocation());
//						break;
					}
				}
//			}
		} else {

			if (bestCount > 1) {
				// determine up, down, left or right based on density
				for (CandidateEvaluation x : bestList) {
					x.setScore(x.getScore()/(x.getLeftToRightDensity()*x.getTopToBottomDensity()));
				}
				
				try {
					Collections.sort(bestList);
					ce = bestList.get(0);
//					if (dppList.size() > 0)
//						ce = pointAway(dppList.get(0), bestList);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			} else {

				ce = evalsList.get(0);
			}
			
			for (CandidateEvaluation x : bestList) {
				possibleLocations.add(x.getCandidateLocation());
			}
			for (CandidateEvaluation x : notQuiteBestList) {
				possibleLocations.add(x.getCandidateLocation());
			}
			
		}
		
		if (possibleLocations.size() > 0) {
			return possibleLocations;
		} else {
//			System.out.println("ptr: "+ptr+" best score: "+bestScore+" Couldn't find suitable location");
			return null;
		}
	}
	
	private CandidateEvaluation pointAway(DataPointProximity dpp, List<CandidateEvaluation> bestList) {
		CandidateEvaluation ce = null;
		int maxDist = -1;
		double bestScore = bestList.get(0).getScore();
		for (CandidateEvaluation e : bestList) {
			if (e.getScore() == bestScore) {
				int dist = Math.abs(dpp.getSlot() - e.getCandidateLocation().getSlot());
				if (dist > maxDist) {
					ce = e;
					maxDist = dist;
				}
			}
		}
		
		return ce;
	}
	
	private double getSlotAngle(int slot) {
		
		if (slot < 8)
			return SLICE*slot;
		else
			return SLICE*(slot-8) * -1.0;	
	}
	
	private double getReverseSlotAngle(int slot) {
		
		if (slot < 8)
			return SLICE*slot * -1.0;
		else
			return SLICE*(slot-8);	
	}
	
	private boolean isColocated(int ptr1, int ptr2) {
		if (colocated.containsKey(ptr1)) {
			if (colocated.get(ptr1).contains(ptr2))
				return true;
		}
		return false;
	}
	
	private DataPointProximitySummary analyzeProximity(int ptr) {
		

		
		// Will not handle colocated points
		List<DataPointProximity> orderedProximitlyList = proximityList.get(ptr);
		double[] closest = new double[16];
		List<Integer>[] inSlot = new ArrayList[16];
		for (int slot = 0; slot < 16; slot++) {
			inSlot[slot] = new ArrayList<Integer>();
		}
		int[] count = new int[16];
		Arrays.fill(closest, -1.0);
		Collections.sort(orderedProximitlyList);
		
		for (DataPointProximity dpp : orderedProximitlyList) {
			count[dpp.getSlot()]++;
			inSlot[dpp.getSlot()].add(dpp.getPtr2());
			// do not include dist of 0.0 for colocated -- use other distance if there is one
			if (closest[dpp.getSlot()] == -1.0 && !isColocated(dpp.getPtr1(), dpp.getPtr2())) {
				closest[dpp.getSlot()] = dpp.getDistance();
			}
			
		}
		
		// if there are none, set distance to a high/low ?  number that will elevate score
		
		for (int i = 0; i < closest.length; i++) {
			if (count[i] == 0)
				closest[i] = 0.0001;
		}
		
		DataPointProximitySummary dpps = new DataPointProximitySummary(ptr);
		dpps.setClosest(closest);
		dpps.setCount(count);
		dpps.setInSlot(inSlot);
		dpps.setOrdered(orderedProximitlyList);
		dpps.initialize();
		
		return dpps;
	}
	
	private int oppositeSlot(int slot) {
		if (slot < 8)
			return 15 - slot;
		else
			return 7 - (slot-8);
	}
	
	private void analyzeDensity() {
//		List<DistanceDensity> al = new ArrayList<DistanceDensity>();
//		System.out.println("### Density ###");
//		for (int i = 0; i < centroids.length; i++) {
//			StringBuilder sb = new StringBuilder();
//			for (int j = 0; j < centroids.length; j++) {
//				sb.append(" "+pointCount[i][j]);
//			}
//			System.out.println(i+": "+sb.toString());
//		}
		
	}
	
	public List<Coordinate> orderLeftToRight(Set<Coordinate> list) {
		List<Coordinate> al = new ArrayList<Coordinate>(list);

		try {
			Collections.sort(al, new Comparator<Coordinate>() {
			    @Override
			    public int compare(Coordinate arg0, Coordinate arg1) {
			    	try {
			    		if (arg0.getY() < arg1.getY())
			    			return -1;
			    		if (arg0.getY() > arg1.getY())
			    			return 1;
			    		if (arg0.getX() < arg1.getX())
			    			return 1;
			    		if (arg0.getX() > arg1.getX())
			    			return -1;
			    		return 0;
						
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
			    }
			  });
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return al;
	}
	
	public List<Coordinate> orderRightToLeft(Set<Coordinate> list) {
		List<Coordinate> al = new ArrayList<Coordinate>(list);

		try {
			Collections.sort(al, new Comparator<Coordinate>() {
			    @Override
			    public int compare(Coordinate arg0, Coordinate arg1) {
			    	try {
			    		if (arg0.getY() < arg1.getY())
			    			return -1;
			    		if (arg0.getY() > arg1.getY())
			    			return 1;
			    		if (arg0.getX() < arg1.getX())
			    			return -1;
			    		if (arg0.getX() > arg1.getX())
			    			return 1;
			    		return 0;
						
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
						return 0;
					}
			    }
			  });
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return al;
	}
	private void moveUnassigned(int ptr) {

		// need to find closest cell with lowest point count
		List<DistanceDensity> al = new ArrayList<DistanceDensity>();
		for (int i = 0; i < centroids.length; i++) {
			for (int j = 0; j < centroids.length; j++) {
				Coordinate coord = boundingBoxes[ptr][1][1].getCoordinate();
				Coordinate cent = new Coordinate(centroids[i][j].getX(), centroids[i][j].getY(), getMinX(), getMaxX(), minY, maxY);
				al.add(new DistanceDensity(cent,
						distance(coord.getX(), coord.getY(), centroids[i][j].getX(), centroids[i][j].getY()),
						pointCount[i][j]));
			}
		}
		
		// st
		Collections.sort(al, new DistanceDensityComparator());
		DistanceDensity dd = al.get(0);
//		System.out.println(al);
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				BoundingBox tempBB = buildBoundingBox(annotationIndex.get(ptr), i, j, 
						dd.getCoordinate().getX(), 
						dd.getCoordinate().getY());
				boundingBoxes[ptr][i][j].relocate(tempBB);
			}
		}
		
		clearIntersectors(ptr);
		clearBoundingBoxes(ptr, 0, 1);
		selectedAnchor[ptr] = TEXT_ANCHORS[0][1];
		iteration[ptr] = ptr;
	}
	
	private void moveUnassigned() {
		for (int ptr = 0; ptr < x.length; ptr++) {
			if (selectedAnchor[ptr] == null) {
				moveUnassigned(ptr);
			}
		}
	}
	
	private boolean notDone() {
		int count = 0;
	
		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			if (selectedAnchor[ptr] == null) {
				count++;
			}
			
		}
		
		return count != 0;
	}
	
	public double[] getShiftValues(int ptr) {
		double cutoff = 1.0;
		double xAdj = boundingBoxes[ptr][0][0].height()/boundingBoxes[ptr][0][0].width();
		double xShift = (boundingBoxes[ptr][0][0].width() * xAdj)/2.0;
		double yShift = boundingBoxes[ptr][0][0].height();
		
		xShift /= 4.0;
		yShift /= 4.0;
		double[] shiftValues = new double[2];
		shiftValues[0] = xShift; // boundingBoxes[ptr][0][0].getX() <= cutoff ? xShift : -xShift;
		shiftValues[1] = yShift; // boundingBoxes[ptr][0][0].getY() <= cutoff ? yShift : -yShift;
		return shiftValues;
		
	}
	
	private void shiftBB(int ptr) {
		
		double cutoff = 1.0;
		double xAdj = boundingBoxes[ptr][0][0].height()/boundingBoxes[ptr][0][0].width();
		double xShift = (boundingBoxes[ptr][0][0].width() * xAdj)/2.0;
		double yShift = boundingBoxes[ptr][0][0].height();
		
		xShift /= 4.0;
		yShift /= 4.0;
//		System.out.println("Shifting "+ptr+" xShift: "+(boundingBoxes[ptr][0][0].getX() <= cutoff ? xShift : -xShift)+ 
//				" yShift: "+ (boundingBoxes[ptr][0][0].getX() <= cutoff ? yShift : -yShift));
//		printBoundingBoxes(ptr);
		
		for (int i=0; i<3; i++) {
			for (int j=0; j<3; j++) {
				boundingBoxes[ptr][i][j].shift(boundingBoxes[ptr][0][0].getX() <= cutoff ? xShift : -xShift, 
						boundingBoxes[ptr][0][0].getY() <= cutoff ? yShift : -yShift);
			}
		}
//		printBoundingBoxes(ptr);
	}
	
	private void shiftBBs() {
		// if the annotation has not be assigned a place, shift the bounding boxes
		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			if (selectedAnchor[ptr] == null) {
				shiftBB(ptr);
			}
		}
	}
	
//	public double getDistance(double x, double y, int ptr) {
//		// need to update iteration
//		// .02 = dist of ;
//		Coordinate dot = new Coordinate(x,y, getMinX(), getMaxX(), minY, maxY);
//		double[] ll = getUsedBoundingBox(ptr).getLowerLeft();
//		Coordinate text = new Coordinate(ll[0], ll[1], getMinX(), getMaxX(), minY, maxY);
//		double dist = dot.distanceTo(text);
//		double adjDist = dist * MicMacPlot.DISTANCE_ADJUSTMENT * (MicMacPlot.PLOT_SIZE_SCALE_FACTOR);
//		return adjDist;
//	}
	
	public double getDistance(int ptr) {
		// need to update iteration
		// .02 = dist of ;
		return BASE_DISTANCE + iteration[ptr]*BASE_DISTANCE;
	}
	
	private boolean makeChoice() {
		// we are left with some points with no clear place to put annotation
		// let's pick one based on the smallest number of intersects and then free that bounding box
		Map<Integer, Set<String>> byCount = new HashMap<Integer, Set<String>>();
		int max = 0;
		for (String key : intersectors.keySet()) {
			Set<String> set = new LinkedHashSet<String>(intersectors.get(key));
			set.remove(EDGE);
			int siz = set.size();
			if (!byCount.containsKey(siz))
				byCount.put(siz, new LinkedHashSet<String>());
			byCount.get(siz).add(key);
			if (siz > max)
				max = siz;
		}
		
		for (int s = 1; s < max; s++) {
			if (byCount.containsKey(s)) {
				Set<String> set = byCount.get(s);
				for (String x : set) {
					// we cannot use this one if one of its intersection bb is already in use
					Set<String> interSet = intersectors.get(x);
					boolean okToUse = true;
					for (String bb : interSet) {
						if (usedBoundingBoxes.contains(bb))
							okToUse = false;
					}
					if (okToUse) {
//						System.out.println("MakeChoice: "+x);
						String[] parts = x.split("#");
						int ptr = Integer.parseInt(parts[0]);
						int i = Integer.parseInt(parts[1]);
						int j = Integer.parseInt(parts[2]);
						// let's use this one
						candidate[ptr][i][j] = true;
						selectedAnchor[ptr] = preSelect3(ptr);
						return true;
					}
				}
			}
		}
		return false;
	}
	
	private void printIntersectors() {
		List<String> al = new ArrayList<String>(intersectors.keySet());
		Collections.sort(al);
		for (String key : al) {
			int ptr = Integer.parseInt(key.split("#")[0]);
//			System.out.println(annotationIndex.get(ptr)+": "+key+" -> "+intersectors.get(key));
		}
	}

	private void preSelecter(int ptr) {
		
			if (selectedAnchor[ptr] == null) {
				selectedAnchor[ptr] = preSelect3(ptr);
			}
	}
	
	private void colocated() {
		if (colocated.size() > 0) {
			for (int a : colocated.keySet()) {
				Set<Integer> c = colocated.get(a);
				for (Integer i : c) {
					if (selectedAnchor[a] == selectedAnchor[i]) {
						TextAnchor anAlternate = alternateTextAnchor(a, i);
						int ptr = i;
						int oldi = mapAnchorToI(selectedAnchor[i]);
						int oldj = mapAnchorToJ(selectedAnchor[i]);
						usedBoundingBoxes.remove(ptr+"#"+oldi+"#"+oldj);
						int newi = mapAnchorToI(anAlternate);
						int newj = mapAnchorToJ(anAlternate);
						selectedAnchor[i] = anAlternate;
						usedBoundingBoxes.add(ptr+"#"+newi+"#"+newj);
					}
				}
			}
		}
	}

	private TextAnchor alternateTextAnchor(int i, int j) {
		TextAnchor oppos = opposite(selectedAnchor[i]);
		if (oppos == null) {

		}
		return oppos;
	}

	private TextAnchor opposite(TextAnchor anchor) {

		if (anchor == TextAnchor.BOTTOM_RIGHT) return TextAnchor.TOP_LEFT;
		if (anchor == TextAnchor.BOTTOM_CENTER) return TextAnchor.TOP_CENTER;
		if (anchor == TextAnchor.BOTTOM_LEFT) return TextAnchor.TOP_RIGHT;
		if (anchor == TextAnchor.CENTER_RIGHT) return TextAnchor.CENTER_LEFT;
		if (anchor == TextAnchor.CENTER_LEFT) return TextAnchor.CENTER_RIGHT;
		if (anchor == TextAnchor.TOP_RIGHT) return TextAnchor.BOTTOM_LEFT;
		if (anchor == TextAnchor.TOP_CENTER) return TextAnchor.BOTTOM_CENTER;
		if (anchor == TextAnchor.TOP_LEFT) return TextAnchor.BOTTOM_RIGHT;
		return null;

	}

	private void initializeColocated() {
		for (int ptr = 0; ptr < x.length; ptr++) {

			for (int p2 = 0; p2 < x.length; p2++) {
				if (p2 == ptr)
					continue;
				if (x[p2] == x[ptr] && y[p2] == y[ptr]) {
					if (!colocated.containsKey(ptr)) {
						colocated.put(ptr, new LinkedHashSet<Integer>());
					}
					colocated.get(ptr).add(p2);
				}
			}
		}
//		if (colocated.size() > 0) {
//			System.out.println("colocated: "+colocated);
//		}
	}

	public TextAnchor determineTextAnchorPoint(int ptr) {
		if (selectedAnchor[ptr] == null) {
			noPlace.add(ptr);
		}
		return selectedAnchor[ptr];
	}
	
	private void clearBoundingBoxes(int ptr, int noti, int notj) {
		for (int i=0; i<3; i++) {
			for (int j=0; j<3; j++) {
				if (i==noti && j==notj)
					continue;
				boundingBoxes[ptr][i][j] = null;
				usedBoundingBoxes.remove(ptr+"#"+i+"#"+j);
				
			}
		}
		usedBoundingBoxes.add(ptr+"#"+noti+"#"+notj);
		selectedBoundingBoxes[ptr] = boundingBoxes[ptr][noti][notj];
	}
	
	public BoundingBox getUsedBoundingBox(int ptr) {
		for (int i=0; i<3; i++) {
			for (int j=0; j<3; j++) {
				if (boundingBoxes[ptr][i][j] != null)
					return boundingBoxes[ptr][i][j];
			}
		}
		return null;
	}
	
	private void clearIntersectors(int ptr) {
		List<String> al = new ArrayList<String>();
		for (String key : intersectors.keySet()) {
			if (key.startsWith(ptr+"#"))
				al.add(key);
		}
		for (String remov : al) {
			intersectors.remove(remov);
		}
	}
	
	public double dist(double d1, double d2) {
		return Math.abs(d1 - d2);
	}
	
	public double midPoint(double d1, double d2) {
		return Math.abs((d1 + d2)/2.0);
	}
	
	public void anchorShift(TextAnchor anchor, BoundingBox bb) {

		// shift bounding box such that the anchor point takes on the existing centerpoint


		double[] to;
		double[] to2;

		double fromx = bb.getX();
		double fromy = bb.getY();
		
		double newx;
		double newy;
		
		
		int X = BoundingBox.X;
		int Y = BoundingBox.Y;

		if (anchor == TextAnchor.BOTTOM_RIGHT) {
			to = bb.getUpperLeft();
			bb.moveTo(to[X], to[Y]);
		}

		else if (anchor == TextAnchor.BOTTOM_CENTER) { 
			newx = bb.getX();
			newy = bb.getY() + bb.height()/2;
			bb.moveTo(newx, newy);
		}

		else if (anchor == TextAnchor.BOTTOM_LEFT) {
			to = bb.getUpperRight();
			bb.moveTo(to[X], to[Y]);
		}

		else if (anchor == TextAnchor.CENTER_RIGHT) {
			newx = bb.getX() - bb.width()/2;
			newy = bb.getY();
			bb.moveTo(newx, newy);
		}

		else if (anchor == TextAnchor.CENTER_LEFT) {
			newx = bb.getX() + bb.width()/2;
			newy = bb.getY();
			bb.moveTo(newx, newy);
		}

		else if (anchor ==TextAnchor.TOP_RIGHT) {
			to = bb.getLowerLeft();
			bb.moveTo(to[X], to[Y]);
		}

		else if (anchor ==  TextAnchor.TOP_CENTER) { 
			newx = bb.getX();
			newy = bb.getY() - bb.height()/2;
			bb.moveTo(newx, newy);
		}

		else if (anchor == TextAnchor.TOP_LEFT) { 
			to = bb.getLowerRight();
			bb.moveTo(to[X], to[Y]);
		}

	}
	
	public TextAnchor preSelect(int ptr) {
		
//		private final static TextAnchor[][] TEXT_ANCHORS = 
//			{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
//			{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
//			{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
//			};

		boolean[][] candidate = this.candidate[ptr];
		
		
		if (candidate[1][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 1, 2);
			return TEXT_ANCHORS[1][2];
		}
		if (candidate[1][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 1, 0);
			return TEXT_ANCHORS[1][0];
		}
		
		if (candidate[0][1]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 1);
			return TEXT_ANCHORS[0][1];
		}
		if (candidate[2][1]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 1);
			return TEXT_ANCHORS[2][1];
		}
		
		if (candidate[0][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 2);
			return TEXT_ANCHORS[0][2];
		}
		if (candidate[0][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 0);
			return TEXT_ANCHORS[0][0];
		}
		if (candidate[2][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 0);
			return TEXT_ANCHORS[2][0];
		}
		if (candidate[2][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 2);
			return TEXT_ANCHORS[2][2];
		}
		return null;
	}
	
	public TextAnchor preSelect3(int ptr) {
		
//		private final static TextAnchor[][] TEXT_ANCHORS = 
//			{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
//			{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
//			{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
//			};

		boolean[][] candidate = this.candidate[ptr];
		
		if (candidate[0][1]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 1);
			return TEXT_ANCHORS[0][1];
		}
		
		if (candidate[2][1]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 1);
			return TEXT_ANCHORS[2][1];
		}
		
		if (candidate[1][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 1, 0);
			return TEXT_ANCHORS[1][0];
		}
		
		
		if (candidate[1][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 1, 2);
			return TEXT_ANCHORS[1][2];
		}

		if (candidate[0][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 2);
			return TEXT_ANCHORS[0][2];
		}
		if (candidate[0][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 0, 0);
			return TEXT_ANCHORS[0][0];
		}
		if (candidate[2][0]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 0);
			return TEXT_ANCHORS[2][0];
		}
		if (candidate[2][2]) {
			clearIntersectors(ptr);
			clearBoundingBoxes(ptr, 2, 2);
			return TEXT_ANCHORS[2][2];
		}
		return null;
	}
	
	public TextAnchor translateUsedAnchor(int ptr, TextAnchor anchor) {
		if (getUsedBoundingBox(ptr).getNumShifts() == 0) {
			return anchor;
		} else {
//			if (anchor == TextAnchor.CENTER_LEFT || anchor == TextAnchor.CENTER_RIGHT) {
//				return anchor;
			if (getUsedBoundingBox(ptr).getLowerLeft()[0] > getUsedBoundingBox(ptr).getX())
				return TextAnchor.CENTER_LEFT;
			else
				return TextAnchor.CENTER_RIGHT;
			// TextAnchor.BOTTOM_CENTER
			// above or below?
		}
	}

	public TextAnchor preSelect2(int ptr) {

		boolean[][] candidate = this.candidate[ptr];

		List<Integer> is = new ArrayList<Integer>();
		List<Integer> js = new ArrayList<Integer>();
		is.add(0); is.add(1); is.add(2);
		js.add(0); js.add(1); js.add(2);
		Collections.shuffle(is);
		Collections.shuffle(js);
		
		for (int i : is) {
			for (int j : js) {
				if (candidate[i][j]) {
					clearIntersectors(ptr);
					clearBoundingBoxes(ptr, i, j);
					return TEXT_ANCHORS[i][j];
				}
			}
		}
		return null;
	}

	private void evaluateCandidate(int ptr, int iter) {

		int localIter = iter;
//		int maxIter = 10;

//		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			localIter = iter;

			if (selectedAnchor[ptr] == null) {
				updateAvailability(ptr);
				if (hasCandidate(ptr)) {	
					iteration[ptr] = localIter;
//					continue;
				} else {
//					while (localIter < maxIter && !hasCandidate(ptr)) {
//						shiftBB(ptr);
//						updateAvailability(ptr);
//						if (hasCandidate(ptr)) {	
//							iteration[ptr] = localIter;
//							continue;
//						} else {
//							localIter++;
//						}
//					}
				}
			}
//		}
	}

	private void updateAvailability(int ptr) {
		
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {
				if (i == 1 && j == 1) {
					candidate[ptr][i][j] = false;
				} else {
					candidate[ptr][i][j] = !intersects(boundingBoxes[ptr][i][j], i, j); // /* candidate[ptr][i][j] &&*/ !intersects(boundingBoxes[ptr][i][j], i, j); // true;
				}
			}
		}
	}
	
	private boolean hasCandidate(int ptr) {
		for (int i = 0; i < 3; i++) {
			for (int j = 0; j < 3; j++) {

				if (candidate[ptr][i][j])
					return true;
			}
		}
		return false;
	}
	
	private boolean intersects(BoundingBox bb, int myi, int myj) {
		// single bounding box as input, compare against all other ptr 8 bounding boxes
		// return a single boolean
		boolean intersect = false;
		// if text extends off left side of graph, don't allow
		if (bb == null)
			return false;
		
		if (bb.getLowerLeft()[0] <= 0.0 ||
				bb.getUpperLeft()[1] >= maxY ||
				bb.getLowerRight()[0] >= maxX ||
				bb.getLowerRight()[1] <= 0.0) {
			intersect = true;
			String key = bb.getPtr()+"#"+myi+"#"+myj;
			String value = "-1#-1#-1";
			if (!intersectors.containsKey(key))
				intersectors.put(key, new LinkedHashSet<String>());
			intersectors.get(key).add(value);
			return true;
		}

		String key = bb.getPtr()+"#"+myi+"#"+myj;

		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			// don't compare against self
			if (ptr == bb.getPtr())
				continue;
				
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j <3; j++) {
					String ptrKey = ptr +"#"+i+"#"+j;
					
					// concerned about BB's that are used and have intersection
					
					boolean i1a = bb.intersect(boundingBoxes[ptr][i][j]);
					boolean i1b = usedBoundingBoxes.contains(ptrKey);
					boolean i2a = boundingBoxes[ptr][i][j] != null && boundingBoxes[ptr][i][j].intersect(bb);
					boolean i2b = usedBoundingBoxes.contains(ptrKey);
					
					// does bounding box interest with the actual dot on the graph?
					boolean i3 = boundingBoxes[ptr][i][j] != null && bb.inside(boundingBoxes[ptr][i][j].getLocation());
					
					boolean inters = (i1a && i1b) || (i2a && i2b) || i3;
					
//					boolean inters = (bb.intersect(boundingBoxes[ptr][i][j]) && (usedBoundingBoxes.contains(key) || i == j))
//							|| ( boundingBoxes[ptr][i][j] != null && boundingBoxes[ptr][i][j].intersect(bb))
//							&& (usedBoundingBoxes.contains(ptrKey) || i == j);
					
					if (inters) {
						intersect = true;
						String value = ptr +"#"+i+"#"+j;
						if (!intersectors.containsKey(key))
							intersectors.put(key, new LinkedHashSet<String>());
						intersectors.get(key).add(value);
					} else {
						
					}
				}
			}
		}

		return intersect;
	}
	
	public boolean boundingBoxOnChart(BoundingBox bb) {
		return bb.onChart(minX, minY, plotMaxX, plotMaxY);
	}
	
	public boolean isAvailable(Annotation annot, boolean lineBoxIntersectEnabled) {

		double height = getHeight(annot.getText());
		double width = getWidth(annot.getText());
		double adjustedDistance = (annot.getDistanceFromXY() /  DISTANCE_ADJUSTMENT); //  * (MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR);
		double x2 = annot.getX() + Math.cos(annot.getAngle()) * adjustedDistance;
		double y2 = annot.getY() - Math.sin(annot.getAngle()) * adjustedDistance;

		BoundingBox bb = new BoundingBox(this, x2, y2, width, height, annot.getPtr());
		anchorShift(annot.getAnchor(), bb);
		
		boolean lineIntersection;
		if (lineBoxIntersectEnabled) {
			lineIntersection = intersectsWithAUsedBoundingBox(annot.getPtr(), annot.getX(), annot.getY(), x2, y2);
		} else {
			lineIntersection = false;
		}

		if (intersectsWithAUsedBoundingBox(bb)  || 
				!boundingBoxOnChart(bb) ||
				lineIntersection) {
			return false;
		}
		annot.setBoundingBox(bb);
		usedAnnotionSpace.add(bb);
		return true;
	}
	
	public boolean squeezeInAnnotation(Annotation annot, boolean lineBoxIntersectEnabled) {

		int sweepCount = 0;

		// immediately lengthen the distance since we already search at the original distance
		
		double height = getHeight(annot.getText());
		double width = getWidth(annot.getText());
		annot.setDistanceFromXY(annot.getDistanceFromXY()+35);
		double adjustedDistance = (annot.getDistanceFromXY()) /  
				DISTANCE_ADJUSTMENT; // *MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR;
		double x2 = annot.getX() + Math.cos(annot.getAngle()) * adjustedDistance;
		double y2 = annot.getY() - Math.sin(annot.getAngle()) * adjustedDistance;
	
		boolean lineBoxIntersect = false;
		
		BoundingBox bb = new BoundingBox(this, x2, y2, width, height, annot.getPtr());
		
		anchorShift(annot.getAnchor(), bb);
		BoundingBox startingBB = bb.copy();
		double startingDistance = annot.getDistanceFromXY();
		
		if (lineBoxIntersectEnabled)
			lineBoxIntersect = intersectsWithAUsedBoundingBox(annot.getPtr(), annot.getX(), annot.getY(), x2, y2);
		else
			lineBoxIntersect = false;
		

		int shiftCount = 0;
		boolean restart = false;
		
		boolean extendDistance = false;
		boolean localSearch = true;
		boolean changeMethod = true;

		while (true) {
			restart = false;
			if (intersectsWithAUsedBoundingBox(bb)  || 
					!boundingBoxOnChart(bb) ||
					lineBoxIntersect) {
				
				lineBoxIntersect = false;
				
					if (extendDistance) {
						
						if (!boundingBoxOnChart(bb)) {
//							System.out.println("OFF CHART");
							return false;
						}
						
						annot.setDistanceFromXY(annot.getDistanceFromXY()+(MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR * 35));
						
						shiftCount++;
						if (shiftCount > 1) {
							sweepCount++;
						}
						if (shiftCount > 20) {
							System.out.println(" SHIFT COUNT EXCEEDED");
							return false;
						} else {
							restart = true;
							changeMethod = true;
							extendDistance = false;
							localSearch = true;
							continue;
						}
					} else if (localSearch){
						changeMethod = true;
						SlotIterator iter = new SlotIterator(annot.getSlot());
						while (iter.isHasMoreCounterClockwise()) {
							int slot = iter.nextCounterClockwise();
//							System.out.println("localSearch: try slot: "+slot);
							adjustedDistance = (annot.getDistanceFromXY() / DISTANCE_ADJUSTMENT); //  * (MicMacScatterPlot.PLOT_SIZE_SCALE_FACTOR); 
							x2 = annot.getX() + Math.cos(getReverseSlotAngle(slot)/*+Math.PI/2*/) * adjustedDistance;  // 900
							y2 = annot.getY() - Math.sin(getReverseSlotAngle(slot)/*+Math.PI/2*/) * adjustedDistance;  // 900
							
//							System.out.println(annot.getPtr()+" dist "+NumberFormatter.formatForSmall(annot.getDistanceFromXY())+" localSearch: "+"try slot: "+slot+" "+x2+", "+y2+" slotAngle: "+getReverseSlotAngle(slot)+
//									" annot angle "+annot.getAngle());
							
							
//							if (x2 < 0.0 || y2 < 0.0 || x2 > maxX + (range/2) || y2 > maxY + (range/2)) {
								if (x2 < 0.0 || y2 < 0.0 || x2 > maxX  || y2 > maxY ) {
								
								continue;
							}
							bb = new BoundingBox(this, x2, y2, width, height, annot.getPtr());
							anchorShift(getSlotAnchor(slot), bb);
							if (lineBoxIntersectEnabled)
								lineBoxIntersect = intersectsWithAUsedBoundingBox(annot.getPtr(), annot.getX(), annot.getY(), x2, y2);
							else
								lineBoxIntersect = false;
							if (!intersectsWithAUsedBoundingBox(bb) && 
									!lineBoxIntersect && 
									boundingBoxOnChart(bb)) {
								restart = true;
								changeMethod = false;
								annot.setAngle(getReverseSlotAngle(slot));
								annot.setAnchor(getSlotAnchor(slot));
								annot.setBoundingBox(bb);
								annot.setSlot(slot);
								usedAnnotionSpace.add(bb);
								return true;
							}
						}
						if (changeMethod) {
							if (sweepCount > 2) {
//								System.out.println("Disable line box intersect");
								// we may have passed up closer bounding boxes because of line intersection.
								// when this changes, go back to the original distance and begin search again
								lineBoxIntersectEnabled = false;
								annot.setDistanceFromXY(startingDistance);
								return squeezeInAnnotation(annot, lineBoxIntersectEnabled);
							}
							extendDistance = true;
							localSearch = false;
							restart = true;
							bb = startingBB;
							
							// this will force while true loop to get past initial intersection checks
							lineBoxIntersect = true;
						}
					}
				} else {
					// this is a special case where previous locations failed because of line intersections
					if (shiftCount == 0) {
						int slot = annot.getSlot();
						annot.setAngle(getReverseSlotAngle(slot));
						annot.setAnchor(getSlotAnchor(slot));
						annot.setBoundingBox(bb);
						usedAnnotionSpace.add(bb);
						return true;
					}
				}

			if (!restart)
				break;
		}
		return false;
	}
	
	public boolean intersectsWithAUsedBoundingBox(int ptr, double x1, double y1, double x2, double y2) {
		boolean intersects = false;
		
		MutableDouble ax1 = new MutableDouble(x1);
		MutableDouble ax2 = new MutableDouble(x2);
		MutableDouble ay1 = new MutableDouble(y1);
		MutableDouble ay2 = new MutableDouble(y2);
		shortenLine(ax1, ay1, ax2, ay2);
		for (BoundingBox abb : usedAnnotionSpace) {
			if (abb.lineIntersectsBoundingBox(ax1.doubleValue(), ay1.doubleValue(), ax2.doubleValue(), ay2.doubleValue())) {
				// don't intersect with self
				if (abb.getPtr() < -4 && Math.abs(abb.getPtr()+10) == ptr) {
					continue;
				}
//				System.out.println(NumberFormatter.formatForSmall(ax1.doubleValue())+","+ NumberFormatter.formatForSmall(ay1.doubleValue())+
//						","+ NumberFormatter.formatForSmall(ax2.doubleValue())+", "+ NumberFormatter.formatForSmall(ay2.doubleValue())+
//						" Intersects with BB: "+abb);
				intersects = true;;
			}
		}
		return intersects;
	}
	
	public void shortenLine(MutableDouble x1, MutableDouble y1, MutableDouble x2, MutableDouble y2) {
		double t = .02;
		double adjust = 0.02;
        
//		System.out.println("shortenLine in: "+x1.doubleValue()+","+y1.doubleValue()+","+x2.doubleValue()+","+y2.doubleValue());
		if (x2.doubleValue() == x1.doubleValue()) {
			y1.setValue((1.0+t)*y1.doubleValue());
			y2.setValue((1.0-t)*y2.doubleValue());
		} else if (y2.doubleValue() == y1.doubleValue()) {
			x1.setValue((1.0+t)*x1.doubleValue());
			x2.setValue((1.0-t)*x2.doubleValue());
		} else {
			double slope = (y2.doubleValue() - y1.doubleValue()) / (x2.doubleValue() - x1.doubleValue());
			double dx = (adjust/ Math.sqrt(1 + (slope * slope))); 
	        double dy = slope * dx;
	        
//	        System.out.println("slope: "+slope+" dx: "+dx+" dy: "+dy);
	        
	        if (x1.doubleValue() < x2.doubleValue()) {
	        	x1.setValue(x1.doubleValue() + dx); 
	        	x2.setValue(x2.doubleValue() - dx);
	        	y1.setValue(y1.doubleValue() + dy); 
		        y2.setValue(y2.doubleValue() - dy);
	        } else {
	        	x1.setValue(x1.doubleValue() - dx); 
	        	x2.setValue(x2.doubleValue() + dx);
	        	y1.setValue(y1.doubleValue() - dy); 
		        y2.setValue(y2.doubleValue() + dy);
	        }
		}
		
//		System.out.println("shortenLine out: "+x1.doubleValue()+","+y1.doubleValue()+","+x2.doubleValue()+","+y2.doubleValue());
	}
	
	public boolean intersectsWithAUsedBoundingBox(BoundingBox bb) {
		boolean intersects = false;
		for (BoundingBox abb : usedAnnotionSpace) {
			if (bb.intersect(abb) || abb.intersect(bb)) {
				// make sure we don't intersect with self
				if (abb.getPtr() < -4 && Math.abs(abb.getPtr()+10) == bb.getPtr()) {
						continue;
				}
//				System.out.println(bb+" intersects "+abb);
				intersects = true;;
			}
		}
		return intersects;
	}
	
	private void initializeCentroids() {
		double[] centerI = new double[] {1.1, .9, .7, .5, .3, .1}; // {.1,.3,.5,.7,.9,1.1};
		double[] centerJ = new double[] {.1,.3,.5,.7,.9,1.1};
		for (int i = 0; i < centerI.length; i++) {
			for (int j = 0; j < centerJ.length; j++) {
				centroids[i][j] = new Coordinate(centerJ[j], centerI[i], getMinX(), getMaxX(), minY, maxY);
			}
		}
	}

//	private boolean (double[] ll, double[] ul, double[] ur, double[] lr , int ptr) {
//
//		boolean inter = false;
//
//		// if text extends off left side of graph, don't allow
//		if (ll[0] <= 0.0)
//			return true;
//
//		for (int i = 0; i < numAnnotations; i++) {
//			if (i == ptr)
//				continue;
//			if (x[i] >= ll[0] && x[i] <= lr[0] &&
//					y[i] >= ll[1] && y[i] <= ul[1]) {
//				inter = true;
//
//				if (!intersectors.containsKey(ptr))
//					intersectors.put(ptr, new LinkedHashSet<Integer>());
//				intersectors.get(ptr).add(i);
//			}
//		}
//
//		return inter;
//	}

	private void initializeCandidateToAll() {
		for (int ptr = 0; ptr < x.length; ptr++) {
			double myX = x[ptr];
			double myY = y[ptr];
			boolean[][] candidate = this.candidate[ptr];
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					candidate[i][j] = true;    				
				}
			}
			candidate[1][1] = false;

//			if (closeToLeftSide(myX, myY)) {
//				for (int i = 0; i < 3; i++)
//					candidate[i][0] = false;
//			}
//
//			if (closeToRightSide(myX, myY)) {
//				for (int i = 0; i < 3; i++)
//					candidate[i][2] = false;
//			}
//
//			if (closeToTopSide(myX, myY)) {
//				for (int i = 0; i < 3; i++)
//					candidate[0][i] = false;
//			}
//
//			if (closeToBottomSide(myX, myY)) {
//				for (int i = 0; i < 3; i++)
//					candidate[2][i] = false;
//			}

//			printCandidate(annotationIndex.get(ptr),candidate);
		}
		
	}
	
	private double getHeight(String text) {
		return HEIGHT_FACTOR; //  * maxY/SIZE;
	}
	
	private double getWidth(String text) {
		return text.length() * WIDTH_FACTOR; //  * maxX/SIZE;
	}
	
	private BoundingBox buildBoundingBox(String text, int i, int j, double x, double y) {

		double[] ll = new double[2];
		double[] ul = new double[2];
		double[] ur = new double[2];
		double[] lr = new double[2];


		double height = getHeight(text);
		double width = getWidth(text);

		double myX = x;
		double myY = y;

		// note that we must adjust x and y coordinates to account for arrow

		// 10.5 
		if (i == 0 && j == 0) {
			ll[0] = myX - width - ARROW_OFFSET; lr[0] = myX - ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}
		// 12 
		if (i == 0 && j == 1) {
			ll[0] = myX - width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}
		// 1.5 
		if (i == 0 && j == 2) {
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return  new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}
		// 3
		if (i == 1 && j == 2) {
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			return  new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}

		// point
		if (i == 1 && j == 1) {
			ll[0] = myX - width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}

		// 4.5
		if (i == 2 && j == 2) {
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}

		// 6
		if (i == 2 && j == 1) {
			ll[0] = myX-width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}

		// 7.5
		if (i == 2 && j == 0) {
			ll[0] = myX-width-ARROW_OFFSET; lr[0] = myX-ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}

		// 9
		if (i == 1 && j == 0) {
			ll[0] = myX-width-ARROW_OFFSET; lr[0] = myX-ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			return new BoundingBox(this, myX, myY, ll, lr, ul, ur, -1);
		}
		
		return null;
	}
	private void initializeBoundingBoxes() {

		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			
			double[] ll = new double[2];
			double[] ul = new double[2];
			double[] ur = new double[2];
			double[] lr = new double[2];

			
			String text = annotationIndex.get(ptr);
			double height = getHeight(text);
			double width = getWidth(text);
//			System.out.println("Text: "+text+" len = "+text.length());
//			System.out.println("height = "+height+" maxY "+maxY);
//			System.out.println("width = "+width+" maxX "+maxX);
			
			double myX = annotations.get(text).getX();
			double myY = annotations.get(text).getY();
			
			myX = x[ptr];
			myY = y[ptr];

			// note that we must adjust x and y coordinates to account for arrow

			// 10.5 
			ll[0] = myX - width - ARROW_OFFSET; lr[0] = myX - ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][0][0] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 12 
			ll[0] = myX - width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][0][1] =new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 1.5 
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY+ARROW_OFFSET; ul[1] = myY + height+ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][0][2] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);
			// 3
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][1][2] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);
			
			// point
			ll[0] = myX - width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][1][1] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 4.5
			ll[0] = myX+ARROW_OFFSET; lr[0] = myX + width+ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][2][2] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 6
			ll[0] = myX-width/2.0; lr[0] = myX + width/2.0; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][2][1] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 7.5
			ll[0] = myX-width-ARROW_OFFSET; lr[0] = myX-ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height-ARROW_OFFSET; ul[1] = myY-ARROW_OFFSET; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][2][0] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);

			// 9
			ll[0] = myX-width-ARROW_OFFSET; lr[0] = myX-ARROW_OFFSET; ul[0] = ll[0]; ur[0] = lr[0];
			ll[1] = myY-height/2.0; ul[1] = myY + height/2.0; lr[1] = ll[1]; ur[1] = ul[1];
			boundingBoxes[ptr][1][0] = new BoundingBox(this, myX, myY, ll, lr, ul, ur, ptr);
			
			// moved "to close" decision to here so that text length can be considered
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					if (i == j)
						continue;
					candidate[ptr][i][j] = !boundingBoxes[ptr][i][j].isTooCloseToPlotEdge();
				}
			}
		}
//		printBoundingBoxes();
	}
	private void printBoundingBoxes() {
		for (int ptr = 0; ptr < numAnnotations; ptr++) {
			printBoundingBoxes(ptr);
		}
	}
	
    public double computeAngle(Coordinate dataCoord1, Coordinate dataCoord2) {
    	double tan = Math.atan2(dataCoord2.getY()-dataCoord1.getY(), dataCoord2.getX()-dataCoord1.getX());

    	
    	if (tan == 0.0)
    		return 0.0;
    	if (tan == Math.PI)
    		return Math.PI;
//		return -tan;
		return tan;
    }
    
    private double computeDistance(Coordinate dataCoord1, Coordinate dataCoord2) {
    	return dataCoord1.distanceTo(dataCoord2);
    }
    	

	
	private void printBoundingBoxes(int ptr) {
			System.out.println("Text: "+annotationIndex.get(ptr));
			for (int i = 0; i < 3; i++) {
				for (int j = 0; j < 3; j++) {
					System.out.println("BB,"+ptr+","+i+","+j+","+boundingBoxes[ptr][i][j]+" candidate: "+candidate[ptr][i][j]);
				}
			}
	}

	public double percentNoPlaceForText() {

		double pct = 0.0;
		double numNoPlace = 0.0;

		for (int ptr = 0; ptr < x.length; ptr++) {

			boolean[][] candidate = this.candidate[ptr];

			boolean hasPlace = false;
			for (int i = 0; i < candidate.length; i++) {
//				System.out.println(candidate[i][0]+" "+candidate[i][1]+" "+candidate[i][2]);
				hasPlace = hasPlace || candidate[i][0] || candidate[i][1] || candidate[i][2];
			}
			if (!hasPlace)
				numNoPlace += 1.0;
		}

		pct = numNoPlace/x.length * 100.0;

		return pct;
	}
	
	private int mapAnchorToI(TextAnchor ta) {
//		private final static TextAnchor[][] TEXT_ANCHORS = 
//			{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
//			{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
//			{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
//			};
		
		if (ta == TextAnchor.BOTTOM_RIGHT || ta == TextAnchor.BOTTOM_CENTER || ta == TextAnchor.BOTTOM_LEFT)
			return 0;
		if (ta == TextAnchor.CENTER_RIGHT || ta == TextAnchor.CENTER || ta == TextAnchor.CENTER_LEFT)
			return 1;
		return 2;
	}
	
	private int mapAnchorToJ(TextAnchor ta) {
//		private final static TextAnchor[][] TEXT_ANCHORS = 
//			{{TextAnchor.BOTTOM_RIGHT, TextAnchor.BOTTOM_CENTER, TextAnchor.BOTTOM_LEFT},
//			{TextAnchor.CENTER_RIGHT, TextAnchor.CENTER, TextAnchor.CENTER_LEFT},
//			{TextAnchor.TOP_RIGHT, TextAnchor.TOP_CENTER, TextAnchor.TOP_LEFT}
//			};
		
		if (ta == TextAnchor.BOTTOM_RIGHT || ta == TextAnchor.CENTER_RIGHT || ta == TextAnchor.TOP_RIGHT)
			return 0;
		if (ta == TextAnchor.BOTTOM_CENTER || ta == TextAnchor.CENTER || ta == TextAnchor.TOP_CENTER)
			return 1;
		return 2;
	}

	public void printCandidate(String text, boolean[][] candidate) {
		System.out.println("### "+text+" ###");
		boolean hasPlace = false;
		for (int i = 0; i < candidate.length; i++) {
			System.out.println(candidate[i][0]+" "+candidate[i][1]+" "+candidate[i][2]);
			hasPlace = hasPlace || candidate[i][0] || candidate[i][1] || candidate[i][2];
		}
		System.out.println("HAS PLACE = "+hasPlace);
	}
	
	public boolean closeToLeftSide(int ptr) {
		return closeToLeftSide(pointCoordinates.get(ptr).getX(), pointCoordinates.get(ptr).getY());
	}
	
	public boolean closeToRightSide(int ptr) {
		return closeToRightSide(pointCoordinates.get(ptr).getX(), pointCoordinates.get(ptr).getY());
	}
	
	public boolean closeToTopSide(int ptr) {
		return closeToTopSide(pointCoordinates.get(ptr).getX(), pointCoordinates.get(ptr).getY());
	}
	
	public boolean closeToBottomSide(int ptr) {
		return closeToBottomSide(pointCoordinates.get(ptr).getX(), pointCoordinates.get(ptr).getY());
	}

	public boolean closeToLeftSide(double x, double y) {
		// 550 pixels wide
		// x/maxX  % 
		// say maxX = 20, width factor = 0.01 so, 550/20 = 25.5 perunit
		return x/maxX < CLOSE;
	}

	public boolean closeToRightSide(double x, double y) {
		return x/maxX > 1.0-CLOSE;
	}

	public boolean closeToTopSide(double x, double y) {
		return y/maxY > 1.0-CLOSE; 
	}

	public boolean closeToBottomSide(double x, double y) {
		return y/maxY < CLOSE;
	}

	public static double distance(double x1, double y1, double x2, double y2) {
		return Math.sqrt((x1-x2)*(x1-x2) + (y1-y2)*(y1-y2));
	}

	public static double getClose() {
		return CLOSE;
	}

	public static TextAnchor[][] getTextAnchors() {
		return TEXT_ANCHORS;
	}

	public double getHeigthFactor() {
		return HEIGHT_FACTOR;
	}

	public double getWidthFactor() {
		return WIDTH_FACTOR;
	}

	public Map<String, Coordinate> getAnnotations() {
		return annotations;
	}

	public Map<Integer, String> getAnnotationIndex() {
		return annotationIndex;
	}

	public int getNumAnnotations() {
		return numAnnotations;
	}

	public double getMaxX() {
		return maxX;
	}

	public double getMaxY() {
		return maxY;
	}

	public double[] getX() {
		return x;
	}

	public double[] getY() {
		return y;
	}

	public boolean[][][] getCandidate() {
		return candidate;
	}

	public Map<String, Set<String>> getIntersectors() {
		return intersectors;
	}

	public Map<Integer, Set<Integer>> getColocated() {
		return colocated;
	}

	public int getCurrentAnnotation() {
		return currentAnnotation;
	}
	
	public double getMinX() {
		return minX;
	}

	public void setMinX(double minX) {
		this.minX = minX;
	}

	public double getMinY() {
		return minY;
	}

	public void setMinY(double minY) {
		this.minY = minY;
	}

	public double getRange() {
		return rangeX;
	}

	public void setRange(double range) {
		this.rangeX = range;
	}

	public void setMaxX(double maxX) {
		this.maxX = maxX;
	}

	public void setMaxY(double maxY) {
		this.maxY = maxY;
	}

	public Map<Integer, Coordinate> getPointCoordinates() {
		return pointCoordinates;
	}
	
	

	public Map<Integer, DataPointProximitySummary> getProximitySummaryList() {
		return proximitySummaryList;
	}



	private class DistanceDensityComparator implements Comparator<DistanceDensity> {

		@Override
		public int compare(DistanceDensity o1, DistanceDensity o2) {
			int density1 = o1.getDensity();
			int density2 = o2.getDensity();
			double distance1 = o1.getDistance();
			double distance2 = o2.getDistance();

			if (density1 < density2)  {
				return -1;
			} else 
				if (density1 == density2) {
					return Double.compare(distance1, distance2);
				} else {
					return 1;
				}

		}
	}
}
