package gov.anl.gss.visualization;

import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.WorkbookUtils;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.JFrame;

import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.jfree.data.xy.XYSeries;
import org.jfree.ui.RefineryUtilities;

public class SampleVisualization {
	
	public static int PLOT_WIDTH = 1200; // 500;
	public static int PLOT_HEIGHT = 1200; // 400; // 440
	public static int FONT_SIZE = 9;
	
	public static final String DATAPOINT_ROW = "DPROW";
	public static final String DATAPOINT_COL = "DPCOL";
	public static final String CONNECTOR = "C";
	
	private Map<Integer, String> seriesType;
	
	private String excelFile;
	private String pngFile;
	private String tab;
	private List<String> columnHeaders;
	private Set<String> rowHeaders;
	
	private Set<String> noInputs;
	private Set<String> noOutputs;
	private Set<String> inputsOutputs;
	private Set<String> noConnectivity;
	
	Map<String, Set<String>> connectivity;
	Map<String, Set<String>> reverseConnectivity;
	Map<String, MutableInteger> outputLinks;
	
	Map<String, Color> colorMap;
	NextColor nextColor;
	
	private Set<String> all;
	
	public SampleVisualization(String excelFile, String tab) {
		this.excelFile = excelFile;
		this.tab = tab;
		rowHeaders = new LinkedHashSet<String>();
		noInputs = new LinkedHashSet<String>();
		noOutputs = new LinkedHashSet<String>();
		inputsOutputs = new LinkedHashSet<String>();
		noConnectivity = new LinkedHashSet<String>();
		all = new LinkedHashSet<String>();
		seriesType = new HashMap<Integer, String>();
		outputLinks = new HashMap<String, MutableInteger>();
		nextColor = new NextColor();
		colorMap = new HashMap<String, Color>();
		reverseConnectivity = new HashMap<String, Set<String>>();
	}
	
	public boolean generate(String pngFile) {
		this.pngFile = pngFile;
		
		 XSSFWorkbook wb = WorkbookUtils.openWorkbook(excelFile);
		 XSSFSheet sheet = wb.getSheet(tab);
		 List<List<String>> table = WorkbookUtils.getSheetAsTable(sheet);
		 table.remove(table.size()-1);
		 
		 int startRow = 2;  // zero-based counting
		 int startCol = 1;  // zero-based counting
		 
		 connectivity = generateConnectivity(table, startRow, startCol);
		 
		 for (String s : connectivity.keySet()) {
			 System.out.println("KEY: "+s);
			 System.out.println("SIZE: "+connectivity.get(s).size());
			 
			 all.addAll(connectivity.get(s));
//			 System.out.println(connectivity.get(s));
		 }
		 all.addAll(rowHeaders);
		 
		 rowToRowConnections(connectivity);
		 
		 for (String s : rowHeaders) {
			 colorMap.put(s, nextColor.nextNotGray());
		 }
		 
		 buildSingleChart(connectivity);
//		 buildChart(connectivity);
		
		return true;
	}
	
	public void buildSingleChart(Map<String, Set<String>> connectivity) {
		int fnum = 0;
		SFEScatterPlotDataSet dataSet = new SFEScatterPlotDataSet();
		double x = 5.0;
		double y = 3.0;
		int series = 0;
		
		List<String> rowHeadersSorted = sortRowHeadersByInputLinkCount();
		List<String> allSorted = sortAllByOutputLinkCount(rowHeadersSorted);
		
		
		for (String s : allSorted) {
			if (rowHeaders.contains(s))
				continue;
			dataSet.addDataPoint(new SFEScatterPlotDataPoint(s, x, y));
			seriesType.put(series, DATAPOINT_COL);
			System.out.println("DATAPOINT: "+series+" "+s+" "+x+" "+y);
			series++;
			
			y += 1.0;
		}
		
		double yInc = y/rowHeaders.size();
		y = 3.0;
		x = 1.0;
		double xInc = 0.2;
		
		for (String h : rowHeadersSorted) {
			
			
			seriesType.put(series, DATAPOINT_ROW);
			if (noConnectivity.contains(h)) {
				x = 1.0;
			} else if (noOutputs.contains(h)) {
				x = 1.0;
			} else if (noInputs.contains(h)) {
				x = 1.8;
			} else {
				x = 1.4;
			}
			dataSet.addDataPoint(new SFEScatterPlotDataPoint(h, x, y));
			System.out.println("DATAPOINT: "+series+" "+h+" "+x+" "+y);
			series++;
			y += yInc;
		}
		
		for (String h : rowHeaders) {
			Set<String> tails = connectivity.get(h);
			for (String t : tails) {
				connect(dataSet, t, h, series);
				series++;
			}
		}
		
		
		dataSet.addDataPoint(new SFEScatterPlotDataPoint("XXX", 6.0, 1));
		seriesType.put(series, DATAPOINT_COL);
		System.out.println("DATAPOINT: "+series+" "+"XXX");
		series++;

			SFESimpleAnnotatedScatterPlot chart = new SFESimpleAnnotatedScatterPlot(" Sample Bolt_Mini_Database5-4-17-NB Visualization", 
					dataSet, 6.0, y, seriesType, colorMap);

			chart.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			chart.pack();
			chart.setLocationRelativeTo(null);
			RefineryUtilities.centerFrameOnScreen(chart);
			chart.setVisible(false);

			Writer.renderPanelAndWrite(GraphTable.wrap(chart.getChart(), 
					PLOT_WIDTH, PLOT_HEIGHT), Integer.toString(fnum)+"_Vis.png");
			fnum++;
			


	}
	
	public void connect(SFEScatterPlotDataSet dataSet, String tail, String head, int series) {
		
		XYSeries connector = new XYSeries(tail+"###"+(head));
		seriesType.put(series, CONNECTOR);
		System.out.println("CONNECTOR: "+(series++)+" "+tail+"###"+(head));
		SFEScatterPlotDataPoint t = dataSet.getPoint(tail);
		SFEScatterPlotDataPoint h = dataSet.getPoint(head);
		connector.add(t.getX(), t.getY());
		connector.add(h.getX(), h.getY());
		dataSet.addConnector(connector);
	}
	
	public void buildChart(Map<String, Set<String>> connectivity) {
		int fnum = 0;
		for (String s : connectivity.keySet()) {
//		for (String s : rowHeaders) {
			SFEScatterPlotDataSet dataSet = new SFEScatterPlotDataSet();

			double x = 5.0;
			double y = 3.0;
			int series = 0;

			

			//		for (String s : connectivity.keySet()) {

			double pty = (y+connectivity.get(s).size())/2;
			double ptx = 2.0;
			dataSet.addDataPoint(new SFEScatterPlotDataPoint(s, ptx, pty));
			seriesType.put(series, DATAPOINT_ROW);
			System.out.println("DATAPOINT: "+(series++)+" "+s);
			XYSeries connector = new XYSeries(s+"###"+(s));
			seriesType.put(series, CONNECTOR);
			System.out.println("CONNECTOR: "+(series++)+" "+s+"###"+(s));
			connector.add(x, y);
			connector.add(ptx, pty);
			dataSet.addConnector(connector);
			//			System.out.println("KEY: "+s);
			Set<String> set = connectivity.get(s);
			for (String v : set) {
				if (rowHeaders.contains(v))
					continue;
				dataSet.addDataPoint(new SFEScatterPlotDataPoint(v, x, y));
				seriesType.put(series, DATAPOINT_COL);
				System.out.println("DATAPOINT: "+(series++)+" "+v);

				connector = new XYSeries(v+"###"+(s));
				seriesType.put(series, CONNECTOR);
				System.out.println("CONNECTOR: "+(series++)+" "+v+"###"+(s));
				connector.add(x, y);
				connector.add(ptx, pty);
				dataSet.addConnector(connector);

				y++;
			}


			dataSet.addDataPoint(new SFEScatterPlotDataPoint("XXX", x+2.0, 1));
			seriesType.put(series, DATAPOINT_COL);
			System.out.println("DATAPOINT: "+(series++)+" "+"XXX");
			connector = new XYSeries("XXX###"+(s));
			seriesType.put(series, CONNECTOR);
			System.out.println("CONNECTOR: "+(series++)+" "+"XXX###"+(s));
			connector.add(x, y);
			connector.add(ptx, pty);
			dataSet.addConnector(connector);

			SFESimpleAnnotatedScatterPlot chart = new SFESimpleAnnotatedScatterPlot("title", 
					dataSet, 6.0, 40.0, seriesType, colorMap);

			chart.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			chart.pack();
			chart.setLocationRelativeTo(null);
			RefineryUtilities.centerFrameOnScreen(chart);
			chart.setVisible(false);

			Writer.renderPanelAndWrite(GraphTable.wrap(chart.getChart(), 
					PLOT_WIDTH, PLOT_HEIGHT), Integer.toString(fnum)+"_testVis.png");
			fnum++;
			
			break;

		}

	}
	
	
	public void rowToRowConnections(Map<String, Set<String>> connectivity ) {
		Set<String> hasInput = new LinkedHashSet<String>();
		Set<String> hasOutput = new LinkedHashSet<String>();

		for (String s : rowHeaders) {
			Set<String> set = connectivity.get(s);
			for (String s2 : connectivity.keySet()) {
				if (set.contains(s2)) {
					System.out.println(s+" <- "+s2);
					hasOutput.add(s2);
					hasInput.add(s);
				}
			}
		}

		//		private Set<String> noInputs;
		//		private Set<String> noOutputs;
		//		private Set<String> inputsOutputs;
		//		private Set<String> noConnectivity;

		for (String s : rowHeaders) {
			if (hasInput.contains(s) && hasOutput.contains(s)) {
				inputsOutputs.add(s);
			} else if (hasOutput.contains(s)) {
				noInputs.add(s);
			} else if (hasInput.contains(s)) {
				noOutputs.add(s);
			} else {
				noConnectivity.add(s);
			}
		}
	}
	
	Map<String, Set<String>> generateConnectivity(List<List<String>> table, int startRow, int startCol) {
		Map<String, Set<String>> connects = new HashMap<String, Set<String>>();
		
		columnHeaders = table.get(startRow);
		for (String ch : columnHeaders) {
			if (ch.replace("Title", "").trim().length() > 0)
				outputLinks.put(ch, new MutableInteger(0));
			if (!reverseConnectivity.containsKey(ch)) {
				reverseConnectivity.put(ch, new LinkedHashSet<String>());
			}
		}
		
		
		
		for (int r = startRow+1; r < table.size(); r++) {
			List<String> row = table.get(r);
			connects.put(row.get(startCol), new LinkedHashSet<String>());
			rowHeaders.add(row.get(startCol));
			Set<String> set = connects.get(table.get(r).get(startCol));
			for (int c = startCol+1; c < table.get(r).size(); c++) {
				if (row.get(c).trim().equalsIgnoreCase("X")) {
					set.add(columnHeaders.get(c));
					outputLinks.get(columnHeaders.get(c)).add(1);
					
					reverseConnectivity.get(columnHeaders.get(c)).add(row.get(startCol));
				}
			}
		}
		
		for (String ch : rowHeaders) {
			if (ch.replace("Title", "").trim().length() > 0) {
				if (!outputLinks.containsKey(ch))
					outputLinks.put(ch, new MutableInteger(0));
				if (!reverseConnectivity.containsKey(ch)) {
					reverseConnectivity.put(ch, new LinkedHashSet<String>());
				}
			}
		}
		
		return connects;
	}
	
	public List<String> sortAllByOutputLinkCount(final List<String> rowHeadersSorted) {
		
		List<String> al = new ArrayList<String>(all);
		Collections.sort(al, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				
				if (outputLinks.get(arg0) == null || outputLinks.get(arg1) == null) {
					System.out.println("output arg0: "+arg0+" "+outputLinks.get(arg0));
					System.out.println("output arg1: "+arg1+" "+outputLinks.get(arg1));
					return 0;
				}
				if (reverseConnectivity.get(arg0) == null || reverseConnectivity.get(arg1) == null) {
					System.out.println("reverse arg0: "+arg0+" "+reverseConnectivity.get(arg0));
					System.out.println("reverse arg1: "+arg1+" "+reverseConnectivity.get(arg1));
					return 0;
				}
				
				try {
					int cnt0 = outputLinks.get(arg0).value();
					int cnt1 = outputLinks.get(arg1).value();
					if (cnt0 == cnt1) {
						int header0 = 0;
						int header1 = 0;
						for (int i = 0; i < rowHeadersSorted.size(); i++) {
							String rheader = rowHeadersSorted.get(i);
							if (reverseConnectivity.get(arg0).contains(rheader)) 
								header0 = i;
							if (reverseConnectivity.get(arg1).contains(rheader)) 
								header1 = i;
						}
						return header0 == header1 ? 0 : header0 < header1 ? -1 : 1;
					} else {
						return cnt0 < cnt1 ? -1 : 1;
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return 0;
				}
			}
		});
		
		return al;
		
	}
	public List<String> sortRowHeadersByInputLinkCount() {
		
		List<String> al = new ArrayList<String>(rowHeaders);
		Collections.sort(al, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				try {
					int cnt0 = connectivity.get(arg0).size();
					int cnt1 = connectivity.get(arg1).size();
					return cnt0 == cnt1 ? 0 : cnt0 < cnt1 ? -1 : 11;
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					return 0;
				}
			}
		});
		
		return al;
	}
	
	public static void main(String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		
		try {
			final SampleVisualization viz = new SampleVisualization("Bolt_Mini_Database5-4-17-NB.xlsx", "Sheet1");
			viz.generate("SampleVis.png");
			System.exit(0);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(0);
		}
		
	}

}
