package gov.anl.gss.sdm.server.model.translator;

import gov.anl.gss.sdm.server.model.analysis.AnalysisManager;
import gov.anl.gss.sdm.server.model.analysis.polarity.VariableComplexity;
import gov.anl.gss.sdm.server.model.analysis.polarity.VariableComplexityData;
import gov.anl.gss.sdm.server.model.documentor.SDMDocLanguageSpecific;
import gov.anl.gss.sdm.server.model.documentor.VariableInformation;
import gov.anl.gss.sdm.server.model.documentor.plots.GetXlsLookups;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.StringConstants;
import gov.anl.gss.sdm.server.model.documentor.xmlwriter.variableselector.VariableSelector;
import gov.anl.gss.sdm.server.model.support.ArrayReference;
import gov.anl.gss.sdm.server.model.support.MemoryMapperInterface;
import gov.anl.gss.sdm.server.model.support.MutableBoolean;
import gov.anl.gss.sdm.server.model.support.MutableInteger;
import gov.anl.gss.sdm.server.model.support.NamedSubscriptManager;
import gov.anl.gss.sdm.server.model.support.SDFunctionsWithXLSColt;
import gov.anl.gss.sdm.server.model.support.SubscriptCombination;
import gov.anl.gss.sdm.server.model.support.TimeSeriesInstance;
import gov.anl.gss.sdm.server.model.support.Utilities;
import gov.anl.gss.sdm.server.model.tools.FileOrganizer;
import gov.anl.gss.sdm.server.model.xmile.utilities.FunctionMapping;

import java.awt.Cursor;
import java.io.BufferedWriter;
import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.apache.commons.lang3.mutable.MutableDouble;
import org.apache.log4j.Logger;

public class Equation {

	static Logger log = Logger.getLogger(Equation.class.getSimpleName());

	private EquationComponents equationComponents;
	private EquationCharacteristics equationCharacteristics;
	private VariableInformation variableInformation;
	private EquationVariable equationVariable;

//	private boolean nativeMemoryImplementation;

	private int vectorSpaceRequirements = 1;

	private String module = "Default";
	private String group = "Default";
	
	private Boolean exogenous = null;

	private int equationNumber = -1;
	private int variableNumber = -1;
	private int totalEquationCount = -1;

	private String vensimEquation;
	private String originalVensimEquation;
	private String equation;
	private List<String> multiEquations;
	private String cleanEquation = "";
	private String lhs;
	private String lhsAsInMdl;
	private String rhs;
	private HashSet<String> rhsTokens;
	private HashSet<String> rhsTokensStripped;
	private String units = null;
	private String comment = null;
	private String other;
	private HashSet<String> references;
	private HashSet<String> referencedBy;
	private Node treeRoot;
	
	private ArrayReference lhsArrayReference;
	
	private boolean doNotSimplifyLoops = false;
	
	private boolean includeInTranslation = true;

	private boolean syntacticallyCorrect = true;
	private boolean usageCorrect = true;
	private boolean fatal = false;

	private boolean abort = false;

	private boolean justMap = false;

	private boolean valueRangeSpecified = false;
	
	private int numInputLinks = 0;
	private int numOutputLinks = 0;

	private List<String> syntaxMessages = new ArrayList<String>();
	private List<String> usageMessages = new ArrayList<String>();
	private List<String> unitsMessages = new ArrayList<String>();
	private List<String> fatalMessages = new ArrayList<String>();

	private List<String> views; // = new ArrayList<String>();
	private Set<String> functionsCalled = new LinkedHashSet<String>();

	private OperationResult opRes = new OperationResult(); // reusable

	private boolean stockVariable = false;

	private String btwnMode = "INTERPOLATE";

	private boolean arrayInitialization = false;

	private static int nextArray = 0;
	private static int nextInt = 0;
	private static int nextInteger = 0;
	private static int nextTimeSeries = 0;

	private boolean oneTime = false;
	private boolean vdmLookup = false;

	private List<String> tokens = new ArrayList<String>();
	private List<String> rpn = new ArrayList<String>();

	private boolean getXls = false;

	private boolean definesLookup = false;
	private boolean definesLookupGetXls = false;
	private boolean lookupXlsReadable = false;
	private boolean definesLookupWithRange = false;
	private boolean referencesLookup = false;
	private boolean hasInitialValue = false;
	private boolean assignment = false;
	private boolean typeString = false;
	private boolean definesSubscript = false;
	private boolean usesInitialFunction = false;
	private boolean callsFunction = false;
	private boolean usesRangeFunction = false;

	private boolean activeInitial = false;
	private boolean sampleIfTrue = false;

	// special circumstances we want to track for documentation

	private boolean usesMinMax = false;
	private boolean usesIfThenElse = false;
	private boolean usesStepPulse = false;

	// START need to add code to initialize
	private boolean usesSmooth = false;
	private boolean smooth3 = false;
	private boolean smoothn = false;
	private int numSmooth = 0;
	private int numSmooth3 = 0;
	private int numSmoothN = 0;
	int nValue = 1;

	private boolean usesDelay = false;
	private boolean delay3 = false;
	private boolean delayn = false;
	
	private int numDelay = 0;
	private int numDelay3 = 0;
	private int numDelayN = 0;

	private boolean levelInitialization = false;
	private boolean initial = false;
	private boolean constant = false;
	private boolean flow = false;
	private boolean auxiliary = true;
	private boolean subscripts = false;
	private boolean data = false;
	private boolean game = false;

	private boolean undocumented = false;
	private boolean embeddedDataNot01 = false;
	private boolean incompletelyDefinedSubscript = false;
	private boolean nonmonotonicLookup = false;
	private boolean cascadingLookup = false;
	private boolean unused = true;
	private boolean supplementary = false;
	private boolean supplementaryUsed = false;
	private boolean embeddedData01 = false;
	private boolean complexFormulation = false;
	private int complexity;
	private boolean complexStock = false;

	private String complexFormulationCount = "";

	// END need to add code to initialize

	private boolean hasMacroInvocation = false;

	private boolean usesTimeSeries = false;

	private boolean hasException = false;
	private int leftBracketCount = 0;
	private List<String> exceptions;

	private boolean hasMultipleEquations = false;
	private String lhsArray;
	private List<String> lhsSubscripts = null;

	private boolean hasLHSArrayReference = false;
	private boolean hasRHSArrayReference = false;

	private EquationArrayReferenceStructure ears;

	private boolean orderedWithInitialValue = false;

	private boolean hasVectorSortOrder = false;
	private boolean hasVectorReorder = false;
	private boolean hasVectorElmMap = false;
	private boolean hasVectorSelect = false;
	private boolean hasVectorRank = false;
	
	private MutableInteger lookAhead1;

	public boolean isHasVectorRank() {
		return hasVectorRank;
	}

	public void setHasVectorRank(boolean hasVectorRank) {
		this.hasVectorRank = hasVectorRank;
	}

	private UnitExpression unitExpression;

	private boolean autoGenerated = false;

	private boolean treeCodeGenerated = false;
	private boolean ite = false;

	private static HashSet<Character> terminatorSet = new LinkedHashSet<Character>();
	private static char[] terminators = { '+', '-', '*', '/', '(', ')', '=',
			',', '^', '`', '>', '<', ':', ';' };
	private static boolean terminatorsInitialized = false;

	private static HashSet<String> lookupTables = new LinkedHashSet<String>();

	private static HashMap<String, Integer> precedence = new LinkedHashMap<String, Integer>();

	private static HashMap<String, Boolean> leftAssociative = new LinkedHashMap<String, Boolean>();

	static {
		precedence.put("_", 8);
		precedence.put("^", 7);
		precedence.put("*", 6);
		precedence.put("/", 6);
		precedence.put("+", 5);
		precedence.put("-", 5);
		precedence.put("==", 4);
		precedence.put("<>", 4);
		precedence.put(">", 4);
		precedence.put(">=", 4);
		precedence.put("<", 4);
		precedence.put("<=", 4);
		precedence.put(":AND:", 2);
		precedence.put(":OR:", 1);
		precedence.put(":NOT:", 3);

		leftAssociative.put("_", false);
		leftAssociative.put("^", false);
		leftAssociative.put("*", true);
		leftAssociative.put("/", true);
		leftAssociative.put("+", true);
		leftAssociative.put("-", true);
		leftAssociative.put("==", true);
		leftAssociative.put("<>", true);
		leftAssociative.put(">", true);
		leftAssociative.put(">=", true);
		leftAssociative.put("<", true);
		leftAssociative.put("<=", true);
		leftAssociative.put(":AND:", true);
		leftAssociative.put(":OR:", true);
		leftAssociative.put(":NOT:", false);

	}
	
	private String timeSeriesUpdate;
	
	public String getTimeSeriesUpdate() {
		return timeSeriesUpdate;
	}

	public void setTimeSeriesUpdate(String timeSeriesUpdate) {
		this.timeSeriesUpdate = timeSeriesUpdate;
	}

	public Equation() {
		references = new LinkedHashSet<String>();
		referencedBy = new LinkedHashSet<String>();
		rhsTokens = new LinkedHashSet<String>();
		views = new ArrayList<String>();
		views.add(" Not in View");

		if (!terminatorsInitialized)
			initializeTerminators();
	}

	private static void initializeTerminators() {
		for (char c : terminators) {
			terminatorSet.add(c);
		}

		terminatorsInitialized = true;
	}

	public static void initializeCounts() {
		nextArray = 0;
		nextInt = 0;
		nextInteger = 0;
		nextTimeSeries = 0;
	}

	// this is the entry point from all outside locations (Map)
	public Equation(EquationComponents equationComponents, boolean map) {
		this();

		
		// maps and only maps create their equation here
		this.equationComponents = equationComponents;
		units = equationComponents.getUnitsPart();
		comment = equationComponents.getCommentPart();
		vensimEquation = equationComponents.getVensimEquation();
		
		originalVensimEquation = equationComponents.getVensimEquation();
		
		justMap = !equationComponents.isHasEquation();
		
		System.out.println(vensimEquation);

		String ve = equationComponents.getEquationPart();
		if (ve.contains("=")) {
			String lhs = Parser.extractLHS(ve); // ve.split("=")[0].trim();
			if (ArrayReference.isArrayReference(lhs))
				lhs += "array.";
			setLhs(lhs);
			InformationManagers
					.getInstance()
					.getNativeDataTypeManager()
					.addVariable(this, this.getLhs(),
							typeString ? "String" : "double");
			
			// by definition, a map must have an equation of the form: a = A FUNCTION OF (a,b,c)
			// for exogenous variables analysis, we need to specify the number of known input links.
			String rhs = Parser.extractRHS(ve);
			if (containsInput()) {
				String[] parts = rhs.split(",");
				setNumInputLinks(parts.length);
			}
		}
		InformationManagers.getInstance().getEquationManager().registerEquation(this);
	}
	
	public boolean callsFunction(String func) {
		return functionsCalled.contains(func.replace(" ", ""));
	}
	
	private boolean containsInput() {
		return !vensimEquation.replace(" ", "").contains("AFUNCTIONOF()");
	}

	// this is the entry point from all outside locations
	public Equation(EquationComponents equationComponents) {

		this(equationComponents.getVensimEquation(), equationComponents, 0);

	}

	public Equation(EquationComponents equationComponents, int doNotRegister) {
		this(equationComponents.getVensimEquation(), equationComponents,
				doNotRegister);

	}

	public Equation(String vensimEquation,
			EquationComponents equationComponents, int doNotRegister) {
		this();
		this.equationComponents = equationComponents;

		this.hasMacroInvocation = equationComponents.isHasMacroInvocation();
		this.vensimEquation = vensimEquation.replaceAll("\t", "");
		this.originalVensimEquation = vensimEquation;

		// if any subscript ranges have blanks before the "!", get rid of them
		if (this.vensimEquation.contains("!")) {
			this.vensimEquation = vensimEquation.replaceAll(" *!", "!");
		}
		
		if (vensimEquation.startsWith("capital investment"))
			System.out.println("capital investment");

		// tokenize will detect certain types of syntax errors

		tokenize();
	
		setModule("Default");
		// if we were able to cleanly tokenize the equation, let's check for
		// other syntax errors

		if (tokens.size() == 0)
			log.debug("Tokens empty");
		if (syntacticallyCorrect) {
			GrammarChecker checker = new GrammarChecker(this, tokens);
			OperationResult or = checker.checkGrammar();
			if (!or.isOk()) {
				checker = new GrammarChecker(this, tokens);
				 or = checker.checkGrammar();
				setSyntacticallyCorrect(false);
				syntaxMessages.add(or.getMessage());
				log.debug("Bad Grammar Check: " + vensimEquation);
				log.debug(or.getMessage());
			}
		}
		
		if (syntacticallyCorrect) {

			// we are done, register this equation
			if (doNotRegister == 0) {
				InformationManagers.getInstance().getEquationManager().registerEquation(this);
			}

			generateTree();
		}
		
		equationVariable = new EquationVariable(this);
	}

	public boolean checkUsage(Map<String, Equation> equations) {
		OperationResult or = (new UsageChecker(equations, this)).checkUsage();
		if (!or.isOk()) {
			usageMessages.add(or.getMessage());
		}

		usageCorrect = or.isOk();
		return usageCorrect;

	}

	private String[] getPair(MutableInteger pos, char[] data) {
		String[] pair = new String[2];
		pair[0] = getNextNumber(pos, data);
		if (pair[0] == null)
			return null;
		pair[1] = getNextNumber(pos, data);
		if (pair[1] == null)
			return null;
		return pair;

	}

	private String getNextNumber(MutableInteger pos, char[] data) {

		String num = "";
		if (pos.value() >= data.length)
			return null;
		while (!(Character.isDigit(data[pos.value()])
				|| data[pos.value()] == '+' || data[pos.value()] == '-')) {
			pos.add(1);
			if (pos.value() >= data.length)
				return null;
		}

		num += data[pos.value()];
		pos.add(1);
		while (Character.isDigit(data[pos.value()]) || data[pos.value()] == '.') {
			num += data[pos.value()];
			pos.add(1);
		}

		return num;
	}

	private void createEars() {
		ears = new EquationArrayReferenceStructure(this);
	}

	private String processPairs(String lookupData) {

		String minX = "", minXY = "", maxX = "", maxXY = "";
		StringBuilder x = new StringBuilder();
		StringBuilder y = new StringBuilder();
		MutableInteger pos = new MutableInteger(0);
		String[] pair = null;
		int numValues = 0;

		// [(0,0)-(1,1)],(0,0),(0.1,0.2),(0.2,0.38),(0.3,0.55),(0.4,0.69),(0.5,0.8),(0.6,0.88),(0.7,0.94),(0.8,0.98),(0.9,0.995),(1,1),(1e+009,1)
		// need to worry about pairs with with no range designation
		// by this time, the blanks have been removed so we can check for ")-("

		char[] lookupDataChars = lookupData.replace(")-(", ")(").toCharArray();

		if (definesLookupWithRange) {
			pair = getPair(pos, lookupDataChars);
			minX = Parser.forceDouble(pair[0]);
			minXY = Parser.forceDouble(pair[1]);

			pair = getPair(pos, lookupDataChars);
			maxX = Parser.forceDouble(pair[0]);
			maxXY = Parser.forceDouble(pair[1]);
		}

		while (pos.value() < lookupDataChars.length) {
			pair = getPair(pos, lookupDataChars);
			if (pair == null)
				break;
			numValues++;
			x.append("," + Parser.forceDouble(pair[0]));
			y.append("," + Parser.forceDouble(pair[1]));
		}

		vectorSpaceRequirements = numValues * 2;

		if (definesLookupWithRange) {
			return numValues + "," + minX + "," + minXY + "," + maxX + ","
					+ maxXY + x.toString() + y.toString();
		} else {
			return numValues + "," + "Double.MIN_VALUE" + ","
					+ "Double.MIN_VALUE" + "," + "Double.MAX_VALUE" + ","
					+ "Double.MAX_VALUE" + x.toString() + y.toString();
//			return numValues +  x.toString() + y.toString();
		}

	}

	public void processExponentiation() {
		cleanEquation = processExponentiation(cleanEquation);
	}

	public String processExponentiation(String eqn) {

		if (eqn.contains("^")) {
			// printTokens();

			int pos = eqn.indexOf("^");

			// find lhs
			// return l will be the first character position of the begining of
			// the lhs
			MutableInteger l = new MutableInteger(pos - 1);
			String lhs = getExpLHS(l, eqn);

			// returning r will be the position of the last position of the rhs
			MutableInteger r = new MutableInteger(pos + 1);
			String rhs = getExpRHS(r, eqn);

			eqn = eqn.substring(0, l.value()) + "Math.pow(" + lhs + "," + rhs
					+ ")" + eqn.substring(r.value() + 1);
			return processExponentiation(eqn);

		}
		return eqn;
	}

	private String getExpLHS(MutableInteger r, String eqn) {
		String c;
		c = eqn.substring(r.value(), r.value() + 1);

		String rhs = "";
		// check if it is paren
		if (c.equals(")")) {
			rhs = toOpeningParen(r, eqn);
		} else {
			// not a paren
			// or a number
			// or a variable
			rhs = toBeginningOfLHS(r, eqn);
		}
		return reverse(rhs);
	}

	private String getExpRHS(MutableInteger r, String eqn) {

		String c;
		c = eqn.substring(r.value(), r.value() + 1);

		String rhs = "";
		// check if it is paren
		if (c.equals("(")) {
			rhs = toClosingParen(r, eqn);
		} else {
			// not a paren
			// could be a function invocation
			// or a number
			// or a variable
			rhs = toEndOfRHS(r, eqn);
		}
		return rhs;
	}

	/**
	 * 
	 * @param r
	 * @param eqn
	 * @return
	 */

	public String toBeginningOfLHS(MutableInteger r, String eqn) {
		String rhs = "";

		String c;
		c = eqn.substring(r.value(), r.value() + 1);

		// is it number?
		// must be among: [0-9]+-. // we will assume that that the vensim model
		// compiles
		// don't need to worry about exponential notation

		if (Character.isDigit(c.toCharArray()[0])) {
			rhs = c;
			r.add(-1);
			c = eqn.substring(r.value(), r.value() + 1);
			while (c.equals(".") || Character.isDigit(c.toCharArray()[0])) {
				rhs += c;
				r.add(-1);
				c = eqn.substring(r.value(), r.value() + 1);
			}
			r.add(1);
		} else {
			// must be either a variable or a function call
			rhs = c;
			r.add(-1);
			c = eqn.substring(r.value(), r.value() + 1);
			while (c.equals("_")
					|| Character.isLetterOrDigit(c.toCharArray()[0])) {
				rhs += c;
				r.add(-1);
				c = eqn.substring(r.value(), r.value() + 1);
			}
			r.add(1);
		}
		return rhs;
	}

	public String toEndOfRHS(MutableInteger r, String eqn) {
		String rhs = "";

		String c;
		c = eqn.substring(r.value(), r.value() + 1);

		// is it number?
		// must be among: [0-9]+-. // we will assume that that the vensim model
		// compiles
		// don't need to worry about exponential notation

		if (c.equals("-") || c.equals("+") || c.equals(".")
				|| Character.isDigit(c.toCharArray()[0])) {
			rhs = c;
			r.add(1);
			// need to check if r points past end of eqn
			if (r.value() < eqn.length()) {
				c = eqn.substring(r.value(), r.value() + 1);
				while ((c.equals(".") || Character.isDigit(c.toCharArray()[0]))
						&& r.value() < eqn.length()) {
					rhs += c;
					r.add(1);
					if (r.value() < eqn.length())
						c = eqn.substring(r.value(), r.value() + 1);
				}
			}
			r.add(-1);
		} else {
			// must be either a variable or a function call
			rhs = c;
			r.add(1);
			c = eqn.substring(r.value(), r.value() + 1);
			while ((c.equals("_") || Character
					.isLetterOrDigit(c.toCharArray()[0]))
					&& r.value() < eqn.length()) {
				rhs += c;
				r.add(1);
				if (r.value() < eqn.length())
					c = eqn.substring(r.value(), r.value() + 1);
			}
			if (c.equals("(")) {
				rhs += toClosingParen(r, eqn);
			} else {
				r.add(-1);
			}
		}
		return rhs;
	}

	public String reverse(String s) {
		String rev = "";

		char[] chars = s.toCharArray();
		for (int i = chars.length - 1; i >= 0; i--)
			rev += chars[i];

		return rev;
	}

	public String toClosingParen(MutableInteger r, String eqn) {
		String c;
		c = eqn.substring(r.value(), r.value() + 1);
		String rhs = c;
		// find matching paren
		int paren = 1;
		while (paren > 0) {
			r.add(1);
			c = eqn.substring(r.value(), r.value() + 1);
			rhs += c;
			if (c.equals(")"))
				paren--;
			else if (c.equals("("))
				paren++;
		}
		return rhs;
	}

	public String toOpeningParen(MutableInteger r, String eqn) {
		String c;
		c = eqn.substring(r.value(), r.value() + 1);
		String rhs = c;
		// find matching parent
		int paren = 1;
		while (paren > 0) {
			r.add(-1);
			c = eqn.substring(r.value(), r.value() + 1);
			rhs += c;
			if (c.equals(")"))
				paren++;
			else if (c.equals("("))
				paren--;
		}
		r.add(-1);
		c = eqn.substring(r.value(), r.value() + 1);
		if (c.equals("_") || Character.isLetterOrDigit(c.toCharArray()[0])) {
			rhs += toFunctionStart(r, eqn);
		} else {
			r.add(1); // restore pointer
		}
		return rhs;
	}

	public String toFunctionStart(MutableInteger r, String eqn) {
		String rhs = "";
		String c;

		c = eqn.substring(r.value(), r.value() + 1);
		while (c.equals("_") || Character.isLetterOrDigit(c.toCharArray()[0])) {
			rhs += c;
			r.add(-1);
			c = eqn.substring(r.value(), r.value() + 1);
		}

		r.add(1);
		return rhs;
	}

	public void processSubscriptDefinition() {
		if (definesSubscript) {
			// registerSubscript(subscript, String ...)
			StringBuilder sb = new StringBuilder("\tregisterSubscript(\""
					+ tokens.get(0) + "\",");
			for (int i = 1; i < tokens.size(); i++) {
				String token = tokens.get(i);
				if (token.equals(","))
					sb.append(token);
				else
					sb.append("\"" + token + "\"");
			}
			sb.append(")");
			cleanEquation = sb.toString();
			setOneTime(true);
		}
	}

	private void createLookupLists(List<Double> x, List<Double> y,
			MutableDouble minX, MutableDouble maxX, MutableDouble minY,
			MutableDouble maxY) {

		String[] lookup = cleanEquation.split("\\(", 2);

		// the specification of the lookup data can appear as
		// [(0,0)-(1,1)],(0,0),(0.1,0.2),(0.2,0.38),(0.3,0.55),(0.4,0.69),(0.5,0.8),(0.6,0.88),(0.7,0.94),(0.8,0.98),(0.9,0.995),(1,1),(1e+009,1)
					// where [] defines range and then pairs of data points
		

		// cannot just split on comma, comma may appear in quoted variable namew
		// :-(
		
		if (lookup[0].contains("ADDLOOKUPPAIRSNATIVE")) {

//		if (!lookup[1].contains("(")) {
			int skip = 6;
			String[] allValues = Parser.splitQuoted(lookup[1], ",");
			int numValues = (allValues.length - skip) / 2;

			for (int i = 0 + skip; i < numValues + skip; i++) {
				x.add(new Double(allValues[i].trim()));
			}
			for (int i = numValues + skip; i < allValues.length; i++) {
				y.add(new Double(allValues[i].replace(")", "").trim()));
			}

			if (minX != null) {
				if (allValues[2].contains("Double.")) {
					minX.setValue(Double.MIN_VALUE);
					maxX.setValue(Double.MAX_VALUE);
					minY.setValue(Double.MIN_VALUE);
					maxY.setValue(Double.MAX_VALUE);
				} else {
					minX.setValue(Double.parseDouble(allValues[2].trim()));
					maxX.setValue(Double.parseDouble(allValues[4].trim()));
					minY.setValue(Double.parseDouble(allValues[3].trim()));
					maxY.setValue(Double.parseDouble(allValues[5].trim()));
				}
			}
		} else {
			// or
			// x[0],x[n],y[0],,,y[n]
			int skip = 2; // skip count and name
			String[] allValues = Parser.splitQuoted(lookup[1],",");
			int numValues = Integer.parseInt(allValues[1]); 

			for (int i = 0 + skip; i < numValues + skip; i++) {
				try {
					x.add(new Double(allValues[i].trim()));
				} catch (NumberFormatException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			for (int i = numValues + skip; i < allValues.length; i++) {
				try {
					y.add(new Double(allValues[i].trim().replace(")", "")));
				} catch (NumberFormatException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		// log.debug(cleanEquation);
		// log.debug(getVensimEquationOnly());
		// log.debug(lookup[1]);

	}

	public String[] splitLookup(String eqn) {
		String[] parts = new String[2];
		parts[0] = new String(lhs);
		parts[1] = eqn.replace(lhs, "").replaceAll("^ *\\(", "");
		return parts;
	}
	
	private void registerArrayLookup() {
		registerArrayLookup(getOriginalLhs());
	}
	
	private void registerArrayLookup(String array) {
		if (getVensimEquationOnly().toUpperCase().contains("GET XLS LOOKUPS")) {
			NativeDataTypeManager ndtm = InformationManagers
					.getInstance()
					.getNativeDataTypeManager();
			ArrayManager am = InformationManagers
					.getInstance()
					.getArrayManager();
			am.arrayReference(array, new ArrayList<String>());
			if (!ndtm.isArray(array)) {
				String definition = array;
				ndtm.addVariableAsArray(this, definition,typeString ? "String" : "double");
			} else {
				int numOrigDim = am.getNumDimensions(array);
				int numThisDim = 2;
				if (ArrayReference.isArrayReference(array)) {
					numThisDim += new ArrayReference(array).getSubscripts().size();
				}
				if (numThisDim - numOrigDim != 2) {
					System.out.println("registerArrayLookup already registered "+array);
				}
			}
		}
	}

	public void processLookup() {
		// first change the statement if it is a lookup table definition
		// sample:
		// foodpollutionmulttab(0,10,20,30,40,50,60,1.02,0.9,0.65,0.35,0.2,0.1,0.05);
		// translate to VensimFunctions.addLookup("foodpollutionmulttab",
		// 0,10,20,30,40,50,60,1.02,0.9,0.65,0.35,0.2,0.1,0.05);
		

		
		if (this.isGetExternalData()) {
//			if (getVensimEquationOnly().toUpperCase().contains("GET XLS LOOKUPS")) {
//				String definition = getOriginalLhs()+"[holder1,holder2]";
//				InformationManagers
//				.getInstance()
//				.getNativeDataTypeManager()
//				.addVariable(this, definition,
//						typeString ? "String" : "double");
//			}
			return;
		}
		if (definesLookup) {
			// THIS HANDLES ONLY INLINE DATA LOOKUPS

			// String[] lookup = cleanEquation.split("\\(", 2);
			String[] lookup = splitLookup(cleanEquation);
			String alternateName = Utilities.makeLegalJavaName(lookup[0]);
			String definition = lookup[0];
			definition = alternateName;
			
			if (definition.contains("]")) {
				definition = definition.replace("]", "") + ",holder1,holder2]";
			} else {
				definition = definition + "[holder1,holder2]";
			}

			// log.debug("Process Lookup add variable");
			InformationManagers
					.getInstance()
					.getNativeDataTypeManager()
					.addVariable(this, definition,
							typeString ? "String" : "double");

			// the specification of the lookup data can appear as
			// x[0],x[n],y[0],,,y[n]

			if (!lookup[1].contains("(")) {
				int numValues = lookup[1].split(",").length / 2;
//				if (nativeMemoryImplementation) {
					cleanEquation = "sdFunctions.ADDLOOKUPNATIVE(\""
							+ /*lookup[0]*/ alternateName + "\", " + numValues + "," + lookup[1];
//				} else {
//					String name = InformationManagers.getInstance()
//							.getNativeDataTypeManager()
//							.getOriginalName(lookup[0]);
//					cleanEquation = "sdFunctions.ADDLOOKUPNATIVE(\"" + name
//							+ "\", " + numValues + "," + lookup[1];
//				}
			} else {
				// or
				// [(0,0)-(1,1)],(0,0),(0.1,0.2),(0.2,0.38),(0.3,0.55),(0.4,0.69),(0.5,0.8),(0.6,0.88),(0.7,0.94),(0.8,0.98),(0.9,0.995),(1,1),(1e+009,1)
				// where [] defines range and then pairs of data points
//				if (nativeMemoryImplementation) {
					cleanEquation = "sdFunctions.ADDLOOKUPPAIRSNATIVE(\""
							+ /*lookup[0]*/ alternateName + "\", " + processPairs(lookup[1])
							+ ")";
//				} else {
//					String name = InformationManagers.getInstance()
//							.getNativeDataTypeManager()
//							.getOriginalName(lookup[0]);
//					cleanEquation = "sdFunctions.ADDLOOKUPPAIRSNATIVE(\""
//							+ name + "\", " + processPairs(lookup[1]) + ")";
//				}
			}

			setOneTime(true);
		}

		// if not a definition, then a reference
		// sample: birthscrowdingmultiplier=birthscrowdingmulttab(crowding);
		// translate to:
		// birthscrowdingmultiplier=VensimFunctions.LOOKUP("birthscrowdingmulttab",
		// crowding);
		// need to locate the lookup and then the

		// Need to worry about subscripted lookup tables that are defined with
		// static subscripts while access
		// is via named subscripts with multiple terminal values.
		// sample: x[y]( (1,2), (2,3)...)
		// ref via: x[named](q) ->
		// sdFunctions.LOOKUP("x"+"["+concatAsSubscript(outerSub.getSubscriptValue("named"))+"]",
		// (q)
		// can also have x[named!](q) ->
		// sdFunctions.LOOKUP("x"+"["+concatAsSubscript(rangeSub.getSubscriptValue("named"))+"]",
		// (q)
		// y[z,named!] -
		// sdFunctions.LOOKUP("y"+"["+concatAsSubscript(outerSub.getSubscriptValue("z"),
		// rangeSub.getSubscriptValue("named"))+"]", (q)
		// we can detect this and generate appropriate subscript code since the
		// we will know outer and range
		// subscripts

		// work directly with tokens
		if (referencesLookup)
			return;
		if (referencesLookup) {
			for (String tableReference : lookupTables) {
				if (cleanEquation.contains(tableReference)) {
					cleanEquation = cleanEquation.replace(tableReference + "(",
							"sdFunctions.LOOKUP(\"" + tableReference + "\",");
					ArrayList<String> newTokens = new ArrayList<String>();
					int skipParenAt = -1;
					for (int i = 0; i < tokens.size(); i++) {
						String t = tokens.get(i);
						if (t.equals(tableReference.replaceAll("\"", ""))) {
							tokens.set(i, "sdFunctions.LOOKUP(\""
									+ tableReference + "\",");
							newTokens.add("sdFunctions.LOOKUP");
							newTokens.add("(");
							newTokens.add("\"" + tableReference + "\"");
							skipParenAt = i + 1;
						} else {
							if (!(t.equals("(") && i == skipParenAt))
								newTokens.add(t);
						}
					}
					tokens = newTokens;
					// printTokens();
				}

			}

			// code for array references -- none will be processed by above if
			// named are used on lhs

			int tokNum = 0;
			boolean skipParen = false;
			ArrayList<String> newTokens = new ArrayList<String>();
			for (int i = 0; i < tokens.size(); i++) {
				String token = tokens.get(i);
				if (token.equals("(") && skipParen) {
					skipParen = false;
					continue;
				}
				// if (isArrayLookupWithSubscript(token)) {
				//
				// // the access is as defined -- subscript matches
				//
				// log.debug("isArrayLookupWithSubscript "+token);
				// ArrayReference ar = new ArrayReference(token);
				// List<String> subscripts = ar.getSubscripts();
				// List<String> rangeSubscripts = ar.getRangeSubscriptsNames();
				// List<String> nonRangeSubscripts = ar.getNonRangeSubscripts();
				// // sample: x[y]( (1,2), (2,3)...)
				// // x[y] -
				// sdFunctions.LOOKUP("x"+"["+concatAsSubscript(outerSub.getSubscriptValue("y"))+"]",
				// (q)
				// StringBuilder sb = new StringBuilder();
				// sb.append("sdFunctions.LOOKUP(\""+token+"\"");
				// }
				if (isArrayLookup(token)) {
					// log.debug("isArrayLookup "+token);

					// the access is different that as defined i.e. terminal
					// definition
					// named subscript access.

					ArrayReference ar = new ArrayReference(token);
					List<String> subscripts = ar.getSubscripts();
					List<String> rangeSubscripts = ar.getRangeSubscriptsNames();
					List<String> nonRangeSubscripts = ar
							.getNonRangeSubscripts();
					// sample: x[y]( (1,2), (2,3)...)
					// ref via: x[named](q) ->
					// sdFunctions.LOOKUP("x"+"["+concatAsSubscript(outerSub.getSubscriptValue("named"))+"]",
					// (q)
					// y[z,named!] -
					// sdFunctions.LOOKUP("y"+"["+concatAsSubscript(outerSub.getSubscriptValue("z"),
					// rangeSub.getSubscriptValue("named"))+"]", (q)
					// Each array lookup can be different in equations

					newTokens.add("sdFunctions.LOOKUP");
					newTokens.add("(");

					skipParen = true;
					StringBuilder sb = new StringBuilder();
					// sb.append("sdFunctions.LOOKUP(\""+token+"\""); // token
					// -> is array name of lookup
					sb.append("\"" + token + "+[\"+concatAsSubscriptEQUATION("); // start
																					// of
																					// subscript
																					// builder
					int subr = 0;
					for (String sub : subscripts) {
						if (subr++ > 0)
							sb.append(",");
						if (nonRangeSubscripts.contains(sub)) {
							sb.append("outerSub.getSubscriptValue(\"" + sub
									+ "\")");
						} else {
							sb.append("rangeSub.getSubscriptValue(\"" + sub
									+ "\")");
						}
					}
					sb.append(")+\"]\"");

					newTokens.add(sb.toString());
					newTokens.add(",");

					// Here!
				} else {
					newTokens.add(token);
				}
				tokNum++;
			}
			tokens = newTokens;
		}

	}
	
	public int getTypeSortValue() {
		if (isStock())
			return 0;
		if (isFlow())
			return 1;
		if (isAuxiliary())
			return 2;
		return 3;
	}

	private boolean isArrayLookupWithSubscript(String s) {
		if (!ArrayReference.isArrayReference(s))
			return false;
		// have an array reference, is it a lookup?
		if (lookupTables.contains(s.replace("array.", "")))
			return true;
		else
			return false;
	}

	private boolean isArrayLookup(String s) {
		if (!ArrayReference.isArrayReference(s))
			return false;
		// have an array reference, is it a lookup?
		if (getLookupArrays().contains(
				new ArrayReference(s.replace("array.", "")).getArrayName()))
			return true;
		else
			return false;
	}

	private Set<String> getLookupArrays() {
		Set<String> lookupArrays = new LinkedHashSet<String>();

		for (String lookup : lookupTables) {
			if (ArrayReference.isArrayReference(lookup))
				lookupArrays.add(new ArrayReference(lookup).getArrayName());
		}

		return lookupArrays;
	}

	private static String convertExponentialNotation(String equation) {

		// if (equation == null)
		// log.debug("equation is null");
		String noExp = new String(equation);

		String notation = "[-]?[0-9]?\\.?[0-9]+[Ee][-+]?[0-9]+";
		Pattern p = Pattern.compile(notation);
		Matcher m = p.matcher(equation);
		while (m.find()) {
			String not = m.group();

			try {
				BigDecimal bd = null;
				bd = BigDecimal.valueOf(Double.parseDouble(not));
				String bdString = bd.toPlainString();
				if (!bdString.contains("."))
					bdString = bdString + ".0";
				noExp = noExp.replaceFirst(not.replace("+", "\\+"), bdString);
			} catch (NumberFormatException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return noExp;
	}

	private boolean canRemoveColonEqual() {

		String[] btwnModes = { ":INTERPOLATE:", ":LOOK FORWARD:",
				":HOLD BACKWORD:", ":RAW:" };

		boolean canRemove = true;

		boolean containsColonEqual = equation.contains(":=");
		if (!containsColonEqual)
			return canRemove;

		String tempEqn = new String(equation);

		for (String mode : btwnModes) {
			if (tempEqn.contains(mode)) {
				tempEqn = tempEqn.replace(mode, "");
				return tempEqn.contains(":=");
			}
		}

		return canRemove;

	}
	


	private void tokenize() {
		
		MutableInteger position = new MutableInteger(0);

		// is this a string assignment?
		if (vensimEquation.contains(":IS:")) {
			vensimEquation = vensimEquation.replace(":IS:", "=");
			typeString = true;
		}

		// reference predefined variables?
		if (vensimEquation.contains(":NA:")) {
			vensimEquation = vensimEquation.replaceAll(":NA:", "NAREPLACEMENT");
		}

		// equation consists of three parts
		String[] eqn = vensimEquation.split("~");


		if (containsInterpolationMode(eqn[0])) {
			btwnMode = getInterpolationMode(eqn[0]);
//			eqn[0] = removeInterpolationMode(eqn[0]);
		}

		// if no rhs is evident, then need to get data from external source
		if (eqn[0].endsWith("=")) {
//			eqn[0] = eqn[0] + " "+Parser.externalDataMethod()+"(\""	+ eqn[0].replace("=", "").trim() + "\")";
			eqn[0] = eqn[0] + " "+Parser.externalDataMethod()+"(1.0)";
			setVdmLookup(true);
		}
		if (endsWithInterpolationMode(eqn[0])) {
//			eqn[0] = removeInterpolationMode(eqn[0]) + " = "+ " "+Parser.externalDataMethod()+"(\""
//					+ removeInterpolationMode(eqn[0]).replace("=", "").trim() + "\")";
			eqn[0] = removeInterpolationMode(eqn[0]) + " = "+ " "+Parser.externalDataMethod()+"(2.0)";			
			setVdmLookup(true);
		}
		// if no = or no : assume it is from an external source

		if (!eqn[0].contains(":") && !eqn[0].contains("=") && !eqn[0].contains(")") && !eqn[0].contains("(") &&
				!eqn[0].contains("<->")) {
//			eqn[0] = removeInterpolationMode(eqn[0]) + " = "+ " "+Parser.externalDataMethod()+"(\""
//					+ removeInterpolationMode(eqn[0]).replace("=", "").trim() + "\")";
			eqn[0] = removeInterpolationMode(eqn[0]) + " = "+ " "+Parser.externalDataMethod()+"(3.0)";	
			setVdmLookup(true);
		}

		// do some preliminary cleanup
		// replace := with = not that options such as :INTERPOLATE: can occur on
		// the LHS
		equation = eqn[0];

		if (equation.contains("=")) {

			rhs = equation.split("=", 2)[1];
		}

		if (canRemoveColonEqual())
			equation = eqn[0].replace(":=", "=").trim();
		if (eqn.length > 1)
			units = eqn[1].replaceAll("  *", " ").trim();
		if (eqn.length > 2)
			comment = eqn[2].replaceAll("  *", " ").trim();

		// log.debug("New Equation: "+equation);

		// TODO: delay subscript processing until code generation --

		// convert scientific notation

		equation = convertExponentialNotation(equation);

		// first process the LHS
		// if this is an assignment statement, then there will be a right had
		// side.
		// the only statement that will not have a RHS is the definition of a
		// lookup table
		// or a subscript definition

		boolean hasRHS = tokenizeLHS(position);

		// was LHS syntactically correct?
		if (!this.isSyntacticallyCorrect()) {
			// abort tokenizing -- may disrupt future processing
			return;
		}

		// take care of the right hand side
		
		if (hasRHS) {
			tokenizeRHS(position);
			if (!this.isSyntacticallyCorrect()) {
				// abort tokenizing -- may disrupt future processing
				return;
			}
		}

		if (this.isDefinesLookup()) {
			this.processLookup();

			if (!this.isSyntacticallyCorrect()) {
				// abort tokenizing -- may disrupt future processing
				return;
			}
		}

		if (this.isAssignment()) {
			// log.debug("Adding from Equation "+getLhs());
			InformationManagers
			.getInstance()
			.getNativeDataTypeManager()
			.addVariable(this, this.getLhs(),
					typeString ? "String" : "double");
		}

		// set characteristics of this equation for documentation purposes

		// only for assignment, data initialization
		if (isAssignment() || isDefinesLookup()) {

			// these are set elsewhere
			// private boolean usesSmooth = false;
			// private boolean usesDelay = false;

			// not sure what these represent
			// private boolean levelInitialization = false;
			// private boolean initial = false;

			// private boolean constant = false;
			if (isOneTime() && !isDefinesLookup())
				constant = true;

			// private boolean flow = false;
			// private boolean auxiliary = false;

			// private boolean subscripts = false;
			if (hasLHSArrayReference)
				subscripts = true;

			// private boolean data = false;
			// private boolean game = false;
			//
			// private boolean undocumented = false;
			undocumented = comment == null || comment.length() == 0;
			// private boolean embeddedDataNot01 = false;
			// private boolean embeddedData01 = false;

			if (isAssignment() && !isConstant()) {
				for (String token : deriveRhsTokens(tokens)) {
					if (Parser.isNumber(token)) {
						double number = Double.parseDouble(token);
						if (number == 0.0 || number == 1.0) {
							embeddedData01 = true;

							// special case: hardcoded 0 in integ initialization
							// argument
							// always mark this as embeddedDataNot01
							if (isStock()) {
								String init = getInitialValue();
								if (Parser.isNumber(init)) {
									number = Double.parseDouble(init);
									if (number == 0.0)
										embeddedDataNot01 = true;
								}
							}
							if (isGame()) {
								embeddedDataNot01 = true;
							}

						} else {
							embeddedDataNot01 = true;
						}
					}
				}
			}

			// private boolean incompletelyDefinedSubscript = false;

			// private boolean nonmonotonicLookup = false;
			nonmonotonicLookup = isNonmonotonicLookup();
			// private boolean cascadingLookup = false;
			// cascadingLookup = isCascadingLookup();
			// private boolean unused = false;

			// private boolean supplementary = false;
			supplementary = this.equationComponents.isSupplementary();
			// private boolean supplementaryUsed = false;
			supplementaryUsed = supplementary && !unused;
			// private boolean complexFormulation = false;
			if (isAssignment()) {
				//				if (lhsAsInMdl.startsWith("prio "))
				//					System.out.println("prio ");
				complexFormulation = true;

				//				complexFormulationCount = Integer
				//						.toString(getRHSVariablesForOrdering().size());
				//				complexity = getRHSVariablesForOrdering().size();

				complexFormulationCount = Integer
						.toString(rhsTokens.size());
				complexity = rhsTokens.size();
				// }
				// private boolean complexStock = false;
				try {
					if (isStock() && getTreeRoot() != null) {
						Node root = getTreeRoot();
						Node rhs = TreeTraversal.getRhs(root);
						Node arg1 = TreeTraversal.getFunctionArgument(rhs, 1);
						List<Node> nodeList = TreeTraversal.getSubTreeAsList(arg1);
						if (nodeList == null) {
							//							System.out.println("Huh?");
						} else {
							for (Node node : nodeList) {

								String token = node.getToken();
								if (Parser.isOperator(token)) {
									if (token.equals("+")
											|| token.equals("-")
											|| token.equals(Parser.INTERNAL_UNARY_MINUS))
										continue;
									complexStock = true;
									break;
								}
							}
						}
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		}

		// a final check -- make sure that there are not 2 consecutive commas --
		// indicates missing param
		// probably due to not specifying an equation for the stock

		for (int i = 1; i < tokens.size(); i++) {
			if (tokens.get(i - 1).equals(",") && tokens.get(i).equals(",")) {
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages
				.add("Parsing Equation: Missing argument to function invocation");
			}
		}
	}

	private List<String> deriveRhsTokens(List<String> tokens) {
		List<String> rhsTokens = new ArrayList<String>();

		int i = 0;
		while (tokens.get(i++) != "=")
			;
		for (; i < tokens.size(); i++)
			rhsTokens.add(tokens.get(i));

		return rhsTokens;
	}

	private boolean tokenizeLHS(MutableInteger position) {

		boolean hasRHS = true;
		
		// this is used solely for detecting subscripted lookups and should
		// not be used for any other purpose!

		boolean hasEqualSign = equation.contains("=");

		String token = "";
		skipWhiteSpace(position);

		if (!inRange(position)) {
			syntacticallyCorrect = false;
			setSyntacticallyCorrect(false);
			syntaxMessages.add("Parsing LHS: No Equation provided "
					+ vensimEquation);
			lhs = vensimEquation;
			return false;
		}

		if (characterAt(position).equals("\"")) {
			// quoted string -- either a variable name or a lookup table name
			token = getQuotedStringStartingAt(position, opRes);
			lhsAsInMdl = "\"" + token + "\"";
			if (!opRes.isOk()) {
				// failure finding closing quote
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add("Parsing LHS: " + opRes.getMessage());
				return false;
			}
		} else {
			// non quoted String
			boolean allowColon = false;
			token = getStringStartingAt(position, allowColon, opRes);
//			token = getNonQuotedStringStartingAt(position, allowColon, opRes);
			lhsAsInMdl = token;
		}

		skipWhiteSpace(position);

		cleanEquation = token;

		lhs = token; // NativeDataTypeManager.getLegalName(token);

		boolean ignore = true;
		
		// need to determine what we have based on next token "(", "=", or "["
		// -- array processing for later
		if (inRange(position) && characterAt(position).equals("=")) {

			// log.debug("Think this is assignment");

			// need to check if this is a "==" for an unchangeable variable
			// in the immediate future, this restriction will not be applied

			if (inRange(position.value() + 1)
					&& characterAt(position.value() + 1).equals("=")) {
				position.add(1);
			}
			tokens.add(InformationManagers.getInstance()
					.getNativeDataTypeManager().getLegalName(token));
			assignment = true;
			tokens.add("=");
			cleanEquation += "=";
			ignore = false;
		} else if (inRange(position) && characterAt(position).equals("(")) {
			// log.debug("Think this is lookup "+this.getVensimEquationOnly());
			// need to tokenize the remaining characters in the lookup
			// definition
			ignore = false;
			tokens.add(token);
			ArrayList<String> tokenizedLookup;
			// what format of lookup are we defining?
			// need to detect "tokenizeWithLookup" case when there is no range
			// "[()-()]" included
			// need to look ahead in the equation past position to see if a "("
			// exists
			if (equation.contains("[")
					|| beyondPosition(position).contains("(")) {
				tokenizedLookup = tokenizeWithLookup(position, opRes);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(opRes.getMessage());
					return false;
				}
			} else {
				tokenizedLookup = tokenizeLookup(position, opRes);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(opRes.getMessage());
					return false;
				}
			}
			tokens.addAll(tokenizedLookup);
			for (String s : tokenizedLookup)
				cleanEquation += s;
			hasRHS = false;
			definesLookup = true;
			
			// can we just legalaize token at this point?
			token = Utilities.makeLegalJavaName(token);
			
			registerArrayLookup(token);
			
			
			InformationManagers.getInstance().getArrayManager()
					.setUsedAsLookup(token); // BAD BAD BAD can we trick code?
			lookupTables.add(token); // used to be clean(token)
			EquationProcessor.lookups.add(token); // used to be clean(token)

		} else if (inRange(position) && characterAt(position).equals("[")
				&& !hasEqualSign && equation.contains("(")) {

			// log.debug("Think this is a subscripted lookup definition: "+
			// vensimEquation);
			// as of 8 Dec 2011, I believe that only a subscripted lookup
			// definition can have this pattern

			// need to gobble up the square brackets and subscript name and make
			// then part of the first
			// token as we try to fake out subscripted lookups without actually
			// storing as arrays

			// something like abc[def]((22,22)...)
			String pastLeftBracket = equation.split("\\[", 2)[1].trim();
			String subscript = pastLeftBracket.split("\\]", 2)[0].trim();
			String origLHS = token;
			token += "[" + subscript + "]";

			// MJB 9/11/2012 Problem that lhs is defined improperly

			setLhs(token);

			// position to the opening left paren
			boolean positioned = false;
			while (inRange(position)) {
				String aChar = characterAt(position);
				if (aChar.equals("(")) {
					positioned = true;
					break;
				}
				position.add(1);
			}
			
			if (!positioned) {
				System.out.println("GET XLS LOOKUPS problem");
			}

			// need to tokenize the remaining characters in the lookup
			// definition
			ignore = false;
			tokens.add(token);
			ArrayList<String> tokenizedLookup;

			// what format of lookup are we defining?
			// need to detect "tokenizeWithLookup" case when there is no range
			// "[()-()]" included
			// need to look ahead in the equation past position to see if a "("
			// exists
			if (beyondPosition(position).contains("[")
					|| beyondPosition(position).contains("(")) {
				tokenizedLookup = tokenizeWithLookup(position, opRes);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(opRes.getMessage());
					return false;
				}
			} else {
				tokenizedLookup = tokenizeLookup(position, opRes);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(opRes.getMessage());
					return false;
				}
			}

			// log.debug("Defining as lookup: "+token);

			tokens.addAll(tokenizedLookup);
			// for (String s : tokenizedLookup)
			// log.debug(s);
			// cleanEquation += s;
			hasRHS = false;
			definesLookup = true;
			lookupTables.add(token); // used to be clean(token)
			
			registerArrayLookup(token);
			
			InformationManagers.getInstance().getArrayManager()
					.setUsedAsLookup(token);
			EquationProcessor.lookups.add(token); // used to be clean(token)
			// List<String> s = new ArrayList<String>();
			// s.add(subscript);
			List<String> s = this.extractSubscripts(subscript);
			InformationManagers.getInstance().getArrayManager()
					.arrayReference(origLHS, s);

			// } else if (inRange(position) && characterAt(position).equals("[")
			// && !hasEqualSign && !equation.contains("(")) {

			// this is a construct of a variable on the LHS and nothing else
			// I think that this is a lookup out of a data file
			// this should be converted into a method call for a data lookup
			
			// need more intelligent code to determine if a subscript

//		} else if (inRange(position) && characterAt(position).equals(":")
//				&& !beyondPosition(position).contains(":")) {
			
		} else if (inRange(position) && characterAt(position).equals(":")
				&& !isStartOfInterpolationMode(position.value())) {


			ignore = false;
			// need to tokenize the remaining characters in the subscript
			// definition
			tokens.add(token);
			tokens.add(":");
			
			hasRHS = false;
			definesSubscript = true;

			// this will be comma separated in tokens
			// tokenizeSubscripts stops if there is a mapping token "->"
			// we will need to check for <-> also.
			
			List<String> tokenizedSubscripts;
			
			// look for GET DIRECT SUBSCRIPT
			
			if (vensimEquation.contains("GET DIRECT SUBSCRIPT")) {
				List<String> tt = Parser.tokenizeMaintainSingleQuotes(equationComponents.getEquationPart() /*vensimEquation */);
				tokenizedSubscripts = Utilities.loadDirectSubscripts(tt);
			} else {
				tokenizedSubscripts = tokenizeSubscripts(position, opRes);
			}
			if (!opRes.isOk()) {
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add(opRes.getMessage());
				return false;
			}

			cleanEquation += ":";
			tokens.addAll(tokenizedSubscripts);
			for (String s : tokenizedSubscripts)
				cleanEquation += s;
			
			EquationProcessor.subscripts.add(token);
			registerSubscripts(tokens.get(0), tokenizedSubscripts);
			InformationManagers
					.getInstance()
					.getNamedSubscriptManager()
					.subscriptDefinition(tokens.get(0),
							extractSubscripts(tokenizedSubscripts));

			// check for ->
			// if we find it, we can find multiple formats for the mapped
			// subscripts
			// (1) a: b, c, d -> q
			// (2) a: b, c, d -> (q: x, y, z)
			// (3) a: b, c, d -> (q: x, y, z), (q2: x2, y2, z2)

			lookAhead1 = new MutableInteger(position.value() + 1);

			if (inRange(position) && characterAt(position).equals("-")
					&& inRange(lookAhead1)
					&& characterAt(lookAhead1).equals(">")) {
				
				// need to tell model Manager that special processing will be required
				
				doNotSimplifyLoops = false; // true;
				InformationManagers.getInstance().getModelManager().setRequiresExpandedSubscriptOrdering(false); // (true);
				processSubscriptMapping(tokens.get(0),
						extractSubscripts(tokenizedSubscripts), position);
			}
		} else if (inRange(position) && characterAt(position).equals(":")
				&& beyondPosition(position).contains(":")) {
			ignore = false;
			btwnMode = getStringDelimitedBy(position);
			skipWhiteSpace(position);

			tokens.add(InformationManagers.getInstance()
					.getNativeDataTypeManager()
					.getLegalName(token.replace(":" + btwnMode + ":", "")));
			assignment = true;
			tokens.add("=");
			cleanEquation += "=";

			if (inRange(position) && !characterAt(position).equals("=")) {
				System.out.println("Didn't find = after btwnMode as expected");
			}
			skipWhiteSpace(position);

		} else if (inRange(position) && characterAt(position).equals("<")
				&& inRange(position.value() + 1)
				&& characterAt(position.value() + 1).equals("-")
				&& inRange(position.value() + 2)
				&& characterAt(position.value() + 2).equals(">")) {
			ignore = false;
			// have a full subrange definition

			// NOTE: this does not directly generate code -- it is done
			// indirectly by MappedSubscriptManager

			hasRHS = false;
			definesSubscript = true;

			doNotSimplifyLoops = false; // true;
			InformationManagers.getInstance().getModelManager().setRequiresExpandedSubscriptOrdering(false); // (true);
			
			String rhsSubscript = equation.split("<->")[1].trim();
			InformationManagers.getInstance().getMappedSubscriptManager()
					.addSubscriptNameFullSubrangeMapping(lhs, rhsSubscript);
			tokens.add(lhs);
			tokens.add("<->");
			tokens.add(rhsSubscript);
			// this.fatal = true;
			// fatalMessages.add("Mapped Subscripts are not supported in this version of Repast Simphony System Dynamics");

		} else if (inRange(position) && characterAt(position).equals("[")) {
			leftBracketCount++;
			ignore = false;
			// todo: nested array References
			// cleanEquation

			lhsArray = token;

			// need to reconstruct the array reference into a single token
			StringBuilder sb = new StringBuilder("array." + token);
			
			// tokens.add("array."+token);
			this.hasLHSArrayReference = true;

			hasRHS = true;

			lhsSubscripts = getSubscriptsStartingAt(position, opRes);
			if (!opRes.isOk()) {
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add("Tokenize LHS: " + opRes.getMessage());
			}
			InformationManagers.getInstance().getArrayManager()
					.arrayReference(lhsArray, extractSubscripts(lhsSubscripts));
			lhsArrayReference = new ArrayReference(token+createSubscript(lhsSubscripts));
			skipWhiteSpace(position);

			// 2/22/13 - ADD PROCESSING
			// look ahead to see if the next character is the start of an
			// interpolation mode definition
			// e.g. :INTERPOLATE:

			if (inRange(position) && characterAt(position).equals(":")) {
				btwnMode = getStringDelimitedBy(position);
				skipWhiteSpace(position);
			}
			if (inRange(position) && characterAt(position).equals("=")) {
				String subs = "";
				for (String s : lhsSubscripts)
					subs += s;
				sb.append(subs);
				// tokens.addAll(lhsSubscripts);
				lhs = sb.toString();

				tokens.add(sb.toString());
				assignment = true;
				tokens.add("=");

				// need to skip past all "="
				if (inRange(position.value() + 1)
						&& characterAt(position.value() + 1).equals("="))
					position.add(1);

			} else if (inRange(position) && characterAt(position).equals("[")) {
				leftBracketCount++;
				// This is the start of the exception list
				exceptions = extractExceptions(position, opRes);
				
				// will this help?
				InformationManagers.getInstance().getModelManager().setRequiresExpandedSubscriptOrdering(false); // (true);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add("Tokenize LHS: " + opRes.getMessage());
					return false;
				}
				this.setHasException(true);
				// should be pointing at "=+
				skipWhiteSpace(position);
				if (inRange(position) && characterAt(position).equals("=")) {
					String subs = "";
					for (String s : lhsSubscripts)
						subs += s;
					sb.append(subs);
					// tokens.addAll(lhsSubscripts);
					lhs = sb.toString();

					tokens.add(sb.toString());
					assignment = true;
					tokens.add("=");

					// need to skip past all "="
					if (inRange(position.value() + 1)
							&& characterAt(position.value() + 1).equals("="))
						position.add(1);
				} else {
					log.debug("ERROR: Bad Array Reference (A)!character found is <"
							+ characterAt(position) + "> position " + position);
					ignore = true;
				}
			} else {
				log.debug("ERROR: Bad Array Reference (B)! character found is <"
						+ characterAt(position)
						+ "> position "
						+ position
						+ " ###" + equation + "###");
				ignore = true;
			}
			// Left Bracket Processing Ends

		} else {
			syntacticallyCorrect = false;
			setSyntacticallyCorrect(false);
			syntaxMessages.add("LHS: Missing or misplaced character");
			return false;
		}
		if (ignore) {
			log.debug("IGNORING <" + equation + ">");
			syntacticallyCorrect = false;
			setSyntacticallyCorrect(false);
			syntaxMessages.add("Ignoring: " + equation);
			return false;
		}
		cleanEquation = "";
		for (String s : tokens)
			cleanEquation += s;
		position.add(1);

		// if (definesLookup)
		// log.debug("defines lookup");

		if (hasEqualSign || definesLookup) {
			InformationManagers.getInstance().getUnitsManager()
					.addLhsUnits(lhs, units);
		}

		if (isHasLHSArrayReference() && isOneTime()) {
			definesLookup = true;
			registerArrayLookup();
			InformationManagers.getInstance().getArrayManager()
					.setUsedAsLookup(getOriginalLhs()); // BAD BAD BAD can we
														// trick code?
			lookupTables.add(getOriginalLhs()); // used to be clean(token)
			EquationProcessor.lookups.add(getOriginalLhs()); // used to be
																// clean(token)
		}

		return hasRHS;
	}
	
	private String createSubscript(List<String> lhsSubscripts) {
		StringBuilder sb = new StringBuilder();
		for (String s : lhsSubscripts) {
			sb.append(s);
		}
		
		return sb.toString();
	}

	private String getStringDelimitedBy(MutableInteger position) {

		StringBuilder sb = new StringBuilder();
		// the character pointed at in position defines the delimiting character
		String delim = characterAt(position);
		position.add(1);
		while (!characterAt(position).equals(delim)) {
			sb.append(characterAt(position));
			position.add(1);
		}
		position.add(1);
		return sb.toString();

	}

	public void removeExceptions(Set<String> arrayReferenceSet) {
		List<String> removeList = new ArrayList<String>();

		for (String arrayReference : arrayReferenceSet) {
			if (containsException(arrayReference))
				removeList.add(arrayReference);
		}
		arrayReferenceSet.removeAll(removeList);
	}

	public boolean containsException(String arrayReference) {
		if (exceptions == null)
			return false;
		boolean contains = false;
		for (String except : exceptions) {
			if (arrayReference.contains(except))
				contains = true;
		}

		return contains;
	}

	private List<String> extractExceptions(MutableInteger position,
			OperationResult or) {
		List<String> exceptions = new ArrayList<String>();
		while (getNextNonBlankAnd(position, ",").equals("[")) {
			String essa = getExceptionSubscriptStartingAt(position, or).replace("[", "");
			if (!or.isOk()) {
				return exceptions;
			}
			exceptions.add(essa);
			log.debug("Detected Exception: "
					+ exceptions.get(exceptions.size() - 1));
		}
		return exceptions;
	}

	private int computeN() {
		int n = 1;
		Set<String> functions = new LinkedHashSet<String>();
		functions.add("DELAY N");
		functions.add("SMOOTH N");
		for (Node node : TreeTraversal.getFunctionCallsInTree(getTreeRoot(),
				functions)) {
			String nString = TreeTraversal.getFunctionArgument(node, 4)
					.getToken();
			int numToAdd = 1;
			if (Parser.isNumber(nString))
				numToAdd = Integer.parseInt(nString) - 1;
			n += numToAdd;
		}
		if (smooth3)
			n += 2;
		if (delay3)
			n += 2;
//		if (n != 1)
//			System.out.println("Num  Equations");
		return n;
	}

	private void checkSpecialFunctions(String token) {
		if (token.equalsIgnoreCase("Vector Sort Order")) {
			hasVectorSortOrder = true;
		} else if (token.equalsIgnoreCase("Vector REORDER")) {
			hasVectorReorder = true;
		} else if (token.equalsIgnoreCase("Vector Elm Map")) {
			hasVectorElmMap = true;
		} else if (token.equalsIgnoreCase("Vector Select")) {
			hasVectorSelect = true;
		} else if (token.equalsIgnoreCase("Vector Rank")) {
			hasVectorRank = true;
		} else if (token.equalsIgnoreCase("min")
				|| token.equalsIgnoreCase("max")) {
			usesMinMax = true;
		} else if (token.equalsIgnoreCase("step")
				|| token.equalsIgnoreCase("pulse")) {
			usesStepPulse = true;
		} else if (token.equalsIgnoreCase("if then else") || token.replace(" ", "_").equalsIgnoreCase("if then else")) {
			usesIfThenElse = true;
			// SMOOTH(X,T) Returns a first order exponential smooth of X over
			// time T.
			// SMOOTH N(X,T,I,N) Returns a N'th order exponential smooth of X
			// over time T starting at I.
			// SMOOTH3(X,T) Returns a third order exponential smooth of X over
			// time T.
			// SMOOTH3I(X,T,I) Returns a third order exponential smooth of X
			// over time T starting at I.
			// SMOOTHI(X,T,I)
		} else if (token.equalsIgnoreCase("smooth")
				|| token.equalsIgnoreCase("smooth n")
				|| token.equalsIgnoreCase("smooth3")
				|| token.equalsIgnoreCase("smooth3i")
				|| token.equalsIgnoreCase("smoothi")) {
			usesSmooth = true;
			numSmooth++;
			if (token.equalsIgnoreCase("smooth3")
					|| token.equalsIgnoreCase("smooth3i")) {
				smooth3 = true;
				numSmooth3++;
			}
			if (token.equalsIgnoreCase("smooth n")) {
				smoothn = true;
				numSmoothN++;
			}

			// DELAY CONVEYOR(X,T,L,IP,I,IT) Delays the input X on a conveyor
			// for time T with leakage
			// L. IP is the time profile of the initial material I which will
			// has an initial
			// convey time IT.
			// DELAY FIXED(X,T,I) Delays the input X for a fixed time T starting
			// with I.
			// DELAY INFORMATION(X,T,I) Delays the input X for a variable time T
			// starting with I discarding
			// inputs if T decreases and holding inputs if T increases.
			// DELAY MATERIAL(X,T,I,M) Delays the input X for a variable time T
			// starting with I and
			// conserving X except that M is used if no delayed inputs are
			// available.
			// DELAY N(X,T,I,N) N'th order exponential delay of the input X
			// starting at I and conserving
			// X.
			// DELAY PROFILE(P,X,T,I,G) An arbitrary delay of X with average
			// delay time T distributed
			// over a profile P (a Lookup) with initial value I initially
			// growing at rate G.
			// DELAY1(X,T) First order exponential delay of X for time T
			// conserving X.
			// DELAY1I(X,T,I) Same as DELAY1 but starting at I.
			// DELAY3(X,T) A third order exponential delay of X for time T
			// conserving X.
			// DELAY3I(X,T,I) Same as DELAY3 but starting at I.
			// DELAYP(X,T:P) Same as DELAY3 but also returning P the amount in
			// the pipeline.
		} else if (token.equalsIgnoreCase("delay")
				|| token.equalsIgnoreCase("delay conveyor")
				|| token.equalsIgnoreCase("delay fixed")
				|| token.equalsIgnoreCase("delay information")
				|| token.equalsIgnoreCase("delay material")
				|| token.equalsIgnoreCase("delay n")
				|| token.equalsIgnoreCase("delay profile")
				|| token.equalsIgnoreCase("delay1")
				|| token.equalsIgnoreCase("delay1i")
				|| token.equalsIgnoreCase("delay3")
				|| token.equalsIgnoreCase("delay3i")
				|| token.equalsIgnoreCase("delay p")) {
			usesDelay = true;
			numDelay++;
			if (token.equalsIgnoreCase("delay3")
					|| token.equalsIgnoreCase("delay3i")) {
				delay3 = true;
				numDelay3++;
			}
			if (token.equalsIgnoreCase("delay n")) {
				delayn = true;
				numDelayN++;
			}
		}
	}

	private void tokenizeRHS(MutableInteger position) {
		
		// break into tokens and determine as much as possible
		// about the equation as we pass through it.
		String token = "";
		String unaryMinus = "";

		boolean function = false;
		boolean variable = false;
		boolean array = false;
		boolean macro = false;
		boolean stringConstant = false;
		boolean lookupRef = false;

		skipWhiteSpace(position);
		boolean done = false;

		int lastPosition = -1;
		int stuck = 0;

		while (!done) {

			function = false;
			variable = false;
			array = false;
			stringConstant = false;
			macro = false;
			lookupRef = false;

			if (position.value() >= equation.length())
				break;

			String theChar = characterAt(position);
		

			if (position.value() == lastPosition) {
				stuck++;

				System.out.println("Stuck on " + theChar + " in position "
						+ lastPosition + " " + getVensimEquationOnly());
				log.debug("Stuck on " + theChar + " in position "
						+ lastPosition + " " + getVensimEquationOnly());
				
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add("Parsing RHS: " + "Stuck on " + theChar + " in position "
						+ lastPosition + " " + getVensimEquationOnly());
				if (stuck > 1)
					return;
				
			}

			lastPosition = position.value();

			if (theChar.equals("{")) {
				Parser.skipToEndDelimiter(position, equation, "}");
				position.add(1);
			}

			skipWhiteSpace(position);
			if (position.value() >= equation.length())
				break;
			if (characterAt(position).equals("\"")
					|| characterAt(position).equals("'")) {
				if (characterAt(position).equals("'"))
					stringConstant = true;

				if (typeString)
					token = getQuotedStringWithQuotesStartingAt(position, opRes);
				else
					token = getQuotedStringStartingAt(position, opRes);
				if (!opRes.isOk()) {
					// failure finding closing quote
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add("Parsing RHS: " + opRes.getMessage());
					return;
				}

				// we have a quoted string. Her are possible situations:
				// (1) a function reference
				// (2) a lookup reference -- Can be an reference to an array
				// Lookup table
				// (3) an array reference
				// (4) a simple variable reference
				// (5) String for variable assignment (typeString = "true")

				// ArrayLookupTable MARK HERE

				skipWhiteSpace(position);
				if (inRange(position)) {
					theChar = characterAt(position);

					// if the char == "(", we have either a function reference
					// or a lookup reference
					if (characterAt(position).equals("(")) {
						// log.debug("TokRHS: func or lookup token: "+token);
						if (InformationManagers.getInstance()
								.getFunctionManager().isVensimFunction(token) || FunctionMapping.isXmileFunction(token)) { // HERE~~~
						// log.debug("TokRHS: Function");
							checkSpecialFunctions(token);
							function = true;
							if (isGetXlsDataFunction(token))
								usesTimeSeries = true;

							if (isGetXlsLookupsFunction(token))
								definesLookupGetXls = true;

							if (token.toUpperCase().equals("INTEG"))
								stockVariable = true;

							if (token.toUpperCase().equals("ACTIVE INITIAL"))
								activeInitial = true;

							if (token.toUpperCase().equals("SAMPLE IF TRUE"))
								sampleIfTrue = true;

							if (token.toUpperCase().equals("GAME"))
								game = true;

							// move past "(" as code below will be adding it
							// back
							position.add(1);

							// try this MJB 5/5/2014
							if (characterAt(position).equals("+")) { // why are
																		// there
																		// "unary"
																		// plus
																		// signs?
								position.add(1);
							}
						} else if (isMacroName(token)) {
							// log.debug("TokRHS: Macro");
							macro = true;
							// move past "(" as code below will be adding it
							// back
							position.add(1);
						} else {
							// log.debug("TokRHS: Lookup");
							referencesLookup = true;
							lookupRef = true;
							rhsTokens.add(token);
						}
						// if the char == "[", we have either an array reference
						// or a array lookup reference
					} else if (characterAt(position).equals("[")) {
						array = true;
						hasRHSArrayReference = true;
						token = tokenizeArrayReference(token, position, opRes);
						if (!opRes.isOk()) {
							syntacticallyCorrect = false;
							setSyntacticallyCorrect(false);
							syntaxMessages.add("Parsing RHS: "
									+ opRes.getMessage());
							return;
						}
						position.add(1);
						skipWhiteSpace(position);
						if (inRange(position.value())
								&& characterAt(position.value()).equals("(")) {
							// log.debug("TokRHS: Lookup2");
							referencesLookup = true;
							lookupRef = true;
							token = token.replace("array.", "");
							rhsTokens.add(token);

						} else {
							rhsTokens.add(token);
							// position.add(1);
						}

					} else {
						if (!stringConstant)
							variable = true;
						rhsTokens.add(token);
					}
				} else {
					// don't look ahead since it is beyond the end of statement
					// so this must be a variable
					variable = true;
					rhsTokens.add(token);
				}
				// this is where we grab either a funtion, a lookup, or variable
				// need to detect arrays
				// or special variable :NA:
			} else if (Character.isLetter(characterAt(position).charAt(0))) {
				boolean allowColon = false;
//				token = unaryMinus
//						+ getNonQuotedStringStartingAt(position, allowColon,
//								opRes);
				token = unaryMinus
						+ getStringStartingAt(position, allowColon,
								opRes);

				skipWhiteSpace(position);
				if (inRange(position)) {
					if (characterAt(position).equals("(")) { // checking for
																// function
																// invocation
					// log.debug("TokRHS: Lookup3 based on token: "+token);
						if (InformationManagers.getInstance()
								.getFunctionManager().isVensimFunction(token) || FunctionMapping.isXmileFunction(token)) { // Here
							checkSpecialFunctions(token);
							function = true;
							if (isGetXlsDataFunction(token)) {
								usesTimeSeries = true;
							}
							if (isGetXlsLookupsFunction(token)) {
								// usesTimeSeries = true;
								definesLookupGetXls = true;
							}

							if (token.toUpperCase().equals("INTEG")) {
								stockVariable = true;
								// try this MJB 5/5/2014
								if (characterAt(position.value() + 1).equals(
										"+")) { // why are there "unary" plus
												// signs?
									position.add(1);
								}
							}
							
							if (token.toUpperCase().equals("ACTIVE INITIAL"))
								activeInitial = true;

							if (token.toUpperCase().equals("GAME"))
								game = true;

							if (token.toUpperCase().equals("INITIAL"))
								usesInitialFunction = true;

							if (token.toUpperCase().equals("SUM")
									|| token.toUpperCase().equals("PROD")
									|| token.toUpperCase().equals("VMIN")
									|| token.toUpperCase().equals("VMAX"))
								usesRangeFunction = true;

							token = token.toUpperCase();
							// move past "(" as code below will be adding it
							// back
							position.add(1);
						} else if (isMacroName(token)) {
							macro = true;
							// move past "(" as code below will be adding it
							// back
							position.add(1);
						} else if (FunctionMapping.isXmileFunction(token)) {
						} else {
							// log.debug("TokRHS: Lookup3");
							lookupRef = true;
							referencesLookup = true;
							rhsTokens.add(token);
						}
						// array reference?
					} else if (characterAt(position).equals("[")) {
						array = true;
						hasRHSArrayReference = true;
						token = tokenizeArrayReference(token, position, opRes);
						if (!opRes.isOk()) {
							syntacticallyCorrect = false;
							setSyntacticallyCorrect(false);
							syntaxMessages.add("Parsing RHS: "
									+ opRes.getMessage());
							return;
						}
						position.add(1);
						skipWhiteSpace(position);
						// theChar = characterAt(position.value());
						if (inRange(position.value())
								&& characterAt(position.value()).equals("(")) { // MARK
																				// HERE
							referencesLookup = true;
							// log.debug("TokRHS: Lookup4");
							lookupRef = true;
							token = token.replace("array.", "");
							rhsTokens.add(token);

						} else {
							rhsTokens.add(token);
							// position.add(1);
						}

					} else {
						if (!token.equals("Double.MIN_VALUE")) {
							variable = true;
							rhsTokens.add(token);
						} else {
//							System.out.println("Double.MIN_VALUE");
						}
					}
				} else {
					// don't look ahead since it is beyond the end of statement
					// so this must be a variable
					if (!token.equals("Double.MIN_VALUE")) {
						variable = true;
						rhsTokens.add(token);
					}  else {
//						System.out.println("Double.MIN_VALUE");
					}
				}
			} else if (Character.isDigit(characterAt(position).charAt(0)) ||
					characterAt(position).equals(".")) {
				String numeric = Parser.forceDouble(getNumberStartingAt(position, opRes));
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add("RHS: " + opRes.getMessage());
					return;
				}
				token = unaryMinus + numeric;
			} else if (characterAt(position).equals(":")) {
				token = getBooleanOperatorStartingAt(position, opRes);
				if (!opRes.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add("RHS: " + opRes.getMessage());
					return;
				}
			} else if (isTerminator(characterAt(position).charAt(0))) {
				// special cases:
				// if "-" need to check if unary and attach to next token
				// if "=" need to change to "=="
				// if "<" need to check to "<>"
				// if "+" need to check if this is a unary + (previous token =
				// "("
				if (characterAt(position).equals("-")) {
					String previousToken = tokens.get(tokens.size() - 1);
					if (previousToken.equals("(") || previousToken.equals("=") || previousToken.equals(";")
							|| Parser.isOperator(previousToken)
							|| isFunctionArgumentSeparator(previousToken)) {
						token = "_"; // unary minus
					} else {
						token = "-";
					}
				} else if (characterAt(position).equals("+")) {
					String previousToken = tokens.get(tokens.size() - 1);
					if (previousToken.equals("(") || previousToken.equals("=")
							|| Parser.isOperator(previousToken)
							|| isFunctionArgumentSeparator(previousToken)) {
						token = " "; // discard unary plus will this work?
					} else {
						token = "+";
					}
				} else if (characterAt(position).equals("=")) {
					token = "==";
				} else if (characterAt(position).equals("<")) { // look ahead
																// for ">" i.e.
																// != "<>"
					if (characterAt(position.value() + 1).equals(">")) {
						token = "<>";
						position.add(1);
					}
					if (characterAt(position.value() + 1).equals("=")) {
						token = "<=";
						position.add(1);
					} else {
						token = characterAt(position);
					}
				} else if (characterAt(position).equals(">")) { // look ahead
																// for "=" i.e.
																// != "<>"
					if (characterAt(position.value() + 1).equals("=")) {
						token = ">=";
						position.add(1);
					} else {
						token = characterAt(position);
					}
				} else {
					token = characterAt(position);
				}
				position.add(1);
			}

			// for functions
			if (function) {
				setCallsFunction(true);
				functionsCalled.add(clean(token).replace(" ", ""));
				if (token.toUpperCase().startsWith("GET XLS LOOK")) {
					definesLookup = true;
					
					String usedAsLookup = lhsArray;
					if (usedAsLookup == null)
						usedAsLookup = lhs;
					registerArrayLookup(usedAsLookup);
					InformationManagers.getInstance().getArrayManager()
							.setUsedAsLookup(usedAsLookup); // BAD BAD BAD can we
														// trick code?
					lookupTables.add(usedAsLookup); // used to be clean(token)
					EquationProcessor.lookups.add(usedAsLookup); // used to be
																// clean(token)
				}
				if (clean(token).startsWith("GETXLSXXX")) {
					tokens.add("sdFunctions." + clean(token));
					tokens.add("(");
					cleanEquation += "sdFunctions." + clean(token) + "(";
				
				} else if (InformationManagers.getInstance().getFunctionManager().isVensimFunction(token)) {
					tokens.add("sdFunctions." + clean(token));
					tokens.add("(");
					FunctionDescription fd = InformationManagers.getInstance()
							.getFunctionManager().getDescription(token);
					if (!fd.isSupported()) {
						InformationManagers
								.getInstance()
								.getModelManager()
								.setRunnable(false,
										"Unsupported Function " + token);
					}
					
					if (fd.isSuppliesInitialValue())
						hasInitialValue = true;

					cleanEquation += "sdFunctions." + clean(token) + "(";
					boolean addComma = false;

					// if (fd == null)
					// log.debug("NULL FD");
					if (fd.isRequiresName()) {
						cleanEquation += "\"" + lhs + "\"";
						addComma = true;
						tokens.add("\"" + lhs + "\"");
						tokens.add(",");
					}

					String valueOf = "";
					valueOf = InformationManagers.getInstance()
							.getNativeDataTypeManager().getLegalName(this, lhs);

					// log.debug("TokRHS: valueof ="+valueOf+" lhs = "+lhs);

					int goober;
					if (ArrayReference.isArrayReference(lhs)) {
						goober = 1;
						valueOf = lhs;
						// valueOf = new ArrayReferenceNative(lhs,
						// this).generateRHSImplementation(); // I added native
						// 1/17
					} else {
						goober = 2;
						// valueOf = NativeDataTypeManager.getLegalName(lhs);
					}

					if (fd.isRequiresValue()) {
						cleanEquation += (addComma ? "," : " ") + valueOf;
						addComma = true;

						tokens.add(valueOf);
						tokens.add(",");
					}
					if (fd.isRequiresTime()) {
						cleanEquation += (addComma ? "," : " ") + "time";
						tokens.add("time");
						tokens.add(",");
					}
					if (fd.isRequiresTimeStep()) {
						cleanEquation += (addComma ? "," : " ") + "timeStep,";
						tokens.add("timeStep");
						tokens.add(",");
					}

					// cleanEquation +=
					// "sdFunctions."+clean(token)+"(\""+lhs+"\","+valueOf+", time, timeStep, ";
				} else if (FunctionMapping.isXmileFunction(clean(token))) {
					tokens.add("sdFunctions." + clean(token));
					tokens.add("(");
					cleanEquation += "sdFunctions." + clean(token) + "(";
				}
			} else if (macro) {
				tokens.add(token);
				// tokens.add(clean(MacroManager.makeLegal(token)));
				tokens.add("(");
				// cleanEquation += clean(MacroManager.makeLegal(token))+"(";
				cleanEquation += clean(token) + "(";
			} else if (lookupRef) {
				// token = lookup name
				tokens.add("sdFunctions.LOOKUP");
				tokens.add("(");

				tokens.add("lookup." + token);

				tokens.add(",");
				// move past the "(", we've already processed it
				position.add(1);
				skipWhiteSpace(position);

				cleanEquation += "sdFunctions.LOOKUP" + "(" + "lookup." + token
						+ ",";
			} else if (array) {
				tokens.add(token);
				cleanEquation += token;

			} else if (variable) {

				tokens.add(InformationManagers.getInstance()
						.getNativeDataTypeManager().getLegalName(token));
				cleanEquation += InformationManagers.getInstance()
						.getNativeDataTypeManager().getLegalName(token);

			} else if (stringConstant) {
				tokens.add("\"" + token + "\"");
				cleanEquation += "\"" + token + "\"";
			} else {
				tokens.add(token);
				cleanEquation += token.equals("_") ? "-" : token;
			}
			skipWhiteSpace(position);
		}
		// if this rhs contains an invocation of GETXLS...
		// we do not want for it to have RHS tokens!

		// set our conditions

		if (vensimEquation.contains(Parser.externalDataMethod()))
			setVdmLookup(true);
		if (rhsTokens.size() == 0 || equation.toUpperCase().contains("GET XLS")
				|| equation.contains(Parser.externalDataMethod()) || definesLookup
			/*	|| hasInitialValue */ || definesSubscript || isUsesInitialFunction()
				|| definesLookupWithRange  || isGame() )
			setOneTime(true);

		// need to call this multiple times even though it meets our "one-time"
		// criteria
		if (equation.contains("RANDOM"))
			setOneTime(false);

		if (tokens.contains("time"))
			setOneTime(false);

	}

	public String getInitialValue() {
		// valid on for stock variables
		// return empty string is not stock
		if (!isStock())
			return "";

		// need rpn and treeRoot
		if (rpn.size() == 0)
			generateRpn();
		if (treeRoot == null)
			generateTree();

		StringBuilder sb = new StringBuilder();

		// extract from tree -- last Argument of the Integ function;
		
		if (treeRoot == null || treeRoot.getChild() == null) {
			generateTree();
		}

		Node integ = treeRoot.getChild().getNext();

		Node arg = integ.getChild();
		while (arg.getNext() != null)
			arg = arg.getNext();

		// this is last arg, but can have a subtree

		sb.append(Node.expand(arg));

		return sb.toString();
	}

	private boolean isArrayReference(MutableInteger position) {

		MutableInteger pos = new MutableInteger(position);

		// look ahead to determine if this is an array reference
		// skip to ending ]
		// find next now-white space
		// ( indicates a lookup
		while (!characterAt(pos).equals("]"))
			pos.add(1);
		pos.add(1);
		skipWhiteSpace(pos);
		if (!characterAt(pos).equals("("))
			return true;
		else
			return false;
	}

	private void validateIFTHENELSEInTree(Node root) {
		// NOTE: we do not support boolean variables. Need to adjust the code to
		// check for a
		// non-zero value rather than just a variable

		List<Node> iteNodes = new ArrayList<Node>();
		findITE(root, iteNodes);
		for (Node node : iteNodes) {
			ite = true;
			validateIFTHENELSE(node);
		}
	}

	public void findITE(Node node, List<Node> iteNodes) {
		if (node == null)
			return;
		if (isITENode(node))
			iteNodes.add(node);
		findITE(node.getChild(), iteNodes);
		findITE(node.getNext(), iteNodes);
	}

	public static boolean isITENode(Node node) {
		if (node.getToken().equals("sdFunctions.IFTHENELSE"))
			return true;
		else
			return false;
	}

	private void validateIFTHENELSE(Node iteNode) {
		// our condition node is the fifth argument -- based on standard calling
		// convention
		// of 4 informational arguments appearing first

//		Node c = iteNode.getChild();
//		if (c == null)
//			log.debug("MISSING ARG");
//		c = c.getNext();
//		if (c == null)
//			log.debug("MISSING ARG");
//		c = c.getNext();
//		if (c == null)
//			log.debug("MISSING ARG");
//		c = c.getNext();
//		if (c == null)
//			log.debug("MISSING ARG");
//		c = c.getNext();
//		if (c == null)
//			log.debug("MISSING ARG");

		// Node conditionNode =
		// iteNode.getChild().getNext().getNext().getNext().getNext();
		Node conditionNode = TreeTraversal.getFunctionArgument(iteNode, 1);
		if (conditionNode.isTerminal()) {
			replaceBooleanVariable(conditionNode);
		} else {
			List<Node> terminals = new ArrayList<Node>();
			findTerminal(conditionNode.getChild(), terminals);
			for (Node node : terminals) {
				if (!Parser.isRelationalOperator(node.getParent().getToken())
						&& !Parser.isArithmeticOperator(node.getParent()
								.getToken())
						&& !Parser.isFunctionInvocation(node.getParent()
								.getToken()))
					replaceBooleanVariable(node);
			}
		}

	}

	private void findTerminal(Node node, List<Node> terminals) {
		if (node == null)
			return;
		if (node.isTerminal())
			terminals.add(node);
		findTerminal(node.getChild(), terminals);
		findTerminal(node.getNext(), terminals);
	}

	private void replaceBooleanVariable(Node node) {
		// need two new nodes
		Node equalsNode = new Node("==");
		Node one = new Node("1.0");
		Node previous = node.getPrevious();
		Node next = node.getNext();
		Node parent = node.getParent();

		// reset previous, and next nodes to point to new node
		if (previous != null) {
			previous.setNext(equalsNode);
		} else {
			parent.setChild(equalsNode);
		}
		if (next != null) {
			next.setPrevious(equalsNode);
		}

		// point equalsNode to conditionNodes pointers
		equalsNode.setParent(node.getParent());
		equalsNode.setNext(node.getNext());
		equalsNode.setPrevious(node.getPrevious());

		// create the condition expression
		equalsNode.setChild(node);
		node.setPrevious(null);
		node.setNext(one);
	}

	private void processSubscriptMapping(String rhsSubscriptName,
			List<String> rhsSubscriptValues, MutableInteger position) {

		// this.fatal = true;
		// fatalMessages.add("Mapped Subscripts are not supported in this version of Repast Simphony System Dynamics");

		// position points to "-" of "->"

		// found ->
		// we can find multiple formats for the mapped subscripts
		// (1) a: b, c, d -> q
		// (2) a: b, c, d -> (q: x, y, z)
		// (3) a: b, c, d -> (q: x, y, z), (q2: x2, y2, z2)

		// position points to the "->"
		String mappedTo = equation.split("->")[1].trim();

		// format (1)
		if (!mappedTo.contains("(")) {
			InformationManagers.getInstance().getMappedSubscriptManager()
					.addSubscriptNameMapping(rhsSubscriptName, mappedTo);

			if (InformationManagers.getInstance().getNamedSubscriptManager()
					.isNamedSubscript(mappedTo)) {
				List<String> mappedToValues = InformationManagers.getInstance()
						.getNamedSubscriptManager().getValuesFor(mappedTo);
				List<String> tempRhsValues;
				if (rhsSubscriptValues.size() == 1 && InformationManagers.getInstance().getNamedSubscriptManager()
						.isNamedSubscript(rhsSubscriptValues.get(0))) {
					tempRhsValues = InformationManagers.getInstance()
							.getNamedSubscriptManager().getValuesFor(rhsSubscriptValues.get(0));
				} else {
					tempRhsValues = rhsSubscriptValues;
				}
				
				for (int i = 0; i < mappedToValues.size(); i++)
					InformationManagers
							.getInstance()
							.getMappedSubscriptManager()
							.addSubscriptValueMapping(rhsSubscriptName,
									mappedTo, tempRhsValues.get(i),
									mappedToValues.get(i));
			} else {
				InformationManagers
						.getInstance()
						.getMappedSubscriptManager()
						.addSubscriptValueMappingDelayed(rhsSubscriptName,
								mappedTo, rhsSubscriptValues);
			}
			return;
		} else {
			// formats (2) & (3)
			String s = characterAt(position);
			position.add(2); // skip "->"
			s = characterAt(position);
			skipWhiteSpace(position);
			s = characterAt(position);
			// should be pointing at a "("
			// if not, we are done
			while (inRange(position) && characterAt(position).equals("(")) {
				List<String> tokens = tokenizeMappedSubscriptInParens(position);

				// first token is the mappedTo name, the rest are subscript
				// values

				mappedTo = tokens.get(0);
				tokens.remove(0);

				InformationManagers.getInstance().getMappedSubscriptManager()
						.addSubscriptNameMapping(rhsSubscriptName, mappedTo);

				if (InformationManagers.getInstance()
						.getNamedSubscriptManager().isNamedSubscript(mappedTo)) {
					for (int i = 0; i < tokens.size(); i++)
						InformationManagers
								.getInstance()
								.getMappedSubscriptManager()
								.addSubscriptValueMapping(rhsSubscriptName,
										mappedTo, rhsSubscriptValues.get(i),
										tokens.get(i));
				} else {
					InformationManagers
							.getInstance()
							.getMappedSubscriptManager()
							.addSubscriptValueMappingDelayed(rhsSubscriptName,
									mappedTo, rhsSubscriptValues, tokens);
				}
			}
			// at this point, we are either pointing to another "(" defining
			// another mapping, or we are done.
		}

	}

	private List<String> tokenizeMappedSubscriptInParens(MutableInteger position) {
		// come in pointing to "("
		List<String> tokens = new ArrayList<String>();
		int matchingParen = equation.indexOf(")", position.value());

		String s = equation.substring(position.value() + 1, matchingParen);
		String[] t = s.split(",");
		String name = t[0].split(":")[0].trim();
		String first = t[0].split(":")[1].trim();
		tokens.add(name);
		tokens.add(first);
		for (int i = 1; i < t.length; i++)
			tokens.add(t[i].trim());

		position.setValue(matchingParen + 1);

		if (!inRange(position))
			return tokens;

		String s1 = characterAt(position);
		skipWhiteSpace(position);
		s1 = characterAt(position);
		if (inRange(position.value())
				&& characterAt(position.value()).equals(",")) {

			position.add(1); // point to ( or whitespace
			s1 = characterAt(position);
			skipWhiteSpace(position);
			s1 = characterAt(position);
		}

		return tokens;

	}

	private String tokenizeArrayReference(String arrayName,
			MutableInteger position, OperationResult or) {
		or.clear();
		StringBuilder sb = new StringBuilder();

		sb.append("array." + arrayName);

		sb.append("[");
		List<String> tokenList = tokenizeSubscripts(position, or);
		if (!or.isOk()) {
			return "";
		}
		for (String token : tokenList) {

			sb.append(token);
		}
		
		List<String> es = extractSubscripts(tokenList);

		InformationManagers.getInstance().getArrayManager()
				.arrayReference(arrayName, es);
		// position should be correct
		return sb.toString();
	}

	// private boolean isArrayConstantInitialization() {
	// // scan the equation
	// if (!this.isHasLHSArrayReference())
	// return false;
	// for (String value : getRHStokens(tokens)) {
	// if (!Parser.isNumber(value))
	// return false;
	// }
	// return true;
	// }

	private List<String> getRHStokens(List<String> tokens) {
		List<String> al = new ArrayList<String>();
		boolean afterEquals = false;
		for (String token : tokens) {
			if (afterEquals) {
				if (!token.equals(",") && !token.equals(";"))
					al.add(token);
			} else if (token.equals("="))
				afterEquals = true;
		}
		return al;
	}

	private void registerSubscripts(String subscript,
			List<String> tokenizedSubscripts) {
		List<String> al = new ArrayList<String>();
		for (String token : tokenizedSubscripts)
			if (!token.equals(","))
				al.add(token);
		EquationProcessor.subscriptValues.put(subscript, al);
	}

	private String getOffset(SubscriptCombination combo1,
			SubscriptCombination combo2) {
		// only need to find offset between last two subscripts
		return "";

	}

	public List<SubscriptCombination> getSubscriptValueCombinations(
			String... strings) {
		// given a list of subscripts return all combinations of values
		List<List<String>> bySubscriptValue = new ArrayList<List<String>>();

		List<SubscriptCombination> values = new ArrayList<SubscriptCombination>();
		List<String> order = new ArrayList<String>();

		// generate list of lists of values
		for (String subscriptName : strings) {
			bySubscriptValue
					.add(extractSubscripts(getSubscriptValues(subscriptName)));
			order.add(subscriptName);
		}
		int numPermutations = 1;
		for (int i = 0; i < bySubscriptValue.size(); i++) {
			numPermutations *= bySubscriptValue.get(i).size();
		}
		// create all permutations
		SubscriptCombination[] combinations = new SubscriptCombination[numPermutations];

		// initialize the empty buffers
		for (int i = 0; i < numPermutations; i++)
			combinations[i] = new SubscriptCombination(order);

		// for each of the subscripts
		for (int subscriptNumber = 0; subscriptNumber < strings.length; subscriptNumber++) {

			// determine the number of times that the same value should be
			// outputed
			int numPerValue = 1;
			for (int i = subscriptNumber + 1; i < bySubscriptValue.size(); i++) {
				numPerValue *= bySubscriptValue.get(i).size();
			}
			int row = 0;
			while (row < numPermutations) {
				for (String value : bySubscriptValue.get(subscriptNumber)) {

					for (int i = 0; i < numPerValue; i++) {
						combinations[row].addSubscriptValue(
								strings[subscriptNumber], value);
						row++;
					}
				}
			}
		}

		for (SubscriptCombination combo : combinations) {
			values.add(combo);
		}

		return values;
	}

	public List<String> getSubscriptValues(String subscriptName) {
		List<String> al = new ArrayList<String>();
		if (isNamedSubscript(subscriptName)) {
			for (String v : EquationProcessor.subscriptValues
					.get(subscriptName)) {
				if (isNamedSubscript(v))
					al.addAll(getSubscriptValues(v));
				else
					al.add(v);
			}
		} else {
			al.add(subscriptName);
		}
		return al;
	}

	public boolean isNamedSubscript(String subscriptName) {
		if (EquationProcessor.subscriptValues.containsKey(subscriptName))
			return true;
		else
			return false;
	}

	private List<String> extractSubscripts(List<String> subscriptList) {
		List<String> al = new ArrayList<String>();
		if (subscriptList == null)
			return al;
		for (String token : subscriptList)
			if (!token.equals("[") && !token.equals(",") && !token.equals("]")) {
//				if (token.equals("*")) { // this is the arg that runs across all values in a sum function
//					al.add("!");
//				} else {
//					al.add(token);
//				}
				al.add(token);
			}
		
		return al;
	}

	private List<String> extractSubscripts(String subscript) {
		List<String> al = new ArrayList<String>();
		String[] subrs = subscript.split(",");
		for (String token : subrs)
			al.add(token.trim());
		return al;
	}

	public String generateArrayConstantsInitialization(
			/*boolean nativeMemoryImplementation*/ /*, boolean newMemoryDefinition */) {
		
		
		// here
		
		StringBuilder sb = new StringBuilder();
		String[] values = asArray(getRHStokens(tokens));
		
		if (values.length > 1) {
//			System.out.println("Multiple values identified on initialization");

			List<String> subscripts = lhsArrayReference.getSubscripts();
			if (subscripts.size() > 1) {

			} else {
				boolean namedSubscript = InformationManagers.getInstance().getNamedSubscriptManager().isNamedSubscript(lhsArrayReference.getSubscripts().get(0));

				if (namedSubscript) {
					InformationManagers.getInstance().getNamedSubscriptManager().getValuesFor(lhsArrayReference.getSubscripts().get(0));
					// for now assume that just numeric values have been provided
				} else {
					StringBuilder sb2 = new StringBuilder();
					for (String v : values) {
						sb2.append(v);
					}
					values = new String[1];
					values[0] = sb2.toString();
				}
			}
		}
		
		// need to parse these values
		
		// is this a list of values? an single expression, a list of expressions?

		if (!equation.toUpperCase().contains("GET XLS")) {
			if (cleanEquation.contains("sdFunction")) {
				// or of the form X[a,b] = SDFuntion...

				String funcCallHead = "";
				String funcCallTail = "";
				funcCallHead = tokens.get(2) + "("; // actual function call

				// 4 ,
				// 5 var name
				// 6 ,
				// 7 var value

				for (int t = 7; t < tokens.size(); t++)
					funcCallTail += tokens.get(t);

				funcCallTail += ";\n";

				sb.append("// funcCallTail = " + funcCallTail);

				// generate the entire group of code.

				sb.append("// getOuter Equation 1691\n");
				sb.append(getEars().getOuterLoops());
//				if (newMemoryDefinition) {
					sb.append(ears.getLHSassignmentVectorMemory());
//				} else {
//					sb.append(ears.getLHSassignment());
//				}
				sb.append(" = " + funcCallHead + "\"" + ears.getLHSassignment()
						+ "\"," + ears.getLHSassignment() + funcCallTail);
				for (int i = 0; i < ears.getOuterClosingCount(); i++)
					sb.append("} // close outer \n");

			} else {

				// need to create a new array
				String vArray = getNextArray();
				if (!SDM_Doc.target.equals(ReaderConstants.C))
					sb.append("double[] ");
				else
					sb.append("double* ");
				sb.append(vArray + " = newDoubleArray( ");
				

				int len = values.length;
				for (int j = 0; j < values.length; j++) {
					if (values[j].equals("_"))
						len--;
				}
				sb.append(len);
				
				boolean needsIncrement = false;
				if (len > 1)
					needsIncrement = true;
				for (int j = 0; j < values.length; j++) {

					sb.append(",");
					if (values[j].startsWith("_")) {
						values[j] = values[j].replace("_", "-");
					}
					sb.append(values[j]);
				}
				sb.append(");\n");

				String integer = getNextInt();
				sb.append("int " + integer + " = 0;\n");

				// sb.append("// getOuter Equation 1742\n");
				sb.append(getEars().getOuterLoops());

//				if (nativeMemoryImplementation) {
//					if (newMemoryDefinition) {
						sb.append(getEars().getLHSassignmentVectorMemory());
//					} else {
//						sb.append(getEars().getLHSassignment());
//					}
					sb.append(" = " + vArray + "[" + integer + "];\n");
					// log

					// sb.append("/* generateArrayConstantsInitialization 1 */\n");
//					if (!newMemoryDefinition) {
//						sb.append("/* log7 */logit(stringConcat(\""
//								+ getEars().getLhsArrayReference()
//										.getArrayName() + "\"");
//						for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//							sb.append(",\"[\",intToString(outer" + i
//									+ "),\"]\"");
//						}
//						sb.append("),0.0,");
//						sb.append(vArray + "[" + integer
//								+ "],memory.get_SAVEPER());\n");
//					}
//				} else {
//					sb.append("/* array init1 */ set(asExpandedArrayRef(\""
//							+ getEars().getLhsArrayReference().getArrayName()
//							+ "\"");
//
//					for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//						sb.append(",outer" + i);
//					}
//					sb.append("), " + vArray + "[" + integer + "]);\n");
//					// // log
//					// sb.append("/* generateArrayConstantsInitialization 1 */\n");
//					// sb.append("/* log7 */logit(stringConcat(\""+getEars().getLhsArrayReference().getArrayName()+"\"");
//					//
//					// for (int i = 0; i < getEars().getOuterClosingCount();
//					// i++) {
//					// sb.append(",\"[\",intToString(outer"+i+"),\"]\"");
//					// }
//					// sb.append("),0.0,");
//					// sb.append(vArray+"["+integer+"],memory.get_SAVEPER());\n");
//				}

				// inc pointer
				if (needsIncrement)
					sb.append(integer + "++;\n");
				for (int i = 0; i < getEars().getOuterClosingCount(); i++)
					sb.append("}\n");

			}
		} else {
			if (equation.toUpperCase().contains("GET XLS LOOKUPS")) {
//				if (!isHasLHSArrayReference()) {
//					return cleanEquation+";";
//				} else {
					// we should only have 2 3d array at most
					
					String arrayName = getLhs();
					if (isHasLHSArrayReference())
						arrayName = getLhsArrayReference().getArrayName();
					String legalName = InformationManagers.getInstance().getNativeDataTypeManager().getLegalName(arrayName);
					
					
					int numTokens = tokens.size();
					int cellLocation = numTokens - 2; // 0 index and next to last
														// token (last arg to
														// function)
					String funcCallHead = "";
					String funcCallCellStart = "";
					String funcCallCell = "";
					String funcCallTail = ")";
					for (int t = 2; t < cellLocation; t++)
						if (t == 4) {
							ModelManager mm = InformationManagers.getInstance()
									.getModelManager();
							FileOrganizer fo = InformationManagers.getInstance().getFileOrganizer();
							funcCallHead += tokens.get(t).replaceFirst("\"",
									"\"" + fo.getRelativeToApplicationDirectory(mm.getModelDirectory()) + "/");
						} else {
							funcCallHead += tokens.get(t);
						}
					funcCallCellStart =  Parser.removeQuotes(tokens.get(cellLocation)); // tokens.get(cellLocation).replace("\"", "");

					int numRows = 1;
					int numCols = -1;

					List<String> subs = extractSubscripts(lhsSubscripts);
					for (int i = subs.size() - 1; i >= 0; i--) {
						if (InformationManagers.getInstance()
								.getNamedSubscriptManager()
								.isNamedSubscript(subs.get(i))) {
							if (numCols == -1) {
								numCols = InformationManagers.getInstance()
										.getNamedSubscriptManager()
										.getValuesFor(subs.get(i)).size();
							} else {
								numRows *= InformationManagers.getInstance()
										.getNamedSubscriptManager()
										.getValuesFor(subs.get(i)).size();
							}
						}
					}

					String v;
					if (this.isHasLHSArrayReference()) {
						v = funcCallHead + "\"" + funcCallCellStart + "\"" + ","
							+ numCols + funcCallTail;
					} else {
						v = funcCallHead + "\"" + funcCallCellStart + "\"" 
								 + funcCallTail;
					}
					
					sb.append(legalName+" = "+v + ";\n");
//					return sb.toString();
					
//					String array = getNextArray();
//					sb.append("double[] " + array + " = " + v + ";\n");
//					String integer = getNextInt();
//					sb.append("int " + integer + " = 0;\n");
//
//					sb.append("// getOuter Equation 1811\n");
//					sb.append(getEars().getOuterLoops());
//
//					if (nativeMemoryImplementation) {
//						if (newMemoryDefinition) {
//							sb.append(getEars().getLHSassignmentVectorMemory());
//							sb.append(" = " + array + "[" + integer + "];\n");
//						} else {
//							sb.append(getEars().getLHSassignment());
//							sb.append(" = " + array + "[" + integer + "];\n");
//
//							// log
//							sb.append("/* generateArrayConstantsInitialization 2 */\n");
//							sb.append("/* log8 */logit(\""
//									+ getEars().getLhsArrayReference()
//											.getArrayName() + "\"");
//							for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//								sb.append("+\"[\"+outer" + i + "+\"]\"");
//							}
//							sb.append(",0.0,");
//							sb.append(array + "[" + integer
//									+ "],memory.get_SAVEPER());\n");
//						}

//					} else {
//						sb.append("/* array init 2 */set(asExpandedArrayRef(\""
//								+ getEars().getLhsArrayReference().getArrayName()
//								+ "\"");
//						for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//							sb.append(",outer" + i);
//						}
//						sb.append("), " + array + "[" + integer + "]);\n");
//						// log
//						// sb.append("/* generateArrayConstantsInitialization 2 */\n");
//						// sb.append("/* log8 */logit(\""+getEars().getLhsArrayReference().getArrayName()+"\"");
//						// for (int i = 0; i < getEars().getOuterClosingCount();
//						// i++) {
//						// sb.append("+\"[\"+outer"+i+"+\"]\"");
//						// }
//						// sb.append(",0.0,");
//						// sb.append(array+"["+integer+"],memory.get_SAVEPER());\n");
//					}
//					// inc pointer
//					sb.append(integer + "++;\n");
//					for (int i = 0; i < getEars().getOuterClosingCount(); i++)
//						sb.append("}\n");

//					return "// this needs to be implemented as a loop returning multiple get xls lookups";
//				}
			} else if (equation.toUpperCase().contains("GET XLS CONSTANTS")) {
				// or of the form X[a,b] = GETXLSCONSTANTS...
				int numTokens = tokens.size();
				int cellLocation = numTokens - 2; // 0 index and next to last
													// token (last arg to
													// function)
				String funcCallHead = "";
				String funcCallCellStart = "";
				String funcCallCell = "";
				String funcCallTail = ")";
				for (int t = 2; t < cellLocation; t++)
					if (t == 4) {
						ModelManager mm = InformationManagers.getInstance()
								.getModelManager();
						FileOrganizer fo = InformationManagers.getInstance().getFileOrganizer();
						funcCallHead += tokens.get(t).replaceFirst("\"",
								"\"" + fo.getRelativeToApplicationDirectory(mm.getModelDirectory()) + "/");
					} else {
						funcCallHead += tokens.get(t);
					}
				funcCallCellStart =  Parser.removeQuotes(tokens.get(cellLocation)); // tokens.get(cellLocation).replace("\"", "");
				;
//				int numDimensions = asArray(extractSubscripts(lhsSubscripts)).length;
//				String columnStart = Parser
//						.getColumnLettersFromCellAddress(funcCallCellStart);
//				int rowStart = Parser.getRowFromCellAddress(funcCallCellStart);

				int numRows = 1;
				int numCols = -1;

				List<String> subs = extractSubscripts(lhsSubscripts);
				for (int i = subs.size() - 1; i >= 0; i--) {
					if (InformationManagers.getInstance()
							.getNamedSubscriptManager()
							.isNamedSubscript(subs.get(i))) {
						if (numCols == -1) {
							numCols = InformationManagers.getInstance()
									.getNamedSubscriptManager()
									.getValuesFor(subs.get(i)).size();
						} else {
							numRows *= InformationManagers.getInstance()
									.getNamedSubscriptManager()
									.getValuesFor(subs.get(i)).size();
						}
					}
				}

//				List<SubscriptCombination> combos = getSubscriptValueCombinations(asArray(extractSubscripts(lhsSubscripts)));
				// String r = getNextInteger();
				// String c = getNextInteger();
				//
				// sb.append("Integer "+r+" = new MutableInteger();\n");
				// sb.append("Integer "+r+" = new MutableInteger();\n");
				// // getNumRowsAndCols(String arrayName, String subscript,
				// MutableInteger numRows, MutableInteger numCols)
				// sb.append("ArrayManager.getNumRowsAndCols(\""+lhsArray+"\", subscript, numRows, numCols);\n");
				// String v = funcCallHead + "\"" + funcCallCellStart + "\"" +
				// ","+ r + ".intValue(),"+ c+ ".intValue()" +funcCallTail;
				String v;
				if (this.isHasLHSArrayReference()) {
					v = funcCallHead + "\"" + funcCallCellStart + "\"" + ","
						+ numRows + "," + numCols + funcCallTail;
				} else {
					v = funcCallHead + "\"" + funcCallCellStart + "\"" 
							 + funcCallTail;
				}
				
				String array = getNextArray();
				sb.append("double[] " + array + " = " + v + ";\n");
				String integer = getNextInt();
				sb.append("int " + integer + " = 0;\n");

				sb.append("// getOuter Equation 1811\n");
				sb.append(getEars().getOuterLoops());

//				if (nativeMemoryImplementation) {
//					if (newMemoryDefinition) {
						sb.append(getEars().getLHSassignmentVectorMemory());
						sb.append(" = " + array + "[" + integer + "];\n");
//					} else {
//						sb.append(getEars().getLHSassignment());
//						sb.append(" = " + array + "[" + integer + "];\n");
//
//						// log
//						sb.append("/* generateArrayConstantsInitialization 2 */\n");
//						sb.append("/* log8 */logit(\""
//								+ getEars().getLhsArrayReference()
//										.getArrayName() + "\"");
//						for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//							sb.append("+\"[\"+outer" + i + "+\"]\"");
//						}
//						sb.append(",0.0,");
//						sb.append(array + "[" + integer
//								+ "],memory.get_SAVEPER());\n");
//					}

//				} else {
//					sb.append("/* array init 2 */set(asExpandedArrayRef(\""
//							+ getEars().getLhsArrayReference().getArrayName()
//							+ "\"");
//					for (int i = 0; i < getEars().getOuterClosingCount(); i++) {
//						sb.append(",outer" + i);
//					}
//					sb.append("), " + array + "[" + integer + "]);\n");
//					// log
//					// sb.append("/* generateArrayConstantsInitialization 2 */\n");
//					// sb.append("/* log8 */logit(\""+getEars().getLhsArrayReference().getArrayName()+"\"");
//					// for (int i = 0; i < getEars().getOuterClosingCount();
//					// i++) {
//					// sb.append("+\"[\"+outer"+i+"+\"]\"");
//					// }
//					// sb.append(",0.0,");
//					// sb.append(array+"["+integer+"],memory.get_SAVEPER());\n");
//				}
				// inc pointer
				sb.append(integer + "++;\n");
				for (int i = 0; i < getEars().getOuterClosingCount(); i++)
					sb.append("}\n");

			} else {
				// or of the form X[a,b] = GETXLSDATA...
//				if (this.isHasLHSArrayReference()) {
				int numTokens = tokens.size();
				int cellLocation = numTokens - 2; // 0 index and next to last
													// token (last arg to
													// function)
				String funcCallHead = "";
				String funcCallCellStart = "";

				String funcCallTail = ")";
				for (int t = 2; t < cellLocation; t++) {
					if (t == 4) {
						ModelManager mm = InformationManagers.getInstance()
								.getModelManager();
						FileOrganizer fo = InformationManagers.getInstance().getFileOrganizer();
						funcCallHead += tokens.get(t).replaceFirst("\"",
								"\"" + fo.getRelativeToApplicationDirectory(mm.getModelDirectory()) + "/");
					} else {
						funcCallHead += tokens.get(t);
					}
				}
				funcCallCellStart =  Parser.removeQuotes(tokens.get(cellLocation)); // tokens.get(cellLocation).replace("\"", "");

				int numRows = 1;
				int numCols = -1;

				List<String> subs = extractSubscripts(lhsSubscripts);
				for (int i = subs.size() - 1; i >= 0; i--) {
					if (InformationManagers.getInstance()
							.getNamedSubscriptManager()
							.isNamedSubscript(subs.get(i))) {
						if (numCols == -1) {
							numCols = InformationManagers.getInstance()
									.getNamedSubscriptManager()
									.getValuesFor(subs.get(i)).size();
						} else {
							numRows *= InformationManagers.getInstance()
									.getNamedSubscriptManager()
									.getValuesFor(subs.get(i)).size();
						}
					}
				}

				List<SubscriptCombination> combos = getSubscriptValueCombinations(asArray(extractSubscripts(lhsSubscripts)));
				// String r = getNextInteger();
				// String c = getNextInteger();
				//
				// sb.append("Integer "+r+" = new MutableInteger();\n");
				// sb.append("Integer "+r+" = new MutableInteger();\n");
				// // getNumRowsAndCols(String arrayName, String subscript,
				// MutableInteger numRows, MutableInteger numCols)
				// sb.append("ArrayManager.getNumRowsAndCols(\""+lhsArray+"\", subscript, numRows, numCols);\n");
				// String v = funcCallHead + "\"" + funcCallCellStart + "\"" +
				// ","+ r + ".intValue(),"+ c+ ".intValue()" +funcCallTail;
				String timeSeries = getNextTimeSeries();

				String v;
				if (this.isHasLHSArrayReference()) {
					v = funcCallHead + "\"" + funcCallCellStart + "\"" + ","
						+ numRows + "," + numCols + funcCallTail;
				} else {
					v = funcCallHead + "\"" + funcCallCellStart + "\"" 
							 + funcCallTail;
				}
				
//				String v = funcCallHead + "\"" + funcCallCellStart + "\"" + ","
//						+ numRows + "," + numCols + funcCallTail;
				sb.append("TimeSeriesInstance " + timeSeries + " = " + v
						+ ";\n");
				String integer = getNextInt();
				sb.append("int " + integer + " = 0;\n");

				int outerClosingLoopCount = 0;
				if (this.hasLHSArrayReference) {
					sb.append("// getOuter Equation 1877\n");
					sb.append(getEars().getOuterLoops());
					outerClosingLoopCount = getEars().getOuterClosingCount();
				}

				// special case: using GET XLS LOOKUPS rather than GET XLS DATA
//				if (this.nativeMemoryImplementation) {
					if (this.isDefinesLookupGetXls()) {
						sb.append(getEars().getLHSassignment() + " = ");
						sb.append("sdFunctions.ADDLOOKUPNATIVE(\"" + lhsArray
								+ "\"+ " + getEars().getLHSsubscript() + ", "
								+ timeSeries + ".getTime()," + timeSeries
								+ ".valuesForIndex(" + integer + "++));");
					} else {
						if (this.hasLHSArrayReference) {
						sb.append("\ttimeSeriesData.addTimeSeries(data, \""
								+ lhsArray + "\", "
								+ getEars().getLHSsubscript() + ", "
								+ timeSeries + ".getTime()," + timeSeries
								+ ".valuesForIndex(" + integer + "++));");
						} else {

							sb.append("\ttimeSeriesData.addTimeSeries(data, \""+lhs+"\", \"\""+ ", "+timeSeries+".getTime(),"+timeSeries+".getValues(),\""+this.btwnMode+"\");\n");

							MemoryMapperInterface memoryMapper = InformationManagers.getInstance().getMemoryMapper();

							String vectorMemoryWrite = memoryMapper.getDataStore(lhs)+"["+memoryMapper.getOffset(lhs) +"]"+" = data.valueOf(\""+lhs+"\");\n";

							sb.append(vectorMemoryWrite);

							String debugVectorMemoryWriteTag = memoryMapper.getDataStore(lhs)+"["+memoryMapper.getOffset(lhs) +"]"+" = data.valueOf("+lhs+")";
							String debugVectorMemoryWriteValue = memoryMapper.getDataStore(lhs)+"["+memoryMapper.getOffset(lhs) +"]";

							sb.append("debugValue(\""+debugVectorMemoryWriteTag+"\", "+debugVectorMemoryWriteValue+");\n");

							setTimeSeriesUpdate(vectorMemoryWrite);
						}
					}
					for (int i = 0; i < outerClosingLoopCount; i++)
						sb.append("}\n");
//				} else {
//
//					// code.append("set("+ears.getLHSassignmentName());
//					// code.append(", "+valueVariable+");\n");
//					if (this.isDefinesLookupGetXls()) {
//						sb.append("set(" + getEars().getLHSassignmentName()
//								+ ",");
//						sb.append("sdFunctions.ADDLOOKUPNATIVE(\"" + lhsArray
//								+ "\"+ " + getEars().getLHSsubscript() + ", "
//								+ timeSeries + ".getTime()," + timeSeries
//								+ ".valuesForIndex(" + integer + "++)));");
//					} else {
//						sb.append("\ttimeSeriesData.addTimeSeries(data, \""
//								+ lhsArray + "\", "
//								+ getEars().getLHSsubscript() + ", "
//								+ timeSeries + ".getTime()," + timeSeries
//								+ ".valuesForIndex(" + integer + "++));");
//					}
//					for (int i = 0; i < getEars().getOuterClosingCount(); i++)
//						sb.append("}\n");
//				}

			}
		}

		return sb.toString();
	}

	// private String getCellLocation( List<SubscriptCombination> combos,
	// SubscriptCombination sub,
	// String funcCallCellStart, int numDimensions, String columnStart, int
	// rowStart) {
	// StringBuilder sb = new StringBuilder();
	//
	// // vectors and scalars are easy
	// if (numDimensions == 1) {
	// int columnOffset = 0;
	// for (columnOffset = 0; columnOffset < combos.size(); columnOffset++)
	// if (combos.get(columnOffset).equals(sub)) {
	// sb.append(getColumn(columnStart, columnOffset) + rowStart);
	// }
	// } else {
	// int columnOffset = 0;
	// for (columnOffset = 0; columnOffset < combos.size(); columnOffset++)
	// if (combos.get(columnOffset).equals(sub)) {
	// sb.append(getColumn(columnStart, columnOffset) + rowStart);
	// }
	// }
	//
	//
	// return sb.toString();
	//
	// }

	// private String getColumn(String column, int offset) {
	//
	// String allColumns = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	// String[] asArray = toStringArray(column);
	// String lastChar = asArray[asArray.length-1];
	// int pos = allColumns.indexOf(lastChar);
	//
	// if (pos + offset < allColumns.length()) {
	// asArray[asArray.length-1] = allColumns.substring(pos+offset,
	// pos+offset+1);
	// return asString(asArray);
	// } else {
	// }
	// return asString(asArray);
	// }

	private String asString(String[] asArray) {
		StringBuilder sb = new StringBuilder();
		for (String s : asArray)
			sb.append(s);
		return sb.toString();
	}

	// private String[] toStringArray(String column) {
	// String[] newCol = new String[column.length()];
	// for (int i = 0; i < newCol.length; i++)
	// newCol[i] = column.substring(i, i+1);
	// return newCol;
	// }

	private String[] asArray(List<String> items) {
		String[] array = new String[items.size()];
		for (int i = 0; i < items.size(); i++) {
			array[i] = items.get(i);
		}
		return array;
	}

	private ArrayList<String> tokenizeWithLookup(MutableInteger position,
			OperationResult or) {
		or.clear();
		ArrayList<String> al = new ArrayList<String>();
		// "Op: Backlog Saturation Effect"= WITH LOOKUP (
		// "Op: Checkpoint Service Backlog"/"Op: Initial Service Backlog",
		// ([(0,0)-(100,10)],(0,0),(0.001,1),(1,1),(100,1) ))

		al.add("(");
		position.add(1);
		skipWhiteSpace(position);

		// process min and max
		// if they exist

		String numeric;

		if (characterAt(position).equals("[")) {

			definesLookupWithRange = true;

			if (!checkIf(characterAt(position), "[", or))
				return al;
			al.add("[");
			position.add(1);
			skipWhiteSpace(position);

			if (!checkIf(characterAt(position), "(", or))
				return al;
			al.add("(");
			position.add(1);
			skipWhiteSpace(position);

			numeric = getNumberStartingAt(position, or);
			if (!or.isOk())
				return al;
			al.add(numeric);
			// position.add(1);
			skipWhiteSpace(position);

			if (!checkIf(characterAt(position), ",", or))
				return al;
			al.add(",");
			position.add(1);
			skipWhiteSpace(position);

			numeric = getNumberStartingAt(position, or);
			if (!or.isOk())
				return al;
			al.add(numeric);
			// position.add(1);
			skipWhiteSpace(position);

			if (!checkIf(characterAt(position), ")", or))
				return al;
			al.add(")");
			position.add(1);
			skipWhiteSpace(position);

			if (!checkIf(characterAt(position), "-", or))
				return al;
			al.add("-");
			position.add(1);
			skipWhiteSpace(position);

			al.add("(");
			position.add(1);
			skipWhiteSpace(position);

			numeric = getNumberStartingAt(position, or);
			if (!or.isOk())
				return al;
			al.add(numeric);
			// position.add(1);
			skipWhiteSpace(position);

			al.add(",");
			position.add(1);
			skipWhiteSpace(position);

			numeric = getNumberStartingAt(position, or);
			if (!or.isOk())
				return al;
			al.add(numeric);
			// position.add(1);
			skipWhiteSpace(position);

			al.add(")");
			position.add(1);
			skipWhiteSpace(position);

			String theChar = characterAt(position);
			Parser.skipToEndDelimiter(position, equation, "]");
			theChar = characterAt(position);

			al.add("]");
			position.add(1);
			skipWhiteSpace(position);

			theChar = characterAt(position);

		}

		boolean done = false;
		int openParens = 1;

		while (inRange(position) && !done) {
			if (Character.isDigit(characterAt(position).charAt(0))
					|| characterAt(position).equals("-")) {
				numeric = getNumberStartingAt(position, or);
				if (!or.isOk())
					return al;
				al.add(numeric);
			} else {
				if (!Character
						.isWhitespace(characterAt(position).toCharArray()[0]))
					al.add(characterAt(position));
				if (characterAt(position).equals("(")) {
					openParens++;
				} else if (characterAt(position).equals(")")) {
					openParens--;
					if (openParens == 0)
						done = true;
				}
				position.add(1);
			}
		}
		if (!done) {

			or.setErrorMessage("Closing paren(s) missing");
			return al;
		}

		// al.add(")");

		return al;
	}

	private boolean checkIf(String s1, String s2, OperationResult or) {
		or.clear();
		if (!s1.equals(s2)) {
			or.setErrorMessage("Expecting <" + s1 + "> found <" + s2 + ">");
		}
		return or.isOk();
	}

	private List<String> tokenizeSubscripts(MutableInteger position,
			OperationResult or) {
		or.clear();
		
		// position is pointing to "["
		List<String> al = new ArrayList<String>();
		position.add(1);
		skipWhiteSpace(position);
		
		// look ahead for GET XLS SUBSCRIPTS
		String startDef = getStringStartingAtPreservePosition(position, or);
		if (startDef.toUpperCase().equals("GET XLS SUBSCRIPT")) {
			getXls = true;
			// build the list based on the contents of the specified in the excel file
			List<String> funcCall = Parser.tokenize(Parser.getRestOfStringStartingAt(position, equation));
			System.out.println("funcCall -> "+funcCall);
			SDFunctionsWithXLSColt sdf = new SDFunctionsWithXLSColt();
			String modelDir = InformationManagers.getInstance().getModelManager().getModelDirectory();
			al = sdf.GETXLSSUBSCRIPT(modelDir+"/"+funcCall.get(2), funcCall.get(4), funcCall.get(6), funcCall.get(8), funcCall.size() > 10 ? funcCall.get(10) : "");
			if (al == null) {
				or.setErrorMessage("Could not extract subscript from file "+funcCall.get(2));
				InformationManagers
				.getInstance()
				.getModelManager()
				.setRunnable(false,
						"Could not extract subscript from file "+funcCall.get(2));
				return al;
			} else {
				return al;
			}
		}

		// need to check for numeric ranges of the form "(X1-X12)"
		// expand these out

		if (characterAt(position).equals("(")) {
			expandNumericRange(position, al, or);
			if (!or.isOk()) {
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add(or.getMessage());
				return al;
			}
		} else {
			MutableInteger mi1 = new MutableInteger(position);
			MutableInteger mi2 = new MutableInteger(position);
			
			String newTok = Parser.getStringStartingAtWithNonQuotedDelimitor( mi1, equation, new String[]{",", "]", "-"});
			
			al.add(Parser.getStringStartingAtWithNonQuotedDelimitor( position, equation, new String[]{",", "]","-"}));
//			al.add(getStringStartingAt(position, /* allow colon */ false, or));
			if (!or.isOk()) {
				syntacticallyCorrect = false;
				setSyntacticallyCorrect(false);
				syntaxMessages.add(or.getMessage());
				return al;
			}
		}
		skipWhiteSpace(position);
		while (inRange(position)) {

			

			skipWhiteSpace(position);
			
			MutableInteger lookAhead1 = new MutableInteger(position);
			lookAhead1.add(1);
			
			if (characterAt(position).equals("]")) {
				al.add("]");   
				break;
			}
			

			if (characterAt(position).equals("-")
					&& characterAt(lookAhead1).equals(">")) {
				return al;
			}
			al.add(characterAt(position)); // comma
			position.add(1);
			skipWhiteSpace(position);
			


			// need to check for numeric ranges of the form "(X1-X12)"
			// expand these out
			
			if (characterAt(position).equals("(")) {
				expandNumericRange(position, al, or);
				if (!or.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(or.getMessage());
					return al;
				}
			} else {
				
				MutableInteger mi1 = new MutableInteger(position);
				MutableInteger mi2 = new MutableInteger(position);
				String newTok = Parser.getStringStartingAtWithNonQuotedDelimitor( mi1, equation, new String[]{",", "]"});

//				getStringStartingAtWithNonQuotedDelimitor(MutableInteger position, String equation, String delimitor)
				al.add(Parser.getStringStartingAtWithNonQuotedDelimitor( position, equation, new String[]{",", "]"}));
//				al.add(getStringStartingAt(position, /* allow colon */ false,or));
				if (!or.isOk()) {
					syntacticallyCorrect = false;
					setSyntacticallyCorrect(false);
					syntaxMessages.add(or.getMessage());
					return al;
				}
			}
			// number
			skipWhiteSpace(position);
		}

		return al;

	}

	private void expandNumericRange(MutableInteger position,
			List<String> tokenList, OperationResult or) {

		or.clear();

		// we have a string of the form: ( x1-x23 )
		// we process up to the closing ) and expand to the appropriate number
		// of entries adding them
		// and a separating comma to the token list

		if (!or.isOk()) {
			this.syntacticallyCorrect = false;
			setSyntacticallyCorrect(false);
			this.syntaxMessages.add("Subscript Definition: " + or.getMessage());
			return;
		}

		String str = equation.substring(position.value());
		// is there a closing ")"
		if (!str.contains(")")) {

			or.setErrorMessage("Numeric Range Subscript Definition: Missing closing paren");
			return;
		}
		str = str.substring(1, str.indexOf(")")).trim();

		// str now contains x1-x23

		String l = str.split("-")[0].trim();
		String r = str.split("-")[1].trim();
		String lHead = extractHeadOfSubscript(l, or);
		if (!or.isOk()) {
			return;
		}
		String start = extractNumberFromEndOfSubscript(l, or);
		if (!or.isOk()) {
			return;
		}
		String rHead = extractHeadOfSubscript(l, or);
		if (!or.isOk()) {
			return;
		}
		String end = extractNumberFromEndOfSubscript(r, or);
		if (!or.isOk()) {
			return;
		}
		if (!lHead.equals(rHead)) {

			or.setErrorMessage("Numeric Subscript Range: subscript heads do not match");
		}
		String chars = l.replaceAll(start, "");

		int numIndex = 0;

		for (int i = Integer.parseInt(start); i <= Integer.parseInt(end); i++) {
			if (numIndex++ > 0)
				tokenList.add(",");
			tokenList.add(chars + Integer.toString(i));
		}

		// need to reposition position
		while (!characterAt(position).equals(")")) {
			position.add(1);
		}
		position.add(1);
		skipWhiteSpace(position);

	}

	private String extractHeadOfSubscript(String s, OperationResult or) {
		or.clear();
		String n = "";
		for (int i = 0; i < s.length(); i++) {
			if (!Character.isDigit(s.charAt(i))) {
				n += s.charAt(i);
			} else {
				return n;
			}
		}

		or.setErrorMessage("Non-numeric Subscript head not found");
		return "";
	}

	private String extractNumberFromEndOfSubscript(String s, OperationResult or) {
		String n = "";
		for (int i = s.length() - 1; i >= 0; i--) {
			if (Character.isDigit(s.charAt(i))) {
				n = s.charAt(i) + n;
			} else {
				return n;
			}
		}

		or.setErrorMessage("Numeric Subscript tail not found");
		return "";
	}

	private ArrayList<String> tokenizeLookup(MutableInteger position,
			OperationResult or) {
		or.clear();
		String numeric;
		// position is pointing to "("
		ArrayList<String> al = new ArrayList<String>();
		al.add("(");
		position.add(1);
		skipWhiteSpace(position);
		numeric = getNumberStartingAt(position, or);
		if (!or.isOk())
			return al;
		al.add(numeric);
		skipWhiteSpace(position);
		while (!characterAt(position).equals(")")) {
			skipWhiteSpace(position);
			al.add(characterAt(position)); // comma
			position.add(1);
			skipWhiteSpace(position);
			numeric = getNumberStartingAt(position, or);
			if (!or.isOk())
				return al;
			al.add(numeric); // number
			skipWhiteSpace(position);
		}
		al.add(")");

		return al;

	}

	private boolean inRange(int position) {
		if (position <= equation.length() - 1)
			return true;
		else
			return false;
	}

	private boolean inRange(MutableInteger position) {
		if (position.value() <= equation.length() - 1)
			return true;
		else
			return false;
	}

	private String getBooleanOperatorStartingAt(MutableInteger position,
			OperationResult or) {
		String booleanOperator = ":";
		position.add(1);
		while (inRange(position) && !characterAt(position).equals(":")) {
			booleanOperator += characterAt(position);
			position.add(1);
		}
		booleanOperator += ":";
		if (!inRange(position) || !Parser.isLogicalOperator(booleanOperator)) {

			or.setErrorMessage("Expecting boolean operator");
			return booleanOperator;
		}

		position.add(1);

		return booleanOperator;
	}

	private List<String> getSubscriptsStartingAt(MutableInteger position,
			OperationResult or) {
		// position is pointing to "("
		ArrayList<String> al = new ArrayList<String>();
		al.add("[");
		position.add(1);
		skipWhiteSpace(position);
		al.add(getStringStartingAt(position, /* allow colon */ false,or));
		if (!or.isOk()) {
			return al;
		}
		skipWhiteSpace(position);
		while (!characterAt(position).equals("]")) {
			skipWhiteSpace(position);
			al.add(characterAt(position)); // comma
			position.add(1);
			skipWhiteSpace(position);
			al.add(getStringStartingAt(position, /* allow colon */ false,or)); // number
			if (!or.isOk()) {
				return al;
			}
			skipWhiteSpace(position);
		}
		position.add(1);
		al.add("]");
		return al;
	}

	private String getExceptionSubscriptStartingAt(MutableInteger position,
			OperationResult or) {
		// position is pointing to "["
		String exceptionSubscript = "";
		// skip over the "["
		position.add(1);

		// grab what ever's nexr
		skipWhiteSpace(position);
		exceptionSubscript += getStringStartingAt(position, /* allow colon */ false,or);
		if (!or.isOk()) {
			return "";
		}
		skipWhiteSpace(position);
		// until we reach the final
		while (!characterAt(position).equals("]")) {
			skipWhiteSpace(position);
			exceptionSubscript += characterAt(position); // comma
			position.add(1);
			skipWhiteSpace(position);
			exceptionSubscript += getStringStartingAt(position, /* allow colon */ false,or); // number
			if (!or.isOk()) {
				return "";
			}
			skipWhiteSpace(position);
		}
		position.add(1);
		return exceptionSubscript;
	}

	private String getNextNonBlank(MutableInteger position) {
		MutableInteger pos = new MutableInteger(position);
		skipWhiteSpace(pos);
		return characterAt(position);
	}
	
	private String getNextNonBlankAnd(MutableInteger position, String s) {
		MutableInteger pos = new MutableInteger(position);
		skipWhiteSpaceAnd(pos, s);
//		return characterAt(position);
		return characterAt(pos);
	}
	
	private String getStringStartingAtPreservePosition(MutableInteger position,
			OperationResult or) {
		MutableInteger pos = new MutableInteger(position);
		String s = getStringStartingAt(pos, /* allow colon */ false,or);
		return s;
		
	}

	private String getStringStartingAt(MutableInteger position, boolean allowColon,
			OperationResult or) {
		String string = "";
		or.clear();
		String debugChar = null;

		if (inRange(position) && characterAt(position).equals("\"")) {
			return "\""+getQuotedStringStartingAt(position, or)+"\"";
		} else {
			while (inRange(position)
					&&
					!characterAt(position).equals("[")
					&& !characterAt(position).equals("]")
					&& (Character.isLetterOrDigit(characterAt(position).charAt(0))
							|| characterAt(position).equals("_")
							|| characterAt(position).equals("$")
							|| characterAt(position).equals(".")
							|| characterAt(position).equals("'")
						//	|| characterAt(position).equals("*")  // <<<<<<<<<<<<<<<<<<<< problem
							|| // allow single quote if in a non-quoted string
							(allowColon && characterAt(position).equals(":"))
							|| !isTerminator(characterAt(position).charAt(0)) || characterAt(
								position).equals(" "))) {
//					&& (Character.isLetter(characterAt(position).charAt(0))
//							|| Character.isDigit(characterAt(position)
//									.charAt(0))
//							|| characterAt(position).equals(" ")
//							|| characterAt(position).equals("_")
//							|| characterAt(position).equals("*")
//							|| characterAt(position).equals("\"") || characterAt(
//								position).equals("!"))) {
				string += characterAt(position);
				position.add(1);
			}
			
//			while (inRange(position)
//					&&
//					/*
//					 * Here we list any characters not part of the ordinary
//					 * terminators that we want to consider as terminators For
//					 * example, we need to stop on left and right parens
//					 */
//					!characterAt(position).equals("[")
//					&& !characterAt(position).equals("]")
//					&& (Character.isLetterOrDigit(characterAt(position).charAt(0))
//							|| characterAt(position).equals("_")
//							|| characterAt(position).equals("$")
//							|| characterAt(position).equals(".")
//							|| characterAt(position).equals("'")
//							|| // allow single quote if in a non-quoted string
//							(allowColon && characterAt(position).equals(":"))
//							|| !isTerminator(characterAt(position).charAt(0)) || characterAt(
//								position).equals(" "))) {
//				nonQuotedString += characterAt(position);
//				position.add(1);
//			}

			// return an OK result -- this is context specific and calling code
			// must determine

			return string.trim();
		}
	}

	private String getNumberStartingAt(MutableInteger position,
			OperationResult or) {
		String number = "";

		// unary minus?

		if (characterAt(position).equals("-")) {
			number = "-";
			position.add(1);
		}
		while (inRange(position)
				&& (Character.isDigit(characterAt(position).charAt(0)) || characterAt(
						position).equals("."))) {
			number += characterAt(position);
			position.add(1);
		}

		if (number.length() == 0) {

			or.setErrorMessage("Expecting numeric value");
		}

		return number;
	}

	private String getNonQuotedStringStartingAt(MutableInteger position,
			boolean allowColon, OperationResult or) {
		or.clear();
		// gobble up spaces too! (for functions)
		String nonQuotedString = "";
		while (inRange(position)
				&&
				/*
				 * Here we list any characters not part of the ordinary
				 * terminators that we want to consider as terminators For
				 * example, we need to stop on left and right parens
				 */
				!characterAt(position).equals("[")
				&& !characterAt(position).equals("]")
				&& (Character.isLetterOrDigit(characterAt(position).charAt(0))
						|| characterAt(position).equals("_")
						|| characterAt(position).equals("$")
						|| characterAt(position).equals(".")
						|| characterAt(position).equals("'")
						|| // allow single quote if in a non-quoted string
						(allowColon && characterAt(position).equals(":"))
						|| !isTerminator(characterAt(position).charAt(0)) || characterAt(
							position).equals(" "))) {
			nonQuotedString += characterAt(position);
			position.add(1);
		}

		// no error detection can be performed at this point -- in is context
		// sensitive

		return nonQuotedString.trim();

	}

	private String getQuotedStringStartingAt(MutableInteger position,
			OperationResult or) {
		or.clear();
		String quoteChar = characterAt(position);
		String quotedString = "";
		position.add(1);
		while (inRange(position) && !characterAt(position).equals(quoteChar)) {
			if (characterAt(position).equals("[")) {
				quotedString += "(";
			} else if (characterAt(position).equals("]")) {
				quotedString += ")";
			} else {
				quotedString += characterAt(position);
			}
			position.add(1);
		}

		if (!inRange(position)) {

			or.setErrorMessage("Missing end quote");
			return quotedString;
		}

		if (characterAt(position).equals(quoteChar)) {
			or.setOk(true);
		}

		position.add(1);
		// add "!" to string if present
		if (inRange(position)) {
			if (characterAt(position).equals("!")) {
				quotedString += characterAt(position);
				position.add(1);
			}
		}

		return quotedString;
	}

	private String getQuotedStringWithQuotesStartingAt(MutableInteger position,
			OperationResult or) {
		or.clear();
		String quoteChar = characterAt(position);
		String quotedString = "\\\"";
		position.add(1);
		while (inRange(position) && !characterAt(position).equals(quoteChar)) {
			quotedString += characterAt(position);
			position.add(1);
		}
		quotedString += "\\\"";

		if (characterAt(position).equals(quoteChar)) {
			or.setOk(true);
		}

		if (!inRange(position)) {

			or.setErrorMessage("Missing end quote");
		}

		position.add(1);

		return quotedString;
	}
	
	private boolean isStartOfInterpolationMode(int position) {
		if (position > equation.length() - 1) {
			log.debug("OutOfRange CharacterAt: int " + position + " EQN: "
					+ equation);
			StackTraceElement[] elements = Thread.currentThread()
					.getStackTrace();

			// for(int i=0; i<elements.length; i++) {
			// System.out.println(elements[i]);
			// }
			return false;
		}
		
		String theRest = equation.substring(position);
		return startsWithInterpolationMode(theRest);
	}
	
	private String nextCharacter(int position) {
		if (position > equation.length() - 1) {
			log.debug("OutOfRange CharacterAt: int " + position + " EQN: "
					+ equation);
			StackTraceElement[] elements = Thread.currentThread()
					.getStackTrace();

			// for(int i=0; i<elements.length; i++) {
			// System.out.println(elements[i]);
			// }
			return null;
		}
		MutableInteger pos = new MutableInteger(position+1);
		Parser.skipWhiteSpace(pos, equation);
		int p = pos.value();
		String s = equation.substring(p, p + 1);
		return equation.substring(p, p + 1);
	}

	private String characterAt(int position) {
		if (position > equation.length() - 1) {
			log.debug("OutOfRange CharacterAt: int " + position + " EQN: "
					+ equation);
			StackTraceElement[] elements = Thread.currentThread()
					.getStackTrace();

			// for(int i=0; i<elements.length; i++) {
			// System.out.println(elements[i]);
			// }
			return null;
		}
		String s = equation.substring(position, position + 1);
		return equation.substring(position, position + 1);
	}

	private String characterAt(MutableInteger position) {
		if (inRange(position)) {
			String s = equation.substring(position.value(),
					position.value() + 1);
			return equation.substring(position.value(), position.value() + 1);
		} else {
			log.debug("OutOfRange CharacterAt: MutInt " + position.value()
					+ " EQN: " + equation);
			StackTraceElement[] elements = Thread.currentThread()
					.getStackTrace();

			// for(int i=0; i<elements.length; i++) {
			// log.debug(elements[i]);
			// }
			return null;
		}
	}

	private String beyondPosition(MutableInteger position) {
		return equation.substring(position.value() + 1);
	}
	
	private void skipWhiteSpaceAnd(MutableInteger position, String s) {
		MutableInteger mi = new MutableInteger(position);
//		System.out.println("skipWhiteSpaceAnd: "+mi.value()+" "+characterAt(mi));
		while (inRange(mi)
				&& (Character.isWhitespace(characterAt(mi).charAt(0)) || characterAt(mi).equals(s))) {
//			System.out.println("skipWhiteSpaceAnd: "+mi.value()+" "+characterAt(mi));
			mi.add(1);
		}
		while (inRange(position)
				&& (Character.isWhitespace(characterAt(position).charAt(0)) || characterAt(position).equals(s))) {
			position.add(1);
		}
	}

	private void skipWhiteSpace(MutableInteger position) {
		while (inRange(position)
				&& Character.isWhitespace(characterAt(position).charAt(0)))
			position.add(1);
	}

	private boolean requiresLHS(String func) {
		if (func.equals("LOOKUP)"))
			return false;
		else
			return true;
	}

	public static boolean isSeparator(String token) {
		if (token.trim().length() == 0)
			return true;
		if (token.equals(")") || token.equals("(") || token.equals("[")
				|| token.equals("]"))
			return true;
		else
			return false;
	}

	private String clean(String s) {
		return  Parser.removeQuotes(s).replace(" ", "").replace(":", "_").replace("\t", "")
				.replace("&", "_").replace("-", "_")
				.replace("(", "_").replace(")", "_").replace("'", "");
	}

	public static boolean isTerminator(char c) {
		if (terminatorSet.contains(c))
			return true;
		else
			return false;
	}

	public static boolean isGetXlsDataFunction(String s) {
		return s.toUpperCase().equals("GET XLS DATA");
	}

	public static boolean isGetXlsConstantsFunction(String s) {
		return s.toUpperCase().equals("GET XLS CONSTANTS");
	}

	public static boolean isGetXlsLookupsFunction(String s) {
		return s.toUpperCase().equals("GET XLS LOOKUPS");
	}

	public static boolean isVectorSortOrder(String s) {
		return s.equals("VECTOR SORT ORDER");
	}

	public static boolean isVectorElmMap(String s) {
		return s.equals("VECTOR ELM MAP");
	}

	// public static boolean isFunction(String s) {
	// if (functionSet.contains(s.toUpperCase()))
	// return true;
	// else
	// return false;
	// }

	public static boolean isMacroName(String s) {
		return InformationManagers.getInstance().getMacroManager()
				.isMacroName(s);
	}

	public String getEquation() {
		return equation;
	}

	public void setEquation(String equation) {
		this.equation = equation;
	}

	public String getLhs() {
		return lhs;
	}
	
	public String getRhsFromTokens() {
		StringBuilder sb = new StringBuilder();
		int eq = tokens.indexOf("=");
		for (int i = eq+1; i < tokens.size(); i++) {
			boolean forceQuote = true;
			sb.append(InformationManagers.getInstance().getNativeDataTypeManager().
					getOriginalNameQuotedIfNecessary(Parser.removeAnnotation(tokens.get(i)), forceQuote));
		}
		
		
		return sb.toString();
	}

	// InformationManagers.getInstance().getNativeDataTypeManager()
	public String getOriginalLhs() {
		String origLHS = InformationManagers.getInstance()
				.getNativeDataTypeManager().getOriginalName(lhs);
		if (origLHS == null) {
			origLHS = InformationManagers.getInstance()
					.getNativeDataTypeManager().getOriginalName(lhs);
		}
		return origLHS;
	}

	public String getOriginalLhsQuotedIfNecessary() {
		return InformationManagers.getInstance().getNativeDataTypeManager()
				.getOriginalNameQuotedIfNecessary(lhs);
	}

	public void setLhs(String lhs) {
		this.lhs = lhs;
	}

	public String getRhs() {
		return rhs;
	}

	public void setRhs(String rhs) {
		this.rhs = rhs;
	}

	public void addReferencedBy(String equation) {
		referencedBy.add(equation);
	}

	public List<String> getReferences() {
		List<String> refs = new ArrayList<String>();

		return refs;
	}
	
	public String getOriginalVensimEquation() {
		return originalVensimEquation;
	}

	public String getOriginalVensimEquationOnly() {
		return originalVensimEquation.split("~")[0].replaceAll("\n", " ");
	}

	public String getVensimEquation() {
		return vensimEquation;
	}

	public String getVensimEquationOnly() {
		return vensimEquation.split("~")[0].replaceAll("\n", " ");
	}
	
	public String getVensimEquationOnlyExtended() {
		String normal = vensimEquation.split("~")[0].replaceAll("\n", " ");
		if (definesSubscript && getXls && syntacticallyCorrect) {
			normal += " {"+cleanEquation+"}";
		} else if (definesSubscript && getXls && !syntacticallyCorrect) {
			normal += " {"+"Unable to load subscript definition"+"}";
		}
		return normal;
	}
	
	public String getOriginalUnits() {
		return (units == null ? "" : units);
	}

	public String getUnits() {
		if (units == null)
			return "";
		return units.split("\\[")[0];
	}

	public String getMinValFromUnits() {
		if (units == null)
			return "";
		String[] range = units.split("\\[");
		if (range.length == 1)
			return "";
		return range[1].split(",")[0].trim();
	}

	public String getMaxValFromUnits() {
		if (units == null)
			return "";
		String[] range = units.split("\\[");
		if (range.length == 1)
			return "";
		String max = range[1].split(",")[1].replace("]", "").trim();
		return max;
	}

	public void setUnits(String units) {
		this.units = units;
	}
	/*
	  	public static final String SENSITIVITY = "[Function Sensitivity Parameter:";
		public static final String SOURCE = "[Source:";
		public static final String LOOKUP = "[Data Lookup:";
	 */
	
	public boolean isContainsSource() {
		if (getComment() != null) {
			return getComment().toLowerCase().contains(VariableSelector.SOURCE.toLowerCase());
		} else {
			return false;
		}
	}
	
	public String getSource() {
		String pat = "\\"+VariableSelector.SOURCE+"[^\\]]*\\]";
		Pattern p = Pattern.compile(pat);
		String comm = getComment();
		 Matcher m = p.matcher(comm);
		 boolean b = m.find();
		 if (b) {
			 return m.group(0).replace(VariableSelector.SOURCE, "").replace("]", "");
		 } else {
			 return "";
		 }
	}
	
	public String getStrippedComment() {
		return strip(comment);
	}
	
	private String strip(String s) {
		String pat = "\\"+VariableSelector.SOURCE+"[^\\]]*\\]";
//		Pattern p = Pattern.compile(pat);
		return s.replaceAll(pat, "");
	}

	public String getComment() {
		return comment;
	}

	public void setComment(String comment) {
		this.comment = comment;
	}

	public String getOther() {
		return other;
	}

	public void setOther(String other) {
		this.other = other;
	}

	public String getCleanEquation() {
		return cleanEquation;
	}

	public void setCleanEquation(String cleanEquation) {
		this.cleanEquation = cleanEquation;
	}

	public Set<String> getInvokedFunctions() {
		Set<String> s = new LinkedHashSet<String>();
		for (String t : tokens)
			if (Parser.isFunctionInvocation(t))
				s.add(getFunctionName(t));

		return s;
	}

	public HashSet<String> getRHSVariables() {
		return new LinkedHashSet<String>(rhsTokens);
	}

	public HashSet<String> getRHSVariablesExpanded() {
		HashSet<String> s = new LinkedHashSet<String>();
		if (rhsTokens == null)
			return s;
		for (String t : rhsTokens) {
			String tNoBang = t.replaceAll("!", "");
			if (ArrayReference.isArrayReference(tNoBang)) {
				s.addAll(InformationManagers.getInstance().getArrayManager()
						.expand(new ArrayReference(tNoBang)));
			} else {
				if (Parser.isFunctionInvocation(t))
					continue;
				s.add(t);
			}
		}
		removeExceptions(s);
		return s;
	}

	public boolean isDefinesLookup() {
		return definesLookup;
	}

	public void setDefinesLookup(boolean definesLookup) {
		this.definesLookup = definesLookup;
	}

	public boolean isReferencesLookup() {
		return referencesLookup;
	}

	public void setReferencesLookup(boolean referencesLookup) {
		this.referencesLookup = referencesLookup;
	}

	public boolean isHasInitialValue() {
		return hasInitialValue;
	}

	public void setHasInitialValue(boolean hasInitialValue) {
		this.hasInitialValue = hasInitialValue;
	}

	public List<String> getTokens() {
		return tokens;
	}

	public void setTokens(ArrayList<String> tokens) {
		this.tokens = tokens;
	}

	public void printTokens() {
		log.debug("EQ: " + getVensimEquation());
		log.debug("EQ: " + getCleanEquation());
		for (String t : getTokens()) {
			log.debug("   <" + t + ">");
		}
	}

	public String getTokensOneLine() {

		StringBuilder sb = new StringBuilder();
//		for (String t : getTokens()) {
//			sb.append(" <" + t + ">");
//			sb.append(Parser.removeAnnotation(t) + " ");
//		}
		sb.append(getVensimEquationOnly());

		return sb.toString();
	}

	public void printTokensOneLine() {

		for (String t : getTokens()) {
			log.debug(" <" + t + ">");
		}
		log.debug("\n");
	}

	public void printRpn() {
		log.debug("EQ: " + getCleanEquation());
		for (String t : getRpn()) {
			log.debug("   <" + t + ">");
		}
	}

	public void printRpnUnits() {
		log.debug("EQ: " + getCleanEquation());
		for (String t : getRpn()) {
			String u = "";
			if (InformationManagers.getInstance().getUnitsManager().hasUnits(t))
				u = InformationManagers.getInstance().getUnitsManager()
						.getUnits(t);
			log.debug("   <" + t + "> <" + u + ">");
		}
	}

	public void generateRpn() {
		boolean printRPN = false;
		if (!isAssignment())
			return;
		// log.debug(cleanEquation);

		rpn.clear();
		Stack<String> stack = new Stack<String>();

		int tPtr = 0;

		boolean locatedEquals = false;

		// work with a temporary version of tokens in which arrays and
		// subscripts are combined
		// into a single token for each occurance

		int functionsActive = 0;
		int activeParens = 0;

		List<String> combinedTokens = new ArrayList<String>(tokens);

		int j = 0;

		for (String tokenIn : combinedTokens) {

			String token = tokenIn; // .replace("lookup.", "memory.");

			if (token.length() == 0) {
				continue;
			}
			// log.debug("### "+token);
			// skip the first two tokens (varname and =)
			if (token.equals("=")) {
				locatedEquals = true;
				continue;
			}
			if (!locatedEquals)
				continue;
			if (Parser.isNumber(token) || isVariableReference(token)
					|| isLookupReference(token)) {
				rpn.add(token);
			} else if (isFunctionInvocation(token) || isMacroInvocation(token)) {
				functionsActive++;
				stack.push(token);
			} else if (isFunctionArgumentSeparator(token)
					&& functionsActive > 0) {
				while (!stack.peek().equals("(")) {
					rpn.add(stack.pop());
				}
			} else if (Parser.isOperator(token)
					|| Parser.isBooleanOperator(token)) {
				while (stack.size() > 0
						&& (Parser.isOperator(stack.peek()) || Parser
								.isBooleanOperator(stack.peek()))
						&& ((leftAssociative.get(token) && precedence
								.get(token).intValue() <= precedence.get(
								stack.peek()).intValue()) || (!leftAssociative
								.get(token) && precedence.get(token).intValue() < precedence
								.get(stack.peek()).intValue()))) {
					rpn.add(stack.pop());
				}
				stack.push(token);

			} else if (isLeftParen(token)) {
				activeParens++;
				stack.push(token);
			} else if (isRightParen(token)) {
				try {
					while (!stack.peek().equals("(")) {
						rpn.add(stack.pop());
						// need to insert an end of argument list marker to
						// support variable length argument lists
						// rpn.add(ARG_LIST_END_MARKER);
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				stack.pop();
				if (activeParens == functionsActive)
					functionsActive--; // where does this belong?
				activeParens--;
				if (stack.size() > 0
						&& (isFunctionInvocation(stack.peek()) || isMacroInvocation(stack
								.peek())))
					rpn.add(stack.pop());
			}
		}

		while (stack.size() > 0)
			rpn.add(stack.pop());

		ArrayList<String> al = new ArrayList<String>();
		al.add(combinedTokens.get(0));
		al.addAll(rpn);
		al.add("=");
		rpn = al;
	}

	private boolean isLookupReference(String token) {
		if (token.startsWith("lookup."))
			return true;
		else
			return false;
	}

	private boolean isVariableReference(String token) {
		if (token.startsWith("\"") || token.startsWith("valueOf")
				|| token.startsWith("memory.") || token.startsWith("time")
				|| token.startsWith("array.")
				|| token.startsWith("arrayValueOf"))
			return true;
		else
			return false;
	}

	private boolean isFunctionInvocation(String token) {
		if (token.startsWith("sdFunction"))
			return true;
		else
			return false;
	}

	private boolean isMacroInvocation(String token) {
		// always return false as we have already processed
		return false;
	}

	private boolean isFunctionArgumentSeparator(String token) {
		if (token.equals(","))
			return true;
		else
			return false;
	}

	private boolean isLeftParen(String token) {
		if (token.equals("("))
			return true;
		else
			return false;
	}

	private boolean isRightParen(String token) {
		if (token.equals(")"))
			return true;
		else
			return false;
	}

	public List<String> getRpn() {
		return rpn;
	}

	public void setRpn(ArrayList<String> rpn) {
		this.rpn = rpn;
	}

	public boolean isAssignment() {
		return assignment;
	}

	public void setAssignment(boolean assignment) {
		this.assignment = assignment;
	}

	public boolean isGetExternalData() {
		if (getVensimEquationOnly().toUpperCase().contains("GET XLS LOOKUPS")
				|| getVensimEquationOnly().toUpperCase().contains("GET XLS DATA")
				|| getVensimEquationOnly().toUpperCase().contains("GET XLS CONSTANTS")
				|| getVensimEquationOnly().toUpperCase().contains("GET DIRECT LOOKUPS")
				|| getVensimEquationOnly().toUpperCase().contains("GET DIRECT DATA")
				|| ( getEquation() != null && getEquation().toUpperCase().contains(Parser.externalDataMethod()))
				|| getVensimEquationOnly().toUpperCase().contains("GET DIRECT CONSTANTS"))
			return true;
		else
			return false;
	}

	public String generateExpressionForInitialization(Node node) {
		StringBuilder sb = new StringBuilder();

		NativeDataTypeManager ndtm = InformationManagers.getInstance()
				.getNativeDataTypeManager();

		if (node != null) {

			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child
			if (Parser.isArithmeticOperator(node.getToken())
					|| Parser.isEqualSign(node.getToken())
					|| Parser.isRelationalOperator(node.getToken())) {
				if (!Parser.isEqualSign(node.getToken()))
					sb.append("(");
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(generateExpressionForInitialization(TreeTraversal
							.getLhs(node))); // node.getChild()
					sb.append(node.getToken());
					sb.append(generateExpressionForInitialization(TreeTraversal
							.getRhs(node))); // node.getChild().getNext()
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(generateExpressionForInitialization(node
							.getChild())); // this is really a LHS location, but
											// RHS of unary
				}
				if (!Parser.isEqualSign(node.getToken()))
					sb.append(")");
				return sb.toString();

			} else if (Parser.isFunctionInvocation(node.getToken())) {
				// sb.append(node.getToken());
				sb.append(generateFunctionCall(node));
			} else {
				sb.append(ndtm.getOriginalName(node.getToken()));
			}

		}
		return sb.toString();
	}

	public String generateFunctionCall(Node node) {
		StringBuilder sb = new StringBuilder();

		if (node != null) {
			// create the entire invocation
			// node is the function name
			sb.append(node.getToken());
			sb.append("(");
			int i = 0;

			Node arg = TreeTraversal.getFunctionArgument(node, 1);
			while (arg != null) {
				if (i++ > 0)
					sb.append(",");
				sb.append(generateExpression(arg));
				arg = arg.getNext();
			}

			sb.append(")");

		}

		return sb.toString();
	}

	public String generateExpression(Node node) {
		StringBuilder sb = new StringBuilder();

		if (node != null) {

			// node is the root node of an expression
			// if an operator:
			// left child
			// operator
			// right child
			if (Parser.isArithmeticOperator(node.getToken())
					|| Parser.isEqualSign(node.getToken())
					|| Parser.isRelationalOperator(node.getToken())) {
				if (!Parser.isEqualSign(node.getToken()))
					sb.append("(");
				if (!Parser.isUnaryOperator(node.getToken())) {
					sb.append(generateExpression(TreeTraversal.getLhs(node))); // node.getChild()
					sb.append(node.getToken());
					sb.append(generateExpression(TreeTraversal.getRhs(node))); // node.getChild().getNext()
				} else {
					sb.append(Parser.translateUnaryOperator(node.getToken()));
					sb.append(generateExpression(node.getChild())); // this is
																	// really a
																	// LHS
																	// location,
																	// but RHS
																	// of unary
				}
				if (!Parser.isEqualSign(node.getToken()))
					sb.append(")");
				return sb.toString();

			} else if (Parser.isFunctionInvocation(node.getToken())) {
				// sb.append(node.getToken());
				sb.append(generateFunctionCall(node));
			} else {
				sb.append(node.getToken());
			}

		}

		return sb.toString();
	}
	
	public void inlineInteg() {
		generateTree();
		
		Node root = getTreeRoot();
		Node lhs = root.getChild();
		Node integ = lhs.getNext();
		
		
		Node arg1 = integ.getChild();
		Node arg2 = arg1.getNext();
		
		Node times = new Node("*");
		Node ts = new Node("TIME STEP");
		
		// integ now become the increment
		// the first argument remains its child
		integ.setToken("+");
		
		// arg1 is the current value (i.e. the lhs variable)
		// its next is the times operator
		
		arg1.setNext(times);
		
		times.setChild(arg2);
		times.setParent(integ);
		times.setNext(null);
		times.setPrevious(arg1);
		
		
		arg2.setPrevious(null);
		arg2.setParent(times);
		arg2.setNext(ts);		
		
		ts.setPrevious(arg2);
		ts.setParent(times);
		
	}

	public void generateTree() {
		treeRoot = generateEquationTree();
		validateIFTHENELSEInTree(treeRoot);
	}

	public Node generateEquationTree() {
		

		boolean printit = false;

		Node start = null;
		Node end = null;

		int nPtr = 0;

		if (rpn.size() == 0) {
			this.generateRpn();

		}

		for (String token : rpn) {
			if (start != null && printit) {
				printTree(start);
			}

			Node aNode = new Node(rpn.get(nPtr));
			if (nPtr == 0) {
				start = aNode;
				end = aNode;
			} else if (Parser.isOperator(token)
					|| Parser.isBooleanOperator(token)) {

				Node p1 = null;
				Node p2 = null;
				Node p3 = null;

				// if binary take 2

				if (Parser.isBinaryOperator(token)) {

					p1 = end;
					p2 = end.getPrevious();
					if (p2 == null) {
						System.out.println("Expecting a previous pointer for p1 "
								+ getVensimEquationOnly());
						System.out.println("token = " + token + " nPtr = " + nPtr);
//						System.out.println(p1.getInfo());
					}
					
					try {
						p3 = end.getPrevious().getPrevious();
					} catch (Exception e) {
						// TODO Auto-generated catch block
						return null;
					}

					// make required changes to structure
					aNode.setChild(p2);
					aNode.setPrevious(p3);
					if (p2 != null) {
						p2.setPrevious(null);
						p2.setParent(aNode);
						p2.setNext(p1);
					}
					p1.setParent(aNode);
					end = aNode;

					if (p3 != null) {
						p3.setNext(aNode);
						aNode.setPrevious(p3);
					} else {
						start = aNode;
					}
				} else {
					// unary operator
					p1 = end;
					p2 = end.getPrevious();

					aNode.setChild(p1);
					aNode.setPrevious(p2);
					if (p2 != null) {
						p2.setNext(aNode);
					} else {
						start = aNode;
					}
					if (p1 != null) {
						p1.setPrevious(null);
						p1.setParent(aNode);
						p1.setNext(null);

					}
					p1.setParent(aNode);
					end = aNode;
				}

			} else if (isFunctionInvocation(token)) {
				// if a functionInvocation
				// int numArgs =
				// numberArguments.get(getFunctionName(token).toUpperCase());
				FunctionDescription fd = InformationManagers.getInstance()
						.getFunctionManager().getDescription(token);
				if (fd == null)
					System.out.println("Null Description");

				// need to worry about functions that have no arguments

				int numArgs = fd.getNumArgsAll();
				boolean varArg = fd.isVariableLengthArguments();

				if (varArg) {
					// this.printRpn();
					// need some tricks here to deteemine variable arg lists
					// search for "name" entry as first arg
					String marker = "\"";
					Node p = end;
					int n = 1;
					while (!p.getToken().startsWith(marker)) {
						n++;
						p = p.getPrevious();
					}
					numArgs = n;
				}

				if (numArgs > 0) {

					Node[] args = new Node[numArgs];
					args[numArgs - 1] = end;
					for (int i = numArgs - 2; i >= 0; i--) {
						args[i] = end.getPrevious();
						end = end.getPrevious();
					}

					if (end != null)
						end = end.getPrevious();

					if (end != null) {
						aNode.setPrevious(end);
						end.setNext(aNode);
					} else {
						start = aNode;
					}
					end = aNode;
					for (int i = 0; i < numArgs; i++) {
						args[i].setParent(aNode);
						// log.debug("    Funtion arg "+i+" "+args[i].getToken());
					}
					aNode.setChild(args[0]);
					args[0].setPrevious(null);
				} else {
					// if there are no args, add to end
					if (end != null) {
						aNode.setPrevious(end);
						end.setNext(aNode);
					} else {
						start = aNode;
					}
					end = aNode;

				}
			} else if (isMacroInvocation(token)) {
				// if a macro Invocation
				int numArgs = InformationManagers.getInstance()
						.getMacroManager().getNumArgumentsFor(token);

				Node[] args = new Node[numArgs];
				args[numArgs - 1] = end;
				for (int i = numArgs - 2; i >= 0; i--) {
					args[i] = end.getPrevious();
					end = end.getPrevious();
				}

				if (end != null)
					end = end.getPrevious();

				if (end != null) {
					aNode.setPrevious(end);
					end.setNext(aNode);
				} else {
					start = aNode;
				}
				end = aNode;
				for (int i = 0; i < numArgs; i++) {
					args[i].setParent(aNode);
				}
				aNode.setChild(args[0]);
				args[0].setPrevious(null);
			} else {
				end.setNext(aNode);
				aNode.setPrevious(end);
				end = aNode;
			}

			// printTreeLevel1(start);
			nPtr++;
		}
		if (start != null && printit) {
			this.printTokens();
			this.printRpn();
			printTree(start);

		}
		printit = false;

		// catch array initializtion?

		if (start == null) {
			log.debug("What is this with no tree?");
		} else {
			if (!Parser.isOperator(start.getToken())) {
				// if end is "=", promote it to root and make start it's child
				if (end.getToken().equals("=")) {
					arrayInitialization = true;

					// MJB 4/6/14 These next four lines of code were commented
					// out. WHY???
					// end.setChild(start);
					// end.setNext(null);
					// start.setParent(end);
					// start = end;
				} else {
					log.debug("Start node is not an operator! Nor array initialization");
					log.debug(getVensimEquation());
				}

			}
		}

		return start;

	}

	public void printTreeLevel1(Node node) {
		if (node == null) {
			log.debug("End of List");
			return;
		}
		log.debug("NODE: " + node.getToken());
		printTreeLevel1(node.getNext());
	}

	private String getFunctionName(String token) {
		return token.replace("sdFunctions.", "");
	}

	public void printTree(Node node) {
		log.debug(cleanEquation);
		System.out.println("Print Tree: node = "+node.getToken());
		printTree(node, 0);
	}

	public void printTree() {
		log.debug(getVensimEquationOnly());
		printTree(getTreeRoot(), 0);
	}

	private void printTree(Node node, int level) {
		if (node == null)
			return;
		String blanks = "";
		for (int i = 0; i < level; i++)
			blanks += "    ";
		System.out.println(blanks
				+ "Level "
				+ level
				+ " token "
				+ node.getToken()
				+ "\n"
				+ blanks
				+ "   parent: "
				+ (node.getParent() != null ? node.getParent().getToken()
						: "NULL") + "\n" + blanks + "   next: "
				+ (node.getNext() != null ? node.getNext().getToken() : "NULL"));
		printTree(node.getChild(), level + 1);
		printTree(node.getNext(), level);
	}

	public void printTreeTerse() {
		log.debug(cleanEquation);
		printTreeTerse(treeRoot, 0);
	}

	public static void printTreeCausal(Node node, int level,
			Map<String, Equation> equations) {
		if (node == null)
			return;
		String blanks = "";
		for (int i = 0; i < level; i++)
			blanks += "    ";
		if (node.getToken() == null) {
			log.debug(blanks + "Level " + level + " token " + node.getToken()
					+ "????? NULL TOKEN");
		} else if (equations.get(node.getToken()) == null) {
			log.debug(blanks + "Level " + level + " token " + node.getToken()
					+ "????? NULL EQUATION");
		} else {
			log.debug(blanks
					+ "Level "
					+ level
					+ " token "
					+ node.getToken()
					+ (equations.get(node.getToken()).isStock() ? "<STOCK>"
							: ""));
		}
		printTreeCausal(node.getChild(), level + 1, equations);
		printTreeCausal(node.getNext(), level, equations);
	}

	private void printTreeTerse(Node node, int level) {
		if (node == null)
			return;
		String blanks = "";
		for (int i = 0; i < level; i++)
			blanks += "    ";
		log.debug(blanks + "Level " + level + " token " + node.getToken());
		System.out.println(blanks + "Level " + level + " token " + Parser.removeAnnotation(node.getToken()));
		printTreeTerse(node.getChild(), level + 1);
		printTreeTerse(node.getNext(), level);
	}

	public void printTreeCode() {
		log.debug("######################################################");
		log.debug(cleanEquation);
		printTreeCode(treeRoot, 0);
	}

	private void printTreeCode(Node node, int level) {
		if (node == null)
			return;
		String blanks = "";
		for (int i = 0; i < level; i++)
			blanks += "    ";
		log.debug(blanks + "Level " + level + "\n" + node.getInfo(blanks));
		printTreeCode(node.getChild(), level + 1);
		printTreeCode(node.getNext(), level);
	}

	public List<String> getEquationUnits() {
		List<String> eqn = new ArrayList<String>();
		// log.debug("=====================================================");
		// log.debug(cleanEquation);
		printTreeCodeUnits(eqn, treeRoot, 0);
		// need to get rid of first and last entries in eqn (uneeded parens)
		if (eqn.size() > 1) {
			eqn.remove(eqn.size() - 1);
			eqn.remove(0);
		}
		return eqn;
	}

	private void printTreeCodeUnits(List<String> eqn, Node node, int level) {
		if (node == null) {
			// log.debug("printTreeCodeUnits: terminate");
			return;
		}
		if (node.isTerminal()) {
			if (Parser.isNumber(node.getToken())) {
				// log.debug(node.getToken()+"<"+"constant"+">");
				eqn.add("constant" + "#" + node.getToken());
			} else {
				// log.debug(node.getToken()+"<"+InformationManagers.getInstance().getUnitsManager().getUnits(node.getToken())+">");
				String v = InformationManagers.getInstance().getUnitsManager()
						.getUnits(node.getToken());
				// if (v == null)
				// log.debug("V is null");
				eqn.add(InformationManagers.getInstance().getUnitsManager()
						.getUnits(node.getToken()));
			}
		}

		if (Parser.isOperator(node.getToken())) {
			Node leftChild = node.getChild();
			Node rightChild = leftChild != null ? leftChild.getNext() : null;
			eqn.add("(");
			printTreeCodeUnits(eqn, leftChild, level + 1);
			// log.debug(level+" "+node.getToken()+"<"+node.getUnits()+">");
			eqn.add(node.getToken());
			// log.debug(node.getToken());
			printTreeCodeUnits(eqn, rightChild, level + 1);
			eqn.add(")");
		}

		if (Parser.isFunctionInvocation(node.getToken())) {
			eqn.add(node.getToken()
					+ "<"
					+ InformationManagers.getInstance().getUnitsManager()
							.getUnits(node.getToken()) + ">");
			eqn.add("(");
			// log.debug(node.getToken()+"<"+InformationManagers.getInstance().getUnitsManager().getUnits(node.getToken())+">");
			Node child = node.getChild();
			int c = 0;
			while (child != null) {
				if (c++ > 0)
					eqn.add(",");
				printTreeCodeUnits(eqn, child, level + 1);
				child = child.getNext();
			}
			eqn.add(")");
		}
	}

	public ArrayList<String> getFunctionInitialVariablesAndConstants() {
		ArrayList<String> al = new ArrayList<String>();
		getFunctionInitialVariablesAndConstants(treeRoot, al);
		return al;
	}

	public void getFunctionInitialVariablesAndConstants(Node node,
			ArrayList<String> al) {
		if (node == null)
			return;

		if (Parser.isFunctionInvocation(node.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (fd.isSuppliesInitialValue()) {
				Node child = node.getChild();
				while (child.getNext() != null)
					child = child.getNext();
				// al.add(child.getToken()); // mjb need to traverse this tree
				// root for all variables

				// System.out.println("Old initVar: "+child.getToken());
				List<Node> nodeList = new ArrayList<Node>();
				findTerminal(child, nodeList);
				// System.out.println(" - Start -");

				for (Node n : nodeList) {
					if (Parser.isQuotedString(n.getToken()))
						continue;
					// allow constants to pass through
					// if (Parser.isNumber(n.getToken()))
					// continue;
					// System.out.println("initVar: "+n.getToken());
					if (!al.contains(n.getToken()))
						al.add(n.getToken());
				}

				// System.out.println(" - End -");
			}
		}

		getFunctionInitialVariablesAndConstants(node.getNext(), al);
		getFunctionInitialVariablesAndConstants(node.getChild(), al);

		return;

	}

	public Set<String> getFunctionInitialVariables() {
		Set<String> set1 = new LinkedHashSet<String>();
		Set<String> set2 = new LinkedHashSet<String>();
		getFunctionInitialVariables(TreeTraversal.getRhs(treeRoot), set1);
		// getNonFunctionInitialVariables(TreeTraversal.getRhs(treeRoot), set2);

		set1.removeAll(set2);
		return set1;
	}

	public void getFunctionInitialVariables(Node node, Set<String> al) {
		if (node == null)
			return;

		if (Parser.isFunctionInvocation(node.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (fd.isSuppliesInitialValue()) {
				// this finds the last arg to the function, the initial value
//				Node child = TreeTraversal.getLastArgument(node);
				Node child = TreeTraversal.getFunctionArgument(node, fd.getInitialValueArg());
				while (child.getNext() != null)
					child = child.getNext();

				List<Node> nodeList = new ArrayList<Node>();
				findTerminal(child, nodeList);

				for (Node n : nodeList) {
					if (n.getParent() != null && Parser.isLogicalOperator(n.getParent().getToken()))
						continue;
					if (Parser.isQuotedString(n.getToken()))
						continue;
					if (Parser.isNumber(n.getToken()))
						continue;
					if (!al.contains(n.getToken()))
						al.add(n.getToken());
				}
			}
		}

//		for (Node node : terminals) {
//			if (!Parser.isRelationalOperator(node.getParent().getToken())
//					&& !Parser.isArithmeticOperator(node.getParent()
//							.getToken())
//					&& !Parser.isFunctionInvocation(node.getParent()
//							.getToken()))
//				replaceBooleanVariable(node);
//		}
		// getFunctionInitialVariables(node.getNext(), al);
		// getFunctionInitialVariables(node.getChild(), al);

		return;

	}
	
	public Set<String> getNonFunctionInitialVariables() {
		Set<String> set1 = new LinkedHashSet<String>();
		Set<String> set2 = new LinkedHashSet<String>();
		getNonFunctionInitialVariables(TreeTraversal.getRhs(treeRoot), set1);
		// getNonFunctionInitialVariables(TreeTraversal.getRhs(treeRoot), set2);

		set1.removeAll(set2);
		return set1;
	}

	public void getNonFunctionInitialVariables(Node node, Set<String> al) {
		if (node == null)
			return;

		if (Parser.isFunctionInvocation(node.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (fd.isSuppliesInitialValue()) {
				// this finds the last arg to the function, the initial value
				Node child = TreeTraversal.getFunctionArgument(node, 2);  // 1

				List<Node> nodeList = new ArrayList<Node>();

				Node limitSearch = child.getNext();
				child.setNext(null);
				findTerminal(child, nodeList);
				child.setNext(limitSearch);

				for (Node n : nodeList) {
					if (Parser.isQuotedString(n.getToken()))
						continue;
					if (Parser.isNumber(n.getToken()))
						continue;
					if (!al.contains(n.getToken()))
						al.add(n.getToken());
				}
			}
		}

		// getFunctionInitialVariables(node.getNext(), al);
		// getFunctionInitialVariables(node.getChild(), al);

		return;

	}

	// =================

	public Set<String> getRHSVariablesForOrderingExpanded() {
		Set<String> s = new LinkedHashSet<String>();

		for (String rhsVar : getRHSVariablesForOrdering()) {
			if (ArrayReference.isArrayReference(rhsVar)) {
				s.addAll(InformationManagers.getInstance().getArrayManager()
						.expand(new ArrayReference(rhsVar)));
			} else {
				s.add(rhsVar);
			}
		}
		removeExceptions(s);
		return s;
	}

	public Set<String> getRHSVariablesForOrdering(boolean trace) {
		Set<String> al = new LinkedHashSet<String>();
		getRHSVariablesForOrdering(TreeTraversal.getRhs(getTreeRoot()), al,
				trace);
		return al;
	}

	public void getRHSVariablesForOrdering(Node node, Set<String> al,
			boolean trace) {
		// if (node == null) {
		// if (trace)
		// log.debug("Null Node return");
		// return;
		// }
		// Node aNode = node;
		// if (trace)
		// log.debug("Node: "+aNode.getToken());
		// // looking at a function, need to strip generated arguments and
		// process other args
		// if (Parser.isFunctionInvocation(aNode.getToken())) {
		// if (trace) {
		// log.debug("Function: "+aNode.getToken());
		// }
		// FunctionDescription fd =
		// InformationManagers.getInstance().getFunctionManager().getDescription(node.getToken());
		// aNode = TreeTraversal.getFunctionArgument(aNode, 1);
		// if (trace)
		// log.debug("arg node: "+aNode.getToken());
		// getRHSVariablesForOrdering(aNode, al, trace);
		// } else {
		// String token = aNode.getToken();
		// if (!Parser.isOperator(token) &&
		// !Parser.isQuotedString(token) &&
		// !Parser.isNumber(token) &&
		// !Parser.isLeftParen(token) &&
		// !Parser.isRightParen(token)) {
		// al.add(token);
		// if (trace)
		// log.debug("var? "+token);
		// }
		//
		// }
		//
		// if (aNode != null)
		// getRHSVariablesForOrdering(aNode.getNext(), al, trace);
		// if (aNode != null)
		// getRHSVariablesForOrdering(aNode.getChild(), al, trace);
		al.addAll(rhsTokens);
	}

	// =======================

	public Set<Node> getLookupFunctionNodes(boolean trace) {
		Set<Node> aSet = new LinkedHashSet<Node>();
		getLookupFunctionNodes(TreeTraversal.getRhs(getTreeRoot()), aSet, trace);
		if (trace)
			log.debug("Returning Set of size: " + aSet.size());
		return aSet;
	}

	public void getLookupFunctionNodes(Node node, Set<Node> aSet, boolean trace) {
		if (node == null) {
			if (trace)
				log.debug("Null Node return");
			return;
		}
		Node aNode = node;
		if (trace)
			log.debug("Node: " + aNode.getToken());
		// looking at a function, need to strip generated arguments and process
		// other args
		if (Parser.isFunctionInvocation(aNode.getToken())
				&& Parser.getFunctionName(aNode.getToken()).equals("LOOKUP")) {
			if (trace) {
				log.debug("Function: " + aNode.getToken());
			}
			aSet.add(aNode);
		}

		if (aNode != null)
			getLookupFunctionNodes(aNode.getNext(), aSet, trace);
		if (aNode != null)
			getLookupFunctionNodes(aNode.getChild(), aSet, trace);
	}
	
	public Set<String> getRHSVariablesForOrderingActiveInitial() {
		Set<String> al = new LinkedHashSet<String>();
		if (this.isStock()) {
			Node root = this.getTreeRoot();
			Node integ = TreeTraversal.getRhs(root);
			Node arg2 = TreeTraversal.getFunctionArgument(integ, 1);
			al.addAll(Node.extractVariables(arg2));
		} else {
			// if is an activeinitial, we need the ctive values
			if (isActiveInitial()) {
				getRHSVariablesForOrderingNotInitial(TreeTraversal.getRhs(getTreeRoot()), al);
			} else {
				// but if another function, we'd want initial values
				getRHSVariablesForOrdering(TreeTraversal.getRhs(getTreeRoot()), al);
			}
		}
		return al;
	}
	
	public void getRHSVariablesForOrderingNotInitial(Node node, Set<String> al) {
		if (node == null)
			return;
		Node aNode = node;

		// looking at a function, need to strip generated arguments and process
		// other args
		if (Parser.isFunctionInvocation(aNode.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (!fd.getExternalName().equals("ELMCOUNT")) {
				int startArg = 1;
				int endArg = fd.getNumArgs();
				if (fd.isSuppliesInitialValue())
					endArg--;
				for (int argNum = startArg; argNum <= endArg; argNum++) {
					Node anotherNode = TreeTraversal.getFunctionArgument(aNode, argNum);
					getRHSVariablesForOrderingNotInitial(anotherNode, al);
				}
			} else {
				aNode = aNode.getNext();
			}
		} else {
			String token = aNode.getToken();
			if (!Parser.isOperator(token) && !Parser.isQuotedString(token)
					&& !Parser.isNumber(token) && !Parser.isLeftParen(token)
					&& !Parser.isRightParen(token))

				al.add(token);
		}

		if (aNode != null)
			getRHSVariablesForOrderingNotInitial(aNode.getChild(), al);
		// this is where we terminate too soon, we never get through next of
		// original node!
		if (aNode != null) {
			// getRHSVariablesForOrdering(aNode.getNext(), al);
			if (!(aNode.getNext() != null && aNode.getNext().getNext() == null))
				getRHSVariablesForOrderingNotInitial(node.getNext(), al);
		}

		return;

	}
	
	public Set<String> getStockArg1Variables() {
		Set<String> al = new LinkedHashSet<String>();
		if (this.isStock()) {
			Node root = this.getTreeRoot();
			Node integ = TreeTraversal.getRhs(root);
			Node arg1 = TreeTraversal.getFunctionArgument(integ, 1);
			al.addAll(Node.extractVariables(arg1));
		}
		return al;
	}
	
	public Set<String> getRHSVariablesForObjectManager() {
		Set<String> al = new LinkedHashSet<String>();
		
			getRHSVariablesForOrdering(TreeTraversal.getRhs(getTreeRoot()), al);
		return al;
	}

	public Set<String> getRHSVariablesForOrdering() {
		Set<String> al = new LinkedHashSet<String>();
		if (this.isStock()) {
			Node root = this.getTreeRoot();
//			Node.printTreeToConsole(root);
			Node integ = TreeTraversal.getRhs(root);
			Node arg2 = TreeTraversal.getFunctionArgument(integ, 2);
			al.addAll(Node.extractVariables(arg2));
		} else {
			getRHSVariablesForOrdering(TreeTraversal.getRhs(getTreeRoot()), al);
		}
		return al;
	}

	public List<String> getRHSVariablesForOrderingSorted() {

		Set<String> al = new LinkedHashSet<String>();
		getRHSVariablesForOrdering(TreeTraversal.getRhs(getTreeRoot()), al);
		List<String> aList = new ArrayList<String>(al);
		Collections.sort(aList, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				return arg0.length() == arg1.length() ? 0
						: arg0.length() < arg1.length() ? 1 : -1;
			}
		});
		return aList;
	}

	public List<String> getRHSVariablesForHREF() {

		Set<String> al = new LinkedHashSet<String>();
		getRHSVariablesForHREF(TreeTraversal.getRhs(getTreeRoot()), al);
		List<String> aList = new ArrayList<String>(al);
		Collections.sort(aList, new Comparator<String>() {
			@Override
			public int compare(String arg0, String arg1) {
				return arg0.length() == arg1.length() ? 0
						: arg0.length() < arg1.length() ? 1 : -1;
			}
		});
		return aList;
	}

	public void getRHSVariablesForOrdering(Node node, Set<String> al) {
		if (node == null)
			return;
		Node aNode = node;

		// looking at a function, need to strip generated arguments and process
		// other args
		if (Parser.isFunctionInvocation(aNode.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (!fd.getExternalName().equals("ELMCOUNT")) {
				int startArg = 1;
				if (fd.isSuppliesInitialValue())
					startArg = fd.getNumArgs();
				aNode = TreeTraversal.getFunctionArgument(aNode, startArg);
				getRHSVariablesForOrdering(aNode, al);
			} else {
				aNode = aNode.getNext();
			}
		} else {
			String token = aNode.getToken();
			if (!Parser.isOperator(token) && !Parser.isQuotedString(token)
					&& !Parser.isNumber(token) && !Parser.isLeftParen(token)
					&& !Parser.isRightParen(token)) {

				if (!al.contains(token))
						al.add(token);
			}
		}

		if (aNode != null)
			getRHSVariablesForOrdering(aNode.getChild(), al);
		// this is where we terminate too soon, we never get through next of
		// original node!
		if (aNode != null) {
			// getRHSVariablesForOrdering(aNode.getNext(), al);
			getRHSVariablesForOrdering(node.getNext(), al);
		}

		return;

	}

	public void getRHSVariablesForHREF(Node node, Set<String> al) {
		if (node == null)
			return;
		Node aNode = node;

		// looking at a function, need to strip generated arguments and process
		// other args
		if (Parser.isFunctionInvocation(aNode.getToken())) {
			FunctionDescription fd = InformationManagers.getInstance()
					.getFunctionManager().getDescription(node.getToken());
			if (!fd.getExternalName().equals("ELMCOUNT")) {
				int startArg = 1;
				// if (fd.isSuppliesInitialValue())
				startArg = 1;
				aNode = TreeTraversal.getFunctionArgument(aNode, startArg);
				getRHSVariablesForOrdering(aNode, al);
			} else {
				aNode = aNode.getNext();
			}
		} else {
			String token = aNode.getToken();
			if (!Parser.isOperator(token) && !Parser.isQuotedString(token)
					&& !Parser.isNumber(token) && !Parser.isLeftParen(token)
					&& !Parser.isRightParen(token))

				al.add(token);
		}

		if (aNode != null)getRHSVariablesForHREF
			(aNode.getChild(), al); // getRHSVariablesForOrdering
		// this is where we terminate too soon, we never get through next of
		// original node!
		if (aNode != null) {
			// getRHSVariablesForOrdering(aNode.getNext(), al);
			getRHSVariablesForHREF(node.getNext(), al); // getRHSVariablesForOrdering
		}

		return;

	}

	public Node getCopyOfTree() {
		return generateEquationTree();
	}

	public Node getTreeRoot() {
		if (treeRoot == null)
			this.generateTree();
		if (treeRoot == null) {
			if (!this.isArrayInitialization() && !definesLookup) {
//				log.debug("Cannot generate tree: " + getVensimEquationOnly());
				System.out.println("Cannot generate tree: " + getVensimEquationOnly());
//				this.generateTree();
			}
		}
		return treeRoot;
	}

	public void setTreeRoot(Node treeRoot) {
		this.treeRoot = treeRoot;
	}

	public boolean isDefinesSubscript() {
		return definesSubscript;
	}

	public void setDefinesSubscript(boolean definesSubscript) {
		this.definesSubscript = definesSubscript;
	}

	public boolean isHasLHSArrayReference() {
		return hasLHSArrayReference;
	}

	public void setHasLHSArrayReference(boolean hasLHSArrayReference) {
		this.hasLHSArrayReference = hasLHSArrayReference;
	}

	public boolean isHasRHSArrayReference() {
		return hasRHSArrayReference;
	}

	public void setHasRHSArrayReference(boolean hasRHSArrayReference) {
		this.hasRHSArrayReference = hasRHSArrayReference;
	}

	public boolean isHasMultipleEquations() {
		return hasMultipleEquations;
	}

	public void setHasMultipleEquations(boolean hasMultipleEquations) {
		this.hasMultipleEquations = hasMultipleEquations;
	}

	public String getUnitsAndComment() {
		StringBuilder sb = new StringBuilder();
		sb.append("/*\n");
		sb.append("\tEquation: " + equation + "\n");
		if (units != null || comment != null) {
			sb.append("\tUnits:" + (units == null ? "None Provided" : units)
					+ "\n");
			sb.append("\tComment: "
					+ (comment == null ? "None Provided" : comment) + "\n");
		}
		sb.append("*/\n");
		return sb.toString();
	}

	public boolean isOneTime() {
		return oneTime;
	}

	public void setOneTime(boolean oneTime) {
		this.oneTime = oneTime;
	}

	public boolean isRepeated() {
		return !isOneTime();
	}

	private String getNextArray() {
		String next = "_a" + nextArray++;
		return next;
	}

	private String getNextInt() {
		String next = "_i" + nextInt++;
		return next;
	}

	// private String getNextInteger() {
	// String next = "_I" + nextInt++;
	// return next;
	// }

	private String getNextTimeSeries() {
		String next = "_ts" + nextTimeSeries++;
		return next;
	}

	public boolean isHasException() {
		return hasException;
	}

	public void setHasException(boolean hasException) {
		this.hasException = hasException;
	}

	public int getLeftBracketCount() {
		return leftBracketCount;
	}

	public void setLeftBracketCount(int leftBracketCount) {
		this.leftBracketCount = leftBracketCount;
	}

	public List<String> getExceptions() {

		// Note that we expand the subscript to terminal values

		List<String> al = new ArrayList<String>();
		for (String subscript : exceptions)
			al.addAll(InformationManagers.getInstance()
					.getNamedSubscriptManager().getValuesFor(subscript));
		return al;
	}

	public boolean getsExternalData() {
		if (equation.toUpperCase().contains(Parser.externalDataMethod()) || equation.toUpperCase().contains("GET XLS DATA")
				|| equation.toUpperCase().contains("GET XLS LOOKUP")
				|| equation.toUpperCase().contains("GET XLS CONSTANT"))
			return true;
		else
			return false;
	}

	public boolean isVdmLookup() {
		return vdmLookup;
	}

	public void setVdmLookup(boolean vdmLookup) {
		this.vdmLookup = vdmLookup;
	}

	public String toString() {
		return getVensimEquationOnly();
//		if (getTreeRoot() != null) {
//		EquationComponents lEqn = new EquationComponents(getOriginalLhsQuotedIfNecessary()+" = " + 
//    			Node.generateExpression(TreeTraversal.getRhs(getTreeRoot()))+"~~autogenerated11");
//		Equation t = new Equation(lEqn, 1);
//		return t.getVensimEquationOnly();
//		} else {
//			return getVensimEquationOnly();
//		}
	}

	// public void convertToNativeDataTypes() {
	// // if (vensimEquation.startsWith("Production Capacity")) {
	// // log.debug("Problem");
	// // }
	// List<String> nativeTypes = new ArrayList<String>();
	// for (String token : tokens) {
	// if (token.startsWith("valueOf")) {
	// String tmp = token.replace("valueOf(\"", "");
	// tmp = tmp.replace("\")", "");
	// nativeTypes.add(NativeDataTypeManager.getLegalName(tmp));
	// } else if (token.startsWith("arrayValueOf")) {
	// // rip appart the invocation
	// //
	// arrayValueOf("Production Capacity",concatAsSubscript("Production Mode","Country"))
	// // becomes
	// // Memory.Production_Capacity[Production_Mode][Country]
	// String tmp = token.replace("arrayValueOf(", "");
	// String[] t = tmp.split(",", 2);
	// StringBuilder sb = new StringBuilder();
	// sb.append(NativeDataTypeManager.getLegalName(t[0].replace("\"", "")));
	// sb.append(t[1].replace("concatAsSubscript(\"","[").replace("\",\"",
	// "][").replace("\"", "]").replace("))", ""));
	// nativeTypes.add(sb.toString());
	// } else if (token.startsWith("array.")) {
	// // rip appart the invocation
	// // array.Initial Production Capacity[Production Mode,Country]
	// // becomes
	// // Memory.Initial_Production_Capacity[Production_Mode][Country]
	// ArrayReference ar = new ArrayReference(token);
	// StringBuilder sb = new StringBuilder();
	// sb.append(NativeDataTypeManager.getLegalName(ar.getArrayName()));
	// for (String subr : ar.getSubscripts())
	// sb.append("["+subr+"]");
	// nativeTypes.add(sb.toString());
	// } else {
	// nativeTypes.add(token);
	// }
	// }
	//
	// tokens = nativeTypes;
	//
	// }

	public List<Node> getTreeAsList() {
		if (treeRoot == null) {
			// generateRpn();
			// generateTree();
			return null;
		}
		List<Node> al = new ArrayList<Node>();

		Node lhs = treeRoot.getChild();

		// hack!
		// need to worry about array initialization, since its syntax violates
		// normal
		// binary operator use ("=")

		if (arrayInitialization) {
			traverse(treeRoot, al);
			al.remove(al.size() - 1);
		} else {

			al.add(lhs);
			traverse(lhs.getNext(), al);
		}

		return al;
	}

	public void traverse(Node node, List<Node> al) {
		if (node == null)
			return;
		al.add(node);
		traverse(node.getChild(), al);
		traverse(node.getNext(), al);

	}

	public EquationArrayReferenceStructure getEars() {
		if (ears == null)
			createEars();
		return ears;
	}

	public void setEars(EquationArrayReferenceStructure ears) {
		this.ears = ears;
	}

	public boolean isArrayInitialization() {
		return arrayInitialization;
	}

	public void setArrayInitialization(boolean arrayInitialization) {
		this.arrayInitialization = arrayInitialization;
	}

	public boolean isUsesTimeSeries() {
		return usesTimeSeries;
	}

	public void setUsesTimeSeries(boolean usesTimeSeries) {
		this.usesTimeSeries = usesTimeSeries;
	}

	public boolean isHasMacroInvocation() {
		return hasMacroInvocation;
	}

	public void setHasMacroInvocation(boolean hasMacroInvocation) {
		this.hasMacroInvocation = hasMacroInvocation;
	}

	public boolean isOrderedWithInitialValue() {
		return orderedWithInitialValue;
	}

	public void setOrderedWithInitialValue(boolean orderedWithInitialValue) {
		this.orderedWithInitialValue = orderedWithInitialValue;
	}

	public boolean isHasVectorSortOrder() {
		return hasVectorSortOrder;
	}

	public void setHasVectorSortOrder(boolean hasVectorSortOrder) {
		this.hasVectorSortOrder = hasVectorSortOrder;
	}

	public boolean isHasVectorElmMap() {
		return hasVectorElmMap;
	}

	public void setHasVectorElmMap(boolean hasVectorElmMap) {
		this.hasVectorElmMap = hasVectorElmMap;
	}

	public boolean requiresPostGenerationProcessing() {
		return hasVectorElmMap || hasVectorSortOrder;
	}

	public String getBtwnMode() {
		return btwnMode;
	}

	public void setBtwnMode(String btwnMode) {
		this.btwnMode = btwnMode;
	}

	public boolean isGetXlsConstants() {

		return vensimEquation.toUpperCase().contains("GET XLS CONSTANTS");
		// return definesLookupGetXls;
	}

	public boolean isDefinesLookupGetXls() {

		return vensimEquation.toUpperCase().contains("GET XLS LOOKUPS");
		// return definesLookupGetXls;
	}

	public boolean isStock() {
		return stockVariable;
	}

	public boolean isSyntacticallyCorrect() {
		return syntacticallyCorrect;
	}

	public void setSyntacticallyCorrect(boolean syntacticallyCorrect) {
		this.syntacticallyCorrect = syntacticallyCorrect;
	}

	public boolean isUsageCorrect() {
		return usageCorrect;
	}

	public void setUsageCorrect(boolean semanticallyCorrect) {
		this.usageCorrect = semanticallyCorrect;
	}

	public List<String> getSyntaxMessages() {
		return syntaxMessages;
	}

	public List<String> getSemanticMessages() {
		return usageMessages;
	}

	public List<String> getUnitsMessages() {

		return unitsMessages;
	}

	public List<String> getUnitsInconsistencyReport() {

		List<String> bw = new ArrayList<String>();
		// bw.add("Equation:\n");
		// bw.add("\t"+StringEscapeUtils.escapeHtml(unitExpression.getVensimEquation().split("~")[0]));
		SDMDocLanguageSpecific language = SDMDocLanguageSpecific.getInstance();
		if (units == null || units.equals("")) {
			bw.add(language.getTextHtml(StringConstants.LHS_UNITS)+": "+
					language.getTextHtml(StringConstants.NONE_SPECIFIED));
//			bw.add("LHS Units: None Specified");
		} else if (unitExpression != null) {
			bw.add(language.getTextHtml(StringConstants.LHS_UNITS)+": "); // "\n"
//			bw.add("LHS Units: "); // "\n"
			bw.add("\t" + unitExpression.getLhsUnitsString());
			bw.add(language.getTextHtml(StringConstants.RHS_UNITS)+": "); // "\n"
//			bw.add("RHS Units: "); // "\n"
			bw.add("\t" + unitExpression.getRhsUnitsString());
			bw.add(language.getTextHtml(StringConstants.COMPLETE_RHS_UNITS)+": "); // "\n"
//			bw.add("Complete RHS Units: "); // "\n"
			bw.add("\t" + unitExpression.getCompleteRhsUnitsString());
		}

		return bw;
	}

	public UnitExpression getUnitExpression() {
		return unitExpression;
	}

	public void setUnitExpression(UnitExpression unitExpression) {
		this.unitExpression = unitExpression;
	}

	public String getEquationFromTree(Node root) {
		StringBuilder sb = new StringBuilder();

		if (getVensimEquationOnly().contains("^"))
			log.debug("STOP");

		Node lhs = root.getChild();
		Node rhs = lhs.getNext();

		sb.append(lhs.getToken() + " = ");
		buildEquation(sb, rhs);

		return sb.toString();

	}

	public void buildEquation(StringBuilder sb, Node node) {
		if (node == null)
			return;
		// unary operator need special processing
		String token = node.getToken();
		if (token.equals("^")) {
			// need to generate a function call here
			sb.append("Math.pow(");
			buildEquation(sb, node.getChild());
			sb.append(",");
			buildEquation(sb, node.getChild().getNext());
			sb.append(")");
			return;
		}
		if (Parser.isArithmeticOperator(token) && Parser.isUnaryOperator(token)) {
			if (token.equals(Parser.INTERNAL_UNARY_MINUS))
				sb.append("-");
			else
				sb.append(token);
			buildEquation(sb, node.getChild());
		} else {
			// need to handle function invocations differently
			if (Parser.isFunctionInvocation(node.getToken())) {
				sb.append(node.getToken());
				sb.append("(");
				int argNum = 0;
				Node args = node.getChild();
				while (args != null) {
					if (argNum++ > 0)
						sb.append(",");
					if (args != null && !args.isTerminal())
						sb.append("(");
					buildEquation(sb, args);
					if (args != null && !args.isTerminal())
						sb.append(")");
					args = args.getNext();
				}
				sb.append(")");
			} else {
				Node child = node.getChild();
				if (child != null && !child.isTerminal())
					sb.append("(");
				buildEquation(sb, child);
				if (child != null && !child.isTerminal())
					sb.append(")");
				sb.append(node.getToken());
				if (child == null)
					return;
				Node next = child.getNext();
				if (next != null && !next.isTerminal())
					sb.append("(");
				buildEquation(sb, next);
				if (next != null && !next.isTerminal())
					sb.append(")");
			}
		}
	}
	
	public String getLEEAEquationFromTree(Node root) {
		StringBuilder sb = new StringBuilder();

		if (getVensimEquationOnly().contains("^"))
			log.debug("STOP");

		Node lhs = root.getChild();
		Node rhs = lhs.getNext();

		sb.append(lhs.getToken() + " = ");
		buildLEEAEquation(sb, rhs);

		return sb.toString();

	}

	public void buildLEEAEquation(StringBuilder sb, Node node) {
		if (node == null)
			return;
		// unary operator need special processing
		String token = node.getToken();
		if (token.equals("^")) {
			// need to generate a function call here
			sb.append("Math.pow(");
			buildEquation(sb, node.getChild());
			sb.append(",");
			buildEquation(sb, node.getChild().getNext());
			sb.append(")");
			return;
		}
		if (Parser.isArithmeticOperator(token) && Parser.isUnaryOperator(token)) {
			if (token.equals(Parser.INTERNAL_UNARY_MINUS))
				sb.append("-");
			else
				sb.append(token);
			buildEquation(sb, node.getChild());
		} else {
			// need to handle function invocations differently
			if (Parser.isFunctionInvocation(node.getToken())) {
				boolean lookup = node.getToken().equals("sdFunctions.LOOKUP");
				sb.append(node.getToken());
				sb.append("(");
				int argNum = 0;
				Node args = node.getChild();
				while (args != null) {
					if (argNum++ > 0)
						sb.append(",");
					if (args != null && !args.isTerminal())
						sb.append("(");
					if (lookup) {
						sb.append("DBREPLACELOOKUP "+args.getToken());
						lookup = false;
					} else {
						buildLEEAEquation(sb, args);
					}
					if (args != null && !args.isTerminal())
						sb.append(")");
					args = args.getNext();
				}
				sb.append(")");
			} else {
				Node child = node.getChild();
				if (child != null && !child.isTerminal())
					sb.append("(");
				buildLEEAEquation(sb, child);
				if (child != null && !child.isTerminal())
					sb.append(")");
				sb.append(node.getToken());
				if (child == null)
					return;
				Node next = child.getNext();
				if (next != null && !next.isTerminal())
					sb.append("(");
				buildLEEAEquation(sb, next);
				if (next != null && !next.isTerminal())
					sb.append(")");
			}
		}
	}

	// bad hack
	
	public boolean isAutoGenerated() {
		return autoGenerated;
	}

	public void setAutoGenerated(boolean autoGenerated) {
		this.autoGenerated = autoGenerated;
	}

	public boolean isTreeCodeGenerated() {
		return treeCodeGenerated;
	}

	public void setTreeCodeGenerated(boolean treeCodeGenerated) {
		this.treeCodeGenerated = treeCodeGenerated;
	}

	public boolean isFatal() {
		return fatal;
	}

	public void setFatal(boolean fatal) {
		this.fatal = fatal;
	}

	public List<String> getFatalMessages() {
		return fatalMessages;
	}

	public boolean isUsesInitialFunction() {
		return usesInitialFunction;
	}

	public boolean isCallsFunction() {
		return callsFunction;
	}

	public void setCallsFunction(boolean callsFunction) {
		this.callsFunction = callsFunction;
	}

	public EquationComponents getEquationComponents() {
		return equationComponents;
	}

	public String getModule() {
		return module;
	}

	public void setModule(String module) {
		if ((isAssignment() || isDefinesLookup()) && lhs.contains(":")) {
			this.module = (lhs.split(":")[0]/* +":" */).replace("array.", "")
					.replace("lookup.", "");
		} else {
			this.module = module;
		}
	}
	
	public boolean isControlGroup() {
		if (lhsAsInMdl == null)
			return false;
		return Parser.isSystemVariable(lhsAsInMdl);
	}

	public String getGroup() {
		return group;
	}

	public void setGroup(String group) {
		this.group = group;
	}

	public List<String> getViews() {
		return views;
	}

	public boolean isUsesMinMax() {
		return usesMinMax;
	}

	public boolean isUsesIfThenElse() {
		return usesIfThenElse;
	}

	public boolean isUsesStepPulse() {
		return usesStepPulse;
	}

	public boolean isUsesSmooth() {
		return usesSmooth;
	}

	public boolean isUsesDelay() {
		return usesDelay;
	}

	public boolean isLevelInitialization() {
		return levelInitialization;
	}

	public boolean isInitial() {
		initial = usesInitialFunction;
		return initial;
	}

	public boolean isConstant() {
		return constant;
	}

	public boolean isFlow() {
		return flow;
	}

	public boolean isAuxiliary() {
		return auxiliary;
	}

	public boolean isSubscripts() {
		subscripts = isDefinesSubscript();
		return subscripts;
	}

	public boolean isDataByMeta() {
		if (getComment() == null) {
			data = false;
			return data;
		}
		
		
		if (isGetExternalData() || (isDefinesLookup() && getComment().toLowerCase().contains(VariableSelector.LOOKUP.toLowerCase())))
			data = true;
		else
			data = false;
		return data;
	}
	
	public boolean isData() {
//		return isDataByMeta();
		if (isGetExternalData() || isDefinesLookup() )
			data = true;
		else
			data = false;
		return data;
	}

	public boolean isGame() {
		return game;
	}
	
	public boolean isGameOrConstant() {
		return game || isConstant();
	}


	public boolean isEmbeddedDataNot01() {
		return embeddedDataNot01;
	}

	public boolean isIncompletelyDefinedSubscript() {
		return incompletelyDefinedSubscript;
	}

	public boolean isUnused() {
		return unused;
	}

	public boolean isSupplementary() {
		return supplementary;
	}

	public boolean isSupplementaryUsed() {
		return supplementaryUsed;
	}

	public boolean isEmbeddedData() {
		return embeddedData01 || embeddedDataNot01;
	}

	public boolean isEmbeddedData01() {
		return embeddedData01;
	}

	public boolean isComplexFormulation() {
		return complexFormulation;
	}

	public boolean isComplexStock() {
		return complexStock;
	}

	public boolean isLookup() {
		return isDefinesLookup() || isDefinesLookupGetXls();
	}

	public boolean isUndocumented() {
		return getComment() == null || getComment().length() == 0;
	}

	public boolean isUnitErrorsWarnings() {
		return unitsMessages.size() > 0;
	}

	public boolean isNotInView() {
		return views.size() == 1 && views.get(0).equals(" Not in View");
	}

	public int getEquationCount() {
		return 1;
	}

	public int getTotalEquationCount() {
		if (totalEquationCount != -1)
			return totalEquationCount;
		if (!isSyntacticallyCorrect())
			return 0;
		if (smooth3 || smoothn || delay3 || delayn)
			nValue = computeN();
		if (isHasLHSArrayReference()) {
			int numCombos = InformationManagers.getInstance()
					.getNamedSubscriptManager()
					.getNumSubscriptCombinations(new ArrayReference(lhs));
			totalEquationCount = numCombos * nValue;
			return numCombos * nValue;
			// return getEars().getNumValues();
		} else {
			totalEquationCount = nValue;
			return nValue;
		}
	}

	public boolean createXYData(List<Double> x, List<Double> y,
			MutableDouble minX, MutableDouble maxX, MutableDouble minY,
			MutableDouble maxY) {
		if (this.isDefinesLookupGetXls()) {
			return (createLookupListsFromXLS(x, y, minX, maxX, minY, maxY));
		} else {
			createLookupLists(x, y, minX, maxX, minY, maxY);
			return true;
		}
	}

	public boolean createLookupListsFromXLS(List<Double> x, List<Double> y,
			MutableDouble minX, MutableDouble maxX, MutableDouble minY,
			MutableDouble maxY) {
		
		if (lhs.contains("Influence of Attackers Relative Success on Motivation Table")) {
			System.out.println("Influence of Attackers Relative Success on Motivation Table");
		}
		// use the equation try to retrieve arguments
		Node func = TreeTraversal.getRhs(getTreeRoot());
		String file =  Parser.removeQuotes(TreeTraversal.getFunctionArgument(func, 1).getToken()); // .replace("\"", "");
		String tab =  Parser.removeQuotes(TreeTraversal.getFunctionArgument(func, 2).getToken()); // .replace("\"", "");
		String rowOrCol =  Parser.removeQuotes(TreeTraversal.getFunctionArgument(func, 3).getToken()); // .replace("\"", "");
		String cell =  Parser.removeQuotes(TreeTraversal.getFunctionArgument(func, 4).getToken()); // .replace("\"", "");
		int numCombo = 1;
		NamedSubscriptManager am = InformationManagers.getInstance()
				.getNamedSubscriptManager();
		if (ArrayReference.isArrayReference(this.getLhs())) {
			numCombo = am.getNumSubscriptCombinations(new ArrayReference(this
					.getLhs()));
		}

		String modelDir = InformationManagers.getInstance().getModelManager()
				.getModelDirectory();
		String fullPath = modelDir + "\\" + file;
		File xls = new File(fullPath);
		if (xls.exists() && xls.canRead()) {
			
			GetXlsLookups gxl = new GetXlsLookups(fullPath, tab, rowOrCol,cell, numCombo);
			TimeSeriesInstance tsi = gxl.getLookupData();
			if (tsi != null) {
			lookupXlsReadable = true;
			minX.setValue(Double.MIN_VALUE);
			minY.setValue(Double.MIN_VALUE);
			maxX.setValue(Double.MAX_VALUE);
			maxY.setValue(Double.MAX_VALUE);

			for (Double d : tsi.getTime())
				x.add(d);
			for (Double d : tsi.getValues())
				y.add(d);
			} else {
				lookupXlsReadable = false;
				InformationManagers
				.getInstance()
				.getModelManager()
				.setRunnable(
						false,
						"Required Excel tab not readable: "
								+ xls.getPath());
			}
		} else {
			lookupXlsReadable = false;
			InformationManagers
					.getInstance()
					.getModelManager()
					.setRunnable(
							false,
							"Required Excel file not readable: "
									+ xls.getPath());
		}
		return lookupXlsReadable;

	}

	public boolean isNonmonotonicLookup() {
		if (!isDefinesLookup() && !isDefinesLookupGetXls())
			return false;
		if (isDataByMeta())
			return false;
		if (isDefinesLookupGetXls()) {
			log.debug(getVensimEquationOnly()
					+ "\nNonmonotonic Lookup Analysis for GET XLS not supported ");
			return false;
		}
		List<Double> x = new ArrayList<Double>();
		List<Double> y = new ArrayList<Double>();
		createLookupLists(x, y, null, null, null, null);
		return isNonmonotinic(x, y);
	}

	public boolean getSlopeDirections(MutableBoolean positiveSlope,
			MutableBoolean negativeSlope, MutableBoolean nonZeroSlope) {

		try {
			List<Double> x = new ArrayList<Double>();
			List<Double> y = new ArrayList<Double>();
			createLookupLists(x, y, null, null, null, null);

			for (int i = 0; i < y.size() - 1; i++) {
				if (y.get(i + 1).doubleValue() > y.get(i).doubleValue())
					positiveSlope.setValue(true);
				if (y.get(i + 1).doubleValue() < y.get(i).doubleValue())
					negativeSlope.setValue(true);
				if (y.get(i + 1).doubleValue() != y.get(i).doubleValue())
					nonZeroSlope.setValue(true);
			}

			return true;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return false;
		}

	}

	private boolean isNonmonotinic(List<Double> x, List<Double> y) {
		boolean positiveSlope = false;
		boolean negativeSlope = false;

		for (int i = 0; i < y.size() - 1; i++) {
			if (y.get(i + 1).doubleValue() > y.get(i).doubleValue())
				positiveSlope = true;
			if (y.get(i + 1).doubleValue() < y.get(i).doubleValue())
				negativeSlope = true;
		}

		return positiveSlope && negativeSlope;

	}

	public boolean isCascadingLookup() {
		// this is a source code check, can be done via tree
		if (!isReferencesLookup())
			return false;
		// printTree();

		boolean cascading = false;
		List<Node> treeAsList = TreeTraversal.getTreeAsList(getTreeRoot());
		for (Node n : treeAsList) {
			if (isLookupReference(n)) {
				for (Node sub : TreeTraversal.getTreeAsList(n)) {
					if (!sub.equals(n)
							&& (isLookupReference(sub) || isIndirectLookupReference(sub))) {
						cascading = true;
						return cascading;
					}
				}
			}
		}
		return cascading;
	}

	private boolean isIndirectLookupReference(Node node) {

		String screenName = InformationManagers.getInstance().getEquationLookup().getScreenName(node.getToken());
		if (screenName == null)
			return false;
		Equation eqn = InformationManagers.getInstance().getEquationLookup().getEquation(screenName);
		if (eqn == null)
			return false;
		return eqn.isReferencesLookup();

	}

	private boolean isLookupReference(Node node) {
		if (node.getToken().startsWith("sdFunctions.LOOKUP"))
			return true;
		else
			return false;
	}

	public boolean isNonzeroEndSlopeLookup(MutableBoolean left,
			MutableBoolean right) {
		if (!isDefinesLookup() && !isDefinesLookupGetXls())
			return false;
		if (isDataByMeta())
			return false;
		if (isDefinesLookupGetXls()) {
			log.debug(getVensimEquationOnly()
					+ "\nNon-zero End Slope Lookup Analysis for GET XLS not supported ");
			return false;
		}
		List<Double> x = new ArrayList<Double>();
		List<Double> y = new ArrayList<Double>();
		createLookupLists(x, y, null, null, null, null);
		return isNonzeroEndSlopeLookup(x, y, left, right);
	}

	private boolean isNonzeroEndSlopeLookup(List<Double> x, List<Double> y,
			MutableBoolean left, MutableBoolean right) {

		int lastPos = y.size() - 1;

		if (lastPos < 1)
			return false;

		if (y.get(lastPos).doubleValue() != y.get(lastPos - 1).doubleValue())
			right.setValue(true);

		if (y.get(0).doubleValue() != y.get(1).doubleValue())
			left.setValue(true);

		return left.value() || right.value();

	}

	public boolean isValueRangeSpecified() {

		// sample: Week [0,?]
		// sample: Week [0,52]

		// I am going to assume that "?" is defined

		String units = equationComponents.getUnitsPart();
		if (!units.contains("\\[")) {
			valueRangeSpecified = false;
		} else {
			// let's parse this for grins

			String range = units.split("[")[1].replace("]", "").trim();
			if (!range.contains(",")) {
				valueRangeSpecified = false;
			} else {
				String[] m = range.split(",");
				if ((m[0].equals("?") || Parser.isNumber(m[0]))
						&& (m[1].equals("?") || !Parser.isNumber(m[1]))) {
					valueRangeSpecified = true;
				} else {
					valueRangeSpecified = false;
				}
			}
			valueRangeSpecified = true;
		}

		return valueRangeSpecified;
	}

	public int getEquationNumber() {
		return equationNumber;
	}

	public void setEquationNumber(int variableNumber) {
		this.equationNumber = variableNumber;
	}

	public EquationCharacteristics getEquationCharacteristics() {
		return equationCharacteristics;
	}

	public void setEquationCharacteristics(
			EquationCharacteristics equationCharacteristics) {
		this.equationCharacteristics = equationCharacteristics;
	}

	public void setFlow(boolean flow) {
		this.flow = flow;
	}

	public void setAuxiliary(boolean auxiliary) {
		this.auxiliary = auxiliary;
	}

	public void addView(String view) {
		if (views.contains(" Not in View"))
			views.remove(" Not in View");
		if (!views.contains(view))
			views.add(view);
	}

	public VariableInformation getVariableInformation() {
		return variableInformation;
	}

	public void setVariableInformation(VariableInformation variableInformation) {
		this.variableInformation = variableInformation;
	}

	public List<String> getUsedBy() {
		if (variableInformation == null) {
			return new ArrayList<String>();
		}
		return variableInformation.getUsedBy();
	}

	public void setLevelInitialization(boolean levelInitialization) {
		this.levelInitialization = levelInitialization;
	}

	public String getType() {
		if (equationCharacteristics == null)
			return "Unavailable";
		return equationCharacteristics.getType();
	}

	public boolean isSensitivity() {
		return false;
	}

	public boolean isSourceInformation() {
		return false;
	}

	public void setUnused(boolean unused) {
		this.unused = unused;
		if (equationCharacteristics != null)
			equationCharacteristics.setUnused(unused);
	}

	public String getComplexFormulationCount() {
		return complexFormulationCount;
	}
	
	public VariableComplexityData getComplexityData() {
		AnalysisManager am = InformationManagers.getInstance().getAnalysisManager();
		if (am != null) {
			VariableComplexity vc = InformationManagers.getInstance().getAnalysisManager().getVariableComplexity();
			if (vc != null) {
				VariableComplexityData vcd = vc.getDataFor(lhsAsInMdl != null ? lhsAsInMdl : lhs);
				if (vcd != null) {
					return vcd;
				} else {
					return null;
				}
			} else {
				return null;
			}
		} else {
			return null;
		}
	}

	public void setComplexFormulationCount(String complexFormulationCount) {
		this.complexFormulationCount = complexFormulationCount;
	}

	public int getVariableNumber() {
		return variableNumber;
	}
	
	public String getVariableName() {
		if (equationVariable != null)
			return equationVariable.getOriginalVariableNameNoQuote();
		else
			return Parser.removeQuotes(Parser.removeAnnotation(Parser.removeAnySubscripts(lhs)));  //remove/*All*/Quotes
	}

	public void setVariableNumber(int variableNumber) {
		this.variableNumber = variableNumber;
		// for a map, this variable not created yet. do it now.
		
		if (equationVariable == null)
			equationVariable = new EquationVariable(this);
		equationVariable.setVariableNumber(variableNumber);
	}

	public int getComplexity() {
		return complexity;
	}

	public void setComplexity(int complexity) {
		this.complexity = complexity;
	}

	public boolean isTypeString() {
		return typeString;
	}

	public boolean isSmooth3() {
		return smooth3;
	}

	public boolean isSmoothn() {
		return smoothn;
	}

	public boolean isDelay3() {
		return delay3;
	}

	public boolean isDelayn() {
		return delayn;
	}

	public int getnValue() {
		return nValue;
	}

	public String getLhsAsInMdl() {
		return lhsAsInMdl;
	}

	public void setLhsAsInMdl(String lhsAsInMdl) {
		this.lhsAsInMdl = lhsAsInMdl;
	}
	
	private String getInterpolationMode(String s) {
		if (s.toUpperCase().contains(":INTERPOLATE:"))
			return "INTERPOLATE";
		else if (s.toUpperCase().contains(":LOOK FORWARD:"))
			return "LOOK FORWARD";
		else if (s.toUpperCase().contains(":HOLD BACKWARD:"))
			return "HOLD BACKWARD";
		else if (s.toUpperCase().contains(":RAW:"))
			return "RAW";
		else
			return "";
	}
	
	private String removeInterpolationMode(String s) {
		return s.replace(":INTERPOLATE:", "").replace(":LOOK FORWARD:", "").replace(":HOLD BACKWARD:", "").replace(":RAW:", "");
	}
	
	private boolean containsInterpolationMode(String s) {
		if (s.toUpperCase().contains(":INTERPOLATE:"))
			return true;
		else if (s.toUpperCase().contains(":LOOK FORWARD:"))
			return true;
		else if (s.toUpperCase().contains(":HOLD BACKWARD:"))
			return true;
		else if (s.toUpperCase().contains(":RAW:"))
			return true;
		else
			return false;
	}
	
	private boolean startsWithInterpolationMode(String s) {
		if (s.toUpperCase().startsWith(":INTERPOLATE:"))
			return true;
		else if (s.toUpperCase().startsWith(":LOOK FORWARD:"))
			return true;
		else if (s.toUpperCase().startsWith(":HOLD BACKWARD:"))
			return true;
		else if (s.toUpperCase().startsWith(":RAW:"))
			return true;
		else
			return false;
	}

	private boolean endsWithInterpolationMode(String s) {
		if (s.toUpperCase().endsWith(":INTERPOLATE:"))
			return true;
		else if (s.toUpperCase().endsWith(":LOOK FORWARD:"))
			return true;
		else if (s.toUpperCase().endsWith(":HOLD BACKWARD:"))
			return true;
		else if (s.toUpperCase().endsWith(":RAW:"))
			return true;
		else
			return false;
	}

	/**
	 * @return the abort
	 */
	public boolean isAbort() {
		return abort;
	}

	/**
	 * @return the lookupXlsReadable
	 */
	public boolean isLookupXlsReadable() {
		return lookupXlsReadable;
	}

	public boolean isActiveInitial() {
		return activeInitial;
	}

	public boolean isSampleIfTrue() {
		return sampleIfTrue;
	}

	public void setSampleIfTrue(boolean sampleIfTrue) {
		this.sampleIfTrue = sampleIfTrue;
	}

	public boolean writeCode(CodeGenerator cg, BufferedWriter bw) {
		boolean success = true;

		processExponentiation();
		generateRpn();
		generateTree();
		cg.generateCode(this, bw, CodeGenerator.DEBUG_VALUES);

		return success;
	}

	/**
	 * @return the justMap
	 */
	public boolean isJustMap() {
		return justMap;
	}

	/**
	 * @param justMap
	 *            the justMap to set
	 */
	public void setJustMap(boolean justMap) {
		this.justMap = justMap;
	}

	public boolean containsVectorFunction() {
		return hasVectorSortOrder || hasVectorElmMap || hasVectorSelect
				|| hasVectorRank || hasVectorReorder;
	}

	public boolean isVectorResult() {
		return hasVectorSortOrder || hasVectorReorder || hasVectorRank;
	}

	/**
	 * @return the hasVectorSelect
	 */
	public boolean isHasVectorSelect() {
		return hasVectorSelect;
	}

	/**
	 * @param hasVectorSelect
	 *            the hasVectorSelect to set
	 */
	public void setHasVectorSelect(boolean hasVectorSelect) {
		this.hasVectorSelect = hasVectorSelect;
	}

	/**
	 * @return the ite
	 */
	public boolean isIte() {
		return ite;
	}

	/**
	 * @param ite
	 *            the ite to set
	 */
	public void setIte(boolean ite) {
		this.ite = ite;
	}

	/**
	 * @return the usesRangeFunction
	 */
	public boolean isUsesRangeFunction() {
		return usesRangeFunction;
	}

//	/**
//	 * @return the nativeMemoryImplementation
//	 */
//	public boolean isNativeMemoryImplementation() {
//		return nativeMemoryImplementation;
//	}
//
//	/**
//	 * @param nativeMemoryImplementation
//	 *            the nativeMemoryImplementation to set
//	 */
//	public void setNativeMemoryImplementation(boolean nativeMemoryImplementation) {
//		this.nativeMemoryImplementation = nativeMemoryImplementation;
//	}

	public String getRHSConstantValue() {

		Node rhs = TreeTraversal.getRhs(getTreeRoot());
		String val = rhs.getToken();
		double value;
		if (Parser.isNumber(val)) {
			return val;
		} else if (Parser.isNumber(this.rhs)) {
			return this.rhs;
		} else {
			Equation eqn = InformationManagers.getInstance().getEquationManager().getAnEquationFor(val);
			if (eqn != null)
				return eqn.getRHSConstantValue();
			else
				return null;
		}
	}

	public boolean containsInitialValueFunction() {
		// want to traverse tree (via tokens)
		for (String token : getTokens()) {
			if (Parser.isFunctionInvocation(token)) {
				FunctionDescription fd = InformationManagers.getInstance()
						.getFunctionManager().getDescription(token);
				if (fd.isSuppliesInitialValue()) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * @return the rhsTokens
	 */
	public HashSet<String> getRhsTokens() {
		return rhsTokens;
	}

	public String getStockFlowAuxiliaryType() {
		String type = "Unknown";
		if (isStock())
			return "stock";
		else if (isFlow())
			return "flow";
		else if (isAuxiliary())
			return "auxiliary";
		return type;
	}

	public int getVectorSpaceRequirements() {
		return vectorSpaceRequirements;
	}

	public void setAbort(boolean abort) {
		this.abort = abort;
	}

	public boolean isDoNotSimplifyLoops() {
		return doNotSimplifyLoops;
	}
	
	public void markFlows() {

		Equation e = this;

		if (getLhs().equals("Water in the Natural Reservoirs"))
			System.out.println("Water in the Natural Reservoirs");

		// if we are used by a stock, we will either be a flow, or we will be an initializer
		// or both!
		// 4/17 can also be part of If Then Else construct
		if (e.isStock()) {

			Node integFunc = e.getTreeRoot().getChild().getNext();

			Node arg1 = TreeTraversal.getFunctionArgument(integFunc, 1);
			List<String> v = Node.extractVariables(arg1);

			Set<String> s = new LinkedHashSet<String>(v); // e.getNonFunctionInitialVariables();

			for (String i : s) {
				String o = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(i));
				if (Parser.isSystemVariable(i))
					continue;

				Set<Equation> allEqn = InformationManagers.getInstance().getEquationManager().getAllEquationsFor(o);
				if (allEqn == null || allEqn.size() == 0) {
					System.out.println("EQN markFlows: no eqn for <"+o+">");
				} else {
					for (Equation eqn : allEqn)
						eqn.setFlow(true);;
				}
			}
			
			Node arg2 = TreeTraversal.getFunctionArgument(integFunc, 2);
			v = Node.extractVariables(arg2);

			s = new LinkedHashSet<String>(v); // e.getNonFunctionInitialVariables();

			for (String i : s) {
				String o = Parser.noSubscripts(InformationManagers.getInstance().getNativeDataTypeManager().getOriginalName(i));
				if (Parser.isSystemVariable(i))
					continue;

				Set<Equation> allEqn = InformationManagers.getInstance().getEquationManager().getAllEquationsFor(o);
				if (allEqn == null || allEqn.size() == 0) {
					System.out.println("EQN markFlows (LI): no eqn for <"+o+">");
				} else {
					for (Equation eqn : allEqn)
						eqn.setLevelInitialization(true);
				}
			}

		}
	}

	public void setOriginalVensimEquation(String originalVensimEquation) {
		this.originalVensimEquation = originalVensimEquation;
	}

	public boolean isIncludeInTranslation() {
		return includeInTranslation;
	}

	public void setIncludeInTranslation(boolean includeInTranslation) {
		this.includeInTranslation = includeInTranslation;
	}

	public Set<String> getFunctionsCalled() {
		return functionsCalled;
	}
	
	public boolean isExogenous() {

		// this value take precedence 
		if (exogenous!= null)
			return exogenous.booleanValue();
		else
			exogenous = isConstant() || isData() || getNumInputLinks() == 0;
			return exogenous; 
	}
	
	public boolean isEndogenous() {
		return !isExogenous();
	}

	public int getNumInputLinks() {
//		if (numInputLinks == 0)
//			System.out.println("EXOGENOUS");
		return numInputLinks;
	}

	public void setNumInputLinks(int numInputLinks) {
		this.numInputLinks = numInputLinks;
	}
	
	public void incrementNumInputLinksConditional(String var) {
		if (rhsTokensStripped == null) {
			rhsTokensStripped = new LinkedHashSet<String>();
			for (String s : rhsTokens) {
				if (ArrayReference.isArrayReference(s))
					rhsTokensStripped.add(ArrayReference.extractArrayName(s));
				else
					rhsTokensStripped.add(s);
			}
		}
		if (rhsTokensStripped.contains(var))
			this.numInputLinks += 1;
	}
	
	public void incrementNumOutputLinksConditional(String var) {
		this.numOutputLinks += 1;
	}

	public int getNumOutputLinks() {
		return numOutputLinks;
	}

	public void setNumOutputLinks(int numOutputLinks) {
		this.numOutputLinks = numOutputLinks;
	}

	public ArrayReference getLhsArrayReference() {
		return lhsArrayReference;
	}
	
	public void setExogenous(boolean bool) {
		exogenous = new Boolean(bool);
		
	}

	public int getNumSmooth() {
		return numSmooth;
	}

	public int getNumSmooth3() {
		return numSmooth3;
	}

	public int getNumSmoothN() {
		return numSmoothN;
	}
	
	public int getNumDelay() {
		return numDelay;
	}

	public int getNumDelay3() {
		return numDelay3;
	}

	public int getNumDelayN() {
		return numDelayN;
	}

}
